# frozen_string_literal: true

# WARNING ABOUT GENERATED CODE
#
# This file is generated. See the contributing guide for more information:
# https://github.com/aws/aws-sdk-ruby/blob/version-3/CONTRIBUTING.md
#
# WARNING ABOUT GENERATED CODE

require 'seahorse/client/plugins/content_length'
require 'aws-sdk-core/plugins/credentials_configuration'
require 'aws-sdk-core/plugins/logging'
require 'aws-sdk-core/plugins/param_converter'
require 'aws-sdk-core/plugins/param_validator'
require 'aws-sdk-core/plugins/user_agent'
require 'aws-sdk-core/plugins/helpful_socket_errors'
require 'aws-sdk-core/plugins/retry_errors'
require 'aws-sdk-core/plugins/global_configuration'
require 'aws-sdk-core/plugins/regional_endpoint'
require 'aws-sdk-core/plugins/endpoint_discovery'
require 'aws-sdk-core/plugins/endpoint_pattern'
require 'aws-sdk-core/plugins/response_paging'
require 'aws-sdk-core/plugins/stub_responses'
require 'aws-sdk-core/plugins/idempotency_token'
require 'aws-sdk-core/plugins/invocation_id'
require 'aws-sdk-core/plugins/jsonvalue_converter'
require 'aws-sdk-core/plugins/client_metrics_plugin'
require 'aws-sdk-core/plugins/client_metrics_send_plugin'
require 'aws-sdk-core/plugins/transfer_encoding'
require 'aws-sdk-core/plugins/http_checksum'
require 'aws-sdk-core/plugins/checksum_algorithm'
require 'aws-sdk-core/plugins/request_compression'
require 'aws-sdk-core/plugins/defaults_mode'
require 'aws-sdk-core/plugins/recursion_detection'
require 'aws-sdk-core/plugins/telemetry'
require 'aws-sdk-core/plugins/sign'
require 'aws-sdk-core/plugins/protocols/ec2'
require 'aws-sdk-ec2/plugins/copy_encrypted_snapshot'
require 'aws-sdk-ec2/plugins/region_validation'

module Aws::EC2
  # An API client for EC2.  To construct a client, you need to configure a `:region` and `:credentials`.
  #
  #     client = Aws::EC2::Client.new(
  #       region: region_name,
  #       credentials: credentials,
  #       # ...
  #     )
  #
  # For details on configuring region and credentials see
  # the [developer guide](/sdk-for-ruby/v3/developer-guide/setup-config.html).
  #
  # See {#initialize} for a full list of supported configuration options.
  class Client < Seahorse::Client::Base

    include Aws::ClientStubs

    @identifier = :ec2

    set_api(ClientApi::API)

    add_plugin(Seahorse::Client::Plugins::ContentLength)
    add_plugin(Aws::Plugins::CredentialsConfiguration)
    add_plugin(Aws::Plugins::Logging)
    add_plugin(Aws::Plugins::ParamConverter)
    add_plugin(Aws::Plugins::ParamValidator)
    add_plugin(Aws::Plugins::UserAgent)
    add_plugin(Aws::Plugins::HelpfulSocketErrors)
    add_plugin(Aws::Plugins::RetryErrors)
    add_plugin(Aws::Plugins::GlobalConfiguration)
    add_plugin(Aws::Plugins::RegionalEndpoint)
    add_plugin(Aws::Plugins::EndpointDiscovery)
    add_plugin(Aws::Plugins::EndpointPattern)
    add_plugin(Aws::Plugins::ResponsePaging)
    add_plugin(Aws::Plugins::StubResponses)
    add_plugin(Aws::Plugins::IdempotencyToken)
    add_plugin(Aws::Plugins::InvocationId)
    add_plugin(Aws::Plugins::JsonvalueConverter)
    add_plugin(Aws::Plugins::ClientMetricsPlugin)
    add_plugin(Aws::Plugins::ClientMetricsSendPlugin)
    add_plugin(Aws::Plugins::TransferEncoding)
    add_plugin(Aws::Plugins::HttpChecksum)
    add_plugin(Aws::Plugins::ChecksumAlgorithm)
    add_plugin(Aws::Plugins::RequestCompression)
    add_plugin(Aws::Plugins::DefaultsMode)
    add_plugin(Aws::Plugins::RecursionDetection)
    add_plugin(Aws::Plugins::Telemetry)
    add_plugin(Aws::Plugins::Sign)
    add_plugin(Aws::Plugins::Protocols::EC2)
    add_plugin(Aws::EC2::Plugins::CopyEncryptedSnapshot)
    add_plugin(Aws::EC2::Plugins::RegionValidation)
    add_plugin(Aws::EC2::Plugins::Endpoints)

    # @overload initialize(options)
    #   @param [Hash] options
    #
    #   @option options [Array<Seahorse::Client::Plugin>] :plugins ([]])
    #     A list of plugins to apply to the client. Each plugin is either a
    #     class name or an instance of a plugin class.
    #
    #   @option options [required, Aws::CredentialProvider] :credentials
    #     Your AWS credentials. This can be an instance of any one of the
    #     following classes:
    #
    #     * `Aws::Credentials` - Used for configuring static, non-refreshing
    #       credentials.
    #
    #     * `Aws::SharedCredentials` - Used for loading static credentials from a
    #       shared file, such as `~/.aws/config`.
    #
    #     * `Aws::AssumeRoleCredentials` - Used when you need to assume a role.
    #
    #     * `Aws::AssumeRoleWebIdentityCredentials` - Used when you need to
    #       assume a role after providing credentials via the web.
    #
    #     * `Aws::SSOCredentials` - Used for loading credentials from AWS SSO using an
    #       access token generated from `aws login`.
    #
    #     * `Aws::ProcessCredentials` - Used for loading credentials from a
    #       process that outputs to stdout.
    #
    #     * `Aws::InstanceProfileCredentials` - Used for loading credentials
    #       from an EC2 IMDS on an EC2 instance.
    #
    #     * `Aws::ECSCredentials` - Used for loading credentials from
    #       instances running in ECS.
    #
    #     * `Aws::CognitoIdentityCredentials` - Used for loading credentials
    #       from the Cognito Identity service.
    #
    #     When `:credentials` are not configured directly, the following
    #     locations will be searched for credentials:
    #
    #     * `Aws.config[:credentials]`
    #     * The `:access_key_id`, `:secret_access_key`, `:session_token`, and
    #       `:account_id` options.
    #     * ENV['AWS_ACCESS_KEY_ID'], ENV['AWS_SECRET_ACCESS_KEY'],
    #       ENV['AWS_SESSION_TOKEN'], and ENV['AWS_ACCOUNT_ID']
    #     * `~/.aws/credentials`
    #     * `~/.aws/config`
    #     * EC2/ECS IMDS instance profile - When used by default, the timeouts
    #       are very aggressive. Construct and pass an instance of
    #       `Aws::InstanceProfileCredentials` or `Aws::ECSCredentials` to
    #       enable retries and extended timeouts. Instance profile credential
    #       fetching can be disabled by setting ENV['AWS_EC2_METADATA_DISABLED']
    #       to true.
    #
    #   @option options [required, String] :region
    #     The AWS region to connect to.  The configured `:region` is
    #     used to determine the service `:endpoint`. When not passed,
    #     a default `:region` is searched for in the following locations:
    #
    #     * `Aws.config[:region]`
    #     * `ENV['AWS_REGION']`
    #     * `ENV['AMAZON_REGION']`
    #     * `ENV['AWS_DEFAULT_REGION']`
    #     * `~/.aws/credentials`
    #     * `~/.aws/config`
    #
    #   @option options [String] :access_key_id
    #
    #   @option options [String] :account_id
    #
    #   @option options [Boolean] :active_endpoint_cache (false)
    #     When set to `true`, a thread polling for endpoints will be running in
    #     the background every 60 secs (default). Defaults to `false`.
    #
    #   @option options [Boolean] :adaptive_retry_wait_to_fill (true)
    #     Used only in `adaptive` retry mode.  When true, the request will sleep
    #     until there is sufficent client side capacity to retry the request.
    #     When false, the request will raise a `RetryCapacityNotAvailableError` and will
    #     not retry instead of sleeping.
    #
    #   @option options [Boolean] :client_side_monitoring (false)
    #     When `true`, client-side metrics will be collected for all API requests from
    #     this client.
    #
    #   @option options [String] :client_side_monitoring_client_id ("")
    #     Allows you to provide an identifier for this client which will be attached to
    #     all generated client side metrics. Defaults to an empty string.
    #
    #   @option options [String] :client_side_monitoring_host ("127.0.0.1")
    #     Allows you to specify the DNS hostname or IPv4 or IPv6 address that the client
    #     side monitoring agent is running on, where client metrics will be published via UDP.
    #
    #   @option options [Integer] :client_side_monitoring_port (31000)
    #     Required for publishing client metrics. The port that the client side monitoring
    #     agent is running on, where client metrics will be published via UDP.
    #
    #   @option options [Aws::ClientSideMonitoring::Publisher] :client_side_monitoring_publisher (Aws::ClientSideMonitoring::Publisher)
    #     Allows you to provide a custom client-side monitoring publisher class. By default,
    #     will use the Client Side Monitoring Agent Publisher.
    #
    #   @option options [Boolean] :convert_params (true)
    #     When `true`, an attempt is made to coerce request parameters into
    #     the required types.
    #
    #   @option options [Boolean] :correct_clock_skew (true)
    #     Used only in `standard` and adaptive retry modes. Specifies whether to apply
    #     a clock skew correction and retry requests with skewed client clocks.
    #
    #   @option options [String] :defaults_mode ("legacy")
    #     See {Aws::DefaultsModeConfiguration} for a list of the
    #     accepted modes and the configuration defaults that are included.
    #
    #   @option options [Boolean] :disable_host_prefix_injection (false)
    #     Set to true to disable SDK automatically adding host prefix
    #     to default service endpoint when available.
    #
    #   @option options [Boolean] :disable_request_compression (false)
    #     When set to 'true' the request body will not be compressed
    #     for supported operations.
    #
    #   @option options [String, URI::HTTPS, URI::HTTP] :endpoint
    #     Normally you should not configure the `:endpoint` option
    #     directly. This is normally constructed from the `:region`
    #     option. Configuring `:endpoint` is normally reserved for
    #     connecting to test or custom endpoints. The endpoint should
    #     be a URI formatted like:
    #
    #         'http://example.com'
    #         'https://example.com'
    #         'http://example.com:123'
    #
    #   @option options [Integer] :endpoint_cache_max_entries (1000)
    #     Used for the maximum size limit of the LRU cache storing endpoints data
    #     for endpoint discovery enabled operations. Defaults to 1000.
    #
    #   @option options [Integer] :endpoint_cache_max_threads (10)
    #     Used for the maximum threads in use for polling endpoints to be cached, defaults to 10.
    #
    #   @option options [Integer] :endpoint_cache_poll_interval (60)
    #     When :endpoint_discovery and :active_endpoint_cache is enabled,
    #     Use this option to config the time interval in seconds for making
    #     requests fetching endpoints information. Defaults to 60 sec.
    #
    #   @option options [Boolean] :endpoint_discovery (false)
    #     When set to `true`, endpoint discovery will be enabled for operations when available.
    #
    #   @option options [Boolean] :ignore_configured_endpoint_urls
    #     Setting to true disables use of endpoint URLs provided via environment
    #     variables and the shared configuration file.
    #
    #   @option options [Aws::Log::Formatter] :log_formatter (Aws::Log::Formatter.default)
    #     The log formatter.
    #
    #   @option options [Symbol] :log_level (:info)
    #     The log level to send messages to the `:logger` at.
    #
    #   @option options [Logger] :logger
    #     The Logger instance to send log messages to.  If this option
    #     is not set, logging will be disabled.
    #
    #   @option options [Integer] :max_attempts (3)
    #     An integer representing the maximum number attempts that will be made for
    #     a single request, including the initial attempt.  For example,
    #     setting this value to 5 will result in a request being retried up to
    #     4 times. Used in `standard` and `adaptive` retry modes.
    #
    #   @option options [String] :profile ("default")
    #     Used when loading credentials from the shared credentials file
    #     at HOME/.aws/credentials.  When not specified, 'default' is used.
    #
    #   @option options [String] :request_checksum_calculation ("when_supported")
    #     Determines when a checksum will be calculated for request payloads. Values are:
    #
    #     * `when_supported` - (default) When set, a checksum will be
    #       calculated for all request payloads of operations modeled with the
    #       `httpChecksum` trait where `requestChecksumRequired` is `true` and/or a
    #       `requestAlgorithmMember` is modeled.
    #     * `when_required` - When set, a checksum will only be calculated for
    #       request payloads of operations modeled with the  `httpChecksum` trait where
    #       `requestChecksumRequired` is `true` or where a `requestAlgorithmMember`
    #       is modeled and supplied.
    #
    #   @option options [Integer] :request_min_compression_size_bytes (10240)
    #     The minimum size in bytes that triggers compression for request
    #     bodies. The value must be non-negative integer value between 0
    #     and 10485780 bytes inclusive.
    #
    #   @option options [String] :response_checksum_validation ("when_supported")
    #     Determines when checksum validation will be performed on response payloads. Values are:
    #
    #     * `when_supported` - (default) When set, checksum validation is performed on all
    #       response payloads of operations modeled with the `httpChecksum` trait where
    #       `responseAlgorithms` is modeled, except when no modeled checksum algorithms
    #       are supported.
    #     * `when_required` - When set, checksum validation is not performed on
    #       response payloads of operations unless the checksum algorithm is supported and
    #       the `requestValidationModeMember` member is set to `ENABLED`.
    #
    #   @option options [Proc] :retry_backoff
    #     A proc or lambda used for backoff. Defaults to 2**retries * retry_base_delay.
    #     This option is only used in the `legacy` retry mode.
    #
    #   @option options [Float] :retry_base_delay (0.3)
    #     The base delay in seconds used by the default backoff function. This option
    #     is only used in the `legacy` retry mode.
    #
    #   @option options [Symbol] :retry_jitter (:none)
    #     A delay randomiser function used by the default backoff function.
    #     Some predefined functions can be referenced by name - :none, :equal, :full,
    #     otherwise a Proc that takes and returns a number. This option is only used
    #     in the `legacy` retry mode.
    #
    #     @see https://www.awsarchitectureblog.com/2015/03/backoff.html
    #
    #   @option options [Integer] :retry_limit (3)
    #     The maximum number of times to retry failed requests.  Only
    #     ~ 500 level server errors and certain ~ 400 level client errors
    #     are retried.  Generally, these are throttling errors, data
    #     checksum errors, networking errors, timeout errors, auth errors,
    #     endpoint discovery, and errors from expired credentials.
    #     This option is only used in the `legacy` retry mode.
    #
    #   @option options [Integer] :retry_max_delay (0)
    #     The maximum number of seconds to delay between retries (0 for no limit)
    #     used by the default backoff function. This option is only used in the
    #     `legacy` retry mode.
    #
    #   @option options [String] :retry_mode ("legacy")
    #     Specifies which retry algorithm to use. Values are:
    #
    #     * `legacy` - The pre-existing retry behavior.  This is default value if
    #       no retry mode is provided.
    #
    #     * `standard` - A standardized set of retry rules across the AWS SDKs.
    #       This includes support for retry quotas, which limit the number of
    #       unsuccessful retries a client can make.
    #
    #     * `adaptive` - An experimental retry mode that includes all the
    #       functionality of `standard` mode along with automatic client side
    #       throttling.  This is a provisional mode that may change behavior
    #       in the future.
    #
    #   @option options [String] :sdk_ua_app_id
    #     A unique and opaque application ID that is appended to the
    #     User-Agent header as app/sdk_ua_app_id. It should have a
    #     maximum length of 50. This variable is sourced from environment
    #     variable AWS_SDK_UA_APP_ID or the shared config profile attribute sdk_ua_app_id.
    #
    #   @option options [String] :secret_access_key
    #
    #   @option options [String] :session_token
    #
    #   @option options [Array] :sigv4a_signing_region_set
    #     A list of regions that should be signed with SigV4a signing. When
    #     not passed, a default `:sigv4a_signing_region_set` is searched for
    #     in the following locations:
    #
    #     * `Aws.config[:sigv4a_signing_region_set]`
    #     * `ENV['AWS_SIGV4A_SIGNING_REGION_SET']`
    #     * `~/.aws/config`
    #
    #   @option options [Boolean] :stub_responses (false)
    #     Causes the client to return stubbed responses. By default
    #     fake responses are generated and returned. You can specify
    #     the response data to return or errors to raise by calling
    #     {ClientStubs#stub_responses}. See {ClientStubs} for more information.
    #
    #     ** Please note ** When response stubbing is enabled, no HTTP
    #     requests are made, and retries are disabled.
    #
    #   @option options [Aws::Telemetry::TelemetryProviderBase] :telemetry_provider (Aws::Telemetry::NoOpTelemetryProvider)
    #     Allows you to provide a telemetry provider, which is used to
    #     emit telemetry data. By default, uses `NoOpTelemetryProvider` which
    #     will not record or emit any telemetry data. The SDK supports the
    #     following telemetry providers:
    #
    #     * OpenTelemetry (OTel) - To use the OTel provider, install and require the
    #     `opentelemetry-sdk` gem and then, pass in an instance of a
    #     `Aws::Telemetry::OTelProvider` for telemetry provider.
    #
    #   @option options [Aws::TokenProvider] :token_provider
    #     A Bearer Token Provider. This can be an instance of any one of the
    #     following classes:
    #
    #     * `Aws::StaticTokenProvider` - Used for configuring static, non-refreshing
    #       tokens.
    #
    #     * `Aws::SSOTokenProvider` - Used for loading tokens from AWS SSO using an
    #       access token generated from `aws login`.
    #
    #     When `:token_provider` is not configured directly, the `Aws::TokenProviderChain`
    #     will be used to search for tokens configured for your profile in shared configuration files.
    #
    #   @option options [Boolean] :use_dualstack_endpoint
    #     When set to `true`, dualstack enabled endpoints (with `.aws` TLD)
    #     will be used if available.
    #
    #   @option options [Boolean] :use_fips_endpoint
    #     When set to `true`, fips compatible endpoints will be used if available.
    #     When a `fips` region is used, the region is normalized and this config
    #     is set to `true`.
    #
    #   @option options [Boolean] :validate_params (true)
    #     When `true`, request parameters are validated before
    #     sending the request.
    #
    #   @option options [Aws::EC2::EndpointProvider] :endpoint_provider
    #     The endpoint provider used to resolve endpoints. Any object that responds to
    #     `#resolve_endpoint(parameters)` where `parameters` is a Struct similar to
    #     `Aws::EC2::EndpointParameters`.
    #
    #   @option options [Float] :http_continue_timeout (1)
    #     The number of seconds to wait for a 100-continue response before sending the
    #     request body.  This option has no effect unless the request has "Expect"
    #     header set to "100-continue".  Defaults to `nil` which  disables this
    #     behaviour.  This value can safely be set per request on the session.
    #
    #   @option options [Float] :http_idle_timeout (5)
    #     The number of seconds a connection is allowed to sit idle before it
    #     is considered stale.  Stale connections are closed and removed from the
    #     pool before making a request.
    #
    #   @option options [Float] :http_open_timeout (15)
    #     The default number of seconds to wait for response data.
    #     This value can safely be set per-request on the session.
    #
    #   @option options [URI::HTTP,String] :http_proxy
    #     A proxy to send requests through.  Formatted like 'http://proxy.com:123'.
    #
    #   @option options [Float] :http_read_timeout (60)
    #     The default number of seconds to wait for response data.
    #     This value can safely be set per-request on the session.
    #
    #   @option options [Boolean] :http_wire_trace (false)
    #     When `true`,  HTTP debug output will be sent to the `:logger`.
    #
    #   @option options [Proc] :on_chunk_received
    #     When a Proc object is provided, it will be used as callback when each chunk
    #     of the response body is received. It provides three arguments: the chunk,
    #     the number of bytes received, and the total number of
    #     bytes in the response (or nil if the server did not send a `content-length`).
    #
    #   @option options [Proc] :on_chunk_sent
    #     When a Proc object is provided, it will be used as callback when each chunk
    #     of the request body is sent. It provides three arguments: the chunk,
    #     the number of bytes read from the body, and the total number of
    #     bytes in the body.
    #
    #   @option options [Boolean] :raise_response_errors (true)
    #     When `true`, response errors are raised.
    #
    #   @option options [String] :ssl_ca_bundle
    #     Full path to the SSL certificate authority bundle file that should be used when
    #     verifying peer certificates.  If you do not pass `:ssl_ca_bundle` or
    #     `:ssl_ca_directory` the the system default will be used if available.
    #
    #   @option options [String] :ssl_ca_directory
    #     Full path of the directory that contains the unbundled SSL certificate
    #     authority files for verifying peer certificates.  If you do
    #     not pass `:ssl_ca_bundle` or `:ssl_ca_directory` the the system
    #     default will be used if available.
    #
    #   @option options [String] :ssl_ca_store
    #     Sets the X509::Store to verify peer certificate.
    #
    #   @option options [OpenSSL::X509::Certificate] :ssl_cert
    #     Sets a client certificate when creating http connections.
    #
    #   @option options [OpenSSL::PKey] :ssl_key
    #     Sets a client key when creating http connections.
    #
    #   @option options [Float] :ssl_timeout
    #     Sets the SSL timeout in seconds
    #
    #   @option options [Boolean] :ssl_verify_peer (true)
    #     When `true`, SSL peer certificates are verified when establishing a connection.
    #
    def initialize(*args)
      super
    end

    # @!group API Operations

    # Accepts an Elastic IP address transfer. For more information, see
    # [Accept a transferred Elastic IP address][1] in the *Amazon VPC User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/vpc-eips.html#using-instance-addressing-eips-transfer-accept
    #
    # @option params [required, String] :address
    #   The Elastic IP address you are accepting for transfer.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   `tag`:&lt;key&gt; - The key/value combination of a tag assigned to the
    #   resource. Use the tag key in the filter name and the tag value as the
    #   filter value. For example, to find all resources that have a tag with
    #   the key `Owner` and the value `TeamA`, specify `tag:Owner` for the
    #   filter name and `TeamA` for the filter value.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AcceptAddressTransferResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AcceptAddressTransferResult#address_transfer #address_transfer} => Types::AddressTransfer
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.accept_address_transfer({
    #     address: "String", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.address_transfer.public_ip #=> String
    #   resp.address_transfer.allocation_id #=> String
    #   resp.address_transfer.transfer_account_id #=> String
    #   resp.address_transfer.transfer_offer_expiration_timestamp #=> Time
    #   resp.address_transfer.transfer_offer_accepted_timestamp #=> Time
    #   resp.address_transfer.address_transfer_status #=> String, one of "pending", "disabled", "accepted"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AcceptAddressTransfer AWS API Documentation
    #
    # @overload accept_address_transfer(params = {})
    # @param [Hash] params ({})
    def accept_address_transfer(params = {}, options = {})
      req = build_request(:accept_address_transfer, params)
      req.send_request(options)
    end

    # Accepts a request to assign billing of the available capacity of a
    # shared Capacity Reservation to your account. For more information, see
    # [ Billing assignment for shared Amazon EC2 Capacity Reservations][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/assign-billing.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :capacity_reservation_id
    #   The ID of the Capacity Reservation for which to accept the request.
    #
    # @return [Types::AcceptCapacityReservationBillingOwnershipResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AcceptCapacityReservationBillingOwnershipResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.accept_capacity_reservation_billing_ownership({
    #     dry_run: false,
    #     capacity_reservation_id: "CapacityReservationId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AcceptCapacityReservationBillingOwnership AWS API Documentation
    #
    # @overload accept_capacity_reservation_billing_ownership(params = {})
    # @param [Hash] params ({})
    def accept_capacity_reservation_billing_ownership(params = {}, options = {})
      req = build_request(:accept_capacity_reservation_billing_ownership, params)
      req.send_request(options)
    end

    # Accepts the Convertible Reserved Instance exchange quote described in
    # the GetReservedInstancesExchangeQuote call.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :reserved_instance_ids
    #   The IDs of the Convertible Reserved Instances to exchange for another
    #   Convertible Reserved Instance of the same or higher value.
    #
    # @option params [Array<Types::TargetConfigurationRequest>] :target_configurations
    #   The configuration of the target Convertible Reserved Instance to
    #   exchange for your current Convertible Reserved Instances.
    #
    # @return [Types::AcceptReservedInstancesExchangeQuoteResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AcceptReservedInstancesExchangeQuoteResult#exchange_id #exchange_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.accept_reserved_instances_exchange_quote({
    #     dry_run: false,
    #     reserved_instance_ids: ["ReservationId"], # required
    #     target_configurations: [
    #       {
    #         instance_count: 1,
    #         offering_id: "ReservedInstancesOfferingId", # required
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.exchange_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AcceptReservedInstancesExchangeQuote AWS API Documentation
    #
    # @overload accept_reserved_instances_exchange_quote(params = {})
    # @param [Hash] params ({})
    def accept_reserved_instances_exchange_quote(params = {}, options = {})
      req = build_request(:accept_reserved_instances_exchange_quote, params)
      req.send_request(options)
    end

    # Accepts a request to associate subnets with a transit gateway
    # multicast domain.
    #
    # @option params [String] :transit_gateway_multicast_domain_id
    #   The ID of the transit gateway multicast domain.
    #
    # @option params [String] :transit_gateway_attachment_id
    #   The ID of the transit gateway attachment.
    #
    # @option params [Array<String>] :subnet_ids
    #   The IDs of the subnets to associate with the transit gateway multicast
    #   domain.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AcceptTransitGatewayMulticastDomainAssociationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AcceptTransitGatewayMulticastDomainAssociationsResult#associations #associations} => Types::TransitGatewayMulticastDomainAssociations
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.accept_transit_gateway_multicast_domain_associations({
    #     transit_gateway_multicast_domain_id: "TransitGatewayMulticastDomainId",
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId",
    #     subnet_ids: ["String"],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.associations.transit_gateway_multicast_domain_id #=> String
    #   resp.associations.transit_gateway_attachment_id #=> String
    #   resp.associations.resource_id #=> String
    #   resp.associations.resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.associations.resource_owner_id #=> String
    #   resp.associations.subnets #=> Array
    #   resp.associations.subnets[0].subnet_id #=> String
    #   resp.associations.subnets[0].state #=> String, one of "pendingAcceptance", "associating", "associated", "disassociating", "disassociated", "rejected", "failed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AcceptTransitGatewayMulticastDomainAssociations AWS API Documentation
    #
    # @overload accept_transit_gateway_multicast_domain_associations(params = {})
    # @param [Hash] params ({})
    def accept_transit_gateway_multicast_domain_associations(params = {}, options = {})
      req = build_request(:accept_transit_gateway_multicast_domain_associations, params)
      req.send_request(options)
    end

    # Accepts a transit gateway peering attachment request. The peering
    # attachment must be in the `pendingAcceptance` state.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the transit gateway attachment.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AcceptTransitGatewayPeeringAttachmentResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AcceptTransitGatewayPeeringAttachmentResult#transit_gateway_peering_attachment #transit_gateway_peering_attachment} => Types::TransitGatewayPeeringAttachment
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.accept_transit_gateway_peering_attachment({
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_peering_attachment.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.transit_gateway_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.core_network_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.owner_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.region #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.transit_gateway_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.core_network_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.owner_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.region #=> String
    #   resp.transit_gateway_peering_attachment.options.dynamic_routing #=> String, one of "enable", "disable"
    #   resp.transit_gateway_peering_attachment.status.code #=> String
    #   resp.transit_gateway_peering_attachment.status.message #=> String
    #   resp.transit_gateway_peering_attachment.state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_peering_attachment.creation_time #=> Time
    #   resp.transit_gateway_peering_attachment.tags #=> Array
    #   resp.transit_gateway_peering_attachment.tags[0].key #=> String
    #   resp.transit_gateway_peering_attachment.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AcceptTransitGatewayPeeringAttachment AWS API Documentation
    #
    # @overload accept_transit_gateway_peering_attachment(params = {})
    # @param [Hash] params ({})
    def accept_transit_gateway_peering_attachment(params = {}, options = {})
      req = build_request(:accept_transit_gateway_peering_attachment, params)
      req.send_request(options)
    end

    # Accepts a request to attach a VPC to a transit gateway.
    #
    # The VPC attachment must be in the `pendingAcceptance` state. Use
    # DescribeTransitGatewayVpcAttachments to view your pending VPC
    # attachment requests. Use RejectTransitGatewayVpcAttachment to reject a
    # VPC attachment request.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the attachment.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AcceptTransitGatewayVpcAttachmentResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AcceptTransitGatewayVpcAttachmentResult#transit_gateway_vpc_attachment #transit_gateway_vpc_attachment} => Types::TransitGatewayVpcAttachment
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.accept_transit_gateway_vpc_attachment({
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_vpc_attachment.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_vpc_attachment.transit_gateway_id #=> String
    #   resp.transit_gateway_vpc_attachment.vpc_id #=> String
    #   resp.transit_gateway_vpc_attachment.vpc_owner_id #=> String
    #   resp.transit_gateway_vpc_attachment.state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_vpc_attachment.subnet_ids #=> Array
    #   resp.transit_gateway_vpc_attachment.subnet_ids[0] #=> String
    #   resp.transit_gateway_vpc_attachment.creation_time #=> Time
    #   resp.transit_gateway_vpc_attachment.options.dns_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.security_group_referencing_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.ipv_6_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.appliance_mode_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.tags #=> Array
    #   resp.transit_gateway_vpc_attachment.tags[0].key #=> String
    #   resp.transit_gateway_vpc_attachment.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AcceptTransitGatewayVpcAttachment AWS API Documentation
    #
    # @overload accept_transit_gateway_vpc_attachment(params = {})
    # @param [Hash] params ({})
    def accept_transit_gateway_vpc_attachment(params = {}, options = {})
      req = build_request(:accept_transit_gateway_vpc_attachment, params)
      req.send_request(options)
    end

    # Accepts connection requests to your VPC endpoint service.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :service_id
    #   The ID of the VPC endpoint service.
    #
    # @option params [required, Array<String>] :vpc_endpoint_ids
    #   The IDs of the interface VPC endpoints.
    #
    # @return [Types::AcceptVpcEndpointConnectionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AcceptVpcEndpointConnectionsResult#unsuccessful #unsuccessful} => Array&lt;Types::UnsuccessfulItem&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.accept_vpc_endpoint_connections({
    #     dry_run: false,
    #     service_id: "VpcEndpointServiceId", # required
    #     vpc_endpoint_ids: ["VpcEndpointId"], # required
    #   })
    #
    # @example Response structure
    #
    #   resp.unsuccessful #=> Array
    #   resp.unsuccessful[0].error.code #=> String
    #   resp.unsuccessful[0].error.message #=> String
    #   resp.unsuccessful[0].resource_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AcceptVpcEndpointConnections AWS API Documentation
    #
    # @overload accept_vpc_endpoint_connections(params = {})
    # @param [Hash] params ({})
    def accept_vpc_endpoint_connections(params = {}, options = {})
      req = build_request(:accept_vpc_endpoint_connections, params)
      req.send_request(options)
    end

    # Accept a VPC peering connection request. To accept a request, the VPC
    # peering connection must be in the `pending-acceptance` state, and you
    # must be the owner of the peer VPC. Use DescribeVpcPeeringConnections
    # to view your outstanding VPC peering connection requests.
    #
    # For an inter-Region VPC peering connection request, you must accept
    # the VPC peering connection in the Region of the accepter VPC.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :vpc_peering_connection_id
    #   The ID of the VPC peering connection. You must specify this parameter
    #   in the request.
    #
    # @return [Types::AcceptVpcPeeringConnectionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AcceptVpcPeeringConnectionResult#vpc_peering_connection #vpc_peering_connection} => Types::VpcPeeringConnection
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.accept_vpc_peering_connection({
    #     dry_run: false,
    #     vpc_peering_connection_id: "VpcPeeringConnectionIdWithResolver", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc_peering_connection.accepter_vpc_info.cidr_block #=> String
    #   resp.vpc_peering_connection.accepter_vpc_info.ipv_6_cidr_block_set #=> Array
    #   resp.vpc_peering_connection.accepter_vpc_info.ipv_6_cidr_block_set[0].ipv_6_cidr_block #=> String
    #   resp.vpc_peering_connection.accepter_vpc_info.cidr_block_set #=> Array
    #   resp.vpc_peering_connection.accepter_vpc_info.cidr_block_set[0].cidr_block #=> String
    #   resp.vpc_peering_connection.accepter_vpc_info.owner_id #=> String
    #   resp.vpc_peering_connection.accepter_vpc_info.peering_options.allow_dns_resolution_from_remote_vpc #=> Boolean
    #   resp.vpc_peering_connection.accepter_vpc_info.peering_options.allow_egress_from_local_classic_link_to_remote_vpc #=> Boolean
    #   resp.vpc_peering_connection.accepter_vpc_info.peering_options.allow_egress_from_local_vpc_to_remote_classic_link #=> Boolean
    #   resp.vpc_peering_connection.accepter_vpc_info.vpc_id #=> String
    #   resp.vpc_peering_connection.accepter_vpc_info.region #=> String
    #   resp.vpc_peering_connection.expiration_time #=> Time
    #   resp.vpc_peering_connection.requester_vpc_info.cidr_block #=> String
    #   resp.vpc_peering_connection.requester_vpc_info.ipv_6_cidr_block_set #=> Array
    #   resp.vpc_peering_connection.requester_vpc_info.ipv_6_cidr_block_set[0].ipv_6_cidr_block #=> String
    #   resp.vpc_peering_connection.requester_vpc_info.cidr_block_set #=> Array
    #   resp.vpc_peering_connection.requester_vpc_info.cidr_block_set[0].cidr_block #=> String
    #   resp.vpc_peering_connection.requester_vpc_info.owner_id #=> String
    #   resp.vpc_peering_connection.requester_vpc_info.peering_options.allow_dns_resolution_from_remote_vpc #=> Boolean
    #   resp.vpc_peering_connection.requester_vpc_info.peering_options.allow_egress_from_local_classic_link_to_remote_vpc #=> Boolean
    #   resp.vpc_peering_connection.requester_vpc_info.peering_options.allow_egress_from_local_vpc_to_remote_classic_link #=> Boolean
    #   resp.vpc_peering_connection.requester_vpc_info.vpc_id #=> String
    #   resp.vpc_peering_connection.requester_vpc_info.region #=> String
    #   resp.vpc_peering_connection.status.code #=> String, one of "initiating-request", "pending-acceptance", "active", "deleted", "rejected", "failed", "expired", "provisioning", "deleting"
    #   resp.vpc_peering_connection.status.message #=> String
    #   resp.vpc_peering_connection.tags #=> Array
    #   resp.vpc_peering_connection.tags[0].key #=> String
    #   resp.vpc_peering_connection.tags[0].value #=> String
    #   resp.vpc_peering_connection.vpc_peering_connection_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AcceptVpcPeeringConnection AWS API Documentation
    #
    # @overload accept_vpc_peering_connection(params = {})
    # @param [Hash] params ({})
    def accept_vpc_peering_connection(params = {}, options = {})
      req = build_request(:accept_vpc_peering_connection, params)
      req.send_request(options)
    end

    # Advertises an IPv4 or IPv6 address range that is provisioned for use
    # with your Amazon Web Services resources through bring your own IP
    # addresses (BYOIP).
    #
    # You can perform this operation at most once every 10 seconds, even if
    # you specify different address ranges each time.
    #
    # We recommend that you stop advertising the BYOIP CIDR from other
    # locations when you advertise it from Amazon Web Services. To minimize
    # down time, you can configure your Amazon Web Services resources to use
    # an address from a BYOIP CIDR before it is advertised, and then
    # simultaneously stop advertising it from the current location and start
    # advertising it through Amazon Web Services.
    #
    # It can take a few minutes before traffic to the specified addresses
    # starts routing to Amazon Web Services because of BGP propagation
    # delays.
    #
    # To stop advertising the BYOIP CIDR, use WithdrawByoipCidr.
    #
    # @option params [required, String] :cidr
    #   The address range, in CIDR notation. This must be the exact range that
    #   you provisioned. You can't advertise only a portion of the
    #   provisioned range.
    #
    # @option params [String] :asn
    #   The public 2-byte or 4-byte ASN that you want to advertise.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :network_border_group
    #   If you have [Local Zones][1] enabled, you can choose a network border
    #   group for Local Zones when you provision and advertise a BYOIPv4 CIDR.
    #   Choose the network border group carefully as the EIP and the Amazon
    #   Web Services resource it is associated with must reside in the same
    #   network border group.
    #
    #   You can provision BYOIP address ranges to and advertise them in the
    #   following Local Zone network border groups:
    #
    #   * us-east-1-dfw-2
    #
    #   * us-west-2-lax-1
    #
    #   * us-west-2-phx-2
    #
    #   <note markdown="1"> You cannot provision or advertise BYOIPv6 address ranges in Local
    #   Zones at this time.
    #
    #    </note>
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/local-zones/latest/ug/how-local-zones-work.html
    #
    # @return [Types::AdvertiseByoipCidrResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AdvertiseByoipCidrResult#byoip_cidr #byoip_cidr} => Types::ByoipCidr
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.advertise_byoip_cidr({
    #     cidr: "String", # required
    #     asn: "String",
    #     dry_run: false,
    #     network_border_group: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.byoip_cidr.cidr #=> String
    #   resp.byoip_cidr.description #=> String
    #   resp.byoip_cidr.asn_associations #=> Array
    #   resp.byoip_cidr.asn_associations[0].asn #=> String
    #   resp.byoip_cidr.asn_associations[0].cidr #=> String
    #   resp.byoip_cidr.asn_associations[0].status_message #=> String
    #   resp.byoip_cidr.asn_associations[0].state #=> String, one of "disassociated", "failed-disassociation", "failed-association", "pending-disassociation", "pending-association", "associated"
    #   resp.byoip_cidr.status_message #=> String
    #   resp.byoip_cidr.state #=> String, one of "advertised", "deprovisioned", "failed-deprovision", "failed-provision", "pending-deprovision", "pending-provision", "provisioned", "provisioned-not-publicly-advertisable"
    #   resp.byoip_cidr.network_border_group #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AdvertiseByoipCidr AWS API Documentation
    #
    # @overload advertise_byoip_cidr(params = {})
    # @param [Hash] params ({})
    def advertise_byoip_cidr(params = {}, options = {})
      req = build_request(:advertise_byoip_cidr, params)
      req.send_request(options)
    end

    # Allocates an Elastic IP address to your Amazon Web Services account.
    # After you allocate the Elastic IP address you can associate it with an
    # instance or network interface. After you release an Elastic IP
    # address, it is released to the IP address pool and can be allocated to
    # a different Amazon Web Services account.
    #
    # You can allocate an Elastic IP address from an address pool owned by
    # Amazon Web Services or from an address pool created from a public IPv4
    # address range that you have brought to Amazon Web Services for use
    # with your Amazon Web Services resources using bring your own IP
    # addresses (BYOIP). For more information, see [Bring Your Own IP
    # Addresses (BYOIP)][1] in the *Amazon EC2 User Guide*.
    #
    # If you release an Elastic IP address, you might be able to recover it.
    # You cannot recover an Elastic IP address that you released after it is
    # allocated to another Amazon Web Services account. To attempt to
    # recover an Elastic IP address that you released, specify it in this
    # operation.
    #
    # For more information, see [Elastic IP Addresses][2] in the *Amazon EC2
    # User Guide*.
    #
    # You can allocate a carrier IP address which is a public IP address
    # from a telecommunication carrier, to a network interface which resides
    # in a subnet in a Wavelength Zone (for example an EC2 instance).
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-byoip.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/elastic-ip-addresses-eip.html
    #
    # @option params [String] :domain
    #   The network (`vpc`).
    #
    # @option params [String] :address
    #   The Elastic IP address to recover or an IPv4 address from an address
    #   pool.
    #
    # @option params [String] :public_ipv_4_pool
    #   The ID of an address pool that you own. Use this parameter to let
    #   Amazon EC2 select an address from the address pool. To specify a
    #   specific address from the address pool, use the `Address` parameter
    #   instead.
    #
    # @option params [String] :network_border_group
    #   A unique set of Availability Zones, Local Zones, or Wavelength Zones
    #   from which Amazon Web Services advertises IP addresses. Use this
    #   parameter to limit the IP address to this location. IP addresses
    #   cannot move between network border groups.
    #
    # @option params [String] :customer_owned_ipv_4_pool
    #   The ID of a customer-owned address pool. Use this parameter to let
    #   Amazon EC2 select an address from the address pool. Alternatively,
    #   specify a specific address from the address pool.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the Elastic IP address.
    #
    # @option params [String] :ipam_pool_id
    #   The ID of an IPAM pool which has an Amazon-provided or BYOIP public
    #   IPv4 CIDR provisioned to it. For more information, see [Allocate
    #   sequential Elastic IP addresses from an IPAM pool][1] in the *Amazon
    #   VPC IPAM User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/tutorials-eip-pool.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AllocateAddressResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AllocateAddressResult#allocation_id #allocation_id} => String
    #   * {Types::AllocateAddressResult#public_ipv_4_pool #public_ipv_4_pool} => String
    #   * {Types::AllocateAddressResult#network_border_group #network_border_group} => String
    #   * {Types::AllocateAddressResult#domain #domain} => String
    #   * {Types::AllocateAddressResult#customer_owned_ip #customer_owned_ip} => String
    #   * {Types::AllocateAddressResult#customer_owned_ipv_4_pool #customer_owned_ipv_4_pool} => String
    #   * {Types::AllocateAddressResult#carrier_ip #carrier_ip} => String
    #   * {Types::AllocateAddressResult#public_ip #public_ip} => String
    #
    #
    # @example Example: To allocate an Elastic IP address
    #
    #   # This example allocates an Elastic IP address.
    #
    #   resp = client.allocate_address({
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     allocation_id: "eipalloc-64d5890a", 
    #     domain: "vpc", 
    #     network_border_group: "us-east-1", 
    #     public_ip: "203.0.113.0", 
    #     public_ipv_4_pool: "amazon", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.allocate_address({
    #     domain: "vpc", # accepts vpc, standard
    #     address: "PublicIpAddress",
    #     public_ipv_4_pool: "Ipv4PoolEc2Id",
    #     network_border_group: "String",
    #     customer_owned_ipv_4_pool: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     ipam_pool_id: "IpamPoolId",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.allocation_id #=> String
    #   resp.public_ipv_4_pool #=> String
    #   resp.network_border_group #=> String
    #   resp.domain #=> String, one of "vpc", "standard"
    #   resp.customer_owned_ip #=> String
    #   resp.customer_owned_ipv_4_pool #=> String
    #   resp.carrier_ip #=> String
    #   resp.public_ip #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AllocateAddress AWS API Documentation
    #
    # @overload allocate_address(params = {})
    # @param [Hash] params ({})
    def allocate_address(params = {}, options = {})
      req = build_request(:allocate_address, params)
      req.send_request(options)
    end

    # Allocates a Dedicated Host to your account. At a minimum, specify the
    # supported instance type or instance family, the Availability Zone in
    # which to allocate the host, and the number of hosts to allocate.
    #
    # @option params [String] :instance_family
    #   Specifies the instance family to be supported by the Dedicated Hosts.
    #   If you specify an instance family, the Dedicated Hosts support
    #   multiple instance types within that instance family.
    #
    #   If you want the Dedicated Hosts to support a specific instance type
    #   only, omit this parameter and specify **InstanceType** instead. You
    #   cannot specify **InstanceFamily** and **InstanceType** in the same
    #   request.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the Dedicated Host during creation.
    #
    # @option params [String] :host_recovery
    #   Indicates whether to enable or disable host recovery for the Dedicated
    #   Host. Host recovery is disabled by default. For more information, see
    #   [ Host recovery][1] in the *Amazon EC2 User Guide*.
    #
    #   Default: `off`
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-hosts-recovery.html
    #
    # @option params [String] :outpost_arn
    #   The Amazon Resource Name (ARN) of the Amazon Web Services Outpost on
    #   which to allocate the Dedicated Host. If you specify **OutpostArn**,
    #   you can optionally specify **AssetIds**.
    #
    #   If you are allocating the Dedicated Host in a Region, omit this
    #   parameter.
    #
    # @option params [String] :host_maintenance
    #   Indicates whether to enable or disable host maintenance for the
    #   Dedicated Host. For more information, see [Host maintenance][1] in the
    #   *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-hosts-maintenance.html
    #
    # @option params [Array<String>] :asset_ids
    #   The IDs of the Outpost hardware assets on which to allocate the
    #   Dedicated Hosts. Targeting specific hardware assets on an Outpost can
    #   help to minimize latency between your workloads. This parameter is
    #   supported only if you specify **OutpostArn**. If you are allocating
    #   the Dedicated Hosts in a Region, omit this parameter.
    #
    #   * If you specify this parameter, you can omit **Quantity**. In this
    #     case, Amazon EC2 allocates a Dedicated Host on each specified
    #     hardware asset.
    #
    #   * If you specify both **AssetIds** and **Quantity**, then the value
    #     for **Quantity** must be equal to the number of asset IDs specified.
    #
    # @option params [String] :auto_placement
    #   Indicates whether the host accepts any untargeted instance launches
    #   that match its instance type configuration, or if it only accepts Host
    #   tenancy instance launches that specify its unique host ID. For more
    #   information, see [ Understanding auto-placement and affinity][1] in
    #   the *Amazon EC2 User Guide*.
    #
    #   Default: `off`
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/how-dedicated-hosts-work.html#dedicated-hosts-understanding
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   Idempotency][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [String] :instance_type
    #   Specifies the instance type to be supported by the Dedicated Hosts. If
    #   you specify an instance type, the Dedicated Hosts support instances of
    #   the specified instance type only.
    #
    #   If you want the Dedicated Hosts to support multiple instance types in
    #   a specific instance family, omit this parameter and specify
    #   **InstanceFamily** instead. You cannot specify **InstanceType** and
    #   **InstanceFamily** in the same request.
    #
    # @option params [Integer] :quantity
    #   The number of Dedicated Hosts to allocate to your account with these
    #   parameters. If you are allocating the Dedicated Hosts on an Outpost,
    #   and you specify **AssetIds**, you can omit this parameter. In this
    #   case, Amazon EC2 allocates a Dedicated Host on each specified hardware
    #   asset. If you specify both **AssetIds** and **Quantity**, then the
    #   value that you specify for **Quantity** must be equal to the number of
    #   asset IDs specified.
    #
    # @option params [required, String] :availability_zone
    #   The Availability Zone in which to allocate the Dedicated Host.
    #
    # @return [Types::AllocateHostsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AllocateHostsResult#host_ids #host_ids} => Array&lt;String&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.allocate_hosts({
    #     instance_family: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     host_recovery: "on", # accepts on, off
    #     outpost_arn: "String",
    #     host_maintenance: "on", # accepts on, off
    #     asset_ids: ["AssetId"],
    #     auto_placement: "on", # accepts on, off
    #     client_token: "String",
    #     instance_type: "String",
    #     quantity: 1,
    #     availability_zone: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.host_ids #=> Array
    #   resp.host_ids[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AllocateHosts AWS API Documentation
    #
    # @overload allocate_hosts(params = {})
    # @param [Hash] params ({})
    def allocate_hosts(params = {}, options = {})
      req = build_request(:allocate_hosts, params)
      req.send_request(options)
    end

    # Allocate a CIDR from an IPAM pool. The Region you use should be the
    # IPAM pool locale. The locale is the Amazon Web Services Region where
    # this IPAM pool is available for allocations.
    #
    # In IPAM, an allocation is a CIDR assignment from an IPAM pool to
    # another IPAM pool or to a resource. For more information, see
    # [Allocate CIDRs][1] in the *Amazon VPC IPAM User Guide*.
    #
    # <note markdown="1"> This action creates an allocation with strong consistency. The
    # returned CIDR will not overlap with any other allocations from the
    # same pool.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/allocate-cidrs-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_pool_id
    #   The ID of the IPAM pool from which you would like to allocate a CIDR.
    #
    # @option params [String] :cidr
    #   The CIDR you would like to allocate from the IPAM pool. Note the
    #   following:
    #
    #   * If there is no DefaultNetmaskLength allocation rule set on the pool,
    #     you must specify either the NetmaskLength or the CIDR.
    #
    #   * If the DefaultNetmaskLength allocation rule is set on the pool, you
    #     can specify either the NetmaskLength or the CIDR and the
    #     DefaultNetmaskLength allocation rule will be ignored.
    #
    #   Possible values: Any available IPv4 or IPv6 CIDR.
    #
    # @option params [Integer] :netmask_length
    #   The netmask length of the CIDR you would like to allocate from the
    #   IPAM pool. Note the following:
    #
    #   * If there is no DefaultNetmaskLength allocation rule set on the pool,
    #     you must specify either the NetmaskLength or the CIDR.
    #
    #   * If the DefaultNetmaskLength allocation rule is set on the pool, you
    #     can specify either the NetmaskLength or the CIDR and the
    #     DefaultNetmaskLength allocation rule will be ignored.
    #
    #   Possible netmask lengths for IPv4 addresses are 0 - 32. Possible
    #   netmask lengths for IPv6 addresses are 0 - 128.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [String] :description
    #   A description for the allocation.
    #
    # @option params [Boolean] :preview_next_cidr
    #   A preview of the next available CIDR in a pool.
    #
    # @option params [Array<String>] :allowed_cidrs
    #   Include a particular CIDR range that can be returned by the pool.
    #   Allowed CIDRs are only allowed if using netmask length for allocation.
    #
    # @option params [Array<String>] :disallowed_cidrs
    #   Exclude a particular CIDR range from being returned by the pool.
    #   Disallowed CIDRs are only allowed if using netmask length for
    #   allocation.
    #
    # @return [Types::AllocateIpamPoolCidrResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AllocateIpamPoolCidrResult#ipam_pool_allocation #ipam_pool_allocation} => Types::IpamPoolAllocation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.allocate_ipam_pool_cidr({
    #     dry_run: false,
    #     ipam_pool_id: "IpamPoolId", # required
    #     cidr: "String",
    #     netmask_length: 1,
    #     client_token: "String",
    #     description: "String",
    #     preview_next_cidr: false,
    #     allowed_cidrs: ["String"],
    #     disallowed_cidrs: ["String"],
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_pool_allocation.cidr #=> String
    #   resp.ipam_pool_allocation.ipam_pool_allocation_id #=> String
    #   resp.ipam_pool_allocation.description #=> String
    #   resp.ipam_pool_allocation.resource_id #=> String
    #   resp.ipam_pool_allocation.resource_type #=> String, one of "ipam-pool", "vpc", "ec2-public-ipv4-pool", "custom", "subnet", "eip"
    #   resp.ipam_pool_allocation.resource_region #=> String
    #   resp.ipam_pool_allocation.resource_owner #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AllocateIpamPoolCidr AWS API Documentation
    #
    # @overload allocate_ipam_pool_cidr(params = {})
    # @param [Hash] params ({})
    def allocate_ipam_pool_cidr(params = {}, options = {})
      req = build_request(:allocate_ipam_pool_cidr, params)
      req.send_request(options)
    end

    # Applies a security group to the association between the target network
    # and the Client VPN endpoint. This action replaces the existing
    # security groups with the specified security groups.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC in which the associated target network is located.
    #
    # @option params [required, Array<String>] :security_group_ids
    #   The IDs of the security groups to apply to the associated target
    #   network. Up to 5 security groups can be applied to an associated
    #   target network.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ApplySecurityGroupsToClientVpnTargetNetworkResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ApplySecurityGroupsToClientVpnTargetNetworkResult#security_group_ids #security_group_ids} => Array&lt;String&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.apply_security_groups_to_client_vpn_target_network({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     vpc_id: "VpcId", # required
    #     security_group_ids: ["SecurityGroupId"], # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.security_group_ids #=> Array
    #   resp.security_group_ids[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ApplySecurityGroupsToClientVpnTargetNetwork AWS API Documentation
    #
    # @overload apply_security_groups_to_client_vpn_target_network(params = {})
    # @param [Hash] params ({})
    def apply_security_groups_to_client_vpn_target_network(params = {}, options = {})
      req = build_request(:apply_security_groups_to_client_vpn_target_network, params)
      req.send_request(options)
    end

    # Assigns the specified IPv6 addresses to the specified network
    # interface. You can specify specific IPv6 addresses, or you can specify
    # the number of IPv6 addresses to be automatically assigned from the
    # subnet's IPv6 CIDR block range. You can assign as many IPv6 addresses
    # to a network interface as you can assign private IPv4 addresses, and
    # the limit varies by instance type.
    #
    # You must specify either the IPv6 addresses or the IPv6 address count
    # in the request.
    #
    # You can optionally use Prefix Delegation on the network interface. You
    # must specify either the IPV6 Prefix Delegation prefixes, or the IPv6
    # Prefix Delegation count. For information, see [ Assigning prefixes to
    # network interfaces][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-prefix-eni.html
    #
    # @option params [Integer] :ipv_6_prefix_count
    #   The number of IPv6 prefixes that Amazon Web Services automatically
    #   assigns to the network interface. You cannot use this option if you
    #   use the `Ipv6Prefixes` option.
    #
    # @option params [Array<String>] :ipv_6_prefixes
    #   One or more IPv6 prefixes assigned to the network interface. You
    #   can't use this option if you use the `Ipv6PrefixCount` option.
    #
    # @option params [required, String] :network_interface_id
    #   The ID of the network interface.
    #
    # @option params [Array<String>] :ipv_6_addresses
    #   The IPv6 addresses to be assigned to the network interface. You can't
    #   use this option if you're specifying a number of IPv6 addresses.
    #
    # @option params [Integer] :ipv_6_address_count
    #   The number of additional IPv6 addresses to assign to the network
    #   interface. The specified number of IPv6 addresses are assigned in
    #   addition to the existing IPv6 addresses that are already assigned to
    #   the network interface. Amazon EC2 automatically selects the IPv6
    #   addresses from the subnet range. You can't use this option if
    #   specifying specific IPv6 addresses.
    #
    # @return [Types::AssignIpv6AddressesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssignIpv6AddressesResult#assigned_ipv_6_addresses #assigned_ipv_6_addresses} => Array&lt;String&gt;
    #   * {Types::AssignIpv6AddressesResult#assigned_ipv_6_prefixes #assigned_ipv_6_prefixes} => Array&lt;String&gt;
    #   * {Types::AssignIpv6AddressesResult#network_interface_id #network_interface_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.assign_ipv_6_addresses({
    #     ipv_6_prefix_count: 1,
    #     ipv_6_prefixes: ["String"],
    #     network_interface_id: "NetworkInterfaceId", # required
    #     ipv_6_addresses: ["String"],
    #     ipv_6_address_count: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.assigned_ipv_6_addresses #=> Array
    #   resp.assigned_ipv_6_addresses[0] #=> String
    #   resp.assigned_ipv_6_prefixes #=> Array
    #   resp.assigned_ipv_6_prefixes[0] #=> String
    #   resp.network_interface_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssignIpv6Addresses AWS API Documentation
    #
    # @overload assign_ipv_6_addresses(params = {})
    # @param [Hash] params ({})
    def assign_ipv_6_addresses(params = {}, options = {})
      req = build_request(:assign_ipv_6_addresses, params)
      req.send_request(options)
    end

    # Assigns the specified secondary private IP addresses to the specified
    # network interface.
    #
    # You can specify specific secondary IP addresses, or you can specify
    # the number of secondary IP addresses to be automatically assigned from
    # the subnet's CIDR block range. The number of secondary IP addresses
    # that you can assign to an instance varies by instance type. For more
    # information about Elastic IP addresses, see [Elastic IP Addresses][1]
    # in the *Amazon EC2 User Guide*.
    #
    # When you move a secondary private IP address to another network
    # interface, any Elastic IP address that is associated with the IP
    # address is also moved.
    #
    # Remapping an IP address is an asynchronous operation. When you move an
    # IP address from one network interface to another, check
    # `network/interfaces/macs/mac/local-ipv4s` in the instance metadata to
    # confirm that the remapping is complete.
    #
    # You must specify either the IP addresses or the IP address count in
    # the request.
    #
    # You can optionally use Prefix Delegation on the network interface. You
    # must specify either the IPv4 Prefix Delegation prefixes, or the IPv4
    # Prefix Delegation count. For information, see [ Assigning prefixes to
    # network interfaces][2] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/elastic-ip-addresses-eip.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-prefix-eni.html
    #
    # @option params [Array<String>] :ipv_4_prefixes
    #   One or more IPv4 prefixes assigned to the network interface. You
    #   can't use this option if you use the `Ipv4PrefixCount` option.
    #
    # @option params [Integer] :ipv_4_prefix_count
    #   The number of IPv4 prefixes that Amazon Web Services automatically
    #   assigns to the network interface. You can't use this option if you
    #   use the `Ipv4 Prefixes` option.
    #
    # @option params [required, String] :network_interface_id
    #   The ID of the network interface.
    #
    # @option params [Array<String>] :private_ip_addresses
    #   The IP addresses to be assigned as a secondary private IP address to
    #   the network interface. You can't specify this parameter when also
    #   specifying a number of secondary IP addresses.
    #
    #   If you don't specify an IP address, Amazon EC2 automatically selects
    #   an IP address within the subnet range.
    #
    # @option params [Integer] :secondary_private_ip_address_count
    #   The number of secondary IP addresses to assign to the network
    #   interface. You can't specify this parameter when also specifying
    #   private IP addresses.
    #
    # @option params [Boolean] :allow_reassignment
    #   Indicates whether to allow an IP address that is already assigned to
    #   another network interface or instance to be reassigned to the
    #   specified network interface.
    #
    # @return [Types::AssignPrivateIpAddressesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssignPrivateIpAddressesResult#network_interface_id #network_interface_id} => String
    #   * {Types::AssignPrivateIpAddressesResult#assigned_private_ip_addresses #assigned_private_ip_addresses} => Array&lt;Types::AssignedPrivateIpAddress&gt;
    #   * {Types::AssignPrivateIpAddressesResult#assigned_ipv_4_prefixes #assigned_ipv_4_prefixes} => Array&lt;Types::Ipv4PrefixSpecification&gt;
    #
    #
    # @example Example: To assign a specific secondary private IP address to an interface
    #
    #   # This example assigns the specified secondary private IP address to the specified network interface.
    #
    #   resp = client.assign_private_ip_addresses({
    #     network_interface_id: "eni-e5aa89a3", 
    #     private_ip_addresses: [
    #       "10.0.0.82", 
    #     ], 
    #   })
    #
    # @example Example: To assign secondary private IP addresses that Amazon EC2 selects to an interface
    #
    #   # This example assigns two secondary private IP addresses to the specified network interface. Amazon EC2 automatically
    #   # assigns these IP addresses from the available IP addresses in the CIDR block range of the subnet the network interface
    #   # is associated with.
    #
    #   resp = client.assign_private_ip_addresses({
    #     network_interface_id: "eni-e5aa89a3", 
    #     secondary_private_ip_address_count: 2, 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.assign_private_ip_addresses({
    #     ipv_4_prefixes: ["String"],
    #     ipv_4_prefix_count: 1,
    #     network_interface_id: "NetworkInterfaceId", # required
    #     private_ip_addresses: ["String"],
    #     secondary_private_ip_address_count: 1,
    #     allow_reassignment: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.network_interface_id #=> String
    #   resp.assigned_private_ip_addresses #=> Array
    #   resp.assigned_private_ip_addresses[0].private_ip_address #=> String
    #   resp.assigned_ipv_4_prefixes #=> Array
    #   resp.assigned_ipv_4_prefixes[0].ipv_4_prefix #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssignPrivateIpAddresses AWS API Documentation
    #
    # @overload assign_private_ip_addresses(params = {})
    # @param [Hash] params ({})
    def assign_private_ip_addresses(params = {}, options = {})
      req = build_request(:assign_private_ip_addresses, params)
      req.send_request(options)
    end

    # Assigns private IPv4 addresses to a private NAT gateway. For more
    # information, see [Work with NAT gateways][1] in the *Amazon VPC User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/nat-gateway-working-with.html
    #
    # @option params [required, String] :nat_gateway_id
    #   The ID of the NAT gateway.
    #
    # @option params [Array<String>] :private_ip_addresses
    #   The private IPv4 addresses you want to assign to the private NAT
    #   gateway.
    #
    # @option params [Integer] :private_ip_address_count
    #   The number of private IP addresses to assign to the NAT gateway. You
    #   can't specify this parameter when also specifying private IP
    #   addresses.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AssignPrivateNatGatewayAddressResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssignPrivateNatGatewayAddressResult#nat_gateway_id #nat_gateway_id} => String
    #   * {Types::AssignPrivateNatGatewayAddressResult#nat_gateway_addresses #nat_gateway_addresses} => Array&lt;Types::NatGatewayAddress&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.assign_private_nat_gateway_address({
    #     nat_gateway_id: "NatGatewayId", # required
    #     private_ip_addresses: ["String"],
    #     private_ip_address_count: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.nat_gateway_id #=> String
    #   resp.nat_gateway_addresses #=> Array
    #   resp.nat_gateway_addresses[0].allocation_id #=> String
    #   resp.nat_gateway_addresses[0].network_interface_id #=> String
    #   resp.nat_gateway_addresses[0].private_ip #=> String
    #   resp.nat_gateway_addresses[0].public_ip #=> String
    #   resp.nat_gateway_addresses[0].association_id #=> String
    #   resp.nat_gateway_addresses[0].is_primary #=> Boolean
    #   resp.nat_gateway_addresses[0].failure_message #=> String
    #   resp.nat_gateway_addresses[0].status #=> String, one of "assigning", "unassigning", "associating", "disassociating", "succeeded", "failed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssignPrivateNatGatewayAddress AWS API Documentation
    #
    # @overload assign_private_nat_gateway_address(params = {})
    # @param [Hash] params ({})
    def assign_private_nat_gateway_address(params = {}, options = {})
      req = build_request(:assign_private_nat_gateway_address, params)
      req.send_request(options)
    end

    # Associates an Elastic IP address, or carrier IP address (for instances
    # that are in subnets in Wavelength Zones) with an instance or a network
    # interface. Before you can use an Elastic IP address, you must allocate
    # it to your account.
    #
    # If the Elastic IP address is already associated with a different
    # instance, it is disassociated from that instance and associated with
    # the specified instance. If you associate an Elastic IP address with an
    # instance that has an existing Elastic IP address, the existing address
    # is disassociated from the instance, but remains allocated to your
    # account.
    #
    # \[Subnets in Wavelength Zones\] You can associate an IP address from
    # the telecommunication carrier to the instance or network interface.
    #
    # You cannot associate an Elastic IP address with an interface in a
    # different network border group.
    #
    # This is an idempotent operation. If you perform the operation more
    # than once, Amazon EC2 doesn't return an error, and you may be charged
    # for each time the Elastic IP address is remapped to the same instance.
    # For more information, see the *Elastic IP Addresses* section of
    # [Amazon EC2 Pricing][1].
    #
    #
    #
    # [1]: http://aws.amazon.com/ec2/pricing/
    #
    # @option params [String] :allocation_id
    #   The allocation ID. This is required.
    #
    # @option params [String] :instance_id
    #   The ID of the instance. The instance must have exactly one attached
    #   network interface. You can specify either the instance ID or the
    #   network interface ID, but not both.
    #
    # @option params [String] :public_ip
    #   Deprecated.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :network_interface_id
    #   The ID of the network interface. If the instance has more than one
    #   network interface, you must specify a network interface ID.
    #
    #   You can specify either the instance ID or the network interface ID,
    #   but not both.
    #
    # @option params [String] :private_ip_address
    #   The primary or secondary private IP address to associate with the
    #   Elastic IP address. If no private IP address is specified, the Elastic
    #   IP address is associated with the primary private IP address.
    #
    # @option params [Boolean] :allow_reassociation
    #   Reassociation is automatic, but you can specify false to ensure the
    #   operation fails if the Elastic IP address is already associated with
    #   another resource.
    #
    # @return [Types::AssociateAddressResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateAddressResult#association_id #association_id} => String
    #
    #
    # @example Example: To associate an Elastic IP address
    #
    #   # This example associates the specified Elastic IP address with the specified instance.
    #
    #   resp = client.associate_address({
    #     allocation_id: "eipalloc-64d5890a", 
    #     instance_id: "i-0b263919b6498b123", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     association_id: "eipassoc-2bebb745", 
    #   }
    #
    # @example Example: To associate an Elastic IP address with a network interface
    #
    #   # This example associates the specified Elastic IP address with the specified network interface.
    #
    #   resp = client.associate_address({
    #     allocation_id: "eipalloc-64d5890a", 
    #     network_interface_id: "eni-1a2b3c4d", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     association_id: "eipassoc-2bebb745", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_address({
    #     allocation_id: "AllocationId",
    #     instance_id: "InstanceId",
    #     public_ip: "EipAllocationPublicIp",
    #     dry_run: false,
    #     network_interface_id: "NetworkInterfaceId",
    #     private_ip_address: "String",
    #     allow_reassociation: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.association_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateAddress AWS API Documentation
    #
    # @overload associate_address(params = {})
    # @param [Hash] params ({})
    def associate_address(params = {}, options = {})
      req = build_request(:associate_address, params)
      req.send_request(options)
    end

    # Initiates a request to assign billing of the unused capacity of a
    # shared Capacity Reservation to a consumer account that is consolidated
    # under the same Amazon Web Services organizations payer account. For
    # more information, see [Billing assignment for shared Amazon EC2
    # Capacity Reservations][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/assign-billing.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :capacity_reservation_id
    #   The ID of the Capacity Reservation.
    #
    # @option params [required, String] :unused_reservation_billing_owner_id
    #   The ID of the consumer account to which to assign billing.
    #
    # @return [Types::AssociateCapacityReservationBillingOwnerResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateCapacityReservationBillingOwnerResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_capacity_reservation_billing_owner({
    #     dry_run: false,
    #     capacity_reservation_id: "CapacityReservationId", # required
    #     unused_reservation_billing_owner_id: "AccountID", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateCapacityReservationBillingOwner AWS API Documentation
    #
    # @overload associate_capacity_reservation_billing_owner(params = {})
    # @param [Hash] params ({})
    def associate_capacity_reservation_billing_owner(params = {}, options = {})
      req = build_request(:associate_capacity_reservation_billing_owner, params)
      req.send_request(options)
    end

    # Associates a target network with a Client VPN endpoint. A target
    # network is a subnet in a VPC. You can associate multiple subnets from
    # the same VPC with a Client VPN endpoint. You can associate only one
    # subnet in each Availability Zone. We recommend that you associate at
    # least two subnets to provide Availability Zone redundancy.
    #
    # If you specified a VPC when you created the Client VPN endpoint or if
    # you have previous subnet associations, the specified subnet must be in
    # the same VPC. To specify a subnet that's in a different VPC, you must
    # first modify the Client VPN endpoint (ModifyClientVpnEndpoint) and
    # change the VPC that's associated with it.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint.
    #
    # @option params [required, String] :subnet_id
    #   The ID of the subnet to associate with the Client VPN endpoint.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AssociateClientVpnTargetNetworkResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateClientVpnTargetNetworkResult#association_id #association_id} => String
    #   * {Types::AssociateClientVpnTargetNetworkResult#status #status} => Types::AssociationStatus
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_client_vpn_target_network({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     subnet_id: "SubnetId", # required
    #     client_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.association_id #=> String
    #   resp.status.code #=> String, one of "associating", "associated", "association-failed", "disassociating", "disassociated"
    #   resp.status.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateClientVpnTargetNetwork AWS API Documentation
    #
    # @overload associate_client_vpn_target_network(params = {})
    # @param [Hash] params ({})
    def associate_client_vpn_target_network(params = {}, options = {})
      req = build_request(:associate_client_vpn_target_network, params)
      req.send_request(options)
    end

    # Associates a set of DHCP options (that you've previously created)
    # with the specified VPC, or associates no DHCP options with the VPC.
    #
    # After you associate the options with the VPC, any existing instances
    # and all new instances that you launch in that VPC use the options. You
    # don't need to restart or relaunch the instances. They automatically
    # pick up the changes within a few hours, depending on how frequently
    # the instance renews its DHCP lease. You can explicitly renew the lease
    # using the operating system on the instance.
    #
    # For more information, see [DHCP option sets][1] in the *Amazon VPC
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/VPC_DHCP_Options.html
    #
    # @option params [required, String] :dhcp_options_id
    #   The ID of the DHCP options set, or `default` to associate no DHCP
    #   options with the VPC.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To associate a DHCP options set with a VPC
    #
    #   # This example associates the specified DHCP options set with the specified VPC.
    #
    #   resp = client.associate_dhcp_options({
    #     dhcp_options_id: "dopt-d9070ebb", 
    #     vpc_id: "vpc-a01106c2", 
    #   })
    #
    # @example Example: To associate the default DHCP options set with a VPC
    #
    #   # This example associates the default DHCP options set with the specified VPC.
    #
    #   resp = client.associate_dhcp_options({
    #     dhcp_options_id: "default", 
    #     vpc_id: "vpc-a01106c2", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_dhcp_options({
    #     dhcp_options_id: "DefaultingDhcpOptionsId", # required
    #     vpc_id: "VpcId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateDhcpOptions AWS API Documentation
    #
    # @overload associate_dhcp_options(params = {})
    # @param [Hash] params ({})
    def associate_dhcp_options(params = {}, options = {})
      req = build_request(:associate_dhcp_options, params)
      req.send_request(options)
    end

    # Associates an Identity and Access Management (IAM) role with an
    # Certificate Manager (ACM) certificate. This enables the certificate to
    # be used by the ACM for Nitro Enclaves application inside an enclave.
    # For more information, see [Certificate Manager for Nitro Enclaves][1]
    # in the *Amazon Web Services Nitro Enclaves User Guide*.
    #
    # When the IAM role is associated with the ACM certificate, the
    # certificate, certificate chain, and encrypted private key are placed
    # in an Amazon S3 location that only the associated IAM role can access.
    # The private key of the certificate is encrypted with an Amazon Web
    # Services managed key that has an attached attestation-based key
    # policy.
    #
    # To enable the IAM role to access the Amazon S3 object, you must grant
    # it permission to call `s3:GetObject` on the Amazon S3 bucket returned
    # by the command. To enable the IAM role to access the KMS key, you must
    # grant it permission to call `kms:Decrypt` on the KMS key returned by
    # the command. For more information, see [ Grant the role permission to
    # access the certificate and encryption key][2] in the *Amazon Web
    # Services Nitro Enclaves User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/enclaves/latest/user/nitro-enclave-refapp.html
    # [2]: https://docs.aws.amazon.com/enclaves/latest/user/nitro-enclave-refapp.html#add-policy
    #
    # @option params [required, String] :certificate_arn
    #   The ARN of the ACM certificate with which to associate the IAM role.
    #
    # @option params [required, String] :role_arn
    #   The ARN of the IAM role to associate with the ACM certificate. You can
    #   associate up to 16 IAM roles with an ACM certificate.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AssociateEnclaveCertificateIamRoleResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateEnclaveCertificateIamRoleResult#certificate_s3_bucket_name #certificate_s3_bucket_name} => String
    #   * {Types::AssociateEnclaveCertificateIamRoleResult#certificate_s3_object_key #certificate_s3_object_key} => String
    #   * {Types::AssociateEnclaveCertificateIamRoleResult#encryption_kms_key_id #encryption_kms_key_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_enclave_certificate_iam_role({
    #     certificate_arn: "CertificateId", # required
    #     role_arn: "RoleId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.certificate_s3_bucket_name #=> String
    #   resp.certificate_s3_object_key #=> String
    #   resp.encryption_kms_key_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateEnclaveCertificateIamRole AWS API Documentation
    #
    # @overload associate_enclave_certificate_iam_role(params = {})
    # @param [Hash] params ({})
    def associate_enclave_certificate_iam_role(params = {}, options = {})
      req = build_request(:associate_enclave_certificate_iam_role, params)
      req.send_request(options)
    end

    # Associates an IAM instance profile with a running or stopped instance.
    # You cannot associate more than one IAM instance profile with an
    # instance.
    #
    # @option params [required, Types::IamInstanceProfileSpecification] :iam_instance_profile
    #   The IAM instance profile.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @return [Types::AssociateIamInstanceProfileResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateIamInstanceProfileResult#iam_instance_profile_association #iam_instance_profile_association} => Types::IamInstanceProfileAssociation
    #
    #
    # @example Example: To associate an IAM instance profile with an instance
    #
    #   # This example associates an IAM instance profile named admin-role with the specified instance.
    #
    #   resp = client.associate_iam_instance_profile({
    #     iam_instance_profile: {
    #       name: "admin-role", 
    #     }, 
    #     instance_id: "i-123456789abcde123", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     iam_instance_profile_association: {
    #       association_id: "iip-assoc-0e7736511a163c209", 
    #       iam_instance_profile: {
    #         arn: "arn:aws:iam::123456789012:instance-profile/admin-role", 
    #         id: "AIPAJBLK7RKJKWDXVHIEC", 
    #       }, 
    #       instance_id: "i-123456789abcde123", 
    #       state: "associating", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_iam_instance_profile({
    #     iam_instance_profile: { # required
    #       arn: "String",
    #       name: "String",
    #     },
    #     instance_id: "InstanceId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.iam_instance_profile_association.association_id #=> String
    #   resp.iam_instance_profile_association.instance_id #=> String
    #   resp.iam_instance_profile_association.iam_instance_profile.arn #=> String
    #   resp.iam_instance_profile_association.iam_instance_profile.id #=> String
    #   resp.iam_instance_profile_association.state #=> String, one of "associating", "associated", "disassociating", "disassociated"
    #   resp.iam_instance_profile_association.timestamp #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateIamInstanceProfile AWS API Documentation
    #
    # @overload associate_iam_instance_profile(params = {})
    # @param [Hash] params ({})
    def associate_iam_instance_profile(params = {}, options = {})
      req = build_request(:associate_iam_instance_profile, params)
      req.send_request(options)
    end

    # Associates one or more targets with an event window. Only one type of
    # target (instance IDs, Dedicated Host IDs, or tags) can be specified
    # with an event window.
    #
    # For more information, see [Define event windows for scheduled
    # events][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/event-windows.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_event_window_id
    #   The ID of the event window.
    #
    # @option params [required, Types::InstanceEventWindowAssociationRequest] :association_target
    #   One or more targets associated with the specified event window.
    #
    # @return [Types::AssociateInstanceEventWindowResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateInstanceEventWindowResult#instance_event_window #instance_event_window} => Types::InstanceEventWindow
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_instance_event_window({
    #     dry_run: false,
    #     instance_event_window_id: "InstanceEventWindowId", # required
    #     association_target: { # required
    #       instance_ids: ["InstanceId"],
    #       instance_tags: [
    #         {
    #           key: "String",
    #           value: "String",
    #         },
    #       ],
    #       dedicated_host_ids: ["DedicatedHostId"],
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_event_window.instance_event_window_id #=> String
    #   resp.instance_event_window.time_ranges #=> Array
    #   resp.instance_event_window.time_ranges[0].start_week_day #=> String, one of "sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"
    #   resp.instance_event_window.time_ranges[0].start_hour #=> Integer
    #   resp.instance_event_window.time_ranges[0].end_week_day #=> String, one of "sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"
    #   resp.instance_event_window.time_ranges[0].end_hour #=> Integer
    #   resp.instance_event_window.name #=> String
    #   resp.instance_event_window.cron_expression #=> String
    #   resp.instance_event_window.association_target.instance_ids #=> Array
    #   resp.instance_event_window.association_target.instance_ids[0] #=> String
    #   resp.instance_event_window.association_target.tags #=> Array
    #   resp.instance_event_window.association_target.tags[0].key #=> String
    #   resp.instance_event_window.association_target.tags[0].value #=> String
    #   resp.instance_event_window.association_target.dedicated_host_ids #=> Array
    #   resp.instance_event_window.association_target.dedicated_host_ids[0] #=> String
    #   resp.instance_event_window.state #=> String, one of "creating", "deleting", "active", "deleted"
    #   resp.instance_event_window.tags #=> Array
    #   resp.instance_event_window.tags[0].key #=> String
    #   resp.instance_event_window.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateInstanceEventWindow AWS API Documentation
    #
    # @overload associate_instance_event_window(params = {})
    # @param [Hash] params ({})
    def associate_instance_event_window(params = {}, options = {})
      req = build_request(:associate_instance_event_window, params)
      req.send_request(options)
    end

    # Associates your Autonomous System Number (ASN) with a BYOIP CIDR that
    # you own in the same Amazon Web Services Region. For more information,
    # see [Tutorial: Bring your ASN to IPAM][1] in the *Amazon VPC IPAM
    # guide*.
    #
    # After the association succeeds, the ASN is eligible for advertisement.
    # You can view the association with [DescribeByoipCidrs][2]. You can
    # advertise the CIDR with [AdvertiseByoipCidr][3].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/tutorials-byoasn.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeByoipCidrs.html
    # [3]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_AdvertiseByoipCidr.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :asn
    #   A public 2-byte or 4-byte ASN.
    #
    # @option params [required, String] :cidr
    #   The BYOIP CIDR you want to associate with an ASN.
    #
    # @return [Types::AssociateIpamByoasnResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateIpamByoasnResult#asn_association #asn_association} => Types::AsnAssociation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_ipam_byoasn({
    #     dry_run: false,
    #     asn: "String", # required
    #     cidr: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.asn_association.asn #=> String
    #   resp.asn_association.cidr #=> String
    #   resp.asn_association.status_message #=> String
    #   resp.asn_association.state #=> String, one of "disassociated", "failed-disassociation", "failed-association", "pending-disassociation", "pending-association", "associated"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateIpamByoasn AWS API Documentation
    #
    # @overload associate_ipam_byoasn(params = {})
    # @param [Hash] params ({})
    def associate_ipam_byoasn(params = {}, options = {})
      req = build_request(:associate_ipam_byoasn, params)
      req.send_request(options)
    end

    # Associates an IPAM resource discovery with an Amazon VPC IPAM. A
    # resource discovery is an IPAM component that enables IPAM to manage
    # and monitor resources that belong to the owning account.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_id
    #   An IPAM ID.
    #
    # @option params [required, String] :ipam_resource_discovery_id
    #   A resource discovery ID.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   Tag specifications.
    #
    # @option params [String] :client_token
    #   A client token.
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    # @return [Types::AssociateIpamResourceDiscoveryResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateIpamResourceDiscoveryResult#ipam_resource_discovery_association #ipam_resource_discovery_association} => Types::IpamResourceDiscoveryAssociation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_ipam_resource_discovery({
    #     dry_run: false,
    #     ipam_id: "IpamId", # required
    #     ipam_resource_discovery_id: "IpamResourceDiscoveryId", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_resource_discovery_association.owner_id #=> String
    #   resp.ipam_resource_discovery_association.ipam_resource_discovery_association_id #=> String
    #   resp.ipam_resource_discovery_association.ipam_resource_discovery_association_arn #=> String
    #   resp.ipam_resource_discovery_association.ipam_resource_discovery_id #=> String
    #   resp.ipam_resource_discovery_association.ipam_id #=> String
    #   resp.ipam_resource_discovery_association.ipam_arn #=> String
    #   resp.ipam_resource_discovery_association.ipam_region #=> String
    #   resp.ipam_resource_discovery_association.is_default #=> Boolean
    #   resp.ipam_resource_discovery_association.resource_discovery_status #=> String, one of "active", "not-found"
    #   resp.ipam_resource_discovery_association.state #=> String, one of "associate-in-progress", "associate-complete", "associate-failed", "disassociate-in-progress", "disassociate-complete", "disassociate-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_resource_discovery_association.tags #=> Array
    #   resp.ipam_resource_discovery_association.tags[0].key #=> String
    #   resp.ipam_resource_discovery_association.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateIpamResourceDiscovery AWS API Documentation
    #
    # @overload associate_ipam_resource_discovery(params = {})
    # @param [Hash] params ({})
    def associate_ipam_resource_discovery(params = {}, options = {})
      req = build_request(:associate_ipam_resource_discovery, params)
      req.send_request(options)
    end

    # Associates Elastic IP addresses (EIPs) and private IPv4 addresses with
    # a public NAT gateway. For more information, see [Work with NAT
    # gateways][1] in the *Amazon VPC User Guide*.
    #
    # By default, you can associate up to 2 Elastic IP addresses per public
    # NAT gateway. You can increase the limit by requesting a quota
    # adjustment. For more information, see [Elastic IP address quotas][2]
    # in the *Amazon VPC User Guide*.
    #
    # When you associate an EIP or secondary EIPs with a public NAT gateway,
    # the network border group of the EIPs must match the network border
    # group of the Availability Zone (AZ) that the public NAT gateway is in.
    # If it's not the same, the EIP will fail to associate. You can see the
    # network border group for the subnet's AZ by viewing the details of
    # the subnet. Similarly, you can view the network border group of an EIP
    # by viewing the details of the EIP address. For more information about
    # network border groups and EIPs, see [Allocate an Elastic IP
    # address][3] in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/nat-gateway-working-with.html
    # [2]: https://docs.aws.amazon.com/vpc/latest/userguide/amazon-vpc-limits.html#vpc-limits-eips
    # [3]: https://docs.aws.amazon.com/vpc/latest/userguide/WorkWithEIPs.html
    #
    # @option params [required, String] :nat_gateway_id
    #   The ID of the NAT gateway.
    #
    # @option params [required, Array<String>] :allocation_ids
    #   The allocation IDs of EIPs that you want to associate with your NAT
    #   gateway.
    #
    # @option params [Array<String>] :private_ip_addresses
    #   The private IPv4 addresses that you want to assign to the NAT gateway.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AssociateNatGatewayAddressResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateNatGatewayAddressResult#nat_gateway_id #nat_gateway_id} => String
    #   * {Types::AssociateNatGatewayAddressResult#nat_gateway_addresses #nat_gateway_addresses} => Array&lt;Types::NatGatewayAddress&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_nat_gateway_address({
    #     nat_gateway_id: "NatGatewayId", # required
    #     allocation_ids: ["AllocationId"], # required
    #     private_ip_addresses: ["String"],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.nat_gateway_id #=> String
    #   resp.nat_gateway_addresses #=> Array
    #   resp.nat_gateway_addresses[0].allocation_id #=> String
    #   resp.nat_gateway_addresses[0].network_interface_id #=> String
    #   resp.nat_gateway_addresses[0].private_ip #=> String
    #   resp.nat_gateway_addresses[0].public_ip #=> String
    #   resp.nat_gateway_addresses[0].association_id #=> String
    #   resp.nat_gateway_addresses[0].is_primary #=> Boolean
    #   resp.nat_gateway_addresses[0].failure_message #=> String
    #   resp.nat_gateway_addresses[0].status #=> String, one of "assigning", "unassigning", "associating", "disassociating", "succeeded", "failed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateNatGatewayAddress AWS API Documentation
    #
    # @overload associate_nat_gateway_address(params = {})
    # @param [Hash] params ({})
    def associate_nat_gateway_address(params = {}, options = {})
      req = build_request(:associate_nat_gateway_address, params)
      req.send_request(options)
    end

    # Associates a subnet in your VPC or an internet gateway or virtual
    # private gateway attached to your VPC with a route table in your VPC.
    # This association causes traffic from the subnet or gateway to be
    # routed according to the routes in the route table. The action returns
    # an association ID, which you need in order to disassociate the route
    # table later. A route table can be associated with multiple subnets.
    #
    # For more information, see [Route tables][1] in the *Amazon VPC User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Route_Tables.html
    #
    # @option params [String] :gateway_id
    #   The ID of the internet gateway or virtual private gateway.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :subnet_id
    #   The ID of the subnet.
    #
    # @option params [required, String] :route_table_id
    #   The ID of the route table.
    #
    # @return [Types::AssociateRouteTableResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateRouteTableResult#association_id #association_id} => String
    #   * {Types::AssociateRouteTableResult#association_state #association_state} => Types::RouteTableAssociationState
    #
    #
    # @example Example: To associate a route table with a subnet
    #
    #   # This example associates the specified route table with the specified subnet.
    #
    #   resp = client.associate_route_table({
    #     route_table_id: "rtb-22574640", 
    #     subnet_id: "subnet-9d4a7b6", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     association_id: "rtbassoc-781d0d1a", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_route_table({
    #     gateway_id: "RouteGatewayId",
    #     dry_run: false,
    #     subnet_id: "SubnetId",
    #     route_table_id: "RouteTableId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.association_id #=> String
    #   resp.association_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failed"
    #   resp.association_state.status_message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateRouteTable AWS API Documentation
    #
    # @overload associate_route_table(params = {})
    # @param [Hash] params ({})
    def associate_route_table(params = {}, options = {})
      req = build_request(:associate_route_table, params)
      req.send_request(options)
    end

    # Associates a security group with another VPC in the same Region. This
    # enables you to use the same security group with network interfaces and
    # instances in the specified VPC.
    #
    # <note markdown="1"> * The VPC you want to associate the security group with must be in the
    #   same Region.
    #
    # * You can associate the security group with another VPC if your
    #   account owns the VPC or if the VPC was shared with you.
    #
    # * You must own the security group and the VPC that it was created in.
    #
    # * You cannot use this feature with default security groups.
    #
    # * You cannot use this feature with the default VPC.
    #
    #  </note>
    #
    # @option params [required, String] :group_id
    #   A security group ID.
    #
    # @option params [required, String] :vpc_id
    #   A VPC ID.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AssociateSecurityGroupVpcResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateSecurityGroupVpcResult#state #state} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_security_group_vpc({
    #     group_id: "SecurityGroupId", # required
    #     vpc_id: "VpcId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.state #=> String, one of "associating", "associated", "association-failed", "disassociating", "disassociated", "disassociation-failed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateSecurityGroupVpc AWS API Documentation
    #
    # @overload associate_security_group_vpc(params = {})
    # @param [Hash] params ({})
    def associate_security_group_vpc(params = {}, options = {})
      req = build_request(:associate_security_group_vpc, params)
      req.send_request(options)
    end

    # Associates a CIDR block with your subnet. You can only associate a
    # single IPv6 CIDR block with your subnet.
    #
    # @option params [String] :ipv_6_ipam_pool_id
    #   An IPv6 IPAM pool ID.
    #
    # @option params [Integer] :ipv_6_netmask_length
    #   An IPv6 netmask length.
    #
    # @option params [required, String] :subnet_id
    #   The ID of your subnet.
    #
    # @option params [String] :ipv_6_cidr_block
    #   The IPv6 CIDR block for your subnet.
    #
    # @return [Types::AssociateSubnetCidrBlockResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateSubnetCidrBlockResult#ipv_6_cidr_block_association #ipv_6_cidr_block_association} => Types::SubnetIpv6CidrBlockAssociation
    #   * {Types::AssociateSubnetCidrBlockResult#subnet_id #subnet_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_subnet_cidr_block({
    #     ipv_6_ipam_pool_id: "IpamPoolId",
    #     ipv_6_netmask_length: 1,
    #     subnet_id: "SubnetId", # required
    #     ipv_6_cidr_block: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.ipv_6_cidr_block_association.association_id #=> String
    #   resp.ipv_6_cidr_block_association.ipv_6_cidr_block #=> String
    #   resp.ipv_6_cidr_block_association.ipv_6_cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.ipv_6_cidr_block_association.ipv_6_cidr_block_state.status_message #=> String
    #   resp.ipv_6_cidr_block_association.ipv_6_address_attribute #=> String, one of "public", "private"
    #   resp.ipv_6_cidr_block_association.ip_source #=> String, one of "amazon", "byoip", "none"
    #   resp.subnet_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateSubnetCidrBlock AWS API Documentation
    #
    # @overload associate_subnet_cidr_block(params = {})
    # @param [Hash] params ({})
    def associate_subnet_cidr_block(params = {}, options = {})
      req = build_request(:associate_subnet_cidr_block, params)
      req.send_request(options)
    end

    # Associates the specified subnets and transit gateway attachments with
    # the specified transit gateway multicast domain.
    #
    # The transit gateway attachment must be in the available state before
    # you can add a resource. Use [DescribeTransitGatewayAttachments][1] to
    # see the state of the attachment.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeTransitGatewayAttachments.html
    #
    # @option params [required, String] :transit_gateway_multicast_domain_id
    #   The ID of the transit gateway multicast domain.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the transit gateway attachment to associate with the transit
    #   gateway multicast domain.
    #
    # @option params [required, Array<String>] :subnet_ids
    #   The IDs of the subnets to associate with the transit gateway multicast
    #   domain.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AssociateTransitGatewayMulticastDomainResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateTransitGatewayMulticastDomainResult#associations #associations} => Types::TransitGatewayMulticastDomainAssociations
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_transit_gateway_multicast_domain({
    #     transit_gateway_multicast_domain_id: "TransitGatewayMulticastDomainId", # required
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     subnet_ids: ["SubnetId"], # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.associations.transit_gateway_multicast_domain_id #=> String
    #   resp.associations.transit_gateway_attachment_id #=> String
    #   resp.associations.resource_id #=> String
    #   resp.associations.resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.associations.resource_owner_id #=> String
    #   resp.associations.subnets #=> Array
    #   resp.associations.subnets[0].subnet_id #=> String
    #   resp.associations.subnets[0].state #=> String, one of "pendingAcceptance", "associating", "associated", "disassociating", "disassociated", "rejected", "failed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateTransitGatewayMulticastDomain AWS API Documentation
    #
    # @overload associate_transit_gateway_multicast_domain(params = {})
    # @param [Hash] params ({})
    def associate_transit_gateway_multicast_domain(params = {}, options = {})
      req = build_request(:associate_transit_gateway_multicast_domain, params)
      req.send_request(options)
    end

    # Associates the specified transit gateway attachment with a transit
    # gateway policy table.
    #
    # @option params [required, String] :transit_gateway_policy_table_id
    #   The ID of the transit gateway policy table to associate with the
    #   transit gateway attachment.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the transit gateway attachment to associate with the policy
    #   table.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AssociateTransitGatewayPolicyTableResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateTransitGatewayPolicyTableResult#association #association} => Types::TransitGatewayPolicyTableAssociation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_transit_gateway_policy_table({
    #     transit_gateway_policy_table_id: "TransitGatewayPolicyTableId", # required
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.association.transit_gateway_policy_table_id #=> String
    #   resp.association.transit_gateway_attachment_id #=> String
    #   resp.association.resource_id #=> String
    #   resp.association.resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.association.state #=> String, one of "associating", "associated", "disassociating", "disassociated"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateTransitGatewayPolicyTable AWS API Documentation
    #
    # @overload associate_transit_gateway_policy_table(params = {})
    # @param [Hash] params ({})
    def associate_transit_gateway_policy_table(params = {}, options = {})
      req = build_request(:associate_transit_gateway_policy_table, params)
      req.send_request(options)
    end

    # Associates the specified attachment with the specified transit gateway
    # route table. You can associate only one route table with an
    # attachment.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the transit gateway route table.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the attachment.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AssociateTransitGatewayRouteTableResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateTransitGatewayRouteTableResult#association #association} => Types::TransitGatewayAssociation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_transit_gateway_route_table({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.association.transit_gateway_route_table_id #=> String
    #   resp.association.transit_gateway_attachment_id #=> String
    #   resp.association.resource_id #=> String
    #   resp.association.resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.association.state #=> String, one of "associating", "associated", "disassociating", "disassociated"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateTransitGatewayRouteTable AWS API Documentation
    #
    # @overload associate_transit_gateway_route_table(params = {})
    # @param [Hash] params ({})
    def associate_transit_gateway_route_table(params = {}, options = {})
      req = build_request(:associate_transit_gateway_route_table, params)
      req.send_request(options)
    end

    # Associates a branch network interface with a trunk network interface.
    #
    # Before you create the association, use [CreateNetworkInterface][1]
    # command and set the interface type to `trunk`. You must also create a
    # network interface for each branch network interface that you want to
    # associate with the trunk network interface.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateNetworkInterface.html
    #
    # @option params [required, String] :branch_interface_id
    #   The ID of the branch network interface.
    #
    # @option params [required, String] :trunk_interface_id
    #   The ID of the trunk network interface.
    #
    # @option params [Integer] :vlan_id
    #   The ID of the VLAN. This applies to the VLAN protocol.
    #
    # @option params [Integer] :gre_key
    #   The application key. This applies to the GRE protocol.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AssociateTrunkInterfaceResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateTrunkInterfaceResult#interface_association #interface_association} => Types::TrunkInterfaceAssociation
    #   * {Types::AssociateTrunkInterfaceResult#client_token #client_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_trunk_interface({
    #     branch_interface_id: "NetworkInterfaceId", # required
    #     trunk_interface_id: "NetworkInterfaceId", # required
    #     vlan_id: 1,
    #     gre_key: 1,
    #     client_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.interface_association.association_id #=> String
    #   resp.interface_association.branch_interface_id #=> String
    #   resp.interface_association.trunk_interface_id #=> String
    #   resp.interface_association.interface_protocol #=> String, one of "VLAN", "GRE"
    #   resp.interface_association.vlan_id #=> Integer
    #   resp.interface_association.gre_key #=> Integer
    #   resp.interface_association.tags #=> Array
    #   resp.interface_association.tags[0].key #=> String
    #   resp.interface_association.tags[0].value #=> String
    #   resp.client_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateTrunkInterface AWS API Documentation
    #
    # @overload associate_trunk_interface(params = {})
    # @param [Hash] params ({})
    def associate_trunk_interface(params = {}, options = {})
      req = build_request(:associate_trunk_interface, params)
      req.send_request(options)
    end

    # Associates a CIDR block with your VPC. You can associate a secondary
    # IPv4 CIDR block, an Amazon-provided IPv6 CIDR block, or an IPv6 CIDR
    # block from an IPv6 address pool that you provisioned through bring
    # your own IP addresses ([BYOIP][1]).
    #
    # You must specify one of the following in the request: an IPv4 CIDR
    # block, an IPv6 pool, or an Amazon-provided IPv6 CIDR block.
    #
    # For more information about associating CIDR blocks with your VPC and
    # applicable restrictions, see [IP addressing for your VPCs and
    # subnets][2] in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-byoip.html
    # [2]: https://docs.aws.amazon.com/vpc/latest/userguide/vpc-ip-addressing.html
    #
    # @option params [String] :cidr_block
    #   An IPv4 CIDR block to associate with the VPC.
    #
    # @option params [String] :ipv_6_cidr_block_network_border_group
    #   The name of the location from which we advertise the IPV6 CIDR block.
    #   Use this parameter to limit the CIDR block to this location.
    #
    #   You must set `AmazonProvidedIpv6CidrBlock` to `true` to use this
    #   parameter.
    #
    #   You can have one IPv6 CIDR block association per network border group.
    #
    # @option params [String] :ipv_6_pool
    #   The ID of an IPv6 address pool from which to allocate the IPv6 CIDR
    #   block.
    #
    # @option params [String] :ipv_6_cidr_block
    #   An IPv6 CIDR block from the IPv6 address pool. You must also specify
    #   `Ipv6Pool` in the request.
    #
    #   To let Amazon choose the IPv6 CIDR block for you, omit this parameter.
    #
    # @option params [String] :ipv_4_ipam_pool_id
    #   Associate a CIDR allocated from an IPv4 IPAM pool to a VPC. For more
    #   information about Amazon VPC IP Address Manager (IPAM), see [What is
    #   IPAM?][1] in the *Amazon VPC IPAM User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/what-is-it-ipam.html
    #
    # @option params [Integer] :ipv_4_netmask_length
    #   The netmask length of the IPv4 CIDR you would like to associate from
    #   an Amazon VPC IP Address Manager (IPAM) pool. For more information
    #   about IPAM, see [What is IPAM?][1] in the *Amazon VPC IPAM User
    #   Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/what-is-it-ipam.html
    #
    # @option params [String] :ipv_6_ipam_pool_id
    #   Associates a CIDR allocated from an IPv6 IPAM pool to a VPC. For more
    #   information about Amazon VPC IP Address Manager (IPAM), see [What is
    #   IPAM?][1] in the *Amazon VPC IPAM User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/what-is-it-ipam.html
    #
    # @option params [Integer] :ipv_6_netmask_length
    #   The netmask length of the IPv6 CIDR you would like to associate from
    #   an Amazon VPC IP Address Manager (IPAM) pool. For more information
    #   about IPAM, see [What is IPAM?][1] in the *Amazon VPC IPAM User
    #   Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/what-is-it-ipam.html
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @option params [Boolean] :amazon_provided_ipv_6_cidr_block
    #   Requests an Amazon-provided IPv6 CIDR block with a /56 prefix length
    #   for the VPC. You cannot specify the range of IPv6 addresses or the
    #   size of the CIDR block.
    #
    # @return [Types::AssociateVpcCidrBlockResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AssociateVpcCidrBlockResult#ipv_6_cidr_block_association #ipv_6_cidr_block_association} => Types::VpcIpv6CidrBlockAssociation
    #   * {Types::AssociateVpcCidrBlockResult#cidr_block_association #cidr_block_association} => Types::VpcCidrBlockAssociation
    #   * {Types::AssociateVpcCidrBlockResult#vpc_id #vpc_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.associate_vpc_cidr_block({
    #     cidr_block: "String",
    #     ipv_6_cidr_block_network_border_group: "String",
    #     ipv_6_pool: "Ipv6PoolEc2Id",
    #     ipv_6_cidr_block: "String",
    #     ipv_4_ipam_pool_id: "IpamPoolId",
    #     ipv_4_netmask_length: 1,
    #     ipv_6_ipam_pool_id: "IpamPoolId",
    #     ipv_6_netmask_length: 1,
    #     vpc_id: "VpcId", # required
    #     amazon_provided_ipv_6_cidr_block: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.ipv_6_cidr_block_association.association_id #=> String
    #   resp.ipv_6_cidr_block_association.ipv_6_cidr_block #=> String
    #   resp.ipv_6_cidr_block_association.ipv_6_cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.ipv_6_cidr_block_association.ipv_6_cidr_block_state.status_message #=> String
    #   resp.ipv_6_cidr_block_association.network_border_group #=> String
    #   resp.ipv_6_cidr_block_association.ipv_6_pool #=> String
    #   resp.ipv_6_cidr_block_association.ipv_6_address_attribute #=> String, one of "public", "private"
    #   resp.ipv_6_cidr_block_association.ip_source #=> String, one of "amazon", "byoip", "none"
    #   resp.cidr_block_association.association_id #=> String
    #   resp.cidr_block_association.cidr_block #=> String
    #   resp.cidr_block_association.cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.cidr_block_association.cidr_block_state.status_message #=> String
    #   resp.vpc_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AssociateVpcCidrBlock AWS API Documentation
    #
    # @overload associate_vpc_cidr_block(params = {})
    # @param [Hash] params ({})
    def associate_vpc_cidr_block(params = {}, options = {})
      req = build_request(:associate_vpc_cidr_block, params)
      req.send_request(options)
    end

    # <note markdown="1"> This action is deprecated.
    #
    #  </note>
    #
    # Links an EC2-Classic instance to a ClassicLink-enabled VPC through one
    # or more of the VPC security groups. You cannot link an EC2-Classic
    # instance to more than one VPC at a time. You can only link an instance
    # that's in the `running` state. An instance is automatically unlinked
    # from a VPC when it's stopped - you can link it to the VPC again when
    # you restart it.
    #
    # After you've linked an instance, you cannot change the VPC security
    # groups that are associated with it. To change the security groups, you
    # must first unlink the instance, and then link it again.
    #
    # Linking your instance to a VPC is sometimes referred to as *attaching*
    # your instance.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_id
    #   The ID of the EC2-Classic instance.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the ClassicLink-enabled VPC.
    #
    # @option params [required, Array<String>] :groups
    #   The IDs of the security groups. You cannot specify security groups
    #   from a different VPC.
    #
    # @return [Types::AttachClassicLinkVpcResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AttachClassicLinkVpcResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.attach_classic_link_vpc({
    #     dry_run: false,
    #     instance_id: "InstanceId", # required
    #     vpc_id: "VpcId", # required
    #     groups: ["SecurityGroupId"], # required
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AttachClassicLinkVpc AWS API Documentation
    #
    # @overload attach_classic_link_vpc(params = {})
    # @param [Hash] params ({})
    def attach_classic_link_vpc(params = {}, options = {})
      req = build_request(:attach_classic_link_vpc, params)
      req.send_request(options)
    end

    # Attaches an internet gateway or a virtual private gateway to a VPC,
    # enabling connectivity between the internet and the VPC. For more
    # information, see [Internet gateways][1] in the *Amazon VPC User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Internet_Gateway.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :internet_gateway_id
    #   The ID of the internet gateway.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To attach an Internet gateway to a VPC
    #
    #   # This example attaches the specified Internet gateway to the specified VPC.
    #
    #   resp = client.attach_internet_gateway({
    #     internet_gateway_id: "igw-c0a643a9", 
    #     vpc_id: "vpc-a01106c2", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.attach_internet_gateway({
    #     dry_run: false,
    #     internet_gateway_id: "InternetGatewayId", # required
    #     vpc_id: "VpcId", # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AttachInternetGateway AWS API Documentation
    #
    # @overload attach_internet_gateway(params = {})
    # @param [Hash] params ({})
    def attach_internet_gateway(params = {}, options = {})
      req = build_request(:attach_internet_gateway, params)
      req.send_request(options)
    end

    # Attaches a network interface to an instance.
    #
    # @option params [Integer] :network_card_index
    #   The index of the network card. Some instance types support multiple
    #   network cards. The primary network interface must be assigned to
    #   network card index 0. The default is network card index 0.
    #
    # @option params [Types::EnaSrdSpecification] :ena_srd_specification
    #   Configures ENA Express for the network interface that this action
    #   attaches to the instance.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :network_interface_id
    #   The ID of the network interface.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @option params [required, Integer] :device_index
    #   The index of the device for the network interface attachment.
    #
    # @return [Types::AttachNetworkInterfaceResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AttachNetworkInterfaceResult#attachment_id #attachment_id} => String
    #   * {Types::AttachNetworkInterfaceResult#network_card_index #network_card_index} => Integer
    #
    #
    # @example Example: To attach a network interface to an instance
    #
    #   # This example attaches the specified network interface to the specified instance.
    #
    #   resp = client.attach_network_interface({
    #     device_index: 1, 
    #     instance_id: "i-1234567890abcdef0", 
    #     network_interface_id: "eni-e5aa89a3", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     attachment_id: "eni-attach-66c4350a", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.attach_network_interface({
    #     network_card_index: 1,
    #     ena_srd_specification: {
    #       ena_srd_enabled: false,
    #       ena_srd_udp_specification: {
    #         ena_srd_udp_enabled: false,
    #       },
    #     },
    #     dry_run: false,
    #     network_interface_id: "NetworkInterfaceId", # required
    #     instance_id: "InstanceId", # required
    #     device_index: 1, # required
    #   })
    #
    # @example Response structure
    #
    #   resp.attachment_id #=> String
    #   resp.network_card_index #=> Integer
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AttachNetworkInterface AWS API Documentation
    #
    # @overload attach_network_interface(params = {})
    # @param [Hash] params ({})
    def attach_network_interface(params = {}, options = {})
      req = build_request(:attach_network_interface, params)
      req.send_request(options)
    end

    # Attaches the specified Amazon Web Services Verified Access trust
    # provider to the specified Amazon Web Services Verified Access
    # instance.
    #
    # @option params [required, String] :verified_access_instance_id
    #   The ID of the Verified Access instance.
    #
    # @option params [required, String] :verified_access_trust_provider_id
    #   The ID of the Verified Access trust provider.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AttachVerifiedAccessTrustProviderResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AttachVerifiedAccessTrustProviderResult#verified_access_trust_provider #verified_access_trust_provider} => Types::VerifiedAccessTrustProvider
    #   * {Types::AttachVerifiedAccessTrustProviderResult#verified_access_instance #verified_access_instance} => Types::VerifiedAccessInstance
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.attach_verified_access_trust_provider({
    #     verified_access_instance_id: "VerifiedAccessInstanceId", # required
    #     verified_access_trust_provider_id: "VerifiedAccessTrustProviderId", # required
    #     client_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_trust_provider.verified_access_trust_provider_id #=> String
    #   resp.verified_access_trust_provider.description #=> String
    #   resp.verified_access_trust_provider.trust_provider_type #=> String, one of "user", "device"
    #   resp.verified_access_trust_provider.user_trust_provider_type #=> String, one of "iam-identity-center", "oidc"
    #   resp.verified_access_trust_provider.device_trust_provider_type #=> String, one of "jamf", "crowdstrike", "jumpcloud"
    #   resp.verified_access_trust_provider.oidc_options.issuer #=> String
    #   resp.verified_access_trust_provider.oidc_options.authorization_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.token_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.user_info_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.client_id #=> String
    #   resp.verified_access_trust_provider.oidc_options.client_secret #=> String
    #   resp.verified_access_trust_provider.oidc_options.scope #=> String
    #   resp.verified_access_trust_provider.device_options.tenant_id #=> String
    #   resp.verified_access_trust_provider.device_options.public_signing_key_url #=> String
    #   resp.verified_access_trust_provider.policy_reference_name #=> String
    #   resp.verified_access_trust_provider.creation_time #=> String
    #   resp.verified_access_trust_provider.last_updated_time #=> String
    #   resp.verified_access_trust_provider.tags #=> Array
    #   resp.verified_access_trust_provider.tags[0].key #=> String
    #   resp.verified_access_trust_provider.tags[0].value #=> String
    #   resp.verified_access_trust_provider.sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.verified_access_trust_provider.sse_specification.kms_key_arn #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.public_signing_key_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.issuer #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.authorization_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.token_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.user_info_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.client_id #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.scope #=> String
    #   resp.verified_access_instance.verified_access_instance_id #=> String
    #   resp.verified_access_instance.description #=> String
    #   resp.verified_access_instance.verified_access_trust_providers #=> Array
    #   resp.verified_access_instance.verified_access_trust_providers[0].verified_access_trust_provider_id #=> String
    #   resp.verified_access_instance.verified_access_trust_providers[0].description #=> String
    #   resp.verified_access_instance.verified_access_trust_providers[0].trust_provider_type #=> String, one of "user", "device"
    #   resp.verified_access_instance.verified_access_trust_providers[0].user_trust_provider_type #=> String, one of "iam-identity-center", "oidc"
    #   resp.verified_access_instance.verified_access_trust_providers[0].device_trust_provider_type #=> String, one of "jamf", "crowdstrike", "jumpcloud"
    #   resp.verified_access_instance.creation_time #=> String
    #   resp.verified_access_instance.last_updated_time #=> String
    #   resp.verified_access_instance.tags #=> Array
    #   resp.verified_access_instance.tags[0].key #=> String
    #   resp.verified_access_instance.tags[0].value #=> String
    #   resp.verified_access_instance.fips_enabled #=> Boolean
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.sub_domain #=> String
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.nameservers #=> Array
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.nameservers[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AttachVerifiedAccessTrustProvider AWS API Documentation
    #
    # @overload attach_verified_access_trust_provider(params = {})
    # @param [Hash] params ({})
    def attach_verified_access_trust_provider(params = {}, options = {})
      req = build_request(:attach_verified_access_trust_provider, params)
      req.send_request(options)
    end

    # Attaches an EBS volume to a running or stopped instance and exposes it
    # to the instance with the specified device name.
    #
    # Encrypted EBS volumes must be attached to instances that support
    # Amazon EBS encryption. For more information, see [Amazon EBS
    # encryption][1] in the *Amazon EBS User Guide*.
    #
    # After you attach an EBS volume, you must make it available. For more
    # information, see [Make an EBS volume available for use][2].
    #
    # If a volume has an Amazon Web Services Marketplace product code:
    #
    # * The volume can be attached only to a stopped instance.
    #
    # * Amazon Web Services Marketplace product codes are copied from the
    #   volume to the instance.
    #
    # * You must be subscribed to the product.
    #
    # * The instance type and operating system of the instance must support
    #   the product. For example, you can't detach a volume from a Windows
    #   instance and attach it to a Linux instance.
    #
    # For more information, see [Attach an Amazon EBS volume to an
    # instance][3] in the *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-encryption.html
    # [2]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-using-volumes.html
    # [3]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-attaching-volume.html
    #
    # @option params [required, String] :device
    #   The device name (for example, `/dev/sdh` or `xvdh`).
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @option params [required, String] :volume_id
    #   The ID of the EBS volume. The volume and instance must be within the
    #   same Availability Zone.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::VolumeAttachment] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::VolumeAttachment#delete_on_termination #delete_on_termination} => Boolean
    #   * {Types::VolumeAttachment#associated_resource #associated_resource} => String
    #   * {Types::VolumeAttachment#instance_owning_service #instance_owning_service} => String
    #   * {Types::VolumeAttachment#volume_id #volume_id} => String
    #   * {Types::VolumeAttachment#instance_id #instance_id} => String
    #   * {Types::VolumeAttachment#device #device} => String
    #   * {Types::VolumeAttachment#state #state} => String
    #   * {Types::VolumeAttachment#attach_time #attach_time} => Time
    #
    #
    # @example Example: To attach a volume to an instance
    #
    #   # This example attaches a volume (``vol-1234567890abcdef0``) to an instance (``i-01474ef662b89480``) as ``/dev/sdf``.
    #
    #   resp = client.attach_volume({
    #     device: "/dev/sdf", 
    #     instance_id: "i-01474ef662b89480", 
    #     volume_id: "vol-1234567890abcdef0", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     attach_time: Time.parse("2016-08-29T18:52:32.724Z"), 
    #     device: "/dev/sdf", 
    #     instance_id: "i-01474ef662b89480", 
    #     state: "attaching", 
    #     volume_id: "vol-1234567890abcdef0", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.attach_volume({
    #     device: "String", # required
    #     instance_id: "InstanceId", # required
    #     volume_id: "VolumeId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.delete_on_termination #=> Boolean
    #   resp.associated_resource #=> String
    #   resp.instance_owning_service #=> String
    #   resp.volume_id #=> String
    #   resp.instance_id #=> String
    #   resp.device #=> String
    #   resp.state #=> String, one of "attaching", "attached", "detaching", "detached", "busy"
    #   resp.attach_time #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AttachVolume AWS API Documentation
    #
    # @overload attach_volume(params = {})
    # @param [Hash] params ({})
    def attach_volume(params = {}, options = {})
      req = build_request(:attach_volume, params)
      req.send_request(options)
    end

    # Attaches an available virtual private gateway to a VPC. You can attach
    # one virtual private gateway to one VPC at a time.
    #
    # For more information, see [Amazon Web Services Site-to-Site VPN][1] in
    # the *Amazon Web Services Site-to-Site VPN User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @option params [required, String] :vpn_gateway_id
    #   The ID of the virtual private gateway.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AttachVpnGatewayResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AttachVpnGatewayResult#vpc_attachment #vpc_attachment} => Types::VpcAttachment
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.attach_vpn_gateway({
    #     vpc_id: "VpcId", # required
    #     vpn_gateway_id: "VpnGatewayId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc_attachment.vpc_id #=> String
    #   resp.vpc_attachment.state #=> String, one of "attaching", "attached", "detaching", "detached"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AttachVpnGateway AWS API Documentation
    #
    # @overload attach_vpn_gateway(params = {})
    # @param [Hash] params ({})
    def attach_vpn_gateway(params = {}, options = {})
      req = build_request(:attach_vpn_gateway, params)
      req.send_request(options)
    end

    # Adds an ingress authorization rule to a Client VPN endpoint. Ingress
    # authorization rules act as firewall rules that grant access to
    # networks. You must configure ingress authorization rules to enable
    # clients to access resources in Amazon Web Services or on-premises
    # networks.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint.
    #
    # @option params [required, String] :target_network_cidr
    #   The IPv4 address range, in CIDR notation, of the network for which
    #   access is being authorized.
    #
    # @option params [String] :access_group_id
    #   The ID of the group to grant access to, for example, the Active
    #   Directory group or identity provider (IdP) group. Required if
    #   `AuthorizeAllGroups` is `false` or not specified.
    #
    # @option params [Boolean] :authorize_all_groups
    #   Indicates whether to grant access to all clients. Specify `true` to
    #   grant all clients who successfully establish a VPN connection access
    #   to the network. Must be set to `true` if `AccessGroupId` is not
    #   specified.
    #
    # @option params [String] :description
    #   A brief description of the authorization rule.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AuthorizeClientVpnIngressResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AuthorizeClientVpnIngressResult#status #status} => Types::ClientVpnAuthorizationRuleStatus
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.authorize_client_vpn_ingress({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     target_network_cidr: "String", # required
    #     access_group_id: "String",
    #     authorize_all_groups: false,
    #     description: "String",
    #     client_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.status.code #=> String, one of "authorizing", "active", "failed", "revoking"
    #   resp.status.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AuthorizeClientVpnIngress AWS API Documentation
    #
    # @overload authorize_client_vpn_ingress(params = {})
    # @param [Hash] params ({})
    def authorize_client_vpn_ingress(params = {}, options = {})
      req = build_request(:authorize_client_vpn_ingress, params)
      req.send_request(options)
    end

    # Adds the specified outbound (egress) rules to a security group.
    #
    # An outbound rule permits instances to send traffic to the specified
    # IPv4 or IPv6 address ranges, the IP address ranges specified by a
    # prefix list, or the instances that are associated with a source
    # security group. For more information, see [Security group rules][1].
    #
    # You must specify exactly one of the following destinations: an IPv4 or
    # IPv6 address range, a prefix list, or a security group. You must
    # specify a protocol for each rule (for example, TCP). If the protocol
    # is TCP or UDP, you must also specify a port or port range. If the
    # protocol is ICMP or ICMPv6, you must also specify the ICMP type and
    # code.
    #
    # Rule changes are propagated to instances associated with the security
    # group as quickly as possible. However, a small delay might occur.
    #
    # For examples of rules that you can add to security groups for specific
    # access scenarios, see [Security group rules for different use
    # cases][2] in the *Amazon EC2 User Guide*.
    #
    # For information about security group quotas, see [Amazon VPC
    # quotas][3] in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/security-group-rules.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/security-group-rules-reference.html
    # [3]: https://docs.aws.amazon.com/vpc/latest/userguide/amazon-vpc-limits.html
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags applied to the security group rule.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :group_id
    #   The ID of the security group.
    #
    # @option params [String] :source_security_group_name
    #   Not supported. Use IP permissions instead.
    #
    # @option params [String] :source_security_group_owner_id
    #   Not supported. Use IP permissions instead.
    #
    # @option params [String] :ip_protocol
    #   Not supported. Use IP permissions instead.
    #
    # @option params [Integer] :from_port
    #   Not supported. Use IP permissions instead.
    #
    # @option params [Integer] :to_port
    #   Not supported. Use IP permissions instead.
    #
    # @option params [String] :cidr_ip
    #   Not supported. Use IP permissions instead.
    #
    # @option params [Array<Types::IpPermission>] :ip_permissions
    #   The permissions for the security group rules.
    #
    # @return [Types::AuthorizeSecurityGroupEgressResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AuthorizeSecurityGroupEgressResult#return #return} => Boolean
    #   * {Types::AuthorizeSecurityGroupEgressResult#security_group_rules #security_group_rules} => Array&lt;Types::SecurityGroupRule&gt;
    #
    #
    # @example Example: To add a rule that allows outbound traffic to a specific address range
    #
    #   # This example adds a rule that grants access to the specified address ranges on TCP port 80.
    #
    #   resp = client.authorize_security_group_egress({
    #     group_id: "sg-1a2b3c4d", 
    #     ip_permissions: [
    #       {
    #         from_port: 80, 
    #         ip_protocol: "tcp", 
    #         ip_ranges: [
    #           {
    #             cidr_ip: "10.0.0.0/16", 
    #           }, 
    #         ], 
    #         to_port: 80, 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Example: To add a rule that allows outbound traffic to a specific security group
    #
    #   # This example adds a rule that grants access to the specified security group on TCP port 80.
    #
    #   resp = client.authorize_security_group_egress({
    #     group_id: "sg-1a2b3c4d", 
    #     ip_permissions: [
    #       {
    #         from_port: 80, 
    #         ip_protocol: "tcp", 
    #         to_port: 80, 
    #         user_id_group_pairs: [
    #           {
    #             group_id: "sg-4b51a32f", 
    #           }, 
    #         ], 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.authorize_security_group_egress({
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #     group_id: "SecurityGroupId", # required
    #     source_security_group_name: "String",
    #     source_security_group_owner_id: "String",
    #     ip_protocol: "String",
    #     from_port: 1,
    #     to_port: 1,
    #     cidr_ip: "String",
    #     ip_permissions: [
    #       {
    #         ip_protocol: "String",
    #         from_port: 1,
    #         to_port: 1,
    #         user_id_group_pairs: [
    #           {
    #             description: "String",
    #             user_id: "String",
    #             group_name: "String",
    #             group_id: "String",
    #             vpc_id: "String",
    #             vpc_peering_connection_id: "String",
    #             peering_status: "String",
    #           },
    #         ],
    #         ip_ranges: [
    #           {
    #             description: "String",
    #             cidr_ip: "String",
    #           },
    #         ],
    #         ipv_6_ranges: [
    #           {
    #             description: "String",
    #             cidr_ipv_6: "String",
    #           },
    #         ],
    #         prefix_list_ids: [
    #           {
    #             description: "String",
    #             prefix_list_id: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #   resp.security_group_rules #=> Array
    #   resp.security_group_rules[0].security_group_rule_id #=> String
    #   resp.security_group_rules[0].group_id #=> String
    #   resp.security_group_rules[0].group_owner_id #=> String
    #   resp.security_group_rules[0].is_egress #=> Boolean
    #   resp.security_group_rules[0].ip_protocol #=> String
    #   resp.security_group_rules[0].from_port #=> Integer
    #   resp.security_group_rules[0].to_port #=> Integer
    #   resp.security_group_rules[0].cidr_ipv_4 #=> String
    #   resp.security_group_rules[0].cidr_ipv_6 #=> String
    #   resp.security_group_rules[0].prefix_list_id #=> String
    #   resp.security_group_rules[0].referenced_group_info.group_id #=> String
    #   resp.security_group_rules[0].referenced_group_info.peering_status #=> String
    #   resp.security_group_rules[0].referenced_group_info.user_id #=> String
    #   resp.security_group_rules[0].referenced_group_info.vpc_id #=> String
    #   resp.security_group_rules[0].referenced_group_info.vpc_peering_connection_id #=> String
    #   resp.security_group_rules[0].description #=> String
    #   resp.security_group_rules[0].tags #=> Array
    #   resp.security_group_rules[0].tags[0].key #=> String
    #   resp.security_group_rules[0].tags[0].value #=> String
    #   resp.security_group_rules[0].security_group_rule_arn #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AuthorizeSecurityGroupEgress AWS API Documentation
    #
    # @overload authorize_security_group_egress(params = {})
    # @param [Hash] params ({})
    def authorize_security_group_egress(params = {}, options = {})
      req = build_request(:authorize_security_group_egress, params)
      req.send_request(options)
    end

    # Adds the specified inbound (ingress) rules to a security group.
    #
    # An inbound rule permits instances to receive traffic from the
    # specified IPv4 or IPv6 address range, the IP address ranges that are
    # specified by a prefix list, or the instances that are associated with
    # a destination security group. For more information, see [Security
    # group rules][1].
    #
    # You must specify exactly one of the following sources: an IPv4 or IPv6
    # address range, a prefix list, or a security group. You must specify a
    # protocol for each rule (for example, TCP). If the protocol is TCP or
    # UDP, you must also specify a port or port range. If the protocol is
    # ICMP or ICMPv6, you must also specify the ICMP/ICMPv6 type and code.
    #
    # Rule changes are propagated to instances associated with the security
    # group as quickly as possible. However, a small delay might occur.
    #
    # For examples of rules that you can add to security groups for specific
    # access scenarios, see [Security group rules for different use
    # cases][2] in the *Amazon EC2 User Guide*.
    #
    # For more information about security group quotas, see [Amazon VPC
    # quotas][3] in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/security-group-rules.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/security-group-rules-reference.html
    # [3]: https://docs.aws.amazon.com/vpc/latest/userguide/amazon-vpc-limits.html
    #
    # @option params [String] :cidr_ip
    #   The IPv4 address range, in CIDR format.
    #
    #   <note markdown="1"> Amazon Web Services [canonicalizes][1] IPv4 and IPv6 CIDRs. For
    #   example, if you specify 100.68.0.18/18 for the CIDR block, Amazon Web
    #   Services canonicalizes the CIDR block to 100.68.0.0/18. Any subsequent
    #   DescribeSecurityGroups and DescribeSecurityGroupRules calls will
    #   return the canonicalized form of the CIDR block. Additionally, if you
    #   attempt to add another rule with the non-canonical form of the CIDR
    #   (such as 100.68.0.18/18) and there is already a rule for the
    #   canonicalized form of the CIDR block (such as 100.68.0.0/18), the API
    #   throws an duplicate rule error.
    #
    #    </note>
    #
    #   To specify an IPv6 address range, use IP permissions instead.
    #
    #   To specify multiple rules and descriptions for the rules, use IP
    #   permissions instead.
    #
    #
    #
    #   [1]: https://en.wikipedia.org/wiki/Canonicalization
    #
    # @option params [Integer] :from_port
    #   If the protocol is TCP or UDP, this is the start of the port range. If
    #   the protocol is ICMP, this is the ICMP type or -1 (all ICMP types).
    #
    #   To specify multiple rules and descriptions for the rules, use IP
    #   permissions instead.
    #
    # @option params [String] :group_id
    #   The ID of the security group.
    #
    # @option params [String] :group_name
    #   \[Default VPC\] The name of the security group. For security groups
    #   for a default VPC you can specify either the ID or the name of the
    #   security group. For security groups for a nondefault VPC, you must
    #   specify the ID of the security group.
    #
    # @option params [Array<Types::IpPermission>] :ip_permissions
    #   The permissions for the security group rules.
    #
    # @option params [String] :ip_protocol
    #   The IP protocol name (`tcp`, `udp`, `icmp`) or number (see [Protocol
    #   Numbers][1]). To specify all protocols, use `-1`.
    #
    #   To specify `icmpv6`, use IP permissions instead.
    #
    #   If you specify a protocol other than one of the supported values,
    #   traffic is allowed on all ports, regardless of any ports that you
    #   specify.
    #
    #   To specify multiple rules and descriptions for the rules, use IP
    #   permissions instead.
    #
    #
    #
    #   [1]: http://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml
    #
    # @option params [String] :source_security_group_name
    #   \[Default VPC\] The name of the source security group.
    #
    #   The rule grants full ICMP, UDP, and TCP access. To create a rule with
    #   a specific protocol and port range, specify a set of IP permissions
    #   instead.
    #
    # @option params [String] :source_security_group_owner_id
    #   The Amazon Web Services account ID for the source security group, if
    #   the source security group is in a different account.
    #
    #   The rule grants full ICMP, UDP, and TCP access. To create a rule with
    #   a specific protocol and port range, use IP permissions instead.
    #
    # @option params [Integer] :to_port
    #   If the protocol is TCP or UDP, this is the end of the port range. If
    #   the protocol is ICMP, this is the ICMP code or -1 (all ICMP codes). If
    #   the start port is -1 (all ICMP types), then the end port must be -1
    #   (all ICMP codes).
    #
    #   To specify multiple rules and descriptions for the rules, use IP
    #   permissions instead.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags applied to the security group rule.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::AuthorizeSecurityGroupIngressResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::AuthorizeSecurityGroupIngressResult#return #return} => Boolean
    #   * {Types::AuthorizeSecurityGroupIngressResult#security_group_rules #security_group_rules} => Array&lt;Types::SecurityGroupRule&gt;
    #
    #
    # @example Example: To add a rule that allows inbound SSH traffic from an IPv4 address range
    #
    #   # This example enables inbound traffic on TCP port 22 (SSH). The rule includes a description to help you identify it
    #   # later.
    #
    #   resp = client.authorize_security_group_ingress({
    #     group_id: "sg-903004f8", 
    #     ip_permissions: [
    #       {
    #         from_port: 22, 
    #         ip_protocol: "tcp", 
    #         ip_ranges: [
    #           {
    #             cidr_ip: "203.0.113.0/24", 
    #             description: "SSH access from the LA office", 
    #           }, 
    #         ], 
    #         to_port: 22, 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Example: To add a rule that allows inbound HTTP traffic from another security group
    #
    #   # This example enables inbound traffic on TCP port 80 from the specified security group. The group must be in the same VPC
    #   # or a peer VPC. Incoming traffic is allowed based on the private IP addresses of instances that are associated with the
    #   # specified security group.
    #
    #   resp = client.authorize_security_group_ingress({
    #     group_id: "sg-111aaa22", 
    #     ip_permissions: [
    #       {
    #         from_port: 80, 
    #         ip_protocol: "tcp", 
    #         to_port: 80, 
    #         user_id_group_pairs: [
    #           {
    #             description: "HTTP access from other instances", 
    #             group_id: "sg-1a2b3c4d", 
    #           }, 
    #         ], 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Example: To add a rule that allows inbound RDP traffic from an IPv6 address range
    #
    #   # This example adds an inbound rule that allows RDP traffic from the specified IPv6 address range. The rule includes a
    #   # description to help you identify it later.
    #
    #   resp = client.authorize_security_group_ingress({
    #     group_id: "sg-123abc12 ", 
    #     ip_permissions: [
    #       {
    #         from_port: 3389, 
    #         ip_protocol: "tcp", 
    #         ipv_6_ranges: [
    #           {
    #             cidr_ipv_6: "2001:db8:1234:1a00::/64", 
    #             description: "RDP access from the NY office", 
    #           }, 
    #         ], 
    #         to_port: 3389, 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.authorize_security_group_ingress({
    #     cidr_ip: "String",
    #     from_port: 1,
    #     group_id: "SecurityGroupId",
    #     group_name: "SecurityGroupName",
    #     ip_permissions: [
    #       {
    #         ip_protocol: "String",
    #         from_port: 1,
    #         to_port: 1,
    #         user_id_group_pairs: [
    #           {
    #             description: "String",
    #             user_id: "String",
    #             group_name: "String",
    #             group_id: "String",
    #             vpc_id: "String",
    #             vpc_peering_connection_id: "String",
    #             peering_status: "String",
    #           },
    #         ],
    #         ip_ranges: [
    #           {
    #             description: "String",
    #             cidr_ip: "String",
    #           },
    #         ],
    #         ipv_6_ranges: [
    #           {
    #             description: "String",
    #             cidr_ipv_6: "String",
    #           },
    #         ],
    #         prefix_list_ids: [
    #           {
    #             description: "String",
    #             prefix_list_id: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     ip_protocol: "String",
    #     source_security_group_name: "String",
    #     source_security_group_owner_id: "String",
    #     to_port: 1,
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #   resp.security_group_rules #=> Array
    #   resp.security_group_rules[0].security_group_rule_id #=> String
    #   resp.security_group_rules[0].group_id #=> String
    #   resp.security_group_rules[0].group_owner_id #=> String
    #   resp.security_group_rules[0].is_egress #=> Boolean
    #   resp.security_group_rules[0].ip_protocol #=> String
    #   resp.security_group_rules[0].from_port #=> Integer
    #   resp.security_group_rules[0].to_port #=> Integer
    #   resp.security_group_rules[0].cidr_ipv_4 #=> String
    #   resp.security_group_rules[0].cidr_ipv_6 #=> String
    #   resp.security_group_rules[0].prefix_list_id #=> String
    #   resp.security_group_rules[0].referenced_group_info.group_id #=> String
    #   resp.security_group_rules[0].referenced_group_info.peering_status #=> String
    #   resp.security_group_rules[0].referenced_group_info.user_id #=> String
    #   resp.security_group_rules[0].referenced_group_info.vpc_id #=> String
    #   resp.security_group_rules[0].referenced_group_info.vpc_peering_connection_id #=> String
    #   resp.security_group_rules[0].description #=> String
    #   resp.security_group_rules[0].tags #=> Array
    #   resp.security_group_rules[0].tags[0].key #=> String
    #   resp.security_group_rules[0].tags[0].value #=> String
    #   resp.security_group_rules[0].security_group_rule_arn #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/AuthorizeSecurityGroupIngress AWS API Documentation
    #
    # @overload authorize_security_group_ingress(params = {})
    # @param [Hash] params ({})
    def authorize_security_group_ingress(params = {}, options = {})
      req = build_request(:authorize_security_group_ingress, params)
      req.send_request(options)
    end

    # Bundles an Amazon instance store-backed Windows instance.
    #
    # During bundling, only the root device volume (C:\\) is bundled. Data
    # on other instance store volumes is not preserved.
    #
    # <note markdown="1"> This action is not applicable for Linux/Unix instances or Windows
    # instances that are backed by Amazon EBS.
    #
    #  </note>
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance to bundle.
    #
    #   Default: None
    #
    # @option params [required, Types::Storage] :storage
    #   The bucket in which to store the AMI. You can specify a bucket that
    #   you already own or a new bucket that Amazon EC2 creates on your
    #   behalf. If you specify a bucket that belongs to someone else, Amazon
    #   EC2 returns an error.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::BundleInstanceResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::BundleInstanceResult#bundle_task #bundle_task} => Types::BundleTask
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.bundle_instance({
    #     instance_id: "InstanceId", # required
    #     storage: { # required
    #       s3: {
    #         aws_access_key_id: "String",
    #         bucket: "String",
    #         prefix: "String",
    #         upload_policy: "data",
    #         upload_policy_signature: "S3StorageUploadPolicySignature",
    #       },
    #     },
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.bundle_task.instance_id #=> String
    #   resp.bundle_task.bundle_id #=> String
    #   resp.bundle_task.state #=> String, one of "pending", "waiting-for-shutdown", "bundling", "storing", "cancelling", "complete", "failed"
    #   resp.bundle_task.start_time #=> Time
    #   resp.bundle_task.update_time #=> Time
    #   resp.bundle_task.storage.s3.aws_access_key_id #=> String
    #   resp.bundle_task.storage.s3.bucket #=> String
    #   resp.bundle_task.storage.s3.prefix #=> String
    #   resp.bundle_task.storage.s3.upload_policy #=> String
    #   resp.bundle_task.storage.s3.upload_policy_signature #=> String
    #   resp.bundle_task.progress #=> String
    #   resp.bundle_task.bundle_task_error.code #=> String
    #   resp.bundle_task.bundle_task_error.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/BundleInstance AWS API Documentation
    #
    # @overload bundle_instance(params = {})
    # @param [Hash] params ({})
    def bundle_instance(params = {}, options = {})
      req = build_request(:bundle_instance, params)
      req.send_request(options)
    end

    # Cancels a bundling operation for an instance store-backed Windows
    # instance.
    #
    # @option params [required, String] :bundle_id
    #   The ID of the bundle task.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CancelBundleTaskResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CancelBundleTaskResult#bundle_task #bundle_task} => Types::BundleTask
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.cancel_bundle_task({
    #     bundle_id: "BundleId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.bundle_task.instance_id #=> String
    #   resp.bundle_task.bundle_id #=> String
    #   resp.bundle_task.state #=> String, one of "pending", "waiting-for-shutdown", "bundling", "storing", "cancelling", "complete", "failed"
    #   resp.bundle_task.start_time #=> Time
    #   resp.bundle_task.update_time #=> Time
    #   resp.bundle_task.storage.s3.aws_access_key_id #=> String
    #   resp.bundle_task.storage.s3.bucket #=> String
    #   resp.bundle_task.storage.s3.prefix #=> String
    #   resp.bundle_task.storage.s3.upload_policy #=> String
    #   resp.bundle_task.storage.s3.upload_policy_signature #=> String
    #   resp.bundle_task.progress #=> String
    #   resp.bundle_task.bundle_task_error.code #=> String
    #   resp.bundle_task.bundle_task_error.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CancelBundleTask AWS API Documentation
    #
    # @overload cancel_bundle_task(params = {})
    # @param [Hash] params ({})
    def cancel_bundle_task(params = {}, options = {})
      req = build_request(:cancel_bundle_task, params)
      req.send_request(options)
    end

    # Cancels the specified Capacity Reservation, releases the reserved
    # capacity, and changes the Capacity Reservation's state to
    # `cancelled`.
    #
    # You can cancel a Capacity Reservation that is in the following states:
    #
    # * `assessing`
    #
    # * `active` and there is no commitment duration or the commitment
    #   duration has elapsed. You can't cancel a future-dated Capacity
    #   Reservation during the commitment duration.
    #
    # <note markdown="1"> You can't modify or cancel a Capacity Block. For more information,
    # see [Capacity Blocks for ML][1].
    #
    #  </note>
    #
    # If a future-dated Capacity Reservation enters the `delayed` state, the
    # commitment duration is waived, and you can cancel it as soon as it
    # enters the `active` state.
    #
    # Instances running in the reserved capacity continue running until you
    # stop them. Stopped instances that target the Capacity Reservation can
    # no longer launch. Modify these instances to either target a different
    # Capacity Reservation, launch On-Demand Instance capacity, or run in
    # any open Capacity Reservation that has matching attributes and
    # sufficient capacity.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-capacity-blocks.html
    #
    # @option params [required, String] :capacity_reservation_id
    #   The ID of the Capacity Reservation to be cancelled.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CancelCapacityReservationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CancelCapacityReservationResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.cancel_capacity_reservation({
    #     capacity_reservation_id: "CapacityReservationId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CancelCapacityReservation AWS API Documentation
    #
    # @overload cancel_capacity_reservation(params = {})
    # @param [Hash] params ({})
    def cancel_capacity_reservation(params = {}, options = {})
      req = build_request(:cancel_capacity_reservation, params)
      req.send_request(options)
    end

    # Cancels one or more Capacity Reservation Fleets. When you cancel a
    # Capacity Reservation Fleet, the following happens:
    #
    # * The Capacity Reservation Fleet's status changes to `cancelled`.
    #
    # * The individual Capacity Reservations in the Fleet are cancelled.
    #   Instances running in the Capacity Reservations at the time of
    #   cancelling the Fleet continue to run in shared capacity.
    #
    # * The Fleet stops creating new Capacity Reservations.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :capacity_reservation_fleet_ids
    #   The IDs of the Capacity Reservation Fleets to cancel.
    #
    # @return [Types::CancelCapacityReservationFleetsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CancelCapacityReservationFleetsResult#successful_fleet_cancellations #successful_fleet_cancellations} => Array&lt;Types::CapacityReservationFleetCancellationState&gt;
    #   * {Types::CancelCapacityReservationFleetsResult#failed_fleet_cancellations #failed_fleet_cancellations} => Array&lt;Types::FailedCapacityReservationFleetCancellationResult&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.cancel_capacity_reservation_fleets({
    #     dry_run: false,
    #     capacity_reservation_fleet_ids: ["CapacityReservationFleetId"], # required
    #   })
    #
    # @example Response structure
    #
    #   resp.successful_fleet_cancellations #=> Array
    #   resp.successful_fleet_cancellations[0].current_fleet_state #=> String, one of "submitted", "modifying", "active", "partially_fulfilled", "expiring", "expired", "cancelling", "cancelled", "failed"
    #   resp.successful_fleet_cancellations[0].previous_fleet_state #=> String, one of "submitted", "modifying", "active", "partially_fulfilled", "expiring", "expired", "cancelling", "cancelled", "failed"
    #   resp.successful_fleet_cancellations[0].capacity_reservation_fleet_id #=> String
    #   resp.failed_fleet_cancellations #=> Array
    #   resp.failed_fleet_cancellations[0].capacity_reservation_fleet_id #=> String
    #   resp.failed_fleet_cancellations[0].cancel_capacity_reservation_fleet_error.code #=> String
    #   resp.failed_fleet_cancellations[0].cancel_capacity_reservation_fleet_error.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CancelCapacityReservationFleets AWS API Documentation
    #
    # @overload cancel_capacity_reservation_fleets(params = {})
    # @param [Hash] params ({})
    def cancel_capacity_reservation_fleets(params = {}, options = {})
      req = build_request(:cancel_capacity_reservation_fleets, params)
      req.send_request(options)
    end

    # Cancels an active conversion task. The task can be the import of an
    # instance or volume. The action removes all artifacts of the
    # conversion, including a partially uploaded volume or instance. If the
    # conversion is complete or is in the process of transferring the final
    # disk image, the command fails and returns an exception.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :conversion_task_id
    #   The ID of the conversion task.
    #
    # @option params [String] :reason_message
    #   The reason for canceling the conversion task.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.cancel_conversion_task({
    #     dry_run: false,
    #     conversion_task_id: "ConversionTaskId", # required
    #     reason_message: "String",
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CancelConversionTask AWS API Documentation
    #
    # @overload cancel_conversion_task(params = {})
    # @param [Hash] params ({})
    def cancel_conversion_task(params = {}, options = {})
      req = build_request(:cancel_conversion_task, params)
      req.send_request(options)
    end

    # Cancels the generation of an account status report.
    #
    # You can only cancel a report while it has the `running` status.
    # Reports with other statuses (`complete`, `cancelled`, or `error`)
    # can't be canceled.
    #
    # For more information, see [Generating the account status report for
    # declarative policies][1] in the *Amazon Web Services Organizations
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_policies_declarative_status-report.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :report_id
    #   The ID of the report.
    #
    # @return [Types::CancelDeclarativePoliciesReportResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CancelDeclarativePoliciesReportResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.cancel_declarative_policies_report({
    #     dry_run: false,
    #     report_id: "DeclarativePoliciesReportId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CancelDeclarativePoliciesReport AWS API Documentation
    #
    # @overload cancel_declarative_policies_report(params = {})
    # @param [Hash] params ({})
    def cancel_declarative_policies_report(params = {}, options = {})
      req = build_request(:cancel_declarative_policies_report, params)
      req.send_request(options)
    end

    # Cancels an active export task. The request removes all artifacts of
    # the export, including any partially-created Amazon S3 objects. If the
    # export task is complete or is in the process of transferring the final
    # disk image, the command fails and returns an error.
    #
    # @option params [required, String] :export_task_id
    #   The ID of the export task. This is the ID returned by the
    #   `CreateInstanceExportTask` and `ExportImage` operations.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.cancel_export_task({
    #     export_task_id: "ExportVmTaskId", # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CancelExportTask AWS API Documentation
    #
    # @overload cancel_export_task(params = {})
    # @param [Hash] params ({})
    def cancel_export_task(params = {}, options = {})
      req = build_request(:cancel_export_task, params)
      req.send_request(options)
    end

    # Removes your Amazon Web Services account from the launch permissions
    # for the specified AMI. For more information, see [ Cancel having an
    # AMI shared with your Amazon Web Services account][1] in the *Amazon
    # EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/cancel-sharing-an-AMI.html
    #
    # @option params [required, String] :image_id
    #   The ID of the AMI that was shared with your Amazon Web Services
    #   account.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CancelImageLaunchPermissionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CancelImageLaunchPermissionResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.cancel_image_launch_permission({
    #     image_id: "ImageId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CancelImageLaunchPermission AWS API Documentation
    #
    # @overload cancel_image_launch_permission(params = {})
    # @param [Hash] params ({})
    def cancel_image_launch_permission(params = {}, options = {})
      req = build_request(:cancel_image_launch_permission, params)
      req.send_request(options)
    end

    # Cancels an in-process import virtual machine or import snapshot task.
    #
    # @option params [String] :cancel_reason
    #   The reason for canceling the task.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :import_task_id
    #   The ID of the import image or import snapshot task to be canceled.
    #
    # @return [Types::CancelImportTaskResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CancelImportTaskResult#import_task_id #import_task_id} => String
    #   * {Types::CancelImportTaskResult#previous_state #previous_state} => String
    #   * {Types::CancelImportTaskResult#state #state} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.cancel_import_task({
    #     cancel_reason: "String",
    #     dry_run: false,
    #     import_task_id: "ImportTaskId",
    #   })
    #
    # @example Response structure
    #
    #   resp.import_task_id #=> String
    #   resp.previous_state #=> String
    #   resp.state #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CancelImportTask AWS API Documentation
    #
    # @overload cancel_import_task(params = {})
    # @param [Hash] params ({})
    def cancel_import_task(params = {}, options = {})
      req = build_request(:cancel_import_task, params)
      req.send_request(options)
    end

    # Cancels the specified Reserved Instance listing in the Reserved
    # Instance Marketplace.
    #
    # For more information, see [Sell in the Reserved Instance
    # Marketplace][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ri-market-general.html
    #
    # @option params [required, String] :reserved_instances_listing_id
    #   The ID of the Reserved Instance listing.
    #
    # @return [Types::CancelReservedInstancesListingResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CancelReservedInstancesListingResult#reserved_instances_listings #reserved_instances_listings} => Array&lt;Types::ReservedInstancesListing&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.cancel_reserved_instances_listing({
    #     reserved_instances_listing_id: "ReservedInstancesListingId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.reserved_instances_listings #=> Array
    #   resp.reserved_instances_listings[0].client_token #=> String
    #   resp.reserved_instances_listings[0].create_date #=> Time
    #   resp.reserved_instances_listings[0].instance_counts #=> Array
    #   resp.reserved_instances_listings[0].instance_counts[0].instance_count #=> Integer
    #   resp.reserved_instances_listings[0].instance_counts[0].state #=> String, one of "available", "sold", "cancelled", "pending"
    #   resp.reserved_instances_listings[0].price_schedules #=> Array
    #   resp.reserved_instances_listings[0].price_schedules[0].active #=> Boolean
    #   resp.reserved_instances_listings[0].price_schedules[0].currency_code #=> String, one of "USD"
    #   resp.reserved_instances_listings[0].price_schedules[0].price #=> Float
    #   resp.reserved_instances_listings[0].price_schedules[0].term #=> Integer
    #   resp.reserved_instances_listings[0].reserved_instances_id #=> String
    #   resp.reserved_instances_listings[0].reserved_instances_listing_id #=> String
    #   resp.reserved_instances_listings[0].status #=> String, one of "active", "pending", "cancelled", "closed"
    #   resp.reserved_instances_listings[0].status_message #=> String
    #   resp.reserved_instances_listings[0].tags #=> Array
    #   resp.reserved_instances_listings[0].tags[0].key #=> String
    #   resp.reserved_instances_listings[0].tags[0].value #=> String
    #   resp.reserved_instances_listings[0].update_date #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CancelReservedInstancesListing AWS API Documentation
    #
    # @overload cancel_reserved_instances_listing(params = {})
    # @param [Hash] params ({})
    def cancel_reserved_instances_listing(params = {}, options = {})
      req = build_request(:cancel_reserved_instances_listing, params)
      req.send_request(options)
    end

    # Cancels the specified Spot Fleet requests.
    #
    # After you cancel a Spot Fleet request, the Spot Fleet launches no new
    # instances.
    #
    # You must also specify whether a canceled Spot Fleet request should
    # terminate its instances. If you choose to terminate the instances, the
    # Spot Fleet request enters the `cancelled_terminating` state.
    # Otherwise, the Spot Fleet request enters the `cancelled_running` state
    # and the instances continue to run until they are interrupted or you
    # terminate them manually.
    #
    # **Restrictions**
    #
    # * You can delete up to 100 fleets in a single request. If you exceed
    #   the specified number, no fleets are deleted.
    #
    # ^
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :spot_fleet_request_ids
    #   The IDs of the Spot Fleet requests.
    #
    #   Constraint: You can specify up to 100 IDs in a single request.
    #
    # @option params [required, Boolean] :terminate_instances
    #   Indicates whether to terminate the associated instances when the Spot
    #   Fleet request is canceled. The default is to terminate the instances.
    #
    #   To let the instances continue to run after the Spot Fleet request is
    #   canceled, specify `no-terminate-instances`.
    #
    # @return [Types::CancelSpotFleetRequestsResponse] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CancelSpotFleetRequestsResponse#successful_fleet_requests #successful_fleet_requests} => Array&lt;Types::CancelSpotFleetRequestsSuccessItem&gt;
    #   * {Types::CancelSpotFleetRequestsResponse#unsuccessful_fleet_requests #unsuccessful_fleet_requests} => Array&lt;Types::CancelSpotFleetRequestsErrorItem&gt;
    #
    #
    # @example Example: To cancel a Spot fleet request
    #
    #   # This example cancels the specified Spot fleet request and terminates its associated Spot Instances.
    #
    #   resp = client.cancel_spot_fleet_requests({
    #     spot_fleet_request_ids: [
    #       "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #     ], 
    #     terminate_instances: true, 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     successful_fleet_requests: [
    #       {
    #         current_spot_fleet_request_state: "cancelled_running", 
    #         previous_spot_fleet_request_state: "active", 
    #         spot_fleet_request_id: "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Example: To cancel a Spot fleet request without terminating its Spot Instances
    #
    #   # This example cancels the specified Spot fleet request without terminating its associated Spot Instances.
    #
    #   resp = client.cancel_spot_fleet_requests({
    #     spot_fleet_request_ids: [
    #       "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #     ], 
    #     terminate_instances: false, 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     successful_fleet_requests: [
    #       {
    #         current_spot_fleet_request_state: "cancelled_terminating", 
    #         previous_spot_fleet_request_state: "active", 
    #         spot_fleet_request_id: "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.cancel_spot_fleet_requests({
    #     dry_run: false,
    #     spot_fleet_request_ids: ["SpotFleetRequestId"], # required
    #     terminate_instances: false, # required
    #   })
    #
    # @example Response structure
    #
    #   resp.successful_fleet_requests #=> Array
    #   resp.successful_fleet_requests[0].current_spot_fleet_request_state #=> String, one of "submitted", "active", "cancelled", "failed", "cancelled_running", "cancelled_terminating", "modifying"
    #   resp.successful_fleet_requests[0].previous_spot_fleet_request_state #=> String, one of "submitted", "active", "cancelled", "failed", "cancelled_running", "cancelled_terminating", "modifying"
    #   resp.successful_fleet_requests[0].spot_fleet_request_id #=> String
    #   resp.unsuccessful_fleet_requests #=> Array
    #   resp.unsuccessful_fleet_requests[0].error.code #=> String, one of "fleetRequestIdDoesNotExist", "fleetRequestIdMalformed", "fleetRequestNotInCancellableState", "unexpectedError"
    #   resp.unsuccessful_fleet_requests[0].error.message #=> String
    #   resp.unsuccessful_fleet_requests[0].spot_fleet_request_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CancelSpotFleetRequests AWS API Documentation
    #
    # @overload cancel_spot_fleet_requests(params = {})
    # @param [Hash] params ({})
    def cancel_spot_fleet_requests(params = {}, options = {})
      req = build_request(:cancel_spot_fleet_requests, params)
      req.send_request(options)
    end

    # Cancels one or more Spot Instance requests.
    #
    # Canceling a Spot Instance request does not terminate running Spot
    # Instances associated with the request.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :spot_instance_request_ids
    #   The IDs of the Spot Instance requests.
    #
    # @return [Types::CancelSpotInstanceRequestsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CancelSpotInstanceRequestsResult#cancelled_spot_instance_requests #cancelled_spot_instance_requests} => Array&lt;Types::CancelledSpotInstanceRequest&gt;
    #
    #
    # @example Example: To cancel Spot Instance requests
    #
    #   # This example cancels a Spot Instance request.
    #
    #   resp = client.cancel_spot_instance_requests({
    #     spot_instance_request_ids: [
    #       "sir-08b93456", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     cancelled_spot_instance_requests: [
    #       {
    #         spot_instance_request_id: "sir-08b93456", 
    #         state: "cancelled", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.cancel_spot_instance_requests({
    #     dry_run: false,
    #     spot_instance_request_ids: ["SpotInstanceRequestId"], # required
    #   })
    #
    # @example Response structure
    #
    #   resp.cancelled_spot_instance_requests #=> Array
    #   resp.cancelled_spot_instance_requests[0].spot_instance_request_id #=> String
    #   resp.cancelled_spot_instance_requests[0].state #=> String, one of "active", "open", "closed", "cancelled", "completed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CancelSpotInstanceRequests AWS API Documentation
    #
    # @overload cancel_spot_instance_requests(params = {})
    # @param [Hash] params ({})
    def cancel_spot_instance_requests(params = {}, options = {})
      req = build_request(:cancel_spot_instance_requests, params)
      req.send_request(options)
    end

    # Determines whether a product code is associated with an instance. This
    # action can only be used by the owner of the product code. It is useful
    # when a product code owner must verify whether another user's instance
    # is eligible for support.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @option params [required, String] :product_code
    #   The product code. This must be a product code that you own.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ConfirmProductInstanceResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ConfirmProductInstanceResult#return #return} => Boolean
    #   * {Types::ConfirmProductInstanceResult#owner_id #owner_id} => String
    #
    #
    # @example Example: To confirm the product instance
    #
    #   # This example determines whether the specified product code is associated with the specified instance.
    #
    #   resp = client.confirm_product_instance({
    #     instance_id: "i-1234567890abcdef0", 
    #     product_code: "774F4FF8", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     owner_id: "123456789012", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.confirm_product_instance({
    #     instance_id: "InstanceId", # required
    #     product_code: "String", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #   resp.owner_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ConfirmProductInstance AWS API Documentation
    #
    # @overload confirm_product_instance(params = {})
    # @param [Hash] params ({})
    def confirm_product_instance(params = {}, options = {})
      req = build_request(:confirm_product_instance, params)
      req.send_request(options)
    end

    # Copies the specified Amazon FPGA Image (AFI) to the current Region.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :source_fpga_image_id
    #   The ID of the source AFI.
    #
    # @option params [String] :description
    #   The description for the new AFI.
    #
    # @option params [String] :name
    #   The name for the new AFI. The default is the name of the source AFI.
    #
    # @option params [required, String] :source_region
    #   The Region that contains the source AFI.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @return [Types::CopyFpgaImageResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CopyFpgaImageResult#fpga_image_id #fpga_image_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.copy_fpga_image({
    #     dry_run: false,
    #     source_fpga_image_id: "String", # required
    #     description: "String",
    #     name: "String",
    #     source_region: "String", # required
    #     client_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.fpga_image_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CopyFpgaImage AWS API Documentation
    #
    # @overload copy_fpga_image(params = {})
    # @param [Hash] params ({})
    def copy_fpga_image(params = {}, options = {})
      req = build_request(:copy_fpga_image, params)
      req.send_request(options)
    end

    # Initiates an AMI copy operation. You can copy an AMI from one Region
    # to another, or from a Region to an Outpost. You can't copy an AMI
    # from an Outpost to a Region, from one Outpost to another, or within
    # the same Outpost. To copy an AMI to another partition, see
    # [CreateStoreImageTask][1].
    #
    # When you copy an AMI from one Region to another, the destination
    # Region is the current Region.
    #
    # When you copy an AMI from a Region to an Outpost, specify the ARN of
    # the Outpost as the destination. Backing snapshots copied to an Outpost
    # are encrypted by default using the default encryption key for the
    # Region or the key that you specify. Outposts do not support
    # unencrypted snapshots.
    #
    # For information about the prerequisites when copying an AMI, see [Copy
    # an AMI][2] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateStoreImageTask.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/CopyingAMIs.html
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier you provide to ensure idempotency of
    #   the request. For more information, see [Ensuring idempotency][1] in
    #   the *Amazon EC2 API Reference*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [String] :description
    #   A description for the new AMI in the destination Region.
    #
    # @option params [Boolean] :encrypted
    #   Specifies whether the destination snapshots of the copied image should
    #   be encrypted. You can encrypt a copy of an unencrypted snapshot, but
    #   you cannot create an unencrypted copy of an encrypted snapshot. The
    #   default KMS key for Amazon EBS is used unless you specify a
    #   non-default Key Management Service (KMS) KMS key using `KmsKeyId`. For
    #   more information, see [Use encryption with EBS-backed AMIs][1] in the
    #   *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/AMIEncryption.html
    #
    # @option params [String] :kms_key_id
    #   The identifier of the symmetric Key Management Service (KMS) KMS key
    #   to use when creating encrypted volumes. If this parameter is not
    #   specified, your Amazon Web Services managed KMS key for Amazon EBS is
    #   used. If you specify a KMS key, you must also set the encrypted state
    #   to `true`.
    #
    #   You can specify a KMS key using any of the following:
    #
    #   * Key ID. For example, 1234abcd-12ab-34cd-56ef-1234567890ab.
    #
    #   * Key alias. For example, alias/ExampleAlias.
    #
    #   * Key ARN. For example,
    #     arn:aws:kms:us-east-1:012345678910:key/1234abcd-12ab-34cd-56ef-1234567890ab.
    #
    #   * Alias ARN. For example,
    #     arn:aws:kms:us-east-1:012345678910:alias/ExampleAlias.
    #
    #   Amazon Web Services authenticates the KMS key asynchronously.
    #   Therefore, if you specify an identifier that is not valid, the action
    #   can appear to complete, but eventually fails.
    #
    #   The specified KMS key must exist in the destination Region.
    #
    #   Amazon EBS does not support asymmetric KMS keys.
    #
    # @option params [required, String] :name
    #   The name of the new AMI in the destination Region.
    #
    # @option params [required, String] :source_image_id
    #   The ID of the AMI to copy.
    #
    # @option params [required, String] :source_region
    #   The name of the Region that contains the AMI to copy.
    #
    # @option params [String] :destination_outpost_arn
    #   The Amazon Resource Name (ARN) of the Outpost to which to copy the
    #   AMI. Only specify this parameter when copying an AMI from an Amazon
    #   Web Services Region to an Outpost. The AMI must be in the Region of
    #   the destination Outpost. You cannot copy an AMI from an Outpost to a
    #   Region, from one Outpost to another, or within the same Outpost.
    #
    #   For more information, see [Copy AMIs from an Amazon Web Services
    #   Region to an Outpost][1] in the *Amazon EBS User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ebs/latest/userguide/snapshots-outposts.html#copy-amis
    #
    # @option params [Boolean] :copy_image_tags
    #   Indicates whether to include your user-defined AMI tags when copying
    #   the AMI.
    #
    #   The following tags will not be copied:
    #
    #   * System tags (prefixed with `aws:`)
    #
    #   * For public and shared AMIs, user-defined tags that are attached by
    #     other Amazon Web Services accounts
    #
    #   Default: Your user-defined AMI tags are not copied.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the new AMI and new snapshots. You can tag the
    #   AMI, the snapshots, or both.
    #
    #   * To tag the new AMI, the value for `ResourceType` must be `image`.
    #
    #   * To tag the new snapshots, the value for `ResourceType` must be
    #     `snapshot`. The same tag is applied to all the new snapshots.
    #
    #   If you specify other values for `ResourceType`, the request fails.
    #
    #   To tag an AMI or snapshot after it has been created, see
    #   [CreateTags][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTags.html
    #
    # @option params [Integer] :snapshot_copy_completion_duration_minutes
    #   Specify a completion duration, in 15 minute increments, to initiate a
    #   time-based AMI copy. The specified completion duration applies to each
    #   of the snapshots associated with the AMI. Each snapshot associated
    #   with the AMI will be completed within the specified completion
    #   duration, regardless of their size.
    #
    #   If you do not specify a value, the AMI copy operation is completed on
    #   a best-effort basis.
    #
    #   For more information, see [ Time-based copies][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ebs/latest/userguide/time-based-copies.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CopyImageResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CopyImageResult#image_id #image_id} => String
    #
    #
    # @example Example: To copy an AMI to another region
    #
    #   # This example copies the specified AMI from the us-east-1 region to the current region.
    #
    #   resp = client.copy_image({
    #     description: "", 
    #     name: "My server", 
    #     source_image_id: "ami-5731123e", 
    #     source_region: "us-east-1", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     image_id: "ami-438bea42", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.copy_image({
    #     client_token: "String",
    #     description: "String",
    #     encrypted: false,
    #     kms_key_id: "KmsKeyId",
    #     name: "String", # required
    #     source_image_id: "String", # required
    #     source_region: "String", # required
    #     destination_outpost_arn: "String",
    #     copy_image_tags: false,
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     snapshot_copy_completion_duration_minutes: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.image_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CopyImage AWS API Documentation
    #
    # @overload copy_image(params = {})
    # @param [Hash] params ({})
    def copy_image(params = {}, options = {})
      req = build_request(:copy_image, params)
      req.send_request(options)
    end

    # Copies a point-in-time snapshot of an EBS volume and stores it in
    # Amazon S3. You can copy a snapshot within the same Region, from one
    # Region to another, or from a Region to an Outpost. You can't copy a
    # snapshot from an Outpost to a Region, from one Outpost to another, or
    # within the same Outpost.
    #
    # You can use the snapshot to create EBS volumes or Amazon Machine
    # Images (AMIs).
    #
    # When copying snapshots to a Region, copies of encrypted EBS snapshots
    # remain encrypted. Copies of unencrypted snapshots remain unencrypted,
    # unless you enable encryption for the snapshot copy operation. By
    # default, encrypted snapshot copies use the default KMS key; however,
    # you can specify a different KMS key. To copy an encrypted snapshot
    # that has been shared from another account, you must have permissions
    # for the KMS key used to encrypt the snapshot.
    #
    # Snapshots copied to an Outpost are encrypted by default using the
    # default encryption key for the Region, or a different key that you
    # specify in the request using **KmsKeyId**. Outposts do not support
    # unencrypted snapshots. For more information, see [Amazon EBS local
    # snapshots on Outposts][1] in the *Amazon EBS User Guide*.
    #
    # Snapshots created by copying another snapshot have an arbitrary volume
    # ID that should not be used for any purpose.
    #
    # For more information, see [Copy an Amazon EBS snapshot][2] in the
    # *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/snapshots-outposts.html#ami
    # [2]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-copy-snapshot.html
    #
    # @option params [String] :description
    #   A description for the EBS snapshot.
    #
    # @option params [String] :destination_outpost_arn
    #   The Amazon Resource Name (ARN) of the Outpost to which to copy the
    #   snapshot. Only specify this parameter when copying a snapshot from an
    #   Amazon Web Services Region to an Outpost. The snapshot must be in the
    #   Region for the destination Outpost. You cannot copy a snapshot from an
    #   Outpost to a Region, from one Outpost to another, or within the same
    #   Outpost.
    #
    #   For more information, see [ Copy snapshots from an Amazon Web Services
    #   Region to an Outpost][1] in the *Amazon EBS User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ebs/latest/userguide/snapshots-outposts.html#copy-snapshots
    #
    # @option params [String] :destination_region
    #   The destination Region to use in the `PresignedUrl` parameter of a
    #   snapshot copy operation. This parameter is only valid for specifying
    #   the destination Region in a `PresignedUrl` parameter, where it is
    #   required.
    #
    #   The snapshot copy is sent to the regional endpoint that you sent the
    #   HTTP request to (for example, `ec2.us-east-1.amazonaws.com`). With the
    #   CLI, this is specified using the `--region` parameter or the default
    #   Region in your Amazon Web Services configuration file.
    #
    # @option params [Boolean] :encrypted
    #   To encrypt a copy of an unencrypted snapshot if encryption by default
    #   is not enabled, enable encryption using this parameter. Otherwise,
    #   omit this parameter. Encrypted snapshots are encrypted, even if you
    #   omit this parameter and encryption by default is not enabled. You
    #   cannot set this parameter to false. For more information, see [Amazon
    #   EBS encryption][1] in the *Amazon EBS User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-encryption.html
    #
    # @option params [String] :kms_key_id
    #   The identifier of the KMS key to use for Amazon EBS encryption. If
    #   this parameter is not specified, your KMS key for Amazon EBS is used.
    #   If `KmsKeyId` is specified, the encrypted state must be `true`.
    #
    #   You can specify the KMS key using any of the following:
    #
    #   * Key ID. For example, 1234abcd-12ab-34cd-56ef-1234567890ab.
    #
    #   * Key alias. For example, alias/ExampleAlias.
    #
    #   * Key ARN. For example,
    #     arn:aws:kms:us-east-1:012345678910:key/1234abcd-12ab-34cd-56ef-1234567890ab.
    #
    #   * Alias ARN. For example,
    #     arn:aws:kms:us-east-1:012345678910:alias/ExampleAlias.
    #
    #   Amazon Web Services authenticates the KMS key asynchronously.
    #   Therefore, if you specify an ID, alias, or ARN that is not valid, the
    #   action can appear to complete, but eventually fails.
    #
    # @option params [String] :presigned_url
    #   When you copy an encrypted source snapshot using the Amazon EC2 Query
    #   API, you must supply a pre-signed URL. This parameter is optional for
    #   unencrypted snapshots. For more information, see [Query requests][1].
    #
    #   The `PresignedUrl` should use the snapshot source endpoint, the
    #   `CopySnapshot` action, and include the `SourceRegion`,
    #   `SourceSnapshotId`, and `DestinationRegion` parameters. The
    #   `PresignedUrl` must be signed using Amazon Web Services Signature
    #   Version 4. Because EBS snapshots are stored in Amazon S3, the signing
    #   algorithm for this parameter uses the same logic that is described in
    #   [ Authenticating Requests: Using Query Parameters (Amazon Web Services
    #   Signature Version 4)][2] in the *Amazon S3 API Reference*. An invalid
    #   or improperly signed `PresignedUrl` will cause the copy operation to
    #   fail asynchronously, and the snapshot will move to an `error` state.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html
    #   [2]: https://docs.aws.amazon.com/AmazonS3/latest/API/sigv4-query-string-auth.html
    #
    # @option params [required, String] :source_region
    #   The ID of the Region that contains the snapshot to be copied.
    #
    # @option params [required, String] :source_snapshot_id
    #   The ID of the EBS snapshot to copy.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the new snapshot.
    #
    # @option params [Integer] :completion_duration_minutes
    #   Specify a completion duration, in 15 minute increments, to initiate a
    #   time-based snapshot copy. Time-based snapshot copy operations complete
    #   within the specified duration. For more information, see [ Time-based
    #   copies][1].
    #
    #   If you do not specify a value, the snapshot copy operation is
    #   completed on a best-effort basis.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ebs/latest/userguide/time-based-copies.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CopySnapshotResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CopySnapshotResult#tags #tags} => Array&lt;Types::Tag&gt;
    #   * {Types::CopySnapshotResult#snapshot_id #snapshot_id} => String
    #
    #
    # @example Example: To copy a snapshot
    #
    #   # This example copies a snapshot with the snapshot ID of ``snap-066877671789bd71b`` from the ``us-west-2`` region to the
    #   # ``us-east-1`` region and adds a short description to identify the snapshot.
    #
    #   resp = client.copy_snapshot({
    #     description: "This is my copied snapshot.", 
    #     destination_region: "us-east-1", 
    #     source_region: "us-west-2", 
    #     source_snapshot_id: "snap-066877671789bd71b", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     snapshot_id: "snap-066877671789bd71b", 
    #   }
    #
    # @example Copy snapshot example
    #   source_snapshot_id = 'snapshot-id'
    #   source_region = 'us-east-1'
    #   target_region = 'us-west-2'
    #
    #   # You must configure your EC2 client for the destination region to copy
    #   ec2 = Aws::EC2::Client(region: target_region)
    #
    #   resp = ec2.copy_snapshot({
    #       source_region: source_region,
    #       source_snapshot_id: source_snapshot_id,
    #   })
    #
    #   snapshot_id = resp.snapshot_id
    #
    #   ec2.wait_until(:snapshot_completed, snapshot_ids: [snapshot_id])
    #
    # @example Copy an encrypted snapshot
    #   # same as above, expect you must pass `encrypted: true`
    #   resp = ec2.copy_snapshot({
    #       source_region: source_region,
    #       source_snapshot_id: source_snapshot_id,
    #       encrypted: true, # required for encrypted snapshots
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.copy_snapshot({
    #     description: "String",
    #     destination_outpost_arn: "String",
    #     destination_region: "String",
    #     encrypted: false,
    #     kms_key_id: "KmsKeyId",
    #     presigned_url: "CopySnapshotRequestPSU",
    #     source_region: "String", # required
    #     source_snapshot_id: "String", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     completion_duration_minutes: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.tags #=> Array
    #   resp.tags[0].key #=> String
    #   resp.tags[0].value #=> String
    #   resp.snapshot_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CopySnapshot AWS API Documentation
    #
    # @overload copy_snapshot(params = {})
    # @param [Hash] params ({})
    def copy_snapshot(params = {}, options = {})
      req = build_request(:copy_snapshot, params)
      req.send_request(options)
    end

    # Creates a new Capacity Reservation with the specified attributes.
    # Capacity Reservations enable you to reserve capacity for your Amazon
    # EC2 instances in a specific Availability Zone for any duration.
    #
    # You can create a Capacity Reservation at any time, and you can choose
    # when it starts. You can create a Capacity Reservation for immediate
    # use or you can request a Capacity Reservation for a future date.
    #
    # For more information, see [ Reserve compute capacity with On-Demand
    # Capacity Reservations][1] in the *Amazon EC2 User Guide*.
    #
    # Your request to create a Capacity Reservation could fail if:
    #
    # * Amazon EC2 does not have sufficient capacity. In this case, try
    #   again at a later time, try in a different Availability Zone, or
    #   request a smaller Capacity Reservation. If your workload is flexible
    #   across instance types and sizes, try with different instance
    #   attributes.
    #
    # * The requested quantity exceeds your On-Demand Instance quota. In
    #   this case, increase your On-Demand Instance quota for the requested
    #   instance type and try again. For more information, see [ Amazon EC2
    #   Service Quotas][2] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-capacity-reservations.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-resource-limits.html
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensure
    #   Idempotency][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [required, String] :instance_type
    #   The instance type for which to reserve capacity.
    #
    #   <note markdown="1"> You can request future-dated Capacity Reservations for instance types
    #   in the C, M, R, I, and T instance families only.
    #
    #    </note>
    #
    #   For more information, see [Instance types][1] in the *Amazon EC2 User
    #   Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html
    #
    # @option params [required, String] :instance_platform
    #   The type of operating system for which to reserve capacity.
    #
    # @option params [String] :availability_zone
    #   The Availability Zone in which to create the Capacity Reservation.
    #
    # @option params [String] :availability_zone_id
    #   The ID of the Availability Zone in which to create the Capacity
    #   Reservation.
    #
    # @option params [String] :tenancy
    #   Indicates the tenancy of the Capacity Reservation. A Capacity
    #   Reservation can have one of the following tenancy settings:
    #
    #   * `default` - The Capacity Reservation is created on hardware that is
    #     shared with other Amazon Web Services accounts.
    #
    #   * `dedicated` - The Capacity Reservation is created on single-tenant
    #     hardware that is dedicated to a single Amazon Web Services account.
    #
    # @option params [required, Integer] :instance_count
    #   The number of instances for which to reserve capacity.
    #
    #   <note markdown="1"> You can request future-dated Capacity Reservations for an instance
    #   count with a minimum of 100 vCPUs. For example, if you request a
    #   future-dated Capacity Reservation for `m5.xlarge` instances, you must
    #   request at least 25 instances (*25 * m5.xlarge = 100 vCPUs*).
    #
    #    </note>
    #
    #   Valid range: 1 - 1000
    #
    # @option params [Boolean] :ebs_optimized
    #   Indicates whether the Capacity Reservation supports EBS-optimized
    #   instances. This optimization provides dedicated throughput to Amazon
    #   EBS and an optimized configuration stack to provide optimal I/O
    #   performance. This optimization isn't available with all instance
    #   types. Additional usage charges apply when using an EBS- optimized
    #   instance.
    #
    # @option params [Boolean] :ephemeral_storage
    #   *Deprecated.*
    #
    # @option params [Time,DateTime,Date,Integer,String] :end_date
    #   The date and time at which the Capacity Reservation expires. When a
    #   Capacity Reservation expires, the reserved capacity is released and
    #   you can no longer launch instances into it. The Capacity
    #   Reservation's state changes to `expired` when it reaches its end date
    #   and time.
    #
    #   You must provide an `EndDate` value if `EndDateType` is `limited`.
    #   Omit `EndDate` if `EndDateType` is `unlimited`.
    #
    #   If the `EndDateType` is `limited`, the Capacity Reservation is
    #   cancelled within an hour from the specified time. For example, if you
    #   specify 5/31/2019, 13:30:55, the Capacity Reservation is guaranteed to
    #   end between 13:30:55 and 14:30:55 on 5/31/2019.
    #
    #   If you are requesting a future-dated Capacity Reservation, you can't
    #   specify an end date and time that is within the commitment duration.
    #
    # @option params [String] :end_date_type
    #   Indicates the way in which the Capacity Reservation ends. A Capacity
    #   Reservation can have one of the following end types:
    #
    #   * `unlimited` - The Capacity Reservation remains active until you
    #     explicitly cancel it. Do not provide an `EndDate` if the
    #     `EndDateType` is `unlimited`.
    #
    #   * `limited` - The Capacity Reservation expires automatically at a
    #     specified date and time. You must provide an `EndDate` value if the
    #     `EndDateType` value is `limited`.
    #
    # @option params [String] :instance_match_criteria
    #   Indicates the type of instance launches that the Capacity Reservation
    #   accepts. The options include:
    #
    #   * `open` - The Capacity Reservation automatically matches all
    #     instances that have matching attributes (instance type, platform,
    #     and Availability Zone). Instances that have matching attributes run
    #     in the Capacity Reservation automatically without specifying any
    #     additional parameters.
    #
    #   * `targeted` - The Capacity Reservation only accepts instances that
    #     have matching attributes (instance type, platform, and Availability
    #     Zone), and explicitly target the Capacity Reservation. This ensures
    #     that only permitted instances can use the reserved capacity.
    #
    #   <note markdown="1"> If you are requesting a future-dated Capacity Reservation, you must
    #   specify `targeted`.
    #
    #    </note>
    #
    #   Default: `open`
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the Capacity Reservation during launch.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :outpost_arn
    #   <note markdown="1"> Not supported for future-dated Capacity Reservations.
    #
    #    </note>
    #
    #   The Amazon Resource Name (ARN) of the Outpost on which to create the
    #   Capacity Reservation.
    #
    # @option params [String] :placement_group_arn
    #   <note markdown="1"> Not supported for future-dated Capacity Reservations.
    #
    #    </note>
    #
    #   The Amazon Resource Name (ARN) of the cluster placement group in which
    #   to create the Capacity Reservation. For more information, see [
    #   Capacity Reservations for cluster placement groups][1] in the *Amazon
    #   EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/cr-cpg.html
    #
    # @option params [Time,DateTime,Date,Integer,String] :start_date
    #   <note markdown="1"> Required for future-dated Capacity Reservations only. To create a
    #   Capacity Reservation for immediate use, omit this parameter.
    #
    #    </note>
    #
    #   The date and time at which the future-dated Capacity Reservation
    #   should become available for use, in the ISO8601 format in the UTC time
    #   zone (`YYYY-MM-DDThh:mm:ss.sssZ`).
    #
    #   You can request a future-dated Capacity Reservation between 5 and 120
    #   days in advance.
    #
    # @option params [Integer] :commitment_duration
    #   <note markdown="1"> Required for future-dated Capacity Reservations only. To create a
    #   Capacity Reservation for immediate use, omit this parameter.
    #
    #    </note>
    #
    #   Specify a commitment duration, in seconds, for the future-dated
    #   Capacity Reservation.
    #
    #   The commitment duration is a minimum duration for which you commit to
    #   having the future-dated Capacity Reservation in the `active` state in
    #   your account after it has been delivered.
    #
    #   For more information, see [ Commitment duration][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/cr-concepts.html#cr-commitment-duration
    #
    # @option params [String] :delivery_preference
    #   <note markdown="1"> Required for future-dated Capacity Reservations only. To create a
    #   Capacity Reservation for immediate use, omit this parameter.
    #
    #    </note>
    #
    #   Indicates that the requested capacity will be delivered in addition to
    #   any running instances or reserved capacity that you have in your
    #   account at the requested date and time.
    #
    #   The only supported value is `incremental`.
    #
    # @return [Types::CreateCapacityReservationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateCapacityReservationResult#capacity_reservation #capacity_reservation} => Types::CapacityReservation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_capacity_reservation({
    #     client_token: "String",
    #     instance_type: "String", # required
    #     instance_platform: "Linux/UNIX", # required, accepts Linux/UNIX, Red Hat Enterprise Linux, SUSE Linux, Windows, Windows with SQL Server, Windows with SQL Server Enterprise, Windows with SQL Server Standard, Windows with SQL Server Web, Linux with SQL Server Standard, Linux with SQL Server Web, Linux with SQL Server Enterprise, RHEL with SQL Server Standard, RHEL with SQL Server Enterprise, RHEL with SQL Server Web, RHEL with HA, RHEL with HA and SQL Server Standard, RHEL with HA and SQL Server Enterprise, Ubuntu Pro
    #     availability_zone: "AvailabilityZoneName",
    #     availability_zone_id: "AvailabilityZoneId",
    #     tenancy: "default", # accepts default, dedicated
    #     instance_count: 1, # required
    #     ebs_optimized: false,
    #     ephemeral_storage: false,
    #     end_date: Time.now,
    #     end_date_type: "unlimited", # accepts unlimited, limited
    #     instance_match_criteria: "open", # accepts open, targeted
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #     outpost_arn: "OutpostArn",
    #     placement_group_arn: "PlacementGroupArn",
    #     start_date: Time.now,
    #     commitment_duration: 1,
    #     delivery_preference: "fixed", # accepts fixed, incremental
    #   })
    #
    # @example Response structure
    #
    #   resp.capacity_reservation.capacity_reservation_id #=> String
    #   resp.capacity_reservation.owner_id #=> String
    #   resp.capacity_reservation.capacity_reservation_arn #=> String
    #   resp.capacity_reservation.availability_zone_id #=> String
    #   resp.capacity_reservation.instance_type #=> String
    #   resp.capacity_reservation.instance_platform #=> String, one of "Linux/UNIX", "Red Hat Enterprise Linux", "SUSE Linux", "Windows", "Windows with SQL Server", "Windows with SQL Server Enterprise", "Windows with SQL Server Standard", "Windows with SQL Server Web", "Linux with SQL Server Standard", "Linux with SQL Server Web", "Linux with SQL Server Enterprise", "RHEL with SQL Server Standard", "RHEL with SQL Server Enterprise", "RHEL with SQL Server Web", "RHEL with HA", "RHEL with HA and SQL Server Standard", "RHEL with HA and SQL Server Enterprise", "Ubuntu Pro"
    #   resp.capacity_reservation.availability_zone #=> String
    #   resp.capacity_reservation.tenancy #=> String, one of "default", "dedicated"
    #   resp.capacity_reservation.total_instance_count #=> Integer
    #   resp.capacity_reservation.available_instance_count #=> Integer
    #   resp.capacity_reservation.ebs_optimized #=> Boolean
    #   resp.capacity_reservation.ephemeral_storage #=> Boolean
    #   resp.capacity_reservation.state #=> String, one of "active", "expired", "cancelled", "pending", "failed", "scheduled", "payment-pending", "payment-failed", "assessing", "delayed", "unsupported"
    #   resp.capacity_reservation.start_date #=> Time
    #   resp.capacity_reservation.end_date #=> Time
    #   resp.capacity_reservation.end_date_type #=> String, one of "unlimited", "limited"
    #   resp.capacity_reservation.instance_match_criteria #=> String, one of "open", "targeted"
    #   resp.capacity_reservation.create_date #=> Time
    #   resp.capacity_reservation.tags #=> Array
    #   resp.capacity_reservation.tags[0].key #=> String
    #   resp.capacity_reservation.tags[0].value #=> String
    #   resp.capacity_reservation.outpost_arn #=> String
    #   resp.capacity_reservation.capacity_reservation_fleet_id #=> String
    #   resp.capacity_reservation.placement_group_arn #=> String
    #   resp.capacity_reservation.capacity_allocations #=> Array
    #   resp.capacity_reservation.capacity_allocations[0].allocation_type #=> String, one of "used", "future"
    #   resp.capacity_reservation.capacity_allocations[0].count #=> Integer
    #   resp.capacity_reservation.reservation_type #=> String, one of "default", "capacity-block"
    #   resp.capacity_reservation.unused_reservation_billing_owner_id #=> String
    #   resp.capacity_reservation.commitment_info.committed_instance_count #=> Integer
    #   resp.capacity_reservation.commitment_info.commitment_end_date #=> Time
    #   resp.capacity_reservation.delivery_preference #=> String, one of "fixed", "incremental"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateCapacityReservation AWS API Documentation
    #
    # @overload create_capacity_reservation(params = {})
    # @param [Hash] params ({})
    def create_capacity_reservation(params = {}, options = {})
      req = build_request(:create_capacity_reservation, params)
      req.send_request(options)
    end

    # Create a new Capacity Reservation by splitting the capacity of the
    # source Capacity Reservation. The new Capacity Reservation will have
    # the same attributes as the source Capacity Reservation except for
    # tags. The source Capacity Reservation must be `active` and owned by
    # your Amazon Web Services account.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensure
    #   Idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [required, String] :source_capacity_reservation_id
    #   The ID of the Capacity Reservation from which you want to split the
    #   capacity.
    #
    # @option params [required, Integer] :instance_count
    #   The number of instances to split from the source Capacity Reservation.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the new Capacity Reservation.
    #
    # @return [Types::CreateCapacityReservationBySplittingResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateCapacityReservationBySplittingResult#source_capacity_reservation #source_capacity_reservation} => Types::CapacityReservation
    #   * {Types::CreateCapacityReservationBySplittingResult#destination_capacity_reservation #destination_capacity_reservation} => Types::CapacityReservation
    #   * {Types::CreateCapacityReservationBySplittingResult#instance_count #instance_count} => Integer
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_capacity_reservation_by_splitting({
    #     dry_run: false,
    #     client_token: "String",
    #     source_capacity_reservation_id: "CapacityReservationId", # required
    #     instance_count: 1, # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.source_capacity_reservation.capacity_reservation_id #=> String
    #   resp.source_capacity_reservation.owner_id #=> String
    #   resp.source_capacity_reservation.capacity_reservation_arn #=> String
    #   resp.source_capacity_reservation.availability_zone_id #=> String
    #   resp.source_capacity_reservation.instance_type #=> String
    #   resp.source_capacity_reservation.instance_platform #=> String, one of "Linux/UNIX", "Red Hat Enterprise Linux", "SUSE Linux", "Windows", "Windows with SQL Server", "Windows with SQL Server Enterprise", "Windows with SQL Server Standard", "Windows with SQL Server Web", "Linux with SQL Server Standard", "Linux with SQL Server Web", "Linux with SQL Server Enterprise", "RHEL with SQL Server Standard", "RHEL with SQL Server Enterprise", "RHEL with SQL Server Web", "RHEL with HA", "RHEL with HA and SQL Server Standard", "RHEL with HA and SQL Server Enterprise", "Ubuntu Pro"
    #   resp.source_capacity_reservation.availability_zone #=> String
    #   resp.source_capacity_reservation.tenancy #=> String, one of "default", "dedicated"
    #   resp.source_capacity_reservation.total_instance_count #=> Integer
    #   resp.source_capacity_reservation.available_instance_count #=> Integer
    #   resp.source_capacity_reservation.ebs_optimized #=> Boolean
    #   resp.source_capacity_reservation.ephemeral_storage #=> Boolean
    #   resp.source_capacity_reservation.state #=> String, one of "active", "expired", "cancelled", "pending", "failed", "scheduled", "payment-pending", "payment-failed", "assessing", "delayed", "unsupported"
    #   resp.source_capacity_reservation.start_date #=> Time
    #   resp.source_capacity_reservation.end_date #=> Time
    #   resp.source_capacity_reservation.end_date_type #=> String, one of "unlimited", "limited"
    #   resp.source_capacity_reservation.instance_match_criteria #=> String, one of "open", "targeted"
    #   resp.source_capacity_reservation.create_date #=> Time
    #   resp.source_capacity_reservation.tags #=> Array
    #   resp.source_capacity_reservation.tags[0].key #=> String
    #   resp.source_capacity_reservation.tags[0].value #=> String
    #   resp.source_capacity_reservation.outpost_arn #=> String
    #   resp.source_capacity_reservation.capacity_reservation_fleet_id #=> String
    #   resp.source_capacity_reservation.placement_group_arn #=> String
    #   resp.source_capacity_reservation.capacity_allocations #=> Array
    #   resp.source_capacity_reservation.capacity_allocations[0].allocation_type #=> String, one of "used", "future"
    #   resp.source_capacity_reservation.capacity_allocations[0].count #=> Integer
    #   resp.source_capacity_reservation.reservation_type #=> String, one of "default", "capacity-block"
    #   resp.source_capacity_reservation.unused_reservation_billing_owner_id #=> String
    #   resp.source_capacity_reservation.commitment_info.committed_instance_count #=> Integer
    #   resp.source_capacity_reservation.commitment_info.commitment_end_date #=> Time
    #   resp.source_capacity_reservation.delivery_preference #=> String, one of "fixed", "incremental"
    #   resp.destination_capacity_reservation.capacity_reservation_id #=> String
    #   resp.destination_capacity_reservation.owner_id #=> String
    #   resp.destination_capacity_reservation.capacity_reservation_arn #=> String
    #   resp.destination_capacity_reservation.availability_zone_id #=> String
    #   resp.destination_capacity_reservation.instance_type #=> String
    #   resp.destination_capacity_reservation.instance_platform #=> String, one of "Linux/UNIX", "Red Hat Enterprise Linux", "SUSE Linux", "Windows", "Windows with SQL Server", "Windows with SQL Server Enterprise", "Windows with SQL Server Standard", "Windows with SQL Server Web", "Linux with SQL Server Standard", "Linux with SQL Server Web", "Linux with SQL Server Enterprise", "RHEL with SQL Server Standard", "RHEL with SQL Server Enterprise", "RHEL with SQL Server Web", "RHEL with HA", "RHEL with HA and SQL Server Standard", "RHEL with HA and SQL Server Enterprise", "Ubuntu Pro"
    #   resp.destination_capacity_reservation.availability_zone #=> String
    #   resp.destination_capacity_reservation.tenancy #=> String, one of "default", "dedicated"
    #   resp.destination_capacity_reservation.total_instance_count #=> Integer
    #   resp.destination_capacity_reservation.available_instance_count #=> Integer
    #   resp.destination_capacity_reservation.ebs_optimized #=> Boolean
    #   resp.destination_capacity_reservation.ephemeral_storage #=> Boolean
    #   resp.destination_capacity_reservation.state #=> String, one of "active", "expired", "cancelled", "pending", "failed", "scheduled", "payment-pending", "payment-failed", "assessing", "delayed", "unsupported"
    #   resp.destination_capacity_reservation.start_date #=> Time
    #   resp.destination_capacity_reservation.end_date #=> Time
    #   resp.destination_capacity_reservation.end_date_type #=> String, one of "unlimited", "limited"
    #   resp.destination_capacity_reservation.instance_match_criteria #=> String, one of "open", "targeted"
    #   resp.destination_capacity_reservation.create_date #=> Time
    #   resp.destination_capacity_reservation.tags #=> Array
    #   resp.destination_capacity_reservation.tags[0].key #=> String
    #   resp.destination_capacity_reservation.tags[0].value #=> String
    #   resp.destination_capacity_reservation.outpost_arn #=> String
    #   resp.destination_capacity_reservation.capacity_reservation_fleet_id #=> String
    #   resp.destination_capacity_reservation.placement_group_arn #=> String
    #   resp.destination_capacity_reservation.capacity_allocations #=> Array
    #   resp.destination_capacity_reservation.capacity_allocations[0].allocation_type #=> String, one of "used", "future"
    #   resp.destination_capacity_reservation.capacity_allocations[0].count #=> Integer
    #   resp.destination_capacity_reservation.reservation_type #=> String, one of "default", "capacity-block"
    #   resp.destination_capacity_reservation.unused_reservation_billing_owner_id #=> String
    #   resp.destination_capacity_reservation.commitment_info.committed_instance_count #=> Integer
    #   resp.destination_capacity_reservation.commitment_info.commitment_end_date #=> Time
    #   resp.destination_capacity_reservation.delivery_preference #=> String, one of "fixed", "incremental"
    #   resp.instance_count #=> Integer
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateCapacityReservationBySplitting AWS API Documentation
    #
    # @overload create_capacity_reservation_by_splitting(params = {})
    # @param [Hash] params ({})
    def create_capacity_reservation_by_splitting(params = {}, options = {})
      req = build_request(:create_capacity_reservation_by_splitting, params)
      req.send_request(options)
    end

    # Creates a Capacity Reservation Fleet. For more information, see
    # [Create a Capacity Reservation Fleet][1] in the *Amazon EC2 User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/work-with-cr-fleets.html#create-crfleet
    #
    # @option params [String] :allocation_strategy
    #   The strategy used by the Capacity Reservation Fleet to determine which
    #   of the specified instance types to use. Currently, only the
    #   `prioritized` allocation strategy is supported. For more information,
    #   see [ Allocation strategy][1] in the *Amazon EC2 User Guide*.
    #
    #   Valid values: `prioritized`
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/crfleet-concepts.html#allocation-strategy
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensure
    #   Idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [required, Array<Types::ReservationFleetInstanceSpecification>] :instance_type_specifications
    #   Information about the instance types for which to reserve the
    #   capacity.
    #
    # @option params [String] :tenancy
    #   Indicates the tenancy of the Capacity Reservation Fleet. All Capacity
    #   Reservations in the Fleet inherit this tenancy. The Capacity
    #   Reservation Fleet can have one of the following tenancy settings:
    #
    #   * `default` - The Capacity Reservation Fleet is created on hardware
    #     that is shared with other Amazon Web Services accounts.
    #
    #   * `dedicated` - The Capacity Reservations are created on single-tenant
    #     hardware that is dedicated to a single Amazon Web Services account.
    #
    # @option params [required, Integer] :total_target_capacity
    #   The total number of capacity units to be reserved by the Capacity
    #   Reservation Fleet. This value, together with the instance type weights
    #   that you assign to each instance type used by the Fleet determine the
    #   number of instances for which the Fleet reserves capacity. Both values
    #   are based on units that make sense for your workload. For more
    #   information, see [Total target capacity][1] in the *Amazon EC2 User
    #   Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/crfleet-concepts.html#target-capacity
    #
    # @option params [Time,DateTime,Date,Integer,String] :end_date
    #   The date and time at which the Capacity Reservation Fleet expires.
    #   When the Capacity Reservation Fleet expires, its state changes to
    #   `expired` and all of the Capacity Reservations in the Fleet expire.
    #
    #   The Capacity Reservation Fleet expires within an hour after the
    #   specified time. For example, if you specify `5/31/2019`, `13:30:55`,
    #   the Capacity Reservation Fleet is guaranteed to expire between
    #   `13:30:55` and `14:30:55` on `5/31/2019`.
    #
    # @option params [String] :instance_match_criteria
    #   Indicates the type of instance launches that the Capacity Reservation
    #   Fleet accepts. All Capacity Reservations in the Fleet inherit this
    #   instance matching criteria.
    #
    #   Currently, Capacity Reservation Fleets support `open` instance
    #   matching criteria only. This means that instances that have matching
    #   attributes (instance type, platform, and Availability Zone) run in the
    #   Capacity Reservations automatically. Instances do not need to
    #   explicitly target a Capacity Reservation Fleet to use its reserved
    #   capacity.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the Capacity Reservation Fleet. The tags are
    #   automatically assigned to the Capacity Reservations in the Fleet.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateCapacityReservationFleetResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateCapacityReservationFleetResult#capacity_reservation_fleet_id #capacity_reservation_fleet_id} => String
    #   * {Types::CreateCapacityReservationFleetResult#state #state} => String
    #   * {Types::CreateCapacityReservationFleetResult#total_target_capacity #total_target_capacity} => Integer
    #   * {Types::CreateCapacityReservationFleetResult#total_fulfilled_capacity #total_fulfilled_capacity} => Float
    #   * {Types::CreateCapacityReservationFleetResult#instance_match_criteria #instance_match_criteria} => String
    #   * {Types::CreateCapacityReservationFleetResult#allocation_strategy #allocation_strategy} => String
    #   * {Types::CreateCapacityReservationFleetResult#create_time #create_time} => Time
    #   * {Types::CreateCapacityReservationFleetResult#end_date #end_date} => Time
    #   * {Types::CreateCapacityReservationFleetResult#tenancy #tenancy} => String
    #   * {Types::CreateCapacityReservationFleetResult#fleet_capacity_reservations #fleet_capacity_reservations} => Array&lt;Types::FleetCapacityReservation&gt;
    #   * {Types::CreateCapacityReservationFleetResult#tags #tags} => Array&lt;Types::Tag&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_capacity_reservation_fleet({
    #     allocation_strategy: "String",
    #     client_token: "String",
    #     instance_type_specifications: [ # required
    #       {
    #         instance_type: "a1.medium", # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #         instance_platform: "Linux/UNIX", # accepts Linux/UNIX, Red Hat Enterprise Linux, SUSE Linux, Windows, Windows with SQL Server, Windows with SQL Server Enterprise, Windows with SQL Server Standard, Windows with SQL Server Web, Linux with SQL Server Standard, Linux with SQL Server Web, Linux with SQL Server Enterprise, RHEL with SQL Server Standard, RHEL with SQL Server Enterprise, RHEL with SQL Server Web, RHEL with HA, RHEL with HA and SQL Server Standard, RHEL with HA and SQL Server Enterprise, Ubuntu Pro
    #         weight: 1.0,
    #         availability_zone: "String",
    #         availability_zone_id: "String",
    #         ebs_optimized: false,
    #         priority: 1,
    #       },
    #     ],
    #     tenancy: "default", # accepts default
    #     total_target_capacity: 1, # required
    #     end_date: Time.now,
    #     instance_match_criteria: "open", # accepts open
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.capacity_reservation_fleet_id #=> String
    #   resp.state #=> String, one of "submitted", "modifying", "active", "partially_fulfilled", "expiring", "expired", "cancelling", "cancelled", "failed"
    #   resp.total_target_capacity #=> Integer
    #   resp.total_fulfilled_capacity #=> Float
    #   resp.instance_match_criteria #=> String, one of "open"
    #   resp.allocation_strategy #=> String
    #   resp.create_time #=> Time
    #   resp.end_date #=> Time
    #   resp.tenancy #=> String, one of "default"
    #   resp.fleet_capacity_reservations #=> Array
    #   resp.fleet_capacity_reservations[0].capacity_reservation_id #=> String
    #   resp.fleet_capacity_reservations[0].availability_zone_id #=> String
    #   resp.fleet_capacity_reservations[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.fleet_capacity_reservations[0].instance_platform #=> String, one of "Linux/UNIX", "Red Hat Enterprise Linux", "SUSE Linux", "Windows", "Windows with SQL Server", "Windows with SQL Server Enterprise", "Windows with SQL Server Standard", "Windows with SQL Server Web", "Linux with SQL Server Standard", "Linux with SQL Server Web", "Linux with SQL Server Enterprise", "RHEL with SQL Server Standard", "RHEL with SQL Server Enterprise", "RHEL with SQL Server Web", "RHEL with HA", "RHEL with HA and SQL Server Standard", "RHEL with HA and SQL Server Enterprise", "Ubuntu Pro"
    #   resp.fleet_capacity_reservations[0].availability_zone #=> String
    #   resp.fleet_capacity_reservations[0].total_instance_count #=> Integer
    #   resp.fleet_capacity_reservations[0].fulfilled_capacity #=> Float
    #   resp.fleet_capacity_reservations[0].ebs_optimized #=> Boolean
    #   resp.fleet_capacity_reservations[0].create_date #=> Time
    #   resp.fleet_capacity_reservations[0].weight #=> Float
    #   resp.fleet_capacity_reservations[0].priority #=> Integer
    #   resp.tags #=> Array
    #   resp.tags[0].key #=> String
    #   resp.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateCapacityReservationFleet AWS API Documentation
    #
    # @overload create_capacity_reservation_fleet(params = {})
    # @param [Hash] params ({})
    def create_capacity_reservation_fleet(params = {}, options = {})
      req = build_request(:create_capacity_reservation_fleet, params)
      req.send_request(options)
    end

    # Creates a carrier gateway. For more information about carrier
    # gateways, see [Carrier gateways][1] in the *Amazon Web Services
    # Wavelength Developer Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/wavelength/latest/developerguide/how-wavelengths-work.html#wavelength-carrier-gateway
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC to associate with the carrier gateway.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to associate with the carrier gateway.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [How to ensure
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @return [Types::CreateCarrierGatewayResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateCarrierGatewayResult#carrier_gateway #carrier_gateway} => Types::CarrierGateway
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_carrier_gateway({
    #     vpc_id: "VpcId", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #     client_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.carrier_gateway.carrier_gateway_id #=> String
    #   resp.carrier_gateway.vpc_id #=> String
    #   resp.carrier_gateway.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.carrier_gateway.owner_id #=> String
    #   resp.carrier_gateway.tags #=> Array
    #   resp.carrier_gateway.tags[0].key #=> String
    #   resp.carrier_gateway.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateCarrierGateway AWS API Documentation
    #
    # @overload create_carrier_gateway(params = {})
    # @param [Hash] params ({})
    def create_carrier_gateway(params = {}, options = {})
      req = build_request(:create_carrier_gateway, params)
      req.send_request(options)
    end

    # Creates a Client VPN endpoint. A Client VPN endpoint is the resource
    # you create and configure to enable and manage client VPN sessions. It
    # is the destination endpoint at which all client VPN sessions are
    # terminated.
    #
    # @option params [required, String] :client_cidr_block
    #   The IPv4 address range, in CIDR notation, from which to assign client
    #   IP addresses. The address range cannot overlap with the local CIDR of
    #   the VPC in which the associated subnet is located, or the routes that
    #   you add manually. The address range cannot be changed after the Client
    #   VPN endpoint has been created. Client CIDR range must have a size of
    #   at least /22 and must not be greater than /12.
    #
    # @option params [required, String] :server_certificate_arn
    #   The ARN of the server certificate. For more information, see the
    #   [Certificate Manager User Guide][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/acm/latest/userguide/
    #
    # @option params [required, Array<Types::ClientVpnAuthenticationRequest>] :authentication_options
    #   Information about the authentication method to be used to authenticate
    #   clients.
    #
    # @option params [required, Types::ConnectionLogOptions] :connection_log_options
    #   Information about the client connection logging options.
    #
    #   If you enable client connection logging, data about client connections
    #   is sent to a Cloudwatch Logs log stream. The following information is
    #   logged:
    #
    #   * Client connection requests
    #
    #   * Client connection results (successful and unsuccessful)
    #
    #   * Reasons for unsuccessful client connection requests
    #
    #   * Client connection termination time
    #
    # @option params [Array<String>] :dns_servers
    #   Information about the DNS servers to be used for DNS resolution. A
    #   Client VPN endpoint can have up to two DNS servers. If no DNS server
    #   is specified, the DNS address configured on the device is used for the
    #   DNS server.
    #
    # @option params [String] :transport_protocol
    #   The transport protocol to be used by the VPN session.
    #
    #   Default value: `udp`
    #
    # @option params [Integer] :vpn_port
    #   The port number to assign to the Client VPN endpoint for TCP and UDP
    #   traffic.
    #
    #   Valid Values: `443` \| `1194`
    #
    #   Default Value: `443`
    #
    # @option params [String] :description
    #   A brief description of the Client VPN endpoint.
    #
    # @option params [Boolean] :split_tunnel
    #   Indicates whether split-tunnel is enabled on the Client VPN endpoint.
    #
    #   By default, split-tunnel on a VPN endpoint is disabled.
    #
    #   For information about split-tunnel VPN endpoints, see [Split-tunnel
    #   Client VPN endpoint][1] in the *Client VPN Administrator Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpn/latest/clientvpn-admin/split-tunnel-vpn.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the Client VPN endpoint during creation.
    #
    # @option params [Array<String>] :security_group_ids
    #   The IDs of one or more security groups to apply to the target network.
    #   You must also specify the ID of the VPC that contains the security
    #   groups.
    #
    # @option params [String] :vpc_id
    #   The ID of the VPC to associate with the Client VPN endpoint. If no
    #   security group IDs are specified in the request, the default security
    #   group for the VPC is applied.
    #
    # @option params [String] :self_service_portal
    #   Specify whether to enable the self-service portal for the Client VPN
    #   endpoint.
    #
    #   Default Value: `enabled`
    #
    # @option params [Types::ClientConnectOptions] :client_connect_options
    #   The options for managing connection authorization for new client
    #   connections.
    #
    # @option params [Integer] :session_timeout_hours
    #   The maximum VPN session duration time in hours.
    #
    #   Valid values: `8 | 10 | 12 | 24`
    #
    #   Default value: `24`
    #
    # @option params [Types::ClientLoginBannerOptions] :client_login_banner_options
    #   Options for enabling a customizable text banner that will be displayed
    #   on Amazon Web Services provided clients when a VPN session is
    #   established.
    #
    # @option params [Boolean] :disconnect_on_session_timeout
    #   Indicates whether the client VPN session is disconnected after the
    #   maximum timeout specified in `SessionTimeoutHours` is reached. If
    #   `true`, users are prompted to reconnect client VPN. If `false`, client
    #   VPN attempts to reconnect automatically. The default value is `false`.
    #
    # @return [Types::CreateClientVpnEndpointResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateClientVpnEndpointResult#client_vpn_endpoint_id #client_vpn_endpoint_id} => String
    #   * {Types::CreateClientVpnEndpointResult#status #status} => Types::ClientVpnEndpointStatus
    #   * {Types::CreateClientVpnEndpointResult#dns_name #dns_name} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_client_vpn_endpoint({
    #     client_cidr_block: "String", # required
    #     server_certificate_arn: "String", # required
    #     authentication_options: [ # required
    #       {
    #         type: "certificate-authentication", # accepts certificate-authentication, directory-service-authentication, federated-authentication
    #         active_directory: {
    #           directory_id: "String",
    #         },
    #         mutual_authentication: {
    #           client_root_certificate_chain_arn: "String",
    #         },
    #         federated_authentication: {
    #           saml_provider_arn: "String",
    #           self_service_saml_provider_arn: "String",
    #         },
    #       },
    #     ],
    #     connection_log_options: { # required
    #       enabled: false,
    #       cloudwatch_log_group: "String",
    #       cloudwatch_log_stream: "String",
    #     },
    #     dns_servers: ["String"],
    #     transport_protocol: "tcp", # accepts tcp, udp
    #     vpn_port: 1,
    #     description: "String",
    #     split_tunnel: false,
    #     dry_run: false,
    #     client_token: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     security_group_ids: ["SecurityGroupId"],
    #     vpc_id: "VpcId",
    #     self_service_portal: "enabled", # accepts enabled, disabled
    #     client_connect_options: {
    #       enabled: false,
    #       lambda_function_arn: "String",
    #     },
    #     session_timeout_hours: 1,
    #     client_login_banner_options: {
    #       enabled: false,
    #       banner_text: "String",
    #     },
    #     disconnect_on_session_timeout: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.client_vpn_endpoint_id #=> String
    #   resp.status.code #=> String, one of "pending-associate", "available", "deleting", "deleted"
    #   resp.status.message #=> String
    #   resp.dns_name #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateClientVpnEndpoint AWS API Documentation
    #
    # @overload create_client_vpn_endpoint(params = {})
    # @param [Hash] params ({})
    def create_client_vpn_endpoint(params = {}, options = {})
      req = build_request(:create_client_vpn_endpoint, params)
      req.send_request(options)
    end

    # Adds a route to a network to a Client VPN endpoint. Each Client VPN
    # endpoint has a route table that describes the available destination
    # network routes. Each route in the route table specifies the path for
    # traﬃc to speciﬁc resources or networks.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint to which to add the route.
    #
    # @option params [required, String] :destination_cidr_block
    #   The IPv4 address range, in CIDR notation, of the route destination.
    #   For example:
    #
    #   * To add a route for Internet access, enter `0.0.0.0/0`
    #
    #   * To add a route for a peered VPC, enter the peered VPC's IPv4 CIDR
    #     range
    #
    #   * To add a route for an on-premises network, enter the Amazon Web
    #     Services Site-to-Site VPN connection's IPv4 CIDR range
    #
    #   * To add a route for the local network, enter the client CIDR range
    #
    # @option params [required, String] :target_vpc_subnet_id
    #   The ID of the subnet through which you want to route traffic. The
    #   specified subnet must be an existing target network of the Client VPN
    #   endpoint.
    #
    #   Alternatively, if you're adding a route for the local network,
    #   specify `local`.
    #
    # @option params [String] :description
    #   A brief description of the route.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateClientVpnRouteResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateClientVpnRouteResult#status #status} => Types::ClientVpnRouteStatus
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_client_vpn_route({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     destination_cidr_block: "String", # required
    #     target_vpc_subnet_id: "SubnetId", # required
    #     description: "String",
    #     client_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.status.code #=> String, one of "creating", "active", "failed", "deleting"
    #   resp.status.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateClientVpnRoute AWS API Documentation
    #
    # @overload create_client_vpn_route(params = {})
    # @param [Hash] params ({})
    def create_client_vpn_route(params = {}, options = {})
      req = build_request(:create_client_vpn_route, params)
      req.send_request(options)
    end

    # Creates a range of customer-owned IP addresses.
    #
    # @option params [required, String] :cidr
    #   A customer-owned IP address range to create.
    #
    # @option params [required, String] :coip_pool_id
    #   The ID of the address pool.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateCoipCidrResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateCoipCidrResult#coip_cidr #coip_cidr} => Types::CoipCidr
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_coip_cidr({
    #     cidr: "String", # required
    #     coip_pool_id: "Ipv4PoolCoipId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.coip_cidr.cidr #=> String
    #   resp.coip_cidr.coip_pool_id #=> String
    #   resp.coip_cidr.local_gateway_route_table_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateCoipCidr AWS API Documentation
    #
    # @overload create_coip_cidr(params = {})
    # @param [Hash] params ({})
    def create_coip_cidr(params = {}, options = {})
      req = build_request(:create_coip_cidr, params)
      req.send_request(options)
    end

    # Creates a pool of customer-owned IP (CoIP) addresses.
    #
    # @option params [required, String] :local_gateway_route_table_id
    #   The ID of the local gateway route table.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the CoIP address pool.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateCoipPoolResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateCoipPoolResult#coip_pool #coip_pool} => Types::CoipPool
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_coip_pool({
    #     local_gateway_route_table_id: "LocalGatewayRoutetableId", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.coip_pool.pool_id #=> String
    #   resp.coip_pool.pool_cidrs #=> Array
    #   resp.coip_pool.pool_cidrs[0] #=> String
    #   resp.coip_pool.local_gateway_route_table_id #=> String
    #   resp.coip_pool.tags #=> Array
    #   resp.coip_pool.tags[0].key #=> String
    #   resp.coip_pool.tags[0].value #=> String
    #   resp.coip_pool.pool_arn #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateCoipPool AWS API Documentation
    #
    # @overload create_coip_pool(params = {})
    # @param [Hash] params ({})
    def create_coip_pool(params = {}, options = {})
      req = build_request(:create_coip_pool, params)
      req.send_request(options)
    end

    # Provides information to Amazon Web Services about your customer
    # gateway device. The customer gateway device is the appliance at your
    # end of the VPN connection. You must provide the IP address of the
    # customer gateway device’s external interface. The IP address must be
    # static and can be behind a device performing network address
    # translation (NAT).
    #
    # For devices that use Border Gateway Protocol (BGP), you can also
    # provide the device's BGP Autonomous System Number (ASN). You can use
    # an existing ASN assigned to your network. If you don't have an ASN
    # already, you can use a private ASN. For more information, see
    # [Customer gateway options for your Site-to-Site VPN connection][1] in
    # the *Amazon Web Services Site-to-Site VPN User Guide*.
    #
    # To create more than one customer gateway with the same VPN type, IP
    # address, and BGP ASN, specify a unique device name for each customer
    # gateway. An identical request returns information about the existing
    # customer gateway; it doesn't create a new customer gateway.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpn/latest/s2svpn/cgw-options.html
    #
    # @option params [Integer] :bgp_asn
    #   For customer gateway devices that support BGP, specify the device's
    #   ASN. You must specify either `BgpAsn` or `BgpAsnExtended` when
    #   creating the customer gateway. If the ASN is larger than
    #   `2,147,483,647`, you must use `BgpAsnExtended`.
    #
    #   Default: 65000
    #
    #   Valid values: `1` to `2,147,483,647`
    #
    # @option params [String] :public_ip
    #   *This member has been deprecated.* The Internet-routable IP address
    #   for the customer gateway's outside interface. The address must be
    #   static.
    #
    # @option params [String] :certificate_arn
    #   The Amazon Resource Name (ARN) for the customer gateway certificate.
    #
    # @option params [required, String] :type
    #   The type of VPN connection that this customer gateway supports
    #   (`ipsec.1`).
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the customer gateway.
    #
    # @option params [String] :device_name
    #   A name for the customer gateway device.
    #
    #   Length Constraints: Up to 255 characters.
    #
    # @option params [String] :ip_address
    #   IPv4 address for the customer gateway device's outside interface. The
    #   address must be static. If `OutsideIpAddressType` in your VPN
    #   connection options is set to `PrivateIpv4`, you can use an RFC6598 or
    #   RFC1918 private IPv4 address. If `OutsideIpAddressType` is set to
    #   `PublicIpv4`, you can use a public IPv4 address.
    #
    # @option params [Integer] :bgp_asn_extended
    #   For customer gateway devices that support BGP, specify the device's
    #   ASN. You must specify either `BgpAsn` or `BgpAsnExtended` when
    #   creating the customer gateway. If the ASN is larger than
    #   `2,147,483,647`, you must use `BgpAsnExtended`.
    #
    #   Valid values: `2,147,483,648` to `4,294,967,295`
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateCustomerGatewayResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateCustomerGatewayResult#customer_gateway #customer_gateway} => Types::CustomerGateway
    #
    #
    # @example Example: To create a customer gateway
    #
    #   # This example creates a customer gateway with the specified IP address for its outside interface.
    #
    #   resp = client.create_customer_gateway({
    #     bgp_asn: 65534, 
    #     public_ip: "12.1.2.3", 
    #     type: "ipsec.1", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     customer_gateway: {
    #       bgp_asn: "65534", 
    #       customer_gateway_id: "cgw-0e11f167", 
    #       ip_address: "12.1.2.3", 
    #       state: "available", 
    #       type: "ipsec.1", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_customer_gateway({
    #     bgp_asn: 1,
    #     public_ip: "String",
    #     certificate_arn: "String",
    #     type: "ipsec.1", # required, accepts ipsec.1
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     device_name: "String",
    #     ip_address: "String",
    #     bgp_asn_extended: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.customer_gateway.certificate_arn #=> String
    #   resp.customer_gateway.device_name #=> String
    #   resp.customer_gateway.tags #=> Array
    #   resp.customer_gateway.tags[0].key #=> String
    #   resp.customer_gateway.tags[0].value #=> String
    #   resp.customer_gateway.bgp_asn_extended #=> String
    #   resp.customer_gateway.customer_gateway_id #=> String
    #   resp.customer_gateway.state #=> String
    #   resp.customer_gateway.type #=> String
    #   resp.customer_gateway.ip_address #=> String
    #   resp.customer_gateway.bgp_asn #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateCustomerGateway AWS API Documentation
    #
    # @overload create_customer_gateway(params = {})
    # @param [Hash] params ({})
    def create_customer_gateway(params = {}, options = {})
      req = build_request(:create_customer_gateway, params)
      req.send_request(options)
    end

    # Creates a default subnet with a size `/20` IPv4 CIDR block in the
    # specified Availability Zone in your default VPC. You can have only one
    # default subnet per Availability Zone. For more information, see
    # [Create a default subnet][1] in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/work-with-default-vpc.html#create-default-subnet
    #
    # @option params [required, String] :availability_zone
    #   The Availability Zone in which to create the default subnet.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Boolean] :ipv_6_native
    #   Indicates whether to create an IPv6 only subnet. If you already have a
    #   default subnet for this Availability Zone, you must delete it before
    #   you can create an IPv6 only subnet.
    #
    # @return [Types::CreateDefaultSubnetResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateDefaultSubnetResult#subnet #subnet} => Types::Subnet
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_default_subnet({
    #     availability_zone: "AvailabilityZoneName", # required
    #     dry_run: false,
    #     ipv_6_native: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.subnet.availability_zone_id #=> String
    #   resp.subnet.enable_lni_at_device_index #=> Integer
    #   resp.subnet.map_customer_owned_ip_on_launch #=> Boolean
    #   resp.subnet.customer_owned_ipv_4_pool #=> String
    #   resp.subnet.owner_id #=> String
    #   resp.subnet.assign_ipv_6_address_on_creation #=> Boolean
    #   resp.subnet.ipv_6_cidr_block_association_set #=> Array
    #   resp.subnet.ipv_6_cidr_block_association_set[0].association_id #=> String
    #   resp.subnet.ipv_6_cidr_block_association_set[0].ipv_6_cidr_block #=> String
    #   resp.subnet.ipv_6_cidr_block_association_set[0].ipv_6_cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.subnet.ipv_6_cidr_block_association_set[0].ipv_6_cidr_block_state.status_message #=> String
    #   resp.subnet.ipv_6_cidr_block_association_set[0].ipv_6_address_attribute #=> String, one of "public", "private"
    #   resp.subnet.ipv_6_cidr_block_association_set[0].ip_source #=> String, one of "amazon", "byoip", "none"
    #   resp.subnet.tags #=> Array
    #   resp.subnet.tags[0].key #=> String
    #   resp.subnet.tags[0].value #=> String
    #   resp.subnet.subnet_arn #=> String
    #   resp.subnet.outpost_arn #=> String
    #   resp.subnet.enable_dns_64 #=> Boolean
    #   resp.subnet.ipv_6_native #=> Boolean
    #   resp.subnet.private_dns_name_options_on_launch.hostname_type #=> String, one of "ip-name", "resource-name"
    #   resp.subnet.private_dns_name_options_on_launch.enable_resource_name_dns_a_record #=> Boolean
    #   resp.subnet.private_dns_name_options_on_launch.enable_resource_name_dns_aaaa_record #=> Boolean
    #   resp.subnet.block_public_access_states.internet_gateway_block_mode #=> String, one of "off", "block-bidirectional", "block-ingress"
    #   resp.subnet.subnet_id #=> String
    #   resp.subnet.state #=> String, one of "pending", "available", "unavailable"
    #   resp.subnet.vpc_id #=> String
    #   resp.subnet.cidr_block #=> String
    #   resp.subnet.available_ip_address_count #=> Integer
    #   resp.subnet.availability_zone #=> String
    #   resp.subnet.default_for_az #=> Boolean
    #   resp.subnet.map_public_ip_on_launch #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateDefaultSubnet AWS API Documentation
    #
    # @overload create_default_subnet(params = {})
    # @param [Hash] params ({})
    def create_default_subnet(params = {}, options = {})
      req = build_request(:create_default_subnet, params)
      req.send_request(options)
    end

    # Creates a default VPC with a size `/16` IPv4 CIDR block and a default
    # subnet in each Availability Zone. For more information about the
    # components of a default VPC, see [Default VPCs][1] in the *Amazon VPC
    # User Guide*. You cannot specify the components of the default VPC
    # yourself.
    #
    # If you deleted your previous default VPC, you can create a default
    # VPC. You cannot have more than one default VPC per Region.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/default-vpc.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateDefaultVpcResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateDefaultVpcResult#vpc #vpc} => Types::Vpc
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_default_vpc({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc.owner_id #=> String
    #   resp.vpc.instance_tenancy #=> String, one of "default", "dedicated", "host"
    #   resp.vpc.ipv_6_cidr_block_association_set #=> Array
    #   resp.vpc.ipv_6_cidr_block_association_set[0].association_id #=> String
    #   resp.vpc.ipv_6_cidr_block_association_set[0].ipv_6_cidr_block #=> String
    #   resp.vpc.ipv_6_cidr_block_association_set[0].ipv_6_cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.vpc.ipv_6_cidr_block_association_set[0].ipv_6_cidr_block_state.status_message #=> String
    #   resp.vpc.ipv_6_cidr_block_association_set[0].network_border_group #=> String
    #   resp.vpc.ipv_6_cidr_block_association_set[0].ipv_6_pool #=> String
    #   resp.vpc.ipv_6_cidr_block_association_set[0].ipv_6_address_attribute #=> String, one of "public", "private"
    #   resp.vpc.ipv_6_cidr_block_association_set[0].ip_source #=> String, one of "amazon", "byoip", "none"
    #   resp.vpc.cidr_block_association_set #=> Array
    #   resp.vpc.cidr_block_association_set[0].association_id #=> String
    #   resp.vpc.cidr_block_association_set[0].cidr_block #=> String
    #   resp.vpc.cidr_block_association_set[0].cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.vpc.cidr_block_association_set[0].cidr_block_state.status_message #=> String
    #   resp.vpc.is_default #=> Boolean
    #   resp.vpc.encryption_control.vpc_id #=> String
    #   resp.vpc.encryption_control.vpc_encryption_control_id #=> String
    #   resp.vpc.encryption_control.mode #=> String, one of "monitor", "enforce"
    #   resp.vpc.encryption_control.state #=> String, one of "enforce-in-progress", "monitor-in-progress", "enforce-failed", "monitor-failed", "deleting", "deleted", "available"
    #   resp.vpc.encryption_control.state_message #=> String
    #   resp.vpc.encryption_control.resource_exclusions.internet_gateway.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpc.encryption_control.resource_exclusions.internet_gateway.state_message #=> String
    #   resp.vpc.encryption_control.resource_exclusions.egress_only_internet_gateway.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpc.encryption_control.resource_exclusions.egress_only_internet_gateway.state_message #=> String
    #   resp.vpc.encryption_control.resource_exclusions.nat_gateway.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpc.encryption_control.resource_exclusions.nat_gateway.state_message #=> String
    #   resp.vpc.encryption_control.resource_exclusions.virtual_private_gateway.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpc.encryption_control.resource_exclusions.virtual_private_gateway.state_message #=> String
    #   resp.vpc.encryption_control.resource_exclusions.vpc_peering.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpc.encryption_control.resource_exclusions.vpc_peering.state_message #=> String
    #   resp.vpc.encryption_control.tags #=> Array
    #   resp.vpc.encryption_control.tags[0].key #=> String
    #   resp.vpc.encryption_control.tags[0].value #=> String
    #   resp.vpc.tags #=> Array
    #   resp.vpc.tags[0].key #=> String
    #   resp.vpc.tags[0].value #=> String
    #   resp.vpc.block_public_access_states.internet_gateway_block_mode #=> String, one of "off", "block-bidirectional", "block-ingress"
    #   resp.vpc.vpc_id #=> String
    #   resp.vpc.state #=> String, one of "pending", "available"
    #   resp.vpc.cidr_block #=> String
    #   resp.vpc.dhcp_options_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateDefaultVpc AWS API Documentation
    #
    # @overload create_default_vpc(params = {})
    # @param [Hash] params ({})
    def create_default_vpc(params = {}, options = {})
      req = build_request(:create_default_vpc, params)
      req.send_request(options)
    end

    # Creates a custom set of DHCP options. After you create a DHCP option
    # set, you associate it with a VPC. After you associate a DHCP option
    # set with a VPC, all existing and newly launched instances in the VPC
    # use this set of DHCP options.
    #
    # The following are the individual DHCP options you can specify. For
    # more information, see [DHCP option sets][1] in the *Amazon VPC User
    # Guide*.
    #
    # * `domain-name` - If you're using AmazonProvidedDNS in `us-east-1`,
    #   specify `ec2.internal`. If you're using AmazonProvidedDNS in any
    #   other Region, specify `region.compute.internal`. Otherwise, specify
    #   a custom domain name. This value is used to complete unqualified DNS
    #   hostnames.
    #
    #   Some Linux operating systems accept multiple domain names separated
    #   by spaces. However, Windows and other Linux operating systems treat
    #   the value as a single domain, which results in unexpected behavior.
    #   If your DHCP option set is associated with a VPC that has instances
    #   running operating systems that treat the value as a single domain,
    #   specify only one domain name.
    #
    # * `domain-name-servers` - The IP addresses of up to four DNS servers,
    #   or AmazonProvidedDNS. To specify multiple domain name servers in a
    #   single parameter, separate the IP addresses using commas. To have
    #   your instances receive custom DNS hostnames as specified in
    #   `domain-name`, you must specify a custom DNS server.
    #
    # * `ntp-servers` - The IP addresses of up to eight Network Time
    #   Protocol (NTP) servers (four IPv4 addresses and four IPv6
    #   addresses).
    #
    # * `netbios-name-servers` - The IP addresses of up to four NetBIOS name
    #   servers.
    #
    # * `netbios-node-type` - The NetBIOS node type (1, 2, 4, or 8). We
    #   recommend that you specify 2. Broadcast and multicast are not
    #   supported. For more information about NetBIOS node types, see [RFC
    #   2132][2].
    #
    # * `ipv6-address-preferred-lease-time` - A value (in seconds, minutes,
    #   hours, or years) for how frequently a running instance with an IPv6
    #   assigned to it goes through DHCPv6 lease renewal. Acceptable values
    #   are between 140 and 2147483647 seconds (approximately 68 years). If
    #   no value is entered, the default lease time is 140 seconds. If you
    #   use long-term addressing for EC2 instances, you can increase the
    #   lease time and avoid frequent lease renewal requests. Lease renewal
    #   typically occurs when half of the lease time has elapsed.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/VPC_DHCP_Options.html
    # [2]: https://www.ietf.org/rfc/rfc2132.txt
    #
    # @option params [required, Array<Types::NewDhcpConfiguration>] :dhcp_configurations
    #   A DHCP configuration option.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the DHCP option.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateDhcpOptionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateDhcpOptionsResult#dhcp_options #dhcp_options} => Types::DhcpOptions
    #
    #
    # @example Example: To create a DHCP options set
    #
    #   # This example creates a DHCP options set.
    #
    #   resp = client.create_dhcp_options({
    #     dhcp_configurations: [
    #       {
    #         key: "domain-name-servers", 
    #         values: [
    #           "10.2.5.1", 
    #           "10.2.5.2", 
    #         ], 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     dhcp_options: {
    #       dhcp_configurations: [
    #         {
    #           key: "domain-name-servers", 
    #           values: [
    #             {
    #               value: "10.2.5.2", 
    #             }, 
    #             {
    #               value: "10.2.5.1", 
    #             }, 
    #           ], 
    #         }, 
    #       ], 
    #       dhcp_options_id: "dopt-d9070ebb", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_dhcp_options({
    #     dhcp_configurations: [ # required
    #       {
    #         key: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.dhcp_options.owner_id #=> String
    #   resp.dhcp_options.tags #=> Array
    #   resp.dhcp_options.tags[0].key #=> String
    #   resp.dhcp_options.tags[0].value #=> String
    #   resp.dhcp_options.dhcp_options_id #=> String
    #   resp.dhcp_options.dhcp_configurations #=> Array
    #   resp.dhcp_options.dhcp_configurations[0].key #=> String
    #   resp.dhcp_options.dhcp_configurations[0].values #=> Array
    #   resp.dhcp_options.dhcp_configurations[0].values[0] #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateDhcpOptions AWS API Documentation
    #
    # @overload create_dhcp_options(params = {})
    # @param [Hash] params ({})
    def create_dhcp_options(params = {}, options = {})
      req = build_request(:create_dhcp_options, params)
      req.send_request(options)
    end

    # \[IPv6 only\] Creates an egress-only internet gateway for your VPC. An
    # egress-only internet gateway is used to enable outbound communication
    # over IPv6 from instances in your VPC to the internet, and prevents
    # hosts outside of your VPC from initiating an IPv6 connection with your
    # instance.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC for which to create the egress-only internet
    #   gateway.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the egress-only internet gateway.
    #
    # @return [Types::CreateEgressOnlyInternetGatewayResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateEgressOnlyInternetGatewayResult#client_token #client_token} => String
    #   * {Types::CreateEgressOnlyInternetGatewayResult#egress_only_internet_gateway #egress_only_internet_gateway} => Types::EgressOnlyInternetGateway
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_egress_only_internet_gateway({
    #     client_token: "String",
    #     dry_run: false,
    #     vpc_id: "VpcId", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.client_token #=> String
    #   resp.egress_only_internet_gateway.attachments #=> Array
    #   resp.egress_only_internet_gateway.attachments[0].state #=> String, one of "attaching", "attached", "detaching", "detached"
    #   resp.egress_only_internet_gateway.attachments[0].vpc_id #=> String
    #   resp.egress_only_internet_gateway.egress_only_internet_gateway_id #=> String
    #   resp.egress_only_internet_gateway.tags #=> Array
    #   resp.egress_only_internet_gateway.tags[0].key #=> String
    #   resp.egress_only_internet_gateway.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateEgressOnlyInternetGateway AWS API Documentation
    #
    # @overload create_egress_only_internet_gateway(params = {})
    # @param [Hash] params ({})
    def create_egress_only_internet_gateway(params = {}, options = {})
      req = build_request(:create_egress_only_internet_gateway, params)
      req.send_request(options)
    end

    # Creates an EC2 Fleet that contains the configuration information for
    # On-Demand Instances and Spot Instances. Instances are launched
    # immediately if there is available capacity.
    #
    # A single EC2 Fleet can include multiple launch specifications that
    # vary by instance type, AMI, Availability Zone, or subnet.
    #
    # For more information, see [EC2 Fleet][1] in the *Amazon EC2 User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-fleet.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. If you do not specify a client token, a
    #   randomly generated token is used for the request to ensure
    #   idempotency.
    #
    #   For more information, see [Ensuring idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [Types::SpotOptionsRequest] :spot_options
    #   Describes the configuration of Spot Instances in an EC2 Fleet.
    #
    # @option params [Types::OnDemandOptionsRequest] :on_demand_options
    #   Describes the configuration of On-Demand Instances in an EC2 Fleet.
    #
    # @option params [String] :excess_capacity_termination_policy
    #   Indicates whether running instances should be terminated if the total
    #   target capacity of the EC2 Fleet is decreased below the current size
    #   of the EC2 Fleet.
    #
    #   Supported only for fleets of type `maintain`.
    #
    # @option params [required, Array<Types::FleetLaunchTemplateConfigRequest>] :launch_template_configs
    #   The configuration for the EC2 Fleet.
    #
    # @option params [required, Types::TargetCapacitySpecificationRequest] :target_capacity_specification
    #   The number of units to request.
    #
    # @option params [Boolean] :terminate_instances_with_expiration
    #   Indicates whether running instances should be terminated when the EC2
    #   Fleet expires.
    #
    # @option params [String] :type
    #   The fleet type. The default value is `maintain`.
    #
    #   * `maintain` - The EC2 Fleet places an asynchronous request for your
    #     desired capacity, and continues to maintain your desired Spot
    #     capacity by replenishing interrupted Spot Instances.
    #
    #   * `request` - The EC2 Fleet places an asynchronous one-time request
    #     for your desired capacity, but does submit Spot requests in
    #     alternative capacity pools if Spot capacity is unavailable, and does
    #     not maintain Spot capacity if Spot Instances are interrupted.
    #
    #   * `instant` - The EC2 Fleet places a synchronous one-time request for
    #     your desired capacity, and returns errors for any instances that
    #     could not be launched.
    #
    #   For more information, see [EC2 Fleet request types][1] in the *Amazon
    #   EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-fleet-request-type.html
    #
    # @option params [Time,DateTime,Date,Integer,String] :valid_from
    #   The start date and time of the request, in UTC format (for example,
    #   *YYYY*-*MM*-*DD*T*HH*:*MM*:*SS*Z). The default is to start fulfilling
    #   the request immediately.
    #
    # @option params [Time,DateTime,Date,Integer,String] :valid_until
    #   The end date and time of the request, in UTC format (for example,
    #   *YYYY*-*MM*-*DD*T*HH*:*MM*:*SS*Z). At this point, no new EC2 Fleet
    #   requests are placed or able to fulfill the request. If no value is
    #   specified, the request remains until you cancel it.
    #
    # @option params [Boolean] :replace_unhealthy_instances
    #   Indicates whether EC2 Fleet should replace unhealthy Spot Instances.
    #   Supported only for fleets of type `maintain`. For more information,
    #   see [EC2 Fleet health checks][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/manage-ec2-fleet.html#ec2-fleet-health-checks
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The key-value pair for tagging the EC2 Fleet request on creation. For
    #   more information, see [Tag your resources][1].
    #
    #   If the fleet type is `instant`, specify a resource type of `fleet` to
    #   tag the fleet or `instance` to tag the instances at launch.
    #
    #   If the fleet type is `maintain` or `request`, specify a resource type
    #   of `fleet` to tag the fleet. You cannot specify a resource type of
    #   `instance`. To tag instances at launch, specify the tags in a [launch
    #   template][2].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html#tag-resources
    #   [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-launch-templates.html#create-launch-template
    #
    # @option params [String] :context
    #   Reserved.
    #
    # @return [Types::CreateFleetResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateFleetResult#fleet_id #fleet_id} => String
    #   * {Types::CreateFleetResult#errors #errors} => Array&lt;Types::CreateFleetError&gt;
    #   * {Types::CreateFleetResult#instances #instances} => Array&lt;Types::CreateFleetInstance&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_fleet({
    #     dry_run: false,
    #     client_token: "String",
    #     spot_options: {
    #       allocation_strategy: "lowest-price", # accepts lowest-price, diversified, capacity-optimized, capacity-optimized-prioritized, price-capacity-optimized
    #       maintenance_strategies: {
    #         capacity_rebalance: {
    #           replacement_strategy: "launch", # accepts launch, launch-before-terminate
    #           termination_delay: 1,
    #         },
    #       },
    #       instance_interruption_behavior: "hibernate", # accepts hibernate, stop, terminate
    #       instance_pools_to_use_count: 1,
    #       single_instance_type: false,
    #       single_availability_zone: false,
    #       min_target_capacity: 1,
    #       max_total_price: "String",
    #     },
    #     on_demand_options: {
    #       allocation_strategy: "lowest-price", # accepts lowest-price, prioritized
    #       capacity_reservation_options: {
    #         usage_strategy: "use-capacity-reservations-first", # accepts use-capacity-reservations-first
    #       },
    #       single_instance_type: false,
    #       single_availability_zone: false,
    #       min_target_capacity: 1,
    #       max_total_price: "String",
    #     },
    #     excess_capacity_termination_policy: "no-termination", # accepts no-termination, termination
    #     launch_template_configs: [ # required
    #       {
    #         launch_template_specification: {
    #           launch_template_id: "LaunchTemplateId",
    #           launch_template_name: "LaunchTemplateName",
    #           version: "String",
    #         },
    #         overrides: [
    #           {
    #             instance_type: "a1.medium", # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #             max_price: "String",
    #             subnet_id: "SubnetId",
    #             availability_zone: "String",
    #             weighted_capacity: 1.0,
    #             priority: 1.0,
    #             placement: {
    #               affinity: "String",
    #               group_name: "PlacementGroupName",
    #               partition_number: 1,
    #               host_id: "String",
    #               tenancy: "default", # accepts default, dedicated, host
    #               spread_domain: "String",
    #               host_resource_group_arn: "String",
    #               group_id: "PlacementGroupId",
    #               availability_zone: "String",
    #             },
    #             block_device_mappings: [
    #               {
    #                 device_name: "String",
    #                 virtual_name: "String",
    #                 ebs: {
    #                   encrypted: false,
    #                   delete_on_termination: false,
    #                   iops: 1,
    #                   throughput: 1,
    #                   kms_key_id: "KmsKeyId",
    #                   snapshot_id: "SnapshotId",
    #                   volume_size: 1,
    #                   volume_type: "standard", # accepts standard, io1, io2, gp2, sc1, st1, gp3
    #                 },
    #                 no_device: "String",
    #               },
    #             ],
    #             instance_requirements: {
    #               v_cpu_count: { # required
    #                 min: 1, # required
    #                 max: 1,
    #               },
    #               memory_mi_b: { # required
    #                 min: 1, # required
    #                 max: 1,
    #               },
    #               cpu_manufacturers: ["intel"], # accepts intel, amd, amazon-web-services, apple
    #               memory_gi_b_per_v_cpu: {
    #                 min: 1.0,
    #                 max: 1.0,
    #               },
    #               excluded_instance_types: ["ExcludedInstanceType"],
    #               instance_generations: ["current"], # accepts current, previous
    #               spot_max_price_percentage_over_lowest_price: 1,
    #               on_demand_max_price_percentage_over_lowest_price: 1,
    #               bare_metal: "included", # accepts included, required, excluded
    #               burstable_performance: "included", # accepts included, required, excluded
    #               require_hibernate_support: false,
    #               network_interface_count: {
    #                 min: 1,
    #                 max: 1,
    #               },
    #               local_storage: "included", # accepts included, required, excluded
    #               local_storage_types: ["hdd"], # accepts hdd, ssd
    #               total_local_storage_gb: {
    #                 min: 1.0,
    #                 max: 1.0,
    #               },
    #               baseline_ebs_bandwidth_mbps: {
    #                 min: 1,
    #                 max: 1,
    #               },
    #               accelerator_types: ["gpu"], # accepts gpu, fpga, inference
    #               accelerator_count: {
    #                 min: 1,
    #                 max: 1,
    #               },
    #               accelerator_manufacturers: ["amazon-web-services"], # accepts amazon-web-services, amd, nvidia, xilinx, habana
    #               accelerator_names: ["a100"], # accepts a100, inferentia, k520, k80, m60, radeon-pro-v520, t4, vu9p, v100, a10g, h100, t4g
    #               accelerator_total_memory_mi_b: {
    #                 min: 1,
    #                 max: 1,
    #               },
    #               network_bandwidth_gbps: {
    #                 min: 1.0,
    #                 max: 1.0,
    #               },
    #               allowed_instance_types: ["AllowedInstanceType"],
    #               max_spot_price_as_percentage_of_optimal_on_demand_price: 1,
    #               baseline_performance_factors: {
    #                 cpu: {
    #                   references: [
    #                     {
    #                       instance_family: "String",
    #                     },
    #                   ],
    #                 },
    #               },
    #             },
    #             image_id: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     target_capacity_specification: { # required
    #       total_target_capacity: 1, # required
    #       on_demand_target_capacity: 1,
    #       spot_target_capacity: 1,
    #       default_target_capacity_type: "spot", # accepts spot, on-demand, capacity-block
    #       target_capacity_unit_type: "vcpu", # accepts vcpu, memory-mib, units
    #     },
    #     terminate_instances_with_expiration: false,
    #     type: "request", # accepts request, maintain, instant
    #     valid_from: Time.now,
    #     valid_until: Time.now,
    #     replace_unhealthy_instances: false,
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     context: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.fleet_id #=> String
    #   resp.errors #=> Array
    #   resp.errors[0].launch_template_and_overrides.launch_template_specification.launch_template_id #=> String
    #   resp.errors[0].launch_template_and_overrides.launch_template_specification.launch_template_name #=> String
    #   resp.errors[0].launch_template_and_overrides.launch_template_specification.version #=> String
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.errors[0].launch_template_and_overrides.overrides.max_price #=> String
    #   resp.errors[0].launch_template_and_overrides.overrides.subnet_id #=> String
    #   resp.errors[0].launch_template_and_overrides.overrides.availability_zone #=> String
    #   resp.errors[0].launch_template_and_overrides.overrides.weighted_capacity #=> Float
    #   resp.errors[0].launch_template_and_overrides.overrides.priority #=> Float
    #   resp.errors[0].launch_template_and_overrides.overrides.placement.group_name #=> String
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.v_cpu_count.min #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.v_cpu_count.max #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.memory_mi_b.min #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.memory_mi_b.max #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.cpu_manufacturers #=> Array
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.cpu_manufacturers[0] #=> String, one of "intel", "amd", "amazon-web-services", "apple"
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.memory_gi_b_per_v_cpu.min #=> Float
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.memory_gi_b_per_v_cpu.max #=> Float
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.excluded_instance_types #=> Array
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.excluded_instance_types[0] #=> String
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.instance_generations #=> Array
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.instance_generations[0] #=> String, one of "current", "previous"
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.spot_max_price_percentage_over_lowest_price #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.on_demand_max_price_percentage_over_lowest_price #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.bare_metal #=> String, one of "included", "required", "excluded"
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.burstable_performance #=> String, one of "included", "required", "excluded"
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.require_hibernate_support #=> Boolean
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.network_interface_count.min #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.network_interface_count.max #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.local_storage #=> String, one of "included", "required", "excluded"
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.local_storage_types #=> Array
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.local_storage_types[0] #=> String, one of "hdd", "ssd"
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.total_local_storage_gb.min #=> Float
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.total_local_storage_gb.max #=> Float
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.baseline_ebs_bandwidth_mbps.min #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.baseline_ebs_bandwidth_mbps.max #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_types #=> Array
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_types[0] #=> String, one of "gpu", "fpga", "inference"
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_count.min #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_count.max #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_manufacturers #=> Array
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_manufacturers[0] #=> String, one of "amazon-web-services", "amd", "nvidia", "xilinx", "habana"
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_names #=> Array
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_names[0] #=> String, one of "a100", "inferentia", "k520", "k80", "m60", "radeon-pro-v520", "t4", "vu9p", "v100", "a10g", "h100", "t4g"
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_total_memory_mi_b.min #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_total_memory_mi_b.max #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.network_bandwidth_gbps.min #=> Float
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.network_bandwidth_gbps.max #=> Float
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.allowed_instance_types #=> Array
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.allowed_instance_types[0] #=> String
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.max_spot_price_as_percentage_of_optimal_on_demand_price #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.baseline_performance_factors.cpu.references #=> Array
    #   resp.errors[0].launch_template_and_overrides.overrides.instance_requirements.baseline_performance_factors.cpu.references[0].instance_family #=> String
    #   resp.errors[0].launch_template_and_overrides.overrides.image_id #=> String
    #   resp.errors[0].launch_template_and_overrides.overrides.block_device_mappings #=> Array
    #   resp.errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].device_name #=> String
    #   resp.errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].virtual_name #=> String
    #   resp.errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.encrypted #=> Boolean
    #   resp.errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.iops #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.throughput #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.kms_key_id #=> String
    #   resp.errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.snapshot_id #=> String
    #   resp.errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.volume_size #=> Integer
    #   resp.errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].no_device #=> String
    #   resp.errors[0].lifecycle #=> String, one of "spot", "on-demand"
    #   resp.errors[0].error_code #=> String
    #   resp.errors[0].error_message #=> String
    #   resp.instances #=> Array
    #   resp.instances[0].launch_template_and_overrides.launch_template_specification.launch_template_id #=> String
    #   resp.instances[0].launch_template_and_overrides.launch_template_specification.launch_template_name #=> String
    #   resp.instances[0].launch_template_and_overrides.launch_template_specification.version #=> String
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.instances[0].launch_template_and_overrides.overrides.max_price #=> String
    #   resp.instances[0].launch_template_and_overrides.overrides.subnet_id #=> String
    #   resp.instances[0].launch_template_and_overrides.overrides.availability_zone #=> String
    #   resp.instances[0].launch_template_and_overrides.overrides.weighted_capacity #=> Float
    #   resp.instances[0].launch_template_and_overrides.overrides.priority #=> Float
    #   resp.instances[0].launch_template_and_overrides.overrides.placement.group_name #=> String
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.v_cpu_count.min #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.v_cpu_count.max #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.memory_mi_b.min #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.memory_mi_b.max #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.cpu_manufacturers #=> Array
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.cpu_manufacturers[0] #=> String, one of "intel", "amd", "amazon-web-services", "apple"
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.memory_gi_b_per_v_cpu.min #=> Float
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.memory_gi_b_per_v_cpu.max #=> Float
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.excluded_instance_types #=> Array
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.excluded_instance_types[0] #=> String
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.instance_generations #=> Array
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.instance_generations[0] #=> String, one of "current", "previous"
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.spot_max_price_percentage_over_lowest_price #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.on_demand_max_price_percentage_over_lowest_price #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.bare_metal #=> String, one of "included", "required", "excluded"
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.burstable_performance #=> String, one of "included", "required", "excluded"
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.require_hibernate_support #=> Boolean
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.network_interface_count.min #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.network_interface_count.max #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.local_storage #=> String, one of "included", "required", "excluded"
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.local_storage_types #=> Array
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.local_storage_types[0] #=> String, one of "hdd", "ssd"
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.total_local_storage_gb.min #=> Float
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.total_local_storage_gb.max #=> Float
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.baseline_ebs_bandwidth_mbps.min #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.baseline_ebs_bandwidth_mbps.max #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_types #=> Array
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_types[0] #=> String, one of "gpu", "fpga", "inference"
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_count.min #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_count.max #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_manufacturers #=> Array
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_manufacturers[0] #=> String, one of "amazon-web-services", "amd", "nvidia", "xilinx", "habana"
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_names #=> Array
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_names[0] #=> String, one of "a100", "inferentia", "k520", "k80", "m60", "radeon-pro-v520", "t4", "vu9p", "v100", "a10g", "h100", "t4g"
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_total_memory_mi_b.min #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_total_memory_mi_b.max #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.network_bandwidth_gbps.min #=> Float
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.network_bandwidth_gbps.max #=> Float
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.allowed_instance_types #=> Array
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.allowed_instance_types[0] #=> String
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.max_spot_price_as_percentage_of_optimal_on_demand_price #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.baseline_performance_factors.cpu.references #=> Array
    #   resp.instances[0].launch_template_and_overrides.overrides.instance_requirements.baseline_performance_factors.cpu.references[0].instance_family #=> String
    #   resp.instances[0].launch_template_and_overrides.overrides.image_id #=> String
    #   resp.instances[0].launch_template_and_overrides.overrides.block_device_mappings #=> Array
    #   resp.instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].device_name #=> String
    #   resp.instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].virtual_name #=> String
    #   resp.instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.encrypted #=> Boolean
    #   resp.instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.iops #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.throughput #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.kms_key_id #=> String
    #   resp.instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.snapshot_id #=> String
    #   resp.instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.volume_size #=> Integer
    #   resp.instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].no_device #=> String
    #   resp.instances[0].lifecycle #=> String, one of "spot", "on-demand"
    #   resp.instances[0].instance_ids #=> Array
    #   resp.instances[0].instance_ids[0] #=> String
    #   resp.instances[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.instances[0].platform #=> String, one of "Windows"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateFleet AWS API Documentation
    #
    # @overload create_fleet(params = {})
    # @param [Hash] params ({})
    def create_fleet(params = {}, options = {})
      req = build_request(:create_fleet, params)
      req.send_request(options)
    end

    # Creates one or more flow logs to capture information about IP traffic
    # for a specific network interface, subnet, or VPC.
    #
    # Flow log data for a monitored network interface is recorded as flow
    # log records, which are log events consisting of fields that describe
    # the traffic flow. For more information, see [Flow log records][1] in
    # the *Amazon VPC User Guide*.
    #
    # When publishing to CloudWatch Logs, flow log records are published to
    # a log group, and each network interface has a unique log stream in the
    # log group. When publishing to Amazon S3, flow log records for all of
    # the monitored network interfaces are published to a single log file
    # object that is stored in the specified bucket.
    #
    # For more information, see [VPC Flow Logs][2] in the *Amazon VPC User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/flow-log-records.html
    # [2]: https://docs.aws.amazon.com/vpc/latest/userguide/flow-logs.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [How to ensure
    #   idempotency][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [String] :deliver_logs_permission_arn
    #   The ARN of the IAM role that allows Amazon EC2 to publish flow logs to
    #   the log destination.
    #
    #   This parameter is required if the destination type is
    #   `cloud-watch-logs`, or if the destination type is
    #   `kinesis-data-firehose` and the delivery stream and the resources to
    #   monitor are in different accounts.
    #
    # @option params [String] :deliver_cross_account_role
    #   The ARN of the IAM role that allows Amazon EC2 to publish flow logs
    #   across accounts.
    #
    # @option params [String] :log_group_name
    #   The name of a new or existing CloudWatch Logs log group where Amazon
    #   EC2 publishes your flow logs.
    #
    #   This parameter is valid only if the destination type is
    #   `cloud-watch-logs`.
    #
    # @option params [required, Array<String>] :resource_ids
    #   The IDs of the resources to monitor. For example, if the resource type
    #   is `VPC`, specify the IDs of the VPCs.
    #
    #   Constraints: Maximum of 25 for transit gateway resource types. Maximum
    #   of 1000 for the other resource types.
    #
    # @option params [required, String] :resource_type
    #   The type of resource to monitor.
    #
    # @option params [String] :traffic_type
    #   The type of traffic to monitor (accepted traffic, rejected traffic, or
    #   all traffic). This parameter is not supported for transit gateway
    #   resource types. It is required for the other resource types.
    #
    # @option params [String] :log_destination_type
    #   The type of destination for the flow log data.
    #
    #   Default: `cloud-watch-logs`
    #
    # @option params [String] :log_destination
    #   The destination for the flow log data. The meaning of this parameter
    #   depends on the destination type.
    #
    #   * If the destination type is `cloud-watch-logs`, specify the ARN of a
    #     CloudWatch Logs log group. For example:
    #
    #     arn:aws:logs:*region*:*account\_id*:log-group:*my\_group*
    #
    #     Alternatively, use the `LogGroupName` parameter.
    #
    #   * If the destination type is `s3`, specify the ARN of an S3 bucket.
    #     For example:
    #
    #     arn:aws:s3:::*my\_bucket*/*my\_subfolder*/
    #
    #     The subfolder is optional. Note that you can't use `AWSLogs` as a
    #     subfolder name.
    #
    #   * If the destination type is `kinesis-data-firehose`, specify the ARN
    #     of a Kinesis Data Firehose delivery stream. For example:
    #
    #     arn:aws:firehose:*region*:*account\_id*:deliverystream:*my\_stream*
    #
    # @option params [String] :log_format
    #   The fields to include in the flow log record. List the fields in the
    #   order in which they should appear. If you omit this parameter, the
    #   flow log is created using the default format. If you specify this
    #   parameter, you must include at least one field. For more information
    #   about the available fields, see [Flow log records][1] in the *Amazon
    #   VPC User Guide* or [Transit Gateway Flow Log records][2] in the
    #   *Amazon Web Services Transit Gateway Guide*.
    #
    #   Specify the fields using the `${field-id}` format, separated by
    #   spaces.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/userguide/flow-log-records.html
    #   [2]: https://docs.aws.amazon.com/vpc/latest/tgw/tgw-flow-logs.html#flow-log-records
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the flow logs.
    #
    # @option params [Integer] :max_aggregation_interval
    #   The maximum interval of time during which a flow of packets is
    #   captured and aggregated into a flow log record. The possible values
    #   are 60 seconds (1 minute) or 600 seconds (10 minutes). This parameter
    #   must be 60 seconds for transit gateway resource types.
    #
    #   When a network interface is attached to a [Nitro-based instance][1],
    #   the aggregation interval is always 60 seconds or less, regardless of
    #   the value that you specify.
    #
    #   Default: 600
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/instancetypes/ec2-nitro-instances.html
    #
    # @option params [Types::DestinationOptionsRequest] :destination_options
    #   The destination options.
    #
    # @return [Types::CreateFlowLogsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateFlowLogsResult#client_token #client_token} => String
    #   * {Types::CreateFlowLogsResult#flow_log_ids #flow_log_ids} => Array&lt;String&gt;
    #   * {Types::CreateFlowLogsResult#unsuccessful #unsuccessful} => Array&lt;Types::UnsuccessfulItem&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_flow_logs({
    #     dry_run: false,
    #     client_token: "String",
    #     deliver_logs_permission_arn: "String",
    #     deliver_cross_account_role: "String",
    #     log_group_name: "String",
    #     resource_ids: ["FlowLogResourceId"], # required
    #     resource_type: "VPC", # required, accepts VPC, Subnet, NetworkInterface, TransitGateway, TransitGatewayAttachment
    #     traffic_type: "ACCEPT", # accepts ACCEPT, REJECT, ALL
    #     log_destination_type: "cloud-watch-logs", # accepts cloud-watch-logs, s3, kinesis-data-firehose
    #     log_destination: "String",
    #     log_format: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     max_aggregation_interval: 1,
    #     destination_options: {
    #       file_format: "plain-text", # accepts plain-text, parquet
    #       hive_compatible_partitions: false,
    #       per_hour_partition: false,
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.client_token #=> String
    #   resp.flow_log_ids #=> Array
    #   resp.flow_log_ids[0] #=> String
    #   resp.unsuccessful #=> Array
    #   resp.unsuccessful[0].error.code #=> String
    #   resp.unsuccessful[0].error.message #=> String
    #   resp.unsuccessful[0].resource_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateFlowLogs AWS API Documentation
    #
    # @overload create_flow_logs(params = {})
    # @param [Hash] params ({})
    def create_flow_logs(params = {}, options = {})
      req = build_request(:create_flow_logs, params)
      req.send_request(options)
    end

    # Creates an Amazon FPGA Image (AFI) from the specified design
    # checkpoint (DCP).
    #
    # The create operation is asynchronous. To verify that the AFI is ready
    # for use, check the output logs.
    #
    # An AFI contains the FPGA bitstream that is ready to download to an
    # FPGA. You can securely deploy an AFI on multiple FPGA-accelerated
    # instances. For more information, see the [Amazon Web Services FPGA
    # Hardware Development Kit][1].
    #
    #
    #
    # [1]: https://github.com/aws/aws-fpga/
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Types::StorageLocation] :input_storage_location
    #   The location of the encrypted design checkpoint in Amazon S3. The
    #   input must be a tarball.
    #
    # @option params [Types::StorageLocation] :logs_storage_location
    #   The location in Amazon S3 for the output logs.
    #
    # @option params [String] :description
    #   A description for the AFI.
    #
    # @option params [String] :name
    #   A name for the AFI.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   Idempotency][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the FPGA image during creation.
    #
    # @return [Types::CreateFpgaImageResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateFpgaImageResult#fpga_image_id #fpga_image_id} => String
    #   * {Types::CreateFpgaImageResult#fpga_image_global_id #fpga_image_global_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_fpga_image({
    #     dry_run: false,
    #     input_storage_location: { # required
    #       bucket: "String",
    #       key: "String",
    #     },
    #     logs_storage_location: {
    #       bucket: "String",
    #       key: "String",
    #     },
    #     description: "String",
    #     name: "String",
    #     client_token: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.fpga_image_id #=> String
    #   resp.fpga_image_global_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateFpgaImage AWS API Documentation
    #
    # @overload create_fpga_image(params = {})
    # @param [Hash] params ({})
    def create_fpga_image(params = {}, options = {})
      req = build_request(:create_fpga_image, params)
      req.send_request(options)
    end

    # Creates an Amazon EBS-backed AMI from an Amazon EBS-backed instance
    # that is either running or stopped.
    #
    # If you customized your instance with instance store volumes or Amazon
    # EBS volumes in addition to the root device volume, the new AMI
    # contains block device mapping information for those volumes. When you
    # launch an instance from this new AMI, the instance automatically
    # launches with those additional volumes.
    #
    # For more information, see [Create an Amazon EBS-backed Linux AMI][1]
    # in the *Amazon Elastic Compute Cloud User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/creating-an-ami-ebs.html
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the AMI and snapshots on creation. You can tag
    #   the AMI, the snapshots, or both.
    #
    #   * To tag the AMI, the value for `ResourceType` must be `image`.
    #
    #   * To tag the snapshots that are created of the root volume and of
    #     other Amazon EBS volumes that are attached to the instance, the
    #     value for `ResourceType` must be `snapshot`. The same tag is applied
    #     to all of the snapshots that are created.
    #
    #   If you specify other values for `ResourceType`, the request fails.
    #
    #   To tag an AMI or snapshot after it has been created, see
    #   [CreateTags][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTags.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @option params [required, String] :name
    #   A name for the new image.
    #
    #   Constraints: 3-128 alphanumeric characters, parentheses (()), square
    #   brackets (\[\]), spaces ( ), periods (.), slashes (/), dashes (-),
    #   single quotes ('), at-signs (@), or underscores(\_)
    #
    # @option params [String] :description
    #   A description for the new image.
    #
    # @option params [Boolean] :no_reboot
    #   Indicates whether or not the instance should be automatically rebooted
    #   before creating the image. Specify one of the following values:
    #
    #   * `true` - The instance is not rebooted before creating the image.
    #     This creates crash-consistent snapshots that include only the data
    #     that has been written to the volumes at the time the snapshots are
    #     created. Buffered data and data in memory that has not yet been
    #     written to the volumes is not included in the snapshots.
    #
    #   * `false` - The instance is rebooted before creating the image. This
    #     ensures that all buffered data and data in memory is written to the
    #     volumes before the snapshots are created.
    #
    #   Default: `false`
    #
    # @option params [Array<Types::BlockDeviceMapping>] :block_device_mappings
    #   The block device mappings.
    #
    #   When using the CreateImage action:
    #
    #   * You can't change the volume size using the VolumeSize parameter. If
    #     you want a different volume size, you must first change the volume
    #     size of the source instance.
    #
    #   * You can't modify the encryption status of existing volumes or
    #     snapshots. To create an AMI with volumes or snapshots that have a
    #     different encryption status (for example, where the source volume
    #     and snapshots are unencrypted, and you want to create an AMI with
    #     encrypted volumes or snapshots), use the CopyImage action.
    #
    #   * The only option that can be changed for existing mappings or
    #     snapshots is `DeleteOnTermination`.
    #
    # @return [Types::CreateImageResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateImageResult#image_id #image_id} => String
    #
    #
    # @example Example: To create an AMI from an Amazon EBS-backed instance
    #
    #   # This example creates an AMI from the specified instance and adds an EBS volume with the device name /dev/sdh and an
    #   # instance store volume with the device name /dev/sdc.
    #
    #   resp = client.create_image({
    #     block_device_mappings: [
    #       {
    #         device_name: "/dev/sdh", 
    #         ebs: {
    #           volume_size: 100, 
    #         }, 
    #       }, 
    #       {
    #         device_name: "/dev/sdc", 
    #         virtual_name: "ephemeral1", 
    #       }, 
    #     ], 
    #     description: "An AMI for my server", 
    #     instance_id: "i-1234567890abcdef0", 
    #     name: "My server", 
    #     no_reboot: true, 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     image_id: "ami-1a2b3c4d", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_image({
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #     instance_id: "InstanceId", # required
    #     name: "String", # required
    #     description: "String",
    #     no_reboot: false,
    #     block_device_mappings: [
    #       {
    #         ebs: {
    #           delete_on_termination: false,
    #           iops: 1,
    #           snapshot_id: "SnapshotId",
    #           volume_size: 1,
    #           volume_type: "standard", # accepts standard, io1, io2, gp2, sc1, st1, gp3
    #           kms_key_id: "String",
    #           throughput: 1,
    #           outpost_arn: "String",
    #           encrypted: false,
    #         },
    #         no_device: "String",
    #         device_name: "String",
    #         virtual_name: "String",
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.image_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateImage AWS API Documentation
    #
    # @overload create_image(params = {})
    # @param [Hash] params ({})
    def create_image(params = {}, options = {})
      req = build_request(:create_image, params)
      req.send_request(options)
    end

    # Creates an EC2 Instance Connect Endpoint.
    #
    # An EC2 Instance Connect Endpoint allows you to connect to an instance,
    # without requiring the instance to have a public IPv4 address. For more
    # information, see [Connect to your instances without requiring a public
    # IPv4 address using EC2 Instance Connect Endpoint][1] in the *Amazon
    # EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Connect-using-EC2-Instance-Connect-Endpoint.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :subnet_id
    #   The ID of the subnet in which to create the EC2 Instance Connect
    #   Endpoint.
    #
    # @option params [Array<String>] :security_group_ids
    #   One or more security groups to associate with the endpoint. If you
    #   don't specify a security group, the default security group for your
    #   VPC will be associated with the endpoint.
    #
    # @option params [Boolean] :preserve_client_ip
    #   Indicates whether the client IP address is preserved as the source.
    #   The following are the possible values.
    #
    #   * `true` - Use the client IP address as the source.
    #
    #   * `false` - Use the network interface IP address as the source.
    #
    #   Default: `false`
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request.
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the EC2 Instance Connect Endpoint during
    #   creation.
    #
    # @return [Types::CreateInstanceConnectEndpointResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateInstanceConnectEndpointResult#instance_connect_endpoint #instance_connect_endpoint} => Types::Ec2InstanceConnectEndpoint
    #   * {Types::CreateInstanceConnectEndpointResult#client_token #client_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_instance_connect_endpoint({
    #     dry_run: false,
    #     subnet_id: "SubnetId", # required
    #     security_group_ids: ["SecurityGroupId"],
    #     preserve_client_ip: false,
    #     client_token: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_connect_endpoint.owner_id #=> String
    #   resp.instance_connect_endpoint.instance_connect_endpoint_id #=> String
    #   resp.instance_connect_endpoint.instance_connect_endpoint_arn #=> String
    #   resp.instance_connect_endpoint.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "delete-in-progress", "delete-complete", "delete-failed"
    #   resp.instance_connect_endpoint.state_message #=> String
    #   resp.instance_connect_endpoint.dns_name #=> String
    #   resp.instance_connect_endpoint.fips_dns_name #=> String
    #   resp.instance_connect_endpoint.network_interface_ids #=> Array
    #   resp.instance_connect_endpoint.network_interface_ids[0] #=> String
    #   resp.instance_connect_endpoint.vpc_id #=> String
    #   resp.instance_connect_endpoint.availability_zone #=> String
    #   resp.instance_connect_endpoint.created_at #=> Time
    #   resp.instance_connect_endpoint.subnet_id #=> String
    #   resp.instance_connect_endpoint.preserve_client_ip #=> Boolean
    #   resp.instance_connect_endpoint.security_group_ids #=> Array
    #   resp.instance_connect_endpoint.security_group_ids[0] #=> String
    #   resp.instance_connect_endpoint.tags #=> Array
    #   resp.instance_connect_endpoint.tags[0].key #=> String
    #   resp.instance_connect_endpoint.tags[0].value #=> String
    #   resp.client_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateInstanceConnectEndpoint AWS API Documentation
    #
    # @overload create_instance_connect_endpoint(params = {})
    # @param [Hash] params ({})
    def create_instance_connect_endpoint(params = {}, options = {})
      req = build_request(:create_instance_connect_endpoint, params)
      req.send_request(options)
    end

    # Creates an event window in which scheduled events for the associated
    # Amazon EC2 instances can run.
    #
    # You can define either a set of time ranges or a cron expression when
    # creating the event window, but not both. All event window times are in
    # UTC.
    #
    # You can create up to 200 event windows per Amazon Web Services Region.
    #
    # When you create the event window, targets (instance IDs, Dedicated
    # Host IDs, or tags) are not yet associated with it. To ensure that the
    # event window can be used, you must associate one or more targets with
    # it by using the AssociateInstanceEventWindow API.
    #
    # Event windows are applicable only for scheduled events that stop,
    # reboot, or terminate instances.
    #
    #  Event windows are *not* applicable for:
    #
    #  * Expedited scheduled events and network maintenance events.
    #
    # * Unscheduled maintenance such as AutoRecovery and unplanned reboots.
    #
    # For more information, see [Define event windows for scheduled
    # events][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/event-windows.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :name
    #   The name of the event window.
    #
    # @option params [Array<Types::InstanceEventWindowTimeRangeRequest>] :time_ranges
    #   The time range for the event window. If you specify a time range, you
    #   can't specify a cron expression.
    #
    # @option params [String] :cron_expression
    #   The cron expression for the event window, for example, `* 0-4,20-23 *
    #   * 1,5`. If you specify a cron expression, you can't specify a time
    #   range.
    #
    #   Constraints:
    #
    #   * Only hour and day of the week values are supported.
    #
    #   * For day of the week values, you can specify either integers `0`
    #     through `6`, or alternative single values `SUN` through `SAT`.
    #
    #   * The minute, month, and year must be specified by `*`.
    #
    #   * The hour value must be one or a multiple range, for example, `0-4`
    #     or `0-4,20-23`.
    #
    #   * Each hour range must be &gt;= 2 hours, for example, `0-2` or
    #     `20-23`.
    #
    #   * The event window must be &gt;= 4 hours. The combined total time
    #     ranges in the event window must be &gt;= 4 hours.
    #
    #   For more information about cron expressions, see [cron][1] on the
    #   *Wikipedia website*.
    #
    #
    #
    #   [1]: https://en.wikipedia.org/wiki/Cron
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the event window.
    #
    # @return [Types::CreateInstanceEventWindowResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateInstanceEventWindowResult#instance_event_window #instance_event_window} => Types::InstanceEventWindow
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_instance_event_window({
    #     dry_run: false,
    #     name: "String",
    #     time_ranges: [
    #       {
    #         start_week_day: "sunday", # accepts sunday, monday, tuesday, wednesday, thursday, friday, saturday
    #         start_hour: 1,
    #         end_week_day: "sunday", # accepts sunday, monday, tuesday, wednesday, thursday, friday, saturday
    #         end_hour: 1,
    #       },
    #     ],
    #     cron_expression: "InstanceEventWindowCronExpression",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_event_window.instance_event_window_id #=> String
    #   resp.instance_event_window.time_ranges #=> Array
    #   resp.instance_event_window.time_ranges[0].start_week_day #=> String, one of "sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"
    #   resp.instance_event_window.time_ranges[0].start_hour #=> Integer
    #   resp.instance_event_window.time_ranges[0].end_week_day #=> String, one of "sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"
    #   resp.instance_event_window.time_ranges[0].end_hour #=> Integer
    #   resp.instance_event_window.name #=> String
    #   resp.instance_event_window.cron_expression #=> String
    #   resp.instance_event_window.association_target.instance_ids #=> Array
    #   resp.instance_event_window.association_target.instance_ids[0] #=> String
    #   resp.instance_event_window.association_target.tags #=> Array
    #   resp.instance_event_window.association_target.tags[0].key #=> String
    #   resp.instance_event_window.association_target.tags[0].value #=> String
    #   resp.instance_event_window.association_target.dedicated_host_ids #=> Array
    #   resp.instance_event_window.association_target.dedicated_host_ids[0] #=> String
    #   resp.instance_event_window.state #=> String, one of "creating", "deleting", "active", "deleted"
    #   resp.instance_event_window.tags #=> Array
    #   resp.instance_event_window.tags[0].key #=> String
    #   resp.instance_event_window.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateInstanceEventWindow AWS API Documentation
    #
    # @overload create_instance_event_window(params = {})
    # @param [Hash] params ({})
    def create_instance_event_window(params = {}, options = {})
      req = build_request(:create_instance_event_window, params)
      req.send_request(options)
    end

    # Exports a running or stopped instance to an Amazon S3 bucket.
    #
    # For information about the prerequisites for your Amazon S3 bucket,
    # supported operating systems, image formats, and known limitations for
    # the types of instances you can export, see [Exporting an instance as a
    # VM Using VM Import/Export][1] in the *VM Import/Export User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vm-import/latest/userguide/vmexport.html
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the export instance task during creation.
    #
    # @option params [String] :description
    #   A description for the conversion task or the resource being exported.
    #   The maximum length is 255 characters.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @option params [required, String] :target_environment
    #   The target virtualization environment.
    #
    # @option params [required, Types::ExportToS3TaskSpecification] :export_to_s3_task
    #   The format and location for an export instance task.
    #
    # @return [Types::CreateInstanceExportTaskResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateInstanceExportTaskResult#export_task #export_task} => Types::ExportTask
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_instance_export_task({
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     description: "String",
    #     instance_id: "InstanceId", # required
    #     target_environment: "citrix", # required, accepts citrix, vmware, microsoft
    #     export_to_s3_task: { # required
    #       disk_image_format: "VMDK", # accepts VMDK, RAW, VHD
    #       container_format: "ova", # accepts ova
    #       s3_bucket: "String",
    #       s3_prefix: "String",
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.export_task.description #=> String
    #   resp.export_task.export_task_id #=> String
    #   resp.export_task.export_to_s3_task.container_format #=> String, one of "ova"
    #   resp.export_task.export_to_s3_task.disk_image_format #=> String, one of "VMDK", "RAW", "VHD"
    #   resp.export_task.export_to_s3_task.s3_bucket #=> String
    #   resp.export_task.export_to_s3_task.s3_key #=> String
    #   resp.export_task.instance_export_details.instance_id #=> String
    #   resp.export_task.instance_export_details.target_environment #=> String, one of "citrix", "vmware", "microsoft"
    #   resp.export_task.state #=> String, one of "active", "cancelling", "cancelled", "completed"
    #   resp.export_task.status_message #=> String
    #   resp.export_task.tags #=> Array
    #   resp.export_task.tags[0].key #=> String
    #   resp.export_task.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateInstanceExportTask AWS API Documentation
    #
    # @overload create_instance_export_task(params = {})
    # @param [Hash] params ({})
    def create_instance_export_task(params = {}, options = {})
      req = build_request(:create_instance_export_task, params)
      req.send_request(options)
    end

    # Creates an internet gateway for use with a VPC. After creating the
    # internet gateway, you attach it to a VPC using AttachInternetGateway.
    #
    # For more information, see [Internet gateways][1] in the *Amazon VPC
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Internet_Gateway.html
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the internet gateway.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateInternetGatewayResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateInternetGatewayResult#internet_gateway #internet_gateway} => Types::InternetGateway
    #
    #
    # @example Example: To create an Internet gateway
    #
    #   # This example creates an Internet gateway.
    #
    #   resp = client.create_internet_gateway({
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     internet_gateway: {
    #       attachments: [
    #       ], 
    #       internet_gateway_id: "igw-c0a643a9", 
    #       tags: [
    #       ], 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_internet_gateway({
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.internet_gateway.attachments #=> Array
    #   resp.internet_gateway.attachments[0].state #=> String, one of "attaching", "attached", "detaching", "detached"
    #   resp.internet_gateway.attachments[0].vpc_id #=> String
    #   resp.internet_gateway.internet_gateway_id #=> String
    #   resp.internet_gateway.owner_id #=> String
    #   resp.internet_gateway.tags #=> Array
    #   resp.internet_gateway.tags[0].key #=> String
    #   resp.internet_gateway.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateInternetGateway AWS API Documentation
    #
    # @overload create_internet_gateway(params = {})
    # @param [Hash] params ({})
    def create_internet_gateway(params = {}, options = {})
      req = build_request(:create_internet_gateway, params)
      req.send_request(options)
    end

    # Create an IPAM. Amazon VPC IP Address Manager (IPAM) is a VPC feature
    # that you can use to automate your IP address management workflows
    # including assigning, tracking, troubleshooting, and auditing IP
    # addresses across Amazon Web Services Regions and accounts throughout
    # your Amazon Web Services Organization.
    #
    # For more information, see [Create an IPAM][1] in the *Amazon VPC IPAM
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/create-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :description
    #   A description for the IPAM.
    #
    # @option params [Array<Types::AddIpamOperatingRegion>] :operating_regions
    #   The operating Regions for the IPAM. Operating Regions are Amazon Web
    #   Services Regions where the IPAM is allowed to manage IP address CIDRs.
    #   IPAM only discovers and monitors resources in the Amazon Web Services
    #   Regions you select as operating Regions.
    #
    #   For more information about operating Regions, see [Create an IPAM][1]
    #   in the *Amazon VPC IPAM User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/create-ipam.html
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The key/value combination of a tag assigned to the resource. Use the
    #   tag key in the filter name and the tag value as the filter value. For
    #   example, to find all resources that have a tag with the key `Owner`
    #   and the value `TeamA`, specify `tag:Owner` for the filter name and
    #   `TeamA` for the filter value.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [String] :tier
    #   IPAM is offered in a Free Tier and an Advanced Tier. For more
    #   information about the features available in each tier and the costs
    #   associated with the tiers, see [Amazon VPC pricing &gt; IPAM tab][1].
    #
    #
    #
    #   [1]: http://aws.amazon.com/vpc/pricing/
    #
    # @option params [Boolean] :enable_private_gua
    #   Enable this option to use your own GUA ranges as private IPv6
    #   addresses. This option is disabled by default.
    #
    # @return [Types::CreateIpamResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateIpamResult#ipam #ipam} => Types::Ipam
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_ipam({
    #     dry_run: false,
    #     description: "String",
    #     operating_regions: [
    #       {
    #         region_name: "String",
    #       },
    #     ],
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String",
    #     tier: "free", # accepts free, advanced
    #     enable_private_gua: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam.owner_id #=> String
    #   resp.ipam.ipam_id #=> String
    #   resp.ipam.ipam_arn #=> String
    #   resp.ipam.ipam_region #=> String
    #   resp.ipam.public_default_scope_id #=> String
    #   resp.ipam.private_default_scope_id #=> String
    #   resp.ipam.scope_count #=> Integer
    #   resp.ipam.description #=> String
    #   resp.ipam.operating_regions #=> Array
    #   resp.ipam.operating_regions[0].region_name #=> String
    #   resp.ipam.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam.tags #=> Array
    #   resp.ipam.tags[0].key #=> String
    #   resp.ipam.tags[0].value #=> String
    #   resp.ipam.default_resource_discovery_id #=> String
    #   resp.ipam.default_resource_discovery_association_id #=> String
    #   resp.ipam.resource_discovery_association_count #=> Integer
    #   resp.ipam.state_message #=> String
    #   resp.ipam.tier #=> String, one of "free", "advanced"
    #   resp.ipam.enable_private_gua #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateIpam AWS API Documentation
    #
    # @overload create_ipam(params = {})
    # @param [Hash] params ({})
    def create_ipam(params = {}, options = {})
      req = build_request(:create_ipam, params)
      req.send_request(options)
    end

    # Create a verification token. A verification token is an Amazon Web
    # Services-generated random value that you can use to prove ownership of
    # an external resource. For example, you can use a verification token to
    # validate that you control a public IP address range when you bring an
    # IP address range to Amazon Web Services (BYOIP).
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_id
    #   The ID of the IPAM that will create the token.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   Token tags.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @return [Types::CreateIpamExternalResourceVerificationTokenResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateIpamExternalResourceVerificationTokenResult#ipam_external_resource_verification_token #ipam_external_resource_verification_token} => Types::IpamExternalResourceVerificationToken
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_ipam_external_resource_verification_token({
    #     dry_run: false,
    #     ipam_id: "IpamId", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_external_resource_verification_token.ipam_external_resource_verification_token_id #=> String
    #   resp.ipam_external_resource_verification_token.ipam_external_resource_verification_token_arn #=> String
    #   resp.ipam_external_resource_verification_token.ipam_id #=> String
    #   resp.ipam_external_resource_verification_token.ipam_arn #=> String
    #   resp.ipam_external_resource_verification_token.ipam_region #=> String
    #   resp.ipam_external_resource_verification_token.token_value #=> String
    #   resp.ipam_external_resource_verification_token.token_name #=> String
    #   resp.ipam_external_resource_verification_token.not_after #=> Time
    #   resp.ipam_external_resource_verification_token.status #=> String, one of "valid", "expired"
    #   resp.ipam_external_resource_verification_token.tags #=> Array
    #   resp.ipam_external_resource_verification_token.tags[0].key #=> String
    #   resp.ipam_external_resource_verification_token.tags[0].value #=> String
    #   resp.ipam_external_resource_verification_token.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "delete-in-progress", "delete-complete", "delete-failed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateIpamExternalResourceVerificationToken AWS API Documentation
    #
    # @overload create_ipam_external_resource_verification_token(params = {})
    # @param [Hash] params ({})
    def create_ipam_external_resource_verification_token(params = {}, options = {})
      req = build_request(:create_ipam_external_resource_verification_token, params)
      req.send_request(options)
    end

    # Create an IP address pool for Amazon VPC IP Address Manager (IPAM). In
    # IPAM, a pool is a collection of contiguous IP addresses CIDRs. Pools
    # enable you to organize your IP addresses according to your routing and
    # security needs. For example, if you have separate routing and security
    # needs for development and production applications, you can create a
    # pool for each.
    #
    # For more information, see [Create a top-level pool][1] in the *Amazon
    # VPC IPAM User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/create-top-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_scope_id
    #   The ID of the scope in which you would like to create the IPAM pool.
    #
    # @option params [String] :locale
    #   The locale for the pool should be one of the following:
    #
    #   * An Amazon Web Services Region where you want this IPAM pool to be
    #     available for allocations.
    #
    #   * The network border group for an Amazon Web Services Local Zone where
    #     you want this IPAM pool to be available for allocations ([supported
    #     Local Zones][1]). This option is only available for IPAM IPv4 pools
    #     in the public scope.
    #
    #   Possible values: Any Amazon Web Services Region or supported Amazon
    #   Web Services Local Zone. Default is `none` and means any locale.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-byoip.html#byoip-zone-avail
    #
    # @option params [String] :source_ipam_pool_id
    #   The ID of the source IPAM pool. Use this option to create a pool
    #   within an existing pool. Note that the CIDR you provision for the pool
    #   within the source pool must be available in the source pool's CIDR
    #   range.
    #
    # @option params [String] :description
    #   A description for the IPAM pool.
    #
    # @option params [required, String] :address_family
    #   The IP protocol assigned to this IPAM pool. You must choose either
    #   IPv4 or IPv6 protocol for a pool.
    #
    # @option params [Boolean] :auto_import
    #   If selected, IPAM will continuously look for resources within the CIDR
    #   range of this pool and automatically import them as allocations into
    #   your IPAM. The CIDRs that will be allocated for these resources must
    #   not already be allocated to other resources in order for the import to
    #   succeed. IPAM will import a CIDR regardless of its compliance with the
    #   pool's allocation rules, so a resource might be imported and
    #   subsequently marked as noncompliant. If IPAM discovers multiple CIDRs
    #   that overlap, IPAM will import the largest CIDR only. If IPAM
    #   discovers multiple CIDRs with matching CIDRs, IPAM will randomly
    #   import one of them only.
    #
    #   A locale must be set on the pool for this feature to work.
    #
    # @option params [Boolean] :publicly_advertisable
    #   Determines if the pool is publicly advertisable. The request can only
    #   contain `PubliclyAdvertisable` if `AddressFamily` is `ipv6` and
    #   `PublicIpSource` is `byoip`.
    #
    # @option params [Integer] :allocation_min_netmask_length
    #   The minimum netmask length required for CIDR allocations in this IPAM
    #   pool to be compliant. The minimum netmask length must be less than the
    #   maximum netmask length. Possible netmask lengths for IPv4 addresses
    #   are 0 - 32. Possible netmask lengths for IPv6 addresses are 0 - 128.
    #
    # @option params [Integer] :allocation_max_netmask_length
    #   The maximum netmask length possible for CIDR allocations in this IPAM
    #   pool to be compliant. The maximum netmask length must be greater than
    #   the minimum netmask length. Possible netmask lengths for IPv4
    #   addresses are 0 - 32. Possible netmask lengths for IPv6 addresses are
    #   0 - 128.
    #
    # @option params [Integer] :allocation_default_netmask_length
    #   The default netmask length for allocations added to this pool. If, for
    #   example, the CIDR assigned to this pool is 10.0.0.0/8 and you enter 16
    #   here, new allocations will default to 10.0.0.0/16.
    #
    # @option params [Array<Types::RequestIpamResourceTag>] :allocation_resource_tags
    #   Tags that are required for resources that use CIDRs from this IPAM
    #   pool. Resources that do not have these tags will not be allowed to
    #   allocate space from the pool. If the resources have their tags changed
    #   after they have allocated space or if the allocation tagging
    #   requirements are changed on the pool, the resource may be marked as
    #   noncompliant.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The key/value combination of a tag assigned to the resource. Use the
    #   tag key in the filter name and the tag value as the filter value. For
    #   example, to find all resources that have a tag with the key `Owner`
    #   and the value `TeamA`, specify `tag:Owner` for the filter name and
    #   `TeamA` for the filter value.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [String] :aws_service
    #   Limits which service in Amazon Web Services that the pool can be used
    #   in. "ec2", for example, allows users to use space for Elastic IP
    #   addresses and VPCs.
    #
    # @option params [String] :public_ip_source
    #   The IP address source for pools in the public scope. Only used for
    #   provisioning IP address CIDRs to pools in the public scope. Default is
    #   `byoip`. For more information, see [Create IPv6 pools][1] in the
    #   *Amazon VPC IPAM User Guide*. By default, you can add only one
    #   Amazon-provided IPv6 CIDR block to a top-level IPv6 pool if
    #   PublicIpSource is `amazon`. For information on increasing the default
    #   limit, see [ Quotas for your IPAM][2] in the *Amazon VPC IPAM User
    #   Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/intro-create-ipv6-pools.html
    #   [2]: https://docs.aws.amazon.com/vpc/latest/ipam/quotas-ipam.html
    #
    # @option params [Types::IpamPoolSourceResourceRequest] :source_resource
    #   The resource used to provision CIDRs to a resource planning pool.
    #
    # @return [Types::CreateIpamPoolResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateIpamPoolResult#ipam_pool #ipam_pool} => Types::IpamPool
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_ipam_pool({
    #     dry_run: false,
    #     ipam_scope_id: "IpamScopeId", # required
    #     locale: "String",
    #     source_ipam_pool_id: "IpamPoolId",
    #     description: "String",
    #     address_family: "ipv4", # required, accepts ipv4, ipv6
    #     auto_import: false,
    #     publicly_advertisable: false,
    #     allocation_min_netmask_length: 1,
    #     allocation_max_netmask_length: 1,
    #     allocation_default_netmask_length: 1,
    #     allocation_resource_tags: [
    #       {
    #         key: "String",
    #         value: "String",
    #       },
    #     ],
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String",
    #     aws_service: "ec2", # accepts ec2
    #     public_ip_source: "amazon", # accepts amazon, byoip
    #     source_resource: {
    #       resource_id: "String",
    #       resource_type: "vpc", # accepts vpc
    #       resource_region: "String",
    #       resource_owner: "String",
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_pool.owner_id #=> String
    #   resp.ipam_pool.ipam_pool_id #=> String
    #   resp.ipam_pool.source_ipam_pool_id #=> String
    #   resp.ipam_pool.ipam_pool_arn #=> String
    #   resp.ipam_pool.ipam_scope_arn #=> String
    #   resp.ipam_pool.ipam_scope_type #=> String, one of "public", "private"
    #   resp.ipam_pool.ipam_arn #=> String
    #   resp.ipam_pool.ipam_region #=> String
    #   resp.ipam_pool.locale #=> String
    #   resp.ipam_pool.pool_depth #=> Integer
    #   resp.ipam_pool.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_pool.state_message #=> String
    #   resp.ipam_pool.description #=> String
    #   resp.ipam_pool.auto_import #=> Boolean
    #   resp.ipam_pool.publicly_advertisable #=> Boolean
    #   resp.ipam_pool.address_family #=> String, one of "ipv4", "ipv6"
    #   resp.ipam_pool.allocation_min_netmask_length #=> Integer
    #   resp.ipam_pool.allocation_max_netmask_length #=> Integer
    #   resp.ipam_pool.allocation_default_netmask_length #=> Integer
    #   resp.ipam_pool.allocation_resource_tags #=> Array
    #   resp.ipam_pool.allocation_resource_tags[0].key #=> String
    #   resp.ipam_pool.allocation_resource_tags[0].value #=> String
    #   resp.ipam_pool.tags #=> Array
    #   resp.ipam_pool.tags[0].key #=> String
    #   resp.ipam_pool.tags[0].value #=> String
    #   resp.ipam_pool.aws_service #=> String, one of "ec2"
    #   resp.ipam_pool.public_ip_source #=> String, one of "amazon", "byoip"
    #   resp.ipam_pool.source_resource.resource_id #=> String
    #   resp.ipam_pool.source_resource.resource_type #=> String, one of "vpc"
    #   resp.ipam_pool.source_resource.resource_region #=> String
    #   resp.ipam_pool.source_resource.resource_owner #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateIpamPool AWS API Documentation
    #
    # @overload create_ipam_pool(params = {})
    # @param [Hash] params ({})
    def create_ipam_pool(params = {}, options = {})
      req = build_request(:create_ipam_pool, params)
      req.send_request(options)
    end

    # Creates an IPAM resource discovery. A resource discovery is an IPAM
    # component that enables IPAM to manage and monitor resources that
    # belong to the owning account.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :description
    #   A description for the IPAM resource discovery.
    #
    # @option params [Array<Types::AddIpamOperatingRegion>] :operating_regions
    #   Operating Regions for the IPAM resource discovery. Operating Regions
    #   are Amazon Web Services Regions where the IPAM is allowed to manage IP
    #   address CIDRs. IPAM only discovers and monitors resources in the
    #   Amazon Web Services Regions you select as operating Regions.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   Tag specifications for the IPAM resource discovery.
    #
    # @option params [String] :client_token
    #   A client token for the IPAM resource discovery.
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    # @return [Types::CreateIpamResourceDiscoveryResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateIpamResourceDiscoveryResult#ipam_resource_discovery #ipam_resource_discovery} => Types::IpamResourceDiscovery
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_ipam_resource_discovery({
    #     dry_run: false,
    #     description: "String",
    #     operating_regions: [
    #       {
    #         region_name: "String",
    #       },
    #     ],
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_resource_discovery.owner_id #=> String
    #   resp.ipam_resource_discovery.ipam_resource_discovery_id #=> String
    #   resp.ipam_resource_discovery.ipam_resource_discovery_arn #=> String
    #   resp.ipam_resource_discovery.ipam_resource_discovery_region #=> String
    #   resp.ipam_resource_discovery.description #=> String
    #   resp.ipam_resource_discovery.operating_regions #=> Array
    #   resp.ipam_resource_discovery.operating_regions[0].region_name #=> String
    #   resp.ipam_resource_discovery.is_default #=> Boolean
    #   resp.ipam_resource_discovery.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_resource_discovery.tags #=> Array
    #   resp.ipam_resource_discovery.tags[0].key #=> String
    #   resp.ipam_resource_discovery.tags[0].value #=> String
    #   resp.ipam_resource_discovery.organizational_unit_exclusions #=> Array
    #   resp.ipam_resource_discovery.organizational_unit_exclusions[0].organizations_entity_path #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateIpamResourceDiscovery AWS API Documentation
    #
    # @overload create_ipam_resource_discovery(params = {})
    # @param [Hash] params ({})
    def create_ipam_resource_discovery(params = {}, options = {})
      req = build_request(:create_ipam_resource_discovery, params)
      req.send_request(options)
    end

    # Create an IPAM scope. In IPAM, a scope is the highest-level container
    # within IPAM. An IPAM contains two default scopes. Each scope
    # represents the IP space for a single network. The private scope is
    # intended for all private IP address space. The public scope is
    # intended for all public IP address space. Scopes enable you to reuse
    # IP addresses across multiple unconnected networks without causing IP
    # address overlap or conflict.
    #
    # For more information, see [Add a scope][1] in the *Amazon VPC IPAM
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/add-scope-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_id
    #   The ID of the IPAM for which you're creating this scope.
    #
    # @option params [String] :description
    #   A description for the scope you're creating.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The key/value combination of a tag assigned to the resource. Use the
    #   tag key in the filter name and the tag value as the filter value. For
    #   example, to find all resources that have a tag with the key `Owner`
    #   and the value `TeamA`, specify `tag:Owner` for the filter name and
    #   `TeamA` for the filter value.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @return [Types::CreateIpamScopeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateIpamScopeResult#ipam_scope #ipam_scope} => Types::IpamScope
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_ipam_scope({
    #     dry_run: false,
    #     ipam_id: "IpamId", # required
    #     description: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_scope.owner_id #=> String
    #   resp.ipam_scope.ipam_scope_id #=> String
    #   resp.ipam_scope.ipam_scope_arn #=> String
    #   resp.ipam_scope.ipam_arn #=> String
    #   resp.ipam_scope.ipam_region #=> String
    #   resp.ipam_scope.ipam_scope_type #=> String, one of "public", "private"
    #   resp.ipam_scope.is_default #=> Boolean
    #   resp.ipam_scope.description #=> String
    #   resp.ipam_scope.pool_count #=> Integer
    #   resp.ipam_scope.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_scope.tags #=> Array
    #   resp.ipam_scope.tags[0].key #=> String
    #   resp.ipam_scope.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateIpamScope AWS API Documentation
    #
    # @overload create_ipam_scope(params = {})
    # @param [Hash] params ({})
    def create_ipam_scope(params = {}, options = {})
      req = build_request(:create_ipam_scope, params)
      req.send_request(options)
    end

    # Creates an ED25519 or 2048-bit RSA key pair with the specified name
    # and in the specified format. Amazon EC2 stores the public key and
    # displays the private key for you to save to a file. The private key is
    # returned as an unencrypted PEM encoded PKCS#1 private key or an
    # unencrypted PPK formatted private key for use with PuTTY. If a key
    # with the specified name already exists, Amazon EC2 returns an error.
    #
    # The key pair returned to you is available only in the Amazon Web
    # Services Region in which you create it. If you prefer, you can create
    # your own key pair using a third-party tool and upload it to any Region
    # using ImportKeyPair.
    #
    # You can have up to 5,000 key pairs per Amazon Web Services Region.
    #
    # For more information, see [Amazon EC2 key pairs][1] in the *Amazon EC2
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-key-pairs.html
    #
    # @option params [required, String] :key_name
    #   A unique name for the key pair.
    #
    #   Constraints: Up to 255 ASCII characters
    #
    # @option params [String] :key_type
    #   The type of key pair. Note that ED25519 keys are not supported for
    #   Windows instances.
    #
    #   Default: `rsa`
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the new key pair.
    #
    # @option params [String] :key_format
    #   The format of the key pair.
    #
    #   Default: `pem`
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::KeyPair] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::KeyPair#key_pair_id #key_pair_id} => String
    #   * {Types::KeyPair#tags #tags} => Array&lt;Types::Tag&gt;
    #   * {Types::KeyPair#key_name #key_name} => String
    #   * {Types::KeyPair#key_fingerprint #key_fingerprint} => String
    #   * {Types::KeyPair#key_material #key_material} => String
    #
    #
    # @example Example: To create a key pair
    #
    #   # This example creates a key pair named my-key-pair.
    #
    #   resp = client.create_key_pair({
    #     key_name: "my-key-pair", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_key_pair({
    #     key_name: "String", # required
    #     key_type: "rsa", # accepts rsa, ed25519
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     key_format: "pem", # accepts pem, ppk
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.key_pair_id #=> String
    #   resp.tags #=> Array
    #   resp.tags[0].key #=> String
    #   resp.tags[0].value #=> String
    #   resp.key_name #=> String
    #   resp.key_fingerprint #=> String
    #   resp.key_material #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateKeyPair AWS API Documentation
    #
    # @overload create_key_pair(params = {})
    # @param [Hash] params ({})
    def create_key_pair(params = {}, options = {})
      req = build_request(:create_key_pair, params)
      req.send_request(options)
    end

    # Creates a launch template.
    #
    # A launch template contains the parameters to launch an instance. When
    # you launch an instance using RunInstances, you can specify a launch
    # template instead of providing the launch parameters in the request.
    # For more information, see [Store instance launch parameters in Amazon
    # EC2 launch templates][1] in the *Amazon EC2 User Guide*.
    #
    # To clone an existing launch template as the basis for a new launch
    # template, use the Amazon EC2 console. The API, SDKs, and CLI do not
    # support cloning a template. For more information, see [Create a launch
    # template from an existing launch template][2] in the *Amazon EC2 User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-launch-templates.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/create-launch-template.html#create-launch-template-from-existing-launch-template
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier you provide to ensure the
    #   idempotency of the request. If a client token isn't specified, a
    #   randomly generated token is used in the request to ensure idempotency.
    #
    #   For more information, see [Ensuring idempotency][1].
    #
    #   Constraint: Maximum 128 ASCII characters.
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [required, String] :launch_template_name
    #   A name for the launch template.
    #
    # @option params [String] :version_description
    #   A description for the first version of the launch template.
    #
    # @option params [required, Types::RequestLaunchTemplateData] :launch_template_data
    #   The information for the launch template.
    #
    # @option params [Types::OperatorRequest] :operator
    #   Reserved for internal use.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the launch template on creation. To tag the
    #   launch template, the resource type must be `launch-template`.
    #
    #   To specify the tags for the resources that are created when an
    #   instance is launched, you must use the `TagSpecifications` parameter
    #   in the [launch template data][1] structure.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_RequestLaunchTemplateData.html
    #
    # @return [Types::CreateLaunchTemplateResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateLaunchTemplateResult#launch_template #launch_template} => Types::LaunchTemplate
    #   * {Types::CreateLaunchTemplateResult#warning #warning} => Types::ValidationWarning
    #
    #
    # @example Example: To create a launch template
    #
    #   # This example creates a launch template that specifies the subnet in which to launch the instance, assigns a public IP
    #   # address and an IPv6 address to the instance, and creates a tag for the instance.
    #
    #   resp = client.create_launch_template({
    #     launch_template_data: {
    #       image_id: "ami-8c1be5f6", 
    #       instance_type: "t2.small", 
    #       network_interfaces: [
    #         {
    #           associate_public_ip_address: true, 
    #           device_index: 0, 
    #           ipv_6_address_count: 1, 
    #           subnet_id: "subnet-7b16de0c", 
    #         }, 
    #       ], 
    #       tag_specifications: [
    #         {
    #           resource_type: "instance", 
    #           tags: [
    #             {
    #               key: "Name", 
    #               value: "webserver", 
    #             }, 
    #           ], 
    #         }, 
    #       ], 
    #     }, 
    #     launch_template_name: "my-template", 
    #     version_description: "WebVersion1", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     launch_template: {
    #       create_time: Time.parse("2017-11-27T09:13:24.000Z"), 
    #       created_by: "arn:aws:iam::123456789012:root", 
    #       default_version_number: 1, 
    #       latest_version_number: 1, 
    #       launch_template_id: "lt-01238c059e3466abc", 
    #       launch_template_name: "my-template", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_launch_template({
    #     dry_run: false,
    #     client_token: "String",
    #     launch_template_name: "LaunchTemplateName", # required
    #     version_description: "VersionDescription",
    #     launch_template_data: { # required
    #       kernel_id: "KernelId",
    #       ebs_optimized: false,
    #       iam_instance_profile: {
    #         arn: "String",
    #         name: "String",
    #       },
    #       block_device_mappings: [
    #         {
    #           device_name: "String",
    #           virtual_name: "String",
    #           ebs: {
    #             encrypted: false,
    #             delete_on_termination: false,
    #             iops: 1,
    #             kms_key_id: "KmsKeyId",
    #             snapshot_id: "SnapshotId",
    #             volume_size: 1,
    #             volume_type: "standard", # accepts standard, io1, io2, gp2, sc1, st1, gp3
    #             throughput: 1,
    #           },
    #           no_device: "String",
    #         },
    #       ],
    #       network_interfaces: [
    #         {
    #           associate_carrier_ip_address: false,
    #           associate_public_ip_address: false,
    #           delete_on_termination: false,
    #           description: "String",
    #           device_index: 1,
    #           groups: ["SecurityGroupId"],
    #           interface_type: "String",
    #           ipv_6_address_count: 1,
    #           ipv_6_addresses: [
    #             {
    #               ipv_6_address: "String",
    #             },
    #           ],
    #           network_interface_id: "NetworkInterfaceId",
    #           private_ip_address: "String",
    #           private_ip_addresses: [
    #             {
    #               primary: false,
    #               private_ip_address: "String",
    #             },
    #           ],
    #           secondary_private_ip_address_count: 1,
    #           subnet_id: "SubnetId",
    #           network_card_index: 1,
    #           ipv_4_prefixes: [
    #             {
    #               ipv_4_prefix: "String",
    #             },
    #           ],
    #           ipv_4_prefix_count: 1,
    #           ipv_6_prefixes: [
    #             {
    #               ipv_6_prefix: "String",
    #             },
    #           ],
    #           ipv_6_prefix_count: 1,
    #           primary_ipv_6: false,
    #           ena_srd_specification: {
    #             ena_srd_enabled: false,
    #             ena_srd_udp_specification: {
    #               ena_srd_udp_enabled: false,
    #             },
    #           },
    #           connection_tracking_specification: {
    #             tcp_established_timeout: 1,
    #             udp_stream_timeout: 1,
    #             udp_timeout: 1,
    #           },
    #         },
    #       ],
    #       image_id: "ImageId",
    #       instance_type: "a1.medium", # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #       key_name: "KeyPairName",
    #       monitoring: {
    #         enabled: false,
    #       },
    #       placement: {
    #         availability_zone: "String",
    #         affinity: "String",
    #         group_name: "PlacementGroupName",
    #         host_id: "DedicatedHostId",
    #         tenancy: "default", # accepts default, dedicated, host
    #         spread_domain: "String",
    #         host_resource_group_arn: "String",
    #         partition_number: 1,
    #         group_id: "PlacementGroupId",
    #       },
    #       ram_disk_id: "RamdiskId",
    #       disable_api_termination: false,
    #       instance_initiated_shutdown_behavior: "stop", # accepts stop, terminate
    #       user_data: "SensitiveUserData",
    #       tag_specifications: [
    #         {
    #           resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #           tags: [
    #             {
    #               key: "String",
    #               value: "String",
    #             },
    #           ],
    #         },
    #       ],
    #       elastic_gpu_specifications: [
    #         {
    #           type: "String", # required
    #         },
    #       ],
    #       elastic_inference_accelerators: [
    #         {
    #           type: "String", # required
    #           count: 1,
    #         },
    #       ],
    #       security_group_ids: ["SecurityGroupId"],
    #       security_groups: ["SecurityGroupName"],
    #       instance_market_options: {
    #         market_type: "spot", # accepts spot, capacity-block
    #         spot_options: {
    #           max_price: "String",
    #           spot_instance_type: "one-time", # accepts one-time, persistent
    #           block_duration_minutes: 1,
    #           valid_until: Time.now,
    #           instance_interruption_behavior: "hibernate", # accepts hibernate, stop, terminate
    #         },
    #       },
    #       credit_specification: {
    #         cpu_credits: "String", # required
    #       },
    #       cpu_options: {
    #         core_count: 1,
    #         threads_per_core: 1,
    #         amd_sev_snp: "enabled", # accepts enabled, disabled
    #       },
    #       capacity_reservation_specification: {
    #         capacity_reservation_preference: "capacity-reservations-only", # accepts capacity-reservations-only, open, none
    #         capacity_reservation_target: {
    #           capacity_reservation_id: "CapacityReservationId",
    #           capacity_reservation_resource_group_arn: "String",
    #         },
    #       },
    #       license_specifications: [
    #         {
    #           license_configuration_arn: "String",
    #         },
    #       ],
    #       hibernation_options: {
    #         configured: false,
    #       },
    #       metadata_options: {
    #         http_tokens: "optional", # accepts optional, required
    #         http_put_response_hop_limit: 1,
    #         http_endpoint: "disabled", # accepts disabled, enabled
    #         http_protocol_ipv_6: "disabled", # accepts disabled, enabled
    #         instance_metadata_tags: "disabled", # accepts disabled, enabled
    #       },
    #       enclave_options: {
    #         enabled: false,
    #       },
    #       instance_requirements: {
    #         v_cpu_count: { # required
    #           min: 1, # required
    #           max: 1,
    #         },
    #         memory_mi_b: { # required
    #           min: 1, # required
    #           max: 1,
    #         },
    #         cpu_manufacturers: ["intel"], # accepts intel, amd, amazon-web-services, apple
    #         memory_gi_b_per_v_cpu: {
    #           min: 1.0,
    #           max: 1.0,
    #         },
    #         excluded_instance_types: ["ExcludedInstanceType"],
    #         instance_generations: ["current"], # accepts current, previous
    #         spot_max_price_percentage_over_lowest_price: 1,
    #         on_demand_max_price_percentage_over_lowest_price: 1,
    #         bare_metal: "included", # accepts included, required, excluded
    #         burstable_performance: "included", # accepts included, required, excluded
    #         require_hibernate_support: false,
    #         network_interface_count: {
    #           min: 1,
    #           max: 1,
    #         },
    #         local_storage: "included", # accepts included, required, excluded
    #         local_storage_types: ["hdd"], # accepts hdd, ssd
    #         total_local_storage_gb: {
    #           min: 1.0,
    #           max: 1.0,
    #         },
    #         baseline_ebs_bandwidth_mbps: {
    #           min: 1,
    #           max: 1,
    #         },
    #         accelerator_types: ["gpu"], # accepts gpu, fpga, inference
    #         accelerator_count: {
    #           min: 1,
    #           max: 1,
    #         },
    #         accelerator_manufacturers: ["amazon-web-services"], # accepts amazon-web-services, amd, nvidia, xilinx, habana
    #         accelerator_names: ["a100"], # accepts a100, inferentia, k520, k80, m60, radeon-pro-v520, t4, vu9p, v100, a10g, h100, t4g
    #         accelerator_total_memory_mi_b: {
    #           min: 1,
    #           max: 1,
    #         },
    #         network_bandwidth_gbps: {
    #           min: 1.0,
    #           max: 1.0,
    #         },
    #         allowed_instance_types: ["AllowedInstanceType"],
    #         max_spot_price_as_percentage_of_optimal_on_demand_price: 1,
    #         baseline_performance_factors: {
    #           cpu: {
    #             references: [
    #               {
    #                 instance_family: "String",
    #               },
    #             ],
    #           },
    #         },
    #       },
    #       private_dns_name_options: {
    #         hostname_type: "ip-name", # accepts ip-name, resource-name
    #         enable_resource_name_dns_a_record: false,
    #         enable_resource_name_dns_aaaa_record: false,
    #       },
    #       maintenance_options: {
    #         auto_recovery: "default", # accepts default, disabled
    #       },
    #       disable_api_stop: false,
    #       operator: {
    #         principal: "String",
    #       },
    #       network_performance_options: {
    #         bandwidth_weighting: "default", # accepts default, vpc-1, ebs-1
    #       },
    #     },
    #     operator: {
    #       principal: "String",
    #     },
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.launch_template.launch_template_id #=> String
    #   resp.launch_template.launch_template_name #=> String
    #   resp.launch_template.create_time #=> Time
    #   resp.launch_template.created_by #=> String
    #   resp.launch_template.default_version_number #=> Integer
    #   resp.launch_template.latest_version_number #=> Integer
    #   resp.launch_template.tags #=> Array
    #   resp.launch_template.tags[0].key #=> String
    #   resp.launch_template.tags[0].value #=> String
    #   resp.launch_template.operator.managed #=> Boolean
    #   resp.launch_template.operator.principal #=> String
    #   resp.warning.errors #=> Array
    #   resp.warning.errors[0].code #=> String
    #   resp.warning.errors[0].message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateLaunchTemplate AWS API Documentation
    #
    # @overload create_launch_template(params = {})
    # @param [Hash] params ({})
    def create_launch_template(params = {}, options = {})
      req = build_request(:create_launch_template, params)
      req.send_request(options)
    end

    # Creates a new version of a launch template. You must specify an
    # existing launch template, either by name or ID. You can determine
    # whether the new version inherits parameters from a source version, and
    # add or overwrite parameters as needed.
    #
    # Launch template versions are numbered in the order in which they are
    # created. You can't specify, change, or replace the numbering of
    # launch template versions.
    #
    # Launch templates are immutable; after you create a launch template,
    # you can't modify it. Instead, you can create a new version of the
    # launch template that includes the changes that you require.
    #
    # For more information, see [Modify a launch template (manage launch
    # template versions)][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/manage-launch-template-versions.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier you provide to ensure the
    #   idempotency of the request. If a client token isn't specified, a
    #   randomly generated token is used in the request to ensure idempotency.
    #
    #   For more information, see [Ensuring idempotency][1].
    #
    #   Constraint: Maximum 128 ASCII characters.
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [String] :launch_template_id
    #   The ID of the launch template.
    #
    #   You must specify either the launch template ID or the launch template
    #   name, but not both.
    #
    # @option params [String] :launch_template_name
    #   The name of the launch template.
    #
    #   You must specify either the launch template ID or the launch template
    #   name, but not both.
    #
    # @option params [String] :source_version
    #   The version of the launch template on which to base the new version.
    #   Snapshots applied to the block device mapping are ignored when
    #   creating a new version unless they are explicitly included.
    #
    #   If you specify this parameter, the new version inherits the launch
    #   parameters from the source version. If you specify additional launch
    #   parameters for the new version, they overwrite any corresponding
    #   launch parameters inherited from the source version.
    #
    #   If you omit this parameter, the new version contains only the launch
    #   parameters that you specify for the new version.
    #
    # @option params [String] :version_description
    #   A description for the version of the launch template.
    #
    # @option params [required, Types::RequestLaunchTemplateData] :launch_template_data
    #   The information for the launch template.
    #
    # @option params [Boolean] :resolve_alias
    #   If `true`, and if a Systems Manager parameter is specified for
    #   `ImageId`, the AMI ID is displayed in the response for `imageID`. For
    #   more information, see [Use a Systems Manager parameter instead of an
    #   AMI ID][1] in the *Amazon EC2 User Guide*.
    #
    #   Default: `false`
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/create-launch-template.html#use-an-ssm-parameter-instead-of-an-ami-id
    #
    # @return [Types::CreateLaunchTemplateVersionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateLaunchTemplateVersionResult#launch_template_version #launch_template_version} => Types::LaunchTemplateVersion
    #   * {Types::CreateLaunchTemplateVersionResult#warning #warning} => Types::ValidationWarning
    #
    #
    # @example Example: To create a launch template version
    #
    #   # This example creates a new launch template version based on version 1 of the specified launch template and specifies a
    #   # different AMI ID.
    #
    #   resp = client.create_launch_template_version({
    #     launch_template_data: {
    #       image_id: "ami-c998b6b2", 
    #     }, 
    #     launch_template_id: "lt-0abcd290751193123", 
    #     source_version: "1", 
    #     version_description: "WebVersion2", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     launch_template_version: {
    #       create_time: Time.parse("2017-12-01T13:35:46.000Z"), 
    #       created_by: "arn:aws:iam::123456789012:root", 
    #       default_version: false, 
    #       launch_template_data: {
    #         image_id: "ami-c998b6b2", 
    #         instance_type: "t2.micro", 
    #         network_interfaces: [
    #           {
    #             associate_public_ip_address: true, 
    #             device_index: 0, 
    #             ipv_6_addresses: [
    #               {
    #                 ipv_6_address: "2001:db8:1234:1a00::123", 
    #               }, 
    #             ], 
    #             subnet_id: "subnet-7b16de0c", 
    #           }, 
    #         ], 
    #       }, 
    #       launch_template_id: "lt-0abcd290751193123", 
    #       launch_template_name: "my-template", 
    #       version_description: "WebVersion2", 
    #       version_number: 2, 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_launch_template_version({
    #     dry_run: false,
    #     client_token: "String",
    #     launch_template_id: "LaunchTemplateId",
    #     launch_template_name: "LaunchTemplateName",
    #     source_version: "String",
    #     version_description: "VersionDescription",
    #     launch_template_data: { # required
    #       kernel_id: "KernelId",
    #       ebs_optimized: false,
    #       iam_instance_profile: {
    #         arn: "String",
    #         name: "String",
    #       },
    #       block_device_mappings: [
    #         {
    #           device_name: "String",
    #           virtual_name: "String",
    #           ebs: {
    #             encrypted: false,
    #             delete_on_termination: false,
    #             iops: 1,
    #             kms_key_id: "KmsKeyId",
    #             snapshot_id: "SnapshotId",
    #             volume_size: 1,
    #             volume_type: "standard", # accepts standard, io1, io2, gp2, sc1, st1, gp3
    #             throughput: 1,
    #           },
    #           no_device: "String",
    #         },
    #       ],
    #       network_interfaces: [
    #         {
    #           associate_carrier_ip_address: false,
    #           associate_public_ip_address: false,
    #           delete_on_termination: false,
    #           description: "String",
    #           device_index: 1,
    #           groups: ["SecurityGroupId"],
    #           interface_type: "String",
    #           ipv_6_address_count: 1,
    #           ipv_6_addresses: [
    #             {
    #               ipv_6_address: "String",
    #             },
    #           ],
    #           network_interface_id: "NetworkInterfaceId",
    #           private_ip_address: "String",
    #           private_ip_addresses: [
    #             {
    #               primary: false,
    #               private_ip_address: "String",
    #             },
    #           ],
    #           secondary_private_ip_address_count: 1,
    #           subnet_id: "SubnetId",
    #           network_card_index: 1,
    #           ipv_4_prefixes: [
    #             {
    #               ipv_4_prefix: "String",
    #             },
    #           ],
    #           ipv_4_prefix_count: 1,
    #           ipv_6_prefixes: [
    #             {
    #               ipv_6_prefix: "String",
    #             },
    #           ],
    #           ipv_6_prefix_count: 1,
    #           primary_ipv_6: false,
    #           ena_srd_specification: {
    #             ena_srd_enabled: false,
    #             ena_srd_udp_specification: {
    #               ena_srd_udp_enabled: false,
    #             },
    #           },
    #           connection_tracking_specification: {
    #             tcp_established_timeout: 1,
    #             udp_stream_timeout: 1,
    #             udp_timeout: 1,
    #           },
    #         },
    #       ],
    #       image_id: "ImageId",
    #       instance_type: "a1.medium", # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #       key_name: "KeyPairName",
    #       monitoring: {
    #         enabled: false,
    #       },
    #       placement: {
    #         availability_zone: "String",
    #         affinity: "String",
    #         group_name: "PlacementGroupName",
    #         host_id: "DedicatedHostId",
    #         tenancy: "default", # accepts default, dedicated, host
    #         spread_domain: "String",
    #         host_resource_group_arn: "String",
    #         partition_number: 1,
    #         group_id: "PlacementGroupId",
    #       },
    #       ram_disk_id: "RamdiskId",
    #       disable_api_termination: false,
    #       instance_initiated_shutdown_behavior: "stop", # accepts stop, terminate
    #       user_data: "SensitiveUserData",
    #       tag_specifications: [
    #         {
    #           resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #           tags: [
    #             {
    #               key: "String",
    #               value: "String",
    #             },
    #           ],
    #         },
    #       ],
    #       elastic_gpu_specifications: [
    #         {
    #           type: "String", # required
    #         },
    #       ],
    #       elastic_inference_accelerators: [
    #         {
    #           type: "String", # required
    #           count: 1,
    #         },
    #       ],
    #       security_group_ids: ["SecurityGroupId"],
    #       security_groups: ["SecurityGroupName"],
    #       instance_market_options: {
    #         market_type: "spot", # accepts spot, capacity-block
    #         spot_options: {
    #           max_price: "String",
    #           spot_instance_type: "one-time", # accepts one-time, persistent
    #           block_duration_minutes: 1,
    #           valid_until: Time.now,
    #           instance_interruption_behavior: "hibernate", # accepts hibernate, stop, terminate
    #         },
    #       },
    #       credit_specification: {
    #         cpu_credits: "String", # required
    #       },
    #       cpu_options: {
    #         core_count: 1,
    #         threads_per_core: 1,
    #         amd_sev_snp: "enabled", # accepts enabled, disabled
    #       },
    #       capacity_reservation_specification: {
    #         capacity_reservation_preference: "capacity-reservations-only", # accepts capacity-reservations-only, open, none
    #         capacity_reservation_target: {
    #           capacity_reservation_id: "CapacityReservationId",
    #           capacity_reservation_resource_group_arn: "String",
    #         },
    #       },
    #       license_specifications: [
    #         {
    #           license_configuration_arn: "String",
    #         },
    #       ],
    #       hibernation_options: {
    #         configured: false,
    #       },
    #       metadata_options: {
    #         http_tokens: "optional", # accepts optional, required
    #         http_put_response_hop_limit: 1,
    #         http_endpoint: "disabled", # accepts disabled, enabled
    #         http_protocol_ipv_6: "disabled", # accepts disabled, enabled
    #         instance_metadata_tags: "disabled", # accepts disabled, enabled
    #       },
    #       enclave_options: {
    #         enabled: false,
    #       },
    #       instance_requirements: {
    #         v_cpu_count: { # required
    #           min: 1, # required
    #           max: 1,
    #         },
    #         memory_mi_b: { # required
    #           min: 1, # required
    #           max: 1,
    #         },
    #         cpu_manufacturers: ["intel"], # accepts intel, amd, amazon-web-services, apple
    #         memory_gi_b_per_v_cpu: {
    #           min: 1.0,
    #           max: 1.0,
    #         },
    #         excluded_instance_types: ["ExcludedInstanceType"],
    #         instance_generations: ["current"], # accepts current, previous
    #         spot_max_price_percentage_over_lowest_price: 1,
    #         on_demand_max_price_percentage_over_lowest_price: 1,
    #         bare_metal: "included", # accepts included, required, excluded
    #         burstable_performance: "included", # accepts included, required, excluded
    #         require_hibernate_support: false,
    #         network_interface_count: {
    #           min: 1,
    #           max: 1,
    #         },
    #         local_storage: "included", # accepts included, required, excluded
    #         local_storage_types: ["hdd"], # accepts hdd, ssd
    #         total_local_storage_gb: {
    #           min: 1.0,
    #           max: 1.0,
    #         },
    #         baseline_ebs_bandwidth_mbps: {
    #           min: 1,
    #           max: 1,
    #         },
    #         accelerator_types: ["gpu"], # accepts gpu, fpga, inference
    #         accelerator_count: {
    #           min: 1,
    #           max: 1,
    #         },
    #         accelerator_manufacturers: ["amazon-web-services"], # accepts amazon-web-services, amd, nvidia, xilinx, habana
    #         accelerator_names: ["a100"], # accepts a100, inferentia, k520, k80, m60, radeon-pro-v520, t4, vu9p, v100, a10g, h100, t4g
    #         accelerator_total_memory_mi_b: {
    #           min: 1,
    #           max: 1,
    #         },
    #         network_bandwidth_gbps: {
    #           min: 1.0,
    #           max: 1.0,
    #         },
    #         allowed_instance_types: ["AllowedInstanceType"],
    #         max_spot_price_as_percentage_of_optimal_on_demand_price: 1,
    #         baseline_performance_factors: {
    #           cpu: {
    #             references: [
    #               {
    #                 instance_family: "String",
    #               },
    #             ],
    #           },
    #         },
    #       },
    #       private_dns_name_options: {
    #         hostname_type: "ip-name", # accepts ip-name, resource-name
    #         enable_resource_name_dns_a_record: false,
    #         enable_resource_name_dns_aaaa_record: false,
    #       },
    #       maintenance_options: {
    #         auto_recovery: "default", # accepts default, disabled
    #       },
    #       disable_api_stop: false,
    #       operator: {
    #         principal: "String",
    #       },
    #       network_performance_options: {
    #         bandwidth_weighting: "default", # accepts default, vpc-1, ebs-1
    #       },
    #     },
    #     resolve_alias: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.launch_template_version.launch_template_id #=> String
    #   resp.launch_template_version.launch_template_name #=> String
    #   resp.launch_template_version.version_number #=> Integer
    #   resp.launch_template_version.version_description #=> String
    #   resp.launch_template_version.create_time #=> Time
    #   resp.launch_template_version.created_by #=> String
    #   resp.launch_template_version.default_version #=> Boolean
    #   resp.launch_template_version.launch_template_data.kernel_id #=> String
    #   resp.launch_template_version.launch_template_data.ebs_optimized #=> Boolean
    #   resp.launch_template_version.launch_template_data.iam_instance_profile.arn #=> String
    #   resp.launch_template_version.launch_template_data.iam_instance_profile.name #=> String
    #   resp.launch_template_version.launch_template_data.block_device_mappings #=> Array
    #   resp.launch_template_version.launch_template_data.block_device_mappings[0].device_name #=> String
    #   resp.launch_template_version.launch_template_data.block_device_mappings[0].virtual_name #=> String
    #   resp.launch_template_version.launch_template_data.block_device_mappings[0].ebs.encrypted #=> Boolean
    #   resp.launch_template_version.launch_template_data.block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.launch_template_version.launch_template_data.block_device_mappings[0].ebs.iops #=> Integer
    #   resp.launch_template_version.launch_template_data.block_device_mappings[0].ebs.kms_key_id #=> String
    #   resp.launch_template_version.launch_template_data.block_device_mappings[0].ebs.snapshot_id #=> String
    #   resp.launch_template_version.launch_template_data.block_device_mappings[0].ebs.volume_size #=> Integer
    #   resp.launch_template_version.launch_template_data.block_device_mappings[0].ebs.volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.launch_template_version.launch_template_data.block_device_mappings[0].ebs.throughput #=> Integer
    #   resp.launch_template_version.launch_template_data.block_device_mappings[0].no_device #=> String
    #   resp.launch_template_version.launch_template_data.network_interfaces #=> Array
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].associate_carrier_ip_address #=> Boolean
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].associate_public_ip_address #=> Boolean
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].delete_on_termination #=> Boolean
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].description #=> String
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].device_index #=> Integer
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].groups #=> Array
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].groups[0] #=> String
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].interface_type #=> String
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].ipv_6_address_count #=> Integer
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].ipv_6_addresses #=> Array
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].ipv_6_addresses[0].ipv_6_address #=> String
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].ipv_6_addresses[0].is_primary_ipv_6 #=> Boolean
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].network_interface_id #=> String
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].private_ip_address #=> String
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].private_ip_addresses #=> Array
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].private_ip_addresses[0].primary #=> Boolean
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].private_ip_addresses[0].private_ip_address #=> String
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].secondary_private_ip_address_count #=> Integer
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].subnet_id #=> String
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].network_card_index #=> Integer
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].ipv_4_prefixes #=> Array
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].ipv_4_prefixes[0].ipv_4_prefix #=> String
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].ipv_4_prefix_count #=> Integer
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].ipv_6_prefixes #=> Array
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].ipv_6_prefixes[0].ipv_6_prefix #=> String
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].ipv_6_prefix_count #=> Integer
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].primary_ipv_6 #=> Boolean
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].ena_srd_specification.ena_srd_enabled #=> Boolean
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].ena_srd_specification.ena_srd_udp_specification.ena_srd_udp_enabled #=> Boolean
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].connection_tracking_specification.tcp_established_timeout #=> Integer
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].connection_tracking_specification.udp_timeout #=> Integer
    #   resp.launch_template_version.launch_template_data.network_interfaces[0].connection_tracking_specification.udp_stream_timeout #=> Integer
    #   resp.launch_template_version.launch_template_data.image_id #=> String
    #   resp.launch_template_version.launch_template_data.instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.launch_template_version.launch_template_data.key_name #=> String
    #   resp.launch_template_version.launch_template_data.monitoring.enabled #=> Boolean
    #   resp.launch_template_version.launch_template_data.placement.availability_zone #=> String
    #   resp.launch_template_version.launch_template_data.placement.affinity #=> String
    #   resp.launch_template_version.launch_template_data.placement.group_name #=> String
    #   resp.launch_template_version.launch_template_data.placement.host_id #=> String
    #   resp.launch_template_version.launch_template_data.placement.tenancy #=> String, one of "default", "dedicated", "host"
    #   resp.launch_template_version.launch_template_data.placement.spread_domain #=> String
    #   resp.launch_template_version.launch_template_data.placement.host_resource_group_arn #=> String
    #   resp.launch_template_version.launch_template_data.placement.partition_number #=> Integer
    #   resp.launch_template_version.launch_template_data.placement.group_id #=> String
    #   resp.launch_template_version.launch_template_data.ram_disk_id #=> String
    #   resp.launch_template_version.launch_template_data.disable_api_termination #=> Boolean
    #   resp.launch_template_version.launch_template_data.instance_initiated_shutdown_behavior #=> String, one of "stop", "terminate"
    #   resp.launch_template_version.launch_template_data.user_data #=> String
    #   resp.launch_template_version.launch_template_data.tag_specifications #=> Array
    #   resp.launch_template_version.launch_template_data.tag_specifications[0].resource_type #=> String, one of "capacity-reservation", "client-vpn-endpoint", "customer-gateway", "carrier-gateway", "coip-pool", "declarative-policies-report", "dedicated-host", "dhcp-options", "egress-only-internet-gateway", "elastic-ip", "elastic-gpu", "export-image-task", "export-instance-task", "fleet", "fpga-image", "host-reservation", "image", "import-image-task", "import-snapshot-task", "instance", "instance-event-window", "internet-gateway", "ipam", "ipam-pool", "ipam-scope", "ipv4pool-ec2", "ipv6pool-ec2", "key-pair", "launch-template", "local-gateway", "local-gateway-route-table", "local-gateway-virtual-interface", "local-gateway-virtual-interface-group", "local-gateway-route-table-vpc-association", "local-gateway-route-table-virtual-interface-group-association", "natgateway", "network-acl", "network-interface", "network-insights-analysis", "network-insights-path", "network-insights-access-scope", "network-insights-access-scope-analysis", "placement-group", "prefix-list", "replace-root-volume-task", "reserved-instances", "route-table", "security-group", "security-group-rule", "snapshot", "spot-fleet-request", "spot-instances-request", "subnet", "subnet-cidr-reservation", "traffic-mirror-filter", "traffic-mirror-session", "traffic-mirror-target", "transit-gateway", "transit-gateway-attachment", "transit-gateway-connect-peer", "transit-gateway-multicast-domain", "transit-gateway-policy-table", "transit-gateway-route-table", "transit-gateway-route-table-announcement", "volume", "vpc", "vpc-endpoint", "vpc-endpoint-connection", "vpc-endpoint-service", "vpc-endpoint-service-permission", "vpc-peering-connection", "vpn-connection", "vpn-gateway", "vpc-flow-log", "capacity-reservation-fleet", "traffic-mirror-filter-rule", "vpc-endpoint-connection-device-type", "verified-access-instance", "verified-access-group", "verified-access-endpoint", "verified-access-policy", "verified-access-trust-provider", "vpn-connection-device-type", "vpc-block-public-access-exclusion", "ipam-resource-discovery", "ipam-resource-discovery-association", "instance-connect-endpoint", "verified-access-endpoint-target", "ipam-external-resource-verification-token"
    #   resp.launch_template_version.launch_template_data.tag_specifications[0].tags #=> Array
    #   resp.launch_template_version.launch_template_data.tag_specifications[0].tags[0].key #=> String
    #   resp.launch_template_version.launch_template_data.tag_specifications[0].tags[0].value #=> String
    #   resp.launch_template_version.launch_template_data.elastic_gpu_specifications #=> Array
    #   resp.launch_template_version.launch_template_data.elastic_gpu_specifications[0].type #=> String
    #   resp.launch_template_version.launch_template_data.elastic_inference_accelerators #=> Array
    #   resp.launch_template_version.launch_template_data.elastic_inference_accelerators[0].type #=> String
    #   resp.launch_template_version.launch_template_data.elastic_inference_accelerators[0].count #=> Integer
    #   resp.launch_template_version.launch_template_data.security_group_ids #=> Array
    #   resp.launch_template_version.launch_template_data.security_group_ids[0] #=> String
    #   resp.launch_template_version.launch_template_data.security_groups #=> Array
    #   resp.launch_template_version.launch_template_data.security_groups[0] #=> String
    #   resp.launch_template_version.launch_template_data.instance_market_options.market_type #=> String, one of "spot", "capacity-block"
    #   resp.launch_template_version.launch_template_data.instance_market_options.spot_options.max_price #=> String
    #   resp.launch_template_version.launch_template_data.instance_market_options.spot_options.spot_instance_type #=> String, one of "one-time", "persistent"
    #   resp.launch_template_version.launch_template_data.instance_market_options.spot_options.block_duration_minutes #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_market_options.spot_options.valid_until #=> Time
    #   resp.launch_template_version.launch_template_data.instance_market_options.spot_options.instance_interruption_behavior #=> String, one of "hibernate", "stop", "terminate"
    #   resp.launch_template_version.launch_template_data.credit_specification.cpu_credits #=> String
    #   resp.launch_template_version.launch_template_data.cpu_options.core_count #=> Integer
    #   resp.launch_template_version.launch_template_data.cpu_options.threads_per_core #=> Integer
    #   resp.launch_template_version.launch_template_data.cpu_options.amd_sev_snp #=> String, one of "enabled", "disabled"
    #   resp.launch_template_version.launch_template_data.capacity_reservation_specification.capacity_reservation_preference #=> String, one of "capacity-reservations-only", "open", "none"
    #   resp.launch_template_version.launch_template_data.capacity_reservation_specification.capacity_reservation_target.capacity_reservation_id #=> String
    #   resp.launch_template_version.launch_template_data.capacity_reservation_specification.capacity_reservation_target.capacity_reservation_resource_group_arn #=> String
    #   resp.launch_template_version.launch_template_data.license_specifications #=> Array
    #   resp.launch_template_version.launch_template_data.license_specifications[0].license_configuration_arn #=> String
    #   resp.launch_template_version.launch_template_data.hibernation_options.configured #=> Boolean
    #   resp.launch_template_version.launch_template_data.metadata_options.state #=> String, one of "pending", "applied"
    #   resp.launch_template_version.launch_template_data.metadata_options.http_tokens #=> String, one of "optional", "required"
    #   resp.launch_template_version.launch_template_data.metadata_options.http_put_response_hop_limit #=> Integer
    #   resp.launch_template_version.launch_template_data.metadata_options.http_endpoint #=> String, one of "disabled", "enabled"
    #   resp.launch_template_version.launch_template_data.metadata_options.http_protocol_ipv_6 #=> String, one of "disabled", "enabled"
    #   resp.launch_template_version.launch_template_data.metadata_options.instance_metadata_tags #=> String, one of "disabled", "enabled"
    #   resp.launch_template_version.launch_template_data.enclave_options.enabled #=> Boolean
    #   resp.launch_template_version.launch_template_data.instance_requirements.v_cpu_count.min #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.v_cpu_count.max #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.memory_mi_b.min #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.memory_mi_b.max #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.cpu_manufacturers #=> Array
    #   resp.launch_template_version.launch_template_data.instance_requirements.cpu_manufacturers[0] #=> String, one of "intel", "amd", "amazon-web-services", "apple"
    #   resp.launch_template_version.launch_template_data.instance_requirements.memory_gi_b_per_v_cpu.min #=> Float
    #   resp.launch_template_version.launch_template_data.instance_requirements.memory_gi_b_per_v_cpu.max #=> Float
    #   resp.launch_template_version.launch_template_data.instance_requirements.excluded_instance_types #=> Array
    #   resp.launch_template_version.launch_template_data.instance_requirements.excluded_instance_types[0] #=> String
    #   resp.launch_template_version.launch_template_data.instance_requirements.instance_generations #=> Array
    #   resp.launch_template_version.launch_template_data.instance_requirements.instance_generations[0] #=> String, one of "current", "previous"
    #   resp.launch_template_version.launch_template_data.instance_requirements.spot_max_price_percentage_over_lowest_price #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.on_demand_max_price_percentage_over_lowest_price #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.bare_metal #=> String, one of "included", "required", "excluded"
    #   resp.launch_template_version.launch_template_data.instance_requirements.burstable_performance #=> String, one of "included", "required", "excluded"
    #   resp.launch_template_version.launch_template_data.instance_requirements.require_hibernate_support #=> Boolean
    #   resp.launch_template_version.launch_template_data.instance_requirements.network_interface_count.min #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.network_interface_count.max #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.local_storage #=> String, one of "included", "required", "excluded"
    #   resp.launch_template_version.launch_template_data.instance_requirements.local_storage_types #=> Array
    #   resp.launch_template_version.launch_template_data.instance_requirements.local_storage_types[0] #=> String, one of "hdd", "ssd"
    #   resp.launch_template_version.launch_template_data.instance_requirements.total_local_storage_gb.min #=> Float
    #   resp.launch_template_version.launch_template_data.instance_requirements.total_local_storage_gb.max #=> Float
    #   resp.launch_template_version.launch_template_data.instance_requirements.baseline_ebs_bandwidth_mbps.min #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.baseline_ebs_bandwidth_mbps.max #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.accelerator_types #=> Array
    #   resp.launch_template_version.launch_template_data.instance_requirements.accelerator_types[0] #=> String, one of "gpu", "fpga", "inference"
    #   resp.launch_template_version.launch_template_data.instance_requirements.accelerator_count.min #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.accelerator_count.max #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.accelerator_manufacturers #=> Array
    #   resp.launch_template_version.launch_template_data.instance_requirements.accelerator_manufacturers[0] #=> String, one of "amazon-web-services", "amd", "nvidia", "xilinx", "habana"
    #   resp.launch_template_version.launch_template_data.instance_requirements.accelerator_names #=> Array
    #   resp.launch_template_version.launch_template_data.instance_requirements.accelerator_names[0] #=> String, one of "a100", "inferentia", "k520", "k80", "m60", "radeon-pro-v520", "t4", "vu9p", "v100", "a10g", "h100", "t4g"
    #   resp.launch_template_version.launch_template_data.instance_requirements.accelerator_total_memory_mi_b.min #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.accelerator_total_memory_mi_b.max #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.network_bandwidth_gbps.min #=> Float
    #   resp.launch_template_version.launch_template_data.instance_requirements.network_bandwidth_gbps.max #=> Float
    #   resp.launch_template_version.launch_template_data.instance_requirements.allowed_instance_types #=> Array
    #   resp.launch_template_version.launch_template_data.instance_requirements.allowed_instance_types[0] #=> String
    #   resp.launch_template_version.launch_template_data.instance_requirements.max_spot_price_as_percentage_of_optimal_on_demand_price #=> Integer
    #   resp.launch_template_version.launch_template_data.instance_requirements.baseline_performance_factors.cpu.references #=> Array
    #   resp.launch_template_version.launch_template_data.instance_requirements.baseline_performance_factors.cpu.references[0].instance_family #=> String
    #   resp.launch_template_version.launch_template_data.private_dns_name_options.hostname_type #=> String, one of "ip-name", "resource-name"
    #   resp.launch_template_version.launch_template_data.private_dns_name_options.enable_resource_name_dns_a_record #=> Boolean
    #   resp.launch_template_version.launch_template_data.private_dns_name_options.enable_resource_name_dns_aaaa_record #=> Boolean
    #   resp.launch_template_version.launch_template_data.maintenance_options.auto_recovery #=> String, one of "default", "disabled"
    #   resp.launch_template_version.launch_template_data.disable_api_stop #=> Boolean
    #   resp.launch_template_version.launch_template_data.operator.managed #=> Boolean
    #   resp.launch_template_version.launch_template_data.operator.principal #=> String
    #   resp.launch_template_version.launch_template_data.network_performance_options.bandwidth_weighting #=> String, one of "default", "vpc-1", "ebs-1"
    #   resp.launch_template_version.operator.managed #=> Boolean
    #   resp.launch_template_version.operator.principal #=> String
    #   resp.warning.errors #=> Array
    #   resp.warning.errors[0].code #=> String
    #   resp.warning.errors[0].message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateLaunchTemplateVersion AWS API Documentation
    #
    # @overload create_launch_template_version(params = {})
    # @param [Hash] params ({})
    def create_launch_template_version(params = {}, options = {})
      req = build_request(:create_launch_template_version, params)
      req.send_request(options)
    end

    # Creates a static route for the specified local gateway route table.
    # You must specify one of the following targets:
    #
    # * `LocalGatewayVirtualInterfaceGroupId`
    #
    # * `NetworkInterfaceId`
    #
    # @option params [String] :destination_cidr_block
    #   The CIDR range used for destination matches. Routing decisions are
    #   based on the most specific match.
    #
    # @option params [required, String] :local_gateway_route_table_id
    #   The ID of the local gateway route table.
    #
    # @option params [String] :local_gateway_virtual_interface_group_id
    #   The ID of the virtual interface group.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :network_interface_id
    #   The ID of the network interface.
    #
    # @option params [String] :destination_prefix_list_id
    #   The ID of the prefix list. Use a prefix list in place of
    #   `DestinationCidrBlock`. You cannot use `DestinationPrefixListId` and
    #   `DestinationCidrBlock` in the same request.
    #
    # @return [Types::CreateLocalGatewayRouteResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateLocalGatewayRouteResult#route #route} => Types::LocalGatewayRoute
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_local_gateway_route({
    #     destination_cidr_block: "String",
    #     local_gateway_route_table_id: "LocalGatewayRoutetableId", # required
    #     local_gateway_virtual_interface_group_id: "LocalGatewayVirtualInterfaceGroupId",
    #     dry_run: false,
    #     network_interface_id: "NetworkInterfaceId",
    #     destination_prefix_list_id: "PrefixListResourceId",
    #   })
    #
    # @example Response structure
    #
    #   resp.route.destination_cidr_block #=> String
    #   resp.route.local_gateway_virtual_interface_group_id #=> String
    #   resp.route.type #=> String, one of "static", "propagated"
    #   resp.route.state #=> String, one of "pending", "active", "blackhole", "deleting", "deleted"
    #   resp.route.local_gateway_route_table_id #=> String
    #   resp.route.local_gateway_route_table_arn #=> String
    #   resp.route.owner_id #=> String
    #   resp.route.subnet_id #=> String
    #   resp.route.coip_pool_id #=> String
    #   resp.route.network_interface_id #=> String
    #   resp.route.destination_prefix_list_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateLocalGatewayRoute AWS API Documentation
    #
    # @overload create_local_gateway_route(params = {})
    # @param [Hash] params ({})
    def create_local_gateway_route(params = {}, options = {})
      req = build_request(:create_local_gateway_route, params)
      req.send_request(options)
    end

    # Creates a local gateway route table.
    #
    # @option params [required, String] :local_gateway_id
    #   The ID of the local gateway.
    #
    # @option params [String] :mode
    #   The mode of the local gateway route table.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags assigned to the local gateway route table.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateLocalGatewayRouteTableResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateLocalGatewayRouteTableResult#local_gateway_route_table #local_gateway_route_table} => Types::LocalGatewayRouteTable
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_local_gateway_route_table({
    #     local_gateway_id: "LocalGatewayId", # required
    #     mode: "direct-vpc-routing", # accepts direct-vpc-routing, coip
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.local_gateway_route_table.local_gateway_route_table_id #=> String
    #   resp.local_gateway_route_table.local_gateway_route_table_arn #=> String
    #   resp.local_gateway_route_table.local_gateway_id #=> String
    #   resp.local_gateway_route_table.outpost_arn #=> String
    #   resp.local_gateway_route_table.owner_id #=> String
    #   resp.local_gateway_route_table.state #=> String
    #   resp.local_gateway_route_table.tags #=> Array
    #   resp.local_gateway_route_table.tags[0].key #=> String
    #   resp.local_gateway_route_table.tags[0].value #=> String
    #   resp.local_gateway_route_table.mode #=> String, one of "direct-vpc-routing", "coip"
    #   resp.local_gateway_route_table.state_reason.code #=> String
    #   resp.local_gateway_route_table.state_reason.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateLocalGatewayRouteTable AWS API Documentation
    #
    # @overload create_local_gateway_route_table(params = {})
    # @param [Hash] params ({})
    def create_local_gateway_route_table(params = {}, options = {})
      req = build_request(:create_local_gateway_route_table, params)
      req.send_request(options)
    end

    # Creates a local gateway route table virtual interface group
    # association.
    #
    # @option params [required, String] :local_gateway_route_table_id
    #   The ID of the local gateway route table.
    #
    # @option params [required, String] :local_gateway_virtual_interface_group_id
    #   The ID of the local gateway route table virtual interface group
    #   association.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags assigned to the local gateway route table virtual interface
    #   group association.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateLocalGatewayRouteTableVirtualInterfaceGroupAssociationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateLocalGatewayRouteTableVirtualInterfaceGroupAssociationResult#local_gateway_route_table_virtual_interface_group_association #local_gateway_route_table_virtual_interface_group_association} => Types::LocalGatewayRouteTableVirtualInterfaceGroupAssociation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_local_gateway_route_table_virtual_interface_group_association({
    #     local_gateway_route_table_id: "LocalGatewayRoutetableId", # required
    #     local_gateway_virtual_interface_group_id: "LocalGatewayVirtualInterfaceGroupId", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.local_gateway_route_table_virtual_interface_group_association.local_gateway_route_table_virtual_interface_group_association_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.local_gateway_virtual_interface_group_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.local_gateway_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.local_gateway_route_table_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.local_gateway_route_table_arn #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.owner_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.state #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.tags #=> Array
    #   resp.local_gateway_route_table_virtual_interface_group_association.tags[0].key #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateLocalGatewayRouteTableVirtualInterfaceGroupAssociation AWS API Documentation
    #
    # @overload create_local_gateway_route_table_virtual_interface_group_association(params = {})
    # @param [Hash] params ({})
    def create_local_gateway_route_table_virtual_interface_group_association(params = {}, options = {})
      req = build_request(:create_local_gateway_route_table_virtual_interface_group_association, params)
      req.send_request(options)
    end

    # Associates the specified VPC with the specified local gateway route
    # table.
    #
    # @option params [required, String] :local_gateway_route_table_id
    #   The ID of the local gateway route table.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the local gateway route table VPC association.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateLocalGatewayRouteTableVpcAssociationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateLocalGatewayRouteTableVpcAssociationResult#local_gateway_route_table_vpc_association #local_gateway_route_table_vpc_association} => Types::LocalGatewayRouteTableVpcAssociation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_local_gateway_route_table_vpc_association({
    #     local_gateway_route_table_id: "LocalGatewayRoutetableId", # required
    #     vpc_id: "VpcId", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.local_gateway_route_table_vpc_association.local_gateway_route_table_vpc_association_id #=> String
    #   resp.local_gateway_route_table_vpc_association.local_gateway_route_table_id #=> String
    #   resp.local_gateway_route_table_vpc_association.local_gateway_route_table_arn #=> String
    #   resp.local_gateway_route_table_vpc_association.local_gateway_id #=> String
    #   resp.local_gateway_route_table_vpc_association.vpc_id #=> String
    #   resp.local_gateway_route_table_vpc_association.owner_id #=> String
    #   resp.local_gateway_route_table_vpc_association.state #=> String
    #   resp.local_gateway_route_table_vpc_association.tags #=> Array
    #   resp.local_gateway_route_table_vpc_association.tags[0].key #=> String
    #   resp.local_gateway_route_table_vpc_association.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateLocalGatewayRouteTableVpcAssociation AWS API Documentation
    #
    # @overload create_local_gateway_route_table_vpc_association(params = {})
    # @param [Hash] params ({})
    def create_local_gateway_route_table_vpc_association(params = {}, options = {})
      req = build_request(:create_local_gateway_route_table_vpc_association, params)
      req.send_request(options)
    end

    # Creates a managed prefix list. You can specify entries for the prefix
    # list. Each entry consists of a CIDR block and an optional description.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :prefix_list_name
    #   A name for the prefix list.
    #
    #   Constraints: Up to 255 characters in length. The name cannot start
    #   with `com.amazonaws`.
    #
    # @option params [Array<Types::AddPrefixListEntry>] :entries
    #   One or more entries for the prefix list.
    #
    # @option params [required, Integer] :max_entries
    #   The maximum number of entries for the prefix list.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the prefix list during creation.
    #
    # @option params [required, String] :address_family
    #   The IP address type.
    #
    #   Valid Values: `IPv4` \| `IPv6`
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   Constraints: Up to 255 UTF-8 characters in length.
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @return [Types::CreateManagedPrefixListResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateManagedPrefixListResult#prefix_list #prefix_list} => Types::ManagedPrefixList
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_managed_prefix_list({
    #     dry_run: false,
    #     prefix_list_name: "String", # required
    #     entries: [
    #       {
    #         cidr: "String", # required
    #         description: "String",
    #       },
    #     ],
    #     max_entries: 1, # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     address_family: "String", # required
    #     client_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.prefix_list.prefix_list_id #=> String
    #   resp.prefix_list.address_family #=> String
    #   resp.prefix_list.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "restore-in-progress", "restore-complete", "restore-failed", "delete-in-progress", "delete-complete", "delete-failed"
    #   resp.prefix_list.state_message #=> String
    #   resp.prefix_list.prefix_list_arn #=> String
    #   resp.prefix_list.prefix_list_name #=> String
    #   resp.prefix_list.max_entries #=> Integer
    #   resp.prefix_list.version #=> Integer
    #   resp.prefix_list.tags #=> Array
    #   resp.prefix_list.tags[0].key #=> String
    #   resp.prefix_list.tags[0].value #=> String
    #   resp.prefix_list.owner_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateManagedPrefixList AWS API Documentation
    #
    # @overload create_managed_prefix_list(params = {})
    # @param [Hash] params ({})
    def create_managed_prefix_list(params = {}, options = {})
      req = build_request(:create_managed_prefix_list, params)
      req.send_request(options)
    end

    # Creates a NAT gateway in the specified subnet. This action creates a
    # network interface in the specified subnet with a private IP address
    # from the IP address range of the subnet. You can create either a
    # public NAT gateway or a private NAT gateway.
    #
    # With a public NAT gateway, internet-bound traffic from a private
    # subnet can be routed to the NAT gateway, so that instances in a
    # private subnet can connect to the internet.
    #
    # With a private NAT gateway, private communication is routed across
    # VPCs and on-premises networks through a transit gateway or virtual
    # private gateway. Common use cases include running large workloads
    # behind a small pool of allowlisted IPv4 addresses, preserving private
    # IPv4 addresses, and communicating between overlapping networks.
    #
    # For more information, see [NAT gateways][1] in the *Amazon VPC User
    # Guide*.
    #
    # When you create a public NAT gateway and assign it an EIP or secondary
    # EIPs, the network border group of the EIPs must match the network
    # border group of the Availability Zone (AZ) that the public NAT gateway
    # is in. If it's not the same, the NAT gateway will fail to launch. You
    # can see the network border group for the subnet's AZ by viewing the
    # details of the subnet. Similarly, you can view the network border
    # group of an EIP by viewing the details of the EIP address. For more
    # information about network border groups and EIPs, see [Allocate an
    # Elastic IP address][2] in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/vpc-nat-gateway.html
    # [2]: https://docs.aws.amazon.com/vpc/latest/userguide/WorkWithEIPs.html
    #
    # @option params [String] :allocation_id
    #   \[Public NAT gateways only\] The allocation ID of an Elastic IP
    #   address to associate with the NAT gateway. You cannot specify an
    #   Elastic IP address with a private NAT gateway. If the Elastic IP
    #   address is associated with another resource, you must first
    #   disassociate it.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   Constraint: Maximum 64 ASCII characters.
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :subnet_id
    #   The ID of the subnet in which to create the NAT gateway.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the NAT gateway.
    #
    # @option params [String] :connectivity_type
    #   Indicates whether the NAT gateway supports public or private
    #   connectivity. The default is public connectivity.
    #
    # @option params [String] :private_ip_address
    #   The private IPv4 address to assign to the NAT gateway. If you don't
    #   provide an address, a private IPv4 address will be automatically
    #   assigned.
    #
    # @option params [Array<String>] :secondary_allocation_ids
    #   Secondary EIP allocation IDs. For more information, see [Create a NAT
    #   gateway][1] in the *Amazon VPC User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/userguide/nat-gateway-working-with.html
    #
    # @option params [Array<String>] :secondary_private_ip_addresses
    #   Secondary private IPv4 addresses. For more information about secondary
    #   addresses, see [Create a NAT gateway][1] in the *Amazon VPC User
    #   Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/userguide/nat-gateway-working-with.html
    #
    # @option params [Integer] :secondary_private_ip_address_count
    #   \[Private NAT gateway only\] The number of secondary private IPv4
    #   addresses you want to assign to the NAT gateway. For more information
    #   about secondary addresses, see [Create a NAT gateway][1] in the
    #   *Amazon VPC User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/userguide/nat-gateway-working-with.html
    #
    # @return [Types::CreateNatGatewayResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateNatGatewayResult#client_token #client_token} => String
    #   * {Types::CreateNatGatewayResult#nat_gateway #nat_gateway} => Types::NatGateway
    #
    #
    # @example Example: To create a NAT gateway
    #
    #   # This example creates a NAT gateway in subnet subnet-1a2b3c4d and associates an Elastic IP address with the allocation ID
    #   # eipalloc-37fc1a52 with the NAT gateway.
    #
    #   resp = client.create_nat_gateway({
    #     allocation_id: "eipalloc-37fc1a52", 
    #     subnet_id: "subnet-1a2b3c4d", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     nat_gateway: {
    #       create_time: Time.parse("2015-12-17T12:45:26.732Z"), 
    #       nat_gateway_addresses: [
    #         {
    #           allocation_id: "eipalloc-37fc1a52", 
    #         }, 
    #       ], 
    #       nat_gateway_id: "nat-08d48af2a8e83edfd", 
    #       state: "pending", 
    #       subnet_id: "subnet-1a2b3c4d", 
    #       vpc_id: "vpc-1122aabb", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_nat_gateway({
    #     allocation_id: "AllocationId",
    #     client_token: "String",
    #     dry_run: false,
    #     subnet_id: "SubnetId", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     connectivity_type: "private", # accepts private, public
    #     private_ip_address: "String",
    #     secondary_allocation_ids: ["AllocationId"],
    #     secondary_private_ip_addresses: ["String"],
    #     secondary_private_ip_address_count: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.client_token #=> String
    #   resp.nat_gateway.create_time #=> Time
    #   resp.nat_gateway.delete_time #=> Time
    #   resp.nat_gateway.failure_code #=> String
    #   resp.nat_gateway.failure_message #=> String
    #   resp.nat_gateway.nat_gateway_addresses #=> Array
    #   resp.nat_gateway.nat_gateway_addresses[0].allocation_id #=> String
    #   resp.nat_gateway.nat_gateway_addresses[0].network_interface_id #=> String
    #   resp.nat_gateway.nat_gateway_addresses[0].private_ip #=> String
    #   resp.nat_gateway.nat_gateway_addresses[0].public_ip #=> String
    #   resp.nat_gateway.nat_gateway_addresses[0].association_id #=> String
    #   resp.nat_gateway.nat_gateway_addresses[0].is_primary #=> Boolean
    #   resp.nat_gateway.nat_gateway_addresses[0].failure_message #=> String
    #   resp.nat_gateway.nat_gateway_addresses[0].status #=> String, one of "assigning", "unassigning", "associating", "disassociating", "succeeded", "failed"
    #   resp.nat_gateway.nat_gateway_id #=> String
    #   resp.nat_gateway.provisioned_bandwidth.provision_time #=> Time
    #   resp.nat_gateway.provisioned_bandwidth.provisioned #=> String
    #   resp.nat_gateway.provisioned_bandwidth.request_time #=> Time
    #   resp.nat_gateway.provisioned_bandwidth.requested #=> String
    #   resp.nat_gateway.provisioned_bandwidth.status #=> String
    #   resp.nat_gateway.state #=> String, one of "pending", "failed", "available", "deleting", "deleted"
    #   resp.nat_gateway.subnet_id #=> String
    #   resp.nat_gateway.vpc_id #=> String
    #   resp.nat_gateway.tags #=> Array
    #   resp.nat_gateway.tags[0].key #=> String
    #   resp.nat_gateway.tags[0].value #=> String
    #   resp.nat_gateway.connectivity_type #=> String, one of "private", "public"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateNatGateway AWS API Documentation
    #
    # @overload create_nat_gateway(params = {})
    # @param [Hash] params ({})
    def create_nat_gateway(params = {}, options = {})
      req = build_request(:create_nat_gateway, params)
      req.send_request(options)
    end

    # Creates a network ACL in a VPC. Network ACLs provide an optional layer
    # of security (in addition to security groups) for the instances in your
    # VPC.
    #
    # For more information, see [Network ACLs][1] in the *Amazon VPC User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/vpc-network-acls.html
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the network ACL.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @return [Types::CreateNetworkAclResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateNetworkAclResult#network_acl #network_acl} => Types::NetworkAcl
    #   * {Types::CreateNetworkAclResult#client_token #client_token} => String
    #
    #
    # @example Example: To create a network ACL
    #
    #   # This example creates a network ACL for the specified VPC.
    #
    #   resp = client.create_network_acl({
    #     vpc_id: "vpc-a01106c2", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     network_acl: {
    #       associations: [
    #       ], 
    #       entries: [
    #         {
    #           cidr_block: "0.0.0.0/0", 
    #           egress: true, 
    #           protocol: "-1", 
    #           rule_action: "deny", 
    #           rule_number: 32767, 
    #         }, 
    #         {
    #           cidr_block: "0.0.0.0/0", 
    #           egress: false, 
    #           protocol: "-1", 
    #           rule_action: "deny", 
    #           rule_number: 32767, 
    #         }, 
    #       ], 
    #       is_default: false, 
    #       network_acl_id: "acl-5fb85d36", 
    #       tags: [
    #       ], 
    #       vpc_id: "vpc-a01106c2", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_network_acl({
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String",
    #     dry_run: false,
    #     vpc_id: "VpcId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.network_acl.associations #=> Array
    #   resp.network_acl.associations[0].network_acl_association_id #=> String
    #   resp.network_acl.associations[0].network_acl_id #=> String
    #   resp.network_acl.associations[0].subnet_id #=> String
    #   resp.network_acl.entries #=> Array
    #   resp.network_acl.entries[0].cidr_block #=> String
    #   resp.network_acl.entries[0].egress #=> Boolean
    #   resp.network_acl.entries[0].icmp_type_code.code #=> Integer
    #   resp.network_acl.entries[0].icmp_type_code.type #=> Integer
    #   resp.network_acl.entries[0].ipv_6_cidr_block #=> String
    #   resp.network_acl.entries[0].port_range.from #=> Integer
    #   resp.network_acl.entries[0].port_range.to #=> Integer
    #   resp.network_acl.entries[0].protocol #=> String
    #   resp.network_acl.entries[0].rule_action #=> String, one of "allow", "deny"
    #   resp.network_acl.entries[0].rule_number #=> Integer
    #   resp.network_acl.is_default #=> Boolean
    #   resp.network_acl.network_acl_id #=> String
    #   resp.network_acl.tags #=> Array
    #   resp.network_acl.tags[0].key #=> String
    #   resp.network_acl.tags[0].value #=> String
    #   resp.network_acl.vpc_id #=> String
    #   resp.network_acl.owner_id #=> String
    #   resp.client_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateNetworkAcl AWS API Documentation
    #
    # @overload create_network_acl(params = {})
    # @param [Hash] params ({})
    def create_network_acl(params = {}, options = {})
      req = build_request(:create_network_acl, params)
      req.send_request(options)
    end

    # Creates an entry (a rule) in a network ACL with the specified rule
    # number. Each network ACL has a set of numbered ingress rules and a
    # separate set of numbered egress rules. When determining whether a
    # packet should be allowed in or out of a subnet associated with the
    # ACL, we process the entries in the ACL according to the rule numbers,
    # in ascending order. Each network ACL has a set of ingress rules and a
    # separate set of egress rules.
    #
    # We recommend that you leave room between the rule numbers (for
    # example, 100, 110, 120, ...), and not number them one right after the
    # other (for example, 101, 102, 103, ...). This makes it easier to add a
    # rule between existing ones without having to renumber the rules.
    #
    # After you add an entry, you can't modify it; you must either replace
    # it, or create an entry and delete the old one.
    #
    # For more information about network ACLs, see [Network ACLs][1] in the
    # *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/vpc-network-acls.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :network_acl_id
    #   The ID of the network ACL.
    #
    # @option params [required, Integer] :rule_number
    #   The rule number for the entry (for example, 100). ACL entries are
    #   processed in ascending order by rule number.
    #
    #   Constraints: Positive integer from 1 to 32766. The range 32767 to
    #   65535 is reserved for internal use.
    #
    # @option params [required, String] :protocol
    #   The protocol number. A value of "-1" means all protocols. If you
    #   specify "-1" or a protocol number other than "6" (TCP), "17"
    #   (UDP), or "1" (ICMP), traffic on all ports is allowed, regardless of
    #   any ports or ICMP types or codes that you specify. If you specify
    #   protocol "58" (ICMPv6) and specify an IPv4 CIDR block, traffic for
    #   all ICMP types and codes allowed, regardless of any that you specify.
    #   If you specify protocol "58" (ICMPv6) and specify an IPv6 CIDR
    #   block, you must specify an ICMP type and code.
    #
    # @option params [required, String] :rule_action
    #   Indicates whether to allow or deny the traffic that matches the rule.
    #
    # @option params [required, Boolean] :egress
    #   Indicates whether this is an egress rule (rule is applied to traffic
    #   leaving the subnet).
    #
    # @option params [String] :cidr_block
    #   The IPv4 network range to allow or deny, in CIDR notation (for example
    #   `172.16.0.0/24`). We modify the specified CIDR block to its canonical
    #   form; for example, if you specify `100.68.0.18/18`, we modify it to
    #   `100.68.0.0/18`.
    #
    # @option params [String] :ipv_6_cidr_block
    #   The IPv6 network range to allow or deny, in CIDR notation (for example
    #   `2001:db8:1234:1a00::/64`).
    #
    # @option params [Types::IcmpTypeCode] :icmp_type_code
    #   ICMP protocol: The ICMP or ICMPv6 type and code. Required if
    #   specifying protocol 1 (ICMP) or protocol 58 (ICMPv6) with an IPv6 CIDR
    #   block.
    #
    # @option params [Types::PortRange] :port_range
    #   TCP or UDP protocols: The range of ports the rule applies to. Required
    #   if specifying protocol 6 (TCP) or 17 (UDP).
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To create a network ACL entry
    #
    #   # This example creates an entry for the specified network ACL. The rule allows ingress traffic from anywhere (0.0.0.0/0)
    #   # on UDP port 53 (DNS) into any associated subnet.
    #
    #   resp = client.create_network_acl_entry({
    #     cidr_block: "0.0.0.0/0", 
    #     egress: false, 
    #     network_acl_id: "acl-5fb85d36", 
    #     port_range: {
    #       from: 53, 
    #       to: 53, 
    #     }, 
    #     protocol: "17", 
    #     rule_action: "allow", 
    #     rule_number: 100, 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_network_acl_entry({
    #     dry_run: false,
    #     network_acl_id: "NetworkAclId", # required
    #     rule_number: 1, # required
    #     protocol: "String", # required
    #     rule_action: "allow", # required, accepts allow, deny
    #     egress: false, # required
    #     cidr_block: "String",
    #     ipv_6_cidr_block: "String",
    #     icmp_type_code: {
    #       code: 1,
    #       type: 1,
    #     },
    #     port_range: {
    #       from: 1,
    #       to: 1,
    #     },
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateNetworkAclEntry AWS API Documentation
    #
    # @overload create_network_acl_entry(params = {})
    # @param [Hash] params ({})
    def create_network_acl_entry(params = {}, options = {})
      req = build_request(:create_network_acl_entry, params)
      req.send_request(options)
    end

    # Creates a Network Access Scope.
    #
    # Amazon Web Services Network Access Analyzer enables cloud networking
    # and cloud operations teams to verify that their networks on Amazon Web
    # Services conform to their network security and governance objectives.
    # For more information, see the [Amazon Web Services Network Access
    # Analyzer Guide][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/network-access-analyzer/
    #
    # @option params [Array<Types::AccessScopePathRequest>] :match_paths
    #   The paths to match.
    #
    # @option params [Array<Types::AccessScopePathRequest>] :exclude_paths
    #   The paths to exclude.
    #
    # @option params [required, String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [How to ensure
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateNetworkInsightsAccessScopeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateNetworkInsightsAccessScopeResult#network_insights_access_scope #network_insights_access_scope} => Types::NetworkInsightsAccessScope
    #   * {Types::CreateNetworkInsightsAccessScopeResult#network_insights_access_scope_content #network_insights_access_scope_content} => Types::NetworkInsightsAccessScopeContent
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_network_insights_access_scope({
    #     match_paths: [
    #       {
    #         source: {
    #           packet_header_statement: {
    #             source_addresses: ["String"],
    #             destination_addresses: ["String"],
    #             source_ports: ["String"],
    #             destination_ports: ["String"],
    #             source_prefix_lists: ["String"],
    #             destination_prefix_lists: ["String"],
    #             protocols: ["tcp"], # accepts tcp, udp
    #           },
    #           resource_statement: {
    #             resources: ["String"],
    #             resource_types: ["String"],
    #           },
    #         },
    #         destination: {
    #           packet_header_statement: {
    #             source_addresses: ["String"],
    #             destination_addresses: ["String"],
    #             source_ports: ["String"],
    #             destination_ports: ["String"],
    #             source_prefix_lists: ["String"],
    #             destination_prefix_lists: ["String"],
    #             protocols: ["tcp"], # accepts tcp, udp
    #           },
    #           resource_statement: {
    #             resources: ["String"],
    #             resource_types: ["String"],
    #           },
    #         },
    #         through_resources: [
    #           {
    #             resource_statement: {
    #               resources: ["String"],
    #               resource_types: ["String"],
    #             },
    #           },
    #         ],
    #       },
    #     ],
    #     exclude_paths: [
    #       {
    #         source: {
    #           packet_header_statement: {
    #             source_addresses: ["String"],
    #             destination_addresses: ["String"],
    #             source_ports: ["String"],
    #             destination_ports: ["String"],
    #             source_prefix_lists: ["String"],
    #             destination_prefix_lists: ["String"],
    #             protocols: ["tcp"], # accepts tcp, udp
    #           },
    #           resource_statement: {
    #             resources: ["String"],
    #             resource_types: ["String"],
    #           },
    #         },
    #         destination: {
    #           packet_header_statement: {
    #             source_addresses: ["String"],
    #             destination_addresses: ["String"],
    #             source_ports: ["String"],
    #             destination_ports: ["String"],
    #             source_prefix_lists: ["String"],
    #             destination_prefix_lists: ["String"],
    #             protocols: ["tcp"], # accepts tcp, udp
    #           },
    #           resource_statement: {
    #             resources: ["String"],
    #             resource_types: ["String"],
    #           },
    #         },
    #         through_resources: [
    #           {
    #             resource_statement: {
    #               resources: ["String"],
    #               resource_types: ["String"],
    #             },
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.network_insights_access_scope.network_insights_access_scope_id #=> String
    #   resp.network_insights_access_scope.network_insights_access_scope_arn #=> String
    #   resp.network_insights_access_scope.created_date #=> Time
    #   resp.network_insights_access_scope.updated_date #=> Time
    #   resp.network_insights_access_scope.tags #=> Array
    #   resp.network_insights_access_scope.tags[0].key #=> String
    #   resp.network_insights_access_scope.tags[0].value #=> String
    #   resp.network_insights_access_scope_content.network_insights_access_scope_id #=> String
    #   resp.network_insights_access_scope_content.match_paths #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.source_addresses #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.source_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.destination_addresses #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.destination_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.source_ports #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.source_ports[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.destination_ports #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.destination_ports[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.source_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.source_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.destination_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.destination_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.protocols #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.protocols[0] #=> String, one of "tcp", "udp"
    #   resp.network_insights_access_scope_content.match_paths[0].source.resource_statement.resources #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.resource_statement.resources[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].source.resource_statement.resource_types #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.resource_statement.resource_types[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.source_addresses #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.source_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.destination_addresses #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.destination_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.source_ports #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.source_ports[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.destination_ports #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.destination_ports[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.source_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.source_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.destination_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.destination_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.protocols #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.protocols[0] #=> String, one of "tcp", "udp"
    #   resp.network_insights_access_scope_content.match_paths[0].destination.resource_statement.resources #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.resource_statement.resources[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.resource_statement.resource_types #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.resource_statement.resource_types[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].through_resources #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].through_resources[0].resource_statement.resources #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].through_resources[0].resource_statement.resources[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].through_resources[0].resource_statement.resource_types #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].through_resources[0].resource_statement.resource_types[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.source_addresses #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.source_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.destination_addresses #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.destination_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.source_ports #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.source_ports[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.destination_ports #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.destination_ports[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.source_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.source_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.destination_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.destination_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.protocols #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.protocols[0] #=> String, one of "tcp", "udp"
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.resource_statement.resources #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.resource_statement.resources[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.resource_statement.resource_types #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.resource_statement.resource_types[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.source_addresses #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.source_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.destination_addresses #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.destination_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.source_ports #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.source_ports[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.destination_ports #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.destination_ports[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.source_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.source_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.destination_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.destination_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.protocols #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.protocols[0] #=> String, one of "tcp", "udp"
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.resource_statement.resources #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.resource_statement.resources[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.resource_statement.resource_types #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.resource_statement.resource_types[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].through_resources #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].through_resources[0].resource_statement.resources #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].through_resources[0].resource_statement.resources[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].through_resources[0].resource_statement.resource_types #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].through_resources[0].resource_statement.resource_types[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateNetworkInsightsAccessScope AWS API Documentation
    #
    # @overload create_network_insights_access_scope(params = {})
    # @param [Hash] params ({})
    def create_network_insights_access_scope(params = {}, options = {})
      req = build_request(:create_network_insights_access_scope, params)
      req.send_request(options)
    end

    # Creates a path to analyze for reachability.
    #
    # Reachability Analyzer enables you to analyze and debug network
    # reachability between two resources in your virtual private cloud
    # (VPC). For more information, see the [Reachability Analyzer Guide][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/reachability/
    #
    # @option params [String] :source_ip
    #   The IP address of the source.
    #
    # @option params [String] :destination_ip
    #   The IP address of the destination.
    #
    # @option params [required, String] :source
    #   The ID or ARN of the source. If the resource is in another account,
    #   you must specify an ARN.
    #
    # @option params [String] :destination
    #   The ID or ARN of the destination. If the resource is in another
    #   account, you must specify an ARN.
    #
    # @option params [required, String] :protocol
    #   The protocol.
    #
    # @option params [Integer] :destination_port
    #   The destination port.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to add to the path.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [How to ensure
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Types::PathRequestFilter] :filter_at_source
    #   Scopes the analysis to network paths that match specific filters at
    #   the source. If you specify this parameter, you can't specify the
    #   parameters for the source IP address or the destination port.
    #
    # @option params [Types::PathRequestFilter] :filter_at_destination
    #   Scopes the analysis to network paths that match specific filters at
    #   the destination. If you specify this parameter, you can't specify the
    #   parameter for the destination IP address.
    #
    # @return [Types::CreateNetworkInsightsPathResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateNetworkInsightsPathResult#network_insights_path #network_insights_path} => Types::NetworkInsightsPath
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_network_insights_path({
    #     source_ip: "IpAddress",
    #     destination_ip: "IpAddress",
    #     source: "NetworkInsightsResourceId", # required
    #     destination: "NetworkInsightsResourceId",
    #     protocol: "tcp", # required, accepts tcp, udp
    #     destination_port: 1,
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #     client_token: "String", # required
    #     filter_at_source: {
    #       source_address: "IpAddress",
    #       source_port_range: {
    #         from_port: 1,
    #         to_port: 1,
    #       },
    #       destination_address: "IpAddress",
    #       destination_port_range: {
    #         from_port: 1,
    #         to_port: 1,
    #       },
    #     },
    #     filter_at_destination: {
    #       source_address: "IpAddress",
    #       source_port_range: {
    #         from_port: 1,
    #         to_port: 1,
    #       },
    #       destination_address: "IpAddress",
    #       destination_port_range: {
    #         from_port: 1,
    #         to_port: 1,
    #       },
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.network_insights_path.network_insights_path_id #=> String
    #   resp.network_insights_path.network_insights_path_arn #=> String
    #   resp.network_insights_path.created_date #=> Time
    #   resp.network_insights_path.source #=> String
    #   resp.network_insights_path.destination #=> String
    #   resp.network_insights_path.source_arn #=> String
    #   resp.network_insights_path.destination_arn #=> String
    #   resp.network_insights_path.source_ip #=> String
    #   resp.network_insights_path.destination_ip #=> String
    #   resp.network_insights_path.protocol #=> String, one of "tcp", "udp"
    #   resp.network_insights_path.destination_port #=> Integer
    #   resp.network_insights_path.tags #=> Array
    #   resp.network_insights_path.tags[0].key #=> String
    #   resp.network_insights_path.tags[0].value #=> String
    #   resp.network_insights_path.filter_at_source.source_address #=> String
    #   resp.network_insights_path.filter_at_source.source_port_range.from_port #=> Integer
    #   resp.network_insights_path.filter_at_source.source_port_range.to_port #=> Integer
    #   resp.network_insights_path.filter_at_source.destination_address #=> String
    #   resp.network_insights_path.filter_at_source.destination_port_range.from_port #=> Integer
    #   resp.network_insights_path.filter_at_source.destination_port_range.to_port #=> Integer
    #   resp.network_insights_path.filter_at_destination.source_address #=> String
    #   resp.network_insights_path.filter_at_destination.source_port_range.from_port #=> Integer
    #   resp.network_insights_path.filter_at_destination.source_port_range.to_port #=> Integer
    #   resp.network_insights_path.filter_at_destination.destination_address #=> String
    #   resp.network_insights_path.filter_at_destination.destination_port_range.from_port #=> Integer
    #   resp.network_insights_path.filter_at_destination.destination_port_range.to_port #=> Integer
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateNetworkInsightsPath AWS API Documentation
    #
    # @overload create_network_insights_path(params = {})
    # @param [Hash] params ({})
    def create_network_insights_path(params = {}, options = {})
      req = build_request(:create_network_insights_path, params)
      req.send_request(options)
    end

    # Creates a network interface in the specified subnet.
    #
    # The number of IP addresses you can assign to a network interface
    # varies by instance type.
    #
    # For more information about network interfaces, see [Elastic network
    # interfaces][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-eni.html
    #
    # @option params [Array<Types::Ipv4PrefixSpecificationRequest>] :ipv_4_prefixes
    #   The IPv4 prefixes assigned to the network interface.
    #
    #   You can't specify IPv4 prefixes if you've specified one of the
    #   following: a count of IPv4 prefixes, specific private IPv4 addresses,
    #   or a count of private IPv4 addresses.
    #
    # @option params [Integer] :ipv_4_prefix_count
    #   The number of IPv4 prefixes that Amazon Web Services automatically
    #   assigns to the network interface.
    #
    #   You can't specify a count of IPv4 prefixes if you've specified one
    #   of the following: specific IPv4 prefixes, specific private IPv4
    #   addresses, or a count of private IPv4 addresses.
    #
    # @option params [Array<Types::Ipv6PrefixSpecificationRequest>] :ipv_6_prefixes
    #   The IPv6 prefixes assigned to the network interface.
    #
    #   You can't specify IPv6 prefixes if you've specified one of the
    #   following: a count of IPv6 prefixes, specific IPv6 addresses, or a
    #   count of IPv6 addresses.
    #
    # @option params [Integer] :ipv_6_prefix_count
    #   The number of IPv6 prefixes that Amazon Web Services automatically
    #   assigns to the network interface.
    #
    #   You can't specify a count of IPv6 prefixes if you've specified one
    #   of the following: specific IPv6 prefixes, specific IPv6 addresses, or
    #   a count of IPv6 addresses.
    #
    # @option params [String] :interface_type
    #   The type of network interface. The default is `interface`.
    #
    #   If you specify `efa-only`, do not assign any IP addresses to the
    #   network interface. EFA-only network interfaces do not support IP
    #   addresses.
    #
    #   The only supported values are `interface`, `efa`, `efa-only`, and
    #   `trunk`.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the new network interface.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :enable_primary_ipv_6
    #   If you’re creating a network interface in a dual-stack or IPv6-only
    #   subnet, you have the option to assign a primary IPv6 IP address. A
    #   primary IPv6 address is an IPv6 GUA address associated with an ENI
    #   that you have enabled to use a primary IPv6 address. Use this option
    #   if the instance that this ENI will be attached to relies on its IPv6
    #   address not changing. Amazon Web Services will automatically assign an
    #   IPv6 address associated with the ENI attached to your instance to be
    #   the primary IPv6 address. Once you enable an IPv6 GUA address to be a
    #   primary IPv6, you cannot disable it. When you enable an IPv6 GUA
    #   address to be a primary IPv6, the first IPv6 GUA will be made the
    #   primary IPv6 address until the instance is terminated or the network
    #   interface is detached. If you have multiple IPv6 addresses associated
    #   with an ENI attached to your instance and you enable a primary IPv6
    #   address, the first IPv6 GUA address associated with the ENI becomes
    #   the primary IPv6 address.
    #
    # @option params [Types::ConnectionTrackingSpecificationRequest] :connection_tracking_specification
    #   A connection tracking specification for the network interface.
    #
    # @option params [Types::OperatorRequest] :operator
    #   Reserved for internal use.
    #
    # @option params [required, String] :subnet_id
    #   The ID of the subnet to associate with the network interface.
    #
    # @option params [String] :description
    #   A description for the network interface.
    #
    # @option params [String] :private_ip_address
    #   The primary private IPv4 address of the network interface. If you
    #   don't specify an IPv4 address, Amazon EC2 selects one for you from
    #   the subnet's IPv4 CIDR range. If you specify an IP address, you
    #   cannot indicate any IP addresses specified in `privateIpAddresses` as
    #   primary (only one IP address can be designated as primary).
    #
    # @option params [Array<String>] :groups
    #   The IDs of the security groups.
    #
    # @option params [Array<Types::PrivateIpAddressSpecification>] :private_ip_addresses
    #   The private IPv4 addresses.
    #
    #   You can't specify private IPv4 addresses if you've specified one of
    #   the following: a count of private IPv4 addresses, specific IPv4
    #   prefixes, or a count of IPv4 prefixes.
    #
    # @option params [Integer] :secondary_private_ip_address_count
    #   The number of secondary private IPv4 addresses to assign to a network
    #   interface. When you specify a number of secondary IPv4 addresses,
    #   Amazon EC2 selects these IP addresses within the subnet's IPv4 CIDR
    #   range. You can't specify this option and specify more than one
    #   private IP address using `privateIpAddresses`.
    #
    #   You can't specify a count of private IPv4 addresses if you've
    #   specified one of the following: specific private IPv4 addresses,
    #   specific IPv4 prefixes, or a count of IPv4 prefixes.
    #
    # @option params [Array<Types::InstanceIpv6Address>] :ipv_6_addresses
    #   The IPv6 addresses from the IPv6 CIDR block range of your subnet.
    #
    #   You can't specify IPv6 addresses using this parameter if you've
    #   specified one of the following: a count of IPv6 addresses, specific
    #   IPv6 prefixes, or a count of IPv6 prefixes.
    #
    # @option params [Integer] :ipv_6_address_count
    #   The number of IPv6 addresses to assign to a network interface. Amazon
    #   EC2 automatically selects the IPv6 addresses from the subnet range.
    #
    #   You can't specify a count of IPv6 addresses using this parameter if
    #   you've specified one of the following: specific IPv6 addresses,
    #   specific IPv6 prefixes, or a count of IPv6 prefixes.
    #
    #   If your subnet has the `AssignIpv6AddressOnCreation` attribute set,
    #   you can override that setting by specifying 0 as the IPv6 address
    #   count.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateNetworkInterfaceResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateNetworkInterfaceResult#network_interface #network_interface} => Types::NetworkInterface
    #   * {Types::CreateNetworkInterfaceResult#client_token #client_token} => String
    #
    #
    # @example Example: To create a network interface
    #
    #   # This example creates a network interface for the specified subnet.
    #
    #   resp = client.create_network_interface({
    #     description: "my network interface", 
    #     groups: [
    #       "sg-903004f8", 
    #     ], 
    #     private_ip_address: "10.0.2.17", 
    #     subnet_id: "subnet-9d4a7b6c", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     network_interface: {
    #       availability_zone: "us-east-1d", 
    #       description: "my network interface", 
    #       groups: [
    #         {
    #           group_id: "sg-903004f8", 
    #           group_name: "default", 
    #         }, 
    #       ], 
    #       mac_address: "02:1a:80:41:52:9c", 
    #       network_interface_id: "eni-e5aa89a3", 
    #       owner_id: "123456789012", 
    #       private_ip_address: "10.0.2.17", 
    #       private_ip_addresses: [
    #         {
    #           primary: true, 
    #           private_ip_address: "10.0.2.17", 
    #         }, 
    #       ], 
    #       requester_managed: false, 
    #       source_dest_check: true, 
    #       status: "pending", 
    #       subnet_id: "subnet-9d4a7b6c", 
    #       tag_set: [
    #       ], 
    #       vpc_id: "vpc-a01106c2", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_network_interface({
    #     ipv_4_prefixes: [
    #       {
    #         ipv_4_prefix: "String",
    #       },
    #     ],
    #     ipv_4_prefix_count: 1,
    #     ipv_6_prefixes: [
    #       {
    #         ipv_6_prefix: "String",
    #       },
    #     ],
    #     ipv_6_prefix_count: 1,
    #     interface_type: "efa", # accepts efa, efa-only, branch, trunk
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String",
    #     enable_primary_ipv_6: false,
    #     connection_tracking_specification: {
    #       tcp_established_timeout: 1,
    #       udp_stream_timeout: 1,
    #       udp_timeout: 1,
    #     },
    #     operator: {
    #       principal: "String",
    #     },
    #     subnet_id: "SubnetId", # required
    #     description: "String",
    #     private_ip_address: "String",
    #     groups: ["SecurityGroupId"],
    #     private_ip_addresses: [
    #       {
    #         primary: false,
    #         private_ip_address: "String",
    #       },
    #     ],
    #     secondary_private_ip_address_count: 1,
    #     ipv_6_addresses: [
    #       {
    #         ipv_6_address: "String",
    #         is_primary_ipv_6: false,
    #       },
    #     ],
    #     ipv_6_address_count: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.network_interface.association.allocation_id #=> String
    #   resp.network_interface.association.association_id #=> String
    #   resp.network_interface.association.ip_owner_id #=> String
    #   resp.network_interface.association.public_dns_name #=> String
    #   resp.network_interface.association.public_ip #=> String
    #   resp.network_interface.association.customer_owned_ip #=> String
    #   resp.network_interface.association.carrier_ip #=> String
    #   resp.network_interface.attachment.attach_time #=> Time
    #   resp.network_interface.attachment.attachment_id #=> String
    #   resp.network_interface.attachment.delete_on_termination #=> Boolean
    #   resp.network_interface.attachment.device_index #=> Integer
    #   resp.network_interface.attachment.network_card_index #=> Integer
    #   resp.network_interface.attachment.instance_id #=> String
    #   resp.network_interface.attachment.instance_owner_id #=> String
    #   resp.network_interface.attachment.status #=> String, one of "attaching", "attached", "detaching", "detached"
    #   resp.network_interface.attachment.ena_srd_specification.ena_srd_enabled #=> Boolean
    #   resp.network_interface.attachment.ena_srd_specification.ena_srd_udp_specification.ena_srd_udp_enabled #=> Boolean
    #   resp.network_interface.availability_zone #=> String
    #   resp.network_interface.connection_tracking_configuration.tcp_established_timeout #=> Integer
    #   resp.network_interface.connection_tracking_configuration.udp_stream_timeout #=> Integer
    #   resp.network_interface.connection_tracking_configuration.udp_timeout #=> Integer
    #   resp.network_interface.description #=> String
    #   resp.network_interface.groups #=> Array
    #   resp.network_interface.groups[0].group_id #=> String
    #   resp.network_interface.groups[0].group_name #=> String
    #   resp.network_interface.interface_type #=> String, one of "interface", "natGateway", "efa", "efa-only", "trunk", "load_balancer", "network_load_balancer", "vpc_endpoint", "branch", "transit_gateway", "lambda", "quicksight", "global_accelerator_managed", "api_gateway_managed", "gateway_load_balancer", "gateway_load_balancer_endpoint", "iot_rules_managed", "aws_codestar_connections_managed"
    #   resp.network_interface.ipv_6_addresses #=> Array
    #   resp.network_interface.ipv_6_addresses[0].ipv_6_address #=> String
    #   resp.network_interface.ipv_6_addresses[0].is_primary_ipv_6 #=> Boolean
    #   resp.network_interface.mac_address #=> String
    #   resp.network_interface.network_interface_id #=> String
    #   resp.network_interface.outpost_arn #=> String
    #   resp.network_interface.owner_id #=> String
    #   resp.network_interface.private_dns_name #=> String
    #   resp.network_interface.private_ip_address #=> String
    #   resp.network_interface.private_ip_addresses #=> Array
    #   resp.network_interface.private_ip_addresses[0].association.allocation_id #=> String
    #   resp.network_interface.private_ip_addresses[0].association.association_id #=> String
    #   resp.network_interface.private_ip_addresses[0].association.ip_owner_id #=> String
    #   resp.network_interface.private_ip_addresses[0].association.public_dns_name #=> String
    #   resp.network_interface.private_ip_addresses[0].association.public_ip #=> String
    #   resp.network_interface.private_ip_addresses[0].association.customer_owned_ip #=> String
    #   resp.network_interface.private_ip_addresses[0].association.carrier_ip #=> String
    #   resp.network_interface.private_ip_addresses[0].primary #=> Boolean
    #   resp.network_interface.private_ip_addresses[0].private_dns_name #=> String
    #   resp.network_interface.private_ip_addresses[0].private_ip_address #=> String
    #   resp.network_interface.ipv_4_prefixes #=> Array
    #   resp.network_interface.ipv_4_prefixes[0].ipv_4_prefix #=> String
    #   resp.network_interface.ipv_6_prefixes #=> Array
    #   resp.network_interface.ipv_6_prefixes[0].ipv_6_prefix #=> String
    #   resp.network_interface.requester_id #=> String
    #   resp.network_interface.requester_managed #=> Boolean
    #   resp.network_interface.source_dest_check #=> Boolean
    #   resp.network_interface.status #=> String, one of "available", "associated", "attaching", "in-use", "detaching"
    #   resp.network_interface.subnet_id #=> String
    #   resp.network_interface.tag_set #=> Array
    #   resp.network_interface.tag_set[0].key #=> String
    #   resp.network_interface.tag_set[0].value #=> String
    #   resp.network_interface.vpc_id #=> String
    #   resp.network_interface.deny_all_igw_traffic #=> Boolean
    #   resp.network_interface.ipv_6_native #=> Boolean
    #   resp.network_interface.ipv_6_address #=> String
    #   resp.network_interface.operator.managed #=> Boolean
    #   resp.network_interface.operator.principal #=> String
    #   resp.client_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateNetworkInterface AWS API Documentation
    #
    # @overload create_network_interface(params = {})
    # @param [Hash] params ({})
    def create_network_interface(params = {}, options = {})
      req = build_request(:create_network_interface, params)
      req.send_request(options)
    end

    # Grants an Amazon Web Services-authorized account permission to attach
    # the specified network interface to an instance in their account.
    #
    # You can grant permission to a single Amazon Web Services account only,
    # and only one account at a time.
    #
    # @option params [required, String] :network_interface_id
    #   The ID of the network interface.
    #
    # @option params [String] :aws_account_id
    #   The Amazon Web Services account ID.
    #
    # @option params [String] :aws_service
    #   The Amazon Web Services service. Currently not supported.
    #
    # @option params [required, String] :permission
    #   The type of permission to grant.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateNetworkInterfacePermissionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateNetworkInterfacePermissionResult#interface_permission #interface_permission} => Types::NetworkInterfacePermission
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_network_interface_permission({
    #     network_interface_id: "NetworkInterfaceId", # required
    #     aws_account_id: "String",
    #     aws_service: "String",
    #     permission: "INSTANCE-ATTACH", # required, accepts INSTANCE-ATTACH, EIP-ASSOCIATE
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.interface_permission.network_interface_permission_id #=> String
    #   resp.interface_permission.network_interface_id #=> String
    #   resp.interface_permission.aws_account_id #=> String
    #   resp.interface_permission.aws_service #=> String
    #   resp.interface_permission.permission #=> String, one of "INSTANCE-ATTACH", "EIP-ASSOCIATE"
    #   resp.interface_permission.permission_state.state #=> String, one of "pending", "granted", "revoking", "revoked"
    #   resp.interface_permission.permission_state.status_message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateNetworkInterfacePermission AWS API Documentation
    #
    # @overload create_network_interface_permission(params = {})
    # @param [Hash] params ({})
    def create_network_interface_permission(params = {}, options = {})
      req = build_request(:create_network_interface_permission, params)
      req.send_request(options)
    end

    # Creates a placement group in which to launch instances. The strategy
    # of the placement group determines how the instances are organized
    # within the group.
    #
    # A `cluster` placement group is a logical grouping of instances within
    # a single Availability Zone that benefit from low network latency, high
    # network throughput. A `spread` placement group places instances on
    # distinct hardware. A `partition` placement group places groups of
    # instances in different partitions, where instances in one partition do
    # not share the same hardware with instances in another partition.
    #
    # For more information, see [Placement groups][1] in the *Amazon EC2
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/placement-groups.html
    #
    # @option params [Integer] :partition_count
    #   The number of partitions. Valid only when **Strategy** is set to
    #   `partition`.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the new placement group.
    #
    # @option params [String] :spread_level
    #   Determines how placement groups spread instances.
    #
    #   * Host – You can use `host` only with Outpost placement groups.
    #
    #   * Rack – No usage restrictions.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :group_name
    #   A name for the placement group. Must be unique within the scope of
    #   your account for the Region.
    #
    #   Constraints: Up to 255 ASCII characters
    #
    # @option params [String] :strategy
    #   The placement strategy.
    #
    # @return [Types::CreatePlacementGroupResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreatePlacementGroupResult#placement_group #placement_group} => Types::PlacementGroup
    #
    #
    # @example Example: To create a placement group
    #
    #   # This example creates a placement group with the specified name.
    #
    #   resp = client.create_placement_group({
    #     group_name: "my-cluster", 
    #     strategy: "cluster", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_placement_group({
    #     partition_count: 1,
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     spread_level: "host", # accepts host, rack
    #     dry_run: false,
    #     group_name: "String",
    #     strategy: "cluster", # accepts cluster, spread, partition
    #   })
    #
    # @example Response structure
    #
    #   resp.placement_group.group_name #=> String
    #   resp.placement_group.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.placement_group.strategy #=> String, one of "cluster", "spread", "partition"
    #   resp.placement_group.partition_count #=> Integer
    #   resp.placement_group.group_id #=> String
    #   resp.placement_group.tags #=> Array
    #   resp.placement_group.tags[0].key #=> String
    #   resp.placement_group.tags[0].value #=> String
    #   resp.placement_group.group_arn #=> String
    #   resp.placement_group.spread_level #=> String, one of "host", "rack"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreatePlacementGroup AWS API Documentation
    #
    # @overload create_placement_group(params = {})
    # @param [Hash] params ({})
    def create_placement_group(params = {}, options = {})
      req = build_request(:create_placement_group, params)
      req.send_request(options)
    end

    # Creates a public IPv4 address pool. A public IPv4 pool is an EC2 IP
    # address pool required for the public IPv4 CIDRs that you own and bring
    # to Amazon Web Services to manage with IPAM. IPv6 addresses you bring
    # to Amazon Web Services, however, use IPAM pools only. To monitor the
    # status of pool creation, use [DescribePublicIpv4Pools][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribePublicIpv4Pools.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The key/value combination of a tag assigned to the resource. Use the
    #   tag key in the filter name and the tag value as the filter value. For
    #   example, to find all resources that have a tag with the key `Owner`
    #   and the value `TeamA`, specify `tag:Owner` for the filter name and
    #   `TeamA` for the filter value.
    #
    # @option params [String] :network_border_group
    #   The Availability Zone (AZ) or Local Zone (LZ) network border group
    #   that the resource that the IP address is assigned to is in. Defaults
    #   to an AZ network border group. For more information on available Local
    #   Zones, see [Local Zone availability][1] in the *Amazon EC2 User
    #   Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-byoip.html#byoip-zone-avail
    #
    # @return [Types::CreatePublicIpv4PoolResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreatePublicIpv4PoolResult#pool_id #pool_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_public_ipv_4_pool({
    #     dry_run: false,
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     network_border_group: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.pool_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreatePublicIpv4Pool AWS API Documentation
    #
    # @overload create_public_ipv_4_pool(params = {})
    # @param [Hash] params ({})
    def create_public_ipv_4_pool(params = {}, options = {})
      req = build_request(:create_public_ipv_4_pool, params)
      req.send_request(options)
    end

    # Replaces the EBS-backed root volume for a `running` instance with a
    # new volume that is restored to the original root volume's launch
    # state, that is restored to a specific snapshot taken from the original
    # root volume, or that is restored from an AMI that has the same key
    # characteristics as that of the instance.
    #
    # For more information, see [Replace a root volume][1] in the *Amazon
    # EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/replace-root.html
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance for which to replace the root volume.
    #
    # @option params [String] :snapshot_id
    #   The ID of the snapshot from which to restore the replacement root
    #   volume. The specified snapshot must be a snapshot that you previously
    #   created from the original root volume.
    #
    #   If you want to restore the replacement root volume to the initial
    #   launch state, or if you want to restore the replacement root volume
    #   from an AMI, omit this parameter.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier you provide to ensure the
    #   idempotency of the request. If you do not specify a client token, a
    #   randomly generated token is used for the request to ensure
    #   idempotency. For more information, see [Ensuring idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the root volume replacement task.
    #
    # @option params [String] :image_id
    #   The ID of the AMI to use to restore the root volume. The specified AMI
    #   must have the same product code, billing information, architecture
    #   type, and virtualization type as that of the instance.
    #
    #   If you want to restore the replacement volume from a specific
    #   snapshot, or if you want to restore it to its launch state, omit this
    #   parameter.
    #
    # @option params [Boolean] :delete_replaced_root_volume
    #   Indicates whether to automatically delete the original root volume
    #   after the root volume replacement task completes. To delete the
    #   original root volume, specify `true`. If you choose to keep the
    #   original root volume after the replacement task completes, you must
    #   manually delete it when you no longer need it.
    #
    # @return [Types::CreateReplaceRootVolumeTaskResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateReplaceRootVolumeTaskResult#replace_root_volume_task #replace_root_volume_task} => Types::ReplaceRootVolumeTask
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_replace_root_volume_task({
    #     instance_id: "InstanceId", # required
    #     snapshot_id: "SnapshotId",
    #     client_token: "String",
    #     dry_run: false,
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     image_id: "ImageId",
    #     delete_replaced_root_volume: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.replace_root_volume_task.replace_root_volume_task_id #=> String
    #   resp.replace_root_volume_task.instance_id #=> String
    #   resp.replace_root_volume_task.task_state #=> String, one of "pending", "in-progress", "failing", "succeeded", "failed", "failed-detached"
    #   resp.replace_root_volume_task.start_time #=> String
    #   resp.replace_root_volume_task.complete_time #=> String
    #   resp.replace_root_volume_task.tags #=> Array
    #   resp.replace_root_volume_task.tags[0].key #=> String
    #   resp.replace_root_volume_task.tags[0].value #=> String
    #   resp.replace_root_volume_task.image_id #=> String
    #   resp.replace_root_volume_task.snapshot_id #=> String
    #   resp.replace_root_volume_task.delete_replaced_root_volume #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateReplaceRootVolumeTask AWS API Documentation
    #
    # @overload create_replace_root_volume_task(params = {})
    # @param [Hash] params ({})
    def create_replace_root_volume_task(params = {}, options = {})
      req = build_request(:create_replace_root_volume_task, params)
      req.send_request(options)
    end

    # Creates a listing for Amazon EC2 Standard Reserved Instances to be
    # sold in the Reserved Instance Marketplace. You can submit one Standard
    # Reserved Instance listing at a time. To get a list of your Standard
    # Reserved Instances, you can use the DescribeReservedInstances
    # operation.
    #
    # <note markdown="1"> Only Standard Reserved Instances can be sold in the Reserved Instance
    # Marketplace. Convertible Reserved Instances cannot be sold.
    #
    #  </note>
    #
    # The Reserved Instance Marketplace matches sellers who want to resell
    # Standard Reserved Instance capacity that they no longer need with
    # buyers who want to purchase additional capacity. Reserved Instances
    # bought and sold through the Reserved Instance Marketplace work like
    # any other Reserved Instances.
    #
    # To sell your Standard Reserved Instances, you must first register as a
    # seller in the Reserved Instance Marketplace. After completing the
    # registration process, you can create a Reserved Instance Marketplace
    # listing of some or all of your Standard Reserved Instances, and
    # specify the upfront price to receive for them. Your Standard Reserved
    # Instance listings then become available for purchase. To view the
    # details of your Standard Reserved Instance listing, you can use the
    # DescribeReservedInstancesListings operation.
    #
    # For more information, see [Sell in the Reserved Instance
    # Marketplace][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ri-market-general.html
    #
    # @option params [required, String] :reserved_instances_id
    #   The ID of the active Standard Reserved Instance.
    #
    # @option params [required, Integer] :instance_count
    #   The number of instances that are a part of a Reserved Instance account
    #   to be listed in the Reserved Instance Marketplace. This number should
    #   be less than or equal to the instance count associated with the
    #   Reserved Instance ID specified in this call.
    #
    # @option params [required, Array<Types::PriceScheduleSpecification>] :price_schedules
    #   A list specifying the price of the Standard Reserved Instance for each
    #   month remaining in the Reserved Instance term.
    #
    # @option params [required, String] :client_token
    #   Unique, case-sensitive identifier you provide to ensure idempotency of
    #   your listings. This helps avoid duplicate listings. For more
    #   information, see [Ensuring Idempotency][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @return [Types::CreateReservedInstancesListingResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateReservedInstancesListingResult#reserved_instances_listings #reserved_instances_listings} => Array&lt;Types::ReservedInstancesListing&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_reserved_instances_listing({
    #     reserved_instances_id: "ReservationId", # required
    #     instance_count: 1, # required
    #     price_schedules: [ # required
    #       {
    #         term: 1,
    #         price: 1.0,
    #         currency_code: "USD", # accepts USD
    #       },
    #     ],
    #     client_token: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.reserved_instances_listings #=> Array
    #   resp.reserved_instances_listings[0].client_token #=> String
    #   resp.reserved_instances_listings[0].create_date #=> Time
    #   resp.reserved_instances_listings[0].instance_counts #=> Array
    #   resp.reserved_instances_listings[0].instance_counts[0].instance_count #=> Integer
    #   resp.reserved_instances_listings[0].instance_counts[0].state #=> String, one of "available", "sold", "cancelled", "pending"
    #   resp.reserved_instances_listings[0].price_schedules #=> Array
    #   resp.reserved_instances_listings[0].price_schedules[0].active #=> Boolean
    #   resp.reserved_instances_listings[0].price_schedules[0].currency_code #=> String, one of "USD"
    #   resp.reserved_instances_listings[0].price_schedules[0].price #=> Float
    #   resp.reserved_instances_listings[0].price_schedules[0].term #=> Integer
    #   resp.reserved_instances_listings[0].reserved_instances_id #=> String
    #   resp.reserved_instances_listings[0].reserved_instances_listing_id #=> String
    #   resp.reserved_instances_listings[0].status #=> String, one of "active", "pending", "cancelled", "closed"
    #   resp.reserved_instances_listings[0].status_message #=> String
    #   resp.reserved_instances_listings[0].tags #=> Array
    #   resp.reserved_instances_listings[0].tags[0].key #=> String
    #   resp.reserved_instances_listings[0].tags[0].value #=> String
    #   resp.reserved_instances_listings[0].update_date #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateReservedInstancesListing AWS API Documentation
    #
    # @overload create_reserved_instances_listing(params = {})
    # @param [Hash] params ({})
    def create_reserved_instances_listing(params = {}, options = {})
      req = build_request(:create_reserved_instances_listing, params)
      req.send_request(options)
    end

    # Starts a task that restores an AMI from an Amazon S3 object that was
    # previously created by using [CreateStoreImageTask][1].
    #
    # To use this API, you must have the required permissions. For more
    # information, see [Permissions for storing and restoring AMIs using
    # Amazon S3][2] in the *Amazon EC2 User Guide*.
    #
    # For more information, see [Store and restore an AMI using Amazon
    # S3][3] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateStoreImageTask.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ami-store-restore.html#ami-s3-permissions
    # [3]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ami-store-restore.html
    #
    # @option params [required, String] :bucket
    #   The name of the Amazon S3 bucket that contains the stored AMI object.
    #
    # @option params [required, String] :object_key
    #   The name of the stored AMI object in the bucket.
    #
    # @option params [String] :name
    #   The name for the restored AMI. The name must be unique for AMIs in the
    #   Region for this account. If you do not provide a name, the new AMI
    #   gets the same name as the original AMI.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the AMI and snapshots on restoration. You can tag
    #   the AMI, the snapshots, or both.
    #
    #   * To tag the AMI, the value for `ResourceType` must be `image`.
    #
    #   * To tag the snapshots, the value for `ResourceType` must be
    #     `snapshot`. The same tag is applied to all of the snapshots that are
    #     created.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateRestoreImageTaskResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateRestoreImageTaskResult#image_id #image_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_restore_image_task({
    #     bucket: "String", # required
    #     object_key: "String", # required
    #     name: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.image_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateRestoreImageTask AWS API Documentation
    #
    # @overload create_restore_image_task(params = {})
    # @param [Hash] params ({})
    def create_restore_image_task(params = {}, options = {})
      req = build_request(:create_restore_image_task, params)
      req.send_request(options)
    end

    # Creates a route in a route table within a VPC.
    #
    # You must specify either a destination CIDR block or a prefix list ID.
    # You must also specify exactly one of the resources from the parameter
    # list.
    #
    # When determining how to route traffic, we use the route with the most
    # specific match. For example, traffic is destined for the IPv4 address
    # `192.0.2.3`, and the route table includes the following two IPv4
    # routes:
    #
    # * `192.0.2.0/24` (goes to some target A)
    #
    # * `192.0.2.0/28` (goes to some target B)
    #
    # Both routes apply to the traffic destined for `192.0.2.3`. However,
    # the second route in the list covers a smaller number of IP addresses
    # and is therefore more specific, so we use that route to determine
    # where to target the traffic.
    #
    # For more information about route tables, see [Route tables][1] in the
    # *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Route_Tables.html
    #
    # @option params [String] :destination_prefix_list_id
    #   The ID of a prefix list used for the destination match.
    #
    # @option params [String] :vpc_endpoint_id
    #   The ID of a VPC endpoint. Supported for Gateway Load Balancer
    #   endpoints only.
    #
    # @option params [String] :transit_gateway_id
    #   The ID of a transit gateway.
    #
    # @option params [String] :local_gateway_id
    #   The ID of the local gateway.
    #
    # @option params [String] :carrier_gateway_id
    #   The ID of the carrier gateway.
    #
    #   You can only use this option when the VPC contains a subnet which is
    #   associated with a Wavelength Zone.
    #
    # @option params [String] :core_network_arn
    #   The Amazon Resource Name (ARN) of the core network.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :route_table_id
    #   The ID of the route table for the route.
    #
    # @option params [String] :destination_cidr_block
    #   The IPv4 CIDR address block used for the destination match. Routing
    #   decisions are based on the most specific match. We modify the
    #   specified CIDR block to its canonical form; for example, if you
    #   specify `100.68.0.18/18`, we modify it to `100.68.0.0/18`.
    #
    # @option params [String] :gateway_id
    #   The ID of an internet gateway or virtual private gateway attached to
    #   your VPC.
    #
    # @option params [String] :destination_ipv_6_cidr_block
    #   The IPv6 CIDR block used for the destination match. Routing decisions
    #   are based on the most specific match.
    #
    # @option params [String] :egress_only_internet_gateway_id
    #   \[IPv6 traffic only\] The ID of an egress-only internet gateway.
    #
    # @option params [String] :instance_id
    #   The ID of a NAT instance in your VPC. The operation fails if you
    #   specify an instance ID unless exactly one network interface is
    #   attached.
    #
    # @option params [String] :network_interface_id
    #   The ID of a network interface.
    #
    # @option params [String] :vpc_peering_connection_id
    #   The ID of a VPC peering connection.
    #
    # @option params [String] :nat_gateway_id
    #   \[IPv4 traffic only\] The ID of a NAT gateway.
    #
    # @return [Types::CreateRouteResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateRouteResult#return #return} => Boolean
    #
    #
    # @example Example: To create a route
    #
    #   # This example creates a route for the specified route table. The route matches all traffic (0.0.0.0/0) and routes it to
    #   # the specified Internet gateway.
    #
    #   resp = client.create_route({
    #     destination_cidr_block: "0.0.0.0/0", 
    #     gateway_id: "igw-c0a643a9", 
    #     route_table_id: "rtb-22574640", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_route({
    #     destination_prefix_list_id: "PrefixListResourceId",
    #     vpc_endpoint_id: "VpcEndpointId",
    #     transit_gateway_id: "TransitGatewayId",
    #     local_gateway_id: "LocalGatewayId",
    #     carrier_gateway_id: "CarrierGatewayId",
    #     core_network_arn: "CoreNetworkArn",
    #     dry_run: false,
    #     route_table_id: "RouteTableId", # required
    #     destination_cidr_block: "String",
    #     gateway_id: "RouteGatewayId",
    #     destination_ipv_6_cidr_block: "String",
    #     egress_only_internet_gateway_id: "EgressOnlyInternetGatewayId",
    #     instance_id: "InstanceId",
    #     network_interface_id: "NetworkInterfaceId",
    #     vpc_peering_connection_id: "VpcPeeringConnectionId",
    #     nat_gateway_id: "NatGatewayId",
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateRoute AWS API Documentation
    #
    # @overload create_route(params = {})
    # @param [Hash] params ({})
    def create_route(params = {}, options = {})
      req = build_request(:create_route, params)
      req.send_request(options)
    end

    # Creates a route table for the specified VPC. After you create a route
    # table, you can add routes and associate the table with a subnet.
    #
    # For more information, see [Route tables][1] in the *Amazon VPC User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Route_Tables.html
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the route table.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @return [Types::CreateRouteTableResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateRouteTableResult#route_table #route_table} => Types::RouteTable
    #   * {Types::CreateRouteTableResult#client_token #client_token} => String
    #
    #
    # @example Example: To create a route table
    #
    #   # This example creates a route table for the specified VPC.
    #
    #   resp = client.create_route_table({
    #     vpc_id: "vpc-a01106c2", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     route_table: {
    #       associations: [
    #       ], 
    #       propagating_vgws: [
    #       ], 
    #       route_table_id: "rtb-22574640", 
    #       routes: [
    #         {
    #           destination_cidr_block: "10.0.0.0/16", 
    #           gateway_id: "local", 
    #           state: "active", 
    #         }, 
    #       ], 
    #       tags: [
    #       ], 
    #       vpc_id: "vpc-a01106c2", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_route_table({
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String",
    #     dry_run: false,
    #     vpc_id: "VpcId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.route_table.associations #=> Array
    #   resp.route_table.associations[0].main #=> Boolean
    #   resp.route_table.associations[0].route_table_association_id #=> String
    #   resp.route_table.associations[0].route_table_id #=> String
    #   resp.route_table.associations[0].subnet_id #=> String
    #   resp.route_table.associations[0].gateway_id #=> String
    #   resp.route_table.associations[0].association_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failed"
    #   resp.route_table.associations[0].association_state.status_message #=> String
    #   resp.route_table.propagating_vgws #=> Array
    #   resp.route_table.propagating_vgws[0].gateway_id #=> String
    #   resp.route_table.route_table_id #=> String
    #   resp.route_table.routes #=> Array
    #   resp.route_table.routes[0].destination_cidr_block #=> String
    #   resp.route_table.routes[0].destination_ipv_6_cidr_block #=> String
    #   resp.route_table.routes[0].destination_prefix_list_id #=> String
    #   resp.route_table.routes[0].egress_only_internet_gateway_id #=> String
    #   resp.route_table.routes[0].gateway_id #=> String
    #   resp.route_table.routes[0].instance_id #=> String
    #   resp.route_table.routes[0].instance_owner_id #=> String
    #   resp.route_table.routes[0].nat_gateway_id #=> String
    #   resp.route_table.routes[0].transit_gateway_id #=> String
    #   resp.route_table.routes[0].local_gateway_id #=> String
    #   resp.route_table.routes[0].carrier_gateway_id #=> String
    #   resp.route_table.routes[0].network_interface_id #=> String
    #   resp.route_table.routes[0].origin #=> String, one of "CreateRouteTable", "CreateRoute", "EnableVgwRoutePropagation"
    #   resp.route_table.routes[0].state #=> String, one of "active", "blackhole"
    #   resp.route_table.routes[0].vpc_peering_connection_id #=> String
    #   resp.route_table.routes[0].core_network_arn #=> String
    #   resp.route_table.tags #=> Array
    #   resp.route_table.tags[0].key #=> String
    #   resp.route_table.tags[0].value #=> String
    #   resp.route_table.vpc_id #=> String
    #   resp.route_table.owner_id #=> String
    #   resp.client_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateRouteTable AWS API Documentation
    #
    # @overload create_route_table(params = {})
    # @param [Hash] params ({})
    def create_route_table(params = {}, options = {})
      req = build_request(:create_route_table, params)
      req.send_request(options)
    end

    # Creates a security group.
    #
    # A security group acts as a virtual firewall for your instance to
    # control inbound and outbound traffic. For more information, see
    # [Amazon EC2 security groups][1] in the *Amazon EC2 User Guide* and
    # [Security groups for your VPC][2] in the *Amazon VPC User Guide*.
    #
    # When you create a security group, you specify a friendly name of your
    # choice. You can't have two security groups for the same VPC with the
    # same name.
    #
    # You have a default security group for use in your VPC. If you don't
    # specify a security group when you launch an instance, the instance is
    # launched into the appropriate default security group. A default
    # security group includes a default rule that grants instances
    # unrestricted network access to each other.
    #
    # You can add or remove rules from your security groups using
    # AuthorizeSecurityGroupIngress, AuthorizeSecurityGroupEgress,
    # RevokeSecurityGroupIngress, and RevokeSecurityGroupEgress.
    #
    # For more information about VPC security group limits, see [Amazon VPC
    # Limits][3].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-network-security.html
    # [2]: https://docs.aws.amazon.com/AmazonVPC/latest/UserGuide/VPC_SecurityGroups.html
    # [3]: https://docs.aws.amazon.com/vpc/latest/userguide/amazon-vpc-limits.html
    #
    # @option params [required, String] :description
    #   A description for the security group.
    #
    #   Constraints: Up to 255 characters in length
    #
    #   Valid characters: a-z, A-Z, 0-9, spaces, and
    #   .\_-:/()#,@\[\]+=&amp;;\{}!$*
    #
    # @option params [required, String] :group_name
    #   The name of the security group. Names are case-insensitive and must be
    #   unique within the VPC.
    #
    #   Constraints: Up to 255 characters in length. Can't start with `sg-`.
    #
    #   Valid characters: a-z, A-Z, 0-9, spaces, and
    #   .\_-:/()#,@\[\]+=&amp;;\{}!$*
    #
    # @option params [String] :vpc_id
    #   The ID of the VPC. Required for a nondefault VPC.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the security group.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateSecurityGroupResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateSecurityGroupResult#group_id #group_id} => String
    #   * {Types::CreateSecurityGroupResult#tags #tags} => Array&lt;Types::Tag&gt;
    #   * {Types::CreateSecurityGroupResult#security_group_arn #security_group_arn} => String
    #
    #
    # @example Example: To create a security group for a VPC
    #
    #   # This example creates a security group for the specified VPC.
    #
    #   resp = client.create_security_group({
    #     description: "My security group", 
    #     group_name: "my-security-group", 
    #     vpc_id: "vpc-1a2b3c4d", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     group_id: "sg-903004f8", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_security_group({
    #     description: "String", # required
    #     group_name: "String", # required
    #     vpc_id: "VpcId",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.group_id #=> String
    #   resp.tags #=> Array
    #   resp.tags[0].key #=> String
    #   resp.tags[0].value #=> String
    #   resp.security_group_arn #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateSecurityGroup AWS API Documentation
    #
    # @overload create_security_group(params = {})
    # @param [Hash] params ({})
    def create_security_group(params = {}, options = {})
      req = build_request(:create_security_group, params)
      req.send_request(options)
    end

    # Creates a snapshot of an EBS volume and stores it in Amazon S3. You
    # can use snapshots for backups, to make copies of EBS volumes, and to
    # save data before shutting down an instance.
    #
    # The location of the source EBS volume determines where you can create
    # the snapshot.
    #
    # * If the source volume is in a Region, you must create the snapshot in
    #   the same Region as the volume.
    #
    # * If the source volume is in a Local Zone, you can create the snapshot
    #   in the same Local Zone or in its parent Amazon Web Services Region.
    #
    # * If the source volume is on an Outpost, you can create the snapshot
    #   on the same Outpost or in its parent Amazon Web Services Region.
    #
    # When a snapshot is created, any Amazon Web Services Marketplace
    # product codes that are associated with the source volume are
    # propagated to the snapshot.
    #
    # You can take a snapshot of an attached volume that is in use. However,
    # snapshots only capture data that has been written to your Amazon EBS
    # volume at the time the snapshot command is issued; this might exclude
    # any data that has been cached by any applications or the operating
    # system. If you can pause any file systems on the volume long enough to
    # take a snapshot, your snapshot should be complete. However, if you
    # cannot pause all file writes to the volume, you should unmount the
    # volume from within the instance, issue the snapshot command, and then
    # remount the volume to ensure a consistent and complete snapshot. You
    # may remount and use your volume while the snapshot status is
    # `pending`.
    #
    # When you create a snapshot for an EBS volume that serves as a root
    # device, we recommend that you stop the instance before taking the
    # snapshot.
    #
    # Snapshots that are taken from encrypted volumes are automatically
    # encrypted. Volumes that are created from encrypted snapshots are also
    # automatically encrypted. Your encrypted volumes and any associated
    # snapshots always remain protected. For more information, see [Amazon
    # EBS encryption][1] in the *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-encryption.html
    #
    # @option params [String] :description
    #   A description for the snapshot.
    #
    # @option params [String] :outpost_arn
    #   <note markdown="1"> Only supported for volumes on Outposts. If the source volume is not on
    #   an Outpost, omit this parameter.
    #
    #    </note>
    #
    #   * To create the snapshot on the same Outpost as the source volume,
    #     specify the ARN of that Outpost. The snapshot must be created on the
    #     same Outpost as the volume.
    #
    #   * To create the snapshot in the parent Region of the Outpost, omit
    #     this parameter.
    #
    #   For more information, see [Create local snapshots from volumes on an
    #   Outpost][1] in the *Amazon EBS User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ebs/latest/userguide/snapshots-outposts.html#create-snapshot
    #
    # @option params [required, String] :volume_id
    #   The ID of the Amazon EBS volume.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the snapshot during creation.
    #
    # @option params [String] :location
    #   <note markdown="1"> Only supported for volumes in Local Zones. If the source volume is not
    #   in a Local Zone, omit this parameter.
    #
    #    </note>
    #
    #   * To create a local snapshot in the same Local Zone as the source
    #     volume, specify `local`.
    #
    #   * To create a regional snapshot in the parent Region of the Local
    #     Zone, specify `regional` or omit this parameter.
    #
    #   Default value: `regional`
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::Snapshot] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::Snapshot#owner_alias #owner_alias} => String
    #   * {Types::Snapshot#outpost_arn #outpost_arn} => String
    #   * {Types::Snapshot#tags #tags} => Array&lt;Types::Tag&gt;
    #   * {Types::Snapshot#storage_tier #storage_tier} => String
    #   * {Types::Snapshot#restore_expiry_time #restore_expiry_time} => Time
    #   * {Types::Snapshot#sse_type #sse_type} => String
    #   * {Types::Snapshot#availability_zone #availability_zone} => String
    #   * {Types::Snapshot#transfer_type #transfer_type} => String
    #   * {Types::Snapshot#completion_duration_minutes #completion_duration_minutes} => Integer
    #   * {Types::Snapshot#completion_time #completion_time} => Time
    #   * {Types::Snapshot#full_snapshot_size_in_bytes #full_snapshot_size_in_bytes} => Integer
    #   * {Types::Snapshot#snapshot_id #snapshot_id} => String
    #   * {Types::Snapshot#volume_id #volume_id} => String
    #   * {Types::Snapshot#state #state} => String
    #   * {Types::Snapshot#state_message #state_message} => String
    #   * {Types::Snapshot#start_time #start_time} => Time
    #   * {Types::Snapshot#progress #progress} => String
    #   * {Types::Snapshot#owner_id #owner_id} => String
    #   * {Types::Snapshot#description #description} => String
    #   * {Types::Snapshot#volume_size #volume_size} => Integer
    #   * {Types::Snapshot#encrypted #encrypted} => Boolean
    #   * {Types::Snapshot#kms_key_id #kms_key_id} => String
    #   * {Types::Snapshot#data_encryption_key_id #data_encryption_key_id} => String
    #
    #
    # @example Example: To create a snapshot
    #
    #   # This example creates a snapshot of the volume with a volume ID of ``vol-1234567890abcdef0`` and a short description to
    #   # identify the snapshot.
    #
    #   resp = client.create_snapshot({
    #     description: "This is my root volume snapshot.", 
    #     volume_id: "vol-1234567890abcdef0", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     description: "This is my root volume snapshot.", 
    #     owner_id: "012345678910", 
    #     snapshot_id: "snap-066877671789bd71b", 
    #     start_time: Time.parse("2014-02-28T21:06:01.000Z"), 
    #     state: "pending", 
    #     tags: [
    #     ], 
    #     volume_id: "vol-1234567890abcdef0", 
    #     volume_size: 8, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_snapshot({
    #     description: "String",
    #     outpost_arn: "String",
    #     volume_id: "VolumeId", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     location: "regional", # accepts regional, local
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.owner_alias #=> String
    #   resp.outpost_arn #=> String
    #   resp.tags #=> Array
    #   resp.tags[0].key #=> String
    #   resp.tags[0].value #=> String
    #   resp.storage_tier #=> String, one of "archive", "standard"
    #   resp.restore_expiry_time #=> Time
    #   resp.sse_type #=> String, one of "sse-ebs", "sse-kms", "none"
    #   resp.availability_zone #=> String
    #   resp.transfer_type #=> String, one of "time-based", "standard"
    #   resp.completion_duration_minutes #=> Integer
    #   resp.completion_time #=> Time
    #   resp.full_snapshot_size_in_bytes #=> Integer
    #   resp.snapshot_id #=> String
    #   resp.volume_id #=> String
    #   resp.state #=> String, one of "pending", "completed", "error", "recoverable", "recovering"
    #   resp.state_message #=> String
    #   resp.start_time #=> Time
    #   resp.progress #=> String
    #   resp.owner_id #=> String
    #   resp.description #=> String
    #   resp.volume_size #=> Integer
    #   resp.encrypted #=> Boolean
    #   resp.kms_key_id #=> String
    #   resp.data_encryption_key_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateSnapshot AWS API Documentation
    #
    # @overload create_snapshot(params = {})
    # @param [Hash] params ({})
    def create_snapshot(params = {}, options = {})
      req = build_request(:create_snapshot, params)
      req.send_request(options)
    end

    # Creates crash-consistent snapshots of multiple EBS volumes attached to
    # an Amazon EC2 instance. Volumes are chosen by specifying an instance.
    # Each volume attached to the specified instance will produce one
    # snapshot that is crash-consistent across the instance. You can include
    # all of the volumes currently attached to the instance, or you can
    # exclude the root volume or specific data (non-root) volumes from the
    # multi-volume snapshot set.
    #
    # The location of the source instance determines where you can create
    # the snapshots.
    #
    # * If the source instance is in a Region, you must create the snapshots
    #   in the same Region as the instance.
    #
    # * If the source instance is in a Local Zone, you can create the
    #   snapshots in the same Local Zone or in its parent Amazon Web
    #   Services Region.
    #
    # * If the source instance is on an Outpost, you can create the
    #   snapshots on the same Outpost or in its parent Amazon Web Services
    #   Region.
    #
    # @option params [String] :description
    #   A description propagated to every snapshot specified by the instance.
    #
    # @option params [required, Types::InstanceSpecification] :instance_specification
    #   The instance to specify which volumes should be included in the
    #   snapshots.
    #
    # @option params [String] :outpost_arn
    #   <note markdown="1"> Only supported for instances on Outposts. If the source instance is
    #   not on an Outpost, omit this parameter.
    #
    #    </note>
    #
    #   * To create the snapshots on the same Outpost as the source instance,
    #     specify the ARN of that Outpost. The snapshots must be created on
    #     the same Outpost as the instance.
    #
    #   * To create the snapshots in the parent Region of the Outpost, omit
    #     this parameter.
    #
    #   For more information, see [ Create local snapshots from volumes on an
    #   Outpost][1] in the *Amazon EBS User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ebs/latest/userguide/snapshots-outposts.html#create-snapshot
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   Tags to apply to every snapshot specified by the instance.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :copy_tags_from_source
    #   Copies the tags from the specified volume to corresponding snapshot.
    #
    # @option params [String] :location
    #   <note markdown="1"> Only supported for instances in Local Zones. If the source instance is
    #   not in a Local Zone, omit this parameter.
    #
    #    </note>
    #
    #   * To create local snapshots in the same Local Zone as the source
    #     instance, specify `local`.
    #
    #   * To create a regional snapshots in the parent Region of the Local
    #     Zone, specify `regional` or omit this parameter.
    #
    #   Default value: `regional`
    #
    # @return [Types::CreateSnapshotsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateSnapshotsResult#snapshots #snapshots} => Array&lt;Types::SnapshotInfo&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_snapshots({
    #     description: "String",
    #     instance_specification: { # required
    #       instance_id: "InstanceIdWithVolumeResolver", # required
    #       exclude_boot_volume: false,
    #       exclude_data_volume_ids: ["VolumeId"],
    #     },
    #     outpost_arn: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #     copy_tags_from_source: "volume", # accepts volume
    #     location: "regional", # accepts regional, local
    #   })
    #
    # @example Response structure
    #
    #   resp.snapshots #=> Array
    #   resp.snapshots[0].description #=> String
    #   resp.snapshots[0].tags #=> Array
    #   resp.snapshots[0].tags[0].key #=> String
    #   resp.snapshots[0].tags[0].value #=> String
    #   resp.snapshots[0].encrypted #=> Boolean
    #   resp.snapshots[0].volume_id #=> String
    #   resp.snapshots[0].state #=> String, one of "pending", "completed", "error", "recoverable", "recovering"
    #   resp.snapshots[0].volume_size #=> Integer
    #   resp.snapshots[0].start_time #=> Time
    #   resp.snapshots[0].progress #=> String
    #   resp.snapshots[0].owner_id #=> String
    #   resp.snapshots[0].snapshot_id #=> String
    #   resp.snapshots[0].outpost_arn #=> String
    #   resp.snapshots[0].sse_type #=> String, one of "sse-ebs", "sse-kms", "none"
    #   resp.snapshots[0].availability_zone #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateSnapshots AWS API Documentation
    #
    # @overload create_snapshots(params = {})
    # @param [Hash] params ({})
    def create_snapshots(params = {}, options = {})
      req = build_request(:create_snapshots, params)
      req.send_request(options)
    end

    # Creates a data feed for Spot Instances, enabling you to view Spot
    # Instance usage logs. You can create one data feed per Amazon Web
    # Services account. For more information, see [Spot Instance data
    # feed][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-data-feeds.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :bucket
    #   The name of the Amazon S3 bucket in which to store the Spot Instance
    #   data feed. For more information about bucket names, see [Bucket naming
    #   rules][1] in the *Amazon S3 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AmazonS3/latest/userguide/bucketnamingrules.html
    #
    # @option params [String] :prefix
    #   The prefix for the data feed file names.
    #
    # @return [Types::CreateSpotDatafeedSubscriptionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateSpotDatafeedSubscriptionResult#spot_datafeed_subscription #spot_datafeed_subscription} => Types::SpotDatafeedSubscription
    #
    #
    # @example Example: To create a Spot Instance datafeed
    #
    #   # This example creates a Spot Instance data feed for your AWS account.
    #
    #   resp = client.create_spot_datafeed_subscription({
    #     bucket: "my-s3-bucket", 
    #     prefix: "spotdata", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     spot_datafeed_subscription: {
    #       bucket: "my-s3-bucket", 
    #       owner_id: "123456789012", 
    #       prefix: "spotdata", 
    #       state: "Active", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_spot_datafeed_subscription({
    #     dry_run: false,
    #     bucket: "String", # required
    #     prefix: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.spot_datafeed_subscription.bucket #=> String
    #   resp.spot_datafeed_subscription.fault.code #=> String
    #   resp.spot_datafeed_subscription.fault.message #=> String
    #   resp.spot_datafeed_subscription.owner_id #=> String
    #   resp.spot_datafeed_subscription.prefix #=> String
    #   resp.spot_datafeed_subscription.state #=> String, one of "Active", "Inactive"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateSpotDatafeedSubscription AWS API Documentation
    #
    # @overload create_spot_datafeed_subscription(params = {})
    # @param [Hash] params ({})
    def create_spot_datafeed_subscription(params = {}, options = {})
      req = build_request(:create_spot_datafeed_subscription, params)
      req.send_request(options)
    end

    # Stores an AMI as a single object in an Amazon S3 bucket.
    #
    # To use this API, you must have the required permissions. For more
    # information, see [Permissions for storing and restoring AMIs using
    # Amazon S3][1] in the *Amazon EC2 User Guide*.
    #
    # For more information, see [Store and restore an AMI using Amazon
    # S3][2] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ami-store-restore.html#ami-s3-permissions
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ami-store-restore.html
    #
    # @option params [required, String] :image_id
    #   The ID of the AMI.
    #
    # @option params [required, String] :bucket
    #   The name of the Amazon S3 bucket in which the AMI object will be
    #   stored. The bucket must be in the Region in which the request is being
    #   made. The AMI object appears in the bucket only after the upload task
    #   has completed.
    #
    # @option params [Array<Types::S3ObjectTag>] :s3_object_tags
    #   The tags to apply to the AMI object that will be stored in the Amazon
    #   S3 bucket.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateStoreImageTaskResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateStoreImageTaskResult#object_key #object_key} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_store_image_task({
    #     image_id: "ImageId", # required
    #     bucket: "String", # required
    #     s3_object_tags: [
    #       {
    #         key: "String",
    #         value: "String",
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.object_key #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateStoreImageTask AWS API Documentation
    #
    # @overload create_store_image_task(params = {})
    # @param [Hash] params ({})
    def create_store_image_task(params = {}, options = {})
      req = build_request(:create_store_image_task, params)
      req.send_request(options)
    end

    # Creates a subnet in the specified VPC. For an IPv4 only subnet,
    # specify an IPv4 CIDR block. If the VPC has an IPv6 CIDR block, you can
    # create an IPv6 only subnet or a dual stack subnet instead. For an IPv6
    # only subnet, specify an IPv6 CIDR block. For a dual stack subnet,
    # specify both an IPv4 CIDR block and an IPv6 CIDR block.
    #
    # A subnet CIDR block must not overlap the CIDR block of an existing
    # subnet in the VPC. After you create a subnet, you can't change its
    # CIDR block.
    #
    # The allowed size for an IPv4 subnet is between a /28 netmask (16 IP
    # addresses) and a /16 netmask (65,536 IP addresses). Amazon Web
    # Services reserves both the first four and the last IPv4 address in
    # each subnet's CIDR block. They're not available for your use.
    #
    # If you've associated an IPv6 CIDR block with your VPC, you can
    # associate an IPv6 CIDR block with a subnet when you create it.
    #
    # If you add more than one subnet to a VPC, they're set up in a star
    # topology with a logical router in the middle.
    #
    # When you stop an instance in a subnet, it retains its private IPv4
    # address. It's therefore possible to have a subnet with no running
    # instances (they're all stopped), but no remaining IP addresses
    # available.
    #
    # For more information, see [Subnets][1] in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/configure-subnets.html
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the subnet.
    #
    # @option params [String] :availability_zone
    #   The Availability Zone or Local Zone for the subnet.
    #
    #   Default: Amazon Web Services selects one for you. If you create more
    #   than one subnet in your VPC, we do not necessarily select a different
    #   zone for each subnet.
    #
    #   To create a subnet in a Local Zone, set this value to the Local Zone
    #   ID, for example `us-west-2-lax-1a`. For information about the Regions
    #   that support Local Zones, see [Available Local Zones][1].
    #
    #   To create a subnet in an Outpost, set this value to the Availability
    #   Zone for the Outpost and specify the Outpost ARN.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/local-zones/latest/ug/available-local-zones.html
    #
    # @option params [String] :availability_zone_id
    #   The AZ ID or the Local Zone ID of the subnet.
    #
    # @option params [String] :cidr_block
    #   The IPv4 network range for the subnet, in CIDR notation. For example,
    #   `10.0.0.0/24`. We modify the specified CIDR block to its canonical
    #   form; for example, if you specify `100.68.0.18/18`, we modify it to
    #   `100.68.0.0/18`.
    #
    #   This parameter is not supported for an IPv6 only subnet.
    #
    # @option params [String] :ipv_6_cidr_block
    #   The IPv6 network range for the subnet, in CIDR notation. This
    #   parameter is required for an IPv6 only subnet.
    #
    # @option params [String] :outpost_arn
    #   The Amazon Resource Name (ARN) of the Outpost. If you specify an
    #   Outpost ARN, you must also specify the Availability Zone of the
    #   Outpost subnet.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @option params [Boolean] :ipv_6_native
    #   Indicates whether to create an IPv6 only subnet.
    #
    # @option params [String] :ipv_4_ipam_pool_id
    #   An IPv4 IPAM pool ID for the subnet.
    #
    # @option params [Integer] :ipv_4_netmask_length
    #   An IPv4 netmask length for the subnet.
    #
    # @option params [String] :ipv_6_ipam_pool_id
    #   An IPv6 IPAM pool ID for the subnet.
    #
    # @option params [Integer] :ipv_6_netmask_length
    #   An IPv6 netmask length for the subnet.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateSubnetResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateSubnetResult#subnet #subnet} => Types::Subnet
    #
    #
    # @example Example: To create a subnet
    #
    #   # This example creates a subnet in the specified VPC with the specified CIDR block. We recommend that you let us select an
    #   # Availability Zone for you.
    #
    #   resp = client.create_subnet({
    #     cidr_block: "10.0.1.0/24", 
    #     vpc_id: "vpc-a01106c2", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     subnet: {
    #       availability_zone: "us-west-2c", 
    #       available_ip_address_count: 251, 
    #       cidr_block: "10.0.1.0/24", 
    #       state: "pending", 
    #       subnet_id: "subnet-9d4a7b6c", 
    #       vpc_id: "vpc-a01106c2", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_subnet({
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     availability_zone: "String",
    #     availability_zone_id: "String",
    #     cidr_block: "String",
    #     ipv_6_cidr_block: "String",
    #     outpost_arn: "String",
    #     vpc_id: "VpcId", # required
    #     ipv_6_native: false,
    #     ipv_4_ipam_pool_id: "IpamPoolId",
    #     ipv_4_netmask_length: 1,
    #     ipv_6_ipam_pool_id: "IpamPoolId",
    #     ipv_6_netmask_length: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.subnet.availability_zone_id #=> String
    #   resp.subnet.enable_lni_at_device_index #=> Integer
    #   resp.subnet.map_customer_owned_ip_on_launch #=> Boolean
    #   resp.subnet.customer_owned_ipv_4_pool #=> String
    #   resp.subnet.owner_id #=> String
    #   resp.subnet.assign_ipv_6_address_on_creation #=> Boolean
    #   resp.subnet.ipv_6_cidr_block_association_set #=> Array
    #   resp.subnet.ipv_6_cidr_block_association_set[0].association_id #=> String
    #   resp.subnet.ipv_6_cidr_block_association_set[0].ipv_6_cidr_block #=> String
    #   resp.subnet.ipv_6_cidr_block_association_set[0].ipv_6_cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.subnet.ipv_6_cidr_block_association_set[0].ipv_6_cidr_block_state.status_message #=> String
    #   resp.subnet.ipv_6_cidr_block_association_set[0].ipv_6_address_attribute #=> String, one of "public", "private"
    #   resp.subnet.ipv_6_cidr_block_association_set[0].ip_source #=> String, one of "amazon", "byoip", "none"
    #   resp.subnet.tags #=> Array
    #   resp.subnet.tags[0].key #=> String
    #   resp.subnet.tags[0].value #=> String
    #   resp.subnet.subnet_arn #=> String
    #   resp.subnet.outpost_arn #=> String
    #   resp.subnet.enable_dns_64 #=> Boolean
    #   resp.subnet.ipv_6_native #=> Boolean
    #   resp.subnet.private_dns_name_options_on_launch.hostname_type #=> String, one of "ip-name", "resource-name"
    #   resp.subnet.private_dns_name_options_on_launch.enable_resource_name_dns_a_record #=> Boolean
    #   resp.subnet.private_dns_name_options_on_launch.enable_resource_name_dns_aaaa_record #=> Boolean
    #   resp.subnet.block_public_access_states.internet_gateway_block_mode #=> String, one of "off", "block-bidirectional", "block-ingress"
    #   resp.subnet.subnet_id #=> String
    #   resp.subnet.state #=> String, one of "pending", "available", "unavailable"
    #   resp.subnet.vpc_id #=> String
    #   resp.subnet.cidr_block #=> String
    #   resp.subnet.available_ip_address_count #=> Integer
    #   resp.subnet.availability_zone #=> String
    #   resp.subnet.default_for_az #=> Boolean
    #   resp.subnet.map_public_ip_on_launch #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateSubnet AWS API Documentation
    #
    # @overload create_subnet(params = {})
    # @param [Hash] params ({})
    def create_subnet(params = {}, options = {})
      req = build_request(:create_subnet, params)
      req.send_request(options)
    end

    # Creates a subnet CIDR reservation. For more information, see [Subnet
    # CIDR reservations][1] in the *Amazon VPC User Guide* and [Manage
    # prefixes for your network interfaces][2] in the *Amazon EC2 User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/subnet-cidr-reservation.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/work-with-prefixes.html
    #
    # @option params [required, String] :subnet_id
    #   The ID of the subnet.
    #
    # @option params [required, String] :cidr
    #   The IPv4 or IPV6 CIDR range to reserve.
    #
    # @option params [required, String] :reservation_type
    #   The type of reservation. The reservation type determines how the
    #   reserved IP addresses are assigned to resources.
    #
    #   * `prefix` - Amazon Web Services assigns the reserved IP addresses to
    #     network interfaces.
    #
    #   * `explicit` - You assign the reserved IP addresses to network
    #     interfaces.
    #
    # @option params [String] :description
    #   The description to assign to the subnet CIDR reservation.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the subnet CIDR reservation.
    #
    # @return [Types::CreateSubnetCidrReservationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateSubnetCidrReservationResult#subnet_cidr_reservation #subnet_cidr_reservation} => Types::SubnetCidrReservation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_subnet_cidr_reservation({
    #     subnet_id: "SubnetId", # required
    #     cidr: "String", # required
    #     reservation_type: "prefix", # required, accepts prefix, explicit
    #     description: "String",
    #     dry_run: false,
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.subnet_cidr_reservation.subnet_cidr_reservation_id #=> String
    #   resp.subnet_cidr_reservation.subnet_id #=> String
    #   resp.subnet_cidr_reservation.cidr #=> String
    #   resp.subnet_cidr_reservation.reservation_type #=> String, one of "prefix", "explicit"
    #   resp.subnet_cidr_reservation.owner_id #=> String
    #   resp.subnet_cidr_reservation.description #=> String
    #   resp.subnet_cidr_reservation.tags #=> Array
    #   resp.subnet_cidr_reservation.tags[0].key #=> String
    #   resp.subnet_cidr_reservation.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateSubnetCidrReservation AWS API Documentation
    #
    # @overload create_subnet_cidr_reservation(params = {})
    # @param [Hash] params ({})
    def create_subnet_cidr_reservation(params = {}, options = {})
      req = build_request(:create_subnet_cidr_reservation, params)
      req.send_request(options)
    end

    # Adds or overwrites only the specified tags for the specified Amazon
    # EC2 resource or resources. When you specify an existing tag key, the
    # value is overwritten with the new value. Each resource can have a
    # maximum of 50 tags. Each tag consists of a key and optional value. Tag
    # keys must be unique per resource.
    #
    # For more information about tags, see [Tag your Amazon EC2
    # resources][1] in the *Amazon Elastic Compute Cloud User Guide*. For
    # more information about creating IAM policies that control users'
    # access to resources based on tags, see [Supported resource-level
    # permissions for Amazon EC2 API actions][2] in the *Amazon Elastic
    # Compute Cloud User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-supported-iam-actions-resources.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :resources
    #   The IDs of the resources, separated by spaces.
    #
    #   Constraints: Up to 1000 resource IDs. We recommend breaking up this
    #   request into smaller batches.
    #
    # @option params [required, Array<Types::Tag>] :tags
    #   The tags. The `value` parameter is required, but if you don't want
    #   the tag to have a value, specify the parameter with no value, and we
    #   set the value to an empty string.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To add a tag to a resource
    #
    #   # This example adds the tag Stack=production to the specified image, or overwrites an existing tag for the AMI where the
    #   # tag key is Stack.
    #
    #   resp = client.create_tags({
    #     resources: [
    #       "ami-78a54011", 
    #     ], 
    #     tags: [
    #       {
    #         key: "Stack", 
    #         value: "production", 
    #       }, 
    #     ], 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_tags({
    #     dry_run: false,
    #     resources: ["TaggableResourceId"], # required
    #     tags: [ # required
    #       {
    #         key: "String",
    #         value: "String",
    #       },
    #     ],
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTags AWS API Documentation
    #
    # @overload create_tags(params = {})
    # @param [Hash] params ({})
    def create_tags(params = {}, options = {})
      req = build_request(:create_tags, params)
      req.send_request(options)
    end

    # Creates a Traffic Mirror filter.
    #
    # A Traffic Mirror filter is a set of rules that defines the traffic to
    # mirror.
    #
    # By default, no traffic is mirrored. To mirror traffic, use
    # [CreateTrafficMirrorFilterRule][1] to add Traffic Mirror rules to the
    # filter. The rules you add define what traffic gets mirrored. You can
    # also use [ModifyTrafficMirrorFilterNetworkServices][2] to mirror
    # supported network services.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTrafficMirrorFilterRule.htm
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ModifyTrafficMirrorFilterNetworkServices.html
    #
    # @option params [String] :description
    #   The description of the Traffic Mirror filter.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to a Traffic Mirror filter.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [How to ensure
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @return [Types::CreateTrafficMirrorFilterResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTrafficMirrorFilterResult#traffic_mirror_filter #traffic_mirror_filter} => Types::TrafficMirrorFilter
    #   * {Types::CreateTrafficMirrorFilterResult#client_token #client_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_traffic_mirror_filter({
    #     description: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #     client_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_filter.traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_filter.ingress_filter_rules #=> Array
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].traffic_mirror_filter_rule_id #=> String
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].traffic_direction #=> String, one of "ingress", "egress"
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].rule_number #=> Integer
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].rule_action #=> String, one of "accept", "reject"
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].protocol #=> Integer
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].destination_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].destination_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].source_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].source_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].destination_cidr_block #=> String
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].source_cidr_block #=> String
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].description #=> String
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].tags #=> Array
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].tags[0].key #=> String
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].tags[0].value #=> String
    #   resp.traffic_mirror_filter.egress_filter_rules #=> Array
    #   resp.traffic_mirror_filter.egress_filter_rules[0].traffic_mirror_filter_rule_id #=> String
    #   resp.traffic_mirror_filter.egress_filter_rules[0].traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_filter.egress_filter_rules[0].traffic_direction #=> String, one of "ingress", "egress"
    #   resp.traffic_mirror_filter.egress_filter_rules[0].rule_number #=> Integer
    #   resp.traffic_mirror_filter.egress_filter_rules[0].rule_action #=> String, one of "accept", "reject"
    #   resp.traffic_mirror_filter.egress_filter_rules[0].protocol #=> Integer
    #   resp.traffic_mirror_filter.egress_filter_rules[0].destination_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filter.egress_filter_rules[0].destination_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filter.egress_filter_rules[0].source_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filter.egress_filter_rules[0].source_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filter.egress_filter_rules[0].destination_cidr_block #=> String
    #   resp.traffic_mirror_filter.egress_filter_rules[0].source_cidr_block #=> String
    #   resp.traffic_mirror_filter.egress_filter_rules[0].description #=> String
    #   resp.traffic_mirror_filter.egress_filter_rules[0].tags #=> Array
    #   resp.traffic_mirror_filter.egress_filter_rules[0].tags[0].key #=> String
    #   resp.traffic_mirror_filter.egress_filter_rules[0].tags[0].value #=> String
    #   resp.traffic_mirror_filter.network_services #=> Array
    #   resp.traffic_mirror_filter.network_services[0] #=> String, one of "amazon-dns"
    #   resp.traffic_mirror_filter.description #=> String
    #   resp.traffic_mirror_filter.tags #=> Array
    #   resp.traffic_mirror_filter.tags[0].key #=> String
    #   resp.traffic_mirror_filter.tags[0].value #=> String
    #   resp.client_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTrafficMirrorFilter AWS API Documentation
    #
    # @overload create_traffic_mirror_filter(params = {})
    # @param [Hash] params ({})
    def create_traffic_mirror_filter(params = {}, options = {})
      req = build_request(:create_traffic_mirror_filter, params)
      req.send_request(options)
    end

    # Creates a Traffic Mirror filter rule.
    #
    # A Traffic Mirror rule defines the Traffic Mirror source traffic to
    # mirror.
    #
    # You need the Traffic Mirror filter ID when you create the rule.
    #
    # @option params [required, String] :traffic_mirror_filter_id
    #   The ID of the filter that this rule is associated with.
    #
    # @option params [required, String] :traffic_direction
    #   The type of traffic.
    #
    # @option params [required, Integer] :rule_number
    #   The number of the Traffic Mirror rule. This number must be unique for
    #   each Traffic Mirror rule in a given direction. The rules are processed
    #   in ascending order by rule number.
    #
    # @option params [required, String] :rule_action
    #   The action to take on the filtered traffic.
    #
    # @option params [Types::TrafficMirrorPortRangeRequest] :destination_port_range
    #   The destination port range.
    #
    # @option params [Types::TrafficMirrorPortRangeRequest] :source_port_range
    #   The source port range.
    #
    # @option params [Integer] :protocol
    #   The protocol, for example UDP, to assign to the Traffic Mirror rule.
    #
    #   For information about the protocol value, see [Protocol Numbers][1] on
    #   the Internet Assigned Numbers Authority (IANA) website.
    #
    #
    #
    #   [1]: https://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml
    #
    # @option params [required, String] :destination_cidr_block
    #   The destination CIDR block to assign to the Traffic Mirror rule.
    #
    # @option params [required, String] :source_cidr_block
    #   The source CIDR block to assign to the Traffic Mirror rule.
    #
    # @option params [String] :description
    #   The description of the Traffic Mirror rule.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [How to ensure
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   Traffic Mirroring tags specifications.
    #
    # @return [Types::CreateTrafficMirrorFilterRuleResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTrafficMirrorFilterRuleResult#traffic_mirror_filter_rule #traffic_mirror_filter_rule} => Types::TrafficMirrorFilterRule
    #   * {Types::CreateTrafficMirrorFilterRuleResult#client_token #client_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_traffic_mirror_filter_rule({
    #     traffic_mirror_filter_id: "TrafficMirrorFilterId", # required
    #     traffic_direction: "ingress", # required, accepts ingress, egress
    #     rule_number: 1, # required
    #     rule_action: "accept", # required, accepts accept, reject
    #     destination_port_range: {
    #       from_port: 1,
    #       to_port: 1,
    #     },
    #     source_port_range: {
    #       from_port: 1,
    #       to_port: 1,
    #     },
    #     protocol: 1,
    #     destination_cidr_block: "String", # required
    #     source_cidr_block: "String", # required
    #     description: "String",
    #     dry_run: false,
    #     client_token: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_filter_rule.traffic_mirror_filter_rule_id #=> String
    #   resp.traffic_mirror_filter_rule.traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_filter_rule.traffic_direction #=> String, one of "ingress", "egress"
    #   resp.traffic_mirror_filter_rule.rule_number #=> Integer
    #   resp.traffic_mirror_filter_rule.rule_action #=> String, one of "accept", "reject"
    #   resp.traffic_mirror_filter_rule.protocol #=> Integer
    #   resp.traffic_mirror_filter_rule.destination_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filter_rule.destination_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filter_rule.source_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filter_rule.source_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filter_rule.destination_cidr_block #=> String
    #   resp.traffic_mirror_filter_rule.source_cidr_block #=> String
    #   resp.traffic_mirror_filter_rule.description #=> String
    #   resp.traffic_mirror_filter_rule.tags #=> Array
    #   resp.traffic_mirror_filter_rule.tags[0].key #=> String
    #   resp.traffic_mirror_filter_rule.tags[0].value #=> String
    #   resp.client_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTrafficMirrorFilterRule AWS API Documentation
    #
    # @overload create_traffic_mirror_filter_rule(params = {})
    # @param [Hash] params ({})
    def create_traffic_mirror_filter_rule(params = {}, options = {})
      req = build_request(:create_traffic_mirror_filter_rule, params)
      req.send_request(options)
    end

    # Creates a Traffic Mirror session.
    #
    # A Traffic Mirror session actively copies packets from a Traffic Mirror
    # source to a Traffic Mirror target. Create a filter, and then assign it
    # to the session to define a subset of the traffic to mirror, for
    # example all TCP traffic.
    #
    # The Traffic Mirror source and the Traffic Mirror target (monitoring
    # appliances) can be in the same VPC, or in a different VPC connected
    # via VPC peering or a transit gateway.
    #
    # By default, no traffic is mirrored. Use [CreateTrafficMirrorFilter][1]
    # to create filter rules that specify the traffic to mirror.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTrafficMirrorFilter.html
    #
    # @option params [required, String] :network_interface_id
    #   The ID of the source network interface.
    #
    # @option params [required, String] :traffic_mirror_target_id
    #   The ID of the Traffic Mirror target.
    #
    # @option params [required, String] :traffic_mirror_filter_id
    #   The ID of the Traffic Mirror filter.
    #
    # @option params [Integer] :packet_length
    #   The number of bytes in each packet to mirror. These are bytes after
    #   the VXLAN header. Do not specify this parameter when you want to
    #   mirror the entire packet. To mirror a subset of the packet, set this
    #   to the length (in bytes) that you want to mirror. For example, if you
    #   set this value to 100, then the first 100 bytes that meet the filter
    #   criteria are copied to the target.
    #
    #   If you do not want to mirror the entire packet, use the `PacketLength`
    #   parameter to specify the number of bytes in each packet to mirror.
    #
    #   For sessions with Network Load Balancer (NLB) Traffic Mirror targets
    #   the default `PacketLength` will be set to 8500. Valid values are
    #   1-8500. Setting a `PacketLength` greater than 8500 will result in an
    #   error response.
    #
    # @option params [required, Integer] :session_number
    #   The session number determines the order in which sessions are
    #   evaluated when an interface is used by multiple sessions. The first
    #   session with a matching filter is the one that mirrors the packets.
    #
    #   Valid values are 1-32766.
    #
    # @option params [Integer] :virtual_network_id
    #   The VXLAN ID for the Traffic Mirror session. For more information
    #   about the VXLAN protocol, see [RFC 7348][1]. If you do not specify a
    #   `VirtualNetworkId`, an account-wide unique ID is chosen at random.
    #
    #
    #
    #   [1]: https://datatracker.ietf.org/doc/html/rfc7348
    #
    # @option params [String] :description
    #   The description of the Traffic Mirror session.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to a Traffic Mirror session.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [How to ensure
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @return [Types::CreateTrafficMirrorSessionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTrafficMirrorSessionResult#traffic_mirror_session #traffic_mirror_session} => Types::TrafficMirrorSession
    #   * {Types::CreateTrafficMirrorSessionResult#client_token #client_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_traffic_mirror_session({
    #     network_interface_id: "NetworkInterfaceId", # required
    #     traffic_mirror_target_id: "TrafficMirrorTargetId", # required
    #     traffic_mirror_filter_id: "TrafficMirrorFilterId", # required
    #     packet_length: 1,
    #     session_number: 1, # required
    #     virtual_network_id: 1,
    #     description: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #     client_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_session.traffic_mirror_session_id #=> String
    #   resp.traffic_mirror_session.traffic_mirror_target_id #=> String
    #   resp.traffic_mirror_session.traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_session.network_interface_id #=> String
    #   resp.traffic_mirror_session.owner_id #=> String
    #   resp.traffic_mirror_session.packet_length #=> Integer
    #   resp.traffic_mirror_session.session_number #=> Integer
    #   resp.traffic_mirror_session.virtual_network_id #=> Integer
    #   resp.traffic_mirror_session.description #=> String
    #   resp.traffic_mirror_session.tags #=> Array
    #   resp.traffic_mirror_session.tags[0].key #=> String
    #   resp.traffic_mirror_session.tags[0].value #=> String
    #   resp.client_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTrafficMirrorSession AWS API Documentation
    #
    # @overload create_traffic_mirror_session(params = {})
    # @param [Hash] params ({})
    def create_traffic_mirror_session(params = {}, options = {})
      req = build_request(:create_traffic_mirror_session, params)
      req.send_request(options)
    end

    # Creates a target for your Traffic Mirror session.
    #
    # A Traffic Mirror target is the destination for mirrored traffic. The
    # Traffic Mirror source and the Traffic Mirror target (monitoring
    # appliances) can be in the same VPC, or in different VPCs connected via
    # VPC peering or a transit gateway.
    #
    # A Traffic Mirror target can be a network interface, a Network Load
    # Balancer, or a Gateway Load Balancer endpoint.
    #
    # To use the target in a Traffic Mirror session, use
    # [CreateTrafficMirrorSession][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTrafficMirrorSession.htm
    #
    # @option params [String] :network_interface_id
    #   The network interface ID that is associated with the target.
    #
    # @option params [String] :network_load_balancer_arn
    #   The Amazon Resource Name (ARN) of the Network Load Balancer that is
    #   associated with the target.
    #
    # @option params [String] :description
    #   The description of the Traffic Mirror target.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the Traffic Mirror target.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [How to ensure
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [String] :gateway_load_balancer_endpoint_id
    #   The ID of the Gateway Load Balancer endpoint.
    #
    # @return [Types::CreateTrafficMirrorTargetResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTrafficMirrorTargetResult#traffic_mirror_target #traffic_mirror_target} => Types::TrafficMirrorTarget
    #   * {Types::CreateTrafficMirrorTargetResult#client_token #client_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_traffic_mirror_target({
    #     network_interface_id: "NetworkInterfaceId",
    #     network_load_balancer_arn: "String",
    #     description: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #     client_token: "String",
    #     gateway_load_balancer_endpoint_id: "VpcEndpointId",
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_target.traffic_mirror_target_id #=> String
    #   resp.traffic_mirror_target.network_interface_id #=> String
    #   resp.traffic_mirror_target.network_load_balancer_arn #=> String
    #   resp.traffic_mirror_target.type #=> String, one of "network-interface", "network-load-balancer", "gateway-load-balancer-endpoint"
    #   resp.traffic_mirror_target.description #=> String
    #   resp.traffic_mirror_target.owner_id #=> String
    #   resp.traffic_mirror_target.tags #=> Array
    #   resp.traffic_mirror_target.tags[0].key #=> String
    #   resp.traffic_mirror_target.tags[0].value #=> String
    #   resp.traffic_mirror_target.gateway_load_balancer_endpoint_id #=> String
    #   resp.client_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTrafficMirrorTarget AWS API Documentation
    #
    # @overload create_traffic_mirror_target(params = {})
    # @param [Hash] params ({})
    def create_traffic_mirror_target(params = {}, options = {})
      req = build_request(:create_traffic_mirror_target, params)
      req.send_request(options)
    end

    # Creates a transit gateway.
    #
    # You can use a transit gateway to interconnect your virtual private
    # clouds (VPC) and on-premises networks. After the transit gateway
    # enters the `available` state, you can attach your VPCs and VPN
    # connections to the transit gateway.
    #
    # To attach your VPCs, use CreateTransitGatewayVpcAttachment.
    #
    # To attach a VPN connection, use CreateCustomerGateway to create a
    # customer gateway and specify the ID of the customer gateway and the ID
    # of the transit gateway in a call to CreateVpnConnection.
    #
    # When you create a transit gateway, we create a default transit gateway
    # route table and use it as the default association route table and the
    # default propagation route table. You can use
    # CreateTransitGatewayRouteTable to create additional transit gateway
    # route tables. If you disable automatic route propagation, we do not
    # create a default transit gateway route table. You can use
    # EnableTransitGatewayRouteTablePropagation to propagate routes from a
    # resource attachment to a transit gateway route table. If you disable
    # automatic associations, you can use AssociateTransitGatewayRouteTable
    # to associate a resource attachment with a transit gateway route table.
    #
    # @option params [String] :description
    #   A description of the transit gateway.
    #
    # @option params [Types::TransitGatewayRequestOptions] :options
    #   The transit gateway options.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the transit gateway.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateTransitGatewayResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTransitGatewayResult#transit_gateway #transit_gateway} => Types::TransitGateway
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_transit_gateway({
    #     description: "String",
    #     options: {
    #       amazon_side_asn: 1,
    #       auto_accept_shared_attachments: "enable", # accepts enable, disable
    #       default_route_table_association: "enable", # accepts enable, disable
    #       default_route_table_propagation: "enable", # accepts enable, disable
    #       vpn_ecmp_support: "enable", # accepts enable, disable
    #       dns_support: "enable", # accepts enable, disable
    #       security_group_referencing_support: "enable", # accepts enable, disable
    #       multicast_support: "enable", # accepts enable, disable
    #       transit_gateway_cidr_blocks: ["String"],
    #     },
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway.transit_gateway_id #=> String
    #   resp.transit_gateway.transit_gateway_arn #=> String
    #   resp.transit_gateway.state #=> String, one of "pending", "available", "modifying", "deleting", "deleted"
    #   resp.transit_gateway.owner_id #=> String
    #   resp.transit_gateway.description #=> String
    #   resp.transit_gateway.creation_time #=> Time
    #   resp.transit_gateway.options.amazon_side_asn #=> Integer
    #   resp.transit_gateway.options.transit_gateway_cidr_blocks #=> Array
    #   resp.transit_gateway.options.transit_gateway_cidr_blocks[0] #=> String
    #   resp.transit_gateway.options.auto_accept_shared_attachments #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.default_route_table_association #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.association_default_route_table_id #=> String
    #   resp.transit_gateway.options.default_route_table_propagation #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.propagation_default_route_table_id #=> String
    #   resp.transit_gateway.options.vpn_ecmp_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.dns_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.security_group_referencing_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.multicast_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway.tags #=> Array
    #   resp.transit_gateway.tags[0].key #=> String
    #   resp.transit_gateway.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTransitGateway AWS API Documentation
    #
    # @overload create_transit_gateway(params = {})
    # @param [Hash] params ({})
    def create_transit_gateway(params = {}, options = {})
      req = build_request(:create_transit_gateway, params)
      req.send_request(options)
    end

    # Creates a Connect attachment from a specified transit gateway
    # attachment. A Connect attachment is a GRE-based tunnel attachment that
    # you can use to establish a connection between a transit gateway and an
    # appliance.
    #
    # A Connect attachment uses an existing VPC or Amazon Web Services
    # Direct Connect attachment as the underlying transport mechanism.
    #
    # @option params [required, String] :transport_transit_gateway_attachment_id
    #   The ID of the transit gateway attachment. You can specify a VPC
    #   attachment or Amazon Web Services Direct Connect attachment.
    #
    # @option params [required, Types::CreateTransitGatewayConnectRequestOptions] :options
    #   The Connect attachment options.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the Connect attachment.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateTransitGatewayConnectResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTransitGatewayConnectResult#transit_gateway_connect #transit_gateway_connect} => Types::TransitGatewayConnect
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_transit_gateway_connect({
    #     transport_transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     options: { # required
    #       protocol: "gre", # required, accepts gre
    #     },
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_connect.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_connect.transport_transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_connect.transit_gateway_id #=> String
    #   resp.transit_gateway_connect.state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_connect.creation_time #=> Time
    #   resp.transit_gateway_connect.options.protocol #=> String, one of "gre"
    #   resp.transit_gateway_connect.tags #=> Array
    #   resp.transit_gateway_connect.tags[0].key #=> String
    #   resp.transit_gateway_connect.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTransitGatewayConnect AWS API Documentation
    #
    # @overload create_transit_gateway_connect(params = {})
    # @param [Hash] params ({})
    def create_transit_gateway_connect(params = {}, options = {})
      req = build_request(:create_transit_gateway_connect, params)
      req.send_request(options)
    end

    # Creates a Connect peer for a specified transit gateway Connect
    # attachment between a transit gateway and an appliance.
    #
    # The peer address and transit gateway address must be the same IP
    # address family (IPv4 or IPv6).
    #
    # For more information, see [Connect peers][1] in the *Amazon Web
    # Services Transit Gateways Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/tgw/tgw-connect.html#tgw-connect-peer
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the Connect attachment.
    #
    # @option params [String] :transit_gateway_address
    #   The peer IP address (GRE outer IP address) on the transit gateway side
    #   of the Connect peer, which must be specified from a transit gateway
    #   CIDR block. If not specified, Amazon automatically assigns the first
    #   available IP address from the transit gateway CIDR block.
    #
    # @option params [required, String] :peer_address
    #   The peer IP address (GRE outer IP address) on the appliance side of
    #   the Connect peer.
    #
    # @option params [Types::TransitGatewayConnectRequestBgpOptions] :bgp_options
    #   The BGP options for the Connect peer.
    #
    # @option params [required, Array<String>] :inside_cidr_blocks
    #   The range of inside IP addresses that are used for BGP peering. You
    #   must specify a size /29 IPv4 CIDR block from the `169.254.0.0/16`
    #   range. The first address from the range must be configured on the
    #   appliance as the BGP IP address. You can also optionally specify a
    #   size /125 IPv6 CIDR block from the `fd00::/8` range.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the Connect peer.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateTransitGatewayConnectPeerResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTransitGatewayConnectPeerResult#transit_gateway_connect_peer #transit_gateway_connect_peer} => Types::TransitGatewayConnectPeer
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_transit_gateway_connect_peer({
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     transit_gateway_address: "String",
    #     peer_address: "String", # required
    #     bgp_options: {
    #       peer_asn: 1,
    #     },
    #     inside_cidr_blocks: ["String"], # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_connect_peer.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_connect_peer.transit_gateway_connect_peer_id #=> String
    #   resp.transit_gateway_connect_peer.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.transit_gateway_connect_peer.creation_time #=> Time
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.transit_gateway_address #=> String
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.peer_address #=> String
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.inside_cidr_blocks #=> Array
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.inside_cidr_blocks[0] #=> String
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.protocol #=> String, one of "gre"
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.bgp_configurations #=> Array
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.bgp_configurations[0].transit_gateway_asn #=> Integer
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.bgp_configurations[0].peer_asn #=> Integer
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.bgp_configurations[0].transit_gateway_address #=> String
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.bgp_configurations[0].peer_address #=> String
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.bgp_configurations[0].bgp_status #=> String, one of "up", "down"
    #   resp.transit_gateway_connect_peer.tags #=> Array
    #   resp.transit_gateway_connect_peer.tags[0].key #=> String
    #   resp.transit_gateway_connect_peer.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTransitGatewayConnectPeer AWS API Documentation
    #
    # @overload create_transit_gateway_connect_peer(params = {})
    # @param [Hash] params ({})
    def create_transit_gateway_connect_peer(params = {}, options = {})
      req = build_request(:create_transit_gateway_connect_peer, params)
      req.send_request(options)
    end

    # Creates a multicast domain using the specified transit gateway.
    #
    # The transit gateway must be in the available state before you create a
    # domain. Use [DescribeTransitGateways][1] to see the state of transit
    # gateway.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeTransitGateways.html
    #
    # @option params [required, String] :transit_gateway_id
    #   The ID of the transit gateway.
    #
    # @option params [Types::CreateTransitGatewayMulticastDomainRequestOptions] :options
    #   The options for the transit gateway multicast domain.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags for the transit gateway multicast domain.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateTransitGatewayMulticastDomainResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTransitGatewayMulticastDomainResult#transit_gateway_multicast_domain #transit_gateway_multicast_domain} => Types::TransitGatewayMulticastDomain
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_transit_gateway_multicast_domain({
    #     transit_gateway_id: "TransitGatewayId", # required
    #     options: {
    #       igmpv_2_support: "enable", # accepts enable, disable
    #       static_sources_support: "enable", # accepts enable, disable
    #       auto_accept_shared_associations: "enable", # accepts enable, disable
    #     },
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_multicast_domain.transit_gateway_multicast_domain_id #=> String
    #   resp.transit_gateway_multicast_domain.transit_gateway_id #=> String
    #   resp.transit_gateway_multicast_domain.transit_gateway_multicast_domain_arn #=> String
    #   resp.transit_gateway_multicast_domain.owner_id #=> String
    #   resp.transit_gateway_multicast_domain.options.igmpv_2_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_multicast_domain.options.static_sources_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_multicast_domain.options.auto_accept_shared_associations #=> String, one of "enable", "disable"
    #   resp.transit_gateway_multicast_domain.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.transit_gateway_multicast_domain.creation_time #=> Time
    #   resp.transit_gateway_multicast_domain.tags #=> Array
    #   resp.transit_gateway_multicast_domain.tags[0].key #=> String
    #   resp.transit_gateway_multicast_domain.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTransitGatewayMulticastDomain AWS API Documentation
    #
    # @overload create_transit_gateway_multicast_domain(params = {})
    # @param [Hash] params ({})
    def create_transit_gateway_multicast_domain(params = {}, options = {})
      req = build_request(:create_transit_gateway_multicast_domain, params)
      req.send_request(options)
    end

    # Requests a transit gateway peering attachment between the specified
    # transit gateway (requester) and a peer transit gateway (accepter). The
    # peer transit gateway can be in your account or a different Amazon Web
    # Services account.
    #
    # After you create the peering attachment, the owner of the accepter
    # transit gateway must accept the attachment request.
    #
    # @option params [required, String] :transit_gateway_id
    #   The ID of the transit gateway.
    #
    # @option params [required, String] :peer_transit_gateway_id
    #   The ID of the peer transit gateway with which to create the peering
    #   attachment.
    #
    # @option params [required, String] :peer_account_id
    #   The ID of the Amazon Web Services account that owns the peer transit
    #   gateway.
    #
    # @option params [required, String] :peer_region
    #   The Region where the peer transit gateway is located.
    #
    # @option params [Types::CreateTransitGatewayPeeringAttachmentRequestOptions] :options
    #   Requests a transit gateway peering attachment.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the transit gateway peering attachment.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateTransitGatewayPeeringAttachmentResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTransitGatewayPeeringAttachmentResult#transit_gateway_peering_attachment #transit_gateway_peering_attachment} => Types::TransitGatewayPeeringAttachment
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_transit_gateway_peering_attachment({
    #     transit_gateway_id: "TransitGatewayId", # required
    #     peer_transit_gateway_id: "TransitAssociationGatewayId", # required
    #     peer_account_id: "String", # required
    #     peer_region: "String", # required
    #     options: {
    #       dynamic_routing: "enable", # accepts enable, disable
    #     },
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_peering_attachment.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.transit_gateway_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.core_network_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.owner_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.region #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.transit_gateway_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.core_network_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.owner_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.region #=> String
    #   resp.transit_gateway_peering_attachment.options.dynamic_routing #=> String, one of "enable", "disable"
    #   resp.transit_gateway_peering_attachment.status.code #=> String
    #   resp.transit_gateway_peering_attachment.status.message #=> String
    #   resp.transit_gateway_peering_attachment.state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_peering_attachment.creation_time #=> Time
    #   resp.transit_gateway_peering_attachment.tags #=> Array
    #   resp.transit_gateway_peering_attachment.tags[0].key #=> String
    #   resp.transit_gateway_peering_attachment.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTransitGatewayPeeringAttachment AWS API Documentation
    #
    # @overload create_transit_gateway_peering_attachment(params = {})
    # @param [Hash] params ({})
    def create_transit_gateway_peering_attachment(params = {}, options = {})
      req = build_request(:create_transit_gateway_peering_attachment, params)
      req.send_request(options)
    end

    # Creates a transit gateway policy table.
    #
    # @option params [required, String] :transit_gateway_id
    #   The ID of the transit gateway used for the policy table.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags specification for the transit gateway policy table created
    #   during the request.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateTransitGatewayPolicyTableResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTransitGatewayPolicyTableResult#transit_gateway_policy_table #transit_gateway_policy_table} => Types::TransitGatewayPolicyTable
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_transit_gateway_policy_table({
    #     transit_gateway_id: "TransitGatewayId", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_policy_table.transit_gateway_policy_table_id #=> String
    #   resp.transit_gateway_policy_table.transit_gateway_id #=> String
    #   resp.transit_gateway_policy_table.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.transit_gateway_policy_table.creation_time #=> Time
    #   resp.transit_gateway_policy_table.tags #=> Array
    #   resp.transit_gateway_policy_table.tags[0].key #=> String
    #   resp.transit_gateway_policy_table.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTransitGatewayPolicyTable AWS API Documentation
    #
    # @overload create_transit_gateway_policy_table(params = {})
    # @param [Hash] params ({})
    def create_transit_gateway_policy_table(params = {}, options = {})
      req = build_request(:create_transit_gateway_policy_table, params)
      req.send_request(options)
    end

    # Creates a reference (route) to a prefix list in a specified transit
    # gateway route table.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the transit gateway route table.
    #
    # @option params [required, String] :prefix_list_id
    #   The ID of the prefix list that is used for destination matches.
    #
    # @option params [String] :transit_gateway_attachment_id
    #   The ID of the attachment to which traffic is routed.
    #
    # @option params [Boolean] :blackhole
    #   Indicates whether to drop traffic that matches this route.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateTransitGatewayPrefixListReferenceResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTransitGatewayPrefixListReferenceResult#transit_gateway_prefix_list_reference #transit_gateway_prefix_list_reference} => Types::TransitGatewayPrefixListReference
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_transit_gateway_prefix_list_reference({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     prefix_list_id: "PrefixListResourceId", # required
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId",
    #     blackhole: false,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_prefix_list_reference.transit_gateway_route_table_id #=> String
    #   resp.transit_gateway_prefix_list_reference.prefix_list_id #=> String
    #   resp.transit_gateway_prefix_list_reference.prefix_list_owner_id #=> String
    #   resp.transit_gateway_prefix_list_reference.state #=> String, one of "pending", "available", "modifying", "deleting"
    #   resp.transit_gateway_prefix_list_reference.blackhole #=> Boolean
    #   resp.transit_gateway_prefix_list_reference.transit_gateway_attachment.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_prefix_list_reference.transit_gateway_attachment.resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.transit_gateway_prefix_list_reference.transit_gateway_attachment.resource_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTransitGatewayPrefixListReference AWS API Documentation
    #
    # @overload create_transit_gateway_prefix_list_reference(params = {})
    # @param [Hash] params ({})
    def create_transit_gateway_prefix_list_reference(params = {}, options = {})
      req = build_request(:create_transit_gateway_prefix_list_reference, params)
      req.send_request(options)
    end

    # Creates a static route for the specified transit gateway route table.
    #
    # @option params [required, String] :destination_cidr_block
    #   The CIDR range used for destination matches. Routing decisions are
    #   based on the most specific match.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the transit gateway route table.
    #
    # @option params [String] :transit_gateway_attachment_id
    #   The ID of the attachment.
    #
    # @option params [Boolean] :blackhole
    #   Indicates whether to drop traffic that matches this route.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateTransitGatewayRouteResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTransitGatewayRouteResult#route #route} => Types::TransitGatewayRoute
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_transit_gateway_route({
    #     destination_cidr_block: "String", # required
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId",
    #     blackhole: false,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.route.destination_cidr_block #=> String
    #   resp.route.prefix_list_id #=> String
    #   resp.route.transit_gateway_route_table_announcement_id #=> String
    #   resp.route.transit_gateway_attachments #=> Array
    #   resp.route.transit_gateway_attachments[0].resource_id #=> String
    #   resp.route.transit_gateway_attachments[0].transit_gateway_attachment_id #=> String
    #   resp.route.transit_gateway_attachments[0].resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.route.type #=> String, one of "static", "propagated"
    #   resp.route.state #=> String, one of "pending", "active", "blackhole", "deleting", "deleted"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTransitGatewayRoute AWS API Documentation
    #
    # @overload create_transit_gateway_route(params = {})
    # @param [Hash] params ({})
    def create_transit_gateway_route(params = {}, options = {})
      req = build_request(:create_transit_gateway_route, params)
      req.send_request(options)
    end

    # Creates a route table for the specified transit gateway.
    #
    # @option params [required, String] :transit_gateway_id
    #   The ID of the transit gateway.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the transit gateway route table.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateTransitGatewayRouteTableResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTransitGatewayRouteTableResult#transit_gateway_route_table #transit_gateway_route_table} => Types::TransitGatewayRouteTable
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_transit_gateway_route_table({
    #     transit_gateway_id: "TransitGatewayId", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_route_table.transit_gateway_route_table_id #=> String
    #   resp.transit_gateway_route_table.transit_gateway_id #=> String
    #   resp.transit_gateway_route_table.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.transit_gateway_route_table.default_association_route_table #=> Boolean
    #   resp.transit_gateway_route_table.default_propagation_route_table #=> Boolean
    #   resp.transit_gateway_route_table.creation_time #=> Time
    #   resp.transit_gateway_route_table.tags #=> Array
    #   resp.transit_gateway_route_table.tags[0].key #=> String
    #   resp.transit_gateway_route_table.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTransitGatewayRouteTable AWS API Documentation
    #
    # @overload create_transit_gateway_route_table(params = {})
    # @param [Hash] params ({})
    def create_transit_gateway_route_table(params = {}, options = {})
      req = build_request(:create_transit_gateway_route_table, params)
      req.send_request(options)
    end

    # Advertises a new transit gateway route table.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the transit gateway route table.
    #
    # @option params [required, String] :peering_attachment_id
    #   The ID of the peering attachment.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags specifications applied to the transit gateway route table
    #   announcement.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateTransitGatewayRouteTableAnnouncementResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTransitGatewayRouteTableAnnouncementResult#transit_gateway_route_table_announcement #transit_gateway_route_table_announcement} => Types::TransitGatewayRouteTableAnnouncement
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_transit_gateway_route_table_announcement({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     peering_attachment_id: "TransitGatewayAttachmentId", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_route_table_announcement.transit_gateway_route_table_announcement_id #=> String
    #   resp.transit_gateway_route_table_announcement.transit_gateway_id #=> String
    #   resp.transit_gateway_route_table_announcement.core_network_id #=> String
    #   resp.transit_gateway_route_table_announcement.peer_transit_gateway_id #=> String
    #   resp.transit_gateway_route_table_announcement.peer_core_network_id #=> String
    #   resp.transit_gateway_route_table_announcement.peering_attachment_id #=> String
    #   resp.transit_gateway_route_table_announcement.announcement_direction #=> String, one of "outgoing", "incoming"
    #   resp.transit_gateway_route_table_announcement.transit_gateway_route_table_id #=> String
    #   resp.transit_gateway_route_table_announcement.state #=> String, one of "available", "pending", "failing", "failed", "deleting", "deleted"
    #   resp.transit_gateway_route_table_announcement.creation_time #=> Time
    #   resp.transit_gateway_route_table_announcement.tags #=> Array
    #   resp.transit_gateway_route_table_announcement.tags[0].key #=> String
    #   resp.transit_gateway_route_table_announcement.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTransitGatewayRouteTableAnnouncement AWS API Documentation
    #
    # @overload create_transit_gateway_route_table_announcement(params = {})
    # @param [Hash] params ({})
    def create_transit_gateway_route_table_announcement(params = {}, options = {})
      req = build_request(:create_transit_gateway_route_table_announcement, params)
      req.send_request(options)
    end

    # Attaches the specified VPC to the specified transit gateway.
    #
    # If you attach a VPC with a CIDR range that overlaps the CIDR range of
    # a VPC that is already attached, the new VPC CIDR range is not
    # propagated to the default propagation route table.
    #
    # To send VPC traffic to an attached transit gateway, add a route to the
    # VPC route table using CreateRoute.
    #
    # @option params [required, String] :transit_gateway_id
    #   The ID of the transit gateway.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @option params [required, Array<String>] :subnet_ids
    #   The IDs of one or more subnets. You can specify only one subnet per
    #   Availability Zone. You must specify at least one subnet, but we
    #   recommend that you specify two subnets for better availability. The
    #   transit gateway uses one IP address from each specified subnet.
    #
    # @option params [Types::CreateTransitGatewayVpcAttachmentRequestOptions] :options
    #   The VPC attachment options.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the VPC attachment.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateTransitGatewayVpcAttachmentResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateTransitGatewayVpcAttachmentResult#transit_gateway_vpc_attachment #transit_gateway_vpc_attachment} => Types::TransitGatewayVpcAttachment
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_transit_gateway_vpc_attachment({
    #     transit_gateway_id: "TransitGatewayId", # required
    #     vpc_id: "VpcId", # required
    #     subnet_ids: ["SubnetId"], # required
    #     options: {
    #       dns_support: "enable", # accepts enable, disable
    #       security_group_referencing_support: "enable", # accepts enable, disable
    #       ipv_6_support: "enable", # accepts enable, disable
    #       appliance_mode_support: "enable", # accepts enable, disable
    #     },
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_vpc_attachment.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_vpc_attachment.transit_gateway_id #=> String
    #   resp.transit_gateway_vpc_attachment.vpc_id #=> String
    #   resp.transit_gateway_vpc_attachment.vpc_owner_id #=> String
    #   resp.transit_gateway_vpc_attachment.state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_vpc_attachment.subnet_ids #=> Array
    #   resp.transit_gateway_vpc_attachment.subnet_ids[0] #=> String
    #   resp.transit_gateway_vpc_attachment.creation_time #=> Time
    #   resp.transit_gateway_vpc_attachment.options.dns_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.security_group_referencing_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.ipv_6_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.appliance_mode_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.tags #=> Array
    #   resp.transit_gateway_vpc_attachment.tags[0].key #=> String
    #   resp.transit_gateway_vpc_attachment.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateTransitGatewayVpcAttachment AWS API Documentation
    #
    # @overload create_transit_gateway_vpc_attachment(params = {})
    # @param [Hash] params ({})
    def create_transit_gateway_vpc_attachment(params = {}, options = {})
      req = build_request(:create_transit_gateway_vpc_attachment, params)
      req.send_request(options)
    end

    # An Amazon Web Services Verified Access endpoint is where you define
    # your application along with an optional endpoint-level access policy.
    #
    # @option params [required, String] :verified_access_group_id
    #   The ID of the Verified Access group to associate the endpoint with.
    #
    # @option params [required, String] :endpoint_type
    #   The type of Verified Access endpoint to create.
    #
    # @option params [required, String] :attachment_type
    #   The type of attachment.
    #
    # @option params [String] :domain_certificate_arn
    #   The ARN of the public TLS/SSL certificate in Amazon Web Services
    #   Certificate Manager to associate with the endpoint. The CN in the
    #   certificate must match the DNS name your end users will use to reach
    #   your application.
    #
    # @option params [String] :application_domain
    #   The DNS name for users to reach your application.
    #
    # @option params [String] :endpoint_domain_prefix
    #   A custom identifier that is prepended to the DNS name that is
    #   generated for the endpoint.
    #
    # @option params [Array<String>] :security_group_ids
    #   The IDs of the security groups to associate with the Verified Access
    #   endpoint. Required if `AttachmentType` is set to `vpc`.
    #
    # @option params [Types::CreateVerifiedAccessEndpointLoadBalancerOptions] :load_balancer_options
    #   The load balancer details. This parameter is required if the endpoint
    #   type is `load-balancer`.
    #
    # @option params [Types::CreateVerifiedAccessEndpointEniOptions] :network_interface_options
    #   The network interface details. This parameter is required if the
    #   endpoint type is `network-interface`.
    #
    # @option params [String] :description
    #   A description for the Verified Access endpoint.
    #
    # @option params [String] :policy_document
    #   The Verified Access policy document.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the Verified Access endpoint.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Types::VerifiedAccessSseSpecificationRequest] :sse_specification
    #   The options for server side encryption.
    #
    # @option params [Types::CreateVerifiedAccessEndpointRdsOptions] :rds_options
    #   The RDS details. This parameter is required if the endpoint type is
    #   `rds`.
    #
    # @option params [Types::CreateVerifiedAccessEndpointCidrOptions] :cidr_options
    #   The CIDR options. This parameter is required if the endpoint type is
    #   `cidr`.
    #
    # @return [Types::CreateVerifiedAccessEndpointResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateVerifiedAccessEndpointResult#verified_access_endpoint #verified_access_endpoint} => Types::VerifiedAccessEndpoint
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_verified_access_endpoint({
    #     verified_access_group_id: "VerifiedAccessGroupId", # required
    #     endpoint_type: "load-balancer", # required, accepts load-balancer, network-interface, rds, cidr
    #     attachment_type: "vpc", # required, accepts vpc
    #     domain_certificate_arn: "CertificateArn",
    #     application_domain: "String",
    #     endpoint_domain_prefix: "String",
    #     security_group_ids: ["SecurityGroupId"],
    #     load_balancer_options: {
    #       protocol: "http", # accepts http, https, tcp
    #       port: 1,
    #       load_balancer_arn: "LoadBalancerArn",
    #       subnet_ids: ["SubnetId"],
    #       port_ranges: [
    #         {
    #           from_port: 1,
    #           to_port: 1,
    #         },
    #       ],
    #     },
    #     network_interface_options: {
    #       network_interface_id: "NetworkInterfaceId",
    #       protocol: "http", # accepts http, https, tcp
    #       port: 1,
    #       port_ranges: [
    #         {
    #           from_port: 1,
    #           to_port: 1,
    #         },
    #       ],
    #     },
    #     description: "String",
    #     policy_document: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String",
    #     dry_run: false,
    #     sse_specification: {
    #       customer_managed_key_enabled: false,
    #       kms_key_arn: "KmsKeyArn",
    #     },
    #     rds_options: {
    #       protocol: "http", # accepts http, https, tcp
    #       port: 1,
    #       rds_db_instance_arn: "RdsDbInstanceArn",
    #       rds_db_cluster_arn: "RdsDbClusterArn",
    #       rds_db_proxy_arn: "RdsDbProxyArn",
    #       rds_endpoint: "String",
    #       subnet_ids: ["SubnetId"],
    #     },
    #     cidr_options: {
    #       protocol: "http", # accepts http, https, tcp
    #       subnet_ids: ["SubnetId"],
    #       cidr: "String",
    #       port_ranges: [
    #         {
    #           from_port: 1,
    #           to_port: 1,
    #         },
    #       ],
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_endpoint.verified_access_instance_id #=> String
    #   resp.verified_access_endpoint.verified_access_group_id #=> String
    #   resp.verified_access_endpoint.verified_access_endpoint_id #=> String
    #   resp.verified_access_endpoint.application_domain #=> String
    #   resp.verified_access_endpoint.endpoint_type #=> String, one of "load-balancer", "network-interface", "rds", "cidr"
    #   resp.verified_access_endpoint.attachment_type #=> String, one of "vpc"
    #   resp.verified_access_endpoint.domain_certificate_arn #=> String
    #   resp.verified_access_endpoint.endpoint_domain #=> String
    #   resp.verified_access_endpoint.device_validation_domain #=> String
    #   resp.verified_access_endpoint.security_group_ids #=> Array
    #   resp.verified_access_endpoint.security_group_ids[0] #=> String
    #   resp.verified_access_endpoint.load_balancer_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoint.load_balancer_options.port #=> Integer
    #   resp.verified_access_endpoint.load_balancer_options.load_balancer_arn #=> String
    #   resp.verified_access_endpoint.load_balancer_options.subnet_ids #=> Array
    #   resp.verified_access_endpoint.load_balancer_options.subnet_ids[0] #=> String
    #   resp.verified_access_endpoint.load_balancer_options.port_ranges #=> Array
    #   resp.verified_access_endpoint.load_balancer_options.port_ranges[0].from_port #=> Integer
    #   resp.verified_access_endpoint.load_balancer_options.port_ranges[0].to_port #=> Integer
    #   resp.verified_access_endpoint.network_interface_options.network_interface_id #=> String
    #   resp.verified_access_endpoint.network_interface_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoint.network_interface_options.port #=> Integer
    #   resp.verified_access_endpoint.network_interface_options.port_ranges #=> Array
    #   resp.verified_access_endpoint.network_interface_options.port_ranges[0].from_port #=> Integer
    #   resp.verified_access_endpoint.network_interface_options.port_ranges[0].to_port #=> Integer
    #   resp.verified_access_endpoint.status.code #=> String, one of "pending", "active", "updating", "deleting", "deleted"
    #   resp.verified_access_endpoint.status.message #=> String
    #   resp.verified_access_endpoint.description #=> String
    #   resp.verified_access_endpoint.creation_time #=> String
    #   resp.verified_access_endpoint.last_updated_time #=> String
    #   resp.verified_access_endpoint.deletion_time #=> String
    #   resp.verified_access_endpoint.tags #=> Array
    #   resp.verified_access_endpoint.tags[0].key #=> String
    #   resp.verified_access_endpoint.tags[0].value #=> String
    #   resp.verified_access_endpoint.sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.verified_access_endpoint.sse_specification.kms_key_arn #=> String
    #   resp.verified_access_endpoint.rds_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoint.rds_options.port #=> Integer
    #   resp.verified_access_endpoint.rds_options.rds_db_instance_arn #=> String
    #   resp.verified_access_endpoint.rds_options.rds_db_cluster_arn #=> String
    #   resp.verified_access_endpoint.rds_options.rds_db_proxy_arn #=> String
    #   resp.verified_access_endpoint.rds_options.rds_endpoint #=> String
    #   resp.verified_access_endpoint.rds_options.subnet_ids #=> Array
    #   resp.verified_access_endpoint.rds_options.subnet_ids[0] #=> String
    #   resp.verified_access_endpoint.cidr_options.cidr #=> String
    #   resp.verified_access_endpoint.cidr_options.port_ranges #=> Array
    #   resp.verified_access_endpoint.cidr_options.port_ranges[0].from_port #=> Integer
    #   resp.verified_access_endpoint.cidr_options.port_ranges[0].to_port #=> Integer
    #   resp.verified_access_endpoint.cidr_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoint.cidr_options.subnet_ids #=> Array
    #   resp.verified_access_endpoint.cidr_options.subnet_ids[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateVerifiedAccessEndpoint AWS API Documentation
    #
    # @overload create_verified_access_endpoint(params = {})
    # @param [Hash] params ({})
    def create_verified_access_endpoint(params = {}, options = {})
      req = build_request(:create_verified_access_endpoint, params)
      req.send_request(options)
    end

    # An Amazon Web Services Verified Access group is a collection of Amazon
    # Web Services Verified Access endpoints who's associated applications
    # have similar security requirements. Each instance within a Verified
    # Access group shares an Verified Access policy. For example, you can
    # group all Verified Access instances associated with "sales"
    # applications together and use one common Verified Access policy.
    #
    # @option params [required, String] :verified_access_instance_id
    #   The ID of the Verified Access instance.
    #
    # @option params [String] :description
    #   A description for the Verified Access group.
    #
    # @option params [String] :policy_document
    #   The Verified Access policy document.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the Verified Access group.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Types::VerifiedAccessSseSpecificationRequest] :sse_specification
    #   The options for server side encryption.
    #
    # @return [Types::CreateVerifiedAccessGroupResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateVerifiedAccessGroupResult#verified_access_group #verified_access_group} => Types::VerifiedAccessGroup
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_verified_access_group({
    #     verified_access_instance_id: "VerifiedAccessInstanceId", # required
    #     description: "String",
    #     policy_document: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String",
    #     dry_run: false,
    #     sse_specification: {
    #       customer_managed_key_enabled: false,
    #       kms_key_arn: "KmsKeyArn",
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_group.verified_access_group_id #=> String
    #   resp.verified_access_group.verified_access_instance_id #=> String
    #   resp.verified_access_group.description #=> String
    #   resp.verified_access_group.owner #=> String
    #   resp.verified_access_group.verified_access_group_arn #=> String
    #   resp.verified_access_group.creation_time #=> String
    #   resp.verified_access_group.last_updated_time #=> String
    #   resp.verified_access_group.deletion_time #=> String
    #   resp.verified_access_group.tags #=> Array
    #   resp.verified_access_group.tags[0].key #=> String
    #   resp.verified_access_group.tags[0].value #=> String
    #   resp.verified_access_group.sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.verified_access_group.sse_specification.kms_key_arn #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateVerifiedAccessGroup AWS API Documentation
    #
    # @overload create_verified_access_group(params = {})
    # @param [Hash] params ({})
    def create_verified_access_group(params = {}, options = {})
      req = build_request(:create_verified_access_group, params)
      req.send_request(options)
    end

    # An Amazon Web Services Verified Access instance is a regional entity
    # that evaluates application requests and grants access only when your
    # security requirements are met.
    #
    # @option params [String] :description
    #   A description for the Verified Access instance.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the Verified Access instance.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Boolean] :fips_enabled
    #   Enable or disable support for Federal Information Processing Standards
    #   (FIPS) on the instance.
    #
    # @option params [String] :cidr_endpoints_custom_sub_domain
    #   The custom subdomain.
    #
    # @return [Types::CreateVerifiedAccessInstanceResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateVerifiedAccessInstanceResult#verified_access_instance #verified_access_instance} => Types::VerifiedAccessInstance
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_verified_access_instance({
    #     description: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String",
    #     dry_run: false,
    #     fips_enabled: false,
    #     cidr_endpoints_custom_sub_domain: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_instance.verified_access_instance_id #=> String
    #   resp.verified_access_instance.description #=> String
    #   resp.verified_access_instance.verified_access_trust_providers #=> Array
    #   resp.verified_access_instance.verified_access_trust_providers[0].verified_access_trust_provider_id #=> String
    #   resp.verified_access_instance.verified_access_trust_providers[0].description #=> String
    #   resp.verified_access_instance.verified_access_trust_providers[0].trust_provider_type #=> String, one of "user", "device"
    #   resp.verified_access_instance.verified_access_trust_providers[0].user_trust_provider_type #=> String, one of "iam-identity-center", "oidc"
    #   resp.verified_access_instance.verified_access_trust_providers[0].device_trust_provider_type #=> String, one of "jamf", "crowdstrike", "jumpcloud"
    #   resp.verified_access_instance.creation_time #=> String
    #   resp.verified_access_instance.last_updated_time #=> String
    #   resp.verified_access_instance.tags #=> Array
    #   resp.verified_access_instance.tags[0].key #=> String
    #   resp.verified_access_instance.tags[0].value #=> String
    #   resp.verified_access_instance.fips_enabled #=> Boolean
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.sub_domain #=> String
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.nameservers #=> Array
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.nameservers[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateVerifiedAccessInstance AWS API Documentation
    #
    # @overload create_verified_access_instance(params = {})
    # @param [Hash] params ({})
    def create_verified_access_instance(params = {}, options = {})
      req = build_request(:create_verified_access_instance, params)
      req.send_request(options)
    end

    # A trust provider is a third-party entity that creates, maintains, and
    # manages identity information for users and devices. When an
    # application request is made, the identity information sent by the
    # trust provider is evaluated by Verified Access before allowing or
    # denying the application request.
    #
    # @option params [required, String] :trust_provider_type
    #   The type of trust provider.
    #
    # @option params [String] :user_trust_provider_type
    #   The type of user-based trust provider. This parameter is required when
    #   the provider type is `user`.
    #
    # @option params [String] :device_trust_provider_type
    #   The type of device-based trust provider. This parameter is required
    #   when the provider type is `device`.
    #
    # @option params [Types::CreateVerifiedAccessTrustProviderOidcOptions] :oidc_options
    #   The options for a OpenID Connect-compatible user-identity trust
    #   provider. This parameter is required when the provider type is `user`.
    #
    # @option params [Types::CreateVerifiedAccessTrustProviderDeviceOptions] :device_options
    #   The options for a device-based trust provider. This parameter is
    #   required when the provider type is `device`.
    #
    # @option params [required, String] :policy_reference_name
    #   The identifier to be used when working with policy rules.
    #
    # @option params [String] :description
    #   A description for the Verified Access trust provider.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the Verified Access trust provider.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Types::VerifiedAccessSseSpecificationRequest] :sse_specification
    #   The options for server side encryption.
    #
    # @option params [Types::CreateVerifiedAccessNativeApplicationOidcOptions] :native_application_oidc_options
    #   The OpenID Connect (OIDC) options.
    #
    # @return [Types::CreateVerifiedAccessTrustProviderResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateVerifiedAccessTrustProviderResult#verified_access_trust_provider #verified_access_trust_provider} => Types::VerifiedAccessTrustProvider
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_verified_access_trust_provider({
    #     trust_provider_type: "user", # required, accepts user, device
    #     user_trust_provider_type: "iam-identity-center", # accepts iam-identity-center, oidc
    #     device_trust_provider_type: "jamf", # accepts jamf, crowdstrike, jumpcloud
    #     oidc_options: {
    #       issuer: "String",
    #       authorization_endpoint: "String",
    #       token_endpoint: "String",
    #       user_info_endpoint: "String",
    #       client_id: "String",
    #       client_secret: "ClientSecretType",
    #       scope: "String",
    #     },
    #     device_options: {
    #       tenant_id: "String",
    #       public_signing_key_url: "String",
    #     },
    #     policy_reference_name: "String", # required
    #     description: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String",
    #     dry_run: false,
    #     sse_specification: {
    #       customer_managed_key_enabled: false,
    #       kms_key_arn: "KmsKeyArn",
    #     },
    #     native_application_oidc_options: {
    #       public_signing_key_endpoint: "String",
    #       issuer: "String",
    #       authorization_endpoint: "String",
    #       token_endpoint: "String",
    #       user_info_endpoint: "String",
    #       client_id: "String",
    #       client_secret: "ClientSecretType",
    #       scope: "String",
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_trust_provider.verified_access_trust_provider_id #=> String
    #   resp.verified_access_trust_provider.description #=> String
    #   resp.verified_access_trust_provider.trust_provider_type #=> String, one of "user", "device"
    #   resp.verified_access_trust_provider.user_trust_provider_type #=> String, one of "iam-identity-center", "oidc"
    #   resp.verified_access_trust_provider.device_trust_provider_type #=> String, one of "jamf", "crowdstrike", "jumpcloud"
    #   resp.verified_access_trust_provider.oidc_options.issuer #=> String
    #   resp.verified_access_trust_provider.oidc_options.authorization_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.token_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.user_info_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.client_id #=> String
    #   resp.verified_access_trust_provider.oidc_options.client_secret #=> String
    #   resp.verified_access_trust_provider.oidc_options.scope #=> String
    #   resp.verified_access_trust_provider.device_options.tenant_id #=> String
    #   resp.verified_access_trust_provider.device_options.public_signing_key_url #=> String
    #   resp.verified_access_trust_provider.policy_reference_name #=> String
    #   resp.verified_access_trust_provider.creation_time #=> String
    #   resp.verified_access_trust_provider.last_updated_time #=> String
    #   resp.verified_access_trust_provider.tags #=> Array
    #   resp.verified_access_trust_provider.tags[0].key #=> String
    #   resp.verified_access_trust_provider.tags[0].value #=> String
    #   resp.verified_access_trust_provider.sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.verified_access_trust_provider.sse_specification.kms_key_arn #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.public_signing_key_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.issuer #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.authorization_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.token_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.user_info_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.client_id #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.scope #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateVerifiedAccessTrustProvider AWS API Documentation
    #
    # @overload create_verified_access_trust_provider(params = {})
    # @param [Hash] params ({})
    def create_verified_access_trust_provider(params = {}, options = {})
      req = build_request(:create_verified_access_trust_provider, params)
      req.send_request(options)
    end

    # Creates an EBS volume that can be attached to an instance in the same
    # Availability Zone.
    #
    # You can create a new empty volume or restore a volume from an EBS
    # snapshot. Any Amazon Web Services Marketplace product codes from the
    # snapshot are propagated to the volume.
    #
    # You can create encrypted volumes. Encrypted volumes must be attached
    # to instances that support Amazon EBS encryption. Volumes that are
    # created from encrypted snapshots are also automatically encrypted. For
    # more information, see [Amazon EBS encryption][1] in the *Amazon EBS
    # User Guide*.
    #
    # You can tag your volumes during creation. For more information, see
    # [Tag your Amazon EC2 resources][2] in the *Amazon EC2 User Guide*.
    #
    # For more information, see [Create an Amazon EBS volume][3] in the
    # *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-encryption.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html
    # [3]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-creating-volume.html
    #
    # @option params [required, String] :availability_zone
    #   The ID of the Availability Zone in which to create the volume. For
    #   example, `us-east-1a`.
    #
    # @option params [Boolean] :encrypted
    #   Indicates whether the volume should be encrypted. The effect of
    #   setting the encryption state to `true` depends on the volume origin
    #   (new or from a snapshot), starting encryption state, ownership, and
    #   whether encryption by default is enabled. For more information, see
    #   [Encryption by default][1] in the *Amazon EBS User Guide*.
    #
    #   Encrypted Amazon EBS volumes must be attached to instances that
    #   support Amazon EBS encryption. For more information, see [Supported
    #   instance types][2].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ebs/latest/userguide/work-with-ebs-encr.html#encryption-by-default
    #   [2]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-encryption-requirements.html#ebs-encryption_supported_instances
    #
    # @option params [Integer] :iops
    #   The number of I/O operations per second (IOPS). For `gp3`, `io1`, and
    #   `io2` volumes, this represents the number of IOPS that are provisioned
    #   for the volume. For `gp2` volumes, this represents the baseline
    #   performance of the volume and the rate at which the volume accumulates
    #   I/O credits for bursting.
    #
    #   The following are the supported values for each volume type:
    #
    #   * `gp3`: 3,000 - 16,000 IOPS
    #
    #   * `io1`: 100 - 64,000 IOPS
    #
    #   * `io2`: 100 - 256,000 IOPS
    #
    #   For `io2` volumes, you can achieve up to 256,000 IOPS on [instances
    #   built on the Nitro System][1]. On other instances, you can achieve
    #   performance up to 32,000 IOPS.
    #
    #   This parameter is required for `io1` and `io2` volumes. The default
    #   for `gp3` volumes is 3,000 IOPS. This parameter is not supported for
    #   `gp2`, `st1`, `sc1`, or `standard` volumes.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/instancetypes/ec2-nitro-instances.html
    #
    # @option params [String] :kms_key_id
    #   The identifier of the KMS key to use for Amazon EBS encryption. If
    #   this parameter is not specified, your KMS key for Amazon EBS is used.
    #   If `KmsKeyId` is specified, the encrypted state must be `true`.
    #
    #   You can specify the KMS key using any of the following:
    #
    #   * Key ID. For example, 1234abcd-12ab-34cd-56ef-1234567890ab.
    #
    #   * Key alias. For example, alias/ExampleAlias.
    #
    #   * Key ARN. For example,
    #     arn:aws:kms:us-east-1:012345678910:key/1234abcd-12ab-34cd-56ef-1234567890ab.
    #
    #   * Alias ARN. For example,
    #     arn:aws:kms:us-east-1:012345678910:alias/ExampleAlias.
    #
    #   Amazon Web Services authenticates the KMS key asynchronously.
    #   Therefore, if you specify an ID, alias, or ARN that is not valid, the
    #   action can appear to complete, but eventually fails.
    #
    # @option params [String] :outpost_arn
    #   The Amazon Resource Name (ARN) of the Outpost on which to create the
    #   volume.
    #
    #   If you intend to use a volume with an instance running on an outpost,
    #   then you must create the volume on the same outpost as the instance.
    #   You can't use a volume created in an Amazon Web Services Region with
    #   an instance on an Amazon Web Services outpost, or the other way
    #   around.
    #
    # @option params [Integer] :size
    #   The size of the volume, in GiBs. You must specify either a snapshot ID
    #   or a volume size. If you specify a snapshot, the default is the
    #   snapshot size. You can specify a volume size that is equal to or
    #   larger than the snapshot size.
    #
    #   The following are the supported volumes sizes for each volume type:
    #
    #   * `gp2` and `gp3`: 1 - 16,384 GiB
    #
    #   * `io1`: 4 - 16,384 GiB
    #
    #   * `io2`: 4 - 65,536 GiB
    #
    #   * `st1` and `sc1`: 125 - 16,384 GiB
    #
    #   * `standard`: 1 - 1024 GiB
    #
    # @option params [String] :snapshot_id
    #   The snapshot from which to create the volume. You must specify either
    #   a snapshot ID or a volume size.
    #
    # @option params [String] :volume_type
    #   The volume type. This parameter can be one of the following values:
    #
    #   * General Purpose SSD: `gp2` \| `gp3`
    #
    #   * Provisioned IOPS SSD: `io1` \| `io2`
    #
    #   * Throughput Optimized HDD: `st1`
    #
    #   * Cold HDD: `sc1`
    #
    #   * Magnetic: `standard`
    #
    #   Throughput Optimized HDD (`st1`) and Cold HDD (`sc1`) volumes can't
    #   be used as boot volumes.
    #
    #   For more information, see [Amazon EBS volume types][1] in the *Amazon
    #   EBS User Guide*.
    #
    #   Default: `gp2`
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-volume-types.html
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the volume during creation.
    #
    # @option params [Boolean] :multi_attach_enabled
    #   Indicates whether to enable Amazon EBS Multi-Attach. If you enable
    #   Multi-Attach, you can attach the volume to up to 16 [Instances built
    #   on the Nitro System][1] in the same Availability Zone. This parameter
    #   is supported with `io1` and `io2` volumes only. For more information,
    #   see [ Amazon EBS Multi-Attach][2] in the *Amazon EBS User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/instancetypes/ec2-nitro-instances.html
    #   [2]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-volumes-multi.html
    #
    # @option params [Integer] :throughput
    #   The throughput to provision for a volume, with a maximum of 1,000
    #   MiB/s.
    #
    #   This parameter is valid only for `gp3` volumes.
    #
    #   Valid Range: Minimum value of 125. Maximum value of 1000.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensure
    #   Idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Types::OperatorRequest] :operator
    #   Reserved for internal use.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::Volume] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::Volume#outpost_arn #outpost_arn} => String
    #   * {Types::Volume#iops #iops} => Integer
    #   * {Types::Volume#tags #tags} => Array&lt;Types::Tag&gt;
    #   * {Types::Volume#volume_type #volume_type} => String
    #   * {Types::Volume#fast_restored #fast_restored} => Boolean
    #   * {Types::Volume#multi_attach_enabled #multi_attach_enabled} => Boolean
    #   * {Types::Volume#throughput #throughput} => Integer
    #   * {Types::Volume#sse_type #sse_type} => String
    #   * {Types::Volume#operator #operator} => Types::OperatorResponse
    #   * {Types::Volume#volume_id #volume_id} => String
    #   * {Types::Volume#size #size} => Integer
    #   * {Types::Volume#snapshot_id #snapshot_id} => String
    #   * {Types::Volume#availability_zone #availability_zone} => String
    #   * {Types::Volume#state #state} => String
    #   * {Types::Volume#create_time #create_time} => Time
    #   * {Types::Volume#attachments #attachments} => Array&lt;Types::VolumeAttachment&gt;
    #   * {Types::Volume#encrypted #encrypted} => Boolean
    #   * {Types::Volume#kms_key_id #kms_key_id} => String
    #
    #
    # @example Example: To create a new volume
    #
    #   # This example creates an 80 GiB General Purpose (SSD) volume in the Availability Zone ``us-east-1a``.
    #
    #   resp = client.create_volume({
    #     availability_zone: "us-east-1a", 
    #     size: 80, 
    #     volume_type: "gp2", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     availability_zone: "us-east-1a", 
    #     create_time: Time.parse("2016-08-29T18:52:32.724Z"), 
    #     encrypted: false, 
    #     iops: 240, 
    #     size: 80, 
    #     snapshot_id: "", 
    #     state: "creating", 
    #     volume_id: "vol-6b60b7c7", 
    #     volume_type: "gp2", 
    #   }
    #
    # @example Example: To create a new Provisioned IOPS (SSD) volume from a snapshot
    #
    #   # This example creates a new Provisioned IOPS (SSD) volume with 1000 provisioned IOPS from a snapshot in the Availability
    #   # Zone ``us-east-1a``.
    #
    #   resp = client.create_volume({
    #     availability_zone: "us-east-1a", 
    #     iops: 1000, 
    #     snapshot_id: "snap-066877671789bd71b", 
    #     volume_type: "io1", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     attachments: [
    #     ], 
    #     availability_zone: "us-east-1a", 
    #     create_time: Time.parse("2016-08-29T18:52:32.724Z"), 
    #     iops: 1000, 
    #     size: 500, 
    #     snapshot_id: "snap-066877671789bd71b", 
    #     state: "creating", 
    #     tags: [
    #     ], 
    #     volume_id: "vol-1234567890abcdef0", 
    #     volume_type: "io1", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_volume({
    #     availability_zone: "AvailabilityZoneName", # required
    #     encrypted: false,
    #     iops: 1,
    #     kms_key_id: "KmsKeyId",
    #     outpost_arn: "String",
    #     size: 1,
    #     snapshot_id: "SnapshotId",
    #     volume_type: "standard", # accepts standard, io1, io2, gp2, sc1, st1, gp3
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     multi_attach_enabled: false,
    #     throughput: 1,
    #     client_token: "String",
    #     operator: {
    #       principal: "String",
    #     },
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.outpost_arn #=> String
    #   resp.iops #=> Integer
    #   resp.tags #=> Array
    #   resp.tags[0].key #=> String
    #   resp.tags[0].value #=> String
    #   resp.volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.fast_restored #=> Boolean
    #   resp.multi_attach_enabled #=> Boolean
    #   resp.throughput #=> Integer
    #   resp.sse_type #=> String, one of "sse-ebs", "sse-kms", "none"
    #   resp.operator.managed #=> Boolean
    #   resp.operator.principal #=> String
    #   resp.volume_id #=> String
    #   resp.size #=> Integer
    #   resp.snapshot_id #=> String
    #   resp.availability_zone #=> String
    #   resp.state #=> String, one of "creating", "available", "in-use", "deleting", "deleted", "error"
    #   resp.create_time #=> Time
    #   resp.attachments #=> Array
    #   resp.attachments[0].delete_on_termination #=> Boolean
    #   resp.attachments[0].associated_resource #=> String
    #   resp.attachments[0].instance_owning_service #=> String
    #   resp.attachments[0].volume_id #=> String
    #   resp.attachments[0].instance_id #=> String
    #   resp.attachments[0].device #=> String
    #   resp.attachments[0].state #=> String, one of "attaching", "attached", "detaching", "detached", "busy"
    #   resp.attachments[0].attach_time #=> Time
    #   resp.encrypted #=> Boolean
    #   resp.kms_key_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateVolume AWS API Documentation
    #
    # @overload create_volume(params = {})
    # @param [Hash] params ({})
    def create_volume(params = {}, options = {})
      req = build_request(:create_volume, params)
      req.send_request(options)
    end

    # Creates a VPC with the specified CIDR blocks. For more information,
    # see [IP addressing for your VPCs and subnets][1] in the *Amazon VPC
    # User Guide*.
    #
    # You can optionally request an IPv6 CIDR block for the VPC. You can
    # request an Amazon-provided IPv6 CIDR block from Amazon's pool of IPv6
    # addresses or an IPv6 CIDR block from an IPv6 address pool that you
    # provisioned through bring your own IP addresses ([BYOIP][2]).
    #
    # By default, each instance that you launch in the VPC has the default
    # DHCP options, which include only a default DNS server that we provide
    # (AmazonProvidedDNS). For more information, see [DHCP option sets][3]
    # in the *Amazon VPC User Guide*.
    #
    # You can specify the instance tenancy value for the VPC when you create
    # it. You can't change this value for the VPC after you create it. For
    # more information, see [Dedicated Instances][4] in the *Amazon EC2 User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/vpc-ip-addressing.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-byoip.html
    # [3]: https://docs.aws.amazon.com/vpc/latest/userguide/VPC_DHCP_Options.html
    # [4]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-instance.html
    #
    # @option params [String] :cidr_block
    #   The IPv4 network range for the VPC, in CIDR notation. For example,
    #   `10.0.0.0/16`. We modify the specified CIDR block to its canonical
    #   form; for example, if you specify `100.68.0.18/18`, we modify it to
    #   `100.68.0.0/18`.
    #
    # @option params [String] :ipv_6_pool
    #   The ID of an IPv6 address pool from which to allocate the IPv6 CIDR
    #   block.
    #
    # @option params [String] :ipv_6_cidr_block
    #   The IPv6 CIDR block from the IPv6 address pool. You must also specify
    #   `Ipv6Pool` in the request.
    #
    #   To let Amazon choose the IPv6 CIDR block for you, omit this parameter.
    #
    # @option params [String] :ipv_4_ipam_pool_id
    #   The ID of an IPv4 IPAM pool you want to use for allocating this VPC's
    #   CIDR. For more information, see [What is IPAM?][1] in the *Amazon VPC
    #   IPAM User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/what-is-it-ipam.html
    #
    # @option params [Integer] :ipv_4_netmask_length
    #   The netmask length of the IPv4 CIDR you want to allocate to this VPC
    #   from an Amazon VPC IP Address Manager (IPAM) pool. For more
    #   information about IPAM, see [What is IPAM?][1] in the *Amazon VPC IPAM
    #   User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/what-is-it-ipam.html
    #
    # @option params [String] :ipv_6_ipam_pool_id
    #   The ID of an IPv6 IPAM pool which will be used to allocate this VPC an
    #   IPv6 CIDR. IPAM is a VPC feature that you can use to automate your IP
    #   address management workflows including assigning, tracking,
    #   troubleshooting, and auditing IP addresses across Amazon Web Services
    #   Regions and accounts throughout your Amazon Web Services Organization.
    #   For more information, see [What is IPAM?][1] in the *Amazon VPC IPAM
    #   User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/what-is-it-ipam.html
    #
    # @option params [Integer] :ipv_6_netmask_length
    #   The netmask length of the IPv6 CIDR you want to allocate to this VPC
    #   from an Amazon VPC IP Address Manager (IPAM) pool. For more
    #   information about IPAM, see [What is IPAM?][1] in the *Amazon VPC IPAM
    #   User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/what-is-it-ipam.html
    #
    # @option params [String] :ipv_6_cidr_block_network_border_group
    #   The name of the location from which we advertise the IPV6 CIDR block.
    #   Use this parameter to limit the address to this location.
    #
    #   You must set `AmazonProvidedIpv6CidrBlock` to `true` to use this
    #   parameter.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the VPC.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :instance_tenancy
    #   The tenancy options for instances launched into the VPC. For
    #   `default`, instances are launched with shared tenancy by default. You
    #   can launch instances with any tenancy into a shared tenancy VPC. For
    #   `dedicated`, instances are launched as dedicated tenancy instances by
    #   default. You can only launch instances with a tenancy of `dedicated`
    #   or `host` into a dedicated tenancy VPC.
    #
    #   **Important:** The `host` value cannot be used with this parameter.
    #   Use the `default` or `dedicated` values only.
    #
    #   Default: `default`
    #
    # @option params [Boolean] :amazon_provided_ipv_6_cidr_block
    #   Requests an Amazon-provided IPv6 CIDR block with a /56 prefix length
    #   for the VPC. You cannot specify the range of IP addresses, or the size
    #   of the CIDR block.
    #
    # @return [Types::CreateVpcResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateVpcResult#vpc #vpc} => Types::Vpc
    #
    #
    # @example Example: To create a VPC
    #
    #   # This example creates a VPC with the specified CIDR block.
    #
    #   resp = client.create_vpc({
    #     cidr_block: "10.0.0.0/16", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     vpc: {
    #       cidr_block: "10.0.0.0/16", 
    #       dhcp_options_id: "dopt-7a8b9c2d", 
    #       instance_tenancy: "default", 
    #       state: "pending", 
    #       vpc_id: "vpc-a01106c2", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_vpc({
    #     cidr_block: "String",
    #     ipv_6_pool: "Ipv6PoolEc2Id",
    #     ipv_6_cidr_block: "String",
    #     ipv_4_ipam_pool_id: "IpamPoolId",
    #     ipv_4_netmask_length: 1,
    #     ipv_6_ipam_pool_id: "IpamPoolId",
    #     ipv_6_netmask_length: 1,
    #     ipv_6_cidr_block_network_border_group: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #     instance_tenancy: "default", # accepts default, dedicated, host
    #     amazon_provided_ipv_6_cidr_block: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc.owner_id #=> String
    #   resp.vpc.instance_tenancy #=> String, one of "default", "dedicated", "host"
    #   resp.vpc.ipv_6_cidr_block_association_set #=> Array
    #   resp.vpc.ipv_6_cidr_block_association_set[0].association_id #=> String
    #   resp.vpc.ipv_6_cidr_block_association_set[0].ipv_6_cidr_block #=> String
    #   resp.vpc.ipv_6_cidr_block_association_set[0].ipv_6_cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.vpc.ipv_6_cidr_block_association_set[0].ipv_6_cidr_block_state.status_message #=> String
    #   resp.vpc.ipv_6_cidr_block_association_set[0].network_border_group #=> String
    #   resp.vpc.ipv_6_cidr_block_association_set[0].ipv_6_pool #=> String
    #   resp.vpc.ipv_6_cidr_block_association_set[0].ipv_6_address_attribute #=> String, one of "public", "private"
    #   resp.vpc.ipv_6_cidr_block_association_set[0].ip_source #=> String, one of "amazon", "byoip", "none"
    #   resp.vpc.cidr_block_association_set #=> Array
    #   resp.vpc.cidr_block_association_set[0].association_id #=> String
    #   resp.vpc.cidr_block_association_set[0].cidr_block #=> String
    #   resp.vpc.cidr_block_association_set[0].cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.vpc.cidr_block_association_set[0].cidr_block_state.status_message #=> String
    #   resp.vpc.is_default #=> Boolean
    #   resp.vpc.encryption_control.vpc_id #=> String
    #   resp.vpc.encryption_control.vpc_encryption_control_id #=> String
    #   resp.vpc.encryption_control.mode #=> String, one of "monitor", "enforce"
    #   resp.vpc.encryption_control.state #=> String, one of "enforce-in-progress", "monitor-in-progress", "enforce-failed", "monitor-failed", "deleting", "deleted", "available"
    #   resp.vpc.encryption_control.state_message #=> String
    #   resp.vpc.encryption_control.resource_exclusions.internet_gateway.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpc.encryption_control.resource_exclusions.internet_gateway.state_message #=> String
    #   resp.vpc.encryption_control.resource_exclusions.egress_only_internet_gateway.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpc.encryption_control.resource_exclusions.egress_only_internet_gateway.state_message #=> String
    #   resp.vpc.encryption_control.resource_exclusions.nat_gateway.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpc.encryption_control.resource_exclusions.nat_gateway.state_message #=> String
    #   resp.vpc.encryption_control.resource_exclusions.virtual_private_gateway.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpc.encryption_control.resource_exclusions.virtual_private_gateway.state_message #=> String
    #   resp.vpc.encryption_control.resource_exclusions.vpc_peering.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpc.encryption_control.resource_exclusions.vpc_peering.state_message #=> String
    #   resp.vpc.encryption_control.tags #=> Array
    #   resp.vpc.encryption_control.tags[0].key #=> String
    #   resp.vpc.encryption_control.tags[0].value #=> String
    #   resp.vpc.tags #=> Array
    #   resp.vpc.tags[0].key #=> String
    #   resp.vpc.tags[0].value #=> String
    #   resp.vpc.block_public_access_states.internet_gateway_block_mode #=> String, one of "off", "block-bidirectional", "block-ingress"
    #   resp.vpc.vpc_id #=> String
    #   resp.vpc.state #=> String, one of "pending", "available"
    #   resp.vpc.cidr_block #=> String
    #   resp.vpc.dhcp_options_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateVpc AWS API Documentation
    #
    # @overload create_vpc(params = {})
    # @param [Hash] params ({})
    def create_vpc(params = {}, options = {})
      req = build_request(:create_vpc, params)
      req.send_request(options)
    end

    # Create a VPC Block Public Access (BPA) exclusion. A VPC BPA exclusion
    # is a mode that can be applied to a single VPC or subnet that exempts
    # it from the account’s BPA mode and will allow bidirectional or
    # egress-only access. You can create BPA exclusions for VPCs and subnets
    # even when BPA is not enabled on the account to ensure that there is no
    # traffic disruption to the exclusions when VPC BPA is turned on. To
    # learn more about VPC BPA, see [Block public access to VPCs and
    # subnets][1] in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/security-vpc-bpa.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :subnet_id
    #   A subnet ID.
    #
    # @option params [String] :vpc_id
    #   A VPC ID.
    #
    # @option params [required, String] :internet_gateway_exclusion_mode
    #   The exclusion mode for internet gateway traffic.
    #
    #   * `allow-bidirectional`: Allow all internet traffic to and from the
    #     excluded VPCs and subnets.
    #
    #   * `allow-egress`: Allow outbound internet traffic from the excluded
    #     VPCs and subnets. Block inbound internet traffic to the excluded
    #     VPCs and subnets. Only applies when VPC Block Public Access is set
    #     to Bidirectional.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   `tag` - The key/value combination of a tag assigned to the resource.
    #   Use the tag key in the filter name and the tag value as the filter
    #   value. For example, to find all resources that have a tag with the key
    #   `Owner` and the value `TeamA`, specify `tag:Owner` for the filter name
    #   and `TeamA` for the filter value.
    #
    # @return [Types::CreateVpcBlockPublicAccessExclusionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateVpcBlockPublicAccessExclusionResult#vpc_block_public_access_exclusion #vpc_block_public_access_exclusion} => Types::VpcBlockPublicAccessExclusion
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_vpc_block_public_access_exclusion({
    #     dry_run: false,
    #     subnet_id: "SubnetId",
    #     vpc_id: "VpcId",
    #     internet_gateway_exclusion_mode: "allow-bidirectional", # required, accepts allow-bidirectional, allow-egress
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc_block_public_access_exclusion.exclusion_id #=> String
    #   resp.vpc_block_public_access_exclusion.internet_gateway_exclusion_mode #=> String, one of "allow-bidirectional", "allow-egress"
    #   resp.vpc_block_public_access_exclusion.resource_arn #=> String
    #   resp.vpc_block_public_access_exclusion.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "update-in-progress", "update-complete", "update-failed", "delete-in-progress", "delete-complete", "disable-in-progress", "disable-complete"
    #   resp.vpc_block_public_access_exclusion.reason #=> String
    #   resp.vpc_block_public_access_exclusion.creation_timestamp #=> Time
    #   resp.vpc_block_public_access_exclusion.last_update_timestamp #=> Time
    #   resp.vpc_block_public_access_exclusion.deletion_timestamp #=> Time
    #   resp.vpc_block_public_access_exclusion.tags #=> Array
    #   resp.vpc_block_public_access_exclusion.tags[0].key #=> String
    #   resp.vpc_block_public_access_exclusion.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateVpcBlockPublicAccessExclusion AWS API Documentation
    #
    # @overload create_vpc_block_public_access_exclusion(params = {})
    # @param [Hash] params ({})
    def create_vpc_block_public_access_exclusion(params = {}, options = {})
      req = build_request(:create_vpc_block_public_access_exclusion, params)
      req.send_request(options)
    end

    # Creates a VPC endpoint. A VPC endpoint provides a private connection
    # between the specified VPC and the specified endpoint service. You can
    # use an endpoint service provided by Amazon Web Services, an Amazon Web
    # Services Marketplace Partner, or another Amazon Web Services account.
    # For more information, see the [Amazon Web Services PrivateLink User
    # Guide][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/privatelink/
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :vpc_endpoint_type
    #   The type of endpoint.
    #
    #   Default: Gateway
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @option params [String] :service_name
    #   The name of the endpoint service.
    #
    # @option params [String] :policy_document
    #   (Interface and gateway endpoints) A policy to attach to the endpoint
    #   that controls access to the service. The policy must be in valid JSON
    #   format. If this parameter is not specified, we attach a default policy
    #   that allows full access to the service.
    #
    # @option params [Array<String>] :route_table_ids
    #   (Gateway endpoint) The route table IDs.
    #
    # @option params [Array<String>] :subnet_ids
    #   (Interface and Gateway Load Balancer endpoints) The IDs of the subnets
    #   in which to create endpoint network interfaces. For a Gateway Load
    #   Balancer endpoint, you can specify only one subnet.
    #
    # @option params [Array<String>] :security_group_ids
    #   (Interface endpoint) The IDs of the security groups to associate with
    #   the endpoint network interfaces. If this parameter is not specified,
    #   we use the default security group for the VPC.
    #
    # @option params [String] :ip_address_type
    #   The IP address type for the endpoint.
    #
    # @option params [Types::DnsOptionsSpecification] :dns_options
    #   The DNS options for the endpoint.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [How to ensure
    #   idempotency][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :private_dns_enabled
    #   (Interface endpoint) Indicates whether to associate a private hosted
    #   zone with the specified VPC. The private hosted zone contains a record
    #   set for the default public DNS name for the service for the Region
    #   (for example, `kinesis.us-east-1.amazonaws.com`), which resolves to
    #   the private IP addresses of the endpoint network interfaces in the
    #   VPC. This enables you to make requests to the default public DNS name
    #   for the service instead of the public DNS names that are automatically
    #   generated by the VPC endpoint service.
    #
    #   To use a private hosted zone, you must set the following VPC
    #   attributes to `true`: `enableDnsHostnames` and `enableDnsSupport`. Use
    #   ModifyVpcAttribute to set the VPC attributes.
    #
    #   Default: `true`
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to associate with the endpoint.
    #
    # @option params [Array<Types::SubnetConfiguration>] :subnet_configurations
    #   The subnet configurations for the endpoint.
    #
    # @option params [String] :service_network_arn
    #   The Amazon Resource Name (ARN) of a service network that will be
    #   associated with the VPC endpoint of type service-network.
    #
    # @option params [String] :resource_configuration_arn
    #   The Amazon Resource Name (ARN) of a resource configuration that will
    #   be associated with the VPC endpoint of type resource.
    #
    # @option params [String] :service_region
    #   The Region where the service is hosted. The default is the current
    #   Region.
    #
    # @return [Types::CreateVpcEndpointResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateVpcEndpointResult#vpc_endpoint #vpc_endpoint} => Types::VpcEndpoint
    #   * {Types::CreateVpcEndpointResult#client_token #client_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_vpc_endpoint({
    #     dry_run: false,
    #     vpc_endpoint_type: "Interface", # accepts Interface, Gateway, GatewayLoadBalancer, Resource, ServiceNetwork
    #     vpc_id: "VpcId", # required
    #     service_name: "String",
    #     policy_document: "String",
    #     route_table_ids: ["RouteTableId"],
    #     subnet_ids: ["SubnetId"],
    #     security_group_ids: ["SecurityGroupId"],
    #     ip_address_type: "ipv4", # accepts ipv4, dualstack, ipv6
    #     dns_options: {
    #       dns_record_ip_type: "ipv4", # accepts ipv4, dualstack, ipv6, service-defined
    #       private_dns_only_for_inbound_resolver_endpoint: false,
    #     },
    #     client_token: "String",
    #     private_dns_enabled: false,
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     subnet_configurations: [
    #       {
    #         subnet_id: "SubnetId",
    #         ipv_4: "String",
    #         ipv_6: "String",
    #       },
    #     ],
    #     service_network_arn: "ServiceNetworkArn",
    #     resource_configuration_arn: "ResourceConfigurationArn",
    #     service_region: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc_endpoint.vpc_endpoint_id #=> String
    #   resp.vpc_endpoint.vpc_endpoint_type #=> String, one of "Interface", "Gateway", "GatewayLoadBalancer", "Resource", "ServiceNetwork"
    #   resp.vpc_endpoint.vpc_id #=> String
    #   resp.vpc_endpoint.service_name #=> String
    #   resp.vpc_endpoint.state #=> String, one of "PendingAcceptance", "Pending", "Available", "Deleting", "Deleted", "Rejected", "Failed", "Expired", "Partial"
    #   resp.vpc_endpoint.policy_document #=> String
    #   resp.vpc_endpoint.route_table_ids #=> Array
    #   resp.vpc_endpoint.route_table_ids[0] #=> String
    #   resp.vpc_endpoint.subnet_ids #=> Array
    #   resp.vpc_endpoint.subnet_ids[0] #=> String
    #   resp.vpc_endpoint.groups #=> Array
    #   resp.vpc_endpoint.groups[0].group_id #=> String
    #   resp.vpc_endpoint.groups[0].group_name #=> String
    #   resp.vpc_endpoint.ip_address_type #=> String, one of "ipv4", "dualstack", "ipv6"
    #   resp.vpc_endpoint.dns_options.dns_record_ip_type #=> String, one of "ipv4", "dualstack", "ipv6", "service-defined"
    #   resp.vpc_endpoint.dns_options.private_dns_only_for_inbound_resolver_endpoint #=> Boolean
    #   resp.vpc_endpoint.private_dns_enabled #=> Boolean
    #   resp.vpc_endpoint.requester_managed #=> Boolean
    #   resp.vpc_endpoint.network_interface_ids #=> Array
    #   resp.vpc_endpoint.network_interface_ids[0] #=> String
    #   resp.vpc_endpoint.dns_entries #=> Array
    #   resp.vpc_endpoint.dns_entries[0].dns_name #=> String
    #   resp.vpc_endpoint.dns_entries[0].hosted_zone_id #=> String
    #   resp.vpc_endpoint.creation_timestamp #=> Time
    #   resp.vpc_endpoint.tags #=> Array
    #   resp.vpc_endpoint.tags[0].key #=> String
    #   resp.vpc_endpoint.tags[0].value #=> String
    #   resp.vpc_endpoint.owner_id #=> String
    #   resp.vpc_endpoint.last_error.message #=> String
    #   resp.vpc_endpoint.last_error.code #=> String
    #   resp.vpc_endpoint.ipv_4_prefixes #=> Array
    #   resp.vpc_endpoint.ipv_4_prefixes[0].subnet_id #=> String
    #   resp.vpc_endpoint.ipv_4_prefixes[0].ip_prefixes #=> Array
    #   resp.vpc_endpoint.ipv_4_prefixes[0].ip_prefixes[0] #=> String
    #   resp.vpc_endpoint.ipv_6_prefixes #=> Array
    #   resp.vpc_endpoint.ipv_6_prefixes[0].subnet_id #=> String
    #   resp.vpc_endpoint.ipv_6_prefixes[0].ip_prefixes #=> Array
    #   resp.vpc_endpoint.ipv_6_prefixes[0].ip_prefixes[0] #=> String
    #   resp.vpc_endpoint.failure_reason #=> String
    #   resp.vpc_endpoint.service_network_arn #=> String
    #   resp.vpc_endpoint.resource_configuration_arn #=> String
    #   resp.vpc_endpoint.service_region #=> String
    #   resp.client_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateVpcEndpoint AWS API Documentation
    #
    # @overload create_vpc_endpoint(params = {})
    # @param [Hash] params ({})
    def create_vpc_endpoint(params = {}, options = {})
      req = build_request(:create_vpc_endpoint, params)
      req.send_request(options)
    end

    # Creates a connection notification for a specified VPC endpoint or VPC
    # endpoint service. A connection notification notifies you of specific
    # endpoint events. You must create an SNS topic to receive
    # notifications. For more information, see [Creating an Amazon SNS
    # topic][1] in the *Amazon SNS Developer Guide*.
    #
    # You can create a connection notification for interface endpoints only.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/sns/latest/dg/CreateTopic.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :service_id
    #   The ID of the endpoint service.
    #
    # @option params [String] :vpc_endpoint_id
    #   The ID of the endpoint.
    #
    # @option params [required, String] :connection_notification_arn
    #   The ARN of the SNS topic for the notifications.
    #
    # @option params [required, Array<String>] :connection_events
    #   The endpoint events for which to receive notifications. Valid values
    #   are `Accept`, `Connect`, `Delete`, and `Reject`.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [How to ensure
    #   idempotency][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @return [Types::CreateVpcEndpointConnectionNotificationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateVpcEndpointConnectionNotificationResult#connection_notification #connection_notification} => Types::ConnectionNotification
    #   * {Types::CreateVpcEndpointConnectionNotificationResult#client_token #client_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_vpc_endpoint_connection_notification({
    #     dry_run: false,
    #     service_id: "VpcEndpointServiceId",
    #     vpc_endpoint_id: "VpcEndpointId",
    #     connection_notification_arn: "String", # required
    #     connection_events: ["String"], # required
    #     client_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.connection_notification.connection_notification_id #=> String
    #   resp.connection_notification.service_id #=> String
    #   resp.connection_notification.vpc_endpoint_id #=> String
    #   resp.connection_notification.connection_notification_type #=> String, one of "Topic"
    #   resp.connection_notification.connection_notification_arn #=> String
    #   resp.connection_notification.connection_events #=> Array
    #   resp.connection_notification.connection_events[0] #=> String
    #   resp.connection_notification.connection_notification_state #=> String, one of "Enabled", "Disabled"
    #   resp.connection_notification.service_region #=> String
    #   resp.client_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateVpcEndpointConnectionNotification AWS API Documentation
    #
    # @overload create_vpc_endpoint_connection_notification(params = {})
    # @param [Hash] params ({})
    def create_vpc_endpoint_connection_notification(params = {}, options = {})
      req = build_request(:create_vpc_endpoint_connection_notification, params)
      req.send_request(options)
    end

    # Creates a VPC endpoint service to which service consumers (Amazon Web
    # Services accounts, users, and IAM roles) can connect.
    #
    # Before you create an endpoint service, you must create one of the
    # following for your service:
    #
    # * A [Network Load Balancer][1]. Service consumers connect to your
    #   service using an interface endpoint.
    #
    # * A [Gateway Load Balancer][2]. Service consumers connect to your
    #   service using a Gateway Load Balancer endpoint.
    #
    # If you set the private DNS name, you must prove that you own the
    # private DNS domain name.
    #
    # For more information, see the [Amazon Web Services PrivateLink
    # Guide][3].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/elasticloadbalancing/latest/network/
    # [2]: https://docs.aws.amazon.com/elasticloadbalancing/latest/gateway/
    # [3]: https://docs.aws.amazon.com/vpc/latest/privatelink/
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Boolean] :acceptance_required
    #   Indicates whether requests from service consumers to create an
    #   endpoint to your service must be accepted manually.
    #
    # @option params [String] :private_dns_name
    #   (Interface endpoint configuration) The private DNS name to assign to
    #   the VPC endpoint service.
    #
    # @option params [Array<String>] :network_load_balancer_arns
    #   The Amazon Resource Names (ARNs) of the Network Load Balancers.
    #
    # @option params [Array<String>] :gateway_load_balancer_arns
    #   The Amazon Resource Names (ARNs) of the Gateway Load Balancers.
    #
    # @option params [Array<String>] :supported_ip_address_types
    #   The supported IP address types. The possible values are `ipv4` and
    #   `ipv6`.
    #
    # @option params [Array<String>] :supported_regions
    #   The Regions from which service consumers can access the service.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [How to ensure
    #   idempotency][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to associate with the service.
    #
    # @return [Types::CreateVpcEndpointServiceConfigurationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateVpcEndpointServiceConfigurationResult#service_configuration #service_configuration} => Types::ServiceConfiguration
    #   * {Types::CreateVpcEndpointServiceConfigurationResult#client_token #client_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_vpc_endpoint_service_configuration({
    #     dry_run: false,
    #     acceptance_required: false,
    #     private_dns_name: "String",
    #     network_load_balancer_arns: ["String"],
    #     gateway_load_balancer_arns: ["String"],
    #     supported_ip_address_types: ["String"],
    #     supported_regions: ["String"],
    #     client_token: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.service_configuration.service_type #=> Array
    #   resp.service_configuration.service_type[0].service_type #=> String, one of "Interface", "Gateway", "GatewayLoadBalancer"
    #   resp.service_configuration.service_id #=> String
    #   resp.service_configuration.service_name #=> String
    #   resp.service_configuration.service_state #=> String, one of "Pending", "Available", "Deleting", "Deleted", "Failed"
    #   resp.service_configuration.availability_zones #=> Array
    #   resp.service_configuration.availability_zones[0] #=> String
    #   resp.service_configuration.acceptance_required #=> Boolean
    #   resp.service_configuration.manages_vpc_endpoints #=> Boolean
    #   resp.service_configuration.network_load_balancer_arns #=> Array
    #   resp.service_configuration.network_load_balancer_arns[0] #=> String
    #   resp.service_configuration.gateway_load_balancer_arns #=> Array
    #   resp.service_configuration.gateway_load_balancer_arns[0] #=> String
    #   resp.service_configuration.supported_ip_address_types #=> Array
    #   resp.service_configuration.supported_ip_address_types[0] #=> String, one of "ipv4", "ipv6"
    #   resp.service_configuration.base_endpoint_dns_names #=> Array
    #   resp.service_configuration.base_endpoint_dns_names[0] #=> String
    #   resp.service_configuration.private_dns_name #=> String
    #   resp.service_configuration.private_dns_name_configuration.state #=> String, one of "pendingVerification", "verified", "failed"
    #   resp.service_configuration.private_dns_name_configuration.type #=> String
    #   resp.service_configuration.private_dns_name_configuration.value #=> String
    #   resp.service_configuration.private_dns_name_configuration.name #=> String
    #   resp.service_configuration.payer_responsibility #=> String, one of "ServiceOwner"
    #   resp.service_configuration.tags #=> Array
    #   resp.service_configuration.tags[0].key #=> String
    #   resp.service_configuration.tags[0].value #=> String
    #   resp.service_configuration.supported_regions #=> Array
    #   resp.service_configuration.supported_regions[0].region #=> String
    #   resp.service_configuration.supported_regions[0].service_state #=> String
    #   resp.service_configuration.remote_access_enabled #=> Boolean
    #   resp.client_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateVpcEndpointServiceConfiguration AWS API Documentation
    #
    # @overload create_vpc_endpoint_service_configuration(params = {})
    # @param [Hash] params ({})
    def create_vpc_endpoint_service_configuration(params = {}, options = {})
      req = build_request(:create_vpc_endpoint_service_configuration, params)
      req.send_request(options)
    end

    # Requests a VPC peering connection between two VPCs: a requester VPC
    # that you own and an accepter VPC with which to create the connection.
    # The accepter VPC can belong to another Amazon Web Services account and
    # can be in a different Region to the requester VPC. The requester VPC
    # and accepter VPC cannot have overlapping CIDR blocks.
    #
    # <note markdown="1"> Limitations and rules apply to a VPC peering connection. For more
    # information, see the [VPC peering limitations][1] in the *VPC Peering
    # Guide*.
    #
    #  </note>
    #
    # The owner of the accepter VPC must accept the peering request to
    # activate the peering connection. The VPC peering connection request
    # expires after 7 days, after which it cannot be accepted or rejected.
    #
    # If you create a VPC peering connection request between VPCs with
    # overlapping CIDR blocks, the VPC peering connection has a status of
    # `failed`.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/peering/vpc-peering-basics.html#vpc-peering-limitations
    #
    # @option params [String] :peer_region
    #   The Region code for the accepter VPC, if the accepter VPC is located
    #   in a Region other than the Region in which you make the request.
    #
    #   Default: The Region in which you make the request.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to assign to the peering connection.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the requester VPC. You must specify this parameter in the
    #   request.
    #
    # @option params [String] :peer_vpc_id
    #   The ID of the VPC with which you are creating the VPC peering
    #   connection. You must specify this parameter in the request.
    #
    # @option params [String] :peer_owner_id
    #   The Amazon Web Services account ID of the owner of the accepter VPC.
    #
    #   Default: Your Amazon Web Services account ID
    #
    # @return [Types::CreateVpcPeeringConnectionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateVpcPeeringConnectionResult#vpc_peering_connection #vpc_peering_connection} => Types::VpcPeeringConnection
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_vpc_peering_connection({
    #     peer_region: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #     vpc_id: "VpcId", # required
    #     peer_vpc_id: "String",
    #     peer_owner_id: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc_peering_connection.accepter_vpc_info.cidr_block #=> String
    #   resp.vpc_peering_connection.accepter_vpc_info.ipv_6_cidr_block_set #=> Array
    #   resp.vpc_peering_connection.accepter_vpc_info.ipv_6_cidr_block_set[0].ipv_6_cidr_block #=> String
    #   resp.vpc_peering_connection.accepter_vpc_info.cidr_block_set #=> Array
    #   resp.vpc_peering_connection.accepter_vpc_info.cidr_block_set[0].cidr_block #=> String
    #   resp.vpc_peering_connection.accepter_vpc_info.owner_id #=> String
    #   resp.vpc_peering_connection.accepter_vpc_info.peering_options.allow_dns_resolution_from_remote_vpc #=> Boolean
    #   resp.vpc_peering_connection.accepter_vpc_info.peering_options.allow_egress_from_local_classic_link_to_remote_vpc #=> Boolean
    #   resp.vpc_peering_connection.accepter_vpc_info.peering_options.allow_egress_from_local_vpc_to_remote_classic_link #=> Boolean
    #   resp.vpc_peering_connection.accepter_vpc_info.vpc_id #=> String
    #   resp.vpc_peering_connection.accepter_vpc_info.region #=> String
    #   resp.vpc_peering_connection.expiration_time #=> Time
    #   resp.vpc_peering_connection.requester_vpc_info.cidr_block #=> String
    #   resp.vpc_peering_connection.requester_vpc_info.ipv_6_cidr_block_set #=> Array
    #   resp.vpc_peering_connection.requester_vpc_info.ipv_6_cidr_block_set[0].ipv_6_cidr_block #=> String
    #   resp.vpc_peering_connection.requester_vpc_info.cidr_block_set #=> Array
    #   resp.vpc_peering_connection.requester_vpc_info.cidr_block_set[0].cidr_block #=> String
    #   resp.vpc_peering_connection.requester_vpc_info.owner_id #=> String
    #   resp.vpc_peering_connection.requester_vpc_info.peering_options.allow_dns_resolution_from_remote_vpc #=> Boolean
    #   resp.vpc_peering_connection.requester_vpc_info.peering_options.allow_egress_from_local_classic_link_to_remote_vpc #=> Boolean
    #   resp.vpc_peering_connection.requester_vpc_info.peering_options.allow_egress_from_local_vpc_to_remote_classic_link #=> Boolean
    #   resp.vpc_peering_connection.requester_vpc_info.vpc_id #=> String
    #   resp.vpc_peering_connection.requester_vpc_info.region #=> String
    #   resp.vpc_peering_connection.status.code #=> String, one of "initiating-request", "pending-acceptance", "active", "deleted", "rejected", "failed", "expired", "provisioning", "deleting"
    #   resp.vpc_peering_connection.status.message #=> String
    #   resp.vpc_peering_connection.tags #=> Array
    #   resp.vpc_peering_connection.tags[0].key #=> String
    #   resp.vpc_peering_connection.tags[0].value #=> String
    #   resp.vpc_peering_connection.vpc_peering_connection_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateVpcPeeringConnection AWS API Documentation
    #
    # @overload create_vpc_peering_connection(params = {})
    # @param [Hash] params ({})
    def create_vpc_peering_connection(params = {}, options = {})
      req = build_request(:create_vpc_peering_connection, params)
      req.send_request(options)
    end

    # Creates a VPN connection between an existing virtual private gateway
    # or transit gateway and a customer gateway. The supported connection
    # type is `ipsec.1`.
    #
    # The response includes information that you need to give to your
    # network administrator to configure your customer gateway.
    #
    # We strongly recommend that you use HTTPS when calling this operation
    # because the response contains sensitive cryptographic information for
    # configuring your customer gateway device.
    #
    # If you decide to shut down your VPN connection for any reason and
    # later create a new VPN connection, you must reconfigure your customer
    # gateway with the new information returned from this call.
    #
    # This is an idempotent operation. If you perform the operation more
    # than once, Amazon EC2 doesn't return an error.
    #
    # For more information, see [Amazon Web Services Site-to-Site VPN][1] in
    # the *Amazon Web Services Site-to-Site VPN User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html
    #
    # @option params [required, String] :customer_gateway_id
    #   The ID of the customer gateway.
    #
    # @option params [required, String] :type
    #   The type of VPN connection (`ipsec.1`).
    #
    # @option params [String] :vpn_gateway_id
    #   The ID of the virtual private gateway. If you specify a virtual
    #   private gateway, you cannot specify a transit gateway.
    #
    # @option params [String] :transit_gateway_id
    #   The ID of the transit gateway. If you specify a transit gateway, you
    #   cannot specify a virtual private gateway.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the VPN connection.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Types::VpnConnectionOptionsSpecification] :options
    #   The options for the VPN connection.
    #
    # @return [Types::CreateVpnConnectionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateVpnConnectionResult#vpn_connection #vpn_connection} => Types::VpnConnection
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_vpn_connection({
    #     customer_gateway_id: "CustomerGatewayId", # required
    #     type: "String", # required
    #     vpn_gateway_id: "VpnGatewayId",
    #     transit_gateway_id: "TransitGatewayId",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #     options: {
    #       enable_acceleration: false,
    #       tunnel_inside_ip_version: "ipv4", # accepts ipv4, ipv6
    #       tunnel_options: [
    #         {
    #           tunnel_inside_cidr: "String",
    #           tunnel_inside_ipv_6_cidr: "String",
    #           pre_shared_key: "preSharedKey",
    #           phase_1_lifetime_seconds: 1,
    #           phase_2_lifetime_seconds: 1,
    #           rekey_margin_time_seconds: 1,
    #           rekey_fuzz_percentage: 1,
    #           replay_window_size: 1,
    #           dpd_timeout_seconds: 1,
    #           dpd_timeout_action: "String",
    #           phase_1_encryption_algorithms: [
    #             {
    #               value: "String",
    #             },
    #           ],
    #           phase_2_encryption_algorithms: [
    #             {
    #               value: "String",
    #             },
    #           ],
    #           phase_1_integrity_algorithms: [
    #             {
    #               value: "String",
    #             },
    #           ],
    #           phase_2_integrity_algorithms: [
    #             {
    #               value: "String",
    #             },
    #           ],
    #           phase_1_dh_group_numbers: [
    #             {
    #               value: 1,
    #             },
    #           ],
    #           phase_2_dh_group_numbers: [
    #             {
    #               value: 1,
    #             },
    #           ],
    #           ike_versions: [
    #             {
    #               value: "String",
    #             },
    #           ],
    #           startup_action: "String",
    #           log_options: {
    #             cloud_watch_log_options: {
    #               log_enabled: false,
    #               log_group_arn: "CloudWatchLogGroupArn",
    #               log_output_format: "String",
    #             },
    #           },
    #           enable_tunnel_lifecycle_control: false,
    #         },
    #       ],
    #       local_ipv_4_network_cidr: "String",
    #       remote_ipv_4_network_cidr: "String",
    #       local_ipv_6_network_cidr: "String",
    #       remote_ipv_6_network_cidr: "String",
    #       outside_ip_address_type: "String",
    #       transport_transit_gateway_attachment_id: "TransitGatewayAttachmentId",
    #       static_routes_only: false,
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.vpn_connection.category #=> String
    #   resp.vpn_connection.transit_gateway_id #=> String
    #   resp.vpn_connection.core_network_arn #=> String
    #   resp.vpn_connection.core_network_attachment_arn #=> String
    #   resp.vpn_connection.gateway_association_state #=> String, one of "associated", "not-associated", "associating", "disassociating"
    #   resp.vpn_connection.options.enable_acceleration #=> Boolean
    #   resp.vpn_connection.options.static_routes_only #=> Boolean
    #   resp.vpn_connection.options.local_ipv_4_network_cidr #=> String
    #   resp.vpn_connection.options.remote_ipv_4_network_cidr #=> String
    #   resp.vpn_connection.options.local_ipv_6_network_cidr #=> String
    #   resp.vpn_connection.options.remote_ipv_6_network_cidr #=> String
    #   resp.vpn_connection.options.outside_ip_address_type #=> String
    #   resp.vpn_connection.options.transport_transit_gateway_attachment_id #=> String
    #   resp.vpn_connection.options.tunnel_inside_ip_version #=> String, one of "ipv4", "ipv6"
    #   resp.vpn_connection.options.tunnel_options #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].outside_ip_address #=> String
    #   resp.vpn_connection.options.tunnel_options[0].tunnel_inside_cidr #=> String
    #   resp.vpn_connection.options.tunnel_options[0].tunnel_inside_ipv_6_cidr #=> String
    #   resp.vpn_connection.options.tunnel_options[0].pre_shared_key #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_lifetime_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_lifetime_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].rekey_margin_time_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].rekey_fuzz_percentage #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].replay_window_size #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].dpd_timeout_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].dpd_timeout_action #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_encryption_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_encryption_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_encryption_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_encryption_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_integrity_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_integrity_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_integrity_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_integrity_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_dh_group_numbers #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_dh_group_numbers[0].value #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_dh_group_numbers #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_dh_group_numbers[0].value #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].ike_versions #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].ike_versions[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].startup_action #=> String
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_enabled #=> Boolean
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_group_arn #=> String
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_output_format #=> String
    #   resp.vpn_connection.options.tunnel_options[0].enable_tunnel_lifecycle_control #=> Boolean
    #   resp.vpn_connection.routes #=> Array
    #   resp.vpn_connection.routes[0].destination_cidr_block #=> String
    #   resp.vpn_connection.routes[0].source #=> String, one of "Static"
    #   resp.vpn_connection.routes[0].state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.vpn_connection.tags #=> Array
    #   resp.vpn_connection.tags[0].key #=> String
    #   resp.vpn_connection.tags[0].value #=> String
    #   resp.vpn_connection.vgw_telemetry #=> Array
    #   resp.vpn_connection.vgw_telemetry[0].accepted_route_count #=> Integer
    #   resp.vpn_connection.vgw_telemetry[0].last_status_change #=> Time
    #   resp.vpn_connection.vgw_telemetry[0].outside_ip_address #=> String
    #   resp.vpn_connection.vgw_telemetry[0].status #=> String, one of "UP", "DOWN"
    #   resp.vpn_connection.vgw_telemetry[0].status_message #=> String
    #   resp.vpn_connection.vgw_telemetry[0].certificate_arn #=> String
    #   resp.vpn_connection.vpn_connection_id #=> String
    #   resp.vpn_connection.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.vpn_connection.customer_gateway_configuration #=> String
    #   resp.vpn_connection.type #=> String, one of "ipsec.1"
    #   resp.vpn_connection.customer_gateway_id #=> String
    #   resp.vpn_connection.vpn_gateway_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateVpnConnection AWS API Documentation
    #
    # @overload create_vpn_connection(params = {})
    # @param [Hash] params ({})
    def create_vpn_connection(params = {}, options = {})
      req = build_request(:create_vpn_connection, params)
      req.send_request(options)
    end

    # Creates a static route associated with a VPN connection between an
    # existing virtual private gateway and a VPN customer gateway. The
    # static route allows traffic to be routed from the virtual private
    # gateway to the VPN customer gateway.
    #
    # For more information, see [Amazon Web Services Site-to-Site VPN][1] in
    # the *Amazon Web Services Site-to-Site VPN User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html
    #
    # @option params [required, String] :destination_cidr_block
    #   The CIDR block associated with the local subnet of the customer
    #   network.
    #
    # @option params [required, String] :vpn_connection_id
    #   The ID of the VPN connection.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_vpn_connection_route({
    #     destination_cidr_block: "String", # required
    #     vpn_connection_id: "VpnConnectionId", # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateVpnConnectionRoute AWS API Documentation
    #
    # @overload create_vpn_connection_route(params = {})
    # @param [Hash] params ({})
    def create_vpn_connection_route(params = {}, options = {})
      req = build_request(:create_vpn_connection_route, params)
      req.send_request(options)
    end

    # Creates a virtual private gateway. A virtual private gateway is the
    # endpoint on the VPC side of your VPN connection. You can create a
    # virtual private gateway before creating the VPC itself.
    #
    # For more information, see [Amazon Web Services Site-to-Site VPN][1] in
    # the *Amazon Web Services Site-to-Site VPN User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html
    #
    # @option params [String] :availability_zone
    #   The Availability Zone for the virtual private gateway.
    #
    # @option params [required, String] :type
    #   The type of VPN connection this virtual private gateway supports.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the virtual private gateway.
    #
    # @option params [Integer] :amazon_side_asn
    #   A private Autonomous System Number (ASN) for the Amazon side of a BGP
    #   session. If you're using a 16-bit ASN, it must be in the 64512 to
    #   65534 range. If you're using a 32-bit ASN, it must be in the
    #   4200000000 to 4294967294 range.
    #
    #   Default: 64512
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::CreateVpnGatewayResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::CreateVpnGatewayResult#vpn_gateway #vpn_gateway} => Types::VpnGateway
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.create_vpn_gateway({
    #     availability_zone: "String",
    #     type: "ipsec.1", # required, accepts ipsec.1
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     amazon_side_asn: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpn_gateway.amazon_side_asn #=> Integer
    #   resp.vpn_gateway.tags #=> Array
    #   resp.vpn_gateway.tags[0].key #=> String
    #   resp.vpn_gateway.tags[0].value #=> String
    #   resp.vpn_gateway.vpn_gateway_id #=> String
    #   resp.vpn_gateway.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.vpn_gateway.type #=> String, one of "ipsec.1"
    #   resp.vpn_gateway.availability_zone #=> String
    #   resp.vpn_gateway.vpc_attachments #=> Array
    #   resp.vpn_gateway.vpc_attachments[0].vpc_id #=> String
    #   resp.vpn_gateway.vpc_attachments[0].state #=> String, one of "attaching", "attached", "detaching", "detached"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/CreateVpnGateway AWS API Documentation
    #
    # @overload create_vpn_gateway(params = {})
    # @param [Hash] params ({})
    def create_vpn_gateway(params = {}, options = {})
      req = build_request(:create_vpn_gateway, params)
      req.send_request(options)
    end

    # Deletes a carrier gateway.
    #
    # If you do not delete the route that contains the carrier gateway as
    # the Target, the route is a blackhole route. For information about how
    # to delete a route, see [DeleteRoute][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DeleteRoute.html
    #
    # @option params [required, String] :carrier_gateway_id
    #   The ID of the carrier gateway.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteCarrierGatewayResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteCarrierGatewayResult#carrier_gateway #carrier_gateway} => Types::CarrierGateway
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_carrier_gateway({
    #     carrier_gateway_id: "CarrierGatewayId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.carrier_gateway.carrier_gateway_id #=> String
    #   resp.carrier_gateway.vpc_id #=> String
    #   resp.carrier_gateway.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.carrier_gateway.owner_id #=> String
    #   resp.carrier_gateway.tags #=> Array
    #   resp.carrier_gateway.tags[0].key #=> String
    #   resp.carrier_gateway.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteCarrierGateway AWS API Documentation
    #
    # @overload delete_carrier_gateway(params = {})
    # @param [Hash] params ({})
    def delete_carrier_gateway(params = {}, options = {})
      req = build_request(:delete_carrier_gateway, params)
      req.send_request(options)
    end

    # Deletes the specified Client VPN endpoint. You must disassociate all
    # target networks before you can delete a Client VPN endpoint.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN to be deleted.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteClientVpnEndpointResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteClientVpnEndpointResult#status #status} => Types::ClientVpnEndpointStatus
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_client_vpn_endpoint({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.status.code #=> String, one of "pending-associate", "available", "deleting", "deleted"
    #   resp.status.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteClientVpnEndpoint AWS API Documentation
    #
    # @overload delete_client_vpn_endpoint(params = {})
    # @param [Hash] params ({})
    def delete_client_vpn_endpoint(params = {}, options = {})
      req = build_request(:delete_client_vpn_endpoint, params)
      req.send_request(options)
    end

    # Deletes a route from a Client VPN endpoint. You can only delete routes
    # that you manually added using the **CreateClientVpnRoute** action. You
    # cannot delete routes that were automatically added when associating a
    # subnet. To remove routes that have been automatically added,
    # disassociate the target subnet from the Client VPN endpoint.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint from which the route is to be
    #   deleted.
    #
    # @option params [String] :target_vpc_subnet_id
    #   The ID of the target subnet used by the route.
    #
    # @option params [required, String] :destination_cidr_block
    #   The IPv4 address range, in CIDR notation, of the route to be deleted.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteClientVpnRouteResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteClientVpnRouteResult#status #status} => Types::ClientVpnRouteStatus
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_client_vpn_route({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     target_vpc_subnet_id: "SubnetId",
    #     destination_cidr_block: "String", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.status.code #=> String, one of "creating", "active", "failed", "deleting"
    #   resp.status.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteClientVpnRoute AWS API Documentation
    #
    # @overload delete_client_vpn_route(params = {})
    # @param [Hash] params ({})
    def delete_client_vpn_route(params = {}, options = {})
      req = build_request(:delete_client_vpn_route, params)
      req.send_request(options)
    end

    # Deletes a range of customer-owned IP addresses.
    #
    # @option params [required, String] :cidr
    #   A customer-owned IP address range that you want to delete.
    #
    # @option params [required, String] :coip_pool_id
    #   The ID of the customer-owned address pool.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteCoipCidrResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteCoipCidrResult#coip_cidr #coip_cidr} => Types::CoipCidr
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_coip_cidr({
    #     cidr: "String", # required
    #     coip_pool_id: "Ipv4PoolCoipId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.coip_cidr.cidr #=> String
    #   resp.coip_cidr.coip_pool_id #=> String
    #   resp.coip_cidr.local_gateway_route_table_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteCoipCidr AWS API Documentation
    #
    # @overload delete_coip_cidr(params = {})
    # @param [Hash] params ({})
    def delete_coip_cidr(params = {}, options = {})
      req = build_request(:delete_coip_cidr, params)
      req.send_request(options)
    end

    # Deletes a pool of customer-owned IP (CoIP) addresses.
    #
    # @option params [required, String] :coip_pool_id
    #   The ID of the CoIP pool that you want to delete.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteCoipPoolResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteCoipPoolResult#coip_pool #coip_pool} => Types::CoipPool
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_coip_pool({
    #     coip_pool_id: "Ipv4PoolCoipId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.coip_pool.pool_id #=> String
    #   resp.coip_pool.pool_cidrs #=> Array
    #   resp.coip_pool.pool_cidrs[0] #=> String
    #   resp.coip_pool.local_gateway_route_table_id #=> String
    #   resp.coip_pool.tags #=> Array
    #   resp.coip_pool.tags[0].key #=> String
    #   resp.coip_pool.tags[0].value #=> String
    #   resp.coip_pool.pool_arn #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteCoipPool AWS API Documentation
    #
    # @overload delete_coip_pool(params = {})
    # @param [Hash] params ({})
    def delete_coip_pool(params = {}, options = {})
      req = build_request(:delete_coip_pool, params)
      req.send_request(options)
    end

    # Deletes the specified customer gateway. You must delete the VPN
    # connection before you can delete the customer gateway.
    #
    # @option params [required, String] :customer_gateway_id
    #   The ID of the customer gateway.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To delete a customer gateway
    #
    #   # This example deletes the specified customer gateway.
    #
    #   resp = client.delete_customer_gateway({
    #     customer_gateway_id: "cgw-0e11f167", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_customer_gateway({
    #     customer_gateway_id: "CustomerGatewayId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteCustomerGateway AWS API Documentation
    #
    # @overload delete_customer_gateway(params = {})
    # @param [Hash] params ({})
    def delete_customer_gateway(params = {}, options = {})
      req = build_request(:delete_customer_gateway, params)
      req.send_request(options)
    end

    # Deletes the specified set of DHCP options. You must disassociate the
    # set of DHCP options before you can delete it. You can disassociate the
    # set of DHCP options by associating either a new set of options or the
    # default set of options with the VPC.
    #
    # @option params [required, String] :dhcp_options_id
    #   The ID of the DHCP options set.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To delete a DHCP options set
    #
    #   # This example deletes the specified DHCP options set.
    #
    #   resp = client.delete_dhcp_options({
    #     dhcp_options_id: "dopt-d9070ebb", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_dhcp_options({
    #     dhcp_options_id: "DhcpOptionsId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteDhcpOptions AWS API Documentation
    #
    # @overload delete_dhcp_options(params = {})
    # @param [Hash] params ({})
    def delete_dhcp_options(params = {}, options = {})
      req = build_request(:delete_dhcp_options, params)
      req.send_request(options)
    end

    # Deletes an egress-only internet gateway.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :egress_only_internet_gateway_id
    #   The ID of the egress-only internet gateway.
    #
    # @return [Types::DeleteEgressOnlyInternetGatewayResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteEgressOnlyInternetGatewayResult#return_code #return_code} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_egress_only_internet_gateway({
    #     dry_run: false,
    #     egress_only_internet_gateway_id: "EgressOnlyInternetGatewayId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.return_code #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteEgressOnlyInternetGateway AWS API Documentation
    #
    # @overload delete_egress_only_internet_gateway(params = {})
    # @param [Hash] params ({})
    def delete_egress_only_internet_gateway(params = {}, options = {})
      req = build_request(:delete_egress_only_internet_gateway, params)
      req.send_request(options)
    end

    # Deletes the specified EC2 Fleet request.
    #
    # After you delete an EC2 Fleet request, it launches no new instances.
    #
    # You must also specify whether a deleted EC2 Fleet request should
    # terminate its instances. If you choose to terminate the instances, the
    # EC2 Fleet request enters the `deleted_terminating` state. Otherwise,
    # it enters the `deleted_running` state, and the instances continue to
    # run until they are interrupted or you terminate them manually.
    #
    # A deleted `instant` fleet with running instances is not supported.
    # When you delete an `instant` fleet, Amazon EC2 automatically
    # terminates all its instances. For fleets with more than 1000
    # instances, the deletion request might fail. If your fleet has more
    # than 1000 instances, first terminate most of the instances manually,
    # leaving 1000 or fewer. Then delete the fleet, and the remaining
    # instances will be terminated automatically.
    #
    # **Restrictions**
    #
    # * You can delete up to 25 fleets of type `instant` in a single
    #   request.
    #
    # * You can delete up to 100 fleets of type `maintain` or `request` in a
    #   single request.
    #
    # * You can delete up to 125 fleets in a single request, provided you do
    #   not exceed the quota for each fleet type, as specified above.
    #
    # * If you exceed the specified number of fleets to delete, no fleets
    #   are deleted.
    #
    # For more information, see [Delete an EC2 Fleet request and the
    # instances in the fleet][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/delete-fleet.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :fleet_ids
    #   The IDs of the EC2 Fleets.
    #
    #   Constraints: In a single request, you can specify up to 25 `instant`
    #   fleet IDs and up to 100 `maintain` or `request` fleet IDs.
    #
    # @option params [required, Boolean] :terminate_instances
    #   Indicates whether to terminate the associated instances when the EC2
    #   Fleet is deleted. The default is to terminate the instances.
    #
    #   To let the instances continue to run after the EC2 Fleet is deleted,
    #   specify `no-terminate-instances`. Supported only for fleets of type
    #   `maintain` and `request`.
    #
    #   For `instant` fleets, you cannot specify `NoTerminateInstances`. A
    #   deleted `instant` fleet with running instances is not supported.
    #
    # @return [Types::DeleteFleetsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteFleetsResult#successful_fleet_deletions #successful_fleet_deletions} => Array&lt;Types::DeleteFleetSuccessItem&gt;
    #   * {Types::DeleteFleetsResult#unsuccessful_fleet_deletions #unsuccessful_fleet_deletions} => Array&lt;Types::DeleteFleetErrorItem&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_fleets({
    #     dry_run: false,
    #     fleet_ids: ["FleetId"], # required
    #     terminate_instances: false, # required
    #   })
    #
    # @example Response structure
    #
    #   resp.successful_fleet_deletions #=> Array
    #   resp.successful_fleet_deletions[0].current_fleet_state #=> String, one of "submitted", "active", "deleted", "failed", "deleted_running", "deleted_terminating", "modifying"
    #   resp.successful_fleet_deletions[0].previous_fleet_state #=> String, one of "submitted", "active", "deleted", "failed", "deleted_running", "deleted_terminating", "modifying"
    #   resp.successful_fleet_deletions[0].fleet_id #=> String
    #   resp.unsuccessful_fleet_deletions #=> Array
    #   resp.unsuccessful_fleet_deletions[0].error.code #=> String, one of "fleetIdDoesNotExist", "fleetIdMalformed", "fleetNotInDeletableState", "unexpectedError"
    #   resp.unsuccessful_fleet_deletions[0].error.message #=> String
    #   resp.unsuccessful_fleet_deletions[0].fleet_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteFleets AWS API Documentation
    #
    # @overload delete_fleets(params = {})
    # @param [Hash] params ({})
    def delete_fleets(params = {}, options = {})
      req = build_request(:delete_fleets, params)
      req.send_request(options)
    end

    # Deletes one or more flow logs.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :flow_log_ids
    #   One or more flow log IDs.
    #
    #   Constraint: Maximum of 1000 flow log IDs.
    #
    # @return [Types::DeleteFlowLogsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteFlowLogsResult#unsuccessful #unsuccessful} => Array&lt;Types::UnsuccessfulItem&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_flow_logs({
    #     dry_run: false,
    #     flow_log_ids: ["VpcFlowLogId"], # required
    #   })
    #
    # @example Response structure
    #
    #   resp.unsuccessful #=> Array
    #   resp.unsuccessful[0].error.code #=> String
    #   resp.unsuccessful[0].error.message #=> String
    #   resp.unsuccessful[0].resource_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteFlowLogs AWS API Documentation
    #
    # @overload delete_flow_logs(params = {})
    # @param [Hash] params ({})
    def delete_flow_logs(params = {}, options = {})
      req = build_request(:delete_flow_logs, params)
      req.send_request(options)
    end

    # Deletes the specified Amazon FPGA Image (AFI).
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :fpga_image_id
    #   The ID of the AFI.
    #
    # @return [Types::DeleteFpgaImageResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteFpgaImageResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_fpga_image({
    #     dry_run: false,
    #     fpga_image_id: "FpgaImageId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteFpgaImage AWS API Documentation
    #
    # @overload delete_fpga_image(params = {})
    # @param [Hash] params ({})
    def delete_fpga_image(params = {}, options = {})
      req = build_request(:delete_fpga_image, params)
      req.send_request(options)
    end

    # Deletes the specified EC2 Instance Connect Endpoint.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_connect_endpoint_id
    #   The ID of the EC2 Instance Connect Endpoint to delete.
    #
    # @return [Types::DeleteInstanceConnectEndpointResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteInstanceConnectEndpointResult#instance_connect_endpoint #instance_connect_endpoint} => Types::Ec2InstanceConnectEndpoint
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_instance_connect_endpoint({
    #     dry_run: false,
    #     instance_connect_endpoint_id: "InstanceConnectEndpointId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_connect_endpoint.owner_id #=> String
    #   resp.instance_connect_endpoint.instance_connect_endpoint_id #=> String
    #   resp.instance_connect_endpoint.instance_connect_endpoint_arn #=> String
    #   resp.instance_connect_endpoint.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "delete-in-progress", "delete-complete", "delete-failed"
    #   resp.instance_connect_endpoint.state_message #=> String
    #   resp.instance_connect_endpoint.dns_name #=> String
    #   resp.instance_connect_endpoint.fips_dns_name #=> String
    #   resp.instance_connect_endpoint.network_interface_ids #=> Array
    #   resp.instance_connect_endpoint.network_interface_ids[0] #=> String
    #   resp.instance_connect_endpoint.vpc_id #=> String
    #   resp.instance_connect_endpoint.availability_zone #=> String
    #   resp.instance_connect_endpoint.created_at #=> Time
    #   resp.instance_connect_endpoint.subnet_id #=> String
    #   resp.instance_connect_endpoint.preserve_client_ip #=> Boolean
    #   resp.instance_connect_endpoint.security_group_ids #=> Array
    #   resp.instance_connect_endpoint.security_group_ids[0] #=> String
    #   resp.instance_connect_endpoint.tags #=> Array
    #   resp.instance_connect_endpoint.tags[0].key #=> String
    #   resp.instance_connect_endpoint.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteInstanceConnectEndpoint AWS API Documentation
    #
    # @overload delete_instance_connect_endpoint(params = {})
    # @param [Hash] params ({})
    def delete_instance_connect_endpoint(params = {}, options = {})
      req = build_request(:delete_instance_connect_endpoint, params)
      req.send_request(options)
    end

    # Deletes the specified event window.
    #
    # For more information, see [Define event windows for scheduled
    # events][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/event-windows.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Boolean] :force_delete
    #   Specify `true` to force delete the event window. Use the force delete
    #   parameter if the event window is currently associated with targets.
    #
    # @option params [required, String] :instance_event_window_id
    #   The ID of the event window.
    #
    # @return [Types::DeleteInstanceEventWindowResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteInstanceEventWindowResult#instance_event_window_state #instance_event_window_state} => Types::InstanceEventWindowStateChange
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_instance_event_window({
    #     dry_run: false,
    #     force_delete: false,
    #     instance_event_window_id: "InstanceEventWindowId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_event_window_state.instance_event_window_id #=> String
    #   resp.instance_event_window_state.state #=> String, one of "creating", "deleting", "active", "deleted"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteInstanceEventWindow AWS API Documentation
    #
    # @overload delete_instance_event_window(params = {})
    # @param [Hash] params ({})
    def delete_instance_event_window(params = {}, options = {})
      req = build_request(:delete_instance_event_window, params)
      req.send_request(options)
    end

    # Deletes the specified internet gateway. You must detach the internet
    # gateway from the VPC before you can delete it.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :internet_gateway_id
    #   The ID of the internet gateway.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To delete an Internet gateway
    #
    #   # This example deletes the specified Internet gateway.
    #
    #   resp = client.delete_internet_gateway({
    #     internet_gateway_id: "igw-c0a643a9", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_internet_gateway({
    #     dry_run: false,
    #     internet_gateway_id: "InternetGatewayId", # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteInternetGateway AWS API Documentation
    #
    # @overload delete_internet_gateway(params = {})
    # @param [Hash] params ({})
    def delete_internet_gateway(params = {}, options = {})
      req = build_request(:delete_internet_gateway, params)
      req.send_request(options)
    end

    # Delete an IPAM. Deleting an IPAM removes all monitored data associated
    # with the IPAM including the historical data for CIDRs.
    #
    # For more information, see [Delete an IPAM][1] in the *Amazon VPC IPAM
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/delete-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_id
    #   The ID of the IPAM to delete.
    #
    # @option params [Boolean] :cascade
    #   Enables you to quickly delete an IPAM, private scopes, pools in
    #   private scopes, and any allocations in the pools in private scopes.
    #   You cannot delete the IPAM with this option if there is a pool in your
    #   public scope. If you use this option, IPAM does the following:
    #
    #   * Deallocates any CIDRs allocated to VPC resources (such as VPCs) in
    #     pools in private scopes.
    #
    #     <note markdown="1"> No VPC resources are deleted as a result of enabling this option.
    #     The CIDR associated with the resource will no longer be allocated
    #     from an IPAM pool, but the CIDR itself will remain unchanged.
    #
    #      </note>
    #
    #   * Deprovisions all IPv4 CIDRs provisioned to IPAM pools in private
    #     scopes.
    #
    #   * Deletes all IPAM pools in private scopes.
    #
    #   * Deletes all non-default private scopes in the IPAM.
    #
    #   * Deletes the default public and private scopes and the IPAM.
    #
    # @return [Types::DeleteIpamResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteIpamResult#ipam #ipam} => Types::Ipam
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_ipam({
    #     dry_run: false,
    #     ipam_id: "IpamId", # required
    #     cascade: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam.owner_id #=> String
    #   resp.ipam.ipam_id #=> String
    #   resp.ipam.ipam_arn #=> String
    #   resp.ipam.ipam_region #=> String
    #   resp.ipam.public_default_scope_id #=> String
    #   resp.ipam.private_default_scope_id #=> String
    #   resp.ipam.scope_count #=> Integer
    #   resp.ipam.description #=> String
    #   resp.ipam.operating_regions #=> Array
    #   resp.ipam.operating_regions[0].region_name #=> String
    #   resp.ipam.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam.tags #=> Array
    #   resp.ipam.tags[0].key #=> String
    #   resp.ipam.tags[0].value #=> String
    #   resp.ipam.default_resource_discovery_id #=> String
    #   resp.ipam.default_resource_discovery_association_id #=> String
    #   resp.ipam.resource_discovery_association_count #=> Integer
    #   resp.ipam.state_message #=> String
    #   resp.ipam.tier #=> String, one of "free", "advanced"
    #   resp.ipam.enable_private_gua #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteIpam AWS API Documentation
    #
    # @overload delete_ipam(params = {})
    # @param [Hash] params ({})
    def delete_ipam(params = {}, options = {})
      req = build_request(:delete_ipam, params)
      req.send_request(options)
    end

    # Delete a verification token. A verification token is an Amazon Web
    # Services-generated random value that you can use to prove ownership of
    # an external resource. For example, you can use a verification token to
    # validate that you control a public IP address range when you bring an
    # IP address range to Amazon Web Services (BYOIP).
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_external_resource_verification_token_id
    #   The token ID.
    #
    # @return [Types::DeleteIpamExternalResourceVerificationTokenResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteIpamExternalResourceVerificationTokenResult#ipam_external_resource_verification_token #ipam_external_resource_verification_token} => Types::IpamExternalResourceVerificationToken
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_ipam_external_resource_verification_token({
    #     dry_run: false,
    #     ipam_external_resource_verification_token_id: "IpamExternalResourceVerificationTokenId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_external_resource_verification_token.ipam_external_resource_verification_token_id #=> String
    #   resp.ipam_external_resource_verification_token.ipam_external_resource_verification_token_arn #=> String
    #   resp.ipam_external_resource_verification_token.ipam_id #=> String
    #   resp.ipam_external_resource_verification_token.ipam_arn #=> String
    #   resp.ipam_external_resource_verification_token.ipam_region #=> String
    #   resp.ipam_external_resource_verification_token.token_value #=> String
    #   resp.ipam_external_resource_verification_token.token_name #=> String
    #   resp.ipam_external_resource_verification_token.not_after #=> Time
    #   resp.ipam_external_resource_verification_token.status #=> String, one of "valid", "expired"
    #   resp.ipam_external_resource_verification_token.tags #=> Array
    #   resp.ipam_external_resource_verification_token.tags[0].key #=> String
    #   resp.ipam_external_resource_verification_token.tags[0].value #=> String
    #   resp.ipam_external_resource_verification_token.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "delete-in-progress", "delete-complete", "delete-failed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteIpamExternalResourceVerificationToken AWS API Documentation
    #
    # @overload delete_ipam_external_resource_verification_token(params = {})
    # @param [Hash] params ({})
    def delete_ipam_external_resource_verification_token(params = {}, options = {})
      req = build_request(:delete_ipam_external_resource_verification_token, params)
      req.send_request(options)
    end

    # Delete an IPAM pool.
    #
    # <note markdown="1"> You cannot delete an IPAM pool if there are allocations in it or CIDRs
    # provisioned to it. To release allocations, see
    # [ReleaseIpamPoolAllocation][1]. To deprovision pool CIDRs, see
    # [DeprovisionIpamPoolCidr][2].
    #
    #  </note>
    #
    # For more information, see [Delete a pool][3] in the *Amazon VPC IPAM
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ReleaseIpamPoolAllocation.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DeprovisionIpamPoolCidr.html
    # [3]: https://docs.aws.amazon.com/vpc/latest/ipam/delete-pool-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_pool_id
    #   The ID of the pool to delete.
    #
    # @option params [Boolean] :cascade
    #   Enables you to quickly delete an IPAM pool and all resources within
    #   that pool, including provisioned CIDRs, allocations, and other pools.
    #
    #   You can only use this option to delete pools in the private scope or
    #   pools in the public scope with a source resource. A source resource is
    #   a resource used to provision CIDRs to a resource planning pool.
    #
    # @return [Types::DeleteIpamPoolResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteIpamPoolResult#ipam_pool #ipam_pool} => Types::IpamPool
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_ipam_pool({
    #     dry_run: false,
    #     ipam_pool_id: "IpamPoolId", # required
    #     cascade: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_pool.owner_id #=> String
    #   resp.ipam_pool.ipam_pool_id #=> String
    #   resp.ipam_pool.source_ipam_pool_id #=> String
    #   resp.ipam_pool.ipam_pool_arn #=> String
    #   resp.ipam_pool.ipam_scope_arn #=> String
    #   resp.ipam_pool.ipam_scope_type #=> String, one of "public", "private"
    #   resp.ipam_pool.ipam_arn #=> String
    #   resp.ipam_pool.ipam_region #=> String
    #   resp.ipam_pool.locale #=> String
    #   resp.ipam_pool.pool_depth #=> Integer
    #   resp.ipam_pool.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_pool.state_message #=> String
    #   resp.ipam_pool.description #=> String
    #   resp.ipam_pool.auto_import #=> Boolean
    #   resp.ipam_pool.publicly_advertisable #=> Boolean
    #   resp.ipam_pool.address_family #=> String, one of "ipv4", "ipv6"
    #   resp.ipam_pool.allocation_min_netmask_length #=> Integer
    #   resp.ipam_pool.allocation_max_netmask_length #=> Integer
    #   resp.ipam_pool.allocation_default_netmask_length #=> Integer
    #   resp.ipam_pool.allocation_resource_tags #=> Array
    #   resp.ipam_pool.allocation_resource_tags[0].key #=> String
    #   resp.ipam_pool.allocation_resource_tags[0].value #=> String
    #   resp.ipam_pool.tags #=> Array
    #   resp.ipam_pool.tags[0].key #=> String
    #   resp.ipam_pool.tags[0].value #=> String
    #   resp.ipam_pool.aws_service #=> String, one of "ec2"
    #   resp.ipam_pool.public_ip_source #=> String, one of "amazon", "byoip"
    #   resp.ipam_pool.source_resource.resource_id #=> String
    #   resp.ipam_pool.source_resource.resource_type #=> String, one of "vpc"
    #   resp.ipam_pool.source_resource.resource_region #=> String
    #   resp.ipam_pool.source_resource.resource_owner #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteIpamPool AWS API Documentation
    #
    # @overload delete_ipam_pool(params = {})
    # @param [Hash] params ({})
    def delete_ipam_pool(params = {}, options = {})
      req = build_request(:delete_ipam_pool, params)
      req.send_request(options)
    end

    # Deletes an IPAM resource discovery. A resource discovery is an IPAM
    # component that enables IPAM to manage and monitor resources that
    # belong to the owning account.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_resource_discovery_id
    #   The IPAM resource discovery ID.
    #
    # @return [Types::DeleteIpamResourceDiscoveryResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteIpamResourceDiscoveryResult#ipam_resource_discovery #ipam_resource_discovery} => Types::IpamResourceDiscovery
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_ipam_resource_discovery({
    #     dry_run: false,
    #     ipam_resource_discovery_id: "IpamResourceDiscoveryId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_resource_discovery.owner_id #=> String
    #   resp.ipam_resource_discovery.ipam_resource_discovery_id #=> String
    #   resp.ipam_resource_discovery.ipam_resource_discovery_arn #=> String
    #   resp.ipam_resource_discovery.ipam_resource_discovery_region #=> String
    #   resp.ipam_resource_discovery.description #=> String
    #   resp.ipam_resource_discovery.operating_regions #=> Array
    #   resp.ipam_resource_discovery.operating_regions[0].region_name #=> String
    #   resp.ipam_resource_discovery.is_default #=> Boolean
    #   resp.ipam_resource_discovery.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_resource_discovery.tags #=> Array
    #   resp.ipam_resource_discovery.tags[0].key #=> String
    #   resp.ipam_resource_discovery.tags[0].value #=> String
    #   resp.ipam_resource_discovery.organizational_unit_exclusions #=> Array
    #   resp.ipam_resource_discovery.organizational_unit_exclusions[0].organizations_entity_path #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteIpamResourceDiscovery AWS API Documentation
    #
    # @overload delete_ipam_resource_discovery(params = {})
    # @param [Hash] params ({})
    def delete_ipam_resource_discovery(params = {}, options = {})
      req = build_request(:delete_ipam_resource_discovery, params)
      req.send_request(options)
    end

    # Delete the scope for an IPAM. You cannot delete the default scopes.
    #
    # For more information, see [Delete a scope][1] in the *Amazon VPC IPAM
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/delete-scope-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_scope_id
    #   The ID of the scope to delete.
    #
    # @return [Types::DeleteIpamScopeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteIpamScopeResult#ipam_scope #ipam_scope} => Types::IpamScope
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_ipam_scope({
    #     dry_run: false,
    #     ipam_scope_id: "IpamScopeId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_scope.owner_id #=> String
    #   resp.ipam_scope.ipam_scope_id #=> String
    #   resp.ipam_scope.ipam_scope_arn #=> String
    #   resp.ipam_scope.ipam_arn #=> String
    #   resp.ipam_scope.ipam_region #=> String
    #   resp.ipam_scope.ipam_scope_type #=> String, one of "public", "private"
    #   resp.ipam_scope.is_default #=> Boolean
    #   resp.ipam_scope.description #=> String
    #   resp.ipam_scope.pool_count #=> Integer
    #   resp.ipam_scope.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_scope.tags #=> Array
    #   resp.ipam_scope.tags[0].key #=> String
    #   resp.ipam_scope.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteIpamScope AWS API Documentation
    #
    # @overload delete_ipam_scope(params = {})
    # @param [Hash] params ({})
    def delete_ipam_scope(params = {}, options = {})
      req = build_request(:delete_ipam_scope, params)
      req.send_request(options)
    end

    # Deletes the specified key pair, by removing the public key from Amazon
    # EC2.
    #
    # @option params [String] :key_name
    #   The name of the key pair.
    #
    # @option params [String] :key_pair_id
    #   The ID of the key pair.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteKeyPairResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteKeyPairResult#return #return} => Boolean
    #   * {Types::DeleteKeyPairResult#key_pair_id #key_pair_id} => String
    #
    #
    # @example Example: To delete a key pair
    #
    #   # This example deletes the specified key pair.
    #
    #   resp = client.delete_key_pair({
    #     key_name: "my-key-pair", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_key_pair({
    #     key_name: "KeyPairNameWithResolver",
    #     key_pair_id: "KeyPairId",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #   resp.key_pair_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteKeyPair AWS API Documentation
    #
    # @overload delete_key_pair(params = {})
    # @param [Hash] params ({})
    def delete_key_pair(params = {}, options = {})
      req = build_request(:delete_key_pair, params)
      req.send_request(options)
    end

    # Deletes a launch template. Deleting a launch template deletes all of
    # its versions.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :launch_template_id
    #   The ID of the launch template.
    #
    #   You must specify either the launch template ID or the launch template
    #   name, but not both.
    #
    # @option params [String] :launch_template_name
    #   The name of the launch template.
    #
    #   You must specify either the launch template ID or the launch template
    #   name, but not both.
    #
    # @return [Types::DeleteLaunchTemplateResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteLaunchTemplateResult#launch_template #launch_template} => Types::LaunchTemplate
    #
    #
    # @example Example: To delete a launch template
    #
    #   # This example deletes the specified launch template.
    #
    #   resp = client.delete_launch_template({
    #     launch_template_id: "lt-0abcd290751193123", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     launch_template: {
    #       create_time: Time.parse("2017-11-23T16:46:25.000Z"), 
    #       created_by: "arn:aws:iam::123456789012:root", 
    #       default_version_number: 2, 
    #       latest_version_number: 2, 
    #       launch_template_id: "lt-0abcd290751193123", 
    #       launch_template_name: "my-template", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_launch_template({
    #     dry_run: false,
    #     launch_template_id: "LaunchTemplateId",
    #     launch_template_name: "LaunchTemplateName",
    #   })
    #
    # @example Response structure
    #
    #   resp.launch_template.launch_template_id #=> String
    #   resp.launch_template.launch_template_name #=> String
    #   resp.launch_template.create_time #=> Time
    #   resp.launch_template.created_by #=> String
    #   resp.launch_template.default_version_number #=> Integer
    #   resp.launch_template.latest_version_number #=> Integer
    #   resp.launch_template.tags #=> Array
    #   resp.launch_template.tags[0].key #=> String
    #   resp.launch_template.tags[0].value #=> String
    #   resp.launch_template.operator.managed #=> Boolean
    #   resp.launch_template.operator.principal #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteLaunchTemplate AWS API Documentation
    #
    # @overload delete_launch_template(params = {})
    # @param [Hash] params ({})
    def delete_launch_template(params = {}, options = {})
      req = build_request(:delete_launch_template, params)
      req.send_request(options)
    end

    # Deletes one or more versions of a launch template.
    #
    # You can't delete the default version of a launch template; you must
    # first assign a different version as the default. If the default
    # version is the only version for the launch template, you must delete
    # the entire launch template using DeleteLaunchTemplate.
    #
    # You can delete up to 200 launch template versions in a single request.
    # To delete more than 200 versions in a single request, use
    # DeleteLaunchTemplate, which deletes the launch template and all of its
    # versions.
    #
    # For more information, see [Delete a launch template version][1] in the
    # *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/delete-launch-template.html#delete-launch-template-version
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :launch_template_id
    #   The ID of the launch template.
    #
    #   You must specify either the launch template ID or the launch template
    #   name, but not both.
    #
    # @option params [String] :launch_template_name
    #   The name of the launch template.
    #
    #   You must specify either the launch template ID or the launch template
    #   name, but not both.
    #
    # @option params [required, Array<String>] :versions
    #   The version numbers of one or more launch template versions to delete.
    #   You can specify up to 200 launch template version numbers.
    #
    # @return [Types::DeleteLaunchTemplateVersionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteLaunchTemplateVersionsResult#successfully_deleted_launch_template_versions #successfully_deleted_launch_template_versions} => Array&lt;Types::DeleteLaunchTemplateVersionsResponseSuccessItem&gt;
    #   * {Types::DeleteLaunchTemplateVersionsResult#unsuccessfully_deleted_launch_template_versions #unsuccessfully_deleted_launch_template_versions} => Array&lt;Types::DeleteLaunchTemplateVersionsResponseErrorItem&gt;
    #
    #
    # @example Example: To delete a launch template version
    #
    #   # This example deletes the specified launch template version.
    #
    #   resp = client.delete_launch_template_versions({
    #     launch_template_id: "lt-0abcd290751193123", 
    #     versions: [
    #       "1", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     successfully_deleted_launch_template_versions: [
    #       {
    #         launch_template_id: "lt-0abcd290751193123", 
    #         launch_template_name: "my-template", 
    #         version_number: 1, 
    #       }, 
    #     ], 
    #     unsuccessfully_deleted_launch_template_versions: [
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_launch_template_versions({
    #     dry_run: false,
    #     launch_template_id: "LaunchTemplateId",
    #     launch_template_name: "LaunchTemplateName",
    #     versions: ["String"], # required
    #   })
    #
    # @example Response structure
    #
    #   resp.successfully_deleted_launch_template_versions #=> Array
    #   resp.successfully_deleted_launch_template_versions[0].launch_template_id #=> String
    #   resp.successfully_deleted_launch_template_versions[0].launch_template_name #=> String
    #   resp.successfully_deleted_launch_template_versions[0].version_number #=> Integer
    #   resp.unsuccessfully_deleted_launch_template_versions #=> Array
    #   resp.unsuccessfully_deleted_launch_template_versions[0].launch_template_id #=> String
    #   resp.unsuccessfully_deleted_launch_template_versions[0].launch_template_name #=> String
    #   resp.unsuccessfully_deleted_launch_template_versions[0].version_number #=> Integer
    #   resp.unsuccessfully_deleted_launch_template_versions[0].response_error.code #=> String, one of "launchTemplateIdDoesNotExist", "launchTemplateIdMalformed", "launchTemplateNameDoesNotExist", "launchTemplateNameMalformed", "launchTemplateVersionDoesNotExist", "unexpectedError"
    #   resp.unsuccessfully_deleted_launch_template_versions[0].response_error.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteLaunchTemplateVersions AWS API Documentation
    #
    # @overload delete_launch_template_versions(params = {})
    # @param [Hash] params ({})
    def delete_launch_template_versions(params = {}, options = {})
      req = build_request(:delete_launch_template_versions, params)
      req.send_request(options)
    end

    # Deletes the specified route from the specified local gateway route
    # table.
    #
    # @option params [String] :destination_cidr_block
    #   The CIDR range for the route. This must match the CIDR for the route
    #   exactly.
    #
    # @option params [required, String] :local_gateway_route_table_id
    #   The ID of the local gateway route table.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :destination_prefix_list_id
    #   Use a prefix list in place of `DestinationCidrBlock`. You cannot use
    #   `DestinationPrefixListId` and `DestinationCidrBlock` in the same
    #   request.
    #
    # @return [Types::DeleteLocalGatewayRouteResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteLocalGatewayRouteResult#route #route} => Types::LocalGatewayRoute
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_local_gateway_route({
    #     destination_cidr_block: "String",
    #     local_gateway_route_table_id: "LocalGatewayRoutetableId", # required
    #     dry_run: false,
    #     destination_prefix_list_id: "PrefixListResourceId",
    #   })
    #
    # @example Response structure
    #
    #   resp.route.destination_cidr_block #=> String
    #   resp.route.local_gateway_virtual_interface_group_id #=> String
    #   resp.route.type #=> String, one of "static", "propagated"
    #   resp.route.state #=> String, one of "pending", "active", "blackhole", "deleting", "deleted"
    #   resp.route.local_gateway_route_table_id #=> String
    #   resp.route.local_gateway_route_table_arn #=> String
    #   resp.route.owner_id #=> String
    #   resp.route.subnet_id #=> String
    #   resp.route.coip_pool_id #=> String
    #   resp.route.network_interface_id #=> String
    #   resp.route.destination_prefix_list_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteLocalGatewayRoute AWS API Documentation
    #
    # @overload delete_local_gateway_route(params = {})
    # @param [Hash] params ({})
    def delete_local_gateway_route(params = {}, options = {})
      req = build_request(:delete_local_gateway_route, params)
      req.send_request(options)
    end

    # Deletes a local gateway route table.
    #
    # @option params [required, String] :local_gateway_route_table_id
    #   The ID of the local gateway route table.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteLocalGatewayRouteTableResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteLocalGatewayRouteTableResult#local_gateway_route_table #local_gateway_route_table} => Types::LocalGatewayRouteTable
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_local_gateway_route_table({
    #     local_gateway_route_table_id: "LocalGatewayRoutetableId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.local_gateway_route_table.local_gateway_route_table_id #=> String
    #   resp.local_gateway_route_table.local_gateway_route_table_arn #=> String
    #   resp.local_gateway_route_table.local_gateway_id #=> String
    #   resp.local_gateway_route_table.outpost_arn #=> String
    #   resp.local_gateway_route_table.owner_id #=> String
    #   resp.local_gateway_route_table.state #=> String
    #   resp.local_gateway_route_table.tags #=> Array
    #   resp.local_gateway_route_table.tags[0].key #=> String
    #   resp.local_gateway_route_table.tags[0].value #=> String
    #   resp.local_gateway_route_table.mode #=> String, one of "direct-vpc-routing", "coip"
    #   resp.local_gateway_route_table.state_reason.code #=> String
    #   resp.local_gateway_route_table.state_reason.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteLocalGatewayRouteTable AWS API Documentation
    #
    # @overload delete_local_gateway_route_table(params = {})
    # @param [Hash] params ({})
    def delete_local_gateway_route_table(params = {}, options = {})
      req = build_request(:delete_local_gateway_route_table, params)
      req.send_request(options)
    end

    # Deletes a local gateway route table virtual interface group
    # association.
    #
    # @option params [required, String] :local_gateway_route_table_virtual_interface_group_association_id
    #   The ID of the local gateway route table virtual interface group
    #   association.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteLocalGatewayRouteTableVirtualInterfaceGroupAssociationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteLocalGatewayRouteTableVirtualInterfaceGroupAssociationResult#local_gateway_route_table_virtual_interface_group_association #local_gateway_route_table_virtual_interface_group_association} => Types::LocalGatewayRouteTableVirtualInterfaceGroupAssociation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_local_gateway_route_table_virtual_interface_group_association({
    #     local_gateway_route_table_virtual_interface_group_association_id: "LocalGatewayRouteTableVirtualInterfaceGroupAssociationId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.local_gateway_route_table_virtual_interface_group_association.local_gateway_route_table_virtual_interface_group_association_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.local_gateway_virtual_interface_group_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.local_gateway_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.local_gateway_route_table_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.local_gateway_route_table_arn #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.owner_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.state #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.tags #=> Array
    #   resp.local_gateway_route_table_virtual_interface_group_association.tags[0].key #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_association.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteLocalGatewayRouteTableVirtualInterfaceGroupAssociation AWS API Documentation
    #
    # @overload delete_local_gateway_route_table_virtual_interface_group_association(params = {})
    # @param [Hash] params ({})
    def delete_local_gateway_route_table_virtual_interface_group_association(params = {}, options = {})
      req = build_request(:delete_local_gateway_route_table_virtual_interface_group_association, params)
      req.send_request(options)
    end

    # Deletes the specified association between a VPC and local gateway
    # route table.
    #
    # @option params [required, String] :local_gateway_route_table_vpc_association_id
    #   The ID of the association.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteLocalGatewayRouteTableVpcAssociationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteLocalGatewayRouteTableVpcAssociationResult#local_gateway_route_table_vpc_association #local_gateway_route_table_vpc_association} => Types::LocalGatewayRouteTableVpcAssociation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_local_gateway_route_table_vpc_association({
    #     local_gateway_route_table_vpc_association_id: "LocalGatewayRouteTableVpcAssociationId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.local_gateway_route_table_vpc_association.local_gateway_route_table_vpc_association_id #=> String
    #   resp.local_gateway_route_table_vpc_association.local_gateway_route_table_id #=> String
    #   resp.local_gateway_route_table_vpc_association.local_gateway_route_table_arn #=> String
    #   resp.local_gateway_route_table_vpc_association.local_gateway_id #=> String
    #   resp.local_gateway_route_table_vpc_association.vpc_id #=> String
    #   resp.local_gateway_route_table_vpc_association.owner_id #=> String
    #   resp.local_gateway_route_table_vpc_association.state #=> String
    #   resp.local_gateway_route_table_vpc_association.tags #=> Array
    #   resp.local_gateway_route_table_vpc_association.tags[0].key #=> String
    #   resp.local_gateway_route_table_vpc_association.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteLocalGatewayRouteTableVpcAssociation AWS API Documentation
    #
    # @overload delete_local_gateway_route_table_vpc_association(params = {})
    # @param [Hash] params ({})
    def delete_local_gateway_route_table_vpc_association(params = {}, options = {})
      req = build_request(:delete_local_gateway_route_table_vpc_association, params)
      req.send_request(options)
    end

    # Deletes the specified managed prefix list. You must first remove all
    # references to the prefix list in your resources.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :prefix_list_id
    #   The ID of the prefix list.
    #
    # @return [Types::DeleteManagedPrefixListResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteManagedPrefixListResult#prefix_list #prefix_list} => Types::ManagedPrefixList
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_managed_prefix_list({
    #     dry_run: false,
    #     prefix_list_id: "PrefixListResourceId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.prefix_list.prefix_list_id #=> String
    #   resp.prefix_list.address_family #=> String
    #   resp.prefix_list.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "restore-in-progress", "restore-complete", "restore-failed", "delete-in-progress", "delete-complete", "delete-failed"
    #   resp.prefix_list.state_message #=> String
    #   resp.prefix_list.prefix_list_arn #=> String
    #   resp.prefix_list.prefix_list_name #=> String
    #   resp.prefix_list.max_entries #=> Integer
    #   resp.prefix_list.version #=> Integer
    #   resp.prefix_list.tags #=> Array
    #   resp.prefix_list.tags[0].key #=> String
    #   resp.prefix_list.tags[0].value #=> String
    #   resp.prefix_list.owner_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteManagedPrefixList AWS API Documentation
    #
    # @overload delete_managed_prefix_list(params = {})
    # @param [Hash] params ({})
    def delete_managed_prefix_list(params = {}, options = {})
      req = build_request(:delete_managed_prefix_list, params)
      req.send_request(options)
    end

    # Deletes the specified NAT gateway. Deleting a public NAT gateway
    # disassociates its Elastic IP address, but does not release the address
    # from your account. Deleting a NAT gateway does not delete any NAT
    # gateway routes in your route tables.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :nat_gateway_id
    #   The ID of the NAT gateway.
    #
    # @return [Types::DeleteNatGatewayResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteNatGatewayResult#nat_gateway_id #nat_gateway_id} => String
    #
    #
    # @example Example: To delete a NAT gateway
    #
    #   # This example deletes the specified NAT gateway.
    #
    #   resp = client.delete_nat_gateway({
    #     nat_gateway_id: "nat-04ae55e711cec5680", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     nat_gateway_id: "nat-04ae55e711cec5680", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_nat_gateway({
    #     dry_run: false,
    #     nat_gateway_id: "NatGatewayId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.nat_gateway_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteNatGateway AWS API Documentation
    #
    # @overload delete_nat_gateway(params = {})
    # @param [Hash] params ({})
    def delete_nat_gateway(params = {}, options = {})
      req = build_request(:delete_nat_gateway, params)
      req.send_request(options)
    end

    # Deletes the specified network ACL. You can't delete the ACL if it's
    # associated with any subnets. You can't delete the default network
    # ACL.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :network_acl_id
    #   The ID of the network ACL.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To delete a network ACL
    #
    #   # This example deletes the specified network ACL.
    #
    #   resp = client.delete_network_acl({
    #     network_acl_id: "acl-5fb85d36", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_network_acl({
    #     dry_run: false,
    #     network_acl_id: "NetworkAclId", # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteNetworkAcl AWS API Documentation
    #
    # @overload delete_network_acl(params = {})
    # @param [Hash] params ({})
    def delete_network_acl(params = {}, options = {})
      req = build_request(:delete_network_acl, params)
      req.send_request(options)
    end

    # Deletes the specified ingress or egress entry (rule) from the
    # specified network ACL.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :network_acl_id
    #   The ID of the network ACL.
    #
    # @option params [required, Integer] :rule_number
    #   The rule number of the entry to delete.
    #
    # @option params [required, Boolean] :egress
    #   Indicates whether the rule is an egress rule.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To delete a network ACL entry
    #
    #   # This example deletes ingress rule number 100 from the specified network ACL.
    #
    #   resp = client.delete_network_acl_entry({
    #     egress: true, 
    #     network_acl_id: "acl-5fb85d36", 
    #     rule_number: 100, 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_network_acl_entry({
    #     dry_run: false,
    #     network_acl_id: "NetworkAclId", # required
    #     rule_number: 1, # required
    #     egress: false, # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteNetworkAclEntry AWS API Documentation
    #
    # @overload delete_network_acl_entry(params = {})
    # @param [Hash] params ({})
    def delete_network_acl_entry(params = {}, options = {})
      req = build_request(:delete_network_acl_entry, params)
      req.send_request(options)
    end

    # Deletes the specified Network Access Scope.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :network_insights_access_scope_id
    #   The ID of the Network Access Scope.
    #
    # @return [Types::DeleteNetworkInsightsAccessScopeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteNetworkInsightsAccessScopeResult#network_insights_access_scope_id #network_insights_access_scope_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_network_insights_access_scope({
    #     dry_run: false,
    #     network_insights_access_scope_id: "NetworkInsightsAccessScopeId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.network_insights_access_scope_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteNetworkInsightsAccessScope AWS API Documentation
    #
    # @overload delete_network_insights_access_scope(params = {})
    # @param [Hash] params ({})
    def delete_network_insights_access_scope(params = {}, options = {})
      req = build_request(:delete_network_insights_access_scope, params)
      req.send_request(options)
    end

    # Deletes the specified Network Access Scope analysis.
    #
    # @option params [required, String] :network_insights_access_scope_analysis_id
    #   The ID of the Network Access Scope analysis.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteNetworkInsightsAccessScopeAnalysisResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteNetworkInsightsAccessScopeAnalysisResult#network_insights_access_scope_analysis_id #network_insights_access_scope_analysis_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_network_insights_access_scope_analysis({
    #     network_insights_access_scope_analysis_id: "NetworkInsightsAccessScopeAnalysisId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.network_insights_access_scope_analysis_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteNetworkInsightsAccessScopeAnalysis AWS API Documentation
    #
    # @overload delete_network_insights_access_scope_analysis(params = {})
    # @param [Hash] params ({})
    def delete_network_insights_access_scope_analysis(params = {}, options = {})
      req = build_request(:delete_network_insights_access_scope_analysis, params)
      req.send_request(options)
    end

    # Deletes the specified network insights analysis.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :network_insights_analysis_id
    #   The ID of the network insights analysis.
    #
    # @return [Types::DeleteNetworkInsightsAnalysisResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteNetworkInsightsAnalysisResult#network_insights_analysis_id #network_insights_analysis_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_network_insights_analysis({
    #     dry_run: false,
    #     network_insights_analysis_id: "NetworkInsightsAnalysisId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.network_insights_analysis_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteNetworkInsightsAnalysis AWS API Documentation
    #
    # @overload delete_network_insights_analysis(params = {})
    # @param [Hash] params ({})
    def delete_network_insights_analysis(params = {}, options = {})
      req = build_request(:delete_network_insights_analysis, params)
      req.send_request(options)
    end

    # Deletes the specified path.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :network_insights_path_id
    #   The ID of the path.
    #
    # @return [Types::DeleteNetworkInsightsPathResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteNetworkInsightsPathResult#network_insights_path_id #network_insights_path_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_network_insights_path({
    #     dry_run: false,
    #     network_insights_path_id: "NetworkInsightsPathId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.network_insights_path_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteNetworkInsightsPath AWS API Documentation
    #
    # @overload delete_network_insights_path(params = {})
    # @param [Hash] params ({})
    def delete_network_insights_path(params = {}, options = {})
      req = build_request(:delete_network_insights_path, params)
      req.send_request(options)
    end

    # Deletes the specified network interface. You must detach the network
    # interface before you can delete it.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :network_interface_id
    #   The ID of the network interface.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To delete a network interface
    #
    #   # This example deletes the specified network interface.
    #
    #   resp = client.delete_network_interface({
    #     network_interface_id: "eni-e5aa89a3", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_network_interface({
    #     dry_run: false,
    #     network_interface_id: "NetworkInterfaceId", # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteNetworkInterface AWS API Documentation
    #
    # @overload delete_network_interface(params = {})
    # @param [Hash] params ({})
    def delete_network_interface(params = {}, options = {})
      req = build_request(:delete_network_interface, params)
      req.send_request(options)
    end

    # Deletes a permission for a network interface. By default, you cannot
    # delete the permission if the account for which you're removing the
    # permission has attached the network interface to an instance. However,
    # you can force delete the permission, regardless of any attachment.
    #
    # @option params [required, String] :network_interface_permission_id
    #   The ID of the network interface permission.
    #
    # @option params [Boolean] :force
    #   Specify `true` to remove the permission even if the network interface
    #   is attached to an instance.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteNetworkInterfacePermissionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteNetworkInterfacePermissionResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_network_interface_permission({
    #     network_interface_permission_id: "NetworkInterfacePermissionId", # required
    #     force: false,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteNetworkInterfacePermission AWS API Documentation
    #
    # @overload delete_network_interface_permission(params = {})
    # @param [Hash] params ({})
    def delete_network_interface_permission(params = {}, options = {})
      req = build_request(:delete_network_interface_permission, params)
      req.send_request(options)
    end

    # Deletes the specified placement group. You must terminate all
    # instances in the placement group before you can delete the placement
    # group. For more information, see [Placement groups][1] in the *Amazon
    # EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/placement-groups.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :group_name
    #   The name of the placement group.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To delete a placement group
    #
    #   # This example deletes the specified placement group.
    #
    #   resp = client.delete_placement_group({
    #     group_name: "my-cluster", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_placement_group({
    #     dry_run: false,
    #     group_name: "PlacementGroupName", # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeletePlacementGroup AWS API Documentation
    #
    # @overload delete_placement_group(params = {})
    # @param [Hash] params ({})
    def delete_placement_group(params = {}, options = {})
      req = build_request(:delete_placement_group, params)
      req.send_request(options)
    end

    # Delete a public IPv4 pool. A public IPv4 pool is an EC2 IP address
    # pool required for the public IPv4 CIDRs that you own and bring to
    # Amazon Web Services to manage with IPAM. IPv6 addresses you bring to
    # Amazon Web Services, however, use IPAM pools only.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :pool_id
    #   The ID of the public IPv4 pool you want to delete.
    #
    # @option params [String] :network_border_group
    #   The Availability Zone (AZ) or Local Zone (LZ) network border group
    #   that the resource that the IP address is assigned to is in. Defaults
    #   to an AZ network border group. For more information on available Local
    #   Zones, see [Local Zone availability][1] in the *Amazon EC2 User
    #   Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-byoip.html#byoip-zone-avail
    #
    # @return [Types::DeletePublicIpv4PoolResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeletePublicIpv4PoolResult#return_value #return_value} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_public_ipv_4_pool({
    #     dry_run: false,
    #     pool_id: "Ipv4PoolEc2Id", # required
    #     network_border_group: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.return_value #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeletePublicIpv4Pool AWS API Documentation
    #
    # @overload delete_public_ipv_4_pool(params = {})
    # @param [Hash] params ({})
    def delete_public_ipv_4_pool(params = {}, options = {})
      req = build_request(:delete_public_ipv_4_pool, params)
      req.send_request(options)
    end

    # Deletes the queued purchases for the specified Reserved Instances.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :reserved_instances_ids
    #   The IDs of the Reserved Instances.
    #
    # @return [Types::DeleteQueuedReservedInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteQueuedReservedInstancesResult#successful_queued_purchase_deletions #successful_queued_purchase_deletions} => Array&lt;Types::SuccessfulQueuedPurchaseDeletion&gt;
    #   * {Types::DeleteQueuedReservedInstancesResult#failed_queued_purchase_deletions #failed_queued_purchase_deletions} => Array&lt;Types::FailedQueuedPurchaseDeletion&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_queued_reserved_instances({
    #     dry_run: false,
    #     reserved_instances_ids: ["ReservationId"], # required
    #   })
    #
    # @example Response structure
    #
    #   resp.successful_queued_purchase_deletions #=> Array
    #   resp.successful_queued_purchase_deletions[0].reserved_instances_id #=> String
    #   resp.failed_queued_purchase_deletions #=> Array
    #   resp.failed_queued_purchase_deletions[0].error.code #=> String, one of "reserved-instances-id-invalid", "reserved-instances-not-in-queued-state", "unexpected-error"
    #   resp.failed_queued_purchase_deletions[0].error.message #=> String
    #   resp.failed_queued_purchase_deletions[0].reserved_instances_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteQueuedReservedInstances AWS API Documentation
    #
    # @overload delete_queued_reserved_instances(params = {})
    # @param [Hash] params ({})
    def delete_queued_reserved_instances(params = {}, options = {})
      req = build_request(:delete_queued_reserved_instances, params)
      req.send_request(options)
    end

    # Deletes the specified route from the specified route table.
    #
    # @option params [String] :destination_prefix_list_id
    #   The ID of the prefix list for the route.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :route_table_id
    #   The ID of the route table.
    #
    # @option params [String] :destination_cidr_block
    #   The IPv4 CIDR range for the route. The value you specify must match
    #   the CIDR for the route exactly.
    #
    # @option params [String] :destination_ipv_6_cidr_block
    #   The IPv6 CIDR range for the route. The value you specify must match
    #   the CIDR for the route exactly.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To delete a route
    #
    #   # This example deletes the specified route from the specified route table.
    #
    #   resp = client.delete_route({
    #     destination_cidr_block: "0.0.0.0/0", 
    #     route_table_id: "rtb-22574640", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_route({
    #     destination_prefix_list_id: "PrefixListResourceId",
    #     dry_run: false,
    #     route_table_id: "RouteTableId", # required
    #     destination_cidr_block: "String",
    #     destination_ipv_6_cidr_block: "String",
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteRoute AWS API Documentation
    #
    # @overload delete_route(params = {})
    # @param [Hash] params ({})
    def delete_route(params = {}, options = {})
      req = build_request(:delete_route, params)
      req.send_request(options)
    end

    # Deletes the specified route table. You must disassociate the route
    # table from any subnets before you can delete it. You can't delete the
    # main route table.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :route_table_id
    #   The ID of the route table.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To delete a route table
    #
    #   # This example deletes the specified route table.
    #
    #   resp = client.delete_route_table({
    #     route_table_id: "rtb-22574640", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_route_table({
    #     dry_run: false,
    #     route_table_id: "RouteTableId", # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteRouteTable AWS API Documentation
    #
    # @overload delete_route_table(params = {})
    # @param [Hash] params ({})
    def delete_route_table(params = {}, options = {})
      req = build_request(:delete_route_table, params)
      req.send_request(options)
    end

    # Deletes a security group.
    #
    # If you attempt to delete a security group that is associated with an
    # instance or network interface, is referenced by another security group
    # in the same VPC, or has a VPC association, the operation fails with
    # `DependencyViolation`.
    #
    # @option params [String] :group_id
    #   The ID of the security group.
    #
    # @option params [String] :group_name
    #   \[Default VPC\] The name of the security group. You can specify either
    #   the security group name or the security group ID. For security groups
    #   in a nondefault VPC, you must specify the security group ID.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteSecurityGroupResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteSecurityGroupResult#return #return} => Boolean
    #   * {Types::DeleteSecurityGroupResult#group_id #group_id} => String
    #
    #
    # @example Example: To delete a security group
    #
    #   # This example deletes the specified security group.
    #
    #   resp = client.delete_security_group({
    #     group_id: "sg-903004f8", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_security_group({
    #     group_id: "SecurityGroupId",
    #     group_name: "SecurityGroupName",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #   resp.group_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteSecurityGroup AWS API Documentation
    #
    # @overload delete_security_group(params = {})
    # @param [Hash] params ({})
    def delete_security_group(params = {}, options = {})
      req = build_request(:delete_security_group, params)
      req.send_request(options)
    end

    # Deletes the specified snapshot.
    #
    # When you make periodic snapshots of a volume, the snapshots are
    # incremental, and only the blocks on the device that have changed since
    # your last snapshot are saved in the new snapshot. When you delete a
    # snapshot, only the data not needed for any other snapshot is removed.
    # So regardless of which prior snapshots have been deleted, all active
    # snapshots will have access to all the information needed to restore
    # the volume.
    #
    # You cannot delete a snapshot of the root device of an EBS volume used
    # by a registered AMI. You must first deregister the AMI before you can
    # delete the snapshot.
    #
    # For more information, see [Delete an Amazon EBS snapshot][1] in the
    # *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-deleting-snapshot.html
    #
    # @option params [required, String] :snapshot_id
    #   The ID of the EBS snapshot.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To delete a snapshot
    #
    #   # This example deletes a snapshot with the snapshot ID of ``snap-1234567890abcdef0``. If the command succeeds, no output
    #   # is returned.
    #
    #   resp = client.delete_snapshot({
    #     snapshot_id: "snap-1234567890abcdef0", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_snapshot({
    #     snapshot_id: "SnapshotId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteSnapshot AWS API Documentation
    #
    # @overload delete_snapshot(params = {})
    # @param [Hash] params ({})
    def delete_snapshot(params = {}, options = {})
      req = build_request(:delete_snapshot, params)
      req.send_request(options)
    end

    # Deletes the data feed for Spot Instances.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To cancel a Spot Instance data feed subscription
    #
    #   # This example deletes a Spot data feed subscription for the account.
    #
    #   resp = client.delete_spot_datafeed_subscription({
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_spot_datafeed_subscription({
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteSpotDatafeedSubscription AWS API Documentation
    #
    # @overload delete_spot_datafeed_subscription(params = {})
    # @param [Hash] params ({})
    def delete_spot_datafeed_subscription(params = {}, options = {})
      req = build_request(:delete_spot_datafeed_subscription, params)
      req.send_request(options)
    end

    # Deletes the specified subnet. You must terminate all running instances
    # in the subnet before you can delete the subnet.
    #
    # @option params [required, String] :subnet_id
    #   The ID of the subnet.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To delete a subnet
    #
    #   # This example deletes the specified subnet.
    #
    #   resp = client.delete_subnet({
    #     subnet_id: "subnet-9d4a7b6c", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_subnet({
    #     subnet_id: "SubnetId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteSubnet AWS API Documentation
    #
    # @overload delete_subnet(params = {})
    # @param [Hash] params ({})
    def delete_subnet(params = {}, options = {})
      req = build_request(:delete_subnet, params)
      req.send_request(options)
    end

    # Deletes a subnet CIDR reservation.
    #
    # @option params [required, String] :subnet_cidr_reservation_id
    #   The ID of the subnet CIDR reservation.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteSubnetCidrReservationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteSubnetCidrReservationResult#deleted_subnet_cidr_reservation #deleted_subnet_cidr_reservation} => Types::SubnetCidrReservation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_subnet_cidr_reservation({
    #     subnet_cidr_reservation_id: "SubnetCidrReservationId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.deleted_subnet_cidr_reservation.subnet_cidr_reservation_id #=> String
    #   resp.deleted_subnet_cidr_reservation.subnet_id #=> String
    #   resp.deleted_subnet_cidr_reservation.cidr #=> String
    #   resp.deleted_subnet_cidr_reservation.reservation_type #=> String, one of "prefix", "explicit"
    #   resp.deleted_subnet_cidr_reservation.owner_id #=> String
    #   resp.deleted_subnet_cidr_reservation.description #=> String
    #   resp.deleted_subnet_cidr_reservation.tags #=> Array
    #   resp.deleted_subnet_cidr_reservation.tags[0].key #=> String
    #   resp.deleted_subnet_cidr_reservation.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteSubnetCidrReservation AWS API Documentation
    #
    # @overload delete_subnet_cidr_reservation(params = {})
    # @param [Hash] params ({})
    def delete_subnet_cidr_reservation(params = {}, options = {})
      req = build_request(:delete_subnet_cidr_reservation, params)
      req.send_request(options)
    end

    # Deletes the specified set of tags from the specified set of resources.
    #
    # To list the current tags, use DescribeTags. For more information about
    # tags, see [Tag your Amazon EC2 resources][1] in the *Amazon Elastic
    # Compute Cloud User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :resources
    #   The IDs of the resources, separated by spaces.
    #
    #   Constraints: Up to 1000 resource IDs. We recommend breaking up this
    #   request into smaller batches.
    #
    # @option params [Array<Types::Tag>] :tags
    #   The tags to delete. Specify a tag key and an optional tag value to
    #   delete specific tags. If you specify a tag key without a tag value, we
    #   delete any tag with this key regardless of its value. If you specify a
    #   tag key with an empty string as the tag value, we delete the tag only
    #   if its value is an empty string.
    #
    #   If you omit this parameter, we delete all user-defined tags for the
    #   specified resources. We do not delete Amazon Web Services-generated
    #   tags (tags that have the `aws:` prefix).
    #
    #   Constraints: Up to 1000 tags.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To delete a tag from a resource
    #
    #   # This example deletes the tag Stack=test from the specified image.
    #
    #   resp = client.delete_tags({
    #     resources: [
    #       "ami-78a54011", 
    #     ], 
    #     tags: [
    #       {
    #         key: "Stack", 
    #         value: "test", 
    #       }, 
    #     ], 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_tags({
    #     dry_run: false,
    #     resources: ["TaggableResourceId"], # required
    #     tags: [
    #       {
    #         key: "String",
    #         value: "String",
    #       },
    #     ],
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTags AWS API Documentation
    #
    # @overload delete_tags(params = {})
    # @param [Hash] params ({})
    def delete_tags(params = {}, options = {})
      req = build_request(:delete_tags, params)
      req.send_request(options)
    end

    # Deletes the specified Traffic Mirror filter.
    #
    # You cannot delete a Traffic Mirror filter that is in use by a Traffic
    # Mirror session.
    #
    # @option params [required, String] :traffic_mirror_filter_id
    #   The ID of the Traffic Mirror filter.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTrafficMirrorFilterResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTrafficMirrorFilterResult#traffic_mirror_filter_id #traffic_mirror_filter_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_traffic_mirror_filter({
    #     traffic_mirror_filter_id: "TrafficMirrorFilterId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_filter_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTrafficMirrorFilter AWS API Documentation
    #
    # @overload delete_traffic_mirror_filter(params = {})
    # @param [Hash] params ({})
    def delete_traffic_mirror_filter(params = {}, options = {})
      req = build_request(:delete_traffic_mirror_filter, params)
      req.send_request(options)
    end

    # Deletes the specified Traffic Mirror rule.
    #
    # @option params [required, String] :traffic_mirror_filter_rule_id
    #   The ID of the Traffic Mirror rule.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTrafficMirrorFilterRuleResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTrafficMirrorFilterRuleResult#traffic_mirror_filter_rule_id #traffic_mirror_filter_rule_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_traffic_mirror_filter_rule({
    #     traffic_mirror_filter_rule_id: "TrafficMirrorFilterRuleIdWithResolver", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_filter_rule_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTrafficMirrorFilterRule AWS API Documentation
    #
    # @overload delete_traffic_mirror_filter_rule(params = {})
    # @param [Hash] params ({})
    def delete_traffic_mirror_filter_rule(params = {}, options = {})
      req = build_request(:delete_traffic_mirror_filter_rule, params)
      req.send_request(options)
    end

    # Deletes the specified Traffic Mirror session.
    #
    # @option params [required, String] :traffic_mirror_session_id
    #   The ID of the Traffic Mirror session.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTrafficMirrorSessionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTrafficMirrorSessionResult#traffic_mirror_session_id #traffic_mirror_session_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_traffic_mirror_session({
    #     traffic_mirror_session_id: "TrafficMirrorSessionId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_session_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTrafficMirrorSession AWS API Documentation
    #
    # @overload delete_traffic_mirror_session(params = {})
    # @param [Hash] params ({})
    def delete_traffic_mirror_session(params = {}, options = {})
      req = build_request(:delete_traffic_mirror_session, params)
      req.send_request(options)
    end

    # Deletes the specified Traffic Mirror target.
    #
    # You cannot delete a Traffic Mirror target that is in use by a Traffic
    # Mirror session.
    #
    # @option params [required, String] :traffic_mirror_target_id
    #   The ID of the Traffic Mirror target.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTrafficMirrorTargetResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTrafficMirrorTargetResult#traffic_mirror_target_id #traffic_mirror_target_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_traffic_mirror_target({
    #     traffic_mirror_target_id: "TrafficMirrorTargetId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_target_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTrafficMirrorTarget AWS API Documentation
    #
    # @overload delete_traffic_mirror_target(params = {})
    # @param [Hash] params ({})
    def delete_traffic_mirror_target(params = {}, options = {})
      req = build_request(:delete_traffic_mirror_target, params)
      req.send_request(options)
    end

    # Deletes the specified transit gateway.
    #
    # @option params [required, String] :transit_gateway_id
    #   The ID of the transit gateway.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTransitGatewayResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTransitGatewayResult#transit_gateway #transit_gateway} => Types::TransitGateway
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_transit_gateway({
    #     transit_gateway_id: "TransitGatewayId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway.transit_gateway_id #=> String
    #   resp.transit_gateway.transit_gateway_arn #=> String
    #   resp.transit_gateway.state #=> String, one of "pending", "available", "modifying", "deleting", "deleted"
    #   resp.transit_gateway.owner_id #=> String
    #   resp.transit_gateway.description #=> String
    #   resp.transit_gateway.creation_time #=> Time
    #   resp.transit_gateway.options.amazon_side_asn #=> Integer
    #   resp.transit_gateway.options.transit_gateway_cidr_blocks #=> Array
    #   resp.transit_gateway.options.transit_gateway_cidr_blocks[0] #=> String
    #   resp.transit_gateway.options.auto_accept_shared_attachments #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.default_route_table_association #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.association_default_route_table_id #=> String
    #   resp.transit_gateway.options.default_route_table_propagation #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.propagation_default_route_table_id #=> String
    #   resp.transit_gateway.options.vpn_ecmp_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.dns_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.security_group_referencing_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.multicast_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway.tags #=> Array
    #   resp.transit_gateway.tags[0].key #=> String
    #   resp.transit_gateway.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTransitGateway AWS API Documentation
    #
    # @overload delete_transit_gateway(params = {})
    # @param [Hash] params ({})
    def delete_transit_gateway(params = {}, options = {})
      req = build_request(:delete_transit_gateway, params)
      req.send_request(options)
    end

    # Deletes the specified Connect attachment. You must first delete any
    # Connect peers for the attachment.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the Connect attachment.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTransitGatewayConnectResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTransitGatewayConnectResult#transit_gateway_connect #transit_gateway_connect} => Types::TransitGatewayConnect
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_transit_gateway_connect({
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_connect.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_connect.transport_transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_connect.transit_gateway_id #=> String
    #   resp.transit_gateway_connect.state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_connect.creation_time #=> Time
    #   resp.transit_gateway_connect.options.protocol #=> String, one of "gre"
    #   resp.transit_gateway_connect.tags #=> Array
    #   resp.transit_gateway_connect.tags[0].key #=> String
    #   resp.transit_gateway_connect.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTransitGatewayConnect AWS API Documentation
    #
    # @overload delete_transit_gateway_connect(params = {})
    # @param [Hash] params ({})
    def delete_transit_gateway_connect(params = {}, options = {})
      req = build_request(:delete_transit_gateway_connect, params)
      req.send_request(options)
    end

    # Deletes the specified Connect peer.
    #
    # @option params [required, String] :transit_gateway_connect_peer_id
    #   The ID of the Connect peer.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTransitGatewayConnectPeerResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTransitGatewayConnectPeerResult#transit_gateway_connect_peer #transit_gateway_connect_peer} => Types::TransitGatewayConnectPeer
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_transit_gateway_connect_peer({
    #     transit_gateway_connect_peer_id: "TransitGatewayConnectPeerId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_connect_peer.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_connect_peer.transit_gateway_connect_peer_id #=> String
    #   resp.transit_gateway_connect_peer.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.transit_gateway_connect_peer.creation_time #=> Time
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.transit_gateway_address #=> String
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.peer_address #=> String
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.inside_cidr_blocks #=> Array
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.inside_cidr_blocks[0] #=> String
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.protocol #=> String, one of "gre"
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.bgp_configurations #=> Array
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.bgp_configurations[0].transit_gateway_asn #=> Integer
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.bgp_configurations[0].peer_asn #=> Integer
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.bgp_configurations[0].transit_gateway_address #=> String
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.bgp_configurations[0].peer_address #=> String
    #   resp.transit_gateway_connect_peer.connect_peer_configuration.bgp_configurations[0].bgp_status #=> String, one of "up", "down"
    #   resp.transit_gateway_connect_peer.tags #=> Array
    #   resp.transit_gateway_connect_peer.tags[0].key #=> String
    #   resp.transit_gateway_connect_peer.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTransitGatewayConnectPeer AWS API Documentation
    #
    # @overload delete_transit_gateway_connect_peer(params = {})
    # @param [Hash] params ({})
    def delete_transit_gateway_connect_peer(params = {}, options = {})
      req = build_request(:delete_transit_gateway_connect_peer, params)
      req.send_request(options)
    end

    # Deletes the specified transit gateway multicast domain.
    #
    # @option params [required, String] :transit_gateway_multicast_domain_id
    #   The ID of the transit gateway multicast domain.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTransitGatewayMulticastDomainResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTransitGatewayMulticastDomainResult#transit_gateway_multicast_domain #transit_gateway_multicast_domain} => Types::TransitGatewayMulticastDomain
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_transit_gateway_multicast_domain({
    #     transit_gateway_multicast_domain_id: "TransitGatewayMulticastDomainId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_multicast_domain.transit_gateway_multicast_domain_id #=> String
    #   resp.transit_gateway_multicast_domain.transit_gateway_id #=> String
    #   resp.transit_gateway_multicast_domain.transit_gateway_multicast_domain_arn #=> String
    #   resp.transit_gateway_multicast_domain.owner_id #=> String
    #   resp.transit_gateway_multicast_domain.options.igmpv_2_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_multicast_domain.options.static_sources_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_multicast_domain.options.auto_accept_shared_associations #=> String, one of "enable", "disable"
    #   resp.transit_gateway_multicast_domain.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.transit_gateway_multicast_domain.creation_time #=> Time
    #   resp.transit_gateway_multicast_domain.tags #=> Array
    #   resp.transit_gateway_multicast_domain.tags[0].key #=> String
    #   resp.transit_gateway_multicast_domain.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTransitGatewayMulticastDomain AWS API Documentation
    #
    # @overload delete_transit_gateway_multicast_domain(params = {})
    # @param [Hash] params ({})
    def delete_transit_gateway_multicast_domain(params = {}, options = {})
      req = build_request(:delete_transit_gateway_multicast_domain, params)
      req.send_request(options)
    end

    # Deletes a transit gateway peering attachment.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the transit gateway peering attachment.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTransitGatewayPeeringAttachmentResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTransitGatewayPeeringAttachmentResult#transit_gateway_peering_attachment #transit_gateway_peering_attachment} => Types::TransitGatewayPeeringAttachment
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_transit_gateway_peering_attachment({
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_peering_attachment.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.transit_gateway_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.core_network_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.owner_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.region #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.transit_gateway_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.core_network_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.owner_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.region #=> String
    #   resp.transit_gateway_peering_attachment.options.dynamic_routing #=> String, one of "enable", "disable"
    #   resp.transit_gateway_peering_attachment.status.code #=> String
    #   resp.transit_gateway_peering_attachment.status.message #=> String
    #   resp.transit_gateway_peering_attachment.state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_peering_attachment.creation_time #=> Time
    #   resp.transit_gateway_peering_attachment.tags #=> Array
    #   resp.transit_gateway_peering_attachment.tags[0].key #=> String
    #   resp.transit_gateway_peering_attachment.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTransitGatewayPeeringAttachment AWS API Documentation
    #
    # @overload delete_transit_gateway_peering_attachment(params = {})
    # @param [Hash] params ({})
    def delete_transit_gateway_peering_attachment(params = {}, options = {})
      req = build_request(:delete_transit_gateway_peering_attachment, params)
      req.send_request(options)
    end

    # Deletes the specified transit gateway policy table.
    #
    # @option params [required, String] :transit_gateway_policy_table_id
    #   The transit gateway policy table to delete.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTransitGatewayPolicyTableResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTransitGatewayPolicyTableResult#transit_gateway_policy_table #transit_gateway_policy_table} => Types::TransitGatewayPolicyTable
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_transit_gateway_policy_table({
    #     transit_gateway_policy_table_id: "TransitGatewayPolicyTableId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_policy_table.transit_gateway_policy_table_id #=> String
    #   resp.transit_gateway_policy_table.transit_gateway_id #=> String
    #   resp.transit_gateway_policy_table.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.transit_gateway_policy_table.creation_time #=> Time
    #   resp.transit_gateway_policy_table.tags #=> Array
    #   resp.transit_gateway_policy_table.tags[0].key #=> String
    #   resp.transit_gateway_policy_table.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTransitGatewayPolicyTable AWS API Documentation
    #
    # @overload delete_transit_gateway_policy_table(params = {})
    # @param [Hash] params ({})
    def delete_transit_gateway_policy_table(params = {}, options = {})
      req = build_request(:delete_transit_gateway_policy_table, params)
      req.send_request(options)
    end

    # Deletes a reference (route) to a prefix list in a specified transit
    # gateway route table.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the route table.
    #
    # @option params [required, String] :prefix_list_id
    #   The ID of the prefix list.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTransitGatewayPrefixListReferenceResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTransitGatewayPrefixListReferenceResult#transit_gateway_prefix_list_reference #transit_gateway_prefix_list_reference} => Types::TransitGatewayPrefixListReference
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_transit_gateway_prefix_list_reference({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     prefix_list_id: "PrefixListResourceId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_prefix_list_reference.transit_gateway_route_table_id #=> String
    #   resp.transit_gateway_prefix_list_reference.prefix_list_id #=> String
    #   resp.transit_gateway_prefix_list_reference.prefix_list_owner_id #=> String
    #   resp.transit_gateway_prefix_list_reference.state #=> String, one of "pending", "available", "modifying", "deleting"
    #   resp.transit_gateway_prefix_list_reference.blackhole #=> Boolean
    #   resp.transit_gateway_prefix_list_reference.transit_gateway_attachment.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_prefix_list_reference.transit_gateway_attachment.resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.transit_gateway_prefix_list_reference.transit_gateway_attachment.resource_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTransitGatewayPrefixListReference AWS API Documentation
    #
    # @overload delete_transit_gateway_prefix_list_reference(params = {})
    # @param [Hash] params ({})
    def delete_transit_gateway_prefix_list_reference(params = {}, options = {})
      req = build_request(:delete_transit_gateway_prefix_list_reference, params)
      req.send_request(options)
    end

    # Deletes the specified route from the specified transit gateway route
    # table.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the transit gateway route table.
    #
    # @option params [required, String] :destination_cidr_block
    #   The CIDR range for the route. This must match the CIDR for the route
    #   exactly.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTransitGatewayRouteResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTransitGatewayRouteResult#route #route} => Types::TransitGatewayRoute
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_transit_gateway_route({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     destination_cidr_block: "String", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.route.destination_cidr_block #=> String
    #   resp.route.prefix_list_id #=> String
    #   resp.route.transit_gateway_route_table_announcement_id #=> String
    #   resp.route.transit_gateway_attachments #=> Array
    #   resp.route.transit_gateway_attachments[0].resource_id #=> String
    #   resp.route.transit_gateway_attachments[0].transit_gateway_attachment_id #=> String
    #   resp.route.transit_gateway_attachments[0].resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.route.type #=> String, one of "static", "propagated"
    #   resp.route.state #=> String, one of "pending", "active", "blackhole", "deleting", "deleted"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTransitGatewayRoute AWS API Documentation
    #
    # @overload delete_transit_gateway_route(params = {})
    # @param [Hash] params ({})
    def delete_transit_gateway_route(params = {}, options = {})
      req = build_request(:delete_transit_gateway_route, params)
      req.send_request(options)
    end

    # Deletes the specified transit gateway route table. If there are any
    # route tables associated with the transit gateway route table, you must
    # first run DisassociateRouteTable before you can delete the transit
    # gateway route table. This removes any route tables associated with the
    # transit gateway route table.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the transit gateway route table.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTransitGatewayRouteTableResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTransitGatewayRouteTableResult#transit_gateway_route_table #transit_gateway_route_table} => Types::TransitGatewayRouteTable
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_transit_gateway_route_table({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_route_table.transit_gateway_route_table_id #=> String
    #   resp.transit_gateway_route_table.transit_gateway_id #=> String
    #   resp.transit_gateway_route_table.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.transit_gateway_route_table.default_association_route_table #=> Boolean
    #   resp.transit_gateway_route_table.default_propagation_route_table #=> Boolean
    #   resp.transit_gateway_route_table.creation_time #=> Time
    #   resp.transit_gateway_route_table.tags #=> Array
    #   resp.transit_gateway_route_table.tags[0].key #=> String
    #   resp.transit_gateway_route_table.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTransitGatewayRouteTable AWS API Documentation
    #
    # @overload delete_transit_gateway_route_table(params = {})
    # @param [Hash] params ({})
    def delete_transit_gateway_route_table(params = {}, options = {})
      req = build_request(:delete_transit_gateway_route_table, params)
      req.send_request(options)
    end

    # Advertises to the transit gateway that a transit gateway route table
    # is deleted.
    #
    # @option params [required, String] :transit_gateway_route_table_announcement_id
    #   The transit gateway route table ID that's being deleted.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTransitGatewayRouteTableAnnouncementResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTransitGatewayRouteTableAnnouncementResult#transit_gateway_route_table_announcement #transit_gateway_route_table_announcement} => Types::TransitGatewayRouteTableAnnouncement
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_transit_gateway_route_table_announcement({
    #     transit_gateway_route_table_announcement_id: "TransitGatewayRouteTableAnnouncementId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_route_table_announcement.transit_gateway_route_table_announcement_id #=> String
    #   resp.transit_gateway_route_table_announcement.transit_gateway_id #=> String
    #   resp.transit_gateway_route_table_announcement.core_network_id #=> String
    #   resp.transit_gateway_route_table_announcement.peer_transit_gateway_id #=> String
    #   resp.transit_gateway_route_table_announcement.peer_core_network_id #=> String
    #   resp.transit_gateway_route_table_announcement.peering_attachment_id #=> String
    #   resp.transit_gateway_route_table_announcement.announcement_direction #=> String, one of "outgoing", "incoming"
    #   resp.transit_gateway_route_table_announcement.transit_gateway_route_table_id #=> String
    #   resp.transit_gateway_route_table_announcement.state #=> String, one of "available", "pending", "failing", "failed", "deleting", "deleted"
    #   resp.transit_gateway_route_table_announcement.creation_time #=> Time
    #   resp.transit_gateway_route_table_announcement.tags #=> Array
    #   resp.transit_gateway_route_table_announcement.tags[0].key #=> String
    #   resp.transit_gateway_route_table_announcement.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTransitGatewayRouteTableAnnouncement AWS API Documentation
    #
    # @overload delete_transit_gateway_route_table_announcement(params = {})
    # @param [Hash] params ({})
    def delete_transit_gateway_route_table_announcement(params = {}, options = {})
      req = build_request(:delete_transit_gateway_route_table_announcement, params)
      req.send_request(options)
    end

    # Deletes the specified VPC attachment.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the attachment.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteTransitGatewayVpcAttachmentResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteTransitGatewayVpcAttachmentResult#transit_gateway_vpc_attachment #transit_gateway_vpc_attachment} => Types::TransitGatewayVpcAttachment
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_transit_gateway_vpc_attachment({
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_vpc_attachment.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_vpc_attachment.transit_gateway_id #=> String
    #   resp.transit_gateway_vpc_attachment.vpc_id #=> String
    #   resp.transit_gateway_vpc_attachment.vpc_owner_id #=> String
    #   resp.transit_gateway_vpc_attachment.state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_vpc_attachment.subnet_ids #=> Array
    #   resp.transit_gateway_vpc_attachment.subnet_ids[0] #=> String
    #   resp.transit_gateway_vpc_attachment.creation_time #=> Time
    #   resp.transit_gateway_vpc_attachment.options.dns_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.security_group_referencing_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.ipv_6_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.appliance_mode_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.tags #=> Array
    #   resp.transit_gateway_vpc_attachment.tags[0].key #=> String
    #   resp.transit_gateway_vpc_attachment.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteTransitGatewayVpcAttachment AWS API Documentation
    #
    # @overload delete_transit_gateway_vpc_attachment(params = {})
    # @param [Hash] params ({})
    def delete_transit_gateway_vpc_attachment(params = {}, options = {})
      req = build_request(:delete_transit_gateway_vpc_attachment, params)
      req.send_request(options)
    end

    # Delete an Amazon Web Services Verified Access endpoint.
    #
    # @option params [required, String] :verified_access_endpoint_id
    #   The ID of the Verified Access endpoint.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteVerifiedAccessEndpointResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteVerifiedAccessEndpointResult#verified_access_endpoint #verified_access_endpoint} => Types::VerifiedAccessEndpoint
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_verified_access_endpoint({
    #     verified_access_endpoint_id: "VerifiedAccessEndpointId", # required
    #     client_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_endpoint.verified_access_instance_id #=> String
    #   resp.verified_access_endpoint.verified_access_group_id #=> String
    #   resp.verified_access_endpoint.verified_access_endpoint_id #=> String
    #   resp.verified_access_endpoint.application_domain #=> String
    #   resp.verified_access_endpoint.endpoint_type #=> String, one of "load-balancer", "network-interface", "rds", "cidr"
    #   resp.verified_access_endpoint.attachment_type #=> String, one of "vpc"
    #   resp.verified_access_endpoint.domain_certificate_arn #=> String
    #   resp.verified_access_endpoint.endpoint_domain #=> String
    #   resp.verified_access_endpoint.device_validation_domain #=> String
    #   resp.verified_access_endpoint.security_group_ids #=> Array
    #   resp.verified_access_endpoint.security_group_ids[0] #=> String
    #   resp.verified_access_endpoint.load_balancer_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoint.load_balancer_options.port #=> Integer
    #   resp.verified_access_endpoint.load_balancer_options.load_balancer_arn #=> String
    #   resp.verified_access_endpoint.load_balancer_options.subnet_ids #=> Array
    #   resp.verified_access_endpoint.load_balancer_options.subnet_ids[0] #=> String
    #   resp.verified_access_endpoint.load_balancer_options.port_ranges #=> Array
    #   resp.verified_access_endpoint.load_balancer_options.port_ranges[0].from_port #=> Integer
    #   resp.verified_access_endpoint.load_balancer_options.port_ranges[0].to_port #=> Integer
    #   resp.verified_access_endpoint.network_interface_options.network_interface_id #=> String
    #   resp.verified_access_endpoint.network_interface_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoint.network_interface_options.port #=> Integer
    #   resp.verified_access_endpoint.network_interface_options.port_ranges #=> Array
    #   resp.verified_access_endpoint.network_interface_options.port_ranges[0].from_port #=> Integer
    #   resp.verified_access_endpoint.network_interface_options.port_ranges[0].to_port #=> Integer
    #   resp.verified_access_endpoint.status.code #=> String, one of "pending", "active", "updating", "deleting", "deleted"
    #   resp.verified_access_endpoint.status.message #=> String
    #   resp.verified_access_endpoint.description #=> String
    #   resp.verified_access_endpoint.creation_time #=> String
    #   resp.verified_access_endpoint.last_updated_time #=> String
    #   resp.verified_access_endpoint.deletion_time #=> String
    #   resp.verified_access_endpoint.tags #=> Array
    #   resp.verified_access_endpoint.tags[0].key #=> String
    #   resp.verified_access_endpoint.tags[0].value #=> String
    #   resp.verified_access_endpoint.sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.verified_access_endpoint.sse_specification.kms_key_arn #=> String
    #   resp.verified_access_endpoint.rds_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoint.rds_options.port #=> Integer
    #   resp.verified_access_endpoint.rds_options.rds_db_instance_arn #=> String
    #   resp.verified_access_endpoint.rds_options.rds_db_cluster_arn #=> String
    #   resp.verified_access_endpoint.rds_options.rds_db_proxy_arn #=> String
    #   resp.verified_access_endpoint.rds_options.rds_endpoint #=> String
    #   resp.verified_access_endpoint.rds_options.subnet_ids #=> Array
    #   resp.verified_access_endpoint.rds_options.subnet_ids[0] #=> String
    #   resp.verified_access_endpoint.cidr_options.cidr #=> String
    #   resp.verified_access_endpoint.cidr_options.port_ranges #=> Array
    #   resp.verified_access_endpoint.cidr_options.port_ranges[0].from_port #=> Integer
    #   resp.verified_access_endpoint.cidr_options.port_ranges[0].to_port #=> Integer
    #   resp.verified_access_endpoint.cidr_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoint.cidr_options.subnet_ids #=> Array
    #   resp.verified_access_endpoint.cidr_options.subnet_ids[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteVerifiedAccessEndpoint AWS API Documentation
    #
    # @overload delete_verified_access_endpoint(params = {})
    # @param [Hash] params ({})
    def delete_verified_access_endpoint(params = {}, options = {})
      req = build_request(:delete_verified_access_endpoint, params)
      req.send_request(options)
    end

    # Delete an Amazon Web Services Verified Access group.
    #
    # @option params [required, String] :verified_access_group_id
    #   The ID of the Verified Access group.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeleteVerifiedAccessGroupResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteVerifiedAccessGroupResult#verified_access_group #verified_access_group} => Types::VerifiedAccessGroup
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_verified_access_group({
    #     verified_access_group_id: "VerifiedAccessGroupId", # required
    #     client_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_group.verified_access_group_id #=> String
    #   resp.verified_access_group.verified_access_instance_id #=> String
    #   resp.verified_access_group.description #=> String
    #   resp.verified_access_group.owner #=> String
    #   resp.verified_access_group.verified_access_group_arn #=> String
    #   resp.verified_access_group.creation_time #=> String
    #   resp.verified_access_group.last_updated_time #=> String
    #   resp.verified_access_group.deletion_time #=> String
    #   resp.verified_access_group.tags #=> Array
    #   resp.verified_access_group.tags[0].key #=> String
    #   resp.verified_access_group.tags[0].value #=> String
    #   resp.verified_access_group.sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.verified_access_group.sse_specification.kms_key_arn #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteVerifiedAccessGroup AWS API Documentation
    #
    # @overload delete_verified_access_group(params = {})
    # @param [Hash] params ({})
    def delete_verified_access_group(params = {}, options = {})
      req = build_request(:delete_verified_access_group, params)
      req.send_request(options)
    end

    # Delete an Amazon Web Services Verified Access instance.
    #
    # @option params [required, String] :verified_access_instance_id
    #   The ID of the Verified Access instance.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @return [Types::DeleteVerifiedAccessInstanceResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteVerifiedAccessInstanceResult#verified_access_instance #verified_access_instance} => Types::VerifiedAccessInstance
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_verified_access_instance({
    #     verified_access_instance_id: "VerifiedAccessInstanceId", # required
    #     dry_run: false,
    #     client_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_instance.verified_access_instance_id #=> String
    #   resp.verified_access_instance.description #=> String
    #   resp.verified_access_instance.verified_access_trust_providers #=> Array
    #   resp.verified_access_instance.verified_access_trust_providers[0].verified_access_trust_provider_id #=> String
    #   resp.verified_access_instance.verified_access_trust_providers[0].description #=> String
    #   resp.verified_access_instance.verified_access_trust_providers[0].trust_provider_type #=> String, one of "user", "device"
    #   resp.verified_access_instance.verified_access_trust_providers[0].user_trust_provider_type #=> String, one of "iam-identity-center", "oidc"
    #   resp.verified_access_instance.verified_access_trust_providers[0].device_trust_provider_type #=> String, one of "jamf", "crowdstrike", "jumpcloud"
    #   resp.verified_access_instance.creation_time #=> String
    #   resp.verified_access_instance.last_updated_time #=> String
    #   resp.verified_access_instance.tags #=> Array
    #   resp.verified_access_instance.tags[0].key #=> String
    #   resp.verified_access_instance.tags[0].value #=> String
    #   resp.verified_access_instance.fips_enabled #=> Boolean
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.sub_domain #=> String
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.nameservers #=> Array
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.nameservers[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteVerifiedAccessInstance AWS API Documentation
    #
    # @overload delete_verified_access_instance(params = {})
    # @param [Hash] params ({})
    def delete_verified_access_instance(params = {}, options = {})
      req = build_request(:delete_verified_access_instance, params)
      req.send_request(options)
    end

    # Delete an Amazon Web Services Verified Access trust provider.
    #
    # @option params [required, String] :verified_access_trust_provider_id
    #   The ID of the Verified Access trust provider.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @return [Types::DeleteVerifiedAccessTrustProviderResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteVerifiedAccessTrustProviderResult#verified_access_trust_provider #verified_access_trust_provider} => Types::VerifiedAccessTrustProvider
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_verified_access_trust_provider({
    #     verified_access_trust_provider_id: "VerifiedAccessTrustProviderId", # required
    #     dry_run: false,
    #     client_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_trust_provider.verified_access_trust_provider_id #=> String
    #   resp.verified_access_trust_provider.description #=> String
    #   resp.verified_access_trust_provider.trust_provider_type #=> String, one of "user", "device"
    #   resp.verified_access_trust_provider.user_trust_provider_type #=> String, one of "iam-identity-center", "oidc"
    #   resp.verified_access_trust_provider.device_trust_provider_type #=> String, one of "jamf", "crowdstrike", "jumpcloud"
    #   resp.verified_access_trust_provider.oidc_options.issuer #=> String
    #   resp.verified_access_trust_provider.oidc_options.authorization_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.token_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.user_info_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.client_id #=> String
    #   resp.verified_access_trust_provider.oidc_options.client_secret #=> String
    #   resp.verified_access_trust_provider.oidc_options.scope #=> String
    #   resp.verified_access_trust_provider.device_options.tenant_id #=> String
    #   resp.verified_access_trust_provider.device_options.public_signing_key_url #=> String
    #   resp.verified_access_trust_provider.policy_reference_name #=> String
    #   resp.verified_access_trust_provider.creation_time #=> String
    #   resp.verified_access_trust_provider.last_updated_time #=> String
    #   resp.verified_access_trust_provider.tags #=> Array
    #   resp.verified_access_trust_provider.tags[0].key #=> String
    #   resp.verified_access_trust_provider.tags[0].value #=> String
    #   resp.verified_access_trust_provider.sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.verified_access_trust_provider.sse_specification.kms_key_arn #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.public_signing_key_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.issuer #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.authorization_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.token_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.user_info_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.client_id #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.scope #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteVerifiedAccessTrustProvider AWS API Documentation
    #
    # @overload delete_verified_access_trust_provider(params = {})
    # @param [Hash] params ({})
    def delete_verified_access_trust_provider(params = {}, options = {})
      req = build_request(:delete_verified_access_trust_provider, params)
      req.send_request(options)
    end

    # Deletes the specified EBS volume. The volume must be in the
    # `available` state (not attached to an instance).
    #
    # The volume can remain in the `deleting` state for several minutes.
    #
    # For more information, see [Delete an Amazon EBS volume][1] in the
    # *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-deleting-volume.html
    #
    # @option params [required, String] :volume_id
    #   The ID of the volume.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To delete a volume
    #
    #   # This example deletes an available volume with the volume ID of ``vol-049df61146c4d7901``. If the command succeeds, no
    #   # output is returned.
    #
    #   resp = client.delete_volume({
    #     volume_id: "vol-049df61146c4d7901", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_volume({
    #     volume_id: "VolumeId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteVolume AWS API Documentation
    #
    # @overload delete_volume(params = {})
    # @param [Hash] params ({})
    def delete_volume(params = {}, options = {})
      req = build_request(:delete_volume, params)
      req.send_request(options)
    end

    # Deletes the specified VPC. You must detach or delete all gateways and
    # resources that are associated with the VPC before you can delete it.
    # For example, you must terminate all instances running in the VPC,
    # delete all security groups associated with the VPC (except the default
    # one), delete all route tables associated with the VPC (except the
    # default one), and so on. When you delete the VPC, it deletes the
    # default security group, network ACL, and route table for the VPC.
    #
    # If you created a flow log for the VPC that you are deleting, note that
    # flow logs for deleted VPCs are eventually automatically removed.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To delete a VPC
    #
    #   # This example deletes the specified VPC.
    #
    #   resp = client.delete_vpc({
    #     vpc_id: "vpc-a01106c2", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_vpc({
    #     vpc_id: "VpcId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteVpc AWS API Documentation
    #
    # @overload delete_vpc(params = {})
    # @param [Hash] params ({})
    def delete_vpc(params = {}, options = {})
      req = build_request(:delete_vpc, params)
      req.send_request(options)
    end

    # Delete a VPC Block Public Access (BPA) exclusion. A VPC BPA exclusion
    # is a mode that can be applied to a single VPC or subnet that exempts
    # it from the account’s BPA mode and will allow bidirectional or
    # egress-only access. You can create BPA exclusions for VPCs and subnets
    # even when BPA is not enabled on the account to ensure that there is no
    # traffic disruption to the exclusions when VPC BPA is turned on. To
    # learn more about VPC BPA, see [Block public access to VPCs and
    # subnets][1] in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/security-vpc-bpa.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :exclusion_id
    #   The ID of the exclusion.
    #
    # @return [Types::DeleteVpcBlockPublicAccessExclusionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteVpcBlockPublicAccessExclusionResult#vpc_block_public_access_exclusion #vpc_block_public_access_exclusion} => Types::VpcBlockPublicAccessExclusion
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_vpc_block_public_access_exclusion({
    #     dry_run: false,
    #     exclusion_id: "VpcBlockPublicAccessExclusionId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc_block_public_access_exclusion.exclusion_id #=> String
    #   resp.vpc_block_public_access_exclusion.internet_gateway_exclusion_mode #=> String, one of "allow-bidirectional", "allow-egress"
    #   resp.vpc_block_public_access_exclusion.resource_arn #=> String
    #   resp.vpc_block_public_access_exclusion.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "update-in-progress", "update-complete", "update-failed", "delete-in-progress", "delete-complete", "disable-in-progress", "disable-complete"
    #   resp.vpc_block_public_access_exclusion.reason #=> String
    #   resp.vpc_block_public_access_exclusion.creation_timestamp #=> Time
    #   resp.vpc_block_public_access_exclusion.last_update_timestamp #=> Time
    #   resp.vpc_block_public_access_exclusion.deletion_timestamp #=> Time
    #   resp.vpc_block_public_access_exclusion.tags #=> Array
    #   resp.vpc_block_public_access_exclusion.tags[0].key #=> String
    #   resp.vpc_block_public_access_exclusion.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteVpcBlockPublicAccessExclusion AWS API Documentation
    #
    # @overload delete_vpc_block_public_access_exclusion(params = {})
    # @param [Hash] params ({})
    def delete_vpc_block_public_access_exclusion(params = {}, options = {})
      req = build_request(:delete_vpc_block_public_access_exclusion, params)
      req.send_request(options)
    end

    # Deletes the specified VPC endpoint connection notifications.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :connection_notification_ids
    #   The IDs of the notifications.
    #
    # @return [Types::DeleteVpcEndpointConnectionNotificationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteVpcEndpointConnectionNotificationsResult#unsuccessful #unsuccessful} => Array&lt;Types::UnsuccessfulItem&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_vpc_endpoint_connection_notifications({
    #     dry_run: false,
    #     connection_notification_ids: ["ConnectionNotificationId"], # required
    #   })
    #
    # @example Response structure
    #
    #   resp.unsuccessful #=> Array
    #   resp.unsuccessful[0].error.code #=> String
    #   resp.unsuccessful[0].error.message #=> String
    #   resp.unsuccessful[0].resource_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteVpcEndpointConnectionNotifications AWS API Documentation
    #
    # @overload delete_vpc_endpoint_connection_notifications(params = {})
    # @param [Hash] params ({})
    def delete_vpc_endpoint_connection_notifications(params = {}, options = {})
      req = build_request(:delete_vpc_endpoint_connection_notifications, params)
      req.send_request(options)
    end

    # Deletes the specified VPC endpoint service configurations. Before you
    # can delete an endpoint service configuration, you must reject any
    # `Available` or `PendingAcceptance` interface endpoint connections that
    # are attached to the service.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :service_ids
    #   The IDs of the services.
    #
    # @return [Types::DeleteVpcEndpointServiceConfigurationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteVpcEndpointServiceConfigurationsResult#unsuccessful #unsuccessful} => Array&lt;Types::UnsuccessfulItem&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_vpc_endpoint_service_configurations({
    #     dry_run: false,
    #     service_ids: ["VpcEndpointServiceId"], # required
    #   })
    #
    # @example Response structure
    #
    #   resp.unsuccessful #=> Array
    #   resp.unsuccessful[0].error.code #=> String
    #   resp.unsuccessful[0].error.message #=> String
    #   resp.unsuccessful[0].resource_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteVpcEndpointServiceConfigurations AWS API Documentation
    #
    # @overload delete_vpc_endpoint_service_configurations(params = {})
    # @param [Hash] params ({})
    def delete_vpc_endpoint_service_configurations(params = {}, options = {})
      req = build_request(:delete_vpc_endpoint_service_configurations, params)
      req.send_request(options)
    end

    # Deletes the specified VPC endpoints.
    #
    # When you delete a gateway endpoint, we delete the endpoint routes in
    # the route tables for the endpoint.
    #
    # When you delete a Gateway Load Balancer endpoint, we delete its
    # endpoint network interfaces. You can only delete Gateway Load Balancer
    # endpoints when the routes that are associated with the endpoint are
    # deleted.
    #
    # When you delete an interface endpoint, we delete its endpoint network
    # interfaces.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :vpc_endpoint_ids
    #   The IDs of the VPC endpoints.
    #
    # @return [Types::DeleteVpcEndpointsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteVpcEndpointsResult#unsuccessful #unsuccessful} => Array&lt;Types::UnsuccessfulItem&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_vpc_endpoints({
    #     dry_run: false,
    #     vpc_endpoint_ids: ["VpcEndpointId"], # required
    #   })
    #
    # @example Response structure
    #
    #   resp.unsuccessful #=> Array
    #   resp.unsuccessful[0].error.code #=> String
    #   resp.unsuccessful[0].error.message #=> String
    #   resp.unsuccessful[0].resource_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteVpcEndpoints AWS API Documentation
    #
    # @overload delete_vpc_endpoints(params = {})
    # @param [Hash] params ({})
    def delete_vpc_endpoints(params = {}, options = {})
      req = build_request(:delete_vpc_endpoints, params)
      req.send_request(options)
    end

    # Deletes a VPC peering connection. Either the owner of the requester
    # VPC or the owner of the accepter VPC can delete the VPC peering
    # connection if it's in the `active` state. The owner of the requester
    # VPC can delete a VPC peering connection in the `pending-acceptance`
    # state. You cannot delete a VPC peering connection that's in the
    # `failed` or `rejected` state.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :vpc_peering_connection_id
    #   The ID of the VPC peering connection.
    #
    # @return [Types::DeleteVpcPeeringConnectionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeleteVpcPeeringConnectionResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_vpc_peering_connection({
    #     dry_run: false,
    #     vpc_peering_connection_id: "VpcPeeringConnectionId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteVpcPeeringConnection AWS API Documentation
    #
    # @overload delete_vpc_peering_connection(params = {})
    # @param [Hash] params ({})
    def delete_vpc_peering_connection(params = {}, options = {})
      req = build_request(:delete_vpc_peering_connection, params)
      req.send_request(options)
    end

    # Deletes the specified VPN connection.
    #
    # If you're deleting the VPC and its associated components, we
    # recommend that you detach the virtual private gateway from the VPC and
    # delete the VPC before deleting the VPN connection. If you believe that
    # the tunnel credentials for your VPN connection have been compromised,
    # you can delete the VPN connection and create a new one that has new
    # keys, without needing to delete the VPC or virtual private gateway. If
    # you create a new VPN connection, you must reconfigure the customer
    # gateway device using the new configuration information returned with
    # the new VPN connection ID.
    #
    # For certificate-based authentication, delete all Certificate Manager
    # (ACM) private certificates used for the Amazon Web Services-side
    # tunnel endpoints for the VPN connection before deleting the VPN
    # connection.
    #
    # @option params [required, String] :vpn_connection_id
    #   The ID of the VPN connection.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_vpn_connection({
    #     vpn_connection_id: "VpnConnectionId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteVpnConnection AWS API Documentation
    #
    # @overload delete_vpn_connection(params = {})
    # @param [Hash] params ({})
    def delete_vpn_connection(params = {}, options = {})
      req = build_request(:delete_vpn_connection, params)
      req.send_request(options)
    end

    # Deletes the specified static route associated with a VPN connection
    # between an existing virtual private gateway and a VPN customer
    # gateway. The static route allows traffic to be routed from the virtual
    # private gateway to the VPN customer gateway.
    #
    # @option params [required, String] :destination_cidr_block
    #   The CIDR block associated with the local subnet of the customer
    #   network.
    #
    # @option params [required, String] :vpn_connection_id
    #   The ID of the VPN connection.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_vpn_connection_route({
    #     destination_cidr_block: "String", # required
    #     vpn_connection_id: "VpnConnectionId", # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteVpnConnectionRoute AWS API Documentation
    #
    # @overload delete_vpn_connection_route(params = {})
    # @param [Hash] params ({})
    def delete_vpn_connection_route(params = {}, options = {})
      req = build_request(:delete_vpn_connection_route, params)
      req.send_request(options)
    end

    # Deletes the specified virtual private gateway. You must first detach
    # the virtual private gateway from the VPC. Note that you don't need to
    # delete the virtual private gateway if you plan to delete and recreate
    # the VPN connection between your VPC and your network.
    #
    # @option params [required, String] :vpn_gateway_id
    #   The ID of the virtual private gateway.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.delete_vpn_gateway({
    #     vpn_gateway_id: "VpnGatewayId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeleteVpnGateway AWS API Documentation
    #
    # @overload delete_vpn_gateway(params = {})
    # @param [Hash] params ({})
    def delete_vpn_gateway(params = {}, options = {})
      req = build_request(:delete_vpn_gateway, params)
      req.send_request(options)
    end

    # Releases the specified address range that you provisioned for use with
    # your Amazon Web Services resources through bring your own IP addresses
    # (BYOIP) and deletes the corresponding address pool.
    #
    # Before you can release an address range, you must stop advertising it
    # using WithdrawByoipCidr and you must not have any IP addresses
    # allocated from its address range.
    #
    # @option params [required, String] :cidr
    #   The address range, in CIDR notation. The prefix must be the same
    #   prefix that you specified when you provisioned the address range.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeprovisionByoipCidrResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeprovisionByoipCidrResult#byoip_cidr #byoip_cidr} => Types::ByoipCidr
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.deprovision_byoip_cidr({
    #     cidr: "String", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.byoip_cidr.cidr #=> String
    #   resp.byoip_cidr.description #=> String
    #   resp.byoip_cidr.asn_associations #=> Array
    #   resp.byoip_cidr.asn_associations[0].asn #=> String
    #   resp.byoip_cidr.asn_associations[0].cidr #=> String
    #   resp.byoip_cidr.asn_associations[0].status_message #=> String
    #   resp.byoip_cidr.asn_associations[0].state #=> String, one of "disassociated", "failed-disassociation", "failed-association", "pending-disassociation", "pending-association", "associated"
    #   resp.byoip_cidr.status_message #=> String
    #   resp.byoip_cidr.state #=> String, one of "advertised", "deprovisioned", "failed-deprovision", "failed-provision", "pending-deprovision", "pending-provision", "provisioned", "provisioned-not-publicly-advertisable"
    #   resp.byoip_cidr.network_border_group #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeprovisionByoipCidr AWS API Documentation
    #
    # @overload deprovision_byoip_cidr(params = {})
    # @param [Hash] params ({})
    def deprovision_byoip_cidr(params = {}, options = {})
      req = build_request(:deprovision_byoip_cidr, params)
      req.send_request(options)
    end

    # Deprovisions your Autonomous System Number (ASN) from your Amazon Web
    # Services account. This action can only be called after any BYOIP CIDR
    # associations are removed from your Amazon Web Services account with
    # [DisassociateIpamByoasn][1]. For more information, see [Tutorial:
    # Bring your ASN to IPAM][2] in the *Amazon VPC IPAM guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DisassociateIpamByoasn.html
    # [2]: https://docs.aws.amazon.com/vpc/latest/ipam/tutorials-byoasn.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_id
    #   The IPAM ID.
    #
    # @option params [required, String] :asn
    #   An ASN.
    #
    # @return [Types::DeprovisionIpamByoasnResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeprovisionIpamByoasnResult#byoasn #byoasn} => Types::Byoasn
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.deprovision_ipam_byoasn({
    #     dry_run: false,
    #     ipam_id: "IpamId", # required
    #     asn: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.byoasn.asn #=> String
    #   resp.byoasn.ipam_id #=> String
    #   resp.byoasn.status_message #=> String
    #   resp.byoasn.state #=> String, one of "deprovisioned", "failed-deprovision", "failed-provision", "pending-deprovision", "pending-provision", "provisioned"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeprovisionIpamByoasn AWS API Documentation
    #
    # @overload deprovision_ipam_byoasn(params = {})
    # @param [Hash] params ({})
    def deprovision_ipam_byoasn(params = {}, options = {})
      req = build_request(:deprovision_ipam_byoasn, params)
      req.send_request(options)
    end

    # Deprovision a CIDR provisioned from an IPAM pool. If you deprovision a
    # CIDR from a pool that has a source pool, the CIDR is recycled back
    # into the source pool. For more information, see [Deprovision pool
    # CIDRs][1] in the *Amazon VPC IPAM User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/depro-pool-cidr-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_pool_id
    #   The ID of the pool that has the CIDR you want to deprovision.
    #
    # @option params [String] :cidr
    #   The CIDR which you want to deprovision from the pool.
    #
    # @return [Types::DeprovisionIpamPoolCidrResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeprovisionIpamPoolCidrResult#ipam_pool_cidr #ipam_pool_cidr} => Types::IpamPoolCidr
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.deprovision_ipam_pool_cidr({
    #     dry_run: false,
    #     ipam_pool_id: "IpamPoolId", # required
    #     cidr: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_pool_cidr.cidr #=> String
    #   resp.ipam_pool_cidr.state #=> String, one of "pending-provision", "provisioned", "failed-provision", "pending-deprovision", "deprovisioned", "failed-deprovision", "pending-import", "failed-import"
    #   resp.ipam_pool_cidr.failure_reason.code #=> String, one of "cidr-not-available", "limit-exceeded"
    #   resp.ipam_pool_cidr.failure_reason.message #=> String
    #   resp.ipam_pool_cidr.ipam_pool_cidr_id #=> String
    #   resp.ipam_pool_cidr.netmask_length #=> Integer
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeprovisionIpamPoolCidr AWS API Documentation
    #
    # @overload deprovision_ipam_pool_cidr(params = {})
    # @param [Hash] params ({})
    def deprovision_ipam_pool_cidr(params = {}, options = {})
      req = build_request(:deprovision_ipam_pool_cidr, params)
      req.send_request(options)
    end

    # Deprovision a CIDR from a public IPv4 pool.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :pool_id
    #   The ID of the pool that you want to deprovision the CIDR from.
    #
    # @option params [required, String] :cidr
    #   The CIDR you want to deprovision from the pool. Enter the CIDR you
    #   want to deprovision with a netmask of `/32`. You must rerun this
    #   command for each IP address in the CIDR range. If your CIDR is a
    #   `/24`, you will have to run this command to deprovision each of the
    #   256 IP addresses in the `/24` CIDR.
    #
    # @return [Types::DeprovisionPublicIpv4PoolCidrResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeprovisionPublicIpv4PoolCidrResult#pool_id #pool_id} => String
    #   * {Types::DeprovisionPublicIpv4PoolCidrResult#deprovisioned_addresses #deprovisioned_addresses} => Array&lt;String&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.deprovision_public_ipv_4_pool_cidr({
    #     dry_run: false,
    #     pool_id: "Ipv4PoolEc2Id", # required
    #     cidr: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.pool_id #=> String
    #   resp.deprovisioned_addresses #=> Array
    #   resp.deprovisioned_addresses[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeprovisionPublicIpv4PoolCidr AWS API Documentation
    #
    # @overload deprovision_public_ipv_4_pool_cidr(params = {})
    # @param [Hash] params ({})
    def deprovision_public_ipv_4_pool_cidr(params = {}, options = {})
      req = build_request(:deprovision_public_ipv_4_pool_cidr, params)
      req.send_request(options)
    end

    # Deregisters the specified AMI. After you deregister an AMI, it can't
    # be used to launch new instances.
    #
    # If you deregister an AMI that matches a Recycle Bin retention rule,
    # the AMI is retained in the Recycle Bin for the specified retention
    # period. For more information, see [Recycle Bin][1] in the *Amazon EC2
    # User Guide*.
    #
    # When you deregister an AMI, it doesn't affect any instances that
    # you've already launched from the AMI. You'll continue to incur usage
    # costs for those instances until you terminate them.
    #
    # When you deregister an Amazon EBS-backed AMI, it doesn't affect the
    # snapshot that was created for the root volume of the instance during
    # the AMI creation process. When you deregister an instance store-backed
    # AMI, it doesn't affect the files that you uploaded to Amazon S3 when
    # you created the AMI.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/recycle-bin.html
    #
    # @option params [required, String] :image_id
    #   The ID of the AMI.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.deregister_image({
    #     image_id: "ImageId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeregisterImage AWS API Documentation
    #
    # @overload deregister_image(params = {})
    # @param [Hash] params ({})
    def deregister_image(params = {}, options = {})
      req = build_request(:deregister_image, params)
      req.send_request(options)
    end

    # Deregisters tag keys to prevent tags that have the specified tag keys
    # from being included in scheduled event notifications for resources in
    # the Region.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Types::DeregisterInstanceTagAttributeRequest] :instance_tag_attribute
    #   Information about the tag keys to deregister.
    #
    # @return [Types::DeregisterInstanceEventNotificationAttributesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeregisterInstanceEventNotificationAttributesResult#instance_tag_attribute #instance_tag_attribute} => Types::InstanceTagNotificationAttribute
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.deregister_instance_event_notification_attributes({
    #     dry_run: false,
    #     instance_tag_attribute: { # required
    #       include_all_tags_of_instance: false,
    #       instance_tag_keys: ["String"],
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_tag_attribute.instance_tag_keys #=> Array
    #   resp.instance_tag_attribute.instance_tag_keys[0] #=> String
    #   resp.instance_tag_attribute.include_all_tags_of_instance #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeregisterInstanceEventNotificationAttributes AWS API Documentation
    #
    # @overload deregister_instance_event_notification_attributes(params = {})
    # @param [Hash] params ({})
    def deregister_instance_event_notification_attributes(params = {}, options = {})
      req = build_request(:deregister_instance_event_notification_attributes, params)
      req.send_request(options)
    end

    # Deregisters the specified members (network interfaces) from the
    # transit gateway multicast group.
    #
    # @option params [String] :transit_gateway_multicast_domain_id
    #   The ID of the transit gateway multicast domain.
    #
    # @option params [String] :group_ip_address
    #   The IP address assigned to the transit gateway multicast group.
    #
    # @option params [Array<String>] :network_interface_ids
    #   The IDs of the group members' network interfaces.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeregisterTransitGatewayMulticastGroupMembersResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeregisterTransitGatewayMulticastGroupMembersResult#deregistered_multicast_group_members #deregistered_multicast_group_members} => Types::TransitGatewayMulticastDeregisteredGroupMembers
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.deregister_transit_gateway_multicast_group_members({
    #     transit_gateway_multicast_domain_id: "TransitGatewayMulticastDomainId",
    #     group_ip_address: "String",
    #     network_interface_ids: ["NetworkInterfaceId"],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.deregistered_multicast_group_members.transit_gateway_multicast_domain_id #=> String
    #   resp.deregistered_multicast_group_members.deregistered_network_interface_ids #=> Array
    #   resp.deregistered_multicast_group_members.deregistered_network_interface_ids[0] #=> String
    #   resp.deregistered_multicast_group_members.group_ip_address #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeregisterTransitGatewayMulticastGroupMembers AWS API Documentation
    #
    # @overload deregister_transit_gateway_multicast_group_members(params = {})
    # @param [Hash] params ({})
    def deregister_transit_gateway_multicast_group_members(params = {}, options = {})
      req = build_request(:deregister_transit_gateway_multicast_group_members, params)
      req.send_request(options)
    end

    # Deregisters the specified sources (network interfaces) from the
    # transit gateway multicast group.
    #
    # @option params [String] :transit_gateway_multicast_domain_id
    #   The ID of the transit gateway multicast domain.
    #
    # @option params [String] :group_ip_address
    #   The IP address assigned to the transit gateway multicast group.
    #
    # @option params [Array<String>] :network_interface_ids
    #   The IDs of the group sources' network interfaces.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DeregisterTransitGatewayMulticastGroupSourcesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DeregisterTransitGatewayMulticastGroupSourcesResult#deregistered_multicast_group_sources #deregistered_multicast_group_sources} => Types::TransitGatewayMulticastDeregisteredGroupSources
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.deregister_transit_gateway_multicast_group_sources({
    #     transit_gateway_multicast_domain_id: "TransitGatewayMulticastDomainId",
    #     group_ip_address: "String",
    #     network_interface_ids: ["NetworkInterfaceId"],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.deregistered_multicast_group_sources.transit_gateway_multicast_domain_id #=> String
    #   resp.deregistered_multicast_group_sources.deregistered_network_interface_ids #=> Array
    #   resp.deregistered_multicast_group_sources.deregistered_network_interface_ids[0] #=> String
    #   resp.deregistered_multicast_group_sources.group_ip_address #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DeregisterTransitGatewayMulticastGroupSources AWS API Documentation
    #
    # @overload deregister_transit_gateway_multicast_group_sources(params = {})
    # @param [Hash] params ({})
    def deregister_transit_gateway_multicast_group_sources(params = {}, options = {})
      req = build_request(:deregister_transit_gateway_multicast_group_sources, params)
      req.send_request(options)
    end

    # Describes attributes of your Amazon Web Services account. The
    # following are the supported account attributes:
    #
    # * `default-vpc`: The ID of the default VPC for your account, or
    #   `none`.
    #
    # * `max-instances`: This attribute is no longer supported. The returned
    #   value does not reflect your actual vCPU limit for running On-Demand
    #   Instances. For more information, see [On-Demand Instance Limits][1]
    #   in the *Amazon Elastic Compute Cloud User Guide*.
    #
    # * `max-elastic-ips`: The maximum number of Elastic IP addresses that
    #   you can allocate.
    #
    # * `supported-platforms`: This attribute is deprecated.
    #
    # * `vpc-max-elastic-ips`: The maximum number of Elastic IP addresses
    #   that you can allocate.
    #
    # * `vpc-max-security-groups-per-interface`: The maximum number of
    #   security groups that you can assign to a network interface.
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-on-demand-instances.html#ec2-on-demand-instances-limits
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :attribute_names
    #   The account attribute names.
    #
    # @return [Types::DescribeAccountAttributesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeAccountAttributesResult#account_attributes #account_attributes} => Array&lt;Types::AccountAttribute&gt;
    #
    #
    # @example Example: To describe a single attribute for your AWS account
    #
    #   # This example describes the supported-platforms attribute for your AWS account.
    #
    #   resp = client.describe_account_attributes({
    #     attribute_names: [
    #       "supported-platforms", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     account_attributes: [
    #       {
    #         attribute_name: "supported-platforms", 
    #         attribute_values: [
    #           {
    #             attribute_value: "EC2", 
    #           }, 
    #           {
    #             attribute_value: "VPC", 
    #           }, 
    #         ], 
    #       }, 
    #     ], 
    #   }
    #
    # @example Example: To describe all attributes for your AWS account
    #
    #   # This example describes the attributes for your AWS account.
    #
    #   resp = client.describe_account_attributes({
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     account_attributes: [
    #       {
    #         attribute_name: "supported-platforms", 
    #         attribute_values: [
    #           {
    #             attribute_value: "EC2", 
    #           }, 
    #           {
    #             attribute_value: "VPC", 
    #           }, 
    #         ], 
    #       }, 
    #       {
    #         attribute_name: "vpc-max-security-groups-per-interface", 
    #         attribute_values: [
    #           {
    #             attribute_value: "5", 
    #           }, 
    #         ], 
    #       }, 
    #       {
    #         attribute_name: "max-elastic-ips", 
    #         attribute_values: [
    #           {
    #             attribute_value: "5", 
    #           }, 
    #         ], 
    #       }, 
    #       {
    #         attribute_name: "max-instances", 
    #         attribute_values: [
    #           {
    #             attribute_value: "20", 
    #           }, 
    #         ], 
    #       }, 
    #       {
    #         attribute_name: "vpc-max-elastic-ips", 
    #         attribute_values: [
    #           {
    #             attribute_value: "5", 
    #           }, 
    #         ], 
    #       }, 
    #       {
    #         attribute_name: "default-vpc", 
    #         attribute_values: [
    #           {
    #             attribute_value: "none", 
    #           }, 
    #         ], 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_account_attributes({
    #     dry_run: false,
    #     attribute_names: ["supported-platforms"], # accepts supported-platforms, default-vpc
    #   })
    #
    # @example Response structure
    #
    #   resp.account_attributes #=> Array
    #   resp.account_attributes[0].attribute_name #=> String
    #   resp.account_attributes[0].attribute_values #=> Array
    #   resp.account_attributes[0].attribute_values[0].attribute_value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeAccountAttributes AWS API Documentation
    #
    # @overload describe_account_attributes(params = {})
    # @param [Hash] params ({})
    def describe_account_attributes(params = {}, options = {})
      req = build_request(:describe_account_attributes, params)
      req.send_request(options)
    end

    # Describes an Elastic IP address transfer. For more information, see
    # [Transfer Elastic IP addresses][1] in the *Amazon VPC User Guide*.
    #
    # When you transfer an Elastic IP address, there is a two-step handshake
    # between the source and transfer Amazon Web Services accounts. When the
    # source account starts the transfer, the transfer account has seven
    # days to accept the Elastic IP address transfer. During those seven
    # days, the source account can view the pending transfer by using this
    # action. After seven days, the transfer expires and ownership of the
    # Elastic IP address returns to the source account. Accepted transfers
    # are visible to the source account for 14 days after the transfers have
    # been accepted.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/vpc-eips.html#transfer-EIPs-intro
    #
    # @option params [Array<String>] :allocation_ids
    #   The allocation IDs of Elastic IP addresses.
    #
    # @option params [String] :next_token
    #   Specify the pagination token from a previous request to retrieve the
    #   next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of address transfers to return in one page of
    #   results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeAddressTransfersResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeAddressTransfersResult#address_transfers #address_transfers} => Array&lt;Types::AddressTransfer&gt;
    #   * {Types::DescribeAddressTransfersResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_address_transfers({
    #     allocation_ids: ["AllocationId"],
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.address_transfers #=> Array
    #   resp.address_transfers[0].public_ip #=> String
    #   resp.address_transfers[0].allocation_id #=> String
    #   resp.address_transfers[0].transfer_account_id #=> String
    #   resp.address_transfers[0].transfer_offer_expiration_timestamp #=> Time
    #   resp.address_transfers[0].transfer_offer_accepted_timestamp #=> Time
    #   resp.address_transfers[0].address_transfer_status #=> String, one of "pending", "disabled", "accepted"
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeAddressTransfers AWS API Documentation
    #
    # @overload describe_address_transfers(params = {})
    # @param [Hash] params ({})
    def describe_address_transfers(params = {}, options = {})
      req = build_request(:describe_address_transfers, params)
      req.send_request(options)
    end

    # Describes the specified Elastic IP addresses or all of your Elastic IP
    # addresses.
    #
    # @option params [Array<String>] :public_ips
    #   One or more Elastic IP addresses.
    #
    #   Default: Describes all your Elastic IP addresses.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. Filter names and values are case-sensitive.
    #
    #   * `allocation-id` - The allocation ID for the address.
    #
    #   * `association-id` - The association ID for the address.
    #
    #   * `instance-id` - The ID of the instance the address is associated
    #     with, if any.
    #
    #   * `network-border-group` - A unique set of Availability Zones, Local
    #     Zones, or Wavelength Zones from where Amazon Web Services advertises
    #     IP addresses.
    #
    #   * `network-interface-id` - The ID of the network interface that the
    #     address is associated with, if any.
    #
    #   * `network-interface-owner-id` - The Amazon Web Services account ID of
    #     the owner.
    #
    #   * `private-ip-address` - The private IP address associated with the
    #     Elastic IP address.
    #
    #   * `public-ip` - The Elastic IP address, or the carrier IP address.
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @option params [Array<String>] :allocation_ids
    #   Information about the allocation IDs.
    #
    # @return [Types::DescribeAddressesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeAddressesResult#addresses #addresses} => Array&lt;Types::Address&gt;
    #
    #
    # @example Example: To describe your Elastic IP addresses
    #
    #   # This example describes your Elastic IP addresses.
    #
    #   resp = client.describe_addresses({
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     addresses: [
    #       {
    #         domain: "standard", 
    #         instance_id: "i-1234567890abcdef0", 
    #         public_ip: "198.51.100.0", 
    #       }, 
    #       {
    #         allocation_id: "eipalloc-12345678", 
    #         association_id: "eipassoc-12345678", 
    #         domain: "vpc", 
    #         instance_id: "i-1234567890abcdef0", 
    #         network_interface_id: "eni-12345678", 
    #         network_interface_owner_id: "123456789012", 
    #         private_ip_address: "10.0.1.241", 
    #         public_ip: "203.0.113.0", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_addresses({
    #     public_ips: ["String"],
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     allocation_ids: ["AllocationId"],
    #   })
    #
    # @example Response structure
    #
    #   resp.addresses #=> Array
    #   resp.addresses[0].allocation_id #=> String
    #   resp.addresses[0].association_id #=> String
    #   resp.addresses[0].domain #=> String, one of "vpc", "standard"
    #   resp.addresses[0].network_interface_id #=> String
    #   resp.addresses[0].network_interface_owner_id #=> String
    #   resp.addresses[0].private_ip_address #=> String
    #   resp.addresses[0].tags #=> Array
    #   resp.addresses[0].tags[0].key #=> String
    #   resp.addresses[0].tags[0].value #=> String
    #   resp.addresses[0].public_ipv_4_pool #=> String
    #   resp.addresses[0].network_border_group #=> String
    #   resp.addresses[0].customer_owned_ip #=> String
    #   resp.addresses[0].customer_owned_ipv_4_pool #=> String
    #   resp.addresses[0].carrier_ip #=> String
    #   resp.addresses[0].service_managed #=> String, one of "alb", "nlb"
    #   resp.addresses[0].instance_id #=> String
    #   resp.addresses[0].public_ip #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeAddresses AWS API Documentation
    #
    # @overload describe_addresses(params = {})
    # @param [Hash] params ({})
    def describe_addresses(params = {}, options = {})
      req = build_request(:describe_addresses, params)
      req.send_request(options)
    end

    # Describes the attributes of the specified Elastic IP addresses. For
    # requirements, see [Using reverse DNS for email applications][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/elastic-ip-addresses-eip.html#Using_Elastic_Addressing_Reverse_DNS
    #
    # @option params [Array<String>] :allocation_ids
    #   \[EC2-VPC\] The allocation IDs.
    #
    # @option params [String] :attribute
    #   The attribute of the IP address.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeAddressesAttributeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeAddressesAttributeResult#addresses #addresses} => Array&lt;Types::AddressAttribute&gt;
    #   * {Types::DescribeAddressesAttributeResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_addresses_attribute({
    #     allocation_ids: ["AllocationId"],
    #     attribute: "domain-name", # accepts domain-name
    #     next_token: "NextToken",
    #     max_results: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.addresses #=> Array
    #   resp.addresses[0].public_ip #=> String
    #   resp.addresses[0].allocation_id #=> String
    #   resp.addresses[0].ptr_record #=> String
    #   resp.addresses[0].ptr_record_update.value #=> String
    #   resp.addresses[0].ptr_record_update.status #=> String
    #   resp.addresses[0].ptr_record_update.reason #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeAddressesAttribute AWS API Documentation
    #
    # @overload describe_addresses_attribute(params = {})
    # @param [Hash] params ({})
    def describe_addresses_attribute(params = {}, options = {})
      req = build_request(:describe_addresses_attribute, params)
      req.send_request(options)
    end

    # Describes the longer ID format settings for all resource types in a
    # specific Region. This request is useful for performing a quick audit
    # to determine whether a specific Region is fully opted in for longer
    # IDs (17-character IDs).
    #
    # This request only returns information about resource types that
    # support longer IDs.
    #
    # The following resource types support longer IDs: `bundle` \|
    # `conversion-task` \| `customer-gateway` \| `dhcp-options` \|
    # `elastic-ip-allocation` \| `elastic-ip-association` \| `export-task`
    # \| `flow-log` \| `image` \| `import-task` \| `instance` \|
    # `internet-gateway` \| `network-acl` \| `network-acl-association` \|
    # `network-interface` \| `network-interface-attachment` \| `prefix-list`
    # \| `reservation` \| `route-table` \| `route-table-association` \|
    # `security-group` \| `snapshot` \| `subnet` \|
    # `subnet-cidr-block-association` \| `volume` \| `vpc` \|
    # `vpc-cidr-block-association` \| `vpc-endpoint` \|
    # `vpc-peering-connection` \| `vpn-connection` \| `vpn-gateway`.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeAggregateIdFormatResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeAggregateIdFormatResult#use_long_ids_aggregated #use_long_ids_aggregated} => Boolean
    #   * {Types::DescribeAggregateIdFormatResult#statuses #statuses} => Array&lt;Types::IdFormat&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_aggregate_id_format({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.use_long_ids_aggregated #=> Boolean
    #   resp.statuses #=> Array
    #   resp.statuses[0].deadline #=> Time
    #   resp.statuses[0].resource #=> String
    #   resp.statuses[0].use_long_ids #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeAggregateIdFormat AWS API Documentation
    #
    # @overload describe_aggregate_id_format(params = {})
    # @param [Hash] params ({})
    def describe_aggregate_id_format(params = {}, options = {})
      req = build_request(:describe_aggregate_id_format, params)
      req.send_request(options)
    end

    # Describes the Availability Zones, Local Zones, and Wavelength Zones
    # that are available to you.
    #
    # For more information about Availability Zones, Local Zones, and
    # Wavelength Zones, see [Regions and zones][1] in the *Amazon EC2 User
    # Guide*.
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-regions-availability-zones.html
    #
    # @option params [Array<String>] :zone_names
    #   The names of the Availability Zones, Local Zones, and Wavelength
    #   Zones.
    #
    # @option params [Array<String>] :zone_ids
    #   The IDs of the Availability Zones, Local Zones, and Wavelength Zones.
    #
    # @option params [Boolean] :all_availability_zones
    #   Include all Availability Zones, Local Zones, and Wavelength Zones
    #   regardless of your opt-in status.
    #
    #   If you do not use this parameter, the results include only the zones
    #   for the Regions where you have chosen the option to opt in.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `group-long-name` - The long name of the zone group for the
    #     Availability Zone (for example, `US West (Oregon) 1`), the Local
    #     Zone (for example, for Zone group `us-west-2-lax-1`, it is `US West
    #     (Los Angeles)`, or the Wavelength Zone (for example, for Zone group
    #     `us-east-1-wl1`, it is `US East (Verizon)`.
    #
    #   * `group-name` - The name of the zone group for the Availability Zone
    #     (for example, `us-east-1-zg-1`), the Local Zone (for example,
    #     `us-west-2-lax-1`), or the Wavelength Zone (for example,
    #     `us-east-1-wl1`).
    #
    #   * `message` - The Zone message.
    #
    #   * `opt-in-status` - The opt-in status (`opted-in` \| `not-opted-in` \|
    #     `opt-in-not-required`).
    #
    #   * `parent-zone-id` - The ID of the zone that handles some of the Local
    #     Zone and Wavelength Zone control plane operations, such as API
    #     calls.
    #
    #   * `parent-zone-name` - The ID of the zone that handles some of the
    #     Local Zone and Wavelength Zone control plane operations, such as API
    #     calls.
    #
    #   * `region-name` - The name of the Region for the Zone (for example,
    #     `us-east-1`).
    #
    #   * `state` - The state of the Availability Zone, the Local Zone, or the
    #     Wavelength Zone (`available`).
    #
    #   * `zone-id` - The ID of the Availability Zone (for example,
    #     `use1-az1`), the Local Zone (for example, `usw2-lax1-az1`), or the
    #     Wavelength Zone (for example, `us-east-1-wl1-bos-wlz-1`).
    #
    #   * `zone-name` - The name of the Availability Zone (for example,
    #     `us-east-1a`), the Local Zone (for example, `us-west-2-lax-1a`), or
    #     the Wavelength Zone (for example, `us-east-1-wl1-bos-wlz-1`).
    #
    #   * `zone-type` - The type of zone (`availability-zone` \| `local-zone`
    #     \| `wavelength-zone`).
    #
    # @return [Types::DescribeAvailabilityZonesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeAvailabilityZonesResult#availability_zones #availability_zones} => Array&lt;Types::AvailabilityZone&gt;
    #
    #
    # @example Example: To describe your Availability Zones
    #
    #   # This example describes the Availability Zones that are available to you. The response includes Availability Zones only
    #   # for the current region.
    #
    #   resp = client.describe_availability_zones({
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     availability_zones: [
    #       {
    #         messages: [
    #         ], 
    #         region_name: "us-east-1", 
    #         state: "available", 
    #         zone_name: "us-east-1b", 
    #       }, 
    #       {
    #         messages: [
    #         ], 
    #         region_name: "us-east-1", 
    #         state: "available", 
    #         zone_name: "us-east-1c", 
    #       }, 
    #       {
    #         messages: [
    #         ], 
    #         region_name: "us-east-1", 
    #         state: "available", 
    #         zone_name: "us-east-1d", 
    #       }, 
    #       {
    #         messages: [
    #         ], 
    #         region_name: "us-east-1", 
    #         state: "available", 
    #         zone_name: "us-east-1e", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_availability_zones({
    #     zone_names: ["String"],
    #     zone_ids: ["String"],
    #     all_availability_zones: false,
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.availability_zones #=> Array
    #   resp.availability_zones[0].opt_in_status #=> String, one of "opt-in-not-required", "opted-in", "not-opted-in"
    #   resp.availability_zones[0].messages #=> Array
    #   resp.availability_zones[0].messages[0].message #=> String
    #   resp.availability_zones[0].region_name #=> String
    #   resp.availability_zones[0].zone_name #=> String
    #   resp.availability_zones[0].zone_id #=> String
    #   resp.availability_zones[0].group_name #=> String
    #   resp.availability_zones[0].network_border_group #=> String
    #   resp.availability_zones[0].zone_type #=> String
    #   resp.availability_zones[0].parent_zone_name #=> String
    #   resp.availability_zones[0].parent_zone_id #=> String
    #   resp.availability_zones[0].group_long_name #=> String
    #   resp.availability_zones[0].state #=> String, one of "available", "information", "impaired", "unavailable", "constrained"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeAvailabilityZones AWS API Documentation
    #
    # @overload describe_availability_zones(params = {})
    # @param [Hash] params ({})
    def describe_availability_zones(params = {}, options = {})
      req = build_request(:describe_availability_zones, params)
      req.send_request(options)
    end

    # Describes the current Infrastructure Performance metric subscriptions.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeAwsNetworkPerformanceMetricSubscriptionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeAwsNetworkPerformanceMetricSubscriptionsResult#next_token #next_token} => String
    #   * {Types::DescribeAwsNetworkPerformanceMetricSubscriptionsResult#subscriptions #subscriptions} => Array&lt;Types::Subscription&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_aws_network_performance_metric_subscriptions({
    #     max_results: 1,
    #     next_token: "String",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.subscriptions #=> Array
    #   resp.subscriptions[0].source #=> String
    #   resp.subscriptions[0].destination #=> String
    #   resp.subscriptions[0].metric #=> String, one of "aggregate-latency"
    #   resp.subscriptions[0].statistic #=> String, one of "p50"
    #   resp.subscriptions[0].period #=> String, one of "five-minutes", "fifteen-minutes", "one-hour", "three-hours", "one-day", "one-week"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeAwsNetworkPerformanceMetricSubscriptions AWS API Documentation
    #
    # @overload describe_aws_network_performance_metric_subscriptions(params = {})
    # @param [Hash] params ({})
    def describe_aws_network_performance_metric_subscriptions(params = {}, options = {})
      req = build_request(:describe_aws_network_performance_metric_subscriptions, params)
      req.send_request(options)
    end

    # Describes the specified bundle tasks or all of your bundle tasks.
    #
    # <note markdown="1"> Completed bundle tasks are listed for only a limited time. If your
    # bundle task is no longer in the list, you can still register an AMI
    # from it. Just use `RegisterImage` with the Amazon S3 bucket name and
    # image manifest name you provided to the bundle task.
    #
    #  </note>
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    # @option params [Array<String>] :bundle_ids
    #   The bundle task IDs.
    #
    #   Default: Describes all your bundle tasks.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `bundle-id` - The ID of the bundle task.
    #
    #   * `error-code` - If the task failed, the error code returned.
    #
    #   * `error-message` - If the task failed, the error message returned.
    #
    #   * `instance-id` - The ID of the instance.
    #
    #   * `progress` - The level of task completion, as a percentage (for
    #     example, 20%).
    #
    #   * `s3-bucket` - The Amazon S3 bucket to store the AMI.
    #
    #   * `s3-prefix` - The beginning of the AMI name.
    #
    #   * `start-time` - The time the task started (for example,
    #     2013-09-15T17:15:20.000Z).
    #
    #   * `state` - The state of the task (`pending` \| `waiting-for-shutdown`
    #     \| `bundling` \| `storing` \| `cancelling` \| `complete` \|
    #     `failed`).
    #
    #   * `update-time` - The time of the most recent update for the task.
    #
    # @return [Types::DescribeBundleTasksResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeBundleTasksResult#bundle_tasks #bundle_tasks} => Array&lt;Types::BundleTask&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_bundle_tasks({
    #     bundle_ids: ["BundleId"],
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.bundle_tasks #=> Array
    #   resp.bundle_tasks[0].instance_id #=> String
    #   resp.bundle_tasks[0].bundle_id #=> String
    #   resp.bundle_tasks[0].state #=> String, one of "pending", "waiting-for-shutdown", "bundling", "storing", "cancelling", "complete", "failed"
    #   resp.bundle_tasks[0].start_time #=> Time
    #   resp.bundle_tasks[0].update_time #=> Time
    #   resp.bundle_tasks[0].storage.s3.aws_access_key_id #=> String
    #   resp.bundle_tasks[0].storage.s3.bucket #=> String
    #   resp.bundle_tasks[0].storage.s3.prefix #=> String
    #   resp.bundle_tasks[0].storage.s3.upload_policy #=> String
    #   resp.bundle_tasks[0].storage.s3.upload_policy_signature #=> String
    #   resp.bundle_tasks[0].progress #=> String
    #   resp.bundle_tasks[0].bundle_task_error.code #=> String
    #   resp.bundle_tasks[0].bundle_task_error.message #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * bundle_task_complete
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeBundleTasks AWS API Documentation
    #
    # @overload describe_bundle_tasks(params = {})
    # @param [Hash] params ({})
    def describe_bundle_tasks(params = {}, options = {})
      req = build_request(:describe_bundle_tasks, params)
      req.send_request(options)
    end

    # Describes the IP address ranges that were specified in calls to
    # ProvisionByoipCidr.
    #
    # To describe the address pools that were created when you provisioned
    # the address ranges, use DescribePublicIpv4Pools or DescribeIpv6Pools.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::DescribeByoipCidrsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeByoipCidrsResult#byoip_cidrs #byoip_cidrs} => Array&lt;Types::ByoipCidr&gt;
    #   * {Types::DescribeByoipCidrsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_byoip_cidrs({
    #     dry_run: false,
    #     max_results: 1, # required
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.byoip_cidrs #=> Array
    #   resp.byoip_cidrs[0].cidr #=> String
    #   resp.byoip_cidrs[0].description #=> String
    #   resp.byoip_cidrs[0].asn_associations #=> Array
    #   resp.byoip_cidrs[0].asn_associations[0].asn #=> String
    #   resp.byoip_cidrs[0].asn_associations[0].cidr #=> String
    #   resp.byoip_cidrs[0].asn_associations[0].status_message #=> String
    #   resp.byoip_cidrs[0].asn_associations[0].state #=> String, one of "disassociated", "failed-disassociation", "failed-association", "pending-disassociation", "pending-association", "associated"
    #   resp.byoip_cidrs[0].status_message #=> String
    #   resp.byoip_cidrs[0].state #=> String, one of "advertised", "deprovisioned", "failed-deprovision", "failed-provision", "pending-deprovision", "pending-provision", "provisioned", "provisioned-not-publicly-advertisable"
    #   resp.byoip_cidrs[0].network_border_group #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeByoipCidrs AWS API Documentation
    #
    # @overload describe_byoip_cidrs(params = {})
    # @param [Hash] params ({})
    def describe_byoip_cidrs(params = {}, options = {})
      req = build_request(:describe_byoip_cidrs, params)
      req.send_request(options)
    end

    # Describes the events for the specified Capacity Block extension during
    # the specified time.
    #
    # @option params [Array<String>] :capacity_reservation_ids
    #   The IDs of Capacity Block reservations that you want to display the
    #   history for.
    #
    # @option params [String] :next_token
    #   The token to use to retrieve the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters
    #
    #   * `availability-zone` - The Availability Zone of the extension.
    #
    #   * `availability-zone-id` - The Availability Zone ID of the extension.
    #
    #   * `capacity-block-extension-offering-id` - The ID of the extension
    #     offering.
    #
    #   * `capacity-block-extension-status` - The status of the extension
    #     (`payment-pending` \| `payment-failed` \| `payment-succeeded`).
    #
    #   * `capacity-reservation-id` - The reservation ID of the extension.
    #
    #   * `instance-type` - The instance type of the extension.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeCapacityBlockExtensionHistoryResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeCapacityBlockExtensionHistoryResult#capacity_block_extensions #capacity_block_extensions} => Array&lt;Types::CapacityBlockExtension&gt;
    #   * {Types::DescribeCapacityBlockExtensionHistoryResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_capacity_block_extension_history({
    #     capacity_reservation_ids: ["CapacityReservationId"],
    #     next_token: "String",
    #     max_results: 1,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.capacity_block_extensions #=> Array
    #   resp.capacity_block_extensions[0].capacity_reservation_id #=> String
    #   resp.capacity_block_extensions[0].instance_type #=> String
    #   resp.capacity_block_extensions[0].instance_count #=> Integer
    #   resp.capacity_block_extensions[0].availability_zone #=> String
    #   resp.capacity_block_extensions[0].availability_zone_id #=> String
    #   resp.capacity_block_extensions[0].capacity_block_extension_offering_id #=> String
    #   resp.capacity_block_extensions[0].capacity_block_extension_duration_hours #=> Integer
    #   resp.capacity_block_extensions[0].capacity_block_extension_status #=> String, one of "payment-pending", "payment-failed", "payment-succeeded"
    #   resp.capacity_block_extensions[0].capacity_block_extension_purchase_date #=> Time
    #   resp.capacity_block_extensions[0].capacity_block_extension_start_date #=> Time
    #   resp.capacity_block_extensions[0].capacity_block_extension_end_date #=> Time
    #   resp.capacity_block_extensions[0].upfront_fee #=> String
    #   resp.capacity_block_extensions[0].currency_code #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeCapacityBlockExtensionHistory AWS API Documentation
    #
    # @overload describe_capacity_block_extension_history(params = {})
    # @param [Hash] params ({})
    def describe_capacity_block_extension_history(params = {}, options = {})
      req = build_request(:describe_capacity_block_extension_history, params)
      req.send_request(options)
    end

    # Describes Capacity Block extension offerings available for purchase in
    # the Amazon Web Services Region that you're currently using.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Integer] :capacity_block_extension_duration_hours
    #   The duration of the Capacity Block extension offering in hours.
    #
    # @option params [required, String] :capacity_reservation_id
    #   The ID of the Capacity reservation to be extended.
    #
    # @option params [String] :next_token
    #   The token to use to retrieve the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @return [Types::DescribeCapacityBlockExtensionOfferingsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeCapacityBlockExtensionOfferingsResult#capacity_block_extension_offerings #capacity_block_extension_offerings} => Array&lt;Types::CapacityBlockExtensionOffering&gt;
    #   * {Types::DescribeCapacityBlockExtensionOfferingsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_capacity_block_extension_offerings({
    #     dry_run: false,
    #     capacity_block_extension_duration_hours: 1, # required
    #     capacity_reservation_id: "CapacityReservationId", # required
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.capacity_block_extension_offerings #=> Array
    #   resp.capacity_block_extension_offerings[0].capacity_block_extension_offering_id #=> String
    #   resp.capacity_block_extension_offerings[0].instance_type #=> String
    #   resp.capacity_block_extension_offerings[0].instance_count #=> Integer
    #   resp.capacity_block_extension_offerings[0].availability_zone #=> String
    #   resp.capacity_block_extension_offerings[0].availability_zone_id #=> String
    #   resp.capacity_block_extension_offerings[0].start_date #=> Time
    #   resp.capacity_block_extension_offerings[0].capacity_block_extension_start_date #=> Time
    #   resp.capacity_block_extension_offerings[0].capacity_block_extension_end_date #=> Time
    #   resp.capacity_block_extension_offerings[0].capacity_block_extension_duration_hours #=> Integer
    #   resp.capacity_block_extension_offerings[0].upfront_fee #=> String
    #   resp.capacity_block_extension_offerings[0].currency_code #=> String
    #   resp.capacity_block_extension_offerings[0].tenancy #=> String, one of "default", "dedicated"
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeCapacityBlockExtensionOfferings AWS API Documentation
    #
    # @overload describe_capacity_block_extension_offerings(params = {})
    # @param [Hash] params ({})
    def describe_capacity_block_extension_offerings(params = {}, options = {})
      req = build_request(:describe_capacity_block_extension_offerings, params)
      req.send_request(options)
    end

    # Describes Capacity Block offerings available for purchase in the
    # Amazon Web Services Region that you're currently using. With Capacity
    # Blocks, you purchase a specific instance type for a period of time.
    #
    # To search for an available Capacity Block offering, you specify a
    # reservation duration and instance count. You must select one of the
    # following options.
    #
    # * For reservation durations<b> 1-day increments up 14 days and 7-day
    #   increments up to 182 days total</b>
    #
    # * For instance count<b> 1, 2, 4, 8, 16, 32, or 64 instances</b>
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :instance_type
    #   The type of instance for which the Capacity Block offering reserves
    #   capacity.
    #
    # @option params [Integer] :instance_count
    #   The number of instances for which to reserve capacity.
    #
    # @option params [Time,DateTime,Date,Integer,String] :start_date_range
    #   The earliest start date for the Capacity Block offering.
    #
    # @option params [Time,DateTime,Date,Integer,String] :end_date_range
    #   The latest end date for the Capacity Block offering.
    #
    # @option params [required, Integer] :capacity_duration_hours
    #   The number of hours for which to reserve Capacity Block.
    #
    # @option params [String] :next_token
    #   The token to use to retrieve the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @return [Types::DescribeCapacityBlockOfferingsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeCapacityBlockOfferingsResult#capacity_block_offerings #capacity_block_offerings} => Array&lt;Types::CapacityBlockOffering&gt;
    #   * {Types::DescribeCapacityBlockOfferingsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_capacity_block_offerings({
    #     dry_run: false,
    #     instance_type: "String",
    #     instance_count: 1,
    #     start_date_range: Time.now,
    #     end_date_range: Time.now,
    #     capacity_duration_hours: 1, # required
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.capacity_block_offerings #=> Array
    #   resp.capacity_block_offerings[0].capacity_block_offering_id #=> String
    #   resp.capacity_block_offerings[0].instance_type #=> String
    #   resp.capacity_block_offerings[0].availability_zone #=> String
    #   resp.capacity_block_offerings[0].instance_count #=> Integer
    #   resp.capacity_block_offerings[0].start_date #=> Time
    #   resp.capacity_block_offerings[0].end_date #=> Time
    #   resp.capacity_block_offerings[0].capacity_block_duration_hours #=> Integer
    #   resp.capacity_block_offerings[0].upfront_fee #=> String
    #   resp.capacity_block_offerings[0].currency_code #=> String
    #   resp.capacity_block_offerings[0].tenancy #=> String, one of "default", "dedicated"
    #   resp.capacity_block_offerings[0].capacity_block_duration_minutes #=> Integer
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeCapacityBlockOfferings AWS API Documentation
    #
    # @overload describe_capacity_block_offerings(params = {})
    # @param [Hash] params ({})
    def describe_capacity_block_offerings(params = {}, options = {})
      req = build_request(:describe_capacity_block_offerings, params)
      req.send_request(options)
    end

    # Describes a request to assign the billing of the unused capacity of a
    # Capacity Reservation. For more information, see [ Billing assignment
    # for shared Amazon EC2 Capacity Reservations][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/transfer-billing.html
    #
    # @option params [Array<String>] :capacity_reservation_ids
    #   The ID of the Capacity Reservation.
    #
    # @option params [required, String] :role
    #   Specify one of the following:
    #
    #   * `odcr-owner` - If you are the Capacity Reservation owner, specify
    #     this value to view requests that you have initiated. Not supported
    #     with the `requested-by` filter.
    #
    #   * `unused-reservation-billing-owner` - If you are the consumer
    #     account, specify this value to view requests that have been sent to
    #     you. Not supported with the `unused-reservation-billing-owner`
    #     filter.
    #
    # @option params [String] :next_token
    #   The token to use to retrieve the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `status` - The state of the request (`pending` \| `accepted` \|
    #     `rejected` \| `cancelled` \| `revoked` \| `expired`).
    #
    #   * `requested-by` - The account ID of the Capacity Reservation owner
    #     that initiated the request. Not supported if you specify
    #     `requested-by` for **Role**.
    #
    #   * `unused-reservation-billing-owner` - The ID of the consumer account
    #     to which the request was sent. Not supported if you specify
    #     `unused-reservation-billing-owner` for **Role**.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeCapacityReservationBillingRequestsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeCapacityReservationBillingRequestsResult#next_token #next_token} => String
    #   * {Types::DescribeCapacityReservationBillingRequestsResult#capacity_reservation_billing_requests #capacity_reservation_billing_requests} => Array&lt;Types::CapacityReservationBillingRequest&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_capacity_reservation_billing_requests({
    #     capacity_reservation_ids: ["CapacityReservationId"],
    #     role: "odcr-owner", # required, accepts odcr-owner, unused-reservation-billing-owner
    #     next_token: "String",
    #     max_results: 1,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.capacity_reservation_billing_requests #=> Array
    #   resp.capacity_reservation_billing_requests[0].capacity_reservation_id #=> String
    #   resp.capacity_reservation_billing_requests[0].requested_by #=> String
    #   resp.capacity_reservation_billing_requests[0].unused_reservation_billing_owner_id #=> String
    #   resp.capacity_reservation_billing_requests[0].last_update_time #=> Time
    #   resp.capacity_reservation_billing_requests[0].status #=> String, one of "pending", "accepted", "rejected", "cancelled", "revoked", "expired"
    #   resp.capacity_reservation_billing_requests[0].status_message #=> String
    #   resp.capacity_reservation_billing_requests[0].capacity_reservation_info.instance_type #=> String
    #   resp.capacity_reservation_billing_requests[0].capacity_reservation_info.availability_zone #=> String
    #   resp.capacity_reservation_billing_requests[0].capacity_reservation_info.tenancy #=> String, one of "default", "dedicated"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeCapacityReservationBillingRequests AWS API Documentation
    #
    # @overload describe_capacity_reservation_billing_requests(params = {})
    # @param [Hash] params ({})
    def describe_capacity_reservation_billing_requests(params = {}, options = {})
      req = build_request(:describe_capacity_reservation_billing_requests, params)
      req.send_request(options)
    end

    # Describes one or more Capacity Reservation Fleets.
    #
    # @option params [Array<String>] :capacity_reservation_fleet_ids
    #   The IDs of the Capacity Reservation Fleets to describe.
    #
    # @option params [String] :next_token
    #   The token to use to retrieve the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `state` - The state of the Fleet (`submitted` \| `modifying` \|
    #     `active` \| `partially_fulfilled` \| `expiring` \| `expired` \|
    #     `cancelling` \| `cancelled` \| `failed`).
    #
    #   * `instance-match-criteria` - The instance matching criteria for the
    #     Fleet. Only `open` is supported.
    #
    #   * `tenancy` - The tenancy of the Fleet (`default` \| `dedicated`).
    #
    #   * `allocation-strategy` - The allocation strategy used by the Fleet.
    #     Only `prioritized` is supported.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeCapacityReservationFleetsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeCapacityReservationFleetsResult#capacity_reservation_fleets #capacity_reservation_fleets} => Array&lt;Types::CapacityReservationFleet&gt;
    #   * {Types::DescribeCapacityReservationFleetsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_capacity_reservation_fleets({
    #     capacity_reservation_fleet_ids: ["CapacityReservationFleetId"],
    #     next_token: "String",
    #     max_results: 1,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.capacity_reservation_fleets #=> Array
    #   resp.capacity_reservation_fleets[0].capacity_reservation_fleet_id #=> String
    #   resp.capacity_reservation_fleets[0].capacity_reservation_fleet_arn #=> String
    #   resp.capacity_reservation_fleets[0].state #=> String, one of "submitted", "modifying", "active", "partially_fulfilled", "expiring", "expired", "cancelling", "cancelled", "failed"
    #   resp.capacity_reservation_fleets[0].total_target_capacity #=> Integer
    #   resp.capacity_reservation_fleets[0].total_fulfilled_capacity #=> Float
    #   resp.capacity_reservation_fleets[0].tenancy #=> String, one of "default"
    #   resp.capacity_reservation_fleets[0].end_date #=> Time
    #   resp.capacity_reservation_fleets[0].create_time #=> Time
    #   resp.capacity_reservation_fleets[0].instance_match_criteria #=> String, one of "open"
    #   resp.capacity_reservation_fleets[0].allocation_strategy #=> String
    #   resp.capacity_reservation_fleets[0].instance_type_specifications #=> Array
    #   resp.capacity_reservation_fleets[0].instance_type_specifications[0].capacity_reservation_id #=> String
    #   resp.capacity_reservation_fleets[0].instance_type_specifications[0].availability_zone_id #=> String
    #   resp.capacity_reservation_fleets[0].instance_type_specifications[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.capacity_reservation_fleets[0].instance_type_specifications[0].instance_platform #=> String, one of "Linux/UNIX", "Red Hat Enterprise Linux", "SUSE Linux", "Windows", "Windows with SQL Server", "Windows with SQL Server Enterprise", "Windows with SQL Server Standard", "Windows with SQL Server Web", "Linux with SQL Server Standard", "Linux with SQL Server Web", "Linux with SQL Server Enterprise", "RHEL with SQL Server Standard", "RHEL with SQL Server Enterprise", "RHEL with SQL Server Web", "RHEL with HA", "RHEL with HA and SQL Server Standard", "RHEL with HA and SQL Server Enterprise", "Ubuntu Pro"
    #   resp.capacity_reservation_fleets[0].instance_type_specifications[0].availability_zone #=> String
    #   resp.capacity_reservation_fleets[0].instance_type_specifications[0].total_instance_count #=> Integer
    #   resp.capacity_reservation_fleets[0].instance_type_specifications[0].fulfilled_capacity #=> Float
    #   resp.capacity_reservation_fleets[0].instance_type_specifications[0].ebs_optimized #=> Boolean
    #   resp.capacity_reservation_fleets[0].instance_type_specifications[0].create_date #=> Time
    #   resp.capacity_reservation_fleets[0].instance_type_specifications[0].weight #=> Float
    #   resp.capacity_reservation_fleets[0].instance_type_specifications[0].priority #=> Integer
    #   resp.capacity_reservation_fleets[0].tags #=> Array
    #   resp.capacity_reservation_fleets[0].tags[0].key #=> String
    #   resp.capacity_reservation_fleets[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeCapacityReservationFleets AWS API Documentation
    #
    # @overload describe_capacity_reservation_fleets(params = {})
    # @param [Hash] params ({})
    def describe_capacity_reservation_fleets(params = {}, options = {})
      req = build_request(:describe_capacity_reservation_fleets, params)
      req.send_request(options)
    end

    # Describes one or more of your Capacity Reservations. The results
    # describe only the Capacity Reservations in the Amazon Web Services
    # Region that you're currently using.
    #
    # @option params [Array<String>] :capacity_reservation_ids
    #   The ID of the Capacity Reservation.
    #
    # @option params [String] :next_token
    #   The token to use to retrieve the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `instance-type` - The type of instance for which the Capacity
    #     Reservation reserves capacity.
    #
    #   * `owner-id` - The ID of the Amazon Web Services account that owns the
    #     Capacity Reservation.
    #
    #   * `instance-platform` - The type of operating system for which the
    #     Capacity Reservation reserves capacity.
    #
    #   * `availability-zone` - The Availability Zone of the Capacity
    #     Reservation.
    #
    #   * `tenancy` - Indicates the tenancy of the Capacity Reservation. A
    #     Capacity Reservation can have one of the following tenancy settings:
    #
    #     * `default` - The Capacity Reservation is created on hardware that
    #       is shared with other Amazon Web Services accounts.
    #
    #     * `dedicated` - The Capacity Reservation is created on single-tenant
    #       hardware that is dedicated to a single Amazon Web Services
    #       account.
    #   * `outpost-arn` - The Amazon Resource Name (ARN) of the Outpost on
    #     which the Capacity Reservation was created.
    #
    #   * `state` - The current state of the Capacity Reservation. A Capacity
    #     Reservation can be in one of the following states:
    #
    #     * `active`- The Capacity Reservation is active and the capacity is
    #       available for your use.
    #
    #     * `expired` - The Capacity Reservation expired automatically at the
    #       date and time specified in your request. The reserved capacity is
    #       no longer available for your use.
    #
    #     * `cancelled` - The Capacity Reservation was cancelled. The reserved
    #       capacity is no longer available for your use.
    #
    #     * `pending` - The Capacity Reservation request was successful but
    #       the capacity provisioning is still pending.
    #
    #     * `failed` - The Capacity Reservation request has failed. A request
    #       might fail due to invalid request parameters, capacity
    #       constraints, or instance limit constraints. Failed requests are
    #       retained for 60 minutes.
    #   * `start-date` - The date and time at which the Capacity Reservation
    #     was started.
    #
    #   * `end-date` - The date and time at which the Capacity Reservation
    #     expires. When a Capacity Reservation expires, the reserved capacity
    #     is released and you can no longer launch instances into it. The
    #     Capacity Reservation's state changes to expired when it reaches its
    #     end date and time.
    #
    #   * `end-date-type` - Indicates the way in which the Capacity
    #     Reservation ends. A Capacity Reservation can have one of the
    #     following end types:
    #
    #     * `unlimited` - The Capacity Reservation remains active until you
    #       explicitly cancel it.
    #
    #     * `limited` - The Capacity Reservation expires automatically at a
    #       specified date and time.
    #   * `instance-match-criteria` - Indicates the type of instance launches
    #     that the Capacity Reservation accepts. The options include:
    #
    #     * `open` - The Capacity Reservation accepts all instances that have
    #       matching attributes (instance type, platform, and Availability
    #       Zone). Instances that have matching attributes launch into the
    #       Capacity Reservation automatically without specifying any
    #       additional parameters.
    #
    #     * `targeted` - The Capacity Reservation only accepts instances that
    #       have matching attributes (instance type, platform, and
    #       Availability Zone), and explicitly target the Capacity
    #       Reservation. This ensures that only permitted instances can use
    #       the reserved capacity.
    #   * `placement-group-arn` - The ARN of the cluster placement group in
    #     which the Capacity Reservation was created.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeCapacityReservationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeCapacityReservationsResult#next_token #next_token} => String
    #   * {Types::DescribeCapacityReservationsResult#capacity_reservations #capacity_reservations} => Array&lt;Types::CapacityReservation&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_capacity_reservations({
    #     capacity_reservation_ids: ["CapacityReservationId"],
    #     next_token: "String",
    #     max_results: 1,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.capacity_reservations #=> Array
    #   resp.capacity_reservations[0].capacity_reservation_id #=> String
    #   resp.capacity_reservations[0].owner_id #=> String
    #   resp.capacity_reservations[0].capacity_reservation_arn #=> String
    #   resp.capacity_reservations[0].availability_zone_id #=> String
    #   resp.capacity_reservations[0].instance_type #=> String
    #   resp.capacity_reservations[0].instance_platform #=> String, one of "Linux/UNIX", "Red Hat Enterprise Linux", "SUSE Linux", "Windows", "Windows with SQL Server", "Windows with SQL Server Enterprise", "Windows with SQL Server Standard", "Windows with SQL Server Web", "Linux with SQL Server Standard", "Linux with SQL Server Web", "Linux with SQL Server Enterprise", "RHEL with SQL Server Standard", "RHEL with SQL Server Enterprise", "RHEL with SQL Server Web", "RHEL with HA", "RHEL with HA and SQL Server Standard", "RHEL with HA and SQL Server Enterprise", "Ubuntu Pro"
    #   resp.capacity_reservations[0].availability_zone #=> String
    #   resp.capacity_reservations[0].tenancy #=> String, one of "default", "dedicated"
    #   resp.capacity_reservations[0].total_instance_count #=> Integer
    #   resp.capacity_reservations[0].available_instance_count #=> Integer
    #   resp.capacity_reservations[0].ebs_optimized #=> Boolean
    #   resp.capacity_reservations[0].ephemeral_storage #=> Boolean
    #   resp.capacity_reservations[0].state #=> String, one of "active", "expired", "cancelled", "pending", "failed", "scheduled", "payment-pending", "payment-failed", "assessing", "delayed", "unsupported"
    #   resp.capacity_reservations[0].start_date #=> Time
    #   resp.capacity_reservations[0].end_date #=> Time
    #   resp.capacity_reservations[0].end_date_type #=> String, one of "unlimited", "limited"
    #   resp.capacity_reservations[0].instance_match_criteria #=> String, one of "open", "targeted"
    #   resp.capacity_reservations[0].create_date #=> Time
    #   resp.capacity_reservations[0].tags #=> Array
    #   resp.capacity_reservations[0].tags[0].key #=> String
    #   resp.capacity_reservations[0].tags[0].value #=> String
    #   resp.capacity_reservations[0].outpost_arn #=> String
    #   resp.capacity_reservations[0].capacity_reservation_fleet_id #=> String
    #   resp.capacity_reservations[0].placement_group_arn #=> String
    #   resp.capacity_reservations[0].capacity_allocations #=> Array
    #   resp.capacity_reservations[0].capacity_allocations[0].allocation_type #=> String, one of "used", "future"
    #   resp.capacity_reservations[0].capacity_allocations[0].count #=> Integer
    #   resp.capacity_reservations[0].reservation_type #=> String, one of "default", "capacity-block"
    #   resp.capacity_reservations[0].unused_reservation_billing_owner_id #=> String
    #   resp.capacity_reservations[0].commitment_info.committed_instance_count #=> Integer
    #   resp.capacity_reservations[0].commitment_info.commitment_end_date #=> Time
    #   resp.capacity_reservations[0].delivery_preference #=> String, one of "fixed", "incremental"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeCapacityReservations AWS API Documentation
    #
    # @overload describe_capacity_reservations(params = {})
    # @param [Hash] params ({})
    def describe_capacity_reservations(params = {}, options = {})
      req = build_request(:describe_capacity_reservations, params)
      req.send_request(options)
    end

    # Describes one or more of your carrier gateways.
    #
    # @option params [Array<String>] :carrier_gateway_ids
    #   One or more carrier gateway IDs.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `carrier-gateway-id` - The ID of the carrier gateway.
    #
    #   * `state` - The state of the carrier gateway (`pending` \| `failed` \|
    #     `available` \| `deleting` \| `deleted`).
    #
    #   * `owner-id` - The Amazon Web Services account ID of the owner of the
    #     carrier gateway.
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `vpc-id` - The ID of the VPC associated with the carrier gateway.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeCarrierGatewaysResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeCarrierGatewaysResult#carrier_gateways #carrier_gateways} => Array&lt;Types::CarrierGateway&gt;
    #   * {Types::DescribeCarrierGatewaysResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_carrier_gateways({
    #     carrier_gateway_ids: ["CarrierGatewayId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.carrier_gateways #=> Array
    #   resp.carrier_gateways[0].carrier_gateway_id #=> String
    #   resp.carrier_gateways[0].vpc_id #=> String
    #   resp.carrier_gateways[0].state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.carrier_gateways[0].owner_id #=> String
    #   resp.carrier_gateways[0].tags #=> Array
    #   resp.carrier_gateways[0].tags[0].key #=> String
    #   resp.carrier_gateways[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeCarrierGateways AWS API Documentation
    #
    # @overload describe_carrier_gateways(params = {})
    # @param [Hash] params ({})
    def describe_carrier_gateways(params = {}, options = {})
      req = build_request(:describe_carrier_gateways, params)
      req.send_request(options)
    end

    # <note markdown="1"> This action is deprecated.
    #
    #  </note>
    #
    # Describes your linked EC2-Classic instances. This request only returns
    # information about EC2-Classic instances linked to a VPC through
    # ClassicLink. You cannot use this request to return information about
    # other instances.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :instance_ids
    #   The instance IDs. Must be instances linked to a VPC through
    #   ClassicLink.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `group-id` - The ID of a VPC security group that's associated with
    #     the instance.
    #
    #   * `instance-id` - The ID of the instance.
    #
    #   * `tag` - The key/value combination of a tag assigned to the resource.
    #     Use the tag key in the filter name and the tag value as the filter
    #     value. For example, to find all resources that have a tag with the
    #     key `Owner` and the value `TeamA`, specify `tag:Owner` for the
    #     filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `vpc-id` - The ID of the VPC to which the instance is linked.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #   Constraint: If the value is greater than 1000, we return only 1000
    #   items.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @return [Types::DescribeClassicLinkInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeClassicLinkInstancesResult#instances #instances} => Array&lt;Types::ClassicLinkInstance&gt;
    #   * {Types::DescribeClassicLinkInstancesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_classic_link_instances({
    #     dry_run: false,
    #     instance_ids: ["InstanceId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.instances #=> Array
    #   resp.instances[0].groups #=> Array
    #   resp.instances[0].groups[0].group_id #=> String
    #   resp.instances[0].groups[0].group_name #=> String
    #   resp.instances[0].instance_id #=> String
    #   resp.instances[0].tags #=> Array
    #   resp.instances[0].tags[0].key #=> String
    #   resp.instances[0].tags[0].value #=> String
    #   resp.instances[0].vpc_id #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeClassicLinkInstances AWS API Documentation
    #
    # @overload describe_classic_link_instances(params = {})
    # @param [Hash] params ({})
    def describe_classic_link_instances(params = {}, options = {})
      req = build_request(:describe_classic_link_instances, params)
      req.send_request(options)
    end

    # Describes the authorization rules for a specified Client VPN endpoint.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :next_token
    #   The token to retrieve the next page of results.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. Filter names and values are case-sensitive.
    #
    #   * `description` - The description of the authorization rule.
    #
    #   * `destination-cidr` - The CIDR of the network to which the
    #     authorization rule applies.
    #
    #   * `group-id` - The ID of the Active Directory group to which the
    #     authorization rule grants access.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return for the request in a single
    #   page. The remaining results can be seen by sending another request
    #   with the nextToken value.
    #
    # @return [Types::DescribeClientVpnAuthorizationRulesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeClientVpnAuthorizationRulesResult#authorization_rules #authorization_rules} => Array&lt;Types::AuthorizationRule&gt;
    #   * {Types::DescribeClientVpnAuthorizationRulesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_client_vpn_authorization_rules({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     dry_run: false,
    #     next_token: "NextToken",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.authorization_rules #=> Array
    #   resp.authorization_rules[0].client_vpn_endpoint_id #=> String
    #   resp.authorization_rules[0].description #=> String
    #   resp.authorization_rules[0].group_id #=> String
    #   resp.authorization_rules[0].access_all #=> Boolean
    #   resp.authorization_rules[0].destination_cidr #=> String
    #   resp.authorization_rules[0].status.code #=> String, one of "authorizing", "active", "failed", "revoking"
    #   resp.authorization_rules[0].status.message #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeClientVpnAuthorizationRules AWS API Documentation
    #
    # @overload describe_client_vpn_authorization_rules(params = {})
    # @param [Hash] params ({})
    def describe_client_vpn_authorization_rules(params = {}, options = {})
      req = build_request(:describe_client_vpn_authorization_rules, params)
      req.send_request(options)
    end

    # Describes active client connections and connections that have been
    # terminated within the last 60 minutes for the specified Client VPN
    # endpoint.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. Filter names and values are case-sensitive.
    #
    #   * `connection-id` - The ID of the connection.
    #
    #   * `username` - For Active Directory client authentication, the user
    #     name of the client who established the client connection.
    #
    # @option params [String] :next_token
    #   The token to retrieve the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return for the request in a single
    #   page. The remaining results can be seen by sending another request
    #   with the nextToken value.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeClientVpnConnectionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeClientVpnConnectionsResult#connections #connections} => Array&lt;Types::ClientVpnConnection&gt;
    #   * {Types::DescribeClientVpnConnectionsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_client_vpn_connections({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "NextToken",
    #     max_results: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.connections #=> Array
    #   resp.connections[0].client_vpn_endpoint_id #=> String
    #   resp.connections[0].timestamp #=> String
    #   resp.connections[0].connection_id #=> String
    #   resp.connections[0].username #=> String
    #   resp.connections[0].connection_established_time #=> String
    #   resp.connections[0].ingress_bytes #=> String
    #   resp.connections[0].egress_bytes #=> String
    #   resp.connections[0].ingress_packets #=> String
    #   resp.connections[0].egress_packets #=> String
    #   resp.connections[0].client_ip #=> String
    #   resp.connections[0].common_name #=> String
    #   resp.connections[0].status.code #=> String, one of "active", "failed-to-terminate", "terminating", "terminated"
    #   resp.connections[0].status.message #=> String
    #   resp.connections[0].connection_end_time #=> String
    #   resp.connections[0].posture_compliance_statuses #=> Array
    #   resp.connections[0].posture_compliance_statuses[0] #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeClientVpnConnections AWS API Documentation
    #
    # @overload describe_client_vpn_connections(params = {})
    # @param [Hash] params ({})
    def describe_client_vpn_connections(params = {}, options = {})
      req = build_request(:describe_client_vpn_connections, params)
      req.send_request(options)
    end

    # Describes one or more Client VPN endpoints in the account.
    #
    # @option params [Array<String>] :client_vpn_endpoint_ids
    #   The ID of the Client VPN endpoint.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return for the request in a single
    #   page. The remaining results can be seen by sending another request
    #   with the nextToken value.
    #
    # @option params [String] :next_token
    #   The token to retrieve the next page of results.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. Filter names and values are case-sensitive.
    #
    #   * `endpoint-id` - The ID of the Client VPN endpoint.
    #
    #   * `transport-protocol` - The transport protocol (`tcp` \| `udp`).
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeClientVpnEndpointsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeClientVpnEndpointsResult#client_vpn_endpoints #client_vpn_endpoints} => Array&lt;Types::ClientVpnEndpoint&gt;
    #   * {Types::DescribeClientVpnEndpointsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_client_vpn_endpoints({
    #     client_vpn_endpoint_ids: ["ClientVpnEndpointId"],
    #     max_results: 1,
    #     next_token: "NextToken",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.client_vpn_endpoints #=> Array
    #   resp.client_vpn_endpoints[0].client_vpn_endpoint_id #=> String
    #   resp.client_vpn_endpoints[0].description #=> String
    #   resp.client_vpn_endpoints[0].status.code #=> String, one of "pending-associate", "available", "deleting", "deleted"
    #   resp.client_vpn_endpoints[0].status.message #=> String
    #   resp.client_vpn_endpoints[0].creation_time #=> String
    #   resp.client_vpn_endpoints[0].deletion_time #=> String
    #   resp.client_vpn_endpoints[0].dns_name #=> String
    #   resp.client_vpn_endpoints[0].client_cidr_block #=> String
    #   resp.client_vpn_endpoints[0].dns_servers #=> Array
    #   resp.client_vpn_endpoints[0].dns_servers[0] #=> String
    #   resp.client_vpn_endpoints[0].split_tunnel #=> Boolean
    #   resp.client_vpn_endpoints[0].vpn_protocol #=> String, one of "openvpn"
    #   resp.client_vpn_endpoints[0].transport_protocol #=> String, one of "tcp", "udp"
    #   resp.client_vpn_endpoints[0].vpn_port #=> Integer
    #   resp.client_vpn_endpoints[0].associated_target_networks #=> Array
    #   resp.client_vpn_endpoints[0].associated_target_networks[0].network_id #=> String
    #   resp.client_vpn_endpoints[0].associated_target_networks[0].network_type #=> String, one of "vpc"
    #   resp.client_vpn_endpoints[0].server_certificate_arn #=> String
    #   resp.client_vpn_endpoints[0].authentication_options #=> Array
    #   resp.client_vpn_endpoints[0].authentication_options[0].type #=> String, one of "certificate-authentication", "directory-service-authentication", "federated-authentication"
    #   resp.client_vpn_endpoints[0].authentication_options[0].active_directory.directory_id #=> String
    #   resp.client_vpn_endpoints[0].authentication_options[0].mutual_authentication.client_root_certificate_chain #=> String
    #   resp.client_vpn_endpoints[0].authentication_options[0].federated_authentication.saml_provider_arn #=> String
    #   resp.client_vpn_endpoints[0].authentication_options[0].federated_authentication.self_service_saml_provider_arn #=> String
    #   resp.client_vpn_endpoints[0].connection_log_options.enabled #=> Boolean
    #   resp.client_vpn_endpoints[0].connection_log_options.cloudwatch_log_group #=> String
    #   resp.client_vpn_endpoints[0].connection_log_options.cloudwatch_log_stream #=> String
    #   resp.client_vpn_endpoints[0].tags #=> Array
    #   resp.client_vpn_endpoints[0].tags[0].key #=> String
    #   resp.client_vpn_endpoints[0].tags[0].value #=> String
    #   resp.client_vpn_endpoints[0].security_group_ids #=> Array
    #   resp.client_vpn_endpoints[0].security_group_ids[0] #=> String
    #   resp.client_vpn_endpoints[0].vpc_id #=> String
    #   resp.client_vpn_endpoints[0].self_service_portal_url #=> String
    #   resp.client_vpn_endpoints[0].client_connect_options.enabled #=> Boolean
    #   resp.client_vpn_endpoints[0].client_connect_options.lambda_function_arn #=> String
    #   resp.client_vpn_endpoints[0].client_connect_options.status.code #=> String, one of "applying", "applied"
    #   resp.client_vpn_endpoints[0].client_connect_options.status.message #=> String
    #   resp.client_vpn_endpoints[0].session_timeout_hours #=> Integer
    #   resp.client_vpn_endpoints[0].client_login_banner_options.enabled #=> Boolean
    #   resp.client_vpn_endpoints[0].client_login_banner_options.banner_text #=> String
    #   resp.client_vpn_endpoints[0].disconnect_on_session_timeout #=> Boolean
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeClientVpnEndpoints AWS API Documentation
    #
    # @overload describe_client_vpn_endpoints(params = {})
    # @param [Hash] params ({})
    def describe_client_vpn_endpoints(params = {}, options = {})
      req = build_request(:describe_client_vpn_endpoints, params)
      req.send_request(options)
    end

    # Describes the routes for the specified Client VPN endpoint.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. Filter names and values are case-sensitive.
    #
    #   * `destination-cidr` - The CIDR of the route destination.
    #
    #   * `origin` - How the route was associated with the Client VPN endpoint
    #     (`associate` \| `add-route`).
    #
    #   * `target-subnet` - The ID of the subnet through which traffic is
    #     routed.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return for the request in a single
    #   page. The remaining results can be seen by sending another request
    #   with the nextToken value.
    #
    # @option params [String] :next_token
    #   The token to retrieve the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeClientVpnRoutesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeClientVpnRoutesResult#routes #routes} => Array&lt;Types::ClientVpnRoute&gt;
    #   * {Types::DescribeClientVpnRoutesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_client_vpn_routes({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.routes #=> Array
    #   resp.routes[0].client_vpn_endpoint_id #=> String
    #   resp.routes[0].destination_cidr #=> String
    #   resp.routes[0].target_subnet #=> String
    #   resp.routes[0].type #=> String
    #   resp.routes[0].origin #=> String
    #   resp.routes[0].status.code #=> String, one of "creating", "active", "failed", "deleting"
    #   resp.routes[0].status.message #=> String
    #   resp.routes[0].description #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeClientVpnRoutes AWS API Documentation
    #
    # @overload describe_client_vpn_routes(params = {})
    # @param [Hash] params ({})
    def describe_client_vpn_routes(params = {}, options = {})
      req = build_request(:describe_client_vpn_routes, params)
      req.send_request(options)
    end

    # Describes the target networks associated with the specified Client VPN
    # endpoint.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint.
    #
    # @option params [Array<String>] :association_ids
    #   The IDs of the target network associations.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return for the request in a single
    #   page. The remaining results can be seen by sending another request
    #   with the nextToken value.
    #
    # @option params [String] :next_token
    #   The token to retrieve the next page of results.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. Filter names and values are case-sensitive.
    #
    #   * `association-id` - The ID of the association.
    #
    #   * `target-network-id` - The ID of the subnet specified as the target
    #     network.
    #
    #   * `vpc-id` - The ID of the VPC in which the target network is located.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeClientVpnTargetNetworksResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeClientVpnTargetNetworksResult#client_vpn_target_networks #client_vpn_target_networks} => Array&lt;Types::TargetNetwork&gt;
    #   * {Types::DescribeClientVpnTargetNetworksResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_client_vpn_target_networks({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     association_ids: ["String"],
    #     max_results: 1,
    #     next_token: "NextToken",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.client_vpn_target_networks #=> Array
    #   resp.client_vpn_target_networks[0].association_id #=> String
    #   resp.client_vpn_target_networks[0].vpc_id #=> String
    #   resp.client_vpn_target_networks[0].target_network_id #=> String
    #   resp.client_vpn_target_networks[0].client_vpn_endpoint_id #=> String
    #   resp.client_vpn_target_networks[0].status.code #=> String, one of "associating", "associated", "association-failed", "disassociating", "disassociated"
    #   resp.client_vpn_target_networks[0].status.message #=> String
    #   resp.client_vpn_target_networks[0].security_groups #=> Array
    #   resp.client_vpn_target_networks[0].security_groups[0] #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeClientVpnTargetNetworks AWS API Documentation
    #
    # @overload describe_client_vpn_target_networks(params = {})
    # @param [Hash] params ({})
    def describe_client_vpn_target_networks(params = {}, options = {})
      req = build_request(:describe_client_vpn_target_networks, params)
      req.send_request(options)
    end

    # Describes the specified customer-owned address pools or all of your
    # customer-owned address pools.
    #
    # @option params [Array<String>] :pool_ids
    #   The IDs of the address pools.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `coip-pool.local-gateway-route-table-id` - The ID of the local
    #     gateway route table.
    #
    #   * `coip-pool.pool-id` - The ID of the address pool.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeCoipPoolsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeCoipPoolsResult#coip_pools #coip_pools} => Array&lt;Types::CoipPool&gt;
    #   * {Types::DescribeCoipPoolsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_coip_pools({
    #     pool_ids: ["Ipv4PoolCoipId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.coip_pools #=> Array
    #   resp.coip_pools[0].pool_id #=> String
    #   resp.coip_pools[0].pool_cidrs #=> Array
    #   resp.coip_pools[0].pool_cidrs[0] #=> String
    #   resp.coip_pools[0].local_gateway_route_table_id #=> String
    #   resp.coip_pools[0].tags #=> Array
    #   resp.coip_pools[0].tags[0].key #=> String
    #   resp.coip_pools[0].tags[0].value #=> String
    #   resp.coip_pools[0].pool_arn #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeCoipPools AWS API Documentation
    #
    # @overload describe_coip_pools(params = {})
    # @param [Hash] params ({})
    def describe_coip_pools(params = {}, options = {})
      req = build_request(:describe_coip_pools, params)
      req.send_request(options)
    end

    # Describes the specified conversion tasks or all your conversion tasks.
    # For more information, see the [VM Import/Export User Guide][1].
    #
    # For information about the import manifest referenced by this API
    # action, see [VM Import Manifest][2].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vm-import/latest/userguide/
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/manifest.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :conversion_task_ids
    #   The conversion task IDs.
    #
    # @return [Types::DescribeConversionTasksResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeConversionTasksResult#conversion_tasks #conversion_tasks} => Array&lt;Types::ConversionTask&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_conversion_tasks({
    #     dry_run: false,
    #     conversion_task_ids: ["ConversionTaskId"],
    #   })
    #
    # @example Response structure
    #
    #   resp.conversion_tasks #=> Array
    #   resp.conversion_tasks[0].conversion_task_id #=> String
    #   resp.conversion_tasks[0].expiration_time #=> String
    #   resp.conversion_tasks[0].import_instance.description #=> String
    #   resp.conversion_tasks[0].import_instance.instance_id #=> String
    #   resp.conversion_tasks[0].import_instance.platform #=> String, one of "Windows"
    #   resp.conversion_tasks[0].import_instance.volumes #=> Array
    #   resp.conversion_tasks[0].import_instance.volumes[0].availability_zone #=> String
    #   resp.conversion_tasks[0].import_instance.volumes[0].bytes_converted #=> Integer
    #   resp.conversion_tasks[0].import_instance.volumes[0].description #=> String
    #   resp.conversion_tasks[0].import_instance.volumes[0].image.checksum #=> String
    #   resp.conversion_tasks[0].import_instance.volumes[0].image.format #=> String, one of "VMDK", "RAW", "VHD"
    #   resp.conversion_tasks[0].import_instance.volumes[0].image.import_manifest_url #=> String
    #   resp.conversion_tasks[0].import_instance.volumes[0].image.size #=> Integer
    #   resp.conversion_tasks[0].import_instance.volumes[0].status #=> String
    #   resp.conversion_tasks[0].import_instance.volumes[0].status_message #=> String
    #   resp.conversion_tasks[0].import_instance.volumes[0].volume.id #=> String
    #   resp.conversion_tasks[0].import_instance.volumes[0].volume.size #=> Integer
    #   resp.conversion_tasks[0].import_volume.availability_zone #=> String
    #   resp.conversion_tasks[0].import_volume.bytes_converted #=> Integer
    #   resp.conversion_tasks[0].import_volume.description #=> String
    #   resp.conversion_tasks[0].import_volume.image.checksum #=> String
    #   resp.conversion_tasks[0].import_volume.image.format #=> String, one of "VMDK", "RAW", "VHD"
    #   resp.conversion_tasks[0].import_volume.image.import_manifest_url #=> String
    #   resp.conversion_tasks[0].import_volume.image.size #=> Integer
    #   resp.conversion_tasks[0].import_volume.volume.id #=> String
    #   resp.conversion_tasks[0].import_volume.volume.size #=> Integer
    #   resp.conversion_tasks[0].state #=> String, one of "active", "cancelling", "cancelled", "completed"
    #   resp.conversion_tasks[0].status_message #=> String
    #   resp.conversion_tasks[0].tags #=> Array
    #   resp.conversion_tasks[0].tags[0].key #=> String
    #   resp.conversion_tasks[0].tags[0].value #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * conversion_task_cancelled
    #   * conversion_task_completed
    #   * conversion_task_deleted
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeConversionTasks AWS API Documentation
    #
    # @overload describe_conversion_tasks(params = {})
    # @param [Hash] params ({})
    def describe_conversion_tasks(params = {}, options = {})
      req = build_request(:describe_conversion_tasks, params)
      req.send_request(options)
    end

    # Describes one or more of your VPN customer gateways.
    #
    # For more information, see [Amazon Web Services Site-to-Site VPN][1] in
    # the *Amazon Web Services Site-to-Site VPN User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html
    #
    # @option params [Array<String>] :customer_gateway_ids
    #   One or more customer gateway IDs.
    #
    #   Default: Describes all your customer gateways.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `bgp-asn` - The customer gateway's Border Gateway Protocol (BGP)
    #     Autonomous System Number (ASN).
    #
    #   * `customer-gateway-id` - The ID of the customer gateway.
    #
    #   * `ip-address` - The IP address of the customer gateway device's
    #     external interface.
    #
    #   * `state` - The state of the customer gateway (`pending` \|
    #     `available` \| `deleting` \| `deleted`).
    #
    #   * `type` - The type of customer gateway. Currently, the only supported
    #     type is `ipsec.1`.
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeCustomerGatewaysResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeCustomerGatewaysResult#customer_gateways #customer_gateways} => Array&lt;Types::CustomerGateway&gt;
    #
    #
    # @example Example: To describe a customer gateway
    #
    #   # This example describes the specified customer gateway.
    #
    #   resp = client.describe_customer_gateways({
    #     customer_gateway_ids: [
    #       "cgw-0e11f167", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     customer_gateways: [
    #       {
    #         bgp_asn: "65534", 
    #         customer_gateway_id: "cgw-0e11f167", 
    #         ip_address: "12.1.2.3", 
    #         state: "available", 
    #         type: "ipsec.1", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_customer_gateways({
    #     customer_gateway_ids: ["CustomerGatewayId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.customer_gateways #=> Array
    #   resp.customer_gateways[0].certificate_arn #=> String
    #   resp.customer_gateways[0].device_name #=> String
    #   resp.customer_gateways[0].tags #=> Array
    #   resp.customer_gateways[0].tags[0].key #=> String
    #   resp.customer_gateways[0].tags[0].value #=> String
    #   resp.customer_gateways[0].bgp_asn_extended #=> String
    #   resp.customer_gateways[0].customer_gateway_id #=> String
    #   resp.customer_gateways[0].state #=> String
    #   resp.customer_gateways[0].type #=> String
    #   resp.customer_gateways[0].ip_address #=> String
    #   resp.customer_gateways[0].bgp_asn #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * customer_gateway_available
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeCustomerGateways AWS API Documentation
    #
    # @overload describe_customer_gateways(params = {})
    # @param [Hash] params ({})
    def describe_customer_gateways(params = {}, options = {})
      req = build_request(:describe_customer_gateways, params)
      req.send_request(options)
    end

    # Describes the metadata of an account status report, including the
    # status of the report.
    #
    # To view the full report, download it from the Amazon S3 bucket where
    # it was saved. Reports are accessible only when they have the
    # `complete` status. Reports with other statuses (`running`,
    # `cancelled`, or `error`) are not available in the S3 bucket. For more
    # information about downloading objects from an S3 bucket, see
    # [Downloading objects][1] in the *Amazon Simple Storage Service User
    # Guide*.
    #
    # For more information, see [Generating the account status report for
    # declarative policies][2] in the *Amazon Web Services Organizations
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AmazonS3/latest/userguide/download-objects.html
    # [2]: https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_policies_declarative_status-report.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Array<String>] :report_ids
    #   One or more report IDs.
    #
    # @return [Types::DescribeDeclarativePoliciesReportsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeDeclarativePoliciesReportsResult#next_token #next_token} => String
    #   * {Types::DescribeDeclarativePoliciesReportsResult#reports #reports} => Array&lt;Types::DeclarativePoliciesReport&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_declarative_policies_reports({
    #     dry_run: false,
    #     next_token: "String",
    #     max_results: 1,
    #     report_ids: ["String"],
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.reports #=> Array
    #   resp.reports[0].report_id #=> String
    #   resp.reports[0].s3_bucket #=> String
    #   resp.reports[0].s3_prefix #=> String
    #   resp.reports[0].target_id #=> String
    #   resp.reports[0].start_time #=> Time
    #   resp.reports[0].end_time #=> Time
    #   resp.reports[0].status #=> String, one of "running", "cancelled", "complete", "error"
    #   resp.reports[0].tags #=> Array
    #   resp.reports[0].tags[0].key #=> String
    #   resp.reports[0].tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeDeclarativePoliciesReports AWS API Documentation
    #
    # @overload describe_declarative_policies_reports(params = {})
    # @param [Hash] params ({})
    def describe_declarative_policies_reports(params = {}, options = {})
      req = build_request(:describe_declarative_policies_reports, params)
      req.send_request(options)
    end

    # Describes your DHCP option sets. The default is to describe all your
    # DHCP option sets. Alternatively, you can specify specific DHCP option
    # set IDs or filter the results to include only the DHCP option sets
    # that match specific criteria.
    #
    # For more information, see [DHCP option sets][1] in the *Amazon VPC
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/VPC_DHCP_Options.html
    #
    # @option params [Array<String>] :dhcp_options_ids
    #   The IDs of DHCP option sets.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `dhcp-options-id` - The ID of a DHCP options set.
    #
    #   * `key` - The key for one of the options (for example, `domain-name`).
    #
    #   * `value` - The value for one of the options.
    #
    #   * `owner-id` - The ID of the Amazon Web Services account that owns the
    #     DHCP options set.
    #
    #   * `tag` - The key/value combination of a tag assigned to the resource.
    #     Use the tag key in the filter name and the tag value as the filter
    #     value. For example, to find all resources that have a tag with the
    #     key `Owner` and the value `TeamA`, specify `tag:Owner` for the
    #     filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @return [Types::DescribeDhcpOptionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeDhcpOptionsResult#next_token #next_token} => String
    #   * {Types::DescribeDhcpOptionsResult#dhcp_options #dhcp_options} => Array&lt;Types::DhcpOptions&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe a DHCP options set
    #
    #   # This example describes the specified DHCP options set.
    #
    #   resp = client.describe_dhcp_options({
    #     dhcp_options_ids: [
    #       "dopt-d9070ebb", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     dhcp_options: [
    #       {
    #         dhcp_configurations: [
    #           {
    #             key: "domain-name-servers", 
    #             values: [
    #               {
    #                 value: "10.2.5.2", 
    #               }, 
    #               {
    #                 value: "10.2.5.1", 
    #               }, 
    #             ], 
    #           }, 
    #         ], 
    #         dhcp_options_id: "dopt-d9070ebb", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_dhcp_options({
    #     dhcp_options_ids: ["DhcpOptionsId"],
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.dhcp_options #=> Array
    #   resp.dhcp_options[0].owner_id #=> String
    #   resp.dhcp_options[0].tags #=> Array
    #   resp.dhcp_options[0].tags[0].key #=> String
    #   resp.dhcp_options[0].tags[0].value #=> String
    #   resp.dhcp_options[0].dhcp_options_id #=> String
    #   resp.dhcp_options[0].dhcp_configurations #=> Array
    #   resp.dhcp_options[0].dhcp_configurations[0].key #=> String
    #   resp.dhcp_options[0].dhcp_configurations[0].values #=> Array
    #   resp.dhcp_options[0].dhcp_configurations[0].values[0] #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeDhcpOptions AWS API Documentation
    #
    # @overload describe_dhcp_options(params = {})
    # @param [Hash] params ({})
    def describe_dhcp_options(params = {}, options = {})
      req = build_request(:describe_dhcp_options, params)
      req.send_request(options)
    end

    # Describes your egress-only internet gateways. The default is to
    # describe all your egress-only internet gateways. Alternatively, you
    # can specify specific egress-only internet gateway IDs or filter the
    # results to include only the egress-only internet gateways that match
    # specific criteria.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :egress_only_internet_gateway_ids
    #   The IDs of the egress-only internet gateways.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `tag` - The key/value combination of a tag assigned to the resource.
    #     Use the tag key in the filter name and the tag value as the filter
    #     value. For example, to find all resources that have a tag with the
    #     key `Owner` and the value `TeamA`, specify `tag:Owner` for the
    #     filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @return [Types::DescribeEgressOnlyInternetGatewaysResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeEgressOnlyInternetGatewaysResult#egress_only_internet_gateways #egress_only_internet_gateways} => Array&lt;Types::EgressOnlyInternetGateway&gt;
    #   * {Types::DescribeEgressOnlyInternetGatewaysResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_egress_only_internet_gateways({
    #     dry_run: false,
    #     egress_only_internet_gateway_ids: ["EgressOnlyInternetGatewayId"],
    #     max_results: 1,
    #     next_token: "String",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.egress_only_internet_gateways #=> Array
    #   resp.egress_only_internet_gateways[0].attachments #=> Array
    #   resp.egress_only_internet_gateways[0].attachments[0].state #=> String, one of "attaching", "attached", "detaching", "detached"
    #   resp.egress_only_internet_gateways[0].attachments[0].vpc_id #=> String
    #   resp.egress_only_internet_gateways[0].egress_only_internet_gateway_id #=> String
    #   resp.egress_only_internet_gateways[0].tags #=> Array
    #   resp.egress_only_internet_gateways[0].tags[0].key #=> String
    #   resp.egress_only_internet_gateways[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeEgressOnlyInternetGateways AWS API Documentation
    #
    # @overload describe_egress_only_internet_gateways(params = {})
    # @param [Hash] params ({})
    def describe_egress_only_internet_gateways(params = {}, options = {})
      req = build_request(:describe_egress_only_internet_gateways, params)
      req.send_request(options)
    end

    # <note markdown="1"> Amazon Elastic Graphics reached end of life on January 8, 2024.
    #
    #  </note>
    #
    # Describes the Elastic Graphics accelerator associated with your
    # instances.
    #
    # @option params [Array<String>] :elastic_gpu_ids
    #   The Elastic Graphics accelerator IDs.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `availability-zone` - The Availability Zone in which the Elastic
    #     Graphics accelerator resides.
    #
    #   * `elastic-gpu-health` - The status of the Elastic Graphics
    #     accelerator (`OK` \| `IMPAIRED`).
    #
    #   * `elastic-gpu-state` - The state of the Elastic Graphics accelerator
    #     (`ATTACHED`).
    #
    #   * `elastic-gpu-type` - The type of Elastic Graphics accelerator; for
    #     example, `eg1.medium`.
    #
    #   * `instance-id` - The ID of the instance to which the Elastic Graphics
    #     accelerator is associated.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in a single call. To retrieve
    #   the remaining results, make another call with the returned `NextToken`
    #   value. This value can be between 5 and 1000.
    #
    # @option params [String] :next_token
    #   The token to request the next page of results.
    #
    # @return [Types::DescribeElasticGpusResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeElasticGpusResult#elastic_gpu_set #elastic_gpu_set} => Array&lt;Types::ElasticGpus&gt;
    #   * {Types::DescribeElasticGpusResult#max_results #max_results} => Integer
    #   * {Types::DescribeElasticGpusResult#next_token #next_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_elastic_gpus({
    #     elastic_gpu_ids: ["ElasticGpuId"],
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.elastic_gpu_set #=> Array
    #   resp.elastic_gpu_set[0].elastic_gpu_id #=> String
    #   resp.elastic_gpu_set[0].availability_zone #=> String
    #   resp.elastic_gpu_set[0].elastic_gpu_type #=> String
    #   resp.elastic_gpu_set[0].elastic_gpu_health.status #=> String, one of "OK", "IMPAIRED"
    #   resp.elastic_gpu_set[0].elastic_gpu_state #=> String, one of "ATTACHED"
    #   resp.elastic_gpu_set[0].instance_id #=> String
    #   resp.elastic_gpu_set[0].tags #=> Array
    #   resp.elastic_gpu_set[0].tags[0].key #=> String
    #   resp.elastic_gpu_set[0].tags[0].value #=> String
    #   resp.max_results #=> Integer
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeElasticGpus AWS API Documentation
    #
    # @overload describe_elastic_gpus(params = {})
    # @param [Hash] params ({})
    def describe_elastic_gpus(params = {}, options = {})
      req = build_request(:describe_elastic_gpus, params)
      req.send_request(options)
    end

    # Describes the specified export image tasks or all of your export image
    # tasks.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   Filter tasks using the `task-state` filter and one of the following
    #   values: `active`, `completed`, `deleting`, or `deleted`.
    #
    # @option params [Array<String>] :export_image_task_ids
    #   The IDs of the export image tasks.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in a single call.
    #
    # @option params [String] :next_token
    #   A token that indicates the next page of results.
    #
    # @return [Types::DescribeExportImageTasksResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeExportImageTasksResult#export_image_tasks #export_image_tasks} => Array&lt;Types::ExportImageTask&gt;
    #   * {Types::DescribeExportImageTasksResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_export_image_tasks({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     export_image_task_ids: ["ExportImageTaskId"],
    #     max_results: 1,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.export_image_tasks #=> Array
    #   resp.export_image_tasks[0].description #=> String
    #   resp.export_image_tasks[0].export_image_task_id #=> String
    #   resp.export_image_tasks[0].image_id #=> String
    #   resp.export_image_tasks[0].progress #=> String
    #   resp.export_image_tasks[0].s3_export_location.s3_bucket #=> String
    #   resp.export_image_tasks[0].s3_export_location.s3_prefix #=> String
    #   resp.export_image_tasks[0].status #=> String
    #   resp.export_image_tasks[0].status_message #=> String
    #   resp.export_image_tasks[0].tags #=> Array
    #   resp.export_image_tasks[0].tags[0].key #=> String
    #   resp.export_image_tasks[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeExportImageTasks AWS API Documentation
    #
    # @overload describe_export_image_tasks(params = {})
    # @param [Hash] params ({})
    def describe_export_image_tasks(params = {}, options = {})
      req = build_request(:describe_export_image_tasks, params)
      req.send_request(options)
    end

    # Describes the specified export instance tasks or all of your export
    # instance tasks.
    #
    # @option params [Array<Types::Filter>] :filters
    #   the filters for the export tasks.
    #
    # @option params [Array<String>] :export_task_ids
    #   The export task IDs.
    #
    # @return [Types::DescribeExportTasksResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeExportTasksResult#export_tasks #export_tasks} => Array&lt;Types::ExportTask&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_export_tasks({
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     export_task_ids: ["ExportTaskId"],
    #   })
    #
    # @example Response structure
    #
    #   resp.export_tasks #=> Array
    #   resp.export_tasks[0].description #=> String
    #   resp.export_tasks[0].export_task_id #=> String
    #   resp.export_tasks[0].export_to_s3_task.container_format #=> String, one of "ova"
    #   resp.export_tasks[0].export_to_s3_task.disk_image_format #=> String, one of "VMDK", "RAW", "VHD"
    #   resp.export_tasks[0].export_to_s3_task.s3_bucket #=> String
    #   resp.export_tasks[0].export_to_s3_task.s3_key #=> String
    #   resp.export_tasks[0].instance_export_details.instance_id #=> String
    #   resp.export_tasks[0].instance_export_details.target_environment #=> String, one of "citrix", "vmware", "microsoft"
    #   resp.export_tasks[0].state #=> String, one of "active", "cancelling", "cancelled", "completed"
    #   resp.export_tasks[0].status_message #=> String
    #   resp.export_tasks[0].tags #=> Array
    #   resp.export_tasks[0].tags[0].key #=> String
    #   resp.export_tasks[0].tags[0].value #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * export_task_cancelled
    #   * export_task_completed
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeExportTasks AWS API Documentation
    #
    # @overload describe_export_tasks(params = {})
    # @param [Hash] params ({})
    def describe_export_tasks(params = {}, options = {})
      req = build_request(:describe_export_tasks, params)
      req.send_request(options)
    end

    # Describe details for Windows AMIs that are configured for Windows fast
    # launch.
    #
    # @option params [Array<String>] :image_ids
    #   Specify one or more Windows AMI image IDs for the request.
    #
    # @option params [Array<Types::Filter>] :filters
    #   Use the following filters to streamline results.
    #
    #   * `resource-type` - The resource type for pre-provisioning.
    #
    #   * `owner-id` - The owner ID for the pre-provisioning resource.
    #
    #   * `state` - The current state of fast launching for the Windows AMI.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeFastLaunchImagesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeFastLaunchImagesResult#fast_launch_images #fast_launch_images} => Array&lt;Types::DescribeFastLaunchImagesSuccessItem&gt;
    #   * {Types::DescribeFastLaunchImagesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_fast_launch_images({
    #     image_ids: ["ImageId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.fast_launch_images #=> Array
    #   resp.fast_launch_images[0].image_id #=> String
    #   resp.fast_launch_images[0].resource_type #=> String, one of "snapshot"
    #   resp.fast_launch_images[0].snapshot_configuration.target_resource_count #=> Integer
    #   resp.fast_launch_images[0].launch_template.launch_template_id #=> String
    #   resp.fast_launch_images[0].launch_template.launch_template_name #=> String
    #   resp.fast_launch_images[0].launch_template.version #=> String
    #   resp.fast_launch_images[0].max_parallel_launches #=> Integer
    #   resp.fast_launch_images[0].owner_id #=> String
    #   resp.fast_launch_images[0].state #=> String, one of "enabling", "enabling-failed", "enabled", "enabled-failed", "disabling", "disabling-failed"
    #   resp.fast_launch_images[0].state_transition_reason #=> String
    #   resp.fast_launch_images[0].state_transition_time #=> Time
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeFastLaunchImages AWS API Documentation
    #
    # @overload describe_fast_launch_images(params = {})
    # @param [Hash] params ({})
    def describe_fast_launch_images(params = {}, options = {})
      req = build_request(:describe_fast_launch_images, params)
      req.send_request(options)
    end

    # Describes the state of fast snapshot restores for your snapshots.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters. The possible values are:
    #
    #   * `availability-zone`: The Availability Zone of the snapshot.
    #
    #   * `owner-id`: The ID of the Amazon Web Services account that enabled
    #     fast snapshot restore on the snapshot.
    #
    #   * `snapshot-id`: The ID of the snapshot.
    #
    #   * `state`: The state of fast snapshot restores for the snapshot
    #     (`enabling` \| `optimizing` \| `enabled` \| `disabling` \|
    #     `disabled`).
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeFastSnapshotRestoresResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeFastSnapshotRestoresResult#fast_snapshot_restores #fast_snapshot_restores} => Array&lt;Types::DescribeFastSnapshotRestoreSuccessItem&gt;
    #   * {Types::DescribeFastSnapshotRestoresResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_fast_snapshot_restores({
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.fast_snapshot_restores #=> Array
    #   resp.fast_snapshot_restores[0].snapshot_id #=> String
    #   resp.fast_snapshot_restores[0].availability_zone #=> String
    #   resp.fast_snapshot_restores[0].state #=> String, one of "enabling", "optimizing", "enabled", "disabling", "disabled"
    #   resp.fast_snapshot_restores[0].state_transition_reason #=> String
    #   resp.fast_snapshot_restores[0].owner_id #=> String
    #   resp.fast_snapshot_restores[0].owner_alias #=> String
    #   resp.fast_snapshot_restores[0].enabling_time #=> Time
    #   resp.fast_snapshot_restores[0].optimizing_time #=> Time
    #   resp.fast_snapshot_restores[0].enabled_time #=> Time
    #   resp.fast_snapshot_restores[0].disabling_time #=> Time
    #   resp.fast_snapshot_restores[0].disabled_time #=> Time
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeFastSnapshotRestores AWS API Documentation
    #
    # @overload describe_fast_snapshot_restores(params = {})
    # @param [Hash] params ({})
    def describe_fast_snapshot_restores(params = {}, options = {})
      req = build_request(:describe_fast_snapshot_restores, params)
      req.send_request(options)
    end

    # Describes the events for the specified EC2 Fleet during the specified
    # time.
    #
    # EC2 Fleet events are delayed by up to 30 seconds before they can be
    # described. This ensures that you can query by the last evaluated time
    # and not miss a recorded event. EC2 Fleet events are available for 48
    # hours.
    #
    # For more information, see [Monitor fleet events using Amazon
    # EventBridge][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/fleet-monitor.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :event_type
    #   The type of events to describe. By default, all events are described.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [required, String] :fleet_id
    #   The ID of the EC2 Fleet.
    #
    # @option params [required, Time,DateTime,Date,Integer,String] :start_time
    #   The start date and time for the events, in UTC format (for example,
    #   *YYYY*-*MM*-*DD*T*HH*:*MM*:*SS*Z).
    #
    # @return [Types::DescribeFleetHistoryResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeFleetHistoryResult#history_records #history_records} => Array&lt;Types::HistoryRecordEntry&gt;
    #   * {Types::DescribeFleetHistoryResult#last_evaluated_time #last_evaluated_time} => Time
    #   * {Types::DescribeFleetHistoryResult#next_token #next_token} => String
    #   * {Types::DescribeFleetHistoryResult#fleet_id #fleet_id} => String
    #   * {Types::DescribeFleetHistoryResult#start_time #start_time} => Time
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_fleet_history({
    #     dry_run: false,
    #     event_type: "instance-change", # accepts instance-change, fleet-change, service-error
    #     max_results: 1,
    #     next_token: "String",
    #     fleet_id: "FleetId", # required
    #     start_time: Time.now, # required
    #   })
    #
    # @example Response structure
    #
    #   resp.history_records #=> Array
    #   resp.history_records[0].event_information.event_description #=> String
    #   resp.history_records[0].event_information.event_sub_type #=> String
    #   resp.history_records[0].event_information.instance_id #=> String
    #   resp.history_records[0].event_type #=> String, one of "instance-change", "fleet-change", "service-error"
    #   resp.history_records[0].timestamp #=> Time
    #   resp.last_evaluated_time #=> Time
    #   resp.next_token #=> String
    #   resp.fleet_id #=> String
    #   resp.start_time #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeFleetHistory AWS API Documentation
    #
    # @overload describe_fleet_history(params = {})
    # @param [Hash] params ({})
    def describe_fleet_history(params = {}, options = {})
      req = build_request(:describe_fleet_history, params)
      req.send_request(options)
    end

    # Describes the running instances for the specified EC2 Fleet.
    #
    # <note markdown="1"> Currently, `DescribeFleetInstances` does not support fleets of type
    # `instant`. Instead, use `DescribeFleets`, specifying the `instant`
    # fleet ID in the request.
    #
    #  </note>
    #
    # For more information, see [Describe your EC2 Fleet][1] in the *Amazon
    # EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/manage-ec2-fleet.html#monitor-ec2-fleet
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [required, String] :fleet_id
    #   The ID of the EC2 Fleet.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `instance-type` - The instance type.
    #
    #   ^
    #
    # @return [Types::DescribeFleetInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeFleetInstancesResult#active_instances #active_instances} => Array&lt;Types::ActiveInstance&gt;
    #   * {Types::DescribeFleetInstancesResult#next_token #next_token} => String
    #   * {Types::DescribeFleetInstancesResult#fleet_id #fleet_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_fleet_instances({
    #     dry_run: false,
    #     max_results: 1,
    #     next_token: "String",
    #     fleet_id: "FleetId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.active_instances #=> Array
    #   resp.active_instances[0].instance_id #=> String
    #   resp.active_instances[0].instance_type #=> String
    #   resp.active_instances[0].spot_instance_request_id #=> String
    #   resp.active_instances[0].instance_health #=> String, one of "healthy", "unhealthy"
    #   resp.next_token #=> String
    #   resp.fleet_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeFleetInstances AWS API Documentation
    #
    # @overload describe_fleet_instances(params = {})
    # @param [Hash] params ({})
    def describe_fleet_instances(params = {}, options = {})
      req = build_request(:describe_fleet_instances, params)
      req.send_request(options)
    end

    # Describes the specified EC2 Fleet or all of your EC2 Fleets.
    #
    # If a fleet is of type `instant`, you must specify the fleet ID in the
    # request, otherwise the fleet does not appear in the response.
    #
    # For more information, see [Describe your EC2 Fleet][1] in the *Amazon
    # EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/manage-ec2-fleet.html#monitor-ec2-fleet
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Array<String>] :fleet_ids
    #   The IDs of the EC2 Fleets.
    #
    #   <note markdown="1"> If a fleet is of type `instant`, you must specify the fleet ID,
    #   otherwise it does not appear in the response.
    #
    #    </note>
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `activity-status` - The progress of the EC2 Fleet ( `error` \|
    #     `pending-fulfillment` \| `pending-termination` \| `fulfilled`).
    #
    #   * `excess-capacity-termination-policy` - Indicates whether to
    #     terminate running instances if the target capacity is decreased
    #     below the current EC2 Fleet size (`true` \| `false`).
    #
    #   * `fleet-state` - The state of the EC2 Fleet (`submitted` \| `active`
    #     \| `deleted` \| `failed` \| `deleted-running` \|
    #     `deleted-terminating` \| `modifying`).
    #
    #   * `replace-unhealthy-instances` - Indicates whether EC2 Fleet should
    #     replace unhealthy instances (`true` \| `false`).
    #
    #   * `type` - The type of request (`instant` \| `request` \| `maintain`).
    #
    # @return [Types::DescribeFleetsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeFleetsResult#next_token #next_token} => String
    #   * {Types::DescribeFleetsResult#fleets #fleets} => Array&lt;Types::FleetData&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_fleets({
    #     dry_run: false,
    #     max_results: 1,
    #     next_token: "String",
    #     fleet_ids: ["FleetId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.fleets #=> Array
    #   resp.fleets[0].activity_status #=> String, one of "error", "pending_fulfillment", "pending_termination", "fulfilled"
    #   resp.fleets[0].create_time #=> Time
    #   resp.fleets[0].fleet_id #=> String
    #   resp.fleets[0].fleet_state #=> String, one of "submitted", "active", "deleted", "failed", "deleted_running", "deleted_terminating", "modifying"
    #   resp.fleets[0].client_token #=> String
    #   resp.fleets[0].excess_capacity_termination_policy #=> String, one of "no-termination", "termination"
    #   resp.fleets[0].fulfilled_capacity #=> Float
    #   resp.fleets[0].fulfilled_on_demand_capacity #=> Float
    #   resp.fleets[0].launch_template_configs #=> Array
    #   resp.fleets[0].launch_template_configs[0].launch_template_specification.launch_template_id #=> String
    #   resp.fleets[0].launch_template_configs[0].launch_template_specification.launch_template_name #=> String
    #   resp.fleets[0].launch_template_configs[0].launch_template_specification.version #=> String
    #   resp.fleets[0].launch_template_configs[0].overrides #=> Array
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.fleets[0].launch_template_configs[0].overrides[0].max_price #=> String
    #   resp.fleets[0].launch_template_configs[0].overrides[0].subnet_id #=> String
    #   resp.fleets[0].launch_template_configs[0].overrides[0].availability_zone #=> String
    #   resp.fleets[0].launch_template_configs[0].overrides[0].weighted_capacity #=> Float
    #   resp.fleets[0].launch_template_configs[0].overrides[0].priority #=> Float
    #   resp.fleets[0].launch_template_configs[0].overrides[0].placement.group_name #=> String
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.v_cpu_count.min #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.v_cpu_count.max #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.memory_mi_b.min #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.memory_mi_b.max #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.cpu_manufacturers #=> Array
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.cpu_manufacturers[0] #=> String, one of "intel", "amd", "amazon-web-services", "apple"
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.memory_gi_b_per_v_cpu.min #=> Float
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.memory_gi_b_per_v_cpu.max #=> Float
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.excluded_instance_types #=> Array
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.excluded_instance_types[0] #=> String
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.instance_generations #=> Array
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.instance_generations[0] #=> String, one of "current", "previous"
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.spot_max_price_percentage_over_lowest_price #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.on_demand_max_price_percentage_over_lowest_price #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.bare_metal #=> String, one of "included", "required", "excluded"
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.burstable_performance #=> String, one of "included", "required", "excluded"
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.require_hibernate_support #=> Boolean
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.network_interface_count.min #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.network_interface_count.max #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.local_storage #=> String, one of "included", "required", "excluded"
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.local_storage_types #=> Array
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.local_storage_types[0] #=> String, one of "hdd", "ssd"
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.total_local_storage_gb.min #=> Float
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.total_local_storage_gb.max #=> Float
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.baseline_ebs_bandwidth_mbps.min #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.baseline_ebs_bandwidth_mbps.max #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.accelerator_types #=> Array
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.accelerator_types[0] #=> String, one of "gpu", "fpga", "inference"
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.accelerator_count.min #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.accelerator_count.max #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.accelerator_manufacturers #=> Array
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.accelerator_manufacturers[0] #=> String, one of "amazon-web-services", "amd", "nvidia", "xilinx", "habana"
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.accelerator_names #=> Array
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.accelerator_names[0] #=> String, one of "a100", "inferentia", "k520", "k80", "m60", "radeon-pro-v520", "t4", "vu9p", "v100", "a10g", "h100", "t4g"
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.accelerator_total_memory_mi_b.min #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.accelerator_total_memory_mi_b.max #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.network_bandwidth_gbps.min #=> Float
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.network_bandwidth_gbps.max #=> Float
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.allowed_instance_types #=> Array
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.allowed_instance_types[0] #=> String
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.max_spot_price_as_percentage_of_optimal_on_demand_price #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.baseline_performance_factors.cpu.references #=> Array
    #   resp.fleets[0].launch_template_configs[0].overrides[0].instance_requirements.baseline_performance_factors.cpu.references[0].instance_family #=> String
    #   resp.fleets[0].launch_template_configs[0].overrides[0].image_id #=> String
    #   resp.fleets[0].launch_template_configs[0].overrides[0].block_device_mappings #=> Array
    #   resp.fleets[0].launch_template_configs[0].overrides[0].block_device_mappings[0].device_name #=> String
    #   resp.fleets[0].launch_template_configs[0].overrides[0].block_device_mappings[0].virtual_name #=> String
    #   resp.fleets[0].launch_template_configs[0].overrides[0].block_device_mappings[0].ebs.encrypted #=> Boolean
    #   resp.fleets[0].launch_template_configs[0].overrides[0].block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.fleets[0].launch_template_configs[0].overrides[0].block_device_mappings[0].ebs.iops #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].block_device_mappings[0].ebs.throughput #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].block_device_mappings[0].ebs.kms_key_id #=> String
    #   resp.fleets[0].launch_template_configs[0].overrides[0].block_device_mappings[0].ebs.snapshot_id #=> String
    #   resp.fleets[0].launch_template_configs[0].overrides[0].block_device_mappings[0].ebs.volume_size #=> Integer
    #   resp.fleets[0].launch_template_configs[0].overrides[0].block_device_mappings[0].ebs.volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.fleets[0].launch_template_configs[0].overrides[0].block_device_mappings[0].no_device #=> String
    #   resp.fleets[0].target_capacity_specification.total_target_capacity #=> Integer
    #   resp.fleets[0].target_capacity_specification.on_demand_target_capacity #=> Integer
    #   resp.fleets[0].target_capacity_specification.spot_target_capacity #=> Integer
    #   resp.fleets[0].target_capacity_specification.default_target_capacity_type #=> String, one of "spot", "on-demand", "capacity-block"
    #   resp.fleets[0].target_capacity_specification.target_capacity_unit_type #=> String, one of "vcpu", "memory-mib", "units"
    #   resp.fleets[0].terminate_instances_with_expiration #=> Boolean
    #   resp.fleets[0].type #=> String, one of "request", "maintain", "instant"
    #   resp.fleets[0].valid_from #=> Time
    #   resp.fleets[0].valid_until #=> Time
    #   resp.fleets[0].replace_unhealthy_instances #=> Boolean
    #   resp.fleets[0].spot_options.allocation_strategy #=> String, one of "lowest-price", "diversified", "capacity-optimized", "capacity-optimized-prioritized", "price-capacity-optimized"
    #   resp.fleets[0].spot_options.maintenance_strategies.capacity_rebalance.replacement_strategy #=> String, one of "launch", "launch-before-terminate"
    #   resp.fleets[0].spot_options.maintenance_strategies.capacity_rebalance.termination_delay #=> Integer
    #   resp.fleets[0].spot_options.instance_interruption_behavior #=> String, one of "hibernate", "stop", "terminate"
    #   resp.fleets[0].spot_options.instance_pools_to_use_count #=> Integer
    #   resp.fleets[0].spot_options.single_instance_type #=> Boolean
    #   resp.fleets[0].spot_options.single_availability_zone #=> Boolean
    #   resp.fleets[0].spot_options.min_target_capacity #=> Integer
    #   resp.fleets[0].spot_options.max_total_price #=> String
    #   resp.fleets[0].on_demand_options.allocation_strategy #=> String, one of "lowest-price", "prioritized"
    #   resp.fleets[0].on_demand_options.capacity_reservation_options.usage_strategy #=> String, one of "use-capacity-reservations-first"
    #   resp.fleets[0].on_demand_options.single_instance_type #=> Boolean
    #   resp.fleets[0].on_demand_options.single_availability_zone #=> Boolean
    #   resp.fleets[0].on_demand_options.min_target_capacity #=> Integer
    #   resp.fleets[0].on_demand_options.max_total_price #=> String
    #   resp.fleets[0].tags #=> Array
    #   resp.fleets[0].tags[0].key #=> String
    #   resp.fleets[0].tags[0].value #=> String
    #   resp.fleets[0].errors #=> Array
    #   resp.fleets[0].errors[0].launch_template_and_overrides.launch_template_specification.launch_template_id #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.launch_template_specification.launch_template_name #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.launch_template_specification.version #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.max_price #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.subnet_id #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.availability_zone #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.weighted_capacity #=> Float
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.priority #=> Float
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.placement.group_name #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.v_cpu_count.min #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.v_cpu_count.max #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.memory_mi_b.min #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.memory_mi_b.max #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.cpu_manufacturers #=> Array
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.cpu_manufacturers[0] #=> String, one of "intel", "amd", "amazon-web-services", "apple"
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.memory_gi_b_per_v_cpu.min #=> Float
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.memory_gi_b_per_v_cpu.max #=> Float
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.excluded_instance_types #=> Array
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.excluded_instance_types[0] #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.instance_generations #=> Array
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.instance_generations[0] #=> String, one of "current", "previous"
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.spot_max_price_percentage_over_lowest_price #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.on_demand_max_price_percentage_over_lowest_price #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.bare_metal #=> String, one of "included", "required", "excluded"
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.burstable_performance #=> String, one of "included", "required", "excluded"
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.require_hibernate_support #=> Boolean
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.network_interface_count.min #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.network_interface_count.max #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.local_storage #=> String, one of "included", "required", "excluded"
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.local_storage_types #=> Array
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.local_storage_types[0] #=> String, one of "hdd", "ssd"
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.total_local_storage_gb.min #=> Float
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.total_local_storage_gb.max #=> Float
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.baseline_ebs_bandwidth_mbps.min #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.baseline_ebs_bandwidth_mbps.max #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_types #=> Array
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_types[0] #=> String, one of "gpu", "fpga", "inference"
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_count.min #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_count.max #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_manufacturers #=> Array
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_manufacturers[0] #=> String, one of "amazon-web-services", "amd", "nvidia", "xilinx", "habana"
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_names #=> Array
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_names[0] #=> String, one of "a100", "inferentia", "k520", "k80", "m60", "radeon-pro-v520", "t4", "vu9p", "v100", "a10g", "h100", "t4g"
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_total_memory_mi_b.min #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_total_memory_mi_b.max #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.network_bandwidth_gbps.min #=> Float
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.network_bandwidth_gbps.max #=> Float
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.allowed_instance_types #=> Array
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.allowed_instance_types[0] #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.max_spot_price_as_percentage_of_optimal_on_demand_price #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.baseline_performance_factors.cpu.references #=> Array
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.instance_requirements.baseline_performance_factors.cpu.references[0].instance_family #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.image_id #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.block_device_mappings #=> Array
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].device_name #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].virtual_name #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.encrypted #=> Boolean
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.iops #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.throughput #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.kms_key_id #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.snapshot_id #=> String
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.volume_size #=> Integer
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.fleets[0].errors[0].launch_template_and_overrides.overrides.block_device_mappings[0].no_device #=> String
    #   resp.fleets[0].errors[0].lifecycle #=> String, one of "spot", "on-demand"
    #   resp.fleets[0].errors[0].error_code #=> String
    #   resp.fleets[0].errors[0].error_message #=> String
    #   resp.fleets[0].instances #=> Array
    #   resp.fleets[0].instances[0].launch_template_and_overrides.launch_template_specification.launch_template_id #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.launch_template_specification.launch_template_name #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.launch_template_specification.version #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.max_price #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.subnet_id #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.availability_zone #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.weighted_capacity #=> Float
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.priority #=> Float
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.placement.group_name #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.v_cpu_count.min #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.v_cpu_count.max #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.memory_mi_b.min #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.memory_mi_b.max #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.cpu_manufacturers #=> Array
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.cpu_manufacturers[0] #=> String, one of "intel", "amd", "amazon-web-services", "apple"
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.memory_gi_b_per_v_cpu.min #=> Float
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.memory_gi_b_per_v_cpu.max #=> Float
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.excluded_instance_types #=> Array
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.excluded_instance_types[0] #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.instance_generations #=> Array
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.instance_generations[0] #=> String, one of "current", "previous"
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.spot_max_price_percentage_over_lowest_price #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.on_demand_max_price_percentage_over_lowest_price #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.bare_metal #=> String, one of "included", "required", "excluded"
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.burstable_performance #=> String, one of "included", "required", "excluded"
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.require_hibernate_support #=> Boolean
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.network_interface_count.min #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.network_interface_count.max #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.local_storage #=> String, one of "included", "required", "excluded"
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.local_storage_types #=> Array
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.local_storage_types[0] #=> String, one of "hdd", "ssd"
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.total_local_storage_gb.min #=> Float
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.total_local_storage_gb.max #=> Float
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.baseline_ebs_bandwidth_mbps.min #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.baseline_ebs_bandwidth_mbps.max #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_types #=> Array
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_types[0] #=> String, one of "gpu", "fpga", "inference"
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_count.min #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_count.max #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_manufacturers #=> Array
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_manufacturers[0] #=> String, one of "amazon-web-services", "amd", "nvidia", "xilinx", "habana"
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_names #=> Array
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_names[0] #=> String, one of "a100", "inferentia", "k520", "k80", "m60", "radeon-pro-v520", "t4", "vu9p", "v100", "a10g", "h100", "t4g"
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_total_memory_mi_b.min #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.accelerator_total_memory_mi_b.max #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.network_bandwidth_gbps.min #=> Float
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.network_bandwidth_gbps.max #=> Float
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.allowed_instance_types #=> Array
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.allowed_instance_types[0] #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.max_spot_price_as_percentage_of_optimal_on_demand_price #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.baseline_performance_factors.cpu.references #=> Array
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.instance_requirements.baseline_performance_factors.cpu.references[0].instance_family #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.image_id #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.block_device_mappings #=> Array
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].device_name #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].virtual_name #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.encrypted #=> Boolean
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.iops #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.throughput #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.kms_key_id #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.snapshot_id #=> String
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.volume_size #=> Integer
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].ebs.volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.fleets[0].instances[0].launch_template_and_overrides.overrides.block_device_mappings[0].no_device #=> String
    #   resp.fleets[0].instances[0].lifecycle #=> String, one of "spot", "on-demand"
    #   resp.fleets[0].instances[0].instance_ids #=> Array
    #   resp.fleets[0].instances[0].instance_ids[0] #=> String
    #   resp.fleets[0].instances[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.fleets[0].instances[0].platform #=> String, one of "Windows"
    #   resp.fleets[0].context #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeFleets AWS API Documentation
    #
    # @overload describe_fleets(params = {})
    # @param [Hash] params ({})
    def describe_fleets(params = {}, options = {})
      req = build_request(:describe_fleets, params)
      req.send_request(options)
    end

    # Describes one or more flow logs.
    #
    # To view the published flow log records, you must view the log
    # destination. For example, the CloudWatch Logs log group, the Amazon S3
    # bucket, or the Kinesis Data Firehose delivery stream.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filter
    #   One or more filters.
    #
    #   * `deliver-log-status` - The status of the logs delivery (`SUCCESS` \|
    #     `FAILED`).
    #
    #   * `log-destination-type` - The type of destination for the flow log
    #     data (`cloud-watch-logs` \| `s3` \| `kinesis-data-firehose`).
    #
    #   * `flow-log-id` - The ID of the flow log.
    #
    #   * `log-group-name` - The name of the log group.
    #
    #   * `resource-id` - The ID of the VPC, subnet, or network interface.
    #
    #   * `traffic-type` - The type of traffic (`ACCEPT` \| `REJECT` \|
    #     `ALL`).
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @option params [Array<String>] :flow_log_ids
    #   One or more flow log IDs.
    #
    #   Constraint: Maximum of 1000 flow log IDs.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token to request the next page of items. Pagination continues from
    #   the end of the items returned by the previous request.
    #
    # @return [Types::DescribeFlowLogsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeFlowLogsResult#flow_logs #flow_logs} => Array&lt;Types::FlowLog&gt;
    #   * {Types::DescribeFlowLogsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_flow_logs({
    #     dry_run: false,
    #     filter: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     flow_log_ids: ["VpcFlowLogId"],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.flow_logs #=> Array
    #   resp.flow_logs[0].creation_time #=> Time
    #   resp.flow_logs[0].deliver_logs_error_message #=> String
    #   resp.flow_logs[0].deliver_logs_permission_arn #=> String
    #   resp.flow_logs[0].deliver_cross_account_role #=> String
    #   resp.flow_logs[0].deliver_logs_status #=> String
    #   resp.flow_logs[0].flow_log_id #=> String
    #   resp.flow_logs[0].flow_log_status #=> String
    #   resp.flow_logs[0].log_group_name #=> String
    #   resp.flow_logs[0].resource_id #=> String
    #   resp.flow_logs[0].traffic_type #=> String, one of "ACCEPT", "REJECT", "ALL"
    #   resp.flow_logs[0].log_destination_type #=> String, one of "cloud-watch-logs", "s3", "kinesis-data-firehose"
    #   resp.flow_logs[0].log_destination #=> String
    #   resp.flow_logs[0].log_format #=> String
    #   resp.flow_logs[0].tags #=> Array
    #   resp.flow_logs[0].tags[0].key #=> String
    #   resp.flow_logs[0].tags[0].value #=> String
    #   resp.flow_logs[0].max_aggregation_interval #=> Integer
    #   resp.flow_logs[0].destination_options.file_format #=> String, one of "plain-text", "parquet"
    #   resp.flow_logs[0].destination_options.hive_compatible_partitions #=> Boolean
    #   resp.flow_logs[0].destination_options.per_hour_partition #=> Boolean
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeFlowLogs AWS API Documentation
    #
    # @overload describe_flow_logs(params = {})
    # @param [Hash] params ({})
    def describe_flow_logs(params = {}, options = {})
      req = build_request(:describe_flow_logs, params)
      req.send_request(options)
    end

    # Describes the specified attribute of the specified Amazon FPGA Image
    # (AFI).
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :fpga_image_id
    #   The ID of the AFI.
    #
    # @option params [required, String] :attribute
    #   The AFI attribute.
    #
    # @return [Types::DescribeFpgaImageAttributeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeFpgaImageAttributeResult#fpga_image_attribute #fpga_image_attribute} => Types::FpgaImageAttribute
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_fpga_image_attribute({
    #     dry_run: false,
    #     fpga_image_id: "FpgaImageId", # required
    #     attribute: "description", # required, accepts description, name, loadPermission, productCodes
    #   })
    #
    # @example Response structure
    #
    #   resp.fpga_image_attribute.fpga_image_id #=> String
    #   resp.fpga_image_attribute.name #=> String
    #   resp.fpga_image_attribute.description #=> String
    #   resp.fpga_image_attribute.load_permissions #=> Array
    #   resp.fpga_image_attribute.load_permissions[0].user_id #=> String
    #   resp.fpga_image_attribute.load_permissions[0].group #=> String, one of "all"
    #   resp.fpga_image_attribute.product_codes #=> Array
    #   resp.fpga_image_attribute.product_codes[0].product_code_id #=> String
    #   resp.fpga_image_attribute.product_codes[0].product_code_type #=> String, one of "devpay", "marketplace"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeFpgaImageAttribute AWS API Documentation
    #
    # @overload describe_fpga_image_attribute(params = {})
    # @param [Hash] params ({})
    def describe_fpga_image_attribute(params = {}, options = {})
      req = build_request(:describe_fpga_image_attribute, params)
      req.send_request(options)
    end

    # Describes the Amazon FPGA Images (AFIs) available to you. These
    # include public AFIs, private AFIs that you own, and AFIs owned by
    # other Amazon Web Services accounts for which you have load
    # permissions.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :fpga_image_ids
    #   The AFI IDs.
    #
    # @option params [Array<String>] :owners
    #   Filters the AFI by owner. Specify an Amazon Web Services account ID,
    #   `self` (owner is the sender of the request), or an Amazon Web Services
    #   owner alias (valid values are `amazon` \| `aws-marketplace`).
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `create-time` - The creation time of the AFI.
    #
    #   * `fpga-image-id` - The FPGA image identifier (AFI ID).
    #
    #   * `fpga-image-global-id` - The global FPGA image identifier (AGFI ID).
    #
    #   * `name` - The name of the AFI.
    #
    #   * `owner-id` - The Amazon Web Services account ID of the AFI owner.
    #
    #   * `product-code` - The product code.
    #
    #   * `shell-version` - The version of the Amazon Web Services Shell that
    #     was used to create the bitstream.
    #
    #   * `state` - The state of the AFI (`pending` \| `failed` \| `available`
    #     \| `unavailable`).
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `update-time` - The time of the most recent update.
    #
    # @option params [String] :next_token
    #   The token to retrieve the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in a single call.
    #
    # @return [Types::DescribeFpgaImagesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeFpgaImagesResult#fpga_images #fpga_images} => Array&lt;Types::FpgaImage&gt;
    #   * {Types::DescribeFpgaImagesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_fpga_images({
    #     dry_run: false,
    #     fpga_image_ids: ["FpgaImageId"],
    #     owners: ["String"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "NextToken",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.fpga_images #=> Array
    #   resp.fpga_images[0].fpga_image_id #=> String
    #   resp.fpga_images[0].fpga_image_global_id #=> String
    #   resp.fpga_images[0].name #=> String
    #   resp.fpga_images[0].description #=> String
    #   resp.fpga_images[0].shell_version #=> String
    #   resp.fpga_images[0].pci_id.device_id #=> String
    #   resp.fpga_images[0].pci_id.vendor_id #=> String
    #   resp.fpga_images[0].pci_id.subsystem_id #=> String
    #   resp.fpga_images[0].pci_id.subsystem_vendor_id #=> String
    #   resp.fpga_images[0].state.code #=> String, one of "pending", "failed", "available", "unavailable"
    #   resp.fpga_images[0].state.message #=> String
    #   resp.fpga_images[0].create_time #=> Time
    #   resp.fpga_images[0].update_time #=> Time
    #   resp.fpga_images[0].owner_id #=> String
    #   resp.fpga_images[0].owner_alias #=> String
    #   resp.fpga_images[0].product_codes #=> Array
    #   resp.fpga_images[0].product_codes[0].product_code_id #=> String
    #   resp.fpga_images[0].product_codes[0].product_code_type #=> String, one of "devpay", "marketplace"
    #   resp.fpga_images[0].tags #=> Array
    #   resp.fpga_images[0].tags[0].key #=> String
    #   resp.fpga_images[0].tags[0].value #=> String
    #   resp.fpga_images[0].public #=> Boolean
    #   resp.fpga_images[0].data_retention_support #=> Boolean
    #   resp.fpga_images[0].instance_types #=> Array
    #   resp.fpga_images[0].instance_types[0] #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeFpgaImages AWS API Documentation
    #
    # @overload describe_fpga_images(params = {})
    # @param [Hash] params ({})
    def describe_fpga_images(params = {}, options = {})
      req = build_request(:describe_fpga_images, params)
      req.send_request(options)
    end

    # Describes the Dedicated Host reservations that are available to
    # purchase.
    #
    # The results describe all of the Dedicated Host reservation offerings,
    # including offerings that might not match the instance family and
    # Region of your Dedicated Hosts. When purchasing an offering, ensure
    # that the instance family and Region of the offering matches that of
    # the Dedicated Hosts with which it is to be associated. For more
    # information about supported instance types, see [Dedicated Hosts][1]
    # in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-hosts-overview.html
    #
    # @option params [Array<Types::Filter>] :filter
    #   The filters.
    #
    #   * `instance-family` - The instance family of the offering (for
    #     example, `m4`).
    #
    #   * `payment-option` - The payment option (`NoUpfront` \|
    #     `PartialUpfront` \| `AllUpfront`).
    #
    # @option params [Integer] :max_duration
    #   This is the maximum duration of the reservation to purchase, specified
    #   in seconds. Reservations are available in one-year and three-year
    #   terms. The number of seconds specified must be the number of seconds
    #   in a year (365x24x60x60) times one of the supported durations (1 or
    #   3). For example, specify 94608000 for three years.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return for the request in a single
    #   page. The remaining results can be seen by sending another request
    #   with the returned `nextToken` value. This value can be between 5 and
    #   500. If `maxResults` is given a larger value than 500, you receive an
    #   error.
    #
    # @option params [Integer] :min_duration
    #   This is the minimum duration of the reservation you'd like to
    #   purchase, specified in seconds. Reservations are available in one-year
    #   and three-year terms. The number of seconds specified must be the
    #   number of seconds in a year (365x24x60x60) times one of the supported
    #   durations (1 or 3). For example, specify 31536000 for one year.
    #
    # @option params [String] :next_token
    #   The token to use to retrieve the next page of results.
    #
    # @option params [String] :offering_id
    #   The ID of the reservation offering.
    #
    # @return [Types::DescribeHostReservationOfferingsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeHostReservationOfferingsResult#next_token #next_token} => String
    #   * {Types::DescribeHostReservationOfferingsResult#offering_set #offering_set} => Array&lt;Types::HostOffering&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_host_reservation_offerings({
    #     filter: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_duration: 1,
    #     max_results: 1,
    #     min_duration: 1,
    #     next_token: "String",
    #     offering_id: "OfferingId",
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.offering_set #=> Array
    #   resp.offering_set[0].currency_code #=> String, one of "USD"
    #   resp.offering_set[0].duration #=> Integer
    #   resp.offering_set[0].hourly_price #=> String
    #   resp.offering_set[0].instance_family #=> String
    #   resp.offering_set[0].offering_id #=> String
    #   resp.offering_set[0].payment_option #=> String, one of "AllUpfront", "PartialUpfront", "NoUpfront"
    #   resp.offering_set[0].upfront_price #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeHostReservationOfferings AWS API Documentation
    #
    # @overload describe_host_reservation_offerings(params = {})
    # @param [Hash] params ({})
    def describe_host_reservation_offerings(params = {}, options = {})
      req = build_request(:describe_host_reservation_offerings, params)
      req.send_request(options)
    end

    # Describes reservations that are associated with Dedicated Hosts in
    # your account.
    #
    # @option params [Array<Types::Filter>] :filter
    #   The filters.
    #
    #   * `instance-family` - The instance family (for example, `m4`).
    #
    #   * `payment-option` - The payment option (`NoUpfront` \|
    #     `PartialUpfront` \| `AllUpfront`).
    #
    #   * `state` - The state of the reservation (`payment-pending` \|
    #     `payment-failed` \| `active` \| `retired`).
    #
    #   * `tag:<key>` - The key/value combination of a tag assigned to the
    #     resource. Use the tag key in the filter name and the tag value as
    #     the filter value. For example, to find all resources that have a tag
    #     with the key `Owner` and the value `TeamA`, specify `tag:Owner` for
    #     the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @option params [Array<String>] :host_reservation_id_set
    #   The host reservation IDs.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return for the request in a single
    #   page. The remaining results can be seen by sending another request
    #   with the returned `nextToken` value. This value can be between 5 and
    #   500. If `maxResults` is given a larger value than 500, you receive an
    #   error.
    #
    # @option params [String] :next_token
    #   The token to use to retrieve the next page of results.
    #
    # @return [Types::DescribeHostReservationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeHostReservationsResult#host_reservation_set #host_reservation_set} => Array&lt;Types::HostReservation&gt;
    #   * {Types::DescribeHostReservationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_host_reservations({
    #     filter: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     host_reservation_id_set: ["HostReservationId"],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.host_reservation_set #=> Array
    #   resp.host_reservation_set[0].count #=> Integer
    #   resp.host_reservation_set[0].currency_code #=> String, one of "USD"
    #   resp.host_reservation_set[0].duration #=> Integer
    #   resp.host_reservation_set[0].end #=> Time
    #   resp.host_reservation_set[0].host_id_set #=> Array
    #   resp.host_reservation_set[0].host_id_set[0] #=> String
    #   resp.host_reservation_set[0].host_reservation_id #=> String
    #   resp.host_reservation_set[0].hourly_price #=> String
    #   resp.host_reservation_set[0].instance_family #=> String
    #   resp.host_reservation_set[0].offering_id #=> String
    #   resp.host_reservation_set[0].payment_option #=> String, one of "AllUpfront", "PartialUpfront", "NoUpfront"
    #   resp.host_reservation_set[0].start #=> Time
    #   resp.host_reservation_set[0].state #=> String, one of "payment-pending", "payment-failed", "active", "retired"
    #   resp.host_reservation_set[0].upfront_price #=> String
    #   resp.host_reservation_set[0].tags #=> Array
    #   resp.host_reservation_set[0].tags[0].key #=> String
    #   resp.host_reservation_set[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeHostReservations AWS API Documentation
    #
    # @overload describe_host_reservations(params = {})
    # @param [Hash] params ({})
    def describe_host_reservations(params = {}, options = {})
      req = build_request(:describe_host_reservations, params)
      req.send_request(options)
    end

    # Describes the specified Dedicated Hosts or all your Dedicated Hosts.
    #
    # The results describe only the Dedicated Hosts in the Region you're
    # currently using. All listed instances consume capacity on your
    # Dedicated Host. Dedicated Hosts that have recently been released are
    # listed with the state `released`.
    #
    # @option params [Array<String>] :host_ids
    #   The IDs of the Dedicated Hosts. The IDs are used for targeted instance
    #   launches.
    #
    # @option params [String] :next_token
    #   The token to use to retrieve the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return for the request in a single
    #   page. The remaining results can be seen by sending another request
    #   with the returned `nextToken` value. This value can be between 5 and
    #   500. If `maxResults` is given a larger value than 500, you receive an
    #   error.
    #
    #   You cannot specify this parameter and the host IDs parameter in the
    #   same request.
    #
    # @option params [Array<Types::Filter>] :filter
    #   The filters.
    #
    #   * `auto-placement` - Whether auto-placement is enabled or disabled
    #     (`on` \| `off`).
    #
    #   * `availability-zone` - The Availability Zone of the host.
    #
    #   * `client-token` - The idempotency token that you provided when you
    #     allocated the host.
    #
    #   * `host-reservation-id` - The ID of the reservation assigned to this
    #     host.
    #
    #   * `instance-type` - The instance type size that the Dedicated Host is
    #     configured to support.
    #
    #   * `state` - The allocation state of the Dedicated Host (`available` \|
    #     `under-assessment` \| `permanent-failure` \| `released` \|
    #     `released-permanent-failure`).
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @return [Types::DescribeHostsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeHostsResult#hosts #hosts} => Array&lt;Types::Host&gt;
    #   * {Types::DescribeHostsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_hosts({
    #     host_ids: ["DedicatedHostId"],
    #     next_token: "String",
    #     max_results: 1,
    #     filter: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.hosts #=> Array
    #   resp.hosts[0].auto_placement #=> String, one of "on", "off"
    #   resp.hosts[0].availability_zone #=> String
    #   resp.hosts[0].available_capacity.available_instance_capacity #=> Array
    #   resp.hosts[0].available_capacity.available_instance_capacity[0].available_capacity #=> Integer
    #   resp.hosts[0].available_capacity.available_instance_capacity[0].instance_type #=> String
    #   resp.hosts[0].available_capacity.available_instance_capacity[0].total_capacity #=> Integer
    #   resp.hosts[0].available_capacity.available_v_cpus #=> Integer
    #   resp.hosts[0].client_token #=> String
    #   resp.hosts[0].host_id #=> String
    #   resp.hosts[0].host_properties.cores #=> Integer
    #   resp.hosts[0].host_properties.instance_type #=> String
    #   resp.hosts[0].host_properties.instance_family #=> String
    #   resp.hosts[0].host_properties.sockets #=> Integer
    #   resp.hosts[0].host_properties.total_v_cpus #=> Integer
    #   resp.hosts[0].host_reservation_id #=> String
    #   resp.hosts[0].instances #=> Array
    #   resp.hosts[0].instances[0].instance_id #=> String
    #   resp.hosts[0].instances[0].instance_type #=> String
    #   resp.hosts[0].instances[0].owner_id #=> String
    #   resp.hosts[0].state #=> String, one of "available", "under-assessment", "permanent-failure", "released", "released-permanent-failure", "pending"
    #   resp.hosts[0].allocation_time #=> Time
    #   resp.hosts[0].release_time #=> Time
    #   resp.hosts[0].tags #=> Array
    #   resp.hosts[0].tags[0].key #=> String
    #   resp.hosts[0].tags[0].value #=> String
    #   resp.hosts[0].host_recovery #=> String, one of "on", "off"
    #   resp.hosts[0].allows_multiple_instance_types #=> String, one of "on", "off"
    #   resp.hosts[0].owner_id #=> String
    #   resp.hosts[0].availability_zone_id #=> String
    #   resp.hosts[0].member_of_service_linked_resource_group #=> Boolean
    #   resp.hosts[0].outpost_arn #=> String
    #   resp.hosts[0].host_maintenance #=> String, one of "on", "off"
    #   resp.hosts[0].asset_id #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeHosts AWS API Documentation
    #
    # @overload describe_hosts(params = {})
    # @param [Hash] params ({})
    def describe_hosts(params = {}, options = {})
      req = build_request(:describe_hosts, params)
      req.send_request(options)
    end

    # Describes your IAM instance profile associations.
    #
    # @option params [Array<String>] :association_ids
    #   The IAM instance profile associations.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `instance-id` - The ID of the instance.
    #
    #   * `state` - The state of the association (`associating` \|
    #     `associated` \| `disassociating`).
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @return [Types::DescribeIamInstanceProfileAssociationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeIamInstanceProfileAssociationsResult#iam_instance_profile_associations #iam_instance_profile_associations} => Array&lt;Types::IamInstanceProfileAssociation&gt;
    #   * {Types::DescribeIamInstanceProfileAssociationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe an IAM instance profile association
    #
    #   # This example describes the specified IAM instance profile association.
    #
    #   resp = client.describe_iam_instance_profile_associations({
    #     association_ids: [
    #       "iip-assoc-0db249b1f25fa24b8", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     iam_instance_profile_associations: [
    #       {
    #         association_id: "iip-assoc-0db249b1f25fa24b8", 
    #         iam_instance_profile: {
    #           arn: "arn:aws:iam::123456789012:instance-profile/admin-role", 
    #           id: "AIPAJVQN4F5WVLGCJDRGM", 
    #         }, 
    #         instance_id: "i-09eb09efa73ec1dee", 
    #         state: "associated", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_iam_instance_profile_associations({
    #     association_ids: ["IamInstanceProfileAssociationId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.iam_instance_profile_associations #=> Array
    #   resp.iam_instance_profile_associations[0].association_id #=> String
    #   resp.iam_instance_profile_associations[0].instance_id #=> String
    #   resp.iam_instance_profile_associations[0].iam_instance_profile.arn #=> String
    #   resp.iam_instance_profile_associations[0].iam_instance_profile.id #=> String
    #   resp.iam_instance_profile_associations[0].state #=> String, one of "associating", "associated", "disassociating", "disassociated"
    #   resp.iam_instance_profile_associations[0].timestamp #=> Time
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeIamInstanceProfileAssociations AWS API Documentation
    #
    # @overload describe_iam_instance_profile_associations(params = {})
    # @param [Hash] params ({})
    def describe_iam_instance_profile_associations(params = {}, options = {})
      req = build_request(:describe_iam_instance_profile_associations, params)
      req.send_request(options)
    end

    # Describes the ID format settings for your resources on a per-Region
    # basis, for example, to view which resource types are enabled for
    # longer IDs. This request only returns information about resource types
    # whose ID formats can be modified; it does not return information about
    # other resource types.
    #
    # The following resource types support longer IDs: `bundle` \|
    # `conversion-task` \| `customer-gateway` \| `dhcp-options` \|
    # `elastic-ip-allocation` \| `elastic-ip-association` \| `export-task`
    # \| `flow-log` \| `image` \| `import-task` \| `instance` \|
    # `internet-gateway` \| `network-acl` \| `network-acl-association` \|
    # `network-interface` \| `network-interface-attachment` \| `prefix-list`
    # \| `reservation` \| `route-table` \| `route-table-association` \|
    # `security-group` \| `snapshot` \| `subnet` \|
    # `subnet-cidr-block-association` \| `volume` \| `vpc` \|
    # `vpc-cidr-block-association` \| `vpc-endpoint` \|
    # `vpc-peering-connection` \| `vpn-connection` \| `vpn-gateway`.
    #
    # These settings apply to the IAM user who makes the request; they do
    # not apply to the entire Amazon Web Services account. By default, an
    # IAM user defaults to the same settings as the root user, unless they
    # explicitly override the settings by running the ModifyIdFormat
    # command. Resources created with longer IDs are visible to all IAM
    # users, regardless of these settings and provided that they have
    # permission to use the relevant `Describe` command for the resource
    # type.
    #
    # @option params [String] :resource
    #   The type of resource: `bundle` \| `conversion-task` \|
    #   `customer-gateway` \| `dhcp-options` \| `elastic-ip-allocation` \|
    #   `elastic-ip-association` \| `export-task` \| `flow-log` \| `image` \|
    #   `import-task` \| `instance` \| `internet-gateway` \| `network-acl` \|
    #   `network-acl-association` \| `network-interface` \|
    #   `network-interface-attachment` \| `prefix-list` \| `reservation` \|
    #   `route-table` \| `route-table-association` \| `security-group` \|
    #   `snapshot` \| `subnet` \| `subnet-cidr-block-association` \| `volume`
    #   \| `vpc` \| `vpc-cidr-block-association` \| `vpc-endpoint` \|
    #   `vpc-peering-connection` \| `vpn-connection` \| `vpn-gateway`
    #
    # @return [Types::DescribeIdFormatResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeIdFormatResult#statuses #statuses} => Array&lt;Types::IdFormat&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_id_format({
    #     resource: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.statuses #=> Array
    #   resp.statuses[0].deadline #=> Time
    #   resp.statuses[0].resource #=> String
    #   resp.statuses[0].use_long_ids #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeIdFormat AWS API Documentation
    #
    # @overload describe_id_format(params = {})
    # @param [Hash] params ({})
    def describe_id_format(params = {}, options = {})
      req = build_request(:describe_id_format, params)
      req.send_request(options)
    end

    # Describes the ID format settings for resources for the specified IAM
    # user, IAM role, or root user. For example, you can view the resource
    # types that are enabled for longer IDs. This request only returns
    # information about resource types whose ID formats can be modified; it
    # does not return information about other resource types. For more
    # information, see [Resource IDs][1] in the *Amazon Elastic Compute
    # Cloud User Guide*.
    #
    # The following resource types support longer IDs: `bundle` \|
    # `conversion-task` \| `customer-gateway` \| `dhcp-options` \|
    # `elastic-ip-allocation` \| `elastic-ip-association` \| `export-task`
    # \| `flow-log` \| `image` \| `import-task` \| `instance` \|
    # `internet-gateway` \| `network-acl` \| `network-acl-association` \|
    # `network-interface` \| `network-interface-attachment` \| `prefix-list`
    # \| `reservation` \| `route-table` \| `route-table-association` \|
    # `security-group` \| `snapshot` \| `subnet` \|
    # `subnet-cidr-block-association` \| `volume` \| `vpc` \|
    # `vpc-cidr-block-association` \| `vpc-endpoint` \|
    # `vpc-peering-connection` \| `vpn-connection` \| `vpn-gateway`.
    #
    # These settings apply to the principal specified in the request. They
    # do not apply to the principal that makes the request.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/resource-ids.html
    #
    # @option params [String] :resource
    #   The type of resource: `bundle` \| `conversion-task` \|
    #   `customer-gateway` \| `dhcp-options` \| `elastic-ip-allocation` \|
    #   `elastic-ip-association` \| `export-task` \| `flow-log` \| `image` \|
    #   `import-task` \| `instance` \| `internet-gateway` \| `network-acl` \|
    #   `network-acl-association` \| `network-interface` \|
    #   `network-interface-attachment` \| `prefix-list` \| `reservation` \|
    #   `route-table` \| `route-table-association` \| `security-group` \|
    #   `snapshot` \| `subnet` \| `subnet-cidr-block-association` \| `volume`
    #   \| `vpc` \| `vpc-cidr-block-association` \| `vpc-endpoint` \|
    #   `vpc-peering-connection` \| `vpn-connection` \| `vpn-gateway`
    #
    # @option params [required, String] :principal_arn
    #   The ARN of the principal, which can be an IAM role, IAM user, or the
    #   root user.
    #
    # @return [Types::DescribeIdentityIdFormatResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeIdentityIdFormatResult#statuses #statuses} => Array&lt;Types::IdFormat&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_identity_id_format({
    #     resource: "String",
    #     principal_arn: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.statuses #=> Array
    #   resp.statuses[0].deadline #=> Time
    #   resp.statuses[0].resource #=> String
    #   resp.statuses[0].use_long_ids #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeIdentityIdFormat AWS API Documentation
    #
    # @overload describe_identity_id_format(params = {})
    # @param [Hash] params ({})
    def describe_identity_id_format(params = {}, options = {})
      req = build_request(:describe_identity_id_format, params)
      req.send_request(options)
    end

    # Describes the specified attribute of the specified AMI. You can
    # specify only one attribute at a time.
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    # @option params [required, String] :attribute
    #   The AMI attribute.
    #
    #   **Note**: The `blockDeviceMapping` attribute is deprecated. Using this
    #   attribute returns the `Client.AuthFailure` error. To get information
    #   about the block device mappings for an AMI, use the DescribeImages
    #   action.
    #
    # @option params [required, String] :image_id
    #   The ID of the AMI.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ImageAttribute] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ImageAttribute#description #description} => Types::AttributeValue
    #   * {Types::ImageAttribute#kernel_id #kernel_id} => Types::AttributeValue
    #   * {Types::ImageAttribute#ramdisk_id #ramdisk_id} => Types::AttributeValue
    #   * {Types::ImageAttribute#sriov_net_support #sriov_net_support} => Types::AttributeValue
    #   * {Types::ImageAttribute#boot_mode #boot_mode} => Types::AttributeValue
    #   * {Types::ImageAttribute#tpm_support #tpm_support} => Types::AttributeValue
    #   * {Types::ImageAttribute#uefi_data #uefi_data} => Types::AttributeValue
    #   * {Types::ImageAttribute#last_launched_time #last_launched_time} => Types::AttributeValue
    #   * {Types::ImageAttribute#imds_support #imds_support} => Types::AttributeValue
    #   * {Types::ImageAttribute#deregistration_protection #deregistration_protection} => Types::AttributeValue
    #   * {Types::ImageAttribute#image_id #image_id} => String
    #   * {Types::ImageAttribute#launch_permissions #launch_permissions} => Array&lt;Types::LaunchPermission&gt;
    #   * {Types::ImageAttribute#product_codes #product_codes} => Array&lt;Types::ProductCode&gt;
    #   * {Types::ImageAttribute#block_device_mappings #block_device_mappings} => Array&lt;Types::BlockDeviceMapping&gt;
    #
    #
    # @example Example: To describe the launch permissions for an AMI
    #
    #   # This example describes the launch permissions for the specified AMI.
    #
    #   resp = client.describe_image_attribute({
    #     attribute: "launchPermission", 
    #     image_id: "ami-5731123e", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     image_id: "ami-5731123e", 
    #     launch_permissions: [
    #       {
    #         user_id: "123456789012", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_image_attribute({
    #     attribute: "description", # required, accepts description, kernel, ramdisk, launchPermission, productCodes, blockDeviceMapping, sriovNetSupport, bootMode, tpmSupport, uefiData, lastLaunchedTime, imdsSupport, deregistrationProtection
    #     image_id: "ImageId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.description #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.kernel_id #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.ramdisk_id #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.sriov_net_support #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.boot_mode #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.tpm_support #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.uefi_data #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.last_launched_time #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.imds_support #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.deregistration_protection #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.image_id #=> String
    #   resp.launch_permissions #=> Array
    #   resp.launch_permissions[0].organization_arn #=> String
    #   resp.launch_permissions[0].organizational_unit_arn #=> String
    #   resp.launch_permissions[0].user_id #=> String
    #   resp.launch_permissions[0].group #=> String, one of "all"
    #   resp.product_codes #=> Array
    #   resp.product_codes[0].product_code_id #=> String
    #   resp.product_codes[0].product_code_type #=> String, one of "devpay", "marketplace"
    #   resp.block_device_mappings #=> Array
    #   resp.block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.block_device_mappings[0].ebs.iops #=> Integer
    #   resp.block_device_mappings[0].ebs.snapshot_id #=> String
    #   resp.block_device_mappings[0].ebs.volume_size #=> Integer
    #   resp.block_device_mappings[0].ebs.volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.block_device_mappings[0].ebs.kms_key_id #=> String
    #   resp.block_device_mappings[0].ebs.throughput #=> Integer
    #   resp.block_device_mappings[0].ebs.outpost_arn #=> String
    #   resp.block_device_mappings[0].ebs.encrypted #=> Boolean
    #   resp.block_device_mappings[0].no_device #=> String
    #   resp.block_device_mappings[0].device_name #=> String
    #   resp.block_device_mappings[0].virtual_name #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeImageAttribute AWS API Documentation
    #
    # @overload describe_image_attribute(params = {})
    # @param [Hash] params ({})
    def describe_image_attribute(params = {}, options = {})
      req = build_request(:describe_image_attribute, params)
      req.send_request(options)
    end

    # Describes the specified images (AMIs, AKIs, and ARIs) available to you
    # or all of the images available to you.
    #
    # The images available to you include public images, private images that
    # you own, and private images owned by other Amazon Web Services
    # accounts for which you have explicit launch permissions.
    #
    # Recently deregistered images appear in the returned results for a
    # short interval and then return empty results. After all instances that
    # reference a deregistered AMI are terminated, specifying the ID of the
    # image will eventually return an error indicating that the AMI ID
    # cannot be found.
    #
    # When Allowed AMIs is set to `enabled`, only allowed images are
    # returned in the results, with the `imageAllowed` field set to `true`
    # for each image. In `audit-mode`, the `imageAllowed` field is set to
    # `true` for images that meet the account's Allowed AMIs criteria, and
    # `false` for images that don't meet the criteria. For more
    # information, see EnableAllowedImagesSettings.
    #
    # We strongly recommend using only paginated requests. Unpaginated
    # requests are susceptible to throttling and timeouts.
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    # @option params [Array<String>] :executable_users
    #   Scopes the images by users with explicit launch permissions. Specify
    #   an Amazon Web Services account ID, `self` (the sender of the request),
    #   or `all` (public AMIs).
    #
    #   * If you specify an Amazon Web Services account ID that is not your
    #     own, only AMIs shared with that specific Amazon Web Services account
    #     ID are returned. However, AMIs that are shared with the account’s
    #     organization or organizational unit (OU) are not returned.
    #
    #   * If you specify `self` or your own Amazon Web Services account ID,
    #     AMIs shared with your account are returned. In addition, AMIs that
    #     are shared with the organization or OU of which you are member are
    #     also returned.
    #
    #   * If you specify `all`, all public AMIs are returned.
    #
    # @option params [Array<String>] :image_ids
    #   The image IDs.
    #
    #   Default: Describes all images available to you.
    #
    # @option params [Array<String>] :owners
    #   Scopes the results to images with the specified owners. You can
    #   specify a combination of Amazon Web Services account IDs, `self`,
    #   `amazon`, `aws-backup-vault`, and `aws-marketplace`. If you omit this
    #   parameter, the results include all images for which you have launch
    #   permissions, regardless of ownership.
    #
    # @option params [Boolean] :include_deprecated
    #   Specifies whether to include deprecated AMIs.
    #
    #   Default: No deprecated AMIs are included in the response.
    #
    #   <note markdown="1"> If you are the AMI owner, all deprecated AMIs appear in the response
    #   regardless of what you specify for this parameter.
    #
    #    </note>
    #
    # @option params [Boolean] :include_disabled
    #   Specifies whether to include disabled AMIs.
    #
    #   Default: No disabled AMIs are included in the response.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `architecture` - The image architecture (`i386` \| `x86_64` \|
    #     `arm64` \| `x86_64_mac` \| `arm64_mac`).
    #
    #   * `block-device-mapping.delete-on-termination` - A Boolean value that
    #     indicates whether the Amazon EBS volume is deleted on instance
    #     termination.
    #
    #   * `block-device-mapping.device-name` - The device name specified in
    #     the block device mapping (for example, `/dev/sdh` or `xvdh`).
    #
    #   * `block-device-mapping.snapshot-id` - The ID of the snapshot used for
    #     the Amazon EBS volume.
    #
    #   * `block-device-mapping.volume-size` - The volume size of the Amazon
    #     EBS volume, in GiB.
    #
    #   * `block-device-mapping.volume-type` - The volume type of the Amazon
    #     EBS volume (`io1` \| `io2` \| `gp2` \| `gp3` \| `sc1 `\| `st1` \|
    #     `standard`).
    #
    #   * `block-device-mapping.encrypted` - A Boolean that indicates whether
    #     the Amazon EBS volume is encrypted.
    #
    #   * `creation-date` - The time when the image was created, in the ISO
    #     8601 format in the UTC time zone (YYYY-MM-DDThh:mm:ss.sssZ), for
    #     example, `2021-09-29T11:04:43.305Z`. You can use a wildcard (`*`),
    #     for example, `2021-09-29T*`, which matches an entire day.
    #
    #   * `description` - The description of the image (provided during image
    #     creation).
    #
    #   * `ena-support` - A Boolean that indicates whether enhanced networking
    #     with ENA is enabled.
    #
    #   * `hypervisor` - The hypervisor type (`ovm` \| `xen`).
    #
    #   * `image-allowed` - A Boolean that indicates whether the image meets
    #     the criteria specified for Allowed AMIs.
    #
    #   * `image-id` - The ID of the image.
    #
    #   * `image-type` - The image type (`machine` \| `kernel` \| `ramdisk`).
    #
    #   * `is-public` - A Boolean that indicates whether the image is public.
    #
    #   * `kernel-id` - The kernel ID.
    #
    #   * `manifest-location` - The location of the image manifest.
    #
    #   * `name` - The name of the AMI (provided during image creation).
    #
    #   * `owner-alias` - The owner alias (`amazon` \| `aws-backup-vault` \|
    #     `aws-marketplace`). The valid aliases are defined in an
    #     Amazon-maintained list. This is not the Amazon Web Services account
    #     alias that can be set using the IAM console. We recommend that you
    #     use the **Owner** request parameter instead of this filter.
    #
    #   * `owner-id` - The Amazon Web Services account ID of the owner. We
    #     recommend that you use the **Owner** request parameter instead of
    #     this filter.
    #
    #   * `platform` - The platform. The only supported value is `windows`.
    #
    #   * `product-code` - The product code.
    #
    #   * `product-code.type` - The type of the product code (`marketplace`).
    #
    #   * `ramdisk-id` - The RAM disk ID.
    #
    #   * `root-device-name` - The device name of the root device volume (for
    #     example, `/dev/sda1`).
    #
    #   * `root-device-type` - The type of the root device volume (`ebs` \|
    #     `instance-store`).
    #
    #   * `source-image-id` - The ID of the source AMI from which the AMI was
    #     created.
    #
    #   * `source-image-region` - The Region of the source AMI.
    #
    #   * `source-instance-id` - The ID of the instance that the AMI was
    #     created from if the AMI was created using CreateImage. This filter
    #     is applicable only if the AMI was created using [CreateImage][1].
    #
    #   * `state` - The state of the image (`available` \| `pending` \|
    #     `failed`).
    #
    #   * `state-reason-code` - The reason code for the state change.
    #
    #   * `state-reason-message` - The message for the state change.
    #
    #   * `sriov-net-support` - A value of `simple` indicates that enhanced
    #     networking with the Intel 82599 VF interface is enabled.
    #
    #   * `tag:<key>` - The key/value combination of a tag assigned to the
    #     resource. Use the tag key in the filter name and the tag value as
    #     the filter value. For example, to find all resources that have a tag
    #     with the key `Owner` and the value `TeamA`, specify `tag:Owner` for
    #     the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `virtualization-type` - The virtualization type (`paravirtual` \|
    #     `hvm`).
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateImage.html
    #
    # @return [Types::DescribeImagesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeImagesResult#next_token #next_token} => String
    #   * {Types::DescribeImagesResult#images #images} => Array&lt;Types::Image&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe an AMI
    #
    #   # This example describes the specified AMI.
    #
    #   resp = client.describe_images({
    #     image_ids: [
    #       "ami-5731123e", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     images: [
    #       {
    #         architecture: "x86_64", 
    #         block_device_mappings: [
    #           {
    #             device_name: "/dev/sda1", 
    #             ebs: {
    #               delete_on_termination: true, 
    #               snapshot_id: "snap-1234567890abcdef0", 
    #               volume_size: 8, 
    #               volume_type: "standard", 
    #             }, 
    #           }, 
    #         ], 
    #         description: "An AMI for my server", 
    #         hypervisor: "xen", 
    #         image_id: "ami-5731123e", 
    #         image_location: "123456789012/My server", 
    #         image_type: "machine", 
    #         kernel_id: "aki-88aa75e1", 
    #         name: "My server", 
    #         owner_id: "123456789012", 
    #         public: false, 
    #         root_device_name: "/dev/sda1", 
    #         root_device_type: "ebs", 
    #         state: "available", 
    #         virtualization_type: "paravirtual", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_images({
    #     executable_users: ["String"],
    #     image_ids: ["ImageId"],
    #     owners: ["String"],
    #     include_deprecated: false,
    #     include_disabled: false,
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.images #=> Array
    #   resp.images[0].platform_details #=> String
    #   resp.images[0].usage_operation #=> String
    #   resp.images[0].block_device_mappings #=> Array
    #   resp.images[0].block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.images[0].block_device_mappings[0].ebs.iops #=> Integer
    #   resp.images[0].block_device_mappings[0].ebs.snapshot_id #=> String
    #   resp.images[0].block_device_mappings[0].ebs.volume_size #=> Integer
    #   resp.images[0].block_device_mappings[0].ebs.volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.images[0].block_device_mappings[0].ebs.kms_key_id #=> String
    #   resp.images[0].block_device_mappings[0].ebs.throughput #=> Integer
    #   resp.images[0].block_device_mappings[0].ebs.outpost_arn #=> String
    #   resp.images[0].block_device_mappings[0].ebs.encrypted #=> Boolean
    #   resp.images[0].block_device_mappings[0].no_device #=> String
    #   resp.images[0].block_device_mappings[0].device_name #=> String
    #   resp.images[0].block_device_mappings[0].virtual_name #=> String
    #   resp.images[0].description #=> String
    #   resp.images[0].ena_support #=> Boolean
    #   resp.images[0].hypervisor #=> String, one of "ovm", "xen"
    #   resp.images[0].image_owner_alias #=> String
    #   resp.images[0].name #=> String
    #   resp.images[0].root_device_name #=> String
    #   resp.images[0].root_device_type #=> String, one of "ebs", "instance-store"
    #   resp.images[0].sriov_net_support #=> String
    #   resp.images[0].state_reason.code #=> String
    #   resp.images[0].state_reason.message #=> String
    #   resp.images[0].tags #=> Array
    #   resp.images[0].tags[0].key #=> String
    #   resp.images[0].tags[0].value #=> String
    #   resp.images[0].virtualization_type #=> String, one of "hvm", "paravirtual"
    #   resp.images[0].boot_mode #=> String, one of "legacy-bios", "uefi", "uefi-preferred"
    #   resp.images[0].tpm_support #=> String, one of "v2.0"
    #   resp.images[0].deprecation_time #=> String
    #   resp.images[0].imds_support #=> String, one of "v2.0"
    #   resp.images[0].source_instance_id #=> String
    #   resp.images[0].deregistration_protection #=> String
    #   resp.images[0].last_launched_time #=> String
    #   resp.images[0].image_allowed #=> Boolean
    #   resp.images[0].source_image_id #=> String
    #   resp.images[0].source_image_region #=> String
    #   resp.images[0].image_id #=> String
    #   resp.images[0].image_location #=> String
    #   resp.images[0].state #=> String, one of "pending", "available", "invalid", "deregistered", "transient", "failed", "error", "disabled"
    #   resp.images[0].owner_id #=> String
    #   resp.images[0].creation_date #=> String
    #   resp.images[0].public #=> Boolean
    #   resp.images[0].product_codes #=> Array
    #   resp.images[0].product_codes[0].product_code_id #=> String
    #   resp.images[0].product_codes[0].product_code_type #=> String, one of "devpay", "marketplace"
    #   resp.images[0].architecture #=> String, one of "i386", "x86_64", "arm64", "x86_64_mac", "arm64_mac"
    #   resp.images[0].image_type #=> String, one of "machine", "kernel", "ramdisk"
    #   resp.images[0].kernel_id #=> String
    #   resp.images[0].ramdisk_id #=> String
    #   resp.images[0].platform #=> String, one of "Windows"
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * image_available
    #   * image_exists
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeImages AWS API Documentation
    #
    # @overload describe_images(params = {})
    # @param [Hash] params ({})
    def describe_images(params = {}, options = {})
      req = build_request(:describe_images, params)
      req.send_request(options)
    end

    # Displays details about an import virtual machine or import snapshot
    # tasks that are already created.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   Filter tasks using the `task-state` filter and one of the following
    #   values: `active`, `completed`, `deleting`, or `deleted`.
    #
    # @option params [Array<String>] :import_task_ids
    #   The IDs of the import image tasks.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in a single call.
    #
    # @option params [String] :next_token
    #   A token that indicates the next page of results.
    #
    # @return [Types::DescribeImportImageTasksResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeImportImageTasksResult#import_image_tasks #import_image_tasks} => Array&lt;Types::ImportImageTask&gt;
    #   * {Types::DescribeImportImageTasksResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_import_image_tasks({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     import_task_ids: ["ImportImageTaskId"],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.import_image_tasks #=> Array
    #   resp.import_image_tasks[0].architecture #=> String
    #   resp.import_image_tasks[0].description #=> String
    #   resp.import_image_tasks[0].encrypted #=> Boolean
    #   resp.import_image_tasks[0].hypervisor #=> String
    #   resp.import_image_tasks[0].image_id #=> String
    #   resp.import_image_tasks[0].import_task_id #=> String
    #   resp.import_image_tasks[0].kms_key_id #=> String
    #   resp.import_image_tasks[0].license_type #=> String
    #   resp.import_image_tasks[0].platform #=> String
    #   resp.import_image_tasks[0].progress #=> String
    #   resp.import_image_tasks[0].snapshot_details #=> Array
    #   resp.import_image_tasks[0].snapshot_details[0].description #=> String
    #   resp.import_image_tasks[0].snapshot_details[0].device_name #=> String
    #   resp.import_image_tasks[0].snapshot_details[0].disk_image_size #=> Float
    #   resp.import_image_tasks[0].snapshot_details[0].format #=> String
    #   resp.import_image_tasks[0].snapshot_details[0].progress #=> String
    #   resp.import_image_tasks[0].snapshot_details[0].snapshot_id #=> String
    #   resp.import_image_tasks[0].snapshot_details[0].status #=> String
    #   resp.import_image_tasks[0].snapshot_details[0].status_message #=> String
    #   resp.import_image_tasks[0].snapshot_details[0].url #=> String
    #   resp.import_image_tasks[0].snapshot_details[0].user_bucket.s3_bucket #=> String
    #   resp.import_image_tasks[0].snapshot_details[0].user_bucket.s3_key #=> String
    #   resp.import_image_tasks[0].status #=> String
    #   resp.import_image_tasks[0].status_message #=> String
    #   resp.import_image_tasks[0].tags #=> Array
    #   resp.import_image_tasks[0].tags[0].key #=> String
    #   resp.import_image_tasks[0].tags[0].value #=> String
    #   resp.import_image_tasks[0].license_specifications #=> Array
    #   resp.import_image_tasks[0].license_specifications[0].license_configuration_arn #=> String
    #   resp.import_image_tasks[0].usage_operation #=> String
    #   resp.import_image_tasks[0].boot_mode #=> String, one of "legacy-bios", "uefi", "uefi-preferred"
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeImportImageTasks AWS API Documentation
    #
    # @overload describe_import_image_tasks(params = {})
    # @param [Hash] params ({})
    def describe_import_image_tasks(params = {}, options = {})
      req = build_request(:describe_import_image_tasks, params)
      req.send_request(options)
    end

    # Describes your import snapshot tasks.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    # @option params [Array<String>] :import_task_ids
    #   A list of import snapshot task IDs.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in a single call. To retrieve
    #   the remaining results, make another call with the returned `NextToken`
    #   value.
    #
    # @option params [String] :next_token
    #   A token that indicates the next page of results.
    #
    # @return [Types::DescribeImportSnapshotTasksResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeImportSnapshotTasksResult#import_snapshot_tasks #import_snapshot_tasks} => Array&lt;Types::ImportSnapshotTask&gt;
    #   * {Types::DescribeImportSnapshotTasksResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_import_snapshot_tasks({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     import_task_ids: ["ImportSnapshotTaskId"],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.import_snapshot_tasks #=> Array
    #   resp.import_snapshot_tasks[0].description #=> String
    #   resp.import_snapshot_tasks[0].import_task_id #=> String
    #   resp.import_snapshot_tasks[0].snapshot_task_detail.description #=> String
    #   resp.import_snapshot_tasks[0].snapshot_task_detail.disk_image_size #=> Float
    #   resp.import_snapshot_tasks[0].snapshot_task_detail.encrypted #=> Boolean
    #   resp.import_snapshot_tasks[0].snapshot_task_detail.format #=> String
    #   resp.import_snapshot_tasks[0].snapshot_task_detail.kms_key_id #=> String
    #   resp.import_snapshot_tasks[0].snapshot_task_detail.progress #=> String
    #   resp.import_snapshot_tasks[0].snapshot_task_detail.snapshot_id #=> String
    #   resp.import_snapshot_tasks[0].snapshot_task_detail.status #=> String
    #   resp.import_snapshot_tasks[0].snapshot_task_detail.status_message #=> String
    #   resp.import_snapshot_tasks[0].snapshot_task_detail.url #=> String
    #   resp.import_snapshot_tasks[0].snapshot_task_detail.user_bucket.s3_bucket #=> String
    #   resp.import_snapshot_tasks[0].snapshot_task_detail.user_bucket.s3_key #=> String
    #   resp.import_snapshot_tasks[0].tags #=> Array
    #   resp.import_snapshot_tasks[0].tags[0].key #=> String
    #   resp.import_snapshot_tasks[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * snapshot_imported
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeImportSnapshotTasks AWS API Documentation
    #
    # @overload describe_import_snapshot_tasks(params = {})
    # @param [Hash] params ({})
    def describe_import_snapshot_tasks(params = {}, options = {})
      req = build_request(:describe_import_snapshot_tasks, params)
      req.send_request(options)
    end

    # Describes the specified attribute of the specified instance. You can
    # specify only one attribute at a time.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @option params [required, String] :attribute
    #   The instance attribute.
    #
    #   Note that the `enaSupport` attribute is not supported.
    #
    # @return [Types::InstanceAttribute] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::InstanceAttribute#block_device_mappings #block_device_mappings} => Array&lt;Types::InstanceBlockDeviceMapping&gt;
    #   * {Types::InstanceAttribute#disable_api_termination #disable_api_termination} => Types::AttributeBooleanValue
    #   * {Types::InstanceAttribute#ena_support #ena_support} => Types::AttributeBooleanValue
    #   * {Types::InstanceAttribute#enclave_options #enclave_options} => Types::EnclaveOptions
    #   * {Types::InstanceAttribute#ebs_optimized #ebs_optimized} => Types::AttributeBooleanValue
    #   * {Types::InstanceAttribute#instance_id #instance_id} => String
    #   * {Types::InstanceAttribute#instance_initiated_shutdown_behavior #instance_initiated_shutdown_behavior} => Types::AttributeValue
    #   * {Types::InstanceAttribute#instance_type #instance_type} => Types::AttributeValue
    #   * {Types::InstanceAttribute#kernel_id #kernel_id} => Types::AttributeValue
    #   * {Types::InstanceAttribute#product_codes #product_codes} => Array&lt;Types::ProductCode&gt;
    #   * {Types::InstanceAttribute#ramdisk_id #ramdisk_id} => Types::AttributeValue
    #   * {Types::InstanceAttribute#root_device_name #root_device_name} => Types::AttributeValue
    #   * {Types::InstanceAttribute#source_dest_check #source_dest_check} => Types::AttributeBooleanValue
    #   * {Types::InstanceAttribute#sriov_net_support #sriov_net_support} => Types::AttributeValue
    #   * {Types::InstanceAttribute#user_data #user_data} => Types::AttributeValue
    #   * {Types::InstanceAttribute#disable_api_stop #disable_api_stop} => Types::AttributeBooleanValue
    #   * {Types::InstanceAttribute#groups #groups} => Array&lt;Types::GroupIdentifier&gt;
    #
    #
    # @example Example: To describe the instance type
    #
    #   # This example describes the instance type of the specified instance.
    #
    #   resp = client.describe_instance_attribute({
    #     attribute: "instanceType", 
    #     instance_id: "i-1234567890abcdef0", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     instance_id: "i-1234567890abcdef0", 
    #     instance_type: {
    #       value: "t1.micro", 
    #     }, 
    #   }
    #
    # @example Example: To describe the disableApiTermination attribute
    #
    #   # This example describes the ``disableApiTermination`` attribute of the specified instance.
    #
    #   resp = client.describe_instance_attribute({
    #     attribute: "disableApiTermination", 
    #     instance_id: "i-1234567890abcdef0", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     disable_api_termination: {
    #       value: false, 
    #     }, 
    #     instance_id: "i-1234567890abcdef0", 
    #   }
    #
    # @example Example: To describe the block device mapping for an instance
    #
    #   # This example describes the ``blockDeviceMapping`` attribute of the specified instance.
    #
    #   resp = client.describe_instance_attribute({
    #     attribute: "blockDeviceMapping", 
    #     instance_id: "i-1234567890abcdef0", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     block_device_mappings: [
    #       {
    #         device_name: "/dev/sda1", 
    #         ebs: {
    #           attach_time: Time.parse("2013-05-17T22:42:34.000Z"), 
    #           delete_on_termination: true, 
    #           status: "attached", 
    #           volume_id: "vol-049df61146c4d7901", 
    #         }, 
    #       }, 
    #       {
    #         device_name: "/dev/sdf", 
    #         ebs: {
    #           attach_time: Time.parse("2013-09-10T23:07:00.000Z"), 
    #           delete_on_termination: false, 
    #           status: "attached", 
    #           volume_id: "vol-049df61146c4d7901", 
    #         }, 
    #       }, 
    #     ], 
    #     instance_id: "i-1234567890abcdef0", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_instance_attribute({
    #     dry_run: false,
    #     instance_id: "InstanceId", # required
    #     attribute: "instanceType", # required, accepts instanceType, kernel, ramdisk, userData, disableApiTermination, instanceInitiatedShutdownBehavior, rootDeviceName, blockDeviceMapping, productCodes, sourceDestCheck, groupSet, ebsOptimized, sriovNetSupport, enaSupport, enclaveOptions, disableApiStop
    #   })
    #
    # @example Response structure
    #
    #   resp.block_device_mappings #=> Array
    #   resp.block_device_mappings[0].device_name #=> String
    #   resp.block_device_mappings[0].ebs.attach_time #=> Time
    #   resp.block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.block_device_mappings[0].ebs.status #=> String, one of "attaching", "attached", "detaching", "detached"
    #   resp.block_device_mappings[0].ebs.volume_id #=> String
    #   resp.block_device_mappings[0].ebs.associated_resource #=> String
    #   resp.block_device_mappings[0].ebs.volume_owner_id #=> String
    #   resp.block_device_mappings[0].ebs.operator.managed #=> Boolean
    #   resp.block_device_mappings[0].ebs.operator.principal #=> String
    #   resp.disable_api_termination.value #=> Boolean
    #   resp.ena_support.value #=> Boolean
    #   resp.enclave_options.enabled #=> Boolean
    #   resp.ebs_optimized.value #=> Boolean
    #   resp.instance_id #=> String
    #   resp.instance_initiated_shutdown_behavior #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.instance_type #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.kernel_id #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.product_codes #=> Array
    #   resp.product_codes[0].product_code_id #=> String
    #   resp.product_codes[0].product_code_type #=> String, one of "devpay", "marketplace"
    #   resp.ramdisk_id #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.root_device_name #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.source_dest_check.value #=> Boolean
    #   resp.sriov_net_support #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.user_data #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.disable_api_stop.value #=> Boolean
    #   resp.groups #=> Array
    #   resp.groups[0].group_id #=> String
    #   resp.groups[0].group_name #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeInstanceAttribute AWS API Documentation
    #
    # @overload describe_instance_attribute(params = {})
    # @param [Hash] params ({})
    def describe_instance_attribute(params = {}, options = {})
      req = build_request(:describe_instance_attribute, params)
      req.send_request(options)
    end

    # Describes the specified EC2 Instance Connect Endpoints or all EC2
    # Instance Connect Endpoints.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `instance-connect-endpoint-id` - The ID of the EC2 Instance Connect
    #     Endpoint.
    #
    #   * `state` - The state of the EC2 Instance Connect Endpoint
    #     (`create-in-progress` \| `create-complete` \| `create-failed` \|
    #     `delete-in-progress` \| `delete-complete` \| `delete-failed`).
    #
    #   * `subnet-id` - The ID of the subnet in which the EC2 Instance Connect
    #     Endpoint was created.
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `tag-value` - The value of a tag assigned to the resource. Use this
    #     filter to find all resources that have a tag with a specific value,
    #     regardless of tag key.
    #
    #   * `vpc-id` - The ID of the VPC in which the EC2 Instance Connect
    #     Endpoint was created.
    #
    # @option params [Array<String>] :instance_connect_endpoint_ids
    #   One or more EC2 Instance Connect Endpoint IDs.
    #
    # @return [Types::DescribeInstanceConnectEndpointsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeInstanceConnectEndpointsResult#instance_connect_endpoints #instance_connect_endpoints} => Array&lt;Types::Ec2InstanceConnectEndpoint&gt;
    #   * {Types::DescribeInstanceConnectEndpointsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_instance_connect_endpoints({
    #     dry_run: false,
    #     max_results: 1,
    #     next_token: "NextToken",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     instance_connect_endpoint_ids: ["String"],
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_connect_endpoints #=> Array
    #   resp.instance_connect_endpoints[0].owner_id #=> String
    #   resp.instance_connect_endpoints[0].instance_connect_endpoint_id #=> String
    #   resp.instance_connect_endpoints[0].instance_connect_endpoint_arn #=> String
    #   resp.instance_connect_endpoints[0].state #=> String, one of "create-in-progress", "create-complete", "create-failed", "delete-in-progress", "delete-complete", "delete-failed"
    #   resp.instance_connect_endpoints[0].state_message #=> String
    #   resp.instance_connect_endpoints[0].dns_name #=> String
    #   resp.instance_connect_endpoints[0].fips_dns_name #=> String
    #   resp.instance_connect_endpoints[0].network_interface_ids #=> Array
    #   resp.instance_connect_endpoints[0].network_interface_ids[0] #=> String
    #   resp.instance_connect_endpoints[0].vpc_id #=> String
    #   resp.instance_connect_endpoints[0].availability_zone #=> String
    #   resp.instance_connect_endpoints[0].created_at #=> Time
    #   resp.instance_connect_endpoints[0].subnet_id #=> String
    #   resp.instance_connect_endpoints[0].preserve_client_ip #=> Boolean
    #   resp.instance_connect_endpoints[0].security_group_ids #=> Array
    #   resp.instance_connect_endpoints[0].security_group_ids[0] #=> String
    #   resp.instance_connect_endpoints[0].tags #=> Array
    #   resp.instance_connect_endpoints[0].tags[0].key #=> String
    #   resp.instance_connect_endpoints[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeInstanceConnectEndpoints AWS API Documentation
    #
    # @overload describe_instance_connect_endpoints(params = {})
    # @param [Hash] params ({})
    def describe_instance_connect_endpoints(params = {}, options = {})
      req = build_request(:describe_instance_connect_endpoints, params)
      req.send_request(options)
    end

    # Describes the credit option for CPU usage of the specified burstable
    # performance instances. The credit options are `standard` and
    # `unlimited`.
    #
    # If you do not specify an instance ID, Amazon EC2 returns burstable
    # performance instances with the `unlimited` credit option, as well as
    # instances that were previously configured as T2, T3, and T3a with the
    # `unlimited` credit option. For example, if you resize a T2 instance,
    # while it is configured as `unlimited`, to an M4 instance, Amazon EC2
    # returns the M4 instance.
    #
    # If you specify one or more instance IDs, Amazon EC2 returns the credit
    # option (`standard` or `unlimited`) of those instances. If you specify
    # an instance ID that is not valid, such as an instance that is not a
    # burstable performance instance, an error is returned.
    #
    # Recently terminated instances might appear in the returned results.
    # This interval is usually less than one hour.
    #
    # If an Availability Zone is experiencing a service disruption and you
    # specify instance IDs in the affected zone, or do not specify any
    # instance IDs at all, the call fails. If you specify only instance IDs
    # in an unaffected zone, the call works normally.
    #
    # For more information, see [Burstable performance instances][1] in the
    # *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/burstable-performance-instances.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `instance-id` - The ID of the instance.
    #
    #   ^
    #
    # @option params [Array<String>] :instance_ids
    #   The instance IDs.
    #
    #   Default: Describes all your instances.
    #
    #   Constraints: Maximum 1000 explicitly specified instance IDs.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #   You cannot specify this parameter and the instance IDs parameter in
    #   the same call.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @return [Types::DescribeInstanceCreditSpecificationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeInstanceCreditSpecificationsResult#instance_credit_specifications #instance_credit_specifications} => Array&lt;Types::InstanceCreditSpecification&gt;
    #   * {Types::DescribeInstanceCreditSpecificationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_instance_credit_specifications({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     instance_ids: ["InstanceId"],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_credit_specifications #=> Array
    #   resp.instance_credit_specifications[0].instance_id #=> String
    #   resp.instance_credit_specifications[0].cpu_credits #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeInstanceCreditSpecifications AWS API Documentation
    #
    # @overload describe_instance_credit_specifications(params = {})
    # @param [Hash] params ({})
    def describe_instance_credit_specifications(params = {}, options = {})
      req = build_request(:describe_instance_credit_specifications, params)
      req.send_request(options)
    end

    # Describes the tag keys that are registered to appear in scheduled
    # event notifications for resources in the current Region.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeInstanceEventNotificationAttributesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeInstanceEventNotificationAttributesResult#instance_tag_attribute #instance_tag_attribute} => Types::InstanceTagNotificationAttribute
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_instance_event_notification_attributes({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_tag_attribute.instance_tag_keys #=> Array
    #   resp.instance_tag_attribute.instance_tag_keys[0] #=> String
    #   resp.instance_tag_attribute.include_all_tags_of_instance #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeInstanceEventNotificationAttributes AWS API Documentation
    #
    # @overload describe_instance_event_notification_attributes(params = {})
    # @param [Hash] params ({})
    def describe_instance_event_notification_attributes(params = {}, options = {})
      req = build_request(:describe_instance_event_notification_attributes, params)
      req.send_request(options)
    end

    # Describes the specified event windows or all event windows.
    #
    # If you specify event window IDs, the output includes information for
    # only the specified event windows. If you specify filters, the output
    # includes information for only those event windows that meet the filter
    # criteria. If you do not specify event windows IDs or filters, the
    # output includes information for all event windows, which can affect
    # performance. We recommend that you use pagination to ensure that the
    # operation returns quickly and successfully.
    #
    # For more information, see [Define event windows for scheduled
    # events][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/event-windows.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :instance_event_window_ids
    #   The IDs of the event windows.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `dedicated-host-id` - The event windows associated with the
    #     specified Dedicated Host ID.
    #
    #   * `event-window-name` - The event windows associated with the
    #     specified names.
    #
    #   * `instance-id` - The event windows associated with the specified
    #     instance ID.
    #
    #   * `instance-tag` - The event windows associated with the specified tag
    #     and value.
    #
    #   * `instance-tag-key` - The event windows associated with the specified
    #     tag key, regardless of the value.
    #
    #   * `instance-tag-value` - The event windows associated with the
    #     specified tag value, regardless of the key.
    #
    #   * `tag:<key>` - The key/value combination of a tag assigned to the
    #     event window. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `CMX`, specify `tag:Owner`
    #     for the filter name and `CMX` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the event window. Use this
    #     filter to find all event windows that have a tag with a specific
    #     key, regardless of the tag value.
    #
    #   * `tag-value` - The value of a tag assigned to the event window. Use
    #     this filter to find all event windows that have a tag with a
    #     specific value, regardless of the tag key.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in a single call. To retrieve
    #   the remaining results, make another call with the returned `NextToken`
    #   value. This value can be between 20 and 500. You cannot specify this
    #   parameter and the event window IDs parameter in the same call.
    #
    # @option params [String] :next_token
    #   The token to request the next page of results.
    #
    # @return [Types::DescribeInstanceEventWindowsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeInstanceEventWindowsResult#instance_event_windows #instance_event_windows} => Array&lt;Types::InstanceEventWindow&gt;
    #   * {Types::DescribeInstanceEventWindowsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_instance_event_windows({
    #     dry_run: false,
    #     instance_event_window_ids: ["InstanceEventWindowId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_event_windows #=> Array
    #   resp.instance_event_windows[0].instance_event_window_id #=> String
    #   resp.instance_event_windows[0].time_ranges #=> Array
    #   resp.instance_event_windows[0].time_ranges[0].start_week_day #=> String, one of "sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"
    #   resp.instance_event_windows[0].time_ranges[0].start_hour #=> Integer
    #   resp.instance_event_windows[0].time_ranges[0].end_week_day #=> String, one of "sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"
    #   resp.instance_event_windows[0].time_ranges[0].end_hour #=> Integer
    #   resp.instance_event_windows[0].name #=> String
    #   resp.instance_event_windows[0].cron_expression #=> String
    #   resp.instance_event_windows[0].association_target.instance_ids #=> Array
    #   resp.instance_event_windows[0].association_target.instance_ids[0] #=> String
    #   resp.instance_event_windows[0].association_target.tags #=> Array
    #   resp.instance_event_windows[0].association_target.tags[0].key #=> String
    #   resp.instance_event_windows[0].association_target.tags[0].value #=> String
    #   resp.instance_event_windows[0].association_target.dedicated_host_ids #=> Array
    #   resp.instance_event_windows[0].association_target.dedicated_host_ids[0] #=> String
    #   resp.instance_event_windows[0].state #=> String, one of "creating", "deleting", "active", "deleted"
    #   resp.instance_event_windows[0].tags #=> Array
    #   resp.instance_event_windows[0].tags[0].key #=> String
    #   resp.instance_event_windows[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeInstanceEventWindows AWS API Documentation
    #
    # @overload describe_instance_event_windows(params = {})
    # @param [Hash] params ({})
    def describe_instance_event_windows(params = {}, options = {})
      req = build_request(:describe_instance_event_windows, params)
      req.send_request(options)
    end

    # Describes the AMI that was used to launch an instance, even if the AMI
    # is deprecated, deregistered, made private (no longer public or shared
    # with your account), or not allowed.
    #
    # If you specify instance IDs, the output includes information for only
    # the specified instances. If you specify filters, the output includes
    # information for only those instances that meet the filter criteria. If
    # you do not specify instance IDs or filters, the output includes
    # information for all instances, which can affect performance.
    #
    # If you specify an instance ID that is not valid, an instance that
    # doesn't exist, or an instance that you do not own, an error
    # (`InvalidInstanceID.NotFound`) is returned.
    #
    # Recently terminated instances might appear in the returned results.
    # This interval is usually less than one hour.
    #
    # In the rare case where an Availability Zone is experiencing a service
    # disruption and you specify instance IDs that are in the affected
    # Availability Zone, or do not specify any instance IDs at all, the call
    # fails. If you specify only instance IDs that are in an unaffected
    # Availability Zone, the call works normally.
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `availability-zone` - The name of the Availability Zone (for
    #     example, `us-west-2a`) or Local Zone (for example,
    #     `us-west-2-lax-1b`) of the instance.
    #
    #   * `instance-id` - The ID of the instance.
    #
    #   * `image-allowed` - A Boolean that indicates whether the image meets
    #     the criteria specified for Allowed AMIs.
    #
    #   * `instance-state-name` - The state of the instance (`pending` \|
    #     `running` \| `shutting-down` \| `terminated` \| `stopping` \|
    #     `stopped`).
    #
    #   * `instance-type` - The type of instance (for example, `t3.micro`).
    #
    #   * `launch-time` - The time when the instance was launched, in the ISO
    #     8601 format in the UTC time zone (YYYY-MM-DDThh:mm:ss.sssZ), for
    #     example, `2023-09-29T11:04:43.305Z`. You can use a wildcard (`*`),
    #     for example, `2023-09-29T*`, which matches an entire day.
    #
    #   * `owner-alias` - The owner alias (`amazon` \| `aws-marketplace` \|
    #     `aws-backup-vault`). The valid aliases are defined in an
    #     Amazon-maintained list. This is not the Amazon Web Services account
    #     alias that can be set using the IAM console. We recommend that you
    #     use the `Owner` request parameter instead of this filter.
    #
    #   * `owner-id` - The Amazon Web Services account ID of the owner. We
    #     recommend that you use the `Owner` request parameter instead of this
    #     filter.
    #
    #   * `tag:<key>` - The key/value combination of a tag assigned to the
    #     resource. Use the tag key in the filter name and the tag value as
    #     the filter value. For example, to find all resources that have a tag
    #     with the key `Owner` and the value `TeamA`, specify `tag:Owner` for
    #     the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `zone-id` - The ID of the Availability Zone (for example,
    #     `usw2-az2`) or Local Zone (for example, `usw2-lax1-az1`) of the
    #     instance.
    #
    # @option params [Array<String>] :instance_ids
    #   The instance IDs.
    #
    #   If you don't specify an instance ID or filters, the output includes
    #   information for all instances.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #   Default: 1000
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeInstanceImageMetadataResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeInstanceImageMetadataResult#instance_image_metadata #instance_image_metadata} => Array&lt;Types::InstanceImageMetadata&gt;
    #   * {Types::DescribeInstanceImageMetadataResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_instance_image_metadata({
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     instance_ids: ["InstanceId"],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_image_metadata #=> Array
    #   resp.instance_image_metadata[0].instance_id #=> String
    #   resp.instance_image_metadata[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.instance_image_metadata[0].launch_time #=> Time
    #   resp.instance_image_metadata[0].availability_zone #=> String
    #   resp.instance_image_metadata[0].zone_id #=> String
    #   resp.instance_image_metadata[0].state.code #=> Integer
    #   resp.instance_image_metadata[0].state.name #=> String, one of "pending", "running", "shutting-down", "terminated", "stopping", "stopped"
    #   resp.instance_image_metadata[0].owner_id #=> String
    #   resp.instance_image_metadata[0].tags #=> Array
    #   resp.instance_image_metadata[0].tags[0].key #=> String
    #   resp.instance_image_metadata[0].tags[0].value #=> String
    #   resp.instance_image_metadata[0].image_metadata.image_id #=> String
    #   resp.instance_image_metadata[0].image_metadata.name #=> String
    #   resp.instance_image_metadata[0].image_metadata.owner_id #=> String
    #   resp.instance_image_metadata[0].image_metadata.state #=> String, one of "pending", "available", "invalid", "deregistered", "transient", "failed", "error", "disabled"
    #   resp.instance_image_metadata[0].image_metadata.image_owner_alias #=> String
    #   resp.instance_image_metadata[0].image_metadata.creation_date #=> String
    #   resp.instance_image_metadata[0].image_metadata.deprecation_time #=> String
    #   resp.instance_image_metadata[0].image_metadata.image_allowed #=> Boolean
    #   resp.instance_image_metadata[0].image_metadata.is_public #=> Boolean
    #   resp.instance_image_metadata[0].operator.managed #=> Boolean
    #   resp.instance_image_metadata[0].operator.principal #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeInstanceImageMetadata AWS API Documentation
    #
    # @overload describe_instance_image_metadata(params = {})
    # @param [Hash] params ({})
    def describe_instance_image_metadata(params = {}, options = {})
      req = build_request(:describe_instance_image_metadata, params)
      req.send_request(options)
    end

    # Describes the status of the specified instances or all of your
    # instances. By default, only running instances are described, unless
    # you specifically indicate to return the status of all instances.
    #
    # Instance status includes the following components:
    #
    # * **Status checks** - Amazon EC2 performs status checks on running EC2
    #   instances to identify hardware and software issues. For more
    #   information, see [Status checks for your instances][1] and
    #   [Troubleshoot instances with failed status checks][2] in the *Amazon
    #   EC2 User Guide*.
    #
    # * **Scheduled events** - Amazon EC2 can schedule events (such as
    #   reboot, stop, or terminate) for your instances related to hardware
    #   issues, software updates, or system maintenance. For more
    #   information, see [Scheduled events for your instances][3] in the
    #   *Amazon EC2 User Guide*.
    #
    # * **Instance state** - You can manage your instances from the moment
    #   you launch them through their termination. For more information, see
    #   [Instance lifecycle][4] in the *Amazon EC2 User Guide*.
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/monitoring-system-instance-status-check.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/TroubleshootingInstances.html
    # [3]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/monitoring-instances-status-check_sched.html
    # [4]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-lifecycle.html
    #
    # @option params [Array<String>] :instance_ids
    #   The instance IDs.
    #
    #   Default: Describes all your instances.
    #
    #   Constraints: Maximum 100 explicitly specified instance IDs.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #   You cannot specify this parameter and the instance IDs parameter in
    #   the same request.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `availability-zone` - The Availability Zone of the instance.
    #
    #   * `event.code` - The code for the scheduled event (`instance-reboot`
    #     \| `system-reboot` \| `system-maintenance` \| `instance-retirement`
    #     \| `instance-stop`).
    #
    #   * `event.description` - A description of the event.
    #
    #   * `event.instance-event-id` - The ID of the event whose date and time
    #     you are modifying.
    #
    #   * `event.not-after` - The latest end time for the scheduled event (for
    #     example, `2014-09-15T17:15:20.000Z`).
    #
    #   * `event.not-before` - The earliest start time for the scheduled event
    #     (for example, `2014-09-15T17:15:20.000Z`).
    #
    #   * `event.not-before-deadline` - The deadline for starting the event
    #     (for example, `2014-09-15T17:15:20.000Z`).
    #
    #   * `instance-state-code` - The code for the instance state, as a 16-bit
    #     unsigned integer. The high byte is used for internal purposes and
    #     should be ignored. The low byte is set based on the state
    #     represented. The valid values are 0 (pending), 16 (running), 32
    #     (shutting-down), 48 (terminated), 64 (stopping), and 80 (stopped).
    #
    #   * `instance-state-name` - The state of the instance (`pending` \|
    #     `running` \| `shutting-down` \| `terminated` \| `stopping` \|
    #     `stopped`).
    #
    #   * `instance-status.reachability` - Filters on instance status where
    #     the name is `reachability` (`passed` \| `failed` \| `initializing`
    #     \| `insufficient-data`).
    #
    #   * `instance-status.status` - The status of the instance (`ok` \|
    #     `impaired` \| `initializing` \| `insufficient-data` \|
    #     `not-applicable`).
    #
    #   * `operator.managed` - A Boolean that indicates whether this is a
    #     managed instance.
    #
    #   * `operator.principal` - The principal that manages the instance. Only
    #     valid for managed instances, where `managed` is `true`.
    #
    #   * `system-status.reachability` - Filters on system status where the
    #     name is `reachability` (`passed` \| `failed` \| `initializing` \|
    #     `insufficient-data`).
    #
    #   * `system-status.status` - The system status of the instance (`ok` \|
    #     `impaired` \| `initializing` \| `insufficient-data` \|
    #     `not-applicable`).
    #
    #   * `attached-ebs-status.status` - The status of the attached EBS volume
    #     for the instance (`ok` \| `impaired` \| `initializing` \|
    #     `insufficient-data` \| `not-applicable`).
    #
    # @option params [Boolean] :include_all_instances
    #   When `true`, includes the health status for all instances. When
    #   `false`, includes the health status for running instances only.
    #
    #   Default: `false`
    #
    # @return [Types::DescribeInstanceStatusResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeInstanceStatusResult#instance_statuses #instance_statuses} => Array&lt;Types::InstanceStatus&gt;
    #   * {Types::DescribeInstanceStatusResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe the status of an instance
    #
    #   # This example describes the current status of the specified instance.
    #
    #   resp = client.describe_instance_status({
    #     instance_ids: [
    #       "i-1234567890abcdef0", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     instance_statuses: [
    #       {
    #         availability_zone: "us-east-1d", 
    #         instance_id: "i-1234567890abcdef0", 
    #         instance_state: {
    #           code: 16, 
    #           name: "running", 
    #         }, 
    #         instance_status: {
    #           details: [
    #             {
    #               name: "reachability", 
    #               status: "passed", 
    #             }, 
    #           ], 
    #           status: "ok", 
    #         }, 
    #         system_status: {
    #           details: [
    #             {
    #               name: "reachability", 
    #               status: "passed", 
    #             }, 
    #           ], 
    #           status: "ok", 
    #         }, 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_instance_status({
    #     instance_ids: ["InstanceId"],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     include_all_instances: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_statuses #=> Array
    #   resp.instance_statuses[0].availability_zone #=> String
    #   resp.instance_statuses[0].outpost_arn #=> String
    #   resp.instance_statuses[0].operator.managed #=> Boolean
    #   resp.instance_statuses[0].operator.principal #=> String
    #   resp.instance_statuses[0].events #=> Array
    #   resp.instance_statuses[0].events[0].instance_event_id #=> String
    #   resp.instance_statuses[0].events[0].code #=> String, one of "instance-reboot", "system-reboot", "system-maintenance", "instance-retirement", "instance-stop"
    #   resp.instance_statuses[0].events[0].description #=> String
    #   resp.instance_statuses[0].events[0].not_after #=> Time
    #   resp.instance_statuses[0].events[0].not_before #=> Time
    #   resp.instance_statuses[0].events[0].not_before_deadline #=> Time
    #   resp.instance_statuses[0].instance_id #=> String
    #   resp.instance_statuses[0].instance_state.code #=> Integer
    #   resp.instance_statuses[0].instance_state.name #=> String, one of "pending", "running", "shutting-down", "terminated", "stopping", "stopped"
    #   resp.instance_statuses[0].instance_status.details #=> Array
    #   resp.instance_statuses[0].instance_status.details[0].impaired_since #=> Time
    #   resp.instance_statuses[0].instance_status.details[0].name #=> String, one of "reachability"
    #   resp.instance_statuses[0].instance_status.details[0].status #=> String, one of "passed", "failed", "insufficient-data", "initializing"
    #   resp.instance_statuses[0].instance_status.status #=> String, one of "ok", "impaired", "insufficient-data", "not-applicable", "initializing"
    #   resp.instance_statuses[0].system_status.details #=> Array
    #   resp.instance_statuses[0].system_status.details[0].impaired_since #=> Time
    #   resp.instance_statuses[0].system_status.details[0].name #=> String, one of "reachability"
    #   resp.instance_statuses[0].system_status.details[0].status #=> String, one of "passed", "failed", "insufficient-data", "initializing"
    #   resp.instance_statuses[0].system_status.status #=> String, one of "ok", "impaired", "insufficient-data", "not-applicable", "initializing"
    #   resp.instance_statuses[0].attached_ebs_status.details #=> Array
    #   resp.instance_statuses[0].attached_ebs_status.details[0].impaired_since #=> Time
    #   resp.instance_statuses[0].attached_ebs_status.details[0].name #=> String, one of "reachability"
    #   resp.instance_statuses[0].attached_ebs_status.details[0].status #=> String, one of "passed", "failed", "insufficient-data", "initializing"
    #   resp.instance_statuses[0].attached_ebs_status.status #=> String, one of "ok", "impaired", "insufficient-data", "not-applicable", "initializing"
    #   resp.next_token #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * instance_status_ok
    #   * system_status_ok
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeInstanceStatus AWS API Documentation
    #
    # @overload describe_instance_status(params = {})
    # @param [Hash] params ({})
    def describe_instance_status(params = {}, options = {})
      req = build_request(:describe_instance_status, params)
      req.send_request(options)
    end

    # Describes a tree-based hierarchy that represents the physical host
    # placement of your EC2 instances within an Availability Zone or Local
    # Zone. You can use this information to determine the relative proximity
    # of your EC2 instances within the Amazon Web Services network to
    # support your tightly coupled workloads.
    #
    # **Limitations**
    #
    # * Supported zones
    #
    #   * Availability Zone
    #
    #   * Local Zone
    # * Supported instance types
    #
    #   * `hpc6a.48xlarge` \| `hpc6id.32xlarge` \| `hpc7a.12xlarge` \|
    #     `hpc7a.24xlarge` \| `hpc7a.48xlarge` \| `hpc7a.96xlarge` \|
    #     `hpc7g.4xlarge` \| `hpc7g.8xlarge` \| `hpc7g.16xlarge`
    #
    #   * `p3dn.24xlarge` \| `p4d.24xlarge` \| `p4de.24xlarge` \|
    #     `p5.48xlarge` \| `p5e.48xlarge` \| `p5en.48xlarge`
    #
    #   * `trn1.2xlarge` \| `trn1.32xlarge` \| `trn1n.32xlarge` \|
    #     `trn2.48xlarge` \| `trn2u.48xlarge`
    #
    # For more information, see [Amazon EC2 instance topology][1] in the
    # *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-topology.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #   You can't specify this parameter and the instance IDs parameter in
    #   the same request.
    #
    #   Default: `20`
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Array<String>] :instance_ids
    #   The instance IDs.
    #
    #   Default: Describes all your instances.
    #
    #   Constraints: Maximum 100 explicitly specified instance IDs.
    #
    # @option params [Array<String>] :group_names
    #   The name of the placement group that each instance is in.
    #
    #   Constraints: Maximum 100 explicitly specified placement group names.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `availability-zone` - The name of the Availability Zone (for
    #     example, `us-west-2a`) or Local Zone (for example,
    #     `us-west-2-lax-1b`) that the instance is in.
    #
    #   * `instance-type` - The instance type (for example, `p4d.24xlarge`) or
    #     instance family (for example, `p4d*`). You can use the `*` wildcard
    #     to match zero or more characters, or the `?` wildcard to match zero
    #     or one character.
    #
    #   * `zone-id` - The ID of the Availability Zone (for example,
    #     `usw2-az2`) or Local Zone (for example, `usw2-lax1-az1`) that the
    #     instance is in.
    #
    # @return [Types::DescribeInstanceTopologyResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeInstanceTopologyResult#instances #instances} => Array&lt;Types::InstanceTopology&gt;
    #   * {Types::DescribeInstanceTopologyResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_instance_topology({
    #     dry_run: false,
    #     next_token: "String",
    #     max_results: 1,
    #     instance_ids: ["InstanceId"],
    #     group_names: ["PlacementGroupName"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.instances #=> Array
    #   resp.instances[0].instance_id #=> String
    #   resp.instances[0].instance_type #=> String
    #   resp.instances[0].group_name #=> String
    #   resp.instances[0].network_nodes #=> Array
    #   resp.instances[0].network_nodes[0] #=> String
    #   resp.instances[0].availability_zone #=> String
    #   resp.instances[0].zone_id #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeInstanceTopology AWS API Documentation
    #
    # @overload describe_instance_topology(params = {})
    # @param [Hash] params ({})
    def describe_instance_topology(params = {}, options = {})
      req = build_request(:describe_instance_topology, params)
      req.send_request(options)
    end

    # Lists the instance types that are offered for the specified location.
    # If no location is specified, the default is to list the instance types
    # that are offered in the current Region.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :location_type
    #   The location type.
    #
    #   * `availability-zone` - The Availability Zone. When you specify a
    #     location filter, it must be an Availability Zone for the current
    #     Region.
    #
    #   * `availability-zone-id` - The AZ ID. When you specify a location
    #     filter, it must be an AZ ID for the current Region.
    #
    #   * `outpost` - The Outpost ARN. When you specify a location filter, it
    #     must be an Outpost ARN for the current Region.
    #
    #   * `region` - The current Region. If you specify a location filter, it
    #     must match the current Region.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. Filter names and values are case-sensitive.
    #
    #   * `instance-type` - The instance type. For a list of possible values,
    #     see [Instance][1].
    #
    #   * `location` - The location. For a list of possible identifiers, see
    #     [Regions and Zones][2].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_Instance.html
    #   [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-regions-availability-zones.html
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @return [Types::DescribeInstanceTypeOfferingsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeInstanceTypeOfferingsResult#instance_type_offerings #instance_type_offerings} => Array&lt;Types::InstanceTypeOffering&gt;
    #   * {Types::DescribeInstanceTypeOfferingsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_instance_type_offerings({
    #     dry_run: false,
    #     location_type: "region", # accepts region, availability-zone, availability-zone-id, outpost
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_type_offerings #=> Array
    #   resp.instance_type_offerings[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.instance_type_offerings[0].location_type #=> String, one of "region", "availability-zone", "availability-zone-id", "outpost"
    #   resp.instance_type_offerings[0].location #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeInstanceTypeOfferings AWS API Documentation
    #
    # @overload describe_instance_type_offerings(params = {})
    # @param [Hash] params ({})
    def describe_instance_type_offerings(params = {}, options = {})
      req = build_request(:describe_instance_type_offerings, params)
      req.send_request(options)
    end

    # Describes the specified instance types. By default, all instance types
    # for the current Region are described. Alternatively, you can filter
    # the results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :instance_types
    #   The instance types.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. Filter names and values are case-sensitive.
    #
    #   * `auto-recovery-supported` - Indicates whether Amazon CloudWatch
    #     action based recovery is supported (`true` \| `false`).
    #
    #   * `bare-metal` - Indicates whether it is a bare metal instance type
    #     (`true` \| `false`).
    #
    #   * `burstable-performance-supported` - Indicates whether the instance
    #     type is a burstable performance T instance type (`true` \| `false`).
    #
    #   * `current-generation` - Indicates whether this instance type is the
    #     latest generation instance type of an instance family (`true` \|
    #     `false`).
    #
    #   * `ebs-info.ebs-optimized-info.baseline-bandwidth-in-mbps` - The
    #     baseline bandwidth performance for an EBS-optimized instance type,
    #     in Mbps.
    #
    #   * `ebs-info.ebs-optimized-info.baseline-iops` - The baseline
    #     input/output storage operations per second for an EBS-optimized
    #     instance type.
    #
    #   * `ebs-info.ebs-optimized-info.baseline-throughput-in-mbps` - The
    #     baseline throughput performance for an EBS-optimized instance type,
    #     in MB/s.
    #
    #   * `ebs-info.ebs-optimized-info.maximum-bandwidth-in-mbps` - The
    #     maximum bandwidth performance for an EBS-optimized instance type, in
    #     Mbps.
    #
    #   * `ebs-info.ebs-optimized-info.maximum-iops` - The maximum
    #     input/output storage operations per second for an EBS-optimized
    #     instance type.
    #
    #   * `ebs-info.ebs-optimized-info.maximum-throughput-in-mbps` - The
    #     maximum throughput performance for an EBS-optimized instance type,
    #     in MB/s.
    #
    #   * `ebs-info.ebs-optimized-support` - Indicates whether the instance
    #     type is EBS-optimized (`supported` \| `unsupported` \| `default`).
    #
    #   * `ebs-info.encryption-support` - Indicates whether EBS encryption is
    #     supported (`supported` \| `unsupported`).
    #
    #   * `ebs-info.nvme-support` - Indicates whether non-volatile memory
    #     express (NVMe) is supported for EBS volumes (`required` \|
    #     `supported` \| `unsupported`).
    #
    #   * `free-tier-eligible` - Indicates whether the instance type is
    #     eligible to use in the free tier (`true` \| `false`).
    #
    #   * `hibernation-supported` - Indicates whether On-Demand hibernation is
    #     supported (`true` \| `false`).
    #
    #   * `hypervisor` - The hypervisor (`nitro` \| `xen`).
    #
    #   * `instance-storage-info.disk.count` - The number of local disks.
    #
    #   * `instance-storage-info.disk.size-in-gb` - The storage size of each
    #     instance storage disk, in GB.
    #
    #   * `instance-storage-info.disk.type` - The storage technology for the
    #     local instance storage disks (`hdd` \| `ssd`).
    #
    #   * `instance-storage-info.encryption-support` - Indicates whether data
    #     is encrypted at rest (`required` \| `supported` \| `unsupported`).
    #
    #   * `instance-storage-info.nvme-support` - Indicates whether
    #     non-volatile memory express (NVMe) is supported for instance store
    #     (`required` \| `supported` \| `unsupported`).
    #
    #   * `instance-storage-info.total-size-in-gb` - The total amount of
    #     storage available from all local instance storage, in GB.
    #
    #   * `instance-storage-supported` - Indicates whether the instance type
    #     has local instance storage (`true` \| `false`).
    #
    #   * `instance-type` - The instance type (for example `c5.2xlarge` or
    #     c5*).
    #
    #   * `memory-info.size-in-mib` - The memory size.
    #
    #   * `network-info.bandwidth-weightings` - For instances that support
    #     bandwidth weighting to boost performance (`default`, `vpc-1`,
    #     `ebs-1`).
    #
    #   * `network-info.efa-info.maximum-efa-interfaces` - The maximum number
    #     of Elastic Fabric Adapters (EFAs) per instance.
    #
    #   * `network-info.efa-supported` - Indicates whether the instance type
    #     supports Elastic Fabric Adapter (EFA) (`true` \| `false`).
    #
    #   * `network-info.ena-support` - Indicates whether Elastic Network
    #     Adapter (ENA) is supported or required (`required` \| `supported` \|
    #     `unsupported`).
    #
    #   * `network-info.encryption-in-transit-supported` - Indicates whether
    #     the instance type automatically encrypts in-transit traffic between
    #     instances (`true` \| `false`).
    #
    #   * `network-info.ipv4-addresses-per-interface` - The maximum number of
    #     private IPv4 addresses per network interface.
    #
    #   * `network-info.ipv6-addresses-per-interface` - The maximum number of
    #     private IPv6 addresses per network interface.
    #
    #   * `network-info.ipv6-supported` - Indicates whether the instance type
    #     supports IPv6 (`true` \| `false`).
    #
    #   * `network-info.maximum-network-cards` - The maximum number of network
    #     cards per instance.
    #
    #   * `network-info.maximum-network-interfaces` - The maximum number of
    #     network interfaces per instance.
    #
    #   * `network-info.network-performance` - The network performance (for
    #     example, "25 Gigabit").
    #
    #   * `nitro-enclaves-support` - Indicates whether Nitro Enclaves is
    #     supported (`supported` \| `unsupported`).
    #
    #   * `nitro-tpm-support` - Indicates whether NitroTPM is supported
    #     (`supported` \| `unsupported`).
    #
    #   * `nitro-tpm-info.supported-versions` - The supported NitroTPM version
    #     (`2.0`).
    #
    #   * `processor-info.supported-architecture` - The CPU architecture
    #     (`arm64` \| `i386` \| `x86_64`).
    #
    #   * `processor-info.sustained-clock-speed-in-ghz` - The CPU clock speed,
    #     in GHz.
    #
    #   * `processor-info.supported-features` - The supported CPU features
    #     (`amd-sev-snp`).
    #
    #   * `supported-boot-mode` - The boot mode (`legacy-bios` \| `uefi`).
    #
    #   * `supported-root-device-type` - The root device type (`ebs` \|
    #     `instance-store`).
    #
    #   * `supported-usage-class` - The usage class (`on-demand` \| `spot` \|
    #     `capacity-block`).
    #
    #   * `supported-virtualization-type` - The virtualization type (`hvm` \|
    #     `paravirtual`).
    #
    #   * `vcpu-info.default-cores` - The default number of cores for the
    #     instance type.
    #
    #   * `vcpu-info.default-threads-per-core` - The default number of threads
    #     per core for the instance type.
    #
    #   * `vcpu-info.default-vcpus` - The default number of vCPUs for the
    #     instance type.
    #
    #   * `vcpu-info.valid-cores` - The number of cores that can be configured
    #     for the instance type.
    #
    #   * `vcpu-info.valid-threads-per-core` - The number of threads per core
    #     that can be configured for the instance type. For example, "1" or
    #     "1,2".
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @return [Types::DescribeInstanceTypesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeInstanceTypesResult#instance_types #instance_types} => Array&lt;Types::InstanceTypeInfo&gt;
    #   * {Types::DescribeInstanceTypesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_instance_types({
    #     dry_run: false,
    #     instance_types: ["a1.medium"], # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_types #=> Array
    #   resp.instance_types[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.instance_types[0].current_generation #=> Boolean
    #   resp.instance_types[0].free_tier_eligible #=> Boolean
    #   resp.instance_types[0].supported_usage_classes #=> Array
    #   resp.instance_types[0].supported_usage_classes[0] #=> String, one of "spot", "on-demand", "capacity-block"
    #   resp.instance_types[0].supported_root_device_types #=> Array
    #   resp.instance_types[0].supported_root_device_types[0] #=> String, one of "ebs", "instance-store"
    #   resp.instance_types[0].supported_virtualization_types #=> Array
    #   resp.instance_types[0].supported_virtualization_types[0] #=> String, one of "hvm", "paravirtual"
    #   resp.instance_types[0].bare_metal #=> Boolean
    #   resp.instance_types[0].hypervisor #=> String, one of "nitro", "xen"
    #   resp.instance_types[0].processor_info.supported_architectures #=> Array
    #   resp.instance_types[0].processor_info.supported_architectures[0] #=> String, one of "i386", "x86_64", "arm64", "x86_64_mac", "arm64_mac"
    #   resp.instance_types[0].processor_info.sustained_clock_speed_in_ghz #=> Float
    #   resp.instance_types[0].processor_info.supported_features #=> Array
    #   resp.instance_types[0].processor_info.supported_features[0] #=> String, one of "amd-sev-snp"
    #   resp.instance_types[0].processor_info.manufacturer #=> String
    #   resp.instance_types[0].v_cpu_info.default_v_cpus #=> Integer
    #   resp.instance_types[0].v_cpu_info.default_cores #=> Integer
    #   resp.instance_types[0].v_cpu_info.default_threads_per_core #=> Integer
    #   resp.instance_types[0].v_cpu_info.valid_cores #=> Array
    #   resp.instance_types[0].v_cpu_info.valid_cores[0] #=> Integer
    #   resp.instance_types[0].v_cpu_info.valid_threads_per_core #=> Array
    #   resp.instance_types[0].v_cpu_info.valid_threads_per_core[0] #=> Integer
    #   resp.instance_types[0].memory_info.size_in_mi_b #=> Integer
    #   resp.instance_types[0].instance_storage_supported #=> Boolean
    #   resp.instance_types[0].instance_storage_info.total_size_in_gb #=> Integer
    #   resp.instance_types[0].instance_storage_info.disks #=> Array
    #   resp.instance_types[0].instance_storage_info.disks[0].size_in_gb #=> Integer
    #   resp.instance_types[0].instance_storage_info.disks[0].count #=> Integer
    #   resp.instance_types[0].instance_storage_info.disks[0].type #=> String, one of "hdd", "ssd"
    #   resp.instance_types[0].instance_storage_info.nvme_support #=> String, one of "unsupported", "supported", "required"
    #   resp.instance_types[0].instance_storage_info.encryption_support #=> String, one of "unsupported", "required"
    #   resp.instance_types[0].ebs_info.ebs_optimized_support #=> String, one of "unsupported", "supported", "default"
    #   resp.instance_types[0].ebs_info.encryption_support #=> String, one of "unsupported", "supported"
    #   resp.instance_types[0].ebs_info.ebs_optimized_info.baseline_bandwidth_in_mbps #=> Integer
    #   resp.instance_types[0].ebs_info.ebs_optimized_info.baseline_throughput_in_m_bps #=> Float
    #   resp.instance_types[0].ebs_info.ebs_optimized_info.baseline_iops #=> Integer
    #   resp.instance_types[0].ebs_info.ebs_optimized_info.maximum_bandwidth_in_mbps #=> Integer
    #   resp.instance_types[0].ebs_info.ebs_optimized_info.maximum_throughput_in_m_bps #=> Float
    #   resp.instance_types[0].ebs_info.ebs_optimized_info.maximum_iops #=> Integer
    #   resp.instance_types[0].ebs_info.nvme_support #=> String, one of "unsupported", "supported", "required"
    #   resp.instance_types[0].network_info.network_performance #=> String
    #   resp.instance_types[0].network_info.maximum_network_interfaces #=> Integer
    #   resp.instance_types[0].network_info.maximum_network_cards #=> Integer
    #   resp.instance_types[0].network_info.default_network_card_index #=> Integer
    #   resp.instance_types[0].network_info.network_cards #=> Array
    #   resp.instance_types[0].network_info.network_cards[0].network_card_index #=> Integer
    #   resp.instance_types[0].network_info.network_cards[0].network_performance #=> String
    #   resp.instance_types[0].network_info.network_cards[0].maximum_network_interfaces #=> Integer
    #   resp.instance_types[0].network_info.network_cards[0].baseline_bandwidth_in_gbps #=> Float
    #   resp.instance_types[0].network_info.network_cards[0].peak_bandwidth_in_gbps #=> Float
    #   resp.instance_types[0].network_info.ipv_4_addresses_per_interface #=> Integer
    #   resp.instance_types[0].network_info.ipv_6_addresses_per_interface #=> Integer
    #   resp.instance_types[0].network_info.ipv_6_supported #=> Boolean
    #   resp.instance_types[0].network_info.ena_support #=> String, one of "unsupported", "supported", "required"
    #   resp.instance_types[0].network_info.efa_supported #=> Boolean
    #   resp.instance_types[0].network_info.efa_info.maximum_efa_interfaces #=> Integer
    #   resp.instance_types[0].network_info.encryption_in_transit_supported #=> Boolean
    #   resp.instance_types[0].network_info.ena_srd_supported #=> Boolean
    #   resp.instance_types[0].network_info.bandwidth_weightings #=> Array
    #   resp.instance_types[0].network_info.bandwidth_weightings[0] #=> String, one of "default", "vpc-1", "ebs-1"
    #   resp.instance_types[0].gpu_info.gpus #=> Array
    #   resp.instance_types[0].gpu_info.gpus[0].name #=> String
    #   resp.instance_types[0].gpu_info.gpus[0].manufacturer #=> String
    #   resp.instance_types[0].gpu_info.gpus[0].count #=> Integer
    #   resp.instance_types[0].gpu_info.gpus[0].memory_info.size_in_mi_b #=> Integer
    #   resp.instance_types[0].gpu_info.total_gpu_memory_in_mi_b #=> Integer
    #   resp.instance_types[0].fpga_info.fpgas #=> Array
    #   resp.instance_types[0].fpga_info.fpgas[0].name #=> String
    #   resp.instance_types[0].fpga_info.fpgas[0].manufacturer #=> String
    #   resp.instance_types[0].fpga_info.fpgas[0].count #=> Integer
    #   resp.instance_types[0].fpga_info.fpgas[0].memory_info.size_in_mi_b #=> Integer
    #   resp.instance_types[0].fpga_info.total_fpga_memory_in_mi_b #=> Integer
    #   resp.instance_types[0].placement_group_info.supported_strategies #=> Array
    #   resp.instance_types[0].placement_group_info.supported_strategies[0] #=> String, one of "cluster", "partition", "spread"
    #   resp.instance_types[0].inference_accelerator_info.accelerators #=> Array
    #   resp.instance_types[0].inference_accelerator_info.accelerators[0].count #=> Integer
    #   resp.instance_types[0].inference_accelerator_info.accelerators[0].name #=> String
    #   resp.instance_types[0].inference_accelerator_info.accelerators[0].manufacturer #=> String
    #   resp.instance_types[0].inference_accelerator_info.accelerators[0].memory_info.size_in_mi_b #=> Integer
    #   resp.instance_types[0].inference_accelerator_info.total_inference_memory_in_mi_b #=> Integer
    #   resp.instance_types[0].hibernation_supported #=> Boolean
    #   resp.instance_types[0].burstable_performance_supported #=> Boolean
    #   resp.instance_types[0].dedicated_hosts_supported #=> Boolean
    #   resp.instance_types[0].auto_recovery_supported #=> Boolean
    #   resp.instance_types[0].supported_boot_modes #=> Array
    #   resp.instance_types[0].supported_boot_modes[0] #=> String, one of "legacy-bios", "uefi"
    #   resp.instance_types[0].nitro_enclaves_support #=> String, one of "unsupported", "supported"
    #   resp.instance_types[0].nitro_tpm_support #=> String, one of "unsupported", "supported"
    #   resp.instance_types[0].nitro_tpm_info.supported_versions #=> Array
    #   resp.instance_types[0].nitro_tpm_info.supported_versions[0] #=> String
    #   resp.instance_types[0].media_accelerator_info.accelerators #=> Array
    #   resp.instance_types[0].media_accelerator_info.accelerators[0].count #=> Integer
    #   resp.instance_types[0].media_accelerator_info.accelerators[0].name #=> String
    #   resp.instance_types[0].media_accelerator_info.accelerators[0].manufacturer #=> String
    #   resp.instance_types[0].media_accelerator_info.accelerators[0].memory_info.size_in_mi_b #=> Integer
    #   resp.instance_types[0].media_accelerator_info.total_media_memory_in_mi_b #=> Integer
    #   resp.instance_types[0].neuron_info.neuron_devices #=> Array
    #   resp.instance_types[0].neuron_info.neuron_devices[0].count #=> Integer
    #   resp.instance_types[0].neuron_info.neuron_devices[0].name #=> String
    #   resp.instance_types[0].neuron_info.neuron_devices[0].core_info.count #=> Integer
    #   resp.instance_types[0].neuron_info.neuron_devices[0].core_info.version #=> Integer
    #   resp.instance_types[0].neuron_info.neuron_devices[0].memory_info.size_in_mi_b #=> Integer
    #   resp.instance_types[0].neuron_info.total_neuron_device_memory_in_mi_b #=> Integer
    #   resp.instance_types[0].phc_support #=> String, one of "unsupported", "supported"
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeInstanceTypes AWS API Documentation
    #
    # @overload describe_instance_types(params = {})
    # @param [Hash] params ({})
    def describe_instance_types(params = {}, options = {})
      req = build_request(:describe_instance_types, params)
      req.send_request(options)
    end

    # Describes the specified instances or all instances.
    #
    # If you specify instance IDs, the output includes information for only
    # the specified instances. If you specify filters, the output includes
    # information for only those instances that meet the filter criteria. If
    # you do not specify instance IDs or filters, the output includes
    # information for all instances, which can affect performance. We
    # recommend that you use pagination to ensure that the operation returns
    # quickly and successfully.
    #
    # If you specify an instance ID that is not valid, an error is returned.
    # If you specify an instance that you do not own, it is not included in
    # the output.
    #
    # Recently terminated instances might appear in the returned results.
    # This interval is usually less than one hour.
    #
    # If you describe instances in the rare case where an Availability Zone
    # is experiencing a service disruption and you specify instance IDs that
    # are in the affected zone, or do not specify any instance IDs at all,
    # the call fails. If you describe instances and specify only instance
    # IDs that are in an unaffected zone, the call works normally.
    #
    # We strongly recommend using only paginated requests. Unpaginated
    # requests are susceptible to throttling and timeouts.
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    # @option params [Array<String>] :instance_ids
    #   The instance IDs.
    #
    #   Default: Describes all your instances.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `affinity` - The affinity setting for an instance running on a
    #     Dedicated Host (`default` \| `host`).
    #
    #   * `architecture` - The instance architecture (`i386` \| `x86_64` \|
    #     `arm64`).
    #
    #   * `availability-zone` - The Availability Zone of the instance.
    #
    #   * `block-device-mapping.attach-time` - The attach time for an EBS
    #     volume mapped to the instance, for example,
    #     `2022-09-15T17:15:20.000Z`.
    #
    #   * `block-device-mapping.delete-on-termination` - A Boolean that
    #     indicates whether the EBS volume is deleted on instance termination.
    #
    #   * `block-device-mapping.device-name` - The device name specified in
    #     the block device mapping (for example, `/dev/sdh` or `xvdh`).
    #
    #   * `block-device-mapping.status` - The status for the EBS volume
    #     (`attaching` \| `attached` \| `detaching` \| `detached`).
    #
    #   * `block-device-mapping.volume-id` - The volume ID of the EBS volume.
    #
    #   * `boot-mode` - The boot mode that was specified by the AMI
    #     (`legacy-bios` \| `uefi` \| `uefi-preferred`).
    #
    #   * `capacity-reservation-id` - The ID of the Capacity Reservation into
    #     which the instance was launched.
    #
    #   * `capacity-reservation-specification.capacity-reservation-preference`
    #     - The instance's Capacity Reservation preference (`open` \|
    #     `none`).
    #
    #   * `capacity-reservation-specification.capacity-reservation-target.capacity-reservation-id`
    #     - The ID of the targeted Capacity Reservation.
    #
    #   * `capacity-reservation-specification.capacity-reservation-target.capacity-reservation-resource-group-arn`
    #     - The ARN of the targeted Capacity Reservation group.
    #
    #   * `client-token` - The idempotency token you provided when you
    #     launched the instance.
    #
    #   * `current-instance-boot-mode` - The boot mode that is used to launch
    #     the instance at launch or start (`legacy-bios` \| `uefi`).
    #
    #   * `dns-name` - The public DNS name of the instance.
    #
    #   * `ebs-optimized` - A Boolean that indicates whether the instance is
    #     optimized for Amazon EBS I/O.
    #
    #   * `ena-support` - A Boolean that indicates whether the instance is
    #     enabled for enhanced networking with ENA.
    #
    #   * `enclave-options.enabled` - A Boolean that indicates whether the
    #     instance is enabled for Amazon Web Services Nitro Enclaves.
    #
    #   * `hibernation-options.configured` - A Boolean that indicates whether
    #     the instance is enabled for hibernation. A value of `true` means
    #     that the instance is enabled for hibernation.
    #
    #   * `host-id` - The ID of the Dedicated Host on which the instance is
    #     running, if applicable.
    #
    #   * `hypervisor` - The hypervisor type of the instance (`ovm` \| `xen`).
    #     The value `xen` is used for both Xen and Nitro hypervisors.
    #
    #   * `iam-instance-profile.arn` - The instance profile associated with
    #     the instance. Specified as an ARN.
    #
    #   * `iam-instance-profile.id` - The instance profile associated with the
    #     instance. Specified as an ID.
    #
    #   * `image-id` - The ID of the image used to launch the instance.
    #
    #   * `instance-id` - The ID of the instance.
    #
    #   * `instance-lifecycle` - Indicates whether this is a Spot Instance, a
    #     Scheduled Instance, or a Capacity Block (`spot` \| `scheduled` \|
    #     `capacity-block`).
    #
    #   * `instance-state-code` - The state of the instance, as a 16-bit
    #     unsigned integer. The high byte is used for internal purposes and
    #     should be ignored. The low byte is set based on the state
    #     represented. The valid values are: 0 (pending), 16 (running), 32
    #     (shutting-down), 48 (terminated), 64 (stopping), and 80 (stopped).
    #
    #   * `instance-state-name` - The state of the instance (`pending` \|
    #     `running` \| `shutting-down` \| `terminated` \| `stopping` \|
    #     `stopped`).
    #
    #   * `instance-type` - The type of instance (for example, `t2.micro`).
    #
    #   * `instance.group-id` - The ID of the security group for the instance.
    #
    #   * `instance.group-name` - The name of the security group for the
    #     instance.
    #
    #   * `ip-address` - The public IPv4 address of the instance.
    #
    #   * `ipv6-address` - The IPv6 address of the instance.
    #
    #   * `kernel-id` - The kernel ID.
    #
    #   * `key-name` - The name of the key pair used when the instance was
    #     launched.
    #
    #   * `launch-index` - When launching multiple instances, this is the
    #     index for the instance in the launch group (for example, 0, 1, 2,
    #     and so on).
    #
    #   * `launch-time` - The time when the instance was launched, in the ISO
    #     8601 format in the UTC time zone (YYYY-MM-DDThh:mm:ss.sssZ), for
    #     example, `2021-09-29T11:04:43.305Z`. You can use a wildcard (`*`),
    #     for example, `2021-09-29T*`, which matches an entire day.
    #
    #   * `maintenance-options.auto-recovery` - The current automatic recovery
    #     behavior of the instance (`disabled` \| `default`).
    #
    #   * `metadata-options.http-endpoint` - The status of access to the HTTP
    #     metadata endpoint on your instance (`enabled` \| `disabled`)
    #
    #   * `metadata-options.http-protocol-ipv4` - Indicates whether the IPv4
    #     endpoint is enabled (`disabled` \| `enabled`).
    #
    #   * `metadata-options.http-protocol-ipv6` - Indicates whether the IPv6
    #     endpoint is enabled (`disabled` \| `enabled`).
    #
    #   * `metadata-options.http-put-response-hop-limit` - The HTTP metadata
    #     request put response hop limit (integer, possible values `1` to
    #     `64`)
    #
    #   * `metadata-options.http-tokens` - The metadata request authorization
    #     state (`optional` \| `required`)
    #
    #   * `metadata-options.instance-metadata-tags` - The status of access to
    #     instance tags from the instance metadata (`enabled` \| `disabled`)
    #
    #   * `metadata-options.state` - The state of the metadata option changes
    #     (`pending` \| `applied`).
    #
    #   * `monitoring-state` - Indicates whether detailed monitoring is
    #     enabled (`disabled` \| `enabled`).
    #
    #   * `network-interface.addresses.association.allocation-id` - The
    #     allocation ID.
    #
    #   * `network-interface.addresses.association.association-id` - The
    #     association ID.
    #
    #   * `network-interface.addresses.association.carrier-ip` - The carrier
    #     IP address.
    #
    #   * `network-interface.addresses.association.customer-owned-ip` - The
    #     customer-owned IP address.
    #
    #   * `network-interface.addresses.association.ip-owner-id` - The owner ID
    #     of the private IPv4 address associated with the network interface.
    #
    #   * `network-interface.addresses.association.public-dns-name` - The
    #     public DNS name.
    #
    #   * `network-interface.addresses.association.public-ip` - The ID of the
    #     association of an Elastic IP address (IPv4) with a network
    #     interface.
    #
    #   * `network-interface.addresses.primary` - Specifies whether the IPv4
    #     address of the network interface is the primary private IPv4
    #     address.
    #
    #   * `network-interface.addresses.private-dns-name` - The private DNS
    #     name.
    #
    #   * `network-interface.addresses.private-ip-address` - The private IPv4
    #     address associated with the network interface.
    #
    #   * `network-interface.association.allocation-id` - The allocation ID
    #     returned when you allocated the Elastic IP address (IPv4) for your
    #     network interface.
    #
    #   * `network-interface.association.association-id` - The association ID
    #     returned when the network interface was associated with an IPv4
    #     address.
    #
    #   * `network-interface.association.carrier-ip` - The customer-owned IP
    #     address.
    #
    #   * `network-interface.association.customer-owned-ip` - The
    #     customer-owned IP address.
    #
    #   * `network-interface.association.ip-owner-id` - The owner of the
    #     Elastic IP address (IPv4) associated with the network interface.
    #
    #   * `network-interface.association.public-dns-name` - The public DNS
    #     name.
    #
    #   * `network-interface.association.public-ip` - The address of the
    #     Elastic IP address (IPv4) bound to the network interface.
    #
    #   * `network-interface.attachment.attach-time` - The time that the
    #     network interface was attached to an instance.
    #
    #   * `network-interface.attachment.attachment-id` - The ID of the
    #     interface attachment.
    #
    #   * `network-interface.attachment.delete-on-termination` - Specifies
    #     whether the attachment is deleted when an instance is terminated.
    #
    #   * `network-interface.attachment.device-index` - The device index to
    #     which the network interface is attached.
    #
    #   * `network-interface.attachment.instance-id` - The ID of the instance
    #     to which the network interface is attached.
    #
    #   * `network-interface.attachment.instance-owner-id` - The owner ID of
    #     the instance to which the network interface is attached.
    #
    #   * `network-interface.attachment.network-card-index` - The index of the
    #     network card.
    #
    #   * `network-interface.attachment.status` - The status of the attachment
    #     (`attaching` \| `attached` \| `detaching` \| `detached`).
    #
    #   * `network-interface.availability-zone` - The Availability Zone for
    #     the network interface.
    #
    #   * `network-interface.deny-all-igw-traffic` - A Boolean that indicates
    #     whether a network interface with an IPv6 address is unreachable from
    #     the public internet.
    #
    #   * `network-interface.description` - The description of the network
    #     interface.
    #
    #   * `network-interface.group-id` - The ID of a security group associated
    #     with the network interface.
    #
    #   * `network-interface.group-name` - The name of a security group
    #     associated with the network interface.
    #
    #   * `network-interface.ipv4-prefixes.ipv4-prefix` - The IPv4 prefixes
    #     that are assigned to the network interface.
    #
    #   * `network-interface.ipv6-address` - The IPv6 address associated with
    #     the network interface.
    #
    #   * `network-interface.ipv6-addresses.ipv6-address` - The IPv6 address
    #     associated with the network interface.
    #
    #   * `network-interface.ipv6-addresses.is-primary-ipv6` - A Boolean that
    #     indicates whether this is the primary IPv6 address.
    #
    #   * `network-interface.ipv6-native` - A Boolean that indicates whether
    #     this is an IPv6 only network interface.
    #
    #   * `network-interface.ipv6-prefixes.ipv6-prefix` - The IPv6 prefix
    #     assigned to the network interface.
    #
    #   * `network-interface.mac-address` - The MAC address of the network
    #     interface.
    #
    #   * `network-interface.network-interface-id` - The ID of the network
    #     interface.
    #
    #   * `network-interface.operator.managed` - A Boolean that indicates
    #     whether the instance has a managed network interface.
    #
    #   * `network-interface.operator.principal` - The principal that manages
    #     the network interface. Only valid for instances with managed network
    #     interfaces, where `managed` is `true`.
    #
    #   * `network-interface.outpost-arn` - The ARN of the Outpost.
    #
    #   * `network-interface.owner-id` - The ID of the owner of the network
    #     interface.
    #
    #   * `network-interface.private-dns-name` - The private DNS name of the
    #     network interface.
    #
    #   * `network-interface.private-ip-address` - The private IPv4 address.
    #
    #   * `network-interface.public-dns-name` - The public DNS name.
    #
    #   * `network-interface.requester-id` - The requester ID for the network
    #     interface.
    #
    #   * `network-interface.requester-managed` - Indicates whether the
    #     network interface is being managed by Amazon Web Services.
    #
    #   * `network-interface.status` - The status of the network interface
    #     (`available`) \| `in-use`).
    #
    #   * `network-interface.source-dest-check` - Whether the network
    #     interface performs source/destination checking. A value of `true`
    #     means that checking is enabled, and `false` means that checking is
    #     disabled. The value must be `false` for the network interface to
    #     perform network address translation (NAT) in your VPC.
    #
    #   * `network-interface.subnet-id` - The ID of the subnet for the network
    #     interface.
    #
    #   * `network-interface.tag-key` - The key of a tag assigned to the
    #     network interface.
    #
    #   * `network-interface.tag-value` - The value of a tag assigned to the
    #     network interface.
    #
    #   * `network-interface.vpc-id` - The ID of the VPC for the network
    #     interface.
    #
    #   * `network-performance-options.bandwidth-weighting` - Where the
    #     performance boost is applied, if applicable. Valid values:
    #     `default`, `vpc-1`, `ebs-1`.
    #
    #   * `operator.managed` - A Boolean that indicates whether this is a
    #     managed instance.
    #
    #   * `operator.principal` - The principal that manages the instance. Only
    #     valid for managed instances, where `managed` is `true`.
    #
    #   * `outpost-arn` - The Amazon Resource Name (ARN) of the Outpost.
    #
    #   * `owner-id` - The Amazon Web Services account ID of the instance
    #     owner.
    #
    #   * `placement-group-name` - The name of the placement group for the
    #     instance.
    #
    #   * `placement-partition-number` - The partition in which the instance
    #     is located.
    #
    #   * `platform` - The platform. To list only Windows instances, use
    #     `windows`.
    #
    #   * `platform-details` - The platform (`Linux/UNIX` \| `Red Hat BYOL
    #     Linux` \| ` Red Hat Enterprise Linux` \| `Red Hat Enterprise Linux
    #     with HA` \| `Red Hat Enterprise Linux with SQL Server Standard and
    #     HA` \| `Red Hat Enterprise Linux with SQL Server Enterprise and HA`
    #     \| `Red Hat Enterprise Linux with SQL Server Standard` \| `Red Hat
    #     Enterprise Linux with SQL Server Web` \| `Red Hat Enterprise Linux
    #     with SQL Server Enterprise` \| `SQL Server Enterprise` \| `SQL
    #     Server Standard` \| `SQL Server Web` \| `SUSE Linux` \| `Ubuntu Pro`
    #     \| `Windows` \| `Windows BYOL` \| `Windows with SQL Server
    #     Enterprise` \| `Windows with SQL Server Standard` \| `Windows with
    #     SQL Server Web`).
    #
    #   * `private-dns-name` - The private IPv4 DNS name of the instance.
    #
    #   * `private-dns-name-options.enable-resource-name-dns-a-record` - A
    #     Boolean that indicates whether to respond to DNS queries for
    #     instance hostnames with DNS A records.
    #
    #   * `private-dns-name-options.enable-resource-name-dns-aaaa-record` - A
    #     Boolean that indicates whether to respond to DNS queries for
    #     instance hostnames with DNS AAAA records.
    #
    #   * `private-dns-name-options.hostname-type` - The type of hostname
    #     (`ip-name` \| `resource-name`).
    #
    #   * `private-ip-address` - The private IPv4 address of the instance.
    #     This can only be used to filter by the primary IP address of the
    #     network interface attached to the instance. To filter by additional
    #     IP addresses assigned to the network interface, use the filter
    #     `network-interface.addresses.private-ip-address`.
    #
    #   * `product-code` - The product code associated with the AMI used to
    #     launch the instance.
    #
    #   * `product-code.type` - The type of product code (`devpay` \|
    #     `marketplace`).
    #
    #   * `ramdisk-id` - The RAM disk ID.
    #
    #   * `reason` - The reason for the current state of the instance (for
    #     example, shows "User Initiated \[date\]" when you stop or
    #     terminate the instance). Similar to the state-reason-code filter.
    #
    #   * `requester-id` - The ID of the entity that launched the instance on
    #     your behalf (for example, Amazon Web Services Management Console,
    #     Auto Scaling, and so on).
    #
    #   * `reservation-id` - The ID of the instance's reservation. A
    #     reservation ID is created any time you launch an instance. A
    #     reservation ID has a one-to-one relationship with an instance launch
    #     request, but can be associated with more than one instance if you
    #     launch multiple instances using the same launch request. For
    #     example, if you launch one instance, you get one reservation ID. If
    #     you launch ten instances using the same launch request, you also get
    #     one reservation ID.
    #
    #   * `root-device-name` - The device name of the root device volume (for
    #     example, `/dev/sda1`).
    #
    #   * `root-device-type` - The type of the root device volume (`ebs` \|
    #     `instance-store`).
    #
    #   * `source-dest-check` - Indicates whether the instance performs
    #     source/destination checking. A value of `true` means that checking
    #     is enabled, and `false` means that checking is disabled. The value
    #     must be `false` for the instance to perform network address
    #     translation (NAT) in your VPC.
    #
    #   * `spot-instance-request-id` - The ID of the Spot Instance request.
    #
    #   * `state-reason-code` - The reason code for the state change.
    #
    #   * `state-reason-message` - A message that describes the state change.
    #
    #   * `subnet-id` - The ID of the subnet for the instance.
    #
    #   * `tag:<key>` - The key/value combination of a tag assigned to the
    #     resource. Use the tag key in the filter name and the tag value as
    #     the filter value. For example, to find all resources that have a tag
    #     with the key `Owner` and the value `TeamA`, specify `tag:Owner` for
    #     the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources that have a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `tenancy` - The tenancy of an instance (`dedicated` \| `default` \|
    #     `host`).
    #
    #   * `tpm-support` - Indicates if the instance is configured for NitroTPM
    #     support (`v2.0`).
    #
    #   * `usage-operation` - The usage operation value for the instance
    #     (`RunInstances` \| `RunInstances:00g0` \| `RunInstances:0010` \|
    #     `RunInstances:1010` \| `RunInstances:1014` \| `RunInstances:1110` \|
    #     `RunInstances:0014` \| `RunInstances:0210` \| `RunInstances:0110` \|
    #     `RunInstances:0100` \| `RunInstances:0004` \| `RunInstances:0200` \|
    #     `RunInstances:000g` \| `RunInstances:0g00` \| `RunInstances:0002` \|
    #     `RunInstances:0800` \| `RunInstances:0102` \| `RunInstances:0006` \|
    #     `RunInstances:0202`).
    #
    #   * `usage-operation-update-time` - The time that the usage operation
    #     was last updated, for example, `2022-09-15T17:15:20.000Z`.
    #
    #   * `virtualization-type` - The virtualization type of the instance
    #     (`paravirtual` \| `hvm`).
    #
    #   * `vpc-id` - The ID of the VPC that the instance is running in.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #   You cannot specify this parameter and the instance IDs parameter in
    #   the same request.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @return [Types::DescribeInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeInstancesResult#next_token #next_token} => String
    #   * {Types::DescribeInstancesResult#reservations #reservations} => Array&lt;Types::Reservation&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe an Amazon EC2 instance
    #
    #   # This example describes the specified instance.
    #
    #   resp = client.describe_instances({
    #     instance_ids: [
    #       "i-1234567890abcdef0", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Example: To describe the instances with a specific instance type
    #
    #   # This example describes the instances with the t2.micro instance type.
    #
    #   resp = client.describe_instances({
    #     filters: [
    #       {
    #         name: "instance-type", 
    #         values: [
    #           "t2.micro", 
    #         ], 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Example: To describe the instances with a specific tag
    #
    #   # This example describes the instances with the Purpose=test tag.
    #
    #   resp = client.describe_instances({
    #     filters: [
    #       {
    #         name: "tag:Purpose", 
    #         values: [
    #           "test", 
    #         ], 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Filtering by tags examples
    #   # filtering by tag keys "key1" or "key2"
    #   ec2.describe_instances(filters:[{ name: 'tag-key', values: ['key1', 'key2'] }])
    #
    #   # filtering by tag values "value1" or "value1"
    #   ec2.describe_instances(filters:[{ name: 'tag-value', values: ['value1', 'value2'] }])
    #
    #   # filtering by key and value, key equals "key1" and value equals "value1" or "value2"
    #   ec2.describe_instances(filters:[{ name: "tag:key1", values: ['value1', 'value2'] }])
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_instances({
    #     instance_ids: ["InstanceId"],
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.reservations #=> Array
    #   resp.reservations[0].reservation_id #=> String
    #   resp.reservations[0].owner_id #=> String
    #   resp.reservations[0].requester_id #=> String
    #   resp.reservations[0].groups #=> Array
    #   resp.reservations[0].groups[0].group_id #=> String
    #   resp.reservations[0].groups[0].group_name #=> String
    #   resp.reservations[0].instances #=> Array
    #   resp.reservations[0].instances[0].architecture #=> String, one of "i386", "x86_64", "arm64", "x86_64_mac", "arm64_mac"
    #   resp.reservations[0].instances[0].block_device_mappings #=> Array
    #   resp.reservations[0].instances[0].block_device_mappings[0].device_name #=> String
    #   resp.reservations[0].instances[0].block_device_mappings[0].ebs.attach_time #=> Time
    #   resp.reservations[0].instances[0].block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.reservations[0].instances[0].block_device_mappings[0].ebs.status #=> String, one of "attaching", "attached", "detaching", "detached"
    #   resp.reservations[0].instances[0].block_device_mappings[0].ebs.volume_id #=> String
    #   resp.reservations[0].instances[0].block_device_mappings[0].ebs.associated_resource #=> String
    #   resp.reservations[0].instances[0].block_device_mappings[0].ebs.volume_owner_id #=> String
    #   resp.reservations[0].instances[0].block_device_mappings[0].ebs.operator.managed #=> Boolean
    #   resp.reservations[0].instances[0].block_device_mappings[0].ebs.operator.principal #=> String
    #   resp.reservations[0].instances[0].client_token #=> String
    #   resp.reservations[0].instances[0].ebs_optimized #=> Boolean
    #   resp.reservations[0].instances[0].ena_support #=> Boolean
    #   resp.reservations[0].instances[0].hypervisor #=> String, one of "ovm", "xen"
    #   resp.reservations[0].instances[0].iam_instance_profile.arn #=> String
    #   resp.reservations[0].instances[0].iam_instance_profile.id #=> String
    #   resp.reservations[0].instances[0].instance_lifecycle #=> String, one of "spot", "scheduled", "capacity-block"
    #   resp.reservations[0].instances[0].elastic_gpu_associations #=> Array
    #   resp.reservations[0].instances[0].elastic_gpu_associations[0].elastic_gpu_id #=> String
    #   resp.reservations[0].instances[0].elastic_gpu_associations[0].elastic_gpu_association_id #=> String
    #   resp.reservations[0].instances[0].elastic_gpu_associations[0].elastic_gpu_association_state #=> String
    #   resp.reservations[0].instances[0].elastic_gpu_associations[0].elastic_gpu_association_time #=> String
    #   resp.reservations[0].instances[0].elastic_inference_accelerator_associations #=> Array
    #   resp.reservations[0].instances[0].elastic_inference_accelerator_associations[0].elastic_inference_accelerator_arn #=> String
    #   resp.reservations[0].instances[0].elastic_inference_accelerator_associations[0].elastic_inference_accelerator_association_id #=> String
    #   resp.reservations[0].instances[0].elastic_inference_accelerator_associations[0].elastic_inference_accelerator_association_state #=> String
    #   resp.reservations[0].instances[0].elastic_inference_accelerator_associations[0].elastic_inference_accelerator_association_time #=> Time
    #   resp.reservations[0].instances[0].network_interfaces #=> Array
    #   resp.reservations[0].instances[0].network_interfaces[0].association.carrier_ip #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].association.customer_owned_ip #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].association.ip_owner_id #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].association.public_dns_name #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].association.public_ip #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].attachment.attach_time #=> Time
    #   resp.reservations[0].instances[0].network_interfaces[0].attachment.attachment_id #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].attachment.delete_on_termination #=> Boolean
    #   resp.reservations[0].instances[0].network_interfaces[0].attachment.device_index #=> Integer
    #   resp.reservations[0].instances[0].network_interfaces[0].attachment.status #=> String, one of "attaching", "attached", "detaching", "detached"
    #   resp.reservations[0].instances[0].network_interfaces[0].attachment.network_card_index #=> Integer
    #   resp.reservations[0].instances[0].network_interfaces[0].attachment.ena_srd_specification.ena_srd_enabled #=> Boolean
    #   resp.reservations[0].instances[0].network_interfaces[0].attachment.ena_srd_specification.ena_srd_udp_specification.ena_srd_udp_enabled #=> Boolean
    #   resp.reservations[0].instances[0].network_interfaces[0].description #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].groups #=> Array
    #   resp.reservations[0].instances[0].network_interfaces[0].groups[0].group_id #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].groups[0].group_name #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].ipv_6_addresses #=> Array
    #   resp.reservations[0].instances[0].network_interfaces[0].ipv_6_addresses[0].ipv_6_address #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].ipv_6_addresses[0].is_primary_ipv_6 #=> Boolean
    #   resp.reservations[0].instances[0].network_interfaces[0].mac_address #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].network_interface_id #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].owner_id #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].private_dns_name #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].private_ip_address #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].private_ip_addresses #=> Array
    #   resp.reservations[0].instances[0].network_interfaces[0].private_ip_addresses[0].association.carrier_ip #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].private_ip_addresses[0].association.customer_owned_ip #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].private_ip_addresses[0].association.ip_owner_id #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].private_ip_addresses[0].association.public_dns_name #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].private_ip_addresses[0].association.public_ip #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].private_ip_addresses[0].primary #=> Boolean
    #   resp.reservations[0].instances[0].network_interfaces[0].private_ip_addresses[0].private_dns_name #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].private_ip_addresses[0].private_ip_address #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].source_dest_check #=> Boolean
    #   resp.reservations[0].instances[0].network_interfaces[0].status #=> String, one of "available", "associated", "attaching", "in-use", "detaching"
    #   resp.reservations[0].instances[0].network_interfaces[0].subnet_id #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].vpc_id #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].interface_type #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].ipv_4_prefixes #=> Array
    #   resp.reservations[0].instances[0].network_interfaces[0].ipv_4_prefixes[0].ipv_4_prefix #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].ipv_6_prefixes #=> Array
    #   resp.reservations[0].instances[0].network_interfaces[0].ipv_6_prefixes[0].ipv_6_prefix #=> String
    #   resp.reservations[0].instances[0].network_interfaces[0].connection_tracking_configuration.tcp_established_timeout #=> Integer
    #   resp.reservations[0].instances[0].network_interfaces[0].connection_tracking_configuration.udp_stream_timeout #=> Integer
    #   resp.reservations[0].instances[0].network_interfaces[0].connection_tracking_configuration.udp_timeout #=> Integer
    #   resp.reservations[0].instances[0].network_interfaces[0].operator.managed #=> Boolean
    #   resp.reservations[0].instances[0].network_interfaces[0].operator.principal #=> String
    #   resp.reservations[0].instances[0].outpost_arn #=> String
    #   resp.reservations[0].instances[0].root_device_name #=> String
    #   resp.reservations[0].instances[0].root_device_type #=> String, one of "ebs", "instance-store"
    #   resp.reservations[0].instances[0].security_groups #=> Array
    #   resp.reservations[0].instances[0].security_groups[0].group_id #=> String
    #   resp.reservations[0].instances[0].security_groups[0].group_name #=> String
    #   resp.reservations[0].instances[0].source_dest_check #=> Boolean
    #   resp.reservations[0].instances[0].spot_instance_request_id #=> String
    #   resp.reservations[0].instances[0].sriov_net_support #=> String
    #   resp.reservations[0].instances[0].state_reason.code #=> String
    #   resp.reservations[0].instances[0].state_reason.message #=> String
    #   resp.reservations[0].instances[0].tags #=> Array
    #   resp.reservations[0].instances[0].tags[0].key #=> String
    #   resp.reservations[0].instances[0].tags[0].value #=> String
    #   resp.reservations[0].instances[0].virtualization_type #=> String, one of "hvm", "paravirtual"
    #   resp.reservations[0].instances[0].cpu_options.core_count #=> Integer
    #   resp.reservations[0].instances[0].cpu_options.threads_per_core #=> Integer
    #   resp.reservations[0].instances[0].cpu_options.amd_sev_snp #=> String, one of "enabled", "disabled"
    #   resp.reservations[0].instances[0].capacity_reservation_id #=> String
    #   resp.reservations[0].instances[0].capacity_reservation_specification.capacity_reservation_preference #=> String, one of "capacity-reservations-only", "open", "none"
    #   resp.reservations[0].instances[0].capacity_reservation_specification.capacity_reservation_target.capacity_reservation_id #=> String
    #   resp.reservations[0].instances[0].capacity_reservation_specification.capacity_reservation_target.capacity_reservation_resource_group_arn #=> String
    #   resp.reservations[0].instances[0].hibernation_options.configured #=> Boolean
    #   resp.reservations[0].instances[0].licenses #=> Array
    #   resp.reservations[0].instances[0].licenses[0].license_configuration_arn #=> String
    #   resp.reservations[0].instances[0].metadata_options.state #=> String, one of "pending", "applied"
    #   resp.reservations[0].instances[0].metadata_options.http_tokens #=> String, one of "optional", "required"
    #   resp.reservations[0].instances[0].metadata_options.http_put_response_hop_limit #=> Integer
    #   resp.reservations[0].instances[0].metadata_options.http_endpoint #=> String, one of "disabled", "enabled"
    #   resp.reservations[0].instances[0].metadata_options.http_protocol_ipv_6 #=> String, one of "disabled", "enabled"
    #   resp.reservations[0].instances[0].metadata_options.instance_metadata_tags #=> String, one of "disabled", "enabled"
    #   resp.reservations[0].instances[0].enclave_options.enabled #=> Boolean
    #   resp.reservations[0].instances[0].boot_mode #=> String, one of "legacy-bios", "uefi", "uefi-preferred"
    #   resp.reservations[0].instances[0].platform_details #=> String
    #   resp.reservations[0].instances[0].usage_operation #=> String
    #   resp.reservations[0].instances[0].usage_operation_update_time #=> Time
    #   resp.reservations[0].instances[0].private_dns_name_options.hostname_type #=> String, one of "ip-name", "resource-name"
    #   resp.reservations[0].instances[0].private_dns_name_options.enable_resource_name_dns_a_record #=> Boolean
    #   resp.reservations[0].instances[0].private_dns_name_options.enable_resource_name_dns_aaaa_record #=> Boolean
    #   resp.reservations[0].instances[0].ipv_6_address #=> String
    #   resp.reservations[0].instances[0].tpm_support #=> String
    #   resp.reservations[0].instances[0].maintenance_options.auto_recovery #=> String, one of "disabled", "default"
    #   resp.reservations[0].instances[0].current_instance_boot_mode #=> String, one of "legacy-bios", "uefi"
    #   resp.reservations[0].instances[0].network_performance_options.bandwidth_weighting #=> String, one of "default", "vpc-1", "ebs-1"
    #   resp.reservations[0].instances[0].operator.managed #=> Boolean
    #   resp.reservations[0].instances[0].operator.principal #=> String
    #   resp.reservations[0].instances[0].instance_id #=> String
    #   resp.reservations[0].instances[0].image_id #=> String
    #   resp.reservations[0].instances[0].state.code #=> Integer
    #   resp.reservations[0].instances[0].state.name #=> String, one of "pending", "running", "shutting-down", "terminated", "stopping", "stopped"
    #   resp.reservations[0].instances[0].private_dns_name #=> String
    #   resp.reservations[0].instances[0].public_dns_name #=> String
    #   resp.reservations[0].instances[0].state_transition_reason #=> String
    #   resp.reservations[0].instances[0].key_name #=> String
    #   resp.reservations[0].instances[0].ami_launch_index #=> Integer
    #   resp.reservations[0].instances[0].product_codes #=> Array
    #   resp.reservations[0].instances[0].product_codes[0].product_code_id #=> String
    #   resp.reservations[0].instances[0].product_codes[0].product_code_type #=> String, one of "devpay", "marketplace"
    #   resp.reservations[0].instances[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.reservations[0].instances[0].launch_time #=> Time
    #   resp.reservations[0].instances[0].placement.affinity #=> String
    #   resp.reservations[0].instances[0].placement.group_name #=> String
    #   resp.reservations[0].instances[0].placement.partition_number #=> Integer
    #   resp.reservations[0].instances[0].placement.host_id #=> String
    #   resp.reservations[0].instances[0].placement.tenancy #=> String, one of "default", "dedicated", "host"
    #   resp.reservations[0].instances[0].placement.spread_domain #=> String
    #   resp.reservations[0].instances[0].placement.host_resource_group_arn #=> String
    #   resp.reservations[0].instances[0].placement.group_id #=> String
    #   resp.reservations[0].instances[0].placement.availability_zone #=> String
    #   resp.reservations[0].instances[0].kernel_id #=> String
    #   resp.reservations[0].instances[0].ramdisk_id #=> String
    #   resp.reservations[0].instances[0].platform #=> String, one of "Windows"
    #   resp.reservations[0].instances[0].monitoring.state #=> String, one of "disabled", "disabling", "enabled", "pending"
    #   resp.reservations[0].instances[0].subnet_id #=> String
    #   resp.reservations[0].instances[0].vpc_id #=> String
    #   resp.reservations[0].instances[0].private_ip_address #=> String
    #   resp.reservations[0].instances[0].public_ip_address #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * instance_exists
    #   * instance_running
    #   * instance_stopped
    #   * instance_terminated
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeInstances AWS API Documentation
    #
    # @overload describe_instances(params = {})
    # @param [Hash] params ({})
    def describe_instances(params = {}, options = {})
      req = build_request(:describe_instances, params)
      req.send_request(options)
    end

    # Describes your internet gateways. The default is to describe all your
    # internet gateways. Alternatively, you can specify specific internet
    # gateway IDs or filter the results to include only the internet
    # gateways that match specific criteria.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :internet_gateway_ids
    #   The IDs of the internet gateways.
    #
    #   Default: Describes all your internet gateways.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `attachment.state` - The current state of the attachment between the
    #     gateway and the VPC (`available`). Present only if a VPC is
    #     attached.
    #
    #   * `attachment.vpc-id` - The ID of an attached VPC.
    #
    #   * `internet-gateway-id` - The ID of the Internet gateway.
    #
    #   * `owner-id` - The ID of the Amazon Web Services account that owns the
    #     internet gateway.
    #
    #   * `tag` - The key/value combination of a tag assigned to the resource.
    #     Use the tag key in the filter name and the tag value as the filter
    #     value. For example, to find all resources that have a tag with the
    #     key `Owner` and the value `TeamA`, specify `tag:Owner` for the
    #     filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @return [Types::DescribeInternetGatewaysResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeInternetGatewaysResult#internet_gateways #internet_gateways} => Array&lt;Types::InternetGateway&gt;
    #   * {Types::DescribeInternetGatewaysResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe the Internet gateway for a VPC
    #
    #   # This example describes the Internet gateway for the specified VPC.
    #
    #   resp = client.describe_internet_gateways({
    #     filters: [
    #       {
    #         name: "attachment.vpc-id", 
    #         values: [
    #           "vpc-a01106c2", 
    #         ], 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     internet_gateways: [
    #       {
    #         attachments: [
    #           {
    #             state: "attached", 
    #             vpc_id: "vpc-a01106c2", 
    #           }, 
    #         ], 
    #         internet_gateway_id: "igw-c0a643a9", 
    #         tags: [
    #         ], 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_internet_gateways({
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #     internet_gateway_ids: ["InternetGatewayId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.internet_gateways #=> Array
    #   resp.internet_gateways[0].attachments #=> Array
    #   resp.internet_gateways[0].attachments[0].state #=> String, one of "attaching", "attached", "detaching", "detached"
    #   resp.internet_gateways[0].attachments[0].vpc_id #=> String
    #   resp.internet_gateways[0].internet_gateway_id #=> String
    #   resp.internet_gateways[0].owner_id #=> String
    #   resp.internet_gateways[0].tags #=> Array
    #   resp.internet_gateways[0].tags[0].key #=> String
    #   resp.internet_gateways[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * internet_gateway_exists
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeInternetGateways AWS API Documentation
    #
    # @overload describe_internet_gateways(params = {})
    # @param [Hash] params ({})
    def describe_internet_gateways(params = {}, options = {})
      req = build_request(:describe_internet_gateways, params)
      req.send_request(options)
    end

    # Describes your Autonomous System Numbers (ASNs), their provisioning
    # statuses, and the BYOIP CIDRs with which they are associated. For more
    # information, see [Tutorial: Bring your ASN to IPAM][1] in the *Amazon
    # VPC IPAM guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/tutorials-byoasn.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::DescribeIpamByoasnResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeIpamByoasnResult#byoasns #byoasns} => Array&lt;Types::Byoasn&gt;
    #   * {Types::DescribeIpamByoasnResult#next_token #next_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_ipam_byoasn({
    #     dry_run: false,
    #     max_results: 1,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.byoasns #=> Array
    #   resp.byoasns[0].asn #=> String
    #   resp.byoasns[0].ipam_id #=> String
    #   resp.byoasns[0].status_message #=> String
    #   resp.byoasns[0].state #=> String, one of "deprovisioned", "failed-deprovision", "failed-provision", "pending-deprovision", "pending-provision", "provisioned"
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeIpamByoasn AWS API Documentation
    #
    # @overload describe_ipam_byoasn(params = {})
    # @param [Hash] params ({})
    def describe_ipam_byoasn(params = {}, options = {})
      req = build_request(:describe_ipam_byoasn, params)
      req.send_request(options)
    end

    # Describe verification tokens. A verification token is an Amazon Web
    # Services-generated random value that you can use to prove ownership of
    # an external resource. For example, you can use a verification token to
    # validate that you control a public IP address range when you bring an
    # IP address range to Amazon Web Services (BYOIP).
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters for the request. For more information about
    #   filtering, see [Filtering CLI output][1].
    #
    #   Available filters:
    #
    #   * `ipam-arn`
    #
    #   * `ipam-external-resource-verification-token-arn`
    #
    #   * `ipam-external-resource-verification-token-id`
    #
    #   * `ipam-id`
    #
    #   * `ipam-region`
    #
    #   * `state`
    #
    #   * `status`
    #
    #   * `token-name`
    #
    #   * `token-value`
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/cli/latest/userguide/cli-usage-filter.html
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of tokens to return in one page of results.
    #
    # @option params [Array<String>] :ipam_external_resource_verification_token_ids
    #   Verification token IDs.
    #
    # @return [Types::DescribeIpamExternalResourceVerificationTokensResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeIpamExternalResourceVerificationTokensResult#next_token #next_token} => String
    #   * {Types::DescribeIpamExternalResourceVerificationTokensResult#ipam_external_resource_verification_tokens #ipam_external_resource_verification_tokens} => Array&lt;Types::IpamExternalResourceVerificationToken&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_ipam_external_resource_verification_tokens({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "NextToken",
    #     max_results: 1,
    #     ipam_external_resource_verification_token_ids: ["String"],
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.ipam_external_resource_verification_tokens #=> Array
    #   resp.ipam_external_resource_verification_tokens[0].ipam_external_resource_verification_token_id #=> String
    #   resp.ipam_external_resource_verification_tokens[0].ipam_external_resource_verification_token_arn #=> String
    #   resp.ipam_external_resource_verification_tokens[0].ipam_id #=> String
    #   resp.ipam_external_resource_verification_tokens[0].ipam_arn #=> String
    #   resp.ipam_external_resource_verification_tokens[0].ipam_region #=> String
    #   resp.ipam_external_resource_verification_tokens[0].token_value #=> String
    #   resp.ipam_external_resource_verification_tokens[0].token_name #=> String
    #   resp.ipam_external_resource_verification_tokens[0].not_after #=> Time
    #   resp.ipam_external_resource_verification_tokens[0].status #=> String, one of "valid", "expired"
    #   resp.ipam_external_resource_verification_tokens[0].tags #=> Array
    #   resp.ipam_external_resource_verification_tokens[0].tags[0].key #=> String
    #   resp.ipam_external_resource_verification_tokens[0].tags[0].value #=> String
    #   resp.ipam_external_resource_verification_tokens[0].state #=> String, one of "create-in-progress", "create-complete", "create-failed", "delete-in-progress", "delete-complete", "delete-failed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeIpamExternalResourceVerificationTokens AWS API Documentation
    #
    # @overload describe_ipam_external_resource_verification_tokens(params = {})
    # @param [Hash] params ({})
    def describe_ipam_external_resource_verification_tokens(params = {}, options = {})
      req = build_request(:describe_ipam_external_resource_verification_tokens, params)
      req.send_request(options)
    end

    # Get information about your IPAM pools.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters for the request. For more information about
    #   filtering, see [Filtering CLI output][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/cli/latest/userguide/cli-usage-filter.html
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in the request.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Array<String>] :ipam_pool_ids
    #   The IDs of the IPAM pools you would like information on.
    #
    # @return [Types::DescribeIpamPoolsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeIpamPoolsResult#next_token #next_token} => String
    #   * {Types::DescribeIpamPoolsResult#ipam_pools #ipam_pools} => Array&lt;Types::IpamPool&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_ipam_pools({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #     ipam_pool_ids: ["String"],
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.ipam_pools #=> Array
    #   resp.ipam_pools[0].owner_id #=> String
    #   resp.ipam_pools[0].ipam_pool_id #=> String
    #   resp.ipam_pools[0].source_ipam_pool_id #=> String
    #   resp.ipam_pools[0].ipam_pool_arn #=> String
    #   resp.ipam_pools[0].ipam_scope_arn #=> String
    #   resp.ipam_pools[0].ipam_scope_type #=> String, one of "public", "private"
    #   resp.ipam_pools[0].ipam_arn #=> String
    #   resp.ipam_pools[0].ipam_region #=> String
    #   resp.ipam_pools[0].locale #=> String
    #   resp.ipam_pools[0].pool_depth #=> Integer
    #   resp.ipam_pools[0].state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_pools[0].state_message #=> String
    #   resp.ipam_pools[0].description #=> String
    #   resp.ipam_pools[0].auto_import #=> Boolean
    #   resp.ipam_pools[0].publicly_advertisable #=> Boolean
    #   resp.ipam_pools[0].address_family #=> String, one of "ipv4", "ipv6"
    #   resp.ipam_pools[0].allocation_min_netmask_length #=> Integer
    #   resp.ipam_pools[0].allocation_max_netmask_length #=> Integer
    #   resp.ipam_pools[0].allocation_default_netmask_length #=> Integer
    #   resp.ipam_pools[0].allocation_resource_tags #=> Array
    #   resp.ipam_pools[0].allocation_resource_tags[0].key #=> String
    #   resp.ipam_pools[0].allocation_resource_tags[0].value #=> String
    #   resp.ipam_pools[0].tags #=> Array
    #   resp.ipam_pools[0].tags[0].key #=> String
    #   resp.ipam_pools[0].tags[0].value #=> String
    #   resp.ipam_pools[0].aws_service #=> String, one of "ec2"
    #   resp.ipam_pools[0].public_ip_source #=> String, one of "amazon", "byoip"
    #   resp.ipam_pools[0].source_resource.resource_id #=> String
    #   resp.ipam_pools[0].source_resource.resource_type #=> String, one of "vpc"
    #   resp.ipam_pools[0].source_resource.resource_region #=> String
    #   resp.ipam_pools[0].source_resource.resource_owner #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeIpamPools AWS API Documentation
    #
    # @overload describe_ipam_pools(params = {})
    # @param [Hash] params ({})
    def describe_ipam_pools(params = {}, options = {})
      req = build_request(:describe_ipam_pools, params)
      req.send_request(options)
    end

    # Describes IPAM resource discoveries. A resource discovery is an IPAM
    # component that enables IPAM to manage and monitor resources that
    # belong to the owning account.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :ipam_resource_discovery_ids
    #   The IPAM resource discovery IDs.
    #
    # @option params [String] :next_token
    #   Specify the pagination token from a previous request to retrieve the
    #   next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of resource discoveries to return in one page of
    #   results.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The resource discovery filters.
    #
    # @return [Types::DescribeIpamResourceDiscoveriesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeIpamResourceDiscoveriesResult#ipam_resource_discoveries #ipam_resource_discoveries} => Array&lt;Types::IpamResourceDiscovery&gt;
    #   * {Types::DescribeIpamResourceDiscoveriesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_ipam_resource_discoveries({
    #     dry_run: false,
    #     ipam_resource_discovery_ids: ["String"],
    #     next_token: "NextToken",
    #     max_results: 1,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_resource_discoveries #=> Array
    #   resp.ipam_resource_discoveries[0].owner_id #=> String
    #   resp.ipam_resource_discoveries[0].ipam_resource_discovery_id #=> String
    #   resp.ipam_resource_discoveries[0].ipam_resource_discovery_arn #=> String
    #   resp.ipam_resource_discoveries[0].ipam_resource_discovery_region #=> String
    #   resp.ipam_resource_discoveries[0].description #=> String
    #   resp.ipam_resource_discoveries[0].operating_regions #=> Array
    #   resp.ipam_resource_discoveries[0].operating_regions[0].region_name #=> String
    #   resp.ipam_resource_discoveries[0].is_default #=> Boolean
    #   resp.ipam_resource_discoveries[0].state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_resource_discoveries[0].tags #=> Array
    #   resp.ipam_resource_discoveries[0].tags[0].key #=> String
    #   resp.ipam_resource_discoveries[0].tags[0].value #=> String
    #   resp.ipam_resource_discoveries[0].organizational_unit_exclusions #=> Array
    #   resp.ipam_resource_discoveries[0].organizational_unit_exclusions[0].organizations_entity_path #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeIpamResourceDiscoveries AWS API Documentation
    #
    # @overload describe_ipam_resource_discoveries(params = {})
    # @param [Hash] params ({})
    def describe_ipam_resource_discoveries(params = {}, options = {})
      req = build_request(:describe_ipam_resource_discoveries, params)
      req.send_request(options)
    end

    # Describes resource discovery association with an Amazon VPC IPAM. An
    # associated resource discovery is a resource discovery that has been
    # associated with an IPAM..
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :ipam_resource_discovery_association_ids
    #   The resource discovery association IDs.
    #
    # @option params [String] :next_token
    #   Specify the pagination token from a previous request to retrieve the
    #   next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of resource discovery associations to return in one
    #   page of results.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The resource discovery association filters.
    #
    # @return [Types::DescribeIpamResourceDiscoveryAssociationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeIpamResourceDiscoveryAssociationsResult#ipam_resource_discovery_associations #ipam_resource_discovery_associations} => Array&lt;Types::IpamResourceDiscoveryAssociation&gt;
    #   * {Types::DescribeIpamResourceDiscoveryAssociationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_ipam_resource_discovery_associations({
    #     dry_run: false,
    #     ipam_resource_discovery_association_ids: ["String"],
    #     next_token: "NextToken",
    #     max_results: 1,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_resource_discovery_associations #=> Array
    #   resp.ipam_resource_discovery_associations[0].owner_id #=> String
    #   resp.ipam_resource_discovery_associations[0].ipam_resource_discovery_association_id #=> String
    #   resp.ipam_resource_discovery_associations[0].ipam_resource_discovery_association_arn #=> String
    #   resp.ipam_resource_discovery_associations[0].ipam_resource_discovery_id #=> String
    #   resp.ipam_resource_discovery_associations[0].ipam_id #=> String
    #   resp.ipam_resource_discovery_associations[0].ipam_arn #=> String
    #   resp.ipam_resource_discovery_associations[0].ipam_region #=> String
    #   resp.ipam_resource_discovery_associations[0].is_default #=> Boolean
    #   resp.ipam_resource_discovery_associations[0].resource_discovery_status #=> String, one of "active", "not-found"
    #   resp.ipam_resource_discovery_associations[0].state #=> String, one of "associate-in-progress", "associate-complete", "associate-failed", "disassociate-in-progress", "disassociate-complete", "disassociate-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_resource_discovery_associations[0].tags #=> Array
    #   resp.ipam_resource_discovery_associations[0].tags[0].key #=> String
    #   resp.ipam_resource_discovery_associations[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeIpamResourceDiscoveryAssociations AWS API Documentation
    #
    # @overload describe_ipam_resource_discovery_associations(params = {})
    # @param [Hash] params ({})
    def describe_ipam_resource_discovery_associations(params = {}, options = {})
      req = build_request(:describe_ipam_resource_discovery_associations, params)
      req.send_request(options)
    end

    # Get information about your IPAM scopes.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters for the request. For more information about
    #   filtering, see [Filtering CLI output][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/cli/latest/userguide/cli-usage-filter.html
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in the request.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Array<String>] :ipam_scope_ids
    #   The IDs of the scopes you want information on.
    #
    # @return [Types::DescribeIpamScopesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeIpamScopesResult#next_token #next_token} => String
    #   * {Types::DescribeIpamScopesResult#ipam_scopes #ipam_scopes} => Array&lt;Types::IpamScope&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_ipam_scopes({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #     ipam_scope_ids: ["String"],
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.ipam_scopes #=> Array
    #   resp.ipam_scopes[0].owner_id #=> String
    #   resp.ipam_scopes[0].ipam_scope_id #=> String
    #   resp.ipam_scopes[0].ipam_scope_arn #=> String
    #   resp.ipam_scopes[0].ipam_arn #=> String
    #   resp.ipam_scopes[0].ipam_region #=> String
    #   resp.ipam_scopes[0].ipam_scope_type #=> String, one of "public", "private"
    #   resp.ipam_scopes[0].is_default #=> Boolean
    #   resp.ipam_scopes[0].description #=> String
    #   resp.ipam_scopes[0].pool_count #=> Integer
    #   resp.ipam_scopes[0].state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_scopes[0].tags #=> Array
    #   resp.ipam_scopes[0].tags[0].key #=> String
    #   resp.ipam_scopes[0].tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeIpamScopes AWS API Documentation
    #
    # @overload describe_ipam_scopes(params = {})
    # @param [Hash] params ({})
    def describe_ipam_scopes(params = {}, options = {})
      req = build_request(:describe_ipam_scopes, params)
      req.send_request(options)
    end

    # Get information about your IPAM pools.
    #
    # For more information, see [What is IPAM?][1] in the *Amazon VPC IPAM
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/what-is-it-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters for the request. For more information about
    #   filtering, see [Filtering CLI output][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/cli/latest/userguide/cli-usage-filter.html
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in the request.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Array<String>] :ipam_ids
    #   The IDs of the IPAMs you want information on.
    #
    # @return [Types::DescribeIpamsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeIpamsResult#next_token #next_token} => String
    #   * {Types::DescribeIpamsResult#ipams #ipams} => Array&lt;Types::Ipam&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_ipams({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #     ipam_ids: ["String"],
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.ipams #=> Array
    #   resp.ipams[0].owner_id #=> String
    #   resp.ipams[0].ipam_id #=> String
    #   resp.ipams[0].ipam_arn #=> String
    #   resp.ipams[0].ipam_region #=> String
    #   resp.ipams[0].public_default_scope_id #=> String
    #   resp.ipams[0].private_default_scope_id #=> String
    #   resp.ipams[0].scope_count #=> Integer
    #   resp.ipams[0].description #=> String
    #   resp.ipams[0].operating_regions #=> Array
    #   resp.ipams[0].operating_regions[0].region_name #=> String
    #   resp.ipams[0].state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipams[0].tags #=> Array
    #   resp.ipams[0].tags[0].key #=> String
    #   resp.ipams[0].tags[0].value #=> String
    #   resp.ipams[0].default_resource_discovery_id #=> String
    #   resp.ipams[0].default_resource_discovery_association_id #=> String
    #   resp.ipams[0].resource_discovery_association_count #=> Integer
    #   resp.ipams[0].state_message #=> String
    #   resp.ipams[0].tier #=> String, one of "free", "advanced"
    #   resp.ipams[0].enable_private_gua #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeIpams AWS API Documentation
    #
    # @overload describe_ipams(params = {})
    # @param [Hash] params ({})
    def describe_ipams(params = {}, options = {})
      req = build_request(:describe_ipams, params)
      req.send_request(options)
    end

    # Describes your IPv6 address pools.
    #
    # @option params [Array<String>] :pool_ids
    #   The IDs of the IPv6 address pools.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @return [Types::DescribeIpv6PoolsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeIpv6PoolsResult#ipv_6_pools #ipv_6_pools} => Array&lt;Types::Ipv6Pool&gt;
    #   * {Types::DescribeIpv6PoolsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_ipv_6_pools({
    #     pool_ids: ["Ipv6PoolEc2Id"],
    #     next_token: "NextToken",
    #     max_results: 1,
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.ipv_6_pools #=> Array
    #   resp.ipv_6_pools[0].pool_id #=> String
    #   resp.ipv_6_pools[0].description #=> String
    #   resp.ipv_6_pools[0].pool_cidr_blocks #=> Array
    #   resp.ipv_6_pools[0].pool_cidr_blocks[0].cidr #=> String
    #   resp.ipv_6_pools[0].tags #=> Array
    #   resp.ipv_6_pools[0].tags[0].key #=> String
    #   resp.ipv_6_pools[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeIpv6Pools AWS API Documentation
    #
    # @overload describe_ipv_6_pools(params = {})
    # @param [Hash] params ({})
    def describe_ipv_6_pools(params = {}, options = {})
      req = build_request(:describe_ipv_6_pools, params)
      req.send_request(options)
    end

    # Describes the specified key pairs or all of your key pairs.
    #
    # For more information about key pairs, see [Amazon EC2 key pairs][1] in
    # the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-key-pairs.html
    #
    # @option params [Array<String>] :key_names
    #   The key pair names.
    #
    #   Default: Describes all of your key pairs.
    #
    # @option params [Array<String>] :key_pair_ids
    #   The IDs of the key pairs.
    #
    # @option params [Boolean] :include_public_key
    #   If `true`, the public key material is included in the response.
    #
    #   Default: `false`
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `key-pair-id` - The ID of the key pair.
    #
    #   * `fingerprint` - The fingerprint of the key pair.
    #
    #   * `key-name` - The name of the key pair.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    # @return [Types::DescribeKeyPairsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeKeyPairsResult#key_pairs #key_pairs} => Array&lt;Types::KeyPairInfo&gt;
    #
    #
    # @example Example: To display a key pair
    #
    #   # This example displays the fingerprint for the specified key.
    #
    #   resp = client.describe_key_pairs({
    #     key_names: [
    #       "my-key-pair", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     key_pairs: [
    #       {
    #         key_fingerprint: "1f:51:ae:28:bf:89:e9:d8:1f:25:5d:37:2d:7d:b8:ca:9f:f5:f1:6f", 
    #         key_name: "my-key-pair", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_key_pairs({
    #     key_names: ["KeyPairName"],
    #     key_pair_ids: ["KeyPairId"],
    #     include_public_key: false,
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.key_pairs #=> Array
    #   resp.key_pairs[0].key_pair_id #=> String
    #   resp.key_pairs[0].key_type #=> String, one of "rsa", "ed25519"
    #   resp.key_pairs[0].tags #=> Array
    #   resp.key_pairs[0].tags[0].key #=> String
    #   resp.key_pairs[0].tags[0].value #=> String
    #   resp.key_pairs[0].public_key #=> String
    #   resp.key_pairs[0].create_time #=> Time
    #   resp.key_pairs[0].key_name #=> String
    #   resp.key_pairs[0].key_fingerprint #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * key_pair_exists
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeKeyPairs AWS API Documentation
    #
    # @overload describe_key_pairs(params = {})
    # @param [Hash] params ({})
    def describe_key_pairs(params = {}, options = {})
      req = build_request(:describe_key_pairs, params)
      req.send_request(options)
    end

    # Describes one or more versions of a specified launch template. You can
    # describe all versions, individual versions, or a range of versions.
    # You can also describe all the latest versions or all the default
    # versions of all the launch templates in your account.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :launch_template_id
    #   The ID of the launch template.
    #
    #   To describe one or more versions of a specified launch template, you
    #   must specify either the launch template ID or the launch template
    #   name, but not both.
    #
    #   To describe all the latest or default launch template versions in your
    #   account, you must omit this parameter.
    #
    # @option params [String] :launch_template_name
    #   The name of the launch template.
    #
    #   To describe one or more versions of a specified launch template, you
    #   must specify either the launch template name or the launch template
    #   ID, but not both.
    #
    #   To describe all the latest or default launch template versions in your
    #   account, you must omit this parameter.
    #
    # @option params [Array<String>] :versions
    #   One or more versions of the launch template. Valid values depend on
    #   whether you are describing a specified launch template (by ID or name)
    #   or all launch templates in your account.
    #
    #   To describe one or more versions of a specified launch template, valid
    #   values are `$Latest`, `$Default`, and numbers.
    #
    #   To describe all launch templates in your account that are defined as
    #   the latest version, the valid value is `$Latest`. To describe all
    #   launch templates in your account that are defined as the default
    #   version, the valid value is `$Default`. You can specify `$Latest` and
    #   `$Default` in the same request. You cannot specify numbers.
    #
    # @option params [String] :min_version
    #   The version number after which to describe launch template versions.
    #
    # @option params [String] :max_version
    #   The version number up to which to describe launch template versions.
    #
    # @option params [String] :next_token
    #   The token to request the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in a single call. To retrieve
    #   the remaining results, make another call with the returned `NextToken`
    #   value. This value can be between 1 and 200.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `create-time` - The time the launch template version was created.
    #
    #   * `ebs-optimized` - A boolean that indicates whether the instance is
    #     optimized for Amazon EBS I/O.
    #
    #   * `http-endpoint` - Indicates whether the HTTP metadata endpoint on
    #     your instances is enabled (`enabled` \| `disabled`).
    #
    #   * `http-protocol-ipv4` - Indicates whether the IPv4 endpoint for the
    #     instance metadata service is enabled (`enabled` \| `disabled`).
    #
    #   * `host-resource-group-arn` - The ARN of the host resource group in
    #     which to launch the instances.
    #
    #   * `http-tokens` - The state of token usage for your instance metadata
    #     requests (`optional` \| `required`).
    #
    #   * `iam-instance-profile` - The ARN of the IAM instance profile.
    #
    #   * `image-id` - The ID of the AMI.
    #
    #   * `instance-type` - The instance type.
    #
    #   * `is-default-version` - A boolean that indicates whether the launch
    #     template version is the default version.
    #
    #   * `kernel-id` - The kernel ID.
    #
    #   * `license-configuration-arn` - The ARN of the license configuration.
    #
    #   * `network-card-index` - The index of the network card.
    #
    #   * `ram-disk-id` - The RAM disk ID.
    #
    # @option params [Boolean] :resolve_alias
    #   If `true`, and if a Systems Manager parameter is specified for
    #   `ImageId`, the AMI ID is displayed in the response for `imageId`.
    #
    #   If `false`, and if a Systems Manager parameter is specified for
    #   `ImageId`, the parameter is displayed in the response for `imageId`.
    #
    #   For more information, see [Use a Systems Manager parameter instead of
    #   an AMI ID][1] in the *Amazon EC2 User Guide*.
    #
    #   Default: `false`
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/create-launch-template.html#use-an-ssm-parameter-instead-of-an-ami-id
    #
    # @return [Types::DescribeLaunchTemplateVersionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeLaunchTemplateVersionsResult#launch_template_versions #launch_template_versions} => Array&lt;Types::LaunchTemplateVersion&gt;
    #   * {Types::DescribeLaunchTemplateVersionsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe the versions for a launch template
    #
    #   # This example describes the versions for the specified launch template.
    #
    #   resp = client.describe_launch_template_versions({
    #     launch_template_id: "068f72b72934aff71", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     launch_template_versions: [
    #       {
    #         create_time: Time.parse("2017-11-20T13:12:32.000Z"), 
    #         created_by: "arn:aws:iam::123456789102:root", 
    #         default_version: false, 
    #         launch_template_data: {
    #           image_id: "ami-6057e21a", 
    #           instance_type: "t2.medium", 
    #           key_name: "kp-us-east", 
    #           network_interfaces: [
    #             {
    #               device_index: 0, 
    #               groups: [
    #                 "sg-7c227019", 
    #               ], 
    #               subnet_id: "subnet-1a2b3c4d", 
    #             }, 
    #           ], 
    #         }, 
    #         launch_template_id: "lt-068f72b72934aff71", 
    #         launch_template_name: "Webservers", 
    #         version_number: 2, 
    #       }, 
    #       {
    #         create_time: Time.parse("2017-11-20T12:52:33.000Z"), 
    #         created_by: "arn:aws:iam::123456789102:root", 
    #         default_version: true, 
    #         launch_template_data: {
    #           image_id: "ami-aabbcc11", 
    #           instance_type: "t2.medium", 
    #           key_name: "kp-us-east", 
    #           network_interfaces: [
    #             {
    #               associate_public_ip_address: true, 
    #               delete_on_termination: false, 
    #               device_index: 0, 
    #               groups: [
    #                 "sg-7c227019", 
    #               ], 
    #               subnet_id: "subnet-7b16de0c", 
    #             }, 
    #           ], 
    #           user_data: "", 
    #         }, 
    #         launch_template_id: "lt-068f72b72934aff71", 
    #         launch_template_name: "Webservers", 
    #         version_number: 1, 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_launch_template_versions({
    #     dry_run: false,
    #     launch_template_id: "LaunchTemplateId",
    #     launch_template_name: "LaunchTemplateName",
    #     versions: ["String"],
    #     min_version: "String",
    #     max_version: "String",
    #     next_token: "String",
    #     max_results: 1,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     resolve_alias: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.launch_template_versions #=> Array
    #   resp.launch_template_versions[0].launch_template_id #=> String
    #   resp.launch_template_versions[0].launch_template_name #=> String
    #   resp.launch_template_versions[0].version_number #=> Integer
    #   resp.launch_template_versions[0].version_description #=> String
    #   resp.launch_template_versions[0].create_time #=> Time
    #   resp.launch_template_versions[0].created_by #=> String
    #   resp.launch_template_versions[0].default_version #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.kernel_id #=> String
    #   resp.launch_template_versions[0].launch_template_data.ebs_optimized #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.iam_instance_profile.arn #=> String
    #   resp.launch_template_versions[0].launch_template_data.iam_instance_profile.name #=> String
    #   resp.launch_template_versions[0].launch_template_data.block_device_mappings #=> Array
    #   resp.launch_template_versions[0].launch_template_data.block_device_mappings[0].device_name #=> String
    #   resp.launch_template_versions[0].launch_template_data.block_device_mappings[0].virtual_name #=> String
    #   resp.launch_template_versions[0].launch_template_data.block_device_mappings[0].ebs.encrypted #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.block_device_mappings[0].ebs.iops #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.block_device_mappings[0].ebs.kms_key_id #=> String
    #   resp.launch_template_versions[0].launch_template_data.block_device_mappings[0].ebs.snapshot_id #=> String
    #   resp.launch_template_versions[0].launch_template_data.block_device_mappings[0].ebs.volume_size #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.block_device_mappings[0].ebs.volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.launch_template_versions[0].launch_template_data.block_device_mappings[0].ebs.throughput #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.block_device_mappings[0].no_device #=> String
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces #=> Array
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].associate_carrier_ip_address #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].associate_public_ip_address #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].delete_on_termination #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].description #=> String
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].device_index #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].groups #=> Array
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].groups[0] #=> String
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].interface_type #=> String
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].ipv_6_address_count #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].ipv_6_addresses #=> Array
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].ipv_6_addresses[0].ipv_6_address #=> String
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].ipv_6_addresses[0].is_primary_ipv_6 #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].network_interface_id #=> String
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].private_ip_address #=> String
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].private_ip_addresses #=> Array
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].private_ip_addresses[0].primary #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].private_ip_addresses[0].private_ip_address #=> String
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].secondary_private_ip_address_count #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].subnet_id #=> String
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].network_card_index #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].ipv_4_prefixes #=> Array
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].ipv_4_prefixes[0].ipv_4_prefix #=> String
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].ipv_4_prefix_count #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].ipv_6_prefixes #=> Array
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].ipv_6_prefixes[0].ipv_6_prefix #=> String
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].ipv_6_prefix_count #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].primary_ipv_6 #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].ena_srd_specification.ena_srd_enabled #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].ena_srd_specification.ena_srd_udp_specification.ena_srd_udp_enabled #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].connection_tracking_specification.tcp_established_timeout #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].connection_tracking_specification.udp_timeout #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.network_interfaces[0].connection_tracking_specification.udp_stream_timeout #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.image_id #=> String
    #   resp.launch_template_versions[0].launch_template_data.instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.launch_template_versions[0].launch_template_data.key_name #=> String
    #   resp.launch_template_versions[0].launch_template_data.monitoring.enabled #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.placement.availability_zone #=> String
    #   resp.launch_template_versions[0].launch_template_data.placement.affinity #=> String
    #   resp.launch_template_versions[0].launch_template_data.placement.group_name #=> String
    #   resp.launch_template_versions[0].launch_template_data.placement.host_id #=> String
    #   resp.launch_template_versions[0].launch_template_data.placement.tenancy #=> String, one of "default", "dedicated", "host"
    #   resp.launch_template_versions[0].launch_template_data.placement.spread_domain #=> String
    #   resp.launch_template_versions[0].launch_template_data.placement.host_resource_group_arn #=> String
    #   resp.launch_template_versions[0].launch_template_data.placement.partition_number #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.placement.group_id #=> String
    #   resp.launch_template_versions[0].launch_template_data.ram_disk_id #=> String
    #   resp.launch_template_versions[0].launch_template_data.disable_api_termination #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.instance_initiated_shutdown_behavior #=> String, one of "stop", "terminate"
    #   resp.launch_template_versions[0].launch_template_data.user_data #=> String
    #   resp.launch_template_versions[0].launch_template_data.tag_specifications #=> Array
    #   resp.launch_template_versions[0].launch_template_data.tag_specifications[0].resource_type #=> String, one of "capacity-reservation", "client-vpn-endpoint", "customer-gateway", "carrier-gateway", "coip-pool", "declarative-policies-report", "dedicated-host", "dhcp-options", "egress-only-internet-gateway", "elastic-ip", "elastic-gpu", "export-image-task", "export-instance-task", "fleet", "fpga-image", "host-reservation", "image", "import-image-task", "import-snapshot-task", "instance", "instance-event-window", "internet-gateway", "ipam", "ipam-pool", "ipam-scope", "ipv4pool-ec2", "ipv6pool-ec2", "key-pair", "launch-template", "local-gateway", "local-gateway-route-table", "local-gateway-virtual-interface", "local-gateway-virtual-interface-group", "local-gateway-route-table-vpc-association", "local-gateway-route-table-virtual-interface-group-association", "natgateway", "network-acl", "network-interface", "network-insights-analysis", "network-insights-path", "network-insights-access-scope", "network-insights-access-scope-analysis", "placement-group", "prefix-list", "replace-root-volume-task", "reserved-instances", "route-table", "security-group", "security-group-rule", "snapshot", "spot-fleet-request", "spot-instances-request", "subnet", "subnet-cidr-reservation", "traffic-mirror-filter", "traffic-mirror-session", "traffic-mirror-target", "transit-gateway", "transit-gateway-attachment", "transit-gateway-connect-peer", "transit-gateway-multicast-domain", "transit-gateway-policy-table", "transit-gateway-route-table", "transit-gateway-route-table-announcement", "volume", "vpc", "vpc-endpoint", "vpc-endpoint-connection", "vpc-endpoint-service", "vpc-endpoint-service-permission", "vpc-peering-connection", "vpn-connection", "vpn-gateway", "vpc-flow-log", "capacity-reservation-fleet", "traffic-mirror-filter-rule", "vpc-endpoint-connection-device-type", "verified-access-instance", "verified-access-group", "verified-access-endpoint", "verified-access-policy", "verified-access-trust-provider", "vpn-connection-device-type", "vpc-block-public-access-exclusion", "ipam-resource-discovery", "ipam-resource-discovery-association", "instance-connect-endpoint", "verified-access-endpoint-target", "ipam-external-resource-verification-token"
    #   resp.launch_template_versions[0].launch_template_data.tag_specifications[0].tags #=> Array
    #   resp.launch_template_versions[0].launch_template_data.tag_specifications[0].tags[0].key #=> String
    #   resp.launch_template_versions[0].launch_template_data.tag_specifications[0].tags[0].value #=> String
    #   resp.launch_template_versions[0].launch_template_data.elastic_gpu_specifications #=> Array
    #   resp.launch_template_versions[0].launch_template_data.elastic_gpu_specifications[0].type #=> String
    #   resp.launch_template_versions[0].launch_template_data.elastic_inference_accelerators #=> Array
    #   resp.launch_template_versions[0].launch_template_data.elastic_inference_accelerators[0].type #=> String
    #   resp.launch_template_versions[0].launch_template_data.elastic_inference_accelerators[0].count #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.security_group_ids #=> Array
    #   resp.launch_template_versions[0].launch_template_data.security_group_ids[0] #=> String
    #   resp.launch_template_versions[0].launch_template_data.security_groups #=> Array
    #   resp.launch_template_versions[0].launch_template_data.security_groups[0] #=> String
    #   resp.launch_template_versions[0].launch_template_data.instance_market_options.market_type #=> String, one of "spot", "capacity-block"
    #   resp.launch_template_versions[0].launch_template_data.instance_market_options.spot_options.max_price #=> String
    #   resp.launch_template_versions[0].launch_template_data.instance_market_options.spot_options.spot_instance_type #=> String, one of "one-time", "persistent"
    #   resp.launch_template_versions[0].launch_template_data.instance_market_options.spot_options.block_duration_minutes #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_market_options.spot_options.valid_until #=> Time
    #   resp.launch_template_versions[0].launch_template_data.instance_market_options.spot_options.instance_interruption_behavior #=> String, one of "hibernate", "stop", "terminate"
    #   resp.launch_template_versions[0].launch_template_data.credit_specification.cpu_credits #=> String
    #   resp.launch_template_versions[0].launch_template_data.cpu_options.core_count #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.cpu_options.threads_per_core #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.cpu_options.amd_sev_snp #=> String, one of "enabled", "disabled"
    #   resp.launch_template_versions[0].launch_template_data.capacity_reservation_specification.capacity_reservation_preference #=> String, one of "capacity-reservations-only", "open", "none"
    #   resp.launch_template_versions[0].launch_template_data.capacity_reservation_specification.capacity_reservation_target.capacity_reservation_id #=> String
    #   resp.launch_template_versions[0].launch_template_data.capacity_reservation_specification.capacity_reservation_target.capacity_reservation_resource_group_arn #=> String
    #   resp.launch_template_versions[0].launch_template_data.license_specifications #=> Array
    #   resp.launch_template_versions[0].launch_template_data.license_specifications[0].license_configuration_arn #=> String
    #   resp.launch_template_versions[0].launch_template_data.hibernation_options.configured #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.metadata_options.state #=> String, one of "pending", "applied"
    #   resp.launch_template_versions[0].launch_template_data.metadata_options.http_tokens #=> String, one of "optional", "required"
    #   resp.launch_template_versions[0].launch_template_data.metadata_options.http_put_response_hop_limit #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.metadata_options.http_endpoint #=> String, one of "disabled", "enabled"
    #   resp.launch_template_versions[0].launch_template_data.metadata_options.http_protocol_ipv_6 #=> String, one of "disabled", "enabled"
    #   resp.launch_template_versions[0].launch_template_data.metadata_options.instance_metadata_tags #=> String, one of "disabled", "enabled"
    #   resp.launch_template_versions[0].launch_template_data.enclave_options.enabled #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.v_cpu_count.min #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.v_cpu_count.max #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.memory_mi_b.min #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.memory_mi_b.max #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.cpu_manufacturers #=> Array
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.cpu_manufacturers[0] #=> String, one of "intel", "amd", "amazon-web-services", "apple"
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.memory_gi_b_per_v_cpu.min #=> Float
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.memory_gi_b_per_v_cpu.max #=> Float
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.excluded_instance_types #=> Array
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.excluded_instance_types[0] #=> String
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.instance_generations #=> Array
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.instance_generations[0] #=> String, one of "current", "previous"
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.spot_max_price_percentage_over_lowest_price #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.on_demand_max_price_percentage_over_lowest_price #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.bare_metal #=> String, one of "included", "required", "excluded"
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.burstable_performance #=> String, one of "included", "required", "excluded"
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.require_hibernate_support #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.network_interface_count.min #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.network_interface_count.max #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.local_storage #=> String, one of "included", "required", "excluded"
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.local_storage_types #=> Array
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.local_storage_types[0] #=> String, one of "hdd", "ssd"
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.total_local_storage_gb.min #=> Float
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.total_local_storage_gb.max #=> Float
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.baseline_ebs_bandwidth_mbps.min #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.baseline_ebs_bandwidth_mbps.max #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.accelerator_types #=> Array
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.accelerator_types[0] #=> String, one of "gpu", "fpga", "inference"
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.accelerator_count.min #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.accelerator_count.max #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.accelerator_manufacturers #=> Array
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.accelerator_manufacturers[0] #=> String, one of "amazon-web-services", "amd", "nvidia", "xilinx", "habana"
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.accelerator_names #=> Array
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.accelerator_names[0] #=> String, one of "a100", "inferentia", "k520", "k80", "m60", "radeon-pro-v520", "t4", "vu9p", "v100", "a10g", "h100", "t4g"
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.accelerator_total_memory_mi_b.min #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.accelerator_total_memory_mi_b.max #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.network_bandwidth_gbps.min #=> Float
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.network_bandwidth_gbps.max #=> Float
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.allowed_instance_types #=> Array
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.allowed_instance_types[0] #=> String
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.max_spot_price_as_percentage_of_optimal_on_demand_price #=> Integer
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.baseline_performance_factors.cpu.references #=> Array
    #   resp.launch_template_versions[0].launch_template_data.instance_requirements.baseline_performance_factors.cpu.references[0].instance_family #=> String
    #   resp.launch_template_versions[0].launch_template_data.private_dns_name_options.hostname_type #=> String, one of "ip-name", "resource-name"
    #   resp.launch_template_versions[0].launch_template_data.private_dns_name_options.enable_resource_name_dns_a_record #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.private_dns_name_options.enable_resource_name_dns_aaaa_record #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.maintenance_options.auto_recovery #=> String, one of "default", "disabled"
    #   resp.launch_template_versions[0].launch_template_data.disable_api_stop #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.operator.managed #=> Boolean
    #   resp.launch_template_versions[0].launch_template_data.operator.principal #=> String
    #   resp.launch_template_versions[0].launch_template_data.network_performance_options.bandwidth_weighting #=> String, one of "default", "vpc-1", "ebs-1"
    #   resp.launch_template_versions[0].operator.managed #=> Boolean
    #   resp.launch_template_versions[0].operator.principal #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeLaunchTemplateVersions AWS API Documentation
    #
    # @overload describe_launch_template_versions(params = {})
    # @param [Hash] params ({})
    def describe_launch_template_versions(params = {}, options = {})
      req = build_request(:describe_launch_template_versions, params)
      req.send_request(options)
    end

    # Describes one or more launch templates.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :launch_template_ids
    #   One or more launch template IDs.
    #
    # @option params [Array<String>] :launch_template_names
    #   One or more launch template names.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `create-time` - The time the launch template was created.
    #
    #   * `launch-template-name` - The name of the launch template.
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @option params [String] :next_token
    #   The token to request the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in a single call. To retrieve
    #   the remaining results, make another call with the returned `NextToken`
    #   value. This value can be between 1 and 200.
    #
    # @return [Types::DescribeLaunchTemplatesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeLaunchTemplatesResult#launch_templates #launch_templates} => Array&lt;Types::LaunchTemplate&gt;
    #   * {Types::DescribeLaunchTemplatesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe a launch template
    #
    #   # This example describes the specified launch template.
    #
    #   resp = client.describe_launch_templates({
    #     launch_template_ids: [
    #       "lt-01238c059e3466abc", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     launch_templates: [
    #       {
    #         create_time: Time.parse("2018-01-16T04:32:57.000Z"), 
    #         created_by: "arn:aws:iam::123456789012:root", 
    #         default_version_number: 1, 
    #         latest_version_number: 1, 
    #         launch_template_id: "lt-01238c059e3466abc", 
    #         launch_template_name: "my-template", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_launch_templates({
    #     dry_run: false,
    #     launch_template_ids: ["LaunchTemplateId"],
    #     launch_template_names: ["LaunchTemplateName"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.launch_templates #=> Array
    #   resp.launch_templates[0].launch_template_id #=> String
    #   resp.launch_templates[0].launch_template_name #=> String
    #   resp.launch_templates[0].create_time #=> Time
    #   resp.launch_templates[0].created_by #=> String
    #   resp.launch_templates[0].default_version_number #=> Integer
    #   resp.launch_templates[0].latest_version_number #=> Integer
    #   resp.launch_templates[0].tags #=> Array
    #   resp.launch_templates[0].tags[0].key #=> String
    #   resp.launch_templates[0].tags[0].value #=> String
    #   resp.launch_templates[0].operator.managed #=> Boolean
    #   resp.launch_templates[0].operator.principal #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeLaunchTemplates AWS API Documentation
    #
    # @overload describe_launch_templates(params = {})
    # @param [Hash] params ({})
    def describe_launch_templates(params = {}, options = {})
      req = build_request(:describe_launch_templates, params)
      req.send_request(options)
    end

    # Describes the associations between virtual interface groups and local
    # gateway route tables.
    #
    # @option params [Array<String>] :local_gateway_route_table_virtual_interface_group_association_ids
    #   The IDs of the associations.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `local-gateway-id` - The ID of a local gateway.
    #
    #   * `local-gateway-route-table-arn` - The Amazon Resource Name (ARN) of
    #     the local gateway route table for the virtual interface group.
    #
    #   * `local-gateway-route-table-id` - The ID of the local gateway route
    #     table.
    #
    #   * `local-gateway-route-table-virtual-interface-group-association-id` -
    #     The ID of the association.
    #
    #   * `local-gateway-route-table-virtual-interface-group-id` - The ID of
    #     the virtual interface group.
    #
    #   * `owner-id` - The ID of the Amazon Web Services account that owns the
    #     local gateway virtual interface group association.
    #
    #   * `state` - The state of the association.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociationsResult#local_gateway_route_table_virtual_interface_group_associations #local_gateway_route_table_virtual_interface_group_associations} => Array&lt;Types::LocalGatewayRouteTableVirtualInterfaceGroupAssociation&gt;
    #   * {Types::DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_local_gateway_route_table_virtual_interface_group_associations({
    #     local_gateway_route_table_virtual_interface_group_association_ids: ["LocalGatewayRouteTableVirtualInterfaceGroupAssociationId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.local_gateway_route_table_virtual_interface_group_associations #=> Array
    #   resp.local_gateway_route_table_virtual_interface_group_associations[0].local_gateway_route_table_virtual_interface_group_association_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_associations[0].local_gateway_virtual_interface_group_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_associations[0].local_gateway_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_associations[0].local_gateway_route_table_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_associations[0].local_gateway_route_table_arn #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_associations[0].owner_id #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_associations[0].state #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_associations[0].tags #=> Array
    #   resp.local_gateway_route_table_virtual_interface_group_associations[0].tags[0].key #=> String
    #   resp.local_gateway_route_table_virtual_interface_group_associations[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeLocalGatewayRouteTableVirtualInterfaceGroupAssociations AWS API Documentation
    #
    # @overload describe_local_gateway_route_table_virtual_interface_group_associations(params = {})
    # @param [Hash] params ({})
    def describe_local_gateway_route_table_virtual_interface_group_associations(params = {}, options = {})
      req = build_request(:describe_local_gateway_route_table_virtual_interface_group_associations, params)
      req.send_request(options)
    end

    # Describes the specified associations between VPCs and local gateway
    # route tables.
    #
    # @option params [Array<String>] :local_gateway_route_table_vpc_association_ids
    #   The IDs of the associations.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `local-gateway-id` - The ID of a local gateway.
    #
    #   * `local-gateway-route-table-arn` - The Amazon Resource Name (ARN) of
    #     the local gateway route table for the association.
    #
    #   * `local-gateway-route-table-id` - The ID of the local gateway route
    #     table.
    #
    #   * `local-gateway-route-table-vpc-association-id` - The ID of the
    #     association.
    #
    #   * `owner-id` - The ID of the Amazon Web Services account that owns the
    #     local gateway route table for the association.
    #
    #   * `state` - The state of the association.
    #
    #   * `vpc-id` - The ID of the VPC.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeLocalGatewayRouteTableVpcAssociationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeLocalGatewayRouteTableVpcAssociationsResult#local_gateway_route_table_vpc_associations #local_gateway_route_table_vpc_associations} => Array&lt;Types::LocalGatewayRouteTableVpcAssociation&gt;
    #   * {Types::DescribeLocalGatewayRouteTableVpcAssociationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_local_gateway_route_table_vpc_associations({
    #     local_gateway_route_table_vpc_association_ids: ["LocalGatewayRouteTableVpcAssociationId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.local_gateway_route_table_vpc_associations #=> Array
    #   resp.local_gateway_route_table_vpc_associations[0].local_gateway_route_table_vpc_association_id #=> String
    #   resp.local_gateway_route_table_vpc_associations[0].local_gateway_route_table_id #=> String
    #   resp.local_gateway_route_table_vpc_associations[0].local_gateway_route_table_arn #=> String
    #   resp.local_gateway_route_table_vpc_associations[0].local_gateway_id #=> String
    #   resp.local_gateway_route_table_vpc_associations[0].vpc_id #=> String
    #   resp.local_gateway_route_table_vpc_associations[0].owner_id #=> String
    #   resp.local_gateway_route_table_vpc_associations[0].state #=> String
    #   resp.local_gateway_route_table_vpc_associations[0].tags #=> Array
    #   resp.local_gateway_route_table_vpc_associations[0].tags[0].key #=> String
    #   resp.local_gateway_route_table_vpc_associations[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeLocalGatewayRouteTableVpcAssociations AWS API Documentation
    #
    # @overload describe_local_gateway_route_table_vpc_associations(params = {})
    # @param [Hash] params ({})
    def describe_local_gateway_route_table_vpc_associations(params = {}, options = {})
      req = build_request(:describe_local_gateway_route_table_vpc_associations, params)
      req.send_request(options)
    end

    # Describes one or more local gateway route tables. By default, all
    # local gateway route tables are described. Alternatively, you can
    # filter the results.
    #
    # @option params [Array<String>] :local_gateway_route_table_ids
    #   The IDs of the local gateway route tables.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `local-gateway-id` - The ID of a local gateway.
    #
    #   * `local-gateway-route-table-arn` - The Amazon Resource Name (ARN) of
    #     the local gateway route table.
    #
    #   * `local-gateway-route-table-id` - The ID of a local gateway route
    #     table.
    #
    #   * `outpost-arn` - The Amazon Resource Name (ARN) of the Outpost.
    #
    #   * `owner-id` - The ID of the Amazon Web Services account that owns the
    #     local gateway route table.
    #
    #   * `state` - The state of the local gateway route table.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeLocalGatewayRouteTablesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeLocalGatewayRouteTablesResult#local_gateway_route_tables #local_gateway_route_tables} => Array&lt;Types::LocalGatewayRouteTable&gt;
    #   * {Types::DescribeLocalGatewayRouteTablesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_local_gateway_route_tables({
    #     local_gateway_route_table_ids: ["LocalGatewayRoutetableId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.local_gateway_route_tables #=> Array
    #   resp.local_gateway_route_tables[0].local_gateway_route_table_id #=> String
    #   resp.local_gateway_route_tables[0].local_gateway_route_table_arn #=> String
    #   resp.local_gateway_route_tables[0].local_gateway_id #=> String
    #   resp.local_gateway_route_tables[0].outpost_arn #=> String
    #   resp.local_gateway_route_tables[0].owner_id #=> String
    #   resp.local_gateway_route_tables[0].state #=> String
    #   resp.local_gateway_route_tables[0].tags #=> Array
    #   resp.local_gateway_route_tables[0].tags[0].key #=> String
    #   resp.local_gateway_route_tables[0].tags[0].value #=> String
    #   resp.local_gateway_route_tables[0].mode #=> String, one of "direct-vpc-routing", "coip"
    #   resp.local_gateway_route_tables[0].state_reason.code #=> String
    #   resp.local_gateway_route_tables[0].state_reason.message #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeLocalGatewayRouteTables AWS API Documentation
    #
    # @overload describe_local_gateway_route_tables(params = {})
    # @param [Hash] params ({})
    def describe_local_gateway_route_tables(params = {}, options = {})
      req = build_request(:describe_local_gateway_route_tables, params)
      req.send_request(options)
    end

    # Describes the specified local gateway virtual interface groups.
    #
    # @option params [Array<String>] :local_gateway_virtual_interface_group_ids
    #   The IDs of the virtual interface groups.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `local-gateway-id` - The ID of a local gateway.
    #
    #   * `local-gateway-virtual-interface-group-id` - The ID of the virtual
    #     interface group.
    #
    #   * `local-gateway-virtual-interface-id` - The ID of the virtual
    #     interface.
    #
    #   * `owner-id` - The ID of the Amazon Web Services account that owns the
    #     local gateway virtual interface group.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeLocalGatewayVirtualInterfaceGroupsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeLocalGatewayVirtualInterfaceGroupsResult#local_gateway_virtual_interface_groups #local_gateway_virtual_interface_groups} => Array&lt;Types::LocalGatewayVirtualInterfaceGroup&gt;
    #   * {Types::DescribeLocalGatewayVirtualInterfaceGroupsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_local_gateway_virtual_interface_groups({
    #     local_gateway_virtual_interface_group_ids: ["LocalGatewayVirtualInterfaceGroupId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.local_gateway_virtual_interface_groups #=> Array
    #   resp.local_gateway_virtual_interface_groups[0].local_gateway_virtual_interface_group_id #=> String
    #   resp.local_gateway_virtual_interface_groups[0].local_gateway_virtual_interface_ids #=> Array
    #   resp.local_gateway_virtual_interface_groups[0].local_gateway_virtual_interface_ids[0] #=> String
    #   resp.local_gateway_virtual_interface_groups[0].local_gateway_id #=> String
    #   resp.local_gateway_virtual_interface_groups[0].owner_id #=> String
    #   resp.local_gateway_virtual_interface_groups[0].tags #=> Array
    #   resp.local_gateway_virtual_interface_groups[0].tags[0].key #=> String
    #   resp.local_gateway_virtual_interface_groups[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeLocalGatewayVirtualInterfaceGroups AWS API Documentation
    #
    # @overload describe_local_gateway_virtual_interface_groups(params = {})
    # @param [Hash] params ({})
    def describe_local_gateway_virtual_interface_groups(params = {}, options = {})
      req = build_request(:describe_local_gateway_virtual_interface_groups, params)
      req.send_request(options)
    end

    # Describes the specified local gateway virtual interfaces.
    #
    # @option params [Array<String>] :local_gateway_virtual_interface_ids
    #   The IDs of the virtual interfaces.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `local-address` - The local address.
    #
    #   * `local-bgp-asn` - The Border Gateway Protocol (BGP) Autonomous
    #     System Number (ASN) of the local gateway.
    #
    #   * `local-gateway-id` - The ID of the local gateway.
    #
    #   * `local-gateway-virtual-interface-id` - The ID of the virtual
    #     interface.
    #
    #   * `owner-id` - The ID of the Amazon Web Services account that owns the
    #     local gateway virtual interface.
    #
    #   * `peer-address` - The peer address.
    #
    #   * `peer-bgp-asn` - The peer BGP ASN.
    #
    #   * `vlan` - The ID of the VLAN.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeLocalGatewayVirtualInterfacesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeLocalGatewayVirtualInterfacesResult#local_gateway_virtual_interfaces #local_gateway_virtual_interfaces} => Array&lt;Types::LocalGatewayVirtualInterface&gt;
    #   * {Types::DescribeLocalGatewayVirtualInterfacesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_local_gateway_virtual_interfaces({
    #     local_gateway_virtual_interface_ids: ["LocalGatewayVirtualInterfaceId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.local_gateway_virtual_interfaces #=> Array
    #   resp.local_gateway_virtual_interfaces[0].local_gateway_virtual_interface_id #=> String
    #   resp.local_gateway_virtual_interfaces[0].local_gateway_id #=> String
    #   resp.local_gateway_virtual_interfaces[0].vlan #=> Integer
    #   resp.local_gateway_virtual_interfaces[0].local_address #=> String
    #   resp.local_gateway_virtual_interfaces[0].peer_address #=> String
    #   resp.local_gateway_virtual_interfaces[0].local_bgp_asn #=> Integer
    #   resp.local_gateway_virtual_interfaces[0].peer_bgp_asn #=> Integer
    #   resp.local_gateway_virtual_interfaces[0].owner_id #=> String
    #   resp.local_gateway_virtual_interfaces[0].tags #=> Array
    #   resp.local_gateway_virtual_interfaces[0].tags[0].key #=> String
    #   resp.local_gateway_virtual_interfaces[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeLocalGatewayVirtualInterfaces AWS API Documentation
    #
    # @overload describe_local_gateway_virtual_interfaces(params = {})
    # @param [Hash] params ({})
    def describe_local_gateway_virtual_interfaces(params = {}, options = {})
      req = build_request(:describe_local_gateway_virtual_interfaces, params)
      req.send_request(options)
    end

    # Describes one or more local gateways. By default, all local gateways
    # are described. Alternatively, you can filter the results.
    #
    # @option params [Array<String>] :local_gateway_ids
    #   The IDs of the local gateways.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `local-gateway-id` - The ID of a local gateway.
    #
    #   * `outpost-arn` - The Amazon Resource Name (ARN) of the Outpost.
    #
    #   * `owner-id` - The ID of the Amazon Web Services account that owns the
    #     local gateway.
    #
    #   * `state` - The state of the association.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeLocalGatewaysResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeLocalGatewaysResult#local_gateways #local_gateways} => Array&lt;Types::LocalGateway&gt;
    #   * {Types::DescribeLocalGatewaysResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_local_gateways({
    #     local_gateway_ids: ["LocalGatewayId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.local_gateways #=> Array
    #   resp.local_gateways[0].local_gateway_id #=> String
    #   resp.local_gateways[0].outpost_arn #=> String
    #   resp.local_gateways[0].owner_id #=> String
    #   resp.local_gateways[0].state #=> String
    #   resp.local_gateways[0].tags #=> Array
    #   resp.local_gateways[0].tags[0].key #=> String
    #   resp.local_gateways[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeLocalGateways AWS API Documentation
    #
    # @overload describe_local_gateways(params = {})
    # @param [Hash] params ({})
    def describe_local_gateways(params = {}, options = {})
      req = build_request(:describe_local_gateways, params)
      req.send_request(options)
    end

    # Describes the lock status for a snapshot.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `lock-state` - The state of the snapshot lock (`compliance-cooloff`
    #     \| `governance` \| `compliance` \| `expired`).
    #
    #   ^
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Array<String>] :snapshot_ids
    #   The IDs of the snapshots for which to view the lock status.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeLockedSnapshotsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeLockedSnapshotsResult#snapshots #snapshots} => Array&lt;Types::LockedSnapshotsInfo&gt;
    #   * {Types::DescribeLockedSnapshotsResult#next_token #next_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_locked_snapshots({
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     snapshot_ids: ["SnapshotId"],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.snapshots #=> Array
    #   resp.snapshots[0].owner_id #=> String
    #   resp.snapshots[0].snapshot_id #=> String
    #   resp.snapshots[0].lock_state #=> String, one of "compliance", "governance", "compliance-cooloff", "expired"
    #   resp.snapshots[0].lock_duration #=> Integer
    #   resp.snapshots[0].cool_off_period #=> Integer
    #   resp.snapshots[0].cool_off_period_expires_on #=> Time
    #   resp.snapshots[0].lock_created_on #=> Time
    #   resp.snapshots[0].lock_duration_start_time #=> Time
    #   resp.snapshots[0].lock_expires_on #=> Time
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeLockedSnapshots AWS API Documentation
    #
    # @overload describe_locked_snapshots(params = {})
    # @param [Hash] params ({})
    def describe_locked_snapshots(params = {}, options = {})
      req = build_request(:describe_locked_snapshots, params)
      req.send_request(options)
    end

    # Describes the specified EC2 Mac Dedicated Host or all of your EC2 Mac
    # Dedicated Hosts.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `availability-zone` - The Availability Zone of the EC2 Mac Dedicated
    #     Host.
    #
    #   * `instance-type` - The instance type size that the EC2 Mac Dedicated
    #     Host is configured to support.
    #
    # @option params [Array<String>] :host_ids
    #   The IDs of the EC2 Mac Dedicated Hosts.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return for the request in a single
    #   page. The remaining results can be seen by sending another request
    #   with the returned `nextToken` value. This value can be between 5 and
    #   500. If `maxResults` is given a larger value than 500, you receive an
    #   error.
    #
    # @option params [String] :next_token
    #   The token to use to retrieve the next page of results.
    #
    # @return [Types::DescribeMacHostsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeMacHostsResult#mac_hosts #mac_hosts} => Array&lt;Types::MacHost&gt;
    #   * {Types::DescribeMacHostsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_mac_hosts({
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     host_ids: ["DedicatedHostId"],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.mac_hosts #=> Array
    #   resp.mac_hosts[0].host_id #=> String
    #   resp.mac_hosts[0].mac_os_latest_supported_versions #=> Array
    #   resp.mac_hosts[0].mac_os_latest_supported_versions[0] #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeMacHosts AWS API Documentation
    #
    # @overload describe_mac_hosts(params = {})
    # @param [Hash] params ({})
    def describe_mac_hosts(params = {}, options = {})
      req = build_request(:describe_mac_hosts, params)
      req.send_request(options)
    end

    # Describes your managed prefix lists and any Amazon Web
    # Services-managed prefix lists.
    #
    # To view the entries for your prefix list, use
    # GetManagedPrefixListEntries.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `owner-id` - The ID of the prefix list owner.
    #
    #   * `prefix-list-id` - The ID of the prefix list.
    #
    #   * `prefix-list-name` - The name of the prefix list.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Array<String>] :prefix_list_ids
    #   One or more prefix list IDs.
    #
    # @return [Types::DescribeManagedPrefixListsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeManagedPrefixListsResult#next_token #next_token} => String
    #   * {Types::DescribeManagedPrefixListsResult#prefix_lists #prefix_lists} => Array&lt;Types::ManagedPrefixList&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_managed_prefix_lists({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #     prefix_list_ids: ["String"],
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.prefix_lists #=> Array
    #   resp.prefix_lists[0].prefix_list_id #=> String
    #   resp.prefix_lists[0].address_family #=> String
    #   resp.prefix_lists[0].state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "restore-in-progress", "restore-complete", "restore-failed", "delete-in-progress", "delete-complete", "delete-failed"
    #   resp.prefix_lists[0].state_message #=> String
    #   resp.prefix_lists[0].prefix_list_arn #=> String
    #   resp.prefix_lists[0].prefix_list_name #=> String
    #   resp.prefix_lists[0].max_entries #=> Integer
    #   resp.prefix_lists[0].version #=> Integer
    #   resp.prefix_lists[0].tags #=> Array
    #   resp.prefix_lists[0].tags[0].key #=> String
    #   resp.prefix_lists[0].tags[0].value #=> String
    #   resp.prefix_lists[0].owner_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeManagedPrefixLists AWS API Documentation
    #
    # @overload describe_managed_prefix_lists(params = {})
    # @param [Hash] params ({})
    def describe_managed_prefix_lists(params = {}, options = {})
      req = build_request(:describe_managed_prefix_lists, params)
      req.send_request(options)
    end

    # <note markdown="1"> This action is deprecated.
    #
    #  </note>
    #
    # Describes your Elastic IP addresses that are being moved from or being
    # restored to the EC2-Classic platform. This request does not return
    # information about any other Elastic IP addresses in your account.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :public_ips
    #   One or more Elastic IP addresses.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `moving-status` - The status of the Elastic IP address
    #     (`MovingToVpc` \| `RestoringToClassic`).
    #
    #   ^
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return for the request in a single
    #   page. The remaining results of the initial request can be seen by
    #   sending another request with the returned `NextToken` value. This
    #   value can be between 5 and 1000; if `MaxResults` is given a value
    #   outside of this range, an error is returned.
    #
    #   Default: If no value is provided, the default is 1000.
    #
    # @return [Types::DescribeMovingAddressesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeMovingAddressesResult#moving_address_statuses #moving_address_statuses} => Array&lt;Types::MovingAddressStatus&gt;
    #   * {Types::DescribeMovingAddressesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe your moving addresses
    #
    #   # This example describes all of your moving Elastic IP addresses.
    #
    #   resp = client.describe_moving_addresses({
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     moving_address_statuses: [
    #       {
    #         move_status: "movingToVpc", 
    #         public_ip: "198.51.100.0", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_moving_addresses({
    #     dry_run: false,
    #     public_ips: ["String"],
    #     next_token: "String",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.moving_address_statuses #=> Array
    #   resp.moving_address_statuses[0].move_status #=> String, one of "movingToVpc", "restoringToClassic"
    #   resp.moving_address_statuses[0].public_ip #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeMovingAddresses AWS API Documentation
    #
    # @overload describe_moving_addresses(params = {})
    # @param [Hash] params ({})
    def describe_moving_addresses(params = {}, options = {})
      req = build_request(:describe_moving_addresses, params)
      req.send_request(options)
    end

    # Describes your NAT gateways. The default is to describe all your NAT
    # gateways. Alternatively, you can specify specific NAT gateway IDs or
    # filter the results to include only the NAT gateways that match
    # specific criteria.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filter
    #   The filters.
    #
    #   * `nat-gateway-id` - The ID of the NAT gateway.
    #
    #   * `state` - The state of the NAT gateway (`pending` \| `failed` \|
    #     `available` \| `deleting` \| `deleted`).
    #
    #   * `subnet-id` - The ID of the subnet in which the NAT gateway resides.
    #
    #   * `tag` - The key/value combination of a tag assigned to the resource.
    #     Use the tag key in the filter name and the tag value as the filter
    #     value. For example, to find all resources that have a tag with the
    #     key `Owner` and the value `TeamA`, specify `tag:Owner` for the
    #     filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `vpc-id` - The ID of the VPC in which the NAT gateway resides.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Array<String>] :nat_gateway_ids
    #   The IDs of the NAT gateways.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @return [Types::DescribeNatGatewaysResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeNatGatewaysResult#nat_gateways #nat_gateways} => Array&lt;Types::NatGateway&gt;
    #   * {Types::DescribeNatGatewaysResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe a NAT gateway
    #
    #   # This example describes the NAT gateway for the specified VPC.
    #
    #   resp = client.describe_nat_gateways({
    #     filter: [
    #       {
    #         name: "vpc-id", 
    #         values: [
    #           "vpc-1a2b3c4d", 
    #         ], 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     nat_gateways: [
    #       {
    #         create_time: Time.parse("2015-12-01T12:26:55.983Z"), 
    #         nat_gateway_addresses: [
    #           {
    #             allocation_id: "eipalloc-89c620ec", 
    #             network_interface_id: "eni-9dec76cd", 
    #             private_ip: "10.0.0.149", 
    #             public_ip: "198.11.222.333", 
    #           }, 
    #         ], 
    #         nat_gateway_id: "nat-05dba92075d71c408", 
    #         state: "available", 
    #         subnet_id: "subnet-847e4dc2", 
    #         vpc_id: "vpc-1a2b3c4d", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_nat_gateways({
    #     dry_run: false,
    #     filter: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     nat_gateway_ids: ["NatGatewayId"],
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.nat_gateways #=> Array
    #   resp.nat_gateways[0].create_time #=> Time
    #   resp.nat_gateways[0].delete_time #=> Time
    #   resp.nat_gateways[0].failure_code #=> String
    #   resp.nat_gateways[0].failure_message #=> String
    #   resp.nat_gateways[0].nat_gateway_addresses #=> Array
    #   resp.nat_gateways[0].nat_gateway_addresses[0].allocation_id #=> String
    #   resp.nat_gateways[0].nat_gateway_addresses[0].network_interface_id #=> String
    #   resp.nat_gateways[0].nat_gateway_addresses[0].private_ip #=> String
    #   resp.nat_gateways[0].nat_gateway_addresses[0].public_ip #=> String
    #   resp.nat_gateways[0].nat_gateway_addresses[0].association_id #=> String
    #   resp.nat_gateways[0].nat_gateway_addresses[0].is_primary #=> Boolean
    #   resp.nat_gateways[0].nat_gateway_addresses[0].failure_message #=> String
    #   resp.nat_gateways[0].nat_gateway_addresses[0].status #=> String, one of "assigning", "unassigning", "associating", "disassociating", "succeeded", "failed"
    #   resp.nat_gateways[0].nat_gateway_id #=> String
    #   resp.nat_gateways[0].provisioned_bandwidth.provision_time #=> Time
    #   resp.nat_gateways[0].provisioned_bandwidth.provisioned #=> String
    #   resp.nat_gateways[0].provisioned_bandwidth.request_time #=> Time
    #   resp.nat_gateways[0].provisioned_bandwidth.requested #=> String
    #   resp.nat_gateways[0].provisioned_bandwidth.status #=> String
    #   resp.nat_gateways[0].state #=> String, one of "pending", "failed", "available", "deleting", "deleted"
    #   resp.nat_gateways[0].subnet_id #=> String
    #   resp.nat_gateways[0].vpc_id #=> String
    #   resp.nat_gateways[0].tags #=> Array
    #   resp.nat_gateways[0].tags[0].key #=> String
    #   resp.nat_gateways[0].tags[0].value #=> String
    #   resp.nat_gateways[0].connectivity_type #=> String, one of "private", "public"
    #   resp.next_token #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * nat_gateway_available
    #   * nat_gateway_deleted
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeNatGateways AWS API Documentation
    #
    # @overload describe_nat_gateways(params = {})
    # @param [Hash] params ({})
    def describe_nat_gateways(params = {}, options = {})
      req = build_request(:describe_nat_gateways, params)
      req.send_request(options)
    end

    # Describes your network ACLs. The default is to describe all your
    # network ACLs. Alternatively, you can specify specific network ACL IDs
    # or filter the results to include only the network ACLs that match
    # specific criteria.
    #
    # For more information, see [Network ACLs][1] in the *Amazon VPC User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/vpc-network-acls.html
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :network_acl_ids
    #   The IDs of the network ACLs.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `association.association-id` - The ID of an association ID for the
    #     ACL.
    #
    #   * `association.network-acl-id` - The ID of the network ACL involved in
    #     the association.
    #
    #   * `association.subnet-id` - The ID of the subnet involved in the
    #     association.
    #
    #   * `default` - Indicates whether the ACL is the default network ACL for
    #     the VPC.
    #
    #   * `entry.cidr` - The IPv4 CIDR range specified in the entry.
    #
    #   * `entry.icmp.code` - The ICMP code specified in the entry, if any.
    #
    #   * `entry.icmp.type` - The ICMP type specified in the entry, if any.
    #
    #   * `entry.ipv6-cidr` - The IPv6 CIDR range specified in the entry.
    #
    #   * `entry.port-range.from` - The start of the port range specified in
    #     the entry.
    #
    #   * `entry.port-range.to` - The end of the port range specified in the
    #     entry.
    #
    #   * `entry.protocol` - The protocol specified in the entry (`tcp` \|
    #     `udp` \| `icmp` or a protocol number).
    #
    #   * `entry.rule-action` - Allows or denies the matching traffic (`allow`
    #     \| `deny`).
    #
    #   * `entry.egress` - A Boolean that indicates the type of rule. Specify
    #     `true` for egress rules, or `false` for ingress rules.
    #
    #   * `entry.rule-number` - The number of an entry (in other words, rule)
    #     in the set of ACL entries.
    #
    #   * `network-acl-id` - The ID of the network ACL.
    #
    #   * `owner-id` - The ID of the Amazon Web Services account that owns the
    #     network ACL.
    #
    #   * `tag` - The key/value combination of a tag assigned to the resource.
    #     Use the tag key in the filter name and the tag value as the filter
    #     value. For example, to find all resources that have a tag with the
    #     key `Owner` and the value `TeamA`, specify `tag:Owner` for the
    #     filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `vpc-id` - The ID of the VPC for the network ACL.
    #
    # @return [Types::DescribeNetworkAclsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeNetworkAclsResult#network_acls #network_acls} => Array&lt;Types::NetworkAcl&gt;
    #   * {Types::DescribeNetworkAclsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe a network ACL
    #
    #   # This example describes the specified network ACL.
    #
    #   resp = client.describe_network_acls({
    #     network_acl_ids: [
    #       "acl-5fb85d36", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     network_acls: [
    #       {
    #         associations: [
    #           {
    #             network_acl_association_id: "aclassoc-66ea5f0b", 
    #             network_acl_id: "acl-9aeb5ef7", 
    #             subnet_id: "subnet-65ea5f08", 
    #           }, 
    #         ], 
    #         entries: [
    #           {
    #             cidr_block: "0.0.0.0/0", 
    #             egress: true, 
    #             protocol: "-1", 
    #             rule_action: "deny", 
    #             rule_number: 32767, 
    #           }, 
    #           {
    #             cidr_block: "0.0.0.0/0", 
    #             egress: false, 
    #             protocol: "-1", 
    #             rule_action: "deny", 
    #             rule_number: 32767, 
    #           }, 
    #         ], 
    #         is_default: false, 
    #         network_acl_id: "acl-5fb85d36", 
    #         tags: [
    #         ], 
    #         vpc_id: "vpc-a01106c2", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_network_acls({
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #     network_acl_ids: ["NetworkAclId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.network_acls #=> Array
    #   resp.network_acls[0].associations #=> Array
    #   resp.network_acls[0].associations[0].network_acl_association_id #=> String
    #   resp.network_acls[0].associations[0].network_acl_id #=> String
    #   resp.network_acls[0].associations[0].subnet_id #=> String
    #   resp.network_acls[0].entries #=> Array
    #   resp.network_acls[0].entries[0].cidr_block #=> String
    #   resp.network_acls[0].entries[0].egress #=> Boolean
    #   resp.network_acls[0].entries[0].icmp_type_code.code #=> Integer
    #   resp.network_acls[0].entries[0].icmp_type_code.type #=> Integer
    #   resp.network_acls[0].entries[0].ipv_6_cidr_block #=> String
    #   resp.network_acls[0].entries[0].port_range.from #=> Integer
    #   resp.network_acls[0].entries[0].port_range.to #=> Integer
    #   resp.network_acls[0].entries[0].protocol #=> String
    #   resp.network_acls[0].entries[0].rule_action #=> String, one of "allow", "deny"
    #   resp.network_acls[0].entries[0].rule_number #=> Integer
    #   resp.network_acls[0].is_default #=> Boolean
    #   resp.network_acls[0].network_acl_id #=> String
    #   resp.network_acls[0].tags #=> Array
    #   resp.network_acls[0].tags[0].key #=> String
    #   resp.network_acls[0].tags[0].value #=> String
    #   resp.network_acls[0].vpc_id #=> String
    #   resp.network_acls[0].owner_id #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeNetworkAcls AWS API Documentation
    #
    # @overload describe_network_acls(params = {})
    # @param [Hash] params ({})
    def describe_network_acls(params = {}, options = {})
      req = build_request(:describe_network_acls, params)
      req.send_request(options)
    end

    # Describes the specified Network Access Scope analyses.
    #
    # @option params [Array<String>] :network_insights_access_scope_analysis_ids
    #   The IDs of the Network Access Scope analyses.
    #
    # @option params [String] :network_insights_access_scope_id
    #   The ID of the Network Access Scope.
    #
    # @option params [Time,DateTime,Date,Integer,String] :analysis_start_time_begin
    #   Filters the results based on the start time. The analysis must have
    #   started on or after this time.
    #
    # @option params [Time,DateTime,Date,Integer,String] :analysis_start_time_end
    #   Filters the results based on the start time. The analysis must have
    #   started on or before this time.
    #
    # @option params [Array<Types::Filter>] :filters
    #   There are no supported filters.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::DescribeNetworkInsightsAccessScopeAnalysesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeNetworkInsightsAccessScopeAnalysesResult#network_insights_access_scope_analyses #network_insights_access_scope_analyses} => Array&lt;Types::NetworkInsightsAccessScopeAnalysis&gt;
    #   * {Types::DescribeNetworkInsightsAccessScopeAnalysesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_network_insights_access_scope_analyses({
    #     network_insights_access_scope_analysis_ids: ["NetworkInsightsAccessScopeAnalysisId"],
    #     network_insights_access_scope_id: "NetworkInsightsAccessScopeId",
    #     analysis_start_time_begin: Time.now,
    #     analysis_start_time_end: Time.now,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     dry_run: false,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.network_insights_access_scope_analyses #=> Array
    #   resp.network_insights_access_scope_analyses[0].network_insights_access_scope_analysis_id #=> String
    #   resp.network_insights_access_scope_analyses[0].network_insights_access_scope_analysis_arn #=> String
    #   resp.network_insights_access_scope_analyses[0].network_insights_access_scope_id #=> String
    #   resp.network_insights_access_scope_analyses[0].status #=> String, one of "running", "succeeded", "failed"
    #   resp.network_insights_access_scope_analyses[0].status_message #=> String
    #   resp.network_insights_access_scope_analyses[0].warning_message #=> String
    #   resp.network_insights_access_scope_analyses[0].start_date #=> Time
    #   resp.network_insights_access_scope_analyses[0].end_date #=> Time
    #   resp.network_insights_access_scope_analyses[0].findings_found #=> String, one of "true", "false", "unknown"
    #   resp.network_insights_access_scope_analyses[0].analyzed_eni_count #=> Integer
    #   resp.network_insights_access_scope_analyses[0].tags #=> Array
    #   resp.network_insights_access_scope_analyses[0].tags[0].key #=> String
    #   resp.network_insights_access_scope_analyses[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeNetworkInsightsAccessScopeAnalyses AWS API Documentation
    #
    # @overload describe_network_insights_access_scope_analyses(params = {})
    # @param [Hash] params ({})
    def describe_network_insights_access_scope_analyses(params = {}, options = {})
      req = build_request(:describe_network_insights_access_scope_analyses, params)
      req.send_request(options)
    end

    # Describes the specified Network Access Scopes.
    #
    # @option params [Array<String>] :network_insights_access_scope_ids
    #   The IDs of the Network Access Scopes.
    #
    # @option params [Array<Types::Filter>] :filters
    #   There are no supported filters.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::DescribeNetworkInsightsAccessScopesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeNetworkInsightsAccessScopesResult#network_insights_access_scopes #network_insights_access_scopes} => Array&lt;Types::NetworkInsightsAccessScope&gt;
    #   * {Types::DescribeNetworkInsightsAccessScopesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_network_insights_access_scopes({
    #     network_insights_access_scope_ids: ["NetworkInsightsAccessScopeId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     dry_run: false,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.network_insights_access_scopes #=> Array
    #   resp.network_insights_access_scopes[0].network_insights_access_scope_id #=> String
    #   resp.network_insights_access_scopes[0].network_insights_access_scope_arn #=> String
    #   resp.network_insights_access_scopes[0].created_date #=> Time
    #   resp.network_insights_access_scopes[0].updated_date #=> Time
    #   resp.network_insights_access_scopes[0].tags #=> Array
    #   resp.network_insights_access_scopes[0].tags[0].key #=> String
    #   resp.network_insights_access_scopes[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeNetworkInsightsAccessScopes AWS API Documentation
    #
    # @overload describe_network_insights_access_scopes(params = {})
    # @param [Hash] params ({})
    def describe_network_insights_access_scopes(params = {}, options = {})
      req = build_request(:describe_network_insights_access_scopes, params)
      req.send_request(options)
    end

    # Describes one or more of your network insights analyses.
    #
    # @option params [Array<String>] :network_insights_analysis_ids
    #   The ID of the network insights analyses. You must specify either
    #   analysis IDs or a path ID.
    #
    # @option params [String] :network_insights_path_id
    #   The ID of the path. You must specify either a path ID or analysis IDs.
    #
    # @option params [Time,DateTime,Date,Integer,String] :analysis_start_time
    #   The time when the network insights analyses started.
    #
    # @option params [Time,DateTime,Date,Integer,String] :analysis_end_time
    #   The time when the network insights analyses ended.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters. The following are the possible values:
    #
    #   * path-found - A Boolean value that indicates whether a feasible path
    #     is found.
    #
    #   * status - The status of the analysis (running \| succeeded \|
    #     failed).
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::DescribeNetworkInsightsAnalysesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeNetworkInsightsAnalysesResult#network_insights_analyses #network_insights_analyses} => Array&lt;Types::NetworkInsightsAnalysis&gt;
    #   * {Types::DescribeNetworkInsightsAnalysesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_network_insights_analyses({
    #     network_insights_analysis_ids: ["NetworkInsightsAnalysisId"],
    #     network_insights_path_id: "NetworkInsightsPathId",
    #     analysis_start_time: Time.now,
    #     analysis_end_time: Time.now,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     dry_run: false,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.network_insights_analyses #=> Array
    #   resp.network_insights_analyses[0].network_insights_analysis_id #=> String
    #   resp.network_insights_analyses[0].network_insights_analysis_arn #=> String
    #   resp.network_insights_analyses[0].network_insights_path_id #=> String
    #   resp.network_insights_analyses[0].additional_accounts #=> Array
    #   resp.network_insights_analyses[0].additional_accounts[0] #=> String
    #   resp.network_insights_analyses[0].filter_in_arns #=> Array
    #   resp.network_insights_analyses[0].filter_in_arns[0] #=> String
    #   resp.network_insights_analyses[0].start_date #=> Time
    #   resp.network_insights_analyses[0].status #=> String, one of "running", "succeeded", "failed"
    #   resp.network_insights_analyses[0].status_message #=> String
    #   resp.network_insights_analyses[0].warning_message #=> String
    #   resp.network_insights_analyses[0].network_path_found #=> Boolean
    #   resp.network_insights_analyses[0].forward_path_components #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].sequence_number #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].acl_rule.cidr #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].acl_rule.egress #=> Boolean
    #   resp.network_insights_analyses[0].forward_path_components[0].acl_rule.port_range.from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].acl_rule.port_range.to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].acl_rule.protocol #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].acl_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].acl_rule.rule_number #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].attached_to.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].attached_to.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].attached_to.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].component.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].component.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].component.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].destination_vpc.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].destination_vpc.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].destination_vpc.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].outbound_header.destination_addresses #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].outbound_header.destination_addresses[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].outbound_header.destination_port_ranges #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].outbound_header.destination_port_ranges[0].from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].outbound_header.destination_port_ranges[0].to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].outbound_header.protocol #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].outbound_header.source_addresses #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].outbound_header.source_addresses[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].outbound_header.source_port_ranges #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].outbound_header.source_port_ranges[0].from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].outbound_header.source_port_ranges[0].to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].inbound_header.destination_addresses #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].inbound_header.destination_addresses[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].inbound_header.destination_port_ranges #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].inbound_header.destination_port_ranges[0].from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].inbound_header.destination_port_ranges[0].to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].inbound_header.protocol #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].inbound_header.source_addresses #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].inbound_header.source_addresses[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].inbound_header.source_port_ranges #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].inbound_header.source_port_ranges[0].from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].inbound_header.source_port_ranges[0].to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].route_table_route.destination_cidr #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].route_table_route.destination_prefix_list_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].route_table_route.egress_only_internet_gateway_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].route_table_route.gateway_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].route_table_route.instance_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].route_table_route.nat_gateway_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].route_table_route.network_interface_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].route_table_route.origin #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].route_table_route.transit_gateway_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].route_table_route.vpc_peering_connection_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].route_table_route.state #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].route_table_route.carrier_gateway_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].route_table_route.core_network_arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].route_table_route.local_gateway_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].security_group_rule.cidr #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].security_group_rule.direction #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].security_group_rule.security_group_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].security_group_rule.port_range.from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].security_group_rule.port_range.to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].security_group_rule.prefix_list_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].security_group_rule.protocol #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].source_vpc.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].source_vpc.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].source_vpc.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].subnet.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].subnet.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].subnet.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].vpc.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].vpc.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].vpc.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].additional_detail_type #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].component.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].component.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].component.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].vpc_endpoint_service.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].vpc_endpoint_service.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].vpc_endpoint_service.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].rule_options #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].rule_options[0].keyword #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].rule_options[0].settings #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].rule_options[0].settings[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].rule_group_type_pairs #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].rule_group_type_pairs[0].rule_group_arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].rule_group_type_pairs[0].rule_group_type #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].rule_group_rule_options_pairs #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_group_arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].keyword #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].settings #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].settings[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].service_name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].load_balancers #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].load_balancers[0].id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].load_balancers[0].arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].additional_details[0].load_balancers[0].name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].transit_gateway.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].transit_gateway.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].transit_gateway.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].transit_gateway_route_table_route.destination_cidr #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].transit_gateway_route_table_route.state #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].transit_gateway_route_table_route.route_origin #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].transit_gateway_route_table_route.prefix_list_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].transit_gateway_route_table_route.attachment_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].transit_gateway_route_table_route.resource_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].transit_gateway_route_table_route.resource_type #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].acl.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].acl.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].acl.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].acl_rule.cidr #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].acl_rule.egress #=> Boolean
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].acl_rule.port_range.from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].acl_rule.port_range.to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].acl_rule.protocol #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].acl_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].acl_rule.rule_number #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].address #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].addresses #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].addresses[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].attached_to.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].attached_to.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].attached_to.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].availability_zones #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].availability_zones[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].cidrs #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].cidrs[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].component.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].component.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].component.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].customer_gateway.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].customer_gateway.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].customer_gateway.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].destination.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].destination.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].destination.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].destination_vpc.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].destination_vpc.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].destination_vpc.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].direction #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].explanation_code #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].ingress_route_table.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].ingress_route_table.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].ingress_route_table.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].internet_gateway.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].internet_gateway.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].internet_gateway.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].classic_load_balancer_listener.load_balancer_port #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].classic_load_balancer_listener.instance_port #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_listener_port #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_target.address #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_target.availability_zone #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_target.instance.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_target.instance.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_target.instance.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_target.port #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_target_group.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_target_group.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_target_group.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_target_groups #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_target_groups[0].id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_target_groups[0].arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_target_groups[0].name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].load_balancer_target_port #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].elastic_load_balancer_listener.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].elastic_load_balancer_listener.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].elastic_load_balancer_listener.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].missing_component #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].nat_gateway.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].nat_gateway.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].nat_gateway.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].network_interface.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].network_interface.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].network_interface.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].packet_field #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpc_peering_connection.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpc_peering_connection.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpc_peering_connection.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].port #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].port_ranges #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].port_ranges[0].from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].port_ranges[0].to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].prefix_list.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].prefix_list.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].prefix_list.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].protocols #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].protocols[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table_route.destination_cidr #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table_route.destination_prefix_list_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table_route.egress_only_internet_gateway_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table_route.gateway_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table_route.instance_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table_route.nat_gateway_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table_route.network_interface_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table_route.origin #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table_route.transit_gateway_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table_route.vpc_peering_connection_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table_route.state #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table_route.carrier_gateway_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table_route.core_network_arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table_route.local_gateway_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].route_table.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].security_group.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].security_group.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].security_group.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].security_group_rule.cidr #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].security_group_rule.direction #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].security_group_rule.security_group_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].security_group_rule.port_range.from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].security_group_rule.port_range.to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].security_group_rule.prefix_list_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].security_group_rule.protocol #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].security_groups #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].security_groups[0].id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].security_groups[0].arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].security_groups[0].name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].source_vpc.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].source_vpc.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].source_vpc.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].state #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].subnet.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].subnet.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].subnet.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].subnet_route_table.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].subnet_route_table.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].subnet_route_table.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpc.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpc.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpc.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpc_endpoint.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpc_endpoint.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpc_endpoint.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpn_connection.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpn_connection.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpn_connection.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpn_gateway.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpn_gateway.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].vpn_gateway.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway_route_table.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway_route_table.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway_route_table.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway_route_table_route.destination_cidr #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway_route_table_route.state #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway_route_table_route.route_origin #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway_route_table_route.prefix_list_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway_route_table_route.attachment_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway_route_table_route.resource_id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway_route_table_route.resource_type #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway_attachment.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway_attachment.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].transit_gateway_attachment.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].component_account #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].component_region #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.rule_group_arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.sources #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.sources[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.destinations #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.destinations[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.source_ports #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.destination_ports #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.protocols #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.protocols[0] #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateless_rule.priority #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateful_rule.rule_group_arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateful_rule.sources #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateful_rule.sources[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateful_rule.destinations #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateful_rule.destinations[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateful_rule.source_ports #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateful_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateful_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateful_rule.destination_ports #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateful_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateful_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateful_rule.protocol #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateful_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].explanations[0].firewall_stateful_rule.direction #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].elastic_load_balancer_listener.id #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].elastic_load_balancer_listener.arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].elastic_load_balancer_listener.name #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.rule_group_arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.sources #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.sources[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.destinations #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.destinations[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.source_ports #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.destination_ports #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.protocols #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.protocols[0] #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateless_rule.priority #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateful_rule.rule_group_arn #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateful_rule.sources #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateful_rule.sources[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateful_rule.destinations #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateful_rule.destinations[0] #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateful_rule.source_ports #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateful_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateful_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateful_rule.destination_ports #=> Array
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateful_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateful_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateful_rule.protocol #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateful_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].firewall_stateful_rule.direction #=> String
    #   resp.network_insights_analyses[0].forward_path_components[0].service_name #=> String
    #   resp.network_insights_analyses[0].return_path_components #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].sequence_number #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].acl_rule.cidr #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].acl_rule.egress #=> Boolean
    #   resp.network_insights_analyses[0].return_path_components[0].acl_rule.port_range.from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].acl_rule.port_range.to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].acl_rule.protocol #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].acl_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].acl_rule.rule_number #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].attached_to.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].attached_to.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].attached_to.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].component.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].component.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].component.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].destination_vpc.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].destination_vpc.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].destination_vpc.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].outbound_header.destination_addresses #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].outbound_header.destination_addresses[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].outbound_header.destination_port_ranges #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].outbound_header.destination_port_ranges[0].from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].outbound_header.destination_port_ranges[0].to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].outbound_header.protocol #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].outbound_header.source_addresses #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].outbound_header.source_addresses[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].outbound_header.source_port_ranges #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].outbound_header.source_port_ranges[0].from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].outbound_header.source_port_ranges[0].to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].inbound_header.destination_addresses #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].inbound_header.destination_addresses[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].inbound_header.destination_port_ranges #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].inbound_header.destination_port_ranges[0].from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].inbound_header.destination_port_ranges[0].to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].inbound_header.protocol #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].inbound_header.source_addresses #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].inbound_header.source_addresses[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].inbound_header.source_port_ranges #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].inbound_header.source_port_ranges[0].from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].inbound_header.source_port_ranges[0].to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].route_table_route.destination_cidr #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].route_table_route.destination_prefix_list_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].route_table_route.egress_only_internet_gateway_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].route_table_route.gateway_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].route_table_route.instance_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].route_table_route.nat_gateway_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].route_table_route.network_interface_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].route_table_route.origin #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].route_table_route.transit_gateway_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].route_table_route.vpc_peering_connection_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].route_table_route.state #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].route_table_route.carrier_gateway_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].route_table_route.core_network_arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].route_table_route.local_gateway_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].security_group_rule.cidr #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].security_group_rule.direction #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].security_group_rule.security_group_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].security_group_rule.port_range.from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].security_group_rule.port_range.to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].security_group_rule.prefix_list_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].security_group_rule.protocol #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].source_vpc.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].source_vpc.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].source_vpc.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].subnet.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].subnet.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].subnet.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].vpc.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].vpc.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].vpc.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].additional_detail_type #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].component.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].component.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].component.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].vpc_endpoint_service.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].vpc_endpoint_service.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].vpc_endpoint_service.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].rule_options #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].rule_options[0].keyword #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].rule_options[0].settings #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].rule_options[0].settings[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].rule_group_type_pairs #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].rule_group_type_pairs[0].rule_group_arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].rule_group_type_pairs[0].rule_group_type #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].rule_group_rule_options_pairs #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_group_arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].keyword #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].settings #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].settings[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].service_name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].load_balancers #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].load_balancers[0].id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].load_balancers[0].arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].additional_details[0].load_balancers[0].name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].transit_gateway.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].transit_gateway.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].transit_gateway.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].transit_gateway_route_table_route.destination_cidr #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].transit_gateway_route_table_route.state #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].transit_gateway_route_table_route.route_origin #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].transit_gateway_route_table_route.prefix_list_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].transit_gateway_route_table_route.attachment_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].transit_gateway_route_table_route.resource_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].transit_gateway_route_table_route.resource_type #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].acl.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].acl.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].acl.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].acl_rule.cidr #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].acl_rule.egress #=> Boolean
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].acl_rule.port_range.from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].acl_rule.port_range.to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].acl_rule.protocol #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].acl_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].acl_rule.rule_number #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].address #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].addresses #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].addresses[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].attached_to.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].attached_to.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].attached_to.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].availability_zones #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].availability_zones[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].cidrs #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].cidrs[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].component.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].component.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].component.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].customer_gateway.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].customer_gateway.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].customer_gateway.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].destination.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].destination.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].destination.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].destination_vpc.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].destination_vpc.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].destination_vpc.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].direction #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].explanation_code #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].ingress_route_table.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].ingress_route_table.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].ingress_route_table.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].internet_gateway.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].internet_gateway.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].internet_gateway.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].classic_load_balancer_listener.load_balancer_port #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].classic_load_balancer_listener.instance_port #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_listener_port #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_target.address #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_target.availability_zone #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_target.instance.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_target.instance.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_target.instance.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_target.port #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_target_group.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_target_group.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_target_group.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_target_groups #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_target_groups[0].id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_target_groups[0].arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_target_groups[0].name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].load_balancer_target_port #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].elastic_load_balancer_listener.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].elastic_load_balancer_listener.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].elastic_load_balancer_listener.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].missing_component #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].nat_gateway.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].nat_gateway.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].nat_gateway.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].network_interface.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].network_interface.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].network_interface.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].packet_field #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpc_peering_connection.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpc_peering_connection.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpc_peering_connection.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].port #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].port_ranges #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].port_ranges[0].from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].port_ranges[0].to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].prefix_list.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].prefix_list.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].prefix_list.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].protocols #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].protocols[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table_route.destination_cidr #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table_route.destination_prefix_list_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table_route.egress_only_internet_gateway_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table_route.gateway_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table_route.instance_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table_route.nat_gateway_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table_route.network_interface_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table_route.origin #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table_route.transit_gateway_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table_route.vpc_peering_connection_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table_route.state #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table_route.carrier_gateway_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table_route.core_network_arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table_route.local_gateway_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].route_table.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].security_group.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].security_group.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].security_group.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].security_group_rule.cidr #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].security_group_rule.direction #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].security_group_rule.security_group_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].security_group_rule.port_range.from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].security_group_rule.port_range.to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].security_group_rule.prefix_list_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].security_group_rule.protocol #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].security_groups #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].security_groups[0].id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].security_groups[0].arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].security_groups[0].name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].source_vpc.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].source_vpc.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].source_vpc.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].state #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].subnet.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].subnet.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].subnet.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].subnet_route_table.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].subnet_route_table.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].subnet_route_table.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpc.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpc.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpc.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpc_endpoint.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpc_endpoint.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpc_endpoint.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpn_connection.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpn_connection.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpn_connection.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpn_gateway.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpn_gateway.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].vpn_gateway.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway_route_table.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway_route_table.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway_route_table.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway_route_table_route.destination_cidr #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway_route_table_route.state #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway_route_table_route.route_origin #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway_route_table_route.prefix_list_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway_route_table_route.attachment_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway_route_table_route.resource_id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway_route_table_route.resource_type #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway_attachment.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway_attachment.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].transit_gateway_attachment.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].component_account #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].component_region #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.rule_group_arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.sources #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.sources[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.destinations #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.destinations[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.source_ports #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.destination_ports #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.protocols #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.protocols[0] #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateless_rule.priority #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateful_rule.rule_group_arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateful_rule.sources #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateful_rule.sources[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateful_rule.destinations #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateful_rule.destinations[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateful_rule.source_ports #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateful_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateful_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateful_rule.destination_ports #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateful_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateful_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateful_rule.protocol #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateful_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].explanations[0].firewall_stateful_rule.direction #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].elastic_load_balancer_listener.id #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].elastic_load_balancer_listener.arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].elastic_load_balancer_listener.name #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.rule_group_arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.sources #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.sources[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.destinations #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.destinations[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.source_ports #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.destination_ports #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.protocols #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.protocols[0] #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateless_rule.priority #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateful_rule.rule_group_arn #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateful_rule.sources #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateful_rule.sources[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateful_rule.destinations #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateful_rule.destinations[0] #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateful_rule.source_ports #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateful_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateful_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateful_rule.destination_ports #=> Array
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateful_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateful_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateful_rule.protocol #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateful_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].firewall_stateful_rule.direction #=> String
    #   resp.network_insights_analyses[0].return_path_components[0].service_name #=> String
    #   resp.network_insights_analyses[0].explanations #=> Array
    #   resp.network_insights_analyses[0].explanations[0].acl.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].acl.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].acl.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].acl_rule.cidr #=> String
    #   resp.network_insights_analyses[0].explanations[0].acl_rule.egress #=> Boolean
    #   resp.network_insights_analyses[0].explanations[0].acl_rule.port_range.from #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].acl_rule.port_range.to #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].acl_rule.protocol #=> String
    #   resp.network_insights_analyses[0].explanations[0].acl_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].explanations[0].acl_rule.rule_number #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].address #=> String
    #   resp.network_insights_analyses[0].explanations[0].addresses #=> Array
    #   resp.network_insights_analyses[0].explanations[0].addresses[0] #=> String
    #   resp.network_insights_analyses[0].explanations[0].attached_to.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].attached_to.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].attached_to.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].availability_zones #=> Array
    #   resp.network_insights_analyses[0].explanations[0].availability_zones[0] #=> String
    #   resp.network_insights_analyses[0].explanations[0].cidrs #=> Array
    #   resp.network_insights_analyses[0].explanations[0].cidrs[0] #=> String
    #   resp.network_insights_analyses[0].explanations[0].component.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].component.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].component.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].customer_gateway.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].customer_gateway.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].customer_gateway.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].destination.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].destination.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].destination.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].destination_vpc.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].destination_vpc.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].destination_vpc.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].direction #=> String
    #   resp.network_insights_analyses[0].explanations[0].explanation_code #=> String
    #   resp.network_insights_analyses[0].explanations[0].ingress_route_table.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].ingress_route_table.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].ingress_route_table.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].internet_gateway.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].internet_gateway.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].internet_gateway.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].classic_load_balancer_listener.load_balancer_port #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].classic_load_balancer_listener.instance_port #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_listener_port #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_target.address #=> String
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_target.availability_zone #=> String
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_target.instance.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_target.instance.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_target.instance.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_target.port #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_target_group.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_target_group.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_target_group.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_target_groups #=> Array
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_target_groups[0].id #=> String
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_target_groups[0].arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_target_groups[0].name #=> String
    #   resp.network_insights_analyses[0].explanations[0].load_balancer_target_port #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].elastic_load_balancer_listener.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].elastic_load_balancer_listener.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].elastic_load_balancer_listener.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].missing_component #=> String
    #   resp.network_insights_analyses[0].explanations[0].nat_gateway.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].nat_gateway.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].nat_gateway.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].network_interface.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].network_interface.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].network_interface.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].packet_field #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpc_peering_connection.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpc_peering_connection.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpc_peering_connection.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].port #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].port_ranges #=> Array
    #   resp.network_insights_analyses[0].explanations[0].port_ranges[0].from #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].port_ranges[0].to #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].prefix_list.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].prefix_list.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].prefix_list.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].protocols #=> Array
    #   resp.network_insights_analyses[0].explanations[0].protocols[0] #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table_route.destination_cidr #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table_route.destination_prefix_list_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table_route.egress_only_internet_gateway_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table_route.gateway_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table_route.instance_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table_route.nat_gateway_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table_route.network_interface_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table_route.origin #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table_route.transit_gateway_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table_route.vpc_peering_connection_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table_route.state #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table_route.carrier_gateway_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table_route.core_network_arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table_route.local_gateway_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].route_table.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].security_group.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].security_group.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].security_group.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].security_group_rule.cidr #=> String
    #   resp.network_insights_analyses[0].explanations[0].security_group_rule.direction #=> String
    #   resp.network_insights_analyses[0].explanations[0].security_group_rule.security_group_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].security_group_rule.port_range.from #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].security_group_rule.port_range.to #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].security_group_rule.prefix_list_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].security_group_rule.protocol #=> String
    #   resp.network_insights_analyses[0].explanations[0].security_groups #=> Array
    #   resp.network_insights_analyses[0].explanations[0].security_groups[0].id #=> String
    #   resp.network_insights_analyses[0].explanations[0].security_groups[0].arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].security_groups[0].name #=> String
    #   resp.network_insights_analyses[0].explanations[0].source_vpc.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].source_vpc.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].source_vpc.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].state #=> String
    #   resp.network_insights_analyses[0].explanations[0].subnet.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].subnet.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].subnet.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].subnet_route_table.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].subnet_route_table.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].subnet_route_table.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpc.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpc.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpc.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpc_endpoint.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpc_endpoint.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpc_endpoint.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpn_connection.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpn_connection.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpn_connection.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpn_gateway.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpn_gateway.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].vpn_gateway.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway_route_table.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway_route_table.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway_route_table.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway_route_table_route.destination_cidr #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway_route_table_route.state #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway_route_table_route.route_origin #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway_route_table_route.prefix_list_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway_route_table_route.attachment_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway_route_table_route.resource_id #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway_route_table_route.resource_type #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway_attachment.id #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway_attachment.arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].transit_gateway_attachment.name #=> String
    #   resp.network_insights_analyses[0].explanations[0].component_account #=> String
    #   resp.network_insights_analyses[0].explanations[0].component_region #=> String
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.rule_group_arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.sources #=> Array
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.sources[0] #=> String
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.destinations #=> Array
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.destinations[0] #=> String
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.source_ports #=> Array
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.destination_ports #=> Array
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.protocols #=> Array
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.protocols[0] #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateless_rule.priority #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateful_rule.rule_group_arn #=> String
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateful_rule.sources #=> Array
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateful_rule.sources[0] #=> String
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateful_rule.destinations #=> Array
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateful_rule.destinations[0] #=> String
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateful_rule.source_ports #=> Array
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateful_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateful_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateful_rule.destination_ports #=> Array
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateful_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateful_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateful_rule.protocol #=> String
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateful_rule.rule_action #=> String
    #   resp.network_insights_analyses[0].explanations[0].firewall_stateful_rule.direction #=> String
    #   resp.network_insights_analyses[0].alternate_path_hints #=> Array
    #   resp.network_insights_analyses[0].alternate_path_hints[0].component_id #=> String
    #   resp.network_insights_analyses[0].alternate_path_hints[0].component_arn #=> String
    #   resp.network_insights_analyses[0].suggested_accounts #=> Array
    #   resp.network_insights_analyses[0].suggested_accounts[0] #=> String
    #   resp.network_insights_analyses[0].tags #=> Array
    #   resp.network_insights_analyses[0].tags[0].key #=> String
    #   resp.network_insights_analyses[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeNetworkInsightsAnalyses AWS API Documentation
    #
    # @overload describe_network_insights_analyses(params = {})
    # @param [Hash] params ({})
    def describe_network_insights_analyses(params = {}, options = {})
      req = build_request(:describe_network_insights_analyses, params)
      req.send_request(options)
    end

    # Describes one or more of your paths.
    #
    # @option params [Array<String>] :network_insights_path_ids
    #   The IDs of the paths.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters. The following are the possible values:
    #
    #   * destination - The ID of the resource.
    #
    #   * filter-at-source.source-address - The source IPv4 address at the
    #     source.
    #
    #   * filter-at-source.source-port-range - The source port range at the
    #     source.
    #
    #   * filter-at-source.destination-address - The destination IPv4 address
    #     at the source.
    #
    #   * filter-at-source.destination-port-range - The destination port range
    #     at the source.
    #
    #   * filter-at-destination.source-address - The source IPv4 address at
    #     the destination.
    #
    #   * filter-at-destination.source-port-range - The source port range at
    #     the destination.
    #
    #   * filter-at-destination.destination-address - The destination IPv4
    #     address at the destination.
    #
    #   * filter-at-destination.destination-port-range - The destination port
    #     range at the destination.
    #
    #   * protocol - The protocol.
    #
    #   * source - The ID of the resource.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::DescribeNetworkInsightsPathsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeNetworkInsightsPathsResult#network_insights_paths #network_insights_paths} => Array&lt;Types::NetworkInsightsPath&gt;
    #   * {Types::DescribeNetworkInsightsPathsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_network_insights_paths({
    #     network_insights_path_ids: ["NetworkInsightsPathId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     dry_run: false,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.network_insights_paths #=> Array
    #   resp.network_insights_paths[0].network_insights_path_id #=> String
    #   resp.network_insights_paths[0].network_insights_path_arn #=> String
    #   resp.network_insights_paths[0].created_date #=> Time
    #   resp.network_insights_paths[0].source #=> String
    #   resp.network_insights_paths[0].destination #=> String
    #   resp.network_insights_paths[0].source_arn #=> String
    #   resp.network_insights_paths[0].destination_arn #=> String
    #   resp.network_insights_paths[0].source_ip #=> String
    #   resp.network_insights_paths[0].destination_ip #=> String
    #   resp.network_insights_paths[0].protocol #=> String, one of "tcp", "udp"
    #   resp.network_insights_paths[0].destination_port #=> Integer
    #   resp.network_insights_paths[0].tags #=> Array
    #   resp.network_insights_paths[0].tags[0].key #=> String
    #   resp.network_insights_paths[0].tags[0].value #=> String
    #   resp.network_insights_paths[0].filter_at_source.source_address #=> String
    #   resp.network_insights_paths[0].filter_at_source.source_port_range.from_port #=> Integer
    #   resp.network_insights_paths[0].filter_at_source.source_port_range.to_port #=> Integer
    #   resp.network_insights_paths[0].filter_at_source.destination_address #=> String
    #   resp.network_insights_paths[0].filter_at_source.destination_port_range.from_port #=> Integer
    #   resp.network_insights_paths[0].filter_at_source.destination_port_range.to_port #=> Integer
    #   resp.network_insights_paths[0].filter_at_destination.source_address #=> String
    #   resp.network_insights_paths[0].filter_at_destination.source_port_range.from_port #=> Integer
    #   resp.network_insights_paths[0].filter_at_destination.source_port_range.to_port #=> Integer
    #   resp.network_insights_paths[0].filter_at_destination.destination_address #=> String
    #   resp.network_insights_paths[0].filter_at_destination.destination_port_range.from_port #=> Integer
    #   resp.network_insights_paths[0].filter_at_destination.destination_port_range.to_port #=> Integer
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeNetworkInsightsPaths AWS API Documentation
    #
    # @overload describe_network_insights_paths(params = {})
    # @param [Hash] params ({})
    def describe_network_insights_paths(params = {}, options = {})
      req = build_request(:describe_network_insights_paths, params)
      req.send_request(options)
    end

    # Describes a network interface attribute. You can specify only one
    # attribute at a time.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :network_interface_id
    #   The ID of the network interface.
    #
    # @option params [String] :attribute
    #   The attribute of the network interface. This parameter is required.
    #
    # @return [Types::DescribeNetworkInterfaceAttributeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeNetworkInterfaceAttributeResult#attachment #attachment} => Types::NetworkInterfaceAttachment
    #   * {Types::DescribeNetworkInterfaceAttributeResult#description #description} => Types::AttributeValue
    #   * {Types::DescribeNetworkInterfaceAttributeResult#groups #groups} => Array&lt;Types::GroupIdentifier&gt;
    #   * {Types::DescribeNetworkInterfaceAttributeResult#network_interface_id #network_interface_id} => String
    #   * {Types::DescribeNetworkInterfaceAttributeResult#source_dest_check #source_dest_check} => Types::AttributeBooleanValue
    #   * {Types::DescribeNetworkInterfaceAttributeResult#associate_public_ip_address #associate_public_ip_address} => Boolean
    #
    #
    # @example Example: To describe the attachment attribute of a network interface
    #
    #   # This example describes the attachment attribute of the specified network interface.
    #
    #   resp = client.describe_network_interface_attribute({
    #     attribute: "attachment", 
    #     network_interface_id: "eni-686ea200", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     attachment: {
    #       attach_time: Time.parse("2015-05-21T20:02:20.000Z"), 
    #       attachment_id: "eni-attach-43348162", 
    #       delete_on_termination: true, 
    #       device_index: 0, 
    #       instance_id: "i-1234567890abcdef0", 
    #       instance_owner_id: "123456789012", 
    #       status: "attached", 
    #     }, 
    #     network_interface_id: "eni-686ea200", 
    #   }
    #
    # @example Example: To describe the description attribute of a network interface
    #
    #   # This example describes the description attribute of the specified network interface.
    #
    #   resp = client.describe_network_interface_attribute({
    #     attribute: "description", 
    #     network_interface_id: "eni-686ea200", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     description: {
    #       value: "My description", 
    #     }, 
    #     network_interface_id: "eni-686ea200", 
    #   }
    #
    # @example Example: To describe the groupSet attribute of a network interface
    #
    #   # This example describes the groupSet attribute of the specified network interface.
    #
    #   resp = client.describe_network_interface_attribute({
    #     attribute: "groupSet", 
    #     network_interface_id: "eni-686ea200", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     groups: [
    #       {
    #         group_id: "sg-903004f8", 
    #         group_name: "my-security-group", 
    #       }, 
    #     ], 
    #     network_interface_id: "eni-686ea200", 
    #   }
    #
    # @example Example: To describe the sourceDestCheck attribute of a network interface
    #
    #   # This example describes the sourceDestCheck attribute of the specified network interface.
    #
    #   resp = client.describe_network_interface_attribute({
    #     attribute: "sourceDestCheck", 
    #     network_interface_id: "eni-686ea200", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     network_interface_id: "eni-686ea200", 
    #     source_dest_check: {
    #       value: true, 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_network_interface_attribute({
    #     dry_run: false,
    #     network_interface_id: "NetworkInterfaceId", # required
    #     attribute: "description", # accepts description, groupSet, sourceDestCheck, attachment, associatePublicIpAddress
    #   })
    #
    # @example Response structure
    #
    #   resp.attachment.attach_time #=> Time
    #   resp.attachment.attachment_id #=> String
    #   resp.attachment.delete_on_termination #=> Boolean
    #   resp.attachment.device_index #=> Integer
    #   resp.attachment.network_card_index #=> Integer
    #   resp.attachment.instance_id #=> String
    #   resp.attachment.instance_owner_id #=> String
    #   resp.attachment.status #=> String, one of "attaching", "attached", "detaching", "detached"
    #   resp.attachment.ena_srd_specification.ena_srd_enabled #=> Boolean
    #   resp.attachment.ena_srd_specification.ena_srd_udp_specification.ena_srd_udp_enabled #=> Boolean
    #   resp.description #=> <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #   resp.groups #=> Array
    #   resp.groups[0].group_id #=> String
    #   resp.groups[0].group_name #=> String
    #   resp.network_interface_id #=> String
    #   resp.source_dest_check.value #=> Boolean
    #   resp.associate_public_ip_address #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeNetworkInterfaceAttribute AWS API Documentation
    #
    # @overload describe_network_interface_attribute(params = {})
    # @param [Hash] params ({})
    def describe_network_interface_attribute(params = {}, options = {})
      req = build_request(:describe_network_interface_attribute, params)
      req.send_request(options)
    end

    # Describes the permissions for your network interfaces.
    #
    # @option params [Array<String>] :network_interface_permission_ids
    #   The network interface permission IDs.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `network-interface-permission.network-interface-permission-id` - The
    #     ID of the permission.
    #
    #   * `network-interface-permission.network-interface-id` - The ID of the
    #     network interface.
    #
    #   * `network-interface-permission.aws-account-id` - The Amazon Web
    #     Services account ID.
    #
    #   * `network-interface-permission.aws-service` - The Amazon Web Services
    #     service.
    #
    #   * `network-interface-permission.permission` - The type of permission
    #     (`INSTANCE-ATTACH` \| `EIP-ASSOCIATE`).
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. If this parameter is not specified, up to 50 results are
    #   returned by default. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @return [Types::DescribeNetworkInterfacePermissionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeNetworkInterfacePermissionsResult#network_interface_permissions #network_interface_permissions} => Array&lt;Types::NetworkInterfacePermission&gt;
    #   * {Types::DescribeNetworkInterfacePermissionsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_network_interface_permissions({
    #     network_interface_permission_ids: ["NetworkInterfacePermissionId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.network_interface_permissions #=> Array
    #   resp.network_interface_permissions[0].network_interface_permission_id #=> String
    #   resp.network_interface_permissions[0].network_interface_id #=> String
    #   resp.network_interface_permissions[0].aws_account_id #=> String
    #   resp.network_interface_permissions[0].aws_service #=> String
    #   resp.network_interface_permissions[0].permission #=> String, one of "INSTANCE-ATTACH", "EIP-ASSOCIATE"
    #   resp.network_interface_permissions[0].permission_state.state #=> String, one of "pending", "granted", "revoking", "revoked"
    #   resp.network_interface_permissions[0].permission_state.status_message #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeNetworkInterfacePermissions AWS API Documentation
    #
    # @overload describe_network_interface_permissions(params = {})
    # @param [Hash] params ({})
    def describe_network_interface_permissions(params = {}, options = {})
      req = build_request(:describe_network_interface_permissions, params)
      req.send_request(options)
    end

    # Describes the specified network interfaces or all your network
    # interfaces.
    #
    # If you have a large number of network interfaces, the operation fails
    # unless you use pagination or one of the following filters: `group-id`,
    # `mac-address`, `private-dns-name`, `private-ip-address`, `subnet-id`,
    # or `vpc-id`.
    #
    # We strongly recommend using only paginated requests. Unpaginated
    # requests are susceptible to throttling and timeouts.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. You cannot specify this parameter and the network
    #   interface IDs parameter in the same request. For more information, see
    #   [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :network_interface_ids
    #   The network interface IDs.
    #
    #   Default: Describes all your network interfaces.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `association.allocation-id` - The allocation ID returned when you
    #     allocated the Elastic IP address (IPv4) for your network interface.
    #
    #   * `association.association-id` - The association ID returned when the
    #     network interface was associated with an IPv4 address.
    #
    #   * `addresses.association.owner-id` - The owner ID of the addresses
    #     associated with the network interface.
    #
    #   * `addresses.association.public-ip` - The association ID returned when
    #     the network interface was associated with the Elastic IP address
    #     (IPv4).
    #
    #   * `addresses.primary` - Whether the private IPv4 address is the
    #     primary IP address associated with the network interface.
    #
    #   * `addresses.private-ip-address` - The private IPv4 addresses
    #     associated with the network interface.
    #
    #   * `association.ip-owner-id` - The owner of the Elastic IP address
    #     (IPv4) associated with the network interface.
    #
    #   * `association.public-ip` - The address of the Elastic IP address
    #     (IPv4) bound to the network interface.
    #
    #   * `association.public-dns-name` - The public DNS name for the network
    #     interface (IPv4).
    #
    #   * `attachment.attach-time` - The time that the network interface was
    #     attached to an instance.
    #
    #   * `attachment.attachment-id` - The ID of the interface attachment.
    #
    #   * `attachment.delete-on-termination` - Indicates whether the
    #     attachment is deleted when an instance is terminated.
    #
    #   * `attachment.device-index` - The device index to which the network
    #     interface is attached.
    #
    #   * `attachment.instance-id` - The ID of the instance to which the
    #     network interface is attached.
    #
    #   * `attachment.instance-owner-id` - The owner ID of the instance to
    #     which the network interface is attached.
    #
    #   * `attachment.status` - The status of the attachment (`attaching` \|
    #     `attached` \| `detaching` \| `detached`).
    #
    #   * `availability-zone` - The Availability Zone of the network
    #     interface.
    #
    #   * `description` - The description of the network interface.
    #
    #   * `group-id` - The ID of a security group associated with the network
    #     interface.
    #
    #   * `ipv6-addresses.ipv6-address` - An IPv6 address associated with the
    #     network interface.
    #
    #   * `interface-type` - The type of network interface
    #     (`api_gateway_managed` \| `aws_codestar_connections_managed` \|
    #     `branch` \| `ec2_instance_connect_endpoint` \| `efa` \| `efa-only`
    #     \| `efs` \| `gateway_load_balancer` \|
    #     `gateway_load_balancer_endpoint` \| `global_accelerator_managed` \|
    #     `interface` \| `iot_rules_managed` \| `lambda` \| `load_balancer` \|
    #     `nat_gateway` \| `network_load_balancer` \| `quicksight` \|
    #     `transit_gateway` \| `trunk` \| `vpc_endpoint`).
    #
    #   * `mac-address` - The MAC address of the network interface.
    #
    #   * `network-interface-id` - The ID of the network interface.
    #
    #   * `operator.managed` - A Boolean that indicates whether this is a
    #     managed network interface.
    #
    #   * `operator.principal` - The principal that manages the network
    #     interface. Only valid for managed network interfaces, where
    #     `managed` is `true`.
    #
    #   * `owner-id` - The Amazon Web Services account ID of the network
    #     interface owner.
    #
    #   * `private-dns-name` - The private DNS name of the network interface
    #     (IPv4).
    #
    #   * `private-ip-address` - The private IPv4 address or addresses of the
    #     network interface.
    #
    #   * `requester-id` - The alias or Amazon Web Services account ID of the
    #     principal or service that created the network interface.
    #
    #   * `requester-managed` - Indicates whether the network interface is
    #     being managed by an Amazon Web Services service (for example, Amazon
    #     Web Services Management Console, Auto Scaling, and so on).
    #
    #   * `source-dest-check` - Indicates whether the network interface
    #     performs source/destination checking. A value of `true` means
    #     checking is enabled, and `false` means checking is disabled. The
    #     value must be `false` for the network interface to perform network
    #     address translation (NAT) in your VPC.
    #
    #   * `status` - The status of the network interface. If the network
    #     interface is not attached to an instance, the status is `available`;
    #     if a network interface is attached to an instance the status is
    #     `in-use`.
    #
    #   * `subnet-id` - The ID of the subnet for the network interface.
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `vpc-id` - The ID of the VPC for the network interface.
    #
    # @return [Types::DescribeNetworkInterfacesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeNetworkInterfacesResult#network_interfaces #network_interfaces} => Array&lt;Types::NetworkInterface&gt;
    #   * {Types::DescribeNetworkInterfacesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe a network interface
    #
    #   resp = client.describe_network_interfaces({
    #     network_interface_ids: [
    #       "eni-e5aa89a3", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     network_interfaces: [
    #       {
    #         association: {
    #           association_id: "eipassoc-0fbb766a", 
    #           ip_owner_id: "123456789012", 
    #           public_dns_name: "ec2-203-0-113-12.compute-1.amazonaws.com", 
    #           public_ip: "203.0.113.12", 
    #         }, 
    #         attachment: {
    #           attach_time: Time.parse("2013-11-30T23:36:42.000Z"), 
    #           attachment_id: "eni-attach-66c4350a", 
    #           delete_on_termination: false, 
    #           device_index: 1, 
    #           instance_id: "i-1234567890abcdef0", 
    #           instance_owner_id: "123456789012", 
    #           status: "attached", 
    #         }, 
    #         availability_zone: "us-east-1d", 
    #         description: "my network interface", 
    #         groups: [
    #           {
    #             group_id: "sg-8637d3e3", 
    #             group_name: "default", 
    #           }, 
    #         ], 
    #         mac_address: "02:2f:8f:b0:cf:75", 
    #         network_interface_id: "eni-e5aa89a3", 
    #         owner_id: "123456789012", 
    #         private_dns_name: "ip-10-0-1-17.ec2.internal", 
    #         private_ip_address: "10.0.1.17", 
    #         private_ip_addresses: [
    #           {
    #             association: {
    #               association_id: "eipassoc-0fbb766a", 
    #               ip_owner_id: "123456789012", 
    #               public_dns_name: "ec2-203-0-113-12.compute-1.amazonaws.com", 
    #               public_ip: "203.0.113.12", 
    #             }, 
    #             primary: true, 
    #             private_dns_name: "ip-10-0-1-17.ec2.internal", 
    #             private_ip_address: "10.0.1.17", 
    #           }, 
    #         ], 
    #         requester_managed: false, 
    #         source_dest_check: true, 
    #         status: "in-use", 
    #         subnet_id: "subnet-b61f49f0", 
    #         tag_set: [
    #         ], 
    #         vpc_id: "vpc-a01106c2", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_network_interfaces({
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #     network_interface_ids: ["NetworkInterfaceId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.network_interfaces #=> Array
    #   resp.network_interfaces[0].association.allocation_id #=> String
    #   resp.network_interfaces[0].association.association_id #=> String
    #   resp.network_interfaces[0].association.ip_owner_id #=> String
    #   resp.network_interfaces[0].association.public_dns_name #=> String
    #   resp.network_interfaces[0].association.public_ip #=> String
    #   resp.network_interfaces[0].association.customer_owned_ip #=> String
    #   resp.network_interfaces[0].association.carrier_ip #=> String
    #   resp.network_interfaces[0].attachment.attach_time #=> Time
    #   resp.network_interfaces[0].attachment.attachment_id #=> String
    #   resp.network_interfaces[0].attachment.delete_on_termination #=> Boolean
    #   resp.network_interfaces[0].attachment.device_index #=> Integer
    #   resp.network_interfaces[0].attachment.network_card_index #=> Integer
    #   resp.network_interfaces[0].attachment.instance_id #=> String
    #   resp.network_interfaces[0].attachment.instance_owner_id #=> String
    #   resp.network_interfaces[0].attachment.status #=> String, one of "attaching", "attached", "detaching", "detached"
    #   resp.network_interfaces[0].attachment.ena_srd_specification.ena_srd_enabled #=> Boolean
    #   resp.network_interfaces[0].attachment.ena_srd_specification.ena_srd_udp_specification.ena_srd_udp_enabled #=> Boolean
    #   resp.network_interfaces[0].availability_zone #=> String
    #   resp.network_interfaces[0].connection_tracking_configuration.tcp_established_timeout #=> Integer
    #   resp.network_interfaces[0].connection_tracking_configuration.udp_stream_timeout #=> Integer
    #   resp.network_interfaces[0].connection_tracking_configuration.udp_timeout #=> Integer
    #   resp.network_interfaces[0].description #=> String
    #   resp.network_interfaces[0].groups #=> Array
    #   resp.network_interfaces[0].groups[0].group_id #=> String
    #   resp.network_interfaces[0].groups[0].group_name #=> String
    #   resp.network_interfaces[0].interface_type #=> String, one of "interface", "natGateway", "efa", "efa-only", "trunk", "load_balancer", "network_load_balancer", "vpc_endpoint", "branch", "transit_gateway", "lambda", "quicksight", "global_accelerator_managed", "api_gateway_managed", "gateway_load_balancer", "gateway_load_balancer_endpoint", "iot_rules_managed", "aws_codestar_connections_managed"
    #   resp.network_interfaces[0].ipv_6_addresses #=> Array
    #   resp.network_interfaces[0].ipv_6_addresses[0].ipv_6_address #=> String
    #   resp.network_interfaces[0].ipv_6_addresses[0].is_primary_ipv_6 #=> Boolean
    #   resp.network_interfaces[0].mac_address #=> String
    #   resp.network_interfaces[0].network_interface_id #=> String
    #   resp.network_interfaces[0].outpost_arn #=> String
    #   resp.network_interfaces[0].owner_id #=> String
    #   resp.network_interfaces[0].private_dns_name #=> String
    #   resp.network_interfaces[0].private_ip_address #=> String
    #   resp.network_interfaces[0].private_ip_addresses #=> Array
    #   resp.network_interfaces[0].private_ip_addresses[0].association.allocation_id #=> String
    #   resp.network_interfaces[0].private_ip_addresses[0].association.association_id #=> String
    #   resp.network_interfaces[0].private_ip_addresses[0].association.ip_owner_id #=> String
    #   resp.network_interfaces[0].private_ip_addresses[0].association.public_dns_name #=> String
    #   resp.network_interfaces[0].private_ip_addresses[0].association.public_ip #=> String
    #   resp.network_interfaces[0].private_ip_addresses[0].association.customer_owned_ip #=> String
    #   resp.network_interfaces[0].private_ip_addresses[0].association.carrier_ip #=> String
    #   resp.network_interfaces[0].private_ip_addresses[0].primary #=> Boolean
    #   resp.network_interfaces[0].private_ip_addresses[0].private_dns_name #=> String
    #   resp.network_interfaces[0].private_ip_addresses[0].private_ip_address #=> String
    #   resp.network_interfaces[0].ipv_4_prefixes #=> Array
    #   resp.network_interfaces[0].ipv_4_prefixes[0].ipv_4_prefix #=> String
    #   resp.network_interfaces[0].ipv_6_prefixes #=> Array
    #   resp.network_interfaces[0].ipv_6_prefixes[0].ipv_6_prefix #=> String
    #   resp.network_interfaces[0].requester_id #=> String
    #   resp.network_interfaces[0].requester_managed #=> Boolean
    #   resp.network_interfaces[0].source_dest_check #=> Boolean
    #   resp.network_interfaces[0].status #=> String, one of "available", "associated", "attaching", "in-use", "detaching"
    #   resp.network_interfaces[0].subnet_id #=> String
    #   resp.network_interfaces[0].tag_set #=> Array
    #   resp.network_interfaces[0].tag_set[0].key #=> String
    #   resp.network_interfaces[0].tag_set[0].value #=> String
    #   resp.network_interfaces[0].vpc_id #=> String
    #   resp.network_interfaces[0].deny_all_igw_traffic #=> Boolean
    #   resp.network_interfaces[0].ipv_6_native #=> Boolean
    #   resp.network_interfaces[0].ipv_6_address #=> String
    #   resp.network_interfaces[0].operator.managed #=> Boolean
    #   resp.network_interfaces[0].operator.principal #=> String
    #   resp.next_token #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * network_interface_available
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeNetworkInterfaces AWS API Documentation
    #
    # @overload describe_network_interfaces(params = {})
    # @param [Hash] params ({})
    def describe_network_interfaces(params = {}, options = {})
      req = build_request(:describe_network_interfaces, params)
      req.send_request(options)
    end

    # Describes the specified placement groups or all of your placement
    # groups.
    #
    # <note markdown="1"> To describe a specific placement group that is *shared* with your
    # account, you must specify the ID of the placement group using the
    # `GroupId` parameter. Specifying the name of a *shared* placement group
    # using the `GroupNames` parameter will result in an error.
    #
    #  </note>
    #
    # For more information, see [Placement groups][1] in the *Amazon EC2
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/placement-groups.html
    #
    # @option params [Array<String>] :group_ids
    #   The IDs of the placement groups.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :group_names
    #   The names of the placement groups.
    #
    #   Constraints:
    #
    #   * You can specify a name only if the placement group is owned by your
    #     account.
    #
    #   * If a placement group is *shared* with your account, specifying the
    #     name results in an error. You must use the `GroupId` parameter
    #     instead.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `group-name` - The name of the placement group.
    #
    #   * `group-arn` - The Amazon Resource Name (ARN) of the placement group.
    #
    #   * `spread-level` - The spread level for the placement group (`host` \|
    #     `rack`).
    #
    #   * `state` - The state of the placement group (`pending` \| `available`
    #     \| `deleting` \| `deleted`).
    #
    #   * `strategy` - The strategy of the placement group (`cluster` \|
    #     `spread` \| `partition`).
    #
    #   * `tag:<key>` - The key/value combination of a tag assigned to the
    #     resource. Use the tag key in the filter name and the tag value as
    #     the filter value. For example, to find all resources that have a tag
    #     with the key `Owner` and the value `TeamA`, specify `tag:Owner` for
    #     the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources that have a tag with a specific key,
    #     regardless of the tag value.
    #
    # @return [Types::DescribePlacementGroupsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribePlacementGroupsResult#placement_groups #placement_groups} => Array&lt;Types::PlacementGroup&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_placement_groups({
    #     group_ids: ["PlacementGroupId"],
    #     dry_run: false,
    #     group_names: ["PlacementGroupName"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.placement_groups #=> Array
    #   resp.placement_groups[0].group_name #=> String
    #   resp.placement_groups[0].state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.placement_groups[0].strategy #=> String, one of "cluster", "spread", "partition"
    #   resp.placement_groups[0].partition_count #=> Integer
    #   resp.placement_groups[0].group_id #=> String
    #   resp.placement_groups[0].tags #=> Array
    #   resp.placement_groups[0].tags[0].key #=> String
    #   resp.placement_groups[0].tags[0].value #=> String
    #   resp.placement_groups[0].group_arn #=> String
    #   resp.placement_groups[0].spread_level #=> String, one of "host", "rack"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribePlacementGroups AWS API Documentation
    #
    # @overload describe_placement_groups(params = {})
    # @param [Hash] params ({})
    def describe_placement_groups(params = {}, options = {})
      req = build_request(:describe_placement_groups, params)
      req.send_request(options)
    end

    # Describes available Amazon Web Services services in a prefix list
    # format, which includes the prefix list name and prefix list ID of the
    # service and the IP address range for the service.
    #
    # We recommend that you use DescribeManagedPrefixLists instead.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `prefix-list-id`: The ID of a prefix list.
    #
    #   * `prefix-list-name`: The name of a prefix list.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Array<String>] :prefix_list_ids
    #   One or more prefix list IDs.
    #
    # @return [Types::DescribePrefixListsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribePrefixListsResult#next_token #next_token} => String
    #   * {Types::DescribePrefixListsResult#prefix_lists #prefix_lists} => Array&lt;Types::PrefixList&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_prefix_lists({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     prefix_list_ids: ["PrefixListResourceId"],
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.prefix_lists #=> Array
    #   resp.prefix_lists[0].cidrs #=> Array
    #   resp.prefix_lists[0].cidrs[0] #=> String
    #   resp.prefix_lists[0].prefix_list_id #=> String
    #   resp.prefix_lists[0].prefix_list_name #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribePrefixLists AWS API Documentation
    #
    # @overload describe_prefix_lists(params = {})
    # @param [Hash] params ({})
    def describe_prefix_lists(params = {}, options = {})
      req = build_request(:describe_prefix_lists, params)
      req.send_request(options)
    end

    # Describes the ID format settings for the root user and all IAM roles
    # and IAM users that have explicitly specified a longer ID (17-character
    # ID) preference.
    #
    # By default, all IAM roles and IAM users default to the same ID
    # settings as the root user, unless they explicitly override the
    # settings. This request is useful for identifying those IAM users and
    # IAM roles that have overridden the default ID settings.
    #
    # The following resource types support longer IDs: `bundle` \|
    # `conversion-task` \| `customer-gateway` \| `dhcp-options` \|
    # `elastic-ip-allocation` \| `elastic-ip-association` \| `export-task`
    # \| `flow-log` \| `image` \| `import-task` \| `instance` \|
    # `internet-gateway` \| `network-acl` \| `network-acl-association` \|
    # `network-interface` \| `network-interface-attachment` \| `prefix-list`
    # \| `reservation` \| `route-table` \| `route-table-association` \|
    # `security-group` \| `snapshot` \| `subnet` \|
    # `subnet-cidr-block-association` \| `volume` \| `vpc` \|
    # `vpc-cidr-block-association` \| `vpc-endpoint` \|
    # `vpc-peering-connection` \| `vpn-connection` \| `vpn-gateway`.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :resources
    #   The type of resource: `bundle` \| `conversion-task` \|
    #   `customer-gateway` \| `dhcp-options` \| `elastic-ip-allocation` \|
    #   `elastic-ip-association` \| `export-task` \| `flow-log` \| `image` \|
    #   `import-task` \| `instance` \| `internet-gateway` \| `network-acl` \|
    #   `network-acl-association` \| `network-interface` \|
    #   `network-interface-attachment` \| `prefix-list` \| `reservation` \|
    #   `route-table` \| `route-table-association` \| `security-group` \|
    #   `snapshot` \| `subnet` \| `subnet-cidr-block-association` \| `volume`
    #   \| `vpc` \| `vpc-cidr-block-association` \| `vpc-endpoint` \|
    #   `vpc-peering-connection` \| `vpn-connection` \| `vpn-gateway`
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in a single call. To retrieve
    #   the remaining results, make another call with the returned NextToken
    #   value.
    #
    # @option params [String] :next_token
    #   The token to request the next page of results.
    #
    # @return [Types::DescribePrincipalIdFormatResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribePrincipalIdFormatResult#principals #principals} => Array&lt;Types::PrincipalIdFormat&gt;
    #   * {Types::DescribePrincipalIdFormatResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_principal_id_format({
    #     dry_run: false,
    #     resources: ["String"],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.principals #=> Array
    #   resp.principals[0].arn #=> String
    #   resp.principals[0].statuses #=> Array
    #   resp.principals[0].statuses[0].deadline #=> Time
    #   resp.principals[0].statuses[0].resource #=> String
    #   resp.principals[0].statuses[0].use_long_ids #=> Boolean
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribePrincipalIdFormat AWS API Documentation
    #
    # @overload describe_principal_id_format(params = {})
    # @param [Hash] params ({})
    def describe_principal_id_format(params = {}, options = {})
      req = build_request(:describe_principal_id_format, params)
      req.send_request(options)
    end

    # Describes the specified IPv4 address pools.
    #
    # @option params [Array<String>] :pool_ids
    #   The IDs of the address pools.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @return [Types::DescribePublicIpv4PoolsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribePublicIpv4PoolsResult#public_ipv_4_pools #public_ipv_4_pools} => Array&lt;Types::PublicIpv4Pool&gt;
    #   * {Types::DescribePublicIpv4PoolsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_public_ipv_4_pools({
    #     pool_ids: ["Ipv4PoolEc2Id"],
    #     next_token: "NextToken",
    #     max_results: 1,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.public_ipv_4_pools #=> Array
    #   resp.public_ipv_4_pools[0].pool_id #=> String
    #   resp.public_ipv_4_pools[0].description #=> String
    #   resp.public_ipv_4_pools[0].pool_address_ranges #=> Array
    #   resp.public_ipv_4_pools[0].pool_address_ranges[0].first_address #=> String
    #   resp.public_ipv_4_pools[0].pool_address_ranges[0].last_address #=> String
    #   resp.public_ipv_4_pools[0].pool_address_ranges[0].address_count #=> Integer
    #   resp.public_ipv_4_pools[0].pool_address_ranges[0].available_address_count #=> Integer
    #   resp.public_ipv_4_pools[0].total_address_count #=> Integer
    #   resp.public_ipv_4_pools[0].total_available_address_count #=> Integer
    #   resp.public_ipv_4_pools[0].network_border_group #=> String
    #   resp.public_ipv_4_pools[0].tags #=> Array
    #   resp.public_ipv_4_pools[0].tags[0].key #=> String
    #   resp.public_ipv_4_pools[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribePublicIpv4Pools AWS API Documentation
    #
    # @overload describe_public_ipv_4_pools(params = {})
    # @param [Hash] params ({})
    def describe_public_ipv_4_pools(params = {}, options = {})
      req = build_request(:describe_public_ipv_4_pools, params)
      req.send_request(options)
    end

    # Describes the Regions that are enabled for your account, or all
    # Regions.
    #
    # For a list of the Regions supported by Amazon EC2, see [Amazon EC2
    # service endpoints][1].
    #
    # For information about enabling and disabling Regions for your account,
    # see [Specify which Amazon Web Services Regions your account can
    # use][2] in the *Amazon Web Services Account Management Reference
    # Guide*.
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-endpoints.html
    # [2]: https://docs.aws.amazon.com/accounts/latest/reference/manage-acct-regions.html
    #
    # @option params [Array<String>] :region_names
    #   The names of the Regions. You can specify any Regions, whether they
    #   are enabled and disabled for your account.
    #
    # @option params [Boolean] :all_regions
    #   Indicates whether to display all Regions, including Regions that are
    #   disabled for your account.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `endpoint` - The endpoint of the Region (for example,
    #     `ec2.us-east-1.amazonaws.com`).
    #
    #   * `opt-in-status` - The opt-in status of the Region
    #     (`opt-in-not-required` \| `opted-in` \| `not-opted-in`).
    #
    #   * `region-name` - The name of the Region (for example, `us-east-1`).
    #
    # @return [Types::DescribeRegionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeRegionsResult#regions #regions} => Array&lt;Types::Region&gt;
    #
    #
    # @example Example: To describe your regions
    #
    #   # This example describes all the regions that are available to you.
    #
    #   resp = client.describe_regions({
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     regions: [
    #       {
    #         endpoint: "ec2.ap-south-1.amazonaws.com", 
    #         region_name: "ap-south-1", 
    #       }, 
    #       {
    #         endpoint: "ec2.eu-west-1.amazonaws.com", 
    #         region_name: "eu-west-1", 
    #       }, 
    #       {
    #         endpoint: "ec2.ap-southeast-1.amazonaws.com", 
    #         region_name: "ap-southeast-1", 
    #       }, 
    #       {
    #         endpoint: "ec2.ap-southeast-2.amazonaws.com", 
    #         region_name: "ap-southeast-2", 
    #       }, 
    #       {
    #         endpoint: "ec2.eu-central-1.amazonaws.com", 
    #         region_name: "eu-central-1", 
    #       }, 
    #       {
    #         endpoint: "ec2.ap-northeast-2.amazonaws.com", 
    #         region_name: "ap-northeast-2", 
    #       }, 
    #       {
    #         endpoint: "ec2.ap-northeast-1.amazonaws.com", 
    #         region_name: "ap-northeast-1", 
    #       }, 
    #       {
    #         endpoint: "ec2.us-east-1.amazonaws.com", 
    #         region_name: "us-east-1", 
    #       }, 
    #       {
    #         endpoint: "ec2.sa-east-1.amazonaws.com", 
    #         region_name: "sa-east-1", 
    #       }, 
    #       {
    #         endpoint: "ec2.us-west-1.amazonaws.com", 
    #         region_name: "us-west-1", 
    #       }, 
    #       {
    #         endpoint: "ec2.us-west-2.amazonaws.com", 
    #         region_name: "us-west-2", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_regions({
    #     region_names: ["String"],
    #     all_regions: false,
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.regions #=> Array
    #   resp.regions[0].opt_in_status #=> String
    #   resp.regions[0].region_name #=> String
    #   resp.regions[0].endpoint #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeRegions AWS API Documentation
    #
    # @overload describe_regions(params = {})
    # @param [Hash] params ({})
    def describe_regions(params = {}, options = {})
      req = build_request(:describe_regions, params)
      req.send_request(options)
    end

    # Describes a root volume replacement task. For more information, see
    # [Replace a root volume][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/replace-root.html
    #
    # @option params [Array<String>] :replace_root_volume_task_ids
    #   The ID of the root volume replacement task to view.
    #
    # @option params [Array<Types::Filter>] :filters
    #   Filter to use:
    #
    #   * `instance-id` - The ID of the instance for which the root volume
    #     replacement task was created.
    #
    #   ^
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeReplaceRootVolumeTasksResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeReplaceRootVolumeTasksResult#replace_root_volume_tasks #replace_root_volume_tasks} => Array&lt;Types::ReplaceRootVolumeTask&gt;
    #   * {Types::DescribeReplaceRootVolumeTasksResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_replace_root_volume_tasks({
    #     replace_root_volume_task_ids: ["ReplaceRootVolumeTaskId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.replace_root_volume_tasks #=> Array
    #   resp.replace_root_volume_tasks[0].replace_root_volume_task_id #=> String
    #   resp.replace_root_volume_tasks[0].instance_id #=> String
    #   resp.replace_root_volume_tasks[0].task_state #=> String, one of "pending", "in-progress", "failing", "succeeded", "failed", "failed-detached"
    #   resp.replace_root_volume_tasks[0].start_time #=> String
    #   resp.replace_root_volume_tasks[0].complete_time #=> String
    #   resp.replace_root_volume_tasks[0].tags #=> Array
    #   resp.replace_root_volume_tasks[0].tags[0].key #=> String
    #   resp.replace_root_volume_tasks[0].tags[0].value #=> String
    #   resp.replace_root_volume_tasks[0].image_id #=> String
    #   resp.replace_root_volume_tasks[0].snapshot_id #=> String
    #   resp.replace_root_volume_tasks[0].delete_replaced_root_volume #=> Boolean
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeReplaceRootVolumeTasks AWS API Documentation
    #
    # @overload describe_replace_root_volume_tasks(params = {})
    # @param [Hash] params ({})
    def describe_replace_root_volume_tasks(params = {}, options = {})
      req = build_request(:describe_replace_root_volume_tasks, params)
      req.send_request(options)
    end

    # Describes one or more of the Reserved Instances that you purchased.
    #
    # For more information about Reserved Instances, see [Reserved
    # Instances][1] in the *Amazon EC2 User Guide*.
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/concepts-on-demand-reserved-instances.html
    #
    # @option params [String] :offering_class
    #   Describes whether the Reserved Instance is Standard or Convertible.
    #
    # @option params [Array<String>] :reserved_instances_ids
    #   One or more Reserved Instance IDs.
    #
    #   Default: Describes all your Reserved Instances, or only those
    #   otherwise specified.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `availability-zone` - The Availability Zone where the Reserved
    #     Instance can be used.
    #
    #   * `duration` - The duration of the Reserved Instance (one year or
    #     three years), in seconds (`31536000` \| `94608000`).
    #
    #   * `end` - The time when the Reserved Instance expires (for example,
    #     2015-08-07T11:54:42.000Z).
    #
    #   * `fixed-price` - The purchase price of the Reserved Instance (for
    #     example, 9800.0).
    #
    #   * `instance-type` - The instance type that is covered by the
    #     reservation.
    #
    #   * `scope` - The scope of the Reserved Instance (`Region` or
    #     `Availability Zone`).
    #
    #   * `product-description` - The Reserved Instance product platform
    #     description (`Linux/UNIX` \| `Linux with SQL Server Standard` \|
    #     `Linux with SQL Server Web` \| `Linux with SQL Server Enterprise` \|
    #     `SUSE Linux` \| `Red Hat Enterprise Linux` \| `Red Hat Enterprise
    #     Linux with HA` \| `Windows` \| `Windows with SQL Server Standard` \|
    #     `Windows with SQL Server Web` \| `Windows with SQL Server
    #     Enterprise`).
    #
    #   * `reserved-instances-id` - The ID of the Reserved Instance.
    #
    #   * `start` - The time at which the Reserved Instance purchase request
    #     was placed (for example, 2014-08-07T11:54:42.000Z).
    #
    #   * `state` - The state of the Reserved Instance (`payment-pending` \|
    #     `active` \| `payment-failed` \| `retired`).
    #
    #   * `tag:<key>` - The key/value combination of a tag assigned to the
    #     resource. Use the tag key in the filter name and the tag value as
    #     the filter value. For example, to find all resources that have a tag
    #     with the key `Owner` and the value `TeamA`, specify `tag:Owner` for
    #     the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `usage-price` - The usage price of the Reserved Instance, per hour
    #     (for example, 0.84).
    #
    # @option params [String] :offering_type
    #   The Reserved Instance offering type. If you are using tools that
    #   predate the 2011-11-01 API version, you only have access to the
    #   `Medium Utilization` Reserved Instance offering type.
    #
    # @return [Types::DescribeReservedInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeReservedInstancesResult#reserved_instances #reserved_instances} => Array&lt;Types::ReservedInstances&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_reserved_instances({
    #     offering_class: "standard", # accepts standard, convertible
    #     reserved_instances_ids: ["ReservationId"],
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     offering_type: "Heavy Utilization", # accepts Heavy Utilization, Medium Utilization, Light Utilization, No Upfront, Partial Upfront, All Upfront
    #   })
    #
    # @example Response structure
    #
    #   resp.reserved_instances #=> Array
    #   resp.reserved_instances[0].currency_code #=> String, one of "USD"
    #   resp.reserved_instances[0].instance_tenancy #=> String, one of "default", "dedicated", "host"
    #   resp.reserved_instances[0].offering_class #=> String, one of "standard", "convertible"
    #   resp.reserved_instances[0].offering_type #=> String, one of "Heavy Utilization", "Medium Utilization", "Light Utilization", "No Upfront", "Partial Upfront", "All Upfront"
    #   resp.reserved_instances[0].recurring_charges #=> Array
    #   resp.reserved_instances[0].recurring_charges[0].amount #=> Float
    #   resp.reserved_instances[0].recurring_charges[0].frequency #=> String, one of "Hourly"
    #   resp.reserved_instances[0].scope #=> String, one of "Availability Zone", "Region"
    #   resp.reserved_instances[0].tags #=> Array
    #   resp.reserved_instances[0].tags[0].key #=> String
    #   resp.reserved_instances[0].tags[0].value #=> String
    #   resp.reserved_instances[0].reserved_instances_id #=> String
    #   resp.reserved_instances[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.reserved_instances[0].availability_zone #=> String
    #   resp.reserved_instances[0].start #=> Time
    #   resp.reserved_instances[0].end #=> Time
    #   resp.reserved_instances[0].duration #=> Integer
    #   resp.reserved_instances[0].usage_price #=> Float
    #   resp.reserved_instances[0].fixed_price #=> Float
    #   resp.reserved_instances[0].instance_count #=> Integer
    #   resp.reserved_instances[0].product_description #=> String, one of "Linux/UNIX", "Linux/UNIX (Amazon VPC)", "Windows", "Windows (Amazon VPC)"
    #   resp.reserved_instances[0].state #=> String, one of "payment-pending", "active", "payment-failed", "retired", "queued", "queued-deleted"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeReservedInstances AWS API Documentation
    #
    # @overload describe_reserved_instances(params = {})
    # @param [Hash] params ({})
    def describe_reserved_instances(params = {}, options = {})
      req = build_request(:describe_reserved_instances, params)
      req.send_request(options)
    end

    # Describes your account's Reserved Instance listings in the Reserved
    # Instance Marketplace.
    #
    # The Reserved Instance Marketplace matches sellers who want to resell
    # Reserved Instance capacity that they no longer need with buyers who
    # want to purchase additional capacity. Reserved Instances bought and
    # sold through the Reserved Instance Marketplace work like any other
    # Reserved Instances.
    #
    # As a seller, you choose to list some or all of your Reserved
    # Instances, and you specify the upfront price to receive for them. Your
    # Reserved Instances are then listed in the Reserved Instance
    # Marketplace and are available for purchase.
    #
    # As a buyer, you specify the configuration of the Reserved Instance to
    # purchase, and the Marketplace matches what you're searching for with
    # what's available. The Marketplace first sells the lowest priced
    # Reserved Instances to you, and continues to sell available Reserved
    # Instance listings to you until your demand is met. You are charged
    # based on the total price of all of the listings that you purchase.
    #
    # For more information, see [Sell in the Reserved Instance
    # Marketplace][1] in the *Amazon EC2 User Guide*.
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ri-market-general.html
    #
    # @option params [String] :reserved_instances_id
    #   One or more Reserved Instance IDs.
    #
    # @option params [String] :reserved_instances_listing_id
    #   One or more Reserved Instance listing IDs.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `reserved-instances-id` - The ID of the Reserved Instances.
    #
    #   * `reserved-instances-listing-id` - The ID of the Reserved Instances
    #     listing.
    #
    #   * `status` - The status of the Reserved Instance listing (`pending` \|
    #     `active` \| `cancelled` \| `closed`).
    #
    #   * `status-message` - The reason for the status.
    #
    # @return [Types::DescribeReservedInstancesListingsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeReservedInstancesListingsResult#reserved_instances_listings #reserved_instances_listings} => Array&lt;Types::ReservedInstancesListing&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_reserved_instances_listings({
    #     reserved_instances_id: "ReservationId",
    #     reserved_instances_listing_id: "ReservedInstancesListingId",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.reserved_instances_listings #=> Array
    #   resp.reserved_instances_listings[0].client_token #=> String
    #   resp.reserved_instances_listings[0].create_date #=> Time
    #   resp.reserved_instances_listings[0].instance_counts #=> Array
    #   resp.reserved_instances_listings[0].instance_counts[0].instance_count #=> Integer
    #   resp.reserved_instances_listings[0].instance_counts[0].state #=> String, one of "available", "sold", "cancelled", "pending"
    #   resp.reserved_instances_listings[0].price_schedules #=> Array
    #   resp.reserved_instances_listings[0].price_schedules[0].active #=> Boolean
    #   resp.reserved_instances_listings[0].price_schedules[0].currency_code #=> String, one of "USD"
    #   resp.reserved_instances_listings[0].price_schedules[0].price #=> Float
    #   resp.reserved_instances_listings[0].price_schedules[0].term #=> Integer
    #   resp.reserved_instances_listings[0].reserved_instances_id #=> String
    #   resp.reserved_instances_listings[0].reserved_instances_listing_id #=> String
    #   resp.reserved_instances_listings[0].status #=> String, one of "active", "pending", "cancelled", "closed"
    #   resp.reserved_instances_listings[0].status_message #=> String
    #   resp.reserved_instances_listings[0].tags #=> Array
    #   resp.reserved_instances_listings[0].tags[0].key #=> String
    #   resp.reserved_instances_listings[0].tags[0].value #=> String
    #   resp.reserved_instances_listings[0].update_date #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeReservedInstancesListings AWS API Documentation
    #
    # @overload describe_reserved_instances_listings(params = {})
    # @param [Hash] params ({})
    def describe_reserved_instances_listings(params = {}, options = {})
      req = build_request(:describe_reserved_instances_listings, params)
      req.send_request(options)
    end

    # Describes the modifications made to your Reserved Instances. If no
    # parameter is specified, information about all your Reserved Instances
    # modification requests is returned. If a modification ID is specified,
    # only information about the specific modification is returned.
    #
    # For more information, see [Modify Reserved Instances][1] in the
    # *Amazon EC2 User Guide*.
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ri-modifying.html
    #
    # @option params [Array<String>] :reserved_instances_modification_ids
    #   IDs for the submitted modification request.
    #
    # @option params [String] :next_token
    #   The token to retrieve the next page of results.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `client-token` - The idempotency token for the modification request.
    #
    #   * `create-date` - The time when the modification request was created.
    #
    #   * `effective-date` - The time when the modification becomes effective.
    #
    #   * `modification-result.reserved-instances-id` - The ID for the
    #     Reserved Instances created as part of the modification request. This
    #     ID is only available when the status of the modification is
    #     `fulfilled`.
    #
    #   * `modification-result.target-configuration.availability-zone` - The
    #     Availability Zone for the new Reserved Instances.
    #
    #   * `modification-result.target-configuration.instance-count ` - The
    #     number of new Reserved Instances.
    #
    #   * `modification-result.target-configuration.instance-type` - The
    #     instance type of the new Reserved Instances.
    #
    #   * `reserved-instances-id` - The ID of the Reserved Instances modified.
    #
    #   * `reserved-instances-modification-id` - The ID of the modification
    #     request.
    #
    #   * `status` - The status of the Reserved Instances modification request
    #     (`processing` \| `fulfilled` \| `failed`).
    #
    #   * `status-message` - The reason for the status.
    #
    #   * `update-date` - The time when the modification request was last
    #     updated.
    #
    # @return [Types::DescribeReservedInstancesModificationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeReservedInstancesModificationsResult#next_token #next_token} => String
    #   * {Types::DescribeReservedInstancesModificationsResult#reserved_instances_modifications #reserved_instances_modifications} => Array&lt;Types::ReservedInstancesModification&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_reserved_instances_modifications({
    #     reserved_instances_modification_ids: ["ReservedInstancesModificationId"],
    #     next_token: "String",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.reserved_instances_modifications #=> Array
    #   resp.reserved_instances_modifications[0].client_token #=> String
    #   resp.reserved_instances_modifications[0].create_date #=> Time
    #   resp.reserved_instances_modifications[0].effective_date #=> Time
    #   resp.reserved_instances_modifications[0].modification_results #=> Array
    #   resp.reserved_instances_modifications[0].modification_results[0].reserved_instances_id #=> String
    #   resp.reserved_instances_modifications[0].modification_results[0].target_configuration.availability_zone #=> String
    #   resp.reserved_instances_modifications[0].modification_results[0].target_configuration.instance_count #=> Integer
    #   resp.reserved_instances_modifications[0].modification_results[0].target_configuration.instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.reserved_instances_modifications[0].modification_results[0].target_configuration.platform #=> String
    #   resp.reserved_instances_modifications[0].modification_results[0].target_configuration.scope #=> String, one of "Availability Zone", "Region"
    #   resp.reserved_instances_modifications[0].reserved_instances_ids #=> Array
    #   resp.reserved_instances_modifications[0].reserved_instances_ids[0].reserved_instances_id #=> String
    #   resp.reserved_instances_modifications[0].reserved_instances_modification_id #=> String
    #   resp.reserved_instances_modifications[0].status #=> String
    #   resp.reserved_instances_modifications[0].status_message #=> String
    #   resp.reserved_instances_modifications[0].update_date #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeReservedInstancesModifications AWS API Documentation
    #
    # @overload describe_reserved_instances_modifications(params = {})
    # @param [Hash] params ({})
    def describe_reserved_instances_modifications(params = {}, options = {})
      req = build_request(:describe_reserved_instances_modifications, params)
      req.send_request(options)
    end

    # Describes Reserved Instance offerings that are available for purchase.
    # With Reserved Instances, you purchase the right to launch instances
    # for a period of time. During that time period, you do not receive
    # insufficient capacity errors, and you pay a lower usage rate than the
    # rate charged for On-Demand instances for the actual time used.
    #
    # If you have listed your own Reserved Instances for sale in the
    # Reserved Instance Marketplace, they will be excluded from these
    # results. This is to ensure that you do not purchase your own Reserved
    # Instances.
    #
    # For more information, see [Sell in the Reserved Instance
    # Marketplace][1] in the *Amazon EC2 User Guide*.
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ri-market-general.html
    #
    # @option params [String] :availability_zone
    #   The Availability Zone in which the Reserved Instance can be used.
    #
    # @option params [Boolean] :include_marketplace
    #   Include Reserved Instance Marketplace offerings in the response.
    #
    # @option params [String] :instance_type
    #   The instance type that the reservation will cover (for example,
    #   `m1.small`). For more information, see [Amazon EC2 instance types][1]
    #   in the *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html
    #
    # @option params [Integer] :max_duration
    #   The maximum duration (in seconds) to filter when searching for
    #   offerings.
    #
    #   Default: 94608000 (3 years)
    #
    # @option params [Integer] :max_instance_count
    #   The maximum number of instances to filter when searching for
    #   offerings.
    #
    #   Default: 20
    #
    # @option params [Integer] :min_duration
    #   The minimum duration (in seconds) to filter when searching for
    #   offerings.
    #
    #   Default: 2592000 (1 month)
    #
    # @option params [String] :offering_class
    #   The offering class of the Reserved Instance. Can be `standard` or
    #   `convertible`.
    #
    # @option params [String] :product_description
    #   The Reserved Instance product platform description. Instances that
    #   include `(Amazon VPC)` in the description are for use with Amazon VPC.
    #
    # @option params [Array<String>] :reserved_instances_offering_ids
    #   One or more Reserved Instances offering IDs.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `availability-zone` - The Availability Zone where the Reserved
    #     Instance can be used.
    #
    #   * `duration` - The duration of the Reserved Instance (for example, one
    #     year or three years), in seconds (`31536000` \| `94608000`).
    #
    #   * `fixed-price` - The purchase price of the Reserved Instance (for
    #     example, 9800.0).
    #
    #   * `instance-type` - The instance type that is covered by the
    #     reservation.
    #
    #   * `marketplace` - Set to `true` to show only Reserved Instance
    #     Marketplace offerings. When this filter is not used, which is the
    #     default behavior, all offerings from both Amazon Web Services and
    #     the Reserved Instance Marketplace are listed.
    #
    #   * `product-description` - The Reserved Instance product platform
    #     description (`Linux/UNIX` \| `Linux with SQL Server Standard` \|
    #     `Linux with SQL Server Web` \| `Linux with SQL Server Enterprise` \|
    #     `SUSE Linux` \| `Red Hat Enterprise Linux` \| `Red Hat Enterprise
    #     Linux with HA` \| `Windows` \| `Windows with SQL Server Standard` \|
    #     `Windows with SQL Server Web` \| `Windows with SQL Server
    #     Enterprise`).
    #
    #   * `reserved-instances-offering-id` - The Reserved Instances offering
    #     ID.
    #
    #   * `scope` - The scope of the Reserved Instance (`Availability Zone` or
    #     `Region`).
    #
    #   * `usage-price` - The usage price of the Reserved Instance, per hour
    #     (for example, 0.84).
    #
    # @option params [String] :instance_tenancy
    #   The tenancy of the instances covered by the reservation. A Reserved
    #   Instance with a tenancy of `dedicated` is applied to instances that
    #   run in a VPC on single-tenant hardware (i.e., Dedicated Instances).
    #
    #   **Important:** The `host` value cannot be used with this parameter.
    #   Use the `default` or `dedicated` values only.
    #
    #   Default: `default`
    #
    # @option params [String] :offering_type
    #   The Reserved Instance offering type. If you are using tools that
    #   predate the 2011-11-01 API version, you only have access to the
    #   `Medium Utilization` Reserved Instance offering type.
    #
    # @option params [String] :next_token
    #   The token to retrieve the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return for the request in a single
    #   page. The remaining results of the initial request can be seen by
    #   sending another request with the returned `NextToken` value. The
    #   maximum is 100.
    #
    #   Default: 100
    #
    # @return [Types::DescribeReservedInstancesOfferingsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeReservedInstancesOfferingsResult#next_token #next_token} => String
    #   * {Types::DescribeReservedInstancesOfferingsResult#reserved_instances_offerings #reserved_instances_offerings} => Array&lt;Types::ReservedInstancesOffering&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_reserved_instances_offerings({
    #     availability_zone: "String",
    #     include_marketplace: false,
    #     instance_type: "a1.medium", # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #     max_duration: 1,
    #     max_instance_count: 1,
    #     min_duration: 1,
    #     offering_class: "standard", # accepts standard, convertible
    #     product_description: "Linux/UNIX", # accepts Linux/UNIX, Linux/UNIX (Amazon VPC), Windows, Windows (Amazon VPC)
    #     reserved_instances_offering_ids: ["ReservedInstancesOfferingId"],
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     instance_tenancy: "default", # accepts default, dedicated, host
    #     offering_type: "Heavy Utilization", # accepts Heavy Utilization, Medium Utilization, Light Utilization, No Upfront, Partial Upfront, All Upfront
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.reserved_instances_offerings #=> Array
    #   resp.reserved_instances_offerings[0].currency_code #=> String, one of "USD"
    #   resp.reserved_instances_offerings[0].instance_tenancy #=> String, one of "default", "dedicated", "host"
    #   resp.reserved_instances_offerings[0].marketplace #=> Boolean
    #   resp.reserved_instances_offerings[0].offering_class #=> String, one of "standard", "convertible"
    #   resp.reserved_instances_offerings[0].offering_type #=> String, one of "Heavy Utilization", "Medium Utilization", "Light Utilization", "No Upfront", "Partial Upfront", "All Upfront"
    #   resp.reserved_instances_offerings[0].pricing_details #=> Array
    #   resp.reserved_instances_offerings[0].pricing_details[0].count #=> Integer
    #   resp.reserved_instances_offerings[0].pricing_details[0].price #=> Float
    #   resp.reserved_instances_offerings[0].recurring_charges #=> Array
    #   resp.reserved_instances_offerings[0].recurring_charges[0].amount #=> Float
    #   resp.reserved_instances_offerings[0].recurring_charges[0].frequency #=> String, one of "Hourly"
    #   resp.reserved_instances_offerings[0].scope #=> String, one of "Availability Zone", "Region"
    #   resp.reserved_instances_offerings[0].reserved_instances_offering_id #=> String
    #   resp.reserved_instances_offerings[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.reserved_instances_offerings[0].availability_zone #=> String
    #   resp.reserved_instances_offerings[0].duration #=> Integer
    #   resp.reserved_instances_offerings[0].usage_price #=> Float
    #   resp.reserved_instances_offerings[0].fixed_price #=> Float
    #   resp.reserved_instances_offerings[0].product_description #=> String, one of "Linux/UNIX", "Linux/UNIX (Amazon VPC)", "Windows", "Windows (Amazon VPC)"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeReservedInstancesOfferings AWS API Documentation
    #
    # @overload describe_reserved_instances_offerings(params = {})
    # @param [Hash] params ({})
    def describe_reserved_instances_offerings(params = {}, options = {})
      req = build_request(:describe_reserved_instances_offerings, params)
      req.send_request(options)
    end

    # Describes your route tables. The default is to describe all your route
    # tables. Alternatively, you can specify specific route table IDs or
    # filter the results to include only the route tables that match
    # specific criteria.
    #
    # Each subnet in your VPC must be associated with a route table. If a
    # subnet is not explicitly associated with any route table, it is
    # implicitly associated with the main route table. This command does not
    # return the subnet ID for implicit associations.
    #
    # For more information, see [Route tables][1] in the *Amazon VPC User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Route_Tables.html
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :route_table_ids
    #   The IDs of the route tables.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `association.gateway-id` - The ID of the gateway involved in the
    #     association.
    #
    #   * `association.route-table-association-id` - The ID of an association
    #     ID for the route table.
    #
    #   * `association.route-table-id` - The ID of the route table involved in
    #     the association.
    #
    #   * `association.subnet-id` - The ID of the subnet involved in the
    #     association.
    #
    #   * `association.main` - Indicates whether the route table is the main
    #     route table for the VPC (`true` \| `false`). Route tables that do
    #     not have an association ID are not returned in the response.
    #
    #   * `owner-id` - The ID of the Amazon Web Services account that owns the
    #     route table.
    #
    #   * `route-table-id` - The ID of the route table.
    #
    #   * `route.destination-cidr-block` - The IPv4 CIDR range specified in a
    #     route in the table.
    #
    #   * `route.destination-ipv6-cidr-block` - The IPv6 CIDR range specified
    #     in a route in the route table.
    #
    #   * `route.destination-prefix-list-id` - The ID (prefix) of the Amazon
    #     Web Services service specified in a route in the table.
    #
    #   * `route.egress-only-internet-gateway-id` - The ID of an egress-only
    #     Internet gateway specified in a route in the route table.
    #
    #   * `route.gateway-id` - The ID of a gateway specified in a route in the
    #     table.
    #
    #   * `route.instance-id` - The ID of an instance specified in a route in
    #     the table.
    #
    #   * `route.nat-gateway-id` - The ID of a NAT gateway.
    #
    #   * `route.transit-gateway-id` - The ID of a transit gateway.
    #
    #   * `route.origin` - Describes how the route was created.
    #     `CreateRouteTable` indicates that the route was automatically
    #     created when the route table was created; `CreateRoute` indicates
    #     that the route was manually added to the route table;
    #     `EnableVgwRoutePropagation` indicates that the route was propagated
    #     by route propagation.
    #
    #   * `route.state` - The state of a route in the route table (`active` \|
    #     `blackhole`). The blackhole state indicates that the route's target
    #     isn't available (for example, the specified gateway isn't attached
    #     to the VPC, the specified NAT instance has been terminated, and so
    #     on).
    #
    #   * `route.vpc-peering-connection-id` - The ID of a VPC peering
    #     connection specified in a route in the table.
    #
    #   * `tag` - The key/value combination of a tag assigned to the resource.
    #     Use the tag key in the filter name and the tag value as the filter
    #     value. For example, to find all resources that have a tag with the
    #     key `Owner` and the value `TeamA`, specify `tag:Owner` for the
    #     filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `vpc-id` - The ID of the VPC for the route table.
    #
    # @return [Types::DescribeRouteTablesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeRouteTablesResult#route_tables #route_tables} => Array&lt;Types::RouteTable&gt;
    #   * {Types::DescribeRouteTablesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe a route table
    #
    #   # This example describes the specified route table.
    #
    #   resp = client.describe_route_tables({
    #     route_table_ids: [
    #       "rtb-1f382e7d", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     route_tables: [
    #       {
    #         associations: [
    #           {
    #             main: true, 
    #             route_table_association_id: "rtbassoc-d8ccddba", 
    #             route_table_id: "rtb-1f382e7d", 
    #           }, 
    #         ], 
    #         propagating_vgws: [
    #         ], 
    #         route_table_id: "rtb-1f382e7d", 
    #         routes: [
    #           {
    #             destination_cidr_block: "10.0.0.0/16", 
    #             gateway_id: "local", 
    #             state: "active", 
    #           }, 
    #         ], 
    #         tags: [
    #         ], 
    #         vpc_id: "vpc-a01106c2", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_route_tables({
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #     route_table_ids: ["RouteTableId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.route_tables #=> Array
    #   resp.route_tables[0].associations #=> Array
    #   resp.route_tables[0].associations[0].main #=> Boolean
    #   resp.route_tables[0].associations[0].route_table_association_id #=> String
    #   resp.route_tables[0].associations[0].route_table_id #=> String
    #   resp.route_tables[0].associations[0].subnet_id #=> String
    #   resp.route_tables[0].associations[0].gateway_id #=> String
    #   resp.route_tables[0].associations[0].association_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failed"
    #   resp.route_tables[0].associations[0].association_state.status_message #=> String
    #   resp.route_tables[0].propagating_vgws #=> Array
    #   resp.route_tables[0].propagating_vgws[0].gateway_id #=> String
    #   resp.route_tables[0].route_table_id #=> String
    #   resp.route_tables[0].routes #=> Array
    #   resp.route_tables[0].routes[0].destination_cidr_block #=> String
    #   resp.route_tables[0].routes[0].destination_ipv_6_cidr_block #=> String
    #   resp.route_tables[0].routes[0].destination_prefix_list_id #=> String
    #   resp.route_tables[0].routes[0].egress_only_internet_gateway_id #=> String
    #   resp.route_tables[0].routes[0].gateway_id #=> String
    #   resp.route_tables[0].routes[0].instance_id #=> String
    #   resp.route_tables[0].routes[0].instance_owner_id #=> String
    #   resp.route_tables[0].routes[0].nat_gateway_id #=> String
    #   resp.route_tables[0].routes[0].transit_gateway_id #=> String
    #   resp.route_tables[0].routes[0].local_gateway_id #=> String
    #   resp.route_tables[0].routes[0].carrier_gateway_id #=> String
    #   resp.route_tables[0].routes[0].network_interface_id #=> String
    #   resp.route_tables[0].routes[0].origin #=> String, one of "CreateRouteTable", "CreateRoute", "EnableVgwRoutePropagation"
    #   resp.route_tables[0].routes[0].state #=> String, one of "active", "blackhole"
    #   resp.route_tables[0].routes[0].vpc_peering_connection_id #=> String
    #   resp.route_tables[0].routes[0].core_network_arn #=> String
    #   resp.route_tables[0].tags #=> Array
    #   resp.route_tables[0].tags[0].key #=> String
    #   resp.route_tables[0].tags[0].value #=> String
    #   resp.route_tables[0].vpc_id #=> String
    #   resp.route_tables[0].owner_id #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeRouteTables AWS API Documentation
    #
    # @overload describe_route_tables(params = {})
    # @param [Hash] params ({})
    def describe_route_tables(params = {}, options = {})
      req = build_request(:describe_route_tables, params)
      req.send_request(options)
    end

    # Finds available schedules that meet the specified criteria.
    #
    # You can search for an available schedule no more than 3 months in
    # advance. You must meet the minimum required duration of 1,200 hours
    # per year. For example, the minimum daily schedule is 4 hours, the
    # minimum weekly schedule is 24 hours, and the minimum monthly schedule
    # is 100 hours.
    #
    # After you find a schedule that meets your needs, call
    # PurchaseScheduledInstances to purchase Scheduled Instances with that
    # schedule.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `availability-zone` - The Availability Zone (for example,
    #     `us-west-2a`).
    #
    #   * `instance-type` - The instance type (for example, `c4.large`).
    #
    #   * `platform` - The platform (`Linux/UNIX` or `Windows`).
    #
    # @option params [required, Types::SlotDateTimeRangeRequest] :first_slot_start_time_range
    #   The time period for the first schedule to start.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in a single call. This value
    #   can be between 5 and 300. The default value is 300. To retrieve the
    #   remaining results, make another call with the returned `NextToken`
    #   value.
    #
    # @option params [Integer] :max_slot_duration_in_hours
    #   The maximum available duration, in hours. This value must be greater
    #   than `MinSlotDurationInHours` and less than 1,720.
    #
    # @option params [Integer] :min_slot_duration_in_hours
    #   The minimum available duration, in hours. The minimum required
    #   duration is 1,200 hours per year. For example, the minimum daily
    #   schedule is 4 hours, the minimum weekly schedule is 24 hours, and the
    #   minimum monthly schedule is 100 hours.
    #
    # @option params [String] :next_token
    #   The token for the next set of results.
    #
    # @option params [required, Types::ScheduledInstanceRecurrenceRequest] :recurrence
    #   The schedule recurrence.
    #
    # @return [Types::DescribeScheduledInstanceAvailabilityResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeScheduledInstanceAvailabilityResult#next_token #next_token} => String
    #   * {Types::DescribeScheduledInstanceAvailabilityResult#scheduled_instance_availability_set #scheduled_instance_availability_set} => Array&lt;Types::ScheduledInstanceAvailability&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_scheduled_instance_availability({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     first_slot_start_time_range: { # required
    #       earliest_time: Time.now, # required
    #       latest_time: Time.now, # required
    #     },
    #     max_results: 1,
    #     max_slot_duration_in_hours: 1,
    #     min_slot_duration_in_hours: 1,
    #     next_token: "String",
    #     recurrence: { # required
    #       frequency: "String",
    #       interval: 1,
    #       occurrence_days: [1],
    #       occurrence_relative_to_end: false,
    #       occurrence_unit: "String",
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.scheduled_instance_availability_set #=> Array
    #   resp.scheduled_instance_availability_set[0].availability_zone #=> String
    #   resp.scheduled_instance_availability_set[0].available_instance_count #=> Integer
    #   resp.scheduled_instance_availability_set[0].first_slot_start_time #=> Time
    #   resp.scheduled_instance_availability_set[0].hourly_price #=> String
    #   resp.scheduled_instance_availability_set[0].instance_type #=> String
    #   resp.scheduled_instance_availability_set[0].max_term_duration_in_days #=> Integer
    #   resp.scheduled_instance_availability_set[0].min_term_duration_in_days #=> Integer
    #   resp.scheduled_instance_availability_set[0].network_platform #=> String
    #   resp.scheduled_instance_availability_set[0].platform #=> String
    #   resp.scheduled_instance_availability_set[0].purchase_token #=> String
    #   resp.scheduled_instance_availability_set[0].recurrence.frequency #=> String
    #   resp.scheduled_instance_availability_set[0].recurrence.interval #=> Integer
    #   resp.scheduled_instance_availability_set[0].recurrence.occurrence_day_set #=> Array
    #   resp.scheduled_instance_availability_set[0].recurrence.occurrence_day_set[0] #=> Integer
    #   resp.scheduled_instance_availability_set[0].recurrence.occurrence_relative_to_end #=> Boolean
    #   resp.scheduled_instance_availability_set[0].recurrence.occurrence_unit #=> String
    #   resp.scheduled_instance_availability_set[0].slot_duration_in_hours #=> Integer
    #   resp.scheduled_instance_availability_set[0].total_scheduled_instance_hours #=> Integer
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeScheduledInstanceAvailability AWS API Documentation
    #
    # @overload describe_scheduled_instance_availability(params = {})
    # @param [Hash] params ({})
    def describe_scheduled_instance_availability(params = {}, options = {})
      req = build_request(:describe_scheduled_instance_availability, params)
      req.send_request(options)
    end

    # Describes the specified Scheduled Instances or all your Scheduled
    # Instances.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `availability-zone` - The Availability Zone (for example,
    #     `us-west-2a`).
    #
    #   * `instance-type` - The instance type (for example, `c4.large`).
    #
    #   * `platform` - The platform (`Linux/UNIX` or `Windows`).
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in a single call. This value
    #   can be between 5 and 300. The default value is 100. To retrieve the
    #   remaining results, make another call with the returned `NextToken`
    #   value.
    #
    # @option params [String] :next_token
    #   The token for the next set of results.
    #
    # @option params [Array<String>] :scheduled_instance_ids
    #   The Scheduled Instance IDs.
    #
    # @option params [Types::SlotStartTimeRangeRequest] :slot_start_time_range
    #   The time period for the first schedule to start.
    #
    # @return [Types::DescribeScheduledInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeScheduledInstancesResult#next_token #next_token} => String
    #   * {Types::DescribeScheduledInstancesResult#scheduled_instance_set #scheduled_instance_set} => Array&lt;Types::ScheduledInstance&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_scheduled_instances({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     scheduled_instance_ids: ["ScheduledInstanceId"],
    #     slot_start_time_range: {
    #       earliest_time: Time.now,
    #       latest_time: Time.now,
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.scheduled_instance_set #=> Array
    #   resp.scheduled_instance_set[0].availability_zone #=> String
    #   resp.scheduled_instance_set[0].create_date #=> Time
    #   resp.scheduled_instance_set[0].hourly_price #=> String
    #   resp.scheduled_instance_set[0].instance_count #=> Integer
    #   resp.scheduled_instance_set[0].instance_type #=> String
    #   resp.scheduled_instance_set[0].network_platform #=> String
    #   resp.scheduled_instance_set[0].next_slot_start_time #=> Time
    #   resp.scheduled_instance_set[0].platform #=> String
    #   resp.scheduled_instance_set[0].previous_slot_end_time #=> Time
    #   resp.scheduled_instance_set[0].recurrence.frequency #=> String
    #   resp.scheduled_instance_set[0].recurrence.interval #=> Integer
    #   resp.scheduled_instance_set[0].recurrence.occurrence_day_set #=> Array
    #   resp.scheduled_instance_set[0].recurrence.occurrence_day_set[0] #=> Integer
    #   resp.scheduled_instance_set[0].recurrence.occurrence_relative_to_end #=> Boolean
    #   resp.scheduled_instance_set[0].recurrence.occurrence_unit #=> String
    #   resp.scheduled_instance_set[0].scheduled_instance_id #=> String
    #   resp.scheduled_instance_set[0].slot_duration_in_hours #=> Integer
    #   resp.scheduled_instance_set[0].term_end_date #=> Time
    #   resp.scheduled_instance_set[0].term_start_date #=> Time
    #   resp.scheduled_instance_set[0].total_scheduled_instance_hours #=> Integer
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeScheduledInstances AWS API Documentation
    #
    # @overload describe_scheduled_instances(params = {})
    # @param [Hash] params ({})
    def describe_scheduled_instances(params = {}, options = {})
      req = build_request(:describe_scheduled_instances, params)
      req.send_request(options)
    end

    # Describes the VPCs on the other side of a VPC peering or Transit
    # Gateway connection that are referencing the security groups you've
    # specified in this request.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :group_id
    #   The IDs of the security groups in your account.
    #
    # @return [Types::DescribeSecurityGroupReferencesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeSecurityGroupReferencesResult#security_group_reference_set #security_group_reference_set} => Array&lt;Types::SecurityGroupReference&gt;
    #
    #
    # @example Example: To describe security group references
    #
    #   # This example describes the security group references for the specified security group.
    #
    #   resp = client.describe_security_group_references({
    #     group_id: [
    #       "sg-903004f8", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     security_group_reference_set: [
    #       {
    #         group_id: "sg-903004f8", 
    #         referencing_vpc_id: "vpc-1a2b3c4d", 
    #         vpc_peering_connection_id: "pcx-b04deed9", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_security_group_references({
    #     dry_run: false,
    #     group_id: ["SecurityGroupId"], # required
    #   })
    #
    # @example Response structure
    #
    #   resp.security_group_reference_set #=> Array
    #   resp.security_group_reference_set[0].group_id #=> String
    #   resp.security_group_reference_set[0].referencing_vpc_id #=> String
    #   resp.security_group_reference_set[0].vpc_peering_connection_id #=> String
    #   resp.security_group_reference_set[0].transit_gateway_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSecurityGroupReferences AWS API Documentation
    #
    # @overload describe_security_group_references(params = {})
    # @param [Hash] params ({})
    def describe_security_group_references(params = {}, options = {})
      req = build_request(:describe_security_group_references, params)
      req.send_request(options)
    end

    # Describes one or more of your security group rules.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `group-id` - The ID of the security group.
    #
    #   * `security-group-rule-id` - The ID of the security group rule.
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    # @option params [Array<String>] :security_group_rule_ids
    #   The IDs of the security group rules.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. This value can be between 5 and 1000. If this parameter is
    #   not specified, then all items are returned. For more information, see
    #   [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @return [Types::DescribeSecurityGroupRulesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeSecurityGroupRulesResult#security_group_rules #security_group_rules} => Array&lt;Types::SecurityGroupRule&gt;
    #   * {Types::DescribeSecurityGroupRulesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_security_group_rules({
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     security_group_rule_ids: ["String"],
    #     dry_run: false,
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.security_group_rules #=> Array
    #   resp.security_group_rules[0].security_group_rule_id #=> String
    #   resp.security_group_rules[0].group_id #=> String
    #   resp.security_group_rules[0].group_owner_id #=> String
    #   resp.security_group_rules[0].is_egress #=> Boolean
    #   resp.security_group_rules[0].ip_protocol #=> String
    #   resp.security_group_rules[0].from_port #=> Integer
    #   resp.security_group_rules[0].to_port #=> Integer
    #   resp.security_group_rules[0].cidr_ipv_4 #=> String
    #   resp.security_group_rules[0].cidr_ipv_6 #=> String
    #   resp.security_group_rules[0].prefix_list_id #=> String
    #   resp.security_group_rules[0].referenced_group_info.group_id #=> String
    #   resp.security_group_rules[0].referenced_group_info.peering_status #=> String
    #   resp.security_group_rules[0].referenced_group_info.user_id #=> String
    #   resp.security_group_rules[0].referenced_group_info.vpc_id #=> String
    #   resp.security_group_rules[0].referenced_group_info.vpc_peering_connection_id #=> String
    #   resp.security_group_rules[0].description #=> String
    #   resp.security_group_rules[0].tags #=> Array
    #   resp.security_group_rules[0].tags[0].key #=> String
    #   resp.security_group_rules[0].tags[0].value #=> String
    #   resp.security_group_rules[0].security_group_rule_arn #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSecurityGroupRules AWS API Documentation
    #
    # @overload describe_security_group_rules(params = {})
    # @param [Hash] params ({})
    def describe_security_group_rules(params = {}, options = {})
      req = build_request(:describe_security_group_rules, params)
      req.send_request(options)
    end

    # Describes security group VPC associations made with
    # [AssociateSecurityGroupVpc][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_AssociateSecurityGroupVpc.html
    #
    # @option params [Array<Types::Filter>] :filters
    #   Security group VPC association filters.
    #
    #   * `group-id`: The security group ID.
    #
    #   * `vpc-id`: The ID of the associated VPC.
    #
    #   * `vpc-owner-id`: The account ID of the VPC owner.
    #
    #   * `state`: The state of the association.
    #
    #   * `tag:<key>`: The key/value combination of a tag assigned to the
    #     resource. Use the tag key in the filter name and the tag value as
    #     the filter value. For example, to find all resources that have a tag
    #     with the key `Owner` and the value `TeamA`, specify `tag:Owner` for
    #     the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key`: The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeSecurityGroupVpcAssociationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeSecurityGroupVpcAssociationsResult#security_group_vpc_associations #security_group_vpc_associations} => Array&lt;Types::SecurityGroupVpcAssociation&gt;
    #   * {Types::DescribeSecurityGroupVpcAssociationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_security_group_vpc_associations({
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.security_group_vpc_associations #=> Array
    #   resp.security_group_vpc_associations[0].group_id #=> String
    #   resp.security_group_vpc_associations[0].vpc_id #=> String
    #   resp.security_group_vpc_associations[0].vpc_owner_id #=> String
    #   resp.security_group_vpc_associations[0].state #=> String, one of "associating", "associated", "association-failed", "disassociating", "disassociated", "disassociation-failed"
    #   resp.security_group_vpc_associations[0].state_reason #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSecurityGroupVpcAssociations AWS API Documentation
    #
    # @overload describe_security_group_vpc_associations(params = {})
    # @param [Hash] params ({})
    def describe_security_group_vpc_associations(params = {}, options = {})
      req = build_request(:describe_security_group_vpc_associations, params)
      req.send_request(options)
    end

    # Describes the specified security groups or all of your security
    # groups.
    #
    # @option params [Array<String>] :group_ids
    #   The IDs of the security groups. Required for security groups in a
    #   nondefault VPC.
    #
    #   Default: Describes all of your security groups.
    #
    # @option params [Array<String>] :group_names
    #   \[Default VPC\] The names of the security groups. You can specify
    #   either the security group name or the security group ID.
    #
    #   Default: Describes all of your security groups.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. This value can be between 5 and 1000. If this parameter is
    #   not specified, then all items are returned. For more information, see
    #   [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters. If using multiple filters for rules, the results include
    #   security groups for which any combination of rules - not necessarily a
    #   single rule - match all filters.
    #
    #   * `description` - The description of the security group.
    #
    #   * `egress.ip-permission.cidr` - An IPv4 CIDR block for an outbound
    #     security group rule.
    #
    #   * `egress.ip-permission.from-port` - For an outbound rule, the start
    #     of port range for the TCP and UDP protocols, or an ICMP type number.
    #
    #   * `egress.ip-permission.group-id` - The ID of a security group that
    #     has been referenced in an outbound security group rule.
    #
    #   * `egress.ip-permission.group-name` - The name of a security group
    #     that is referenced in an outbound security group rule.
    #
    #   * `egress.ip-permission.ipv6-cidr` - An IPv6 CIDR block for an
    #     outbound security group rule.
    #
    #   * `egress.ip-permission.prefix-list-id` - The ID of a prefix list to
    #     which a security group rule allows outbound access.
    #
    #   * `egress.ip-permission.protocol` - The IP protocol for an outbound
    #     security group rule (`tcp` \| `udp` \| `icmp`, a protocol number, or
    #     -1 for all protocols).
    #
    #   * `egress.ip-permission.to-port` - For an outbound rule, the end of
    #     port range for the TCP and UDP protocols, or an ICMP code.
    #
    #   * `egress.ip-permission.user-id` - The ID of an Amazon Web Services
    #     account that has been referenced in an outbound security group rule.
    #
    #   * `group-id` - The ID of the security group.
    #
    #   * `group-name` - The name of the security group.
    #
    #   * `ip-permission.cidr` - An IPv4 CIDR block for an inbound security
    #     group rule.
    #
    #   * `ip-permission.from-port` - For an inbound rule, the start of port
    #     range for the TCP and UDP protocols, or an ICMP type number.
    #
    #   * `ip-permission.group-id` - The ID of a security group that has been
    #     referenced in an inbound security group rule.
    #
    #   * `ip-permission.group-name` - The name of a security group that is
    #     referenced in an inbound security group rule.
    #
    #   * `ip-permission.ipv6-cidr` - An IPv6 CIDR block for an inbound
    #     security group rule.
    #
    #   * `ip-permission.prefix-list-id` - The ID of a prefix list from which
    #     a security group rule allows inbound access.
    #
    #   * `ip-permission.protocol` - The IP protocol for an inbound security
    #     group rule (`tcp` \| `udp` \| `icmp`, a protocol number, or -1 for
    #     all protocols).
    #
    #   * `ip-permission.to-port` - For an inbound rule, the end of port range
    #     for the TCP and UDP protocols, or an ICMP code.
    #
    #   * `ip-permission.user-id` - The ID of an Amazon Web Services account
    #     that has been referenced in an inbound security group rule.
    #
    #   * `owner-id` - The Amazon Web Services account ID of the owner of the
    #     security group.
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `vpc-id` - The ID of the VPC specified when the security group was
    #     created.
    #
    # @return [Types::DescribeSecurityGroupsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeSecurityGroupsResult#next_token #next_token} => String
    #   * {Types::DescribeSecurityGroupsResult#security_groups #security_groups} => Array&lt;Types::SecurityGroup&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe a security group
    #
    #   # This example describes the specified security group.
    #
    #   resp = client.describe_security_groups({
    #     group_ids: [
    #       "sg-903004f8", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Example: To describe a tagged security group
    #
    #   # This example describes the security groups that include the specified tag (Purpose=test).
    #
    #   resp = client.describe_security_groups({
    #     filters: [
    #       {
    #         name: "tag:Purpose", 
    #         values: [
    #           "test", 
    #         ], 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_security_groups({
    #     group_ids: ["SecurityGroupId"],
    #     group_names: ["SecurityGroupName"],
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.security_groups #=> Array
    #   resp.security_groups[0].group_id #=> String
    #   resp.security_groups[0].ip_permissions_egress #=> Array
    #   resp.security_groups[0].ip_permissions_egress[0].ip_protocol #=> String
    #   resp.security_groups[0].ip_permissions_egress[0].from_port #=> Integer
    #   resp.security_groups[0].ip_permissions_egress[0].to_port #=> Integer
    #   resp.security_groups[0].ip_permissions_egress[0].user_id_group_pairs #=> Array
    #   resp.security_groups[0].ip_permissions_egress[0].user_id_group_pairs[0].description #=> String
    #   resp.security_groups[0].ip_permissions_egress[0].user_id_group_pairs[0].user_id #=> String
    #   resp.security_groups[0].ip_permissions_egress[0].user_id_group_pairs[0].group_name #=> String
    #   resp.security_groups[0].ip_permissions_egress[0].user_id_group_pairs[0].group_id #=> String
    #   resp.security_groups[0].ip_permissions_egress[0].user_id_group_pairs[0].vpc_id #=> String
    #   resp.security_groups[0].ip_permissions_egress[0].user_id_group_pairs[0].vpc_peering_connection_id #=> String
    #   resp.security_groups[0].ip_permissions_egress[0].user_id_group_pairs[0].peering_status #=> String
    #   resp.security_groups[0].ip_permissions_egress[0].ip_ranges #=> Array
    #   resp.security_groups[0].ip_permissions_egress[0].ip_ranges[0].description #=> String
    #   resp.security_groups[0].ip_permissions_egress[0].ip_ranges[0].cidr_ip #=> String
    #   resp.security_groups[0].ip_permissions_egress[0].ipv_6_ranges #=> Array
    #   resp.security_groups[0].ip_permissions_egress[0].ipv_6_ranges[0].description #=> String
    #   resp.security_groups[0].ip_permissions_egress[0].ipv_6_ranges[0].cidr_ipv_6 #=> String
    #   resp.security_groups[0].ip_permissions_egress[0].prefix_list_ids #=> Array
    #   resp.security_groups[0].ip_permissions_egress[0].prefix_list_ids[0].description #=> String
    #   resp.security_groups[0].ip_permissions_egress[0].prefix_list_ids[0].prefix_list_id #=> String
    #   resp.security_groups[0].tags #=> Array
    #   resp.security_groups[0].tags[0].key #=> String
    #   resp.security_groups[0].tags[0].value #=> String
    #   resp.security_groups[0].vpc_id #=> String
    #   resp.security_groups[0].security_group_arn #=> String
    #   resp.security_groups[0].owner_id #=> String
    #   resp.security_groups[0].group_name #=> String
    #   resp.security_groups[0].description #=> String
    #   resp.security_groups[0].ip_permissions #=> Array
    #   resp.security_groups[0].ip_permissions[0].ip_protocol #=> String
    #   resp.security_groups[0].ip_permissions[0].from_port #=> Integer
    #   resp.security_groups[0].ip_permissions[0].to_port #=> Integer
    #   resp.security_groups[0].ip_permissions[0].user_id_group_pairs #=> Array
    #   resp.security_groups[0].ip_permissions[0].user_id_group_pairs[0].description #=> String
    #   resp.security_groups[0].ip_permissions[0].user_id_group_pairs[0].user_id #=> String
    #   resp.security_groups[0].ip_permissions[0].user_id_group_pairs[0].group_name #=> String
    #   resp.security_groups[0].ip_permissions[0].user_id_group_pairs[0].group_id #=> String
    #   resp.security_groups[0].ip_permissions[0].user_id_group_pairs[0].vpc_id #=> String
    #   resp.security_groups[0].ip_permissions[0].user_id_group_pairs[0].vpc_peering_connection_id #=> String
    #   resp.security_groups[0].ip_permissions[0].user_id_group_pairs[0].peering_status #=> String
    #   resp.security_groups[0].ip_permissions[0].ip_ranges #=> Array
    #   resp.security_groups[0].ip_permissions[0].ip_ranges[0].description #=> String
    #   resp.security_groups[0].ip_permissions[0].ip_ranges[0].cidr_ip #=> String
    #   resp.security_groups[0].ip_permissions[0].ipv_6_ranges #=> Array
    #   resp.security_groups[0].ip_permissions[0].ipv_6_ranges[0].description #=> String
    #   resp.security_groups[0].ip_permissions[0].ipv_6_ranges[0].cidr_ipv_6 #=> String
    #   resp.security_groups[0].ip_permissions[0].prefix_list_ids #=> Array
    #   resp.security_groups[0].ip_permissions[0].prefix_list_ids[0].description #=> String
    #   resp.security_groups[0].ip_permissions[0].prefix_list_ids[0].prefix_list_id #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * security_group_exists
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSecurityGroups AWS API Documentation
    #
    # @overload describe_security_groups(params = {})
    # @param [Hash] params ({})
    def describe_security_groups(params = {}, options = {})
      req = build_request(:describe_security_groups, params)
      req.send_request(options)
    end

    # Describes the specified attribute of the specified snapshot. You can
    # specify only one attribute at a time.
    #
    # For more information about EBS snapshots, see [Amazon EBS
    # snapshots][1] in the *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-snapshots.html
    #
    # @option params [required, String] :attribute
    #   The snapshot attribute you would like to view.
    #
    # @option params [required, String] :snapshot_id
    #   The ID of the EBS snapshot.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeSnapshotAttributeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeSnapshotAttributeResult#product_codes #product_codes} => Array&lt;Types::ProductCode&gt;
    #   * {Types::DescribeSnapshotAttributeResult#snapshot_id #snapshot_id} => String
    #   * {Types::DescribeSnapshotAttributeResult#create_volume_permissions #create_volume_permissions} => Array&lt;Types::CreateVolumePermission&gt;
    #
    #
    # @example Example: To describe snapshot attributes
    #
    #   # This example describes the ``createVolumePermission`` attribute on a snapshot with the snapshot ID of
    #   # ``snap-066877671789bd71b``.
    #
    #   resp = client.describe_snapshot_attribute({
    #     attribute: "createVolumePermission", 
    #     snapshot_id: "snap-066877671789bd71b", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     create_volume_permissions: [
    #     ], 
    #     snapshot_id: "snap-066877671789bd71b", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_snapshot_attribute({
    #     attribute: "productCodes", # required, accepts productCodes, createVolumePermission
    #     snapshot_id: "SnapshotId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.product_codes #=> Array
    #   resp.product_codes[0].product_code_id #=> String
    #   resp.product_codes[0].product_code_type #=> String, one of "devpay", "marketplace"
    #   resp.snapshot_id #=> String
    #   resp.create_volume_permissions #=> Array
    #   resp.create_volume_permissions[0].user_id #=> String
    #   resp.create_volume_permissions[0].group #=> String, one of "all"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSnapshotAttribute AWS API Documentation
    #
    # @overload describe_snapshot_attribute(params = {})
    # @param [Hash] params ({})
    def describe_snapshot_attribute(params = {}, options = {})
      req = build_request(:describe_snapshot_attribute, params)
      req.send_request(options)
    end

    # Describes the storage tier status of one or more Amazon EBS snapshots.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `snapshot-id` - The snapshot ID.
    #
    #   * `volume-id` - The ID of the volume the snapshot is for.
    #
    #   * `last-tiering-operation` - The state of the last archive or restore
    #     action. (`archival-in-progress` \| `archival-completed` \|
    #     `archival-failed` \| `permanent-restore-in-progress` \|
    #     `permanent-restore-completed` \| `permanent-restore-failed` \|
    #     `temporary-restore-in-progress` \| `temporary-restore-completed` \|
    #     `temporary-restore-failed`)
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @return [Types::DescribeSnapshotTierStatusResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeSnapshotTierStatusResult#snapshot_tier_statuses #snapshot_tier_statuses} => Array&lt;Types::SnapshotTierStatus&gt;
    #   * {Types::DescribeSnapshotTierStatusResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_snapshot_tier_status({
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.snapshot_tier_statuses #=> Array
    #   resp.snapshot_tier_statuses[0].snapshot_id #=> String
    #   resp.snapshot_tier_statuses[0].volume_id #=> String
    #   resp.snapshot_tier_statuses[0].status #=> String, one of "pending", "completed", "error", "recoverable", "recovering"
    #   resp.snapshot_tier_statuses[0].owner_id #=> String
    #   resp.snapshot_tier_statuses[0].tags #=> Array
    #   resp.snapshot_tier_statuses[0].tags[0].key #=> String
    #   resp.snapshot_tier_statuses[0].tags[0].value #=> String
    #   resp.snapshot_tier_statuses[0].storage_tier #=> String, one of "archive", "standard"
    #   resp.snapshot_tier_statuses[0].last_tiering_start_time #=> Time
    #   resp.snapshot_tier_statuses[0].last_tiering_progress #=> Integer
    #   resp.snapshot_tier_statuses[0].last_tiering_operation_status #=> String, one of "archival-in-progress", "archival-completed", "archival-failed", "temporary-restore-in-progress", "temporary-restore-completed", "temporary-restore-failed", "permanent-restore-in-progress", "permanent-restore-completed", "permanent-restore-failed"
    #   resp.snapshot_tier_statuses[0].last_tiering_operation_status_detail #=> String
    #   resp.snapshot_tier_statuses[0].archival_complete_time #=> Time
    #   resp.snapshot_tier_statuses[0].restore_expiry_time #=> Time
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSnapshotTierStatus AWS API Documentation
    #
    # @overload describe_snapshot_tier_status(params = {})
    # @param [Hash] params ({})
    def describe_snapshot_tier_status(params = {}, options = {})
      req = build_request(:describe_snapshot_tier_status, params)
      req.send_request(options)
    end

    # Describes the specified EBS snapshots available to you or all of the
    # EBS snapshots available to you.
    #
    # The snapshots available to you include public snapshots, private
    # snapshots that you own, and private snapshots owned by other Amazon
    # Web Services accounts for which you have explicit create volume
    # permissions.
    #
    # The create volume permissions fall into the following categories:
    #
    # * *public*: The owner of the snapshot granted create volume
    #   permissions for the snapshot to the `all` group. All Amazon Web
    #   Services accounts have create volume permissions for these
    #   snapshots.
    #
    # * *explicit*: The owner of the snapshot granted create volume
    #   permissions to a specific Amazon Web Services account.
    #
    # * *implicit*: An Amazon Web Services account has implicit create
    #   volume permissions for all snapshots it owns.
    #
    # The list of snapshots returned can be filtered by specifying snapshot
    # IDs, snapshot owners, or Amazon Web Services accounts with create
    # volume permissions. If no options are specified, Amazon EC2 returns
    # all snapshots for which you have create volume permissions.
    #
    # If you specify one or more snapshot IDs, only snapshots that have the
    # specified IDs are returned. If you specify an invalid snapshot ID, an
    # error is returned. If you specify a snapshot ID for which you do not
    # have access, it is not included in the returned results.
    #
    # If you specify one or more snapshot owners using the `OwnerIds`
    # option, only snapshots from the specified owners and for which you
    # have access are returned. The results can include the Amazon Web
    # Services account IDs of the specified owners, `amazon` for snapshots
    # owned by Amazon, or `self` for snapshots that you own.
    #
    # If you specify a list of restorable users, only snapshots with create
    # snapshot permissions for those users are returned. You can specify
    # Amazon Web Services account IDs (if you own the snapshots), `self` for
    # snapshots for which you own or have explicit permissions, or `all` for
    # public snapshots.
    #
    # If you are describing a long list of snapshots, we recommend that you
    # paginate the output to make the list more manageable. For more
    # information, see [Pagination][1].
    #
    # To get the state of fast snapshot restores for a snapshot, use
    # DescribeFastSnapshotRestores.
    #
    # For more information about EBS snapshots, see [Amazon EBS
    # snapshots][2] in the *Amazon EBS User Guide*.
    #
    # We strongly recommend using only paginated requests. Unpaginated
    # requests are susceptible to throttling and timeouts.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    # [2]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-snapshots.html
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Array<String>] :owner_ids
    #   Scopes the results to snapshots with the specified owners. You can
    #   specify a combination of Amazon Web Services account IDs, `self`, and
    #   `amazon`.
    #
    # @option params [Array<String>] :restorable_by_user_ids
    #   The IDs of the Amazon Web Services accounts that can create volumes
    #   from the snapshot.
    #
    # @option params [Array<String>] :snapshot_ids
    #   The snapshot IDs.
    #
    #   Default: Describes the snapshots for which you have create volume
    #   permissions.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `description` - A description of the snapshot.
    #
    #   * `encrypted` - Indicates whether the snapshot is encrypted (`true` \|
    #     `false`)
    #
    #   * `owner-alias` - The owner alias, from an Amazon-maintained list
    #     (`amazon`). This is not the user-configured Amazon Web Services
    #     account alias set using the IAM console. We recommend that you use
    #     the related parameter instead of this filter.
    #
    #   * `owner-id` - The Amazon Web Services account ID of the owner. We
    #     recommend that you use the related parameter instead of this filter.
    #
    #   * `progress` - The progress of the snapshot, as a percentage (for
    #     example, 80%).
    #
    #   * `snapshot-id` - The snapshot ID.
    #
    #   * `start-time` - The time stamp when the snapshot was initiated.
    #
    #   * `status` - The status of the snapshot (`pending` \| `completed` \|
    #     `error`).
    #
    #   * `storage-tier` - The storage tier of the snapshot (`archive` \|
    #     `standard`).
    #
    #   * `transfer-type` - The type of operation used to create the snapshot
    #     (`time-based` \| `standard`).
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `volume-id` - The ID of the volume the snapshot is for.
    #
    #   * `volume-size` - The size of the volume, in GiB.
    #
    # @return [Types::DescribeSnapshotsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeSnapshotsResult#next_token #next_token} => String
    #   * {Types::DescribeSnapshotsResult#snapshots #snapshots} => Array&lt;Types::Snapshot&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe a snapshot
    #
    #   # This example describes a snapshot with the snapshot ID of ``snap-1234567890abcdef0``.
    #
    #   resp = client.describe_snapshots({
    #     snapshot_ids: [
    #       "snap-1234567890abcdef0", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     next_token: "", 
    #     snapshots: [
    #       {
    #         description: "This is my snapshot.", 
    #         owner_id: "012345678910", 
    #         progress: "100%", 
    #         snapshot_id: "snap-1234567890abcdef0", 
    #         start_time: Time.parse("2014-02-28T21:28:32.000Z"), 
    #         state: "completed", 
    #         volume_id: "vol-049df61146c4d7901", 
    #         volume_size: 8, 
    #       }, 
    #     ], 
    #   }
    #
    # @example Example: To describe snapshots using filters
    #
    #   # This example describes all snapshots owned by the ID 012345678910 that are in the ``pending`` status.
    #
    #   resp = client.describe_snapshots({
    #     filters: [
    #       {
    #         name: "status", 
    #         values: [
    #           "pending", 
    #         ], 
    #       }, 
    #     ], 
    #     owner_ids: [
    #       "012345678910", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     next_token: "", 
    #     snapshots: [
    #       {
    #         description: "This is my copied snapshot.", 
    #         owner_id: "012345678910", 
    #         progress: "87%", 
    #         snapshot_id: "snap-066877671789bd71b", 
    #         start_time: Time.parse("2014-02-28T21:37:27.000Z"), 
    #         state: "pending", 
    #         volume_id: "vol-1234567890abcdef0", 
    #         volume_size: 8, 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_snapshots({
    #     max_results: 1,
    #     next_token: "String",
    #     owner_ids: ["String"],
    #     restorable_by_user_ids: ["String"],
    #     snapshot_ids: ["SnapshotId"],
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.snapshots #=> Array
    #   resp.snapshots[0].owner_alias #=> String
    #   resp.snapshots[0].outpost_arn #=> String
    #   resp.snapshots[0].tags #=> Array
    #   resp.snapshots[0].tags[0].key #=> String
    #   resp.snapshots[0].tags[0].value #=> String
    #   resp.snapshots[0].storage_tier #=> String, one of "archive", "standard"
    #   resp.snapshots[0].restore_expiry_time #=> Time
    #   resp.snapshots[0].sse_type #=> String, one of "sse-ebs", "sse-kms", "none"
    #   resp.snapshots[0].availability_zone #=> String
    #   resp.snapshots[0].transfer_type #=> String, one of "time-based", "standard"
    #   resp.snapshots[0].completion_duration_minutes #=> Integer
    #   resp.snapshots[0].completion_time #=> Time
    #   resp.snapshots[0].full_snapshot_size_in_bytes #=> Integer
    #   resp.snapshots[0].snapshot_id #=> String
    #   resp.snapshots[0].volume_id #=> String
    #   resp.snapshots[0].state #=> String, one of "pending", "completed", "error", "recoverable", "recovering"
    #   resp.snapshots[0].state_message #=> String
    #   resp.snapshots[0].start_time #=> Time
    #   resp.snapshots[0].progress #=> String
    #   resp.snapshots[0].owner_id #=> String
    #   resp.snapshots[0].description #=> String
    #   resp.snapshots[0].volume_size #=> Integer
    #   resp.snapshots[0].encrypted #=> Boolean
    #   resp.snapshots[0].kms_key_id #=> String
    #   resp.snapshots[0].data_encryption_key_id #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * snapshot_completed
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSnapshots AWS API Documentation
    #
    # @overload describe_snapshots(params = {})
    # @param [Hash] params ({})
    def describe_snapshots(params = {}, options = {})
      req = build_request(:describe_snapshots, params)
      req.send_request(options)
    end

    # Describes the data feed for Spot Instances. For more information, see
    # [Spot Instance data feed][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-data-feeds.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeSpotDatafeedSubscriptionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeSpotDatafeedSubscriptionResult#spot_datafeed_subscription #spot_datafeed_subscription} => Types::SpotDatafeedSubscription
    #
    #
    # @example Example: To describe the datafeed for your AWS account
    #
    #   # This example describes the Spot Instance datafeed subscription for your AWS account.
    #
    #   resp = client.describe_spot_datafeed_subscription({
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     spot_datafeed_subscription: {
    #       bucket: "my-s3-bucket", 
    #       owner_id: "123456789012", 
    #       prefix: "spotdata", 
    #       state: "Active", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_spot_datafeed_subscription({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.spot_datafeed_subscription.bucket #=> String
    #   resp.spot_datafeed_subscription.fault.code #=> String
    #   resp.spot_datafeed_subscription.fault.message #=> String
    #   resp.spot_datafeed_subscription.owner_id #=> String
    #   resp.spot_datafeed_subscription.prefix #=> String
    #   resp.spot_datafeed_subscription.state #=> String, one of "Active", "Inactive"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSpotDatafeedSubscription AWS API Documentation
    #
    # @overload describe_spot_datafeed_subscription(params = {})
    # @param [Hash] params ({})
    def describe_spot_datafeed_subscription(params = {}, options = {})
      req = build_request(:describe_spot_datafeed_subscription, params)
      req.send_request(options)
    end

    # Describes the running instances for the specified Spot Fleet.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :spot_fleet_request_id
    #   The ID of the Spot Fleet request.
    #
    # @option params [String] :next_token
    #   The token to include in another request to get the next page of items.
    #   This value is `null` when there are no more items to return.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @return [Types::DescribeSpotFleetInstancesResponse] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeSpotFleetInstancesResponse#active_instances #active_instances} => Array&lt;Types::ActiveInstance&gt;
    #   * {Types::DescribeSpotFleetInstancesResponse#next_token #next_token} => String
    #   * {Types::DescribeSpotFleetInstancesResponse#spot_fleet_request_id #spot_fleet_request_id} => String
    #
    #
    # @example Example: To describe the Spot Instances associated with a Spot fleet
    #
    #   # This example lists the Spot Instances associated with the specified Spot fleet.
    #
    #   resp = client.describe_spot_fleet_instances({
    #     spot_fleet_request_id: "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     active_instances: [
    #       {
    #         instance_id: "i-1234567890abcdef0", 
    #         instance_type: "m3.medium", 
    #         spot_instance_request_id: "sir-08b93456", 
    #       }, 
    #     ], 
    #     spot_fleet_request_id: "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_spot_fleet_instances({
    #     dry_run: false,
    #     spot_fleet_request_id: "SpotFleetRequestId", # required
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.active_instances #=> Array
    #   resp.active_instances[0].instance_id #=> String
    #   resp.active_instances[0].instance_type #=> String
    #   resp.active_instances[0].spot_instance_request_id #=> String
    #   resp.active_instances[0].instance_health #=> String, one of "healthy", "unhealthy"
    #   resp.next_token #=> String
    #   resp.spot_fleet_request_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSpotFleetInstances AWS API Documentation
    #
    # @overload describe_spot_fleet_instances(params = {})
    # @param [Hash] params ({})
    def describe_spot_fleet_instances(params = {}, options = {})
      req = build_request(:describe_spot_fleet_instances, params)
      req.send_request(options)
    end

    # Describes the events for the specified Spot Fleet request during the
    # specified time.
    #
    # Spot Fleet events are delayed by up to 30 seconds before they can be
    # described. This ensures that you can query by the last evaluated time
    # and not miss a recorded event. Spot Fleet events are available for 48
    # hours.
    #
    # For more information, see [Monitor fleet events using Amazon
    # EventBridge][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/fleet-monitor.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :spot_fleet_request_id
    #   The ID of the Spot Fleet request.
    #
    # @option params [String] :event_type
    #   The type of events to describe. By default, all events are described.
    #
    # @option params [required, Time,DateTime,Date,Integer,String] :start_time
    #   The starting date and time for the events, in UTC format (for example,
    #   *YYYY*-*MM*-*DD*T*HH*:*MM*:*SS*Z).
    #
    # @option params [String] :next_token
    #   The token to include in another request to get the next page of items.
    #   This value is `null` when there are no more items to return.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @return [Types::DescribeSpotFleetRequestHistoryResponse] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeSpotFleetRequestHistoryResponse#history_records #history_records} => Array&lt;Types::HistoryRecord&gt;
    #   * {Types::DescribeSpotFleetRequestHistoryResponse#last_evaluated_time #last_evaluated_time} => Time
    #   * {Types::DescribeSpotFleetRequestHistoryResponse#next_token #next_token} => String
    #   * {Types::DescribeSpotFleetRequestHistoryResponse#spot_fleet_request_id #spot_fleet_request_id} => String
    #   * {Types::DescribeSpotFleetRequestHistoryResponse#start_time #start_time} => Time
    #
    #
    # @example Example: To describe Spot fleet history
    #
    #   # This example returns the history for the specified Spot fleet starting at the specified time.
    #
    #   resp = client.describe_spot_fleet_request_history({
    #     spot_fleet_request_id: "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #     start_time: Time.parse("2015-05-26T00:00:00Z"), 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     history_records: [
    #       {
    #         event_information: {
    #           event_sub_type: "submitted", 
    #         }, 
    #         event_type: "fleetRequestChange", 
    #         timestamp: Time.parse("2015-05-26T23:17:20.697Z"), 
    #       }, 
    #       {
    #         event_information: {
    #           event_sub_type: "active", 
    #         }, 
    #         event_type: "fleetRequestChange", 
    #         timestamp: Time.parse("2015-05-26T23:17:20.873Z"), 
    #       }, 
    #       {
    #         event_information: {
    #           event_sub_type: "launched", 
    #           instance_id: "i-1234567890abcdef0", 
    #         }, 
    #         event_type: "instanceChange", 
    #         timestamp: Time.parse("2015-05-26T23:21:21.712Z"), 
    #       }, 
    #       {
    #         event_information: {
    #           event_sub_type: "launched", 
    #           instance_id: "i-1234567890abcdef1", 
    #         }, 
    #         event_type: "instanceChange", 
    #         timestamp: Time.parse("2015-05-26T23:21:21.816Z"), 
    #       }, 
    #     ], 
    #     next_token: "CpHNsscimcV5oH7bSbub03CI2Qms5+ypNpNm+53MNlR0YcXAkp0xFlfKf91yVxSExmbtma3awYxMFzNA663ZskT0AHtJ6TCb2Z8bQC2EnZgyELbymtWPfpZ1ZbauVg+P+TfGlWxWWB/Vr5dk5d4LfdgA/DRAHUrYgxzrEXAMPLE=", 
    #     spot_fleet_request_id: "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #     start_time: Time.parse("2015-05-26T00:00:00Z"), 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_spot_fleet_request_history({
    #     dry_run: false,
    #     spot_fleet_request_id: "SpotFleetRequestId", # required
    #     event_type: "instanceChange", # accepts instanceChange, fleetRequestChange, error, information
    #     start_time: Time.now, # required
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.history_records #=> Array
    #   resp.history_records[0].event_information.event_description #=> String
    #   resp.history_records[0].event_information.event_sub_type #=> String
    #   resp.history_records[0].event_information.instance_id #=> String
    #   resp.history_records[0].event_type #=> String, one of "instanceChange", "fleetRequestChange", "error", "information"
    #   resp.history_records[0].timestamp #=> Time
    #   resp.last_evaluated_time #=> Time
    #   resp.next_token #=> String
    #   resp.spot_fleet_request_id #=> String
    #   resp.start_time #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSpotFleetRequestHistory AWS API Documentation
    #
    # @overload describe_spot_fleet_request_history(params = {})
    # @param [Hash] params ({})
    def describe_spot_fleet_request_history(params = {}, options = {})
      req = build_request(:describe_spot_fleet_request_history, params)
      req.send_request(options)
    end

    # Describes your Spot Fleet requests.
    #
    # Spot Fleet requests are deleted 48 hours after they are canceled and
    # their instances are terminated.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :spot_fleet_request_ids
    #   The IDs of the Spot Fleet requests.
    #
    # @option params [String] :next_token
    #   The token to include in another request to get the next page of items.
    #   This value is `null` when there are no more items to return.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @return [Types::DescribeSpotFleetRequestsResponse] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeSpotFleetRequestsResponse#next_token #next_token} => String
    #   * {Types::DescribeSpotFleetRequestsResponse#spot_fleet_request_configs #spot_fleet_request_configs} => Array&lt;Types::SpotFleetRequestConfig&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe a Spot fleet request
    #
    #   # This example describes the specified Spot fleet request.
    #
    #   resp = client.describe_spot_fleet_requests({
    #     spot_fleet_request_ids: [
    #       "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     spot_fleet_request_configs: [
    #       {
    #         spot_fleet_request_config: {
    #           iam_fleet_role: "arn:aws:iam::123456789012:role/my-spot-fleet-role", 
    #           launch_specifications: [
    #             {
    #               ebs_optimized: false, 
    #               image_id: "ami-1a2b3c4d", 
    #               instance_type: "cc2.8xlarge", 
    #               network_interfaces: [
    #                 {
    #                   associate_public_ip_address: true, 
    #                   delete_on_termination: false, 
    #                   device_index: 0, 
    #                   secondary_private_ip_address_count: 0, 
    #                   subnet_id: "subnet-a61dafcf", 
    #                 }, 
    #               ], 
    #             }, 
    #             {
    #               ebs_optimized: false, 
    #               image_id: "ami-1a2b3c4d", 
    #               instance_type: "r3.8xlarge", 
    #               network_interfaces: [
    #                 {
    #                   associate_public_ip_address: true, 
    #                   delete_on_termination: false, 
    #                   device_index: 0, 
    #                   secondary_private_ip_address_count: 0, 
    #                   subnet_id: "subnet-a61dafcf", 
    #                 }, 
    #               ], 
    #             }, 
    #           ], 
    #           spot_price: "0.05", 
    #           target_capacity: 20, 
    #         }, 
    #         spot_fleet_request_id: "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #         spot_fleet_request_state: "active", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_spot_fleet_requests({
    #     dry_run: false,
    #     spot_fleet_request_ids: ["SpotFleetRequestId"],
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.spot_fleet_request_configs #=> Array
    #   resp.spot_fleet_request_configs[0].activity_status #=> String, one of "error", "pending_fulfillment", "pending_termination", "fulfilled"
    #   resp.spot_fleet_request_configs[0].create_time #=> Time
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.allocation_strategy #=> String, one of "lowestPrice", "diversified", "capacityOptimized", "capacityOptimizedPrioritized", "priceCapacityOptimized"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.on_demand_allocation_strategy #=> String, one of "lowestPrice", "prioritized"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.spot_maintenance_strategies.capacity_rebalance.replacement_strategy #=> String, one of "launch", "launch-before-terminate"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.spot_maintenance_strategies.capacity_rebalance.termination_delay #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.client_token #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.excess_capacity_termination_policy #=> String, one of "noTermination", "default"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.fulfilled_capacity #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.on_demand_fulfilled_capacity #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.iam_fleet_role #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].addressing_type #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].block_device_mappings #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].block_device_mappings[0].ebs.iops #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].block_device_mappings[0].ebs.snapshot_id #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].block_device_mappings[0].ebs.volume_size #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].block_device_mappings[0].ebs.volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].block_device_mappings[0].ebs.kms_key_id #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].block_device_mappings[0].ebs.throughput #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].block_device_mappings[0].ebs.outpost_arn #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].block_device_mappings[0].ebs.encrypted #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].block_device_mappings[0].no_device #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].block_device_mappings[0].device_name #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].block_device_mappings[0].virtual_name #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].ebs_optimized #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].iam_instance_profile.arn #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].iam_instance_profile.name #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].image_id #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].kernel_id #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].key_name #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].monitoring.enabled #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].associate_public_ip_address #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].delete_on_termination #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].description #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].device_index #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].groups #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].groups[0] #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].ipv_6_address_count #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].ipv_6_addresses #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].ipv_6_addresses[0].ipv_6_address #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].ipv_6_addresses[0].is_primary_ipv_6 #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].network_interface_id #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].private_ip_address #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].private_ip_addresses #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].private_ip_addresses[0].primary #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].private_ip_addresses[0].private_ip_address #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].secondary_private_ip_address_count #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].subnet_id #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].associate_carrier_ip_address #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].interface_type #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].network_card_index #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].ipv_4_prefixes #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].ipv_4_prefixes[0].ipv_4_prefix #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].ipv_4_prefix_count #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].ipv_6_prefixes #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].ipv_6_prefixes[0].ipv_6_prefix #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].ipv_6_prefix_count #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].primary_ipv_6 #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].ena_srd_specification.ena_srd_enabled #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].ena_srd_specification.ena_srd_udp_specification.ena_srd_udp_enabled #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].connection_tracking_specification.tcp_established_timeout #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].connection_tracking_specification.udp_stream_timeout #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].network_interfaces[0].connection_tracking_specification.udp_timeout #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].placement.availability_zone #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].placement.group_name #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].placement.tenancy #=> String, one of "default", "dedicated", "host"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].ramdisk_id #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].spot_price #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].subnet_id #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].user_data #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].weighted_capacity #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].tag_specifications #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].tag_specifications[0].resource_type #=> String, one of "capacity-reservation", "client-vpn-endpoint", "customer-gateway", "carrier-gateway", "coip-pool", "declarative-policies-report", "dedicated-host", "dhcp-options", "egress-only-internet-gateway", "elastic-ip", "elastic-gpu", "export-image-task", "export-instance-task", "fleet", "fpga-image", "host-reservation", "image", "import-image-task", "import-snapshot-task", "instance", "instance-event-window", "internet-gateway", "ipam", "ipam-pool", "ipam-scope", "ipv4pool-ec2", "ipv6pool-ec2", "key-pair", "launch-template", "local-gateway", "local-gateway-route-table", "local-gateway-virtual-interface", "local-gateway-virtual-interface-group", "local-gateway-route-table-vpc-association", "local-gateway-route-table-virtual-interface-group-association", "natgateway", "network-acl", "network-interface", "network-insights-analysis", "network-insights-path", "network-insights-access-scope", "network-insights-access-scope-analysis", "placement-group", "prefix-list", "replace-root-volume-task", "reserved-instances", "route-table", "security-group", "security-group-rule", "snapshot", "spot-fleet-request", "spot-instances-request", "subnet", "subnet-cidr-reservation", "traffic-mirror-filter", "traffic-mirror-session", "traffic-mirror-target", "transit-gateway", "transit-gateway-attachment", "transit-gateway-connect-peer", "transit-gateway-multicast-domain", "transit-gateway-policy-table", "transit-gateway-route-table", "transit-gateway-route-table-announcement", "volume", "vpc", "vpc-endpoint", "vpc-endpoint-connection", "vpc-endpoint-service", "vpc-endpoint-service-permission", "vpc-peering-connection", "vpn-connection", "vpn-gateway", "vpc-flow-log", "capacity-reservation-fleet", "traffic-mirror-filter-rule", "vpc-endpoint-connection-device-type", "verified-access-instance", "verified-access-group", "verified-access-endpoint", "verified-access-policy", "verified-access-trust-provider", "vpn-connection-device-type", "vpc-block-public-access-exclusion", "ipam-resource-discovery", "ipam-resource-discovery-association", "instance-connect-endpoint", "verified-access-endpoint-target", "ipam-external-resource-verification-token"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].tag_specifications[0].tags #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].tag_specifications[0].tags[0].key #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].tag_specifications[0].tags[0].value #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.v_cpu_count.min #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.v_cpu_count.max #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.memory_mi_b.min #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.memory_mi_b.max #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.cpu_manufacturers #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.cpu_manufacturers[0] #=> String, one of "intel", "amd", "amazon-web-services", "apple"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.memory_gi_b_per_v_cpu.min #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.memory_gi_b_per_v_cpu.max #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.excluded_instance_types #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.excluded_instance_types[0] #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.instance_generations #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.instance_generations[0] #=> String, one of "current", "previous"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.spot_max_price_percentage_over_lowest_price #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.on_demand_max_price_percentage_over_lowest_price #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.bare_metal #=> String, one of "included", "required", "excluded"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.burstable_performance #=> String, one of "included", "required", "excluded"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.require_hibernate_support #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.network_interface_count.min #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.network_interface_count.max #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.local_storage #=> String, one of "included", "required", "excluded"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.local_storage_types #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.local_storage_types[0] #=> String, one of "hdd", "ssd"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.total_local_storage_gb.min #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.total_local_storage_gb.max #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.baseline_ebs_bandwidth_mbps.min #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.baseline_ebs_bandwidth_mbps.max #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.accelerator_types #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.accelerator_types[0] #=> String, one of "gpu", "fpga", "inference"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.accelerator_count.min #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.accelerator_count.max #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.accelerator_manufacturers #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.accelerator_manufacturers[0] #=> String, one of "amazon-web-services", "amd", "nvidia", "xilinx", "habana"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.accelerator_names #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.accelerator_names[0] #=> String, one of "a100", "inferentia", "k520", "k80", "m60", "radeon-pro-v520", "t4", "vu9p", "v100", "a10g", "h100", "t4g"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.accelerator_total_memory_mi_b.min #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.accelerator_total_memory_mi_b.max #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.network_bandwidth_gbps.min #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.network_bandwidth_gbps.max #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.allowed_instance_types #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.allowed_instance_types[0] #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.max_spot_price_as_percentage_of_optimal_on_demand_price #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.baseline_performance_factors.cpu.references #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].instance_requirements.baseline_performance_factors.cpu.references[0].instance_family #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].security_groups #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].security_groups[0].group_id #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_specifications[0].security_groups[0].group_name #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].launch_template_specification.launch_template_id #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].launch_template_specification.launch_template_name #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].launch_template_specification.version #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].spot_price #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].subnet_id #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].availability_zone #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].weighted_capacity #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].priority #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.v_cpu_count.min #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.v_cpu_count.max #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.memory_mi_b.min #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.memory_mi_b.max #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.cpu_manufacturers #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.cpu_manufacturers[0] #=> String, one of "intel", "amd", "amazon-web-services", "apple"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.memory_gi_b_per_v_cpu.min #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.memory_gi_b_per_v_cpu.max #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.excluded_instance_types #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.excluded_instance_types[0] #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.instance_generations #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.instance_generations[0] #=> String, one of "current", "previous"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.spot_max_price_percentage_over_lowest_price #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.on_demand_max_price_percentage_over_lowest_price #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.bare_metal #=> String, one of "included", "required", "excluded"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.burstable_performance #=> String, one of "included", "required", "excluded"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.require_hibernate_support #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.network_interface_count.min #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.network_interface_count.max #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.local_storage #=> String, one of "included", "required", "excluded"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.local_storage_types #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.local_storage_types[0] #=> String, one of "hdd", "ssd"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.total_local_storage_gb.min #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.total_local_storage_gb.max #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.baseline_ebs_bandwidth_mbps.min #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.baseline_ebs_bandwidth_mbps.max #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.accelerator_types #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.accelerator_types[0] #=> String, one of "gpu", "fpga", "inference"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.accelerator_count.min #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.accelerator_count.max #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.accelerator_manufacturers #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.accelerator_manufacturers[0] #=> String, one of "amazon-web-services", "amd", "nvidia", "xilinx", "habana"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.accelerator_names #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.accelerator_names[0] #=> String, one of "a100", "inferentia", "k520", "k80", "m60", "radeon-pro-v520", "t4", "vu9p", "v100", "a10g", "h100", "t4g"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.accelerator_total_memory_mi_b.min #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.accelerator_total_memory_mi_b.max #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.network_bandwidth_gbps.min #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.network_bandwidth_gbps.max #=> Float
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.allowed_instance_types #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.allowed_instance_types[0] #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.max_spot_price_as_percentage_of_optimal_on_demand_price #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.baseline_performance_factors.cpu.references #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.launch_template_configs[0].overrides[0].instance_requirements.baseline_performance_factors.cpu.references[0].instance_family #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.spot_price #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.target_capacity #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.on_demand_target_capacity #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.on_demand_max_total_price #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.spot_max_total_price #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.terminate_instances_with_expiration #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.type #=> String, one of "request", "maintain", "instant"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.valid_from #=> Time
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.valid_until #=> Time
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.replace_unhealthy_instances #=> Boolean
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.instance_interruption_behavior #=> String, one of "hibernate", "stop", "terminate"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.load_balancers_config.classic_load_balancers_config.classic_load_balancers #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.load_balancers_config.classic_load_balancers_config.classic_load_balancers[0].name #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.load_balancers_config.target_groups_config.target_groups #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.load_balancers_config.target_groups_config.target_groups[0].arn #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.instance_pools_to_use_count #=> Integer
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.context #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.target_capacity_unit_type #=> String, one of "vcpu", "memory-mib", "units"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.tag_specifications #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.tag_specifications[0].resource_type #=> String, one of "capacity-reservation", "client-vpn-endpoint", "customer-gateway", "carrier-gateway", "coip-pool", "declarative-policies-report", "dedicated-host", "dhcp-options", "egress-only-internet-gateway", "elastic-ip", "elastic-gpu", "export-image-task", "export-instance-task", "fleet", "fpga-image", "host-reservation", "image", "import-image-task", "import-snapshot-task", "instance", "instance-event-window", "internet-gateway", "ipam", "ipam-pool", "ipam-scope", "ipv4pool-ec2", "ipv6pool-ec2", "key-pair", "launch-template", "local-gateway", "local-gateway-route-table", "local-gateway-virtual-interface", "local-gateway-virtual-interface-group", "local-gateway-route-table-vpc-association", "local-gateway-route-table-virtual-interface-group-association", "natgateway", "network-acl", "network-interface", "network-insights-analysis", "network-insights-path", "network-insights-access-scope", "network-insights-access-scope-analysis", "placement-group", "prefix-list", "replace-root-volume-task", "reserved-instances", "route-table", "security-group", "security-group-rule", "snapshot", "spot-fleet-request", "spot-instances-request", "subnet", "subnet-cidr-reservation", "traffic-mirror-filter", "traffic-mirror-session", "traffic-mirror-target", "transit-gateway", "transit-gateway-attachment", "transit-gateway-connect-peer", "transit-gateway-multicast-domain", "transit-gateway-policy-table", "transit-gateway-route-table", "transit-gateway-route-table-announcement", "volume", "vpc", "vpc-endpoint", "vpc-endpoint-connection", "vpc-endpoint-service", "vpc-endpoint-service-permission", "vpc-peering-connection", "vpn-connection", "vpn-gateway", "vpc-flow-log", "capacity-reservation-fleet", "traffic-mirror-filter-rule", "vpc-endpoint-connection-device-type", "verified-access-instance", "verified-access-group", "verified-access-endpoint", "verified-access-policy", "verified-access-trust-provider", "vpn-connection-device-type", "vpc-block-public-access-exclusion", "ipam-resource-discovery", "ipam-resource-discovery-association", "instance-connect-endpoint", "verified-access-endpoint-target", "ipam-external-resource-verification-token"
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.tag_specifications[0].tags #=> Array
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.tag_specifications[0].tags[0].key #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_config.tag_specifications[0].tags[0].value #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_id #=> String
    #   resp.spot_fleet_request_configs[0].spot_fleet_request_state #=> String, one of "submitted", "active", "cancelled", "failed", "cancelled_running", "cancelled_terminating", "modifying"
    #   resp.spot_fleet_request_configs[0].tags #=> Array
    #   resp.spot_fleet_request_configs[0].tags[0].key #=> String
    #   resp.spot_fleet_request_configs[0].tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSpotFleetRequests AWS API Documentation
    #
    # @overload describe_spot_fleet_requests(params = {})
    # @param [Hash] params ({})
    def describe_spot_fleet_requests(params = {}, options = {})
      req = build_request(:describe_spot_fleet_requests, params)
      req.send_request(options)
    end

    # Describes the specified Spot Instance requests.
    #
    # You can use `DescribeSpotInstanceRequests` to find a running Spot
    # Instance by examining the response. If the status of the Spot Instance
    # is `fulfilled`, the instance ID appears in the response and contains
    # the identifier of the instance. Alternatively, you can use
    # [DescribeInstances][1] with a filter to look for instances where the
    # instance lifecycle is `spot`.
    #
    # We recommend that you set `MaxResults` to a value between 5 and 1000
    # to limit the number of items returned. This paginates the output,
    # which makes the list more manageable and returns the items faster. If
    # the list of items exceeds your `MaxResults` value, then that number of
    # items is returned along with a `NextToken` value that can be passed to
    # a subsequent `DescribeSpotInstanceRequests` request to retrieve the
    # remaining items.
    #
    # Spot Instance requests are deleted four hours after they are canceled
    # and their instances are terminated.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeInstances
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :spot_instance_request_ids
    #   The IDs of the Spot Instance requests.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `availability-zone-group` - The Availability Zone group.
    #
    #   * `create-time` - The time stamp when the Spot Instance request was
    #     created.
    #
    #   * `fault-code` - The fault code related to the request.
    #
    #   * `fault-message` - The fault message related to the request.
    #
    #   * `instance-id` - The ID of the instance that fulfilled the request.
    #
    #   * `launch-group` - The Spot Instance launch group.
    #
    #   * `launch.block-device-mapping.delete-on-termination` - Indicates
    #     whether the EBS volume is deleted on instance termination.
    #
    #   * `launch.block-device-mapping.device-name` - The device name for the
    #     volume in the block device mapping (for example, `/dev/sdh` or
    #     `xvdh`).
    #
    #   * `launch.block-device-mapping.snapshot-id` - The ID of the snapshot
    #     for the EBS volume.
    #
    #   * `launch.block-device-mapping.volume-size` - The size of the EBS
    #     volume, in GiB.
    #
    #   * `launch.block-device-mapping.volume-type` - The type of EBS volume:
    #     `gp2` or `gp3` for General Purpose SSD, `io1` or `io2` for
    #     Provisioned IOPS SSD, `st1` for Throughput Optimized HDD, `sc1` for
    #     Cold HDD, or `standard` for Magnetic.
    #
    #   * `launch.group-id` - The ID of the security group for the instance.
    #
    #   * `launch.group-name` - The name of the security group for the
    #     instance.
    #
    #   * `launch.image-id` - The ID of the AMI.
    #
    #   * `launch.instance-type` - The type of instance (for example,
    #     `m3.medium`).
    #
    #   * `launch.kernel-id` - The kernel ID.
    #
    #   * `launch.key-name` - The name of the key pair the instance launched
    #     with.
    #
    #   * `launch.monitoring-enabled` - Whether detailed monitoring is enabled
    #     for the Spot Instance.
    #
    #   * `launch.ramdisk-id` - The RAM disk ID.
    #
    #   * `launched-availability-zone` - The Availability Zone in which the
    #     request is launched.
    #
    #   * `network-interface.addresses.primary` - Indicates whether the IP
    #     address is the primary private IP address.
    #
    #   * `network-interface.delete-on-termination` - Indicates whether the
    #     network interface is deleted when the instance is terminated.
    #
    #   * `network-interface.description` - A description of the network
    #     interface.
    #
    #   * `network-interface.device-index` - The index of the device for the
    #     network interface attachment on the instance.
    #
    #   * `network-interface.group-id` - The ID of the security group
    #     associated with the network interface.
    #
    #   * `network-interface.network-interface-id` - The ID of the network
    #     interface.
    #
    #   * `network-interface.private-ip-address` - The primary private IP
    #     address of the network interface.
    #
    #   * `network-interface.subnet-id` - The ID of the subnet for the
    #     instance.
    #
    #   * `product-description` - The product description associated with the
    #     instance (`Linux/UNIX` \| `Windows`).
    #
    #   * `spot-instance-request-id` - The Spot Instance request ID.
    #
    #   * `spot-price` - The maximum hourly price for any Spot Instance
    #     launched to fulfill the request.
    #
    #   * `state` - The state of the Spot Instance request (`open` \| `active`
    #     \| `closed` \| `cancelled` \| `failed`). Spot request status
    #     information can help you track your Amazon EC2 Spot Instance
    #     requests. For more information, see [Spot request status][1] in the
    #     *Amazon EC2 User Guide*.
    #
    #   * `status-code` - The short code describing the most recent evaluation
    #     of your Spot Instance request.
    #
    #   * `status-message` - The message explaining the status of the Spot
    #     Instance request.
    #
    #   * `tag:<key>` - The key/value combination of a tag assigned to the
    #     resource. Use the tag key in the filter name and the tag value as
    #     the filter value. For example, to find all resources that have a tag
    #     with the key `Owner` and the value `TeamA`, specify `tag:Owner` for
    #     the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `type` - The type of Spot Instance request (`one-time` \|
    #     `persistent`).
    #
    #   * `valid-from` - The start date of the request.
    #
    #   * `valid-until` - The end date of the request.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-request-status.html
    #
    # @return [Types::DescribeSpotInstanceRequestsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeSpotInstanceRequestsResult#spot_instance_requests #spot_instance_requests} => Array&lt;Types::SpotInstanceRequest&gt;
    #   * {Types::DescribeSpotInstanceRequestsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe a Spot Instance request
    #
    #   # This example describes the specified Spot Instance request.
    #
    #   resp = client.describe_spot_instance_requests({
    #     spot_instance_request_ids: [
    #       "sir-08b93456", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     spot_instance_requests: [
    #       {
    #         create_time: Time.parse("2014-04-30T18:14:55.000Z"), 
    #         instance_id: "i-1234567890abcdef0", 
    #         launch_specification: {
    #           block_device_mappings: [
    #             {
    #               device_name: "/dev/sda1", 
    #               ebs: {
    #                 delete_on_termination: true, 
    #                 volume_size: 8, 
    #                 volume_type: "standard", 
    #               }, 
    #             }, 
    #           ], 
    #           ebs_optimized: false, 
    #           image_id: "ami-7aba833f", 
    #           instance_type: "m1.small", 
    #           key_name: "my-key-pair", 
    #           security_groups: [
    #             {
    #               group_id: "sg-e38f24a7", 
    #               group_name: "my-security-group", 
    #             }, 
    #           ], 
    #         }, 
    #         launched_availability_zone: "us-west-1b", 
    #         product_description: "Linux/UNIX", 
    #         spot_instance_request_id: "sir-08b93456", 
    #         spot_price: "0.010000", 
    #         state: "active", 
    #         status: {
    #           code: "fulfilled", 
    #           message: "Your Spot request is fulfilled.", 
    #           update_time: Time.parse("2014-04-30T18:16:21.000Z"), 
    #         }, 
    #         type: "one-time", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_spot_instance_requests({
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #     spot_instance_request_ids: ["SpotInstanceRequestId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.spot_instance_requests #=> Array
    #   resp.spot_instance_requests[0].actual_block_hourly_price #=> String
    #   resp.spot_instance_requests[0].availability_zone_group #=> String
    #   resp.spot_instance_requests[0].block_duration_minutes #=> Integer
    #   resp.spot_instance_requests[0].create_time #=> Time
    #   resp.spot_instance_requests[0].fault.code #=> String
    #   resp.spot_instance_requests[0].fault.message #=> String
    #   resp.spot_instance_requests[0].instance_id #=> String
    #   resp.spot_instance_requests[0].launch_group #=> String
    #   resp.spot_instance_requests[0].launch_specification.user_data #=> String
    #   resp.spot_instance_requests[0].launch_specification.addressing_type #=> String
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings #=> Array
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.iops #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.snapshot_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.volume_size #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.kms_key_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.throughput #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.outpost_arn #=> String
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.encrypted #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].no_device #=> String
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].device_name #=> String
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].virtual_name #=> String
    #   resp.spot_instance_requests[0].launch_specification.ebs_optimized #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.iam_instance_profile.arn #=> String
    #   resp.spot_instance_requests[0].launch_specification.iam_instance_profile.name #=> String
    #   resp.spot_instance_requests[0].launch_specification.image_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.spot_instance_requests[0].launch_specification.kernel_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.key_name #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces #=> Array
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].associate_public_ip_address #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].delete_on_termination #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].description #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].device_index #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].groups #=> Array
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].groups[0] #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_6_address_count #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_6_addresses #=> Array
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_6_addresses[0].ipv_6_address #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_6_addresses[0].is_primary_ipv_6 #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].network_interface_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].private_ip_address #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].private_ip_addresses #=> Array
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].private_ip_addresses[0].primary #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].private_ip_addresses[0].private_ip_address #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].secondary_private_ip_address_count #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].subnet_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].associate_carrier_ip_address #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].interface_type #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].network_card_index #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_4_prefixes #=> Array
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_4_prefixes[0].ipv_4_prefix #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_4_prefix_count #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_6_prefixes #=> Array
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_6_prefixes[0].ipv_6_prefix #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_6_prefix_count #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].primary_ipv_6 #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ena_srd_specification.ena_srd_enabled #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ena_srd_specification.ena_srd_udp_specification.ena_srd_udp_enabled #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].connection_tracking_specification.tcp_established_timeout #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].connection_tracking_specification.udp_stream_timeout #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].connection_tracking_specification.udp_timeout #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.placement.availability_zone #=> String
    #   resp.spot_instance_requests[0].launch_specification.placement.group_name #=> String
    #   resp.spot_instance_requests[0].launch_specification.placement.tenancy #=> String, one of "default", "dedicated", "host"
    #   resp.spot_instance_requests[0].launch_specification.ramdisk_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.subnet_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.security_groups #=> Array
    #   resp.spot_instance_requests[0].launch_specification.security_groups[0].group_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.security_groups[0].group_name #=> String
    #   resp.spot_instance_requests[0].launch_specification.monitoring.enabled #=> Boolean
    #   resp.spot_instance_requests[0].launched_availability_zone #=> String
    #   resp.spot_instance_requests[0].product_description #=> String, one of "Linux/UNIX", "Linux/UNIX (Amazon VPC)", "Windows", "Windows (Amazon VPC)"
    #   resp.spot_instance_requests[0].spot_instance_request_id #=> String
    #   resp.spot_instance_requests[0].spot_price #=> String
    #   resp.spot_instance_requests[0].state #=> String, one of "open", "active", "closed", "cancelled", "failed", "disabled"
    #   resp.spot_instance_requests[0].status.code #=> String
    #   resp.spot_instance_requests[0].status.message #=> String
    #   resp.spot_instance_requests[0].status.update_time #=> Time
    #   resp.spot_instance_requests[0].tags #=> Array
    #   resp.spot_instance_requests[0].tags[0].key #=> String
    #   resp.spot_instance_requests[0].tags[0].value #=> String
    #   resp.spot_instance_requests[0].type #=> String, one of "one-time", "persistent"
    #   resp.spot_instance_requests[0].valid_from #=> Time
    #   resp.spot_instance_requests[0].valid_until #=> Time
    #   resp.spot_instance_requests[0].instance_interruption_behavior #=> String, one of "hibernate", "stop", "terminate"
    #   resp.next_token #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * spot_instance_request_fulfilled
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSpotInstanceRequests AWS API Documentation
    #
    # @overload describe_spot_instance_requests(params = {})
    # @param [Hash] params ({})
    def describe_spot_instance_requests(params = {}, options = {})
      req = build_request(:describe_spot_instance_requests, params)
      req.send_request(options)
    end

    # Describes the Spot price history. For more information, see [Spot
    # Instance pricing history][1] in the *Amazon EC2 User Guide*.
    #
    # When you specify a start and end time, the operation returns the
    # prices of the instance types within that time range. It also returns
    # the last price change before the start time, which is the effective
    # price as of the start time.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-spot-instances-history.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Time,DateTime,Date,Integer,String] :start_time
    #   The date and time, up to the past 90 days, from which to start
    #   retrieving the price history data, in UTC format (for example,
    #   *YYYY*-*MM*-*DD*T*HH*:*MM*:*SS*Z).
    #
    # @option params [Time,DateTime,Date,Integer,String] :end_time
    #   The date and time, up to the current date, from which to stop
    #   retrieving the price history data, in UTC format (for example,
    #   *YYYY*-*MM*-*DD*T*HH*:*MM*:*SS*Z).
    #
    # @option params [Array<String>] :instance_types
    #   Filters the results by the specified instance types.
    #
    # @option params [Array<String>] :product_descriptions
    #   Filters the results by the specified basic product descriptions.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `availability-zone` - The Availability Zone for which prices should
    #     be returned.
    #
    #   * `instance-type` - The type of instance (for example, `m3.medium`).
    #
    #   * `product-description` - The product description for the Spot price
    #     (`Linux/UNIX` \| `Red Hat Enterprise Linux` \| `SUSE Linux` \|
    #     `Windows` \| `Linux/UNIX (Amazon VPC)` \| `Red Hat Enterprise Linux
    #     (Amazon VPC)` \| `SUSE Linux (Amazon VPC)` \| `Windows (Amazon
    #     VPC)`).
    #
    #   * `spot-price` - The Spot price. The value must match exactly (or use
    #     wildcards; greater than or less than comparison is not supported).
    #
    #   * `timestamp` - The time stamp of the Spot price history, in UTC
    #     format (for example, *ddd MMM dd HH*:*mm*:*ss* UTC *YYYY*). You can
    #     use wildcards (`*` and `?`). Greater than or less than comparison is
    #     not supported.
    #
    # @option params [String] :availability_zone
    #   Filters the results by the specified Availability Zone.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @return [Types::DescribeSpotPriceHistoryResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeSpotPriceHistoryResult#next_token #next_token} => String
    #   * {Types::DescribeSpotPriceHistoryResult#spot_price_history #spot_price_history} => Array&lt;Types::SpotPrice&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe Spot price history for Linux/UNIX (Amazon VPC)
    #
    #   # This example returns the Spot Price history for m1.xlarge, Linux/UNIX (Amazon VPC) instances for a particular day in
    #   # January.
    #
    #   resp = client.describe_spot_price_history({
    #     end_time: Time.parse("2014-01-06T08:09:10.05Z"), 
    #     instance_types: [
    #       "m1.xlarge", 
    #     ], 
    #     product_descriptions: [
    #       "Linux/UNIX (Amazon VPC)", 
    #     ], 
    #     start_time: Time.parse("2014-01-06T07:08:09.05Z"), 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     spot_price_history: [
    #       {
    #         availability_zone: "us-west-1a", 
    #         instance_type: "m1.xlarge", 
    #         product_description: "Linux/UNIX (Amazon VPC)", 
    #         spot_price: "0.080000", 
    #         timestamp: Time.parse("2014-01-06T04:32:53.000Z"), 
    #       }, 
    #       {
    #         availability_zone: "us-west-1c", 
    #         instance_type: "m1.xlarge", 
    #         product_description: "Linux/UNIX (Amazon VPC)", 
    #         spot_price: "0.080000", 
    #         timestamp: Time.parse("2014-01-05T11:28:26.000Z"), 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_spot_price_history({
    #     dry_run: false,
    #     start_time: Time.now,
    #     end_time: Time.now,
    #     instance_types: ["a1.medium"], # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #     product_descriptions: ["String"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     availability_zone: "String",
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.spot_price_history #=> Array
    #   resp.spot_price_history[0].availability_zone #=> String
    #   resp.spot_price_history[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.spot_price_history[0].product_description #=> String, one of "Linux/UNIX", "Linux/UNIX (Amazon VPC)", "Windows", "Windows (Amazon VPC)"
    #   resp.spot_price_history[0].spot_price #=> String
    #   resp.spot_price_history[0].timestamp #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSpotPriceHistory AWS API Documentation
    #
    # @overload describe_spot_price_history(params = {})
    # @param [Hash] params ({})
    def describe_spot_price_history(params = {}, options = {})
      req = build_request(:describe_spot_price_history, params)
      req.send_request(options)
    end

    # Describes the stale security group rules for security groups
    # referenced across a VPC peering connection, transit gateway
    # connection, or with a security group VPC association. Rules are stale
    # when they reference a deleted security group. Rules can also be stale
    # if they reference a security group in a peer VPC for which the VPC
    # peering connection has been deleted, across a transit gateway where
    # the transit gateway has been deleted (or [the transit gateway security
    # group referencing feature][1] has been disabled), or if a security
    # group VPC association has been disassociated.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/tgw/tgw-vpc-attachments.html#vpc-attachment-security
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @return [Types::DescribeStaleSecurityGroupsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeStaleSecurityGroupsResult#next_token #next_token} => String
    #   * {Types::DescribeStaleSecurityGroupsResult#stale_security_group_set #stale_security_group_set} => Array&lt;Types::StaleSecurityGroup&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_stale_security_groups({
    #     dry_run: false,
    #     max_results: 1,
    #     next_token: "DescribeStaleSecurityGroupsNextToken",
    #     vpc_id: "VpcId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.stale_security_group_set #=> Array
    #   resp.stale_security_group_set[0].description #=> String
    #   resp.stale_security_group_set[0].group_id #=> String
    #   resp.stale_security_group_set[0].group_name #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions #=> Array
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].from_port #=> Integer
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].ip_protocol #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].ip_ranges #=> Array
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].ip_ranges[0] #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].prefix_list_ids #=> Array
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].prefix_list_ids[0] #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].to_port #=> Integer
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].user_id_group_pairs #=> Array
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].user_id_group_pairs[0].description #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].user_id_group_pairs[0].user_id #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].user_id_group_pairs[0].group_name #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].user_id_group_pairs[0].group_id #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].user_id_group_pairs[0].vpc_id #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].user_id_group_pairs[0].vpc_peering_connection_id #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions[0].user_id_group_pairs[0].peering_status #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress #=> Array
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].from_port #=> Integer
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].ip_protocol #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].ip_ranges #=> Array
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].ip_ranges[0] #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].prefix_list_ids #=> Array
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].prefix_list_ids[0] #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].to_port #=> Integer
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].user_id_group_pairs #=> Array
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].user_id_group_pairs[0].description #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].user_id_group_pairs[0].user_id #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].user_id_group_pairs[0].group_name #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].user_id_group_pairs[0].group_id #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].user_id_group_pairs[0].vpc_id #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].user_id_group_pairs[0].vpc_peering_connection_id #=> String
    #   resp.stale_security_group_set[0].stale_ip_permissions_egress[0].user_id_group_pairs[0].peering_status #=> String
    #   resp.stale_security_group_set[0].vpc_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeStaleSecurityGroups AWS API Documentation
    #
    # @overload describe_stale_security_groups(params = {})
    # @param [Hash] params ({})
    def describe_stale_security_groups(params = {}, options = {})
      req = build_request(:describe_stale_security_groups, params)
      req.send_request(options)
    end

    # Describes the progress of the AMI store tasks. You can describe the
    # store tasks for specified AMIs. If you don't specify the AMIs, you
    # get a paginated list of store tasks from the last 31 days.
    #
    # For each AMI task, the response indicates if the task is `InProgress`,
    # `Completed`, or `Failed`. For tasks `InProgress`, the response shows
    # the estimated progress as a percentage.
    #
    # Tasks are listed in reverse chronological order. Currently, only tasks
    # from the past 31 days can be viewed.
    #
    # To use this API, you must have the required permissions. For more
    # information, see [Permissions for storing and restoring AMIs using
    # Amazon S3][1] in the *Amazon EC2 User Guide*.
    #
    # For more information, see [Store and restore an AMI using Amazon
    # S3][2] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ami-store-restore.html#ami-s3-permissions
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ami-store-restore.html
    #
    # @option params [Array<String>] :image_ids
    #   The AMI IDs for which to show progress. Up to 20 AMI IDs can be
    #   included in a request.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `task-state` - Returns tasks in a certain state (`InProgress` \|
    #     `Completed` \| `Failed`)
    #
    #   * `bucket` - Returns task information for tasks that targeted a
    #     specific bucket. For the filter value, specify the bucket name.
    #
    #   <note markdown="1"> When you specify the `ImageIds` parameter, any filters that you
    #   specify are ignored. To use the filters, you must remove the
    #   `ImageIds` parameter.
    #
    #    </note>
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #   You cannot specify this parameter and the `ImageIds` parameter in the
    #   same call.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @return [Types::DescribeStoreImageTasksResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeStoreImageTasksResult#store_image_task_results #store_image_task_results} => Array&lt;Types::StoreImageTaskResult&gt;
    #   * {Types::DescribeStoreImageTasksResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_store_image_tasks({
    #     image_ids: ["ImageId"],
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.store_image_task_results #=> Array
    #   resp.store_image_task_results[0].ami_id #=> String
    #   resp.store_image_task_results[0].task_start_time #=> Time
    #   resp.store_image_task_results[0].bucket #=> String
    #   resp.store_image_task_results[0].s3object_key #=> String
    #   resp.store_image_task_results[0].progress_percentage #=> Integer
    #   resp.store_image_task_results[0].store_task_state #=> String
    #   resp.store_image_task_results[0].store_task_failure_reason #=> String
    #   resp.next_token #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * store_image_task_complete
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeStoreImageTasks AWS API Documentation
    #
    # @overload describe_store_image_tasks(params = {})
    # @param [Hash] params ({})
    def describe_store_image_tasks(params = {}, options = {})
      req = build_request(:describe_store_image_tasks, params)
      req.send_request(options)
    end

    # Describes your subnets. The default is to describe all your subnets.
    # Alternatively, you can specify specific subnet IDs or filter the
    # results to include only the subnets that match specific criteria.
    #
    # For more information, see [Subnets][1] in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/configure-subnets.html
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `availability-zone` - The Availability Zone for the subnet. You can
    #     also use `availabilityZone` as the filter name.
    #
    #   * `availability-zone-id` - The ID of the Availability Zone for the
    #     subnet. You can also use `availabilityZoneId` as the filter name.
    #
    #   * `available-ip-address-count` - The number of IPv4 addresses in the
    #     subnet that are available.
    #
    #   * `cidr-block` - The IPv4 CIDR block of the subnet. The CIDR block you
    #     specify must exactly match the subnet's CIDR block for information
    #     to be returned for the subnet. You can also use `cidr` or
    #     `cidrBlock` as the filter names.
    #
    #   * `customer-owned-ipv4-pool` - The customer-owned IPv4 address pool
    #     associated with the subnet.
    #
    #   * `default-for-az` - Indicates whether this is the default subnet for
    #     the Availability Zone (`true` \| `false`). You can also use
    #     `defaultForAz` as the filter name.
    #
    #   * `enable-dns64` - Indicates whether DNS queries made to the
    #     Amazon-provided DNS Resolver in this subnet should return synthetic
    #     IPv6 addresses for IPv4-only destinations.
    #
    #   * `enable-lni-at-device-index` - Indicates the device position for
    #     local network interfaces in this subnet. For example, `1` indicates
    #     local network interfaces in this subnet are the secondary network
    #     interface (eth1).
    #
    #   * `ipv6-cidr-block-association.ipv6-cidr-block` - An IPv6 CIDR block
    #     associated with the subnet.
    #
    #   * `ipv6-cidr-block-association.association-id` - An association ID for
    #     an IPv6 CIDR block associated with the subnet.
    #
    #   * `ipv6-cidr-block-association.state` - The state of an IPv6 CIDR
    #     block associated with the subnet.
    #
    #   * `ipv6-native` - Indicates whether this is an IPv6 only subnet
    #     (`true` \| `false`).
    #
    #   * `map-customer-owned-ip-on-launch` - Indicates whether a network
    #     interface created in this subnet (including a network interface
    #     created by RunInstances) receives a customer-owned IPv4 address.
    #
    #   * `map-public-ip-on-launch` - Indicates whether instances launched in
    #     this subnet receive a public IPv4 address.
    #
    #   * `outpost-arn` - The Amazon Resource Name (ARN) of the Outpost.
    #
    #   * `owner-id` - The ID of the Amazon Web Services account that owns the
    #     subnet.
    #
    #   * `private-dns-name-options-on-launch.hostname-type` - The type of
    #     hostname to assign to instances in the subnet at launch. For
    #     IPv4-only and dual-stack (IPv4 and IPv6) subnets, an instance DNS
    #     name can be based on the instance IPv4 address (ip-name) or the
    #     instance ID (resource-name). For IPv6 only subnets, an instance DNS
    #     name must be based on the instance ID (resource-name).
    #
    #   * `private-dns-name-options-on-launch.enable-resource-name-dns-a-record`
    #     - Indicates whether to respond to DNS queries for instance hostnames
    #     with DNS A records.
    #
    #   * `private-dns-name-options-on-launch.enable-resource-name-dns-aaaa-record`
    #     - Indicates whether to respond to DNS queries for instance hostnames
    #     with DNS AAAA records.
    #
    #   * `state` - The state of the subnet (`pending` \| `available`).
    #
    #   * `subnet-arn` - The Amazon Resource Name (ARN) of the subnet.
    #
    #   * `subnet-id` - The ID of the subnet.
    #
    #   * `tag` - The key/value combination of a tag assigned to the resource.
    #     Use the tag key in the filter name and the tag value as the filter
    #     value. For example, to find all resources that have a tag with the
    #     key `Owner` and the value `TeamA`, specify `tag:Owner` for the
    #     filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `vpc-id` - The ID of the VPC for the subnet.
    #
    # @option params [Array<String>] :subnet_ids
    #   The IDs of the subnets.
    #
    #   Default: Describes all your subnets.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeSubnetsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeSubnetsResult#next_token #next_token} => String
    #   * {Types::DescribeSubnetsResult#subnets #subnets} => Array&lt;Types::Subnet&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe the subnets for a VPC
    #
    #   # This example describes the subnets for the specified VPC.
    #
    #   resp = client.describe_subnets({
    #     filters: [
    #       {
    #         name: "vpc-id", 
    #         values: [
    #           "vpc-a01106c2", 
    #         ], 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     subnets: [
    #       {
    #         availability_zone: "us-east-1c", 
    #         available_ip_address_count: 251, 
    #         cidr_block: "10.0.1.0/24", 
    #         default_for_az: false, 
    #         map_public_ip_on_launch: false, 
    #         state: "available", 
    #         subnet_id: "subnet-9d4a7b6c", 
    #         vpc_id: "vpc-a01106c2", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_subnets({
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     subnet_ids: ["SubnetId"],
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.subnets #=> Array
    #   resp.subnets[0].availability_zone_id #=> String
    #   resp.subnets[0].enable_lni_at_device_index #=> Integer
    #   resp.subnets[0].map_customer_owned_ip_on_launch #=> Boolean
    #   resp.subnets[0].customer_owned_ipv_4_pool #=> String
    #   resp.subnets[0].owner_id #=> String
    #   resp.subnets[0].assign_ipv_6_address_on_creation #=> Boolean
    #   resp.subnets[0].ipv_6_cidr_block_association_set #=> Array
    #   resp.subnets[0].ipv_6_cidr_block_association_set[0].association_id #=> String
    #   resp.subnets[0].ipv_6_cidr_block_association_set[0].ipv_6_cidr_block #=> String
    #   resp.subnets[0].ipv_6_cidr_block_association_set[0].ipv_6_cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.subnets[0].ipv_6_cidr_block_association_set[0].ipv_6_cidr_block_state.status_message #=> String
    #   resp.subnets[0].ipv_6_cidr_block_association_set[0].ipv_6_address_attribute #=> String, one of "public", "private"
    #   resp.subnets[0].ipv_6_cidr_block_association_set[0].ip_source #=> String, one of "amazon", "byoip", "none"
    #   resp.subnets[0].tags #=> Array
    #   resp.subnets[0].tags[0].key #=> String
    #   resp.subnets[0].tags[0].value #=> String
    #   resp.subnets[0].subnet_arn #=> String
    #   resp.subnets[0].outpost_arn #=> String
    #   resp.subnets[0].enable_dns_64 #=> Boolean
    #   resp.subnets[0].ipv_6_native #=> Boolean
    #   resp.subnets[0].private_dns_name_options_on_launch.hostname_type #=> String, one of "ip-name", "resource-name"
    #   resp.subnets[0].private_dns_name_options_on_launch.enable_resource_name_dns_a_record #=> Boolean
    #   resp.subnets[0].private_dns_name_options_on_launch.enable_resource_name_dns_aaaa_record #=> Boolean
    #   resp.subnets[0].block_public_access_states.internet_gateway_block_mode #=> String, one of "off", "block-bidirectional", "block-ingress"
    #   resp.subnets[0].subnet_id #=> String
    #   resp.subnets[0].state #=> String, one of "pending", "available", "unavailable"
    #   resp.subnets[0].vpc_id #=> String
    #   resp.subnets[0].cidr_block #=> String
    #   resp.subnets[0].available_ip_address_count #=> Integer
    #   resp.subnets[0].availability_zone #=> String
    #   resp.subnets[0].default_for_az #=> Boolean
    #   resp.subnets[0].map_public_ip_on_launch #=> Boolean
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * subnet_available
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeSubnets AWS API Documentation
    #
    # @overload describe_subnets(params = {})
    # @param [Hash] params ({})
    def describe_subnets(params = {}, options = {})
      req = build_request(:describe_subnets, params)
      req.send_request(options)
    end

    # Describes the specified tags for your EC2 resources.
    #
    # For more information about tags, see [Tag your Amazon EC2
    # resources][1] in the *Amazon Elastic Compute Cloud User Guide*.
    #
    # We strongly recommend using only paginated requests. Unpaginated
    # requests are susceptible to throttling and timeouts.
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `key` - The tag key.
    #
    #   * `resource-id` - The ID of the resource.
    #
    #   * `resource-type` - The resource type. For a list of possible values,
    #     see [TagSpecification][1].
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of the tag. For
    #     example, specify "tag:Owner" for the filter name and "TeamA" for
    #     the filter value to find resources with the tag "Owner=TeamA".
    #
    #   * `value` - The tag value.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_TagSpecification.html
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. This value can
    #   be between 5 and 1000. To get the next page of items, make another
    #   request with the token returned in the output. For more information,
    #   see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @return [Types::DescribeTagsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTagsResult#next_token #next_token} => String
    #   * {Types::DescribeTagsResult#tags #tags} => Array&lt;Types::TagDescription&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe the tags for a single resource
    #
    #   # This example describes the tags for the specified instance.
    #
    #   resp = client.describe_tags({
    #     filters: [
    #       {
    #         name: "resource-id", 
    #         values: [
    #           "i-1234567890abcdef8", 
    #         ], 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     tags: [
    #       {
    #         key: "Stack", 
    #         resource_id: "i-1234567890abcdef8", 
    #         resource_type: "instance", 
    #         value: "test", 
    #       }, 
    #       {
    #         key: "Name", 
    #         resource_id: "i-1234567890abcdef8", 
    #         resource_type: "instance", 
    #         value: "Beta Server", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_tags({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.tags #=> Array
    #   resp.tags[0].key #=> String
    #   resp.tags[0].resource_id #=> String
    #   resp.tags[0].resource_type #=> String, one of "capacity-reservation", "client-vpn-endpoint", "customer-gateway", "carrier-gateway", "coip-pool", "declarative-policies-report", "dedicated-host", "dhcp-options", "egress-only-internet-gateway", "elastic-ip", "elastic-gpu", "export-image-task", "export-instance-task", "fleet", "fpga-image", "host-reservation", "image", "import-image-task", "import-snapshot-task", "instance", "instance-event-window", "internet-gateway", "ipam", "ipam-pool", "ipam-scope", "ipv4pool-ec2", "ipv6pool-ec2", "key-pair", "launch-template", "local-gateway", "local-gateway-route-table", "local-gateway-virtual-interface", "local-gateway-virtual-interface-group", "local-gateway-route-table-vpc-association", "local-gateway-route-table-virtual-interface-group-association", "natgateway", "network-acl", "network-interface", "network-insights-analysis", "network-insights-path", "network-insights-access-scope", "network-insights-access-scope-analysis", "placement-group", "prefix-list", "replace-root-volume-task", "reserved-instances", "route-table", "security-group", "security-group-rule", "snapshot", "spot-fleet-request", "spot-instances-request", "subnet", "subnet-cidr-reservation", "traffic-mirror-filter", "traffic-mirror-session", "traffic-mirror-target", "transit-gateway", "transit-gateway-attachment", "transit-gateway-connect-peer", "transit-gateway-multicast-domain", "transit-gateway-policy-table", "transit-gateway-route-table", "transit-gateway-route-table-announcement", "volume", "vpc", "vpc-endpoint", "vpc-endpoint-connection", "vpc-endpoint-service", "vpc-endpoint-service-permission", "vpc-peering-connection", "vpn-connection", "vpn-gateway", "vpc-flow-log", "capacity-reservation-fleet", "traffic-mirror-filter-rule", "vpc-endpoint-connection-device-type", "verified-access-instance", "verified-access-group", "verified-access-endpoint", "verified-access-policy", "verified-access-trust-provider", "vpn-connection-device-type", "vpc-block-public-access-exclusion", "ipam-resource-discovery", "ipam-resource-discovery-association", "instance-connect-endpoint", "verified-access-endpoint-target", "ipam-external-resource-verification-token"
    #   resp.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTags AWS API Documentation
    #
    # @overload describe_tags(params = {})
    # @param [Hash] params ({})
    def describe_tags(params = {}, options = {})
      req = build_request(:describe_tags, params)
      req.send_request(options)
    end

    # Describe traffic mirror filters that determine the traffic that is
    # mirrored.
    #
    # @option params [Array<String>] :traffic_mirror_filter_rule_ids
    #   Traffic filter rule IDs.
    #
    # @option params [String] :traffic_mirror_filter_id
    #   Traffic filter ID.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   Traffic mirror filters.
    #
    #   * `traffic-mirror-filter-rule-id`: The ID of the Traffic Mirror rule.
    #
    #   * `traffic-mirror-filter-id`: The ID of the filter that this rule is
    #     associated with.
    #
    #   * `rule-number`: The number of the Traffic Mirror rule.
    #
    #   * `rule-action`: The action taken on the filtered traffic. Possible
    #     actions are `accept` and `reject`.
    #
    #   * `traffic-direction`: The traffic direction. Possible directions are
    #     `ingress` and `egress`.
    #
    #   * `protocol`: The protocol, for example UDP, assigned to the Traffic
    #     Mirror rule.
    #
    #   * `source-cidr-block`: The source CIDR block assigned to the Traffic
    #     Mirror rule.
    #
    #   * `destination-cidr-block`: The destination CIDR block assigned to the
    #     Traffic Mirror rule.
    #
    #   * `description`: The description of the Traffic Mirror rule.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::DescribeTrafficMirrorFilterRulesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTrafficMirrorFilterRulesResult#traffic_mirror_filter_rules #traffic_mirror_filter_rules} => Array&lt;Types::TrafficMirrorFilterRule&gt;
    #   * {Types::DescribeTrafficMirrorFilterRulesResult#next_token #next_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_traffic_mirror_filter_rules({
    #     traffic_mirror_filter_rule_ids: ["TrafficMirrorFilterRuleIdWithResolver"],
    #     traffic_mirror_filter_id: "TrafficMirrorFilterId",
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_filter_rules #=> Array
    #   resp.traffic_mirror_filter_rules[0].traffic_mirror_filter_rule_id #=> String
    #   resp.traffic_mirror_filter_rules[0].traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_filter_rules[0].traffic_direction #=> String, one of "ingress", "egress"
    #   resp.traffic_mirror_filter_rules[0].rule_number #=> Integer
    #   resp.traffic_mirror_filter_rules[0].rule_action #=> String, one of "accept", "reject"
    #   resp.traffic_mirror_filter_rules[0].protocol #=> Integer
    #   resp.traffic_mirror_filter_rules[0].destination_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filter_rules[0].destination_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filter_rules[0].source_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filter_rules[0].source_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filter_rules[0].destination_cidr_block #=> String
    #   resp.traffic_mirror_filter_rules[0].source_cidr_block #=> String
    #   resp.traffic_mirror_filter_rules[0].description #=> String
    #   resp.traffic_mirror_filter_rules[0].tags #=> Array
    #   resp.traffic_mirror_filter_rules[0].tags[0].key #=> String
    #   resp.traffic_mirror_filter_rules[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTrafficMirrorFilterRules AWS API Documentation
    #
    # @overload describe_traffic_mirror_filter_rules(params = {})
    # @param [Hash] params ({})
    def describe_traffic_mirror_filter_rules(params = {}, options = {})
      req = build_request(:describe_traffic_mirror_filter_rules, params)
      req.send_request(options)
    end

    # Describes one or more Traffic Mirror filters.
    #
    # @option params [Array<String>] :traffic_mirror_filter_ids
    #   The ID of the Traffic Mirror filter.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `description`: The Traffic Mirror filter description.
    #
    #   * `traffic-mirror-filter-id`: The ID of the Traffic Mirror filter.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::DescribeTrafficMirrorFiltersResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTrafficMirrorFiltersResult#traffic_mirror_filters #traffic_mirror_filters} => Array&lt;Types::TrafficMirrorFilter&gt;
    #   * {Types::DescribeTrafficMirrorFiltersResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_traffic_mirror_filters({
    #     traffic_mirror_filter_ids: ["TrafficMirrorFilterId"],
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_filters #=> Array
    #   resp.traffic_mirror_filters[0].traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_filters[0].ingress_filter_rules #=> Array
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].traffic_mirror_filter_rule_id #=> String
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].traffic_direction #=> String, one of "ingress", "egress"
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].rule_number #=> Integer
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].rule_action #=> String, one of "accept", "reject"
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].protocol #=> Integer
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].destination_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].destination_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].source_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].source_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].destination_cidr_block #=> String
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].source_cidr_block #=> String
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].description #=> String
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].tags #=> Array
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].tags[0].key #=> String
    #   resp.traffic_mirror_filters[0].ingress_filter_rules[0].tags[0].value #=> String
    #   resp.traffic_mirror_filters[0].egress_filter_rules #=> Array
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].traffic_mirror_filter_rule_id #=> String
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].traffic_direction #=> String, one of "ingress", "egress"
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].rule_number #=> Integer
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].rule_action #=> String, one of "accept", "reject"
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].protocol #=> Integer
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].destination_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].destination_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].source_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].source_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].destination_cidr_block #=> String
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].source_cidr_block #=> String
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].description #=> String
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].tags #=> Array
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].tags[0].key #=> String
    #   resp.traffic_mirror_filters[0].egress_filter_rules[0].tags[0].value #=> String
    #   resp.traffic_mirror_filters[0].network_services #=> Array
    #   resp.traffic_mirror_filters[0].network_services[0] #=> String, one of "amazon-dns"
    #   resp.traffic_mirror_filters[0].description #=> String
    #   resp.traffic_mirror_filters[0].tags #=> Array
    #   resp.traffic_mirror_filters[0].tags[0].key #=> String
    #   resp.traffic_mirror_filters[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTrafficMirrorFilters AWS API Documentation
    #
    # @overload describe_traffic_mirror_filters(params = {})
    # @param [Hash] params ({})
    def describe_traffic_mirror_filters(params = {}, options = {})
      req = build_request(:describe_traffic_mirror_filters, params)
      req.send_request(options)
    end

    # Describes one or more Traffic Mirror sessions. By default, all Traffic
    # Mirror sessions are described. Alternatively, you can filter the
    # results.
    #
    # @option params [Array<String>] :traffic_mirror_session_ids
    #   The ID of the Traffic Mirror session.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `description`: The Traffic Mirror session description.
    #
    #   * `network-interface-id`: The ID of the Traffic Mirror session network
    #     interface.
    #
    #   * `owner-id`: The ID of the account that owns the Traffic Mirror
    #     session.
    #
    #   * `packet-length`: The assigned number of packets to mirror.
    #
    #   * `session-number`: The assigned session number.
    #
    #   * `traffic-mirror-filter-id`: The ID of the Traffic Mirror filter.
    #
    #   * `traffic-mirror-session-id`: The ID of the Traffic Mirror session.
    #
    #   * `traffic-mirror-target-id`: The ID of the Traffic Mirror target.
    #
    #   * `virtual-network-id`: The virtual network ID of the Traffic Mirror
    #     session.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::DescribeTrafficMirrorSessionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTrafficMirrorSessionsResult#traffic_mirror_sessions #traffic_mirror_sessions} => Array&lt;Types::TrafficMirrorSession&gt;
    #   * {Types::DescribeTrafficMirrorSessionsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_traffic_mirror_sessions({
    #     traffic_mirror_session_ids: ["TrafficMirrorSessionId"],
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_sessions #=> Array
    #   resp.traffic_mirror_sessions[0].traffic_mirror_session_id #=> String
    #   resp.traffic_mirror_sessions[0].traffic_mirror_target_id #=> String
    #   resp.traffic_mirror_sessions[0].traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_sessions[0].network_interface_id #=> String
    #   resp.traffic_mirror_sessions[0].owner_id #=> String
    #   resp.traffic_mirror_sessions[0].packet_length #=> Integer
    #   resp.traffic_mirror_sessions[0].session_number #=> Integer
    #   resp.traffic_mirror_sessions[0].virtual_network_id #=> Integer
    #   resp.traffic_mirror_sessions[0].description #=> String
    #   resp.traffic_mirror_sessions[0].tags #=> Array
    #   resp.traffic_mirror_sessions[0].tags[0].key #=> String
    #   resp.traffic_mirror_sessions[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTrafficMirrorSessions AWS API Documentation
    #
    # @overload describe_traffic_mirror_sessions(params = {})
    # @param [Hash] params ({})
    def describe_traffic_mirror_sessions(params = {}, options = {})
      req = build_request(:describe_traffic_mirror_sessions, params)
      req.send_request(options)
    end

    # Information about one or more Traffic Mirror targets.
    #
    # @option params [Array<String>] :traffic_mirror_target_ids
    #   The ID of the Traffic Mirror targets.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `description`: The Traffic Mirror target description.
    #
    #   * `network-interface-id`: The ID of the Traffic Mirror session network
    #     interface.
    #
    #   * `network-load-balancer-arn`: The Amazon Resource Name (ARN) of the
    #     Network Load Balancer that is associated with the session.
    #
    #   * `owner-id`: The ID of the account that owns the Traffic Mirror
    #     session.
    #
    #   * `traffic-mirror-target-id`: The ID of the Traffic Mirror target.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::DescribeTrafficMirrorTargetsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTrafficMirrorTargetsResult#traffic_mirror_targets #traffic_mirror_targets} => Array&lt;Types::TrafficMirrorTarget&gt;
    #   * {Types::DescribeTrafficMirrorTargetsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_traffic_mirror_targets({
    #     traffic_mirror_target_ids: ["TrafficMirrorTargetId"],
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_targets #=> Array
    #   resp.traffic_mirror_targets[0].traffic_mirror_target_id #=> String
    #   resp.traffic_mirror_targets[0].network_interface_id #=> String
    #   resp.traffic_mirror_targets[0].network_load_balancer_arn #=> String
    #   resp.traffic_mirror_targets[0].type #=> String, one of "network-interface", "network-load-balancer", "gateway-load-balancer-endpoint"
    #   resp.traffic_mirror_targets[0].description #=> String
    #   resp.traffic_mirror_targets[0].owner_id #=> String
    #   resp.traffic_mirror_targets[0].tags #=> Array
    #   resp.traffic_mirror_targets[0].tags[0].key #=> String
    #   resp.traffic_mirror_targets[0].tags[0].value #=> String
    #   resp.traffic_mirror_targets[0].gateway_load_balancer_endpoint_id #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTrafficMirrorTargets AWS API Documentation
    #
    # @overload describe_traffic_mirror_targets(params = {})
    # @param [Hash] params ({})
    def describe_traffic_mirror_targets(params = {}, options = {})
      req = build_request(:describe_traffic_mirror_targets, params)
      req.send_request(options)
    end

    # Describes one or more attachments between resources and transit
    # gateways. By default, all attachments are described. Alternatively,
    # you can filter the results by attachment ID, attachment state,
    # resource ID, or resource owner.
    #
    # @option params [Array<String>] :transit_gateway_attachment_ids
    #   The IDs of the attachments.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `association.state` - The state of the association (`associating` \|
    #     `associated` \| `disassociating`).
    #
    #   * `association.transit-gateway-route-table-id` - The ID of the route
    #     table for the transit gateway.
    #
    #   * `resource-id` - The ID of the resource.
    #
    #   * `resource-owner-id` - The ID of the Amazon Web Services account that
    #     owns the resource.
    #
    #   * `resource-type` - The resource type. Valid values are `vpc` \| `vpn`
    #     \| `direct-connect-gateway` \| `peering` \| `connect`.
    #
    #   * `state` - The state of the attachment. Valid values are `available`
    #     \| `deleted` \| `deleting` \| `failed` \| `failing` \|
    #     `initiatingRequest` \| `modifying` \| `pendingAcceptance` \|
    #     `pending` \| `rollingBack` \| `rejected` \| `rejecting`.
    #
    #   * `transit-gateway-attachment-id` - The ID of the attachment.
    #
    #   * `transit-gateway-id` - The ID of the transit gateway.
    #
    #   * `transit-gateway-owner-id` - The ID of the Amazon Web Services
    #     account that owns the transit gateway.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeTransitGatewayAttachmentsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTransitGatewayAttachmentsResult#transit_gateway_attachments #transit_gateway_attachments} => Array&lt;Types::TransitGatewayAttachment&gt;
    #   * {Types::DescribeTransitGatewayAttachmentsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_transit_gateway_attachments({
    #     transit_gateway_attachment_ids: ["TransitGatewayAttachmentId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_attachments #=> Array
    #   resp.transit_gateway_attachments[0].transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_attachments[0].transit_gateway_id #=> String
    #   resp.transit_gateway_attachments[0].transit_gateway_owner_id #=> String
    #   resp.transit_gateway_attachments[0].resource_owner_id #=> String
    #   resp.transit_gateway_attachments[0].resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.transit_gateway_attachments[0].resource_id #=> String
    #   resp.transit_gateway_attachments[0].state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_attachments[0].association.transit_gateway_route_table_id #=> String
    #   resp.transit_gateway_attachments[0].association.state #=> String, one of "associating", "associated", "disassociating", "disassociated"
    #   resp.transit_gateway_attachments[0].creation_time #=> Time
    #   resp.transit_gateway_attachments[0].tags #=> Array
    #   resp.transit_gateway_attachments[0].tags[0].key #=> String
    #   resp.transit_gateway_attachments[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTransitGatewayAttachments AWS API Documentation
    #
    # @overload describe_transit_gateway_attachments(params = {})
    # @param [Hash] params ({})
    def describe_transit_gateway_attachments(params = {}, options = {})
      req = build_request(:describe_transit_gateway_attachments, params)
      req.send_request(options)
    end

    # Describes one or more Connect peers.
    #
    # @option params [Array<String>] :transit_gateway_connect_peer_ids
    #   The IDs of the Connect peers.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `state` - The state of the Connect peer (`pending` \| `available` \|
    #     `deleting` \| `deleted`).
    #
    #   * `transit-gateway-attachment-id` - The ID of the attachment.
    #
    #   * `transit-gateway-connect-peer-id` - The ID of the Connect peer.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeTransitGatewayConnectPeersResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTransitGatewayConnectPeersResult#transit_gateway_connect_peers #transit_gateway_connect_peers} => Array&lt;Types::TransitGatewayConnectPeer&gt;
    #   * {Types::DescribeTransitGatewayConnectPeersResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_transit_gateway_connect_peers({
    #     transit_gateway_connect_peer_ids: ["TransitGatewayConnectPeerId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_connect_peers #=> Array
    #   resp.transit_gateway_connect_peers[0].transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_connect_peers[0].transit_gateway_connect_peer_id #=> String
    #   resp.transit_gateway_connect_peers[0].state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.transit_gateway_connect_peers[0].creation_time #=> Time
    #   resp.transit_gateway_connect_peers[0].connect_peer_configuration.transit_gateway_address #=> String
    #   resp.transit_gateway_connect_peers[0].connect_peer_configuration.peer_address #=> String
    #   resp.transit_gateway_connect_peers[0].connect_peer_configuration.inside_cidr_blocks #=> Array
    #   resp.transit_gateway_connect_peers[0].connect_peer_configuration.inside_cidr_blocks[0] #=> String
    #   resp.transit_gateway_connect_peers[0].connect_peer_configuration.protocol #=> String, one of "gre"
    #   resp.transit_gateway_connect_peers[0].connect_peer_configuration.bgp_configurations #=> Array
    #   resp.transit_gateway_connect_peers[0].connect_peer_configuration.bgp_configurations[0].transit_gateway_asn #=> Integer
    #   resp.transit_gateway_connect_peers[0].connect_peer_configuration.bgp_configurations[0].peer_asn #=> Integer
    #   resp.transit_gateway_connect_peers[0].connect_peer_configuration.bgp_configurations[0].transit_gateway_address #=> String
    #   resp.transit_gateway_connect_peers[0].connect_peer_configuration.bgp_configurations[0].peer_address #=> String
    #   resp.transit_gateway_connect_peers[0].connect_peer_configuration.bgp_configurations[0].bgp_status #=> String, one of "up", "down"
    #   resp.transit_gateway_connect_peers[0].tags #=> Array
    #   resp.transit_gateway_connect_peers[0].tags[0].key #=> String
    #   resp.transit_gateway_connect_peers[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTransitGatewayConnectPeers AWS API Documentation
    #
    # @overload describe_transit_gateway_connect_peers(params = {})
    # @param [Hash] params ({})
    def describe_transit_gateway_connect_peers(params = {}, options = {})
      req = build_request(:describe_transit_gateway_connect_peers, params)
      req.send_request(options)
    end

    # Describes one or more Connect attachments.
    #
    # @option params [Array<String>] :transit_gateway_attachment_ids
    #   The IDs of the attachments.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `options.protocol` - The tunnel protocol (`gre`).
    #
    #   * `state` - The state of the attachment (`initiating` \|
    #     `initiatingRequest` \| `pendingAcceptance` \| `rollingBack` \|
    #     `pending` \| `available` \| `modifying` \| `deleting` \| `deleted`
    #     \| `failed` \| `rejected` \| `rejecting` \| `failing`).
    #
    #   * `transit-gateway-attachment-id` - The ID of the Connect attachment.
    #
    #   * `transit-gateway-id` - The ID of the transit gateway.
    #
    #   * `transport-transit-gateway-attachment-id` - The ID of the transit
    #     gateway attachment from which the Connect attachment was created.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeTransitGatewayConnectsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTransitGatewayConnectsResult#transit_gateway_connects #transit_gateway_connects} => Array&lt;Types::TransitGatewayConnect&gt;
    #   * {Types::DescribeTransitGatewayConnectsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_transit_gateway_connects({
    #     transit_gateway_attachment_ids: ["TransitGatewayAttachmentId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_connects #=> Array
    #   resp.transit_gateway_connects[0].transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_connects[0].transport_transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_connects[0].transit_gateway_id #=> String
    #   resp.transit_gateway_connects[0].state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_connects[0].creation_time #=> Time
    #   resp.transit_gateway_connects[0].options.protocol #=> String, one of "gre"
    #   resp.transit_gateway_connects[0].tags #=> Array
    #   resp.transit_gateway_connects[0].tags[0].key #=> String
    #   resp.transit_gateway_connects[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTransitGatewayConnects AWS API Documentation
    #
    # @overload describe_transit_gateway_connects(params = {})
    # @param [Hash] params ({})
    def describe_transit_gateway_connects(params = {}, options = {})
      req = build_request(:describe_transit_gateway_connects, params)
      req.send_request(options)
    end

    # Describes one or more transit gateway multicast domains.
    #
    # @option params [Array<String>] :transit_gateway_multicast_domain_ids
    #   The ID of the transit gateway multicast domain.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `state` - The state of the transit gateway multicast domain. Valid
    #     values are `pending` \| `available` \| `deleting` \| `deleted`.
    #
    #   * `transit-gateway-id` - The ID of the transit gateway.
    #
    #   * `transit-gateway-multicast-domain-id` - The ID of the transit
    #     gateway multicast domain.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeTransitGatewayMulticastDomainsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTransitGatewayMulticastDomainsResult#transit_gateway_multicast_domains #transit_gateway_multicast_domains} => Array&lt;Types::TransitGatewayMulticastDomain&gt;
    #   * {Types::DescribeTransitGatewayMulticastDomainsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_transit_gateway_multicast_domains({
    #     transit_gateway_multicast_domain_ids: ["TransitGatewayMulticastDomainId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_multicast_domains #=> Array
    #   resp.transit_gateway_multicast_domains[0].transit_gateway_multicast_domain_id #=> String
    #   resp.transit_gateway_multicast_domains[0].transit_gateway_id #=> String
    #   resp.transit_gateway_multicast_domains[0].transit_gateway_multicast_domain_arn #=> String
    #   resp.transit_gateway_multicast_domains[0].owner_id #=> String
    #   resp.transit_gateway_multicast_domains[0].options.igmpv_2_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_multicast_domains[0].options.static_sources_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_multicast_domains[0].options.auto_accept_shared_associations #=> String, one of "enable", "disable"
    #   resp.transit_gateway_multicast_domains[0].state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.transit_gateway_multicast_domains[0].creation_time #=> Time
    #   resp.transit_gateway_multicast_domains[0].tags #=> Array
    #   resp.transit_gateway_multicast_domains[0].tags[0].key #=> String
    #   resp.transit_gateway_multicast_domains[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTransitGatewayMulticastDomains AWS API Documentation
    #
    # @overload describe_transit_gateway_multicast_domains(params = {})
    # @param [Hash] params ({})
    def describe_transit_gateway_multicast_domains(params = {}, options = {})
      req = build_request(:describe_transit_gateway_multicast_domains, params)
      req.send_request(options)
    end

    # Describes your transit gateway peering attachments.
    #
    # @option params [Array<String>] :transit_gateway_attachment_ids
    #   One or more IDs of the transit gateway peering attachments.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `transit-gateway-attachment-id` - The ID of the transit gateway
    #     attachment.
    #
    #   * `local-owner-id` - The ID of your Amazon Web Services account.
    #
    #   * `remote-owner-id` - The ID of the Amazon Web Services account in the
    #     remote Region that owns the transit gateway.
    #
    #   * `state` - The state of the peering attachment. Valid values are
    #     `available` \| `deleted` \| `deleting` \| `failed` \| `failing` \|
    #     `initiatingRequest` \| `modifying` \| `pendingAcceptance` \|
    #     `pending` \| `rollingBack` \| `rejected` \| `rejecting`).
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources that have a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `transit-gateway-id` - The ID of the transit gateway.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeTransitGatewayPeeringAttachmentsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTransitGatewayPeeringAttachmentsResult#transit_gateway_peering_attachments #transit_gateway_peering_attachments} => Array&lt;Types::TransitGatewayPeeringAttachment&gt;
    #   * {Types::DescribeTransitGatewayPeeringAttachmentsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_transit_gateway_peering_attachments({
    #     transit_gateway_attachment_ids: ["TransitGatewayAttachmentId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_peering_attachments #=> Array
    #   resp.transit_gateway_peering_attachments[0].transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_peering_attachments[0].accepter_transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_peering_attachments[0].requester_tgw_info.transit_gateway_id #=> String
    #   resp.transit_gateway_peering_attachments[0].requester_tgw_info.core_network_id #=> String
    #   resp.transit_gateway_peering_attachments[0].requester_tgw_info.owner_id #=> String
    #   resp.transit_gateway_peering_attachments[0].requester_tgw_info.region #=> String
    #   resp.transit_gateway_peering_attachments[0].accepter_tgw_info.transit_gateway_id #=> String
    #   resp.transit_gateway_peering_attachments[0].accepter_tgw_info.core_network_id #=> String
    #   resp.transit_gateway_peering_attachments[0].accepter_tgw_info.owner_id #=> String
    #   resp.transit_gateway_peering_attachments[0].accepter_tgw_info.region #=> String
    #   resp.transit_gateway_peering_attachments[0].options.dynamic_routing #=> String, one of "enable", "disable"
    #   resp.transit_gateway_peering_attachments[0].status.code #=> String
    #   resp.transit_gateway_peering_attachments[0].status.message #=> String
    #   resp.transit_gateway_peering_attachments[0].state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_peering_attachments[0].creation_time #=> Time
    #   resp.transit_gateway_peering_attachments[0].tags #=> Array
    #   resp.transit_gateway_peering_attachments[0].tags[0].key #=> String
    #   resp.transit_gateway_peering_attachments[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTransitGatewayPeeringAttachments AWS API Documentation
    #
    # @overload describe_transit_gateway_peering_attachments(params = {})
    # @param [Hash] params ({})
    def describe_transit_gateway_peering_attachments(params = {}, options = {})
      req = build_request(:describe_transit_gateway_peering_attachments, params)
      req.send_request(options)
    end

    # Describes one or more transit gateway route policy tables.
    #
    # @option params [Array<String>] :transit_gateway_policy_table_ids
    #   The IDs of the transit gateway policy tables.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters associated with the transit gateway policy table.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeTransitGatewayPolicyTablesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTransitGatewayPolicyTablesResult#transit_gateway_policy_tables #transit_gateway_policy_tables} => Array&lt;Types::TransitGatewayPolicyTable&gt;
    #   * {Types::DescribeTransitGatewayPolicyTablesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_transit_gateway_policy_tables({
    #     transit_gateway_policy_table_ids: ["TransitGatewayPolicyTableId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_policy_tables #=> Array
    #   resp.transit_gateway_policy_tables[0].transit_gateway_policy_table_id #=> String
    #   resp.transit_gateway_policy_tables[0].transit_gateway_id #=> String
    #   resp.transit_gateway_policy_tables[0].state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.transit_gateway_policy_tables[0].creation_time #=> Time
    #   resp.transit_gateway_policy_tables[0].tags #=> Array
    #   resp.transit_gateway_policy_tables[0].tags[0].key #=> String
    #   resp.transit_gateway_policy_tables[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTransitGatewayPolicyTables AWS API Documentation
    #
    # @overload describe_transit_gateway_policy_tables(params = {})
    # @param [Hash] params ({})
    def describe_transit_gateway_policy_tables(params = {}, options = {})
      req = build_request(:describe_transit_gateway_policy_tables, params)
      req.send_request(options)
    end

    # Describes one or more transit gateway route table advertisements.
    #
    # @option params [Array<String>] :transit_gateway_route_table_announcement_ids
    #   The IDs of the transit gateway route tables that are being advertised.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters associated with the transit gateway policy table.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeTransitGatewayRouteTableAnnouncementsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTransitGatewayRouteTableAnnouncementsResult#transit_gateway_route_table_announcements #transit_gateway_route_table_announcements} => Array&lt;Types::TransitGatewayRouteTableAnnouncement&gt;
    #   * {Types::DescribeTransitGatewayRouteTableAnnouncementsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_transit_gateway_route_table_announcements({
    #     transit_gateway_route_table_announcement_ids: ["TransitGatewayRouteTableAnnouncementId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_route_table_announcements #=> Array
    #   resp.transit_gateway_route_table_announcements[0].transit_gateway_route_table_announcement_id #=> String
    #   resp.transit_gateway_route_table_announcements[0].transit_gateway_id #=> String
    #   resp.transit_gateway_route_table_announcements[0].core_network_id #=> String
    #   resp.transit_gateway_route_table_announcements[0].peer_transit_gateway_id #=> String
    #   resp.transit_gateway_route_table_announcements[0].peer_core_network_id #=> String
    #   resp.transit_gateway_route_table_announcements[0].peering_attachment_id #=> String
    #   resp.transit_gateway_route_table_announcements[0].announcement_direction #=> String, one of "outgoing", "incoming"
    #   resp.transit_gateway_route_table_announcements[0].transit_gateway_route_table_id #=> String
    #   resp.transit_gateway_route_table_announcements[0].state #=> String, one of "available", "pending", "failing", "failed", "deleting", "deleted"
    #   resp.transit_gateway_route_table_announcements[0].creation_time #=> Time
    #   resp.transit_gateway_route_table_announcements[0].tags #=> Array
    #   resp.transit_gateway_route_table_announcements[0].tags[0].key #=> String
    #   resp.transit_gateway_route_table_announcements[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTransitGatewayRouteTableAnnouncements AWS API Documentation
    #
    # @overload describe_transit_gateway_route_table_announcements(params = {})
    # @param [Hash] params ({})
    def describe_transit_gateway_route_table_announcements(params = {}, options = {})
      req = build_request(:describe_transit_gateway_route_table_announcements, params)
      req.send_request(options)
    end

    # Describes one or more transit gateway route tables. By default, all
    # transit gateway route tables are described. Alternatively, you can
    # filter the results.
    #
    # @option params [Array<String>] :transit_gateway_route_table_ids
    #   The IDs of the transit gateway route tables.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `default-association-route-table` - Indicates whether this is the
    #     default association route table for the transit gateway (`true` \|
    #     `false`).
    #
    #   * `default-propagation-route-table` - Indicates whether this is the
    #     default propagation route table for the transit gateway (`true` \|
    #     `false`).
    #
    #   * `state` - The state of the route table (`available` \| `deleting` \|
    #     `deleted` \| `pending`).
    #
    #   * `transit-gateway-id` - The ID of the transit gateway.
    #
    #   * `transit-gateway-route-table-id` - The ID of the transit gateway
    #     route table.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeTransitGatewayRouteTablesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTransitGatewayRouteTablesResult#transit_gateway_route_tables #transit_gateway_route_tables} => Array&lt;Types::TransitGatewayRouteTable&gt;
    #   * {Types::DescribeTransitGatewayRouteTablesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_transit_gateway_route_tables({
    #     transit_gateway_route_table_ids: ["TransitGatewayRouteTableId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_route_tables #=> Array
    #   resp.transit_gateway_route_tables[0].transit_gateway_route_table_id #=> String
    #   resp.transit_gateway_route_tables[0].transit_gateway_id #=> String
    #   resp.transit_gateway_route_tables[0].state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.transit_gateway_route_tables[0].default_association_route_table #=> Boolean
    #   resp.transit_gateway_route_tables[0].default_propagation_route_table #=> Boolean
    #   resp.transit_gateway_route_tables[0].creation_time #=> Time
    #   resp.transit_gateway_route_tables[0].tags #=> Array
    #   resp.transit_gateway_route_tables[0].tags[0].key #=> String
    #   resp.transit_gateway_route_tables[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTransitGatewayRouteTables AWS API Documentation
    #
    # @overload describe_transit_gateway_route_tables(params = {})
    # @param [Hash] params ({})
    def describe_transit_gateway_route_tables(params = {}, options = {})
      req = build_request(:describe_transit_gateway_route_tables, params)
      req.send_request(options)
    end

    # Describes one or more VPC attachments. By default, all VPC attachments
    # are described. Alternatively, you can filter the results.
    #
    # @option params [Array<String>] :transit_gateway_attachment_ids
    #   The IDs of the attachments.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `state` - The state of the attachment. Valid values are `available`
    #     \| `deleted` \| `deleting` \| `failed` \| `failing` \|
    #     `initiatingRequest` \| `modifying` \| `pendingAcceptance` \|
    #     `pending` \| `rollingBack` \| `rejected` \| `rejecting`.
    #
    #   * `transit-gateway-attachment-id` - The ID of the attachment.
    #
    #   * `transit-gateway-id` - The ID of the transit gateway.
    #
    #   * `vpc-id` - The ID of the VPC.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeTransitGatewayVpcAttachmentsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTransitGatewayVpcAttachmentsResult#transit_gateway_vpc_attachments #transit_gateway_vpc_attachments} => Array&lt;Types::TransitGatewayVpcAttachment&gt;
    #   * {Types::DescribeTransitGatewayVpcAttachmentsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_transit_gateway_vpc_attachments({
    #     transit_gateway_attachment_ids: ["TransitGatewayAttachmentId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_vpc_attachments #=> Array
    #   resp.transit_gateway_vpc_attachments[0].transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_vpc_attachments[0].transit_gateway_id #=> String
    #   resp.transit_gateway_vpc_attachments[0].vpc_id #=> String
    #   resp.transit_gateway_vpc_attachments[0].vpc_owner_id #=> String
    #   resp.transit_gateway_vpc_attachments[0].state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_vpc_attachments[0].subnet_ids #=> Array
    #   resp.transit_gateway_vpc_attachments[0].subnet_ids[0] #=> String
    #   resp.transit_gateway_vpc_attachments[0].creation_time #=> Time
    #   resp.transit_gateway_vpc_attachments[0].options.dns_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachments[0].options.security_group_referencing_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachments[0].options.ipv_6_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachments[0].options.appliance_mode_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachments[0].tags #=> Array
    #   resp.transit_gateway_vpc_attachments[0].tags[0].key #=> String
    #   resp.transit_gateway_vpc_attachments[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTransitGatewayVpcAttachments AWS API Documentation
    #
    # @overload describe_transit_gateway_vpc_attachments(params = {})
    # @param [Hash] params ({})
    def describe_transit_gateway_vpc_attachments(params = {}, options = {})
      req = build_request(:describe_transit_gateway_vpc_attachments, params)
      req.send_request(options)
    end

    # Describes one or more transit gateways. By default, all transit
    # gateways are described. Alternatively, you can filter the results.
    #
    # @option params [Array<String>] :transit_gateway_ids
    #   The IDs of the transit gateways.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `options.propagation-default-route-table-id` - The ID of the default
    #     propagation route table.
    #
    #   * `options.amazon-side-asn` - The private ASN for the Amazon side of a
    #     BGP session.
    #
    #   * `options.association-default-route-table-id` - The ID of the default
    #     association route table.
    #
    #   * `options.auto-accept-shared-attachments` - Indicates whether there
    #     is automatic acceptance of attachment requests (`enable` \|
    #     `disable`).
    #
    #   * `options.default-route-table-association` - Indicates whether
    #     resource attachments are automatically associated with the default
    #     association route table (`enable` \| `disable`).
    #
    #   * `options.default-route-table-propagation` - Indicates whether
    #     resource attachments automatically propagate routes to the default
    #     propagation route table (`enable` \| `disable`).
    #
    #   * `options.dns-support` - Indicates whether DNS support is enabled
    #     (`enable` \| `disable`).
    #
    #   * `options.vpn-ecmp-support` - Indicates whether Equal Cost Multipath
    #     Protocol support is enabled (`enable` \| `disable`).
    #
    #   * `owner-id` - The ID of the Amazon Web Services account that owns the
    #     transit gateway.
    #
    #   * `state` - The state of the transit gateway (`available` \| `deleted`
    #     \| `deleting` \| `modifying` \| `pending`).
    #
    #   * `transit-gateway-id` - The ID of the transit gateway.
    #
    #   * `tag-key `- The key/value combination of a tag assigned to the
    #     resource. Use the tag key in the filter name and the tag value as
    #     the filter value. For example, to find all resources that have a tag
    #     with the key `Owner` and the value `TeamA`, specify `tag:Owner` for
    #     the filter name and `TeamA` for the filter value.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeTransitGatewaysResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTransitGatewaysResult#transit_gateways #transit_gateways} => Array&lt;Types::TransitGateway&gt;
    #   * {Types::DescribeTransitGatewaysResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_transit_gateways({
    #     transit_gateway_ids: ["TransitGatewayId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateways #=> Array
    #   resp.transit_gateways[0].transit_gateway_id #=> String
    #   resp.transit_gateways[0].transit_gateway_arn #=> String
    #   resp.transit_gateways[0].state #=> String, one of "pending", "available", "modifying", "deleting", "deleted"
    #   resp.transit_gateways[0].owner_id #=> String
    #   resp.transit_gateways[0].description #=> String
    #   resp.transit_gateways[0].creation_time #=> Time
    #   resp.transit_gateways[0].options.amazon_side_asn #=> Integer
    #   resp.transit_gateways[0].options.transit_gateway_cidr_blocks #=> Array
    #   resp.transit_gateways[0].options.transit_gateway_cidr_blocks[0] #=> String
    #   resp.transit_gateways[0].options.auto_accept_shared_attachments #=> String, one of "enable", "disable"
    #   resp.transit_gateways[0].options.default_route_table_association #=> String, one of "enable", "disable"
    #   resp.transit_gateways[0].options.association_default_route_table_id #=> String
    #   resp.transit_gateways[0].options.default_route_table_propagation #=> String, one of "enable", "disable"
    #   resp.transit_gateways[0].options.propagation_default_route_table_id #=> String
    #   resp.transit_gateways[0].options.vpn_ecmp_support #=> String, one of "enable", "disable"
    #   resp.transit_gateways[0].options.dns_support #=> String, one of "enable", "disable"
    #   resp.transit_gateways[0].options.security_group_referencing_support #=> String, one of "enable", "disable"
    #   resp.transit_gateways[0].options.multicast_support #=> String, one of "enable", "disable"
    #   resp.transit_gateways[0].tags #=> Array
    #   resp.transit_gateways[0].tags[0].key #=> String
    #   resp.transit_gateways[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTransitGateways AWS API Documentation
    #
    # @overload describe_transit_gateways(params = {})
    # @param [Hash] params ({})
    def describe_transit_gateways(params = {}, options = {})
      req = build_request(:describe_transit_gateways, params)
      req.send_request(options)
    end

    # Describes one or more network interface trunk associations.
    #
    # @option params [Array<String>] :association_ids
    #   The IDs of the associations.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `gre-key` - The ID of a trunk interface association.
    #
    #   * `interface-protocol` - The interface protocol. Valid values are
    #     `VLAN` and `GRE`.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @return [Types::DescribeTrunkInterfaceAssociationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeTrunkInterfaceAssociationsResult#interface_associations #interface_associations} => Array&lt;Types::TrunkInterfaceAssociation&gt;
    #   * {Types::DescribeTrunkInterfaceAssociationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_trunk_interface_associations({
    #     association_ids: ["TrunkInterfaceAssociationId"],
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.interface_associations #=> Array
    #   resp.interface_associations[0].association_id #=> String
    #   resp.interface_associations[0].branch_interface_id #=> String
    #   resp.interface_associations[0].trunk_interface_id #=> String
    #   resp.interface_associations[0].interface_protocol #=> String, one of "VLAN", "GRE"
    #   resp.interface_associations[0].vlan_id #=> Integer
    #   resp.interface_associations[0].gre_key #=> Integer
    #   resp.interface_associations[0].tags #=> Array
    #   resp.interface_associations[0].tags[0].key #=> String
    #   resp.interface_associations[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeTrunkInterfaceAssociations AWS API Documentation
    #
    # @overload describe_trunk_interface_associations(params = {})
    # @param [Hash] params ({})
    def describe_trunk_interface_associations(params = {}, options = {})
      req = build_request(:describe_trunk_interface_associations, params)
      req.send_request(options)
    end

    # Describes the specified Amazon Web Services Verified Access endpoints.
    #
    # @option params [Array<String>] :verified_access_endpoint_ids
    #   The ID of the Verified Access endpoint.
    #
    # @option params [String] :verified_access_instance_id
    #   The ID of the Verified Access instance.
    #
    # @option params [String] :verified_access_group_id
    #   The ID of the Verified Access group.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. Filter names and values are case-sensitive.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeVerifiedAccessEndpointsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVerifiedAccessEndpointsResult#verified_access_endpoints #verified_access_endpoints} => Array&lt;Types::VerifiedAccessEndpoint&gt;
    #   * {Types::DescribeVerifiedAccessEndpointsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_verified_access_endpoints({
    #     verified_access_endpoint_ids: ["VerifiedAccessEndpointId"],
    #     verified_access_instance_id: "VerifiedAccessInstanceId",
    #     verified_access_group_id: "VerifiedAccessGroupId",
    #     max_results: 1,
    #     next_token: "NextToken",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_endpoints #=> Array
    #   resp.verified_access_endpoints[0].verified_access_instance_id #=> String
    #   resp.verified_access_endpoints[0].verified_access_group_id #=> String
    #   resp.verified_access_endpoints[0].verified_access_endpoint_id #=> String
    #   resp.verified_access_endpoints[0].application_domain #=> String
    #   resp.verified_access_endpoints[0].endpoint_type #=> String, one of "load-balancer", "network-interface", "rds", "cidr"
    #   resp.verified_access_endpoints[0].attachment_type #=> String, one of "vpc"
    #   resp.verified_access_endpoints[0].domain_certificate_arn #=> String
    #   resp.verified_access_endpoints[0].endpoint_domain #=> String
    #   resp.verified_access_endpoints[0].device_validation_domain #=> String
    #   resp.verified_access_endpoints[0].security_group_ids #=> Array
    #   resp.verified_access_endpoints[0].security_group_ids[0] #=> String
    #   resp.verified_access_endpoints[0].load_balancer_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoints[0].load_balancer_options.port #=> Integer
    #   resp.verified_access_endpoints[0].load_balancer_options.load_balancer_arn #=> String
    #   resp.verified_access_endpoints[0].load_balancer_options.subnet_ids #=> Array
    #   resp.verified_access_endpoints[0].load_balancer_options.subnet_ids[0] #=> String
    #   resp.verified_access_endpoints[0].load_balancer_options.port_ranges #=> Array
    #   resp.verified_access_endpoints[0].load_balancer_options.port_ranges[0].from_port #=> Integer
    #   resp.verified_access_endpoints[0].load_balancer_options.port_ranges[0].to_port #=> Integer
    #   resp.verified_access_endpoints[0].network_interface_options.network_interface_id #=> String
    #   resp.verified_access_endpoints[0].network_interface_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoints[0].network_interface_options.port #=> Integer
    #   resp.verified_access_endpoints[0].network_interface_options.port_ranges #=> Array
    #   resp.verified_access_endpoints[0].network_interface_options.port_ranges[0].from_port #=> Integer
    #   resp.verified_access_endpoints[0].network_interface_options.port_ranges[0].to_port #=> Integer
    #   resp.verified_access_endpoints[0].status.code #=> String, one of "pending", "active", "updating", "deleting", "deleted"
    #   resp.verified_access_endpoints[0].status.message #=> String
    #   resp.verified_access_endpoints[0].description #=> String
    #   resp.verified_access_endpoints[0].creation_time #=> String
    #   resp.verified_access_endpoints[0].last_updated_time #=> String
    #   resp.verified_access_endpoints[0].deletion_time #=> String
    #   resp.verified_access_endpoints[0].tags #=> Array
    #   resp.verified_access_endpoints[0].tags[0].key #=> String
    #   resp.verified_access_endpoints[0].tags[0].value #=> String
    #   resp.verified_access_endpoints[0].sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.verified_access_endpoints[0].sse_specification.kms_key_arn #=> String
    #   resp.verified_access_endpoints[0].rds_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoints[0].rds_options.port #=> Integer
    #   resp.verified_access_endpoints[0].rds_options.rds_db_instance_arn #=> String
    #   resp.verified_access_endpoints[0].rds_options.rds_db_cluster_arn #=> String
    #   resp.verified_access_endpoints[0].rds_options.rds_db_proxy_arn #=> String
    #   resp.verified_access_endpoints[0].rds_options.rds_endpoint #=> String
    #   resp.verified_access_endpoints[0].rds_options.subnet_ids #=> Array
    #   resp.verified_access_endpoints[0].rds_options.subnet_ids[0] #=> String
    #   resp.verified_access_endpoints[0].cidr_options.cidr #=> String
    #   resp.verified_access_endpoints[0].cidr_options.port_ranges #=> Array
    #   resp.verified_access_endpoints[0].cidr_options.port_ranges[0].from_port #=> Integer
    #   resp.verified_access_endpoints[0].cidr_options.port_ranges[0].to_port #=> Integer
    #   resp.verified_access_endpoints[0].cidr_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoints[0].cidr_options.subnet_ids #=> Array
    #   resp.verified_access_endpoints[0].cidr_options.subnet_ids[0] #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVerifiedAccessEndpoints AWS API Documentation
    #
    # @overload describe_verified_access_endpoints(params = {})
    # @param [Hash] params ({})
    def describe_verified_access_endpoints(params = {}, options = {})
      req = build_request(:describe_verified_access_endpoints, params)
      req.send_request(options)
    end

    # Describes the specified Verified Access groups.
    #
    # @option params [Array<String>] :verified_access_group_ids
    #   The ID of the Verified Access groups.
    #
    # @option params [String] :verified_access_instance_id
    #   The ID of the Verified Access instance.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. Filter names and values are case-sensitive.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeVerifiedAccessGroupsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVerifiedAccessGroupsResult#verified_access_groups #verified_access_groups} => Array&lt;Types::VerifiedAccessGroup&gt;
    #   * {Types::DescribeVerifiedAccessGroupsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_verified_access_groups({
    #     verified_access_group_ids: ["VerifiedAccessGroupId"],
    #     verified_access_instance_id: "VerifiedAccessInstanceId",
    #     max_results: 1,
    #     next_token: "NextToken",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_groups #=> Array
    #   resp.verified_access_groups[0].verified_access_group_id #=> String
    #   resp.verified_access_groups[0].verified_access_instance_id #=> String
    #   resp.verified_access_groups[0].description #=> String
    #   resp.verified_access_groups[0].owner #=> String
    #   resp.verified_access_groups[0].verified_access_group_arn #=> String
    #   resp.verified_access_groups[0].creation_time #=> String
    #   resp.verified_access_groups[0].last_updated_time #=> String
    #   resp.verified_access_groups[0].deletion_time #=> String
    #   resp.verified_access_groups[0].tags #=> Array
    #   resp.verified_access_groups[0].tags[0].key #=> String
    #   resp.verified_access_groups[0].tags[0].value #=> String
    #   resp.verified_access_groups[0].sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.verified_access_groups[0].sse_specification.kms_key_arn #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVerifiedAccessGroups AWS API Documentation
    #
    # @overload describe_verified_access_groups(params = {})
    # @param [Hash] params ({})
    def describe_verified_access_groups(params = {}, options = {})
      req = build_request(:describe_verified_access_groups, params)
      req.send_request(options)
    end

    # Describes the specified Amazon Web Services Verified Access instances.
    #
    # @option params [Array<String>] :verified_access_instance_ids
    #   The IDs of the Verified Access instances.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. Filter names and values are case-sensitive.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeVerifiedAccessInstanceLoggingConfigurationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVerifiedAccessInstanceLoggingConfigurationsResult#logging_configurations #logging_configurations} => Array&lt;Types::VerifiedAccessInstanceLoggingConfiguration&gt;
    #   * {Types::DescribeVerifiedAccessInstanceLoggingConfigurationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_verified_access_instance_logging_configurations({
    #     verified_access_instance_ids: ["VerifiedAccessInstanceId"],
    #     max_results: 1,
    #     next_token: "NextToken",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.logging_configurations #=> Array
    #   resp.logging_configurations[0].verified_access_instance_id #=> String
    #   resp.logging_configurations[0].access_logs.s3.enabled #=> Boolean
    #   resp.logging_configurations[0].access_logs.s3.delivery_status.code #=> String, one of "success", "failed"
    #   resp.logging_configurations[0].access_logs.s3.delivery_status.message #=> String
    #   resp.logging_configurations[0].access_logs.s3.bucket_name #=> String
    #   resp.logging_configurations[0].access_logs.s3.prefix #=> String
    #   resp.logging_configurations[0].access_logs.s3.bucket_owner #=> String
    #   resp.logging_configurations[0].access_logs.cloud_watch_logs.enabled #=> Boolean
    #   resp.logging_configurations[0].access_logs.cloud_watch_logs.delivery_status.code #=> String, one of "success", "failed"
    #   resp.logging_configurations[0].access_logs.cloud_watch_logs.delivery_status.message #=> String
    #   resp.logging_configurations[0].access_logs.cloud_watch_logs.log_group #=> String
    #   resp.logging_configurations[0].access_logs.kinesis_data_firehose.enabled #=> Boolean
    #   resp.logging_configurations[0].access_logs.kinesis_data_firehose.delivery_status.code #=> String, one of "success", "failed"
    #   resp.logging_configurations[0].access_logs.kinesis_data_firehose.delivery_status.message #=> String
    #   resp.logging_configurations[0].access_logs.kinesis_data_firehose.delivery_stream #=> String
    #   resp.logging_configurations[0].access_logs.log_version #=> String
    #   resp.logging_configurations[0].access_logs.include_trust_context #=> Boolean
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVerifiedAccessInstanceLoggingConfigurations AWS API Documentation
    #
    # @overload describe_verified_access_instance_logging_configurations(params = {})
    # @param [Hash] params ({})
    def describe_verified_access_instance_logging_configurations(params = {}, options = {})
      req = build_request(:describe_verified_access_instance_logging_configurations, params)
      req.send_request(options)
    end

    # Describes the specified Amazon Web Services Verified Access instances.
    #
    # @option params [Array<String>] :verified_access_instance_ids
    #   The IDs of the Verified Access instances.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. Filter names and values are case-sensitive.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeVerifiedAccessInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVerifiedAccessInstancesResult#verified_access_instances #verified_access_instances} => Array&lt;Types::VerifiedAccessInstance&gt;
    #   * {Types::DescribeVerifiedAccessInstancesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_verified_access_instances({
    #     verified_access_instance_ids: ["VerifiedAccessInstanceId"],
    #     max_results: 1,
    #     next_token: "NextToken",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_instances #=> Array
    #   resp.verified_access_instances[0].verified_access_instance_id #=> String
    #   resp.verified_access_instances[0].description #=> String
    #   resp.verified_access_instances[0].verified_access_trust_providers #=> Array
    #   resp.verified_access_instances[0].verified_access_trust_providers[0].verified_access_trust_provider_id #=> String
    #   resp.verified_access_instances[0].verified_access_trust_providers[0].description #=> String
    #   resp.verified_access_instances[0].verified_access_trust_providers[0].trust_provider_type #=> String, one of "user", "device"
    #   resp.verified_access_instances[0].verified_access_trust_providers[0].user_trust_provider_type #=> String, one of "iam-identity-center", "oidc"
    #   resp.verified_access_instances[0].verified_access_trust_providers[0].device_trust_provider_type #=> String, one of "jamf", "crowdstrike", "jumpcloud"
    #   resp.verified_access_instances[0].creation_time #=> String
    #   resp.verified_access_instances[0].last_updated_time #=> String
    #   resp.verified_access_instances[0].tags #=> Array
    #   resp.verified_access_instances[0].tags[0].key #=> String
    #   resp.verified_access_instances[0].tags[0].value #=> String
    #   resp.verified_access_instances[0].fips_enabled #=> Boolean
    #   resp.verified_access_instances[0].cidr_endpoints_custom_sub_domain.sub_domain #=> String
    #   resp.verified_access_instances[0].cidr_endpoints_custom_sub_domain.nameservers #=> Array
    #   resp.verified_access_instances[0].cidr_endpoints_custom_sub_domain.nameservers[0] #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVerifiedAccessInstances AWS API Documentation
    #
    # @overload describe_verified_access_instances(params = {})
    # @param [Hash] params ({})
    def describe_verified_access_instances(params = {}, options = {})
      req = build_request(:describe_verified_access_instances, params)
      req.send_request(options)
    end

    # Describes the specified Amazon Web Services Verified Access trust
    # providers.
    #
    # @option params [Array<String>] :verified_access_trust_provider_ids
    #   The IDs of the Verified Access trust providers.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. Filter names and values are case-sensitive.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeVerifiedAccessTrustProvidersResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVerifiedAccessTrustProvidersResult#verified_access_trust_providers #verified_access_trust_providers} => Array&lt;Types::VerifiedAccessTrustProvider&gt;
    #   * {Types::DescribeVerifiedAccessTrustProvidersResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_verified_access_trust_providers({
    #     verified_access_trust_provider_ids: ["VerifiedAccessTrustProviderId"],
    #     max_results: 1,
    #     next_token: "NextToken",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_trust_providers #=> Array
    #   resp.verified_access_trust_providers[0].verified_access_trust_provider_id #=> String
    #   resp.verified_access_trust_providers[0].description #=> String
    #   resp.verified_access_trust_providers[0].trust_provider_type #=> String, one of "user", "device"
    #   resp.verified_access_trust_providers[0].user_trust_provider_type #=> String, one of "iam-identity-center", "oidc"
    #   resp.verified_access_trust_providers[0].device_trust_provider_type #=> String, one of "jamf", "crowdstrike", "jumpcloud"
    #   resp.verified_access_trust_providers[0].oidc_options.issuer #=> String
    #   resp.verified_access_trust_providers[0].oidc_options.authorization_endpoint #=> String
    #   resp.verified_access_trust_providers[0].oidc_options.token_endpoint #=> String
    #   resp.verified_access_trust_providers[0].oidc_options.user_info_endpoint #=> String
    #   resp.verified_access_trust_providers[0].oidc_options.client_id #=> String
    #   resp.verified_access_trust_providers[0].oidc_options.client_secret #=> String
    #   resp.verified_access_trust_providers[0].oidc_options.scope #=> String
    #   resp.verified_access_trust_providers[0].device_options.tenant_id #=> String
    #   resp.verified_access_trust_providers[0].device_options.public_signing_key_url #=> String
    #   resp.verified_access_trust_providers[0].policy_reference_name #=> String
    #   resp.verified_access_trust_providers[0].creation_time #=> String
    #   resp.verified_access_trust_providers[0].last_updated_time #=> String
    #   resp.verified_access_trust_providers[0].tags #=> Array
    #   resp.verified_access_trust_providers[0].tags[0].key #=> String
    #   resp.verified_access_trust_providers[0].tags[0].value #=> String
    #   resp.verified_access_trust_providers[0].sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.verified_access_trust_providers[0].sse_specification.kms_key_arn #=> String
    #   resp.verified_access_trust_providers[0].native_application_oidc_options.public_signing_key_endpoint #=> String
    #   resp.verified_access_trust_providers[0].native_application_oidc_options.issuer #=> String
    #   resp.verified_access_trust_providers[0].native_application_oidc_options.authorization_endpoint #=> String
    #   resp.verified_access_trust_providers[0].native_application_oidc_options.token_endpoint #=> String
    #   resp.verified_access_trust_providers[0].native_application_oidc_options.user_info_endpoint #=> String
    #   resp.verified_access_trust_providers[0].native_application_oidc_options.client_id #=> String
    #   resp.verified_access_trust_providers[0].native_application_oidc_options.scope #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVerifiedAccessTrustProviders AWS API Documentation
    #
    # @overload describe_verified_access_trust_providers(params = {})
    # @param [Hash] params ({})
    def describe_verified_access_trust_providers(params = {}, options = {})
      req = build_request(:describe_verified_access_trust_providers, params)
      req.send_request(options)
    end

    # Describes the specified attribute of the specified volume. You can
    # specify only one attribute at a time.
    #
    # For more information about EBS volumes, see [Amazon EBS volumes][1] in
    # the *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-volumes.html
    #
    # @option params [required, String] :attribute
    #   The attribute of the volume. This parameter is required.
    #
    # @option params [required, String] :volume_id
    #   The ID of the volume.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeVolumeAttributeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVolumeAttributeResult#auto_enable_io #auto_enable_io} => Types::AttributeBooleanValue
    #   * {Types::DescribeVolumeAttributeResult#product_codes #product_codes} => Array&lt;Types::ProductCode&gt;
    #   * {Types::DescribeVolumeAttributeResult#volume_id #volume_id} => String
    #
    #
    # @example Example: To describe a volume attribute
    #
    #   # This example describes the ``autoEnableIo`` attribute of the volume with the ID ``vol-049df61146c4d7901``.
    #
    #   resp = client.describe_volume_attribute({
    #     attribute: "autoEnableIO", 
    #     volume_id: "vol-049df61146c4d7901", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     auto_enable_io: {
    #       value: false, 
    #     }, 
    #     volume_id: "vol-049df61146c4d7901", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_volume_attribute({
    #     attribute: "autoEnableIO", # required, accepts autoEnableIO, productCodes
    #     volume_id: "VolumeId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.auto_enable_io.value #=> Boolean
    #   resp.product_codes #=> Array
    #   resp.product_codes[0].product_code_id #=> String
    #   resp.product_codes[0].product_code_type #=> String, one of "devpay", "marketplace"
    #   resp.volume_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVolumeAttribute AWS API Documentation
    #
    # @overload describe_volume_attribute(params = {})
    # @param [Hash] params ({})
    def describe_volume_attribute(params = {}, options = {})
      req = build_request(:describe_volume_attribute, params)
      req.send_request(options)
    end

    # Describes the status of the specified volumes. Volume status provides
    # the result of the checks performed on your volumes to determine events
    # that can impair the performance of your volumes. The performance of a
    # volume can be affected if an issue occurs on the volume's underlying
    # host. If the volume's underlying host experiences a power outage or
    # system issue, after the system is restored, there could be data
    # inconsistencies on the volume. Volume events notify you if this
    # occurs. Volume actions notify you if any action needs to be taken in
    # response to the event.
    #
    # The `DescribeVolumeStatus` operation provides the following
    # information about the specified volumes:
    #
    # *Status*: Reflects the current status of the volume. The possible
    # values are `ok`, `impaired` , `warning`, or `insufficient-data`. If
    # all checks pass, the overall status of the volume is `ok`. If the
    # check fails, the overall status is `impaired`. If the status is
    # `insufficient-data`, then the checks might still be taking place on
    # your volume at the time. We recommend that you retry the request. For
    # more information about volume status, see [Monitor the status of your
    # volumes][1] in the *Amazon EBS User Guide*.
    #
    # *Events*: Reflect the cause of a volume status and might require you
    # to take action. For example, if your volume returns an `impaired`
    # status, then the volume event might be `potential-data-inconsistency`.
    # This means that your volume has been affected by an issue with the
    # underlying host, has all I/O operations disabled, and might have
    # inconsistent data.
    #
    # *Actions*: Reflect the actions you might have to take in response to
    # an event. For example, if the status of the volume is `impaired` and
    # the volume event shows `potential-data-inconsistency`, then the action
    # shows `enable-volume-io`. This means that you may want to enable the
    # I/O operations for the volume by calling the EnableVolumeIO action and
    # then check the volume for data consistency.
    #
    # Volume status is based on the volume status checks, and does not
    # reflect the volume state. Therefore, volume status does not indicate
    # volumes in the `error` state (for example, when a volume is incapable
    # of accepting I/O.)
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/monitoring-volume-status.html
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Array<String>] :volume_ids
    #   The IDs of the volumes.
    #
    #   Default: Describes all your volumes.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `action.code` - The action code for the event (for example,
    #     `enable-volume-io`).
    #
    #   * `action.description` - A description of the action.
    #
    #   * `action.event-id` - The event ID associated with the action.
    #
    #   * `availability-zone` - The Availability Zone of the instance.
    #
    #   * `event.description` - A description of the event.
    #
    #   * `event.event-id` - The event ID.
    #
    #   * `event.event-type` - The event type (for `io-enabled`: `passed` \|
    #     `failed`; for `io-performance`: `io-performance:degraded` \|
    #     `io-performance:severely-degraded` \| `io-performance:stalled`).
    #
    #   * `event.not-after` - The latest end time for the event.
    #
    #   * `event.not-before` - The earliest start time for the event.
    #
    #   * `volume-status.details-name` - The cause for `volume-status.status`
    #     (`io-enabled` \| `io-performance`).
    #
    #   * `volume-status.details-status` - The status of
    #     `volume-status.details-name` (for `io-enabled`: `passed` \|
    #     `failed`; for `io-performance`: `normal` \| `degraded` \|
    #     `severely-degraded` \| `stalled`).
    #
    #   * `volume-status.status` - The status of the volume (`ok` \|
    #     `impaired` \| `warning` \| `insufficient-data`).
    #
    # @return [Types::DescribeVolumeStatusResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVolumeStatusResult#next_token #next_token} => String
    #   * {Types::DescribeVolumeStatusResult#volume_statuses #volume_statuses} => Array&lt;Types::VolumeStatusItem&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe the status of a single volume
    #
    #   # This example describes the status for the volume ``vol-1234567890abcdef0``.
    #
    #   resp = client.describe_volume_status({
    #     volume_ids: [
    #       "vol-1234567890abcdef0", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     volume_statuses: [
    #       {
    #         actions: [
    #         ], 
    #         availability_zone: "us-east-1a", 
    #         events: [
    #         ], 
    #         volume_id: "vol-1234567890abcdef0", 
    #         volume_status: {
    #           details: [
    #             {
    #               name: "io-enabled", 
    #               status: "passed", 
    #             }, 
    #             {
    #               name: "io-performance", 
    #               status: "not-applicable", 
    #             }, 
    #           ], 
    #           status: "ok", 
    #         }, 
    #       }, 
    #     ], 
    #   }
    #
    # @example Example: To describe the status of impaired volumes
    #
    #   # This example describes the status for all volumes that are impaired. In this example output, there are no impaired
    #   # volumes.
    #
    #   resp = client.describe_volume_status({
    #     filters: [
    #       {
    #         name: "volume-status.status", 
    #         values: [
    #           "impaired", 
    #         ], 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     volume_statuses: [
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_volume_status({
    #     max_results: 1,
    #     next_token: "String",
    #     volume_ids: ["VolumeId"],
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.volume_statuses #=> Array
    #   resp.volume_statuses[0].actions #=> Array
    #   resp.volume_statuses[0].actions[0].code #=> String
    #   resp.volume_statuses[0].actions[0].description #=> String
    #   resp.volume_statuses[0].actions[0].event_id #=> String
    #   resp.volume_statuses[0].actions[0].event_type #=> String
    #   resp.volume_statuses[0].availability_zone #=> String
    #   resp.volume_statuses[0].outpost_arn #=> String
    #   resp.volume_statuses[0].events #=> Array
    #   resp.volume_statuses[0].events[0].description #=> String
    #   resp.volume_statuses[0].events[0].event_id #=> String
    #   resp.volume_statuses[0].events[0].event_type #=> String
    #   resp.volume_statuses[0].events[0].not_after #=> Time
    #   resp.volume_statuses[0].events[0].not_before #=> Time
    #   resp.volume_statuses[0].events[0].instance_id #=> String
    #   resp.volume_statuses[0].volume_id #=> String
    #   resp.volume_statuses[0].volume_status.details #=> Array
    #   resp.volume_statuses[0].volume_status.details[0].name #=> String, one of "io-enabled", "io-performance"
    #   resp.volume_statuses[0].volume_status.details[0].status #=> String
    #   resp.volume_statuses[0].volume_status.status #=> String, one of "ok", "impaired", "insufficient-data"
    #   resp.volume_statuses[0].attachment_statuses #=> Array
    #   resp.volume_statuses[0].attachment_statuses[0].io_performance #=> String
    #   resp.volume_statuses[0].attachment_statuses[0].instance_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVolumeStatus AWS API Documentation
    #
    # @overload describe_volume_status(params = {})
    # @param [Hash] params ({})
    def describe_volume_status(params = {}, options = {})
      req = build_request(:describe_volume_status, params)
      req.send_request(options)
    end

    # Describes the specified EBS volumes or all of your EBS volumes.
    #
    # If you are describing a long list of volumes, we recommend that you
    # paginate the output to make the list more manageable. For more
    # information, see [Pagination][1].
    #
    # For more information about EBS volumes, see [Amazon EBS volumes][2] in
    # the *Amazon EBS User Guide*.
    #
    # We strongly recommend using only paginated requests. Unpaginated
    # requests are susceptible to throttling and timeouts.
    #
    # <note markdown="1"> The order of the elements in the response, including those within
    # nested structures, might vary. Applications should not assume the
    # elements appear in a particular order.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    # [2]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-volumes.html
    #
    # @option params [Array<String>] :volume_ids
    #   The volume IDs. If not specified, then all volumes are included in the
    #   response.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `attachment.attach-time` - The time stamp when the attachment
    #     initiated.
    #
    #   * `attachment.delete-on-termination` - Whether the volume is deleted
    #     on instance termination.
    #
    #   * `attachment.device` - The device name specified in the block device
    #     mapping (for example, `/dev/sda1`).
    #
    #   * `attachment.instance-id` - The ID of the instance the volume is
    #     attached to.
    #
    #   * `attachment.status` - The attachment state (`attaching` \|
    #     `attached` \| `detaching`).
    #
    #   * `availability-zone` - The Availability Zone in which the volume was
    #     created.
    #
    #   * `create-time` - The time stamp when the volume was created.
    #
    #   * `encrypted` - Indicates whether the volume is encrypted (`true` \|
    #     `false`)
    #
    #   * `fast-restored` - Indicates whether the volume was created from a
    #     snapshot that is enabled for fast snapshot restore (`true` \|
    #     `false`).
    #
    #   * `multi-attach-enabled` - Indicates whether the volume is enabled for
    #     Multi-Attach (`true` \| `false`)
    #
    #   * `operator.managed` - A Boolean that indicates whether this is a
    #     managed volume.
    #
    #   * `operator.principal` - The principal that manages the volume. Only
    #     valid for managed volumes, where `managed` is `true`.
    #
    #   * `size` - The size of the volume, in GiB.
    #
    #   * `snapshot-id` - The snapshot from which the volume was created.
    #
    #   * `status` - The state of the volume (`creating` \| `available` \|
    #     `in-use` \| `deleting` \| `deleted` \| `error`).
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `volume-id` - The volume ID.
    #
    #   * `volume-type` - The Amazon EBS volume type (`gp2` \| `gp3` \| `io1`
    #     \| `io2` \| `st1` \| `sc1`\| `standard`)
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @return [Types::DescribeVolumesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVolumesResult#next_token #next_token} => String
    #   * {Types::DescribeVolumesResult#volumes #volumes} => Array&lt;Types::Volume&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe all volumes
    #
    #   # This example describes all of your volumes in the default region.
    #
    #   resp = client.describe_volumes({
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     next_token: "", 
    #     volumes: [
    #       {
    #         attachments: [
    #           {
    #             attach_time: Time.parse("2013-12-18T22:35:00.000Z"), 
    #             delete_on_termination: true, 
    #             device: "/dev/sda1", 
    #             instance_id: "i-1234567890abcdef0", 
    #             state: "attached", 
    #             volume_id: "vol-049df61146c4d7901", 
    #           }, 
    #         ], 
    #         availability_zone: "us-east-1a", 
    #         create_time: Time.parse("2013-12-18T22:35:00.084Z"), 
    #         size: 8, 
    #         snapshot_id: "snap-1234567890abcdef0", 
    #         state: "in-use", 
    #         volume_id: "vol-049df61146c4d7901", 
    #         volume_type: "standard", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Example: To describe volumes that are attached to a specific instance
    #
    #   # This example describes all volumes that are both attached to the instance with the ID i-1234567890abcdef0 and set to
    #   # delete when the instance terminates.
    #
    #   resp = client.describe_volumes({
    #     filters: [
    #       {
    #         name: "attachment.instance-id", 
    #         values: [
    #           "i-1234567890abcdef0", 
    #         ], 
    #       }, 
    #       {
    #         name: "attachment.delete-on-termination", 
    #         values: [
    #           "true", 
    #         ], 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     volumes: [
    #       {
    #         attachments: [
    #           {
    #             attach_time: Time.parse("2013-12-18T22:35:00.000Z"), 
    #             delete_on_termination: true, 
    #             device: "/dev/sda1", 
    #             instance_id: "i-1234567890abcdef0", 
    #             state: "attached", 
    #             volume_id: "vol-049df61146c4d7901", 
    #           }, 
    #         ], 
    #         availability_zone: "us-east-1a", 
    #         create_time: Time.parse("2013-12-18T22:35:00.084Z"), 
    #         size: 8, 
    #         snapshot_id: "snap-1234567890abcdef0", 
    #         state: "in-use", 
    #         volume_id: "vol-049df61146c4d7901", 
    #         volume_type: "standard", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_volumes({
    #     volume_ids: ["VolumeId"],
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.volumes #=> Array
    #   resp.volumes[0].outpost_arn #=> String
    #   resp.volumes[0].iops #=> Integer
    #   resp.volumes[0].tags #=> Array
    #   resp.volumes[0].tags[0].key #=> String
    #   resp.volumes[0].tags[0].value #=> String
    #   resp.volumes[0].volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.volumes[0].fast_restored #=> Boolean
    #   resp.volumes[0].multi_attach_enabled #=> Boolean
    #   resp.volumes[0].throughput #=> Integer
    #   resp.volumes[0].sse_type #=> String, one of "sse-ebs", "sse-kms", "none"
    #   resp.volumes[0].operator.managed #=> Boolean
    #   resp.volumes[0].operator.principal #=> String
    #   resp.volumes[0].volume_id #=> String
    #   resp.volumes[0].size #=> Integer
    #   resp.volumes[0].snapshot_id #=> String
    #   resp.volumes[0].availability_zone #=> String
    #   resp.volumes[0].state #=> String, one of "creating", "available", "in-use", "deleting", "deleted", "error"
    #   resp.volumes[0].create_time #=> Time
    #   resp.volumes[0].attachments #=> Array
    #   resp.volumes[0].attachments[0].delete_on_termination #=> Boolean
    #   resp.volumes[0].attachments[0].associated_resource #=> String
    #   resp.volumes[0].attachments[0].instance_owning_service #=> String
    #   resp.volumes[0].attachments[0].volume_id #=> String
    #   resp.volumes[0].attachments[0].instance_id #=> String
    #   resp.volumes[0].attachments[0].device #=> String
    #   resp.volumes[0].attachments[0].state #=> String, one of "attaching", "attached", "detaching", "detached", "busy"
    #   resp.volumes[0].attachments[0].attach_time #=> Time
    #   resp.volumes[0].encrypted #=> Boolean
    #   resp.volumes[0].kms_key_id #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * volume_available
    #   * volume_deleted
    #   * volume_in_use
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVolumes AWS API Documentation
    #
    # @overload describe_volumes(params = {})
    # @param [Hash] params ({})
    def describe_volumes(params = {}, options = {})
      req = build_request(:describe_volumes, params)
      req.send_request(options)
    end

    # Describes the most recent volume modification request for the
    # specified EBS volumes.
    #
    # For more information, see [ Monitor the progress of volume
    # modifications][1] in the *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/monitoring-volume-modifications.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :volume_ids
    #   The IDs of the volumes.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `modification-state` - The current modification state (modifying \|
    #     optimizing \| completed \| failed).
    #
    #   * `original-iops` - The original IOPS rate of the volume.
    #
    #   * `original-size` - The original size of the volume, in GiB.
    #
    #   * `original-volume-type` - The original volume type of the volume
    #     (standard \| io1 \| io2 \| gp2 \| sc1 \| st1).
    #
    #   * `originalMultiAttachEnabled` - Indicates whether Multi-Attach
    #     support was enabled (true \| false).
    #
    #   * `start-time` - The modification start time.
    #
    #   * `target-iops` - The target IOPS rate of the volume.
    #
    #   * `target-size` - The target size of the volume, in GiB.
    #
    #   * `target-volume-type` - The target volume type of the volume
    #     (standard \| io1 \| io2 \| gp2 \| sc1 \| st1).
    #
    #   * `targetMultiAttachEnabled` - Indicates whether Multi-Attach support
    #     is to be enabled (true \| false).
    #
    #   * `volume-id` - The ID of the volume.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results (up to a limit of 500) to be returned in
    #   a paginated request. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @return [Types::DescribeVolumesModificationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVolumesModificationsResult#next_token #next_token} => String
    #   * {Types::DescribeVolumesModificationsResult#volumes_modifications #volumes_modifications} => Array&lt;Types::VolumeModification&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_volumes_modifications({
    #     dry_run: false,
    #     volume_ids: ["VolumeId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.volumes_modifications #=> Array
    #   resp.volumes_modifications[0].volume_id #=> String
    #   resp.volumes_modifications[0].modification_state #=> String, one of "modifying", "optimizing", "completed", "failed"
    #   resp.volumes_modifications[0].status_message #=> String
    #   resp.volumes_modifications[0].target_size #=> Integer
    #   resp.volumes_modifications[0].target_iops #=> Integer
    #   resp.volumes_modifications[0].target_volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.volumes_modifications[0].target_throughput #=> Integer
    #   resp.volumes_modifications[0].target_multi_attach_enabled #=> Boolean
    #   resp.volumes_modifications[0].original_size #=> Integer
    #   resp.volumes_modifications[0].original_iops #=> Integer
    #   resp.volumes_modifications[0].original_volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.volumes_modifications[0].original_throughput #=> Integer
    #   resp.volumes_modifications[0].original_multi_attach_enabled #=> Boolean
    #   resp.volumes_modifications[0].progress #=> Integer
    #   resp.volumes_modifications[0].start_time #=> Time
    #   resp.volumes_modifications[0].end_time #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVolumesModifications AWS API Documentation
    #
    # @overload describe_volumes_modifications(params = {})
    # @param [Hash] params ({})
    def describe_volumes_modifications(params = {}, options = {})
      req = build_request(:describe_volumes_modifications, params)
      req.send_request(options)
    end

    # Describes the specified attribute of the specified VPC. You can
    # specify only one attribute at a time.
    #
    # @option params [required, String] :attribute
    #   The VPC attribute.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeVpcAttributeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpcAttributeResult#enable_dns_hostnames #enable_dns_hostnames} => Types::AttributeBooleanValue
    #   * {Types::DescribeVpcAttributeResult#enable_dns_support #enable_dns_support} => Types::AttributeBooleanValue
    #   * {Types::DescribeVpcAttributeResult#enable_network_address_usage_metrics #enable_network_address_usage_metrics} => Types::AttributeBooleanValue
    #   * {Types::DescribeVpcAttributeResult#vpc_id #vpc_id} => String
    #
    #
    # @example Example: To describe the enableDnsSupport attribute
    #
    #   # This example describes the enableDnsSupport attribute. This attribute indicates whether DNS resolution is enabled for
    #   # the VPC. If this attribute is true, the Amazon DNS server resolves DNS hostnames for your instances to their
    #   # corresponding IP addresses; otherwise, it does not.
    #
    #   resp = client.describe_vpc_attribute({
    #     attribute: "enableDnsSupport", 
    #     vpc_id: "vpc-a01106c2", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     enable_dns_support: {
    #       value: true, 
    #     }, 
    #     vpc_id: "vpc-a01106c2", 
    #   }
    #
    # @example Example: To describe the enableDnsHostnames attribute
    #
    #   # This example describes the enableDnsHostnames attribute. This attribute indicates whether the instances launched in the
    #   # VPC get DNS hostnames. If this attribute is true, instances in the VPC get DNS hostnames; otherwise, they do not.
    #
    #   resp = client.describe_vpc_attribute({
    #     attribute: "enableDnsHostnames", 
    #     vpc_id: "vpc-a01106c2", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     enable_dns_hostnames: {
    #       value: true, 
    #     }, 
    #     vpc_id: "vpc-a01106c2", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpc_attribute({
    #     attribute: "enableDnsSupport", # required, accepts enableDnsSupport, enableDnsHostnames, enableNetworkAddressUsageMetrics
    #     vpc_id: "VpcId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.enable_dns_hostnames.value #=> Boolean
    #   resp.enable_dns_support.value #=> Boolean
    #   resp.enable_network_address_usage_metrics.value #=> Boolean
    #   resp.vpc_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpcAttribute AWS API Documentation
    #
    # @overload describe_vpc_attribute(params = {})
    # @param [Hash] params ({})
    def describe_vpc_attribute(params = {}, options = {})
      req = build_request(:describe_vpc_attribute, params)
      req.send_request(options)
    end

    # Describe VPC Block Public Access (BPA) exclusions. A VPC BPA exclusion
    # is a mode that can be applied to a single VPC or subnet that exempts
    # it from the account’s BPA mode and will allow bidirectional or
    # egress-only access. You can create BPA exclusions for VPCs and subnets
    # even when BPA is not enabled on the account to ensure that there is no
    # traffic disruption to the exclusions when VPC BPA is turned on. To
    # learn more about VPC BPA, see [Block public access to VPCs and
    # subnets][1] in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/security-vpc-bpa.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   Filters for the request:
    #
    #   * `resource-arn` - The Amazon Resource Name (ARN) of a exclusion.
    #
    #   * `internet-gateway-exclusion-mode` - The mode of a VPC BPA exclusion.
    #     Possible values: `allow-bidirectional | allow-egress`.
    #
    #   * `state` - The state of VPC BPA. Possible values: `create-in-progress
    #     | create-complete | update-in-progress | update-complete |
    #     delete-in-progress | deleted-complete | disable-in-progress |
    #     disable-complete`
    #
    #   * `tag` - The key/value combination of a tag assigned to the resource.
    #     Use the tag key in the filter name and the tag value as the filter
    #     value. For example, to find all resources that have a tag with the
    #     key `Owner` and the value `TeamA`, specify `tag:Owner` for the
    #     filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `tag-value`: The value of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific value,
    #     regardless of the tag key.
    #
    # @option params [Array<String>] :exclusion_ids
    #   IDs of exclusions.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @return [Types::DescribeVpcBlockPublicAccessExclusionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpcBlockPublicAccessExclusionsResult#vpc_block_public_access_exclusions #vpc_block_public_access_exclusions} => Array&lt;Types::VpcBlockPublicAccessExclusion&gt;
    #   * {Types::DescribeVpcBlockPublicAccessExclusionsResult#next_token #next_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpc_block_public_access_exclusions({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     exclusion_ids: ["VpcBlockPublicAccessExclusionId"],
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc_block_public_access_exclusions #=> Array
    #   resp.vpc_block_public_access_exclusions[0].exclusion_id #=> String
    #   resp.vpc_block_public_access_exclusions[0].internet_gateway_exclusion_mode #=> String, one of "allow-bidirectional", "allow-egress"
    #   resp.vpc_block_public_access_exclusions[0].resource_arn #=> String
    #   resp.vpc_block_public_access_exclusions[0].state #=> String, one of "create-in-progress", "create-complete", "create-failed", "update-in-progress", "update-complete", "update-failed", "delete-in-progress", "delete-complete", "disable-in-progress", "disable-complete"
    #   resp.vpc_block_public_access_exclusions[0].reason #=> String
    #   resp.vpc_block_public_access_exclusions[0].creation_timestamp #=> Time
    #   resp.vpc_block_public_access_exclusions[0].last_update_timestamp #=> Time
    #   resp.vpc_block_public_access_exclusions[0].deletion_timestamp #=> Time
    #   resp.vpc_block_public_access_exclusions[0].tags #=> Array
    #   resp.vpc_block_public_access_exclusions[0].tags[0].key #=> String
    #   resp.vpc_block_public_access_exclusions[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpcBlockPublicAccessExclusions AWS API Documentation
    #
    # @overload describe_vpc_block_public_access_exclusions(params = {})
    # @param [Hash] params ({})
    def describe_vpc_block_public_access_exclusions(params = {}, options = {})
      req = build_request(:describe_vpc_block_public_access_exclusions, params)
      req.send_request(options)
    end

    # Describe VPC Block Public Access (BPA) options. VPC Block Public
    # Access (BPA) enables you to block resources in VPCs and subnets that
    # you own in a Region from reaching or being reached from the internet
    # through internet gateways and egress-only internet gateways. To learn
    # more about VPC BPA, see [Block public access to VPCs and subnets][1]
    # in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/security-vpc-bpa.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeVpcBlockPublicAccessOptionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpcBlockPublicAccessOptionsResult#vpc_block_public_access_options #vpc_block_public_access_options} => Types::VpcBlockPublicAccessOptions
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpc_block_public_access_options({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc_block_public_access_options.aws_account_id #=> String
    #   resp.vpc_block_public_access_options.aws_region #=> String
    #   resp.vpc_block_public_access_options.state #=> String, one of "default-state", "update-in-progress", "update-complete"
    #   resp.vpc_block_public_access_options.internet_gateway_block_mode #=> String, one of "off", "block-bidirectional", "block-ingress"
    #   resp.vpc_block_public_access_options.reason #=> String
    #   resp.vpc_block_public_access_options.last_update_timestamp #=> Time
    #   resp.vpc_block_public_access_options.managed_by #=> String, one of "account", "declarative-policy"
    #   resp.vpc_block_public_access_options.exclusions_allowed #=> String, one of "allowed", "not-allowed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpcBlockPublicAccessOptions AWS API Documentation
    #
    # @overload describe_vpc_block_public_access_options(params = {})
    # @param [Hash] params ({})
    def describe_vpc_block_public_access_options(params = {}, options = {})
      req = build_request(:describe_vpc_block_public_access_options, params)
      req.send_request(options)
    end

    # <note markdown="1"> This action is deprecated.
    #
    #  </note>
    #
    # Describes the ClassicLink status of the specified VPCs.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :vpc_ids
    #   The VPCs for which you want to describe the ClassicLink status.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `is-classic-link-enabled` - Whether the VPC is enabled for
    #     ClassicLink (`true` \| `false`).
    #
    #   * `tag` - The key/value combination of a tag assigned to the resource.
    #     Use the tag key in the filter name and the tag value as the filter
    #     value. For example, to find all resources that have a tag with the
    #     key `Owner` and the value `TeamA`, specify `tag:Owner` for the
    #     filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @return [Types::DescribeVpcClassicLinkResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpcClassicLinkResult#vpcs #vpcs} => Array&lt;Types::VpcClassicLink&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpc_classic_link({
    #     dry_run: false,
    #     vpc_ids: ["VpcId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.vpcs #=> Array
    #   resp.vpcs[0].classic_link_enabled #=> Boolean
    #   resp.vpcs[0].tags #=> Array
    #   resp.vpcs[0].tags[0].key #=> String
    #   resp.vpcs[0].tags[0].value #=> String
    #   resp.vpcs[0].vpc_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpcClassicLink AWS API Documentation
    #
    # @overload describe_vpc_classic_link(params = {})
    # @param [Hash] params ({})
    def describe_vpc_classic_link(params = {}, options = {})
      req = build_request(:describe_vpc_classic_link, params)
      req.send_request(options)
    end

    # <note markdown="1"> This action is deprecated.
    #
    #  </note>
    #
    # Describes the ClassicLink DNS support status of one or more VPCs. If
    # enabled, the DNS hostname of a linked EC2-Classic instance resolves to
    # its private IP address when addressed from an instance in the VPC to
    # which it's linked. Similarly, the DNS hostname of an instance in a
    # VPC resolves to its private IP address when addressed from a linked
    # EC2-Classic instance.
    #
    # @option params [Array<String>] :vpc_ids
    #   The IDs of the VPCs.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @return [Types::DescribeVpcClassicLinkDnsSupportResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpcClassicLinkDnsSupportResult#next_token #next_token} => String
    #   * {Types::DescribeVpcClassicLinkDnsSupportResult#vpcs #vpcs} => Array&lt;Types::ClassicLinkDnsSupport&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpc_classic_link_dns_support({
    #     vpc_ids: ["VpcId"],
    #     max_results: 1,
    #     next_token: "DescribeVpcClassicLinkDnsSupportNextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.vpcs #=> Array
    #   resp.vpcs[0].classic_link_dns_supported #=> Boolean
    #   resp.vpcs[0].vpc_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpcClassicLinkDnsSupport AWS API Documentation
    #
    # @overload describe_vpc_classic_link_dns_support(params = {})
    # @param [Hash] params ({})
    def describe_vpc_classic_link_dns_support(params = {}, options = {})
      req = build_request(:describe_vpc_classic_link_dns_support, params)
      req.send_request(options)
    end

    # Describes the VPC resources, VPC endpoint services, Amazon Lattice
    # services, or service networks associated with the VPC endpoint.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :vpc_endpoint_ids
    #   The IDs of the VPC endpoints.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `vpc-endpoint-id` - The ID of the VPC endpoint.
    #
    #   * `associated-resource-accessibility` - The association state. When
    #     the state is `accessible`, it returns `AVAILABLE`. When the state is
    #     `inaccessible`, it returns `PENDING` or `FAILED`.
    #
    #   * `association-id` - The ID of the VPC endpoint association.
    #
    #   * `associated-resource-id` - The ID of the associated resource
    #     configuration.
    #
    #   * `service-network-arn` - The Amazon Resource Name (ARN) of the
    #     associated service network. Only VPC endpoints of type service
    #     network will be returned.
    #
    #   * `resource-configuration-group-arn` - The Amazon Resource Name (ARN)
    #     of the resource configuration of type GROUP.
    #
    #   * `service-network-resource-association-id` - The ID of the
    #     association.
    #
    # @option params [Integer] :max_results
    #   The maximum page size.
    #
    # @option params [String] :next_token
    #   The pagination token.
    #
    # @return [Types::DescribeVpcEndpointAssociationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpcEndpointAssociationsResult#vpc_endpoint_associations #vpc_endpoint_associations} => Array&lt;Types::VpcEndpointAssociation&gt;
    #   * {Types::DescribeVpcEndpointAssociationsResult#next_token #next_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpc_endpoint_associations({
    #     dry_run: false,
    #     vpc_endpoint_ids: ["VpcEndpointId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc_endpoint_associations #=> Array
    #   resp.vpc_endpoint_associations[0].id #=> String
    #   resp.vpc_endpoint_associations[0].vpc_endpoint_id #=> String
    #   resp.vpc_endpoint_associations[0].service_network_arn #=> String
    #   resp.vpc_endpoint_associations[0].service_network_name #=> String
    #   resp.vpc_endpoint_associations[0].associated_resource_accessibility #=> String
    #   resp.vpc_endpoint_associations[0].failure_reason #=> String
    #   resp.vpc_endpoint_associations[0].failure_code #=> String
    #   resp.vpc_endpoint_associations[0].dns_entry.dns_name #=> String
    #   resp.vpc_endpoint_associations[0].dns_entry.hosted_zone_id #=> String
    #   resp.vpc_endpoint_associations[0].private_dns_entry.dns_name #=> String
    #   resp.vpc_endpoint_associations[0].private_dns_entry.hosted_zone_id #=> String
    #   resp.vpc_endpoint_associations[0].associated_resource_arn #=> String
    #   resp.vpc_endpoint_associations[0].resource_configuration_group_arn #=> String
    #   resp.vpc_endpoint_associations[0].tags #=> Array
    #   resp.vpc_endpoint_associations[0].tags[0].key #=> String
    #   resp.vpc_endpoint_associations[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpcEndpointAssociations AWS API Documentation
    #
    # @overload describe_vpc_endpoint_associations(params = {})
    # @param [Hash] params ({})
    def describe_vpc_endpoint_associations(params = {}, options = {})
      req = build_request(:describe_vpc_endpoint_associations, params)
      req.send_request(options)
    end

    # Describes the connection notifications for VPC endpoints and VPC
    # endpoint services.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :connection_notification_id
    #   The ID of the notification.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `connection-notification-arn` - The ARN of the SNS topic for the
    #     notification.
    #
    #   * `connection-notification-id` - The ID of the notification.
    #
    #   * `connection-notification-state` - The state of the notification
    #     (`Enabled` \| `Disabled`).
    #
    #   * `connection-notification-type` - The type of notification (`Topic`).
    #
    #   * `service-id` - The ID of the endpoint service.
    #
    #   * `vpc-endpoint-id` - The ID of the VPC endpoint.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in a single call. To retrieve
    #   the remaining results, make another request with the returned
    #   `NextToken` value.
    #
    # @option params [String] :next_token
    #   The token to request the next page of results.
    #
    # @return [Types::DescribeVpcEndpointConnectionNotificationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpcEndpointConnectionNotificationsResult#connection_notification_set #connection_notification_set} => Array&lt;Types::ConnectionNotification&gt;
    #   * {Types::DescribeVpcEndpointConnectionNotificationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpc_endpoint_connection_notifications({
    #     dry_run: false,
    #     connection_notification_id: "ConnectionNotificationId",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.connection_notification_set #=> Array
    #   resp.connection_notification_set[0].connection_notification_id #=> String
    #   resp.connection_notification_set[0].service_id #=> String
    #   resp.connection_notification_set[0].vpc_endpoint_id #=> String
    #   resp.connection_notification_set[0].connection_notification_type #=> String, one of "Topic"
    #   resp.connection_notification_set[0].connection_notification_arn #=> String
    #   resp.connection_notification_set[0].connection_events #=> Array
    #   resp.connection_notification_set[0].connection_events[0] #=> String
    #   resp.connection_notification_set[0].connection_notification_state #=> String, one of "Enabled", "Disabled"
    #   resp.connection_notification_set[0].service_region #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpcEndpointConnectionNotifications AWS API Documentation
    #
    # @overload describe_vpc_endpoint_connection_notifications(params = {})
    # @param [Hash] params ({})
    def describe_vpc_endpoint_connection_notifications(params = {}, options = {})
      req = build_request(:describe_vpc_endpoint_connection_notifications, params)
      req.send_request(options)
    end

    # Describes the VPC endpoint connections to your VPC endpoint services,
    # including any endpoints that are pending your acceptance.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `ip-address-type` - The IP address type (`ipv4` \| `ipv6`).
    #
    #   * `service-id` - The ID of the service.
    #
    #   * `vpc-endpoint-owner` - The ID of the Amazon Web Services account ID
    #     that owns the endpoint.
    #
    #   * `vpc-endpoint-region` - The Region of the endpoint or `cross-region`
    #     to find endpoints for other Regions.
    #
    #   * `vpc-endpoint-state` - The state of the endpoint
    #     (`pendingAcceptance` \| `pending` \| `available` \| `deleting` \|
    #     `deleted` \| `rejected` \| `failed`).
    #
    #   * `vpc-endpoint-id` - The ID of the endpoint.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return for the request in a single
    #   page. The remaining results of the initial request can be seen by
    #   sending another request with the returned `NextToken` value. This
    #   value can be between 5 and 1,000; if `MaxResults` is given a value
    #   larger than 1,000, only 1,000 results are returned.
    #
    # @option params [String] :next_token
    #   The token to retrieve the next page of results.
    #
    # @return [Types::DescribeVpcEndpointConnectionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpcEndpointConnectionsResult#vpc_endpoint_connections #vpc_endpoint_connections} => Array&lt;Types::VpcEndpointConnection&gt;
    #   * {Types::DescribeVpcEndpointConnectionsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpc_endpoint_connections({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc_endpoint_connections #=> Array
    #   resp.vpc_endpoint_connections[0].service_id #=> String
    #   resp.vpc_endpoint_connections[0].vpc_endpoint_id #=> String
    #   resp.vpc_endpoint_connections[0].vpc_endpoint_owner #=> String
    #   resp.vpc_endpoint_connections[0].vpc_endpoint_state #=> String, one of "PendingAcceptance", "Pending", "Available", "Deleting", "Deleted", "Rejected", "Failed", "Expired", "Partial"
    #   resp.vpc_endpoint_connections[0].creation_timestamp #=> Time
    #   resp.vpc_endpoint_connections[0].dns_entries #=> Array
    #   resp.vpc_endpoint_connections[0].dns_entries[0].dns_name #=> String
    #   resp.vpc_endpoint_connections[0].dns_entries[0].hosted_zone_id #=> String
    #   resp.vpc_endpoint_connections[0].network_load_balancer_arns #=> Array
    #   resp.vpc_endpoint_connections[0].network_load_balancer_arns[0] #=> String
    #   resp.vpc_endpoint_connections[0].gateway_load_balancer_arns #=> Array
    #   resp.vpc_endpoint_connections[0].gateway_load_balancer_arns[0] #=> String
    #   resp.vpc_endpoint_connections[0].ip_address_type #=> String, one of "ipv4", "dualstack", "ipv6"
    #   resp.vpc_endpoint_connections[0].vpc_endpoint_connection_id #=> String
    #   resp.vpc_endpoint_connections[0].tags #=> Array
    #   resp.vpc_endpoint_connections[0].tags[0].key #=> String
    #   resp.vpc_endpoint_connections[0].tags[0].value #=> String
    #   resp.vpc_endpoint_connections[0].vpc_endpoint_region #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpcEndpointConnections AWS API Documentation
    #
    # @overload describe_vpc_endpoint_connections(params = {})
    # @param [Hash] params ({})
    def describe_vpc_endpoint_connections(params = {}, options = {})
      req = build_request(:describe_vpc_endpoint_connections, params)
      req.send_request(options)
    end

    # Describes the VPC endpoint service configurations in your account
    # (your services).
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :service_ids
    #   The IDs of the endpoint services.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `service-name` - The name of the service.
    #
    #   * `service-id` - The ID of the service.
    #
    #   * `service-state` - The state of the service (`Pending` \| `Available`
    #     \| `Deleting` \| `Deleted` \| `Failed`).
    #
    #   * `supported-ip-address-types` - The IP address type (`ipv4` \|
    #     `ipv6`).
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return for the request in a single
    #   page. The remaining results of the initial request can be seen by
    #   sending another request with the returned `NextToken` value. This
    #   value can be between 5 and 1,000; if `MaxResults` is given a value
    #   larger than 1,000, only 1,000 results are returned.
    #
    # @option params [String] :next_token
    #   The token to retrieve the next page of results.
    #
    # @return [Types::DescribeVpcEndpointServiceConfigurationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpcEndpointServiceConfigurationsResult#service_configurations #service_configurations} => Array&lt;Types::ServiceConfiguration&gt;
    #   * {Types::DescribeVpcEndpointServiceConfigurationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpc_endpoint_service_configurations({
    #     dry_run: false,
    #     service_ids: ["VpcEndpointServiceId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.service_configurations #=> Array
    #   resp.service_configurations[0].service_type #=> Array
    #   resp.service_configurations[0].service_type[0].service_type #=> String, one of "Interface", "Gateway", "GatewayLoadBalancer"
    #   resp.service_configurations[0].service_id #=> String
    #   resp.service_configurations[0].service_name #=> String
    #   resp.service_configurations[0].service_state #=> String, one of "Pending", "Available", "Deleting", "Deleted", "Failed"
    #   resp.service_configurations[0].availability_zones #=> Array
    #   resp.service_configurations[0].availability_zones[0] #=> String
    #   resp.service_configurations[0].acceptance_required #=> Boolean
    #   resp.service_configurations[0].manages_vpc_endpoints #=> Boolean
    #   resp.service_configurations[0].network_load_balancer_arns #=> Array
    #   resp.service_configurations[0].network_load_balancer_arns[0] #=> String
    #   resp.service_configurations[0].gateway_load_balancer_arns #=> Array
    #   resp.service_configurations[0].gateway_load_balancer_arns[0] #=> String
    #   resp.service_configurations[0].supported_ip_address_types #=> Array
    #   resp.service_configurations[0].supported_ip_address_types[0] #=> String, one of "ipv4", "ipv6"
    #   resp.service_configurations[0].base_endpoint_dns_names #=> Array
    #   resp.service_configurations[0].base_endpoint_dns_names[0] #=> String
    #   resp.service_configurations[0].private_dns_name #=> String
    #   resp.service_configurations[0].private_dns_name_configuration.state #=> String, one of "pendingVerification", "verified", "failed"
    #   resp.service_configurations[0].private_dns_name_configuration.type #=> String
    #   resp.service_configurations[0].private_dns_name_configuration.value #=> String
    #   resp.service_configurations[0].private_dns_name_configuration.name #=> String
    #   resp.service_configurations[0].payer_responsibility #=> String, one of "ServiceOwner"
    #   resp.service_configurations[0].tags #=> Array
    #   resp.service_configurations[0].tags[0].key #=> String
    #   resp.service_configurations[0].tags[0].value #=> String
    #   resp.service_configurations[0].supported_regions #=> Array
    #   resp.service_configurations[0].supported_regions[0].region #=> String
    #   resp.service_configurations[0].supported_regions[0].service_state #=> String
    #   resp.service_configurations[0].remote_access_enabled #=> Boolean
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpcEndpointServiceConfigurations AWS API Documentation
    #
    # @overload describe_vpc_endpoint_service_configurations(params = {})
    # @param [Hash] params ({})
    def describe_vpc_endpoint_service_configurations(params = {}, options = {})
      req = build_request(:describe_vpc_endpoint_service_configurations, params)
      req.send_request(options)
    end

    # Describes the principals (service consumers) that are permitted to
    # discover your VPC endpoint service.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :service_id
    #   The ID of the service.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `principal` - The ARN of the principal.
    #
    #   * `principal-type` - The principal type (`All` \| `Service` \|
    #     `OrganizationUnit` \| `Account` \| `User` \| `Role`).
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return for the request in a single
    #   page. The remaining results of the initial request can be seen by
    #   sending another request with the returned `NextToken` value. This
    #   value can be between 5 and 1,000; if `MaxResults` is given a value
    #   larger than 1,000, only 1,000 results are returned.
    #
    # @option params [String] :next_token
    #   The token to retrieve the next page of results.
    #
    # @return [Types::DescribeVpcEndpointServicePermissionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpcEndpointServicePermissionsResult#allowed_principals #allowed_principals} => Array&lt;Types::AllowedPrincipal&gt;
    #   * {Types::DescribeVpcEndpointServicePermissionsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpc_endpoint_service_permissions({
    #     dry_run: false,
    #     service_id: "VpcEndpointServiceId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.allowed_principals #=> Array
    #   resp.allowed_principals[0].principal_type #=> String, one of "All", "Service", "OrganizationUnit", "Account", "User", "Role"
    #   resp.allowed_principals[0].principal #=> String
    #   resp.allowed_principals[0].service_permission_id #=> String
    #   resp.allowed_principals[0].tags #=> Array
    #   resp.allowed_principals[0].tags[0].key #=> String
    #   resp.allowed_principals[0].tags[0].value #=> String
    #   resp.allowed_principals[0].service_id #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpcEndpointServicePermissions AWS API Documentation
    #
    # @overload describe_vpc_endpoint_service_permissions(params = {})
    # @param [Hash] params ({})
    def describe_vpc_endpoint_service_permissions(params = {}, options = {})
      req = build_request(:describe_vpc_endpoint_service_permissions, params)
      req.send_request(options)
    end

    # Describes available services to which you can create a VPC endpoint.
    #
    # When the service provider and the consumer have different accounts in
    # multiple Availability Zones, and the consumer views the VPC endpoint
    # service information, the response only includes the common
    # Availability Zones. For example, when the service provider account
    # uses `us-east-1a` and `us-east-1c` and the consumer uses `us-east-1a`
    # and `us-east-1b`, the response includes the VPC endpoint services in
    # the common Availability Zone, `us-east-1a`.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :service_names
    #   The service names.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `owner` - The ID or alias of the Amazon Web Services account that
    #     owns the service.
    #
    #   * `service-name` - The name of the service.
    #
    #   * `service-region` - The Region of the service.
    #
    #   * `service-type` - The type of service (`Interface` \| `Gateway` \|
    #     `GatewayLoadBalancer`).
    #
    #   * `supported-ip-address-types` - The IP address type (`ipv4` \|
    #     `ipv6`).
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. The request
    #   returns a token that you can specify in a subsequent call to get the
    #   next set of results.
    #
    #   Constraint: If the value is greater than 1,000, we return only 1,000
    #   items.
    #
    # @option params [String] :next_token
    #   The token for the next set of items to return. (You received this
    #   token from a prior call.)
    #
    # @option params [Array<String>] :service_regions
    #   The service Regions.
    #
    # @return [Types::DescribeVpcEndpointServicesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpcEndpointServicesResult#service_names #service_names} => Array&lt;String&gt;
    #   * {Types::DescribeVpcEndpointServicesResult#service_details #service_details} => Array&lt;Types::ServiceDetail&gt;
    #   * {Types::DescribeVpcEndpointServicesResult#next_token #next_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpc_endpoint_services({
    #     dry_run: false,
    #     service_names: ["String"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     service_regions: ["String"],
    #   })
    #
    # @example Response structure
    #
    #   resp.service_names #=> Array
    #   resp.service_names[0] #=> String
    #   resp.service_details #=> Array
    #   resp.service_details[0].service_name #=> String
    #   resp.service_details[0].service_id #=> String
    #   resp.service_details[0].service_type #=> Array
    #   resp.service_details[0].service_type[0].service_type #=> String, one of "Interface", "Gateway", "GatewayLoadBalancer"
    #   resp.service_details[0].service_region #=> String
    #   resp.service_details[0].availability_zones #=> Array
    #   resp.service_details[0].availability_zones[0] #=> String
    #   resp.service_details[0].owner #=> String
    #   resp.service_details[0].base_endpoint_dns_names #=> Array
    #   resp.service_details[0].base_endpoint_dns_names[0] #=> String
    #   resp.service_details[0].private_dns_name #=> String
    #   resp.service_details[0].private_dns_names #=> Array
    #   resp.service_details[0].private_dns_names[0].private_dns_name #=> String
    #   resp.service_details[0].vpc_endpoint_policy_supported #=> Boolean
    #   resp.service_details[0].acceptance_required #=> Boolean
    #   resp.service_details[0].manages_vpc_endpoints #=> Boolean
    #   resp.service_details[0].payer_responsibility #=> String, one of "ServiceOwner"
    #   resp.service_details[0].tags #=> Array
    #   resp.service_details[0].tags[0].key #=> String
    #   resp.service_details[0].tags[0].value #=> String
    #   resp.service_details[0].private_dns_name_verification_state #=> String, one of "pendingVerification", "verified", "failed"
    #   resp.service_details[0].supported_ip_address_types #=> Array
    #   resp.service_details[0].supported_ip_address_types[0] #=> String, one of "ipv4", "ipv6"
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpcEndpointServices AWS API Documentation
    #
    # @overload describe_vpc_endpoint_services(params = {})
    # @param [Hash] params ({})
    def describe_vpc_endpoint_services(params = {}, options = {})
      req = build_request(:describe_vpc_endpoint_services, params)
      req.send_request(options)
    end

    # Describes your VPC endpoints. The default is to describe all your VPC
    # endpoints. Alternatively, you can specify specific VPC endpoint IDs or
    # filter the results to include only the VPC endpoints that match
    # specific criteria.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :vpc_endpoint_ids
    #   The IDs of the VPC endpoints.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `ip-address-type` - The IP address type (`ipv4` \| `ipv6`).
    #
    #   * `service-name` - The name of the service.
    #
    #   * `service-region` - The Region of the service.
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `vpc-id` - The ID of the VPC in which the endpoint resides.
    #
    #   * `vpc-endpoint-id` - The ID of the endpoint.
    #
    #   * `vpc-endpoint-state` - The state of the endpoint
    #     (`pendingAcceptance` \| `pending` \| `available` \| `deleting` \|
    #     `deleted` \| `rejected` \| `failed`).
    #
    #   * `vpc-endpoint-type` - The type of VPC endpoint (`Interface` \|
    #     `Gateway` \| `GatewayLoadBalancer` \| `Resource` \|
    #     `ServiceNetwork`).
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. The request
    #   returns a token that you can specify in a subsequent call to get the
    #   next set of results.
    #
    #   Constraint: If the value is greater than 1,000, we return only 1,000
    #   items.
    #
    # @option params [String] :next_token
    #   The token for the next set of items to return. (You received this
    #   token from a prior call.)
    #
    # @return [Types::DescribeVpcEndpointsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpcEndpointsResult#vpc_endpoints #vpc_endpoints} => Array&lt;Types::VpcEndpoint&gt;
    #   * {Types::DescribeVpcEndpointsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpc_endpoints({
    #     dry_run: false,
    #     vpc_endpoint_ids: ["VpcEndpointId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc_endpoints #=> Array
    #   resp.vpc_endpoints[0].vpc_endpoint_id #=> String
    #   resp.vpc_endpoints[0].vpc_endpoint_type #=> String, one of "Interface", "Gateway", "GatewayLoadBalancer", "Resource", "ServiceNetwork"
    #   resp.vpc_endpoints[0].vpc_id #=> String
    #   resp.vpc_endpoints[0].service_name #=> String
    #   resp.vpc_endpoints[0].state #=> String, one of "PendingAcceptance", "Pending", "Available", "Deleting", "Deleted", "Rejected", "Failed", "Expired", "Partial"
    #   resp.vpc_endpoints[0].policy_document #=> String
    #   resp.vpc_endpoints[0].route_table_ids #=> Array
    #   resp.vpc_endpoints[0].route_table_ids[0] #=> String
    #   resp.vpc_endpoints[0].subnet_ids #=> Array
    #   resp.vpc_endpoints[0].subnet_ids[0] #=> String
    #   resp.vpc_endpoints[0].groups #=> Array
    #   resp.vpc_endpoints[0].groups[0].group_id #=> String
    #   resp.vpc_endpoints[0].groups[0].group_name #=> String
    #   resp.vpc_endpoints[0].ip_address_type #=> String, one of "ipv4", "dualstack", "ipv6"
    #   resp.vpc_endpoints[0].dns_options.dns_record_ip_type #=> String, one of "ipv4", "dualstack", "ipv6", "service-defined"
    #   resp.vpc_endpoints[0].dns_options.private_dns_only_for_inbound_resolver_endpoint #=> Boolean
    #   resp.vpc_endpoints[0].private_dns_enabled #=> Boolean
    #   resp.vpc_endpoints[0].requester_managed #=> Boolean
    #   resp.vpc_endpoints[0].network_interface_ids #=> Array
    #   resp.vpc_endpoints[0].network_interface_ids[0] #=> String
    #   resp.vpc_endpoints[0].dns_entries #=> Array
    #   resp.vpc_endpoints[0].dns_entries[0].dns_name #=> String
    #   resp.vpc_endpoints[0].dns_entries[0].hosted_zone_id #=> String
    #   resp.vpc_endpoints[0].creation_timestamp #=> Time
    #   resp.vpc_endpoints[0].tags #=> Array
    #   resp.vpc_endpoints[0].tags[0].key #=> String
    #   resp.vpc_endpoints[0].tags[0].value #=> String
    #   resp.vpc_endpoints[0].owner_id #=> String
    #   resp.vpc_endpoints[0].last_error.message #=> String
    #   resp.vpc_endpoints[0].last_error.code #=> String
    #   resp.vpc_endpoints[0].ipv_4_prefixes #=> Array
    #   resp.vpc_endpoints[0].ipv_4_prefixes[0].subnet_id #=> String
    #   resp.vpc_endpoints[0].ipv_4_prefixes[0].ip_prefixes #=> Array
    #   resp.vpc_endpoints[0].ipv_4_prefixes[0].ip_prefixes[0] #=> String
    #   resp.vpc_endpoints[0].ipv_6_prefixes #=> Array
    #   resp.vpc_endpoints[0].ipv_6_prefixes[0].subnet_id #=> String
    #   resp.vpc_endpoints[0].ipv_6_prefixes[0].ip_prefixes #=> Array
    #   resp.vpc_endpoints[0].ipv_6_prefixes[0].ip_prefixes[0] #=> String
    #   resp.vpc_endpoints[0].failure_reason #=> String
    #   resp.vpc_endpoints[0].service_network_arn #=> String
    #   resp.vpc_endpoints[0].resource_configuration_arn #=> String
    #   resp.vpc_endpoints[0].service_region #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpcEndpoints AWS API Documentation
    #
    # @overload describe_vpc_endpoints(params = {})
    # @param [Hash] params ({})
    def describe_vpc_endpoints(params = {}, options = {})
      req = build_request(:describe_vpc_endpoints, params)
      req.send_request(options)
    end

    # Describes your VPC peering connections. The default is to describe all
    # your VPC peering connections. Alternatively, you can specify specific
    # VPC peering connection IDs or filter the results to include only the
    # VPC peering connections that match specific criteria.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :vpc_peering_connection_ids
    #   The IDs of the VPC peering connections.
    #
    #   Default: Describes all your VPC peering connections.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `accepter-vpc-info.cidr-block` - The IPv4 CIDR block of the accepter
    #     VPC.
    #
    #   * `accepter-vpc-info.owner-id` - The ID of the Amazon Web Services
    #     account that owns the accepter VPC.
    #
    #   * `accepter-vpc-info.vpc-id` - The ID of the accepter VPC.
    #
    #   * `expiration-time` - The expiration date and time for the VPC peering
    #     connection.
    #
    #   * `requester-vpc-info.cidr-block` - The IPv4 CIDR block of the
    #     requester's VPC.
    #
    #   * `requester-vpc-info.owner-id` - The ID of the Amazon Web Services
    #     account that owns the requester VPC.
    #
    #   * `requester-vpc-info.vpc-id` - The ID of the requester VPC.
    #
    #   * `status-code` - The status of the VPC peering connection
    #     (`pending-acceptance` \| `failed` \| `expired` \| `provisioning` \|
    #     `active` \| `deleting` \| `deleted` \| `rejected`).
    #
    #   * `status-message` - A message that provides more information about
    #     the status of the VPC peering connection, if applicable.
    #
    #   * `tag` - The key/value combination of a tag assigned to the resource.
    #     Use the tag key in the filter name and the tag value as the filter
    #     value. For example, to find all resources that have a tag with the
    #     key `Owner` and the value `TeamA`, specify `tag:Owner` for the
    #     filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `vpc-peering-connection-id` - The ID of the VPC peering connection.
    #
    # @return [Types::DescribeVpcPeeringConnectionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpcPeeringConnectionsResult#vpc_peering_connections #vpc_peering_connections} => Array&lt;Types::VpcPeeringConnection&gt;
    #   * {Types::DescribeVpcPeeringConnectionsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpc_peering_connections({
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #     vpc_peering_connection_ids: ["VpcPeeringConnectionId"],
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc_peering_connections #=> Array
    #   resp.vpc_peering_connections[0].accepter_vpc_info.cidr_block #=> String
    #   resp.vpc_peering_connections[0].accepter_vpc_info.ipv_6_cidr_block_set #=> Array
    #   resp.vpc_peering_connections[0].accepter_vpc_info.ipv_6_cidr_block_set[0].ipv_6_cidr_block #=> String
    #   resp.vpc_peering_connections[0].accepter_vpc_info.cidr_block_set #=> Array
    #   resp.vpc_peering_connections[0].accepter_vpc_info.cidr_block_set[0].cidr_block #=> String
    #   resp.vpc_peering_connections[0].accepter_vpc_info.owner_id #=> String
    #   resp.vpc_peering_connections[0].accepter_vpc_info.peering_options.allow_dns_resolution_from_remote_vpc #=> Boolean
    #   resp.vpc_peering_connections[0].accepter_vpc_info.peering_options.allow_egress_from_local_classic_link_to_remote_vpc #=> Boolean
    #   resp.vpc_peering_connections[0].accepter_vpc_info.peering_options.allow_egress_from_local_vpc_to_remote_classic_link #=> Boolean
    #   resp.vpc_peering_connections[0].accepter_vpc_info.vpc_id #=> String
    #   resp.vpc_peering_connections[0].accepter_vpc_info.region #=> String
    #   resp.vpc_peering_connections[0].expiration_time #=> Time
    #   resp.vpc_peering_connections[0].requester_vpc_info.cidr_block #=> String
    #   resp.vpc_peering_connections[0].requester_vpc_info.ipv_6_cidr_block_set #=> Array
    #   resp.vpc_peering_connections[0].requester_vpc_info.ipv_6_cidr_block_set[0].ipv_6_cidr_block #=> String
    #   resp.vpc_peering_connections[0].requester_vpc_info.cidr_block_set #=> Array
    #   resp.vpc_peering_connections[0].requester_vpc_info.cidr_block_set[0].cidr_block #=> String
    #   resp.vpc_peering_connections[0].requester_vpc_info.owner_id #=> String
    #   resp.vpc_peering_connections[0].requester_vpc_info.peering_options.allow_dns_resolution_from_remote_vpc #=> Boolean
    #   resp.vpc_peering_connections[0].requester_vpc_info.peering_options.allow_egress_from_local_classic_link_to_remote_vpc #=> Boolean
    #   resp.vpc_peering_connections[0].requester_vpc_info.peering_options.allow_egress_from_local_vpc_to_remote_classic_link #=> Boolean
    #   resp.vpc_peering_connections[0].requester_vpc_info.vpc_id #=> String
    #   resp.vpc_peering_connections[0].requester_vpc_info.region #=> String
    #   resp.vpc_peering_connections[0].status.code #=> String, one of "initiating-request", "pending-acceptance", "active", "deleted", "rejected", "failed", "expired", "provisioning", "deleting"
    #   resp.vpc_peering_connections[0].status.message #=> String
    #   resp.vpc_peering_connections[0].tags #=> Array
    #   resp.vpc_peering_connections[0].tags[0].key #=> String
    #   resp.vpc_peering_connections[0].tags[0].value #=> String
    #   resp.vpc_peering_connections[0].vpc_peering_connection_id #=> String
    #   resp.next_token #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * vpc_peering_connection_deleted
    #   * vpc_peering_connection_exists
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpcPeeringConnections AWS API Documentation
    #
    # @overload describe_vpc_peering_connections(params = {})
    # @param [Hash] params ({})
    def describe_vpc_peering_connections(params = {}, options = {})
      req = build_request(:describe_vpc_peering_connections, params)
      req.send_request(options)
    end

    # Describes your VPCs. The default is to describe all your VPCs.
    # Alternatively, you can specify specific VPC IDs or filter the results
    # to include only the VPCs that match specific criteria.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters.
    #
    #   * `cidr` - The primary IPv4 CIDR block of the VPC. The CIDR block you
    #     specify must exactly match the VPC's CIDR block for information to
    #     be returned for the VPC. Must contain the slash followed by one or
    #     two digits (for example, `/28`).
    #
    #   * `cidr-block-association.cidr-block` - An IPv4 CIDR block associated
    #     with the VPC.
    #
    #   * `cidr-block-association.association-id` - The association ID for an
    #     IPv4 CIDR block associated with the VPC.
    #
    #   * `cidr-block-association.state` - The state of an IPv4 CIDR block
    #     associated with the VPC.
    #
    #   * `dhcp-options-id` - The ID of a set of DHCP options.
    #
    #   * `ipv6-cidr-block-association.ipv6-cidr-block` - An IPv6 CIDR block
    #     associated with the VPC.
    #
    #   * `ipv6-cidr-block-association.ipv6-pool` - The ID of the IPv6 address
    #     pool from which the IPv6 CIDR block is allocated.
    #
    #   * `ipv6-cidr-block-association.association-id` - The association ID
    #     for an IPv6 CIDR block associated with the VPC.
    #
    #   * `ipv6-cidr-block-association.state` - The state of an IPv6 CIDR
    #     block associated with the VPC.
    #
    #   * `is-default` - Indicates whether the VPC is the default VPC.
    #
    #   * `owner-id` - The ID of the Amazon Web Services account that owns the
    #     VPC.
    #
    #   * `state` - The state of the VPC (`pending` \| `available`).
    #
    #   * `tag` - The key/value combination of a tag assigned to the resource.
    #     Use the tag key in the filter name and the tag value as the filter
    #     value. For example, to find all resources that have a tag with the
    #     key `Owner` and the value `TeamA`, specify `tag:Owner` for the
    #     filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `vpc-id` - The ID of the VPC.
    #
    # @option params [Array<String>] :vpc_ids
    #   The IDs of the VPCs.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeVpcsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpcsResult#next_token #next_token} => String
    #   * {Types::DescribeVpcsResult#vpcs #vpcs} => Array&lt;Types::Vpc&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    #
    # @example Example: To describe a VPC
    #
    #   # This example describes the specified VPC.
    #
    #   resp = client.describe_vpcs({
    #     vpc_ids: [
    #       "vpc-a01106c2", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     vpcs: [
    #       {
    #         cidr_block: "10.0.0.0/16", 
    #         dhcp_options_id: "dopt-7a8b9c2d", 
    #         instance_tenancy: "default", 
    #         is_default: false, 
    #         state: "available", 
    #         tags: [
    #           {
    #             key: "Name", 
    #             value: "MyVPC", 
    #           }, 
    #         ], 
    #         vpc_id: "vpc-a01106c2", 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpcs({
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     vpc_ids: ["VpcId"],
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.vpcs #=> Array
    #   resp.vpcs[0].owner_id #=> String
    #   resp.vpcs[0].instance_tenancy #=> String, one of "default", "dedicated", "host"
    #   resp.vpcs[0].ipv_6_cidr_block_association_set #=> Array
    #   resp.vpcs[0].ipv_6_cidr_block_association_set[0].association_id #=> String
    #   resp.vpcs[0].ipv_6_cidr_block_association_set[0].ipv_6_cidr_block #=> String
    #   resp.vpcs[0].ipv_6_cidr_block_association_set[0].ipv_6_cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.vpcs[0].ipv_6_cidr_block_association_set[0].ipv_6_cidr_block_state.status_message #=> String
    #   resp.vpcs[0].ipv_6_cidr_block_association_set[0].network_border_group #=> String
    #   resp.vpcs[0].ipv_6_cidr_block_association_set[0].ipv_6_pool #=> String
    #   resp.vpcs[0].ipv_6_cidr_block_association_set[0].ipv_6_address_attribute #=> String, one of "public", "private"
    #   resp.vpcs[0].ipv_6_cidr_block_association_set[0].ip_source #=> String, one of "amazon", "byoip", "none"
    #   resp.vpcs[0].cidr_block_association_set #=> Array
    #   resp.vpcs[0].cidr_block_association_set[0].association_id #=> String
    #   resp.vpcs[0].cidr_block_association_set[0].cidr_block #=> String
    #   resp.vpcs[0].cidr_block_association_set[0].cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.vpcs[0].cidr_block_association_set[0].cidr_block_state.status_message #=> String
    #   resp.vpcs[0].is_default #=> Boolean
    #   resp.vpcs[0].encryption_control.vpc_id #=> String
    #   resp.vpcs[0].encryption_control.vpc_encryption_control_id #=> String
    #   resp.vpcs[0].encryption_control.mode #=> String, one of "monitor", "enforce"
    #   resp.vpcs[0].encryption_control.state #=> String, one of "enforce-in-progress", "monitor-in-progress", "enforce-failed", "monitor-failed", "deleting", "deleted", "available"
    #   resp.vpcs[0].encryption_control.state_message #=> String
    #   resp.vpcs[0].encryption_control.resource_exclusions.internet_gateway.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpcs[0].encryption_control.resource_exclusions.internet_gateway.state_message #=> String
    #   resp.vpcs[0].encryption_control.resource_exclusions.egress_only_internet_gateway.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpcs[0].encryption_control.resource_exclusions.egress_only_internet_gateway.state_message #=> String
    #   resp.vpcs[0].encryption_control.resource_exclusions.nat_gateway.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpcs[0].encryption_control.resource_exclusions.nat_gateway.state_message #=> String
    #   resp.vpcs[0].encryption_control.resource_exclusions.virtual_private_gateway.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpcs[0].encryption_control.resource_exclusions.virtual_private_gateway.state_message #=> String
    #   resp.vpcs[0].encryption_control.resource_exclusions.vpc_peering.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.vpcs[0].encryption_control.resource_exclusions.vpc_peering.state_message #=> String
    #   resp.vpcs[0].encryption_control.tags #=> Array
    #   resp.vpcs[0].encryption_control.tags[0].key #=> String
    #   resp.vpcs[0].encryption_control.tags[0].value #=> String
    #   resp.vpcs[0].tags #=> Array
    #   resp.vpcs[0].tags[0].key #=> String
    #   resp.vpcs[0].tags[0].value #=> String
    #   resp.vpcs[0].block_public_access_states.internet_gateway_block_mode #=> String, one of "off", "block-bidirectional", "block-ingress"
    #   resp.vpcs[0].vpc_id #=> String
    #   resp.vpcs[0].state #=> String, one of "pending", "available"
    #   resp.vpcs[0].cidr_block #=> String
    #   resp.vpcs[0].dhcp_options_id #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * vpc_available
    #   * vpc_exists
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpcs AWS API Documentation
    #
    # @overload describe_vpcs(params = {})
    # @param [Hash] params ({})
    def describe_vpcs(params = {}, options = {})
      req = build_request(:describe_vpcs, params)
      req.send_request(options)
    end

    # Describes one or more of your VPN connections.
    #
    # For more information, see [Amazon Web Services Site-to-Site VPN][1] in
    # the *Amazon Web Services Site-to-Site VPN User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `customer-gateway-configuration` - The configuration information for
    #     the customer gateway.
    #
    #   * `customer-gateway-id` - The ID of a customer gateway associated with
    #     the VPN connection.
    #
    #   * `state` - The state of the VPN connection (`pending` \| `available`
    #     \| `deleting` \| `deleted`).
    #
    #   * `option.static-routes-only` - Indicates whether the connection has
    #     static routes only. Used for devices that do not support Border
    #     Gateway Protocol (BGP).
    #
    #   * `route.destination-cidr-block` - The destination CIDR block. This
    #     corresponds to the subnet used in a customer data center.
    #
    #   * `bgp-asn` - The BGP Autonomous System Number (ASN) associated with a
    #     BGP device.
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `type` - The type of VPN connection. Currently the only supported
    #     type is `ipsec.1`.
    #
    #   * `vpn-connection-id` - The ID of the VPN connection.
    #
    #   * `vpn-gateway-id` - The ID of a virtual private gateway associated
    #     with the VPN connection.
    #
    #   * `transit-gateway-id` - The ID of a transit gateway associated with
    #     the VPN connection.
    #
    # @option params [Array<String>] :vpn_connection_ids
    #   One or more VPN connection IDs.
    #
    #   Default: Describes your VPN connections.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeVpnConnectionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpnConnectionsResult#vpn_connections #vpn_connections} => Array&lt;Types::VpnConnection&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpn_connections({
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     vpn_connection_ids: ["VpnConnectionId"],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpn_connections #=> Array
    #   resp.vpn_connections[0].category #=> String
    #   resp.vpn_connections[0].transit_gateway_id #=> String
    #   resp.vpn_connections[0].core_network_arn #=> String
    #   resp.vpn_connections[0].core_network_attachment_arn #=> String
    #   resp.vpn_connections[0].gateway_association_state #=> String, one of "associated", "not-associated", "associating", "disassociating"
    #   resp.vpn_connections[0].options.enable_acceleration #=> Boolean
    #   resp.vpn_connections[0].options.static_routes_only #=> Boolean
    #   resp.vpn_connections[0].options.local_ipv_4_network_cidr #=> String
    #   resp.vpn_connections[0].options.remote_ipv_4_network_cidr #=> String
    #   resp.vpn_connections[0].options.local_ipv_6_network_cidr #=> String
    #   resp.vpn_connections[0].options.remote_ipv_6_network_cidr #=> String
    #   resp.vpn_connections[0].options.outside_ip_address_type #=> String
    #   resp.vpn_connections[0].options.transport_transit_gateway_attachment_id #=> String
    #   resp.vpn_connections[0].options.tunnel_inside_ip_version #=> String, one of "ipv4", "ipv6"
    #   resp.vpn_connections[0].options.tunnel_options #=> Array
    #   resp.vpn_connections[0].options.tunnel_options[0].outside_ip_address #=> String
    #   resp.vpn_connections[0].options.tunnel_options[0].tunnel_inside_cidr #=> String
    #   resp.vpn_connections[0].options.tunnel_options[0].tunnel_inside_ipv_6_cidr #=> String
    #   resp.vpn_connections[0].options.tunnel_options[0].pre_shared_key #=> String
    #   resp.vpn_connections[0].options.tunnel_options[0].phase_1_lifetime_seconds #=> Integer
    #   resp.vpn_connections[0].options.tunnel_options[0].phase_2_lifetime_seconds #=> Integer
    #   resp.vpn_connections[0].options.tunnel_options[0].rekey_margin_time_seconds #=> Integer
    #   resp.vpn_connections[0].options.tunnel_options[0].rekey_fuzz_percentage #=> Integer
    #   resp.vpn_connections[0].options.tunnel_options[0].replay_window_size #=> Integer
    #   resp.vpn_connections[0].options.tunnel_options[0].dpd_timeout_seconds #=> Integer
    #   resp.vpn_connections[0].options.tunnel_options[0].dpd_timeout_action #=> String
    #   resp.vpn_connections[0].options.tunnel_options[0].phase_1_encryption_algorithms #=> Array
    #   resp.vpn_connections[0].options.tunnel_options[0].phase_1_encryption_algorithms[0].value #=> String
    #   resp.vpn_connections[0].options.tunnel_options[0].phase_2_encryption_algorithms #=> Array
    #   resp.vpn_connections[0].options.tunnel_options[0].phase_2_encryption_algorithms[0].value #=> String
    #   resp.vpn_connections[0].options.tunnel_options[0].phase_1_integrity_algorithms #=> Array
    #   resp.vpn_connections[0].options.tunnel_options[0].phase_1_integrity_algorithms[0].value #=> String
    #   resp.vpn_connections[0].options.tunnel_options[0].phase_2_integrity_algorithms #=> Array
    #   resp.vpn_connections[0].options.tunnel_options[0].phase_2_integrity_algorithms[0].value #=> String
    #   resp.vpn_connections[0].options.tunnel_options[0].phase_1_dh_group_numbers #=> Array
    #   resp.vpn_connections[0].options.tunnel_options[0].phase_1_dh_group_numbers[0].value #=> Integer
    #   resp.vpn_connections[0].options.tunnel_options[0].phase_2_dh_group_numbers #=> Array
    #   resp.vpn_connections[0].options.tunnel_options[0].phase_2_dh_group_numbers[0].value #=> Integer
    #   resp.vpn_connections[0].options.tunnel_options[0].ike_versions #=> Array
    #   resp.vpn_connections[0].options.tunnel_options[0].ike_versions[0].value #=> String
    #   resp.vpn_connections[0].options.tunnel_options[0].startup_action #=> String
    #   resp.vpn_connections[0].options.tunnel_options[0].log_options.cloud_watch_log_options.log_enabled #=> Boolean
    #   resp.vpn_connections[0].options.tunnel_options[0].log_options.cloud_watch_log_options.log_group_arn #=> String
    #   resp.vpn_connections[0].options.tunnel_options[0].log_options.cloud_watch_log_options.log_output_format #=> String
    #   resp.vpn_connections[0].options.tunnel_options[0].enable_tunnel_lifecycle_control #=> Boolean
    #   resp.vpn_connections[0].routes #=> Array
    #   resp.vpn_connections[0].routes[0].destination_cidr_block #=> String
    #   resp.vpn_connections[0].routes[0].source #=> String, one of "Static"
    #   resp.vpn_connections[0].routes[0].state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.vpn_connections[0].tags #=> Array
    #   resp.vpn_connections[0].tags[0].key #=> String
    #   resp.vpn_connections[0].tags[0].value #=> String
    #   resp.vpn_connections[0].vgw_telemetry #=> Array
    #   resp.vpn_connections[0].vgw_telemetry[0].accepted_route_count #=> Integer
    #   resp.vpn_connections[0].vgw_telemetry[0].last_status_change #=> Time
    #   resp.vpn_connections[0].vgw_telemetry[0].outside_ip_address #=> String
    #   resp.vpn_connections[0].vgw_telemetry[0].status #=> String, one of "UP", "DOWN"
    #   resp.vpn_connections[0].vgw_telemetry[0].status_message #=> String
    #   resp.vpn_connections[0].vgw_telemetry[0].certificate_arn #=> String
    #   resp.vpn_connections[0].vpn_connection_id #=> String
    #   resp.vpn_connections[0].state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.vpn_connections[0].customer_gateway_configuration #=> String
    #   resp.vpn_connections[0].type #=> String, one of "ipsec.1"
    #   resp.vpn_connections[0].customer_gateway_id #=> String
    #   resp.vpn_connections[0].vpn_gateway_id #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * vpn_connection_available
    #   * vpn_connection_deleted
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpnConnections AWS API Documentation
    #
    # @overload describe_vpn_connections(params = {})
    # @param [Hash] params ({})
    def describe_vpn_connections(params = {}, options = {})
      req = build_request(:describe_vpn_connections, params)
      req.send_request(options)
    end

    # Describes one or more of your virtual private gateways.
    #
    # For more information, see [Amazon Web Services Site-to-Site VPN][1] in
    # the *Amazon Web Services Site-to-Site VPN User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpn/latest/s2svpn/VPC_VPN.html
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `amazon-side-asn` - The Autonomous System Number (ASN) for the
    #     Amazon side of the gateway.
    #
    #   * `attachment.state` - The current state of the attachment between the
    #     gateway and the VPC (`attaching` \| `attached` \| `detaching` \|
    #     `detached`).
    #
    #   * `attachment.vpc-id` - The ID of an attached VPC.
    #
    #   * `availability-zone` - The Availability Zone for the virtual private
    #     gateway (if applicable).
    #
    #   * `state` - The state of the virtual private gateway (`pending` \|
    #     `available` \| `deleting` \| `deleted`).
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    #   * `type` - The type of virtual private gateway. Currently the only
    #     supported type is `ipsec.1`.
    #
    #   * `vpn-gateway-id` - The ID of the virtual private gateway.
    #
    # @option params [Array<String>] :vpn_gateway_ids
    #   One or more virtual private gateway IDs.
    #
    #   Default: Describes all your virtual private gateways.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DescribeVpnGatewaysResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DescribeVpnGatewaysResult#vpn_gateways #vpn_gateways} => Array&lt;Types::VpnGateway&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.describe_vpn_gateways({
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     vpn_gateway_ids: ["VpnGatewayId"],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpn_gateways #=> Array
    #   resp.vpn_gateways[0].amazon_side_asn #=> Integer
    #   resp.vpn_gateways[0].tags #=> Array
    #   resp.vpn_gateways[0].tags[0].key #=> String
    #   resp.vpn_gateways[0].tags[0].value #=> String
    #   resp.vpn_gateways[0].vpn_gateway_id #=> String
    #   resp.vpn_gateways[0].state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.vpn_gateways[0].type #=> String, one of "ipsec.1"
    #   resp.vpn_gateways[0].availability_zone #=> String
    #   resp.vpn_gateways[0].vpc_attachments #=> Array
    #   resp.vpn_gateways[0].vpc_attachments[0].vpc_id #=> String
    #   resp.vpn_gateways[0].vpc_attachments[0].state #=> String, one of "attaching", "attached", "detaching", "detached"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DescribeVpnGateways AWS API Documentation
    #
    # @overload describe_vpn_gateways(params = {})
    # @param [Hash] params ({})
    def describe_vpn_gateways(params = {}, options = {})
      req = build_request(:describe_vpn_gateways, params)
      req.send_request(options)
    end

    # <note markdown="1"> This action is deprecated.
    #
    #  </note>
    #
    # Unlinks (detaches) a linked EC2-Classic instance from a VPC. After the
    # instance has been unlinked, the VPC security groups are no longer
    # associated with it. An instance is automatically unlinked from a VPC
    # when it's stopped.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance to unlink from the VPC.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC to which the instance is linked.
    #
    # @return [Types::DetachClassicLinkVpcResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DetachClassicLinkVpcResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.detach_classic_link_vpc({
    #     dry_run: false,
    #     instance_id: "InstanceId", # required
    #     vpc_id: "VpcId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DetachClassicLinkVpc AWS API Documentation
    #
    # @overload detach_classic_link_vpc(params = {})
    # @param [Hash] params ({})
    def detach_classic_link_vpc(params = {}, options = {})
      req = build_request(:detach_classic_link_vpc, params)
      req.send_request(options)
    end

    # Detaches an internet gateway from a VPC, disabling connectivity
    # between the internet and the VPC. The VPC must not contain any running
    # instances with Elastic IP addresses or public IPv4 addresses.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :internet_gateway_id
    #   The ID of the internet gateway.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To detach an Internet gateway from a VPC
    #
    #   # This example detaches the specified Internet gateway from the specified VPC.
    #
    #   resp = client.detach_internet_gateway({
    #     internet_gateway_id: "igw-c0a643a9", 
    #     vpc_id: "vpc-a01106c2", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.detach_internet_gateway({
    #     dry_run: false,
    #     internet_gateway_id: "InternetGatewayId", # required
    #     vpc_id: "VpcId", # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DetachInternetGateway AWS API Documentation
    #
    # @overload detach_internet_gateway(params = {})
    # @param [Hash] params ({})
    def detach_internet_gateway(params = {}, options = {})
      req = build_request(:detach_internet_gateway, params)
      req.send_request(options)
    end

    # Detaches a network interface from an instance.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :attachment_id
    #   The ID of the attachment.
    #
    # @option params [Boolean] :force
    #   Specifies whether to force a detachment.
    #
    #   <note markdown="1"> * Use the `Force` parameter only as a last resort to detach a network
    #     interface from a failed instance.
    #
    #   * If you use the `Force` parameter to detach a network interface, you
    #     might not be able to attach a different network interface to the
    #     same index on the instance without first stopping and starting the
    #     instance.
    #
    #   * If you force the detachment of a network interface, the [instance
    #     metadata][1] might not get updated. This means that the attributes
    #     associated with the detached network interface might still be
    #     visible. The instance metadata will get updated when you stop and
    #     start the instance.
    #
    #    </note>
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To detach a network interface from an instance
    #
    #   # This example detaches the specified network interface from its attached instance.
    #
    #   resp = client.detach_network_interface({
    #     attachment_id: "eni-attach-66c4350a", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.detach_network_interface({
    #     dry_run: false,
    #     attachment_id: "NetworkInterfaceAttachmentId", # required
    #     force: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DetachNetworkInterface AWS API Documentation
    #
    # @overload detach_network_interface(params = {})
    # @param [Hash] params ({})
    def detach_network_interface(params = {}, options = {})
      req = build_request(:detach_network_interface, params)
      req.send_request(options)
    end

    # Detaches the specified Amazon Web Services Verified Access trust
    # provider from the specified Amazon Web Services Verified Access
    # instance.
    #
    # @option params [required, String] :verified_access_instance_id
    #   The ID of the Verified Access instance.
    #
    # @option params [required, String] :verified_access_trust_provider_id
    #   The ID of the Verified Access trust provider.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DetachVerifiedAccessTrustProviderResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DetachVerifiedAccessTrustProviderResult#verified_access_trust_provider #verified_access_trust_provider} => Types::VerifiedAccessTrustProvider
    #   * {Types::DetachVerifiedAccessTrustProviderResult#verified_access_instance #verified_access_instance} => Types::VerifiedAccessInstance
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.detach_verified_access_trust_provider({
    #     verified_access_instance_id: "VerifiedAccessInstanceId", # required
    #     verified_access_trust_provider_id: "VerifiedAccessTrustProviderId", # required
    #     client_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_trust_provider.verified_access_trust_provider_id #=> String
    #   resp.verified_access_trust_provider.description #=> String
    #   resp.verified_access_trust_provider.trust_provider_type #=> String, one of "user", "device"
    #   resp.verified_access_trust_provider.user_trust_provider_type #=> String, one of "iam-identity-center", "oidc"
    #   resp.verified_access_trust_provider.device_trust_provider_type #=> String, one of "jamf", "crowdstrike", "jumpcloud"
    #   resp.verified_access_trust_provider.oidc_options.issuer #=> String
    #   resp.verified_access_trust_provider.oidc_options.authorization_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.token_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.user_info_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.client_id #=> String
    #   resp.verified_access_trust_provider.oidc_options.client_secret #=> String
    #   resp.verified_access_trust_provider.oidc_options.scope #=> String
    #   resp.verified_access_trust_provider.device_options.tenant_id #=> String
    #   resp.verified_access_trust_provider.device_options.public_signing_key_url #=> String
    #   resp.verified_access_trust_provider.policy_reference_name #=> String
    #   resp.verified_access_trust_provider.creation_time #=> String
    #   resp.verified_access_trust_provider.last_updated_time #=> String
    #   resp.verified_access_trust_provider.tags #=> Array
    #   resp.verified_access_trust_provider.tags[0].key #=> String
    #   resp.verified_access_trust_provider.tags[0].value #=> String
    #   resp.verified_access_trust_provider.sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.verified_access_trust_provider.sse_specification.kms_key_arn #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.public_signing_key_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.issuer #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.authorization_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.token_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.user_info_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.client_id #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.scope #=> String
    #   resp.verified_access_instance.verified_access_instance_id #=> String
    #   resp.verified_access_instance.description #=> String
    #   resp.verified_access_instance.verified_access_trust_providers #=> Array
    #   resp.verified_access_instance.verified_access_trust_providers[0].verified_access_trust_provider_id #=> String
    #   resp.verified_access_instance.verified_access_trust_providers[0].description #=> String
    #   resp.verified_access_instance.verified_access_trust_providers[0].trust_provider_type #=> String, one of "user", "device"
    #   resp.verified_access_instance.verified_access_trust_providers[0].user_trust_provider_type #=> String, one of "iam-identity-center", "oidc"
    #   resp.verified_access_instance.verified_access_trust_providers[0].device_trust_provider_type #=> String, one of "jamf", "crowdstrike", "jumpcloud"
    #   resp.verified_access_instance.creation_time #=> String
    #   resp.verified_access_instance.last_updated_time #=> String
    #   resp.verified_access_instance.tags #=> Array
    #   resp.verified_access_instance.tags[0].key #=> String
    #   resp.verified_access_instance.tags[0].value #=> String
    #   resp.verified_access_instance.fips_enabled #=> Boolean
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.sub_domain #=> String
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.nameservers #=> Array
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.nameservers[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DetachVerifiedAccessTrustProvider AWS API Documentation
    #
    # @overload detach_verified_access_trust_provider(params = {})
    # @param [Hash] params ({})
    def detach_verified_access_trust_provider(params = {}, options = {})
      req = build_request(:detach_verified_access_trust_provider, params)
      req.send_request(options)
    end

    # Detaches an EBS volume from an instance. Make sure to unmount any file
    # systems on the device within your operating system before detaching
    # the volume. Failure to do so can result in the volume becoming stuck
    # in the `busy` state while detaching. If this happens, detachment can
    # be delayed indefinitely until you unmount the volume, force
    # detachment, reboot the instance, or all three. If an EBS volume is the
    # root device of an instance, it can't be detached while the instance
    # is running. To detach the root volume, stop the instance first.
    #
    # When a volume with an Amazon Web Services Marketplace product code is
    # detached from an instance, the product code is no longer associated
    # with the instance.
    #
    # You can't detach or force detach volumes that are attached to Amazon
    # ECS or Fargate tasks. Attempting to do this results in the
    # `UnsupportedOperationException` exception with the `Unable to detach
    # volume attached to ECS tasks` error message.
    #
    # For more information, see [Detach an Amazon EBS volume][1] in the
    # *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-detaching-volume.html
    #
    # @option params [String] :device
    #   The device name.
    #
    # @option params [Boolean] :force
    #   Forces detachment if the previous detachment attempt did not occur
    #   cleanly (for example, logging into an instance, unmounting the volume,
    #   and detaching normally). This option can lead to data loss or a
    #   corrupted file system. Use this option only as a last resort to detach
    #   a volume from a failed instance. The instance won't have an
    #   opportunity to flush file system caches or file system metadata. If
    #   you use this option, you must perform file system check and repair
    #   procedures.
    #
    # @option params [String] :instance_id
    #   The ID of the instance. If you are detaching a Multi-Attach enabled
    #   volume, you must specify an instance ID.
    #
    # @option params [required, String] :volume_id
    #   The ID of the volume.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::VolumeAttachment] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::VolumeAttachment#delete_on_termination #delete_on_termination} => Boolean
    #   * {Types::VolumeAttachment#associated_resource #associated_resource} => String
    #   * {Types::VolumeAttachment#instance_owning_service #instance_owning_service} => String
    #   * {Types::VolumeAttachment#volume_id #volume_id} => String
    #   * {Types::VolumeAttachment#instance_id #instance_id} => String
    #   * {Types::VolumeAttachment#device #device} => String
    #   * {Types::VolumeAttachment#state #state} => String
    #   * {Types::VolumeAttachment#attach_time #attach_time} => Time
    #
    #
    # @example Example: To detach a volume from an instance
    #
    #   # This example detaches the volume (``vol-049df61146c4d7901``) from the instance it is attached to.
    #
    #   resp = client.detach_volume({
    #     volume_id: "vol-1234567890abcdef0", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     attach_time: Time.parse("2014-02-27T19:23:06.000Z"), 
    #     device: "/dev/sdb", 
    #     instance_id: "i-1234567890abcdef0", 
    #     state: "detaching", 
    #     volume_id: "vol-049df61146c4d7901", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.detach_volume({
    #     device: "String",
    #     force: false,
    #     instance_id: "InstanceIdForResolver",
    #     volume_id: "VolumeIdWithResolver", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.delete_on_termination #=> Boolean
    #   resp.associated_resource #=> String
    #   resp.instance_owning_service #=> String
    #   resp.volume_id #=> String
    #   resp.instance_id #=> String
    #   resp.device #=> String
    #   resp.state #=> String, one of "attaching", "attached", "detaching", "detached", "busy"
    #   resp.attach_time #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DetachVolume AWS API Documentation
    #
    # @overload detach_volume(params = {})
    # @param [Hash] params ({})
    def detach_volume(params = {}, options = {})
      req = build_request(:detach_volume, params)
      req.send_request(options)
    end

    # Detaches a virtual private gateway from a VPC. You do this if you're
    # planning to turn off the VPC and not use it anymore. You can confirm a
    # virtual private gateway has been completely detached from a VPC by
    # describing the virtual private gateway (any attachments to the virtual
    # private gateway are also described).
    #
    # You must wait for the attachment's state to switch to `detached`
    # before you can delete the VPC or attach a different VPC to the virtual
    # private gateway.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @option params [required, String] :vpn_gateway_id
    #   The ID of the virtual private gateway.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.detach_vpn_gateway({
    #     vpc_id: "VpcId", # required
    #     vpn_gateway_id: "VpnGatewayId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DetachVpnGateway AWS API Documentation
    #
    # @overload detach_vpn_gateway(params = {})
    # @param [Hash] params ({})
    def detach_vpn_gateway(params = {}, options = {})
      req = build_request(:detach_vpn_gateway, params)
      req.send_request(options)
    end

    # Disables Elastic IP address transfer. For more information, see
    # [Transfer Elastic IP addresses][1] in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/vpc-eips.html#transfer-EIPs-intro
    #
    # @option params [required, String] :allocation_id
    #   The allocation ID of an Elastic IP address.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisableAddressTransferResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableAddressTransferResult#address_transfer #address_transfer} => Types::AddressTransfer
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_address_transfer({
    #     allocation_id: "AllocationId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.address_transfer.public_ip #=> String
    #   resp.address_transfer.allocation_id #=> String
    #   resp.address_transfer.transfer_account_id #=> String
    #   resp.address_transfer.transfer_offer_expiration_timestamp #=> Time
    #   resp.address_transfer.transfer_offer_accepted_timestamp #=> Time
    #   resp.address_transfer.address_transfer_status #=> String, one of "pending", "disabled", "accepted"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableAddressTransfer AWS API Documentation
    #
    # @overload disable_address_transfer(params = {})
    # @param [Hash] params ({})
    def disable_address_transfer(params = {}, options = {})
      req = build_request(:disable_address_transfer, params)
      req.send_request(options)
    end

    # Disables Allowed AMIs for your account in the specified Amazon Web
    # Services Region. When set to `disabled`, the image criteria in your
    # Allowed AMIs settings do not apply, and no restrictions are placed on
    # AMI discoverability or usage. Users in your account can launch
    # instances using any public AMI or AMI shared with your account.
    #
    # <note markdown="1"> The Allowed AMIs feature does not restrict the AMIs owned by your
    # account. Regardless of the criteria you set, the AMIs created by your
    # account will always be discoverable and usable by users in your
    # account.
    #
    #  </note>
    #
    # For more information, see [Control the discovery and use of AMIs in
    # Amazon EC2 with Allowed AMIs][1] in *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-allowed-amis.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisableAllowedImagesSettingsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableAllowedImagesSettingsResult#allowed_images_settings_state #allowed_images_settings_state} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_allowed_images_settings({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.allowed_images_settings_state #=> String, one of "disabled"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableAllowedImagesSettings AWS API Documentation
    #
    # @overload disable_allowed_images_settings(params = {})
    # @param [Hash] params ({})
    def disable_allowed_images_settings(params = {}, options = {})
      req = build_request(:disable_allowed_images_settings, params)
      req.send_request(options)
    end

    # Disables Infrastructure Performance metric subscriptions.
    #
    # @option params [String] :source
    #   The source Region or Availability Zone that the metric subscription is
    #   disabled for. For example, `us-east-1`.
    #
    # @option params [String] :destination
    #   The target Region or Availability Zone that the metric subscription is
    #   disabled for. For example, `eu-north-1`.
    #
    # @option params [String] :metric
    #   The metric used for the disabled subscription.
    #
    # @option params [String] :statistic
    #   The statistic used for the disabled subscription.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisableAwsNetworkPerformanceMetricSubscriptionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableAwsNetworkPerformanceMetricSubscriptionResult#output #output} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_aws_network_performance_metric_subscription({
    #     source: "String",
    #     destination: "String",
    #     metric: "aggregate-latency", # accepts aggregate-latency
    #     statistic: "p50", # accepts p50
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.output #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableAwsNetworkPerformanceMetricSubscription AWS API Documentation
    #
    # @overload disable_aws_network_performance_metric_subscription(params = {})
    # @param [Hash] params ({})
    def disable_aws_network_performance_metric_subscription(params = {}, options = {})
      req = build_request(:disable_aws_network_performance_metric_subscription, params)
      req.send_request(options)
    end

    # Disables EBS encryption by default for your account in the current
    # Region.
    #
    # After you disable encryption by default, you can still create
    # encrypted volumes by enabling encryption when you create each volume.
    #
    # Disabling encryption by default does not change the encryption status
    # of your existing volumes.
    #
    # For more information, see [Amazon EBS encryption][1] in the *Amazon
    # EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-encryption.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisableEbsEncryptionByDefaultResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableEbsEncryptionByDefaultResult#ebs_encryption_by_default #ebs_encryption_by_default} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_ebs_encryption_by_default({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.ebs_encryption_by_default #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableEbsEncryptionByDefault AWS API Documentation
    #
    # @overload disable_ebs_encryption_by_default(params = {})
    # @param [Hash] params ({})
    def disable_ebs_encryption_by_default(params = {}, options = {})
      req = build_request(:disable_ebs_encryption_by_default, params)
      req.send_request(options)
    end

    # Discontinue Windows fast launch for a Windows AMI, and clean up
    # existing pre-provisioned snapshots. After you disable Windows fast
    # launch, the AMI uses the standard launch process for each new
    # instance. Amazon EC2 must remove all pre-provisioned snapshots before
    # you can enable Windows fast launch again.
    #
    # <note markdown="1"> You can only change these settings for Windows AMIs that you own or
    # that have been shared with you.
    #
    #  </note>
    #
    # @option params [required, String] :image_id
    #   Specify the ID of the image for which to disable Windows fast launch.
    #
    # @option params [Boolean] :force
    #   Forces the image settings to turn off Windows fast launch for your
    #   Windows AMI. This parameter overrides any errors that are encountered
    #   while cleaning up resources in your account.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisableFastLaunchResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableFastLaunchResult#image_id #image_id} => String
    #   * {Types::DisableFastLaunchResult#resource_type #resource_type} => String
    #   * {Types::DisableFastLaunchResult#snapshot_configuration #snapshot_configuration} => Types::FastLaunchSnapshotConfigurationResponse
    #   * {Types::DisableFastLaunchResult#launch_template #launch_template} => Types::FastLaunchLaunchTemplateSpecificationResponse
    #   * {Types::DisableFastLaunchResult#max_parallel_launches #max_parallel_launches} => Integer
    #   * {Types::DisableFastLaunchResult#owner_id #owner_id} => String
    #   * {Types::DisableFastLaunchResult#state #state} => String
    #   * {Types::DisableFastLaunchResult#state_transition_reason #state_transition_reason} => String
    #   * {Types::DisableFastLaunchResult#state_transition_time #state_transition_time} => Time
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_fast_launch({
    #     image_id: "ImageId", # required
    #     force: false,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.image_id #=> String
    #   resp.resource_type #=> String, one of "snapshot"
    #   resp.snapshot_configuration.target_resource_count #=> Integer
    #   resp.launch_template.launch_template_id #=> String
    #   resp.launch_template.launch_template_name #=> String
    #   resp.launch_template.version #=> String
    #   resp.max_parallel_launches #=> Integer
    #   resp.owner_id #=> String
    #   resp.state #=> String, one of "enabling", "enabling-failed", "enabled", "enabled-failed", "disabling", "disabling-failed"
    #   resp.state_transition_reason #=> String
    #   resp.state_transition_time #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableFastLaunch AWS API Documentation
    #
    # @overload disable_fast_launch(params = {})
    # @param [Hash] params ({})
    def disable_fast_launch(params = {}, options = {})
      req = build_request(:disable_fast_launch, params)
      req.send_request(options)
    end

    # Disables fast snapshot restores for the specified snapshots in the
    # specified Availability Zones.
    #
    # @option params [required, Array<String>] :availability_zones
    #   One or more Availability Zones. For example, `us-east-2a`.
    #
    # @option params [required, Array<String>] :source_snapshot_ids
    #   The IDs of one or more snapshots. For example,
    #   `snap-1234567890abcdef0`.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisableFastSnapshotRestoresResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableFastSnapshotRestoresResult#successful #successful} => Array&lt;Types::DisableFastSnapshotRestoreSuccessItem&gt;
    #   * {Types::DisableFastSnapshotRestoresResult#unsuccessful #unsuccessful} => Array&lt;Types::DisableFastSnapshotRestoreErrorItem&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_fast_snapshot_restores({
    #     availability_zones: ["String"], # required
    #     source_snapshot_ids: ["SnapshotId"], # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.successful #=> Array
    #   resp.successful[0].snapshot_id #=> String
    #   resp.successful[0].availability_zone #=> String
    #   resp.successful[0].state #=> String, one of "enabling", "optimizing", "enabled", "disabling", "disabled"
    #   resp.successful[0].state_transition_reason #=> String
    #   resp.successful[0].owner_id #=> String
    #   resp.successful[0].owner_alias #=> String
    #   resp.successful[0].enabling_time #=> Time
    #   resp.successful[0].optimizing_time #=> Time
    #   resp.successful[0].enabled_time #=> Time
    #   resp.successful[0].disabling_time #=> Time
    #   resp.successful[0].disabled_time #=> Time
    #   resp.unsuccessful #=> Array
    #   resp.unsuccessful[0].snapshot_id #=> String
    #   resp.unsuccessful[0].fast_snapshot_restore_state_errors #=> Array
    #   resp.unsuccessful[0].fast_snapshot_restore_state_errors[0].availability_zone #=> String
    #   resp.unsuccessful[0].fast_snapshot_restore_state_errors[0].error.code #=> String
    #   resp.unsuccessful[0].fast_snapshot_restore_state_errors[0].error.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableFastSnapshotRestores AWS API Documentation
    #
    # @overload disable_fast_snapshot_restores(params = {})
    # @param [Hash] params ({})
    def disable_fast_snapshot_restores(params = {}, options = {})
      req = build_request(:disable_fast_snapshot_restores, params)
      req.send_request(options)
    end

    # Sets the AMI state to `disabled` and removes all launch permissions
    # from the AMI. A disabled AMI can't be used for instance launches.
    #
    # A disabled AMI can't be shared. If an AMI was public or previously
    # shared, it is made private. If an AMI was shared with an Amazon Web
    # Services account, organization, or Organizational Unit, they lose
    # access to the disabled AMI.
    #
    # A disabled AMI does not appear in [DescribeImages][1] API calls by
    # default.
    #
    # Only the AMI owner can disable an AMI.
    #
    # You can re-enable a disabled AMI using [EnableImage][2].
    #
    # For more information, see [Disable an AMI][3] in the *Amazon EC2 User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeImages.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_EnableImage.html
    # [3]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/disable-an-ami.html
    #
    # @option params [required, String] :image_id
    #   The ID of the AMI.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisableImageResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableImageResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_image({
    #     image_id: "ImageId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableImage AWS API Documentation
    #
    # @overload disable_image(params = {})
    # @param [Hash] params ({})
    def disable_image(params = {}, options = {})
      req = build_request(:disable_image, params)
      req.send_request(options)
    end

    # Disables *block public access for AMIs* at the account level in the
    # specified Amazon Web Services Region. This removes the *block public
    # access* restriction from your account. With the restriction removed,
    # you can publicly share your AMIs in the specified Amazon Web Services
    # Region.
    #
    # The API can take up to 10 minutes to configure this setting. During
    # this time, if you run [GetImageBlockPublicAccessState][1], the
    # response will be `block-new-sharing`. When the API has completed the
    # configuration, the response will be `unblocked`.
    #
    # For more information, see [Block public access to your AMIs][2] in the
    # *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_GetImageBlockPublicAccessState.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/sharingamis-intro.html#block-public-access-to-amis
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisableImageBlockPublicAccessResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableImageBlockPublicAccessResult#image_block_public_access_state #image_block_public_access_state} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_image_block_public_access({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.image_block_public_access_state #=> String, one of "unblocked"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableImageBlockPublicAccess AWS API Documentation
    #
    # @overload disable_image_block_public_access(params = {})
    # @param [Hash] params ({})
    def disable_image_block_public_access(params = {}, options = {})
      req = build_request(:disable_image_block_public_access, params)
      req.send_request(options)
    end

    # Cancels the deprecation of the specified AMI.
    #
    # For more information, see [Deprecate an AMI][1] in the *Amazon EC2
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ami-deprecate.html
    #
    # @option params [required, String] :image_id
    #   The ID of the AMI.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisableImageDeprecationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableImageDeprecationResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_image_deprecation({
    #     image_id: "ImageId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableImageDeprecation AWS API Documentation
    #
    # @overload disable_image_deprecation(params = {})
    # @param [Hash] params ({})
    def disable_image_deprecation(params = {}, options = {})
      req = build_request(:disable_image_deprecation, params)
      req.send_request(options)
    end

    # Disables deregistration protection for an AMI. When deregistration
    # protection is disabled, the AMI can be deregistered.
    #
    # If you chose to include a 24-hour cooldown period when you enabled
    # deregistration protection for the AMI, then, when you disable
    # deregistration protection, you won’t immediately be able to deregister
    # the AMI.
    #
    # For more information, see [Protect an AMI from deregistration][1] in
    # the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/deregister-ami.html#ami-deregistration-protection
    #
    # @option params [required, String] :image_id
    #   The ID of the AMI.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisableImageDeregistrationProtectionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableImageDeregistrationProtectionResult#return #return} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_image_deregistration_protection({
    #     image_id: "ImageId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableImageDeregistrationProtection AWS API Documentation
    #
    # @overload disable_image_deregistration_protection(params = {})
    # @param [Hash] params ({})
    def disable_image_deregistration_protection(params = {}, options = {})
      req = build_request(:disable_image_deregistration_protection, params)
      req.send_request(options)
    end

    # Disable the IPAM account. For more information, see [Enable
    # integration with Organizations][1] in the *Amazon VPC IPAM User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/enable-integ-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :delegated_admin_account_id
    #   The Organizations member account ID that you want to disable as IPAM
    #   account.
    #
    # @return [Types::DisableIpamOrganizationAdminAccountResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableIpamOrganizationAdminAccountResult#success #success} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_ipam_organization_admin_account({
    #     dry_run: false,
    #     delegated_admin_account_id: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.success #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableIpamOrganizationAdminAccount AWS API Documentation
    #
    # @overload disable_ipam_organization_admin_account(params = {})
    # @param [Hash] params ({})
    def disable_ipam_organization_admin_account(params = {}, options = {})
      req = build_request(:disable_ipam_organization_admin_account, params)
      req.send_request(options)
    end

    # Disables access to the EC2 serial console of all instances for your
    # account. By default, access to the EC2 serial console is disabled for
    # your account. For more information, see [Manage account access to the
    # EC2 serial console][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/configure-access-to-serial-console.html#serial-console-account-access
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisableSerialConsoleAccessResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableSerialConsoleAccessResult#serial_console_access_enabled #serial_console_access_enabled} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_serial_console_access({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.serial_console_access_enabled #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableSerialConsoleAccess AWS API Documentation
    #
    # @overload disable_serial_console_access(params = {})
    # @param [Hash] params ({})
    def disable_serial_console_access(params = {}, options = {})
      req = build_request(:disable_serial_console_access, params)
      req.send_request(options)
    end

    # Disables the *block public access for snapshots* setting at the
    # account level for the specified Amazon Web Services Region. After you
    # disable block public access for snapshots in a Region, users can
    # publicly share snapshots in that Region.
    #
    # Enabling block public access for snapshots in *block-all-sharing* mode
    # does not change the permissions for snapshots that are already
    # publicly shared. Instead, it prevents these snapshots from be publicly
    # visible and publicly accessible. Therefore, the attributes for these
    # snapshots still indicate that they are publicly shared, even though
    # they are not publicly available.
    #
    #  If you disable block public access , these snapshots will become
    # publicly available again.
    #
    # For more information, see [ Block public access for snapshots][1] in
    # the *Amazon EBS User Guide* .
    #
    #
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/block-public-access-snapshots.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisableSnapshotBlockPublicAccessResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableSnapshotBlockPublicAccessResult#state #state} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_snapshot_block_public_access({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.state #=> String, one of "block-all-sharing", "block-new-sharing", "unblocked"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableSnapshotBlockPublicAccess AWS API Documentation
    #
    # @overload disable_snapshot_block_public_access(params = {})
    # @param [Hash] params ({})
    def disable_snapshot_block_public_access(params = {}, options = {})
      req = build_request(:disable_snapshot_block_public_access, params)
      req.send_request(options)
    end

    # Disables the specified resource attachment from propagating routes to
    # the specified propagation route table.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the propagation route table.
    #
    # @option params [String] :transit_gateway_attachment_id
    #   The ID of the attachment.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :transit_gateway_route_table_announcement_id
    #   The ID of the route table announcement.
    #
    # @return [Types::DisableTransitGatewayRouteTablePropagationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableTransitGatewayRouteTablePropagationResult#propagation #propagation} => Types::TransitGatewayPropagation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_transit_gateway_route_table_propagation({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId",
    #     dry_run: false,
    #     transit_gateway_route_table_announcement_id: "TransitGatewayRouteTableAnnouncementId",
    #   })
    #
    # @example Response structure
    #
    #   resp.propagation.transit_gateway_attachment_id #=> String
    #   resp.propagation.resource_id #=> String
    #   resp.propagation.resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.propagation.transit_gateway_route_table_id #=> String
    #   resp.propagation.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.propagation.transit_gateway_route_table_announcement_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableTransitGatewayRouteTablePropagation AWS API Documentation
    #
    # @overload disable_transit_gateway_route_table_propagation(params = {})
    # @param [Hash] params ({})
    def disable_transit_gateway_route_table_propagation(params = {}, options = {})
      req = build_request(:disable_transit_gateway_route_table_propagation, params)
      req.send_request(options)
    end

    # Disables a virtual private gateway (VGW) from propagating routes to a
    # specified route table of a VPC.
    #
    # @option params [required, String] :gateway_id
    #   The ID of the virtual private gateway.
    #
    # @option params [required, String] :route_table_id
    #   The ID of the route table.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To disable route propagation
    #
    #   # This example disables the specified virtual private gateway from propagating static routes to the specified route table.
    #
    #   resp = client.disable_vgw_route_propagation({
    #     gateway_id: "vgw-9a4cacf3", 
    #     route_table_id: "rtb-22574640", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_vgw_route_propagation({
    #     gateway_id: "VpnGatewayId", # required
    #     route_table_id: "RouteTableId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableVgwRoutePropagation AWS API Documentation
    #
    # @overload disable_vgw_route_propagation(params = {})
    # @param [Hash] params ({})
    def disable_vgw_route_propagation(params = {}, options = {})
      req = build_request(:disable_vgw_route_propagation, params)
      req.send_request(options)
    end

    # <note markdown="1"> This action is deprecated.
    #
    #  </note>
    #
    # Disables ClassicLink for a VPC. You cannot disable ClassicLink for a
    # VPC that has EC2-Classic instances linked to it.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @return [Types::DisableVpcClassicLinkResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableVpcClassicLinkResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_vpc_classic_link({
    #     dry_run: false,
    #     vpc_id: "VpcId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableVpcClassicLink AWS API Documentation
    #
    # @overload disable_vpc_classic_link(params = {})
    # @param [Hash] params ({})
    def disable_vpc_classic_link(params = {}, options = {})
      req = build_request(:disable_vpc_classic_link, params)
      req.send_request(options)
    end

    # <note markdown="1"> This action is deprecated.
    #
    #  </note>
    #
    # Disables ClassicLink DNS support for a VPC. If disabled, DNS hostnames
    # resolve to public IP addresses when addressed between a linked
    # EC2-Classic instance and instances in the VPC to which it's linked.
    #
    # You must specify a VPC ID in the request.
    #
    # @option params [String] :vpc_id
    #   The ID of the VPC.
    #
    # @return [Types::DisableVpcClassicLinkDnsSupportResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisableVpcClassicLinkDnsSupportResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disable_vpc_classic_link_dns_support({
    #     vpc_id: "VpcId",
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisableVpcClassicLinkDnsSupport AWS API Documentation
    #
    # @overload disable_vpc_classic_link_dns_support(params = {})
    # @param [Hash] params ({})
    def disable_vpc_classic_link_dns_support(params = {}, options = {})
      req = build_request(:disable_vpc_classic_link_dns_support, params)
      req.send_request(options)
    end

    # Disassociates an Elastic IP address from the instance or network
    # interface it's associated with.
    #
    # This is an idempotent operation. If you perform the operation more
    # than once, Amazon EC2 doesn't return an error.
    #
    # @option params [String] :association_id
    #   The association ID. This parameter is required.
    #
    # @option params [String] :public_ip
    #   Deprecated.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To disassociate an Elastic IP address
    #
    #   # This example disassociates an Elastic IP address from an instance.
    #
    #   resp = client.disassociate_address({
    #     association_id: "eipassoc-2bebb745", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_address({
    #     association_id: "ElasticIpAssociationId",
    #     public_ip: "EipAllocationPublicIp",
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateAddress AWS API Documentation
    #
    # @overload disassociate_address(params = {})
    # @param [Hash] params ({})
    def disassociate_address(params = {}, options = {})
      req = build_request(:disassociate_address, params)
      req.send_request(options)
    end

    # Cancels a pending request to assign billing of the unused capacity of
    # a Capacity Reservation to a consumer account, or revokes a request
    # that has already been accepted. For more information, see [Billing
    # assignment for shared Amazon EC2 Capacity Reservations][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/assign-billing.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :capacity_reservation_id
    #   The ID of the Capacity Reservation.
    #
    # @option params [required, String] :unused_reservation_billing_owner_id
    #   The ID of the consumer account to which the request was sent.
    #
    # @return [Types::DisassociateCapacityReservationBillingOwnerResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateCapacityReservationBillingOwnerResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_capacity_reservation_billing_owner({
    #     dry_run: false,
    #     capacity_reservation_id: "CapacityReservationId", # required
    #     unused_reservation_billing_owner_id: "AccountID", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateCapacityReservationBillingOwner AWS API Documentation
    #
    # @overload disassociate_capacity_reservation_billing_owner(params = {})
    # @param [Hash] params ({})
    def disassociate_capacity_reservation_billing_owner(params = {}, options = {})
      req = build_request(:disassociate_capacity_reservation_billing_owner, params)
      req.send_request(options)
    end

    # Disassociates a target network from the specified Client VPN endpoint.
    # When you disassociate the last target network from a Client VPN, the
    # following happens:
    #
    # * The route that was automatically added for the VPC is deleted
    #
    # * All active client connections are terminated
    #
    # * New client connections are disallowed
    #
    # * The Client VPN endpoint's status changes to `pending-associate`
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint from which to disassociate the
    #   target network.
    #
    # @option params [required, String] :association_id
    #   The ID of the target network association.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisassociateClientVpnTargetNetworkResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateClientVpnTargetNetworkResult#association_id #association_id} => String
    #   * {Types::DisassociateClientVpnTargetNetworkResult#status #status} => Types::AssociationStatus
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_client_vpn_target_network({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     association_id: "String", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.association_id #=> String
    #   resp.status.code #=> String, one of "associating", "associated", "association-failed", "disassociating", "disassociated"
    #   resp.status.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateClientVpnTargetNetwork AWS API Documentation
    #
    # @overload disassociate_client_vpn_target_network(params = {})
    # @param [Hash] params ({})
    def disassociate_client_vpn_target_network(params = {}, options = {})
      req = build_request(:disassociate_client_vpn_target_network, params)
      req.send_request(options)
    end

    # Disassociates an IAM role from an Certificate Manager (ACM)
    # certificate. Disassociating an IAM role from an ACM certificate
    # removes the Amazon S3 object that contains the certificate,
    # certificate chain, and encrypted private key from the Amazon S3
    # bucket. It also revokes the IAM role's permission to use the KMS key
    # used to encrypt the private key. This effectively revokes the role's
    # permission to use the certificate.
    #
    # @option params [required, String] :certificate_arn
    #   The ARN of the ACM certificate from which to disassociate the IAM
    #   role.
    #
    # @option params [required, String] :role_arn
    #   The ARN of the IAM role to disassociate.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisassociateEnclaveCertificateIamRoleResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateEnclaveCertificateIamRoleResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_enclave_certificate_iam_role({
    #     certificate_arn: "CertificateId", # required
    #     role_arn: "RoleId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateEnclaveCertificateIamRole AWS API Documentation
    #
    # @overload disassociate_enclave_certificate_iam_role(params = {})
    # @param [Hash] params ({})
    def disassociate_enclave_certificate_iam_role(params = {}, options = {})
      req = build_request(:disassociate_enclave_certificate_iam_role, params)
      req.send_request(options)
    end

    # Disassociates an IAM instance profile from a running or stopped
    # instance.
    #
    # Use DescribeIamInstanceProfileAssociations to get the association ID.
    #
    # @option params [required, String] :association_id
    #   The ID of the IAM instance profile association.
    #
    # @return [Types::DisassociateIamInstanceProfileResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateIamInstanceProfileResult#iam_instance_profile_association #iam_instance_profile_association} => Types::IamInstanceProfileAssociation
    #
    #
    # @example Example: To disassociate an IAM instance profile
    #
    #   # This example disassociates the specified IAM instance profile from an instance.
    #
    #   resp = client.disassociate_iam_instance_profile({
    #     association_id: "iip-assoc-05020b59952902f5f", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     iam_instance_profile_association: {
    #       association_id: "iip-assoc-05020b59952902f5f", 
    #       iam_instance_profile: {
    #         arn: "arn:aws:iam::123456789012:instance-profile/admin-role", 
    #         id: "AIPAI5IVIHMFFYY2DKV5Y", 
    #       }, 
    #       instance_id: "i-123456789abcde123", 
    #       state: "disassociating", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_iam_instance_profile({
    #     association_id: "IamInstanceProfileAssociationId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.iam_instance_profile_association.association_id #=> String
    #   resp.iam_instance_profile_association.instance_id #=> String
    #   resp.iam_instance_profile_association.iam_instance_profile.arn #=> String
    #   resp.iam_instance_profile_association.iam_instance_profile.id #=> String
    #   resp.iam_instance_profile_association.state #=> String, one of "associating", "associated", "disassociating", "disassociated"
    #   resp.iam_instance_profile_association.timestamp #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateIamInstanceProfile AWS API Documentation
    #
    # @overload disassociate_iam_instance_profile(params = {})
    # @param [Hash] params ({})
    def disassociate_iam_instance_profile(params = {}, options = {})
      req = build_request(:disassociate_iam_instance_profile, params)
      req.send_request(options)
    end

    # Disassociates one or more targets from an event window.
    #
    # For more information, see [Define event windows for scheduled
    # events][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/event-windows.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_event_window_id
    #   The ID of the event window.
    #
    # @option params [required, Types::InstanceEventWindowDisassociationRequest] :association_target
    #   One or more targets to disassociate from the specified event window.
    #
    # @return [Types::DisassociateInstanceEventWindowResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateInstanceEventWindowResult#instance_event_window #instance_event_window} => Types::InstanceEventWindow
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_instance_event_window({
    #     dry_run: false,
    #     instance_event_window_id: "InstanceEventWindowId", # required
    #     association_target: { # required
    #       instance_ids: ["InstanceId"],
    #       instance_tags: [
    #         {
    #           key: "String",
    #           value: "String",
    #         },
    #       ],
    #       dedicated_host_ids: ["DedicatedHostId"],
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_event_window.instance_event_window_id #=> String
    #   resp.instance_event_window.time_ranges #=> Array
    #   resp.instance_event_window.time_ranges[0].start_week_day #=> String, one of "sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"
    #   resp.instance_event_window.time_ranges[0].start_hour #=> Integer
    #   resp.instance_event_window.time_ranges[0].end_week_day #=> String, one of "sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"
    #   resp.instance_event_window.time_ranges[0].end_hour #=> Integer
    #   resp.instance_event_window.name #=> String
    #   resp.instance_event_window.cron_expression #=> String
    #   resp.instance_event_window.association_target.instance_ids #=> Array
    #   resp.instance_event_window.association_target.instance_ids[0] #=> String
    #   resp.instance_event_window.association_target.tags #=> Array
    #   resp.instance_event_window.association_target.tags[0].key #=> String
    #   resp.instance_event_window.association_target.tags[0].value #=> String
    #   resp.instance_event_window.association_target.dedicated_host_ids #=> Array
    #   resp.instance_event_window.association_target.dedicated_host_ids[0] #=> String
    #   resp.instance_event_window.state #=> String, one of "creating", "deleting", "active", "deleted"
    #   resp.instance_event_window.tags #=> Array
    #   resp.instance_event_window.tags[0].key #=> String
    #   resp.instance_event_window.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateInstanceEventWindow AWS API Documentation
    #
    # @overload disassociate_instance_event_window(params = {})
    # @param [Hash] params ({})
    def disassociate_instance_event_window(params = {}, options = {})
      req = build_request(:disassociate_instance_event_window, params)
      req.send_request(options)
    end

    # Remove the association between your Autonomous System Number (ASN) and
    # your BYOIP CIDR. You may want to use this action to disassociate an
    # ASN from a CIDR or if you want to swap ASNs. For more information, see
    # [Tutorial: Bring your ASN to IPAM][1] in the *Amazon VPC IPAM guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/tutorials-byoasn.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :asn
    #   A public 2-byte or 4-byte ASN.
    #
    # @option params [required, String] :cidr
    #   A BYOIP CIDR.
    #
    # @return [Types::DisassociateIpamByoasnResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateIpamByoasnResult#asn_association #asn_association} => Types::AsnAssociation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_ipam_byoasn({
    #     dry_run: false,
    #     asn: "String", # required
    #     cidr: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.asn_association.asn #=> String
    #   resp.asn_association.cidr #=> String
    #   resp.asn_association.status_message #=> String
    #   resp.asn_association.state #=> String, one of "disassociated", "failed-disassociation", "failed-association", "pending-disassociation", "pending-association", "associated"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateIpamByoasn AWS API Documentation
    #
    # @overload disassociate_ipam_byoasn(params = {})
    # @param [Hash] params ({})
    def disassociate_ipam_byoasn(params = {}, options = {})
      req = build_request(:disassociate_ipam_byoasn, params)
      req.send_request(options)
    end

    # Disassociates a resource discovery from an Amazon VPC IPAM. A resource
    # discovery is an IPAM component that enables IPAM to manage and monitor
    # resources that belong to the owning account.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_resource_discovery_association_id
    #   A resource discovery association ID.
    #
    # @return [Types::DisassociateIpamResourceDiscoveryResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateIpamResourceDiscoveryResult#ipam_resource_discovery_association #ipam_resource_discovery_association} => Types::IpamResourceDiscoveryAssociation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_ipam_resource_discovery({
    #     dry_run: false,
    #     ipam_resource_discovery_association_id: "IpamResourceDiscoveryAssociationId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_resource_discovery_association.owner_id #=> String
    #   resp.ipam_resource_discovery_association.ipam_resource_discovery_association_id #=> String
    #   resp.ipam_resource_discovery_association.ipam_resource_discovery_association_arn #=> String
    #   resp.ipam_resource_discovery_association.ipam_resource_discovery_id #=> String
    #   resp.ipam_resource_discovery_association.ipam_id #=> String
    #   resp.ipam_resource_discovery_association.ipam_arn #=> String
    #   resp.ipam_resource_discovery_association.ipam_region #=> String
    #   resp.ipam_resource_discovery_association.is_default #=> Boolean
    #   resp.ipam_resource_discovery_association.resource_discovery_status #=> String, one of "active", "not-found"
    #   resp.ipam_resource_discovery_association.state #=> String, one of "associate-in-progress", "associate-complete", "associate-failed", "disassociate-in-progress", "disassociate-complete", "disassociate-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_resource_discovery_association.tags #=> Array
    #   resp.ipam_resource_discovery_association.tags[0].key #=> String
    #   resp.ipam_resource_discovery_association.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateIpamResourceDiscovery AWS API Documentation
    #
    # @overload disassociate_ipam_resource_discovery(params = {})
    # @param [Hash] params ({})
    def disassociate_ipam_resource_discovery(params = {}, options = {})
      req = build_request(:disassociate_ipam_resource_discovery, params)
      req.send_request(options)
    end

    # Disassociates secondary Elastic IP addresses (EIPs) from a public NAT
    # gateway. You cannot disassociate your primary EIP. For more
    # information, see [Edit secondary IP address associations][1] in the
    # *Amazon VPC User Guide*.
    #
    # While disassociating is in progress, you cannot associate/disassociate
    # additional EIPs while the connections are being drained. You are,
    # however, allowed to delete the NAT gateway.
    #
    # An EIP is released only at the end of MaxDrainDurationSeconds. It
    # stays associated and supports the existing connections but does not
    # support any new connections (new connections are distributed across
    # the remaining associated EIPs). As the existing connections drain out,
    # the EIPs (and the corresponding private IP addresses mapped to them)
    # are released.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/nat-gateway-working-with.html#nat-gateway-edit-secondary
    #
    # @option params [required, String] :nat_gateway_id
    #   The ID of the NAT gateway.
    #
    # @option params [required, Array<String>] :association_ids
    #   The association IDs of EIPs that have been associated with the NAT
    #   gateway.
    #
    # @option params [Integer] :max_drain_duration_seconds
    #   The maximum amount of time to wait (in seconds) before forcibly
    #   releasing the IP addresses if connections are still in progress.
    #   Default value is 350 seconds.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisassociateNatGatewayAddressResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateNatGatewayAddressResult#nat_gateway_id #nat_gateway_id} => String
    #   * {Types::DisassociateNatGatewayAddressResult#nat_gateway_addresses #nat_gateway_addresses} => Array&lt;Types::NatGatewayAddress&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_nat_gateway_address({
    #     nat_gateway_id: "NatGatewayId", # required
    #     association_ids: ["ElasticIpAssociationId"], # required
    #     max_drain_duration_seconds: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.nat_gateway_id #=> String
    #   resp.nat_gateway_addresses #=> Array
    #   resp.nat_gateway_addresses[0].allocation_id #=> String
    #   resp.nat_gateway_addresses[0].network_interface_id #=> String
    #   resp.nat_gateway_addresses[0].private_ip #=> String
    #   resp.nat_gateway_addresses[0].public_ip #=> String
    #   resp.nat_gateway_addresses[0].association_id #=> String
    #   resp.nat_gateway_addresses[0].is_primary #=> Boolean
    #   resp.nat_gateway_addresses[0].failure_message #=> String
    #   resp.nat_gateway_addresses[0].status #=> String, one of "assigning", "unassigning", "associating", "disassociating", "succeeded", "failed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateNatGatewayAddress AWS API Documentation
    #
    # @overload disassociate_nat_gateway_address(params = {})
    # @param [Hash] params ({})
    def disassociate_nat_gateway_address(params = {}, options = {})
      req = build_request(:disassociate_nat_gateway_address, params)
      req.send_request(options)
    end

    # Disassociates a subnet or gateway from a route table.
    #
    # After you perform this action, the subnet no longer uses the routes in
    # the route table. Instead, it uses the routes in the VPC's main route
    # table. For more information about route tables, see [Route tables][1]
    # in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Route_Tables.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :association_id
    #   The association ID representing the current association between the
    #   route table and subnet or gateway.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To disassociate a route table
    #
    #   # This example disassociates the specified route table from its associated subnet.
    #
    #   resp = client.disassociate_route_table({
    #     association_id: "rtbassoc-781d0d1a", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_route_table({
    #     dry_run: false,
    #     association_id: "RouteTableAssociationId", # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateRouteTable AWS API Documentation
    #
    # @overload disassociate_route_table(params = {})
    # @param [Hash] params ({})
    def disassociate_route_table(params = {}, options = {})
      req = build_request(:disassociate_route_table, params)
      req.send_request(options)
    end

    # Disassociates a security group from a VPC. You cannot disassociate the
    # security group if any Elastic network interfaces in the associated VPC
    # are still associated with the security group. Note that the
    # disassociation is asynchronous and you can check the status of the
    # request with [DescribeSecurityGroupVpcAssociations][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DescribeSecurityGroupVpcAssociations.html
    #
    # @option params [required, String] :group_id
    #   A security group ID.
    #
    # @option params [required, String] :vpc_id
    #   A VPC ID.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisassociateSecurityGroupVpcResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateSecurityGroupVpcResult#state #state} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_security_group_vpc({
    #     group_id: "DisassociateSecurityGroupVpcSecurityGroupId", # required
    #     vpc_id: "String", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.state #=> String, one of "associating", "associated", "association-failed", "disassociating", "disassociated", "disassociation-failed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateSecurityGroupVpc AWS API Documentation
    #
    # @overload disassociate_security_group_vpc(params = {})
    # @param [Hash] params ({})
    def disassociate_security_group_vpc(params = {}, options = {})
      req = build_request(:disassociate_security_group_vpc, params)
      req.send_request(options)
    end

    # Disassociates a CIDR block from a subnet. Currently, you can
    # disassociate an IPv6 CIDR block only. You must detach or delete all
    # gateways and resources that are associated with the CIDR block before
    # you can disassociate it.
    #
    # @option params [required, String] :association_id
    #   The association ID for the CIDR block.
    #
    # @return [Types::DisassociateSubnetCidrBlockResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateSubnetCidrBlockResult#ipv_6_cidr_block_association #ipv_6_cidr_block_association} => Types::SubnetIpv6CidrBlockAssociation
    #   * {Types::DisassociateSubnetCidrBlockResult#subnet_id #subnet_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_subnet_cidr_block({
    #     association_id: "SubnetCidrAssociationId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.ipv_6_cidr_block_association.association_id #=> String
    #   resp.ipv_6_cidr_block_association.ipv_6_cidr_block #=> String
    #   resp.ipv_6_cidr_block_association.ipv_6_cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.ipv_6_cidr_block_association.ipv_6_cidr_block_state.status_message #=> String
    #   resp.ipv_6_cidr_block_association.ipv_6_address_attribute #=> String, one of "public", "private"
    #   resp.ipv_6_cidr_block_association.ip_source #=> String, one of "amazon", "byoip", "none"
    #   resp.subnet_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateSubnetCidrBlock AWS API Documentation
    #
    # @overload disassociate_subnet_cidr_block(params = {})
    # @param [Hash] params ({})
    def disassociate_subnet_cidr_block(params = {}, options = {})
      req = build_request(:disassociate_subnet_cidr_block, params)
      req.send_request(options)
    end

    # Disassociates the specified subnets from the transit gateway multicast
    # domain.
    #
    # @option params [required, String] :transit_gateway_multicast_domain_id
    #   The ID of the transit gateway multicast domain.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the attachment.
    #
    # @option params [required, Array<String>] :subnet_ids
    #   The IDs of the subnets;
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisassociateTransitGatewayMulticastDomainResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateTransitGatewayMulticastDomainResult#associations #associations} => Types::TransitGatewayMulticastDomainAssociations
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_transit_gateway_multicast_domain({
    #     transit_gateway_multicast_domain_id: "TransitGatewayMulticastDomainId", # required
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     subnet_ids: ["SubnetId"], # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.associations.transit_gateway_multicast_domain_id #=> String
    #   resp.associations.transit_gateway_attachment_id #=> String
    #   resp.associations.resource_id #=> String
    #   resp.associations.resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.associations.resource_owner_id #=> String
    #   resp.associations.subnets #=> Array
    #   resp.associations.subnets[0].subnet_id #=> String
    #   resp.associations.subnets[0].state #=> String, one of "pendingAcceptance", "associating", "associated", "disassociating", "disassociated", "rejected", "failed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateTransitGatewayMulticastDomain AWS API Documentation
    #
    # @overload disassociate_transit_gateway_multicast_domain(params = {})
    # @param [Hash] params ({})
    def disassociate_transit_gateway_multicast_domain(params = {}, options = {})
      req = build_request(:disassociate_transit_gateway_multicast_domain, params)
      req.send_request(options)
    end

    # Removes the association between an an attachment and a policy table.
    #
    # @option params [required, String] :transit_gateway_policy_table_id
    #   The ID of the disassociated policy table.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the transit gateway attachment to disassociate from the
    #   policy table.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisassociateTransitGatewayPolicyTableResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateTransitGatewayPolicyTableResult#association #association} => Types::TransitGatewayPolicyTableAssociation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_transit_gateway_policy_table({
    #     transit_gateway_policy_table_id: "TransitGatewayPolicyTableId", # required
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.association.transit_gateway_policy_table_id #=> String
    #   resp.association.transit_gateway_attachment_id #=> String
    #   resp.association.resource_id #=> String
    #   resp.association.resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.association.state #=> String, one of "associating", "associated", "disassociating", "disassociated"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateTransitGatewayPolicyTable AWS API Documentation
    #
    # @overload disassociate_transit_gateway_policy_table(params = {})
    # @param [Hash] params ({})
    def disassociate_transit_gateway_policy_table(params = {}, options = {})
      req = build_request(:disassociate_transit_gateway_policy_table, params)
      req.send_request(options)
    end

    # Disassociates a resource attachment from a transit gateway route
    # table.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the transit gateway route table.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the attachment.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisassociateTransitGatewayRouteTableResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateTransitGatewayRouteTableResult#association #association} => Types::TransitGatewayAssociation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_transit_gateway_route_table({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.association.transit_gateway_route_table_id #=> String
    #   resp.association.transit_gateway_attachment_id #=> String
    #   resp.association.resource_id #=> String
    #   resp.association.resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.association.state #=> String, one of "associating", "associated", "disassociating", "disassociated"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateTransitGatewayRouteTable AWS API Documentation
    #
    # @overload disassociate_transit_gateway_route_table(params = {})
    # @param [Hash] params ({})
    def disassociate_transit_gateway_route_table(params = {}, options = {})
      req = build_request(:disassociate_transit_gateway_route_table, params)
      req.send_request(options)
    end

    # Removes an association between a branch network interface with a trunk
    # network interface.
    #
    # @option params [required, String] :association_id
    #   The ID of the association
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::DisassociateTrunkInterfaceResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateTrunkInterfaceResult#return #return} => Boolean
    #   * {Types::DisassociateTrunkInterfaceResult#client_token #client_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_trunk_interface({
    #     association_id: "TrunkInterfaceAssociationId", # required
    #     client_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #   resp.client_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateTrunkInterface AWS API Documentation
    #
    # @overload disassociate_trunk_interface(params = {})
    # @param [Hash] params ({})
    def disassociate_trunk_interface(params = {}, options = {})
      req = build_request(:disassociate_trunk_interface, params)
      req.send_request(options)
    end

    # Disassociates a CIDR block from a VPC. To disassociate the CIDR block,
    # you must specify its association ID. You can get the association ID by
    # using DescribeVpcs. You must detach or delete all gateways and
    # resources that are associated with the CIDR block before you can
    # disassociate it.
    #
    # You cannot disassociate the CIDR block with which you originally
    # created the VPC (the primary CIDR block).
    #
    # @option params [required, String] :association_id
    #   The association ID for the CIDR block.
    #
    # @return [Types::DisassociateVpcCidrBlockResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::DisassociateVpcCidrBlockResult#ipv_6_cidr_block_association #ipv_6_cidr_block_association} => Types::VpcIpv6CidrBlockAssociation
    #   * {Types::DisassociateVpcCidrBlockResult#cidr_block_association #cidr_block_association} => Types::VpcCidrBlockAssociation
    #   * {Types::DisassociateVpcCidrBlockResult#vpc_id #vpc_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.disassociate_vpc_cidr_block({
    #     association_id: "VpcCidrAssociationId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.ipv_6_cidr_block_association.association_id #=> String
    #   resp.ipv_6_cidr_block_association.ipv_6_cidr_block #=> String
    #   resp.ipv_6_cidr_block_association.ipv_6_cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.ipv_6_cidr_block_association.ipv_6_cidr_block_state.status_message #=> String
    #   resp.ipv_6_cidr_block_association.network_border_group #=> String
    #   resp.ipv_6_cidr_block_association.ipv_6_pool #=> String
    #   resp.ipv_6_cidr_block_association.ipv_6_address_attribute #=> String, one of "public", "private"
    #   resp.ipv_6_cidr_block_association.ip_source #=> String, one of "amazon", "byoip", "none"
    #   resp.cidr_block_association.association_id #=> String
    #   resp.cidr_block_association.cidr_block #=> String
    #   resp.cidr_block_association.cidr_block_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failing", "failed"
    #   resp.cidr_block_association.cidr_block_state.status_message #=> String
    #   resp.vpc_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/DisassociateVpcCidrBlock AWS API Documentation
    #
    # @overload disassociate_vpc_cidr_block(params = {})
    # @param [Hash] params ({})
    def disassociate_vpc_cidr_block(params = {}, options = {})
      req = build_request(:disassociate_vpc_cidr_block, params)
      req.send_request(options)
    end

    # Enables Elastic IP address transfer. For more information, see
    # [Transfer Elastic IP addresses][1] in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/vpc-eips.html#transfer-EIPs-intro
    #
    # @option params [required, String] :allocation_id
    #   The allocation ID of an Elastic IP address.
    #
    # @option params [required, String] :transfer_account_id
    #   The ID of the account that you want to transfer the Elastic IP address
    #   to.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::EnableAddressTransferResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableAddressTransferResult#address_transfer #address_transfer} => Types::AddressTransfer
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_address_transfer({
    #     allocation_id: "AllocationId", # required
    #     transfer_account_id: "String", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.address_transfer.public_ip #=> String
    #   resp.address_transfer.allocation_id #=> String
    #   resp.address_transfer.transfer_account_id #=> String
    #   resp.address_transfer.transfer_offer_expiration_timestamp #=> Time
    #   resp.address_transfer.transfer_offer_accepted_timestamp #=> Time
    #   resp.address_transfer.address_transfer_status #=> String, one of "pending", "disabled", "accepted"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableAddressTransfer AWS API Documentation
    #
    # @overload enable_address_transfer(params = {})
    # @param [Hash] params ({})
    def enable_address_transfer(params = {}, options = {})
      req = build_request(:enable_address_transfer, params)
      req.send_request(options)
    end

    # Enables Allowed AMIs for your account in the specified Amazon Web
    # Services Region. Two values are accepted:
    #
    # * `enabled`: The image criteria in your Allowed AMIs settings are
    #   applied. As a result, only AMIs matching these criteria are
    #   discoverable and can be used by your account to launch instances.
    #
    # * `audit-mode`: The image criteria in your Allowed AMIs settings are
    #   not applied. No restrictions are placed on AMI discoverability or
    #   usage. Users in your account can launch instances using any public
    #   AMI or AMI shared with your account.
    #
    #   The purpose of `audit-mode` is to indicate which AMIs will be
    #   affected when Allowed AMIs is `enabled`. In `audit-mode`, each AMI
    #   displays either `"ImageAllowed": true` or `"ImageAllowed": false` to
    #   indicate whether the AMI will be discoverable and available to users
    #   in the account when Allowed AMIs is enabled.
    #
    # <note markdown="1"> The Allowed AMIs feature does not restrict the AMIs owned by your
    # account. Regardless of the criteria you set, the AMIs created by your
    # account will always be discoverable and usable by users in your
    # account.
    #
    #  </note>
    #
    # For more information, see [Control the discovery and use of AMIs in
    # Amazon EC2 with Allowed AMIs][1] in *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-allowed-amis.html
    #
    # @option params [required, String] :allowed_images_settings_state
    #   Specify `enabled` to apply the image criteria specified by the Allowed
    #   AMIs settings. Specify `audit-mode` so that you can check which AMIs
    #   will be allowed or not allowed by the image criteria.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::EnableAllowedImagesSettingsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableAllowedImagesSettingsResult#allowed_images_settings_state #allowed_images_settings_state} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_allowed_images_settings({
    #     allowed_images_settings_state: "enabled", # required, accepts enabled, audit-mode
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.allowed_images_settings_state #=> String, one of "enabled", "audit-mode"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableAllowedImagesSettings AWS API Documentation
    #
    # @overload enable_allowed_images_settings(params = {})
    # @param [Hash] params ({})
    def enable_allowed_images_settings(params = {}, options = {})
      req = build_request(:enable_allowed_images_settings, params)
      req.send_request(options)
    end

    # Enables Infrastructure Performance subscriptions.
    #
    # @option params [String] :source
    #   The source Region (like `us-east-1`) or Availability Zone ID (like
    #   `use1-az1`) that the metric subscription is enabled for. If you use
    #   Availability Zone IDs, the Source and Destination Availability Zones
    #   must be in the same Region.
    #
    # @option params [String] :destination
    #   The target Region (like `us-east-2`) or Availability Zone ID (like
    #   `use2-az2`) that the metric subscription is enabled for. If you use
    #   Availability Zone IDs, the Source and Destination Availability Zones
    #   must be in the same Region.
    #
    # @option params [String] :metric
    #   The metric used for the enabled subscription.
    #
    # @option params [String] :statistic
    #   The statistic used for the enabled subscription.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::EnableAwsNetworkPerformanceMetricSubscriptionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableAwsNetworkPerformanceMetricSubscriptionResult#output #output} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_aws_network_performance_metric_subscription({
    #     source: "String",
    #     destination: "String",
    #     metric: "aggregate-latency", # accepts aggregate-latency
    #     statistic: "p50", # accepts p50
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.output #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableAwsNetworkPerformanceMetricSubscription AWS API Documentation
    #
    # @overload enable_aws_network_performance_metric_subscription(params = {})
    # @param [Hash] params ({})
    def enable_aws_network_performance_metric_subscription(params = {}, options = {})
      req = build_request(:enable_aws_network_performance_metric_subscription, params)
      req.send_request(options)
    end

    # Enables EBS encryption by default for your account in the current
    # Region.
    #
    # After you enable encryption by default, the EBS volumes that you
    # create are always encrypted, either using the default KMS key or the
    # KMS key that you specified when you created each volume. For more
    # information, see [Amazon EBS encryption][1] in the *Amazon EBS User
    # Guide*.
    #
    # You can specify the default KMS key for encryption by default using
    # ModifyEbsDefaultKmsKeyId or ResetEbsDefaultKmsKeyId.
    #
    # Enabling encryption by default has no effect on the encryption status
    # of your existing volumes.
    #
    # After you enable encryption by default, you can no longer launch
    # instances using instance types that do not support encryption. For
    # more information, see [Supported instance types][2].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-encryption.html
    # [2]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-encryption-requirements.html#ebs-encryption_supported_instances
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::EnableEbsEncryptionByDefaultResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableEbsEncryptionByDefaultResult#ebs_encryption_by_default #ebs_encryption_by_default} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_ebs_encryption_by_default({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.ebs_encryption_by_default #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableEbsEncryptionByDefault AWS API Documentation
    #
    # @overload enable_ebs_encryption_by_default(params = {})
    # @param [Hash] params ({})
    def enable_ebs_encryption_by_default(params = {}, options = {})
      req = build_request(:enable_ebs_encryption_by_default, params)
      req.send_request(options)
    end

    # When you enable Windows fast launch for a Windows AMI, images are
    # pre-provisioned, using snapshots to launch instances up to 65% faster.
    # To create the optimized Windows image, Amazon EC2 launches an instance
    # and runs through Sysprep steps, rebooting as required. Then it creates
    # a set of reserved snapshots that are used for subsequent launches. The
    # reserved snapshots are automatically replenished as they are used,
    # depending on your settings for launch frequency.
    #
    # <note markdown="1"> You can only change these settings for Windows AMIs that you own or
    # that have been shared with you.
    #
    #  </note>
    #
    # @option params [required, String] :image_id
    #   Specify the ID of the image for which to enable Windows fast launch.
    #
    # @option params [String] :resource_type
    #   The type of resource to use for pre-provisioning the AMI for Windows
    #   fast launch. Supported values include: `snapshot`, which is the
    #   default value.
    #
    # @option params [Types::FastLaunchSnapshotConfigurationRequest] :snapshot_configuration
    #   Configuration settings for creating and managing the snapshots that
    #   are used for pre-provisioning the AMI for Windows fast launch. The
    #   associated `ResourceType` must be `snapshot`.
    #
    # @option params [Types::FastLaunchLaunchTemplateSpecificationRequest] :launch_template
    #   The launch template to use when launching Windows instances from
    #   pre-provisioned snapshots. Launch template parameters can include
    #   either the name or ID of the launch template, but not both.
    #
    # @option params [Integer] :max_parallel_launches
    #   The maximum number of instances that Amazon EC2 can launch at the same
    #   time to create pre-provisioned snapshots for Windows fast launch.
    #   Value must be `6` or greater.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::EnableFastLaunchResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableFastLaunchResult#image_id #image_id} => String
    #   * {Types::EnableFastLaunchResult#resource_type #resource_type} => String
    #   * {Types::EnableFastLaunchResult#snapshot_configuration #snapshot_configuration} => Types::FastLaunchSnapshotConfigurationResponse
    #   * {Types::EnableFastLaunchResult#launch_template #launch_template} => Types::FastLaunchLaunchTemplateSpecificationResponse
    #   * {Types::EnableFastLaunchResult#max_parallel_launches #max_parallel_launches} => Integer
    #   * {Types::EnableFastLaunchResult#owner_id #owner_id} => String
    #   * {Types::EnableFastLaunchResult#state #state} => String
    #   * {Types::EnableFastLaunchResult#state_transition_reason #state_transition_reason} => String
    #   * {Types::EnableFastLaunchResult#state_transition_time #state_transition_time} => Time
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_fast_launch({
    #     image_id: "ImageId", # required
    #     resource_type: "String",
    #     snapshot_configuration: {
    #       target_resource_count: 1,
    #     },
    #     launch_template: {
    #       launch_template_id: "LaunchTemplateId",
    #       launch_template_name: "String",
    #       version: "String", # required
    #     },
    #     max_parallel_launches: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.image_id #=> String
    #   resp.resource_type #=> String, one of "snapshot"
    #   resp.snapshot_configuration.target_resource_count #=> Integer
    #   resp.launch_template.launch_template_id #=> String
    #   resp.launch_template.launch_template_name #=> String
    #   resp.launch_template.version #=> String
    #   resp.max_parallel_launches #=> Integer
    #   resp.owner_id #=> String
    #   resp.state #=> String, one of "enabling", "enabling-failed", "enabled", "enabled-failed", "disabling", "disabling-failed"
    #   resp.state_transition_reason #=> String
    #   resp.state_transition_time #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableFastLaunch AWS API Documentation
    #
    # @overload enable_fast_launch(params = {})
    # @param [Hash] params ({})
    def enable_fast_launch(params = {}, options = {})
      req = build_request(:enable_fast_launch, params)
      req.send_request(options)
    end

    # Enables fast snapshot restores for the specified snapshots in the
    # specified Availability Zones.
    #
    # You get the full benefit of fast snapshot restores after they enter
    # the `enabled` state. To get the current state of fast snapshot
    # restores, use DescribeFastSnapshotRestores. To disable fast snapshot
    # restores, use DisableFastSnapshotRestores.
    #
    # For more information, see [Amazon EBS fast snapshot restore][1] in the
    # *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-fast-snapshot-restore.html
    #
    # @option params [required, Array<String>] :availability_zones
    #   One or more Availability Zones. For example, `us-east-2a`.
    #
    # @option params [required, Array<String>] :source_snapshot_ids
    #   The IDs of one or more snapshots. For example,
    #   `snap-1234567890abcdef0`. You can specify a snapshot that was shared
    #   with you from another Amazon Web Services account.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::EnableFastSnapshotRestoresResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableFastSnapshotRestoresResult#successful #successful} => Array&lt;Types::EnableFastSnapshotRestoreSuccessItem&gt;
    #   * {Types::EnableFastSnapshotRestoresResult#unsuccessful #unsuccessful} => Array&lt;Types::EnableFastSnapshotRestoreErrorItem&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_fast_snapshot_restores({
    #     availability_zones: ["String"], # required
    #     source_snapshot_ids: ["SnapshotId"], # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.successful #=> Array
    #   resp.successful[0].snapshot_id #=> String
    #   resp.successful[0].availability_zone #=> String
    #   resp.successful[0].state #=> String, one of "enabling", "optimizing", "enabled", "disabling", "disabled"
    #   resp.successful[0].state_transition_reason #=> String
    #   resp.successful[0].owner_id #=> String
    #   resp.successful[0].owner_alias #=> String
    #   resp.successful[0].enabling_time #=> Time
    #   resp.successful[0].optimizing_time #=> Time
    #   resp.successful[0].enabled_time #=> Time
    #   resp.successful[0].disabling_time #=> Time
    #   resp.successful[0].disabled_time #=> Time
    #   resp.unsuccessful #=> Array
    #   resp.unsuccessful[0].snapshot_id #=> String
    #   resp.unsuccessful[0].fast_snapshot_restore_state_errors #=> Array
    #   resp.unsuccessful[0].fast_snapshot_restore_state_errors[0].availability_zone #=> String
    #   resp.unsuccessful[0].fast_snapshot_restore_state_errors[0].error.code #=> String
    #   resp.unsuccessful[0].fast_snapshot_restore_state_errors[0].error.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableFastSnapshotRestores AWS API Documentation
    #
    # @overload enable_fast_snapshot_restores(params = {})
    # @param [Hash] params ({})
    def enable_fast_snapshot_restores(params = {}, options = {})
      req = build_request(:enable_fast_snapshot_restores, params)
      req.send_request(options)
    end

    # Re-enables a disabled AMI. The re-enabled AMI is marked as `available`
    # and can be used for instance launches, appears in describe operations,
    # and can be shared. Amazon Web Services accounts, organizations, and
    # Organizational Units that lost access to the AMI when it was disabled
    # do not regain access automatically. Once the AMI is available, it can
    # be shared with them again.
    #
    # Only the AMI owner can re-enable a disabled AMI.
    #
    # For more information, see [Disable an AMI][1] in the *Amazon EC2 User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/disable-an-ami.html
    #
    # @option params [required, String] :image_id
    #   The ID of the AMI.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::EnableImageResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableImageResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_image({
    #     image_id: "ImageId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableImage AWS API Documentation
    #
    # @overload enable_image(params = {})
    # @param [Hash] params ({})
    def enable_image(params = {}, options = {})
      req = build_request(:enable_image, params)
      req.send_request(options)
    end

    # Enables *block public access for AMIs* at the account level in the
    # specified Amazon Web Services Region. This prevents the public sharing
    # of your AMIs. However, if you already have public AMIs, they will
    # remain publicly available.
    #
    # The API can take up to 10 minutes to configure this setting. During
    # this time, if you run [GetImageBlockPublicAccessState][1], the
    # response will be `unblocked`. When the API has completed the
    # configuration, the response will be `block-new-sharing`.
    #
    # For more information, see [Block public access to your AMIs][2] in the
    # *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_GetImageBlockPublicAccessState.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/sharingamis-intro.html#block-public-access-to-amis
    #
    # @option params [required, String] :image_block_public_access_state
    #   Specify `block-new-sharing` to enable block public access for AMIs at
    #   the account level in the specified Region. This will block any attempt
    #   to publicly share your AMIs in the specified Region.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::EnableImageBlockPublicAccessResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableImageBlockPublicAccessResult#image_block_public_access_state #image_block_public_access_state} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_image_block_public_access({
    #     image_block_public_access_state: "block-new-sharing", # required, accepts block-new-sharing
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.image_block_public_access_state #=> String, one of "block-new-sharing"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableImageBlockPublicAccess AWS API Documentation
    #
    # @overload enable_image_block_public_access(params = {})
    # @param [Hash] params ({})
    def enable_image_block_public_access(params = {}, options = {})
      req = build_request(:enable_image_block_public_access, params)
      req.send_request(options)
    end

    # Enables deprecation of the specified AMI at the specified date and
    # time.
    #
    # For more information, see [Deprecate an AMI][1] in the *Amazon EC2
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ami-deprecate.html
    #
    # @option params [required, String] :image_id
    #   The ID of the AMI.
    #
    # @option params [required, Time,DateTime,Date,Integer,String] :deprecate_at
    #   The date and time to deprecate the AMI, in UTC, in the following
    #   format: *YYYY*-*MM*-*DD*T*HH*:*MM*:*SS*Z. If you specify a value for
    #   seconds, Amazon EC2 rounds the seconds to the nearest minute.
    #
    #   You can’t specify a date in the past. The upper limit for
    #   `DeprecateAt` is 10 years from now, except for public AMIs, where the
    #   upper limit is 2 years from the creation date.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::EnableImageDeprecationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableImageDeprecationResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_image_deprecation({
    #     image_id: "ImageId", # required
    #     deprecate_at: Time.now, # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableImageDeprecation AWS API Documentation
    #
    # @overload enable_image_deprecation(params = {})
    # @param [Hash] params ({})
    def enable_image_deprecation(params = {}, options = {})
      req = build_request(:enable_image_deprecation, params)
      req.send_request(options)
    end

    # Enables deregistration protection for an AMI. When deregistration
    # protection is enabled, the AMI can't be deregistered.
    #
    # To allow the AMI to be deregistered, you must first disable
    # deregistration protection using DisableImageDeregistrationProtection.
    #
    # For more information, see [Protect an AMI from deregistration][1] in
    # the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/deregister-ami.html#ami-deregistration-protection
    #
    # @option params [required, String] :image_id
    #   The ID of the AMI.
    #
    # @option params [Boolean] :with_cooldown
    #   If `true`, enforces deregistration protection for 24 hours after
    #   deregistration protection is disabled.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::EnableImageDeregistrationProtectionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableImageDeregistrationProtectionResult#return #return} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_image_deregistration_protection({
    #     image_id: "ImageId", # required
    #     with_cooldown: false,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableImageDeregistrationProtection AWS API Documentation
    #
    # @overload enable_image_deregistration_protection(params = {})
    # @param [Hash] params ({})
    def enable_image_deregistration_protection(params = {}, options = {})
      req = build_request(:enable_image_deregistration_protection, params)
      req.send_request(options)
    end

    # Enable an Organizations member account as the IPAM admin account. You
    # cannot select the Organizations management account as the IPAM admin
    # account. For more information, see [Enable integration with
    # Organizations][1] in the *Amazon VPC IPAM User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/enable-integ-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :delegated_admin_account_id
    #   The Organizations member account ID that you want to enable as the
    #   IPAM account.
    #
    # @return [Types::EnableIpamOrganizationAdminAccountResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableIpamOrganizationAdminAccountResult#success #success} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_ipam_organization_admin_account({
    #     dry_run: false,
    #     delegated_admin_account_id: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.success #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableIpamOrganizationAdminAccount AWS API Documentation
    #
    # @overload enable_ipam_organization_admin_account(params = {})
    # @param [Hash] params ({})
    def enable_ipam_organization_admin_account(params = {}, options = {})
      req = build_request(:enable_ipam_organization_admin_account, params)
      req.send_request(options)
    end

    # Establishes a trust relationship between Reachability Analyzer and
    # Organizations. This operation must be performed by the management
    # account for the organization.
    #
    # After you establish a trust relationship, a user in the management
    # account or a delegated administrator account can run a cross-account
    # analysis using resources from the member accounts.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::EnableReachabilityAnalyzerOrganizationSharingResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableReachabilityAnalyzerOrganizationSharingResult#return_value #return_value} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_reachability_analyzer_organization_sharing({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return_value #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableReachabilityAnalyzerOrganizationSharing AWS API Documentation
    #
    # @overload enable_reachability_analyzer_organization_sharing(params = {})
    # @param [Hash] params ({})
    def enable_reachability_analyzer_organization_sharing(params = {}, options = {})
      req = build_request(:enable_reachability_analyzer_organization_sharing, params)
      req.send_request(options)
    end

    # Enables access to the EC2 serial console of all instances for your
    # account. By default, access to the EC2 serial console is disabled for
    # your account. For more information, see [Manage account access to the
    # EC2 serial console][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/configure-access-to-serial-console.html#serial-console-account-access
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::EnableSerialConsoleAccessResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableSerialConsoleAccessResult#serial_console_access_enabled #serial_console_access_enabled} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_serial_console_access({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.serial_console_access_enabled #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableSerialConsoleAccess AWS API Documentation
    #
    # @overload enable_serial_console_access(params = {})
    # @param [Hash] params ({})
    def enable_serial_console_access(params = {}, options = {})
      req = build_request(:enable_serial_console_access, params)
      req.send_request(options)
    end

    # Enables or modifies the *block public access for snapshots* setting at
    # the account level for the specified Amazon Web Services Region. After
    # you enable block public access for snapshots in a Region, users can no
    # longer request public sharing for snapshots in that Region. Snapshots
    # that are already publicly shared are either treated as private or they
    # remain publicly shared, depending on the **State** that you specify.
    #
    # Enabling block public access for snapshots in *block all sharing* mode
    # does not change the permissions for snapshots that are already
    # publicly shared. Instead, it prevents these snapshots from be publicly
    # visible and publicly accessible. Therefore, the attributes for these
    # snapshots still indicate that they are publicly shared, even though
    # they are not publicly available.
    #
    #  If you later disable block public access or change the mode to *block
    # new sharing*, these snapshots will become publicly available again.
    #
    # For more information, see [ Block public access for snapshots][1] in
    # the *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/block-public-access-snapshots.html
    #
    # @option params [required, String] :state
    #   The mode in which to enable block public access for snapshots for the
    #   Region. Specify one of the following values:
    #
    #   * `block-all-sharing` - Prevents all public sharing of snapshots in
    #     the Region. Users in the account will no longer be able to request
    #     new public sharing. Additionally, snapshots that are already
    #     publicly shared are treated as private and they are no longer
    #     publicly available.
    #
    #   * `block-new-sharing` - Prevents only new public sharing of snapshots
    #     in the Region. Users in the account will no longer be able to
    #     request new public sharing. However, snapshots that are already
    #     publicly shared, remain publicly available.
    #
    #   `unblocked` is not a valid value for
    #   **EnableSnapshotBlockPublicAccess**.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::EnableSnapshotBlockPublicAccessResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableSnapshotBlockPublicAccessResult#state #state} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_snapshot_block_public_access({
    #     state: "block-all-sharing", # required, accepts block-all-sharing, block-new-sharing, unblocked
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.state #=> String, one of "block-all-sharing", "block-new-sharing", "unblocked"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableSnapshotBlockPublicAccess AWS API Documentation
    #
    # @overload enable_snapshot_block_public_access(params = {})
    # @param [Hash] params ({})
    def enable_snapshot_block_public_access(params = {}, options = {})
      req = build_request(:enable_snapshot_block_public_access, params)
      req.send_request(options)
    end

    # Enables the specified attachment to propagate routes to the specified
    # propagation route table.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the propagation route table.
    #
    # @option params [String] :transit_gateway_attachment_id
    #   The ID of the attachment.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :transit_gateway_route_table_announcement_id
    #   The ID of the transit gateway route table announcement.
    #
    # @return [Types::EnableTransitGatewayRouteTablePropagationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableTransitGatewayRouteTablePropagationResult#propagation #propagation} => Types::TransitGatewayPropagation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_transit_gateway_route_table_propagation({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId",
    #     dry_run: false,
    #     transit_gateway_route_table_announcement_id: "TransitGatewayRouteTableAnnouncementId",
    #   })
    #
    # @example Response structure
    #
    #   resp.propagation.transit_gateway_attachment_id #=> String
    #   resp.propagation.resource_id #=> String
    #   resp.propagation.resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.propagation.transit_gateway_route_table_id #=> String
    #   resp.propagation.state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.propagation.transit_gateway_route_table_announcement_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableTransitGatewayRouteTablePropagation AWS API Documentation
    #
    # @overload enable_transit_gateway_route_table_propagation(params = {})
    # @param [Hash] params ({})
    def enable_transit_gateway_route_table_propagation(params = {}, options = {})
      req = build_request(:enable_transit_gateway_route_table_propagation, params)
      req.send_request(options)
    end

    # Enables a virtual private gateway (VGW) to propagate routes to the
    # specified route table of a VPC.
    #
    # @option params [required, String] :gateway_id
    #   The ID of the virtual private gateway that is attached to a VPC. The
    #   virtual private gateway must be attached to the same VPC that the
    #   routing tables are associated with.
    #
    # @option params [required, String] :route_table_id
    #   The ID of the route table. The routing table must be associated with
    #   the same VPC that the virtual private gateway is attached to.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To enable route propagation
    #
    #   # This example enables the specified virtual private gateway to propagate static routes to the specified route table.
    #
    #   resp = client.enable_vgw_route_propagation({
    #     gateway_id: "vgw-9a4cacf3", 
    #     route_table_id: "rtb-22574640", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_vgw_route_propagation({
    #     gateway_id: "VpnGatewayId", # required
    #     route_table_id: "RouteTableId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableVgwRoutePropagation AWS API Documentation
    #
    # @overload enable_vgw_route_propagation(params = {})
    # @param [Hash] params ({})
    def enable_vgw_route_propagation(params = {}, options = {})
      req = build_request(:enable_vgw_route_propagation, params)
      req.send_request(options)
    end

    # Enables I/O operations for a volume that had I/O operations disabled
    # because the data on the volume was potentially inconsistent.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :volume_id
    #   The ID of the volume.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To enable I/O for a volume
    #
    #   # This example enables I/O on volume ``vol-1234567890abcdef0``.
    #
    #   resp = client.enable_volume_io({
    #     volume_id: "vol-1234567890abcdef0", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_volume_io({
    #     dry_run: false,
    #     volume_id: "VolumeId", # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableVolumeIO AWS API Documentation
    #
    # @overload enable_volume_io(params = {})
    # @param [Hash] params ({})
    def enable_volume_io(params = {}, options = {})
      req = build_request(:enable_volume_io, params)
      req.send_request(options)
    end

    # <note markdown="1"> This action is deprecated.
    #
    #  </note>
    #
    # Enables a VPC for ClassicLink. You can then link EC2-Classic instances
    # to your ClassicLink-enabled VPC to allow communication over private IP
    # addresses. You cannot enable your VPC for ClassicLink if any of your
    # VPC route tables have existing routes for address ranges within the
    # `10.0.0.0/8` IP address range, excluding local routes for VPCs in the
    # `10.0.0.0/16` and `10.1.0.0/16` IP address ranges.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @return [Types::EnableVpcClassicLinkResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableVpcClassicLinkResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_vpc_classic_link({
    #     dry_run: false,
    #     vpc_id: "VpcId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableVpcClassicLink AWS API Documentation
    #
    # @overload enable_vpc_classic_link(params = {})
    # @param [Hash] params ({})
    def enable_vpc_classic_link(params = {}, options = {})
      req = build_request(:enable_vpc_classic_link, params)
      req.send_request(options)
    end

    # <note markdown="1"> This action is deprecated.
    #
    #  </note>
    #
    # Enables a VPC to support DNS hostname resolution for ClassicLink. If
    # enabled, the DNS hostname of a linked EC2-Classic instance resolves to
    # its private IP address when addressed from an instance in the VPC to
    # which it's linked. Similarly, the DNS hostname of an instance in a
    # VPC resolves to its private IP address when addressed from a linked
    # EC2-Classic instance.
    #
    # You must specify a VPC ID in the request.
    #
    # @option params [String] :vpc_id
    #   The ID of the VPC.
    #
    # @return [Types::EnableVpcClassicLinkDnsSupportResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::EnableVpcClassicLinkDnsSupportResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.enable_vpc_classic_link_dns_support({
    #     vpc_id: "VpcId",
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/EnableVpcClassicLinkDnsSupport AWS API Documentation
    #
    # @overload enable_vpc_classic_link_dns_support(params = {})
    # @param [Hash] params ({})
    def enable_vpc_classic_link_dns_support(params = {}, options = {})
      req = build_request(:enable_vpc_classic_link_dns_support, params)
      req.send_request(options)
    end

    # Downloads the client certificate revocation list for the specified
    # Client VPN endpoint.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ExportClientVpnClientCertificateRevocationListResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ExportClientVpnClientCertificateRevocationListResult#certificate_revocation_list #certificate_revocation_list} => String
    #   * {Types::ExportClientVpnClientCertificateRevocationListResult#status #status} => Types::ClientCertificateRevocationListStatus
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.export_client_vpn_client_certificate_revocation_list({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.certificate_revocation_list #=> String
    #   resp.status.code #=> String, one of "pending", "active"
    #   resp.status.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ExportClientVpnClientCertificateRevocationList AWS API Documentation
    #
    # @overload export_client_vpn_client_certificate_revocation_list(params = {})
    # @param [Hash] params ({})
    def export_client_vpn_client_certificate_revocation_list(params = {}, options = {})
      req = build_request(:export_client_vpn_client_certificate_revocation_list, params)
      req.send_request(options)
    end

    # Downloads the contents of the Client VPN endpoint configuration file
    # for the specified Client VPN endpoint. The Client VPN endpoint
    # configuration file includes the Client VPN endpoint and certificate
    # information clients need to establish a connection with the Client VPN
    # endpoint.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ExportClientVpnClientConfigurationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ExportClientVpnClientConfigurationResult#client_configuration #client_configuration} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.export_client_vpn_client_configuration({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.client_configuration #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ExportClientVpnClientConfiguration AWS API Documentation
    #
    # @overload export_client_vpn_client_configuration(params = {})
    # @param [Hash] params ({})
    def export_client_vpn_client_configuration(params = {}, options = {})
      req = build_request(:export_client_vpn_client_configuration, params)
      req.send_request(options)
    end

    # Exports an Amazon Machine Image (AMI) to a VM file. For more
    # information, see [Exporting a VM directly from an Amazon Machine Image
    # (AMI)][1] in the *VM Import/Export User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vm-import/latest/userguide/vmexport_image.html
    #
    # @option params [String] :client_token
    #   Token to enable idempotency for export image requests.
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    # @option params [String] :description
    #   A description of the image being exported. The maximum length is 255
    #   characters.
    #
    # @option params [required, String] :disk_image_format
    #   The disk image format.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :image_id
    #   The ID of the image.
    #
    # @option params [required, Types::ExportTaskS3LocationRequest] :s3_export_location
    #   The Amazon S3 bucket for the destination image. The destination bucket
    #   must exist.
    #
    # @option params [String] :role_name
    #   The name of the role that grants VM Import/Export permission to export
    #   images to your Amazon S3 bucket. If this parameter is not specified,
    #   the default role is named 'vmimport'.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the export image task during creation.
    #
    # @return [Types::ExportImageResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ExportImageResult#description #description} => String
    #   * {Types::ExportImageResult#disk_image_format #disk_image_format} => String
    #   * {Types::ExportImageResult#export_image_task_id #export_image_task_id} => String
    #   * {Types::ExportImageResult#image_id #image_id} => String
    #   * {Types::ExportImageResult#role_name #role_name} => String
    #   * {Types::ExportImageResult#progress #progress} => String
    #   * {Types::ExportImageResult#s3_export_location #s3_export_location} => Types::ExportTaskS3Location
    #   * {Types::ExportImageResult#status #status} => String
    #   * {Types::ExportImageResult#status_message #status_message} => String
    #   * {Types::ExportImageResult#tags #tags} => Array&lt;Types::Tag&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.export_image({
    #     client_token: "String",
    #     description: "String",
    #     disk_image_format: "VMDK", # required, accepts VMDK, RAW, VHD
    #     dry_run: false,
    #     image_id: "ImageId", # required
    #     s3_export_location: { # required
    #       s3_bucket: "String", # required
    #       s3_prefix: "String",
    #     },
    #     role_name: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.description #=> String
    #   resp.disk_image_format #=> String, one of "VMDK", "RAW", "VHD"
    #   resp.export_image_task_id #=> String
    #   resp.image_id #=> String
    #   resp.role_name #=> String
    #   resp.progress #=> String
    #   resp.s3_export_location.s3_bucket #=> String
    #   resp.s3_export_location.s3_prefix #=> String
    #   resp.status #=> String
    #   resp.status_message #=> String
    #   resp.tags #=> Array
    #   resp.tags[0].key #=> String
    #   resp.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ExportImage AWS API Documentation
    #
    # @overload export_image(params = {})
    # @param [Hash] params ({})
    def export_image(params = {}, options = {})
      req = build_request(:export_image, params)
      req.send_request(options)
    end

    # Exports routes from the specified transit gateway route table to the
    # specified S3 bucket. By default, all routes are exported.
    # Alternatively, you can filter by CIDR range.
    #
    # The routes are saved to the specified bucket in a JSON file. For more
    # information, see [Export route tables to Amazon S3][1] in the *Amazon
    # Web Services Transit Gateways Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/tgw/tgw-route-tables.html#tgw-export-route-tables
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the route table.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `attachment.transit-gateway-attachment-id` - The id of the transit
    #     gateway attachment.
    #
    #   * `attachment.resource-id` - The resource id of the transit gateway
    #     attachment.
    #
    #   * `route-search.exact-match` - The exact match of the specified
    #     filter.
    #
    #   * `route-search.longest-prefix-match` - The longest prefix that
    #     matches the route.
    #
    #   * `route-search.subnet-of-match` - The routes with a subnet that match
    #     the specified CIDR filter.
    #
    #   * `route-search.supernet-of-match` - The routes with a CIDR that
    #     encompass the CIDR filter. For example, if you have 10.0.1.0/29 and
    #     10.0.1.0/31 routes in your route table and you specify
    #     supernet-of-match as 10.0.1.0/30, then the result returns
    #     10.0.1.0/29.
    #
    #   * `state` - The state of the route (`active` \| `blackhole`).
    #
    #   * `transit-gateway-route-destination-cidr-block` - The CIDR range.
    #
    #   * `type` - The type of route (`propagated` \| `static`).
    #
    # @option params [required, String] :s3_bucket
    #   The name of the S3 bucket.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ExportTransitGatewayRoutesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ExportTransitGatewayRoutesResult#s3_location #s3_location} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.export_transit_gateway_routes({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     s3_bucket: "String", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.s3_location #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ExportTransitGatewayRoutes AWS API Documentation
    #
    # @overload export_transit_gateway_routes(params = {})
    # @param [Hash] params ({})
    def export_transit_gateway_routes(params = {}, options = {})
      req = build_request(:export_transit_gateway_routes, params)
      req.send_request(options)
    end

    # Exports the client configuration for a Verified Access instance.
    #
    # @option params [required, String] :verified_access_instance_id
    #   The ID of the Verified Access instance.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ExportVerifiedAccessInstanceClientConfigurationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ExportVerifiedAccessInstanceClientConfigurationResult#version #version} => String
    #   * {Types::ExportVerifiedAccessInstanceClientConfigurationResult#verified_access_instance_id #verified_access_instance_id} => String
    #   * {Types::ExportVerifiedAccessInstanceClientConfigurationResult#region #region} => String
    #   * {Types::ExportVerifiedAccessInstanceClientConfigurationResult#device_trust_providers #device_trust_providers} => Array&lt;String&gt;
    #   * {Types::ExportVerifiedAccessInstanceClientConfigurationResult#user_trust_provider #user_trust_provider} => Types::VerifiedAccessInstanceUserTrustProviderClientConfiguration
    #   * {Types::ExportVerifiedAccessInstanceClientConfigurationResult#open_vpn_configurations #open_vpn_configurations} => Array&lt;Types::VerifiedAccessInstanceOpenVpnClientConfiguration&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.export_verified_access_instance_client_configuration({
    #     verified_access_instance_id: "VerifiedAccessInstanceId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.version #=> String
    #   resp.verified_access_instance_id #=> String
    #   resp.region #=> String
    #   resp.device_trust_providers #=> Array
    #   resp.device_trust_providers[0] #=> String, one of "jamf", "crowdstrike", "jumpcloud"
    #   resp.user_trust_provider.type #=> String, one of "iam-identity-center", "oidc"
    #   resp.user_trust_provider.scopes #=> String
    #   resp.user_trust_provider.issuer #=> String
    #   resp.user_trust_provider.authorization_endpoint #=> String
    #   resp.user_trust_provider.public_signing_key_endpoint #=> String
    #   resp.user_trust_provider.token_endpoint #=> String
    #   resp.user_trust_provider.user_info_endpoint #=> String
    #   resp.user_trust_provider.client_id #=> String
    #   resp.user_trust_provider.client_secret #=> String
    #   resp.user_trust_provider.pkce_enabled #=> Boolean
    #   resp.open_vpn_configurations #=> Array
    #   resp.open_vpn_configurations[0].config #=> String
    #   resp.open_vpn_configurations[0].routes #=> Array
    #   resp.open_vpn_configurations[0].routes[0].cidr #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ExportVerifiedAccessInstanceClientConfiguration AWS API Documentation
    #
    # @overload export_verified_access_instance_client_configuration(params = {})
    # @param [Hash] params ({})
    def export_verified_access_instance_client_configuration(params = {}, options = {})
      req = build_request(:export_verified_access_instance_client_configuration, params)
      req.send_request(options)
    end

    # Gets the current state of the Allowed AMIs setting and the list of
    # Allowed AMIs criteria at the account level in the specified Region.
    #
    # <note markdown="1"> The Allowed AMIs feature does not restrict the AMIs owned by your
    # account. Regardless of the criteria you set, the AMIs created by your
    # account will always be discoverable and usable by users in your
    # account.
    #
    #  </note>
    #
    # For more information, see [Control the discovery and use of AMIs in
    # Amazon EC2 with Allowed AMIs][1] in *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-allowed-amis.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetAllowedImagesSettingsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetAllowedImagesSettingsResult#state #state} => String
    #   * {Types::GetAllowedImagesSettingsResult#image_criteria #image_criteria} => Array&lt;Types::ImageCriterion&gt;
    #   * {Types::GetAllowedImagesSettingsResult#managed_by #managed_by} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_allowed_images_settings({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.state #=> String
    #   resp.image_criteria #=> Array
    #   resp.image_criteria[0].image_providers #=> Array
    #   resp.image_criteria[0].image_providers[0] #=> String
    #   resp.managed_by #=> String, one of "account", "declarative-policy"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetAllowedImagesSettings AWS API Documentation
    #
    # @overload get_allowed_images_settings(params = {})
    # @param [Hash] params ({})
    def get_allowed_images_settings(params = {}, options = {})
      req = build_request(:get_allowed_images_settings, params)
      req.send_request(options)
    end

    # Returns the IAM roles that are associated with the specified ACM (ACM)
    # certificate. It also returns the name of the Amazon S3 bucket and the
    # Amazon S3 object key where the certificate, certificate chain, and
    # encrypted private key bundle are stored, and the ARN of the KMS key
    # that's used to encrypt the private key.
    #
    # @option params [required, String] :certificate_arn
    #   The ARN of the ACM certificate for which to view the associated IAM
    #   roles, encryption keys, and Amazon S3 object information.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetAssociatedEnclaveCertificateIamRolesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetAssociatedEnclaveCertificateIamRolesResult#associated_roles #associated_roles} => Array&lt;Types::AssociatedRole&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_associated_enclave_certificate_iam_roles({
    #     certificate_arn: "CertificateId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.associated_roles #=> Array
    #   resp.associated_roles[0].associated_role_arn #=> String
    #   resp.associated_roles[0].certificate_s3_bucket_name #=> String
    #   resp.associated_roles[0].certificate_s3_object_key #=> String
    #   resp.associated_roles[0].encryption_kms_key_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetAssociatedEnclaveCertificateIamRoles AWS API Documentation
    #
    # @overload get_associated_enclave_certificate_iam_roles(params = {})
    # @param [Hash] params ({})
    def get_associated_enclave_certificate_iam_roles(params = {}, options = {})
      req = build_request(:get_associated_enclave_certificate_iam_roles, params)
      req.send_request(options)
    end

    # Gets information about the IPv6 CIDR block associations for a
    # specified IPv6 address pool.
    #
    # @option params [required, String] :pool_id
    #   The ID of the IPv6 address pool.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetAssociatedIpv6PoolCidrsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetAssociatedIpv6PoolCidrsResult#ipv_6_cidr_associations #ipv_6_cidr_associations} => Array&lt;Types::Ipv6CidrAssociation&gt;
    #   * {Types::GetAssociatedIpv6PoolCidrsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_associated_ipv_6_pool_cidrs({
    #     pool_id: "Ipv6PoolEc2Id", # required
    #     next_token: "NextToken",
    #     max_results: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.ipv_6_cidr_associations #=> Array
    #   resp.ipv_6_cidr_associations[0].ipv_6_cidr #=> String
    #   resp.ipv_6_cidr_associations[0].associated_resource #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetAssociatedIpv6PoolCidrs AWS API Documentation
    #
    # @overload get_associated_ipv_6_pool_cidrs(params = {})
    # @param [Hash] params ({})
    def get_associated_ipv_6_pool_cidrs(params = {}, options = {})
      req = build_request(:get_associated_ipv_6_pool_cidrs, params)
      req.send_request(options)
    end

    # Gets network performance data.
    #
    # @option params [Array<Types::DataQuery>] :data_queries
    #   A list of network performance data queries.
    #
    # @option params [Time,DateTime,Date,Integer,String] :start_time
    #   The starting time for the performance data request. The starting time
    #   must be formatted as `yyyy-mm-ddThh:mm:ss`. For example,
    #   `2022-06-10T12:00:00.000Z`.
    #
    # @option params [Time,DateTime,Date,Integer,String] :end_time
    #   The ending time for the performance data request. The end time must be
    #   formatted as `yyyy-mm-ddThh:mm:ss`. For example,
    #   `2022-06-12T12:00:00.000Z`.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetAwsNetworkPerformanceDataResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetAwsNetworkPerformanceDataResult#data_responses #data_responses} => Array&lt;Types::DataResponse&gt;
    #   * {Types::GetAwsNetworkPerformanceDataResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_aws_network_performance_data({
    #     data_queries: [
    #       {
    #         id: "String",
    #         source: "String",
    #         destination: "String",
    #         metric: "aggregate-latency", # accepts aggregate-latency
    #         statistic: "p50", # accepts p50
    #         period: "five-minutes", # accepts five-minutes, fifteen-minutes, one-hour, three-hours, one-day, one-week
    #       },
    #     ],
    #     start_time: Time.now,
    #     end_time: Time.now,
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.data_responses #=> Array
    #   resp.data_responses[0].id #=> String
    #   resp.data_responses[0].source #=> String
    #   resp.data_responses[0].destination #=> String
    #   resp.data_responses[0].metric #=> String, one of "aggregate-latency"
    #   resp.data_responses[0].statistic #=> String, one of "p50"
    #   resp.data_responses[0].period #=> String, one of "five-minutes", "fifteen-minutes", "one-hour", "three-hours", "one-day", "one-week"
    #   resp.data_responses[0].metric_points #=> Array
    #   resp.data_responses[0].metric_points[0].start_date #=> Time
    #   resp.data_responses[0].metric_points[0].end_date #=> Time
    #   resp.data_responses[0].metric_points[0].value #=> Float
    #   resp.data_responses[0].metric_points[0].status #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetAwsNetworkPerformanceData AWS API Documentation
    #
    # @overload get_aws_network_performance_data(params = {})
    # @param [Hash] params ({})
    def get_aws_network_performance_data(params = {}, options = {})
      req = build_request(:get_aws_network_performance_data, params)
      req.send_request(options)
    end

    # Gets usage information about a Capacity Reservation. If the Capacity
    # Reservation is shared, it shows usage information for the Capacity
    # Reservation owner and each Amazon Web Services account that is
    # currently using the shared capacity. If the Capacity Reservation is
    # not shared, it shows only the Capacity Reservation owner's usage.
    #
    # @option params [required, String] :capacity_reservation_id
    #   The ID of the Capacity Reservation.
    #
    # @option params [String] :next_token
    #   The token to use to retrieve the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetCapacityReservationUsageResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetCapacityReservationUsageResult#next_token #next_token} => String
    #   * {Types::GetCapacityReservationUsageResult#capacity_reservation_id #capacity_reservation_id} => String
    #   * {Types::GetCapacityReservationUsageResult#instance_type #instance_type} => String
    #   * {Types::GetCapacityReservationUsageResult#total_instance_count #total_instance_count} => Integer
    #   * {Types::GetCapacityReservationUsageResult#available_instance_count #available_instance_count} => Integer
    #   * {Types::GetCapacityReservationUsageResult#state #state} => String
    #   * {Types::GetCapacityReservationUsageResult#instance_usages #instance_usages} => Array&lt;Types::InstanceUsage&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_capacity_reservation_usage({
    #     capacity_reservation_id: "CapacityReservationId", # required
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.capacity_reservation_id #=> String
    #   resp.instance_type #=> String
    #   resp.total_instance_count #=> Integer
    #   resp.available_instance_count #=> Integer
    #   resp.state #=> String, one of "active", "expired", "cancelled", "pending", "failed", "scheduled", "payment-pending", "payment-failed", "assessing", "delayed", "unsupported"
    #   resp.instance_usages #=> Array
    #   resp.instance_usages[0].account_id #=> String
    #   resp.instance_usages[0].used_instance_count #=> Integer
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetCapacityReservationUsage AWS API Documentation
    #
    # @overload get_capacity_reservation_usage(params = {})
    # @param [Hash] params ({})
    def get_capacity_reservation_usage(params = {}, options = {})
      req = build_request(:get_capacity_reservation_usage, params)
      req.send_request(options)
    end

    # Describes the allocations from the specified customer-owned address
    # pool.
    #
    # @option params [required, String] :pool_id
    #   The ID of the address pool.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `coip-address-usage.allocation-id` - The allocation ID of the
    #     address.
    #
    #   * `coip-address-usage.aws-account-id` - The ID of the Amazon Web
    #     Services account that is using the customer-owned IP address.
    #
    #   * `coip-address-usage.aws-service` - The Amazon Web Services service
    #     that is using the customer-owned IP address.
    #
    #   * `coip-address-usage.co-ip` - The customer-owned IP address.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetCoipPoolUsageResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetCoipPoolUsageResult#coip_pool_id #coip_pool_id} => String
    #   * {Types::GetCoipPoolUsageResult#coip_address_usages #coip_address_usages} => Array&lt;Types::CoipAddressUsage&gt;
    #   * {Types::GetCoipPoolUsageResult#local_gateway_route_table_id #local_gateway_route_table_id} => String
    #   * {Types::GetCoipPoolUsageResult#next_token #next_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_coip_pool_usage({
    #     pool_id: "Ipv4PoolCoipId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.coip_pool_id #=> String
    #   resp.coip_address_usages #=> Array
    #   resp.coip_address_usages[0].allocation_id #=> String
    #   resp.coip_address_usages[0].aws_account_id #=> String
    #   resp.coip_address_usages[0].aws_service #=> String
    #   resp.coip_address_usages[0].co_ip #=> String
    #   resp.local_gateway_route_table_id #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetCoipPoolUsage AWS API Documentation
    #
    # @overload get_coip_pool_usage(params = {})
    # @param [Hash] params ({})
    def get_coip_pool_usage(params = {}, options = {})
      req = build_request(:get_coip_pool_usage, params)
      req.send_request(options)
    end

    # Gets the console output for the specified instance. For Linux
    # instances, the instance console output displays the exact console
    # output that would normally be displayed on a physical monitor attached
    # to a computer. For Windows instances, the instance console output
    # includes the last three system event log errors.
    #
    # For more information, see [Instance console output][1] in the *Amazon
    # EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-console.html#instance-console-console-output
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @option params [Boolean] :latest
    #   When enabled, retrieves the latest console output for the instance.
    #
    #   Default: disabled (`false`)
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetConsoleOutputResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetConsoleOutputResult#instance_id #instance_id} => String
    #   * {Types::GetConsoleOutputResult#timestamp #timestamp} => Time
    #   * {Types::GetConsoleOutputResult#output #output} => String
    #
    #
    # @example Example: To get the console output
    #
    #   # This example gets the console output for the specified instance.
    #
    #   resp = client.get_console_output({
    #     instance_id: "i-1234567890abcdef0", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     instance_id: "i-1234567890abcdef0", 
    #     output: "...", 
    #     timestamp: Time.parse("2018-05-25T21:23:53.000Z"), 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_console_output({
    #     instance_id: "InstanceId", # required
    #     latest: false,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_id #=> String
    #   resp.timestamp #=> Time
    #   resp.output #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetConsoleOutput AWS API Documentation
    #
    # @overload get_console_output(params = {})
    # @param [Hash] params ({})
    def get_console_output(params = {}, options = {})
      req = build_request(:get_console_output, params)
      req.send_request(options)
    end

    # Retrieve a JPG-format screenshot of a running instance to help with
    # troubleshooting.
    #
    # The returned content is Base64-encoded.
    #
    # For more information, see [Instance console output][1] in the *Amazon
    # EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/troubleshoot-unreachable-instance.html#instance-console-console-output
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @option params [Boolean] :wake_up
    #   When set to `true`, acts as keystroke input and wakes up an instance
    #   that's in standby or "sleep" mode.
    #
    # @return [Types::GetConsoleScreenshotResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetConsoleScreenshotResult#image_data #image_data} => String
    #   * {Types::GetConsoleScreenshotResult#instance_id #instance_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_console_screenshot({
    #     dry_run: false,
    #     instance_id: "InstanceId", # required
    #     wake_up: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.image_data #=> String
    #   resp.instance_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetConsoleScreenshot AWS API Documentation
    #
    # @overload get_console_screenshot(params = {})
    # @param [Hash] params ({})
    def get_console_screenshot(params = {}, options = {})
      req = build_request(:get_console_screenshot, params)
      req.send_request(options)
    end

    # Retrieves a summary of the account status report.
    #
    # To view the full report, download it from the Amazon S3 bucket where
    # it was saved. Reports are accessible only when they have the
    # `complete` status. Reports with other statuses (`running`,
    # `cancelled`, or `error`) are not available in the S3 bucket. For more
    # information about downloading objects from an S3 bucket, see
    # [Downloading objects][1] in the *Amazon Simple Storage Service User
    # Guide*.
    #
    # For more information, see [Generating the account status report for
    # declarative policies][2] in the *Amazon Web Services Organizations
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AmazonS3/latest/userguide/download-objects.html
    # [2]: https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_policies_declarative_status-report.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :report_id
    #   The ID of the report.
    #
    # @return [Types::GetDeclarativePoliciesReportSummaryResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetDeclarativePoliciesReportSummaryResult#report_id #report_id} => String
    #   * {Types::GetDeclarativePoliciesReportSummaryResult#s3_bucket #s3_bucket} => String
    #   * {Types::GetDeclarativePoliciesReportSummaryResult#s3_prefix #s3_prefix} => String
    #   * {Types::GetDeclarativePoliciesReportSummaryResult#target_id #target_id} => String
    #   * {Types::GetDeclarativePoliciesReportSummaryResult#start_time #start_time} => Time
    #   * {Types::GetDeclarativePoliciesReportSummaryResult#end_time #end_time} => Time
    #   * {Types::GetDeclarativePoliciesReportSummaryResult#number_of_accounts #number_of_accounts} => Integer
    #   * {Types::GetDeclarativePoliciesReportSummaryResult#number_of_failed_accounts #number_of_failed_accounts} => Integer
    #   * {Types::GetDeclarativePoliciesReportSummaryResult#attribute_summaries #attribute_summaries} => Array&lt;Types::AttributeSummary&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_declarative_policies_report_summary({
    #     dry_run: false,
    #     report_id: "DeclarativePoliciesReportId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.report_id #=> String
    #   resp.s3_bucket #=> String
    #   resp.s3_prefix #=> String
    #   resp.target_id #=> String
    #   resp.start_time #=> Time
    #   resp.end_time #=> Time
    #   resp.number_of_accounts #=> Integer
    #   resp.number_of_failed_accounts #=> Integer
    #   resp.attribute_summaries #=> Array
    #   resp.attribute_summaries[0].attribute_name #=> String
    #   resp.attribute_summaries[0].most_frequent_value #=> String
    #   resp.attribute_summaries[0].number_of_matched_accounts #=> Integer
    #   resp.attribute_summaries[0].number_of_unmatched_accounts #=> Integer
    #   resp.attribute_summaries[0].regional_summaries #=> Array
    #   resp.attribute_summaries[0].regional_summaries[0].region_name #=> String
    #   resp.attribute_summaries[0].regional_summaries[0].number_of_matched_accounts #=> Integer
    #   resp.attribute_summaries[0].regional_summaries[0].number_of_unmatched_accounts #=> Integer
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetDeclarativePoliciesReportSummary AWS API Documentation
    #
    # @overload get_declarative_policies_report_summary(params = {})
    # @param [Hash] params ({})
    def get_declarative_policies_report_summary(params = {}, options = {})
      req = build_request(:get_declarative_policies_report_summary, params)
      req.send_request(options)
    end

    # Describes the default credit option for CPU usage of a burstable
    # performance instance family.
    #
    # For more information, see [Burstable performance instances][1] in the
    # *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/burstable-performance-instances.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_family
    #   The instance family.
    #
    # @return [Types::GetDefaultCreditSpecificationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetDefaultCreditSpecificationResult#instance_family_credit_specification #instance_family_credit_specification} => Types::InstanceFamilyCreditSpecification
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_default_credit_specification({
    #     dry_run: false,
    #     instance_family: "t2", # required, accepts t2, t3, t3a, t4g
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_family_credit_specification.instance_family #=> String, one of "t2", "t3", "t3a", "t4g"
    #   resp.instance_family_credit_specification.cpu_credits #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetDefaultCreditSpecification AWS API Documentation
    #
    # @overload get_default_credit_specification(params = {})
    # @param [Hash] params ({})
    def get_default_credit_specification(params = {}, options = {})
      req = build_request(:get_default_credit_specification, params)
      req.send_request(options)
    end

    # Describes the default KMS key for EBS encryption by default for your
    # account in this Region. You can change the default KMS key for
    # encryption by default using ModifyEbsDefaultKmsKeyId or
    # ResetEbsDefaultKmsKeyId.
    #
    # For more information, see [Amazon EBS encryption][1] in the *Amazon
    # EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-encryption.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetEbsDefaultKmsKeyIdResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetEbsDefaultKmsKeyIdResult#kms_key_id #kms_key_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_ebs_default_kms_key_id({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.kms_key_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetEbsDefaultKmsKeyId AWS API Documentation
    #
    # @overload get_ebs_default_kms_key_id(params = {})
    # @param [Hash] params ({})
    def get_ebs_default_kms_key_id(params = {}, options = {})
      req = build_request(:get_ebs_default_kms_key_id, params)
      req.send_request(options)
    end

    # Describes whether EBS encryption by default is enabled for your
    # account in the current Region.
    #
    # For more information, see [Amazon EBS encryption][1] in the *Amazon
    # EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-encryption.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetEbsEncryptionByDefaultResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetEbsEncryptionByDefaultResult#ebs_encryption_by_default #ebs_encryption_by_default} => Boolean
    #   * {Types::GetEbsEncryptionByDefaultResult#sse_type #sse_type} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_ebs_encryption_by_default({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.ebs_encryption_by_default #=> Boolean
    #   resp.sse_type #=> String, one of "sse-ebs", "sse-kms", "none"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetEbsEncryptionByDefault AWS API Documentation
    #
    # @overload get_ebs_encryption_by_default(params = {})
    # @param [Hash] params ({})
    def get_ebs_encryption_by_default(params = {}, options = {})
      req = build_request(:get_ebs_encryption_by_default, params)
      req.send_request(options)
    end

    # Generates a CloudFormation template that streamlines and automates the
    # integration of VPC flow logs with Amazon Athena. This make it easier
    # for you to query and gain insights from VPC flow logs data. Based on
    # the information that you provide, we configure resources in the
    # template to do the following:
    #
    # * Create a table in Athena that maps fields to a custom log format
    #
    # * Create a Lambda function that updates the table with new partitions
    #   on a daily, weekly, or monthly basis
    #
    # * Create a table partitioned between two timestamps in the past
    #
    # * Create a set of named queries in Athena that you can use to get
    #   started quickly
    #
    # <note markdown="1"> `GetFlowLogsIntegrationTemplate` does not support integration between
    # Amazon Web Services Transit Gateway Flow Logs and Amazon Athena.
    #
    #  </note>
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :flow_log_id
    #   The ID of the flow log.
    #
    # @option params [required, String] :config_delivery_s3_destination_arn
    #   To store the CloudFormation template in Amazon S3, specify the
    #   location in Amazon S3.
    #
    # @option params [required, Types::IntegrateServices] :integrate_services
    #   Information about the service integration.
    #
    # @return [Types::GetFlowLogsIntegrationTemplateResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetFlowLogsIntegrationTemplateResult#result #result} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_flow_logs_integration_template({
    #     dry_run: false,
    #     flow_log_id: "VpcFlowLogId", # required
    #     config_delivery_s3_destination_arn: "String", # required
    #     integrate_services: { # required
    #       athena_integrations: [
    #         {
    #           integration_result_s3_destination_arn: "String", # required
    #           partition_load_frequency: "none", # required, accepts none, daily, weekly, monthly
    #           partition_start_date: Time.now,
    #           partition_end_date: Time.now,
    #         },
    #       ],
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.result #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetFlowLogsIntegrationTemplate AWS API Documentation
    #
    # @overload get_flow_logs_integration_template(params = {})
    # @param [Hash] params ({})
    def get_flow_logs_integration_template(params = {}, options = {})
      req = build_request(:get_flow_logs_integration_template, params)
      req.send_request(options)
    end

    # Lists the resource groups to which a Capacity Reservation has been
    # added.
    #
    # @option params [required, String] :capacity_reservation_id
    #   The ID of the Capacity Reservation. If you specify a Capacity
    #   Reservation that is shared with you, the operation returns only
    #   Capacity Reservation groups that you own.
    #
    # @option params [String] :next_token
    #   The token to use to retrieve the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetGroupsForCapacityReservationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetGroupsForCapacityReservationResult#next_token #next_token} => String
    #   * {Types::GetGroupsForCapacityReservationResult#capacity_reservation_groups #capacity_reservation_groups} => Array&lt;Types::CapacityReservationGroup&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_groups_for_capacity_reservation({
    #     capacity_reservation_id: "CapacityReservationId", # required
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.capacity_reservation_groups #=> Array
    #   resp.capacity_reservation_groups[0].group_arn #=> String
    #   resp.capacity_reservation_groups[0].owner_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetGroupsForCapacityReservation AWS API Documentation
    #
    # @overload get_groups_for_capacity_reservation(params = {})
    # @param [Hash] params ({})
    def get_groups_for_capacity_reservation(params = {}, options = {})
      req = build_request(:get_groups_for_capacity_reservation, params)
      req.send_request(options)
    end

    # Preview a reservation purchase with configurations that match those of
    # your Dedicated Host. You must have active Dedicated Hosts in your
    # account before you purchase a reservation.
    #
    # This is a preview of the PurchaseHostReservation action and does not
    # result in the offering being purchased.
    #
    # @option params [required, Array<String>] :host_id_set
    #   The IDs of the Dedicated Hosts with which the reservation is
    #   associated.
    #
    # @option params [required, String] :offering_id
    #   The offering ID of the reservation.
    #
    # @return [Types::GetHostReservationPurchasePreviewResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetHostReservationPurchasePreviewResult#currency_code #currency_code} => String
    #   * {Types::GetHostReservationPurchasePreviewResult#purchase #purchase} => Array&lt;Types::Purchase&gt;
    #   * {Types::GetHostReservationPurchasePreviewResult#total_hourly_price #total_hourly_price} => String
    #   * {Types::GetHostReservationPurchasePreviewResult#total_upfront_price #total_upfront_price} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_host_reservation_purchase_preview({
    #     host_id_set: ["DedicatedHostId"], # required
    #     offering_id: "OfferingId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.currency_code #=> String, one of "USD"
    #   resp.purchase #=> Array
    #   resp.purchase[0].currency_code #=> String, one of "USD"
    #   resp.purchase[0].duration #=> Integer
    #   resp.purchase[0].host_id_set #=> Array
    #   resp.purchase[0].host_id_set[0] #=> String
    #   resp.purchase[0].host_reservation_id #=> String
    #   resp.purchase[0].hourly_price #=> String
    #   resp.purchase[0].instance_family #=> String
    #   resp.purchase[0].payment_option #=> String, one of "AllUpfront", "PartialUpfront", "NoUpfront"
    #   resp.purchase[0].upfront_price #=> String
    #   resp.total_hourly_price #=> String
    #   resp.total_upfront_price #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetHostReservationPurchasePreview AWS API Documentation
    #
    # @overload get_host_reservation_purchase_preview(params = {})
    # @param [Hash] params ({})
    def get_host_reservation_purchase_preview(params = {}, options = {})
      req = build_request(:get_host_reservation_purchase_preview, params)
      req.send_request(options)
    end

    # Gets the current state of *block public access for AMIs* at the
    # account level in the specified Amazon Web Services Region.
    #
    # For more information, see [Block public access to your AMIs][1] in the
    # *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/sharingamis-intro.html#block-public-access-to-amis
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetImageBlockPublicAccessStateResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetImageBlockPublicAccessStateResult#image_block_public_access_state #image_block_public_access_state} => String
    #   * {Types::GetImageBlockPublicAccessStateResult#managed_by #managed_by} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_image_block_public_access_state({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.image_block_public_access_state #=> String
    #   resp.managed_by #=> String, one of "account", "declarative-policy"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetImageBlockPublicAccessState AWS API Documentation
    #
    # @overload get_image_block_public_access_state(params = {})
    # @param [Hash] params ({})
    def get_image_block_public_access_state(params = {}, options = {})
      req = build_request(:get_image_block_public_access_state, params)
      req.send_request(options)
    end

    # Gets the default instance metadata service (IMDS) settings that are
    # set at the account level in the specified Amazon Web Services  Region.
    #
    # For more information, see [Order of precedence for instance metadata
    # options][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/configuring-instance-metadata-options.html#instance-metadata-options-order-of-precedence
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetInstanceMetadataDefaultsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetInstanceMetadataDefaultsResult#account_level #account_level} => Types::InstanceMetadataDefaultsResponse
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_instance_metadata_defaults({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.account_level.http_tokens #=> String, one of "optional", "required"
    #   resp.account_level.http_put_response_hop_limit #=> Integer
    #   resp.account_level.http_endpoint #=> String, one of "disabled", "enabled"
    #   resp.account_level.instance_metadata_tags #=> String, one of "disabled", "enabled"
    #   resp.account_level.managed_by #=> String, one of "account", "declarative-policy"
    #   resp.account_level.managed_exception_message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetInstanceMetadataDefaults AWS API Documentation
    #
    # @overload get_instance_metadata_defaults(params = {})
    # @param [Hash] params ({})
    def get_instance_metadata_defaults(params = {}, options = {})
      req = build_request(:get_instance_metadata_defaults, params)
      req.send_request(options)
    end

    # Gets the public endorsement key associated with the Nitro Trusted
    # Platform Module (NitroTPM) for the specified instance.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance for which to get the public endorsement key.
    #
    # @option params [required, String] :key_type
    #   The required public endorsement key type.
    #
    # @option params [required, String] :key_format
    #   The required public endorsement key format. Specify `der` for a
    #   DER-encoded public key that is compatible with OpenSSL. Specify `tpmt`
    #   for a TPM 2.0 format that is compatible with tpm2-tools. The returned
    #   key is base64 encoded.
    #
    # @option params [Boolean] :dry_run
    #   Specify this parameter to verify whether the request will succeed,
    #   without actually making the request. If the request will succeed, the
    #   response is `DryRunOperation`. Otherwise, the response is
    #   `UnauthorizedOperation`.
    #
    # @return [Types::GetInstanceTpmEkPubResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetInstanceTpmEkPubResult#instance_id #instance_id} => String
    #   * {Types::GetInstanceTpmEkPubResult#key_type #key_type} => String
    #   * {Types::GetInstanceTpmEkPubResult#key_format #key_format} => String
    #   * {Types::GetInstanceTpmEkPubResult#key_value #key_value} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_instance_tpm_ek_pub({
    #     instance_id: "InstanceId", # required
    #     key_type: "rsa-2048", # required, accepts rsa-2048, ecc-sec-p384
    #     key_format: "der", # required, accepts der, tpmt
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_id #=> String
    #   resp.key_type #=> String, one of "rsa-2048", "ecc-sec-p384"
    #   resp.key_format #=> String, one of "der", "tpmt"
    #   resp.key_value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetInstanceTpmEkPub AWS API Documentation
    #
    # @overload get_instance_tpm_ek_pub(params = {})
    # @param [Hash] params ({})
    def get_instance_tpm_ek_pub(params = {}, options = {})
      req = build_request(:get_instance_tpm_ek_pub, params)
      req.send_request(options)
    end

    # Returns a list of instance types with the specified instance
    # attributes. You can use the response to preview the instance types
    # without launching instances. Note that the response does not consider
    # capacity.
    #
    # When you specify multiple parameters, you get instance types that
    # satisfy all of the specified parameters. If you specify multiple
    # values for a parameter, you get instance types that satisfy any of the
    # specified values.
    #
    # For more information, see [Preview instance types with specified
    # attributes][1], [Specify attributes for instance type selection for
    # EC2 Fleet or Spot Fleet][2], and [Spot placement score][3] in the
    # *Amazon EC2 User Guide*, and [Creating mixed instance groups using
    # attribute-based instance type selection][4] in the *Amazon EC2 Auto
    # Scaling User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-fleet-attribute-based-instance-type-selection.html#ec2fleet-get-instance-types-from-instance-requirements
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-fleet-attribute-based-instance-type-selection.html
    # [3]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-placement-score.html
    # [4]: https://docs.aws.amazon.com/autoscaling/ec2/userguide/create-asg-instance-type-requirements.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :architecture_types
    #   The processor architecture type.
    #
    # @option params [required, Array<String>] :virtualization_types
    #   The virtualization type.
    #
    # @option params [required, Types::InstanceRequirementsRequest] :instance_requirements
    #   The attributes required for the instance types.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @return [Types::GetInstanceTypesFromInstanceRequirementsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetInstanceTypesFromInstanceRequirementsResult#instance_types #instance_types} => Array&lt;Types::InstanceTypeInfoFromInstanceRequirements&gt;
    #   * {Types::GetInstanceTypesFromInstanceRequirementsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_instance_types_from_instance_requirements({
    #     dry_run: false,
    #     architecture_types: ["i386"], # required, accepts i386, x86_64, arm64, x86_64_mac, arm64_mac
    #     virtualization_types: ["hvm"], # required, accepts hvm, paravirtual
    #     instance_requirements: { # required
    #       v_cpu_count: { # required
    #         min: 1, # required
    #         max: 1,
    #       },
    #       memory_mi_b: { # required
    #         min: 1, # required
    #         max: 1,
    #       },
    #       cpu_manufacturers: ["intel"], # accepts intel, amd, amazon-web-services, apple
    #       memory_gi_b_per_v_cpu: {
    #         min: 1.0,
    #         max: 1.0,
    #       },
    #       excluded_instance_types: ["ExcludedInstanceType"],
    #       instance_generations: ["current"], # accepts current, previous
    #       spot_max_price_percentage_over_lowest_price: 1,
    #       on_demand_max_price_percentage_over_lowest_price: 1,
    #       bare_metal: "included", # accepts included, required, excluded
    #       burstable_performance: "included", # accepts included, required, excluded
    #       require_hibernate_support: false,
    #       network_interface_count: {
    #         min: 1,
    #         max: 1,
    #       },
    #       local_storage: "included", # accepts included, required, excluded
    #       local_storage_types: ["hdd"], # accepts hdd, ssd
    #       total_local_storage_gb: {
    #         min: 1.0,
    #         max: 1.0,
    #       },
    #       baseline_ebs_bandwidth_mbps: {
    #         min: 1,
    #         max: 1,
    #       },
    #       accelerator_types: ["gpu"], # accepts gpu, fpga, inference
    #       accelerator_count: {
    #         min: 1,
    #         max: 1,
    #       },
    #       accelerator_manufacturers: ["amazon-web-services"], # accepts amazon-web-services, amd, nvidia, xilinx, habana
    #       accelerator_names: ["a100"], # accepts a100, inferentia, k520, k80, m60, radeon-pro-v520, t4, vu9p, v100, a10g, h100, t4g
    #       accelerator_total_memory_mi_b: {
    #         min: 1,
    #         max: 1,
    #       },
    #       network_bandwidth_gbps: {
    #         min: 1.0,
    #         max: 1.0,
    #       },
    #       allowed_instance_types: ["AllowedInstanceType"],
    #       max_spot_price_as_percentage_of_optimal_on_demand_price: 1,
    #       baseline_performance_factors: {
    #         cpu: {
    #           references: [
    #             {
    #               instance_family: "String",
    #             },
    #           ],
    #         },
    #       },
    #     },
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_types #=> Array
    #   resp.instance_types[0].instance_type #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetInstanceTypesFromInstanceRequirements AWS API Documentation
    #
    # @overload get_instance_types_from_instance_requirements(params = {})
    # @param [Hash] params ({})
    def get_instance_types_from_instance_requirements(params = {}, options = {})
      req = build_request(:get_instance_types_from_instance_requirements, params)
      req.send_request(options)
    end

    # A binary representation of the UEFI variable store. Only non-volatile
    # variables are stored. This is a base64 encoded and zlib compressed
    # binary value that must be properly encoded.
    #
    # When you use [register-image][1] to create an AMI, you can create an
    # exact copy of your variable store by passing the UEFI data in the
    # `UefiData` parameter. You can modify the UEFI data by using the
    # [python-uefivars tool][2] on GitHub. You can use the tool to convert
    # the UEFI data into a human-readable format (JSON), which you can
    # inspect and modify, and then convert back into the binary format to
    # use with register-image.
    #
    # For more information, see [UEFI Secure Boot][3] in the *Amazon EC2
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/cli/latest/reference/ec2/register-image.html
    # [2]: https://github.com/awslabs/python-uefivars
    # [3]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/uefi-secure-boot.html
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance from which to retrieve the UEFI data.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetInstanceUefiDataResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetInstanceUefiDataResult#instance_id #instance_id} => String
    #   * {Types::GetInstanceUefiDataResult#uefi_data #uefi_data} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_instance_uefi_data({
    #     instance_id: "InstanceId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_id #=> String
    #   resp.uefi_data #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetInstanceUefiData AWS API Documentation
    #
    # @overload get_instance_uefi_data(params = {})
    # @param [Hash] params ({})
    def get_instance_uefi_data(params = {}, options = {})
      req = build_request(:get_instance_uefi_data, params)
      req.send_request(options)
    end

    # Retrieve historical information about a CIDR within an IPAM scope. For
    # more information, see [View the history of IP addresses][1] in the
    # *Amazon VPC IPAM User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/view-history-cidr-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :cidr
    #   The CIDR you want the history of. The CIDR can be an IPv4 or IPv6 IP
    #   address range. If you enter a /16 IPv4 CIDR, you will get records that
    #   match it exactly. You will not get records for any subnets within the
    #   /16 CIDR.
    #
    # @option params [required, String] :ipam_scope_id
    #   The ID of the IPAM scope that the CIDR is in.
    #
    # @option params [String] :vpc_id
    #   The ID of the VPC you want your history records filtered by.
    #
    # @option params [Time,DateTime,Date,Integer,String] :start_time
    #   The start of the time period for which you are looking for history. If
    #   you omit this option, it will default to the value of EndTime.
    #
    # @option params [Time,DateTime,Date,Integer,String] :end_time
    #   The end of the time period for which you are looking for history. If
    #   you omit this option, it will default to the current time.
    #
    # @option params [Integer] :max_results
    #   The maximum number of historical results you would like returned per
    #   page. Defaults to 100.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::GetIpamAddressHistoryResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetIpamAddressHistoryResult#history_records #history_records} => Array&lt;Types::IpamAddressHistoryRecord&gt;
    #   * {Types::GetIpamAddressHistoryResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_ipam_address_history({
    #     dry_run: false,
    #     cidr: "String", # required
    #     ipam_scope_id: "IpamScopeId", # required
    #     vpc_id: "String",
    #     start_time: Time.now,
    #     end_time: Time.now,
    #     max_results: 1,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.history_records #=> Array
    #   resp.history_records[0].resource_owner_id #=> String
    #   resp.history_records[0].resource_region #=> String
    #   resp.history_records[0].resource_type #=> String, one of "eip", "vpc", "subnet", "network-interface", "instance"
    #   resp.history_records[0].resource_id #=> String
    #   resp.history_records[0].resource_cidr #=> String
    #   resp.history_records[0].resource_name #=> String
    #   resp.history_records[0].resource_compliance_status #=> String, one of "compliant", "noncompliant", "unmanaged", "ignored"
    #   resp.history_records[0].resource_overlap_status #=> String, one of "overlapping", "nonoverlapping", "ignored"
    #   resp.history_records[0].vpc_id #=> String
    #   resp.history_records[0].sampled_start_time #=> Time
    #   resp.history_records[0].sampled_end_time #=> Time
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetIpamAddressHistory AWS API Documentation
    #
    # @overload get_ipam_address_history(params = {})
    # @param [Hash] params ({})
    def get_ipam_address_history(params = {}, options = {})
      req = build_request(:get_ipam_address_history, params)
      req.send_request(options)
    end

    # Gets IPAM discovered accounts. A discovered account is an Amazon Web
    # Services account that is monitored under a resource discovery. If you
    # have integrated IPAM with Amazon Web Services Organizations, all
    # accounts in the organization are discovered accounts. Only the IPAM
    # account can get all discovered accounts in the organization.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_resource_discovery_id
    #   A resource discovery ID.
    #
    # @option params [required, String] :discovery_region
    #   The Amazon Web Services Region that the account information is
    #   returned from.
    #
    # @option params [Array<Types::Filter>] :filters
    #   Discovered account filters.
    #
    # @option params [String] :next_token
    #   Specify the pagination token from a previous request to retrieve the
    #   next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of discovered accounts to return in one page of
    #   results.
    #
    # @return [Types::GetIpamDiscoveredAccountsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetIpamDiscoveredAccountsResult#ipam_discovered_accounts #ipam_discovered_accounts} => Array&lt;Types::IpamDiscoveredAccount&gt;
    #   * {Types::GetIpamDiscoveredAccountsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_ipam_discovered_accounts({
    #     dry_run: false,
    #     ipam_resource_discovery_id: "IpamResourceDiscoveryId", # required
    #     discovery_region: "String", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "NextToken",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_discovered_accounts #=> Array
    #   resp.ipam_discovered_accounts[0].account_id #=> String
    #   resp.ipam_discovered_accounts[0].discovery_region #=> String
    #   resp.ipam_discovered_accounts[0].failure_reason.code #=> String, one of "assume-role-failure", "throttling-failure", "unauthorized-failure"
    #   resp.ipam_discovered_accounts[0].failure_reason.message #=> String
    #   resp.ipam_discovered_accounts[0].last_attempted_discovery_time #=> Time
    #   resp.ipam_discovered_accounts[0].last_successful_discovery_time #=> Time
    #   resp.ipam_discovered_accounts[0].organizational_unit_id #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetIpamDiscoveredAccounts AWS API Documentation
    #
    # @overload get_ipam_discovered_accounts(params = {})
    # @param [Hash] params ({})
    def get_ipam_discovered_accounts(params = {}, options = {})
      req = build_request(:get_ipam_discovered_accounts, params)
      req.send_request(options)
    end

    # Gets the public IP addresses that have been discovered by IPAM.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_resource_discovery_id
    #   An IPAM resource discovery ID.
    #
    # @option params [required, String] :address_region
    #   The Amazon Web Services Region for the IP address.
    #
    # @option params [Array<Types::Filter>] :filters
    #   Filters.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of IPAM discovered public addresses to return in
    #   one page of results.
    #
    # @return [Types::GetIpamDiscoveredPublicAddressesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetIpamDiscoveredPublicAddressesResult#ipam_discovered_public_addresses #ipam_discovered_public_addresses} => Array&lt;Types::IpamDiscoveredPublicAddress&gt;
    #   * {Types::GetIpamDiscoveredPublicAddressesResult#oldest_sample_time #oldest_sample_time} => Time
    #   * {Types::GetIpamDiscoveredPublicAddressesResult#next_token #next_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_ipam_discovered_public_addresses({
    #     dry_run: false,
    #     ipam_resource_discovery_id: "IpamResourceDiscoveryId", # required
    #     address_region: "String", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "NextToken",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_discovered_public_addresses #=> Array
    #   resp.ipam_discovered_public_addresses[0].ipam_resource_discovery_id #=> String
    #   resp.ipam_discovered_public_addresses[0].address_region #=> String
    #   resp.ipam_discovered_public_addresses[0].address #=> String
    #   resp.ipam_discovered_public_addresses[0].address_owner_id #=> String
    #   resp.ipam_discovered_public_addresses[0].address_allocation_id #=> String
    #   resp.ipam_discovered_public_addresses[0].association_status #=> String, one of "associated", "disassociated"
    #   resp.ipam_discovered_public_addresses[0].address_type #=> String, one of "service-managed-ip", "service-managed-byoip", "amazon-owned-eip", "amazon-owned-contig", "byoip", "ec2-public-ip"
    #   resp.ipam_discovered_public_addresses[0].service #=> String, one of "nat-gateway", "database-migration-service", "redshift", "elastic-container-service", "relational-database-service", "site-to-site-vpn", "load-balancer", "global-accelerator", "other"
    #   resp.ipam_discovered_public_addresses[0].service_resource #=> String
    #   resp.ipam_discovered_public_addresses[0].vpc_id #=> String
    #   resp.ipam_discovered_public_addresses[0].subnet_id #=> String
    #   resp.ipam_discovered_public_addresses[0].public_ipv_4_pool_id #=> String
    #   resp.ipam_discovered_public_addresses[0].network_interface_id #=> String
    #   resp.ipam_discovered_public_addresses[0].network_interface_description #=> String
    #   resp.ipam_discovered_public_addresses[0].instance_id #=> String
    #   resp.ipam_discovered_public_addresses[0].tags.eip_tags #=> Array
    #   resp.ipam_discovered_public_addresses[0].tags.eip_tags[0].key #=> String
    #   resp.ipam_discovered_public_addresses[0].tags.eip_tags[0].value #=> String
    #   resp.ipam_discovered_public_addresses[0].network_border_group #=> String
    #   resp.ipam_discovered_public_addresses[0].security_groups #=> Array
    #   resp.ipam_discovered_public_addresses[0].security_groups[0].group_name #=> String
    #   resp.ipam_discovered_public_addresses[0].security_groups[0].group_id #=> String
    #   resp.ipam_discovered_public_addresses[0].sample_time #=> Time
    #   resp.oldest_sample_time #=> Time
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetIpamDiscoveredPublicAddresses AWS API Documentation
    #
    # @overload get_ipam_discovered_public_addresses(params = {})
    # @param [Hash] params ({})
    def get_ipam_discovered_public_addresses(params = {}, options = {})
      req = build_request(:get_ipam_discovered_public_addresses, params)
      req.send_request(options)
    end

    # Returns the resource CIDRs that are monitored as part of a resource
    # discovery. A discovered resource is a resource CIDR monitored under a
    # resource discovery. The following resources can be discovered: VPCs,
    # Public IPv4 pools, VPC subnets, and Elastic IP addresses.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_resource_discovery_id
    #   A resource discovery ID.
    #
    # @option params [required, String] :resource_region
    #   A resource Region.
    #
    # @option params [Array<Types::Filter>] :filters
    #   Filters.
    #
    # @option params [String] :next_token
    #   Specify the pagination token from a previous request to retrieve the
    #   next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of discovered resource CIDRs to return in one page
    #   of results.
    #
    # @return [Types::GetIpamDiscoveredResourceCidrsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetIpamDiscoveredResourceCidrsResult#ipam_discovered_resource_cidrs #ipam_discovered_resource_cidrs} => Array&lt;Types::IpamDiscoveredResourceCidr&gt;
    #   * {Types::GetIpamDiscoveredResourceCidrsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_ipam_discovered_resource_cidrs({
    #     dry_run: false,
    #     ipam_resource_discovery_id: "IpamResourceDiscoveryId", # required
    #     resource_region: "String", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     next_token: "NextToken",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_discovered_resource_cidrs #=> Array
    #   resp.ipam_discovered_resource_cidrs[0].ipam_resource_discovery_id #=> String
    #   resp.ipam_discovered_resource_cidrs[0].resource_region #=> String
    #   resp.ipam_discovered_resource_cidrs[0].resource_id #=> String
    #   resp.ipam_discovered_resource_cidrs[0].resource_owner_id #=> String
    #   resp.ipam_discovered_resource_cidrs[0].resource_cidr #=> String
    #   resp.ipam_discovered_resource_cidrs[0].ip_source #=> String, one of "amazon", "byoip", "none"
    #   resp.ipam_discovered_resource_cidrs[0].resource_type #=> String, one of "vpc", "subnet", "eip", "public-ipv4-pool", "ipv6-pool", "eni"
    #   resp.ipam_discovered_resource_cidrs[0].resource_tags #=> Array
    #   resp.ipam_discovered_resource_cidrs[0].resource_tags[0].key #=> String
    #   resp.ipam_discovered_resource_cidrs[0].resource_tags[0].value #=> String
    #   resp.ipam_discovered_resource_cidrs[0].ip_usage #=> Float
    #   resp.ipam_discovered_resource_cidrs[0].vpc_id #=> String
    #   resp.ipam_discovered_resource_cidrs[0].subnet_id #=> String
    #   resp.ipam_discovered_resource_cidrs[0].network_interface_attachment_status #=> String, one of "available", "in-use"
    #   resp.ipam_discovered_resource_cidrs[0].sample_time #=> Time
    #   resp.ipam_discovered_resource_cidrs[0].availability_zone_id #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetIpamDiscoveredResourceCidrs AWS API Documentation
    #
    # @overload get_ipam_discovered_resource_cidrs(params = {})
    # @param [Hash] params ({})
    def get_ipam_discovered_resource_cidrs(params = {}, options = {})
      req = build_request(:get_ipam_discovered_resource_cidrs, params)
      req.send_request(options)
    end

    # Get a list of all the CIDR allocations in an IPAM pool. The Region you
    # use should be the IPAM pool locale. The locale is the Amazon Web
    # Services Region where this IPAM pool is available for allocations.
    #
    # <note markdown="1"> If you use this action after [AllocateIpamPoolCidr][1] or
    # [ReleaseIpamPoolAllocation][2], note that all EC2 API actions follow
    # an [eventual consistency][3] model.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_AllocateIpamPoolCidr.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ReleaseIpamPoolAllocation.html
    # [3]: https://docs.aws.amazon.com/ec2/latest/devguide/eventual-consistency.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_pool_id
    #   The ID of the IPAM pool you want to see the allocations for.
    #
    # @option params [String] :ipam_pool_allocation_id
    #   The ID of the allocation.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters for the request. For more information about
    #   filtering, see [Filtering CLI output][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/cli/latest/userguide/cli-usage-filter.html
    #
    # @option params [Integer] :max_results
    #   The maximum number of results you would like returned per page.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::GetIpamPoolAllocationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetIpamPoolAllocationsResult#ipam_pool_allocations #ipam_pool_allocations} => Array&lt;Types::IpamPoolAllocation&gt;
    #   * {Types::GetIpamPoolAllocationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_ipam_pool_allocations({
    #     dry_run: false,
    #     ipam_pool_id: "IpamPoolId", # required
    #     ipam_pool_allocation_id: "IpamPoolAllocationId",
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_pool_allocations #=> Array
    #   resp.ipam_pool_allocations[0].cidr #=> String
    #   resp.ipam_pool_allocations[0].ipam_pool_allocation_id #=> String
    #   resp.ipam_pool_allocations[0].description #=> String
    #   resp.ipam_pool_allocations[0].resource_id #=> String
    #   resp.ipam_pool_allocations[0].resource_type #=> String, one of "ipam-pool", "vpc", "ec2-public-ipv4-pool", "custom", "subnet", "eip"
    #   resp.ipam_pool_allocations[0].resource_region #=> String
    #   resp.ipam_pool_allocations[0].resource_owner #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetIpamPoolAllocations AWS API Documentation
    #
    # @overload get_ipam_pool_allocations(params = {})
    # @param [Hash] params ({})
    def get_ipam_pool_allocations(params = {}, options = {})
      req = build_request(:get_ipam_pool_allocations, params)
      req.send_request(options)
    end

    # Get the CIDRs provisioned to an IPAM pool.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_pool_id
    #   The ID of the IPAM pool you want the CIDR for.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters for the request. For more information about
    #   filtering, see [Filtering CLI output][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/cli/latest/userguide/cli-usage-filter.html
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in the request.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::GetIpamPoolCidrsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetIpamPoolCidrsResult#ipam_pool_cidrs #ipam_pool_cidrs} => Array&lt;Types::IpamPoolCidr&gt;
    #   * {Types::GetIpamPoolCidrsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_ipam_pool_cidrs({
    #     dry_run: false,
    #     ipam_pool_id: "IpamPoolId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_pool_cidrs #=> Array
    #   resp.ipam_pool_cidrs[0].cidr #=> String
    #   resp.ipam_pool_cidrs[0].state #=> String, one of "pending-provision", "provisioned", "failed-provision", "pending-deprovision", "deprovisioned", "failed-deprovision", "pending-import", "failed-import"
    #   resp.ipam_pool_cidrs[0].failure_reason.code #=> String, one of "cidr-not-available", "limit-exceeded"
    #   resp.ipam_pool_cidrs[0].failure_reason.message #=> String
    #   resp.ipam_pool_cidrs[0].ipam_pool_cidr_id #=> String
    #   resp.ipam_pool_cidrs[0].netmask_length #=> Integer
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetIpamPoolCidrs AWS API Documentation
    #
    # @overload get_ipam_pool_cidrs(params = {})
    # @param [Hash] params ({})
    def get_ipam_pool_cidrs(params = {}, options = {})
      req = build_request(:get_ipam_pool_cidrs, params)
      req.send_request(options)
    end

    # Returns resource CIDRs managed by IPAM in a given scope. If an IPAM is
    # associated with more than one resource discovery, the resource CIDRs
    # across all of the resource discoveries is returned. A resource
    # discovery is an IPAM component that enables IPAM to manage and monitor
    # resources that belong to the owning account.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters for the request. For more information about
    #   filtering, see [Filtering CLI output][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/cli/latest/userguide/cli-usage-filter.html
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return in the request.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [required, String] :ipam_scope_id
    #   The ID of the scope that the resource is in.
    #
    # @option params [String] :ipam_pool_id
    #   The ID of the IPAM pool that the resource is in.
    #
    # @option params [String] :resource_id
    #   The ID of the resource.
    #
    # @option params [String] :resource_type
    #   The resource type.
    #
    # @option params [Types::RequestIpamResourceTag] :resource_tag
    #   The resource tag.
    #
    # @option params [String] :resource_owner
    #   The ID of the Amazon Web Services account that owns the resource.
    #
    # @return [Types::GetIpamResourceCidrsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetIpamResourceCidrsResult#next_token #next_token} => String
    #   * {Types::GetIpamResourceCidrsResult#ipam_resource_cidrs #ipam_resource_cidrs} => Array&lt;Types::IpamResourceCidr&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_ipam_resource_cidrs({
    #     dry_run: false,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "NextToken",
    #     ipam_scope_id: "IpamScopeId", # required
    #     ipam_pool_id: "IpamPoolId",
    #     resource_id: "String",
    #     resource_type: "vpc", # accepts vpc, subnet, eip, public-ipv4-pool, ipv6-pool, eni
    #     resource_tag: {
    #       key: "String",
    #       value: "String",
    #     },
    #     resource_owner: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.ipam_resource_cidrs #=> Array
    #   resp.ipam_resource_cidrs[0].ipam_id #=> String
    #   resp.ipam_resource_cidrs[0].ipam_scope_id #=> String
    #   resp.ipam_resource_cidrs[0].ipam_pool_id #=> String
    #   resp.ipam_resource_cidrs[0].resource_region #=> String
    #   resp.ipam_resource_cidrs[0].resource_owner_id #=> String
    #   resp.ipam_resource_cidrs[0].resource_id #=> String
    #   resp.ipam_resource_cidrs[0].resource_name #=> String
    #   resp.ipam_resource_cidrs[0].resource_cidr #=> String
    #   resp.ipam_resource_cidrs[0].resource_type #=> String, one of "vpc", "subnet", "eip", "public-ipv4-pool", "ipv6-pool", "eni"
    #   resp.ipam_resource_cidrs[0].resource_tags #=> Array
    #   resp.ipam_resource_cidrs[0].resource_tags[0].key #=> String
    #   resp.ipam_resource_cidrs[0].resource_tags[0].value #=> String
    #   resp.ipam_resource_cidrs[0].ip_usage #=> Float
    #   resp.ipam_resource_cidrs[0].compliance_status #=> String, one of "compliant", "noncompliant", "unmanaged", "ignored"
    #   resp.ipam_resource_cidrs[0].management_state #=> String, one of "managed", "unmanaged", "ignored"
    #   resp.ipam_resource_cidrs[0].overlap_status #=> String, one of "overlapping", "nonoverlapping", "ignored"
    #   resp.ipam_resource_cidrs[0].vpc_id #=> String
    #   resp.ipam_resource_cidrs[0].availability_zone_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetIpamResourceCidrs AWS API Documentation
    #
    # @overload get_ipam_resource_cidrs(params = {})
    # @param [Hash] params ({})
    def get_ipam_resource_cidrs(params = {}, options = {})
      req = build_request(:get_ipam_resource_cidrs, params)
      req.send_request(options)
    end

    # Retrieves the configuration data of the specified instance. You can
    # use this data to create a launch template.
    #
    # This action calls on other describe actions to get instance
    # information. Depending on your instance configuration, you may need to
    # allow the following actions in your IAM policy:
    # `DescribeSpotInstanceRequests`,
    # `DescribeInstanceCreditSpecifications`, `DescribeVolumes`, and
    # `DescribeInstanceAttribute`. Or, you can allow `describe*` depending
    # on your instance requirements.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @return [Types::GetLaunchTemplateDataResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetLaunchTemplateDataResult#launch_template_data #launch_template_data} => Types::ResponseLaunchTemplateData
    #
    #
    # @example Example: To get the launch template data for an instance 
    #
    #   # This example gets the launch template data for the specified instance.
    #
    #   resp = client.get_launch_template_data({
    #     instance_id: "0123d646e8048babc", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     launch_template_data: {
    #       block_device_mappings: [
    #         {
    #           device_name: "/dev/xvda", 
    #           ebs: {
    #             delete_on_termination: true, 
    #             encrypted: false, 
    #             iops: 100, 
    #             snapshot_id: "snap-02594938353ef77d3", 
    #             volume_size: 8, 
    #             volume_type: "gp2", 
    #           }, 
    #         }, 
    #       ], 
    #       ebs_optimized: false, 
    #       image_id: "ami-32cf7b4a", 
    #       instance_type: "t2.medium", 
    #       key_name: "my-key-pair", 
    #       monitoring: {
    #         enabled: false, 
    #       }, 
    #       network_interfaces: [
    #         {
    #           associate_public_ip_address: false, 
    #           delete_on_termination: true, 
    #           description: "", 
    #           device_index: 0, 
    #           groups: [
    #             "sg-d14e1bb4", 
    #           ], 
    #           ipv_6_addresses: [
    #           ], 
    #           network_interface_id: "eni-4338b5a9", 
    #           private_ip_address: "10.0.3.233", 
    #           private_ip_addresses: [
    #             {
    #               primary: true, 
    #               private_ip_address: "10.0.3.233", 
    #             }, 
    #           ], 
    #           subnet_id: "subnet-5264e837", 
    #         }, 
    #       ], 
    #       placement: {
    #         availability_zone: "us-east-2b", 
    #         group_name: "", 
    #         tenancy: "default", 
    #       }, 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_launch_template_data({
    #     dry_run: false,
    #     instance_id: "InstanceId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.launch_template_data.kernel_id #=> String
    #   resp.launch_template_data.ebs_optimized #=> Boolean
    #   resp.launch_template_data.iam_instance_profile.arn #=> String
    #   resp.launch_template_data.iam_instance_profile.name #=> String
    #   resp.launch_template_data.block_device_mappings #=> Array
    #   resp.launch_template_data.block_device_mappings[0].device_name #=> String
    #   resp.launch_template_data.block_device_mappings[0].virtual_name #=> String
    #   resp.launch_template_data.block_device_mappings[0].ebs.encrypted #=> Boolean
    #   resp.launch_template_data.block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.launch_template_data.block_device_mappings[0].ebs.iops #=> Integer
    #   resp.launch_template_data.block_device_mappings[0].ebs.kms_key_id #=> String
    #   resp.launch_template_data.block_device_mappings[0].ebs.snapshot_id #=> String
    #   resp.launch_template_data.block_device_mappings[0].ebs.volume_size #=> Integer
    #   resp.launch_template_data.block_device_mappings[0].ebs.volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.launch_template_data.block_device_mappings[0].ebs.throughput #=> Integer
    #   resp.launch_template_data.block_device_mappings[0].no_device #=> String
    #   resp.launch_template_data.network_interfaces #=> Array
    #   resp.launch_template_data.network_interfaces[0].associate_carrier_ip_address #=> Boolean
    #   resp.launch_template_data.network_interfaces[0].associate_public_ip_address #=> Boolean
    #   resp.launch_template_data.network_interfaces[0].delete_on_termination #=> Boolean
    #   resp.launch_template_data.network_interfaces[0].description #=> String
    #   resp.launch_template_data.network_interfaces[0].device_index #=> Integer
    #   resp.launch_template_data.network_interfaces[0].groups #=> Array
    #   resp.launch_template_data.network_interfaces[0].groups[0] #=> String
    #   resp.launch_template_data.network_interfaces[0].interface_type #=> String
    #   resp.launch_template_data.network_interfaces[0].ipv_6_address_count #=> Integer
    #   resp.launch_template_data.network_interfaces[0].ipv_6_addresses #=> Array
    #   resp.launch_template_data.network_interfaces[0].ipv_6_addresses[0].ipv_6_address #=> String
    #   resp.launch_template_data.network_interfaces[0].ipv_6_addresses[0].is_primary_ipv_6 #=> Boolean
    #   resp.launch_template_data.network_interfaces[0].network_interface_id #=> String
    #   resp.launch_template_data.network_interfaces[0].private_ip_address #=> String
    #   resp.launch_template_data.network_interfaces[0].private_ip_addresses #=> Array
    #   resp.launch_template_data.network_interfaces[0].private_ip_addresses[0].primary #=> Boolean
    #   resp.launch_template_data.network_interfaces[0].private_ip_addresses[0].private_ip_address #=> String
    #   resp.launch_template_data.network_interfaces[0].secondary_private_ip_address_count #=> Integer
    #   resp.launch_template_data.network_interfaces[0].subnet_id #=> String
    #   resp.launch_template_data.network_interfaces[0].network_card_index #=> Integer
    #   resp.launch_template_data.network_interfaces[0].ipv_4_prefixes #=> Array
    #   resp.launch_template_data.network_interfaces[0].ipv_4_prefixes[0].ipv_4_prefix #=> String
    #   resp.launch_template_data.network_interfaces[0].ipv_4_prefix_count #=> Integer
    #   resp.launch_template_data.network_interfaces[0].ipv_6_prefixes #=> Array
    #   resp.launch_template_data.network_interfaces[0].ipv_6_prefixes[0].ipv_6_prefix #=> String
    #   resp.launch_template_data.network_interfaces[0].ipv_6_prefix_count #=> Integer
    #   resp.launch_template_data.network_interfaces[0].primary_ipv_6 #=> Boolean
    #   resp.launch_template_data.network_interfaces[0].ena_srd_specification.ena_srd_enabled #=> Boolean
    #   resp.launch_template_data.network_interfaces[0].ena_srd_specification.ena_srd_udp_specification.ena_srd_udp_enabled #=> Boolean
    #   resp.launch_template_data.network_interfaces[0].connection_tracking_specification.tcp_established_timeout #=> Integer
    #   resp.launch_template_data.network_interfaces[0].connection_tracking_specification.udp_timeout #=> Integer
    #   resp.launch_template_data.network_interfaces[0].connection_tracking_specification.udp_stream_timeout #=> Integer
    #   resp.launch_template_data.image_id #=> String
    #   resp.launch_template_data.instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.launch_template_data.key_name #=> String
    #   resp.launch_template_data.monitoring.enabled #=> Boolean
    #   resp.launch_template_data.placement.availability_zone #=> String
    #   resp.launch_template_data.placement.affinity #=> String
    #   resp.launch_template_data.placement.group_name #=> String
    #   resp.launch_template_data.placement.host_id #=> String
    #   resp.launch_template_data.placement.tenancy #=> String, one of "default", "dedicated", "host"
    #   resp.launch_template_data.placement.spread_domain #=> String
    #   resp.launch_template_data.placement.host_resource_group_arn #=> String
    #   resp.launch_template_data.placement.partition_number #=> Integer
    #   resp.launch_template_data.placement.group_id #=> String
    #   resp.launch_template_data.ram_disk_id #=> String
    #   resp.launch_template_data.disable_api_termination #=> Boolean
    #   resp.launch_template_data.instance_initiated_shutdown_behavior #=> String, one of "stop", "terminate"
    #   resp.launch_template_data.user_data #=> String
    #   resp.launch_template_data.tag_specifications #=> Array
    #   resp.launch_template_data.tag_specifications[0].resource_type #=> String, one of "capacity-reservation", "client-vpn-endpoint", "customer-gateway", "carrier-gateway", "coip-pool", "declarative-policies-report", "dedicated-host", "dhcp-options", "egress-only-internet-gateway", "elastic-ip", "elastic-gpu", "export-image-task", "export-instance-task", "fleet", "fpga-image", "host-reservation", "image", "import-image-task", "import-snapshot-task", "instance", "instance-event-window", "internet-gateway", "ipam", "ipam-pool", "ipam-scope", "ipv4pool-ec2", "ipv6pool-ec2", "key-pair", "launch-template", "local-gateway", "local-gateway-route-table", "local-gateway-virtual-interface", "local-gateway-virtual-interface-group", "local-gateway-route-table-vpc-association", "local-gateway-route-table-virtual-interface-group-association", "natgateway", "network-acl", "network-interface", "network-insights-analysis", "network-insights-path", "network-insights-access-scope", "network-insights-access-scope-analysis", "placement-group", "prefix-list", "replace-root-volume-task", "reserved-instances", "route-table", "security-group", "security-group-rule", "snapshot", "spot-fleet-request", "spot-instances-request", "subnet", "subnet-cidr-reservation", "traffic-mirror-filter", "traffic-mirror-session", "traffic-mirror-target", "transit-gateway", "transit-gateway-attachment", "transit-gateway-connect-peer", "transit-gateway-multicast-domain", "transit-gateway-policy-table", "transit-gateway-route-table", "transit-gateway-route-table-announcement", "volume", "vpc", "vpc-endpoint", "vpc-endpoint-connection", "vpc-endpoint-service", "vpc-endpoint-service-permission", "vpc-peering-connection", "vpn-connection", "vpn-gateway", "vpc-flow-log", "capacity-reservation-fleet", "traffic-mirror-filter-rule", "vpc-endpoint-connection-device-type", "verified-access-instance", "verified-access-group", "verified-access-endpoint", "verified-access-policy", "verified-access-trust-provider", "vpn-connection-device-type", "vpc-block-public-access-exclusion", "ipam-resource-discovery", "ipam-resource-discovery-association", "instance-connect-endpoint", "verified-access-endpoint-target", "ipam-external-resource-verification-token"
    #   resp.launch_template_data.tag_specifications[0].tags #=> Array
    #   resp.launch_template_data.tag_specifications[0].tags[0].key #=> String
    #   resp.launch_template_data.tag_specifications[0].tags[0].value #=> String
    #   resp.launch_template_data.elastic_gpu_specifications #=> Array
    #   resp.launch_template_data.elastic_gpu_specifications[0].type #=> String
    #   resp.launch_template_data.elastic_inference_accelerators #=> Array
    #   resp.launch_template_data.elastic_inference_accelerators[0].type #=> String
    #   resp.launch_template_data.elastic_inference_accelerators[0].count #=> Integer
    #   resp.launch_template_data.security_group_ids #=> Array
    #   resp.launch_template_data.security_group_ids[0] #=> String
    #   resp.launch_template_data.security_groups #=> Array
    #   resp.launch_template_data.security_groups[0] #=> String
    #   resp.launch_template_data.instance_market_options.market_type #=> String, one of "spot", "capacity-block"
    #   resp.launch_template_data.instance_market_options.spot_options.max_price #=> String
    #   resp.launch_template_data.instance_market_options.spot_options.spot_instance_type #=> String, one of "one-time", "persistent"
    #   resp.launch_template_data.instance_market_options.spot_options.block_duration_minutes #=> Integer
    #   resp.launch_template_data.instance_market_options.spot_options.valid_until #=> Time
    #   resp.launch_template_data.instance_market_options.spot_options.instance_interruption_behavior #=> String, one of "hibernate", "stop", "terminate"
    #   resp.launch_template_data.credit_specification.cpu_credits #=> String
    #   resp.launch_template_data.cpu_options.core_count #=> Integer
    #   resp.launch_template_data.cpu_options.threads_per_core #=> Integer
    #   resp.launch_template_data.cpu_options.amd_sev_snp #=> String, one of "enabled", "disabled"
    #   resp.launch_template_data.capacity_reservation_specification.capacity_reservation_preference #=> String, one of "capacity-reservations-only", "open", "none"
    #   resp.launch_template_data.capacity_reservation_specification.capacity_reservation_target.capacity_reservation_id #=> String
    #   resp.launch_template_data.capacity_reservation_specification.capacity_reservation_target.capacity_reservation_resource_group_arn #=> String
    #   resp.launch_template_data.license_specifications #=> Array
    #   resp.launch_template_data.license_specifications[0].license_configuration_arn #=> String
    #   resp.launch_template_data.hibernation_options.configured #=> Boolean
    #   resp.launch_template_data.metadata_options.state #=> String, one of "pending", "applied"
    #   resp.launch_template_data.metadata_options.http_tokens #=> String, one of "optional", "required"
    #   resp.launch_template_data.metadata_options.http_put_response_hop_limit #=> Integer
    #   resp.launch_template_data.metadata_options.http_endpoint #=> String, one of "disabled", "enabled"
    #   resp.launch_template_data.metadata_options.http_protocol_ipv_6 #=> String, one of "disabled", "enabled"
    #   resp.launch_template_data.metadata_options.instance_metadata_tags #=> String, one of "disabled", "enabled"
    #   resp.launch_template_data.enclave_options.enabled #=> Boolean
    #   resp.launch_template_data.instance_requirements.v_cpu_count.min #=> Integer
    #   resp.launch_template_data.instance_requirements.v_cpu_count.max #=> Integer
    #   resp.launch_template_data.instance_requirements.memory_mi_b.min #=> Integer
    #   resp.launch_template_data.instance_requirements.memory_mi_b.max #=> Integer
    #   resp.launch_template_data.instance_requirements.cpu_manufacturers #=> Array
    #   resp.launch_template_data.instance_requirements.cpu_manufacturers[0] #=> String, one of "intel", "amd", "amazon-web-services", "apple"
    #   resp.launch_template_data.instance_requirements.memory_gi_b_per_v_cpu.min #=> Float
    #   resp.launch_template_data.instance_requirements.memory_gi_b_per_v_cpu.max #=> Float
    #   resp.launch_template_data.instance_requirements.excluded_instance_types #=> Array
    #   resp.launch_template_data.instance_requirements.excluded_instance_types[0] #=> String
    #   resp.launch_template_data.instance_requirements.instance_generations #=> Array
    #   resp.launch_template_data.instance_requirements.instance_generations[0] #=> String, one of "current", "previous"
    #   resp.launch_template_data.instance_requirements.spot_max_price_percentage_over_lowest_price #=> Integer
    #   resp.launch_template_data.instance_requirements.on_demand_max_price_percentage_over_lowest_price #=> Integer
    #   resp.launch_template_data.instance_requirements.bare_metal #=> String, one of "included", "required", "excluded"
    #   resp.launch_template_data.instance_requirements.burstable_performance #=> String, one of "included", "required", "excluded"
    #   resp.launch_template_data.instance_requirements.require_hibernate_support #=> Boolean
    #   resp.launch_template_data.instance_requirements.network_interface_count.min #=> Integer
    #   resp.launch_template_data.instance_requirements.network_interface_count.max #=> Integer
    #   resp.launch_template_data.instance_requirements.local_storage #=> String, one of "included", "required", "excluded"
    #   resp.launch_template_data.instance_requirements.local_storage_types #=> Array
    #   resp.launch_template_data.instance_requirements.local_storage_types[0] #=> String, one of "hdd", "ssd"
    #   resp.launch_template_data.instance_requirements.total_local_storage_gb.min #=> Float
    #   resp.launch_template_data.instance_requirements.total_local_storage_gb.max #=> Float
    #   resp.launch_template_data.instance_requirements.baseline_ebs_bandwidth_mbps.min #=> Integer
    #   resp.launch_template_data.instance_requirements.baseline_ebs_bandwidth_mbps.max #=> Integer
    #   resp.launch_template_data.instance_requirements.accelerator_types #=> Array
    #   resp.launch_template_data.instance_requirements.accelerator_types[0] #=> String, one of "gpu", "fpga", "inference"
    #   resp.launch_template_data.instance_requirements.accelerator_count.min #=> Integer
    #   resp.launch_template_data.instance_requirements.accelerator_count.max #=> Integer
    #   resp.launch_template_data.instance_requirements.accelerator_manufacturers #=> Array
    #   resp.launch_template_data.instance_requirements.accelerator_manufacturers[0] #=> String, one of "amazon-web-services", "amd", "nvidia", "xilinx", "habana"
    #   resp.launch_template_data.instance_requirements.accelerator_names #=> Array
    #   resp.launch_template_data.instance_requirements.accelerator_names[0] #=> String, one of "a100", "inferentia", "k520", "k80", "m60", "radeon-pro-v520", "t4", "vu9p", "v100", "a10g", "h100", "t4g"
    #   resp.launch_template_data.instance_requirements.accelerator_total_memory_mi_b.min #=> Integer
    #   resp.launch_template_data.instance_requirements.accelerator_total_memory_mi_b.max #=> Integer
    #   resp.launch_template_data.instance_requirements.network_bandwidth_gbps.min #=> Float
    #   resp.launch_template_data.instance_requirements.network_bandwidth_gbps.max #=> Float
    #   resp.launch_template_data.instance_requirements.allowed_instance_types #=> Array
    #   resp.launch_template_data.instance_requirements.allowed_instance_types[0] #=> String
    #   resp.launch_template_data.instance_requirements.max_spot_price_as_percentage_of_optimal_on_demand_price #=> Integer
    #   resp.launch_template_data.instance_requirements.baseline_performance_factors.cpu.references #=> Array
    #   resp.launch_template_data.instance_requirements.baseline_performance_factors.cpu.references[0].instance_family #=> String
    #   resp.launch_template_data.private_dns_name_options.hostname_type #=> String, one of "ip-name", "resource-name"
    #   resp.launch_template_data.private_dns_name_options.enable_resource_name_dns_a_record #=> Boolean
    #   resp.launch_template_data.private_dns_name_options.enable_resource_name_dns_aaaa_record #=> Boolean
    #   resp.launch_template_data.maintenance_options.auto_recovery #=> String, one of "default", "disabled"
    #   resp.launch_template_data.disable_api_stop #=> Boolean
    #   resp.launch_template_data.operator.managed #=> Boolean
    #   resp.launch_template_data.operator.principal #=> String
    #   resp.launch_template_data.network_performance_options.bandwidth_weighting #=> String, one of "default", "vpc-1", "ebs-1"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetLaunchTemplateData AWS API Documentation
    #
    # @overload get_launch_template_data(params = {})
    # @param [Hash] params ({})
    def get_launch_template_data(params = {}, options = {})
      req = build_request(:get_launch_template_data, params)
      req.send_request(options)
    end

    # Gets information about the resources that are associated with the
    # specified managed prefix list.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :prefix_list_id
    #   The ID of the prefix list.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::GetManagedPrefixListAssociationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetManagedPrefixListAssociationsResult#prefix_list_associations #prefix_list_associations} => Array&lt;Types::PrefixListAssociation&gt;
    #   * {Types::GetManagedPrefixListAssociationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_managed_prefix_list_associations({
    #     dry_run: false,
    #     prefix_list_id: "PrefixListResourceId", # required
    #     max_results: 1,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.prefix_list_associations #=> Array
    #   resp.prefix_list_associations[0].resource_id #=> String
    #   resp.prefix_list_associations[0].resource_owner #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetManagedPrefixListAssociations AWS API Documentation
    #
    # @overload get_managed_prefix_list_associations(params = {})
    # @param [Hash] params ({})
    def get_managed_prefix_list_associations(params = {}, options = {})
      req = build_request(:get_managed_prefix_list_associations, params)
      req.send_request(options)
    end

    # Gets information about the entries for a specified managed prefix
    # list.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :prefix_list_id
    #   The ID of the prefix list.
    #
    # @option params [Integer] :target_version
    #   The version of the prefix list for which to return the entries. The
    #   default is the current version.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @return [Types::GetManagedPrefixListEntriesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetManagedPrefixListEntriesResult#entries #data.entries} => Array&lt;Types::PrefixListEntry&gt; (This method conflicts with a method on Response, call it through the data member)
    #   * {Types::GetManagedPrefixListEntriesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_managed_prefix_list_entries({
    #     dry_run: false,
    #     prefix_list_id: "PrefixListResourceId", # required
    #     target_version: 1,
    #     max_results: 1,
    #     next_token: "NextToken",
    #   })
    #
    # @example Response structure
    #
    #   resp.data.entries #=> Array
    #   resp.data.entries[0].cidr #=> String
    #   resp.data.entries[0].description #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetManagedPrefixListEntries AWS API Documentation
    #
    # @overload get_managed_prefix_list_entries(params = {})
    # @param [Hash] params ({})
    def get_managed_prefix_list_entries(params = {}, options = {})
      req = build_request(:get_managed_prefix_list_entries, params)
      req.send_request(options)
    end

    # Gets the findings for the specified Network Access Scope analysis.
    #
    # @option params [required, String] :network_insights_access_scope_analysis_id
    #   The ID of the Network Access Scope analysis.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetNetworkInsightsAccessScopeAnalysisFindingsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetNetworkInsightsAccessScopeAnalysisFindingsResult#network_insights_access_scope_analysis_id #network_insights_access_scope_analysis_id} => String
    #   * {Types::GetNetworkInsightsAccessScopeAnalysisFindingsResult#analysis_status #analysis_status} => String
    #   * {Types::GetNetworkInsightsAccessScopeAnalysisFindingsResult#analysis_findings #analysis_findings} => Array&lt;Types::AccessScopeAnalysisFinding&gt;
    #   * {Types::GetNetworkInsightsAccessScopeAnalysisFindingsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_network_insights_access_scope_analysis_findings({
    #     network_insights_access_scope_analysis_id: "NetworkInsightsAccessScopeAnalysisId", # required
    #     max_results: 1,
    #     next_token: "NextToken",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.network_insights_access_scope_analysis_id #=> String
    #   resp.analysis_status #=> String, one of "running", "succeeded", "failed"
    #   resp.analysis_findings #=> Array
    #   resp.analysis_findings[0].network_insights_access_scope_analysis_id #=> String
    #   resp.analysis_findings[0].network_insights_access_scope_id #=> String
    #   resp.analysis_findings[0].finding_id #=> String
    #   resp.analysis_findings[0].finding_components #=> Array
    #   resp.analysis_findings[0].finding_components[0].sequence_number #=> Integer
    #   resp.analysis_findings[0].finding_components[0].acl_rule.cidr #=> String
    #   resp.analysis_findings[0].finding_components[0].acl_rule.egress #=> Boolean
    #   resp.analysis_findings[0].finding_components[0].acl_rule.port_range.from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].acl_rule.port_range.to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].acl_rule.protocol #=> String
    #   resp.analysis_findings[0].finding_components[0].acl_rule.rule_action #=> String
    #   resp.analysis_findings[0].finding_components[0].acl_rule.rule_number #=> Integer
    #   resp.analysis_findings[0].finding_components[0].attached_to.id #=> String
    #   resp.analysis_findings[0].finding_components[0].attached_to.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].attached_to.name #=> String
    #   resp.analysis_findings[0].finding_components[0].component.id #=> String
    #   resp.analysis_findings[0].finding_components[0].component.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].component.name #=> String
    #   resp.analysis_findings[0].finding_components[0].destination_vpc.id #=> String
    #   resp.analysis_findings[0].finding_components[0].destination_vpc.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].destination_vpc.name #=> String
    #   resp.analysis_findings[0].finding_components[0].outbound_header.destination_addresses #=> Array
    #   resp.analysis_findings[0].finding_components[0].outbound_header.destination_addresses[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].outbound_header.destination_port_ranges #=> Array
    #   resp.analysis_findings[0].finding_components[0].outbound_header.destination_port_ranges[0].from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].outbound_header.destination_port_ranges[0].to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].outbound_header.protocol #=> String
    #   resp.analysis_findings[0].finding_components[0].outbound_header.source_addresses #=> Array
    #   resp.analysis_findings[0].finding_components[0].outbound_header.source_addresses[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].outbound_header.source_port_ranges #=> Array
    #   resp.analysis_findings[0].finding_components[0].outbound_header.source_port_ranges[0].from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].outbound_header.source_port_ranges[0].to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].inbound_header.destination_addresses #=> Array
    #   resp.analysis_findings[0].finding_components[0].inbound_header.destination_addresses[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].inbound_header.destination_port_ranges #=> Array
    #   resp.analysis_findings[0].finding_components[0].inbound_header.destination_port_ranges[0].from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].inbound_header.destination_port_ranges[0].to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].inbound_header.protocol #=> String
    #   resp.analysis_findings[0].finding_components[0].inbound_header.source_addresses #=> Array
    #   resp.analysis_findings[0].finding_components[0].inbound_header.source_addresses[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].inbound_header.source_port_ranges #=> Array
    #   resp.analysis_findings[0].finding_components[0].inbound_header.source_port_ranges[0].from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].inbound_header.source_port_ranges[0].to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].route_table_route.destination_cidr #=> String
    #   resp.analysis_findings[0].finding_components[0].route_table_route.destination_prefix_list_id #=> String
    #   resp.analysis_findings[0].finding_components[0].route_table_route.egress_only_internet_gateway_id #=> String
    #   resp.analysis_findings[0].finding_components[0].route_table_route.gateway_id #=> String
    #   resp.analysis_findings[0].finding_components[0].route_table_route.instance_id #=> String
    #   resp.analysis_findings[0].finding_components[0].route_table_route.nat_gateway_id #=> String
    #   resp.analysis_findings[0].finding_components[0].route_table_route.network_interface_id #=> String
    #   resp.analysis_findings[0].finding_components[0].route_table_route.origin #=> String
    #   resp.analysis_findings[0].finding_components[0].route_table_route.transit_gateway_id #=> String
    #   resp.analysis_findings[0].finding_components[0].route_table_route.vpc_peering_connection_id #=> String
    #   resp.analysis_findings[0].finding_components[0].route_table_route.state #=> String
    #   resp.analysis_findings[0].finding_components[0].route_table_route.carrier_gateway_id #=> String
    #   resp.analysis_findings[0].finding_components[0].route_table_route.core_network_arn #=> String
    #   resp.analysis_findings[0].finding_components[0].route_table_route.local_gateway_id #=> String
    #   resp.analysis_findings[0].finding_components[0].security_group_rule.cidr #=> String
    #   resp.analysis_findings[0].finding_components[0].security_group_rule.direction #=> String
    #   resp.analysis_findings[0].finding_components[0].security_group_rule.security_group_id #=> String
    #   resp.analysis_findings[0].finding_components[0].security_group_rule.port_range.from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].security_group_rule.port_range.to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].security_group_rule.prefix_list_id #=> String
    #   resp.analysis_findings[0].finding_components[0].security_group_rule.protocol #=> String
    #   resp.analysis_findings[0].finding_components[0].source_vpc.id #=> String
    #   resp.analysis_findings[0].finding_components[0].source_vpc.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].source_vpc.name #=> String
    #   resp.analysis_findings[0].finding_components[0].subnet.id #=> String
    #   resp.analysis_findings[0].finding_components[0].subnet.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].subnet.name #=> String
    #   resp.analysis_findings[0].finding_components[0].vpc.id #=> String
    #   resp.analysis_findings[0].finding_components[0].vpc.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].vpc.name #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details #=> Array
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].additional_detail_type #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].component.id #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].component.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].component.name #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].vpc_endpoint_service.id #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].vpc_endpoint_service.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].vpc_endpoint_service.name #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].rule_options #=> Array
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].rule_options[0].keyword #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].rule_options[0].settings #=> Array
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].rule_options[0].settings[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].rule_group_type_pairs #=> Array
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].rule_group_type_pairs[0].rule_group_arn #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].rule_group_type_pairs[0].rule_group_type #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].rule_group_rule_options_pairs #=> Array
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_group_arn #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options #=> Array
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].keyword #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].settings #=> Array
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].settings[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].service_name #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].load_balancers #=> Array
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].load_balancers[0].id #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].load_balancers[0].arn #=> String
    #   resp.analysis_findings[0].finding_components[0].additional_details[0].load_balancers[0].name #=> String
    #   resp.analysis_findings[0].finding_components[0].transit_gateway.id #=> String
    #   resp.analysis_findings[0].finding_components[0].transit_gateway.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].transit_gateway.name #=> String
    #   resp.analysis_findings[0].finding_components[0].transit_gateway_route_table_route.destination_cidr #=> String
    #   resp.analysis_findings[0].finding_components[0].transit_gateway_route_table_route.state #=> String
    #   resp.analysis_findings[0].finding_components[0].transit_gateway_route_table_route.route_origin #=> String
    #   resp.analysis_findings[0].finding_components[0].transit_gateway_route_table_route.prefix_list_id #=> String
    #   resp.analysis_findings[0].finding_components[0].transit_gateway_route_table_route.attachment_id #=> String
    #   resp.analysis_findings[0].finding_components[0].transit_gateway_route_table_route.resource_id #=> String
    #   resp.analysis_findings[0].finding_components[0].transit_gateway_route_table_route.resource_type #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].acl.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].acl.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].acl.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].acl_rule.cidr #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].acl_rule.egress #=> Boolean
    #   resp.analysis_findings[0].finding_components[0].explanations[0].acl_rule.port_range.from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].acl_rule.port_range.to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].acl_rule.protocol #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].acl_rule.rule_action #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].acl_rule.rule_number #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].address #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].addresses #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].addresses[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].attached_to.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].attached_to.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].attached_to.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].availability_zones #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].availability_zones[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].cidrs #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].cidrs[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].component.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].component.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].component.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].customer_gateway.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].customer_gateway.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].customer_gateway.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].destination.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].destination.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].destination.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].destination_vpc.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].destination_vpc.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].destination_vpc.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].direction #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].explanation_code #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].ingress_route_table.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].ingress_route_table.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].ingress_route_table.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].internet_gateway.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].internet_gateway.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].internet_gateway.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].classic_load_balancer_listener.load_balancer_port #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].classic_load_balancer_listener.instance_port #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_listener_port #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_target.address #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_target.availability_zone #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_target.instance.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_target.instance.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_target.instance.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_target.port #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_target_group.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_target_group.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_target_group.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_target_groups #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_target_groups[0].id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_target_groups[0].arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_target_groups[0].name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].load_balancer_target_port #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].elastic_load_balancer_listener.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].elastic_load_balancer_listener.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].elastic_load_balancer_listener.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].missing_component #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].nat_gateway.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].nat_gateway.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].nat_gateway.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].network_interface.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].network_interface.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].network_interface.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].packet_field #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpc_peering_connection.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpc_peering_connection.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpc_peering_connection.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].port #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].port_ranges #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].port_ranges[0].from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].port_ranges[0].to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].prefix_list.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].prefix_list.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].prefix_list.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].protocols #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].protocols[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table_route.destination_cidr #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table_route.destination_prefix_list_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table_route.egress_only_internet_gateway_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table_route.gateway_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table_route.instance_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table_route.nat_gateway_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table_route.network_interface_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table_route.origin #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table_route.transit_gateway_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table_route.vpc_peering_connection_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table_route.state #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table_route.carrier_gateway_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table_route.core_network_arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table_route.local_gateway_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].route_table.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].security_group.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].security_group.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].security_group.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].security_group_rule.cidr #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].security_group_rule.direction #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].security_group_rule.security_group_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].security_group_rule.port_range.from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].security_group_rule.port_range.to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].security_group_rule.prefix_list_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].security_group_rule.protocol #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].security_groups #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].security_groups[0].id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].security_groups[0].arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].security_groups[0].name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].source_vpc.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].source_vpc.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].source_vpc.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].state #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].subnet.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].subnet.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].subnet.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].subnet_route_table.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].subnet_route_table.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].subnet_route_table.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpc.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpc.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpc.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpc_endpoint.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpc_endpoint.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpc_endpoint.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpn_connection.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpn_connection.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpn_connection.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpn_gateway.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpn_gateway.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].vpn_gateway.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway_route_table.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway_route_table.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway_route_table.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway_route_table_route.destination_cidr #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway_route_table_route.state #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway_route_table_route.route_origin #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway_route_table_route.prefix_list_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway_route_table_route.attachment_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway_route_table_route.resource_id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway_route_table_route.resource_type #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway_attachment.id #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway_attachment.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].transit_gateway_attachment.name #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].component_account #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].component_region #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.rule_group_arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.sources #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.sources[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.destinations #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.destinations[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.source_ports #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.source_ports[0].from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.source_ports[0].to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.destination_ports #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.destination_ports[0].from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.destination_ports[0].to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.protocols #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.protocols[0] #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.rule_action #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateless_rule.priority #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateful_rule.rule_group_arn #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateful_rule.sources #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateful_rule.sources[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateful_rule.destinations #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateful_rule.destinations[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateful_rule.source_ports #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateful_rule.source_ports[0].from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateful_rule.source_ports[0].to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateful_rule.destination_ports #=> Array
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateful_rule.destination_ports[0].from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateful_rule.destination_ports[0].to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateful_rule.protocol #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateful_rule.rule_action #=> String
    #   resp.analysis_findings[0].finding_components[0].explanations[0].firewall_stateful_rule.direction #=> String
    #   resp.analysis_findings[0].finding_components[0].elastic_load_balancer_listener.id #=> String
    #   resp.analysis_findings[0].finding_components[0].elastic_load_balancer_listener.arn #=> String
    #   resp.analysis_findings[0].finding_components[0].elastic_load_balancer_listener.name #=> String
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.rule_group_arn #=> String
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.sources #=> Array
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.sources[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.destinations #=> Array
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.destinations[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.source_ports #=> Array
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.source_ports[0].from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.source_ports[0].to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.destination_ports #=> Array
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.destination_ports[0].from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.destination_ports[0].to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.protocols #=> Array
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.protocols[0] #=> Integer
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.rule_action #=> String
    #   resp.analysis_findings[0].finding_components[0].firewall_stateless_rule.priority #=> Integer
    #   resp.analysis_findings[0].finding_components[0].firewall_stateful_rule.rule_group_arn #=> String
    #   resp.analysis_findings[0].finding_components[0].firewall_stateful_rule.sources #=> Array
    #   resp.analysis_findings[0].finding_components[0].firewall_stateful_rule.sources[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].firewall_stateful_rule.destinations #=> Array
    #   resp.analysis_findings[0].finding_components[0].firewall_stateful_rule.destinations[0] #=> String
    #   resp.analysis_findings[0].finding_components[0].firewall_stateful_rule.source_ports #=> Array
    #   resp.analysis_findings[0].finding_components[0].firewall_stateful_rule.source_ports[0].from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].firewall_stateful_rule.source_ports[0].to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].firewall_stateful_rule.destination_ports #=> Array
    #   resp.analysis_findings[0].finding_components[0].firewall_stateful_rule.destination_ports[0].from #=> Integer
    #   resp.analysis_findings[0].finding_components[0].firewall_stateful_rule.destination_ports[0].to #=> Integer
    #   resp.analysis_findings[0].finding_components[0].firewall_stateful_rule.protocol #=> String
    #   resp.analysis_findings[0].finding_components[0].firewall_stateful_rule.rule_action #=> String
    #   resp.analysis_findings[0].finding_components[0].firewall_stateful_rule.direction #=> String
    #   resp.analysis_findings[0].finding_components[0].service_name #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetNetworkInsightsAccessScopeAnalysisFindings AWS API Documentation
    #
    # @overload get_network_insights_access_scope_analysis_findings(params = {})
    # @param [Hash] params ({})
    def get_network_insights_access_scope_analysis_findings(params = {}, options = {})
      req = build_request(:get_network_insights_access_scope_analysis_findings, params)
      req.send_request(options)
    end

    # Gets the content for the specified Network Access Scope.
    #
    # @option params [required, String] :network_insights_access_scope_id
    #   The ID of the Network Access Scope.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetNetworkInsightsAccessScopeContentResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetNetworkInsightsAccessScopeContentResult#network_insights_access_scope_content #network_insights_access_scope_content} => Types::NetworkInsightsAccessScopeContent
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_network_insights_access_scope_content({
    #     network_insights_access_scope_id: "NetworkInsightsAccessScopeId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.network_insights_access_scope_content.network_insights_access_scope_id #=> String
    #   resp.network_insights_access_scope_content.match_paths #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.source_addresses #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.source_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.destination_addresses #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.destination_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.source_ports #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.source_ports[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.destination_ports #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.destination_ports[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.source_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.source_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.destination_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.destination_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.protocols #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.packet_header_statement.protocols[0] #=> String, one of "tcp", "udp"
    #   resp.network_insights_access_scope_content.match_paths[0].source.resource_statement.resources #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.resource_statement.resources[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].source.resource_statement.resource_types #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].source.resource_statement.resource_types[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.source_addresses #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.source_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.destination_addresses #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.destination_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.source_ports #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.source_ports[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.destination_ports #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.destination_ports[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.source_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.source_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.destination_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.destination_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.protocols #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.packet_header_statement.protocols[0] #=> String, one of "tcp", "udp"
    #   resp.network_insights_access_scope_content.match_paths[0].destination.resource_statement.resources #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.resource_statement.resources[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].destination.resource_statement.resource_types #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].destination.resource_statement.resource_types[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].through_resources #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].through_resources[0].resource_statement.resources #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].through_resources[0].resource_statement.resources[0] #=> String
    #   resp.network_insights_access_scope_content.match_paths[0].through_resources[0].resource_statement.resource_types #=> Array
    #   resp.network_insights_access_scope_content.match_paths[0].through_resources[0].resource_statement.resource_types[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.source_addresses #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.source_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.destination_addresses #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.destination_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.source_ports #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.source_ports[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.destination_ports #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.destination_ports[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.source_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.source_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.destination_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.destination_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.protocols #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.packet_header_statement.protocols[0] #=> String, one of "tcp", "udp"
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.resource_statement.resources #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.resource_statement.resources[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.resource_statement.resource_types #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].source.resource_statement.resource_types[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.source_addresses #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.source_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.destination_addresses #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.destination_addresses[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.source_ports #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.source_ports[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.destination_ports #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.destination_ports[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.source_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.source_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.destination_prefix_lists #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.destination_prefix_lists[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.protocols #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.packet_header_statement.protocols[0] #=> String, one of "tcp", "udp"
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.resource_statement.resources #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.resource_statement.resources[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.resource_statement.resource_types #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].destination.resource_statement.resource_types[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].through_resources #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].through_resources[0].resource_statement.resources #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].through_resources[0].resource_statement.resources[0] #=> String
    #   resp.network_insights_access_scope_content.exclude_paths[0].through_resources[0].resource_statement.resource_types #=> Array
    #   resp.network_insights_access_scope_content.exclude_paths[0].through_resources[0].resource_statement.resource_types[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetNetworkInsightsAccessScopeContent AWS API Documentation
    #
    # @overload get_network_insights_access_scope_content(params = {})
    # @param [Hash] params ({})
    def get_network_insights_access_scope_content(params = {}, options = {})
      req = build_request(:get_network_insights_access_scope_content, params)
      req.send_request(options)
    end

    # Retrieves the encrypted administrator password for a running Windows
    # instance.
    #
    # The Windows password is generated at boot by the `EC2Config` service
    # or `EC2Launch` scripts (Windows Server 2016 and later). This usually
    # only happens the first time an instance is launched. For more
    # information, see [EC2Config][1] and [EC2Launch][2] in the *Amazon EC2
    # User Guide*.
    #
    # For the `EC2Config` service, the password is not generated for
    # rebundled AMIs unless `Ec2SetPassword` is enabled before bundling.
    #
    # The password is encrypted using the key pair that you specified when
    # you launched the instance. You must provide the corresponding key pair
    # file.
    #
    # When you launch an instance, password generation and encryption may
    # take a few minutes. If you try to retrieve the password before it's
    # available, the output returns an empty string. We recommend that you
    # wait up to 15 minutes after launching an instance before trying to
    # retrieve the generated password.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UsingConfig_WinAMI.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2launch.html
    #
    # @option params [required, String] :instance_id
    #   The ID of the Windows instance.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetPasswordDataResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetPasswordDataResult#instance_id #instance_id} => String
    #   * {Types::GetPasswordDataResult#timestamp #timestamp} => Time
    #   * {Types::GetPasswordDataResult#password_data #password_data} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_password_data({
    #     instance_id: "InstanceId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_id #=> String
    #   resp.timestamp #=> Time
    #   resp.password_data #=> String
    #
    #
    # The following waiters are defined for this operation (see {Client#wait_until} for detailed usage):
    #
    #   * password_data_available
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetPasswordData AWS API Documentation
    #
    # @overload get_password_data(params = {})
    # @param [Hash] params ({})
    def get_password_data(params = {}, options = {})
      req = build_request(:get_password_data, params)
      req.send_request(options)
    end

    # Returns a quote and exchange information for exchanging one or more
    # specified Convertible Reserved Instances for a new Convertible
    # Reserved Instance. If the exchange cannot be performed, the reason is
    # returned in the response. Use AcceptReservedInstancesExchangeQuote to
    # perform the exchange.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :reserved_instance_ids
    #   The IDs of the Convertible Reserved Instances to exchange.
    #
    # @option params [Array<Types::TargetConfigurationRequest>] :target_configurations
    #   The configuration of the target Convertible Reserved Instance to
    #   exchange for your current Convertible Reserved Instances.
    #
    # @return [Types::GetReservedInstancesExchangeQuoteResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetReservedInstancesExchangeQuoteResult#currency_code #currency_code} => String
    #   * {Types::GetReservedInstancesExchangeQuoteResult#is_valid_exchange #is_valid_exchange} => Boolean
    #   * {Types::GetReservedInstancesExchangeQuoteResult#output_reserved_instances_will_expire_at #output_reserved_instances_will_expire_at} => Time
    #   * {Types::GetReservedInstancesExchangeQuoteResult#payment_due #payment_due} => String
    #   * {Types::GetReservedInstancesExchangeQuoteResult#reserved_instance_value_rollup #reserved_instance_value_rollup} => Types::ReservationValue
    #   * {Types::GetReservedInstancesExchangeQuoteResult#reserved_instance_value_set #reserved_instance_value_set} => Array&lt;Types::ReservedInstanceReservationValue&gt;
    #   * {Types::GetReservedInstancesExchangeQuoteResult#target_configuration_value_rollup #target_configuration_value_rollup} => Types::ReservationValue
    #   * {Types::GetReservedInstancesExchangeQuoteResult#target_configuration_value_set #target_configuration_value_set} => Array&lt;Types::TargetReservationValue&gt;
    #   * {Types::GetReservedInstancesExchangeQuoteResult#validation_failure_reason #validation_failure_reason} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_reserved_instances_exchange_quote({
    #     dry_run: false,
    #     reserved_instance_ids: ["ReservationId"], # required
    #     target_configurations: [
    #       {
    #         instance_count: 1,
    #         offering_id: "ReservedInstancesOfferingId", # required
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.currency_code #=> String
    #   resp.is_valid_exchange #=> Boolean
    #   resp.output_reserved_instances_will_expire_at #=> Time
    #   resp.payment_due #=> String
    #   resp.reserved_instance_value_rollup.hourly_price #=> String
    #   resp.reserved_instance_value_rollup.remaining_total_value #=> String
    #   resp.reserved_instance_value_rollup.remaining_upfront_value #=> String
    #   resp.reserved_instance_value_set #=> Array
    #   resp.reserved_instance_value_set[0].reservation_value.hourly_price #=> String
    #   resp.reserved_instance_value_set[0].reservation_value.remaining_total_value #=> String
    #   resp.reserved_instance_value_set[0].reservation_value.remaining_upfront_value #=> String
    #   resp.reserved_instance_value_set[0].reserved_instance_id #=> String
    #   resp.target_configuration_value_rollup.hourly_price #=> String
    #   resp.target_configuration_value_rollup.remaining_total_value #=> String
    #   resp.target_configuration_value_rollup.remaining_upfront_value #=> String
    #   resp.target_configuration_value_set #=> Array
    #   resp.target_configuration_value_set[0].reservation_value.hourly_price #=> String
    #   resp.target_configuration_value_set[0].reservation_value.remaining_total_value #=> String
    #   resp.target_configuration_value_set[0].reservation_value.remaining_upfront_value #=> String
    #   resp.target_configuration_value_set[0].target_configuration.instance_count #=> Integer
    #   resp.target_configuration_value_set[0].target_configuration.offering_id #=> String
    #   resp.validation_failure_reason #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetReservedInstancesExchangeQuote AWS API Documentation
    #
    # @overload get_reserved_instances_exchange_quote(params = {})
    # @param [Hash] params ({})
    def get_reserved_instances_exchange_quote(params = {}, options = {})
      req = build_request(:get_reserved_instances_exchange_quote, params)
      req.send_request(options)
    end

    # Gets security groups that can be associated by the Amazon Web Services
    # account making the request with network interfaces in the specified
    # VPC.
    #
    # @option params [required, String] :vpc_id
    #   The VPC ID where the security group can be used.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters. If using multiple filters, the results include security
    #   groups which match all filters.
    #
    #   * `group-id`: The security group ID.
    #
    #   * `description`: The security group's description.
    #
    #   * `group-name`: The security group name.
    #
    #   * `owner-id`: The security group owner ID.
    #
    #   * `primary-vpc-id`: The VPC ID in which the security group was
    #     created.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetSecurityGroupsForVpcResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetSecurityGroupsForVpcResult#next_token #next_token} => String
    #   * {Types::GetSecurityGroupsForVpcResult#security_group_for_vpcs #security_group_for_vpcs} => Array&lt;Types::SecurityGroupForVpc&gt;
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_security_groups_for_vpc({
    #     vpc_id: "VpcId", # required
    #     next_token: "String",
    #     max_results: 1,
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.next_token #=> String
    #   resp.security_group_for_vpcs #=> Array
    #   resp.security_group_for_vpcs[0].description #=> String
    #   resp.security_group_for_vpcs[0].group_name #=> String
    #   resp.security_group_for_vpcs[0].owner_id #=> String
    #   resp.security_group_for_vpcs[0].group_id #=> String
    #   resp.security_group_for_vpcs[0].tags #=> Array
    #   resp.security_group_for_vpcs[0].tags[0].key #=> String
    #   resp.security_group_for_vpcs[0].tags[0].value #=> String
    #   resp.security_group_for_vpcs[0].primary_vpc_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetSecurityGroupsForVpc AWS API Documentation
    #
    # @overload get_security_groups_for_vpc(params = {})
    # @param [Hash] params ({})
    def get_security_groups_for_vpc(params = {}, options = {})
      req = build_request(:get_security_groups_for_vpc, params)
      req.send_request(options)
    end

    # Retrieves the access status of your account to the EC2 serial console
    # of all instances. By default, access to the EC2 serial console is
    # disabled for your account. For more information, see [Manage account
    # access to the EC2 serial console][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/configure-access-to-serial-console.html#serial-console-account-access
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetSerialConsoleAccessStatusResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetSerialConsoleAccessStatusResult#serial_console_access_enabled #serial_console_access_enabled} => Boolean
    #   * {Types::GetSerialConsoleAccessStatusResult#managed_by #managed_by} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_serial_console_access_status({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.serial_console_access_enabled #=> Boolean
    #   resp.managed_by #=> String, one of "account", "declarative-policy"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetSerialConsoleAccessStatus AWS API Documentation
    #
    # @overload get_serial_console_access_status(params = {})
    # @param [Hash] params ({})
    def get_serial_console_access_status(params = {}, options = {})
      req = build_request(:get_serial_console_access_status, params)
      req.send_request(options)
    end

    # Gets the current state of *block public access for snapshots* setting
    # for the account and Region.
    #
    # For more information, see [ Block public access for snapshots][1] in
    # the *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/block-public-access-snapshots.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetSnapshotBlockPublicAccessStateResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetSnapshotBlockPublicAccessStateResult#state #state} => String
    #   * {Types::GetSnapshotBlockPublicAccessStateResult#managed_by #managed_by} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_snapshot_block_public_access_state({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.state #=> String, one of "block-all-sharing", "block-new-sharing", "unblocked"
    #   resp.managed_by #=> String, one of "account", "declarative-policy"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetSnapshotBlockPublicAccessState AWS API Documentation
    #
    # @overload get_snapshot_block_public_access_state(params = {})
    # @param [Hash] params ({})
    def get_snapshot_block_public_access_state(params = {}, options = {})
      req = build_request(:get_snapshot_block_public_access_state, params)
      req.send_request(options)
    end

    # Calculates the Spot placement score for a Region or Availability Zone
    # based on the specified target capacity and compute requirements.
    #
    # You can specify your compute requirements either by using
    # `InstanceRequirementsWithMetadata` and letting Amazon EC2 choose the
    # optimal instance types to fulfill your Spot request, or you can
    # specify the instance types by using `InstanceTypes`.
    #
    # For more information, see [Spot placement score][1] in the *Amazon EC2
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-placement-score.html
    #
    # @option params [Array<String>] :instance_types
    #   The instance types. We recommend that you specify at least three
    #   instance types. If you specify one or two instance types, or specify
    #   variations of a single instance type (for example, an `m3.xlarge` with
    #   and without instance storage), the returned placement score will
    #   always be low.
    #
    #   If you specify `InstanceTypes`, you can't specify
    #   `InstanceRequirementsWithMetadata`.
    #
    # @option params [required, Integer] :target_capacity
    #   The target capacity.
    #
    # @option params [String] :target_capacity_unit_type
    #   The unit for the target capacity.
    #
    # @option params [Boolean] :single_availability_zone
    #   Specify `true` so that the response returns a list of scored
    #   Availability Zones. Otherwise, the response returns a list of scored
    #   Regions.
    #
    #   A list of scored Availability Zones is useful if you want to launch
    #   all of your Spot capacity into a single Availability Zone.
    #
    # @option params [Array<String>] :region_names
    #   The Regions used to narrow down the list of Regions to be scored.
    #   Enter the Region code, for example, `us-east-1`.
    #
    # @option params [Types::InstanceRequirementsWithMetadataRequest] :instance_requirements_with_metadata
    #   The attributes for the instance types. When you specify instance
    #   attributes, Amazon EC2 will identify instance types with those
    #   attributes.
    #
    #   If you specify `InstanceRequirementsWithMetadata`, you can't specify
    #   `InstanceTypes`.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @return [Types::GetSpotPlacementScoresResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetSpotPlacementScoresResult#spot_placement_scores #spot_placement_scores} => Array&lt;Types::SpotPlacementScore&gt;
    #   * {Types::GetSpotPlacementScoresResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_spot_placement_scores({
    #     instance_types: ["String"],
    #     target_capacity: 1, # required
    #     target_capacity_unit_type: "vcpu", # accepts vcpu, memory-mib, units
    #     single_availability_zone: false,
    #     region_names: ["String"],
    #     instance_requirements_with_metadata: {
    #       architecture_types: ["i386"], # accepts i386, x86_64, arm64, x86_64_mac, arm64_mac
    #       virtualization_types: ["hvm"], # accepts hvm, paravirtual
    #       instance_requirements: {
    #         v_cpu_count: { # required
    #           min: 1, # required
    #           max: 1,
    #         },
    #         memory_mi_b: { # required
    #           min: 1, # required
    #           max: 1,
    #         },
    #         cpu_manufacturers: ["intel"], # accepts intel, amd, amazon-web-services, apple
    #         memory_gi_b_per_v_cpu: {
    #           min: 1.0,
    #           max: 1.0,
    #         },
    #         excluded_instance_types: ["ExcludedInstanceType"],
    #         instance_generations: ["current"], # accepts current, previous
    #         spot_max_price_percentage_over_lowest_price: 1,
    #         on_demand_max_price_percentage_over_lowest_price: 1,
    #         bare_metal: "included", # accepts included, required, excluded
    #         burstable_performance: "included", # accepts included, required, excluded
    #         require_hibernate_support: false,
    #         network_interface_count: {
    #           min: 1,
    #           max: 1,
    #         },
    #         local_storage: "included", # accepts included, required, excluded
    #         local_storage_types: ["hdd"], # accepts hdd, ssd
    #         total_local_storage_gb: {
    #           min: 1.0,
    #           max: 1.0,
    #         },
    #         baseline_ebs_bandwidth_mbps: {
    #           min: 1,
    #           max: 1,
    #         },
    #         accelerator_types: ["gpu"], # accepts gpu, fpga, inference
    #         accelerator_count: {
    #           min: 1,
    #           max: 1,
    #         },
    #         accelerator_manufacturers: ["amazon-web-services"], # accepts amazon-web-services, amd, nvidia, xilinx, habana
    #         accelerator_names: ["a100"], # accepts a100, inferentia, k520, k80, m60, radeon-pro-v520, t4, vu9p, v100, a10g, h100, t4g
    #         accelerator_total_memory_mi_b: {
    #           min: 1,
    #           max: 1,
    #         },
    #         network_bandwidth_gbps: {
    #           min: 1.0,
    #           max: 1.0,
    #         },
    #         allowed_instance_types: ["AllowedInstanceType"],
    #         max_spot_price_as_percentage_of_optimal_on_demand_price: 1,
    #         baseline_performance_factors: {
    #           cpu: {
    #             references: [
    #               {
    #                 instance_family: "String",
    #               },
    #             ],
    #           },
    #         },
    #       },
    #     },
    #     dry_run: false,
    #     max_results: 1,
    #     next_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.spot_placement_scores #=> Array
    #   resp.spot_placement_scores[0].region #=> String
    #   resp.spot_placement_scores[0].availability_zone_id #=> String
    #   resp.spot_placement_scores[0].score #=> Integer
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetSpotPlacementScores AWS API Documentation
    #
    # @overload get_spot_placement_scores(params = {})
    # @param [Hash] params ({})
    def get_spot_placement_scores(params = {}, options = {})
      req = build_request(:get_spot_placement_scores, params)
      req.send_request(options)
    end

    # Gets information about the subnet CIDR reservations.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `reservationType` - The type of reservation (`prefix` \|
    #     `explicit`).
    #
    #   * `subnet-id` - The ID of the subnet.
    #
    #   * `tag`:&lt;key&gt; - The key/value combination of a tag assigned to
    #     the resource. Use the tag key in the filter name and the tag value
    #     as the filter value. For example, to find all resources that have a
    #     tag with the key `Owner` and the value `TeamA`, specify `tag:Owner`
    #     for the filter name and `TeamA` for the filter value.
    #
    #   * `tag-key` - The key of a tag assigned to the resource. Use this
    #     filter to find all resources assigned a tag with a specific key,
    #     regardless of the tag value.
    #
    # @option params [required, String] :subnet_id
    #   The ID of the subnet.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @return [Types::GetSubnetCidrReservationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetSubnetCidrReservationsResult#subnet_ipv_4_cidr_reservations #subnet_ipv_4_cidr_reservations} => Array&lt;Types::SubnetCidrReservation&gt;
    #   * {Types::GetSubnetCidrReservationsResult#subnet_ipv_6_cidr_reservations #subnet_ipv_6_cidr_reservations} => Array&lt;Types::SubnetCidrReservation&gt;
    #   * {Types::GetSubnetCidrReservationsResult#next_token #next_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_subnet_cidr_reservations({
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     subnet_id: "SubnetId", # required
    #     dry_run: false,
    #     next_token: "String",
    #     max_results: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.subnet_ipv_4_cidr_reservations #=> Array
    #   resp.subnet_ipv_4_cidr_reservations[0].subnet_cidr_reservation_id #=> String
    #   resp.subnet_ipv_4_cidr_reservations[0].subnet_id #=> String
    #   resp.subnet_ipv_4_cidr_reservations[0].cidr #=> String
    #   resp.subnet_ipv_4_cidr_reservations[0].reservation_type #=> String, one of "prefix", "explicit"
    #   resp.subnet_ipv_4_cidr_reservations[0].owner_id #=> String
    #   resp.subnet_ipv_4_cidr_reservations[0].description #=> String
    #   resp.subnet_ipv_4_cidr_reservations[0].tags #=> Array
    #   resp.subnet_ipv_4_cidr_reservations[0].tags[0].key #=> String
    #   resp.subnet_ipv_4_cidr_reservations[0].tags[0].value #=> String
    #   resp.subnet_ipv_6_cidr_reservations #=> Array
    #   resp.subnet_ipv_6_cidr_reservations[0].subnet_cidr_reservation_id #=> String
    #   resp.subnet_ipv_6_cidr_reservations[0].subnet_id #=> String
    #   resp.subnet_ipv_6_cidr_reservations[0].cidr #=> String
    #   resp.subnet_ipv_6_cidr_reservations[0].reservation_type #=> String, one of "prefix", "explicit"
    #   resp.subnet_ipv_6_cidr_reservations[0].owner_id #=> String
    #   resp.subnet_ipv_6_cidr_reservations[0].description #=> String
    #   resp.subnet_ipv_6_cidr_reservations[0].tags #=> Array
    #   resp.subnet_ipv_6_cidr_reservations[0].tags[0].key #=> String
    #   resp.subnet_ipv_6_cidr_reservations[0].tags[0].value #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetSubnetCidrReservations AWS API Documentation
    #
    # @overload get_subnet_cidr_reservations(params = {})
    # @param [Hash] params ({})
    def get_subnet_cidr_reservations(params = {}, options = {})
      req = build_request(:get_subnet_cidr_reservations, params)
      req.send_request(options)
    end

    # Lists the route tables to which the specified resource attachment
    # propagates routes.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the attachment.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `transit-gateway-route-table-id` - The ID of the transit gateway
    #     route table.
    #
    #   ^
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetTransitGatewayAttachmentPropagationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetTransitGatewayAttachmentPropagationsResult#transit_gateway_attachment_propagations #transit_gateway_attachment_propagations} => Array&lt;Types::TransitGatewayAttachmentPropagation&gt;
    #   * {Types::GetTransitGatewayAttachmentPropagationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_transit_gateway_attachment_propagations({
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_attachment_propagations #=> Array
    #   resp.transit_gateway_attachment_propagations[0].transit_gateway_route_table_id #=> String
    #   resp.transit_gateway_attachment_propagations[0].state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetTransitGatewayAttachmentPropagations AWS API Documentation
    #
    # @overload get_transit_gateway_attachment_propagations(params = {})
    # @param [Hash] params ({})
    def get_transit_gateway_attachment_propagations(params = {}, options = {})
      req = build_request(:get_transit_gateway_attachment_propagations, params)
      req.send_request(options)
    end

    # Gets information about the associations for the transit gateway
    # multicast domain.
    #
    # @option params [required, String] :transit_gateway_multicast_domain_id
    #   The ID of the transit gateway multicast domain.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `resource-id` - The ID of the resource.
    #
    #   * `resource-type` - The type of resource. The valid value is: `vpc`.
    #
    #   * `state` - The state of the subnet association. Valid values are
    #     `associated` \| `associating` \| `disassociated` \|
    #     `disassociating`.
    #
    #   * `subnet-id` - The ID of the subnet.
    #
    #   * `transit-gateway-attachment-id` - The id of the transit gateway
    #     attachment.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetTransitGatewayMulticastDomainAssociationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetTransitGatewayMulticastDomainAssociationsResult#multicast_domain_associations #multicast_domain_associations} => Array&lt;Types::TransitGatewayMulticastDomainAssociation&gt;
    #   * {Types::GetTransitGatewayMulticastDomainAssociationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_transit_gateway_multicast_domain_associations({
    #     transit_gateway_multicast_domain_id: "TransitGatewayMulticastDomainId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.multicast_domain_associations #=> Array
    #   resp.multicast_domain_associations[0].transit_gateway_attachment_id #=> String
    #   resp.multicast_domain_associations[0].resource_id #=> String
    #   resp.multicast_domain_associations[0].resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.multicast_domain_associations[0].resource_owner_id #=> String
    #   resp.multicast_domain_associations[0].subnet.subnet_id #=> String
    #   resp.multicast_domain_associations[0].subnet.state #=> String, one of "pendingAcceptance", "associating", "associated", "disassociating", "disassociated", "rejected", "failed"
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetTransitGatewayMulticastDomainAssociations AWS API Documentation
    #
    # @overload get_transit_gateway_multicast_domain_associations(params = {})
    # @param [Hash] params ({})
    def get_transit_gateway_multicast_domain_associations(params = {}, options = {})
      req = build_request(:get_transit_gateway_multicast_domain_associations, params)
      req.send_request(options)
    end

    # Gets a list of the transit gateway policy table associations.
    #
    # @option params [required, String] :transit_gateway_policy_table_id
    #   The ID of the transit gateway policy table.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters associated with the transit gateway policy table.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetTransitGatewayPolicyTableAssociationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetTransitGatewayPolicyTableAssociationsResult#associations #associations} => Array&lt;Types::TransitGatewayPolicyTableAssociation&gt;
    #   * {Types::GetTransitGatewayPolicyTableAssociationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_transit_gateway_policy_table_associations({
    #     transit_gateway_policy_table_id: "TransitGatewayPolicyTableId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.associations #=> Array
    #   resp.associations[0].transit_gateway_policy_table_id #=> String
    #   resp.associations[0].transit_gateway_attachment_id #=> String
    #   resp.associations[0].resource_id #=> String
    #   resp.associations[0].resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.associations[0].state #=> String, one of "associating", "associated", "disassociating", "disassociated"
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetTransitGatewayPolicyTableAssociations AWS API Documentation
    #
    # @overload get_transit_gateway_policy_table_associations(params = {})
    # @param [Hash] params ({})
    def get_transit_gateway_policy_table_associations(params = {}, options = {})
      req = build_request(:get_transit_gateway_policy_table_associations, params)
      req.send_request(options)
    end

    # Returns a list of transit gateway policy table entries.
    #
    # @option params [required, String] :transit_gateway_policy_table_id
    #   The ID of the transit gateway policy table.
    #
    # @option params [Array<Types::Filter>] :filters
    #   The filters associated with the transit gateway policy table.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetTransitGatewayPolicyTableEntriesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetTransitGatewayPolicyTableEntriesResult#transit_gateway_policy_table_entries #transit_gateway_policy_table_entries} => Array&lt;Types::TransitGatewayPolicyTableEntry&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_transit_gateway_policy_table_entries({
    #     transit_gateway_policy_table_id: "TransitGatewayPolicyTableId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_policy_table_entries #=> Array
    #   resp.transit_gateway_policy_table_entries[0].policy_rule_number #=> String
    #   resp.transit_gateway_policy_table_entries[0].policy_rule.source_cidr_block #=> String
    #   resp.transit_gateway_policy_table_entries[0].policy_rule.source_port_range #=> String
    #   resp.transit_gateway_policy_table_entries[0].policy_rule.destination_cidr_block #=> String
    #   resp.transit_gateway_policy_table_entries[0].policy_rule.destination_port_range #=> String
    #   resp.transit_gateway_policy_table_entries[0].policy_rule.protocol #=> String
    #   resp.transit_gateway_policy_table_entries[0].policy_rule.meta_data.meta_data_key #=> String
    #   resp.transit_gateway_policy_table_entries[0].policy_rule.meta_data.meta_data_value #=> String
    #   resp.transit_gateway_policy_table_entries[0].target_route_table_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetTransitGatewayPolicyTableEntries AWS API Documentation
    #
    # @overload get_transit_gateway_policy_table_entries(params = {})
    # @param [Hash] params ({})
    def get_transit_gateway_policy_table_entries(params = {}, options = {})
      req = build_request(:get_transit_gateway_policy_table_entries, params)
      req.send_request(options)
    end

    # Gets information about the prefix list references in a specified
    # transit gateway route table.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the transit gateway route table.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `attachment.resource-id` - The ID of the resource for the
    #     attachment.
    #
    #   * `attachment.resource-type` - The type of resource for the
    #     attachment. Valid values are `vpc` \| `vpn` \|
    #     `direct-connect-gateway` \| `peering`.
    #
    #   * `attachment.transit-gateway-attachment-id` - The ID of the
    #     attachment.
    #
    #   * `is-blackhole` - Whether traffic matching the route is blocked
    #     (`true` \| `false`).
    #
    #   * `prefix-list-id` - The ID of the prefix list.
    #
    #   * `prefix-list-owner-id` - The ID of the owner of the prefix list.
    #
    #   * `state` - The state of the prefix list reference (`pending` \|
    #     `available` \| `modifying` \| `deleting`).
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetTransitGatewayPrefixListReferencesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetTransitGatewayPrefixListReferencesResult#transit_gateway_prefix_list_references #transit_gateway_prefix_list_references} => Array&lt;Types::TransitGatewayPrefixListReference&gt;
    #   * {Types::GetTransitGatewayPrefixListReferencesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_transit_gateway_prefix_list_references({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_prefix_list_references #=> Array
    #   resp.transit_gateway_prefix_list_references[0].transit_gateway_route_table_id #=> String
    #   resp.transit_gateway_prefix_list_references[0].prefix_list_id #=> String
    #   resp.transit_gateway_prefix_list_references[0].prefix_list_owner_id #=> String
    #   resp.transit_gateway_prefix_list_references[0].state #=> String, one of "pending", "available", "modifying", "deleting"
    #   resp.transit_gateway_prefix_list_references[0].blackhole #=> Boolean
    #   resp.transit_gateway_prefix_list_references[0].transit_gateway_attachment.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_prefix_list_references[0].transit_gateway_attachment.resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.transit_gateway_prefix_list_references[0].transit_gateway_attachment.resource_id #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetTransitGatewayPrefixListReferences AWS API Documentation
    #
    # @overload get_transit_gateway_prefix_list_references(params = {})
    # @param [Hash] params ({})
    def get_transit_gateway_prefix_list_references(params = {}, options = {})
      req = build_request(:get_transit_gateway_prefix_list_references, params)
      req.send_request(options)
    end

    # Gets information about the associations for the specified transit
    # gateway route table.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the transit gateway route table.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `resource-id` - The ID of the resource.
    #
    #   * `resource-type` - The resource type. Valid values are `vpc` \| `vpn`
    #     \| `direct-connect-gateway` \| `peering` \| `connect`.
    #
    #   * `transit-gateway-attachment-id` - The ID of the attachment.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetTransitGatewayRouteTableAssociationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetTransitGatewayRouteTableAssociationsResult#associations #associations} => Array&lt;Types::TransitGatewayRouteTableAssociation&gt;
    #   * {Types::GetTransitGatewayRouteTableAssociationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_transit_gateway_route_table_associations({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.associations #=> Array
    #   resp.associations[0].transit_gateway_attachment_id #=> String
    #   resp.associations[0].resource_id #=> String
    #   resp.associations[0].resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.associations[0].state #=> String, one of "associating", "associated", "disassociating", "disassociated"
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetTransitGatewayRouteTableAssociations AWS API Documentation
    #
    # @overload get_transit_gateway_route_table_associations(params = {})
    # @param [Hash] params ({})
    def get_transit_gateway_route_table_associations(params = {}, options = {})
      req = build_request(:get_transit_gateway_route_table_associations, params)
      req.send_request(options)
    end

    # Gets information about the route table propagations for the specified
    # transit gateway route table.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the transit gateway route table.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `resource-id` - The ID of the resource.
    #
    #   * `resource-type` - The resource type. Valid values are `vpc` \| `vpn`
    #     \| `direct-connect-gateway` \| `peering` \| `connect`.
    #
    #   * `transit-gateway-attachment-id` - The ID of the attachment.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetTransitGatewayRouteTablePropagationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetTransitGatewayRouteTablePropagationsResult#transit_gateway_route_table_propagations #transit_gateway_route_table_propagations} => Array&lt;Types::TransitGatewayRouteTablePropagation&gt;
    #   * {Types::GetTransitGatewayRouteTablePropagationsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_transit_gateway_route_table_propagations({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_route_table_propagations #=> Array
    #   resp.transit_gateway_route_table_propagations[0].transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_route_table_propagations[0].resource_id #=> String
    #   resp.transit_gateway_route_table_propagations[0].resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.transit_gateway_route_table_propagations[0].state #=> String, one of "enabling", "enabled", "disabling", "disabled"
    #   resp.transit_gateway_route_table_propagations[0].transit_gateway_route_table_announcement_id #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetTransitGatewayRouteTablePropagations AWS API Documentation
    #
    # @overload get_transit_gateway_route_table_propagations(params = {})
    # @param [Hash] params ({})
    def get_transit_gateway_route_table_propagations(params = {}, options = {})
      req = build_request(:get_transit_gateway_route_table_propagations, params)
      req.send_request(options)
    end

    # Get the Verified Access policy associated with the endpoint.
    #
    # @option params [required, String] :verified_access_endpoint_id
    #   The ID of the Verified Access endpoint.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetVerifiedAccessEndpointPolicyResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetVerifiedAccessEndpointPolicyResult#policy_enabled #policy_enabled} => Boolean
    #   * {Types::GetVerifiedAccessEndpointPolicyResult#policy_document #policy_document} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_verified_access_endpoint_policy({
    #     verified_access_endpoint_id: "VerifiedAccessEndpointId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.policy_enabled #=> Boolean
    #   resp.policy_document #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetVerifiedAccessEndpointPolicy AWS API Documentation
    #
    # @overload get_verified_access_endpoint_policy(params = {})
    # @param [Hash] params ({})
    def get_verified_access_endpoint_policy(params = {}, options = {})
      req = build_request(:get_verified_access_endpoint_policy, params)
      req.send_request(options)
    end

    # Gets the targets for the specified network CIDR endpoint for Verified
    # Access.
    #
    # @option params [required, String] :verified_access_endpoint_id
    #   The ID of the network CIDR endpoint.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetVerifiedAccessEndpointTargetsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetVerifiedAccessEndpointTargetsResult#verified_access_endpoint_targets #verified_access_endpoint_targets} => Array&lt;Types::VerifiedAccessEndpointTarget&gt;
    #   * {Types::GetVerifiedAccessEndpointTargetsResult#next_token #next_token} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_verified_access_endpoint_targets({
    #     verified_access_endpoint_id: "VerifiedAccessEndpointId", # required
    #     max_results: 1,
    #     next_token: "NextToken",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_endpoint_targets #=> Array
    #   resp.verified_access_endpoint_targets[0].verified_access_endpoint_id #=> String
    #   resp.verified_access_endpoint_targets[0].verified_access_endpoint_target_ip_address #=> String
    #   resp.verified_access_endpoint_targets[0].verified_access_endpoint_target_dns #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetVerifiedAccessEndpointTargets AWS API Documentation
    #
    # @overload get_verified_access_endpoint_targets(params = {})
    # @param [Hash] params ({})
    def get_verified_access_endpoint_targets(params = {}, options = {})
      req = build_request(:get_verified_access_endpoint_targets, params)
      req.send_request(options)
    end

    # Shows the contents of the Verified Access policy associated with the
    # group.
    #
    # @option params [required, String] :verified_access_group_id
    #   The ID of the Verified Access group.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetVerifiedAccessGroupPolicyResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetVerifiedAccessGroupPolicyResult#policy_enabled #policy_enabled} => Boolean
    #   * {Types::GetVerifiedAccessGroupPolicyResult#policy_document #policy_document} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_verified_access_group_policy({
    #     verified_access_group_id: "VerifiedAccessGroupId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.policy_enabled #=> Boolean
    #   resp.policy_document #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetVerifiedAccessGroupPolicy AWS API Documentation
    #
    # @overload get_verified_access_group_policy(params = {})
    # @param [Hash] params ({})
    def get_verified_access_group_policy(params = {}, options = {})
      req = build_request(:get_verified_access_group_policy, params)
      req.send_request(options)
    end

    # Download an Amazon Web Services-provided sample configuration file to
    # be used with the customer gateway device specified for your
    # Site-to-Site VPN connection.
    #
    # @option params [required, String] :vpn_connection_id
    #   The `VpnConnectionId` specifies the Site-to-Site VPN connection used
    #   for the sample configuration.
    #
    # @option params [required, String] :vpn_connection_device_type_id
    #   Device identifier provided by the `GetVpnConnectionDeviceTypes` API.
    #
    # @option params [String] :internet_key_exchange_version
    #   The IKE version to be used in the sample configuration file for your
    #   customer gateway device. You can specify one of the following
    #   versions: `ikev1` or `ikev2`.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetVpnConnectionDeviceSampleConfigurationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetVpnConnectionDeviceSampleConfigurationResult#vpn_connection_device_sample_configuration #vpn_connection_device_sample_configuration} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_vpn_connection_device_sample_configuration({
    #     vpn_connection_id: "VpnConnectionId", # required
    #     vpn_connection_device_type_id: "VpnConnectionDeviceTypeId", # required
    #     internet_key_exchange_version: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpn_connection_device_sample_configuration #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetVpnConnectionDeviceSampleConfiguration AWS API Documentation
    #
    # @overload get_vpn_connection_device_sample_configuration(params = {})
    # @param [Hash] params ({})
    def get_vpn_connection_device_sample_configuration(params = {}, options = {})
      req = build_request(:get_vpn_connection_device_sample_configuration, params)
      req.send_request(options)
    end

    # Obtain a list of customer gateway devices for which sample
    # configuration files can be provided. The request has no additional
    # parameters. You can also see the list of device types with sample
    # configuration files available under [Your customer gateway device][1]
    # in the *Amazon Web Services Site-to-Site VPN User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpn/latest/s2svpn/your-cgw.html
    #
    # @option params [Integer] :max_results
    #   The maximum number of results returned by
    #   `GetVpnConnectionDeviceTypes` in paginated output. When this parameter
    #   is used, `GetVpnConnectionDeviceTypes` only returns `MaxResults`
    #   results in a single page along with a `NextToken` response element.
    #   The remaining results of the initial request can be seen by sending
    #   another `GetVpnConnectionDeviceTypes` request with the returned
    #   `NextToken` value. This value can be between 200 and 1000. If this
    #   parameter is not used, then `GetVpnConnectionDeviceTypes` returns all
    #   results.
    #
    # @option params [String] :next_token
    #   The `NextToken` value returned from a previous paginated
    #   `GetVpnConnectionDeviceTypes` request where `MaxResults` was used and
    #   the results exceeded the value of that parameter. Pagination continues
    #   from the end of the previous results that returned the `NextToken`
    #   value. This value is null when there are no more results to return.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetVpnConnectionDeviceTypesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetVpnConnectionDeviceTypesResult#vpn_connection_device_types #vpn_connection_device_types} => Array&lt;Types::VpnConnectionDeviceType&gt;
    #   * {Types::GetVpnConnectionDeviceTypesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_vpn_connection_device_types({
    #     max_results: 1,
    #     next_token: "NextToken",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpn_connection_device_types #=> Array
    #   resp.vpn_connection_device_types[0].vpn_connection_device_type_id #=> String
    #   resp.vpn_connection_device_types[0].vendor #=> String
    #   resp.vpn_connection_device_types[0].platform #=> String
    #   resp.vpn_connection_device_types[0].software #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetVpnConnectionDeviceTypes AWS API Documentation
    #
    # @overload get_vpn_connection_device_types(params = {})
    # @param [Hash] params ({})
    def get_vpn_connection_device_types(params = {}, options = {})
      req = build_request(:get_vpn_connection_device_types, params)
      req.send_request(options)
    end

    # Get details of available tunnel endpoint maintenance.
    #
    # @option params [required, String] :vpn_connection_id
    #   The ID of the Site-to-Site VPN connection.
    #
    # @option params [required, String] :vpn_tunnel_outside_ip_address
    #   The external IP address of the VPN tunnel.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::GetVpnTunnelReplacementStatusResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::GetVpnTunnelReplacementStatusResult#vpn_connection_id #vpn_connection_id} => String
    #   * {Types::GetVpnTunnelReplacementStatusResult#transit_gateway_id #transit_gateway_id} => String
    #   * {Types::GetVpnTunnelReplacementStatusResult#customer_gateway_id #customer_gateway_id} => String
    #   * {Types::GetVpnTunnelReplacementStatusResult#vpn_gateway_id #vpn_gateway_id} => String
    #   * {Types::GetVpnTunnelReplacementStatusResult#vpn_tunnel_outside_ip_address #vpn_tunnel_outside_ip_address} => String
    #   * {Types::GetVpnTunnelReplacementStatusResult#maintenance_details #maintenance_details} => Types::MaintenanceDetails
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.get_vpn_tunnel_replacement_status({
    #     vpn_connection_id: "VpnConnectionId", # required
    #     vpn_tunnel_outside_ip_address: "String", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpn_connection_id #=> String
    #   resp.transit_gateway_id #=> String
    #   resp.customer_gateway_id #=> String
    #   resp.vpn_gateway_id #=> String
    #   resp.vpn_tunnel_outside_ip_address #=> String
    #   resp.maintenance_details.pending_maintenance #=> String
    #   resp.maintenance_details.maintenance_auto_applied_after #=> Time
    #   resp.maintenance_details.last_maintenance_applied #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/GetVpnTunnelReplacementStatus AWS API Documentation
    #
    # @overload get_vpn_tunnel_replacement_status(params = {})
    # @param [Hash] params ({})
    def get_vpn_tunnel_replacement_status(params = {}, options = {})
      req = build_request(:get_vpn_tunnel_replacement_status, params)
      req.send_request(options)
    end

    # Uploads a client certificate revocation list to the specified Client
    # VPN endpoint. Uploading a client certificate revocation list
    # overwrites the existing client certificate revocation list.
    #
    # Uploading a client certificate revocation list resets existing client
    # connections.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint to which the client certificate
    #   revocation list applies.
    #
    # @option params [required, String] :certificate_revocation_list
    #   The client certificate revocation list file. For more information, see
    #   [Generate a Client Certificate Revocation List][1] in the *Client VPN
    #   Administrator Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpn/latest/clientvpn-admin/cvpn-working-certificates.html#cvpn-working-certificates-generate
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ImportClientVpnClientCertificateRevocationListResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ImportClientVpnClientCertificateRevocationListResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.import_client_vpn_client_certificate_revocation_list({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     certificate_revocation_list: "String", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ImportClientVpnClientCertificateRevocationList AWS API Documentation
    #
    # @overload import_client_vpn_client_certificate_revocation_list(params = {})
    # @param [Hash] params ({})
    def import_client_vpn_client_certificate_revocation_list(params = {}, options = {})
      req = build_request(:import_client_vpn_client_certificate_revocation_list, params)
      req.send_request(options)
    end

    # <note markdown="1"> To import your virtual machines (VMs) with a console-based experience,
    # you can use the *Import virtual machine images to Amazon Web Services*
    # template in the [Migration Hub Orchestrator console][1]. For more
    # information, see the [ *Migration Hub Orchestrator User Guide* ][2].
    #
    #  </note>
    #
    # Import single or multi-volume disk images or EBS snapshots into an
    # Amazon Machine Image (AMI).
    #
    # Amazon Web Services VM Import/Export strongly recommends specifying a
    # value for either the `--license-type` or `--usage-operation` parameter
    # when you create a new VM Import task. This ensures your operating
    # system is licensed appropriately and your billing is optimized.
    #
    # For more information, see [Importing a VM as an image using VM
    # Import/Export][3] in the *VM Import/Export User Guide*.
    #
    #
    #
    # [1]: https://console.aws.amazon.com/migrationhub/orchestrator
    # [2]: https://docs.aws.amazon.com/migrationhub-orchestrator/latest/userguide/import-vm-images.html
    # [3]: https://docs.aws.amazon.com/vm-import/latest/userguide/vmimport-image-import.html
    #
    # @option params [String] :architecture
    #   The architecture of the virtual machine.
    #
    #   Valid values: `i386` \| `x86_64`
    #
    # @option params [Types::ClientData] :client_data
    #   The client-specific data.
    #
    # @option params [String] :client_token
    #   The token to enable idempotency for VM import requests.
    #
    # @option params [String] :description
    #   A description string for the import image task.
    #
    # @option params [Array<Types::ImageDiskContainer>] :disk_containers
    #   Information about the disk containers.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Boolean] :encrypted
    #   Specifies whether the destination AMI of the imported image should be
    #   encrypted. The default KMS key for EBS is used unless you specify a
    #   non-default KMS key using `KmsKeyId`. For more information, see
    #   [Amazon EBS Encryption][1] in the *Amazon Elastic Compute Cloud User
    #   Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html
    #
    # @option params [String] :hypervisor
    #   The target hypervisor platform.
    #
    #   Valid values: `xen`
    #
    # @option params [String] :kms_key_id
    #   An identifier for the symmetric KMS key to use when creating the
    #   encrypted AMI. This parameter is only required if you want to use a
    #   non-default KMS key; if this parameter is not specified, the default
    #   KMS key for EBS is used. If a `KmsKeyId` is specified, the `Encrypted`
    #   flag must also be set.
    #
    #   The KMS key identifier may be provided in any of the following
    #   formats:
    #
    #   * Key ID
    #
    #   * Key alias
    #
    #   * ARN using key ID. The ID ARN contains the `arn:aws:kms` namespace,
    #     followed by the Region of the key, the Amazon Web Services account
    #     ID of the key owner, the `key` namespace, and then the key ID. For
    #     example,
    #     arn:aws:kms:*us-east-1*:*012345678910*:key/*abcd1234-a123-456a-a12b-a123b4cd56ef*.
    #
    #   * ARN using key alias. The alias ARN contains the `arn:aws:kms`
    #     namespace, followed by the Region of the key, the Amazon Web
    #     Services account ID of the key owner, the `alias` namespace, and
    #     then the key alias. For example,
    #     arn:aws:kms:*us-east-1*:*012345678910*:alias/*ExampleAlias*.
    #
    #   Amazon Web Services parses `KmsKeyId` asynchronously, meaning that the
    #   action you call may appear to complete even though you provided an
    #   invalid identifier. This action will eventually report failure.
    #
    #   The specified KMS key must exist in the Region that the AMI is being
    #   copied to.
    #
    #   Amazon EBS does not support asymmetric KMS keys.
    #
    # @option params [String] :license_type
    #   The license type to be used for the Amazon Machine Image (AMI) after
    #   importing.
    #
    #   Specify `AWS` to replace the source-system license with an Amazon Web
    #   Services license or `BYOL` to retain the source-system license.
    #   Leaving this parameter undefined is the same as choosing `AWS` when
    #   importing a Windows Server operating system, and the same as choosing
    #   `BYOL` when importing a Windows client operating system (such as
    #   Windows 10) or a Linux operating system.
    #
    #   To use `BYOL`, you must have existing licenses with rights to use
    #   these licenses in a third party cloud, such as Amazon Web Services.
    #   For more information, see [Prerequisites][1] in the VM Import/Export
    #   User Guide.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vm-import/latest/userguide/vmimport-image-import.html#prerequisites-image
    #
    # @option params [String] :platform
    #   The operating system of the virtual machine. If you import a VM that
    #   is compatible with Unified Extensible Firmware Interface (UEFI) using
    #   an EBS snapshot, you must specify a value for the platform.
    #
    #   Valid values: `Windows` \| `Linux`
    #
    # @option params [String] :role_name
    #   The name of the role to use when not using the default role,
    #   'vmimport'.
    #
    # @option params [Array<Types::ImportImageLicenseConfigurationRequest>] :license_specifications
    #   The ARNs of the license configurations.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the import image task during creation.
    #
    # @option params [String] :usage_operation
    #   The usage operation value. For more information, see [Licensing
    #   options][1] in the *VM Import/Export User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vm-import/latest/userguide/vmie_prereqs.html#prerequisites
    #
    # @option params [String] :boot_mode
    #   The boot mode of the virtual machine.
    #
    #   <note markdown="1"> The `uefi-preferred` boot mode isn't supported for importing images.
    #   For more information, see [Boot modes][1] in the *VM Import/Export
    #   User Guide*.
    #
    #    </note>
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vm-import/latest/userguide/prerequisites.html#vmimport-boot-modes
    #
    # @return [Types::ImportImageResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ImportImageResult#architecture #architecture} => String
    #   * {Types::ImportImageResult#description #description} => String
    #   * {Types::ImportImageResult#encrypted #encrypted} => Boolean
    #   * {Types::ImportImageResult#hypervisor #hypervisor} => String
    #   * {Types::ImportImageResult#image_id #image_id} => String
    #   * {Types::ImportImageResult#import_task_id #import_task_id} => String
    #   * {Types::ImportImageResult#kms_key_id #kms_key_id} => String
    #   * {Types::ImportImageResult#license_type #license_type} => String
    #   * {Types::ImportImageResult#platform #platform} => String
    #   * {Types::ImportImageResult#progress #progress} => String
    #   * {Types::ImportImageResult#snapshot_details #snapshot_details} => Array&lt;Types::SnapshotDetail&gt;
    #   * {Types::ImportImageResult#status #status} => String
    #   * {Types::ImportImageResult#status_message #status_message} => String
    #   * {Types::ImportImageResult#license_specifications #license_specifications} => Array&lt;Types::ImportImageLicenseConfigurationResponse&gt;
    #   * {Types::ImportImageResult#tags #tags} => Array&lt;Types::Tag&gt;
    #   * {Types::ImportImageResult#usage_operation #usage_operation} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.import_image({
    #     architecture: "String",
    #     client_data: {
    #       comment: "String",
    #       upload_end: Time.now,
    #       upload_size: 1.0,
    #       upload_start: Time.now,
    #     },
    #     client_token: "String",
    #     description: "String",
    #     disk_containers: [
    #       {
    #         description: "String",
    #         device_name: "String",
    #         format: "String",
    #         snapshot_id: "SnapshotId",
    #         url: "SensitiveUrl",
    #         user_bucket: {
    #           s3_bucket: "String",
    #           s3_key: "String",
    #         },
    #       },
    #     ],
    #     dry_run: false,
    #     encrypted: false,
    #     hypervisor: "String",
    #     kms_key_id: "KmsKeyId",
    #     license_type: "String",
    #     platform: "String",
    #     role_name: "String",
    #     license_specifications: [
    #       {
    #         license_configuration_arn: "String",
    #       },
    #     ],
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     usage_operation: "String",
    #     boot_mode: "legacy-bios", # accepts legacy-bios, uefi, uefi-preferred
    #   })
    #
    # @example Response structure
    #
    #   resp.architecture #=> String
    #   resp.description #=> String
    #   resp.encrypted #=> Boolean
    #   resp.hypervisor #=> String
    #   resp.image_id #=> String
    #   resp.import_task_id #=> String
    #   resp.kms_key_id #=> String
    #   resp.license_type #=> String
    #   resp.platform #=> String
    #   resp.progress #=> String
    #   resp.snapshot_details #=> Array
    #   resp.snapshot_details[0].description #=> String
    #   resp.snapshot_details[0].device_name #=> String
    #   resp.snapshot_details[0].disk_image_size #=> Float
    #   resp.snapshot_details[0].format #=> String
    #   resp.snapshot_details[0].progress #=> String
    #   resp.snapshot_details[0].snapshot_id #=> String
    #   resp.snapshot_details[0].status #=> String
    #   resp.snapshot_details[0].status_message #=> String
    #   resp.snapshot_details[0].url #=> String
    #   resp.snapshot_details[0].user_bucket.s3_bucket #=> String
    #   resp.snapshot_details[0].user_bucket.s3_key #=> String
    #   resp.status #=> String
    #   resp.status_message #=> String
    #   resp.license_specifications #=> Array
    #   resp.license_specifications[0].license_configuration_arn #=> String
    #   resp.tags #=> Array
    #   resp.tags[0].key #=> String
    #   resp.tags[0].value #=> String
    #   resp.usage_operation #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ImportImage AWS API Documentation
    #
    # @overload import_image(params = {})
    # @param [Hash] params ({})
    def import_image(params = {}, options = {})
      req = build_request(:import_image, params)
      req.send_request(options)
    end

    # <note markdown="1"> We recommend that you use the [ `ImportImage` ][1] API instead. For
    # more information, see [Importing a VM as an image using VM
    # Import/Export][2] in the *VM Import/Export User Guide*.
    #
    #  </note>
    #
    # Creates an import instance task using metadata from the specified disk
    # image.
    #
    # This API action supports only single-volume VMs. To import
    # multi-volume VMs, use ImportImage instead.
    #
    # For information about the import manifest referenced by this API
    # action, see [VM Import Manifest][3].
    #
    # This API action is not supported by the Command Line Interface (CLI).
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ImportImage.html
    # [2]: https://docs.aws.amazon.com/vm-import/latest/userguide/vmimport-image-import.html
    # [3]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/manifest.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :description
    #   A description for the instance being imported.
    #
    # @option params [Types::ImportInstanceLaunchSpecification] :launch_specification
    #   The launch specification.
    #
    # @option params [Array<Types::DiskImage>] :disk_images
    #   The disk image.
    #
    # @option params [required, String] :platform
    #   The instance operating system.
    #
    # @return [Types::ImportInstanceResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ImportInstanceResult#conversion_task #conversion_task} => Types::ConversionTask
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.import_instance({
    #     dry_run: false,
    #     description: "String",
    #     launch_specification: {
    #       architecture: "i386", # accepts i386, x86_64, arm64, x86_64_mac, arm64_mac
    #       group_names: ["SecurityGroupName"],
    #       group_ids: ["SecurityGroupId"],
    #       additional_info: "String",
    #       user_data: {
    #         data: "String",
    #       },
    #       instance_type: "a1.medium", # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #       placement: {
    #         affinity: "String",
    #         group_name: "PlacementGroupName",
    #         partition_number: 1,
    #         host_id: "String",
    #         tenancy: "default", # accepts default, dedicated, host
    #         spread_domain: "String",
    #         host_resource_group_arn: "String",
    #         group_id: "PlacementGroupId",
    #         availability_zone: "String",
    #       },
    #       monitoring: false,
    #       subnet_id: "SubnetId",
    #       instance_initiated_shutdown_behavior: "stop", # accepts stop, terminate
    #       private_ip_address: "String",
    #     },
    #     disk_images: [
    #       {
    #         description: "String",
    #         image: {
    #           format: "VMDK", # required, accepts VMDK, RAW, VHD
    #           bytes: 1, # required
    #           import_manifest_url: "ImportManifestUrl", # required
    #         },
    #         volume: {
    #           size: 1, # required
    #         },
    #       },
    #     ],
    #     platform: "Windows", # required, accepts Windows
    #   })
    #
    # @example Response structure
    #
    #   resp.conversion_task.conversion_task_id #=> String
    #   resp.conversion_task.expiration_time #=> String
    #   resp.conversion_task.import_instance.description #=> String
    #   resp.conversion_task.import_instance.instance_id #=> String
    #   resp.conversion_task.import_instance.platform #=> String, one of "Windows"
    #   resp.conversion_task.import_instance.volumes #=> Array
    #   resp.conversion_task.import_instance.volumes[0].availability_zone #=> String
    #   resp.conversion_task.import_instance.volumes[0].bytes_converted #=> Integer
    #   resp.conversion_task.import_instance.volumes[0].description #=> String
    #   resp.conversion_task.import_instance.volumes[0].image.checksum #=> String
    #   resp.conversion_task.import_instance.volumes[0].image.format #=> String, one of "VMDK", "RAW", "VHD"
    #   resp.conversion_task.import_instance.volumes[0].image.import_manifest_url #=> String
    #   resp.conversion_task.import_instance.volumes[0].image.size #=> Integer
    #   resp.conversion_task.import_instance.volumes[0].status #=> String
    #   resp.conversion_task.import_instance.volumes[0].status_message #=> String
    #   resp.conversion_task.import_instance.volumes[0].volume.id #=> String
    #   resp.conversion_task.import_instance.volumes[0].volume.size #=> Integer
    #   resp.conversion_task.import_volume.availability_zone #=> String
    #   resp.conversion_task.import_volume.bytes_converted #=> Integer
    #   resp.conversion_task.import_volume.description #=> String
    #   resp.conversion_task.import_volume.image.checksum #=> String
    #   resp.conversion_task.import_volume.image.format #=> String, one of "VMDK", "RAW", "VHD"
    #   resp.conversion_task.import_volume.image.import_manifest_url #=> String
    #   resp.conversion_task.import_volume.image.size #=> Integer
    #   resp.conversion_task.import_volume.volume.id #=> String
    #   resp.conversion_task.import_volume.volume.size #=> Integer
    #   resp.conversion_task.state #=> String, one of "active", "cancelling", "cancelled", "completed"
    #   resp.conversion_task.status_message #=> String
    #   resp.conversion_task.tags #=> Array
    #   resp.conversion_task.tags[0].key #=> String
    #   resp.conversion_task.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ImportInstance AWS API Documentation
    #
    # @overload import_instance(params = {})
    # @param [Hash] params ({})
    def import_instance(params = {}, options = {})
      req = build_request(:import_instance, params)
      req.send_request(options)
    end

    # Imports the public key from an RSA or ED25519 key pair that you
    # created using a third-party tool. You give Amazon Web Services only
    # the public key. The private key is never transferred between you and
    # Amazon Web Services.
    #
    # For more information about the requirements for importing a key pair,
    # see [Create a key pair and import the public key to Amazon EC2][1] in
    # the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/create-key-pairs.html#how-to-generate-your-own-key-and-import-it-to-aws
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the imported key pair.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :key_name
    #   A unique name for the key pair.
    #
    # @option params [required, String, StringIO, File] :public_key_material
    #   The public key.
    #
    # @return [Types::ImportKeyPairResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ImportKeyPairResult#key_fingerprint #key_fingerprint} => String
    #   * {Types::ImportKeyPairResult#key_name #key_name} => String
    #   * {Types::ImportKeyPairResult#key_pair_id #key_pair_id} => String
    #   * {Types::ImportKeyPairResult#tags #tags} => Array&lt;Types::Tag&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.import_key_pair({
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #     key_name: "String", # required
    #     public_key_material: "data", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.key_fingerprint #=> String
    #   resp.key_name #=> String
    #   resp.key_pair_id #=> String
    #   resp.tags #=> Array
    #   resp.tags[0].key #=> String
    #   resp.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ImportKeyPair AWS API Documentation
    #
    # @overload import_key_pair(params = {})
    # @param [Hash] params ({})
    def import_key_pair(params = {}, options = {})
      req = build_request(:import_key_pair, params)
      req.send_request(options)
    end

    # Imports a disk into an EBS snapshot.
    #
    # For more information, see [Importing a disk as a snapshot using VM
    # Import/Export][1] in the *VM Import/Export User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vm-import/latest/userguide/vmimport-import-snapshot.html
    #
    # @option params [Types::ClientData] :client_data
    #   The client-specific data.
    #
    # @option params [String] :client_token
    #   Token to enable idempotency for VM import requests.
    #
    # @option params [String] :description
    #   The description string for the import snapshot task.
    #
    # @option params [Types::SnapshotDiskContainer] :disk_container
    #   Information about the disk container.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Boolean] :encrypted
    #   Specifies whether the destination snapshot of the imported image
    #   should be encrypted. The default KMS key for EBS is used unless you
    #   specify a non-default KMS key using `KmsKeyId`. For more information,
    #   see [Amazon EBS Encryption][1] in the *Amazon Elastic Compute Cloud
    #   User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/EBSEncryption.html
    #
    # @option params [String] :kms_key_id
    #   An identifier for the symmetric KMS key to use when creating the
    #   encrypted snapshot. This parameter is only required if you want to use
    #   a non-default KMS key; if this parameter is not specified, the default
    #   KMS key for EBS is used. If a `KmsKeyId` is specified, the `Encrypted`
    #   flag must also be set.
    #
    #   The KMS key identifier may be provided in any of the following
    #   formats:
    #
    #   * Key ID
    #
    #   * Key alias
    #
    #   * ARN using key ID. The ID ARN contains the `arn:aws:kms` namespace,
    #     followed by the Region of the key, the Amazon Web Services account
    #     ID of the key owner, the `key` namespace, and then the key ID. For
    #     example,
    #     arn:aws:kms:*us-east-1*:*012345678910*:key/*abcd1234-a123-456a-a12b-a123b4cd56ef*.
    #
    #   * ARN using key alias. The alias ARN contains the `arn:aws:kms`
    #     namespace, followed by the Region of the key, the Amazon Web
    #     Services account ID of the key owner, the `alias` namespace, and
    #     then the key alias. For example,
    #     arn:aws:kms:*us-east-1*:*012345678910*:alias/*ExampleAlias*.
    #
    #   Amazon Web Services parses `KmsKeyId` asynchronously, meaning that the
    #   action you call may appear to complete even though you provided an
    #   invalid identifier. This action will eventually report failure.
    #
    #   The specified KMS key must exist in the Region that the snapshot is
    #   being copied to.
    #
    #   Amazon EBS does not support asymmetric KMS keys.
    #
    # @option params [String] :role_name
    #   The name of the role to use when not using the default role,
    #   'vmimport'.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the import snapshot task during creation.
    #
    # @return [Types::ImportSnapshotResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ImportSnapshotResult#description #description} => String
    #   * {Types::ImportSnapshotResult#import_task_id #import_task_id} => String
    #   * {Types::ImportSnapshotResult#snapshot_task_detail #snapshot_task_detail} => Types::SnapshotTaskDetail
    #   * {Types::ImportSnapshotResult#tags #tags} => Array&lt;Types::Tag&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.import_snapshot({
    #     client_data: {
    #       comment: "String",
    #       upload_end: Time.now,
    #       upload_size: 1.0,
    #       upload_start: Time.now,
    #     },
    #     client_token: "String",
    #     description: "String",
    #     disk_container: {
    #       description: "String",
    #       format: "String",
    #       url: "SensitiveUrl",
    #       user_bucket: {
    #         s3_bucket: "String",
    #         s3_key: "String",
    #       },
    #     },
    #     dry_run: false,
    #     encrypted: false,
    #     kms_key_id: "KmsKeyId",
    #     role_name: "String",
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.description #=> String
    #   resp.import_task_id #=> String
    #   resp.snapshot_task_detail.description #=> String
    #   resp.snapshot_task_detail.disk_image_size #=> Float
    #   resp.snapshot_task_detail.encrypted #=> Boolean
    #   resp.snapshot_task_detail.format #=> String
    #   resp.snapshot_task_detail.kms_key_id #=> String
    #   resp.snapshot_task_detail.progress #=> String
    #   resp.snapshot_task_detail.snapshot_id #=> String
    #   resp.snapshot_task_detail.status #=> String
    #   resp.snapshot_task_detail.status_message #=> String
    #   resp.snapshot_task_detail.url #=> String
    #   resp.snapshot_task_detail.user_bucket.s3_bucket #=> String
    #   resp.snapshot_task_detail.user_bucket.s3_key #=> String
    #   resp.tags #=> Array
    #   resp.tags[0].key #=> String
    #   resp.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ImportSnapshot AWS API Documentation
    #
    # @overload import_snapshot(params = {})
    # @param [Hash] params ({})
    def import_snapshot(params = {}, options = {})
      req = build_request(:import_snapshot, params)
      req.send_request(options)
    end

    # <note markdown="1"> This API action supports only single-volume VMs. To import
    # multi-volume VMs, use ImportImage instead. To import a disk to a
    # snapshot, use ImportSnapshot instead.
    #
    #  </note>
    #
    # Creates an import volume task using metadata from the specified disk
    # image.
    #
    # For information about the import manifest referenced by this API
    # action, see [VM Import Manifest][1].
    #
    # This API action is not supported by the Command Line Interface (CLI).
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/manifest.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :availability_zone
    #   The Availability Zone for the resulting EBS volume.
    #
    # @option params [required, Types::DiskImageDetail] :image
    #   The disk image.
    #
    # @option params [String] :description
    #   A description of the volume.
    #
    # @option params [required, Types::VolumeDetail] :volume
    #   The volume size.
    #
    # @return [Types::ImportVolumeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ImportVolumeResult#conversion_task #conversion_task} => Types::ConversionTask
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.import_volume({
    #     dry_run: false,
    #     availability_zone: "String", # required
    #     image: { # required
    #       format: "VMDK", # required, accepts VMDK, RAW, VHD
    #       bytes: 1, # required
    #       import_manifest_url: "ImportManifestUrl", # required
    #     },
    #     description: "String",
    #     volume: { # required
    #       size: 1, # required
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.conversion_task.conversion_task_id #=> String
    #   resp.conversion_task.expiration_time #=> String
    #   resp.conversion_task.import_instance.description #=> String
    #   resp.conversion_task.import_instance.instance_id #=> String
    #   resp.conversion_task.import_instance.platform #=> String, one of "Windows"
    #   resp.conversion_task.import_instance.volumes #=> Array
    #   resp.conversion_task.import_instance.volumes[0].availability_zone #=> String
    #   resp.conversion_task.import_instance.volumes[0].bytes_converted #=> Integer
    #   resp.conversion_task.import_instance.volumes[0].description #=> String
    #   resp.conversion_task.import_instance.volumes[0].image.checksum #=> String
    #   resp.conversion_task.import_instance.volumes[0].image.format #=> String, one of "VMDK", "RAW", "VHD"
    #   resp.conversion_task.import_instance.volumes[0].image.import_manifest_url #=> String
    #   resp.conversion_task.import_instance.volumes[0].image.size #=> Integer
    #   resp.conversion_task.import_instance.volumes[0].status #=> String
    #   resp.conversion_task.import_instance.volumes[0].status_message #=> String
    #   resp.conversion_task.import_instance.volumes[0].volume.id #=> String
    #   resp.conversion_task.import_instance.volumes[0].volume.size #=> Integer
    #   resp.conversion_task.import_volume.availability_zone #=> String
    #   resp.conversion_task.import_volume.bytes_converted #=> Integer
    #   resp.conversion_task.import_volume.description #=> String
    #   resp.conversion_task.import_volume.image.checksum #=> String
    #   resp.conversion_task.import_volume.image.format #=> String, one of "VMDK", "RAW", "VHD"
    #   resp.conversion_task.import_volume.image.import_manifest_url #=> String
    #   resp.conversion_task.import_volume.image.size #=> Integer
    #   resp.conversion_task.import_volume.volume.id #=> String
    #   resp.conversion_task.import_volume.volume.size #=> Integer
    #   resp.conversion_task.state #=> String, one of "active", "cancelling", "cancelled", "completed"
    #   resp.conversion_task.status_message #=> String
    #   resp.conversion_task.tags #=> Array
    #   resp.conversion_task.tags[0].key #=> String
    #   resp.conversion_task.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ImportVolume AWS API Documentation
    #
    # @overload import_volume(params = {})
    # @param [Hash] params ({})
    def import_volume(params = {}, options = {})
      req = build_request(:import_volume, params)
      req.send_request(options)
    end

    # Lists one or more AMIs that are currently in the Recycle Bin. For more
    # information, see [Recycle Bin][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/recycle-bin.html
    #
    # @option params [Array<String>] :image_ids
    #   The IDs of the AMIs to list. Omit this parameter to list all of the
    #   AMIs that are in the Recycle Bin. You can specify up to 20 IDs in a
    #   single request.
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ListImagesInRecycleBinResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ListImagesInRecycleBinResult#images #images} => Array&lt;Types::ImageRecycleBinInfo&gt;
    #   * {Types::ListImagesInRecycleBinResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.list_images_in_recycle_bin({
    #     image_ids: ["ImageId"],
    #     next_token: "String",
    #     max_results: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.images #=> Array
    #   resp.images[0].image_id #=> String
    #   resp.images[0].name #=> String
    #   resp.images[0].description #=> String
    #   resp.images[0].recycle_bin_enter_time #=> Time
    #   resp.images[0].recycle_bin_exit_time #=> Time
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ListImagesInRecycleBin AWS API Documentation
    #
    # @overload list_images_in_recycle_bin(params = {})
    # @param [Hash] params ({})
    def list_images_in_recycle_bin(params = {}, options = {})
      req = build_request(:list_images_in_recycle_bin, params)
      req.send_request(options)
    end

    # Lists one or more snapshots that are currently in the Recycle Bin.
    #
    # @option params [Integer] :max_results
    #   The maximum number of items to return for this request. To get the
    #   next page of items, make another request with the token returned in
    #   the output. For more information, see [Pagination][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Query-Requests.html#api-pagination
    #
    # @option params [String] :next_token
    #   The token returned from a previous paginated request. Pagination
    #   continues from the end of the items returned by the previous request.
    #
    # @option params [Array<String>] :snapshot_ids
    #   The IDs of the snapshots to list. Omit this parameter to list all of
    #   the snapshots that are in the Recycle Bin.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ListSnapshotsInRecycleBinResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ListSnapshotsInRecycleBinResult#snapshots #snapshots} => Array&lt;Types::SnapshotRecycleBinInfo&gt;
    #   * {Types::ListSnapshotsInRecycleBinResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.list_snapshots_in_recycle_bin({
    #     max_results: 1,
    #     next_token: "String",
    #     snapshot_ids: ["SnapshotId"],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.snapshots #=> Array
    #   resp.snapshots[0].snapshot_id #=> String
    #   resp.snapshots[0].recycle_bin_enter_time #=> Time
    #   resp.snapshots[0].recycle_bin_exit_time #=> Time
    #   resp.snapshots[0].description #=> String
    #   resp.snapshots[0].volume_id #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ListSnapshotsInRecycleBin AWS API Documentation
    #
    # @overload list_snapshots_in_recycle_bin(params = {})
    # @param [Hash] params ({})
    def list_snapshots_in_recycle_bin(params = {}, options = {})
      req = build_request(:list_snapshots_in_recycle_bin, params)
      req.send_request(options)
    end

    # Locks an Amazon EBS snapshot in either *governance* or *compliance*
    # mode to protect it against accidental or malicious deletions for a
    # specific duration. A locked snapshot can't be deleted.
    #
    # You can also use this action to modify the lock settings for a
    # snapshot that is already locked. The allowed modifications depend on
    # the lock mode and lock state:
    #
    # * If the snapshot is locked in governance mode, you can modify the
    #   lock mode and the lock duration or lock expiration date.
    #
    # * If the snapshot is locked in compliance mode and it is in the
    #   cooling-off period, you can modify the lock mode and the lock
    #   duration or lock expiration date.
    #
    # * If the snapshot is locked in compliance mode and the cooling-off
    #   period has lapsed, you can only increase the lock duration or extend
    #   the lock expiration date.
    #
    # @option params [required, String] :snapshot_id
    #   The ID of the snapshot to lock.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :lock_mode
    #   The mode in which to lock the snapshot. Specify one of the following:
    #
    #   * `governance` - Locks the snapshot in governance mode. Snapshots
    #     locked in governance mode can't be deleted until one of the
    #     following conditions are met:
    #
    #     * The lock duration expires.
    #
    #     * The snapshot is unlocked by a user with the appropriate
    #       permissions.
    #     Users with the appropriate IAM permissions can unlock the snapshot,
    #     increase or decrease the lock duration, and change the lock mode to
    #     `compliance` at any time.
    #
    #     If you lock a snapshot in `governance` mode, omit <b>
    #     CoolOffPeriod</b>.
    #
    #   * `compliance` - Locks the snapshot in compliance mode. Snapshots
    #     locked in compliance mode can't be unlocked by any user. They can
    #     be deleted only after the lock duration expires. Users can't
    #     decrease the lock duration or change the lock mode to `governance`.
    #     However, users with appropriate IAM permissions can increase the
    #     lock duration at any time.
    #
    #     If you lock a snapshot in `compliance` mode, you can optionally
    #     specify **CoolOffPeriod**.
    #
    # @option params [Integer] :cool_off_period
    #   The cooling-off period during which you can unlock the snapshot or
    #   modify the lock settings after locking the snapshot in compliance
    #   mode, in hours. After the cooling-off period expires, you can't
    #   unlock or delete the snapshot, decrease the lock duration, or change
    #   the lock mode. You can increase the lock duration after the
    #   cooling-off period expires.
    #
    #   The cooling-off period is optional when locking a snapshot in
    #   compliance mode. If you are locking the snapshot in governance mode,
    #   omit this parameter.
    #
    #   To lock the snapshot in compliance mode immediately without a
    #   cooling-off period, omit this parameter.
    #
    #   If you are extending the lock duration for a snapshot that is locked
    #   in compliance mode after the cooling-off period has expired, omit this
    #   parameter. If you specify a cooling-period in a such a request, the
    #   request fails.
    #
    #   Allowed values: Min 1, max 72.
    #
    # @option params [Integer] :lock_duration
    #   The period of time for which to lock the snapshot, in days. The
    #   snapshot lock will automatically expire after this period lapses.
    #
    #   You must specify either this parameter or **ExpirationDate**, but not
    #   both.
    #
    #   Allowed values: Min: 1, max 36500
    #
    # @option params [Time,DateTime,Date,Integer,String] :expiration_date
    #   The date and time at which the snapshot lock is to automatically
    #   expire, in the UTC time zone (`YYYY-MM-DDThh:mm:ss.sssZ`).
    #
    #   You must specify either this parameter or **LockDuration**, but not
    #   both.
    #
    # @return [Types::LockSnapshotResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::LockSnapshotResult#snapshot_id #snapshot_id} => String
    #   * {Types::LockSnapshotResult#lock_state #lock_state} => String
    #   * {Types::LockSnapshotResult#lock_duration #lock_duration} => Integer
    #   * {Types::LockSnapshotResult#cool_off_period #cool_off_period} => Integer
    #   * {Types::LockSnapshotResult#cool_off_period_expires_on #cool_off_period_expires_on} => Time
    #   * {Types::LockSnapshotResult#lock_created_on #lock_created_on} => Time
    #   * {Types::LockSnapshotResult#lock_expires_on #lock_expires_on} => Time
    #   * {Types::LockSnapshotResult#lock_duration_start_time #lock_duration_start_time} => Time
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.lock_snapshot({
    #     snapshot_id: "SnapshotId", # required
    #     dry_run: false,
    #     lock_mode: "compliance", # required, accepts compliance, governance
    #     cool_off_period: 1,
    #     lock_duration: 1,
    #     expiration_date: Time.now,
    #   })
    #
    # @example Response structure
    #
    #   resp.snapshot_id #=> String
    #   resp.lock_state #=> String, one of "compliance", "governance", "compliance-cooloff", "expired"
    #   resp.lock_duration #=> Integer
    #   resp.cool_off_period #=> Integer
    #   resp.cool_off_period_expires_on #=> Time
    #   resp.lock_created_on #=> Time
    #   resp.lock_expires_on #=> Time
    #   resp.lock_duration_start_time #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/LockSnapshot AWS API Documentation
    #
    # @overload lock_snapshot(params = {})
    # @param [Hash] params ({})
    def lock_snapshot(params = {}, options = {})
      req = build_request(:lock_snapshot, params)
      req.send_request(options)
    end

    # Modifies an attribute of the specified Elastic IP address. For
    # requirements, see [Using reverse DNS for email applications][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/elastic-ip-addresses-eip.html#Using_Elastic_Addressing_Reverse_DNS
    #
    # @option params [required, String] :allocation_id
    #   \[EC2-VPC\] The allocation ID.
    #
    # @option params [String] :domain_name
    #   The domain name to modify for the IP address.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyAddressAttributeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyAddressAttributeResult#address #address} => Types::AddressAttribute
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_address_attribute({
    #     allocation_id: "AllocationId", # required
    #     domain_name: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.address.public_ip #=> String
    #   resp.address.allocation_id #=> String
    #   resp.address.ptr_record #=> String
    #   resp.address.ptr_record_update.value #=> String
    #   resp.address.ptr_record_update.status #=> String
    #   resp.address.ptr_record_update.reason #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyAddressAttribute AWS API Documentation
    #
    # @overload modify_address_attribute(params = {})
    # @param [Hash] params ({})
    def modify_address_attribute(params = {}, options = {})
      req = build_request(:modify_address_attribute, params)
      req.send_request(options)
    end

    # Changes the opt-in status of the specified zone group for your
    # account.
    #
    # @option params [required, String] :group_name
    #   The name of the Availability Zone group, Local Zone group, or
    #   Wavelength Zone group.
    #
    # @option params [required, String] :opt_in_status
    #   Indicates whether to opt in to the zone group. The only valid value is
    #   `opted-in`. You must contact Amazon Web Services Support to opt out of
    #   a Local Zone or Wavelength Zone group.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyAvailabilityZoneGroupResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyAvailabilityZoneGroupResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_availability_zone_group({
    #     group_name: "String", # required
    #     opt_in_status: "opted-in", # required, accepts opted-in, not-opted-in
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyAvailabilityZoneGroup AWS API Documentation
    #
    # @overload modify_availability_zone_group(params = {})
    # @param [Hash] params ({})
    def modify_availability_zone_group(params = {}, options = {})
      req = build_request(:modify_availability_zone_group, params)
      req.send_request(options)
    end

    # Modifies a Capacity Reservation's capacity, instance eligibility, and
    # the conditions under which it is to be released. You can't modify a
    # Capacity Reservation's instance type, EBS optimization, platform,
    # instance store settings, Availability Zone, or tenancy. If you need to
    # modify any of these attributes, we recommend that you cancel the
    # Capacity Reservation, and then create a new one with the required
    # attributes. For more information, see [ Modify an active Capacity
    # Reservation][1].
    #
    # The allowed modifications depend on the state of the Capacity
    # Reservation:
    #
    # * `assessing` or `scheduled` state - You can modify the tags only.
    #
    # * `pending` state - You can't modify the Capacity Reservation in any
    #   way.
    #
    # * `active` state but still within the commitment duration - You can't
    #   decrease the instance count or set an end date that is within the
    #   commitment duration. All other modifications are allowed.
    #
    # * `active` state with no commitment duration or elapsed commitment
    #   duration - All modifications are allowed.
    #
    # * `expired`, `cancelled`, `unsupported`, or `failed` state - You
    #   can't modify the Capacity Reservation in any way.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/capacity-reservations-modify.html
    #
    # @option params [required, String] :capacity_reservation_id
    #   The ID of the Capacity Reservation.
    #
    # @option params [Integer] :instance_count
    #   The number of instances for which to reserve capacity. The number of
    #   instances can't be increased or decreased by more than `1000` in a
    #   single request.
    #
    # @option params [Time,DateTime,Date,Integer,String] :end_date
    #   The date and time at which the Capacity Reservation expires. When a
    #   Capacity Reservation expires, the reserved capacity is released and
    #   you can no longer launch instances into it. The Capacity
    #   Reservation's state changes to `expired` when it reaches its end date
    #   and time.
    #
    #   The Capacity Reservation is cancelled within an hour from the
    #   specified time. For example, if you specify 5/31/2019, 13:30:55, the
    #   Capacity Reservation is guaranteed to end between 13:30:55 and
    #   14:30:55 on 5/31/2019.
    #
    #   You must provide an `EndDate` value if `EndDateType` is `limited`.
    #   Omit `EndDate` if `EndDateType` is `unlimited`.
    #
    # @option params [String] :end_date_type
    #   Indicates the way in which the Capacity Reservation ends. A Capacity
    #   Reservation can have one of the following end types:
    #
    #   * `unlimited` - The Capacity Reservation remains active until you
    #     explicitly cancel it. Do not provide an `EndDate` value if
    #     `EndDateType` is `unlimited`.
    #
    #   * `limited` - The Capacity Reservation expires automatically at a
    #     specified date and time. You must provide an `EndDate` value if
    #     `EndDateType` is `limited`.
    #
    # @option params [Boolean] :accept
    #   Reserved. Capacity Reservations you have created are accepted by
    #   default.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :additional_info
    #   Reserved for future use.
    #
    # @option params [String] :instance_match_criteria
    #   The matching criteria (instance eligibility) that you want to use in
    #   the modified Capacity Reservation. If you change the instance
    #   eligibility of an existing Capacity Reservation from `targeted` to
    #   `open`, any running instances that match the attributes of the
    #   Capacity Reservation, have the `CapacityReservationPreference` set to
    #   `open`, and are not yet running in the Capacity Reservation, will
    #   automatically use the modified Capacity Reservation.
    #
    #   To modify the instance eligibility, the Capacity Reservation must be
    #   completely idle (zero usage).
    #
    # @return [Types::ModifyCapacityReservationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyCapacityReservationResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_capacity_reservation({
    #     capacity_reservation_id: "CapacityReservationId", # required
    #     instance_count: 1,
    #     end_date: Time.now,
    #     end_date_type: "unlimited", # accepts unlimited, limited
    #     accept: false,
    #     dry_run: false,
    #     additional_info: "String",
    #     instance_match_criteria: "open", # accepts open, targeted
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyCapacityReservation AWS API Documentation
    #
    # @overload modify_capacity_reservation(params = {})
    # @param [Hash] params ({})
    def modify_capacity_reservation(params = {}, options = {})
      req = build_request(:modify_capacity_reservation, params)
      req.send_request(options)
    end

    # Modifies a Capacity Reservation Fleet.
    #
    # When you modify the total target capacity of a Capacity Reservation
    # Fleet, the Fleet automatically creates new Capacity Reservations, or
    # modifies or cancels existing Capacity Reservations in the Fleet to
    # meet the new total target capacity. When you modify the end date for
    # the Fleet, the end dates for all of the individual Capacity
    # Reservations in the Fleet are updated accordingly.
    #
    # @option params [required, String] :capacity_reservation_fleet_id
    #   The ID of the Capacity Reservation Fleet to modify.
    #
    # @option params [Integer] :total_target_capacity
    #   The total number of capacity units to be reserved by the Capacity
    #   Reservation Fleet. This value, together with the instance type weights
    #   that you assign to each instance type used by the Fleet determine the
    #   number of instances for which the Fleet reserves capacity. Both values
    #   are based on units that make sense for your workload. For more
    #   information, see [Total target capacity][1] in the *Amazon EC2 User
    #   Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/crfleet-concepts.html#target-capacity
    #
    # @option params [Time,DateTime,Date,Integer,String] :end_date
    #   The date and time at which the Capacity Reservation Fleet expires.
    #   When the Capacity Reservation Fleet expires, its state changes to
    #   `expired` and all of the Capacity Reservations in the Fleet expire.
    #
    #   The Capacity Reservation Fleet expires within an hour after the
    #   specified time. For example, if you specify `5/31/2019`, `13:30:55`,
    #   the Capacity Reservation Fleet is guaranteed to expire between
    #   `13:30:55` and `14:30:55` on `5/31/2019`.
    #
    #   You can't specify **EndDate** and <b> RemoveEndDate</b> in the same
    #   request.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Boolean] :remove_end_date
    #   Indicates whether to remove the end date from the Capacity Reservation
    #   Fleet. If you remove the end date, the Capacity Reservation Fleet does
    #   not expire and it remains active until you explicitly cancel it using
    #   the **CancelCapacityReservationFleet** action.
    #
    #   You can't specify **RemoveEndDate** and <b> EndDate</b> in the same
    #   request.
    #
    # @return [Types::ModifyCapacityReservationFleetResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyCapacityReservationFleetResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_capacity_reservation_fleet({
    #     capacity_reservation_fleet_id: "CapacityReservationFleetId", # required
    #     total_target_capacity: 1,
    #     end_date: Time.now,
    #     dry_run: false,
    #     remove_end_date: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyCapacityReservationFleet AWS API Documentation
    #
    # @overload modify_capacity_reservation_fleet(params = {})
    # @param [Hash] params ({})
    def modify_capacity_reservation_fleet(params = {}, options = {})
      req = build_request(:modify_capacity_reservation_fleet, params)
      req.send_request(options)
    end

    # Modifies the specified Client VPN endpoint. Modifying the DNS server
    # resets existing client connections.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint to modify.
    #
    # @option params [String] :server_certificate_arn
    #   The ARN of the server certificate to be used. The server certificate
    #   must be provisioned in Certificate Manager (ACM).
    #
    # @option params [Types::ConnectionLogOptions] :connection_log_options
    #   Information about the client connection logging options.
    #
    #   If you enable client connection logging, data about client connections
    #   is sent to a Cloudwatch Logs log stream. The following information is
    #   logged:
    #
    #   * Client connection requests
    #
    #   * Client connection results (successful and unsuccessful)
    #
    #   * Reasons for unsuccessful client connection requests
    #
    #   * Client connection termination time
    #
    # @option params [Types::DnsServersOptionsModifyStructure] :dns_servers
    #   Information about the DNS servers to be used by Client VPN
    #   connections. A Client VPN endpoint can have up to two DNS servers.
    #
    # @option params [Integer] :vpn_port
    #   The port number to assign to the Client VPN endpoint for TCP and UDP
    #   traffic.
    #
    #   Valid Values: `443` \| `1194`
    #
    #   Default Value: `443`
    #
    # @option params [String] :description
    #   A brief description of the Client VPN endpoint.
    #
    # @option params [Boolean] :split_tunnel
    #   Indicates whether the VPN is split-tunnel.
    #
    #   For information about split-tunnel VPN endpoints, see [Split-tunnel
    #   Client VPN endpoint][1] in the *Client VPN Administrator Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpn/latest/clientvpn-admin/split-tunnel-vpn.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<String>] :security_group_ids
    #   The IDs of one or more security groups to apply to the target network.
    #
    # @option params [String] :vpc_id
    #   The ID of the VPC to associate with the Client VPN endpoint.
    #
    # @option params [String] :self_service_portal
    #   Specify whether to enable the self-service portal for the Client VPN
    #   endpoint.
    #
    # @option params [Types::ClientConnectOptions] :client_connect_options
    #   The options for managing connection authorization for new client
    #   connections.
    #
    # @option params [Integer] :session_timeout_hours
    #   The maximum VPN session duration time in hours.
    #
    #   Valid values: `8 | 10 | 12 | 24`
    #
    #   Default value: `24`
    #
    # @option params [Types::ClientLoginBannerOptions] :client_login_banner_options
    #   Options for enabling a customizable text banner that will be displayed
    #   on Amazon Web Services provided clients when a VPN session is
    #   established.
    #
    # @option params [Boolean] :disconnect_on_session_timeout
    #   Indicates whether the client VPN session is disconnected after the
    #   maximum timeout specified in `sessionTimeoutHours` is reached. If
    #   `true`, users are prompted to reconnect client VPN. If `false`, client
    #   VPN attempts to reconnect automatically. The default value is `false`.
    #
    # @return [Types::ModifyClientVpnEndpointResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyClientVpnEndpointResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_client_vpn_endpoint({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     server_certificate_arn: "String",
    #     connection_log_options: {
    #       enabled: false,
    #       cloudwatch_log_group: "String",
    #       cloudwatch_log_stream: "String",
    #     },
    #     dns_servers: {
    #       custom_dns_servers: ["String"],
    #       enabled: false,
    #     },
    #     vpn_port: 1,
    #     description: "String",
    #     split_tunnel: false,
    #     dry_run: false,
    #     security_group_ids: ["SecurityGroupId"],
    #     vpc_id: "VpcId",
    #     self_service_portal: "enabled", # accepts enabled, disabled
    #     client_connect_options: {
    #       enabled: false,
    #       lambda_function_arn: "String",
    #     },
    #     session_timeout_hours: 1,
    #     client_login_banner_options: {
    #       enabled: false,
    #       banner_text: "String",
    #     },
    #     disconnect_on_session_timeout: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyClientVpnEndpoint AWS API Documentation
    #
    # @overload modify_client_vpn_endpoint(params = {})
    # @param [Hash] params ({})
    def modify_client_vpn_endpoint(params = {}, options = {})
      req = build_request(:modify_client_vpn_endpoint, params)
      req.send_request(options)
    end

    # Modifies the default credit option for CPU usage of burstable
    # performance instances. The default credit option is set at the account
    # level per Amazon Web Services Region, and is specified per instance
    # family. All new burstable performance instances in the account launch
    # using the default credit option.
    #
    # `ModifyDefaultCreditSpecification` is an asynchronous operation, which
    # works at an Amazon Web Services Region level and modifies the credit
    # option for each Availability Zone. All zones in a Region are updated
    # within five minutes. But if instances are launched during this
    # operation, they might not get the new credit option until the zone is
    # updated. To verify whether the update has occurred, you can call
    # `GetDefaultCreditSpecification` and check `DefaultCreditSpecification`
    # for updates.
    #
    # For more information, see [Burstable performance instances][1] in the
    # *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/burstable-performance-instances.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_family
    #   The instance family.
    #
    # @option params [required, String] :cpu_credits
    #   The credit option for CPU usage of the instance family.
    #
    #   Valid Values: `standard` \| `unlimited`
    #
    # @return [Types::ModifyDefaultCreditSpecificationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyDefaultCreditSpecificationResult#instance_family_credit_specification #instance_family_credit_specification} => Types::InstanceFamilyCreditSpecification
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_default_credit_specification({
    #     dry_run: false,
    #     instance_family: "t2", # required, accepts t2, t3, t3a, t4g
    #     cpu_credits: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_family_credit_specification.instance_family #=> String, one of "t2", "t3", "t3a", "t4g"
    #   resp.instance_family_credit_specification.cpu_credits #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyDefaultCreditSpecification AWS API Documentation
    #
    # @overload modify_default_credit_specification(params = {})
    # @param [Hash] params ({})
    def modify_default_credit_specification(params = {}, options = {})
      req = build_request(:modify_default_credit_specification, params)
      req.send_request(options)
    end

    # Changes the default KMS key for EBS encryption by default for your
    # account in this Region.
    #
    # Amazon Web Services creates a unique Amazon Web Services managed KMS
    # key in each Region for use with encryption by default. If you change
    # the default KMS key to a symmetric customer managed KMS key, it is
    # used instead of the Amazon Web Services managed KMS key. To reset the
    # default KMS key to the Amazon Web Services managed KMS key for EBS,
    # use ResetEbsDefaultKmsKeyId. Amazon EBS does not support asymmetric
    # KMS keys.
    #
    # If you delete or disable the customer managed KMS key that you
    # specified for use with encryption by default, your instances will fail
    # to launch.
    #
    # For more information, see [Amazon EBS encryption][1] in the *Amazon
    # EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-encryption.html
    #
    # @option params [required, String] :kms_key_id
    #   The identifier of the KMS key to use for Amazon EBS encryption. If
    #   this parameter is not specified, your KMS key for Amazon EBS is used.
    #   If `KmsKeyId` is specified, the encrypted state must be `true`.
    #
    #   You can specify the KMS key using any of the following:
    #
    #   * Key ID. For example, 1234abcd-12ab-34cd-56ef-1234567890ab.
    #
    #   * Key alias. For example, alias/ExampleAlias.
    #
    #   * Key ARN. For example,
    #     arn:aws:kms:us-east-1:012345678910:key/1234abcd-12ab-34cd-56ef-1234567890ab.
    #
    #   * Alias ARN. For example,
    #     arn:aws:kms:us-east-1:012345678910:alias/ExampleAlias.
    #
    #   Amazon Web Services authenticates the KMS key asynchronously.
    #   Therefore, if you specify an ID, alias, or ARN that is not valid, the
    #   action can appear to complete, but eventually fails.
    #
    #   Amazon EBS does not support asymmetric KMS keys.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyEbsDefaultKmsKeyIdResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyEbsDefaultKmsKeyIdResult#kms_key_id #kms_key_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_ebs_default_kms_key_id({
    #     kms_key_id: "KmsKeyId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.kms_key_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyEbsDefaultKmsKeyId AWS API Documentation
    #
    # @overload modify_ebs_default_kms_key_id(params = {})
    # @param [Hash] params ({})
    def modify_ebs_default_kms_key_id(params = {}, options = {})
      req = build_request(:modify_ebs_default_kms_key_id, params)
      req.send_request(options)
    end

    # Modifies the specified EC2 Fleet.
    #
    # You can only modify an EC2 Fleet request of type `maintain`.
    #
    # While the EC2 Fleet is being modified, it is in the `modifying` state.
    #
    # To scale up your EC2 Fleet, increase its target capacity. The EC2
    # Fleet launches the additional Spot Instances according to the
    # allocation strategy for the EC2 Fleet request. If the allocation
    # strategy is `lowest-price`, the EC2 Fleet launches instances using the
    # Spot Instance pool with the lowest price. If the allocation strategy
    # is `diversified`, the EC2 Fleet distributes the instances across the
    # Spot Instance pools. If the allocation strategy is
    # `capacity-optimized`, EC2 Fleet launches instances from Spot Instance
    # pools with optimal capacity for the number of instances that are
    # launching.
    #
    # To scale down your EC2 Fleet, decrease its target capacity. First, the
    # EC2 Fleet cancels any open requests that exceed the new target
    # capacity. You can request that the EC2 Fleet terminate Spot Instances
    # until the size of the fleet no longer exceeds the new target capacity.
    # If the allocation strategy is `lowest-price`, the EC2 Fleet terminates
    # the instances with the highest price per unit. If the allocation
    # strategy is `capacity-optimized`, the EC2 Fleet terminates the
    # instances in the Spot Instance pools that have the least available
    # Spot Instance capacity. If the allocation strategy is `diversified`,
    # the EC2 Fleet terminates instances across the Spot Instance pools.
    # Alternatively, you can request that the EC2 Fleet keep the fleet at
    # its current size, but not replace any Spot Instances that are
    # interrupted or that you terminate manually.
    #
    # If you are finished with your EC2 Fleet for now, but will use it again
    # later, you can set the target capacity to 0.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :excess_capacity_termination_policy
    #   Indicates whether running instances should be terminated if the total
    #   target capacity of the EC2 Fleet is decreased below the current size
    #   of the EC2 Fleet.
    #
    #   Supported only for fleets of type `maintain`.
    #
    # @option params [Array<Types::FleetLaunchTemplateConfigRequest>] :launch_template_configs
    #   The launch template and overrides.
    #
    # @option params [required, String] :fleet_id
    #   The ID of the EC2 Fleet.
    #
    # @option params [Types::TargetCapacitySpecificationRequest] :target_capacity_specification
    #   The size of the EC2 Fleet.
    #
    # @option params [String] :context
    #   Reserved.
    #
    # @return [Types::ModifyFleetResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyFleetResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_fleet({
    #     dry_run: false,
    #     excess_capacity_termination_policy: "no-termination", # accepts no-termination, termination
    #     launch_template_configs: [
    #       {
    #         launch_template_specification: {
    #           launch_template_id: "LaunchTemplateId",
    #           launch_template_name: "LaunchTemplateName",
    #           version: "String",
    #         },
    #         overrides: [
    #           {
    #             instance_type: "a1.medium", # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #             max_price: "String",
    #             subnet_id: "SubnetId",
    #             availability_zone: "String",
    #             weighted_capacity: 1.0,
    #             priority: 1.0,
    #             placement: {
    #               affinity: "String",
    #               group_name: "PlacementGroupName",
    #               partition_number: 1,
    #               host_id: "String",
    #               tenancy: "default", # accepts default, dedicated, host
    #               spread_domain: "String",
    #               host_resource_group_arn: "String",
    #               group_id: "PlacementGroupId",
    #               availability_zone: "String",
    #             },
    #             block_device_mappings: [
    #               {
    #                 device_name: "String",
    #                 virtual_name: "String",
    #                 ebs: {
    #                   encrypted: false,
    #                   delete_on_termination: false,
    #                   iops: 1,
    #                   throughput: 1,
    #                   kms_key_id: "KmsKeyId",
    #                   snapshot_id: "SnapshotId",
    #                   volume_size: 1,
    #                   volume_type: "standard", # accepts standard, io1, io2, gp2, sc1, st1, gp3
    #                 },
    #                 no_device: "String",
    #               },
    #             ],
    #             instance_requirements: {
    #               v_cpu_count: { # required
    #                 min: 1, # required
    #                 max: 1,
    #               },
    #               memory_mi_b: { # required
    #                 min: 1, # required
    #                 max: 1,
    #               },
    #               cpu_manufacturers: ["intel"], # accepts intel, amd, amazon-web-services, apple
    #               memory_gi_b_per_v_cpu: {
    #                 min: 1.0,
    #                 max: 1.0,
    #               },
    #               excluded_instance_types: ["ExcludedInstanceType"],
    #               instance_generations: ["current"], # accepts current, previous
    #               spot_max_price_percentage_over_lowest_price: 1,
    #               on_demand_max_price_percentage_over_lowest_price: 1,
    #               bare_metal: "included", # accepts included, required, excluded
    #               burstable_performance: "included", # accepts included, required, excluded
    #               require_hibernate_support: false,
    #               network_interface_count: {
    #                 min: 1,
    #                 max: 1,
    #               },
    #               local_storage: "included", # accepts included, required, excluded
    #               local_storage_types: ["hdd"], # accepts hdd, ssd
    #               total_local_storage_gb: {
    #                 min: 1.0,
    #                 max: 1.0,
    #               },
    #               baseline_ebs_bandwidth_mbps: {
    #                 min: 1,
    #                 max: 1,
    #               },
    #               accelerator_types: ["gpu"], # accepts gpu, fpga, inference
    #               accelerator_count: {
    #                 min: 1,
    #                 max: 1,
    #               },
    #               accelerator_manufacturers: ["amazon-web-services"], # accepts amazon-web-services, amd, nvidia, xilinx, habana
    #               accelerator_names: ["a100"], # accepts a100, inferentia, k520, k80, m60, radeon-pro-v520, t4, vu9p, v100, a10g, h100, t4g
    #               accelerator_total_memory_mi_b: {
    #                 min: 1,
    #                 max: 1,
    #               },
    #               network_bandwidth_gbps: {
    #                 min: 1.0,
    #                 max: 1.0,
    #               },
    #               allowed_instance_types: ["AllowedInstanceType"],
    #               max_spot_price_as_percentage_of_optimal_on_demand_price: 1,
    #               baseline_performance_factors: {
    #                 cpu: {
    #                   references: [
    #                     {
    #                       instance_family: "String",
    #                     },
    #                   ],
    #                 },
    #               },
    #             },
    #             image_id: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     fleet_id: "FleetId", # required
    #     target_capacity_specification: {
    #       total_target_capacity: 1, # required
    #       on_demand_target_capacity: 1,
    #       spot_target_capacity: 1,
    #       default_target_capacity_type: "spot", # accepts spot, on-demand, capacity-block
    #       target_capacity_unit_type: "vcpu", # accepts vcpu, memory-mib, units
    #     },
    #     context: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyFleet AWS API Documentation
    #
    # @overload modify_fleet(params = {})
    # @param [Hash] params ({})
    def modify_fleet(params = {}, options = {})
      req = build_request(:modify_fleet, params)
      req.send_request(options)
    end

    # Modifies the specified attribute of the specified Amazon FPGA Image
    # (AFI).
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :fpga_image_id
    #   The ID of the AFI.
    #
    # @option params [String] :attribute
    #   The name of the attribute.
    #
    # @option params [String] :operation_type
    #   The operation type.
    #
    # @option params [Array<String>] :user_ids
    #   The Amazon Web Services account IDs. This parameter is valid only when
    #   modifying the `loadPermission` attribute.
    #
    # @option params [Array<String>] :user_groups
    #   The user groups. This parameter is valid only when modifying the
    #   `loadPermission` attribute.
    #
    # @option params [Array<String>] :product_codes
    #   The product codes. After you add a product code to an AFI, it can't
    #   be removed. This parameter is valid only when modifying the
    #   `productCodes` attribute.
    #
    # @option params [Types::LoadPermissionModifications] :load_permission
    #   The load permission for the AFI.
    #
    # @option params [String] :description
    #   A description for the AFI.
    #
    # @option params [String] :name
    #   A name for the AFI.
    #
    # @return [Types::ModifyFpgaImageAttributeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyFpgaImageAttributeResult#fpga_image_attribute #fpga_image_attribute} => Types::FpgaImageAttribute
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_fpga_image_attribute({
    #     dry_run: false,
    #     fpga_image_id: "FpgaImageId", # required
    #     attribute: "description", # accepts description, name, loadPermission, productCodes
    #     operation_type: "add", # accepts add, remove
    #     user_ids: ["String"],
    #     user_groups: ["String"],
    #     product_codes: ["String"],
    #     load_permission: {
    #       add: [
    #         {
    #           group: "all", # accepts all
    #           user_id: "String",
    #         },
    #       ],
    #       remove: [
    #         {
    #           group: "all", # accepts all
    #           user_id: "String",
    #         },
    #       ],
    #     },
    #     description: "String",
    #     name: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.fpga_image_attribute.fpga_image_id #=> String
    #   resp.fpga_image_attribute.name #=> String
    #   resp.fpga_image_attribute.description #=> String
    #   resp.fpga_image_attribute.load_permissions #=> Array
    #   resp.fpga_image_attribute.load_permissions[0].user_id #=> String
    #   resp.fpga_image_attribute.load_permissions[0].group #=> String, one of "all"
    #   resp.fpga_image_attribute.product_codes #=> Array
    #   resp.fpga_image_attribute.product_codes[0].product_code_id #=> String
    #   resp.fpga_image_attribute.product_codes[0].product_code_type #=> String, one of "devpay", "marketplace"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyFpgaImageAttribute AWS API Documentation
    #
    # @overload modify_fpga_image_attribute(params = {})
    # @param [Hash] params ({})
    def modify_fpga_image_attribute(params = {}, options = {})
      req = build_request(:modify_fpga_image_attribute, params)
      req.send_request(options)
    end

    # Modify the auto-placement setting of a Dedicated Host. When
    # auto-placement is enabled, any instances that you launch with a
    # tenancy of `host` but without a specific host ID are placed onto any
    # available Dedicated Host in your account that has auto-placement
    # enabled. When auto-placement is disabled, you need to provide a host
    # ID to have the instance launch onto a specific host. If no host ID is
    # provided, the instance is launched onto a suitable host with
    # auto-placement enabled.
    #
    # You can also use this API action to modify a Dedicated Host to support
    # either multiple instance types in an instance family, or to support a
    # specific instance type only.
    #
    # @option params [String] :host_recovery
    #   Indicates whether to enable or disable host recovery for the Dedicated
    #   Host. For more information, see [Host recovery][1] in the *Amazon EC2
    #   User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-hosts-recovery.html
    #
    # @option params [String] :instance_type
    #   Specifies the instance type to be supported by the Dedicated Host.
    #   Specify this parameter to modify a Dedicated Host to support only a
    #   specific instance type.
    #
    #   If you want to modify a Dedicated Host to support multiple instance
    #   types in its current instance family, omit this parameter and specify
    #   **InstanceFamily** instead. You cannot specify **InstanceType** and
    #   **InstanceFamily** in the same request.
    #
    # @option params [String] :instance_family
    #   Specifies the instance family to be supported by the Dedicated Host.
    #   Specify this parameter to modify a Dedicated Host to support multiple
    #   instance types within its current instance family.
    #
    #   If you want to modify a Dedicated Host to support a specific instance
    #   type only, omit this parameter and specify **InstanceType** instead.
    #   You cannot specify **InstanceFamily** and **InstanceType** in the same
    #   request.
    #
    # @option params [String] :host_maintenance
    #   Indicates whether to enable or disable host maintenance for the
    #   Dedicated Host. For more information, see [Host maintenance][1] in the
    #   *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-hosts-maintenance.html
    #
    # @option params [required, Array<String>] :host_ids
    #   The IDs of the Dedicated Hosts to modify.
    #
    # @option params [String] :auto_placement
    #   Specify whether to enable or disable auto-placement.
    #
    # @return [Types::ModifyHostsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyHostsResult#successful #successful} => Array&lt;String&gt;
    #   * {Types::ModifyHostsResult#unsuccessful #unsuccessful} => Array&lt;Types::UnsuccessfulItem&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_hosts({
    #     host_recovery: "on", # accepts on, off
    #     instance_type: "String",
    #     instance_family: "String",
    #     host_maintenance: "on", # accepts on, off
    #     host_ids: ["DedicatedHostId"], # required
    #     auto_placement: "on", # accepts on, off
    #   })
    #
    # @example Response structure
    #
    #   resp.successful #=> Array
    #   resp.successful[0] #=> String
    #   resp.unsuccessful #=> Array
    #   resp.unsuccessful[0].error.code #=> String
    #   resp.unsuccessful[0].error.message #=> String
    #   resp.unsuccessful[0].resource_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyHosts AWS API Documentation
    #
    # @overload modify_hosts(params = {})
    # @param [Hash] params ({})
    def modify_hosts(params = {}, options = {})
      req = build_request(:modify_hosts, params)
      req.send_request(options)
    end

    # Modifies the ID format for the specified resource on a per-Region
    # basis. You can specify that resources should receive longer IDs
    # (17-character IDs) when they are created.
    #
    # This request can only be used to modify longer ID settings for
    # resource types that are within the opt-in period. Resources currently
    # in their opt-in period include: `bundle` \| `conversion-task` \|
    # `customer-gateway` \| `dhcp-options` \| `elastic-ip-allocation` \|
    # `elastic-ip-association` \| `export-task` \| `flow-log` \| `image` \|
    # `import-task` \| `internet-gateway` \| `network-acl` \|
    # `network-acl-association` \| `network-interface` \|
    # `network-interface-attachment` \| `prefix-list` \| `route-table` \|
    # `route-table-association` \| `security-group` \| `subnet` \|
    # `subnet-cidr-block-association` \| `vpc` \|
    # `vpc-cidr-block-association` \| `vpc-endpoint` \|
    # `vpc-peering-connection` \| `vpn-connection` \| `vpn-gateway`.
    #
    # This setting applies to the IAM user who makes the request; it does
    # not apply to the entire Amazon Web Services account. By default, an
    # IAM user defaults to the same settings as the root user. If you're
    # using this action as the root user, then these settings apply to the
    # entire account, unless an IAM user explicitly overrides these settings
    # for themselves. For more information, see [Resource IDs][1] in the
    # *Amazon Elastic Compute Cloud User Guide*.
    #
    # Resources created with longer IDs are visible to all IAM roles and
    # users, regardless of these settings and provided that they have
    # permission to use the relevant `Describe` command for the resource
    # type.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/resource-ids.html
    #
    # @option params [required, String] :resource
    #   The type of resource: `bundle` \| `conversion-task` \|
    #   `customer-gateway` \| `dhcp-options` \| `elastic-ip-allocation` \|
    #   `elastic-ip-association` \| `export-task` \| `flow-log` \| `image` \|
    #   `import-task` \| `internet-gateway` \| `network-acl` \|
    #   `network-acl-association` \| `network-interface` \|
    #   `network-interface-attachment` \| `prefix-list` \| `route-table` \|
    #   `route-table-association` \| `security-group` \| `subnet` \|
    #   `subnet-cidr-block-association` \| `vpc` \|
    #   `vpc-cidr-block-association` \| `vpc-endpoint` \|
    #   `vpc-peering-connection` \| `vpn-connection` \| `vpn-gateway`.
    #
    #   Alternatively, use the `all-current` option to include all resource
    #   types that are currently within their opt-in period for longer IDs.
    #
    # @option params [required, Boolean] :use_long_ids
    #   Indicate whether the resource should use longer IDs (17-character
    #   IDs).
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_id_format({
    #     resource: "String", # required
    #     use_long_ids: false, # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyIdFormat AWS API Documentation
    #
    # @overload modify_id_format(params = {})
    # @param [Hash] params ({})
    def modify_id_format(params = {}, options = {})
      req = build_request(:modify_id_format, params)
      req.send_request(options)
    end

    # Modifies the ID format of a resource for a specified IAM user, IAM
    # role, or the root user for an account; or all IAM users, IAM roles,
    # and the root user for an account. You can specify that resources
    # should receive longer IDs (17-character IDs) when they are created.
    #
    # This request can only be used to modify longer ID settings for
    # resource types that are within the opt-in period. Resources currently
    # in their opt-in period include: `bundle` \| `conversion-task` \|
    # `customer-gateway` \| `dhcp-options` \| `elastic-ip-allocation` \|
    # `elastic-ip-association` \| `export-task` \| `flow-log` \| `image` \|
    # `import-task` \| `internet-gateway` \| `network-acl` \|
    # `network-acl-association` \| `network-interface` \|
    # `network-interface-attachment` \| `prefix-list` \| `route-table` \|
    # `route-table-association` \| `security-group` \| `subnet` \|
    # `subnet-cidr-block-association` \| `vpc` \|
    # `vpc-cidr-block-association` \| `vpc-endpoint` \|
    # `vpc-peering-connection` \| `vpn-connection` \| `vpn-gateway`.
    #
    # For more information, see [Resource IDs][1] in the *Amazon Elastic
    # Compute Cloud User Guide*.
    #
    # This setting applies to the principal specified in the request; it
    # does not apply to the principal that makes the request.
    #
    # Resources created with longer IDs are visible to all IAM roles and
    # users, regardless of these settings and provided that they have
    # permission to use the relevant `Describe` command for the resource
    # type.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/resource-ids.html
    #
    # @option params [required, String] :resource
    #   The type of resource: `bundle` \| `conversion-task` \|
    #   `customer-gateway` \| `dhcp-options` \| `elastic-ip-allocation` \|
    #   `elastic-ip-association` \| `export-task` \| `flow-log` \| `image` \|
    #   `import-task` \| `internet-gateway` \| `network-acl` \|
    #   `network-acl-association` \| `network-interface` \|
    #   `network-interface-attachment` \| `prefix-list` \| `route-table` \|
    #   `route-table-association` \| `security-group` \| `subnet` \|
    #   `subnet-cidr-block-association` \| `vpc` \|
    #   `vpc-cidr-block-association` \| `vpc-endpoint` \|
    #   `vpc-peering-connection` \| `vpn-connection` \| `vpn-gateway`.
    #
    #   Alternatively, use the `all-current` option to include all resource
    #   types that are currently within their opt-in period for longer IDs.
    #
    # @option params [required, Boolean] :use_long_ids
    #   Indicates whether the resource should use longer IDs (17-character
    #   IDs)
    #
    # @option params [required, String] :principal_arn
    #   The ARN of the principal, which can be an IAM user, IAM role, or the
    #   root user. Specify `all` to modify the ID format for all IAM users,
    #   IAM roles, and the root user of the account.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_identity_id_format({
    #     resource: "String", # required
    #     use_long_ids: false, # required
    #     principal_arn: "String", # required
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyIdentityIdFormat AWS API Documentation
    #
    # @overload modify_identity_id_format(params = {})
    # @param [Hash] params ({})
    def modify_identity_id_format(params = {}, options = {})
      req = build_request(:modify_identity_id_format, params)
      req.send_request(options)
    end

    # Modifies the specified attribute of the specified AMI. You can specify
    # only one attribute at a time.
    #
    # To specify the attribute, you can use the `Attribute` parameter, or
    # one of the following parameters: `Description`, `ImdsSupport`, or
    # `LaunchPermission`.
    #
    # Images with an Amazon Web Services Marketplace product code cannot be
    # made public.
    #
    # To enable the SriovNetSupport enhanced networking attribute of an
    # image, enable SriovNetSupport on an instance and create an AMI from
    # the instance.
    #
    # @option params [String] :attribute
    #   The name of the attribute to modify.
    #
    #   Valid values: `description` \| `imdsSupport` \| `launchPermission`
    #
    # @option params [Types::AttributeValue] :description
    #   A new description for the AMI.
    #
    # @option params [required, String] :image_id
    #   The ID of the AMI.
    #
    # @option params [Types::LaunchPermissionModifications] :launch_permission
    #   A new launch permission for the AMI.
    #
    # @option params [String] :operation_type
    #   The operation type. This parameter can be used only when the
    #   `Attribute` parameter is `launchPermission`.
    #
    # @option params [Array<String>] :product_codes
    #   Not supported.
    #
    # @option params [Array<String>] :user_groups
    #   The user groups. This parameter can be used only when the `Attribute`
    #   parameter is `launchPermission`.
    #
    # @option params [Array<String>] :user_ids
    #   The Amazon Web Services account IDs. This parameter can be used only
    #   when the `Attribute` parameter is `launchPermission`.
    #
    # @option params [String] :value
    #   The value of the attribute being modified. This parameter can be used
    #   only when the `Attribute` parameter is `description` or `imdsSupport`.
    #
    # @option params [Array<String>] :organization_arns
    #   The Amazon Resource Name (ARN) of an organization. This parameter can
    #   be used only when the `Attribute` parameter is `launchPermission`.
    #
    # @option params [Array<String>] :organizational_unit_arns
    #   The Amazon Resource Name (ARN) of an organizational unit (OU). This
    #   parameter can be used only when the `Attribute` parameter is
    #   `launchPermission`.
    #
    # @option params [Types::AttributeValue] :imds_support
    #   Set to `v2.0` to indicate that IMDSv2 is specified in the AMI.
    #   Instances launched from this AMI will have `HttpTokens` automatically
    #   set to `required` so that, by default, the instance requires that
    #   IMDSv2 is used when requesting instance metadata. In addition,
    #   `HttpPutResponseHopLimit` is set to `2`. For more information, see
    #   [Configure the AMI][1] in the *Amazon EC2 User Guide*.
    #
    #   Do not use this parameter unless your AMI software supports IMDSv2.
    #   After you set the value to `v2.0`, you can't undo it. The only way to
    #   “reset” your AMI is to create a new AMI from the underlying snapshot.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/configuring-IMDS-new-instances.html#configure-IMDS-new-instances-ami-configuration
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To make an AMI public
    #
    #   # This example makes the specified AMI public.
    #
    #   resp = client.modify_image_attribute({
    #     image_id: "ami-5731123e", 
    #     launch_permission: {
    #       add: [
    #         {
    #           group: "all", 
    #         }, 
    #       ], 
    #     }, 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Example: To grant launch permissions
    #
    #   # This example grants launch permissions for the specified AMI to the specified AWS account.
    #
    #   resp = client.modify_image_attribute({
    #     image_id: "ami-5731123e", 
    #     launch_permission: {
    #       add: [
    #         {
    #           user_id: "123456789012", 
    #         }, 
    #       ], 
    #     }, 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_image_attribute({
    #     attribute: "String",
    #     description: "value", # value <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #     image_id: "ImageId", # required
    #     launch_permission: {
    #       add: [
    #         {
    #           organization_arn: "String",
    #           organizational_unit_arn: "String",
    #           user_id: "String",
    #           group: "all", # accepts all
    #         },
    #       ],
    #       remove: [
    #         {
    #           organization_arn: "String",
    #           organizational_unit_arn: "String",
    #           user_id: "String",
    #           group: "all", # accepts all
    #         },
    #       ],
    #     },
    #     operation_type: "add", # accepts add, remove
    #     product_codes: ["String"],
    #     user_groups: ["String"],
    #     user_ids: ["String"],
    #     value: "String",
    #     organization_arns: ["String"],
    #     organizational_unit_arns: ["String"],
    #     imds_support: "value", # value <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyImageAttribute AWS API Documentation
    #
    # @overload modify_image_attribute(params = {})
    # @param [Hash] params ({})
    def modify_image_attribute(params = {}, options = {})
      req = build_request(:modify_image_attribute, params)
      req.send_request(options)
    end

    # Modifies the specified attribute of the specified instance. You can
    # specify only one attribute at a time.
    #
    # <b>Note: </b>Using this action to change the security groups
    # associated with an elastic network interface (ENI) attached to an
    # instance can result in an error if the instance has more than one ENI.
    # To change the security groups associated with an ENI attached to an
    # instance that has multiple ENIs, we recommend that you use the
    # ModifyNetworkInterfaceAttribute action.
    #
    # To modify some attributes, the instance must be stopped. For more
    # information, see [Modify a stopped instance][1] in the *Amazon EC2
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_ChangingAttributesWhileInstanceStopped.html
    #
    # @option params [Types::AttributeBooleanValue] :source_dest_check
    #   Enable or disable source/destination checks, which ensure that the
    #   instance is either the source or the destination of any traffic that
    #   it receives. If the value is `true`, source/destination checks are
    #   enabled; otherwise, they are disabled. The default value is `true`.
    #   You must disable source/destination checks if the instance runs
    #   services such as network address translation, routing, or firewalls.
    #
    # @option params [Types::AttributeBooleanValue] :disable_api_stop
    #   Indicates whether an instance is enabled for stop protection. For more
    #   information, see [Enable stop protection for your instance][1].
    #
    #
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-stop-protection.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @option params [String] :attribute
    #   The name of the attribute to modify.
    #
    #   You can modify the following attributes only: `disableApiTermination`
    #   \| `instanceType` \| `kernel` \| `ramdisk` \|
    #   `instanceInitiatedShutdownBehavior` \| `blockDeviceMapping` \|
    #   `userData` \| `sourceDestCheck` \| `groupSet` \| `ebsOptimized` \|
    #   `sriovNetSupport` \| `enaSupport` \| `nvmeSupport` \| `disableApiStop`
    #   \| `enclaveOptions`
    #
    # @option params [String] :value
    #   A new value for the attribute. Use only with the `kernel`, `ramdisk`,
    #   `userData`, `disableApiTermination`, or
    #   `instanceInitiatedShutdownBehavior` attribute.
    #
    # @option params [Array<Types::InstanceBlockDeviceMappingSpecification>] :block_device_mappings
    #   Modifies the `DeleteOnTermination` attribute for volumes that are
    #   currently attached. The volume must be owned by the caller. If no
    #   value is specified for `DeleteOnTermination`, the default is `true`
    #   and the volume is deleted when the instance is terminated. You can't
    #   modify the `DeleteOnTermination` attribute for volumes that are
    #   attached to Fargate tasks.
    #
    #   To add instance store volumes to an Amazon EBS-backed instance, you
    #   must add them when you launch the instance. For more information, see
    #   [Update the block device mapping when launching an instance][1] in the
    #   *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/block-device-mapping-concepts.html#Using_OverridingAMIBDM
    #
    # @option params [Types::AttributeBooleanValue] :disable_api_termination
    #   Enable or disable termination protection for the instance. If the
    #   value is `true`, you can't terminate the instance using the Amazon
    #   EC2 console, command line interface, or API. You can't enable
    #   termination protection for Spot Instances.
    #
    # @option params [Types::AttributeValue] :instance_type
    #   Changes the instance type to the specified value. For more
    #   information, see [Instance types][1] in the *Amazon EC2 User Guide*.
    #   If the instance type is not valid, the error returned is
    #   `InvalidInstanceAttributeValue`.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html
    #
    # @option params [Types::AttributeValue] :kernel
    #   Changes the instance's kernel to the specified value. We recommend
    #   that you use PV-GRUB instead of kernels and RAM disks. For more
    #   information, see [PV-GRUB][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedKernels.html
    #
    # @option params [Types::AttributeValue] :ramdisk
    #   Changes the instance's RAM disk to the specified value. We recommend
    #   that you use PV-GRUB instead of kernels and RAM disks. For more
    #   information, see [PV-GRUB][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedKernels.html
    #
    # @option params [Types::BlobAttributeValue] :user_data
    #   Changes the instance's user data to the specified value. User data
    #   must be base64-encoded. Depending on the tool or SDK that you're
    #   using, the base64-encoding might be performed for you. For more
    #   information, see [Work with instance user data][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instancedata-add-user-data.html
    #
    # @option params [Types::AttributeValue] :instance_initiated_shutdown_behavior
    #   Specifies whether an instance stops or terminates when you initiate
    #   shutdown from the instance (using the operating system command for
    #   system shutdown).
    #
    # @option params [Array<String>] :groups
    #   Replaces the security groups of the instance with the specified
    #   security groups. You must specify the ID of at least one security
    #   group, even if it's just the default security group for the VPC.
    #
    # @option params [Types::AttributeBooleanValue] :ebs_optimized
    #   Specifies whether the instance is optimized for Amazon EBS I/O. This
    #   optimization provides dedicated throughput to Amazon EBS and an
    #   optimized configuration stack to provide optimal EBS I/O performance.
    #   This optimization isn't available with all instance types. Additional
    #   usage charges apply when using an EBS Optimized instance.
    #
    # @option params [Types::AttributeValue] :sriov_net_support
    #   Set to `simple` to enable enhanced networking with the Intel 82599
    #   Virtual Function interface for the instance.
    #
    #   There is no way to disable enhanced networking with the Intel 82599
    #   Virtual Function interface at this time.
    #
    #   This option is supported only for HVM instances. Specifying this
    #   option with a PV instance can make it unreachable.
    #
    # @option params [Types::AttributeBooleanValue] :ena_support
    #   Set to `true` to enable enhanced networking with ENA for the instance.
    #
    #   This option is supported only for HVM instances. Specifying this
    #   option with a PV instance can make it unreachable.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To modify the instance type
    #
    #   # This example modifies the instance type of the specified stopped instance.
    #
    #   resp = client.modify_instance_attribute({
    #     instance_id: "i-1234567890abcdef0", 
    #     instance_type: {
    #       value: "m5.large", 
    #     }, 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Example: To enable enhanced networking
    #
    #   # This example enables enhanced networking for the specified stopped instance.
    #
    #   resp = client.modify_instance_attribute({
    #     ena_support: {
    #       value: true, 
    #     }, 
    #     instance_id: "i-1234567890abcdef0", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_instance_attribute({
    #     source_dest_check: {
    #       value: false,
    #     },
    #     disable_api_stop: {
    #       value: false,
    #     },
    #     dry_run: false,
    #     instance_id: "InstanceId", # required
    #     attribute: "instanceType", # accepts instanceType, kernel, ramdisk, userData, disableApiTermination, instanceInitiatedShutdownBehavior, rootDeviceName, blockDeviceMapping, productCodes, sourceDestCheck, groupSet, ebsOptimized, sriovNetSupport, enaSupport, enclaveOptions, disableApiStop
    #     value: "String",
    #     block_device_mappings: [
    #       {
    #         device_name: "String",
    #         ebs: {
    #           volume_id: "VolumeId",
    #           delete_on_termination: false,
    #         },
    #         virtual_name: "String",
    #         no_device: "String",
    #       },
    #     ],
    #     disable_api_termination: {
    #       value: false,
    #     },
    #     instance_type: "value", # value <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #     kernel: "value", # value <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #     ramdisk: "value", # value <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #     user_data: {
    #       value: "data",
    #     },
    #     instance_initiated_shutdown_behavior: "value", # value <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #     groups: ["SecurityGroupId"],
    #     ebs_optimized: {
    #       value: false,
    #     },
    #     sriov_net_support: "value", # value <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #     ena_support: {
    #       value: false,
    #     },
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyInstanceAttribute AWS API Documentation
    #
    # @overload modify_instance_attribute(params = {})
    # @param [Hash] params ({})
    def modify_instance_attribute(params = {}, options = {})
      req = build_request(:modify_instance_attribute, params)
      req.send_request(options)
    end

    # Modifies the Capacity Reservation settings for a stopped instance. Use
    # this action to configure an instance to target a specific Capacity
    # Reservation, run in any `open` Capacity Reservation with matching
    # attributes, run in On-Demand Instance capacity, or only run in a
    # Capacity Reservation.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance to be modified.
    #
    # @option params [required, Types::CapacityReservationSpecification] :capacity_reservation_specification
    #   Information about the Capacity Reservation targeting option.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyInstanceCapacityReservationAttributesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyInstanceCapacityReservationAttributesResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_instance_capacity_reservation_attributes({
    #     instance_id: "InstanceId", # required
    #     capacity_reservation_specification: { # required
    #       capacity_reservation_preference: "capacity-reservations-only", # accepts capacity-reservations-only, open, none
    #       capacity_reservation_target: {
    #         capacity_reservation_id: "CapacityReservationId",
    #         capacity_reservation_resource_group_arn: "String",
    #       },
    #     },
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyInstanceCapacityReservationAttributes AWS API Documentation
    #
    # @overload modify_instance_capacity_reservation_attributes(params = {})
    # @param [Hash] params ({})
    def modify_instance_capacity_reservation_attributes(params = {}, options = {})
      req = build_request(:modify_instance_capacity_reservation_attributes, params)
      req.send_request(options)
    end

    # By default, all vCPUs for the instance type are active when you launch
    # an instance. When you configure the number of active vCPUs for the
    # instance, it can help you save on licensing costs and optimize
    # performance. The base cost of the instance remains unchanged.
    #
    # The number of active vCPUs equals the number of threads per CPU core
    # multiplied by the number of cores. The instance must be in a `Stopped`
    # state before you make changes.
    #
    # <note markdown="1"> Some instance type options do not support this capability. For more
    # information, see [Supported CPU options][1] in the *Amazon EC2 User
    # Guide*.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/cpu-options-supported-instances-values.html
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance to update.
    #
    # @option params [required, Integer] :core_count
    #   The number of CPU cores to activate for the specified instance.
    #
    # @option params [required, Integer] :threads_per_core
    #   The number of threads to run for each CPU core.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyInstanceCpuOptionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyInstanceCpuOptionsResult#instance_id #instance_id} => String
    #   * {Types::ModifyInstanceCpuOptionsResult#core_count #core_count} => Integer
    #   * {Types::ModifyInstanceCpuOptionsResult#threads_per_core #threads_per_core} => Integer
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_instance_cpu_options({
    #     instance_id: "InstanceId", # required
    #     core_count: 1, # required
    #     threads_per_core: 1, # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_id #=> String
    #   resp.core_count #=> Integer
    #   resp.threads_per_core #=> Integer
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyInstanceCpuOptions AWS API Documentation
    #
    # @overload modify_instance_cpu_options(params = {})
    # @param [Hash] params ({})
    def modify_instance_cpu_options(params = {}, options = {})
      req = build_request(:modify_instance_cpu_options, params)
      req.send_request(options)
    end

    # Modifies the credit option for CPU usage on a running or stopped
    # burstable performance instance. The credit options are `standard` and
    # `unlimited`.
    #
    # For more information, see [Burstable performance instances][1] in the
    # *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/burstable-performance-instances.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   Idempotency][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [required, Array<Types::InstanceCreditSpecificationRequest>] :instance_credit_specifications
    #   Information about the credit option for CPU usage.
    #
    # @return [Types::ModifyInstanceCreditSpecificationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyInstanceCreditSpecificationResult#successful_instance_credit_specifications #successful_instance_credit_specifications} => Array&lt;Types::SuccessfulInstanceCreditSpecificationItem&gt;
    #   * {Types::ModifyInstanceCreditSpecificationResult#unsuccessful_instance_credit_specifications #unsuccessful_instance_credit_specifications} => Array&lt;Types::UnsuccessfulInstanceCreditSpecificationItem&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_instance_credit_specification({
    #     dry_run: false,
    #     client_token: "String",
    #     instance_credit_specifications: [ # required
    #       {
    #         instance_id: "InstanceId", # required
    #         cpu_credits: "String",
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.successful_instance_credit_specifications #=> Array
    #   resp.successful_instance_credit_specifications[0].instance_id #=> String
    #   resp.unsuccessful_instance_credit_specifications #=> Array
    #   resp.unsuccessful_instance_credit_specifications[0].instance_id #=> String
    #   resp.unsuccessful_instance_credit_specifications[0].error.code #=> String, one of "InvalidInstanceID.Malformed", "InvalidInstanceID.NotFound", "IncorrectInstanceState", "InstanceCreditSpecification.NotSupported"
    #   resp.unsuccessful_instance_credit_specifications[0].error.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyInstanceCreditSpecification AWS API Documentation
    #
    # @overload modify_instance_credit_specification(params = {})
    # @param [Hash] params ({})
    def modify_instance_credit_specification(params = {}, options = {})
      req = build_request(:modify_instance_credit_specification, params)
      req.send_request(options)
    end

    # Modifies the start time for a scheduled Amazon EC2 instance event.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance with the scheduled event.
    #
    # @option params [required, String] :instance_event_id
    #   The ID of the event whose date and time you are modifying.
    #
    # @option params [required, Time,DateTime,Date,Integer,String] :not_before
    #   The new date and time when the event will take place.
    #
    # @return [Types::ModifyInstanceEventStartTimeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyInstanceEventStartTimeResult#event #event} => Types::InstanceStatusEvent
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_instance_event_start_time({
    #     dry_run: false,
    #     instance_id: "InstanceId", # required
    #     instance_event_id: "String", # required
    #     not_before: Time.now, # required
    #   })
    #
    # @example Response structure
    #
    #   resp.event.instance_event_id #=> String
    #   resp.event.code #=> String, one of "instance-reboot", "system-reboot", "system-maintenance", "instance-retirement", "instance-stop"
    #   resp.event.description #=> String
    #   resp.event.not_after #=> Time
    #   resp.event.not_before #=> Time
    #   resp.event.not_before_deadline #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyInstanceEventStartTime AWS API Documentation
    #
    # @overload modify_instance_event_start_time(params = {})
    # @param [Hash] params ({})
    def modify_instance_event_start_time(params = {}, options = {})
      req = build_request(:modify_instance_event_start_time, params)
      req.send_request(options)
    end

    # Modifies the specified event window.
    #
    # You can define either a set of time ranges or a cron expression when
    # modifying the event window, but not both.
    #
    # To modify the targets associated with the event window, use the
    # AssociateInstanceEventWindow and DisassociateInstanceEventWindow API.
    #
    # If Amazon Web Services has already scheduled an event, modifying an
    # event window won't change the time of the scheduled event.
    #
    # For more information, see [Define event windows for scheduled
    # events][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/event-windows.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :name
    #   The name of the event window.
    #
    # @option params [required, String] :instance_event_window_id
    #   The ID of the event window.
    #
    # @option params [Array<Types::InstanceEventWindowTimeRangeRequest>] :time_ranges
    #   The time ranges of the event window.
    #
    # @option params [String] :cron_expression
    #   The cron expression of the event window, for example, `* 0-4,20-23 * *
    #   1,5`.
    #
    #   Constraints:
    #
    #   * Only hour and day of the week values are supported.
    #
    #   * For day of the week values, you can specify either integers `0`
    #     through `6`, or alternative single values `SUN` through `SAT`.
    #
    #   * The minute, month, and year must be specified by `*`.
    #
    #   * The hour value must be one or a multiple range, for example, `0-4`
    #     or `0-4,20-23`.
    #
    #   * Each hour range must be &gt;= 2 hours, for example, `0-2` or
    #     `20-23`.
    #
    #   * The event window must be &gt;= 4 hours. The combined total time
    #     ranges in the event window must be &gt;= 4 hours.
    #
    #   For more information about cron expressions, see [cron][1] on the
    #   *Wikipedia website*.
    #
    #
    #
    #   [1]: https://en.wikipedia.org/wiki/Cron
    #
    # @return [Types::ModifyInstanceEventWindowResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyInstanceEventWindowResult#instance_event_window #instance_event_window} => Types::InstanceEventWindow
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_instance_event_window({
    #     dry_run: false,
    #     name: "String",
    #     instance_event_window_id: "InstanceEventWindowId", # required
    #     time_ranges: [
    #       {
    #         start_week_day: "sunday", # accepts sunday, monday, tuesday, wednesday, thursday, friday, saturday
    #         start_hour: 1,
    #         end_week_day: "sunday", # accepts sunday, monday, tuesday, wednesday, thursday, friday, saturday
    #         end_hour: 1,
    #       },
    #     ],
    #     cron_expression: "InstanceEventWindowCronExpression",
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_event_window.instance_event_window_id #=> String
    #   resp.instance_event_window.time_ranges #=> Array
    #   resp.instance_event_window.time_ranges[0].start_week_day #=> String, one of "sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"
    #   resp.instance_event_window.time_ranges[0].start_hour #=> Integer
    #   resp.instance_event_window.time_ranges[0].end_week_day #=> String, one of "sunday", "monday", "tuesday", "wednesday", "thursday", "friday", "saturday"
    #   resp.instance_event_window.time_ranges[0].end_hour #=> Integer
    #   resp.instance_event_window.name #=> String
    #   resp.instance_event_window.cron_expression #=> String
    #   resp.instance_event_window.association_target.instance_ids #=> Array
    #   resp.instance_event_window.association_target.instance_ids[0] #=> String
    #   resp.instance_event_window.association_target.tags #=> Array
    #   resp.instance_event_window.association_target.tags[0].key #=> String
    #   resp.instance_event_window.association_target.tags[0].value #=> String
    #   resp.instance_event_window.association_target.dedicated_host_ids #=> Array
    #   resp.instance_event_window.association_target.dedicated_host_ids[0] #=> String
    #   resp.instance_event_window.state #=> String, one of "creating", "deleting", "active", "deleted"
    #   resp.instance_event_window.tags #=> Array
    #   resp.instance_event_window.tags[0].key #=> String
    #   resp.instance_event_window.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyInstanceEventWindow AWS API Documentation
    #
    # @overload modify_instance_event_window(params = {})
    # @param [Hash] params ({})
    def modify_instance_event_window(params = {}, options = {})
      req = build_request(:modify_instance_event_window, params)
      req.send_request(options)
    end

    # Modifies the recovery behavior of your instance to disable simplified
    # automatic recovery or set the recovery behavior to default. The
    # default configuration will not enable simplified automatic recovery
    # for an unsupported instance type. For more information, see
    # [Simplified automatic recovery][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-recover.html#instance-configuration-recovery
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @option params [String] :auto_recovery
    #   Disables the automatic recovery behavior of your instance or sets it
    #   to default.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyInstanceMaintenanceOptionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyInstanceMaintenanceOptionsResult#instance_id #instance_id} => String
    #   * {Types::ModifyInstanceMaintenanceOptionsResult#auto_recovery #auto_recovery} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_instance_maintenance_options({
    #     instance_id: "InstanceId", # required
    #     auto_recovery: "disabled", # accepts disabled, default
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_id #=> String
    #   resp.auto_recovery #=> String, one of "disabled", "default"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyInstanceMaintenanceOptions AWS API Documentation
    #
    # @overload modify_instance_maintenance_options(params = {})
    # @param [Hash] params ({})
    def modify_instance_maintenance_options(params = {}, options = {})
      req = build_request(:modify_instance_maintenance_options, params)
      req.send_request(options)
    end

    # Modifies the default instance metadata service (IMDS) settings at the
    # account level in the specified Amazon Web Services  Region.
    #
    # <note markdown="1"> To remove a parameter's account-level default setting, specify
    # `no-preference`. If an account-level setting is cleared with
    # `no-preference`, then the instance launch considers the other instance
    # metadata settings. For more information, see [Order of precedence for
    # instance metadata options][1] in the *Amazon EC2 User Guide*.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/configuring-instance-metadata-options.html#instance-metadata-options-order-of-precedence
    #
    # @option params [String] :http_tokens
    #   Indicates whether IMDSv2 is required.
    #
    #   * `optional` – IMDSv2 is optional, which means that you can use either
    #     IMDSv2 or IMDSv1.
    #
    #   * `required` – IMDSv2 is required, which means that IMDSv1 is
    #     disabled, and you must use IMDSv2.
    #
    # @option params [Integer] :http_put_response_hop_limit
    #   The maximum number of hops that the metadata token can travel. To
    #   indicate no preference, specify `-1`.
    #
    #   Possible values: Integers from `1` to `64`, and `-1` to indicate no
    #   preference
    #
    # @option params [String] :http_endpoint
    #   Enables or disables the IMDS endpoint on an instance. When disabled,
    #   the instance metadata can't be accessed.
    #
    # @option params [String] :instance_metadata_tags
    #   Enables or disables access to an instance's tags from the instance
    #   metadata. For more information, see [Work with instance tags using the
    #   instance metadata][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html#work-with-tags-in-IMDS
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyInstanceMetadataDefaultsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyInstanceMetadataDefaultsResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_instance_metadata_defaults({
    #     http_tokens: "optional", # accepts optional, required, no-preference
    #     http_put_response_hop_limit: 1,
    #     http_endpoint: "disabled", # accepts disabled, enabled, no-preference
    #     instance_metadata_tags: "disabled", # accepts disabled, enabled, no-preference
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyInstanceMetadataDefaults AWS API Documentation
    #
    # @overload modify_instance_metadata_defaults(params = {})
    # @param [Hash] params ({})
    def modify_instance_metadata_defaults(params = {}, options = {})
      req = build_request(:modify_instance_metadata_defaults, params)
      req.send_request(options)
    end

    # Modify the instance metadata parameters on a running or stopped
    # instance. When you modify the parameters on a stopped instance, they
    # are applied when the instance is started. When you modify the
    # parameters on a running instance, the API responds with a state of
    # “pending”. After the parameter modifications are successfully applied
    # to the instance, the state of the modifications changes from “pending”
    # to “applied” in subsequent describe-instances API calls. For more
    # information, see [Instance metadata and user data][1] in the *Amazon
    # EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @option params [String] :http_tokens
    #   Indicates whether IMDSv2 is required.
    #
    #   * `optional` - IMDSv2 is optional. You can choose whether to send a
    #     session token in your instance metadata retrieval requests. If you
    #     retrieve IAM role credentials without a session token, you receive
    #     the IMDSv1 role credentials. If you retrieve IAM role credentials
    #     using a valid session token, you receive the IMDSv2 role
    #     credentials.
    #
    #   * `required` - IMDSv2 is required. You must send a session token in
    #     your instance metadata retrieval requests. With this option,
    #     retrieving the IAM role credentials always returns IMDSv2
    #     credentials; IMDSv1 credentials are not available.
    #
    #   Default:
    #
    #   * If the value of `ImdsSupport` for the Amazon Machine Image (AMI) for
    #     your instance is `v2.0` and the account level default is set to
    #     `no-preference`, the default is `required`.
    #
    #   * If the value of `ImdsSupport` for the Amazon Machine Image (AMI) for
    #     your instance is `v2.0`, but the account level default is set to `V1
    #     or V2`, the default is `optional`.
    #
    #   The default value can also be affected by other combinations of
    #   parameters. For more information, see [Order of precedence for
    #   instance metadata options][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/configuring-instance-metadata-options.html#instance-metadata-options-order-of-precedence
    #
    # @option params [Integer] :http_put_response_hop_limit
    #   The desired HTTP PUT response hop limit for instance metadata
    #   requests. The larger the number, the further instance metadata
    #   requests can travel. If no parameter is specified, the existing state
    #   is maintained.
    #
    #   Possible values: Integers from 1 to 64
    #
    # @option params [String] :http_endpoint
    #   Enables or disables the HTTP metadata endpoint on your instances. If
    #   this parameter is not specified, the existing state is maintained.
    #
    #   If you specify a value of `disabled`, you cannot access your instance
    #   metadata.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :http_protocol_ipv_6
    #   Enables or disables the IPv6 endpoint for the instance metadata
    #   service. Applies only if you enabled the HTTP metadata endpoint.
    #
    # @option params [String] :instance_metadata_tags
    #   Set to `enabled` to allow access to instance tags from the instance
    #   metadata. Set to `disabled` to turn off access to instance tags from
    #   the instance metadata. For more information, see [Work with instance
    #   tags using the instance metadata][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html#work-with-tags-in-IMDS
    #
    # @return [Types::ModifyInstanceMetadataOptionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyInstanceMetadataOptionsResult#instance_id #instance_id} => String
    #   * {Types::ModifyInstanceMetadataOptionsResult#instance_metadata_options #instance_metadata_options} => Types::InstanceMetadataOptionsResponse
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_instance_metadata_options({
    #     instance_id: "InstanceId", # required
    #     http_tokens: "optional", # accepts optional, required
    #     http_put_response_hop_limit: 1,
    #     http_endpoint: "disabled", # accepts disabled, enabled
    #     dry_run: false,
    #     http_protocol_ipv_6: "disabled", # accepts disabled, enabled
    #     instance_metadata_tags: "disabled", # accepts disabled, enabled
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_id #=> String
    #   resp.instance_metadata_options.state #=> String, one of "pending", "applied"
    #   resp.instance_metadata_options.http_tokens #=> String, one of "optional", "required"
    #   resp.instance_metadata_options.http_put_response_hop_limit #=> Integer
    #   resp.instance_metadata_options.http_endpoint #=> String, one of "disabled", "enabled"
    #   resp.instance_metadata_options.http_protocol_ipv_6 #=> String, one of "disabled", "enabled"
    #   resp.instance_metadata_options.instance_metadata_tags #=> String, one of "disabled", "enabled"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyInstanceMetadataOptions AWS API Documentation
    #
    # @overload modify_instance_metadata_options(params = {})
    # @param [Hash] params ({})
    def modify_instance_metadata_options(params = {}, options = {})
      req = build_request(:modify_instance_metadata_options, params)
      req.send_request(options)
    end

    # Change the configuration of the network performance options for an
    # existing instance.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance to update.
    #
    # @option params [required, String] :bandwidth_weighting
    #   Specify the bandwidth weighting option to boost the associated type of
    #   baseline bandwidth, as follows:
    #
    #   default
    #
    #   : This option uses the standard bandwidth configuration for your
    #     instance type.
    #
    #   vpc-1
    #
    #   : This option boosts your networking baseline bandwidth and reduces
    #     your EBS baseline bandwidth.
    #
    #   ebs-1
    #
    #   : This option boosts your EBS baseline bandwidth and reduces your
    #     networking baseline bandwidth.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyInstanceNetworkPerformanceResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyInstanceNetworkPerformanceResult#instance_id #instance_id} => String
    #   * {Types::ModifyInstanceNetworkPerformanceResult#bandwidth_weighting #bandwidth_weighting} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_instance_network_performance_options({
    #     instance_id: "InstanceId", # required
    #     bandwidth_weighting: "default", # required, accepts default, vpc-1, ebs-1
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_id #=> String
    #   resp.bandwidth_weighting #=> String, one of "default", "vpc-1", "ebs-1"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyInstanceNetworkPerformanceOptions AWS API Documentation
    #
    # @overload modify_instance_network_performance_options(params = {})
    # @param [Hash] params ({})
    def modify_instance_network_performance_options(params = {}, options = {})
      req = build_request(:modify_instance_network_performance_options, params)
      req.send_request(options)
    end

    # Modifies the placement attributes for a specified instance. You can do
    # the following:
    #
    # * Modify the affinity between an instance and a [Dedicated Host][1].
    #   When affinity is set to `host` and the instance is not associated
    #   with a specific Dedicated Host, the next time the instance is
    #   started, it is automatically associated with the host on which it
    #   lands. If the instance is restarted or rebooted, this relationship
    #   persists.
    #
    # * Change the Dedicated Host with which an instance is associated.
    #
    # * Change the instance tenancy of an instance.
    #
    # * Move an instance to or from a [placement group][2].
    #
    # At least one attribute for affinity, host ID, tenancy, or placement
    # group name must be specified in the request. Affinity and tenancy can
    # be modified in the same request.
    #
    # To modify the host ID, tenancy, placement group, or partition for an
    # instance, the instance must be in the `stopped` state.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-hosts-overview.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/placement-groups.html
    #
    # @option params [String] :group_name
    #   The name of the placement group in which to place the instance. For
    #   spread placement groups, the instance must have a tenancy of
    #   `default`. For cluster and partition placement groups, the instance
    #   must have a tenancy of `default` or `dedicated`.
    #
    #   To remove an instance from a placement group, specify an empty string
    #   ("").
    #
    # @option params [Integer] :partition_number
    #   The number of the partition in which to place the instance. Valid only
    #   if the placement group strategy is set to `partition`.
    #
    # @option params [String] :host_resource_group_arn
    #   The ARN of the host resource group in which to place the instance. The
    #   instance must have a tenancy of `host` to specify this parameter.
    #
    # @option params [String] :group_id
    #   The Group Id of a placement group. You must specify the Placement
    #   Group **Group Id** to launch an instance in a shared placement group.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance that you are modifying.
    #
    # @option params [String] :tenancy
    #   The tenancy for the instance.
    #
    #   <note markdown="1"> For T3 instances, you must launch the instance on a Dedicated Host to
    #   use a tenancy of `host`. You can't change the tenancy from `host` to
    #   `dedicated` or `default`. Attempting to make one of these unsupported
    #   tenancy changes results in an `InvalidRequest` error code.
    #
    #    </note>
    #
    # @option params [String] :affinity
    #   The affinity setting for the instance. For more information, see [Host
    #   affinity][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/how-dedicated-hosts-work.html#dedicated-hosts-affinity
    #
    # @option params [String] :host_id
    #   The ID of the Dedicated Host with which to associate the instance.
    #
    # @return [Types::ModifyInstancePlacementResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyInstancePlacementResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_instance_placement({
    #     group_name: "PlacementGroupName",
    #     partition_number: 1,
    #     host_resource_group_arn: "String",
    #     group_id: "PlacementGroupId",
    #     instance_id: "InstanceId", # required
    #     tenancy: "default", # accepts default, dedicated, host
    #     affinity: "default", # accepts default, host
    #     host_id: "DedicatedHostId",
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyInstancePlacement AWS API Documentation
    #
    # @overload modify_instance_placement(params = {})
    # @param [Hash] params ({})
    def modify_instance_placement(params = {}, options = {})
      req = build_request(:modify_instance_placement, params)
      req.send_request(options)
    end

    # Modify the configurations of an IPAM.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_id
    #   The ID of the IPAM you want to modify.
    #
    # @option params [String] :description
    #   The description of the IPAM you want to modify.
    #
    # @option params [Array<Types::AddIpamOperatingRegion>] :add_operating_regions
    #   Choose the operating Regions for the IPAM. Operating Regions are
    #   Amazon Web Services Regions where the IPAM is allowed to manage IP
    #   address CIDRs. IPAM only discovers and monitors resources in the
    #   Amazon Web Services Regions you select as operating Regions.
    #
    #   For more information about operating Regions, see [Create an IPAM][1]
    #   in the *Amazon VPC IPAM User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/create-ipam.html
    #
    # @option params [Array<Types::RemoveIpamOperatingRegion>] :remove_operating_regions
    #   The operating Regions to remove.
    #
    # @option params [String] :tier
    #   IPAM is offered in a Free Tier and an Advanced Tier. For more
    #   information about the features available in each tier and the costs
    #   associated with the tiers, see [Amazon VPC pricing &gt; IPAM tab][1].
    #
    #
    #
    #   [1]: http://aws.amazon.com/vpc/pricing/
    #
    # @option params [Boolean] :enable_private_gua
    #   Enable this option to use your own GUA ranges as private IPv6
    #   addresses. This option is disabled by default.
    #
    # @return [Types::ModifyIpamResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyIpamResult#ipam #ipam} => Types::Ipam
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_ipam({
    #     dry_run: false,
    #     ipam_id: "IpamId", # required
    #     description: "String",
    #     add_operating_regions: [
    #       {
    #         region_name: "String",
    #       },
    #     ],
    #     remove_operating_regions: [
    #       {
    #         region_name: "String",
    #       },
    #     ],
    #     tier: "free", # accepts free, advanced
    #     enable_private_gua: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam.owner_id #=> String
    #   resp.ipam.ipam_id #=> String
    #   resp.ipam.ipam_arn #=> String
    #   resp.ipam.ipam_region #=> String
    #   resp.ipam.public_default_scope_id #=> String
    #   resp.ipam.private_default_scope_id #=> String
    #   resp.ipam.scope_count #=> Integer
    #   resp.ipam.description #=> String
    #   resp.ipam.operating_regions #=> Array
    #   resp.ipam.operating_regions[0].region_name #=> String
    #   resp.ipam.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam.tags #=> Array
    #   resp.ipam.tags[0].key #=> String
    #   resp.ipam.tags[0].value #=> String
    #   resp.ipam.default_resource_discovery_id #=> String
    #   resp.ipam.default_resource_discovery_association_id #=> String
    #   resp.ipam.resource_discovery_association_count #=> Integer
    #   resp.ipam.state_message #=> String
    #   resp.ipam.tier #=> String, one of "free", "advanced"
    #   resp.ipam.enable_private_gua #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyIpam AWS API Documentation
    #
    # @overload modify_ipam(params = {})
    # @param [Hash] params ({})
    def modify_ipam(params = {}, options = {})
      req = build_request(:modify_ipam, params)
      req.send_request(options)
    end

    # Modify the configurations of an IPAM pool.
    #
    # For more information, see [Modify a pool][1] in the *Amazon VPC IPAM
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/mod-pool-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_pool_id
    #   The ID of the IPAM pool you want to modify.
    #
    # @option params [String] :description
    #   The description of the IPAM pool you want to modify.
    #
    # @option params [Boolean] :auto_import
    #   If true, IPAM will continuously look for resources within the CIDR
    #   range of this pool and automatically import them as allocations into
    #   your IPAM. The CIDRs that will be allocated for these resources must
    #   not already be allocated to other resources in order for the import to
    #   succeed. IPAM will import a CIDR regardless of its compliance with the
    #   pool's allocation rules, so a resource might be imported and
    #   subsequently marked as noncompliant. If IPAM discovers multiple CIDRs
    #   that overlap, IPAM will import the largest CIDR only. If IPAM
    #   discovers multiple CIDRs with matching CIDRs, IPAM will randomly
    #   import one of them only.
    #
    #   A locale must be set on the pool for this feature to work.
    #
    # @option params [Integer] :allocation_min_netmask_length
    #   The minimum netmask length required for CIDR allocations in this IPAM
    #   pool to be compliant. Possible netmask lengths for IPv4 addresses are
    #   0 - 32. Possible netmask lengths for IPv6 addresses are 0 - 128. The
    #   minimum netmask length must be less than the maximum netmask length.
    #
    # @option params [Integer] :allocation_max_netmask_length
    #   The maximum netmask length possible for CIDR allocations in this IPAM
    #   pool to be compliant. Possible netmask lengths for IPv4 addresses are
    #   0 - 32. Possible netmask lengths for IPv6 addresses are 0 - 128.The
    #   maximum netmask length must be greater than the minimum netmask
    #   length.
    #
    # @option params [Integer] :allocation_default_netmask_length
    #   The default netmask length for allocations added to this pool. If, for
    #   example, the CIDR assigned to this pool is 10.0.0.0/8 and you enter 16
    #   here, new allocations will default to 10.0.0.0/16.
    #
    # @option params [Boolean] :clear_allocation_default_netmask_length
    #   Clear the default netmask length allocation rule for this pool.
    #
    # @option params [Array<Types::RequestIpamResourceTag>] :add_allocation_resource_tags
    #   Add tag allocation rules to a pool. For more information about
    #   allocation rules, see [Create a top-level pool][1] in the *Amazon VPC
    #   IPAM User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/create-top-ipam.html
    #
    # @option params [Array<Types::RequestIpamResourceTag>] :remove_allocation_resource_tags
    #   Remove tag allocation rules from a pool.
    #
    # @return [Types::ModifyIpamPoolResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyIpamPoolResult#ipam_pool #ipam_pool} => Types::IpamPool
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_ipam_pool({
    #     dry_run: false,
    #     ipam_pool_id: "IpamPoolId", # required
    #     description: "String",
    #     auto_import: false,
    #     allocation_min_netmask_length: 1,
    #     allocation_max_netmask_length: 1,
    #     allocation_default_netmask_length: 1,
    #     clear_allocation_default_netmask_length: false,
    #     add_allocation_resource_tags: [
    #       {
    #         key: "String",
    #         value: "String",
    #       },
    #     ],
    #     remove_allocation_resource_tags: [
    #       {
    #         key: "String",
    #         value: "String",
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_pool.owner_id #=> String
    #   resp.ipam_pool.ipam_pool_id #=> String
    #   resp.ipam_pool.source_ipam_pool_id #=> String
    #   resp.ipam_pool.ipam_pool_arn #=> String
    #   resp.ipam_pool.ipam_scope_arn #=> String
    #   resp.ipam_pool.ipam_scope_type #=> String, one of "public", "private"
    #   resp.ipam_pool.ipam_arn #=> String
    #   resp.ipam_pool.ipam_region #=> String
    #   resp.ipam_pool.locale #=> String
    #   resp.ipam_pool.pool_depth #=> Integer
    #   resp.ipam_pool.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_pool.state_message #=> String
    #   resp.ipam_pool.description #=> String
    #   resp.ipam_pool.auto_import #=> Boolean
    #   resp.ipam_pool.publicly_advertisable #=> Boolean
    #   resp.ipam_pool.address_family #=> String, one of "ipv4", "ipv6"
    #   resp.ipam_pool.allocation_min_netmask_length #=> Integer
    #   resp.ipam_pool.allocation_max_netmask_length #=> Integer
    #   resp.ipam_pool.allocation_default_netmask_length #=> Integer
    #   resp.ipam_pool.allocation_resource_tags #=> Array
    #   resp.ipam_pool.allocation_resource_tags[0].key #=> String
    #   resp.ipam_pool.allocation_resource_tags[0].value #=> String
    #   resp.ipam_pool.tags #=> Array
    #   resp.ipam_pool.tags[0].key #=> String
    #   resp.ipam_pool.tags[0].value #=> String
    #   resp.ipam_pool.aws_service #=> String, one of "ec2"
    #   resp.ipam_pool.public_ip_source #=> String, one of "amazon", "byoip"
    #   resp.ipam_pool.source_resource.resource_id #=> String
    #   resp.ipam_pool.source_resource.resource_type #=> String, one of "vpc"
    #   resp.ipam_pool.source_resource.resource_region #=> String
    #   resp.ipam_pool.source_resource.resource_owner #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyIpamPool AWS API Documentation
    #
    # @overload modify_ipam_pool(params = {})
    # @param [Hash] params ({})
    def modify_ipam_pool(params = {}, options = {})
      req = build_request(:modify_ipam_pool, params)
      req.send_request(options)
    end

    # Modify a resource CIDR. You can use this action to transfer resource
    # CIDRs between scopes and ignore resource CIDRs that you do not want to
    # manage. If set to false, the resource will not be tracked for overlap,
    # it cannot be auto-imported into a pool, and it will be removed from
    # any pool it has an allocation in.
    #
    # For more information, see [Move resource CIDRs between scopes][1] and
    # [Change the monitoring state of resource CIDRs][2] in the *Amazon VPC
    # IPAM User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/move-resource-ipam.html
    # [2]: https://docs.aws.amazon.com/vpc/latest/ipam/change-monitoring-state-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :resource_id
    #   The ID of the resource you want to modify.
    #
    # @option params [required, String] :resource_cidr
    #   The CIDR of the resource you want to modify.
    #
    # @option params [required, String] :resource_region
    #   The Amazon Web Services Region of the resource you want to modify.
    #
    # @option params [required, String] :current_ipam_scope_id
    #   The ID of the current scope that the resource CIDR is in.
    #
    # @option params [String] :destination_ipam_scope_id
    #   The ID of the scope you want to transfer the resource CIDR to.
    #
    # @option params [required, Boolean] :monitored
    #   Determines if the resource is monitored by IPAM. If a resource is
    #   monitored, the resource is discovered by IPAM and you can view details
    #   about the resource’s CIDR.
    #
    # @return [Types::ModifyIpamResourceCidrResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyIpamResourceCidrResult#ipam_resource_cidr #ipam_resource_cidr} => Types::IpamResourceCidr
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_ipam_resource_cidr({
    #     dry_run: false,
    #     resource_id: "String", # required
    #     resource_cidr: "String", # required
    #     resource_region: "String", # required
    #     current_ipam_scope_id: "IpamScopeId", # required
    #     destination_ipam_scope_id: "IpamScopeId",
    #     monitored: false, # required
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_resource_cidr.ipam_id #=> String
    #   resp.ipam_resource_cidr.ipam_scope_id #=> String
    #   resp.ipam_resource_cidr.ipam_pool_id #=> String
    #   resp.ipam_resource_cidr.resource_region #=> String
    #   resp.ipam_resource_cidr.resource_owner_id #=> String
    #   resp.ipam_resource_cidr.resource_id #=> String
    #   resp.ipam_resource_cidr.resource_name #=> String
    #   resp.ipam_resource_cidr.resource_cidr #=> String
    #   resp.ipam_resource_cidr.resource_type #=> String, one of "vpc", "subnet", "eip", "public-ipv4-pool", "ipv6-pool", "eni"
    #   resp.ipam_resource_cidr.resource_tags #=> Array
    #   resp.ipam_resource_cidr.resource_tags[0].key #=> String
    #   resp.ipam_resource_cidr.resource_tags[0].value #=> String
    #   resp.ipam_resource_cidr.ip_usage #=> Float
    #   resp.ipam_resource_cidr.compliance_status #=> String, one of "compliant", "noncompliant", "unmanaged", "ignored"
    #   resp.ipam_resource_cidr.management_state #=> String, one of "managed", "unmanaged", "ignored"
    #   resp.ipam_resource_cidr.overlap_status #=> String, one of "overlapping", "nonoverlapping", "ignored"
    #   resp.ipam_resource_cidr.vpc_id #=> String
    #   resp.ipam_resource_cidr.availability_zone_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyIpamResourceCidr AWS API Documentation
    #
    # @overload modify_ipam_resource_cidr(params = {})
    # @param [Hash] params ({})
    def modify_ipam_resource_cidr(params = {}, options = {})
      req = build_request(:modify_ipam_resource_cidr, params)
      req.send_request(options)
    end

    # Modifies a resource discovery. A resource discovery is an IPAM
    # component that enables IPAM to manage and monitor resources that
    # belong to the owning account.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_resource_discovery_id
    #   A resource discovery ID.
    #
    # @option params [String] :description
    #   A resource discovery description.
    #
    # @option params [Array<Types::AddIpamOperatingRegion>] :add_operating_regions
    #   Add operating Regions to the resource discovery. Operating Regions are
    #   Amazon Web Services Regions where the IPAM is allowed to manage IP
    #   address CIDRs. IPAM only discovers and monitors resources in the
    #   Amazon Web Services Regions you select as operating Regions.
    #
    # @option params [Array<Types::RemoveIpamOperatingRegion>] :remove_operating_regions
    #   Remove operating Regions.
    #
    # @option params [Array<Types::AddIpamOrganizationalUnitExclusion>] :add_organizational_unit_exclusions
    #   Add an Organizational Unit (OU) exclusion to your IPAM. If your IPAM
    #   is integrated with Amazon Web Services Organizations and you add an
    #   organizational unit (OU) exclusion, IPAM will not manage the IP
    #   addresses in accounts in that OU exclusion. There is a limit on the
    #   number of exclusions you can create. For more information, see [Quotas
    #   for your IPAM][1] in the *Amazon VPC IPAM User Guide*.
    #
    #   <note markdown="1"> The resulting set of exclusions must not result in "overlap",
    #   meaning two or more OU exclusions must not exclude the same OU. For
    #   more information and examples, see the Amazon Web Services CLI request
    #   process in [Add or remove OU exclusions ][2] in the *Amazon VPC User
    #   Guide*.
    #
    #    </note>
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/quotas-ipam.html
    #   [2]: https://docs.aws.amazon.com/vpc/latest/ipam/exclude-ous.html#exclude-ous-create-delete
    #
    # @option params [Array<Types::RemoveIpamOrganizationalUnitExclusion>] :remove_organizational_unit_exclusions
    #   Remove an Organizational Unit (OU) exclusion to your IPAM. If your
    #   IPAM is integrated with Amazon Web Services Organizations and you add
    #   an organizational unit (OU) exclusion, IPAM will not manage the IP
    #   addresses in accounts in that OU exclusion. There is a limit on the
    #   number of exclusions you can create. For more information, see [Quotas
    #   for your IPAM][1] in the *Amazon VPC IPAM User Guide*.
    #
    #   <note markdown="1"> The resulting set of exclusions must not result in "overlap",
    #   meaning two or more OU exclusions must not exclude the same OU. For
    #   more information and examples, see the Amazon Web Services CLI request
    #   process in [Add or remove OU exclusions ][2] in the *Amazon VPC User
    #   Guide*.
    #
    #    </note>
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/ipam/quotas-ipam.html
    #   [2]: https://docs.aws.amazon.com/vpc/latest/ipam/exclude-ous.html#exclude-ous-create-delete
    #
    # @return [Types::ModifyIpamResourceDiscoveryResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyIpamResourceDiscoveryResult#ipam_resource_discovery #ipam_resource_discovery} => Types::IpamResourceDiscovery
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_ipam_resource_discovery({
    #     dry_run: false,
    #     ipam_resource_discovery_id: "IpamResourceDiscoveryId", # required
    #     description: "String",
    #     add_operating_regions: [
    #       {
    #         region_name: "String",
    #       },
    #     ],
    #     remove_operating_regions: [
    #       {
    #         region_name: "String",
    #       },
    #     ],
    #     add_organizational_unit_exclusions: [
    #       {
    #         organizations_entity_path: "String",
    #       },
    #     ],
    #     remove_organizational_unit_exclusions: [
    #       {
    #         organizations_entity_path: "String",
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_resource_discovery.owner_id #=> String
    #   resp.ipam_resource_discovery.ipam_resource_discovery_id #=> String
    #   resp.ipam_resource_discovery.ipam_resource_discovery_arn #=> String
    #   resp.ipam_resource_discovery.ipam_resource_discovery_region #=> String
    #   resp.ipam_resource_discovery.description #=> String
    #   resp.ipam_resource_discovery.operating_regions #=> Array
    #   resp.ipam_resource_discovery.operating_regions[0].region_name #=> String
    #   resp.ipam_resource_discovery.is_default #=> Boolean
    #   resp.ipam_resource_discovery.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_resource_discovery.tags #=> Array
    #   resp.ipam_resource_discovery.tags[0].key #=> String
    #   resp.ipam_resource_discovery.tags[0].value #=> String
    #   resp.ipam_resource_discovery.organizational_unit_exclusions #=> Array
    #   resp.ipam_resource_discovery.organizational_unit_exclusions[0].organizations_entity_path #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyIpamResourceDiscovery AWS API Documentation
    #
    # @overload modify_ipam_resource_discovery(params = {})
    # @param [Hash] params ({})
    def modify_ipam_resource_discovery(params = {}, options = {})
      req = build_request(:modify_ipam_resource_discovery, params)
      req.send_request(options)
    end

    # Modify an IPAM scope.
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_scope_id
    #   The ID of the scope you want to modify.
    #
    # @option params [String] :description
    #   The description of the scope you want to modify.
    #
    # @return [Types::ModifyIpamScopeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyIpamScopeResult#ipam_scope #ipam_scope} => Types::IpamScope
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_ipam_scope({
    #     dry_run: false,
    #     ipam_scope_id: "IpamScopeId", # required
    #     description: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_scope.owner_id #=> String
    #   resp.ipam_scope.ipam_scope_id #=> String
    #   resp.ipam_scope.ipam_scope_arn #=> String
    #   resp.ipam_scope.ipam_arn #=> String
    #   resp.ipam_scope.ipam_region #=> String
    #   resp.ipam_scope.ipam_scope_type #=> String, one of "public", "private"
    #   resp.ipam_scope.is_default #=> Boolean
    #   resp.ipam_scope.description #=> String
    #   resp.ipam_scope.pool_count #=> Integer
    #   resp.ipam_scope.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "delete-in-progress", "delete-complete", "delete-failed", "isolate-in-progress", "isolate-complete", "restore-in-progress"
    #   resp.ipam_scope.tags #=> Array
    #   resp.ipam_scope.tags[0].key #=> String
    #   resp.ipam_scope.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyIpamScope AWS API Documentation
    #
    # @overload modify_ipam_scope(params = {})
    # @param [Hash] params ({})
    def modify_ipam_scope(params = {}, options = {})
      req = build_request(:modify_ipam_scope, params)
      req.send_request(options)
    end

    # Modifies a launch template. You can specify which version of the
    # launch template to set as the default version. When launching an
    # instance, the default version applies when a launch template version
    # is not specified.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier you provide to ensure the
    #   idempotency of the request. If a client token isn't specified, a
    #   randomly generated token is used in the request to ensure idempotency.
    #
    #   For more information, see [Ensuring idempotency][1].
    #
    #   Constraint: Maximum 128 ASCII characters.
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [String] :launch_template_id
    #   The ID of the launch template.
    #
    #   You must specify either the launch template ID or the launch template
    #   name, but not both.
    #
    # @option params [String] :launch_template_name
    #   The name of the launch template.
    #
    #   You must specify either the launch template ID or the launch template
    #   name, but not both.
    #
    # @option params [String] :default_version
    #   The version number of the launch template to set as the default
    #   version.
    #
    # @return [Types::ModifyLaunchTemplateResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyLaunchTemplateResult#launch_template #launch_template} => Types::LaunchTemplate
    #
    #
    # @example Example: To change the default version of a launch template
    #
    #   # This example specifies version 2 as the default version of the specified launch template.
    #
    #   resp = client.modify_launch_template({
    #     default_version: "2", 
    #     launch_template_id: "lt-0abcd290751193123", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     launch_template: {
    #       create_time: Time.parse("2017-12-01T13:35:46.000Z"), 
    #       created_by: "arn:aws:iam::123456789012:root", 
    #       default_version_number: 2, 
    #       latest_version_number: 2, 
    #       launch_template_id: "lt-0abcd290751193123", 
    #       launch_template_name: "WebServers", 
    #     }, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_launch_template({
    #     dry_run: false,
    #     client_token: "String",
    #     launch_template_id: "LaunchTemplateId",
    #     launch_template_name: "LaunchTemplateName",
    #     default_version: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.launch_template.launch_template_id #=> String
    #   resp.launch_template.launch_template_name #=> String
    #   resp.launch_template.create_time #=> Time
    #   resp.launch_template.created_by #=> String
    #   resp.launch_template.default_version_number #=> Integer
    #   resp.launch_template.latest_version_number #=> Integer
    #   resp.launch_template.tags #=> Array
    #   resp.launch_template.tags[0].key #=> String
    #   resp.launch_template.tags[0].value #=> String
    #   resp.launch_template.operator.managed #=> Boolean
    #   resp.launch_template.operator.principal #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyLaunchTemplate AWS API Documentation
    #
    # @overload modify_launch_template(params = {})
    # @param [Hash] params ({})
    def modify_launch_template(params = {}, options = {})
      req = build_request(:modify_launch_template, params)
      req.send_request(options)
    end

    # Modifies the specified local gateway route.
    #
    # @option params [String] :destination_cidr_block
    #   The CIDR block used for destination matches. The value that you
    #   provide must match the CIDR of an existing route in the table.
    #
    # @option params [required, String] :local_gateway_route_table_id
    #   The ID of the local gateway route table.
    #
    # @option params [String] :local_gateway_virtual_interface_group_id
    #   The ID of the virtual interface group.
    #
    # @option params [String] :network_interface_id
    #   The ID of the network interface.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :destination_prefix_list_id
    #   The ID of the prefix list. Use a prefix list in place of
    #   `DestinationCidrBlock`. You cannot use `DestinationPrefixListId` and
    #   `DestinationCidrBlock` in the same request.
    #
    # @return [Types::ModifyLocalGatewayRouteResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyLocalGatewayRouteResult#route #route} => Types::LocalGatewayRoute
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_local_gateway_route({
    #     destination_cidr_block: "String",
    #     local_gateway_route_table_id: "LocalGatewayRoutetableId", # required
    #     local_gateway_virtual_interface_group_id: "LocalGatewayVirtualInterfaceGroupId",
    #     network_interface_id: "NetworkInterfaceId",
    #     dry_run: false,
    #     destination_prefix_list_id: "PrefixListResourceId",
    #   })
    #
    # @example Response structure
    #
    #   resp.route.destination_cidr_block #=> String
    #   resp.route.local_gateway_virtual_interface_group_id #=> String
    #   resp.route.type #=> String, one of "static", "propagated"
    #   resp.route.state #=> String, one of "pending", "active", "blackhole", "deleting", "deleted"
    #   resp.route.local_gateway_route_table_id #=> String
    #   resp.route.local_gateway_route_table_arn #=> String
    #   resp.route.owner_id #=> String
    #   resp.route.subnet_id #=> String
    #   resp.route.coip_pool_id #=> String
    #   resp.route.network_interface_id #=> String
    #   resp.route.destination_prefix_list_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyLocalGatewayRoute AWS API Documentation
    #
    # @overload modify_local_gateway_route(params = {})
    # @param [Hash] params ({})
    def modify_local_gateway_route(params = {}, options = {})
      req = build_request(:modify_local_gateway_route, params)
      req.send_request(options)
    end

    # Modifies the specified managed prefix list.
    #
    # Adding or removing entries in a prefix list creates a new version of
    # the prefix list. Changing the name of the prefix list does not affect
    # the version.
    #
    # If you specify a current version number that does not match the true
    # current version number, the request fails.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :prefix_list_id
    #   The ID of the prefix list.
    #
    # @option params [Integer] :current_version
    #   The current version of the prefix list.
    #
    # @option params [String] :prefix_list_name
    #   A name for the prefix list.
    #
    # @option params [Array<Types::AddPrefixListEntry>] :add_entries
    #   One or more entries to add to the prefix list.
    #
    # @option params [Array<Types::RemovePrefixListEntry>] :remove_entries
    #   One or more entries to remove from the prefix list.
    #
    # @option params [Integer] :max_entries
    #   The maximum number of entries for the prefix list. You cannot modify
    #   the entries of a prefix list and modify the size of a prefix list at
    #   the same time.
    #
    #   If any of the resources that reference the prefix list cannot support
    #   the new maximum size, the modify operation fails. Check the state
    #   message for the IDs of the first ten resources that do not support the
    #   new maximum size.
    #
    # @return [Types::ModifyManagedPrefixListResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyManagedPrefixListResult#prefix_list #prefix_list} => Types::ManagedPrefixList
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_managed_prefix_list({
    #     dry_run: false,
    #     prefix_list_id: "PrefixListResourceId", # required
    #     current_version: 1,
    #     prefix_list_name: "String",
    #     add_entries: [
    #       {
    #         cidr: "String", # required
    #         description: "String",
    #       },
    #     ],
    #     remove_entries: [
    #       {
    #         cidr: "String", # required
    #       },
    #     ],
    #     max_entries: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.prefix_list.prefix_list_id #=> String
    #   resp.prefix_list.address_family #=> String
    #   resp.prefix_list.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "restore-in-progress", "restore-complete", "restore-failed", "delete-in-progress", "delete-complete", "delete-failed"
    #   resp.prefix_list.state_message #=> String
    #   resp.prefix_list.prefix_list_arn #=> String
    #   resp.prefix_list.prefix_list_name #=> String
    #   resp.prefix_list.max_entries #=> Integer
    #   resp.prefix_list.version #=> Integer
    #   resp.prefix_list.tags #=> Array
    #   resp.prefix_list.tags[0].key #=> String
    #   resp.prefix_list.tags[0].value #=> String
    #   resp.prefix_list.owner_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyManagedPrefixList AWS API Documentation
    #
    # @overload modify_managed_prefix_list(params = {})
    # @param [Hash] params ({})
    def modify_managed_prefix_list(params = {}, options = {})
      req = build_request(:modify_managed_prefix_list, params)
      req.send_request(options)
    end

    # Modifies the specified network interface attribute. You can specify
    # only one attribute at a time. You can use this action to attach and
    # detach security groups from an existing EC2 instance.
    #
    # @option params [Types::EnaSrdSpecification] :ena_srd_specification
    #   Updates the ENA Express configuration for the network interface that’s
    #   attached to the instance.
    #
    # @option params [Boolean] :enable_primary_ipv_6
    #   If you’re modifying a network interface in a dual-stack or IPv6-only
    #   subnet, you have the option to assign a primary IPv6 IP address. A
    #   primary IPv6 address is an IPv6 GUA address associated with an ENI
    #   that you have enabled to use a primary IPv6 address. Use this option
    #   if the instance that this ENI will be attached to relies on its IPv6
    #   address not changing. Amazon Web Services will automatically assign an
    #   IPv6 address associated with the ENI attached to your instance to be
    #   the primary IPv6 address. Once you enable an IPv6 GUA address to be a
    #   primary IPv6, you cannot disable it. When you enable an IPv6 GUA
    #   address to be a primary IPv6, the first IPv6 GUA will be made the
    #   primary IPv6 address until the instance is terminated or the network
    #   interface is detached. If you have multiple IPv6 addresses associated
    #   with an ENI attached to your instance and you enable a primary IPv6
    #   address, the first IPv6 GUA address associated with the ENI becomes
    #   the primary IPv6 address.
    #
    # @option params [Types::ConnectionTrackingSpecificationRequest] :connection_tracking_specification
    #   A connection tracking specification.
    #
    # @option params [Boolean] :associate_public_ip_address
    #   Indicates whether to assign a public IPv4 address to a network
    #   interface. This option can be enabled for any network interface but
    #   will only apply to the primary network interface (eth0).
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :network_interface_id
    #   The ID of the network interface.
    #
    # @option params [Types::AttributeValue] :description
    #   A description for the network interface.
    #
    # @option params [Types::AttributeBooleanValue] :source_dest_check
    #   Enable or disable source/destination checks, which ensure that the
    #   instance is either the source or the destination of any traffic that
    #   it receives. If the value is `true`, source/destination checks are
    #   enabled; otherwise, they are disabled. The default value is `true`.
    #   You must disable source/destination checks if the instance runs
    #   services such as network address translation, routing, or firewalls.
    #
    # @option params [Array<String>] :groups
    #   Changes the security groups for the network interface. The new set of
    #   groups you specify replaces the current set. You must specify at least
    #   one group, even if it's just the default security group in the VPC.
    #   You must specify the ID of the security group, not the name.
    #
    # @option params [Types::NetworkInterfaceAttachmentChanges] :attachment
    #   Information about the interface attachment. If modifying the `delete
    #   on termination` attribute, you must specify the ID of the interface
    #   attachment.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To modify the attachment attribute of a network interface
    #
    #   # This example modifies the attachment attribute of the specified network interface.
    #
    #   resp = client.modify_network_interface_attribute({
    #     attachment: {
    #       attachment_id: "eni-attach-43348162", 
    #       delete_on_termination: false, 
    #     }, 
    #     network_interface_id: "eni-686ea200", 
    #   })
    #
    # @example Example: To modify the description attribute of a network interface
    #
    #   # This example modifies the description attribute of the specified network interface.
    #
    #   resp = client.modify_network_interface_attribute({
    #     description: {
    #       value: "My description", 
    #     }, 
    #     network_interface_id: "eni-686ea200", 
    #   })
    #
    # @example Example: To modify the groupSet attribute of a network interface
    #
    #   # This example command modifies the groupSet attribute of the specified network interface.
    #
    #   resp = client.modify_network_interface_attribute({
    #     groups: [
    #       "sg-903004f8", 
    #       "sg-1a2b3c4d", 
    #     ], 
    #     network_interface_id: "eni-686ea200", 
    #   })
    #
    # @example Example: To modify the sourceDestCheck attribute of a network interface
    #
    #   # This example command modifies the sourceDestCheck attribute of the specified network interface.
    #
    #   resp = client.modify_network_interface_attribute({
    #     network_interface_id: "eni-686ea200", 
    #     source_dest_check: {
    #       value: false, 
    #     }, 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_network_interface_attribute({
    #     ena_srd_specification: {
    #       ena_srd_enabled: false,
    #       ena_srd_udp_specification: {
    #         ena_srd_udp_enabled: false,
    #       },
    #     },
    #     enable_primary_ipv_6: false,
    #     connection_tracking_specification: {
    #       tcp_established_timeout: 1,
    #       udp_stream_timeout: 1,
    #       udp_timeout: 1,
    #     },
    #     associate_public_ip_address: false,
    #     dry_run: false,
    #     network_interface_id: "NetworkInterfaceId", # required
    #     description: "value", # value <Hash,Array,String,Numeric,Boolean,IO,Set,nil>
    #     source_dest_check: {
    #       value: false,
    #     },
    #     groups: ["SecurityGroupId"],
    #     attachment: {
    #       attachment_id: "NetworkInterfaceAttachmentId",
    #       delete_on_termination: false,
    #     },
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyNetworkInterfaceAttribute AWS API Documentation
    #
    # @overload modify_network_interface_attribute(params = {})
    # @param [Hash] params ({})
    def modify_network_interface_attribute(params = {}, options = {})
      req = build_request(:modify_network_interface_attribute, params)
      req.send_request(options)
    end

    # Modifies the options for instance hostnames for the specified
    # instance.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @option params [String] :private_dns_hostname_type
    #   The type of hostname for EC2 instances. For IPv4 only subnets, an
    #   instance DNS name must be based on the instance IPv4 address. For IPv6
    #   only subnets, an instance DNS name must be based on the instance ID.
    #   For dual-stack subnets, you can specify whether DNS names use the
    #   instance IPv4 address or the instance ID.
    #
    # @option params [Boolean] :enable_resource_name_dns_a_record
    #   Indicates whether to respond to DNS queries for instance hostnames
    #   with DNS A records.
    #
    # @option params [Boolean] :enable_resource_name_dns_aaaa_record
    #   Indicates whether to respond to DNS queries for instance hostnames
    #   with DNS AAAA records.
    #
    # @return [Types::ModifyPrivateDnsNameOptionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyPrivateDnsNameOptionsResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_private_dns_name_options({
    #     dry_run: false,
    #     instance_id: "InstanceId", # required
    #     private_dns_hostname_type: "ip-name", # accepts ip-name, resource-name
    #     enable_resource_name_dns_a_record: false,
    #     enable_resource_name_dns_aaaa_record: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyPrivateDnsNameOptions AWS API Documentation
    #
    # @overload modify_private_dns_name_options(params = {})
    # @param [Hash] params ({})
    def modify_private_dns_name_options(params = {}, options = {})
      req = build_request(:modify_private_dns_name_options, params)
      req.send_request(options)
    end

    # Modifies the configuration of your Reserved Instances, such as the
    # Availability Zone, instance count, or instance type. The Reserved
    # Instances to be modified must be identical, except for Availability
    # Zone, network platform, and instance type.
    #
    # For more information, see [Modify Reserved Instances][1] in the
    # *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ri-modifying.html
    #
    # @option params [required, Array<String>] :reserved_instances_ids
    #   The IDs of the Reserved Instances to modify.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token you provide to ensure idempotency of
    #   your modification request. For more information, see [Ensuring
    #   Idempotency][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [required, Array<Types::ReservedInstancesConfiguration>] :target_configurations
    #   The configuration settings for the Reserved Instances to modify.
    #
    # @return [Types::ModifyReservedInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyReservedInstancesResult#reserved_instances_modification_id #reserved_instances_modification_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_reserved_instances({
    #     reserved_instances_ids: ["ReservationId"], # required
    #     client_token: "String",
    #     target_configurations: [ # required
    #       {
    #         availability_zone: "String",
    #         instance_count: 1,
    #         instance_type: "a1.medium", # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #         platform: "String",
    #         scope: "Availability Zone", # accepts Availability Zone, Region
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.reserved_instances_modification_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyReservedInstances AWS API Documentation
    #
    # @overload modify_reserved_instances(params = {})
    # @param [Hash] params ({})
    def modify_reserved_instances(params = {}, options = {})
      req = build_request(:modify_reserved_instances, params)
      req.send_request(options)
    end

    # Modifies the rules of a security group.
    #
    # @option params [required, String] :group_id
    #   The ID of the security group.
    #
    # @option params [required, Array<Types::SecurityGroupRuleUpdate>] :security_group_rules
    #   Information about the security group properties to update.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifySecurityGroupRulesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifySecurityGroupRulesResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_security_group_rules({
    #     group_id: "SecurityGroupId", # required
    #     security_group_rules: [ # required
    #       {
    #         security_group_rule_id: "SecurityGroupRuleId", # required
    #         security_group_rule: {
    #           ip_protocol: "String",
    #           from_port: 1,
    #           to_port: 1,
    #           cidr_ipv_4: "String",
    #           cidr_ipv_6: "String",
    #           prefix_list_id: "PrefixListResourceId",
    #           referenced_group_id: "SecurityGroupId",
    #           description: "String",
    #         },
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifySecurityGroupRules AWS API Documentation
    #
    # @overload modify_security_group_rules(params = {})
    # @param [Hash] params ({})
    def modify_security_group_rules(params = {}, options = {})
      req = build_request(:modify_security_group_rules, params)
      req.send_request(options)
    end

    # Adds or removes permission settings for the specified snapshot. You
    # may add or remove specified Amazon Web Services account IDs from a
    # snapshot's list of create volume permissions, but you cannot do both
    # in a single operation. If you need to both add and remove account IDs
    # for a snapshot, you must use multiple operations. You can make up to
    # 500 modifications to a snapshot in a single operation.
    #
    # Encrypted snapshots and snapshots with Amazon Web Services Marketplace
    # product codes cannot be made public. Snapshots encrypted with your
    # default KMS key cannot be shared with other accounts.
    #
    # For more information about modifying snapshot permissions, see [Share
    # a snapshot][1] in the *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-modifying-snapshot-permissions.html
    #
    # @option params [String] :attribute
    #   The snapshot attribute to modify. Only volume creation permissions can
    #   be modified.
    #
    # @option params [Types::CreateVolumePermissionModifications] :create_volume_permission
    #   A JSON representation of the snapshot attribute modification.
    #
    # @option params [Array<String>] :group_names
    #   The group to modify for the snapshot.
    #
    # @option params [String] :operation_type
    #   The type of operation to perform to the attribute.
    #
    # @option params [required, String] :snapshot_id
    #   The ID of the snapshot.
    #
    # @option params [Array<String>] :user_ids
    #   The account ID to modify for the snapshot.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To modify a snapshot attribute
    #
    #   # This example modifies snapshot ``snap-1234567890abcdef0`` to remove the create volume permission for a user with the
    #   # account ID ``123456789012``. If the command succeeds, no output is returned.
    #
    #   resp = client.modify_snapshot_attribute({
    #     attribute: "createVolumePermission", 
    #     operation_type: "remove", 
    #     snapshot_id: "snap-1234567890abcdef0", 
    #     user_ids: [
    #       "123456789012", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Example: To make a snapshot public
    #
    #   # This example makes the snapshot ``snap-1234567890abcdef0`` public.
    #
    #   resp = client.modify_snapshot_attribute({
    #     attribute: "createVolumePermission", 
    #     group_names: [
    #       "all", 
    #     ], 
    #     operation_type: "add", 
    #     snapshot_id: "snap-1234567890abcdef0", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_snapshot_attribute({
    #     attribute: "productCodes", # accepts productCodes, createVolumePermission
    #     create_volume_permission: {
    #       add: [
    #         {
    #           user_id: "String",
    #           group: "all", # accepts all
    #         },
    #       ],
    #       remove: [
    #         {
    #           user_id: "String",
    #           group: "all", # accepts all
    #         },
    #       ],
    #     },
    #     group_names: ["SecurityGroupName"],
    #     operation_type: "add", # accepts add, remove
    #     snapshot_id: "SnapshotId", # required
    #     user_ids: ["String"],
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifySnapshotAttribute AWS API Documentation
    #
    # @overload modify_snapshot_attribute(params = {})
    # @param [Hash] params ({})
    def modify_snapshot_attribute(params = {}, options = {})
      req = build_request(:modify_snapshot_attribute, params)
      req.send_request(options)
    end

    # Archives an Amazon EBS snapshot. When you archive a snapshot, it is
    # converted to a full snapshot that includes all of the blocks of data
    # that were written to the volume at the time the snapshot was created,
    # and moved from the standard tier to the archive tier. For more
    # information, see [Archive Amazon EBS snapshots][1] in the *Amazon EBS
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/snapshot-archive.html
    #
    # @option params [required, String] :snapshot_id
    #   The ID of the snapshot.
    #
    # @option params [String] :storage_tier
    #   The name of the storage tier. You must specify `archive`.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifySnapshotTierResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifySnapshotTierResult#snapshot_id #snapshot_id} => String
    #   * {Types::ModifySnapshotTierResult#tiering_start_time #tiering_start_time} => Time
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_snapshot_tier({
    #     snapshot_id: "SnapshotId", # required
    #     storage_tier: "archive", # accepts archive
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.snapshot_id #=> String
    #   resp.tiering_start_time #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifySnapshotTier AWS API Documentation
    #
    # @overload modify_snapshot_tier(params = {})
    # @param [Hash] params ({})
    def modify_snapshot_tier(params = {}, options = {})
      req = build_request(:modify_snapshot_tier, params)
      req.send_request(options)
    end

    # Modifies the specified Spot Fleet request.
    #
    # You can only modify a Spot Fleet request of type `maintain`.
    #
    # While the Spot Fleet request is being modified, it is in the
    # `modifying` state.
    #
    # To scale up your Spot Fleet, increase its target capacity. The Spot
    # Fleet launches the additional Spot Instances according to the
    # allocation strategy for the Spot Fleet request. If the allocation
    # strategy is `lowestPrice`, the Spot Fleet launches instances using the
    # Spot Instance pool with the lowest price. If the allocation strategy
    # is `diversified`, the Spot Fleet distributes the instances across the
    # Spot Instance pools. If the allocation strategy is
    # `capacityOptimized`, Spot Fleet launches instances from Spot Instance
    # pools with optimal capacity for the number of instances that are
    # launching.
    #
    # To scale down your Spot Fleet, decrease its target capacity. First,
    # the Spot Fleet cancels any open requests that exceed the new target
    # capacity. You can request that the Spot Fleet terminate Spot Instances
    # until the size of the fleet no longer exceeds the new target capacity.
    # If the allocation strategy is `lowestPrice`, the Spot Fleet terminates
    # the instances with the highest price per unit. If the allocation
    # strategy is `capacityOptimized`, the Spot Fleet terminates the
    # instances in the Spot Instance pools that have the least available
    # Spot Instance capacity. If the allocation strategy is `diversified`,
    # the Spot Fleet terminates instances across the Spot Instance pools.
    # Alternatively, you can request that the Spot Fleet keep the fleet at
    # its current size, but not replace any Spot Instances that are
    # interrupted or that you terminate manually.
    #
    # If you are finished with your Spot Fleet for now, but will use it
    # again later, you can set the target capacity to 0.
    #
    # @option params [Array<Types::LaunchTemplateConfig>] :launch_template_configs
    #   The launch template and overrides. You can only use this parameter if
    #   you specified a launch template (`LaunchTemplateConfigs`) in your Spot
    #   Fleet request. If you specified `LaunchSpecifications` in your Spot
    #   Fleet request, then omit this parameter.
    #
    # @option params [Integer] :on_demand_target_capacity
    #   The number of On-Demand Instances in the fleet.
    #
    # @option params [String] :context
    #   Reserved.
    #
    # @option params [required, String] :spot_fleet_request_id
    #   The ID of the Spot Fleet request.
    #
    # @option params [Integer] :target_capacity
    #   The size of the fleet.
    #
    # @option params [String] :excess_capacity_termination_policy
    #   Indicates whether running instances should be terminated if the target
    #   capacity of the Spot Fleet request is decreased below the current size
    #   of the Spot Fleet.
    #
    #   Supported only for fleets of type `maintain`.
    #
    # @return [Types::ModifySpotFleetRequestResponse] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifySpotFleetRequestResponse#return #return} => Boolean
    #
    #
    # @example Example: To increase the target capacity of a Spot fleet request
    #
    #   # This example increases the target capacity of the specified Spot fleet request.
    #
    #   resp = client.modify_spot_fleet_request({
    #     spot_fleet_request_id: "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #     target_capacity: 20, 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     return: true, 
    #   }
    #
    # @example Example: To decrease the target capacity of a Spot fleet request
    #
    #   # This example decreases the target capacity of the specified Spot fleet request without terminating any Spot Instances as
    #   # a result.
    #
    #   resp = client.modify_spot_fleet_request({
    #     excess_capacity_termination_policy: "NoTermination ", 
    #     spot_fleet_request_id: "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #     target_capacity: 10, 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     return: true, 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_spot_fleet_request({
    #     launch_template_configs: [
    #       {
    #         launch_template_specification: {
    #           launch_template_id: "String",
    #           launch_template_name: "LaunchTemplateName",
    #           version: "String",
    #         },
    #         overrides: [
    #           {
    #             instance_type: "a1.medium", # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #             spot_price: "String",
    #             subnet_id: "SubnetId",
    #             availability_zone: "String",
    #             weighted_capacity: 1.0,
    #             priority: 1.0,
    #             instance_requirements: {
    #               v_cpu_count: {
    #                 min: 1,
    #                 max: 1,
    #               },
    #               memory_mi_b: {
    #                 min: 1,
    #                 max: 1,
    #               },
    #               cpu_manufacturers: ["intel"], # accepts intel, amd, amazon-web-services, apple
    #               memory_gi_b_per_v_cpu: {
    #                 min: 1.0,
    #                 max: 1.0,
    #               },
    #               excluded_instance_types: ["ExcludedInstanceType"],
    #               instance_generations: ["current"], # accepts current, previous
    #               spot_max_price_percentage_over_lowest_price: 1,
    #               on_demand_max_price_percentage_over_lowest_price: 1,
    #               bare_metal: "included", # accepts included, required, excluded
    #               burstable_performance: "included", # accepts included, required, excluded
    #               require_hibernate_support: false,
    #               network_interface_count: {
    #                 min: 1,
    #                 max: 1,
    #               },
    #               local_storage: "included", # accepts included, required, excluded
    #               local_storage_types: ["hdd"], # accepts hdd, ssd
    #               total_local_storage_gb: {
    #                 min: 1.0,
    #                 max: 1.0,
    #               },
    #               baseline_ebs_bandwidth_mbps: {
    #                 min: 1,
    #                 max: 1,
    #               },
    #               accelerator_types: ["gpu"], # accepts gpu, fpga, inference
    #               accelerator_count: {
    #                 min: 1,
    #                 max: 1,
    #               },
    #               accelerator_manufacturers: ["amazon-web-services"], # accepts amazon-web-services, amd, nvidia, xilinx, habana
    #               accelerator_names: ["a100"], # accepts a100, inferentia, k520, k80, m60, radeon-pro-v520, t4, vu9p, v100, a10g, h100, t4g
    #               accelerator_total_memory_mi_b: {
    #                 min: 1,
    #                 max: 1,
    #               },
    #               network_bandwidth_gbps: {
    #                 min: 1.0,
    #                 max: 1.0,
    #               },
    #               allowed_instance_types: ["AllowedInstanceType"],
    #               max_spot_price_as_percentage_of_optimal_on_demand_price: 1,
    #               baseline_performance_factors: {
    #                 cpu: {
    #                   references: [
    #                     {
    #                       instance_family: "String",
    #                     },
    #                   ],
    #                 },
    #               },
    #             },
    #           },
    #         ],
    #       },
    #     ],
    #     on_demand_target_capacity: 1,
    #     context: "String",
    #     spot_fleet_request_id: "SpotFleetRequestId", # required
    #     target_capacity: 1,
    #     excess_capacity_termination_policy: "noTermination", # accepts noTermination, default
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifySpotFleetRequest AWS API Documentation
    #
    # @overload modify_spot_fleet_request(params = {})
    # @param [Hash] params ({})
    def modify_spot_fleet_request(params = {}, options = {})
      req = build_request(:modify_spot_fleet_request, params)
      req.send_request(options)
    end

    # Modifies a subnet attribute. You can only modify one attribute at a
    # time.
    #
    # Use this action to modify subnets on Amazon Web Services Outposts.
    #
    # * To modify a subnet on an Outpost rack, set both
    #   `MapCustomerOwnedIpOnLaunch` and `CustomerOwnedIpv4Pool`. These two
    #   parameters act as a single attribute.
    #
    # * To modify a subnet on an Outpost server, set either
    #   `EnableLniAtDeviceIndex` or `DisableLniAtDeviceIndex`.
    #
    # For more information about Amazon Web Services Outposts, see the
    # following:
    #
    # * [Outpost servers][1]
    #
    # * [Outpost racks][2]
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/outposts/latest/userguide/how-servers-work.html
    # [2]: https://docs.aws.amazon.com/outposts/latest/userguide/how-racks-work.html
    #
    # @option params [Types::AttributeBooleanValue] :assign_ipv_6_address_on_creation
    #   Specify `true` to indicate that network interfaces created in the
    #   specified subnet should be assigned an IPv6 address. This includes a
    #   network interface that's created when launching an instance into the
    #   subnet (the instance therefore receives an IPv6 address).
    #
    #   If you enable the IPv6 addressing feature for your subnet, your
    #   network interface or instance only receives an IPv6 address if it's
    #   created using version `2016-11-15` or later of the Amazon EC2 API.
    #
    # @option params [Types::AttributeBooleanValue] :map_public_ip_on_launch
    #   Specify `true` to indicate that network interfaces attached to
    #   instances created in the specified subnet should be assigned a public
    #   IPv4 address.
    #
    #   Amazon Web Services charges for all public IPv4 addresses, including
    #   public IPv4 addresses associated with running instances and Elastic IP
    #   addresses. For more information, see the *Public IPv4 Address* tab on
    #   the [Amazon VPC pricing page][1].
    #
    #
    #
    #   [1]: http://aws.amazon.com/vpc/pricing/
    #
    # @option params [required, String] :subnet_id
    #   The ID of the subnet.
    #
    # @option params [Types::AttributeBooleanValue] :map_customer_owned_ip_on_launch
    #   Specify `true` to indicate that network interfaces attached to
    #   instances created in the specified subnet should be assigned a
    #   customer-owned IPv4 address.
    #
    #   When this value is `true`, you must specify the customer-owned IP pool
    #   using `CustomerOwnedIpv4Pool`.
    #
    # @option params [String] :customer_owned_ipv_4_pool
    #   The customer-owned IPv4 address pool associated with the subnet.
    #
    #   You must set this value when you specify `true` for
    #   `MapCustomerOwnedIpOnLaunch`.
    #
    # @option params [Types::AttributeBooleanValue] :enable_dns_64
    #   Indicates whether DNS queries made to the Amazon-provided DNS Resolver
    #   in this subnet should return synthetic IPv6 addresses for IPv4-only
    #   destinations.
    #
    #   You must first configure a NAT gateway in a public subnet (separate
    #   from the subnet containing the IPv6-only workloads). For example, the
    #   subnet containing the NAT gateway should have a `0.0.0.0/0` route
    #   pointing to the internet gateway. For more information, see [Configure
    #   DNS64 and NAT64][1] in the *Amazon VPC User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/vpc/latest/userguide/nat-gateway-nat64-dns64.html#nat-gateway-nat64-dns64-walkthrough
    #
    # @option params [String] :private_dns_hostname_type_on_launch
    #   The type of hostname to assign to instances in the subnet at launch.
    #   For IPv4-only and dual-stack (IPv4 and IPv6) subnets, an instance DNS
    #   name can be based on the instance IPv4 address (ip-name) or the
    #   instance ID (resource-name). For IPv6 only subnets, an instance DNS
    #   name must be based on the instance ID (resource-name).
    #
    # @option params [Types::AttributeBooleanValue] :enable_resource_name_dns_a_record_on_launch
    #   Indicates whether to respond to DNS queries for instance hostnames
    #   with DNS A records.
    #
    # @option params [Types::AttributeBooleanValue] :enable_resource_name_dns_aaaa_record_on_launch
    #   Indicates whether to respond to DNS queries for instance hostnames
    #   with DNS AAAA records.
    #
    # @option params [Integer] :enable_lni_at_device_index
    #   Indicates the device position for local network interfaces in this
    #   subnet. For example, `1` indicates local network interfaces in this
    #   subnet are the secondary network interface (eth1). A local network
    #   interface cannot be the primary network interface (eth0).
    #
    # @option params [Types::AttributeBooleanValue] :disable_lni_at_device_index
    #   Specify `true` to indicate that local network interfaces at the
    #   current position should be disabled.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To change a subnet's public IP addressing behavior
    #
    #   # This example modifies the specified subnet so that all instances launched into this subnet are assigned a public IP
    #   # address.
    #
    #   resp = client.modify_subnet_attribute({
    #     map_public_ip_on_launch: {
    #       value: true, 
    #     }, 
    #     subnet_id: "subnet-1a2b3c4d", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_subnet_attribute({
    #     assign_ipv_6_address_on_creation: {
    #       value: false,
    #     },
    #     map_public_ip_on_launch: {
    #       value: false,
    #     },
    #     subnet_id: "SubnetId", # required
    #     map_customer_owned_ip_on_launch: {
    #       value: false,
    #     },
    #     customer_owned_ipv_4_pool: "CoipPoolId",
    #     enable_dns_64: {
    #       value: false,
    #     },
    #     private_dns_hostname_type_on_launch: "ip-name", # accepts ip-name, resource-name
    #     enable_resource_name_dns_a_record_on_launch: {
    #       value: false,
    #     },
    #     enable_resource_name_dns_aaaa_record_on_launch: {
    #       value: false,
    #     },
    #     enable_lni_at_device_index: 1,
    #     disable_lni_at_device_index: {
    #       value: false,
    #     },
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifySubnetAttribute AWS API Documentation
    #
    # @overload modify_subnet_attribute(params = {})
    # @param [Hash] params ({})
    def modify_subnet_attribute(params = {}, options = {})
      req = build_request(:modify_subnet_attribute, params)
      req.send_request(options)
    end

    # Allows or restricts mirroring network services.
    #
    # By default, Amazon DNS network services are not eligible for Traffic
    # Mirror. Use `AddNetworkServices` to add network services to a Traffic
    # Mirror filter. When a network service is added to the Traffic Mirror
    # filter, all traffic related to that network service will be mirrored.
    # When you no longer want to mirror network services, use
    # `RemoveNetworkServices` to remove the network services from the
    # Traffic Mirror filter.
    #
    # @option params [required, String] :traffic_mirror_filter_id
    #   The ID of the Traffic Mirror filter.
    #
    # @option params [Array<String>] :add_network_services
    #   The network service, for example Amazon DNS, that you want to mirror.
    #
    # @option params [Array<String>] :remove_network_services
    #   The network service, for example Amazon DNS, that you no longer want
    #   to mirror.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyTrafficMirrorFilterNetworkServicesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyTrafficMirrorFilterNetworkServicesResult#traffic_mirror_filter #traffic_mirror_filter} => Types::TrafficMirrorFilter
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_traffic_mirror_filter_network_services({
    #     traffic_mirror_filter_id: "TrafficMirrorFilterId", # required
    #     add_network_services: ["amazon-dns"], # accepts amazon-dns
    #     remove_network_services: ["amazon-dns"], # accepts amazon-dns
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_filter.traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_filter.ingress_filter_rules #=> Array
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].traffic_mirror_filter_rule_id #=> String
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].traffic_direction #=> String, one of "ingress", "egress"
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].rule_number #=> Integer
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].rule_action #=> String, one of "accept", "reject"
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].protocol #=> Integer
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].destination_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].destination_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].source_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].source_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].destination_cidr_block #=> String
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].source_cidr_block #=> String
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].description #=> String
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].tags #=> Array
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].tags[0].key #=> String
    #   resp.traffic_mirror_filter.ingress_filter_rules[0].tags[0].value #=> String
    #   resp.traffic_mirror_filter.egress_filter_rules #=> Array
    #   resp.traffic_mirror_filter.egress_filter_rules[0].traffic_mirror_filter_rule_id #=> String
    #   resp.traffic_mirror_filter.egress_filter_rules[0].traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_filter.egress_filter_rules[0].traffic_direction #=> String, one of "ingress", "egress"
    #   resp.traffic_mirror_filter.egress_filter_rules[0].rule_number #=> Integer
    #   resp.traffic_mirror_filter.egress_filter_rules[0].rule_action #=> String, one of "accept", "reject"
    #   resp.traffic_mirror_filter.egress_filter_rules[0].protocol #=> Integer
    #   resp.traffic_mirror_filter.egress_filter_rules[0].destination_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filter.egress_filter_rules[0].destination_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filter.egress_filter_rules[0].source_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filter.egress_filter_rules[0].source_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filter.egress_filter_rules[0].destination_cidr_block #=> String
    #   resp.traffic_mirror_filter.egress_filter_rules[0].source_cidr_block #=> String
    #   resp.traffic_mirror_filter.egress_filter_rules[0].description #=> String
    #   resp.traffic_mirror_filter.egress_filter_rules[0].tags #=> Array
    #   resp.traffic_mirror_filter.egress_filter_rules[0].tags[0].key #=> String
    #   resp.traffic_mirror_filter.egress_filter_rules[0].tags[0].value #=> String
    #   resp.traffic_mirror_filter.network_services #=> Array
    #   resp.traffic_mirror_filter.network_services[0] #=> String, one of "amazon-dns"
    #   resp.traffic_mirror_filter.description #=> String
    #   resp.traffic_mirror_filter.tags #=> Array
    #   resp.traffic_mirror_filter.tags[0].key #=> String
    #   resp.traffic_mirror_filter.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyTrafficMirrorFilterNetworkServices AWS API Documentation
    #
    # @overload modify_traffic_mirror_filter_network_services(params = {})
    # @param [Hash] params ({})
    def modify_traffic_mirror_filter_network_services(params = {}, options = {})
      req = build_request(:modify_traffic_mirror_filter_network_services, params)
      req.send_request(options)
    end

    # Modifies the specified Traffic Mirror rule.
    #
    # `DestinationCidrBlock` and `SourceCidrBlock` must both be an IPv4
    # range or an IPv6 range.
    #
    # @option params [required, String] :traffic_mirror_filter_rule_id
    #   The ID of the Traffic Mirror rule.
    #
    # @option params [String] :traffic_direction
    #   The type of traffic to assign to the rule.
    #
    # @option params [Integer] :rule_number
    #   The number of the Traffic Mirror rule. This number must be unique for
    #   each Traffic Mirror rule in a given direction. The rules are processed
    #   in ascending order by rule number.
    #
    # @option params [String] :rule_action
    #   The action to assign to the rule.
    #
    # @option params [Types::TrafficMirrorPortRangeRequest] :destination_port_range
    #   The destination ports that are associated with the Traffic Mirror
    #   rule.
    #
    # @option params [Types::TrafficMirrorPortRangeRequest] :source_port_range
    #   The port range to assign to the Traffic Mirror rule.
    #
    # @option params [Integer] :protocol
    #   The protocol, for example TCP, to assign to the Traffic Mirror rule.
    #
    # @option params [String] :destination_cidr_block
    #   The destination CIDR block to assign to the Traffic Mirror rule.
    #
    # @option params [String] :source_cidr_block
    #   The source CIDR block to assign to the Traffic Mirror rule.
    #
    # @option params [String] :description
    #   The description to assign to the Traffic Mirror rule.
    #
    # @option params [Array<String>] :remove_fields
    #   The properties that you want to remove from the Traffic Mirror filter
    #   rule.
    #
    #   When you remove a property from a Traffic Mirror filter rule, the
    #   property is set to the default.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyTrafficMirrorFilterRuleResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyTrafficMirrorFilterRuleResult#traffic_mirror_filter_rule #traffic_mirror_filter_rule} => Types::TrafficMirrorFilterRule
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_traffic_mirror_filter_rule({
    #     traffic_mirror_filter_rule_id: "TrafficMirrorFilterRuleIdWithResolver", # required
    #     traffic_direction: "ingress", # accepts ingress, egress
    #     rule_number: 1,
    #     rule_action: "accept", # accepts accept, reject
    #     destination_port_range: {
    #       from_port: 1,
    #       to_port: 1,
    #     },
    #     source_port_range: {
    #       from_port: 1,
    #       to_port: 1,
    #     },
    #     protocol: 1,
    #     destination_cidr_block: "String",
    #     source_cidr_block: "String",
    #     description: "String",
    #     remove_fields: ["destination-port-range"], # accepts destination-port-range, source-port-range, protocol, description
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_filter_rule.traffic_mirror_filter_rule_id #=> String
    #   resp.traffic_mirror_filter_rule.traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_filter_rule.traffic_direction #=> String, one of "ingress", "egress"
    #   resp.traffic_mirror_filter_rule.rule_number #=> Integer
    #   resp.traffic_mirror_filter_rule.rule_action #=> String, one of "accept", "reject"
    #   resp.traffic_mirror_filter_rule.protocol #=> Integer
    #   resp.traffic_mirror_filter_rule.destination_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filter_rule.destination_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filter_rule.source_port_range.from_port #=> Integer
    #   resp.traffic_mirror_filter_rule.source_port_range.to_port #=> Integer
    #   resp.traffic_mirror_filter_rule.destination_cidr_block #=> String
    #   resp.traffic_mirror_filter_rule.source_cidr_block #=> String
    #   resp.traffic_mirror_filter_rule.description #=> String
    #   resp.traffic_mirror_filter_rule.tags #=> Array
    #   resp.traffic_mirror_filter_rule.tags[0].key #=> String
    #   resp.traffic_mirror_filter_rule.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyTrafficMirrorFilterRule AWS API Documentation
    #
    # @overload modify_traffic_mirror_filter_rule(params = {})
    # @param [Hash] params ({})
    def modify_traffic_mirror_filter_rule(params = {}, options = {})
      req = build_request(:modify_traffic_mirror_filter_rule, params)
      req.send_request(options)
    end

    # Modifies a Traffic Mirror session.
    #
    # @option params [required, String] :traffic_mirror_session_id
    #   The ID of the Traffic Mirror session.
    #
    # @option params [String] :traffic_mirror_target_id
    #   The Traffic Mirror target. The target must be in the same VPC as the
    #   source, or have a VPC peering connection with the source.
    #
    # @option params [String] :traffic_mirror_filter_id
    #   The ID of the Traffic Mirror filter.
    #
    # @option params [Integer] :packet_length
    #   The number of bytes in each packet to mirror. These are bytes after
    #   the VXLAN header. To mirror a subset, set this to the length (in
    #   bytes) to mirror. For example, if you set this value to 100, then the
    #   first 100 bytes that meet the filter criteria are copied to the
    #   target. Do not specify this parameter when you want to mirror the
    #   entire packet.
    #
    #   For sessions with Network Load Balancer (NLB) traffic mirror targets,
    #   the default `PacketLength` will be set to 8500. Valid values are
    #   1-8500. Setting a `PacketLength` greater than 8500 will result in an
    #   error response.
    #
    # @option params [Integer] :session_number
    #   The session number determines the order in which sessions are
    #   evaluated when an interface is used by multiple sessions. The first
    #   session with a matching filter is the one that mirrors the packets.
    #
    #   Valid values are 1-32766.
    #
    # @option params [Integer] :virtual_network_id
    #   The virtual network ID of the Traffic Mirror session.
    #
    # @option params [String] :description
    #   The description to assign to the Traffic Mirror session.
    #
    # @option params [Array<String>] :remove_fields
    #   The properties that you want to remove from the Traffic Mirror
    #   session.
    #
    #   When you remove a property from a Traffic Mirror session, the property
    #   is set to the default.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyTrafficMirrorSessionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyTrafficMirrorSessionResult#traffic_mirror_session #traffic_mirror_session} => Types::TrafficMirrorSession
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_traffic_mirror_session({
    #     traffic_mirror_session_id: "TrafficMirrorSessionId", # required
    #     traffic_mirror_target_id: "TrafficMirrorTargetId",
    #     traffic_mirror_filter_id: "TrafficMirrorFilterId",
    #     packet_length: 1,
    #     session_number: 1,
    #     virtual_network_id: 1,
    #     description: "String",
    #     remove_fields: ["packet-length"], # accepts packet-length, description, virtual-network-id
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.traffic_mirror_session.traffic_mirror_session_id #=> String
    #   resp.traffic_mirror_session.traffic_mirror_target_id #=> String
    #   resp.traffic_mirror_session.traffic_mirror_filter_id #=> String
    #   resp.traffic_mirror_session.network_interface_id #=> String
    #   resp.traffic_mirror_session.owner_id #=> String
    #   resp.traffic_mirror_session.packet_length #=> Integer
    #   resp.traffic_mirror_session.session_number #=> Integer
    #   resp.traffic_mirror_session.virtual_network_id #=> Integer
    #   resp.traffic_mirror_session.description #=> String
    #   resp.traffic_mirror_session.tags #=> Array
    #   resp.traffic_mirror_session.tags[0].key #=> String
    #   resp.traffic_mirror_session.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyTrafficMirrorSession AWS API Documentation
    #
    # @overload modify_traffic_mirror_session(params = {})
    # @param [Hash] params ({})
    def modify_traffic_mirror_session(params = {}, options = {})
      req = build_request(:modify_traffic_mirror_session, params)
      req.send_request(options)
    end

    # Modifies the specified transit gateway. When you modify a transit
    # gateway, the modified options are applied to new transit gateway
    # attachments only. Your existing transit gateway attachments are not
    # modified.
    #
    # @option params [required, String] :transit_gateway_id
    #   The ID of the transit gateway.
    #
    # @option params [String] :description
    #   The description for the transit gateway.
    #
    # @option params [Types::ModifyTransitGatewayOptions] :options
    #   The options to modify.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyTransitGatewayResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyTransitGatewayResult#transit_gateway #transit_gateway} => Types::TransitGateway
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_transit_gateway({
    #     transit_gateway_id: "TransitGatewayId", # required
    #     description: "String",
    #     options: {
    #       add_transit_gateway_cidr_blocks: ["String"],
    #       remove_transit_gateway_cidr_blocks: ["String"],
    #       vpn_ecmp_support: "enable", # accepts enable, disable
    #       dns_support: "enable", # accepts enable, disable
    #       security_group_referencing_support: "enable", # accepts enable, disable
    #       auto_accept_shared_attachments: "enable", # accepts enable, disable
    #       default_route_table_association: "enable", # accepts enable, disable
    #       association_default_route_table_id: "TransitGatewayRouteTableId",
    #       default_route_table_propagation: "enable", # accepts enable, disable
    #       propagation_default_route_table_id: "TransitGatewayRouteTableId",
    #       amazon_side_asn: 1,
    #     },
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway.transit_gateway_id #=> String
    #   resp.transit_gateway.transit_gateway_arn #=> String
    #   resp.transit_gateway.state #=> String, one of "pending", "available", "modifying", "deleting", "deleted"
    #   resp.transit_gateway.owner_id #=> String
    #   resp.transit_gateway.description #=> String
    #   resp.transit_gateway.creation_time #=> Time
    #   resp.transit_gateway.options.amazon_side_asn #=> Integer
    #   resp.transit_gateway.options.transit_gateway_cidr_blocks #=> Array
    #   resp.transit_gateway.options.transit_gateway_cidr_blocks[0] #=> String
    #   resp.transit_gateway.options.auto_accept_shared_attachments #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.default_route_table_association #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.association_default_route_table_id #=> String
    #   resp.transit_gateway.options.default_route_table_propagation #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.propagation_default_route_table_id #=> String
    #   resp.transit_gateway.options.vpn_ecmp_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.dns_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.security_group_referencing_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway.options.multicast_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway.tags #=> Array
    #   resp.transit_gateway.tags[0].key #=> String
    #   resp.transit_gateway.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyTransitGateway AWS API Documentation
    #
    # @overload modify_transit_gateway(params = {})
    # @param [Hash] params ({})
    def modify_transit_gateway(params = {}, options = {})
      req = build_request(:modify_transit_gateway, params)
      req.send_request(options)
    end

    # Modifies a reference (route) to a prefix list in a specified transit
    # gateway route table.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the transit gateway route table.
    #
    # @option params [required, String] :prefix_list_id
    #   The ID of the prefix list.
    #
    # @option params [String] :transit_gateway_attachment_id
    #   The ID of the attachment to which traffic is routed.
    #
    # @option params [Boolean] :blackhole
    #   Indicates whether to drop traffic that matches this route.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyTransitGatewayPrefixListReferenceResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyTransitGatewayPrefixListReferenceResult#transit_gateway_prefix_list_reference #transit_gateway_prefix_list_reference} => Types::TransitGatewayPrefixListReference
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_transit_gateway_prefix_list_reference({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     prefix_list_id: "PrefixListResourceId", # required
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId",
    #     blackhole: false,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_prefix_list_reference.transit_gateway_route_table_id #=> String
    #   resp.transit_gateway_prefix_list_reference.prefix_list_id #=> String
    #   resp.transit_gateway_prefix_list_reference.prefix_list_owner_id #=> String
    #   resp.transit_gateway_prefix_list_reference.state #=> String, one of "pending", "available", "modifying", "deleting"
    #   resp.transit_gateway_prefix_list_reference.blackhole #=> Boolean
    #   resp.transit_gateway_prefix_list_reference.transit_gateway_attachment.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_prefix_list_reference.transit_gateway_attachment.resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.transit_gateway_prefix_list_reference.transit_gateway_attachment.resource_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyTransitGatewayPrefixListReference AWS API Documentation
    #
    # @overload modify_transit_gateway_prefix_list_reference(params = {})
    # @param [Hash] params ({})
    def modify_transit_gateway_prefix_list_reference(params = {}, options = {})
      req = build_request(:modify_transit_gateway_prefix_list_reference, params)
      req.send_request(options)
    end

    # Modifies the specified VPC attachment.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the attachment.
    #
    # @option params [Array<String>] :add_subnet_ids
    #   The IDs of one or more subnets to add. You can specify at most one
    #   subnet per Availability Zone.
    #
    # @option params [Array<String>] :remove_subnet_ids
    #   The IDs of one or more subnets to remove.
    #
    # @option params [Types::ModifyTransitGatewayVpcAttachmentRequestOptions] :options
    #   The new VPC attachment options.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyTransitGatewayVpcAttachmentResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyTransitGatewayVpcAttachmentResult#transit_gateway_vpc_attachment #transit_gateway_vpc_attachment} => Types::TransitGatewayVpcAttachment
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_transit_gateway_vpc_attachment({
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     add_subnet_ids: ["SubnetId"],
    #     remove_subnet_ids: ["SubnetId"],
    #     options: {
    #       dns_support: "enable", # accepts enable, disable
    #       security_group_referencing_support: "enable", # accepts enable, disable
    #       ipv_6_support: "enable", # accepts enable, disable
    #       appliance_mode_support: "enable", # accepts enable, disable
    #     },
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_vpc_attachment.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_vpc_attachment.transit_gateway_id #=> String
    #   resp.transit_gateway_vpc_attachment.vpc_id #=> String
    #   resp.transit_gateway_vpc_attachment.vpc_owner_id #=> String
    #   resp.transit_gateway_vpc_attachment.state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_vpc_attachment.subnet_ids #=> Array
    #   resp.transit_gateway_vpc_attachment.subnet_ids[0] #=> String
    #   resp.transit_gateway_vpc_attachment.creation_time #=> Time
    #   resp.transit_gateway_vpc_attachment.options.dns_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.security_group_referencing_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.ipv_6_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.appliance_mode_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.tags #=> Array
    #   resp.transit_gateway_vpc_attachment.tags[0].key #=> String
    #   resp.transit_gateway_vpc_attachment.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyTransitGatewayVpcAttachment AWS API Documentation
    #
    # @overload modify_transit_gateway_vpc_attachment(params = {})
    # @param [Hash] params ({})
    def modify_transit_gateway_vpc_attachment(params = {}, options = {})
      req = build_request(:modify_transit_gateway_vpc_attachment, params)
      req.send_request(options)
    end

    # Modifies the configuration of the specified Amazon Web Services
    # Verified Access endpoint.
    #
    # @option params [required, String] :verified_access_endpoint_id
    #   The ID of the Verified Access endpoint.
    #
    # @option params [String] :verified_access_group_id
    #   The ID of the Verified Access group.
    #
    # @option params [Types::ModifyVerifiedAccessEndpointLoadBalancerOptions] :load_balancer_options
    #   The load balancer details if creating the Verified Access endpoint as
    #   `load-balancer`type.
    #
    # @option params [Types::ModifyVerifiedAccessEndpointEniOptions] :network_interface_options
    #   The network interface options.
    #
    # @option params [String] :description
    #   A description for the Verified Access endpoint.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Types::ModifyVerifiedAccessEndpointRdsOptions] :rds_options
    #   The RDS options.
    #
    # @option params [Types::ModifyVerifiedAccessEndpointCidrOptions] :cidr_options
    #   The CIDR options.
    #
    # @return [Types::ModifyVerifiedAccessEndpointResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVerifiedAccessEndpointResult#verified_access_endpoint #verified_access_endpoint} => Types::VerifiedAccessEndpoint
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_verified_access_endpoint({
    #     verified_access_endpoint_id: "VerifiedAccessEndpointId", # required
    #     verified_access_group_id: "VerifiedAccessGroupId",
    #     load_balancer_options: {
    #       subnet_ids: ["SubnetId"],
    #       protocol: "http", # accepts http, https, tcp
    #       port: 1,
    #       port_ranges: [
    #         {
    #           from_port: 1,
    #           to_port: 1,
    #         },
    #       ],
    #     },
    #     network_interface_options: {
    #       protocol: "http", # accepts http, https, tcp
    #       port: 1,
    #       port_ranges: [
    #         {
    #           from_port: 1,
    #           to_port: 1,
    #         },
    #       ],
    #     },
    #     description: "String",
    #     client_token: "String",
    #     dry_run: false,
    #     rds_options: {
    #       subnet_ids: ["SubnetId"],
    #       port: 1,
    #       rds_endpoint: "String",
    #     },
    #     cidr_options: {
    #       port_ranges: [
    #         {
    #           from_port: 1,
    #           to_port: 1,
    #         },
    #       ],
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_endpoint.verified_access_instance_id #=> String
    #   resp.verified_access_endpoint.verified_access_group_id #=> String
    #   resp.verified_access_endpoint.verified_access_endpoint_id #=> String
    #   resp.verified_access_endpoint.application_domain #=> String
    #   resp.verified_access_endpoint.endpoint_type #=> String, one of "load-balancer", "network-interface", "rds", "cidr"
    #   resp.verified_access_endpoint.attachment_type #=> String, one of "vpc"
    #   resp.verified_access_endpoint.domain_certificate_arn #=> String
    #   resp.verified_access_endpoint.endpoint_domain #=> String
    #   resp.verified_access_endpoint.device_validation_domain #=> String
    #   resp.verified_access_endpoint.security_group_ids #=> Array
    #   resp.verified_access_endpoint.security_group_ids[0] #=> String
    #   resp.verified_access_endpoint.load_balancer_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoint.load_balancer_options.port #=> Integer
    #   resp.verified_access_endpoint.load_balancer_options.load_balancer_arn #=> String
    #   resp.verified_access_endpoint.load_balancer_options.subnet_ids #=> Array
    #   resp.verified_access_endpoint.load_balancer_options.subnet_ids[0] #=> String
    #   resp.verified_access_endpoint.load_balancer_options.port_ranges #=> Array
    #   resp.verified_access_endpoint.load_balancer_options.port_ranges[0].from_port #=> Integer
    #   resp.verified_access_endpoint.load_balancer_options.port_ranges[0].to_port #=> Integer
    #   resp.verified_access_endpoint.network_interface_options.network_interface_id #=> String
    #   resp.verified_access_endpoint.network_interface_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoint.network_interface_options.port #=> Integer
    #   resp.verified_access_endpoint.network_interface_options.port_ranges #=> Array
    #   resp.verified_access_endpoint.network_interface_options.port_ranges[0].from_port #=> Integer
    #   resp.verified_access_endpoint.network_interface_options.port_ranges[0].to_port #=> Integer
    #   resp.verified_access_endpoint.status.code #=> String, one of "pending", "active", "updating", "deleting", "deleted"
    #   resp.verified_access_endpoint.status.message #=> String
    #   resp.verified_access_endpoint.description #=> String
    #   resp.verified_access_endpoint.creation_time #=> String
    #   resp.verified_access_endpoint.last_updated_time #=> String
    #   resp.verified_access_endpoint.deletion_time #=> String
    #   resp.verified_access_endpoint.tags #=> Array
    #   resp.verified_access_endpoint.tags[0].key #=> String
    #   resp.verified_access_endpoint.tags[0].value #=> String
    #   resp.verified_access_endpoint.sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.verified_access_endpoint.sse_specification.kms_key_arn #=> String
    #   resp.verified_access_endpoint.rds_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoint.rds_options.port #=> Integer
    #   resp.verified_access_endpoint.rds_options.rds_db_instance_arn #=> String
    #   resp.verified_access_endpoint.rds_options.rds_db_cluster_arn #=> String
    #   resp.verified_access_endpoint.rds_options.rds_db_proxy_arn #=> String
    #   resp.verified_access_endpoint.rds_options.rds_endpoint #=> String
    #   resp.verified_access_endpoint.rds_options.subnet_ids #=> Array
    #   resp.verified_access_endpoint.rds_options.subnet_ids[0] #=> String
    #   resp.verified_access_endpoint.cidr_options.cidr #=> String
    #   resp.verified_access_endpoint.cidr_options.port_ranges #=> Array
    #   resp.verified_access_endpoint.cidr_options.port_ranges[0].from_port #=> Integer
    #   resp.verified_access_endpoint.cidr_options.port_ranges[0].to_port #=> Integer
    #   resp.verified_access_endpoint.cidr_options.protocol #=> String, one of "http", "https", "tcp"
    #   resp.verified_access_endpoint.cidr_options.subnet_ids #=> Array
    #   resp.verified_access_endpoint.cidr_options.subnet_ids[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVerifiedAccessEndpoint AWS API Documentation
    #
    # @overload modify_verified_access_endpoint(params = {})
    # @param [Hash] params ({})
    def modify_verified_access_endpoint(params = {}, options = {})
      req = build_request(:modify_verified_access_endpoint, params)
      req.send_request(options)
    end

    # Modifies the specified Amazon Web Services Verified Access endpoint
    # policy.
    #
    # @option params [required, String] :verified_access_endpoint_id
    #   The ID of the Verified Access endpoint.
    #
    # @option params [Boolean] :policy_enabled
    #   The status of the Verified Access policy.
    #
    # @option params [String] :policy_document
    #   The Verified Access policy document.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Types::VerifiedAccessSseSpecificationRequest] :sse_specification
    #   The options for server side encryption.
    #
    # @return [Types::ModifyVerifiedAccessEndpointPolicyResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVerifiedAccessEndpointPolicyResult#policy_enabled #policy_enabled} => Boolean
    #   * {Types::ModifyVerifiedAccessEndpointPolicyResult#policy_document #policy_document} => String
    #   * {Types::ModifyVerifiedAccessEndpointPolicyResult#sse_specification #sse_specification} => Types::VerifiedAccessSseSpecificationResponse
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_verified_access_endpoint_policy({
    #     verified_access_endpoint_id: "VerifiedAccessEndpointId", # required
    #     policy_enabled: false,
    #     policy_document: "String",
    #     client_token: "String",
    #     dry_run: false,
    #     sse_specification: {
    #       customer_managed_key_enabled: false,
    #       kms_key_arn: "KmsKeyArn",
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.policy_enabled #=> Boolean
    #   resp.policy_document #=> String
    #   resp.sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.sse_specification.kms_key_arn #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVerifiedAccessEndpointPolicy AWS API Documentation
    #
    # @overload modify_verified_access_endpoint_policy(params = {})
    # @param [Hash] params ({})
    def modify_verified_access_endpoint_policy(params = {}, options = {})
      req = build_request(:modify_verified_access_endpoint_policy, params)
      req.send_request(options)
    end

    # Modifies the specified Amazon Web Services Verified Access group
    # configuration.
    #
    # @option params [required, String] :verified_access_group_id
    #   The ID of the Verified Access group.
    #
    # @option params [String] :verified_access_instance_id
    #   The ID of the Verified Access instance.
    #
    # @option params [String] :description
    #   A description for the Verified Access group.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyVerifiedAccessGroupResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVerifiedAccessGroupResult#verified_access_group #verified_access_group} => Types::VerifiedAccessGroup
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_verified_access_group({
    #     verified_access_group_id: "VerifiedAccessGroupId", # required
    #     verified_access_instance_id: "VerifiedAccessInstanceId",
    #     description: "String",
    #     client_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_group.verified_access_group_id #=> String
    #   resp.verified_access_group.verified_access_instance_id #=> String
    #   resp.verified_access_group.description #=> String
    #   resp.verified_access_group.owner #=> String
    #   resp.verified_access_group.verified_access_group_arn #=> String
    #   resp.verified_access_group.creation_time #=> String
    #   resp.verified_access_group.last_updated_time #=> String
    #   resp.verified_access_group.deletion_time #=> String
    #   resp.verified_access_group.tags #=> Array
    #   resp.verified_access_group.tags[0].key #=> String
    #   resp.verified_access_group.tags[0].value #=> String
    #   resp.verified_access_group.sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.verified_access_group.sse_specification.kms_key_arn #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVerifiedAccessGroup AWS API Documentation
    #
    # @overload modify_verified_access_group(params = {})
    # @param [Hash] params ({})
    def modify_verified_access_group(params = {}, options = {})
      req = build_request(:modify_verified_access_group, params)
      req.send_request(options)
    end

    # Modifies the specified Amazon Web Services Verified Access group
    # policy.
    #
    # @option params [required, String] :verified_access_group_id
    #   The ID of the Verified Access group.
    #
    # @option params [Boolean] :policy_enabled
    #   The status of the Verified Access policy.
    #
    # @option params [String] :policy_document
    #   The Verified Access policy document.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Types::VerifiedAccessSseSpecificationRequest] :sse_specification
    #   The options for server side encryption.
    #
    # @return [Types::ModifyVerifiedAccessGroupPolicyResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVerifiedAccessGroupPolicyResult#policy_enabled #policy_enabled} => Boolean
    #   * {Types::ModifyVerifiedAccessGroupPolicyResult#policy_document #policy_document} => String
    #   * {Types::ModifyVerifiedAccessGroupPolicyResult#sse_specification #sse_specification} => Types::VerifiedAccessSseSpecificationResponse
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_verified_access_group_policy({
    #     verified_access_group_id: "VerifiedAccessGroupId", # required
    #     policy_enabled: false,
    #     policy_document: "String",
    #     client_token: "String",
    #     dry_run: false,
    #     sse_specification: {
    #       customer_managed_key_enabled: false,
    #       kms_key_arn: "KmsKeyArn",
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.policy_enabled #=> Boolean
    #   resp.policy_document #=> String
    #   resp.sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.sse_specification.kms_key_arn #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVerifiedAccessGroupPolicy AWS API Documentation
    #
    # @overload modify_verified_access_group_policy(params = {})
    # @param [Hash] params ({})
    def modify_verified_access_group_policy(params = {}, options = {})
      req = build_request(:modify_verified_access_group_policy, params)
      req.send_request(options)
    end

    # Modifies the configuration of the specified Amazon Web Services
    # Verified Access instance.
    #
    # @option params [required, String] :verified_access_instance_id
    #   The ID of the Verified Access instance.
    #
    # @option params [String] :description
    #   A description for the Verified Access instance.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [String] :cidr_endpoints_custom_sub_domain
    #   The custom subdomain.
    #
    # @return [Types::ModifyVerifiedAccessInstanceResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVerifiedAccessInstanceResult#verified_access_instance #verified_access_instance} => Types::VerifiedAccessInstance
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_verified_access_instance({
    #     verified_access_instance_id: "VerifiedAccessInstanceId", # required
    #     description: "String",
    #     dry_run: false,
    #     client_token: "String",
    #     cidr_endpoints_custom_sub_domain: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_instance.verified_access_instance_id #=> String
    #   resp.verified_access_instance.description #=> String
    #   resp.verified_access_instance.verified_access_trust_providers #=> Array
    #   resp.verified_access_instance.verified_access_trust_providers[0].verified_access_trust_provider_id #=> String
    #   resp.verified_access_instance.verified_access_trust_providers[0].description #=> String
    #   resp.verified_access_instance.verified_access_trust_providers[0].trust_provider_type #=> String, one of "user", "device"
    #   resp.verified_access_instance.verified_access_trust_providers[0].user_trust_provider_type #=> String, one of "iam-identity-center", "oidc"
    #   resp.verified_access_instance.verified_access_trust_providers[0].device_trust_provider_type #=> String, one of "jamf", "crowdstrike", "jumpcloud"
    #   resp.verified_access_instance.creation_time #=> String
    #   resp.verified_access_instance.last_updated_time #=> String
    #   resp.verified_access_instance.tags #=> Array
    #   resp.verified_access_instance.tags[0].key #=> String
    #   resp.verified_access_instance.tags[0].value #=> String
    #   resp.verified_access_instance.fips_enabled #=> Boolean
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.sub_domain #=> String
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.nameservers #=> Array
    #   resp.verified_access_instance.cidr_endpoints_custom_sub_domain.nameservers[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVerifiedAccessInstance AWS API Documentation
    #
    # @overload modify_verified_access_instance(params = {})
    # @param [Hash] params ({})
    def modify_verified_access_instance(params = {}, options = {})
      req = build_request(:modify_verified_access_instance, params)
      req.send_request(options)
    end

    # Modifies the logging configuration for the specified Amazon Web
    # Services Verified Access instance.
    #
    # @option params [required, String] :verified_access_instance_id
    #   The ID of the Verified Access instance.
    #
    # @option params [required, Types::VerifiedAccessLogOptions] :access_logs
    #   The configuration options for Verified Access instances.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @return [Types::ModifyVerifiedAccessInstanceLoggingConfigurationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVerifiedAccessInstanceLoggingConfigurationResult#logging_configuration #logging_configuration} => Types::VerifiedAccessInstanceLoggingConfiguration
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_verified_access_instance_logging_configuration({
    #     verified_access_instance_id: "VerifiedAccessInstanceId", # required
    #     access_logs: { # required
    #       s3: {
    #         enabled: false, # required
    #         bucket_name: "String",
    #         prefix: "String",
    #         bucket_owner: "String",
    #       },
    #       cloud_watch_logs: {
    #         enabled: false, # required
    #         log_group: "String",
    #       },
    #       kinesis_data_firehose: {
    #         enabled: false, # required
    #         delivery_stream: "String",
    #       },
    #       log_version: "String",
    #       include_trust_context: false,
    #     },
    #     dry_run: false,
    #     client_token: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.logging_configuration.verified_access_instance_id #=> String
    #   resp.logging_configuration.access_logs.s3.enabled #=> Boolean
    #   resp.logging_configuration.access_logs.s3.delivery_status.code #=> String, one of "success", "failed"
    #   resp.logging_configuration.access_logs.s3.delivery_status.message #=> String
    #   resp.logging_configuration.access_logs.s3.bucket_name #=> String
    #   resp.logging_configuration.access_logs.s3.prefix #=> String
    #   resp.logging_configuration.access_logs.s3.bucket_owner #=> String
    #   resp.logging_configuration.access_logs.cloud_watch_logs.enabled #=> Boolean
    #   resp.logging_configuration.access_logs.cloud_watch_logs.delivery_status.code #=> String, one of "success", "failed"
    #   resp.logging_configuration.access_logs.cloud_watch_logs.delivery_status.message #=> String
    #   resp.logging_configuration.access_logs.cloud_watch_logs.log_group #=> String
    #   resp.logging_configuration.access_logs.kinesis_data_firehose.enabled #=> Boolean
    #   resp.logging_configuration.access_logs.kinesis_data_firehose.delivery_status.code #=> String, one of "success", "failed"
    #   resp.logging_configuration.access_logs.kinesis_data_firehose.delivery_status.message #=> String
    #   resp.logging_configuration.access_logs.kinesis_data_firehose.delivery_stream #=> String
    #   resp.logging_configuration.access_logs.log_version #=> String
    #   resp.logging_configuration.access_logs.include_trust_context #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVerifiedAccessInstanceLoggingConfiguration AWS API Documentation
    #
    # @overload modify_verified_access_instance_logging_configuration(params = {})
    # @param [Hash] params ({})
    def modify_verified_access_instance_logging_configuration(params = {}, options = {})
      req = build_request(:modify_verified_access_instance_logging_configuration, params)
      req.send_request(options)
    end

    # Modifies the configuration of the specified Amazon Web Services
    # Verified Access trust provider.
    #
    # @option params [required, String] :verified_access_trust_provider_id
    #   The ID of the Verified Access trust provider.
    #
    # @option params [Types::ModifyVerifiedAccessTrustProviderOidcOptions] :oidc_options
    #   The options for an OpenID Connect-compatible user-identity trust
    #   provider.
    #
    # @option params [Types::ModifyVerifiedAccessTrustProviderDeviceOptions] :device_options
    #   The options for a device-based trust provider. This parameter is
    #   required when the provider type is `device`.
    #
    # @option params [String] :description
    #   A description for the Verified Access trust provider.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive token that you provide to ensure idempotency
    #   of your modification request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [Types::VerifiedAccessSseSpecificationRequest] :sse_specification
    #   The options for server side encryption.
    #
    # @option params [Types::ModifyVerifiedAccessNativeApplicationOidcOptions] :native_application_oidc_options
    #   The OpenID Connect (OIDC) options.
    #
    # @return [Types::ModifyVerifiedAccessTrustProviderResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVerifiedAccessTrustProviderResult#verified_access_trust_provider #verified_access_trust_provider} => Types::VerifiedAccessTrustProvider
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_verified_access_trust_provider({
    #     verified_access_trust_provider_id: "VerifiedAccessTrustProviderId", # required
    #     oidc_options: {
    #       issuer: "String",
    #       authorization_endpoint: "String",
    #       token_endpoint: "String",
    #       user_info_endpoint: "String",
    #       client_id: "String",
    #       client_secret: "ClientSecretType",
    #       scope: "String",
    #     },
    #     device_options: {
    #       public_signing_key_url: "String",
    #     },
    #     description: "String",
    #     dry_run: false,
    #     client_token: "String",
    #     sse_specification: {
    #       customer_managed_key_enabled: false,
    #       kms_key_arn: "KmsKeyArn",
    #     },
    #     native_application_oidc_options: {
    #       public_signing_key_endpoint: "String",
    #       issuer: "String",
    #       authorization_endpoint: "String",
    #       token_endpoint: "String",
    #       user_info_endpoint: "String",
    #       client_id: "String",
    #       client_secret: "ClientSecretType",
    #       scope: "String",
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.verified_access_trust_provider.verified_access_trust_provider_id #=> String
    #   resp.verified_access_trust_provider.description #=> String
    #   resp.verified_access_trust_provider.trust_provider_type #=> String, one of "user", "device"
    #   resp.verified_access_trust_provider.user_trust_provider_type #=> String, one of "iam-identity-center", "oidc"
    #   resp.verified_access_trust_provider.device_trust_provider_type #=> String, one of "jamf", "crowdstrike", "jumpcloud"
    #   resp.verified_access_trust_provider.oidc_options.issuer #=> String
    #   resp.verified_access_trust_provider.oidc_options.authorization_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.token_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.user_info_endpoint #=> String
    #   resp.verified_access_trust_provider.oidc_options.client_id #=> String
    #   resp.verified_access_trust_provider.oidc_options.client_secret #=> String
    #   resp.verified_access_trust_provider.oidc_options.scope #=> String
    #   resp.verified_access_trust_provider.device_options.tenant_id #=> String
    #   resp.verified_access_trust_provider.device_options.public_signing_key_url #=> String
    #   resp.verified_access_trust_provider.policy_reference_name #=> String
    #   resp.verified_access_trust_provider.creation_time #=> String
    #   resp.verified_access_trust_provider.last_updated_time #=> String
    #   resp.verified_access_trust_provider.tags #=> Array
    #   resp.verified_access_trust_provider.tags[0].key #=> String
    #   resp.verified_access_trust_provider.tags[0].value #=> String
    #   resp.verified_access_trust_provider.sse_specification.customer_managed_key_enabled #=> Boolean
    #   resp.verified_access_trust_provider.sse_specification.kms_key_arn #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.public_signing_key_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.issuer #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.authorization_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.token_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.user_info_endpoint #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.client_id #=> String
    #   resp.verified_access_trust_provider.native_application_oidc_options.scope #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVerifiedAccessTrustProvider AWS API Documentation
    #
    # @overload modify_verified_access_trust_provider(params = {})
    # @param [Hash] params ({})
    def modify_verified_access_trust_provider(params = {}, options = {})
      req = build_request(:modify_verified_access_trust_provider, params)
      req.send_request(options)
    end

    # You can modify several parameters of an existing EBS volume, including
    # volume size, volume type, and IOPS capacity. If your EBS volume is
    # attached to a current-generation EC2 instance type, you might be able
    # to apply these changes without stopping the instance or detaching the
    # volume from it. For more information about modifying EBS volumes, see
    # [Amazon EBS Elastic Volumes][1] in the *Amazon EBS User Guide*.
    #
    # When you complete a resize operation on your volume, you need to
    # extend the volume's file-system size to take advantage of the new
    # storage capacity. For more information, see [Extend the file
    # system][2].
    #
    # For more information, see [Monitor the progress of volume
    # modifications][3] in the *Amazon EBS User Guide*.
    #
    # With previous-generation instance types, resizing an EBS volume might
    # require detaching and reattaching the volume or stopping and
    # restarting the instance.
    #
    # After modifying a volume, you must wait at least six hours and ensure
    # that the volume is in the `in-use` or `available` state before you can
    # modify the same volume. This is sometimes referred to as a cooldown
    # period.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-modify-volume.html
    # [2]: https://docs.aws.amazon.com/ebs/latest/userguide/recognize-expanded-volume-linux.html
    # [3]: https://docs.aws.amazon.com/ebs/latest/userguide/monitoring-volume-modifications.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :volume_id
    #   The ID of the volume.
    #
    # @option params [Integer] :size
    #   The target size of the volume, in GiB. The target volume size must be
    #   greater than or equal to the existing size of the volume.
    #
    #   The following are the supported volumes sizes for each volume type:
    #
    #   * `gp2` and `gp3`: 1 - 16,384 GiB
    #
    #   * `io1`: 4 - 16,384 GiB
    #
    #   * `io2`: 4 - 65,536 GiB
    #
    #   * `st1` and `sc1`: 125 - 16,384 GiB
    #
    #   * `standard`: 1 - 1024 GiB
    #
    #   Default: The existing size is retained.
    #
    # @option params [String] :volume_type
    #   The target EBS volume type of the volume. For more information, see
    #   [Amazon EBS volume types][1] in the *Amazon EBS User Guide*.
    #
    #   Default: The existing type is retained.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-volume-types.html
    #
    # @option params [Integer] :iops
    #   The target IOPS rate of the volume. This parameter is valid only for
    #   `gp3`, `io1`, and `io2` volumes.
    #
    #   The following are the supported values for each volume type:
    #
    #   * `gp3`: 3,000 - 16,000 IOPS
    #
    #   * `io1`: 100 - 64,000 IOPS
    #
    #   * `io2`: 100 - 256,000 IOPS
    #
    #   For `io2` volumes, you can achieve up to 256,000 IOPS on [instances
    #   built on the Nitro System][1]. On other instances, you can achieve
    #   performance up to 32,000 IOPS.
    #
    #   Default: The existing value is retained if you keep the same volume
    #   type. If you change the volume type to `io1`, `io2`, or `gp3`, the
    #   default is 3,000.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/instancetypes/ec2-nitro-instances.html
    #
    # @option params [Integer] :throughput
    #   The target throughput of the volume, in MiB/s. This parameter is valid
    #   only for `gp3` volumes. The maximum value is 1,000.
    #
    #   Default: The existing value is retained if the source and target
    #   volume type is `gp3`. Otherwise, the default value is 125.
    #
    #   Valid Range: Minimum value of 125. Maximum value of 1000.
    #
    # @option params [Boolean] :multi_attach_enabled
    #   Specifies whether to enable Amazon EBS Multi-Attach. If you enable
    #   Multi-Attach, you can attach the volume to up to 16 [ Nitro-based
    #   instances][1] in the same Availability Zone. This parameter is
    #   supported with `io1` and `io2` volumes only. For more information, see
    #   [ Amazon EBS Multi-Attach][2] in the *Amazon EBS User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/instancetypes/ec2-nitro-instances.html
    #   [2]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-volumes-multi.html
    #
    # @return [Types::ModifyVolumeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVolumeResult#volume_modification #volume_modification} => Types::VolumeModification
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_volume({
    #     dry_run: false,
    #     volume_id: "VolumeId", # required
    #     size: 1,
    #     volume_type: "standard", # accepts standard, io1, io2, gp2, sc1, st1, gp3
    #     iops: 1,
    #     throughput: 1,
    #     multi_attach_enabled: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.volume_modification.volume_id #=> String
    #   resp.volume_modification.modification_state #=> String, one of "modifying", "optimizing", "completed", "failed"
    #   resp.volume_modification.status_message #=> String
    #   resp.volume_modification.target_size #=> Integer
    #   resp.volume_modification.target_iops #=> Integer
    #   resp.volume_modification.target_volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.volume_modification.target_throughput #=> Integer
    #   resp.volume_modification.target_multi_attach_enabled #=> Boolean
    #   resp.volume_modification.original_size #=> Integer
    #   resp.volume_modification.original_iops #=> Integer
    #   resp.volume_modification.original_volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.volume_modification.original_throughput #=> Integer
    #   resp.volume_modification.original_multi_attach_enabled #=> Boolean
    #   resp.volume_modification.progress #=> Integer
    #   resp.volume_modification.start_time #=> Time
    #   resp.volume_modification.end_time #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVolume AWS API Documentation
    #
    # @overload modify_volume(params = {})
    # @param [Hash] params ({})
    def modify_volume(params = {}, options = {})
      req = build_request(:modify_volume, params)
      req.send_request(options)
    end

    # Modifies a volume attribute.
    #
    # By default, all I/O operations for the volume are suspended when the
    # data on the volume is determined to be potentially inconsistent, to
    # prevent undetectable, latent data corruption. The I/O access to the
    # volume can be resumed by first enabling I/O access and then checking
    # the data consistency on your volume.
    #
    # You can change the default behavior to resume I/O operations. We
    # recommend that you change this only for boot volumes or for volumes
    # that are stateless or disposable.
    #
    # @option params [Types::AttributeBooleanValue] :auto_enable_io
    #   Indicates whether the volume should be auto-enabled for I/O
    #   operations.
    #
    # @option params [required, String] :volume_id
    #   The ID of the volume.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To modify a volume attribute
    #
    #   # This example sets the ``autoEnableIo`` attribute of the volume with the ID ``vol-1234567890abcdef0`` to ``true``. If the
    #   # command succeeds, no output is returned.
    #
    #   resp = client.modify_volume_attribute({
    #     auto_enable_io: {
    #       value: true, 
    #     }, 
    #     dry_run: true, 
    #     volume_id: "vol-1234567890abcdef0", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_volume_attribute({
    #     auto_enable_io: {
    #       value: false,
    #     },
    #     volume_id: "VolumeId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVolumeAttribute AWS API Documentation
    #
    # @overload modify_volume_attribute(params = {})
    # @param [Hash] params ({})
    def modify_volume_attribute(params = {}, options = {})
      req = build_request(:modify_volume_attribute, params)
      req.send_request(options)
    end

    # Modifies the specified attribute of the specified VPC.
    #
    # @option params [Types::AttributeBooleanValue] :enable_dns_hostnames
    #   Indicates whether the instances launched in the VPC get DNS hostnames.
    #   If enabled, instances in the VPC get DNS hostnames; otherwise, they do
    #   not.
    #
    #   You cannot modify the DNS resolution and DNS hostnames attributes in
    #   the same request. Use separate requests for each attribute. You can
    #   only enable DNS hostnames if you've enabled DNS support.
    #
    # @option params [Types::AttributeBooleanValue] :enable_dns_support
    #   Indicates whether the DNS resolution is supported for the VPC. If
    #   enabled, queries to the Amazon provided DNS server at the
    #   169.254.169.253 IP address, or the reserved IP address at the base of
    #   the VPC network range "plus two" succeed. If disabled, the Amazon
    #   provided DNS service in the VPC that resolves public DNS hostnames to
    #   IP addresses is not enabled.
    #
    #   You cannot modify the DNS resolution and DNS hostnames attributes in
    #   the same request. Use separate requests for each attribute.
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @option params [Types::AttributeBooleanValue] :enable_network_address_usage_metrics
    #   Indicates whether Network Address Usage metrics are enabled for your
    #   VPC.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To modify the enableDnsSupport attribute
    #
    #   # This example modifies the enableDnsSupport attribute. This attribute indicates whether DNS resolution is enabled for the
    #   # VPC. If this attribute is true, the Amazon DNS server resolves DNS hostnames for instances in the VPC to their
    #   # corresponding IP addresses; otherwise, it does not.
    #
    #   resp = client.modify_vpc_attribute({
    #     enable_dns_support: {
    #       value: false, 
    #     }, 
    #     vpc_id: "vpc-a01106c2", 
    #   })
    #
    # @example Example: To modify the enableDnsHostnames attribute
    #
    #   # This example modifies the enableDnsHostnames attribute. This attribute indicates whether instances launched in the VPC
    #   # get DNS hostnames. If this attribute is true, instances in the VPC get DNS hostnames; otherwise, they do not.
    #
    #   resp = client.modify_vpc_attribute({
    #     enable_dns_hostnames: {
    #       value: false, 
    #     }, 
    #     vpc_id: "vpc-a01106c2", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_vpc_attribute({
    #     enable_dns_hostnames: {
    #       value: false,
    #     },
    #     enable_dns_support: {
    #       value: false,
    #     },
    #     vpc_id: "VpcId", # required
    #     enable_network_address_usage_metrics: {
    #       value: false,
    #     },
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVpcAttribute AWS API Documentation
    #
    # @overload modify_vpc_attribute(params = {})
    # @param [Hash] params ({})
    def modify_vpc_attribute(params = {}, options = {})
      req = build_request(:modify_vpc_attribute, params)
      req.send_request(options)
    end

    # Modify VPC Block Public Access (BPA) exclusions. A VPC BPA exclusion
    # is a mode that can be applied to a single VPC or subnet that exempts
    # it from the account’s BPA mode and will allow bidirectional or
    # egress-only access. You can create BPA exclusions for VPCs and subnets
    # even when BPA is not enabled on the account to ensure that there is no
    # traffic disruption to the exclusions when VPC BPA is turned on.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :exclusion_id
    #   The ID of an exclusion.
    #
    # @option params [required, String] :internet_gateway_exclusion_mode
    #   The exclusion mode for internet gateway traffic.
    #
    #   * `allow-bidirectional`: Allow all internet traffic to and from the
    #     excluded VPCs and subnets.
    #
    #   * `allow-egress`: Allow outbound internet traffic from the excluded
    #     VPCs and subnets. Block inbound internet traffic to the excluded
    #     VPCs and subnets. Only applies when VPC Block Public Access is set
    #     to Bidirectional.
    #
    # @return [Types::ModifyVpcBlockPublicAccessExclusionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVpcBlockPublicAccessExclusionResult#vpc_block_public_access_exclusion #vpc_block_public_access_exclusion} => Types::VpcBlockPublicAccessExclusion
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_vpc_block_public_access_exclusion({
    #     dry_run: false,
    #     exclusion_id: "VpcBlockPublicAccessExclusionId", # required
    #     internet_gateway_exclusion_mode: "allow-bidirectional", # required, accepts allow-bidirectional, allow-egress
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc_block_public_access_exclusion.exclusion_id #=> String
    #   resp.vpc_block_public_access_exclusion.internet_gateway_exclusion_mode #=> String, one of "allow-bidirectional", "allow-egress"
    #   resp.vpc_block_public_access_exclusion.resource_arn #=> String
    #   resp.vpc_block_public_access_exclusion.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "update-in-progress", "update-complete", "update-failed", "delete-in-progress", "delete-complete", "disable-in-progress", "disable-complete"
    #   resp.vpc_block_public_access_exclusion.reason #=> String
    #   resp.vpc_block_public_access_exclusion.creation_timestamp #=> Time
    #   resp.vpc_block_public_access_exclusion.last_update_timestamp #=> Time
    #   resp.vpc_block_public_access_exclusion.deletion_timestamp #=> Time
    #   resp.vpc_block_public_access_exclusion.tags #=> Array
    #   resp.vpc_block_public_access_exclusion.tags[0].key #=> String
    #   resp.vpc_block_public_access_exclusion.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVpcBlockPublicAccessExclusion AWS API Documentation
    #
    # @overload modify_vpc_block_public_access_exclusion(params = {})
    # @param [Hash] params ({})
    def modify_vpc_block_public_access_exclusion(params = {}, options = {})
      req = build_request(:modify_vpc_block_public_access_exclusion, params)
      req.send_request(options)
    end

    # Modify VPC Block Public Access (BPA) options. VPC Block Public Access
    # (BPA) enables you to block resources in VPCs and subnets that you own
    # in a Region from reaching or being reached from the internet through
    # internet gateways and egress-only internet gateways. To learn more
    # about VPC BPA, see [Block public access to VPCs and subnets][1] in the
    # *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/security-vpc-bpa.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :internet_gateway_block_mode
    #   The mode of VPC BPA.
    #
    #   * `off`: VPC BPA is not enabled and traffic is allowed to and from
    #     internet gateways and egress-only internet gateways in this Region.
    #
    #   * `block-bidirectional`: Block all traffic to and from internet
    #     gateways and egress-only internet gateways in this Region (except
    #     for excluded VPCs and subnets).
    #
    #   * `block-ingress`: Block all internet traffic to the VPCs in this
    #     Region (except for VPCs or subnets which are excluded). Only traffic
    #     to and from NAT gateways and egress-only internet gateways is
    #     allowed because these gateways only allow outbound connections to be
    #     established.
    #
    # @return [Types::ModifyVpcBlockPublicAccessOptionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVpcBlockPublicAccessOptionsResult#vpc_block_public_access_options #vpc_block_public_access_options} => Types::VpcBlockPublicAccessOptions
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_vpc_block_public_access_options({
    #     dry_run: false,
    #     internet_gateway_block_mode: "off", # required, accepts off, block-bidirectional, block-ingress
    #   })
    #
    # @example Response structure
    #
    #   resp.vpc_block_public_access_options.aws_account_id #=> String
    #   resp.vpc_block_public_access_options.aws_region #=> String
    #   resp.vpc_block_public_access_options.state #=> String, one of "default-state", "update-in-progress", "update-complete"
    #   resp.vpc_block_public_access_options.internet_gateway_block_mode #=> String, one of "off", "block-bidirectional", "block-ingress"
    #   resp.vpc_block_public_access_options.reason #=> String
    #   resp.vpc_block_public_access_options.last_update_timestamp #=> Time
    #   resp.vpc_block_public_access_options.managed_by #=> String, one of "account", "declarative-policy"
    #   resp.vpc_block_public_access_options.exclusions_allowed #=> String, one of "allowed", "not-allowed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVpcBlockPublicAccessOptions AWS API Documentation
    #
    # @overload modify_vpc_block_public_access_options(params = {})
    # @param [Hash] params ({})
    def modify_vpc_block_public_access_options(params = {}, options = {})
      req = build_request(:modify_vpc_block_public_access_options, params)
      req.send_request(options)
    end

    # Modifies attributes of a specified VPC endpoint. The attributes that
    # you can modify depend on the type of VPC endpoint (interface, gateway,
    # or Gateway Load Balancer). For more information, see the [Amazon Web
    # Services PrivateLink Guide][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/privatelink/
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :vpc_endpoint_id
    #   The ID of the endpoint.
    #
    # @option params [Boolean] :reset_policy
    #   (Gateway endpoint) Specify `true` to reset the policy document to the
    #   default policy. The default policy allows full access to the service.
    #
    # @option params [String] :policy_document
    #   (Interface and gateway endpoints) A policy to attach to the endpoint
    #   that controls access to the service. The policy must be in valid JSON
    #   format.
    #
    # @option params [Array<String>] :add_route_table_ids
    #   (Gateway endpoint) The IDs of the route tables to associate with the
    #   endpoint.
    #
    # @option params [Array<String>] :remove_route_table_ids
    #   (Gateway endpoint) The IDs of the route tables to disassociate from
    #   the endpoint.
    #
    # @option params [Array<String>] :add_subnet_ids
    #   (Interface and Gateway Load Balancer endpoints) The IDs of the subnets
    #   in which to serve the endpoint. For a Gateway Load Balancer endpoint,
    #   you can specify only one subnet.
    #
    # @option params [Array<String>] :remove_subnet_ids
    #   (Interface endpoint) The IDs of the subnets from which to remove the
    #   endpoint.
    #
    # @option params [Array<String>] :add_security_group_ids
    #   (Interface endpoint) The IDs of the security groups to associate with
    #   the endpoint network interfaces.
    #
    # @option params [Array<String>] :remove_security_group_ids
    #   (Interface endpoint) The IDs of the security groups to disassociate
    #   from the endpoint network interfaces.
    #
    # @option params [String] :ip_address_type
    #   The IP address type for the endpoint.
    #
    # @option params [Types::DnsOptionsSpecification] :dns_options
    #   The DNS options for the endpoint.
    #
    # @option params [Boolean] :private_dns_enabled
    #   (Interface endpoint) Indicates whether a private hosted zone is
    #   associated with the VPC.
    #
    # @option params [Array<Types::SubnetConfiguration>] :subnet_configurations
    #   The subnet configurations for the endpoint.
    #
    # @return [Types::ModifyVpcEndpointResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVpcEndpointResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_vpc_endpoint({
    #     dry_run: false,
    #     vpc_endpoint_id: "VpcEndpointId", # required
    #     reset_policy: false,
    #     policy_document: "String",
    #     add_route_table_ids: ["RouteTableId"],
    #     remove_route_table_ids: ["RouteTableId"],
    #     add_subnet_ids: ["SubnetId"],
    #     remove_subnet_ids: ["SubnetId"],
    #     add_security_group_ids: ["SecurityGroupId"],
    #     remove_security_group_ids: ["SecurityGroupId"],
    #     ip_address_type: "ipv4", # accepts ipv4, dualstack, ipv6
    #     dns_options: {
    #       dns_record_ip_type: "ipv4", # accepts ipv4, dualstack, ipv6, service-defined
    #       private_dns_only_for_inbound_resolver_endpoint: false,
    #     },
    #     private_dns_enabled: false,
    #     subnet_configurations: [
    #       {
    #         subnet_id: "SubnetId",
    #         ipv_4: "String",
    #         ipv_6: "String",
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVpcEndpoint AWS API Documentation
    #
    # @overload modify_vpc_endpoint(params = {})
    # @param [Hash] params ({})
    def modify_vpc_endpoint(params = {}, options = {})
      req = build_request(:modify_vpc_endpoint, params)
      req.send_request(options)
    end

    # Modifies a connection notification for VPC endpoint or VPC endpoint
    # service. You can change the SNS topic for the notification, or the
    # events for which to be notified.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :connection_notification_id
    #   The ID of the notification.
    #
    # @option params [String] :connection_notification_arn
    #   The ARN for the SNS topic for the notification.
    #
    # @option params [Array<String>] :connection_events
    #   The events for the endpoint. Valid values are `Accept`, `Connect`,
    #   `Delete`, and `Reject`.
    #
    # @return [Types::ModifyVpcEndpointConnectionNotificationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVpcEndpointConnectionNotificationResult#return_value #return_value} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_vpc_endpoint_connection_notification({
    #     dry_run: false,
    #     connection_notification_id: "ConnectionNotificationId", # required
    #     connection_notification_arn: "String",
    #     connection_events: ["String"],
    #   })
    #
    # @example Response structure
    #
    #   resp.return_value #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVpcEndpointConnectionNotification AWS API Documentation
    #
    # @overload modify_vpc_endpoint_connection_notification(params = {})
    # @param [Hash] params ({})
    def modify_vpc_endpoint_connection_notification(params = {}, options = {})
      req = build_request(:modify_vpc_endpoint_connection_notification, params)
      req.send_request(options)
    end

    # Modifies the attributes of the specified VPC endpoint service
    # configuration.
    #
    # If you set or modify the private DNS name, you must prove that you own
    # the private DNS domain name.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :service_id
    #   The ID of the service.
    #
    # @option params [String] :private_dns_name
    #   (Interface endpoint configuration) The private DNS name to assign to
    #   the endpoint service.
    #
    # @option params [Boolean] :remove_private_dns_name
    #   (Interface endpoint configuration) Removes the private DNS name of the
    #   endpoint service.
    #
    # @option params [Boolean] :acceptance_required
    #   Indicates whether requests to create an endpoint to the service must
    #   be accepted.
    #
    # @option params [Array<String>] :add_network_load_balancer_arns
    #   The Amazon Resource Names (ARNs) of Network Load Balancers to add to
    #   the service configuration.
    #
    # @option params [Array<String>] :remove_network_load_balancer_arns
    #   The Amazon Resource Names (ARNs) of Network Load Balancers to remove
    #   from the service configuration.
    #
    # @option params [Array<String>] :add_gateway_load_balancer_arns
    #   The Amazon Resource Names (ARNs) of Gateway Load Balancers to add to
    #   the service configuration.
    #
    # @option params [Array<String>] :remove_gateway_load_balancer_arns
    #   The Amazon Resource Names (ARNs) of Gateway Load Balancers to remove
    #   from the service configuration.
    #
    # @option params [Array<String>] :add_supported_ip_address_types
    #   The IP address types to add to the service configuration.
    #
    # @option params [Array<String>] :remove_supported_ip_address_types
    #   The IP address types to remove from the service configuration.
    #
    # @option params [Array<String>] :add_supported_regions
    #   The supported Regions to add to the service configuration.
    #
    # @option params [Array<String>] :remove_supported_regions
    #   The supported Regions to remove from the service configuration.
    #
    # @return [Types::ModifyVpcEndpointServiceConfigurationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVpcEndpointServiceConfigurationResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_vpc_endpoint_service_configuration({
    #     dry_run: false,
    #     service_id: "VpcEndpointServiceId", # required
    #     private_dns_name: "String",
    #     remove_private_dns_name: false,
    #     acceptance_required: false,
    #     add_network_load_balancer_arns: ["String"],
    #     remove_network_load_balancer_arns: ["String"],
    #     add_gateway_load_balancer_arns: ["String"],
    #     remove_gateway_load_balancer_arns: ["String"],
    #     add_supported_ip_address_types: ["String"],
    #     remove_supported_ip_address_types: ["String"],
    #     add_supported_regions: ["String"],
    #     remove_supported_regions: ["String"],
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVpcEndpointServiceConfiguration AWS API Documentation
    #
    # @overload modify_vpc_endpoint_service_configuration(params = {})
    # @param [Hash] params ({})
    def modify_vpc_endpoint_service_configuration(params = {}, options = {})
      req = build_request(:modify_vpc_endpoint_service_configuration, params)
      req.send_request(options)
    end

    # Modifies the payer responsibility for your VPC endpoint service.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :service_id
    #   The ID of the service.
    #
    # @option params [required, String] :payer_responsibility
    #   The entity that is responsible for the endpoint costs. The default is
    #   the endpoint owner. If you set the payer responsibility to the service
    #   owner, you cannot set it back to the endpoint owner.
    #
    # @return [Types::ModifyVpcEndpointServicePayerResponsibilityResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVpcEndpointServicePayerResponsibilityResult#return_value #return_value} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_vpc_endpoint_service_payer_responsibility({
    #     dry_run: false,
    #     service_id: "VpcEndpointServiceId", # required
    #     payer_responsibility: "ServiceOwner", # required, accepts ServiceOwner
    #   })
    #
    # @example Response structure
    #
    #   resp.return_value #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVpcEndpointServicePayerResponsibility AWS API Documentation
    #
    # @overload modify_vpc_endpoint_service_payer_responsibility(params = {})
    # @param [Hash] params ({})
    def modify_vpc_endpoint_service_payer_responsibility(params = {}, options = {})
      req = build_request(:modify_vpc_endpoint_service_payer_responsibility, params)
      req.send_request(options)
    end

    # Modifies the permissions for your VPC endpoint service. You can add or
    # remove permissions for service consumers (Amazon Web Services
    # accounts, users, and IAM roles) to connect to your endpoint service.
    #
    # If you grant permissions to all principals, the service is public. Any
    # users who know the name of a public service can send a request to
    # attach an endpoint. If the service does not require manual approval,
    # attachments are automatically approved.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :service_id
    #   The ID of the service.
    #
    # @option params [Array<String>] :add_allowed_principals
    #   The Amazon Resource Names (ARN) of the principals. Permissions are
    #   granted to the principals in this list. To grant permissions to all
    #   principals, specify an asterisk (*).
    #
    # @option params [Array<String>] :remove_allowed_principals
    #   The Amazon Resource Names (ARN) of the principals. Permissions are
    #   revoked for principals in this list.
    #
    # @return [Types::ModifyVpcEndpointServicePermissionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVpcEndpointServicePermissionsResult#added_principals #added_principals} => Array&lt;Types::AddedPrincipal&gt;
    #   * {Types::ModifyVpcEndpointServicePermissionsResult#return_value #return_value} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_vpc_endpoint_service_permissions({
    #     dry_run: false,
    #     service_id: "VpcEndpointServiceId", # required
    #     add_allowed_principals: ["String"],
    #     remove_allowed_principals: ["String"],
    #   })
    #
    # @example Response structure
    #
    #   resp.added_principals #=> Array
    #   resp.added_principals[0].principal_type #=> String, one of "All", "Service", "OrganizationUnit", "Account", "User", "Role"
    #   resp.added_principals[0].principal #=> String
    #   resp.added_principals[0].service_permission_id #=> String
    #   resp.added_principals[0].service_id #=> String
    #   resp.return_value #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVpcEndpointServicePermissions AWS API Documentation
    #
    # @overload modify_vpc_endpoint_service_permissions(params = {})
    # @param [Hash] params ({})
    def modify_vpc_endpoint_service_permissions(params = {}, options = {})
      req = build_request(:modify_vpc_endpoint_service_permissions, params)
      req.send_request(options)
    end

    # Modifies the VPC peering connection options on one side of a VPC
    # peering connection.
    #
    # If the peered VPCs are in the same Amazon Web Services account, you
    # can enable DNS resolution for queries from the local VPC. This ensures
    # that queries from the local VPC resolve to private IP addresses in the
    # peer VPC. This option is not available if the peered VPCs are in
    # different Amazon Web Services accounts or different Regions. For
    # peered VPCs in different Amazon Web Services accounts, each Amazon Web
    # Services account owner must initiate a separate request to modify the
    # peering connection options. For inter-region peering connections, you
    # must use the Region for the requester VPC to modify the requester VPC
    # peering options and the Region for the accepter VPC to modify the
    # accepter VPC peering options. To verify which VPCs are the accepter
    # and the requester for a VPC peering connection, use the
    # DescribeVpcPeeringConnections command.
    #
    # @option params [Types::PeeringConnectionOptionsRequest] :accepter_peering_connection_options
    #   The VPC peering connection options for the accepter VPC.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Types::PeeringConnectionOptionsRequest] :requester_peering_connection_options
    #   The VPC peering connection options for the requester VPC.
    #
    # @option params [required, String] :vpc_peering_connection_id
    #   The ID of the VPC peering connection.
    #
    # @return [Types::ModifyVpcPeeringConnectionOptionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVpcPeeringConnectionOptionsResult#accepter_peering_connection_options #accepter_peering_connection_options} => Types::PeeringConnectionOptions
    #   * {Types::ModifyVpcPeeringConnectionOptionsResult#requester_peering_connection_options #requester_peering_connection_options} => Types::PeeringConnectionOptions
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_vpc_peering_connection_options({
    #     accepter_peering_connection_options: {
    #       allow_dns_resolution_from_remote_vpc: false,
    #       allow_egress_from_local_classic_link_to_remote_vpc: false,
    #       allow_egress_from_local_vpc_to_remote_classic_link: false,
    #     },
    #     dry_run: false,
    #     requester_peering_connection_options: {
    #       allow_dns_resolution_from_remote_vpc: false,
    #       allow_egress_from_local_classic_link_to_remote_vpc: false,
    #       allow_egress_from_local_vpc_to_remote_classic_link: false,
    #     },
    #     vpc_peering_connection_id: "VpcPeeringConnectionId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.accepter_peering_connection_options.allow_dns_resolution_from_remote_vpc #=> Boolean
    #   resp.accepter_peering_connection_options.allow_egress_from_local_classic_link_to_remote_vpc #=> Boolean
    #   resp.accepter_peering_connection_options.allow_egress_from_local_vpc_to_remote_classic_link #=> Boolean
    #   resp.requester_peering_connection_options.allow_dns_resolution_from_remote_vpc #=> Boolean
    #   resp.requester_peering_connection_options.allow_egress_from_local_classic_link_to_remote_vpc #=> Boolean
    #   resp.requester_peering_connection_options.allow_egress_from_local_vpc_to_remote_classic_link #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVpcPeeringConnectionOptions AWS API Documentation
    #
    # @overload modify_vpc_peering_connection_options(params = {})
    # @param [Hash] params ({})
    def modify_vpc_peering_connection_options(params = {}, options = {})
      req = build_request(:modify_vpc_peering_connection_options, params)
      req.send_request(options)
    end

    # Modifies the instance tenancy attribute of the specified VPC. You can
    # change the instance tenancy attribute of a VPC to `default` only. You
    # cannot change the instance tenancy attribute to `dedicated`.
    #
    # After you modify the tenancy of the VPC, any new instances that you
    # launch into the VPC have a tenancy of `default`, unless you specify
    # otherwise during launch. The tenancy of any existing instances in the
    # VPC is not affected.
    #
    # For more information, see [Dedicated Instances][1] in the *Amazon EC2
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/dedicated-instance.html
    #
    # @option params [required, String] :vpc_id
    #   The ID of the VPC.
    #
    # @option params [required, String] :instance_tenancy
    #   The instance tenancy attribute for the VPC.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyVpcTenancyResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVpcTenancyResult#return_value #return_value} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_vpc_tenancy({
    #     vpc_id: "VpcId", # required
    #     instance_tenancy: "default", # required, accepts default
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return_value #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVpcTenancy AWS API Documentation
    #
    # @overload modify_vpc_tenancy(params = {})
    # @param [Hash] params ({})
    def modify_vpc_tenancy(params = {}, options = {})
      req = build_request(:modify_vpc_tenancy, params)
      req.send_request(options)
    end

    # Modifies the customer gateway or the target gateway of an Amazon Web
    # Services Site-to-Site VPN connection. To modify the target gateway,
    # the following migration options are available:
    #
    # * An existing virtual private gateway to a new virtual private gateway
    #
    # * An existing virtual private gateway to a transit gateway
    #
    # * An existing transit gateway to a new transit gateway
    #
    # * An existing transit gateway to a virtual private gateway
    #
    # Before you perform the migration to the new gateway, you must
    # configure the new gateway. Use CreateVpnGateway to create a virtual
    # private gateway, or CreateTransitGateway to create a transit gateway.
    #
    # This step is required when you migrate from a virtual private gateway
    # with static routes to a transit gateway.
    #
    # You must delete the static routes before you migrate to the new
    # gateway.
    #
    # Keep a copy of the static route before you delete it. You will need to
    # add back these routes to the transit gateway after the VPN connection
    # migration is complete.
    #
    # After you migrate to the new gateway, you might need to modify your
    # VPC route table. Use CreateRoute and DeleteRoute to make the changes
    # described in [Update VPC route tables][1] in the *Amazon Web Services
    # Site-to-Site VPN User Guide*.
    #
    # When the new gateway is a transit gateway, modify the transit gateway
    # route table to allow traffic between the VPC and the Amazon Web
    # Services Site-to-Site VPN connection. Use CreateTransitGatewayRoute to
    # add the routes.
    #
    # If you deleted VPN static routes, you must add the static routes to
    # the transit gateway route table.
    #
    # After you perform this operation, the VPN endpoint's IP addresses on
    # the Amazon Web Services side and the tunnel options remain intact.
    # Your Amazon Web Services Site-to-Site VPN connection will be
    # temporarily unavailable for a brief period while we provision the new
    # endpoints.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpn/latest/s2svpn/modify-vpn-target.html#step-update-routing
    #
    # @option params [required, String] :vpn_connection_id
    #   The ID of the VPN connection.
    #
    # @option params [String] :transit_gateway_id
    #   The ID of the transit gateway.
    #
    # @option params [String] :customer_gateway_id
    #   The ID of the customer gateway at your end of the VPN connection.
    #
    # @option params [String] :vpn_gateway_id
    #   The ID of the virtual private gateway at the Amazon Web Services side
    #   of the VPN connection.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyVpnConnectionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVpnConnectionResult#vpn_connection #vpn_connection} => Types::VpnConnection
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_vpn_connection({
    #     vpn_connection_id: "VpnConnectionId", # required
    #     transit_gateway_id: "TransitGatewayId",
    #     customer_gateway_id: "CustomerGatewayId",
    #     vpn_gateway_id: "VpnGatewayId",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpn_connection.category #=> String
    #   resp.vpn_connection.transit_gateway_id #=> String
    #   resp.vpn_connection.core_network_arn #=> String
    #   resp.vpn_connection.core_network_attachment_arn #=> String
    #   resp.vpn_connection.gateway_association_state #=> String, one of "associated", "not-associated", "associating", "disassociating"
    #   resp.vpn_connection.options.enable_acceleration #=> Boolean
    #   resp.vpn_connection.options.static_routes_only #=> Boolean
    #   resp.vpn_connection.options.local_ipv_4_network_cidr #=> String
    #   resp.vpn_connection.options.remote_ipv_4_network_cidr #=> String
    #   resp.vpn_connection.options.local_ipv_6_network_cidr #=> String
    #   resp.vpn_connection.options.remote_ipv_6_network_cidr #=> String
    #   resp.vpn_connection.options.outside_ip_address_type #=> String
    #   resp.vpn_connection.options.transport_transit_gateway_attachment_id #=> String
    #   resp.vpn_connection.options.tunnel_inside_ip_version #=> String, one of "ipv4", "ipv6"
    #   resp.vpn_connection.options.tunnel_options #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].outside_ip_address #=> String
    #   resp.vpn_connection.options.tunnel_options[0].tunnel_inside_cidr #=> String
    #   resp.vpn_connection.options.tunnel_options[0].tunnel_inside_ipv_6_cidr #=> String
    #   resp.vpn_connection.options.tunnel_options[0].pre_shared_key #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_lifetime_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_lifetime_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].rekey_margin_time_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].rekey_fuzz_percentage #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].replay_window_size #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].dpd_timeout_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].dpd_timeout_action #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_encryption_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_encryption_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_encryption_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_encryption_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_integrity_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_integrity_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_integrity_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_integrity_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_dh_group_numbers #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_dh_group_numbers[0].value #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_dh_group_numbers #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_dh_group_numbers[0].value #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].ike_versions #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].ike_versions[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].startup_action #=> String
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_enabled #=> Boolean
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_group_arn #=> String
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_output_format #=> String
    #   resp.vpn_connection.options.tunnel_options[0].enable_tunnel_lifecycle_control #=> Boolean
    #   resp.vpn_connection.routes #=> Array
    #   resp.vpn_connection.routes[0].destination_cidr_block #=> String
    #   resp.vpn_connection.routes[0].source #=> String, one of "Static"
    #   resp.vpn_connection.routes[0].state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.vpn_connection.tags #=> Array
    #   resp.vpn_connection.tags[0].key #=> String
    #   resp.vpn_connection.tags[0].value #=> String
    #   resp.vpn_connection.vgw_telemetry #=> Array
    #   resp.vpn_connection.vgw_telemetry[0].accepted_route_count #=> Integer
    #   resp.vpn_connection.vgw_telemetry[0].last_status_change #=> Time
    #   resp.vpn_connection.vgw_telemetry[0].outside_ip_address #=> String
    #   resp.vpn_connection.vgw_telemetry[0].status #=> String, one of "UP", "DOWN"
    #   resp.vpn_connection.vgw_telemetry[0].status_message #=> String
    #   resp.vpn_connection.vgw_telemetry[0].certificate_arn #=> String
    #   resp.vpn_connection.vpn_connection_id #=> String
    #   resp.vpn_connection.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.vpn_connection.customer_gateway_configuration #=> String
    #   resp.vpn_connection.type #=> String, one of "ipsec.1"
    #   resp.vpn_connection.customer_gateway_id #=> String
    #   resp.vpn_connection.vpn_gateway_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVpnConnection AWS API Documentation
    #
    # @overload modify_vpn_connection(params = {})
    # @param [Hash] params ({})
    def modify_vpn_connection(params = {}, options = {})
      req = build_request(:modify_vpn_connection, params)
      req.send_request(options)
    end

    # Modifies the connection options for your Site-to-Site VPN connection.
    #
    # When you modify the VPN connection options, the VPN endpoint IP
    # addresses on the Amazon Web Services side do not change, and the
    # tunnel options do not change. Your VPN connection will be temporarily
    # unavailable for a brief period while the VPN connection is updated.
    #
    # @option params [required, String] :vpn_connection_id
    #   The ID of the Site-to-Site VPN connection.
    #
    # @option params [String] :local_ipv_4_network_cidr
    #   The IPv4 CIDR on the customer gateway (on-premises) side of the VPN
    #   connection.
    #
    #   Default: `0.0.0.0/0`
    #
    # @option params [String] :remote_ipv_4_network_cidr
    #   The IPv4 CIDR on the Amazon Web Services side of the VPN connection.
    #
    #   Default: `0.0.0.0/0`
    #
    # @option params [String] :local_ipv_6_network_cidr
    #   The IPv6 CIDR on the customer gateway (on-premises) side of the VPN
    #   connection.
    #
    #   Default: `::/0`
    #
    # @option params [String] :remote_ipv_6_network_cidr
    #   The IPv6 CIDR on the Amazon Web Services side of the VPN connection.
    #
    #   Default: `::/0`
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyVpnConnectionOptionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVpnConnectionOptionsResult#vpn_connection #vpn_connection} => Types::VpnConnection
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_vpn_connection_options({
    #     vpn_connection_id: "VpnConnectionId", # required
    #     local_ipv_4_network_cidr: "String",
    #     remote_ipv_4_network_cidr: "String",
    #     local_ipv_6_network_cidr: "String",
    #     remote_ipv_6_network_cidr: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpn_connection.category #=> String
    #   resp.vpn_connection.transit_gateway_id #=> String
    #   resp.vpn_connection.core_network_arn #=> String
    #   resp.vpn_connection.core_network_attachment_arn #=> String
    #   resp.vpn_connection.gateway_association_state #=> String, one of "associated", "not-associated", "associating", "disassociating"
    #   resp.vpn_connection.options.enable_acceleration #=> Boolean
    #   resp.vpn_connection.options.static_routes_only #=> Boolean
    #   resp.vpn_connection.options.local_ipv_4_network_cidr #=> String
    #   resp.vpn_connection.options.remote_ipv_4_network_cidr #=> String
    #   resp.vpn_connection.options.local_ipv_6_network_cidr #=> String
    #   resp.vpn_connection.options.remote_ipv_6_network_cidr #=> String
    #   resp.vpn_connection.options.outside_ip_address_type #=> String
    #   resp.vpn_connection.options.transport_transit_gateway_attachment_id #=> String
    #   resp.vpn_connection.options.tunnel_inside_ip_version #=> String, one of "ipv4", "ipv6"
    #   resp.vpn_connection.options.tunnel_options #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].outside_ip_address #=> String
    #   resp.vpn_connection.options.tunnel_options[0].tunnel_inside_cidr #=> String
    #   resp.vpn_connection.options.tunnel_options[0].tunnel_inside_ipv_6_cidr #=> String
    #   resp.vpn_connection.options.tunnel_options[0].pre_shared_key #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_lifetime_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_lifetime_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].rekey_margin_time_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].rekey_fuzz_percentage #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].replay_window_size #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].dpd_timeout_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].dpd_timeout_action #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_encryption_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_encryption_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_encryption_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_encryption_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_integrity_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_integrity_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_integrity_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_integrity_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_dh_group_numbers #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_dh_group_numbers[0].value #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_dh_group_numbers #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_dh_group_numbers[0].value #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].ike_versions #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].ike_versions[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].startup_action #=> String
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_enabled #=> Boolean
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_group_arn #=> String
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_output_format #=> String
    #   resp.vpn_connection.options.tunnel_options[0].enable_tunnel_lifecycle_control #=> Boolean
    #   resp.vpn_connection.routes #=> Array
    #   resp.vpn_connection.routes[0].destination_cidr_block #=> String
    #   resp.vpn_connection.routes[0].source #=> String, one of "Static"
    #   resp.vpn_connection.routes[0].state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.vpn_connection.tags #=> Array
    #   resp.vpn_connection.tags[0].key #=> String
    #   resp.vpn_connection.tags[0].value #=> String
    #   resp.vpn_connection.vgw_telemetry #=> Array
    #   resp.vpn_connection.vgw_telemetry[0].accepted_route_count #=> Integer
    #   resp.vpn_connection.vgw_telemetry[0].last_status_change #=> Time
    #   resp.vpn_connection.vgw_telemetry[0].outside_ip_address #=> String
    #   resp.vpn_connection.vgw_telemetry[0].status #=> String, one of "UP", "DOWN"
    #   resp.vpn_connection.vgw_telemetry[0].status_message #=> String
    #   resp.vpn_connection.vgw_telemetry[0].certificate_arn #=> String
    #   resp.vpn_connection.vpn_connection_id #=> String
    #   resp.vpn_connection.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.vpn_connection.customer_gateway_configuration #=> String
    #   resp.vpn_connection.type #=> String, one of "ipsec.1"
    #   resp.vpn_connection.customer_gateway_id #=> String
    #   resp.vpn_connection.vpn_gateway_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVpnConnectionOptions AWS API Documentation
    #
    # @overload modify_vpn_connection_options(params = {})
    # @param [Hash] params ({})
    def modify_vpn_connection_options(params = {}, options = {})
      req = build_request(:modify_vpn_connection_options, params)
      req.send_request(options)
    end

    # Modifies the VPN tunnel endpoint certificate.
    #
    # @option params [required, String] :vpn_connection_id
    #   The ID of the Amazon Web Services Site-to-Site VPN connection.
    #
    # @option params [required, String] :vpn_tunnel_outside_ip_address
    #   The external IP address of the VPN tunnel.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ModifyVpnTunnelCertificateResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVpnTunnelCertificateResult#vpn_connection #vpn_connection} => Types::VpnConnection
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_vpn_tunnel_certificate({
    #     vpn_connection_id: "VpnConnectionId", # required
    #     vpn_tunnel_outside_ip_address: "String", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpn_connection.category #=> String
    #   resp.vpn_connection.transit_gateway_id #=> String
    #   resp.vpn_connection.core_network_arn #=> String
    #   resp.vpn_connection.core_network_attachment_arn #=> String
    #   resp.vpn_connection.gateway_association_state #=> String, one of "associated", "not-associated", "associating", "disassociating"
    #   resp.vpn_connection.options.enable_acceleration #=> Boolean
    #   resp.vpn_connection.options.static_routes_only #=> Boolean
    #   resp.vpn_connection.options.local_ipv_4_network_cidr #=> String
    #   resp.vpn_connection.options.remote_ipv_4_network_cidr #=> String
    #   resp.vpn_connection.options.local_ipv_6_network_cidr #=> String
    #   resp.vpn_connection.options.remote_ipv_6_network_cidr #=> String
    #   resp.vpn_connection.options.outside_ip_address_type #=> String
    #   resp.vpn_connection.options.transport_transit_gateway_attachment_id #=> String
    #   resp.vpn_connection.options.tunnel_inside_ip_version #=> String, one of "ipv4", "ipv6"
    #   resp.vpn_connection.options.tunnel_options #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].outside_ip_address #=> String
    #   resp.vpn_connection.options.tunnel_options[0].tunnel_inside_cidr #=> String
    #   resp.vpn_connection.options.tunnel_options[0].tunnel_inside_ipv_6_cidr #=> String
    #   resp.vpn_connection.options.tunnel_options[0].pre_shared_key #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_lifetime_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_lifetime_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].rekey_margin_time_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].rekey_fuzz_percentage #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].replay_window_size #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].dpd_timeout_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].dpd_timeout_action #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_encryption_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_encryption_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_encryption_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_encryption_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_integrity_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_integrity_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_integrity_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_integrity_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_dh_group_numbers #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_dh_group_numbers[0].value #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_dh_group_numbers #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_dh_group_numbers[0].value #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].ike_versions #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].ike_versions[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].startup_action #=> String
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_enabled #=> Boolean
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_group_arn #=> String
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_output_format #=> String
    #   resp.vpn_connection.options.tunnel_options[0].enable_tunnel_lifecycle_control #=> Boolean
    #   resp.vpn_connection.routes #=> Array
    #   resp.vpn_connection.routes[0].destination_cidr_block #=> String
    #   resp.vpn_connection.routes[0].source #=> String, one of "Static"
    #   resp.vpn_connection.routes[0].state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.vpn_connection.tags #=> Array
    #   resp.vpn_connection.tags[0].key #=> String
    #   resp.vpn_connection.tags[0].value #=> String
    #   resp.vpn_connection.vgw_telemetry #=> Array
    #   resp.vpn_connection.vgw_telemetry[0].accepted_route_count #=> Integer
    #   resp.vpn_connection.vgw_telemetry[0].last_status_change #=> Time
    #   resp.vpn_connection.vgw_telemetry[0].outside_ip_address #=> String
    #   resp.vpn_connection.vgw_telemetry[0].status #=> String, one of "UP", "DOWN"
    #   resp.vpn_connection.vgw_telemetry[0].status_message #=> String
    #   resp.vpn_connection.vgw_telemetry[0].certificate_arn #=> String
    #   resp.vpn_connection.vpn_connection_id #=> String
    #   resp.vpn_connection.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.vpn_connection.customer_gateway_configuration #=> String
    #   resp.vpn_connection.type #=> String, one of "ipsec.1"
    #   resp.vpn_connection.customer_gateway_id #=> String
    #   resp.vpn_connection.vpn_gateway_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVpnTunnelCertificate AWS API Documentation
    #
    # @overload modify_vpn_tunnel_certificate(params = {})
    # @param [Hash] params ({})
    def modify_vpn_tunnel_certificate(params = {}, options = {})
      req = build_request(:modify_vpn_tunnel_certificate, params)
      req.send_request(options)
    end

    # Modifies the options for a VPN tunnel in an Amazon Web Services
    # Site-to-Site VPN connection. You can modify multiple options for a
    # tunnel in a single request, but you can only modify one tunnel at a
    # time. For more information, see [Site-to-Site VPN tunnel options for
    # your Site-to-Site VPN connection][1] in the *Amazon Web Services
    # Site-to-Site VPN User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpn/latest/s2svpn/VPNTunnels.html
    #
    # @option params [required, String] :vpn_connection_id
    #   The ID of the Amazon Web Services Site-to-Site VPN connection.
    #
    # @option params [required, String] :vpn_tunnel_outside_ip_address
    #   The external IP address of the VPN tunnel.
    #
    # @option params [required, Types::ModifyVpnTunnelOptionsSpecification] :tunnel_options
    #   The tunnel options to modify.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Boolean] :skip_tunnel_replacement
    #   Choose whether or not to trigger immediate tunnel replacement. This is
    #   only applicable when turning on or off `EnableTunnelLifecycleControl`.
    #
    #   Valid values: `True` \| `False`
    #
    # @return [Types::ModifyVpnTunnelOptionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ModifyVpnTunnelOptionsResult#vpn_connection #vpn_connection} => Types::VpnConnection
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.modify_vpn_tunnel_options({
    #     vpn_connection_id: "VpnConnectionId", # required
    #     vpn_tunnel_outside_ip_address: "String", # required
    #     tunnel_options: { # required
    #       tunnel_inside_cidr: "String",
    #       tunnel_inside_ipv_6_cidr: "String",
    #       pre_shared_key: "preSharedKey",
    #       phase_1_lifetime_seconds: 1,
    #       phase_2_lifetime_seconds: 1,
    #       rekey_margin_time_seconds: 1,
    #       rekey_fuzz_percentage: 1,
    #       replay_window_size: 1,
    #       dpd_timeout_seconds: 1,
    #       dpd_timeout_action: "String",
    #       phase_1_encryption_algorithms: [
    #         {
    #           value: "String",
    #         },
    #       ],
    #       phase_2_encryption_algorithms: [
    #         {
    #           value: "String",
    #         },
    #       ],
    #       phase_1_integrity_algorithms: [
    #         {
    #           value: "String",
    #         },
    #       ],
    #       phase_2_integrity_algorithms: [
    #         {
    #           value: "String",
    #         },
    #       ],
    #       phase_1_dh_group_numbers: [
    #         {
    #           value: 1,
    #         },
    #       ],
    #       phase_2_dh_group_numbers: [
    #         {
    #           value: 1,
    #         },
    #       ],
    #       ike_versions: [
    #         {
    #           value: "String",
    #         },
    #       ],
    #       startup_action: "String",
    #       log_options: {
    #         cloud_watch_log_options: {
    #           log_enabled: false,
    #           log_group_arn: "CloudWatchLogGroupArn",
    #           log_output_format: "String",
    #         },
    #       },
    #       enable_tunnel_lifecycle_control: false,
    #     },
    #     dry_run: false,
    #     skip_tunnel_replacement: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.vpn_connection.category #=> String
    #   resp.vpn_connection.transit_gateway_id #=> String
    #   resp.vpn_connection.core_network_arn #=> String
    #   resp.vpn_connection.core_network_attachment_arn #=> String
    #   resp.vpn_connection.gateway_association_state #=> String, one of "associated", "not-associated", "associating", "disassociating"
    #   resp.vpn_connection.options.enable_acceleration #=> Boolean
    #   resp.vpn_connection.options.static_routes_only #=> Boolean
    #   resp.vpn_connection.options.local_ipv_4_network_cidr #=> String
    #   resp.vpn_connection.options.remote_ipv_4_network_cidr #=> String
    #   resp.vpn_connection.options.local_ipv_6_network_cidr #=> String
    #   resp.vpn_connection.options.remote_ipv_6_network_cidr #=> String
    #   resp.vpn_connection.options.outside_ip_address_type #=> String
    #   resp.vpn_connection.options.transport_transit_gateway_attachment_id #=> String
    #   resp.vpn_connection.options.tunnel_inside_ip_version #=> String, one of "ipv4", "ipv6"
    #   resp.vpn_connection.options.tunnel_options #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].outside_ip_address #=> String
    #   resp.vpn_connection.options.tunnel_options[0].tunnel_inside_cidr #=> String
    #   resp.vpn_connection.options.tunnel_options[0].tunnel_inside_ipv_6_cidr #=> String
    #   resp.vpn_connection.options.tunnel_options[0].pre_shared_key #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_lifetime_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_lifetime_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].rekey_margin_time_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].rekey_fuzz_percentage #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].replay_window_size #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].dpd_timeout_seconds #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].dpd_timeout_action #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_encryption_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_encryption_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_encryption_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_encryption_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_integrity_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_integrity_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_integrity_algorithms #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_integrity_algorithms[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_dh_group_numbers #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_1_dh_group_numbers[0].value #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_dh_group_numbers #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].phase_2_dh_group_numbers[0].value #=> Integer
    #   resp.vpn_connection.options.tunnel_options[0].ike_versions #=> Array
    #   resp.vpn_connection.options.tunnel_options[0].ike_versions[0].value #=> String
    #   resp.vpn_connection.options.tunnel_options[0].startup_action #=> String
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_enabled #=> Boolean
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_group_arn #=> String
    #   resp.vpn_connection.options.tunnel_options[0].log_options.cloud_watch_log_options.log_output_format #=> String
    #   resp.vpn_connection.options.tunnel_options[0].enable_tunnel_lifecycle_control #=> Boolean
    #   resp.vpn_connection.routes #=> Array
    #   resp.vpn_connection.routes[0].destination_cidr_block #=> String
    #   resp.vpn_connection.routes[0].source #=> String, one of "Static"
    #   resp.vpn_connection.routes[0].state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.vpn_connection.tags #=> Array
    #   resp.vpn_connection.tags[0].key #=> String
    #   resp.vpn_connection.tags[0].value #=> String
    #   resp.vpn_connection.vgw_telemetry #=> Array
    #   resp.vpn_connection.vgw_telemetry[0].accepted_route_count #=> Integer
    #   resp.vpn_connection.vgw_telemetry[0].last_status_change #=> Time
    #   resp.vpn_connection.vgw_telemetry[0].outside_ip_address #=> String
    #   resp.vpn_connection.vgw_telemetry[0].status #=> String, one of "UP", "DOWN"
    #   resp.vpn_connection.vgw_telemetry[0].status_message #=> String
    #   resp.vpn_connection.vgw_telemetry[0].certificate_arn #=> String
    #   resp.vpn_connection.vpn_connection_id #=> String
    #   resp.vpn_connection.state #=> String, one of "pending", "available", "deleting", "deleted"
    #   resp.vpn_connection.customer_gateway_configuration #=> String
    #   resp.vpn_connection.type #=> String, one of "ipsec.1"
    #   resp.vpn_connection.customer_gateway_id #=> String
    #   resp.vpn_connection.vpn_gateway_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ModifyVpnTunnelOptions AWS API Documentation
    #
    # @overload modify_vpn_tunnel_options(params = {})
    # @param [Hash] params ({})
    def modify_vpn_tunnel_options(params = {}, options = {})
      req = build_request(:modify_vpn_tunnel_options, params)
      req.send_request(options)
    end

    # Enables detailed monitoring for a running instance. Otherwise, basic
    # monitoring is enabled. For more information, see [Monitor your
    # instances using CloudWatch][1] in the *Amazon EC2 User Guide*.
    #
    # To disable detailed monitoring, see [UnmonitorInstances][2].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-cloudwatch.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_UnmonitorInstances.html
    #
    # @option params [required, Array<String>] :instance_ids
    #   The IDs of the instances.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::MonitorInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::MonitorInstancesResult#instance_monitorings #instance_monitorings} => Array&lt;Types::InstanceMonitoring&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.monitor_instances({
    #     instance_ids: ["InstanceId"], # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_monitorings #=> Array
    #   resp.instance_monitorings[0].instance_id #=> String
    #   resp.instance_monitorings[0].monitoring.state #=> String, one of "disabled", "disabling", "enabled", "pending"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/MonitorInstances AWS API Documentation
    #
    # @overload monitor_instances(params = {})
    # @param [Hash] params ({})
    def monitor_instances(params = {}, options = {})
      req = build_request(:monitor_instances, params)
      req.send_request(options)
    end

    # <note markdown="1"> This action is deprecated.
    #
    #  </note>
    #
    # Moves an Elastic IP address from the EC2-Classic platform to the
    # EC2-VPC platform. The Elastic IP address must be allocated to your
    # account for more than 24 hours, and it must not be associated with an
    # instance. After the Elastic IP address is moved, it is no longer
    # available for use in the EC2-Classic platform, unless you move it back
    # using the RestoreAddressToClassic request. You cannot move an Elastic
    # IP address that was originally allocated for use in the EC2-VPC
    # platform to the EC2-Classic platform.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :public_ip
    #   The Elastic IP address.
    #
    # @return [Types::MoveAddressToVpcResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::MoveAddressToVpcResult#allocation_id #allocation_id} => String
    #   * {Types::MoveAddressToVpcResult#status #status} => String
    #
    #
    # @example Example: To move an address to EC2-VPC
    #
    #   # This example moves the specified Elastic IP address to the EC2-VPC platform.
    #
    #   resp = client.move_address_to_vpc({
    #     public_ip: "54.123.4.56", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     status: "MoveInProgress", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.move_address_to_vpc({
    #     dry_run: false,
    #     public_ip: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.allocation_id #=> String
    #   resp.status #=> String, one of "MoveInProgress", "InVpc", "InClassic"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/MoveAddressToVpc AWS API Documentation
    #
    # @overload move_address_to_vpc(params = {})
    # @param [Hash] params ({})
    def move_address_to_vpc(params = {}, options = {})
      req = build_request(:move_address_to_vpc, params)
      req.send_request(options)
    end

    # Move a BYOIPv4 CIDR to IPAM from a public IPv4 pool.
    #
    # If you already have a BYOIPv4 CIDR with Amazon Web Services, you can
    # move the CIDR to IPAM from a public IPv4 pool. You cannot move an IPv6
    # CIDR to IPAM. If you are bringing a new IP address to Amazon Web
    # Services for the first time, complete the steps in [Tutorial: BYOIP
    # address CIDRs to IPAM][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/tutorials-byoip-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :cidr
    #   The BYOIP CIDR.
    #
    # @option params [required, String] :ipam_pool_id
    #   The IPAM pool ID.
    #
    # @option params [required, String] :ipam_pool_owner
    #   The Amazon Web Services account ID of the owner of the IPAM pool.
    #
    # @return [Types::MoveByoipCidrToIpamResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::MoveByoipCidrToIpamResult#byoip_cidr #byoip_cidr} => Types::ByoipCidr
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.move_byoip_cidr_to_ipam({
    #     dry_run: false,
    #     cidr: "String", # required
    #     ipam_pool_id: "IpamPoolId", # required
    #     ipam_pool_owner: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.byoip_cidr.cidr #=> String
    #   resp.byoip_cidr.description #=> String
    #   resp.byoip_cidr.asn_associations #=> Array
    #   resp.byoip_cidr.asn_associations[0].asn #=> String
    #   resp.byoip_cidr.asn_associations[0].cidr #=> String
    #   resp.byoip_cidr.asn_associations[0].status_message #=> String
    #   resp.byoip_cidr.asn_associations[0].state #=> String, one of "disassociated", "failed-disassociation", "failed-association", "pending-disassociation", "pending-association", "associated"
    #   resp.byoip_cidr.status_message #=> String
    #   resp.byoip_cidr.state #=> String, one of "advertised", "deprovisioned", "failed-deprovision", "failed-provision", "pending-deprovision", "pending-provision", "provisioned", "provisioned-not-publicly-advertisable"
    #   resp.byoip_cidr.network_border_group #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/MoveByoipCidrToIpam AWS API Documentation
    #
    # @overload move_byoip_cidr_to_ipam(params = {})
    # @param [Hash] params ({})
    def move_byoip_cidr_to_ipam(params = {}, options = {})
      req = build_request(:move_byoip_cidr_to_ipam, params)
      req.send_request(options)
    end

    # Move available capacity from a source Capacity Reservation to a
    # destination Capacity Reservation. The source Capacity Reservation and
    # the destination Capacity Reservation must be `active`, owned by your
    # Amazon Web Services account, and share the following:
    #
    # * Instance type
    #
    # * Platform
    #
    # * Availability Zone
    #
    # * Tenancy
    #
    # * Placement group
    #
    # * Capacity Reservation end time - `At specific time` or `Manually`.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensure
    #   Idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [required, String] :source_capacity_reservation_id
    #   The ID of the Capacity Reservation from which you want to move
    #   capacity.
    #
    # @option params [required, String] :destination_capacity_reservation_id
    #   The ID of the Capacity Reservation that you want to move capacity
    #   into.
    #
    # @option params [required, Integer] :instance_count
    #   The number of instances that you want to move from the source Capacity
    #   Reservation.
    #
    # @return [Types::MoveCapacityReservationInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::MoveCapacityReservationInstancesResult#source_capacity_reservation #source_capacity_reservation} => Types::CapacityReservation
    #   * {Types::MoveCapacityReservationInstancesResult#destination_capacity_reservation #destination_capacity_reservation} => Types::CapacityReservation
    #   * {Types::MoveCapacityReservationInstancesResult#instance_count #instance_count} => Integer
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.move_capacity_reservation_instances({
    #     dry_run: false,
    #     client_token: "String",
    #     source_capacity_reservation_id: "CapacityReservationId", # required
    #     destination_capacity_reservation_id: "CapacityReservationId", # required
    #     instance_count: 1, # required
    #   })
    #
    # @example Response structure
    #
    #   resp.source_capacity_reservation.capacity_reservation_id #=> String
    #   resp.source_capacity_reservation.owner_id #=> String
    #   resp.source_capacity_reservation.capacity_reservation_arn #=> String
    #   resp.source_capacity_reservation.availability_zone_id #=> String
    #   resp.source_capacity_reservation.instance_type #=> String
    #   resp.source_capacity_reservation.instance_platform #=> String, one of "Linux/UNIX", "Red Hat Enterprise Linux", "SUSE Linux", "Windows", "Windows with SQL Server", "Windows with SQL Server Enterprise", "Windows with SQL Server Standard", "Windows with SQL Server Web", "Linux with SQL Server Standard", "Linux with SQL Server Web", "Linux with SQL Server Enterprise", "RHEL with SQL Server Standard", "RHEL with SQL Server Enterprise", "RHEL with SQL Server Web", "RHEL with HA", "RHEL with HA and SQL Server Standard", "RHEL with HA and SQL Server Enterprise", "Ubuntu Pro"
    #   resp.source_capacity_reservation.availability_zone #=> String
    #   resp.source_capacity_reservation.tenancy #=> String, one of "default", "dedicated"
    #   resp.source_capacity_reservation.total_instance_count #=> Integer
    #   resp.source_capacity_reservation.available_instance_count #=> Integer
    #   resp.source_capacity_reservation.ebs_optimized #=> Boolean
    #   resp.source_capacity_reservation.ephemeral_storage #=> Boolean
    #   resp.source_capacity_reservation.state #=> String, one of "active", "expired", "cancelled", "pending", "failed", "scheduled", "payment-pending", "payment-failed", "assessing", "delayed", "unsupported"
    #   resp.source_capacity_reservation.start_date #=> Time
    #   resp.source_capacity_reservation.end_date #=> Time
    #   resp.source_capacity_reservation.end_date_type #=> String, one of "unlimited", "limited"
    #   resp.source_capacity_reservation.instance_match_criteria #=> String, one of "open", "targeted"
    #   resp.source_capacity_reservation.create_date #=> Time
    #   resp.source_capacity_reservation.tags #=> Array
    #   resp.source_capacity_reservation.tags[0].key #=> String
    #   resp.source_capacity_reservation.tags[0].value #=> String
    #   resp.source_capacity_reservation.outpost_arn #=> String
    #   resp.source_capacity_reservation.capacity_reservation_fleet_id #=> String
    #   resp.source_capacity_reservation.placement_group_arn #=> String
    #   resp.source_capacity_reservation.capacity_allocations #=> Array
    #   resp.source_capacity_reservation.capacity_allocations[0].allocation_type #=> String, one of "used", "future"
    #   resp.source_capacity_reservation.capacity_allocations[0].count #=> Integer
    #   resp.source_capacity_reservation.reservation_type #=> String, one of "default", "capacity-block"
    #   resp.source_capacity_reservation.unused_reservation_billing_owner_id #=> String
    #   resp.source_capacity_reservation.commitment_info.committed_instance_count #=> Integer
    #   resp.source_capacity_reservation.commitment_info.commitment_end_date #=> Time
    #   resp.source_capacity_reservation.delivery_preference #=> String, one of "fixed", "incremental"
    #   resp.destination_capacity_reservation.capacity_reservation_id #=> String
    #   resp.destination_capacity_reservation.owner_id #=> String
    #   resp.destination_capacity_reservation.capacity_reservation_arn #=> String
    #   resp.destination_capacity_reservation.availability_zone_id #=> String
    #   resp.destination_capacity_reservation.instance_type #=> String
    #   resp.destination_capacity_reservation.instance_platform #=> String, one of "Linux/UNIX", "Red Hat Enterprise Linux", "SUSE Linux", "Windows", "Windows with SQL Server", "Windows with SQL Server Enterprise", "Windows with SQL Server Standard", "Windows with SQL Server Web", "Linux with SQL Server Standard", "Linux with SQL Server Web", "Linux with SQL Server Enterprise", "RHEL with SQL Server Standard", "RHEL with SQL Server Enterprise", "RHEL with SQL Server Web", "RHEL with HA", "RHEL with HA and SQL Server Standard", "RHEL with HA and SQL Server Enterprise", "Ubuntu Pro"
    #   resp.destination_capacity_reservation.availability_zone #=> String
    #   resp.destination_capacity_reservation.tenancy #=> String, one of "default", "dedicated"
    #   resp.destination_capacity_reservation.total_instance_count #=> Integer
    #   resp.destination_capacity_reservation.available_instance_count #=> Integer
    #   resp.destination_capacity_reservation.ebs_optimized #=> Boolean
    #   resp.destination_capacity_reservation.ephemeral_storage #=> Boolean
    #   resp.destination_capacity_reservation.state #=> String, one of "active", "expired", "cancelled", "pending", "failed", "scheduled", "payment-pending", "payment-failed", "assessing", "delayed", "unsupported"
    #   resp.destination_capacity_reservation.start_date #=> Time
    #   resp.destination_capacity_reservation.end_date #=> Time
    #   resp.destination_capacity_reservation.end_date_type #=> String, one of "unlimited", "limited"
    #   resp.destination_capacity_reservation.instance_match_criteria #=> String, one of "open", "targeted"
    #   resp.destination_capacity_reservation.create_date #=> Time
    #   resp.destination_capacity_reservation.tags #=> Array
    #   resp.destination_capacity_reservation.tags[0].key #=> String
    #   resp.destination_capacity_reservation.tags[0].value #=> String
    #   resp.destination_capacity_reservation.outpost_arn #=> String
    #   resp.destination_capacity_reservation.capacity_reservation_fleet_id #=> String
    #   resp.destination_capacity_reservation.placement_group_arn #=> String
    #   resp.destination_capacity_reservation.capacity_allocations #=> Array
    #   resp.destination_capacity_reservation.capacity_allocations[0].allocation_type #=> String, one of "used", "future"
    #   resp.destination_capacity_reservation.capacity_allocations[0].count #=> Integer
    #   resp.destination_capacity_reservation.reservation_type #=> String, one of "default", "capacity-block"
    #   resp.destination_capacity_reservation.unused_reservation_billing_owner_id #=> String
    #   resp.destination_capacity_reservation.commitment_info.committed_instance_count #=> Integer
    #   resp.destination_capacity_reservation.commitment_info.commitment_end_date #=> Time
    #   resp.destination_capacity_reservation.delivery_preference #=> String, one of "fixed", "incremental"
    #   resp.instance_count #=> Integer
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/MoveCapacityReservationInstances AWS API Documentation
    #
    # @overload move_capacity_reservation_instances(params = {})
    # @param [Hash] params ({})
    def move_capacity_reservation_instances(params = {}, options = {})
      req = build_request(:move_capacity_reservation_instances, params)
      req.send_request(options)
    end

    # Provisions an IPv4 or IPv6 address range for use with your Amazon Web
    # Services resources through bring your own IP addresses (BYOIP) and
    # creates a corresponding address pool. After the address range is
    # provisioned, it is ready to be advertised using AdvertiseByoipCidr.
    #
    # Amazon Web Services verifies that you own the address range and are
    # authorized to advertise it. You must ensure that the address range is
    # registered to you and that you created an RPKI ROA to authorize Amazon
    # ASNs 16509 and 14618 to advertise the address range. For more
    # information, see [Bring your own IP addresses (BYOIP)][1] in the
    # *Amazon EC2 User Guide*.
    #
    # Provisioning an address range is an asynchronous operation, so the
    # call returns immediately, but the address range is not ready to use
    # until its status changes from `pending-provision` to `provisioned`. To
    # monitor the status of an address range, use DescribeByoipCidrs. To
    # allocate an Elastic IP address from your IPv4 address pool, use
    # AllocateAddress with either the specific address from the address pool
    # or the ID of the address pool.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-byoip.html
    #
    # @option params [required, String] :cidr
    #   The public IPv4 or IPv6 address range, in CIDR notation. The most
    #   specific IPv4 prefix that you can specify is /24. The most specific
    #   IPv6 address range that you can bring is /48 for CIDRs that are
    #   publicly advertisable and /56 for CIDRs that are not publicly
    #   advertisable. The address range cannot overlap with another address
    #   range that you've brought to this or another Region.
    #
    # @option params [Types::CidrAuthorizationContext] :cidr_authorization_context
    #   A signed document that proves that you are authorized to bring the
    #   specified IP address range to Amazon using BYOIP.
    #
    # @option params [Boolean] :publicly_advertisable
    #   (IPv6 only) Indicate whether the address range will be publicly
    #   advertised to the internet.
    #
    #   Default: true
    #
    # @option params [String] :description
    #   A description for the address range and the address pool.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::TagSpecification>] :pool_tag_specifications
    #   The tags to apply to the address pool.
    #
    # @option params [Boolean] :multi_region
    #   Reserved.
    #
    # @option params [String] :network_border_group
    #   If you have [Local Zones][1] enabled, you can choose a network border
    #   group for Local Zones when you provision and advertise a BYOIPv4 CIDR.
    #   Choose the network border group carefully as the EIP and the Amazon
    #   Web Services resource it is associated with must reside in the same
    #   network border group.
    #
    #   You can provision BYOIP address ranges to and advertise them in the
    #   following Local Zone network border groups:
    #
    #   * us-east-1-dfw-2
    #
    #   * us-west-2-lax-1
    #
    #   * us-west-2-phx-2
    #
    #   <note markdown="1"> You cannot provision or advertise BYOIPv6 address ranges in Local
    #   Zones at this time.
    #
    #    </note>
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/local-zones/latest/ug/how-local-zones-work.html
    #
    # @return [Types::ProvisionByoipCidrResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ProvisionByoipCidrResult#byoip_cidr #byoip_cidr} => Types::ByoipCidr
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.provision_byoip_cidr({
    #     cidr: "String", # required
    #     cidr_authorization_context: {
    #       message: "String", # required
    #       signature: "String", # required
    #     },
    #     publicly_advertisable: false,
    #     description: "String",
    #     dry_run: false,
    #     pool_tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     multi_region: false,
    #     network_border_group: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.byoip_cidr.cidr #=> String
    #   resp.byoip_cidr.description #=> String
    #   resp.byoip_cidr.asn_associations #=> Array
    #   resp.byoip_cidr.asn_associations[0].asn #=> String
    #   resp.byoip_cidr.asn_associations[0].cidr #=> String
    #   resp.byoip_cidr.asn_associations[0].status_message #=> String
    #   resp.byoip_cidr.asn_associations[0].state #=> String, one of "disassociated", "failed-disassociation", "failed-association", "pending-disassociation", "pending-association", "associated"
    #   resp.byoip_cidr.status_message #=> String
    #   resp.byoip_cidr.state #=> String, one of "advertised", "deprovisioned", "failed-deprovision", "failed-provision", "pending-deprovision", "pending-provision", "provisioned", "provisioned-not-publicly-advertisable"
    #   resp.byoip_cidr.network_border_group #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ProvisionByoipCidr AWS API Documentation
    #
    # @overload provision_byoip_cidr(params = {})
    # @param [Hash] params ({})
    def provision_byoip_cidr(params = {}, options = {})
      req = build_request(:provision_byoip_cidr, params)
      req.send_request(options)
    end

    # Provisions your Autonomous System Number (ASN) for use in your Amazon
    # Web Services account. This action requires authorization context for
    # Amazon to bring the ASN to an Amazon Web Services account. For more
    # information, see [Tutorial: Bring your ASN to IPAM][1] in the *Amazon
    # VPC IPAM guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/tutorials-byoasn.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_id
    #   An IPAM ID.
    #
    # @option params [required, String] :asn
    #   A public 2-byte or 4-byte ASN.
    #
    # @option params [required, Types::AsnAuthorizationContext] :asn_authorization_context
    #   An ASN authorization context.
    #
    # @return [Types::ProvisionIpamByoasnResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ProvisionIpamByoasnResult#byoasn #byoasn} => Types::Byoasn
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.provision_ipam_byoasn({
    #     dry_run: false,
    #     ipam_id: "IpamId", # required
    #     asn: "String", # required
    #     asn_authorization_context: { # required
    #       message: "String", # required
    #       signature: "String", # required
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.byoasn.asn #=> String
    #   resp.byoasn.ipam_id #=> String
    #   resp.byoasn.status_message #=> String
    #   resp.byoasn.state #=> String, one of "deprovisioned", "failed-deprovision", "failed-provision", "pending-deprovision", "pending-provision", "provisioned"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ProvisionIpamByoasn AWS API Documentation
    #
    # @overload provision_ipam_byoasn(params = {})
    # @param [Hash] params ({})
    def provision_ipam_byoasn(params = {}, options = {})
      req = build_request(:provision_ipam_byoasn, params)
      req.send_request(options)
    end

    # Provision a CIDR to an IPAM pool. You can use this action to provision
    # new CIDRs to a top-level pool or to transfer a CIDR from a top-level
    # pool to a pool within it.
    #
    # For more information, see [Provision CIDRs to pools][1] in the *Amazon
    # VPC IPAM User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/prov-cidr-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_pool_id
    #   The ID of the IPAM pool to which you want to assign a CIDR.
    #
    # @option params [String] :cidr
    #   The CIDR you want to assign to the IPAM pool. Either "NetmaskLength"
    #   or "Cidr" is required. This value will be null if you specify
    #   "NetmaskLength" and will be filled in during the provisioning
    #   process.
    #
    # @option params [Types::IpamCidrAuthorizationContext] :cidr_authorization_context
    #   A signed document that proves that you are authorized to bring a
    #   specified IP address range to Amazon using BYOIP. This option only
    #   applies to IPv4 and IPv6 pools in the public scope.
    #
    # @option params [Integer] :netmask_length
    #   The netmask length of the CIDR you'd like to provision to a pool. Can
    #   be used for provisioning Amazon-provided IPv6 CIDRs to top-level pools
    #   and for provisioning CIDRs to pools with source pools. Cannot be used
    #   to provision BYOIP CIDRs to top-level pools. Either "NetmaskLength"
    #   or "Cidr" is required.
    #
    # @option params [String] :client_token
    #   A unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @option params [String] :verification_method
    #   The method for verifying control of a public IP address range.
    #   Defaults to `remarks-x509` if not specified. This option only applies
    #   to IPv4 and IPv6 pools in the public scope.
    #
    # @option params [String] :ipam_external_resource_verification_token_id
    #   Verification token ID. This option only applies to IPv4 and IPv6 pools
    #   in the public scope.
    #
    # @return [Types::ProvisionIpamPoolCidrResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ProvisionIpamPoolCidrResult#ipam_pool_cidr #ipam_pool_cidr} => Types::IpamPoolCidr
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.provision_ipam_pool_cidr({
    #     dry_run: false,
    #     ipam_pool_id: "IpamPoolId", # required
    #     cidr: "String",
    #     cidr_authorization_context: {
    #       message: "String",
    #       signature: "String",
    #     },
    #     netmask_length: 1,
    #     client_token: "String",
    #     verification_method: "remarks-x509", # accepts remarks-x509, dns-token
    #     ipam_external_resource_verification_token_id: "IpamExternalResourceVerificationTokenId",
    #   })
    #
    # @example Response structure
    #
    #   resp.ipam_pool_cidr.cidr #=> String
    #   resp.ipam_pool_cidr.state #=> String, one of "pending-provision", "provisioned", "failed-provision", "pending-deprovision", "deprovisioned", "failed-deprovision", "pending-import", "failed-import"
    #   resp.ipam_pool_cidr.failure_reason.code #=> String, one of "cidr-not-available", "limit-exceeded"
    #   resp.ipam_pool_cidr.failure_reason.message #=> String
    #   resp.ipam_pool_cidr.ipam_pool_cidr_id #=> String
    #   resp.ipam_pool_cidr.netmask_length #=> Integer
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ProvisionIpamPoolCidr AWS API Documentation
    #
    # @overload provision_ipam_pool_cidr(params = {})
    # @param [Hash] params ({})
    def provision_ipam_pool_cidr(params = {}, options = {})
      req = build_request(:provision_ipam_pool_cidr, params)
      req.send_request(options)
    end

    # Provision a CIDR to a public IPv4 pool.
    #
    # For more information about IPAM, see [What is IPAM?][1] in the *Amazon
    # VPC IPAM User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/ipam/what-is-it-ipam.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_pool_id
    #   The ID of the IPAM pool you would like to use to allocate this CIDR.
    #
    # @option params [required, String] :pool_id
    #   The ID of the public IPv4 pool you would like to use for this CIDR.
    #
    # @option params [required, Integer] :netmask_length
    #   The netmask length of the CIDR you would like to allocate to the
    #   public IPv4 pool. The least specific netmask length you can define is
    #   24.
    #
    # @option params [String] :network_border_group
    #   The Availability Zone (AZ) or Local Zone (LZ) network border group
    #   that the resource that the IP address is assigned to is in. Defaults
    #   to an AZ network border group. For more information on available Local
    #   Zones, see [Local Zone availability][1] in the *Amazon EC2 User
    #   Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-byoip.html#byoip-zone-avail
    #
    # @return [Types::ProvisionPublicIpv4PoolCidrResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ProvisionPublicIpv4PoolCidrResult#pool_id #pool_id} => String
    #   * {Types::ProvisionPublicIpv4PoolCidrResult#pool_address_range #pool_address_range} => Types::PublicIpv4PoolRange
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.provision_public_ipv_4_pool_cidr({
    #     dry_run: false,
    #     ipam_pool_id: "IpamPoolId", # required
    #     pool_id: "Ipv4PoolEc2Id", # required
    #     netmask_length: 1, # required
    #     network_border_group: "String",
    #   })
    #
    # @example Response structure
    #
    #   resp.pool_id #=> String
    #   resp.pool_address_range.first_address #=> String
    #   resp.pool_address_range.last_address #=> String
    #   resp.pool_address_range.address_count #=> Integer
    #   resp.pool_address_range.available_address_count #=> Integer
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ProvisionPublicIpv4PoolCidr AWS API Documentation
    #
    # @overload provision_public_ipv_4_pool_cidr(params = {})
    # @param [Hash] params ({})
    def provision_public_ipv_4_pool_cidr(params = {}, options = {})
      req = build_request(:provision_public_ipv_4_pool_cidr, params)
      req.send_request(options)
    end

    # Purchase the Capacity Block for use with your account. With Capacity
    # Blocks you ensure GPU capacity is available for machine learning (ML)
    # workloads. You must specify the ID of the Capacity Block offering you
    # are purchasing.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the Capacity Block during launch.
    #
    # @option params [required, String] :capacity_block_offering_id
    #   The ID of the Capacity Block offering.
    #
    # @option params [required, String] :instance_platform
    #   The type of operating system for which to reserve capacity.
    #
    # @return [Types::PurchaseCapacityBlockResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::PurchaseCapacityBlockResult#capacity_reservation #capacity_reservation} => Types::CapacityReservation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.purchase_capacity_block({
    #     dry_run: false,
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     capacity_block_offering_id: "OfferingId", # required
    #     instance_platform: "Linux/UNIX", # required, accepts Linux/UNIX, Red Hat Enterprise Linux, SUSE Linux, Windows, Windows with SQL Server, Windows with SQL Server Enterprise, Windows with SQL Server Standard, Windows with SQL Server Web, Linux with SQL Server Standard, Linux with SQL Server Web, Linux with SQL Server Enterprise, RHEL with SQL Server Standard, RHEL with SQL Server Enterprise, RHEL with SQL Server Web, RHEL with HA, RHEL with HA and SQL Server Standard, RHEL with HA and SQL Server Enterprise, Ubuntu Pro
    #   })
    #
    # @example Response structure
    #
    #   resp.capacity_reservation.capacity_reservation_id #=> String
    #   resp.capacity_reservation.owner_id #=> String
    #   resp.capacity_reservation.capacity_reservation_arn #=> String
    #   resp.capacity_reservation.availability_zone_id #=> String
    #   resp.capacity_reservation.instance_type #=> String
    #   resp.capacity_reservation.instance_platform #=> String, one of "Linux/UNIX", "Red Hat Enterprise Linux", "SUSE Linux", "Windows", "Windows with SQL Server", "Windows with SQL Server Enterprise", "Windows with SQL Server Standard", "Windows with SQL Server Web", "Linux with SQL Server Standard", "Linux with SQL Server Web", "Linux with SQL Server Enterprise", "RHEL with SQL Server Standard", "RHEL with SQL Server Enterprise", "RHEL with SQL Server Web", "RHEL with HA", "RHEL with HA and SQL Server Standard", "RHEL with HA and SQL Server Enterprise", "Ubuntu Pro"
    #   resp.capacity_reservation.availability_zone #=> String
    #   resp.capacity_reservation.tenancy #=> String, one of "default", "dedicated"
    #   resp.capacity_reservation.total_instance_count #=> Integer
    #   resp.capacity_reservation.available_instance_count #=> Integer
    #   resp.capacity_reservation.ebs_optimized #=> Boolean
    #   resp.capacity_reservation.ephemeral_storage #=> Boolean
    #   resp.capacity_reservation.state #=> String, one of "active", "expired", "cancelled", "pending", "failed", "scheduled", "payment-pending", "payment-failed", "assessing", "delayed", "unsupported"
    #   resp.capacity_reservation.start_date #=> Time
    #   resp.capacity_reservation.end_date #=> Time
    #   resp.capacity_reservation.end_date_type #=> String, one of "unlimited", "limited"
    #   resp.capacity_reservation.instance_match_criteria #=> String, one of "open", "targeted"
    #   resp.capacity_reservation.create_date #=> Time
    #   resp.capacity_reservation.tags #=> Array
    #   resp.capacity_reservation.tags[0].key #=> String
    #   resp.capacity_reservation.tags[0].value #=> String
    #   resp.capacity_reservation.outpost_arn #=> String
    #   resp.capacity_reservation.capacity_reservation_fleet_id #=> String
    #   resp.capacity_reservation.placement_group_arn #=> String
    #   resp.capacity_reservation.capacity_allocations #=> Array
    #   resp.capacity_reservation.capacity_allocations[0].allocation_type #=> String, one of "used", "future"
    #   resp.capacity_reservation.capacity_allocations[0].count #=> Integer
    #   resp.capacity_reservation.reservation_type #=> String, one of "default", "capacity-block"
    #   resp.capacity_reservation.unused_reservation_billing_owner_id #=> String
    #   resp.capacity_reservation.commitment_info.committed_instance_count #=> Integer
    #   resp.capacity_reservation.commitment_info.commitment_end_date #=> Time
    #   resp.capacity_reservation.delivery_preference #=> String, one of "fixed", "incremental"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/PurchaseCapacityBlock AWS API Documentation
    #
    # @overload purchase_capacity_block(params = {})
    # @param [Hash] params ({})
    def purchase_capacity_block(params = {}, options = {})
      req = build_request(:purchase_capacity_block, params)
      req.send_request(options)
    end

    # Purchase the Capacity Block extension for use with your account. You
    # must specify the ID of the Capacity Block extension offering you are
    # purchasing.
    #
    # @option params [required, String] :capacity_block_extension_offering_id
    #   The ID of the Capacity Block extension offering to purchase.
    #
    # @option params [required, String] :capacity_reservation_id
    #   The ID of the Capacity reservation to be extended.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::PurchaseCapacityBlockExtensionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::PurchaseCapacityBlockExtensionResult#capacity_block_extensions #capacity_block_extensions} => Array&lt;Types::CapacityBlockExtension&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.purchase_capacity_block_extension({
    #     capacity_block_extension_offering_id: "OfferingId", # required
    #     capacity_reservation_id: "CapacityReservationId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.capacity_block_extensions #=> Array
    #   resp.capacity_block_extensions[0].capacity_reservation_id #=> String
    #   resp.capacity_block_extensions[0].instance_type #=> String
    #   resp.capacity_block_extensions[0].instance_count #=> Integer
    #   resp.capacity_block_extensions[0].availability_zone #=> String
    #   resp.capacity_block_extensions[0].availability_zone_id #=> String
    #   resp.capacity_block_extensions[0].capacity_block_extension_offering_id #=> String
    #   resp.capacity_block_extensions[0].capacity_block_extension_duration_hours #=> Integer
    #   resp.capacity_block_extensions[0].capacity_block_extension_status #=> String, one of "payment-pending", "payment-failed", "payment-succeeded"
    #   resp.capacity_block_extensions[0].capacity_block_extension_purchase_date #=> Time
    #   resp.capacity_block_extensions[0].capacity_block_extension_start_date #=> Time
    #   resp.capacity_block_extensions[0].capacity_block_extension_end_date #=> Time
    #   resp.capacity_block_extensions[0].upfront_fee #=> String
    #   resp.capacity_block_extensions[0].currency_code #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/PurchaseCapacityBlockExtension AWS API Documentation
    #
    # @overload purchase_capacity_block_extension(params = {})
    # @param [Hash] params ({})
    def purchase_capacity_block_extension(params = {}, options = {})
      req = build_request(:purchase_capacity_block_extension, params)
      req.send_request(options)
    end

    # Purchase a reservation with configurations that match those of your
    # Dedicated Host. You must have active Dedicated Hosts in your account
    # before you purchase a reservation. This action results in the
    # specified reservation being purchased and charged to your account.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   Idempotency][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [String] :currency_code
    #   The currency in which the `totalUpfrontPrice`, `LimitPrice`, and
    #   `totalHourlyPrice` amounts are specified. At this time, the only
    #   supported currency is `USD`.
    #
    # @option params [required, Array<String>] :host_id_set
    #   The IDs of the Dedicated Hosts with which the reservation will be
    #   associated.
    #
    # @option params [String] :limit_price
    #   The specified limit is checked against the total upfront cost of the
    #   reservation (calculated as the offering's upfront cost multiplied by
    #   the host count). If the total upfront cost is greater than the
    #   specified price limit, the request fails. This is used to ensure that
    #   the purchase does not exceed the expected upfront cost of the
    #   purchase. At this time, the only supported currency is `USD`. For
    #   example, to indicate a limit price of USD 100, specify 100.00.
    #
    # @option params [required, String] :offering_id
    #   The ID of the offering.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the Dedicated Host Reservation during purchase.
    #
    # @return [Types::PurchaseHostReservationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::PurchaseHostReservationResult#client_token #client_token} => String
    #   * {Types::PurchaseHostReservationResult#currency_code #currency_code} => String
    #   * {Types::PurchaseHostReservationResult#purchase #purchase} => Array&lt;Types::Purchase&gt;
    #   * {Types::PurchaseHostReservationResult#total_hourly_price #total_hourly_price} => String
    #   * {Types::PurchaseHostReservationResult#total_upfront_price #total_upfront_price} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.purchase_host_reservation({
    #     client_token: "String",
    #     currency_code: "USD", # accepts USD
    #     host_id_set: ["DedicatedHostId"], # required
    #     limit_price: "String",
    #     offering_id: "OfferingId", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.client_token #=> String
    #   resp.currency_code #=> String, one of "USD"
    #   resp.purchase #=> Array
    #   resp.purchase[0].currency_code #=> String, one of "USD"
    #   resp.purchase[0].duration #=> Integer
    #   resp.purchase[0].host_id_set #=> Array
    #   resp.purchase[0].host_id_set[0] #=> String
    #   resp.purchase[0].host_reservation_id #=> String
    #   resp.purchase[0].hourly_price #=> String
    #   resp.purchase[0].instance_family #=> String
    #   resp.purchase[0].payment_option #=> String, one of "AllUpfront", "PartialUpfront", "NoUpfront"
    #   resp.purchase[0].upfront_price #=> String
    #   resp.total_hourly_price #=> String
    #   resp.total_upfront_price #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/PurchaseHostReservation AWS API Documentation
    #
    # @overload purchase_host_reservation(params = {})
    # @param [Hash] params ({})
    def purchase_host_reservation(params = {}, options = {})
      req = build_request(:purchase_host_reservation, params)
      req.send_request(options)
    end

    # Purchases a Reserved Instance for use with your account. With Reserved
    # Instances, you pay a lower hourly rate compared to On-Demand instance
    # pricing.
    #
    # Use DescribeReservedInstancesOfferings to get a list of Reserved
    # Instance offerings that match your specifications. After you've
    # purchased a Reserved Instance, you can check for your new Reserved
    # Instance with DescribeReservedInstances.
    #
    # To queue a purchase for a future date and time, specify a purchase
    # time. If you do not specify a purchase time, the default is the
    # current time.
    #
    # For more information, see [Reserved Instances][1] and [Sell in the
    # Reserved Instance Marketplace][2] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/concepts-on-demand-reserved-instances.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ri-market-general.html
    #
    # @option params [required, Integer] :instance_count
    #   The number of Reserved Instances to purchase.
    #
    # @option params [required, String] :reserved_instances_offering_id
    #   The ID of the Reserved Instance offering to purchase.
    #
    # @option params [Time,DateTime,Date,Integer,String] :purchase_time
    #   The time at which to purchase the Reserved Instance, in UTC format
    #   (for example, *YYYY*-*MM*-*DD*T*HH*:*MM*:*SS*Z).
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Types::ReservedInstanceLimitPrice] :limit_price
    #   Specified for Reserved Instance Marketplace offerings to limit the
    #   total order and ensure that the Reserved Instances are not purchased
    #   at unexpected prices.
    #
    # @return [Types::PurchaseReservedInstancesOfferingResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::PurchaseReservedInstancesOfferingResult#reserved_instances_id #reserved_instances_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.purchase_reserved_instances_offering({
    #     instance_count: 1, # required
    #     reserved_instances_offering_id: "ReservedInstancesOfferingId", # required
    #     purchase_time: Time.now,
    #     dry_run: false,
    #     limit_price: {
    #       amount: 1.0,
    #       currency_code: "USD", # accepts USD
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.reserved_instances_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/PurchaseReservedInstancesOffering AWS API Documentation
    #
    # @overload purchase_reserved_instances_offering(params = {})
    # @param [Hash] params ({})
    def purchase_reserved_instances_offering(params = {}, options = {})
      req = build_request(:purchase_reserved_instances_offering, params)
      req.send_request(options)
    end

    # <note markdown="1"> You can no longer purchase Scheduled Instances.
    #
    #  </note>
    #
    # Purchases the Scheduled Instances with the specified schedule.
    #
    # Scheduled Instances enable you to purchase Amazon EC2 compute capacity
    # by the hour for a one-year term. Before you can purchase a Scheduled
    # Instance, you must call DescribeScheduledInstanceAvailability to check
    # for available schedules and obtain a purchase token. After you
    # purchase a Scheduled Instance, you must call RunScheduledInstances
    # during each scheduled time period.
    #
    # After you purchase a Scheduled Instance, you can't cancel, modify, or
    # resell your purchase.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that ensures the idempotency of the
    #   request. For more information, see [Ensuring Idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<Types::PurchaseRequest>] :purchase_requests
    #   The purchase requests.
    #
    # @return [Types::PurchaseScheduledInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::PurchaseScheduledInstancesResult#scheduled_instance_set #scheduled_instance_set} => Array&lt;Types::ScheduledInstance&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.purchase_scheduled_instances({
    #     client_token: "String",
    #     dry_run: false,
    #     purchase_requests: [ # required
    #       {
    #         instance_count: 1, # required
    #         purchase_token: "String", # required
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.scheduled_instance_set #=> Array
    #   resp.scheduled_instance_set[0].availability_zone #=> String
    #   resp.scheduled_instance_set[0].create_date #=> Time
    #   resp.scheduled_instance_set[0].hourly_price #=> String
    #   resp.scheduled_instance_set[0].instance_count #=> Integer
    #   resp.scheduled_instance_set[0].instance_type #=> String
    #   resp.scheduled_instance_set[0].network_platform #=> String
    #   resp.scheduled_instance_set[0].next_slot_start_time #=> Time
    #   resp.scheduled_instance_set[0].platform #=> String
    #   resp.scheduled_instance_set[0].previous_slot_end_time #=> Time
    #   resp.scheduled_instance_set[0].recurrence.frequency #=> String
    #   resp.scheduled_instance_set[0].recurrence.interval #=> Integer
    #   resp.scheduled_instance_set[0].recurrence.occurrence_day_set #=> Array
    #   resp.scheduled_instance_set[0].recurrence.occurrence_day_set[0] #=> Integer
    #   resp.scheduled_instance_set[0].recurrence.occurrence_relative_to_end #=> Boolean
    #   resp.scheduled_instance_set[0].recurrence.occurrence_unit #=> String
    #   resp.scheduled_instance_set[0].scheduled_instance_id #=> String
    #   resp.scheduled_instance_set[0].slot_duration_in_hours #=> Integer
    #   resp.scheduled_instance_set[0].term_end_date #=> Time
    #   resp.scheduled_instance_set[0].term_start_date #=> Time
    #   resp.scheduled_instance_set[0].total_scheduled_instance_hours #=> Integer
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/PurchaseScheduledInstances AWS API Documentation
    #
    # @overload purchase_scheduled_instances(params = {})
    # @param [Hash] params ({})
    def purchase_scheduled_instances(params = {}, options = {})
      req = build_request(:purchase_scheduled_instances, params)
      req.send_request(options)
    end

    # Requests a reboot of the specified instances. This operation is
    # asynchronous; it only queues a request to reboot the specified
    # instances. The operation succeeds if the instances are valid and
    # belong to you. Requests to reboot terminated instances are ignored.
    #
    # If an instance does not cleanly shut down within a few minutes, Amazon
    # EC2 performs a hard reboot.
    #
    # For more information about troubleshooting, see [Troubleshoot an
    # unreachable instance][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-console.html
    #
    # @option params [required, Array<String>] :instance_ids
    #   The instance IDs.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To reboot an EC2 instance
    #
    #   # This example reboots the specified EC2 instance.
    #
    #   resp = client.reboot_instances({
    #     instance_ids: [
    #       "i-1234567890abcdef5", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.reboot_instances({
    #     instance_ids: ["InstanceId"], # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RebootInstances AWS API Documentation
    #
    # @overload reboot_instances(params = {})
    # @param [Hash] params ({})
    def reboot_instances(params = {}, options = {})
      req = build_request(:reboot_instances, params)
      req.send_request(options)
    end

    # Registers an AMI. When you're creating an instance-store backed AMI,
    # registering the AMI is the final step in the creation process. For
    # more information about creating AMIs, see [Create an AMI from a
    # snapshot][1] and [Create an instance-store backed AMI][2] in the
    # *Amazon EC2 User Guide*.
    #
    # <note markdown="1"> For Amazon EBS-backed instances, CreateImage creates and registers the
    # AMI in a single request, so you don't have to register the AMI
    # yourself. We recommend that you always use CreateImage unless you have
    # a specific reason to use RegisterImage.
    #
    #  </note>
    #
    # If needed, you can deregister an AMI at any time. Any modifications
    # you make to an AMI backed by an instance store volume invalidates its
    # registration. If you make changes to an image, deregister the previous
    # image and register the new image.
    #
    # **Register a snapshot of a root device volume**
    #
    # You can use `RegisterImage` to create an Amazon EBS-backed Linux AMI
    # from a snapshot of a root device volume. You specify the snapshot
    # using a block device mapping. You can't set the encryption state of
    # the volume using the block device mapping. If the snapshot is
    # encrypted, or encryption by default is enabled, the root volume of an
    # instance launched from the AMI is encrypted.
    #
    # For more information, see [Create an AMI from a snapshot][1] and [Use
    # encryption with Amazon EBS-backed AMIs][3] in the *Amazon EC2 User
    # Guide*.
    #
    # **Amazon Web Services Marketplace product codes**
    #
    # If any snapshots have Amazon Web Services Marketplace product codes,
    # they are copied to the new AMI.
    #
    # In most cases, AMIs for Windows, RedHat, SUSE, and SQL Server require
    # correct licensing information to be present on the AMI. For more
    # information, see [Understand AMI billing information][4] in the
    # *Amazon EC2 User Guide*. When creating an AMI from a snapshot, the
    # `RegisterImage` operation derives the correct billing information from
    # the snapshot's metadata, but this requires the appropriate metadata
    # to be present. To verify if the correct billing information was
    # applied, check the `PlatformDetails` field on the new AMI. If the
    # field is empty or doesn't match the expected operating system code
    # (for example, Windows, RedHat, SUSE, or SQL), the AMI creation was
    # unsuccessful, and you should discard the AMI and instead create the
    # AMI from an instance using CreateImage. For more information, see
    # [Create an AMI from an instance ][5] in the *Amazon EC2 User Guide*.
    #
    # If you purchase a Reserved Instance to apply to an On-Demand Instance
    # that was launched from an AMI with a billing product code, make sure
    # that the Reserved Instance has the matching billing product code. If
    # you purchase a Reserved Instance without the matching billing product
    # code, the Reserved Instance will not be applied to the On-Demand
    # Instance. For information about how to obtain the platform details and
    # billing information of an AMI, see [Understand AMI billing
    # information][4] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/creating-an-ami-ebs.html#creating-launching-ami-from-snapshot
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/creating-an-ami-instance-store.html
    # [3]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/AMIEncryption.html
    # [4]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ami-billing-info.html
    # [5]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/creating-an-ami-ebs.html#how-to-create-ebs-ami
    #
    # @option params [String] :image_location
    #   The full path to your AMI manifest in Amazon S3 storage. The specified
    #   bucket must have the `aws-exec-read` canned access control list (ACL)
    #   to ensure that it can be accessed by Amazon EC2. For more information,
    #   see [Canned ACLs][1] in the *Amazon S3 Service Developer Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AmazonS3/latest/dev/acl-overview.html#canned-acl
    #
    # @option params [Array<String>] :billing_products
    #   The billing product codes. Your account must be authorized to specify
    #   billing product codes.
    #
    #   If your account is not authorized to specify billing product codes,
    #   you can publish AMIs that include billable software and list them on
    #   the Amazon Web Services Marketplace. You must first register as a
    #   seller on the Amazon Web Services Marketplace. For more information,
    #   see [Getting started as a seller][1] and [AMI-based products][2] in
    #   the *Amazon Web Services Marketplace Seller Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/marketplace/latest/userguide/user-guide-for-sellers.html
    #   [2]: https://docs.aws.amazon.com/marketplace/latest/userguide/ami-products.html
    #
    # @option params [String] :boot_mode
    #   The boot mode of the AMI. A value of `uefi-preferred` indicates that
    #   the AMI supports both UEFI and Legacy BIOS.
    #
    #   <note markdown="1"> The operating system contained in the AMI must be configured to
    #   support the specified boot mode.
    #
    #    </note>
    #
    #   For more information, see [Boot modes][1] in the *Amazon EC2 User
    #   Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ami-boot.html
    #
    # @option params [String] :tpm_support
    #   Set to `v2.0` to enable Trusted Platform Module (TPM) support. For
    #   more information, see [NitroTPM][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/nitrotpm.html
    #
    # @option params [String] :uefi_data
    #   Base64 representation of the non-volatile UEFI variable store. To
    #   retrieve the UEFI data, use the [GetInstanceUefiData][1] command. You
    #   can inspect and modify the UEFI data by using the [python-uefivars
    #   tool][2] on GitHub. For more information, see [UEFI Secure Boot][3] in
    #   the *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_GetInstanceUefiData
    #   [2]: https://github.com/awslabs/python-uefivars
    #   [3]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/uefi-secure-boot.html
    #
    # @option params [String] :imds_support
    #   Set to `v2.0` to indicate that IMDSv2 is specified in the AMI.
    #   Instances launched from this AMI will have `HttpTokens` automatically
    #   set to `required` so that, by default, the instance requires that
    #   IMDSv2 is used when requesting instance metadata. In addition,
    #   `HttpPutResponseHopLimit` is set to `2`. For more information, see
    #   [Configure the AMI][1] in the *Amazon EC2 User Guide*.
    #
    #   <note markdown="1"> If you set the value to `v2.0`, make sure that your AMI software can
    #   support IMDSv2.
    #
    #    </note>
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/configuring-IMDS-new-instances.html#configure-IMDS-new-instances-ami-configuration
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the AMI.
    #
    #   To tag the AMI, the value for `ResourceType` must be `image`. If you
    #   specify another value for `ResourceType`, the request fails.
    #
    #   To tag an AMI after it has been registered, see [CreateTags][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTags.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :name
    #   A name for your AMI.
    #
    #   Constraints: 3-128 alphanumeric characters, parentheses (()), square
    #   brackets (\[\]), spaces ( ), periods (.), slashes (/), dashes (-),
    #   single quotes ('), at-signs (@), or underscores(\_)
    #
    # @option params [String] :description
    #   A description for your AMI.
    #
    # @option params [String] :architecture
    #   The architecture of the AMI.
    #
    #   Default: For Amazon EBS-backed AMIs, `i386`. For instance store-backed
    #   AMIs, the architecture specified in the manifest file.
    #
    # @option params [String] :kernel_id
    #   The ID of the kernel.
    #
    # @option params [String] :ramdisk_id
    #   The ID of the RAM disk.
    #
    # @option params [String] :root_device_name
    #   The device name of the root device volume (for example, `/dev/sda1`).
    #
    # @option params [Array<Types::BlockDeviceMapping>] :block_device_mappings
    #   The block device mapping entries.
    #
    #   If you specify an Amazon EBS volume using the ID of an Amazon EBS
    #   snapshot, you can't specify the encryption state of the volume.
    #
    #   If you create an AMI on an Outpost, then all backing snapshots must be
    #   on the same Outpost or in the Region of that Outpost. AMIs on an
    #   Outpost that include local snapshots can be used to launch instances
    #   on the same Outpost only. For more information, [Amazon EBS local
    #   snapshots on Outposts][1] in the *Amazon EBS User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ebs/latest/userguide/snapshots-outposts.html#ami
    #
    # @option params [String] :virtualization_type
    #   The type of virtualization (`hvm` \| `paravirtual`).
    #
    #   Default: `paravirtual`
    #
    # @option params [String] :sriov_net_support
    #   Set to `simple` to enable enhanced networking with the Intel 82599
    #   Virtual Function interface for the AMI and any instances that you
    #   launch from the AMI.
    #
    #   There is no way to disable `sriovNetSupport` at this time.
    #
    #   This option is supported only for HVM AMIs. Specifying this option
    #   with a PV AMI can make instances launched from the AMI unreachable.
    #
    # @option params [Boolean] :ena_support
    #   Set to `true` to enable enhanced networking with ENA for the AMI and
    #   any instances that you launch from the AMI.
    #
    #   This option is supported only for HVM AMIs. Specifying this option
    #   with a PV AMI can make instances launched from the AMI unreachable.
    #
    # @return [Types::RegisterImageResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RegisterImageResult#image_id #image_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.register_image({
    #     image_location: "String",
    #     billing_products: ["String"],
    #     boot_mode: "legacy-bios", # accepts legacy-bios, uefi, uefi-preferred
    #     tpm_support: "v2.0", # accepts v2.0
    #     uefi_data: "StringType",
    #     imds_support: "v2.0", # accepts v2.0
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     dry_run: false,
    #     name: "String", # required
    #     description: "String",
    #     architecture: "i386", # accepts i386, x86_64, arm64, x86_64_mac, arm64_mac
    #     kernel_id: "KernelId",
    #     ramdisk_id: "RamdiskId",
    #     root_device_name: "String",
    #     block_device_mappings: [
    #       {
    #         ebs: {
    #           delete_on_termination: false,
    #           iops: 1,
    #           snapshot_id: "SnapshotId",
    #           volume_size: 1,
    #           volume_type: "standard", # accepts standard, io1, io2, gp2, sc1, st1, gp3
    #           kms_key_id: "String",
    #           throughput: 1,
    #           outpost_arn: "String",
    #           encrypted: false,
    #         },
    #         no_device: "String",
    #         device_name: "String",
    #         virtual_name: "String",
    #       },
    #     ],
    #     virtualization_type: "String",
    #     sriov_net_support: "String",
    #     ena_support: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.image_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RegisterImage AWS API Documentation
    #
    # @overload register_image(params = {})
    # @param [Hash] params ({})
    def register_image(params = {}, options = {})
      req = build_request(:register_image, params)
      req.send_request(options)
    end

    # Registers a set of tag keys to include in scheduled event
    # notifications for your resources.
    #
    # To remove tags, use
    # [DeregisterInstanceEventNotificationAttributes][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_DeregisterInstanceEventNotificationAttributes.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Types::RegisterInstanceTagAttributeRequest] :instance_tag_attribute
    #   Information about the tag keys to register.
    #
    # @return [Types::RegisterInstanceEventNotificationAttributesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RegisterInstanceEventNotificationAttributesResult#instance_tag_attribute #instance_tag_attribute} => Types::InstanceTagNotificationAttribute
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.register_instance_event_notification_attributes({
    #     dry_run: false,
    #     instance_tag_attribute: { # required
    #       include_all_tags_of_instance: false,
    #       instance_tag_keys: ["String"],
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_tag_attribute.instance_tag_keys #=> Array
    #   resp.instance_tag_attribute.instance_tag_keys[0] #=> String
    #   resp.instance_tag_attribute.include_all_tags_of_instance #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RegisterInstanceEventNotificationAttributes AWS API Documentation
    #
    # @overload register_instance_event_notification_attributes(params = {})
    # @param [Hash] params ({})
    def register_instance_event_notification_attributes(params = {}, options = {})
      req = build_request(:register_instance_event_notification_attributes, params)
      req.send_request(options)
    end

    # Registers members (network interfaces) with the transit gateway
    # multicast group. A member is a network interface associated with a
    # supported EC2 instance that receives multicast traffic. For more
    # information, see [Multicast on transit gateways][1] in the *Amazon Web
    # Services Transit Gateways Guide*.
    #
    # After you add the members, use
    # [SearchTransitGatewayMulticastGroups][2] to verify that the members
    # were added to the transit gateway multicast group.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/tgw/tgw-multicast-overview.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_SearchTransitGatewayMulticastGroups.html
    #
    # @option params [required, String] :transit_gateway_multicast_domain_id
    #   The ID of the transit gateway multicast domain.
    #
    # @option params [String] :group_ip_address
    #   The IP address assigned to the transit gateway multicast group.
    #
    # @option params [required, Array<String>] :network_interface_ids
    #   The group members' network interface IDs to register with the transit
    #   gateway multicast group.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::RegisterTransitGatewayMulticastGroupMembersResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RegisterTransitGatewayMulticastGroupMembersResult#registered_multicast_group_members #registered_multicast_group_members} => Types::TransitGatewayMulticastRegisteredGroupMembers
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.register_transit_gateway_multicast_group_members({
    #     transit_gateway_multicast_domain_id: "TransitGatewayMulticastDomainId", # required
    #     group_ip_address: "String",
    #     network_interface_ids: ["NetworkInterfaceId"], # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.registered_multicast_group_members.transit_gateway_multicast_domain_id #=> String
    #   resp.registered_multicast_group_members.registered_network_interface_ids #=> Array
    #   resp.registered_multicast_group_members.registered_network_interface_ids[0] #=> String
    #   resp.registered_multicast_group_members.group_ip_address #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RegisterTransitGatewayMulticastGroupMembers AWS API Documentation
    #
    # @overload register_transit_gateway_multicast_group_members(params = {})
    # @param [Hash] params ({})
    def register_transit_gateway_multicast_group_members(params = {}, options = {})
      req = build_request(:register_transit_gateway_multicast_group_members, params)
      req.send_request(options)
    end

    # Registers sources (network interfaces) with the specified transit
    # gateway multicast group.
    #
    # A multicast source is a network interface attached to a supported
    # instance that sends multicast traffic. For more information about
    # supported instances, see [Multicast on transit gateways][1] in the
    # *Amazon Web Services Transit Gateways Guide*.
    #
    # After you add the source, use [SearchTransitGatewayMulticastGroups][2]
    # to verify that the source was added to the multicast group.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/tgw/tgw-multicast-overview.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_SearchTransitGatewayMulticastGroups.html
    #
    # @option params [required, String] :transit_gateway_multicast_domain_id
    #   The ID of the transit gateway multicast domain.
    #
    # @option params [String] :group_ip_address
    #   The IP address assigned to the transit gateway multicast group.
    #
    # @option params [required, Array<String>] :network_interface_ids
    #   The group sources' network interface IDs to register with the transit
    #   gateway multicast group.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::RegisterTransitGatewayMulticastGroupSourcesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RegisterTransitGatewayMulticastGroupSourcesResult#registered_multicast_group_sources #registered_multicast_group_sources} => Types::TransitGatewayMulticastRegisteredGroupSources
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.register_transit_gateway_multicast_group_sources({
    #     transit_gateway_multicast_domain_id: "TransitGatewayMulticastDomainId", # required
    #     group_ip_address: "String",
    #     network_interface_ids: ["NetworkInterfaceId"], # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.registered_multicast_group_sources.transit_gateway_multicast_domain_id #=> String
    #   resp.registered_multicast_group_sources.registered_network_interface_ids #=> Array
    #   resp.registered_multicast_group_sources.registered_network_interface_ids[0] #=> String
    #   resp.registered_multicast_group_sources.group_ip_address #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RegisterTransitGatewayMulticastGroupSources AWS API Documentation
    #
    # @overload register_transit_gateway_multicast_group_sources(params = {})
    # @param [Hash] params ({})
    def register_transit_gateway_multicast_group_sources(params = {}, options = {})
      req = build_request(:register_transit_gateway_multicast_group_sources, params)
      req.send_request(options)
    end

    # Rejects a request to assign billing of the available capacity of a
    # shared Capacity Reservation to your account. For more information, see
    # [ Billing assignment for shared Amazon EC2 Capacity Reservations][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/assign-billing.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :capacity_reservation_id
    #   The ID of the Capacity Reservation for which to reject the request.
    #
    # @return [Types::RejectCapacityReservationBillingOwnershipResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RejectCapacityReservationBillingOwnershipResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.reject_capacity_reservation_billing_ownership({
    #     dry_run: false,
    #     capacity_reservation_id: "CapacityReservationId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RejectCapacityReservationBillingOwnership AWS API Documentation
    #
    # @overload reject_capacity_reservation_billing_ownership(params = {})
    # @param [Hash] params ({})
    def reject_capacity_reservation_billing_ownership(params = {}, options = {})
      req = build_request(:reject_capacity_reservation_billing_ownership, params)
      req.send_request(options)
    end

    # Rejects a request to associate cross-account subnets with a transit
    # gateway multicast domain.
    #
    # @option params [String] :transit_gateway_multicast_domain_id
    #   The ID of the transit gateway multicast domain.
    #
    # @option params [String] :transit_gateway_attachment_id
    #   The ID of the transit gateway attachment.
    #
    # @option params [Array<String>] :subnet_ids
    #   The IDs of the subnets to associate with the transit gateway multicast
    #   domain.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::RejectTransitGatewayMulticastDomainAssociationsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RejectTransitGatewayMulticastDomainAssociationsResult#associations #associations} => Types::TransitGatewayMulticastDomainAssociations
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.reject_transit_gateway_multicast_domain_associations({
    #     transit_gateway_multicast_domain_id: "TransitGatewayMulticastDomainId",
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId",
    #     subnet_ids: ["String"],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.associations.transit_gateway_multicast_domain_id #=> String
    #   resp.associations.transit_gateway_attachment_id #=> String
    #   resp.associations.resource_id #=> String
    #   resp.associations.resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.associations.resource_owner_id #=> String
    #   resp.associations.subnets #=> Array
    #   resp.associations.subnets[0].subnet_id #=> String
    #   resp.associations.subnets[0].state #=> String, one of "pendingAcceptance", "associating", "associated", "disassociating", "disassociated", "rejected", "failed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RejectTransitGatewayMulticastDomainAssociations AWS API Documentation
    #
    # @overload reject_transit_gateway_multicast_domain_associations(params = {})
    # @param [Hash] params ({})
    def reject_transit_gateway_multicast_domain_associations(params = {}, options = {})
      req = build_request(:reject_transit_gateway_multicast_domain_associations, params)
      req.send_request(options)
    end

    # Rejects a transit gateway peering attachment request.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the transit gateway peering attachment.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::RejectTransitGatewayPeeringAttachmentResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RejectTransitGatewayPeeringAttachmentResult#transit_gateway_peering_attachment #transit_gateway_peering_attachment} => Types::TransitGatewayPeeringAttachment
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.reject_transit_gateway_peering_attachment({
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_peering_attachment.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.transit_gateway_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.core_network_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.owner_id #=> String
    #   resp.transit_gateway_peering_attachment.requester_tgw_info.region #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.transit_gateway_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.core_network_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.owner_id #=> String
    #   resp.transit_gateway_peering_attachment.accepter_tgw_info.region #=> String
    #   resp.transit_gateway_peering_attachment.options.dynamic_routing #=> String, one of "enable", "disable"
    #   resp.transit_gateway_peering_attachment.status.code #=> String
    #   resp.transit_gateway_peering_attachment.status.message #=> String
    #   resp.transit_gateway_peering_attachment.state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_peering_attachment.creation_time #=> Time
    #   resp.transit_gateway_peering_attachment.tags #=> Array
    #   resp.transit_gateway_peering_attachment.tags[0].key #=> String
    #   resp.transit_gateway_peering_attachment.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RejectTransitGatewayPeeringAttachment AWS API Documentation
    #
    # @overload reject_transit_gateway_peering_attachment(params = {})
    # @param [Hash] params ({})
    def reject_transit_gateway_peering_attachment(params = {}, options = {})
      req = build_request(:reject_transit_gateway_peering_attachment, params)
      req.send_request(options)
    end

    # Rejects a request to attach a VPC to a transit gateway.
    #
    # The VPC attachment must be in the `pendingAcceptance` state. Use
    # DescribeTransitGatewayVpcAttachments to view your pending VPC
    # attachment requests. Use AcceptTransitGatewayVpcAttachment to accept a
    # VPC attachment request.
    #
    # @option params [required, String] :transit_gateway_attachment_id
    #   The ID of the attachment.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::RejectTransitGatewayVpcAttachmentResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RejectTransitGatewayVpcAttachmentResult#transit_gateway_vpc_attachment #transit_gateway_vpc_attachment} => Types::TransitGatewayVpcAttachment
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.reject_transit_gateway_vpc_attachment({
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.transit_gateway_vpc_attachment.transit_gateway_attachment_id #=> String
    #   resp.transit_gateway_vpc_attachment.transit_gateway_id #=> String
    #   resp.transit_gateway_vpc_attachment.vpc_id #=> String
    #   resp.transit_gateway_vpc_attachment.vpc_owner_id #=> String
    #   resp.transit_gateway_vpc_attachment.state #=> String, one of "initiating", "initiatingRequest", "pendingAcceptance", "rollingBack", "pending", "available", "modifying", "deleting", "deleted", "failed", "rejected", "rejecting", "failing"
    #   resp.transit_gateway_vpc_attachment.subnet_ids #=> Array
    #   resp.transit_gateway_vpc_attachment.subnet_ids[0] #=> String
    #   resp.transit_gateway_vpc_attachment.creation_time #=> Time
    #   resp.transit_gateway_vpc_attachment.options.dns_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.security_group_referencing_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.ipv_6_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.options.appliance_mode_support #=> String, one of "enable", "disable"
    #   resp.transit_gateway_vpc_attachment.tags #=> Array
    #   resp.transit_gateway_vpc_attachment.tags[0].key #=> String
    #   resp.transit_gateway_vpc_attachment.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RejectTransitGatewayVpcAttachment AWS API Documentation
    #
    # @overload reject_transit_gateway_vpc_attachment(params = {})
    # @param [Hash] params ({})
    def reject_transit_gateway_vpc_attachment(params = {}, options = {})
      req = build_request(:reject_transit_gateway_vpc_attachment, params)
      req.send_request(options)
    end

    # Rejects VPC endpoint connection requests to your VPC endpoint service.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :service_id
    #   The ID of the service.
    #
    # @option params [required, Array<String>] :vpc_endpoint_ids
    #   The IDs of the VPC endpoints.
    #
    # @return [Types::RejectVpcEndpointConnectionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RejectVpcEndpointConnectionsResult#unsuccessful #unsuccessful} => Array&lt;Types::UnsuccessfulItem&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.reject_vpc_endpoint_connections({
    #     dry_run: false,
    #     service_id: "VpcEndpointServiceId", # required
    #     vpc_endpoint_ids: ["VpcEndpointId"], # required
    #   })
    #
    # @example Response structure
    #
    #   resp.unsuccessful #=> Array
    #   resp.unsuccessful[0].error.code #=> String
    #   resp.unsuccessful[0].error.message #=> String
    #   resp.unsuccessful[0].resource_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RejectVpcEndpointConnections AWS API Documentation
    #
    # @overload reject_vpc_endpoint_connections(params = {})
    # @param [Hash] params ({})
    def reject_vpc_endpoint_connections(params = {}, options = {})
      req = build_request(:reject_vpc_endpoint_connections, params)
      req.send_request(options)
    end

    # Rejects a VPC peering connection request. The VPC peering connection
    # must be in the `pending-acceptance` state. Use the
    # DescribeVpcPeeringConnections request to view your outstanding VPC
    # peering connection requests. To delete an active VPC peering
    # connection, or to delete a VPC peering connection request that you
    # initiated, use DeleteVpcPeeringConnection.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :vpc_peering_connection_id
    #   The ID of the VPC peering connection.
    #
    # @return [Types::RejectVpcPeeringConnectionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RejectVpcPeeringConnectionResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.reject_vpc_peering_connection({
    #     dry_run: false,
    #     vpc_peering_connection_id: "VpcPeeringConnectionId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RejectVpcPeeringConnection AWS API Documentation
    #
    # @overload reject_vpc_peering_connection(params = {})
    # @param [Hash] params ({})
    def reject_vpc_peering_connection(params = {}, options = {})
      req = build_request(:reject_vpc_peering_connection, params)
      req.send_request(options)
    end

    # Releases the specified Elastic IP address.
    #
    # \[Default VPC\] Releasing an Elastic IP address automatically
    # disassociates it from any instance that it's associated with. To
    # disassociate an Elastic IP address without releasing it, use
    # DisassociateAddress.
    #
    # \[Nondefault VPC\] You must use DisassociateAddress to disassociate
    # the Elastic IP address before you can release it. Otherwise, Amazon
    # EC2 returns an error (`InvalidIPAddress.InUse`).
    #
    # After releasing an Elastic IP address, it is released to the IP
    # address pool. Be sure to update your DNS records and any servers or
    # devices that communicate with the address. If you attempt to release
    # an Elastic IP address that you already released, you'll get an
    # `AuthFailure` error if the address is already allocated to another
    # Amazon Web Services account.
    #
    # After you release an Elastic IP address, you might be able to recover
    # it. For more information, see AllocateAddress.
    #
    # @option params [String] :allocation_id
    #   The allocation ID. This parameter is required.
    #
    # @option params [String] :public_ip
    #   Deprecated.
    #
    # @option params [String] :network_border_group
    #   The set of Availability Zones, Local Zones, or Wavelength Zones from
    #   which Amazon Web Services advertises IP addresses.
    #
    #   If you provide an incorrect network border group, you receive an
    #   `InvalidAddress.NotFound` error.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To release an Elastic IP address
    #
    #   # This example releases the specified Elastic IP address.
    #
    #   resp = client.release_address({
    #     allocation_id: "eipalloc-64d5890a", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.release_address({
    #     allocation_id: "AllocationId",
    #     public_ip: "String",
    #     network_border_group: "String",
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ReleaseAddress AWS API Documentation
    #
    # @overload release_address(params = {})
    # @param [Hash] params ({})
    def release_address(params = {}, options = {})
      req = build_request(:release_address, params)
      req.send_request(options)
    end

    # When you no longer want to use an On-Demand Dedicated Host it can be
    # released. On-Demand billing is stopped and the host goes into
    # `released` state. The host ID of Dedicated Hosts that have been
    # released can no longer be specified in another request, for example,
    # to modify the host. You must stop or terminate all instances on a host
    # before it can be released.
    #
    # When Dedicated Hosts are released, it may take some time for them to
    # stop counting toward your limit and you may receive capacity errors
    # when trying to allocate new Dedicated Hosts. Wait a few minutes and
    # then try again.
    #
    # Released hosts still appear in a DescribeHosts response.
    #
    # @option params [required, Array<String>] :host_ids
    #   The IDs of the Dedicated Hosts to release.
    #
    # @return [Types::ReleaseHostsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ReleaseHostsResult#successful #successful} => Array&lt;String&gt;
    #   * {Types::ReleaseHostsResult#unsuccessful #unsuccessful} => Array&lt;Types::UnsuccessfulItem&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.release_hosts({
    #     host_ids: ["DedicatedHostId"], # required
    #   })
    #
    # @example Response structure
    #
    #   resp.successful #=> Array
    #   resp.successful[0] #=> String
    #   resp.unsuccessful #=> Array
    #   resp.unsuccessful[0].error.code #=> String
    #   resp.unsuccessful[0].error.message #=> String
    #   resp.unsuccessful[0].resource_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ReleaseHosts AWS API Documentation
    #
    # @overload release_hosts(params = {})
    # @param [Hash] params ({})
    def release_hosts(params = {}, options = {})
      req = build_request(:release_hosts, params)
      req.send_request(options)
    end

    # Release an allocation within an IPAM pool. The Region you use should
    # be the IPAM pool locale. The locale is the Amazon Web Services Region
    # where this IPAM pool is available for allocations. You can only use
    # this action to release manual allocations. To remove an allocation for
    # a resource without deleting the resource, set its monitored state to
    # false using [ModifyIpamResourceCidr][1]. For more information, see
    # [Release an allocation][2] in the *Amazon VPC IPAM User Guide*.
    #
    # <note markdown="1"> All EC2 API actions follow an [eventual consistency][3] model.
    #
    #  </note>
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ModifyIpamResourceCidr.html
    # [2]: https://docs.aws.amazon.com/vpc/latest/ipam/release-alloc-ipam.html
    # [3]: https://docs.aws.amazon.com/ec2/latest/devguide/eventual-consistency.html
    #
    # @option params [Boolean] :dry_run
    #   A check for whether you have the required permissions for the action
    #   without actually making the request and provides an error response. If
    #   you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :ipam_pool_id
    #   The ID of the IPAM pool which contains the allocation you want to
    #   release.
    #
    # @option params [required, String] :cidr
    #   The CIDR of the allocation you want to release.
    #
    # @option params [required, String] :ipam_pool_allocation_id
    #   The ID of the allocation.
    #
    # @return [Types::ReleaseIpamPoolAllocationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ReleaseIpamPoolAllocationResult#success #success} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.release_ipam_pool_allocation({
    #     dry_run: false,
    #     ipam_pool_id: "IpamPoolId", # required
    #     cidr: "String", # required
    #     ipam_pool_allocation_id: "IpamPoolAllocationId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.success #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ReleaseIpamPoolAllocation AWS API Documentation
    #
    # @overload release_ipam_pool_allocation(params = {})
    # @param [Hash] params ({})
    def release_ipam_pool_allocation(params = {}, options = {})
      req = build_request(:release_ipam_pool_allocation, params)
      req.send_request(options)
    end

    # Replaces an IAM instance profile for the specified running instance.
    # You can use this action to change the IAM instance profile that's
    # associated with an instance without having to disassociate the
    # existing IAM instance profile first.
    #
    # Use DescribeIamInstanceProfileAssociations to get the association ID.
    #
    # @option params [required, Types::IamInstanceProfileSpecification] :iam_instance_profile
    #   The IAM instance profile.
    #
    # @option params [required, String] :association_id
    #   The ID of the existing IAM instance profile association.
    #
    # @return [Types::ReplaceIamInstanceProfileAssociationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ReplaceIamInstanceProfileAssociationResult#iam_instance_profile_association #iam_instance_profile_association} => Types::IamInstanceProfileAssociation
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.replace_iam_instance_profile_association({
    #     iam_instance_profile: { # required
    #       arn: "String",
    #       name: "String",
    #     },
    #     association_id: "IamInstanceProfileAssociationId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.iam_instance_profile_association.association_id #=> String
    #   resp.iam_instance_profile_association.instance_id #=> String
    #   resp.iam_instance_profile_association.iam_instance_profile.arn #=> String
    #   resp.iam_instance_profile_association.iam_instance_profile.id #=> String
    #   resp.iam_instance_profile_association.state #=> String, one of "associating", "associated", "disassociating", "disassociated"
    #   resp.iam_instance_profile_association.timestamp #=> Time
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ReplaceIamInstanceProfileAssociation AWS API Documentation
    #
    # @overload replace_iam_instance_profile_association(params = {})
    # @param [Hash] params ({})
    def replace_iam_instance_profile_association(params = {}, options = {})
      req = build_request(:replace_iam_instance_profile_association, params)
      req.send_request(options)
    end

    # Sets or replaces the criteria for Allowed AMIs.
    #
    # <note markdown="1"> The Allowed AMIs feature does not restrict the AMIs owned by your
    # account. Regardless of the criteria you set, the AMIs created by your
    # account will always be discoverable and usable by users in your
    # account.
    #
    #  </note>
    #
    # For more information, see [Control the discovery and use of AMIs in
    # Amazon EC2 with Allowed AMIs][1] in *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-allowed-amis.html
    #
    # @option params [Array<Types::ImageCriterionRequest>] :image_criteria
    #   The list of criteria that are evaluated to determine whether AMIs are
    #   discoverable and usable in the account in the specified Amazon Web
    #   Services Region.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ReplaceImageCriteriaInAllowedImagesSettingsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ReplaceImageCriteriaInAllowedImagesSettingsResult#return_value #return_value} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.replace_image_criteria_in_allowed_images_settings({
    #     image_criteria: [
    #       {
    #         image_providers: ["ImageProviderRequest"],
    #       },
    #     ],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return_value #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ReplaceImageCriteriaInAllowedImagesSettings AWS API Documentation
    #
    # @overload replace_image_criteria_in_allowed_images_settings(params = {})
    # @param [Hash] params ({})
    def replace_image_criteria_in_allowed_images_settings(params = {}, options = {})
      req = build_request(:replace_image_criteria_in_allowed_images_settings, params)
      req.send_request(options)
    end

    # Changes which network ACL a subnet is associated with. By default when
    # you create a subnet, it's automatically associated with the default
    # network ACL. For more information, see [Network ACLs][1] in the
    # *Amazon VPC User Guide*.
    #
    # This is an idempotent operation.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/vpc-network-acls.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :association_id
    #   The ID of the current association between the original network ACL and
    #   the subnet.
    #
    # @option params [required, String] :network_acl_id
    #   The ID of the new network ACL to associate with the subnet.
    #
    # @return [Types::ReplaceNetworkAclAssociationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ReplaceNetworkAclAssociationResult#new_association_id #new_association_id} => String
    #
    #
    # @example Example: To replace the network ACL associated with a subnet
    #
    #   # This example associates the specified network ACL with the subnet for the specified network ACL association.
    #
    #   resp = client.replace_network_acl_association({
    #     association_id: "aclassoc-e5b95c8c", 
    #     network_acl_id: "acl-5fb85d36", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     new_association_id: "aclassoc-3999875b", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.replace_network_acl_association({
    #     dry_run: false,
    #     association_id: "NetworkAclAssociationId", # required
    #     network_acl_id: "NetworkAclId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.new_association_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ReplaceNetworkAclAssociation AWS API Documentation
    #
    # @overload replace_network_acl_association(params = {})
    # @param [Hash] params ({})
    def replace_network_acl_association(params = {}, options = {})
      req = build_request(:replace_network_acl_association, params)
      req.send_request(options)
    end

    # Replaces an entry (rule) in a network ACL. For more information, see
    # [Network ACLs][1] in the *Amazon VPC User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/vpc-network-acls.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :network_acl_id
    #   The ID of the ACL.
    #
    # @option params [required, Integer] :rule_number
    #   The rule number of the entry to replace.
    #
    # @option params [required, String] :protocol
    #   The protocol number. A value of "-1" means all protocols. If you
    #   specify "-1" or a protocol number other than "6" (TCP), "17"
    #   (UDP), or "1" (ICMP), traffic on all ports is allowed, regardless of
    #   any ports or ICMP types or codes that you specify. If you specify
    #   protocol "58" (ICMPv6) and specify an IPv4 CIDR block, traffic for
    #   all ICMP types and codes allowed, regardless of any that you specify.
    #   If you specify protocol "58" (ICMPv6) and specify an IPv6 CIDR
    #   block, you must specify an ICMP type and code.
    #
    # @option params [required, String] :rule_action
    #   Indicates whether to allow or deny the traffic that matches the rule.
    #
    # @option params [required, Boolean] :egress
    #   Indicates whether to replace the egress rule.
    #
    #   Default: If no value is specified, we replace the ingress rule.
    #
    # @option params [String] :cidr_block
    #   The IPv4 network range to allow or deny, in CIDR notation (for example
    #   `172.16.0.0/24`).
    #
    # @option params [String] :ipv_6_cidr_block
    #   The IPv6 network range to allow or deny, in CIDR notation (for example
    #   `2001:bd8:1234:1a00::/64`).
    #
    # @option params [Types::IcmpTypeCode] :icmp_type_code
    #   ICMP protocol: The ICMP or ICMPv6 type and code. Required if
    #   specifying protocol 1 (ICMP) or protocol 58 (ICMPv6) with an IPv6 CIDR
    #   block.
    #
    # @option params [Types::PortRange] :port_range
    #   TCP or UDP protocols: The range of ports the rule applies to. Required
    #   if specifying protocol 6 (TCP) or 17 (UDP).
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To replace a network ACL entry
    #
    #   # This example replaces an entry for the specified network ACL. The new rule 100 allows ingress traffic from
    #   # 203.0.113.12/24 on UDP port 53 (DNS) into any associated subnet.
    #
    #   resp = client.replace_network_acl_entry({
    #     cidr_block: "203.0.113.12/24", 
    #     egress: false, 
    #     network_acl_id: "acl-5fb85d36", 
    #     port_range: {
    #       from: 53, 
    #       to: 53, 
    #     }, 
    #     protocol: "17", 
    #     rule_action: "allow", 
    #     rule_number: 100, 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.replace_network_acl_entry({
    #     dry_run: false,
    #     network_acl_id: "NetworkAclId", # required
    #     rule_number: 1, # required
    #     protocol: "String", # required
    #     rule_action: "allow", # required, accepts allow, deny
    #     egress: false, # required
    #     cidr_block: "String",
    #     ipv_6_cidr_block: "String",
    #     icmp_type_code: {
    #       code: 1,
    #       type: 1,
    #     },
    #     port_range: {
    #       from: 1,
    #       to: 1,
    #     },
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ReplaceNetworkAclEntry AWS API Documentation
    #
    # @overload replace_network_acl_entry(params = {})
    # @param [Hash] params ({})
    def replace_network_acl_entry(params = {}, options = {})
      req = build_request(:replace_network_acl_entry, params)
      req.send_request(options)
    end

    # Replaces an existing route within a route table in a VPC.
    #
    # You must specify either a destination CIDR block or a prefix list ID.
    # You must also specify exactly one of the resources from the parameter
    # list, or reset the local route to its default target.
    #
    # For more information, see [Route tables][1] in the *Amazon VPC User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Route_Tables.html
    #
    # @option params [String] :destination_prefix_list_id
    #   The ID of the prefix list for the route.
    #
    # @option params [String] :vpc_endpoint_id
    #   The ID of a VPC endpoint. Supported for Gateway Load Balancer
    #   endpoints only.
    #
    # @option params [Boolean] :local_target
    #   Specifies whether to reset the local route to its default target
    #   (`local`).
    #
    # @option params [String] :transit_gateway_id
    #   The ID of a transit gateway.
    #
    # @option params [String] :local_gateway_id
    #   The ID of the local gateway.
    #
    # @option params [String] :carrier_gateway_id
    #   \[IPv4 traffic only\] The ID of a carrier gateway.
    #
    # @option params [String] :core_network_arn
    #   The Amazon Resource Name (ARN) of the core network.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :route_table_id
    #   The ID of the route table.
    #
    # @option params [String] :destination_cidr_block
    #   The IPv4 CIDR address block used for the destination match. The value
    #   that you provide must match the CIDR of an existing route in the
    #   table.
    #
    # @option params [String] :gateway_id
    #   The ID of an internet gateway or virtual private gateway.
    #
    # @option params [String] :destination_ipv_6_cidr_block
    #   The IPv6 CIDR address block used for the destination match. The value
    #   that you provide must match the CIDR of an existing route in the
    #   table.
    #
    # @option params [String] :egress_only_internet_gateway_id
    #   \[IPv6 traffic only\] The ID of an egress-only internet gateway.
    #
    # @option params [String] :instance_id
    #   The ID of a NAT instance in your VPC.
    #
    # @option params [String] :network_interface_id
    #   The ID of a network interface.
    #
    # @option params [String] :vpc_peering_connection_id
    #   The ID of a VPC peering connection.
    #
    # @option params [String] :nat_gateway_id
    #   \[IPv4 traffic only\] The ID of a NAT gateway.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To replace a route
    #
    #   # This example replaces the specified route in the specified table table. The new route matches the specified CIDR and
    #   # sends the traffic to the specified virtual private gateway.
    #
    #   resp = client.replace_route({
    #     destination_cidr_block: "10.0.0.0/16", 
    #     gateway_id: "vgw-9a4cacf3", 
    #     route_table_id: "rtb-22574640", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.replace_route({
    #     destination_prefix_list_id: "PrefixListResourceId",
    #     vpc_endpoint_id: "VpcEndpointId",
    #     local_target: false,
    #     transit_gateway_id: "TransitGatewayId",
    #     local_gateway_id: "LocalGatewayId",
    #     carrier_gateway_id: "CarrierGatewayId",
    #     core_network_arn: "CoreNetworkArn",
    #     dry_run: false,
    #     route_table_id: "RouteTableId", # required
    #     destination_cidr_block: "String",
    #     gateway_id: "RouteGatewayId",
    #     destination_ipv_6_cidr_block: "String",
    #     egress_only_internet_gateway_id: "EgressOnlyInternetGatewayId",
    #     instance_id: "InstanceId",
    #     network_interface_id: "NetworkInterfaceId",
    #     vpc_peering_connection_id: "VpcPeeringConnectionId",
    #     nat_gateway_id: "NatGatewayId",
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ReplaceRoute AWS API Documentation
    #
    # @overload replace_route(params = {})
    # @param [Hash] params ({})
    def replace_route(params = {}, options = {})
      req = build_request(:replace_route, params)
      req.send_request(options)
    end

    # Changes the route table associated with a given subnet, internet
    # gateway, or virtual private gateway in a VPC. After the operation
    # completes, the subnet or gateway uses the routes in the new route
    # table. For more information about route tables, see [Route tables][1]
    # in the *Amazon VPC User Guide*.
    #
    # You can also use this operation to change which table is the main
    # route table in the VPC. Specify the main route table's association ID
    # and the route table ID of the new main route table.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/VPC_Route_Tables.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :association_id
    #   The association ID.
    #
    # @option params [required, String] :route_table_id
    #   The ID of the new route table to associate with the subnet.
    #
    # @return [Types::ReplaceRouteTableAssociationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ReplaceRouteTableAssociationResult#new_association_id #new_association_id} => String
    #   * {Types::ReplaceRouteTableAssociationResult#association_state #association_state} => Types::RouteTableAssociationState
    #
    #
    # @example Example: To replace the route table associated with a subnet
    #
    #   # This example associates the specified route table with the subnet for the specified route table association.
    #
    #   resp = client.replace_route_table_association({
    #     association_id: "rtbassoc-781d0d1a", 
    #     route_table_id: "rtb-22574640", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     new_association_id: "rtbassoc-3a1f0f58", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.replace_route_table_association({
    #     dry_run: false,
    #     association_id: "RouteTableAssociationId", # required
    #     route_table_id: "RouteTableId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.new_association_id #=> String
    #   resp.association_state.state #=> String, one of "associating", "associated", "disassociating", "disassociated", "failed"
    #   resp.association_state.status_message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ReplaceRouteTableAssociation AWS API Documentation
    #
    # @overload replace_route_table_association(params = {})
    # @param [Hash] params ({})
    def replace_route_table_association(params = {}, options = {})
      req = build_request(:replace_route_table_association, params)
      req.send_request(options)
    end

    # Replaces the specified route in the specified transit gateway route
    # table.
    #
    # @option params [required, String] :destination_cidr_block
    #   The CIDR range used for the destination match. Routing decisions are
    #   based on the most specific match.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the route table.
    #
    # @option params [String] :transit_gateway_attachment_id
    #   The ID of the attachment.
    #
    # @option params [Boolean] :blackhole
    #   Indicates whether traffic matching this route is to be dropped.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ReplaceTransitGatewayRouteResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ReplaceTransitGatewayRouteResult#route #route} => Types::TransitGatewayRoute
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.replace_transit_gateway_route({
    #     destination_cidr_block: "String", # required
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     transit_gateway_attachment_id: "TransitGatewayAttachmentId",
    #     blackhole: false,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.route.destination_cidr_block #=> String
    #   resp.route.prefix_list_id #=> String
    #   resp.route.transit_gateway_route_table_announcement_id #=> String
    #   resp.route.transit_gateway_attachments #=> Array
    #   resp.route.transit_gateway_attachments[0].resource_id #=> String
    #   resp.route.transit_gateway_attachments[0].transit_gateway_attachment_id #=> String
    #   resp.route.transit_gateway_attachments[0].resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.route.type #=> String, one of "static", "propagated"
    #   resp.route.state #=> String, one of "pending", "active", "blackhole", "deleting", "deleted"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ReplaceTransitGatewayRoute AWS API Documentation
    #
    # @overload replace_transit_gateway_route(params = {})
    # @param [Hash] params ({})
    def replace_transit_gateway_route(params = {}, options = {})
      req = build_request(:replace_transit_gateway_route, params)
      req.send_request(options)
    end

    # Trigger replacement of specified VPN tunnel.
    #
    # @option params [required, String] :vpn_connection_id
    #   The ID of the Site-to-Site VPN connection.
    #
    # @option params [required, String] :vpn_tunnel_outside_ip_address
    #   The external IP address of the VPN tunnel.
    #
    # @option params [Boolean] :apply_pending_maintenance
    #   Trigger pending tunnel endpoint maintenance.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ReplaceVpnTunnelResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ReplaceVpnTunnelResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.replace_vpn_tunnel({
    #     vpn_connection_id: "VpnConnectionId", # required
    #     vpn_tunnel_outside_ip_address: "String", # required
    #     apply_pending_maintenance: false,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ReplaceVpnTunnel AWS API Documentation
    #
    # @overload replace_vpn_tunnel(params = {})
    # @param [Hash] params ({})
    def replace_vpn_tunnel(params = {}, options = {})
      req = build_request(:replace_vpn_tunnel, params)
      req.send_request(options)
    end

    # Submits feedback about the status of an instance. The instance must be
    # in the `running` state. If your experience with the instance differs
    # from the instance status returned by DescribeInstanceStatus, use
    # ReportInstanceStatus to report your experience with the instance.
    # Amazon EC2 collects this information to improve the accuracy of status
    # checks.
    #
    # Use of this action does not change the value returned by
    # DescribeInstanceStatus.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Array<String>] :instances
    #   The instances.
    #
    # @option params [required, String] :status
    #   The status of all instances listed.
    #
    # @option params [Time,DateTime,Date,Integer,String] :start_time
    #   The time at which the reported instance health state began.
    #
    # @option params [Time,DateTime,Date,Integer,String] :end_time
    #   The time at which the reported instance health state ended.
    #
    # @option params [required, Array<String>] :reason_codes
    #   The reason codes that describe the health state of your instance.
    #
    #   * `instance-stuck-in-state`: My instance is stuck in a state.
    #
    #   * `unresponsive`: My instance is unresponsive.
    #
    #   * `not-accepting-credentials`: My instance is not accepting my
    #     credentials.
    #
    #   * `password-not-available`: A password is not available for my
    #     instance.
    #
    #   * `performance-network`: My instance is experiencing performance
    #     problems that I believe are network related.
    #
    #   * `performance-instance-store`: My instance is experiencing
    #     performance problems that I believe are related to the instance
    #     stores.
    #
    #   * `performance-ebs-volume`: My instance is experiencing performance
    #     problems that I believe are related to an EBS volume.
    #
    #   * `performance-other`: My instance is experiencing performance
    #     problems.
    #
    #   * `other`: \[explain using the description parameter\]
    #
    # @option params [String] :description
    #   Descriptive text about the health state of your instance.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.report_instance_status({
    #     dry_run: false,
    #     instances: ["InstanceId"], # required
    #     status: "ok", # required, accepts ok, impaired
    #     start_time: Time.now,
    #     end_time: Time.now,
    #     reason_codes: ["instance-stuck-in-state"], # required, accepts instance-stuck-in-state, unresponsive, not-accepting-credentials, password-not-available, performance-network, performance-instance-store, performance-ebs-volume, performance-other, other
    #     description: "ReportInstanceStatusRequestDescription",
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ReportInstanceStatus AWS API Documentation
    #
    # @overload report_instance_status(params = {})
    # @param [Hash] params ({})
    def report_instance_status(params = {}, options = {})
      req = build_request(:report_instance_status, params)
      req.send_request(options)
    end

    # Creates a Spot Fleet request.
    #
    # The Spot Fleet request specifies the total target capacity and the
    # On-Demand target capacity. Amazon EC2 calculates the difference
    # between the total capacity and On-Demand capacity, and launches the
    # difference as Spot capacity.
    #
    # You can submit a single request that includes multiple launch
    # specifications that vary by instance type, AMI, Availability Zone, or
    # subnet.
    #
    # By default, the Spot Fleet requests Spot Instances in the Spot
    # Instance pool where the price per unit is the lowest. Each launch
    # specification can include its own instance weighting that reflects the
    # value of the instance type to your application workload.
    #
    # Alternatively, you can specify that the Spot Fleet distribute the
    # target capacity across the Spot pools included in its launch
    # specifications. By ensuring that the Spot Instances in your Spot Fleet
    # are in different Spot pools, you can improve the availability of your
    # fleet.
    #
    # You can specify tags for the Spot Fleet request and instances launched
    # by the fleet. You cannot tag other resource types in a Spot Fleet
    # request because only the `spot-fleet-request` and `instance` resource
    # types are supported.
    #
    # For more information, see [Spot Fleet requests][1] in the *Amazon EC2
    # User Guide*.
    #
    # We strongly discourage using the RequestSpotFleet API because it is a
    # legacy API with no planned investment. For options for requesting Spot
    # Instances, see [Which is the best Spot request method to use?][2] in
    # the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-fleet-requests.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-best-practices.html#which-spot-request-method-to-use
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, Types::SpotFleetRequestConfigData] :spot_fleet_request_config
    #   The configuration for the Spot Fleet request.
    #
    # @return [Types::RequestSpotFleetResponse] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RequestSpotFleetResponse#spot_fleet_request_id #spot_fleet_request_id} => String
    #
    #
    # @example Example: To request a Spot fleet in the subnet with the lowest price
    #
    #   # This example creates a Spot fleet request with two launch specifications that differ only by subnet. The Spot fleet
    #   # launches the instances in the specified subnet with the lowest price. If the instances are launched in a default VPC,
    #   # they receive a public IP address by default. If the instances are launched in a nondefault VPC, they do not receive a
    #   # public IP address by default. Note that you can't specify different subnets from the same Availability Zone in a Spot
    #   # fleet request.
    #
    #   resp = client.request_spot_fleet({
    #     spot_fleet_request_config: {
    #       iam_fleet_role: "arn:aws:iam::123456789012:role/my-spot-fleet-role", 
    #       launch_specifications: [
    #         {
    #           iam_instance_profile: {
    #             arn: "arn:aws:iam::123456789012:instance-profile/my-iam-role", 
    #           }, 
    #           image_id: "ami-1a2b3c4d", 
    #           instance_type: "m3.medium", 
    #           key_name: "my-key-pair", 
    #           security_groups: [
    #             {
    #               group_id: "sg-1a2b3c4d", 
    #             }, 
    #           ], 
    #           subnet_id: "subnet-1a2b3c4d, subnet-3c4d5e6f", 
    #         }, 
    #       ], 
    #       spot_price: "0.04", 
    #       target_capacity: 2, 
    #     }, 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     spot_fleet_request_id: "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #   }
    #
    # @example Example: To request a Spot fleet in the Availability Zone with the lowest price
    #
    #   # This example creates a Spot fleet request with two launch specifications that differ only by Availability Zone. The Spot
    #   # fleet launches the instances in the specified Availability Zone with the lowest price. If your account supports EC2-VPC
    #   # only, Amazon EC2 launches the Spot instances in the default subnet of the Availability Zone.
    #
    #   resp = client.request_spot_fleet({
    #     spot_fleet_request_config: {
    #       iam_fleet_role: "arn:aws:iam::123456789012:role/my-spot-fleet-role", 
    #       launch_specifications: [
    #         {
    #           iam_instance_profile: {
    #             arn: "arn:aws:iam::123456789012:instance-profile/my-iam-role", 
    #           }, 
    #           image_id: "ami-1a2b3c4d", 
    #           instance_type: "m3.medium", 
    #           key_name: "my-key-pair", 
    #           placement: {
    #             availability_zone: "us-west-2a, us-west-2b", 
    #           }, 
    #           security_groups: [
    #             {
    #               group_id: "sg-1a2b3c4d", 
    #             }, 
    #           ], 
    #         }, 
    #       ], 
    #       spot_price: "0.04", 
    #       target_capacity: 2, 
    #     }, 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     spot_fleet_request_id: "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #   }
    #
    # @example Example: To launch Spot instances in a subnet and assign them public IP addresses
    #
    #   # This example assigns public addresses to instances launched in a nondefault VPC. Note that when you specify a network
    #   # interface, you must include the subnet ID and security group ID using the network interface.
    #
    #   resp = client.request_spot_fleet({
    #     spot_fleet_request_config: {
    #       iam_fleet_role: "arn:aws:iam::123456789012:role/my-spot-fleet-role", 
    #       launch_specifications: [
    #         {
    #           iam_instance_profile: {
    #             arn: "arn:aws:iam::880185128111:instance-profile/my-iam-role", 
    #           }, 
    #           image_id: "ami-1a2b3c4d", 
    #           instance_type: "m3.medium", 
    #           key_name: "my-key-pair", 
    #           network_interfaces: [
    #             {
    #               associate_public_ip_address: true, 
    #               device_index: 0, 
    #               groups: [
    #                 "sg-1a2b3c4d", 
    #               ], 
    #               subnet_id: "subnet-1a2b3c4d", 
    #             }, 
    #           ], 
    #         }, 
    #       ], 
    #       spot_price: "0.04", 
    #       target_capacity: 2, 
    #     }, 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     spot_fleet_request_id: "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #   }
    #
    # @example Example: To request a Spot fleet using the diversified allocation strategy
    #
    #   # This example creates a Spot fleet request that launches 30 instances using the diversified allocation strategy. The
    #   # launch specifications differ by instance type. The Spot fleet distributes the instances across the launch specifications
    #   # such that there are 10 instances of each type.
    #
    #   resp = client.request_spot_fleet({
    #     spot_fleet_request_config: {
    #       allocation_strategy: "diversified", 
    #       iam_fleet_role: "arn:aws:iam::123456789012:role/my-spot-fleet-role", 
    #       launch_specifications: [
    #         {
    #           image_id: "ami-1a2b3c4d", 
    #           instance_type: "c4.2xlarge", 
    #           subnet_id: "subnet-1a2b3c4d", 
    #         }, 
    #         {
    #           image_id: "ami-1a2b3c4d", 
    #           instance_type: "m3.2xlarge", 
    #           subnet_id: "subnet-1a2b3c4d", 
    #         }, 
    #         {
    #           image_id: "ami-1a2b3c4d", 
    #           instance_type: "r3.2xlarge", 
    #           subnet_id: "subnet-1a2b3c4d", 
    #         }, 
    #       ], 
    #       spot_price: "0.70", 
    #       target_capacity: 30, 
    #     }, 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     spot_fleet_request_id: "sfr-73fbd2ce-aa30-494c-8788-1cee4EXAMPLE", 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.request_spot_fleet({
    #     dry_run: false,
    #     spot_fleet_request_config: { # required
    #       allocation_strategy: "lowestPrice", # accepts lowestPrice, diversified, capacityOptimized, capacityOptimizedPrioritized, priceCapacityOptimized
    #       on_demand_allocation_strategy: "lowestPrice", # accepts lowestPrice, prioritized
    #       spot_maintenance_strategies: {
    #         capacity_rebalance: {
    #           replacement_strategy: "launch", # accepts launch, launch-before-terminate
    #           termination_delay: 1,
    #         },
    #       },
    #       client_token: "String",
    #       excess_capacity_termination_policy: "noTermination", # accepts noTermination, default
    #       fulfilled_capacity: 1.0,
    #       on_demand_fulfilled_capacity: 1.0,
    #       iam_fleet_role: "String", # required
    #       launch_specifications: [
    #         {
    #           addressing_type: "String",
    #           block_device_mappings: [
    #             {
    #               ebs: {
    #                 delete_on_termination: false,
    #                 iops: 1,
    #                 snapshot_id: "SnapshotId",
    #                 volume_size: 1,
    #                 volume_type: "standard", # accepts standard, io1, io2, gp2, sc1, st1, gp3
    #                 kms_key_id: "String",
    #                 throughput: 1,
    #                 outpost_arn: "String",
    #                 encrypted: false,
    #               },
    #               no_device: "String",
    #               device_name: "String",
    #               virtual_name: "String",
    #             },
    #           ],
    #           ebs_optimized: false,
    #           iam_instance_profile: {
    #             arn: "String",
    #             name: "String",
    #           },
    #           image_id: "ImageId",
    #           instance_type: "a1.medium", # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #           kernel_id: "String",
    #           key_name: "KeyPairName",
    #           monitoring: {
    #             enabled: false,
    #           },
    #           network_interfaces: [
    #             {
    #               associate_public_ip_address: false,
    #               delete_on_termination: false,
    #               description: "String",
    #               device_index: 1,
    #               groups: ["SecurityGroupId"],
    #               ipv_6_address_count: 1,
    #               ipv_6_addresses: [
    #                 {
    #                   ipv_6_address: "String",
    #                   is_primary_ipv_6: false,
    #                 },
    #               ],
    #               network_interface_id: "NetworkInterfaceId",
    #               private_ip_address: "String",
    #               private_ip_addresses: [
    #                 {
    #                   primary: false,
    #                   private_ip_address: "String",
    #                 },
    #               ],
    #               secondary_private_ip_address_count: 1,
    #               subnet_id: "String",
    #               associate_carrier_ip_address: false,
    #               interface_type: "String",
    #               network_card_index: 1,
    #               ipv_4_prefixes: [
    #                 {
    #                   ipv_4_prefix: "String",
    #                 },
    #               ],
    #               ipv_4_prefix_count: 1,
    #               ipv_6_prefixes: [
    #                 {
    #                   ipv_6_prefix: "String",
    #                 },
    #               ],
    #               ipv_6_prefix_count: 1,
    #               primary_ipv_6: false,
    #               ena_srd_specification: {
    #                 ena_srd_enabled: false,
    #                 ena_srd_udp_specification: {
    #                   ena_srd_udp_enabled: false,
    #                 },
    #               },
    #               connection_tracking_specification: {
    #                 tcp_established_timeout: 1,
    #                 udp_stream_timeout: 1,
    #                 udp_timeout: 1,
    #               },
    #             },
    #           ],
    #           placement: {
    #             availability_zone: "String",
    #             group_name: "PlacementGroupName",
    #             tenancy: "default", # accepts default, dedicated, host
    #           },
    #           ramdisk_id: "String",
    #           spot_price: "String",
    #           subnet_id: "SubnetId",
    #           user_data: "SensitiveUserData",
    #           weighted_capacity: 1.0,
    #           tag_specifications: [
    #             {
    #               resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #               tags: [
    #                 {
    #                   key: "String",
    #                   value: "String",
    #                 },
    #               ],
    #             },
    #           ],
    #           instance_requirements: {
    #             v_cpu_count: {
    #               min: 1,
    #               max: 1,
    #             },
    #             memory_mi_b: {
    #               min: 1,
    #               max: 1,
    #             },
    #             cpu_manufacturers: ["intel"], # accepts intel, amd, amazon-web-services, apple
    #             memory_gi_b_per_v_cpu: {
    #               min: 1.0,
    #               max: 1.0,
    #             },
    #             excluded_instance_types: ["ExcludedInstanceType"],
    #             instance_generations: ["current"], # accepts current, previous
    #             spot_max_price_percentage_over_lowest_price: 1,
    #             on_demand_max_price_percentage_over_lowest_price: 1,
    #             bare_metal: "included", # accepts included, required, excluded
    #             burstable_performance: "included", # accepts included, required, excluded
    #             require_hibernate_support: false,
    #             network_interface_count: {
    #               min: 1,
    #               max: 1,
    #             },
    #             local_storage: "included", # accepts included, required, excluded
    #             local_storage_types: ["hdd"], # accepts hdd, ssd
    #             total_local_storage_gb: {
    #               min: 1.0,
    #               max: 1.0,
    #             },
    #             baseline_ebs_bandwidth_mbps: {
    #               min: 1,
    #               max: 1,
    #             },
    #             accelerator_types: ["gpu"], # accepts gpu, fpga, inference
    #             accelerator_count: {
    #               min: 1,
    #               max: 1,
    #             },
    #             accelerator_manufacturers: ["amazon-web-services"], # accepts amazon-web-services, amd, nvidia, xilinx, habana
    #             accelerator_names: ["a100"], # accepts a100, inferentia, k520, k80, m60, radeon-pro-v520, t4, vu9p, v100, a10g, h100, t4g
    #             accelerator_total_memory_mi_b: {
    #               min: 1,
    #               max: 1,
    #             },
    #             network_bandwidth_gbps: {
    #               min: 1.0,
    #               max: 1.0,
    #             },
    #             allowed_instance_types: ["AllowedInstanceType"],
    #             max_spot_price_as_percentage_of_optimal_on_demand_price: 1,
    #             baseline_performance_factors: {
    #               cpu: {
    #                 references: [
    #                   {
    #                     instance_family: "String",
    #                   },
    #                 ],
    #               },
    #             },
    #           },
    #           security_groups: [
    #             {
    #               group_id: "String",
    #               group_name: "String",
    #             },
    #           ],
    #         },
    #       ],
    #       launch_template_configs: [
    #         {
    #           launch_template_specification: {
    #             launch_template_id: "String",
    #             launch_template_name: "LaunchTemplateName",
    #             version: "String",
    #           },
    #           overrides: [
    #             {
    #               instance_type: "a1.medium", # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #               spot_price: "String",
    #               subnet_id: "SubnetId",
    #               availability_zone: "String",
    #               weighted_capacity: 1.0,
    #               priority: 1.0,
    #               instance_requirements: {
    #                 v_cpu_count: {
    #                   min: 1,
    #                   max: 1,
    #                 },
    #                 memory_mi_b: {
    #                   min: 1,
    #                   max: 1,
    #                 },
    #                 cpu_manufacturers: ["intel"], # accepts intel, amd, amazon-web-services, apple
    #                 memory_gi_b_per_v_cpu: {
    #                   min: 1.0,
    #                   max: 1.0,
    #                 },
    #                 excluded_instance_types: ["ExcludedInstanceType"],
    #                 instance_generations: ["current"], # accepts current, previous
    #                 spot_max_price_percentage_over_lowest_price: 1,
    #                 on_demand_max_price_percentage_over_lowest_price: 1,
    #                 bare_metal: "included", # accepts included, required, excluded
    #                 burstable_performance: "included", # accepts included, required, excluded
    #                 require_hibernate_support: false,
    #                 network_interface_count: {
    #                   min: 1,
    #                   max: 1,
    #                 },
    #                 local_storage: "included", # accepts included, required, excluded
    #                 local_storage_types: ["hdd"], # accepts hdd, ssd
    #                 total_local_storage_gb: {
    #                   min: 1.0,
    #                   max: 1.0,
    #                 },
    #                 baseline_ebs_bandwidth_mbps: {
    #                   min: 1,
    #                   max: 1,
    #                 },
    #                 accelerator_types: ["gpu"], # accepts gpu, fpga, inference
    #                 accelerator_count: {
    #                   min: 1,
    #                   max: 1,
    #                 },
    #                 accelerator_manufacturers: ["amazon-web-services"], # accepts amazon-web-services, amd, nvidia, xilinx, habana
    #                 accelerator_names: ["a100"], # accepts a100, inferentia, k520, k80, m60, radeon-pro-v520, t4, vu9p, v100, a10g, h100, t4g
    #                 accelerator_total_memory_mi_b: {
    #                   min: 1,
    #                   max: 1,
    #                 },
    #                 network_bandwidth_gbps: {
    #                   min: 1.0,
    #                   max: 1.0,
    #                 },
    #                 allowed_instance_types: ["AllowedInstanceType"],
    #                 max_spot_price_as_percentage_of_optimal_on_demand_price: 1,
    #                 baseline_performance_factors: {
    #                   cpu: {
    #                     references: [
    #                       {
    #                         instance_family: "String",
    #                       },
    #                     ],
    #                   },
    #                 },
    #               },
    #             },
    #           ],
    #         },
    #       ],
    #       spot_price: "String",
    #       target_capacity: 1, # required
    #       on_demand_target_capacity: 1,
    #       on_demand_max_total_price: "String",
    #       spot_max_total_price: "String",
    #       terminate_instances_with_expiration: false,
    #       type: "request", # accepts request, maintain, instant
    #       valid_from: Time.now,
    #       valid_until: Time.now,
    #       replace_unhealthy_instances: false,
    #       instance_interruption_behavior: "hibernate", # accepts hibernate, stop, terminate
    #       load_balancers_config: {
    #         classic_load_balancers_config: {
    #           classic_load_balancers: [
    #             {
    #               name: "String",
    #             },
    #           ],
    #         },
    #         target_groups_config: {
    #           target_groups: [
    #             {
    #               arn: "String",
    #             },
    #           ],
    #         },
    #       },
    #       instance_pools_to_use_count: 1,
    #       context: "String",
    #       target_capacity_unit_type: "vcpu", # accepts vcpu, memory-mib, units
    #       tag_specifications: [
    #         {
    #           resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #           tags: [
    #             {
    #               key: "String",
    #               value: "String",
    #             },
    #           ],
    #         },
    #       ],
    #     },
    #   })
    #
    # @example Response structure
    #
    #   resp.spot_fleet_request_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RequestSpotFleet AWS API Documentation
    #
    # @overload request_spot_fleet(params = {})
    # @param [Hash] params ({})
    def request_spot_fleet(params = {}, options = {})
      req = build_request(:request_spot_fleet, params)
      req.send_request(options)
    end

    # Creates a Spot Instance request.
    #
    # For more information, see [Work with Spot Instance][1] in the *Amazon
    # EC2 User Guide*.
    #
    # We strongly discourage using the RequestSpotInstances API because it
    # is a legacy API with no planned investment. For options for requesting
    # Spot Instances, see [Which is the best Spot request method to use?][2]
    # in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-requests.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-best-practices.html#which-spot-request-method-to-use
    #
    # @option params [Types::RequestSpotLaunchSpecification] :launch_specification
    #   The launch specification.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The key-value pair for tagging the Spot Instance request on creation.
    #   The value for `ResourceType` must be `spot-instances-request`,
    #   otherwise the Spot Instance request fails. To tag the Spot Instance
    #   request after it has been created, see [CreateTags][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTags.html
    #
    # @option params [String] :instance_interruption_behavior
    #   The behavior when a Spot Instance is interrupted. The default is
    #   `terminate`.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :spot_price
    #   The maximum price per unit hour that you are willing to pay for a Spot
    #   Instance. We do not recommend using this parameter because it can lead
    #   to increased interruptions. If you do not specify this parameter, you
    #   will pay the current Spot price.
    #
    #   If you specify a maximum price, your instances will be interrupted
    #   more frequently than if you do not specify this parameter.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [Ensuring
    #   idempotency in Amazon EC2 API requests][1] in the *Amazon EC2 User
    #   Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Run_Instance_Idempotency.html
    #
    # @option params [Integer] :instance_count
    #   The maximum number of Spot Instances to launch.
    #
    #   Default: 1
    #
    # @option params [String] :type
    #   The Spot Instance request type.
    #
    #   Default: `one-time`
    #
    # @option params [Time,DateTime,Date,Integer,String] :valid_from
    #   The start date of the request. If this is a one-time request, the
    #   request becomes active at this date and time and remains active until
    #   all instances launch, the request expires, or the request is canceled.
    #   If the request is persistent, the request becomes active at this date
    #   and time and remains active until it expires or is canceled.
    #
    #   The specified start date and time cannot be equal to the current date
    #   and time. You must specify a start date and time that occurs after the
    #   current date and time.
    #
    # @option params [Time,DateTime,Date,Integer,String] :valid_until
    #   The end date of the request, in UTC format
    #   (*YYYY*-*MM*-*DD*T*HH*:*MM*:*SS*Z).
    #
    #   * For a persistent request, the request remains active until the
    #     `ValidUntil` date and time is reached. Otherwise, the request
    #     remains active until you cancel it.
    #
    #   * For a one-time request, the request remains active until all
    #     instances launch, the request is canceled, or the `ValidUntil` date
    #     and time is reached. By default, the request is valid for 7 days
    #     from the date the request was created.
    #
    # @option params [String] :launch_group
    #   The instance launch group. Launch groups are Spot Instances that
    #   launch together and terminate together.
    #
    #   Default: Instances are launched and terminated individually
    #
    # @option params [String] :availability_zone_group
    #   The user-specified name for a logical grouping of requests.
    #
    #   When you specify an Availability Zone group in a Spot Instance
    #   request, all Spot Instances in the request are launched in the same
    #   Availability Zone. Instance proximity is maintained with this
    #   parameter, but the choice of Availability Zone is not. The group
    #   applies only to requests for Spot Instances of the same instance type.
    #   Any additional Spot Instance requests that are specified with the same
    #   Availability Zone group name are launched in that same Availability
    #   Zone, as long as at least one instance from the group is still active.
    #
    #   If there is no active instance running in the Availability Zone group
    #   that you specify for a new Spot Instance request (all instances are
    #   terminated, the request is expired, or the maximum price you specified
    #   falls below current Spot price), then Amazon EC2 launches the instance
    #   in any Availability Zone where the constraint can be met.
    #   Consequently, the subsequent set of Spot Instances could be placed in
    #   a different zone from the original request, even if you specified the
    #   same Availability Zone group.
    #
    #   Default: Instances are launched in any available Availability Zone.
    #
    # @option params [Integer] :block_duration_minutes
    #   Deprecated.
    #
    # @return [Types::RequestSpotInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RequestSpotInstancesResult#spot_instance_requests #spot_instance_requests} => Array&lt;Types::SpotInstanceRequest&gt;
    #
    #
    # @example Example: To create a one-time Spot Instance request
    #
    #   # This example creates a one-time Spot Instance request for five instances in the specified Availability Zone. If your
    #   # account supports EC2-VPC only, Amazon EC2 launches the instances in the default subnet of the specified Availability
    #   # Zone.
    #
    #   resp = client.request_spot_instances({
    #     instance_count: 5, 
    #     launch_specification: {
    #       iam_instance_profile: {
    #         arn: "arn:aws:iam::123456789012:instance-profile/my-iam-role", 
    #       }, 
    #       image_id: "ami-1a2b3c4d", 
    #       instance_type: "m3.medium", 
    #       key_name: "my-key-pair", 
    #       placement: {
    #         availability_zone: "us-west-2a", 
    #       }, 
    #       security_group_ids: [
    #         "sg-1a2b3c4d", 
    #       ], 
    #     }, 
    #     spot_price: "0.03", 
    #     type: "one-time", 
    #   })
    #
    # @example Example: To create a one-time Spot Instance request
    #
    #   # This example command creates a one-time Spot Instance request for five instances in the specified subnet. Amazon EC2
    #   # launches the instances in the specified subnet. If the VPC is a nondefault VPC, the instances do not receive a public IP
    #   # address by default.
    #
    #   resp = client.request_spot_instances({
    #     instance_count: 5, 
    #     launch_specification: {
    #       iam_instance_profile: {
    #         arn: "arn:aws:iam::123456789012:instance-profile/my-iam-role", 
    #       }, 
    #       image_id: "ami-1a2b3c4d", 
    #       instance_type: "m3.medium", 
    #       security_group_ids: [
    #         "sg-1a2b3c4d", 
    #       ], 
    #       subnet_id: "subnet-1a2b3c4d", 
    #     }, 
    #     spot_price: "0.050", 
    #     type: "one-time", 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.request_spot_instances({
    #     launch_specification: {
    #       security_group_ids: ["SecurityGroupId"],
    #       security_groups: ["String"],
    #       addressing_type: "String",
    #       block_device_mappings: [
    #         {
    #           ebs: {
    #             delete_on_termination: false,
    #             iops: 1,
    #             snapshot_id: "SnapshotId",
    #             volume_size: 1,
    #             volume_type: "standard", # accepts standard, io1, io2, gp2, sc1, st1, gp3
    #             kms_key_id: "String",
    #             throughput: 1,
    #             outpost_arn: "String",
    #             encrypted: false,
    #           },
    #           no_device: "String",
    #           device_name: "String",
    #           virtual_name: "String",
    #         },
    #       ],
    #       ebs_optimized: false,
    #       iam_instance_profile: {
    #         arn: "String",
    #         name: "String",
    #       },
    #       image_id: "ImageId",
    #       instance_type: "a1.medium", # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #       kernel_id: "KernelId",
    #       key_name: "KeyPairNameWithResolver",
    #       monitoring: {
    #         enabled: false, # required
    #       },
    #       network_interfaces: [
    #         {
    #           associate_public_ip_address: false,
    #           delete_on_termination: false,
    #           description: "String",
    #           device_index: 1,
    #           groups: ["SecurityGroupId"],
    #           ipv_6_address_count: 1,
    #           ipv_6_addresses: [
    #             {
    #               ipv_6_address: "String",
    #               is_primary_ipv_6: false,
    #             },
    #           ],
    #           network_interface_id: "NetworkInterfaceId",
    #           private_ip_address: "String",
    #           private_ip_addresses: [
    #             {
    #               primary: false,
    #               private_ip_address: "String",
    #             },
    #           ],
    #           secondary_private_ip_address_count: 1,
    #           subnet_id: "String",
    #           associate_carrier_ip_address: false,
    #           interface_type: "String",
    #           network_card_index: 1,
    #           ipv_4_prefixes: [
    #             {
    #               ipv_4_prefix: "String",
    #             },
    #           ],
    #           ipv_4_prefix_count: 1,
    #           ipv_6_prefixes: [
    #             {
    #               ipv_6_prefix: "String",
    #             },
    #           ],
    #           ipv_6_prefix_count: 1,
    #           primary_ipv_6: false,
    #           ena_srd_specification: {
    #             ena_srd_enabled: false,
    #             ena_srd_udp_specification: {
    #               ena_srd_udp_enabled: false,
    #             },
    #           },
    #           connection_tracking_specification: {
    #             tcp_established_timeout: 1,
    #             udp_stream_timeout: 1,
    #             udp_timeout: 1,
    #           },
    #         },
    #       ],
    #       placement: {
    #         availability_zone: "String",
    #         group_name: "PlacementGroupName",
    #         tenancy: "default", # accepts default, dedicated, host
    #       },
    #       ramdisk_id: "RamdiskId",
    #       subnet_id: "SubnetId",
    #       user_data: "SensitiveUserData",
    #     },
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     instance_interruption_behavior: "hibernate", # accepts hibernate, stop, terminate
    #     dry_run: false,
    #     spot_price: "String",
    #     client_token: "String",
    #     instance_count: 1,
    #     type: "one-time", # accepts one-time, persistent
    #     valid_from: Time.now,
    #     valid_until: Time.now,
    #     launch_group: "String",
    #     availability_zone_group: "String",
    #     block_duration_minutes: 1,
    #   })
    #
    # @example Response structure
    #
    #   resp.spot_instance_requests #=> Array
    #   resp.spot_instance_requests[0].actual_block_hourly_price #=> String
    #   resp.spot_instance_requests[0].availability_zone_group #=> String
    #   resp.spot_instance_requests[0].block_duration_minutes #=> Integer
    #   resp.spot_instance_requests[0].create_time #=> Time
    #   resp.spot_instance_requests[0].fault.code #=> String
    #   resp.spot_instance_requests[0].fault.message #=> String
    #   resp.spot_instance_requests[0].instance_id #=> String
    #   resp.spot_instance_requests[0].launch_group #=> String
    #   resp.spot_instance_requests[0].launch_specification.user_data #=> String
    #   resp.spot_instance_requests[0].launch_specification.addressing_type #=> String
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings #=> Array
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.iops #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.snapshot_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.volume_size #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.volume_type #=> String, one of "standard", "io1", "io2", "gp2", "sc1", "st1", "gp3"
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.kms_key_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.throughput #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.outpost_arn #=> String
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].ebs.encrypted #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].no_device #=> String
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].device_name #=> String
    #   resp.spot_instance_requests[0].launch_specification.block_device_mappings[0].virtual_name #=> String
    #   resp.spot_instance_requests[0].launch_specification.ebs_optimized #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.iam_instance_profile.arn #=> String
    #   resp.spot_instance_requests[0].launch_specification.iam_instance_profile.name #=> String
    #   resp.spot_instance_requests[0].launch_specification.image_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.spot_instance_requests[0].launch_specification.kernel_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.key_name #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces #=> Array
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].associate_public_ip_address #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].delete_on_termination #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].description #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].device_index #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].groups #=> Array
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].groups[0] #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_6_address_count #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_6_addresses #=> Array
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_6_addresses[0].ipv_6_address #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_6_addresses[0].is_primary_ipv_6 #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].network_interface_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].private_ip_address #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].private_ip_addresses #=> Array
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].private_ip_addresses[0].primary #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].private_ip_addresses[0].private_ip_address #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].secondary_private_ip_address_count #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].subnet_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].associate_carrier_ip_address #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].interface_type #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].network_card_index #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_4_prefixes #=> Array
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_4_prefixes[0].ipv_4_prefix #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_4_prefix_count #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_6_prefixes #=> Array
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_6_prefixes[0].ipv_6_prefix #=> String
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ipv_6_prefix_count #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].primary_ipv_6 #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ena_srd_specification.ena_srd_enabled #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].ena_srd_specification.ena_srd_udp_specification.ena_srd_udp_enabled #=> Boolean
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].connection_tracking_specification.tcp_established_timeout #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].connection_tracking_specification.udp_stream_timeout #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.network_interfaces[0].connection_tracking_specification.udp_timeout #=> Integer
    #   resp.spot_instance_requests[0].launch_specification.placement.availability_zone #=> String
    #   resp.spot_instance_requests[0].launch_specification.placement.group_name #=> String
    #   resp.spot_instance_requests[0].launch_specification.placement.tenancy #=> String, one of "default", "dedicated", "host"
    #   resp.spot_instance_requests[0].launch_specification.ramdisk_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.subnet_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.security_groups #=> Array
    #   resp.spot_instance_requests[0].launch_specification.security_groups[0].group_id #=> String
    #   resp.spot_instance_requests[0].launch_specification.security_groups[0].group_name #=> String
    #   resp.spot_instance_requests[0].launch_specification.monitoring.enabled #=> Boolean
    #   resp.spot_instance_requests[0].launched_availability_zone #=> String
    #   resp.spot_instance_requests[0].product_description #=> String, one of "Linux/UNIX", "Linux/UNIX (Amazon VPC)", "Windows", "Windows (Amazon VPC)"
    #   resp.spot_instance_requests[0].spot_instance_request_id #=> String
    #   resp.spot_instance_requests[0].spot_price #=> String
    #   resp.spot_instance_requests[0].state #=> String, one of "open", "active", "closed", "cancelled", "failed", "disabled"
    #   resp.spot_instance_requests[0].status.code #=> String
    #   resp.spot_instance_requests[0].status.message #=> String
    #   resp.spot_instance_requests[0].status.update_time #=> Time
    #   resp.spot_instance_requests[0].tags #=> Array
    #   resp.spot_instance_requests[0].tags[0].key #=> String
    #   resp.spot_instance_requests[0].tags[0].value #=> String
    #   resp.spot_instance_requests[0].type #=> String, one of "one-time", "persistent"
    #   resp.spot_instance_requests[0].valid_from #=> Time
    #   resp.spot_instance_requests[0].valid_until #=> Time
    #   resp.spot_instance_requests[0].instance_interruption_behavior #=> String, one of "hibernate", "stop", "terminate"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RequestSpotInstances AWS API Documentation
    #
    # @overload request_spot_instances(params = {})
    # @param [Hash] params ({})
    def request_spot_instances(params = {}, options = {})
      req = build_request(:request_spot_instances, params)
      req.send_request(options)
    end

    # Resets the attribute of the specified IP address. For requirements,
    # see [Using reverse DNS for email applications][1].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/elastic-ip-addresses-eip.html#Using_Elastic_Addressing_Reverse_DNS
    #
    # @option params [required, String] :allocation_id
    #   \[EC2-VPC\] The allocation ID.
    #
    # @option params [required, String] :attribute
    #   The attribute of the IP address.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ResetAddressAttributeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ResetAddressAttributeResult#address #address} => Types::AddressAttribute
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.reset_address_attribute({
    #     allocation_id: "AllocationId", # required
    #     attribute: "domain-name", # required, accepts domain-name
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.address.public_ip #=> String
    #   resp.address.allocation_id #=> String
    #   resp.address.ptr_record #=> String
    #   resp.address.ptr_record_update.value #=> String
    #   resp.address.ptr_record_update.status #=> String
    #   resp.address.ptr_record_update.reason #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ResetAddressAttribute AWS API Documentation
    #
    # @overload reset_address_attribute(params = {})
    # @param [Hash] params ({})
    def reset_address_attribute(params = {}, options = {})
      req = build_request(:reset_address_attribute, params)
      req.send_request(options)
    end

    # Resets the default KMS key for EBS encryption for your account in this
    # Region to the Amazon Web Services managed KMS key for EBS.
    #
    # After resetting the default KMS key to the Amazon Web Services managed
    # KMS key, you can continue to encrypt by a customer managed KMS key by
    # specifying it when you create the volume. For more information, see
    # [Amazon EBS encryption][1] in the *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-encryption.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::ResetEbsDefaultKmsKeyIdResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ResetEbsDefaultKmsKeyIdResult#kms_key_id #kms_key_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.reset_ebs_default_kms_key_id({
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.kms_key_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ResetEbsDefaultKmsKeyId AWS API Documentation
    #
    # @overload reset_ebs_default_kms_key_id(params = {})
    # @param [Hash] params ({})
    def reset_ebs_default_kms_key_id(params = {}, options = {})
      req = build_request(:reset_ebs_default_kms_key_id, params)
      req.send_request(options)
    end

    # Resets the specified attribute of the specified Amazon FPGA Image
    # (AFI) to its default value. You can only reset the load permission
    # attribute.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :fpga_image_id
    #   The ID of the AFI.
    #
    # @option params [String] :attribute
    #   The attribute.
    #
    # @return [Types::ResetFpgaImageAttributeResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::ResetFpgaImageAttributeResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.reset_fpga_image_attribute({
    #     dry_run: false,
    #     fpga_image_id: "FpgaImageId", # required
    #     attribute: "loadPermission", # accepts loadPermission
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ResetFpgaImageAttribute AWS API Documentation
    #
    # @overload reset_fpga_image_attribute(params = {})
    # @param [Hash] params ({})
    def reset_fpga_image_attribute(params = {}, options = {})
      req = build_request(:reset_fpga_image_attribute, params)
      req.send_request(options)
    end

    # Resets an attribute of an AMI to its default value.
    #
    # @option params [required, String] :attribute
    #   The attribute to reset (currently you can only reset the launch
    #   permission attribute).
    #
    # @option params [required, String] :image_id
    #   The ID of the AMI.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To reset the launchPermission attribute
    #
    #   # This example resets the launchPermission attribute for the specified AMI. By default, AMIs are private.
    #
    #   resp = client.reset_image_attribute({
    #     attribute: "launchPermission", 
    #     image_id: "ami-5731123e", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.reset_image_attribute({
    #     attribute: "launchPermission", # required, accepts launchPermission
    #     image_id: "ImageId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ResetImageAttribute AWS API Documentation
    #
    # @overload reset_image_attribute(params = {})
    # @param [Hash] params ({})
    def reset_image_attribute(params = {}, options = {})
      req = build_request(:reset_image_attribute, params)
      req.send_request(options)
    end

    # Resets an attribute of an instance to its default value. To reset the
    # `kernel` or `ramdisk`, the instance must be in a stopped state. To
    # reset the `sourceDestCheck`, the instance can be either running or
    # stopped.
    #
    # The `sourceDestCheck` attribute controls whether source/destination
    # checking is enabled. The default value is `true`, which means checking
    # is enabled. This value must be `false` for a NAT instance to perform
    # NAT. For more information, see [NAT instances][1] in the *Amazon VPC
    # User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AmazonVPC/latest/UserGuide/VPC_NAT_Instance.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @option params [required, String] :attribute
    #   The attribute to reset.
    #
    #   You can only reset the following attributes: `kernel` \| `ramdisk` \|
    #   `sourceDestCheck`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To reset the sourceDestCheck attribute
    #
    #   # This example resets the sourceDestCheck attribute for the specified instance.
    #
    #   resp = client.reset_instance_attribute({
    #     attribute: "sourceDestCheck", 
    #     instance_id: "i-1234567890abcdef0", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.reset_instance_attribute({
    #     dry_run: false,
    #     instance_id: "InstanceId", # required
    #     attribute: "instanceType", # required, accepts instanceType, kernel, ramdisk, userData, disableApiTermination, instanceInitiatedShutdownBehavior, rootDeviceName, blockDeviceMapping, productCodes, sourceDestCheck, groupSet, ebsOptimized, sriovNetSupport, enaSupport, enclaveOptions, disableApiStop
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ResetInstanceAttribute AWS API Documentation
    #
    # @overload reset_instance_attribute(params = {})
    # @param [Hash] params ({})
    def reset_instance_attribute(params = {}, options = {})
      req = build_request(:reset_instance_attribute, params)
      req.send_request(options)
    end

    # Resets a network interface attribute. You can specify only one
    # attribute at a time.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :network_interface_id
    #   The ID of the network interface.
    #
    # @option params [String] :source_dest_check
    #   The source/destination checking attribute. Resets the value to `true`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.reset_network_interface_attribute({
    #     dry_run: false,
    #     network_interface_id: "NetworkInterfaceId", # required
    #     source_dest_check: "String",
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ResetNetworkInterfaceAttribute AWS API Documentation
    #
    # @overload reset_network_interface_attribute(params = {})
    # @param [Hash] params ({})
    def reset_network_interface_attribute(params = {}, options = {})
      req = build_request(:reset_network_interface_attribute, params)
      req.send_request(options)
    end

    # Resets permission settings for the specified snapshot.
    #
    # For more information about modifying snapshot permissions, see [Share
    # a snapshot][1] in the *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/ebs-modifying-snapshot-permissions.html
    #
    # @option params [required, String] :attribute
    #   The attribute to reset. Currently, only the attribute for permission
    #   to create volumes can be reset.
    #
    # @option params [required, String] :snapshot_id
    #   The ID of the snapshot.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To reset a snapshot attribute
    #
    #   # This example resets the create volume permissions for snapshot ``snap-1234567890abcdef0``. If the command succeeds, no
    #   # output is returned.
    #
    #   resp = client.reset_snapshot_attribute({
    #     attribute: "createVolumePermission", 
    #     snapshot_id: "snap-1234567890abcdef0", 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.reset_snapshot_attribute({
    #     attribute: "productCodes", # required, accepts productCodes, createVolumePermission
    #     snapshot_id: "SnapshotId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/ResetSnapshotAttribute AWS API Documentation
    #
    # @overload reset_snapshot_attribute(params = {})
    # @param [Hash] params ({})
    def reset_snapshot_attribute(params = {}, options = {})
      req = build_request(:reset_snapshot_attribute, params)
      req.send_request(options)
    end

    # <note markdown="1"> This action is deprecated.
    #
    #  </note>
    #
    # Restores an Elastic IP address that was previously moved to the
    # EC2-VPC platform back to the EC2-Classic platform. You cannot move an
    # Elastic IP address that was originally allocated for use in EC2-VPC.
    # The Elastic IP address must not be associated with an instance or
    # network interface.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :public_ip
    #   The Elastic IP address.
    #
    # @return [Types::RestoreAddressToClassicResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RestoreAddressToClassicResult#public_ip #public_ip} => String
    #   * {Types::RestoreAddressToClassicResult#status #status} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.restore_address_to_classic({
    #     dry_run: false,
    #     public_ip: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.public_ip #=> String
    #   resp.status #=> String, one of "MoveInProgress", "InVpc", "InClassic"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RestoreAddressToClassic AWS API Documentation
    #
    # @overload restore_address_to_classic(params = {})
    # @param [Hash] params ({})
    def restore_address_to_classic(params = {}, options = {})
      req = build_request(:restore_address_to_classic, params)
      req.send_request(options)
    end

    # Restores an AMI from the Recycle Bin. For more information, see
    # [Recycle Bin][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/recycle-bin.html
    #
    # @option params [required, String] :image_id
    #   The ID of the AMI to restore.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::RestoreImageFromRecycleBinResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RestoreImageFromRecycleBinResult#return #return} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.restore_image_from_recycle_bin({
    #     image_id: "ImageId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RestoreImageFromRecycleBin AWS API Documentation
    #
    # @overload restore_image_from_recycle_bin(params = {})
    # @param [Hash] params ({})
    def restore_image_from_recycle_bin(params = {}, options = {})
      req = build_request(:restore_image_from_recycle_bin, params)
      req.send_request(options)
    end

    # Restores the entries from a previous version of a managed prefix list
    # to a new version of the prefix list.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :prefix_list_id
    #   The ID of the prefix list.
    #
    # @option params [required, Integer] :previous_version
    #   The version to restore.
    #
    # @option params [required, Integer] :current_version
    #   The current version number for the prefix list.
    #
    # @return [Types::RestoreManagedPrefixListVersionResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RestoreManagedPrefixListVersionResult#prefix_list #prefix_list} => Types::ManagedPrefixList
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.restore_managed_prefix_list_version({
    #     dry_run: false,
    #     prefix_list_id: "PrefixListResourceId", # required
    #     previous_version: 1, # required
    #     current_version: 1, # required
    #   })
    #
    # @example Response structure
    #
    #   resp.prefix_list.prefix_list_id #=> String
    #   resp.prefix_list.address_family #=> String
    #   resp.prefix_list.state #=> String, one of "create-in-progress", "create-complete", "create-failed", "modify-in-progress", "modify-complete", "modify-failed", "restore-in-progress", "restore-complete", "restore-failed", "delete-in-progress", "delete-complete", "delete-failed"
    #   resp.prefix_list.state_message #=> String
    #   resp.prefix_list.prefix_list_arn #=> String
    #   resp.prefix_list.prefix_list_name #=> String
    #   resp.prefix_list.max_entries #=> Integer
    #   resp.prefix_list.version #=> Integer
    #   resp.prefix_list.tags #=> Array
    #   resp.prefix_list.tags[0].key #=> String
    #   resp.prefix_list.tags[0].value #=> String
    #   resp.prefix_list.owner_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RestoreManagedPrefixListVersion AWS API Documentation
    #
    # @overload restore_managed_prefix_list_version(params = {})
    # @param [Hash] params ({})
    def restore_managed_prefix_list_version(params = {}, options = {})
      req = build_request(:restore_managed_prefix_list_version, params)
      req.send_request(options)
    end

    # Restores a snapshot from the Recycle Bin. For more information, see
    # [Restore snapshots from the Recycle Bin][1] in the *Amazon EBS User
    # Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/recycle-bin-working-with-snaps.html#recycle-bin-restore-snaps
    #
    # @option params [required, String] :snapshot_id
    #   The ID of the snapshot to restore.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::RestoreSnapshotFromRecycleBinResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RestoreSnapshotFromRecycleBinResult#snapshot_id #snapshot_id} => String
    #   * {Types::RestoreSnapshotFromRecycleBinResult#outpost_arn #outpost_arn} => String
    #   * {Types::RestoreSnapshotFromRecycleBinResult#description #description} => String
    #   * {Types::RestoreSnapshotFromRecycleBinResult#encrypted #encrypted} => Boolean
    #   * {Types::RestoreSnapshotFromRecycleBinResult#owner_id #owner_id} => String
    #   * {Types::RestoreSnapshotFromRecycleBinResult#progress #progress} => String
    #   * {Types::RestoreSnapshotFromRecycleBinResult#start_time #start_time} => Time
    #   * {Types::RestoreSnapshotFromRecycleBinResult#state #state} => String
    #   * {Types::RestoreSnapshotFromRecycleBinResult#volume_id #volume_id} => String
    #   * {Types::RestoreSnapshotFromRecycleBinResult#volume_size #volume_size} => Integer
    #   * {Types::RestoreSnapshotFromRecycleBinResult#sse_type #sse_type} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.restore_snapshot_from_recycle_bin({
    #     snapshot_id: "SnapshotId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.snapshot_id #=> String
    #   resp.outpost_arn #=> String
    #   resp.description #=> String
    #   resp.encrypted #=> Boolean
    #   resp.owner_id #=> String
    #   resp.progress #=> String
    #   resp.start_time #=> Time
    #   resp.state #=> String, one of "pending", "completed", "error", "recoverable", "recovering"
    #   resp.volume_id #=> String
    #   resp.volume_size #=> Integer
    #   resp.sse_type #=> String, one of "sse-ebs", "sse-kms", "none"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RestoreSnapshotFromRecycleBin AWS API Documentation
    #
    # @overload restore_snapshot_from_recycle_bin(params = {})
    # @param [Hash] params ({})
    def restore_snapshot_from_recycle_bin(params = {}, options = {})
      req = build_request(:restore_snapshot_from_recycle_bin, params)
      req.send_request(options)
    end

    # Restores an archived Amazon EBS snapshot for use temporarily or
    # permanently, or modifies the restore period or restore type for a
    # snapshot that was previously temporarily restored.
    #
    # For more information see [ Restore an archived snapshot][1] and [
    # modify the restore period or restore type for a temporarily restored
    # snapshot][2] in the *Amazon EBS User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/ebs/latest/userguide/working-with-snapshot-archiving.html#restore-archived-snapshot
    # [2]: https://docs.aws.amazon.com/ebs/latest/userguide/working-with-snapshot-archiving.html#modify-temp-restore-period
    #
    # @option params [required, String] :snapshot_id
    #   The ID of the snapshot to restore.
    #
    # @option params [Integer] :temporary_restore_days
    #   Specifies the number of days for which to temporarily restore an
    #   archived snapshot. Required for temporary restores only. The snapshot
    #   will be automatically re-archived after this period.
    #
    #   To temporarily restore an archived snapshot, specify the number of
    #   days and omit the **PermanentRestore** parameter or set it to `false`.
    #
    # @option params [Boolean] :permanent_restore
    #   Indicates whether to permanently restore an archived snapshot. To
    #   permanently restore an archived snapshot, specify `true` and omit the
    #   **RestoreSnapshotTierRequest$TemporaryRestoreDays** parameter.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::RestoreSnapshotTierResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RestoreSnapshotTierResult#snapshot_id #snapshot_id} => String
    #   * {Types::RestoreSnapshotTierResult#restore_start_time #restore_start_time} => Time
    #   * {Types::RestoreSnapshotTierResult#restore_duration #restore_duration} => Integer
    #   * {Types::RestoreSnapshotTierResult#is_permanent_restore #is_permanent_restore} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.restore_snapshot_tier({
    #     snapshot_id: "SnapshotId", # required
    #     temporary_restore_days: 1,
    #     permanent_restore: false,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.snapshot_id #=> String
    #   resp.restore_start_time #=> Time
    #   resp.restore_duration #=> Integer
    #   resp.is_permanent_restore #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RestoreSnapshotTier AWS API Documentation
    #
    # @overload restore_snapshot_tier(params = {})
    # @param [Hash] params ({})
    def restore_snapshot_tier(params = {}, options = {})
      req = build_request(:restore_snapshot_tier, params)
      req.send_request(options)
    end

    # Removes an ingress authorization rule from a Client VPN endpoint.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint with which the authorization rule is
    #   associated.
    #
    # @option params [required, String] :target_network_cidr
    #   The IPv4 address range, in CIDR notation, of the network for which
    #   access is being removed.
    #
    # @option params [String] :access_group_id
    #   The ID of the Active Directory group for which to revoke access.
    #
    # @option params [Boolean] :revoke_all_groups
    #   Indicates whether access should be revoked for all groups for a single
    #   `TargetNetworkCidr` that earlier authorized ingress for all groups
    #   using `AuthorizeAllGroups`. This does not impact other authorization
    #   rules that allowed ingress to the same `TargetNetworkCidr` with a
    #   specific `AccessGroupId`.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::RevokeClientVpnIngressResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RevokeClientVpnIngressResult#status #status} => Types::ClientVpnAuthorizationRuleStatus
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.revoke_client_vpn_ingress({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     target_network_cidr: "String", # required
    #     access_group_id: "String",
    #     revoke_all_groups: false,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.status.code #=> String, one of "authorizing", "active", "failed", "revoking"
    #   resp.status.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RevokeClientVpnIngress AWS API Documentation
    #
    # @overload revoke_client_vpn_ingress(params = {})
    # @param [Hash] params ({})
    def revoke_client_vpn_ingress(params = {}, options = {})
      req = build_request(:revoke_client_vpn_ingress, params)
      req.send_request(options)
    end

    # Removes the specified outbound (egress) rules from the specified
    # security group.
    #
    # You can specify rules using either rule IDs or security group rule
    # properties. If you use rule properties, the values that you specify
    # (for example, ports) must match the existing rule's values exactly.
    # Each rule has a protocol, from and to ports, and destination (CIDR
    # range, security group, or prefix list). For the TCP and UDP protocols,
    # you must also specify the destination port or range of ports. For the
    # ICMP protocol, you must also specify the ICMP type and code. If the
    # security group rule has a description, you do not need to specify the
    # description to revoke the rule.
    #
    # For a default VPC, if the values you specify do not match the existing
    # rule's values, no error is returned, and the output describes the
    # security group rules that were not revoked.
    #
    # Amazon Web Services recommends that you describe the security group to
    # verify that the rules were removed.
    #
    # Rule changes are propagated to instances within the security group as
    # quickly as possible. However, a small delay might occur.
    #
    # @option params [Array<String>] :security_group_rule_ids
    #   The IDs of the security group rules.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :group_id
    #   The ID of the security group.
    #
    # @option params [String] :source_security_group_name
    #   Not supported. Use a set of IP permissions to specify a destination
    #   security group.
    #
    # @option params [String] :source_security_group_owner_id
    #   Not supported. Use a set of IP permissions to specify a destination
    #   security group.
    #
    # @option params [String] :ip_protocol
    #   Not supported. Use a set of IP permissions to specify the protocol
    #   name or number.
    #
    # @option params [Integer] :from_port
    #   Not supported. Use a set of IP permissions to specify the port.
    #
    # @option params [Integer] :to_port
    #   Not supported. Use a set of IP permissions to specify the port.
    #
    # @option params [String] :cidr_ip
    #   Not supported. Use a set of IP permissions to specify the CIDR.
    #
    # @option params [Array<Types::IpPermission>] :ip_permissions
    #   The sets of IP permissions. You can't specify a destination security
    #   group and a CIDR IP address range in the same set of permissions.
    #
    # @return [Types::RevokeSecurityGroupEgressResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RevokeSecurityGroupEgressResult#return #return} => Boolean
    #   * {Types::RevokeSecurityGroupEgressResult#unknown_ip_permissions #unknown_ip_permissions} => Array&lt;Types::IpPermission&gt;
    #   * {Types::RevokeSecurityGroupEgressResult#revoked_security_group_rules #revoked_security_group_rules} => Array&lt;Types::RevokedSecurityGroupRule&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.revoke_security_group_egress({
    #     security_group_rule_ids: ["String"],
    #     dry_run: false,
    #     group_id: "SecurityGroupId", # required
    #     source_security_group_name: "String",
    #     source_security_group_owner_id: "String",
    #     ip_protocol: "String",
    #     from_port: 1,
    #     to_port: 1,
    #     cidr_ip: "String",
    #     ip_permissions: [
    #       {
    #         ip_protocol: "String",
    #         from_port: 1,
    #         to_port: 1,
    #         user_id_group_pairs: [
    #           {
    #             description: "String",
    #             user_id: "String",
    #             group_name: "String",
    #             group_id: "String",
    #             vpc_id: "String",
    #             vpc_peering_connection_id: "String",
    #             peering_status: "String",
    #           },
    #         ],
    #         ip_ranges: [
    #           {
    #             description: "String",
    #             cidr_ip: "String",
    #           },
    #         ],
    #         ipv_6_ranges: [
    #           {
    #             description: "String",
    #             cidr_ipv_6: "String",
    #           },
    #         ],
    #         prefix_list_ids: [
    #           {
    #             description: "String",
    #             prefix_list_id: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #   resp.unknown_ip_permissions #=> Array
    #   resp.unknown_ip_permissions[0].ip_protocol #=> String
    #   resp.unknown_ip_permissions[0].from_port #=> Integer
    #   resp.unknown_ip_permissions[0].to_port #=> Integer
    #   resp.unknown_ip_permissions[0].user_id_group_pairs #=> Array
    #   resp.unknown_ip_permissions[0].user_id_group_pairs[0].description #=> String
    #   resp.unknown_ip_permissions[0].user_id_group_pairs[0].user_id #=> String
    #   resp.unknown_ip_permissions[0].user_id_group_pairs[0].group_name #=> String
    #   resp.unknown_ip_permissions[0].user_id_group_pairs[0].group_id #=> String
    #   resp.unknown_ip_permissions[0].user_id_group_pairs[0].vpc_id #=> String
    #   resp.unknown_ip_permissions[0].user_id_group_pairs[0].vpc_peering_connection_id #=> String
    #   resp.unknown_ip_permissions[0].user_id_group_pairs[0].peering_status #=> String
    #   resp.unknown_ip_permissions[0].ip_ranges #=> Array
    #   resp.unknown_ip_permissions[0].ip_ranges[0].description #=> String
    #   resp.unknown_ip_permissions[0].ip_ranges[0].cidr_ip #=> String
    #   resp.unknown_ip_permissions[0].ipv_6_ranges #=> Array
    #   resp.unknown_ip_permissions[0].ipv_6_ranges[0].description #=> String
    #   resp.unknown_ip_permissions[0].ipv_6_ranges[0].cidr_ipv_6 #=> String
    #   resp.unknown_ip_permissions[0].prefix_list_ids #=> Array
    #   resp.unknown_ip_permissions[0].prefix_list_ids[0].description #=> String
    #   resp.unknown_ip_permissions[0].prefix_list_ids[0].prefix_list_id #=> String
    #   resp.revoked_security_group_rules #=> Array
    #   resp.revoked_security_group_rules[0].security_group_rule_id #=> String
    #   resp.revoked_security_group_rules[0].group_id #=> String
    #   resp.revoked_security_group_rules[0].is_egress #=> Boolean
    #   resp.revoked_security_group_rules[0].ip_protocol #=> String
    #   resp.revoked_security_group_rules[0].from_port #=> Integer
    #   resp.revoked_security_group_rules[0].to_port #=> Integer
    #   resp.revoked_security_group_rules[0].cidr_ipv_4 #=> String
    #   resp.revoked_security_group_rules[0].cidr_ipv_6 #=> String
    #   resp.revoked_security_group_rules[0].prefix_list_id #=> String
    #   resp.revoked_security_group_rules[0].referenced_group_id #=> String
    #   resp.revoked_security_group_rules[0].description #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RevokeSecurityGroupEgress AWS API Documentation
    #
    # @overload revoke_security_group_egress(params = {})
    # @param [Hash] params ({})
    def revoke_security_group_egress(params = {}, options = {})
      req = build_request(:revoke_security_group_egress, params)
      req.send_request(options)
    end

    # Removes the specified inbound (ingress) rules from a security group.
    #
    # You can specify rules using either rule IDs or security group rule
    # properties. If you use rule properties, the values that you specify
    # (for example, ports) must match the existing rule's values exactly.
    # Each rule has a protocol, from and to ports, and source (CIDR range,
    # security group, or prefix list). For the TCP and UDP protocols, you
    # must also specify the destination port or range of ports. For the ICMP
    # protocol, you must also specify the ICMP type and code. If the
    # security group rule has a description, you do not need to specify the
    # description to revoke the rule.
    #
    # For a default VPC, if the values you specify do not match the existing
    # rule's values, no error is returned, and the output describes the
    # security group rules that were not revoked.
    #
    # For a non-default VPC, if the values you specify do not match the
    # existing rule's values, an `InvalidPermission.NotFound` client error
    # is returned, and no rules are revoked.
    #
    # Amazon Web Services recommends that you describe the security group to
    # verify that the rules were removed.
    #
    # Rule changes are propagated to instances within the security group as
    # quickly as possible. However, a small delay might occur.
    #
    # @option params [String] :cidr_ip
    #   The CIDR IP address range. You can't specify this parameter when
    #   specifying a source security group.
    #
    # @option params [Integer] :from_port
    #   If the protocol is TCP or UDP, this is the start of the port range. If
    #   the protocol is ICMP, this is the ICMP type or -1 (all ICMP types).
    #
    # @option params [String] :group_id
    #   The ID of the security group.
    #
    # @option params [String] :group_name
    #   \[Default VPC\] The name of the security group. You must specify
    #   either the security group ID or the security group name in the
    #   request. For security groups in a nondefault VPC, you must specify the
    #   security group ID.
    #
    # @option params [Array<Types::IpPermission>] :ip_permissions
    #   The sets of IP permissions. You can't specify a source security group
    #   and a CIDR IP address range in the same set of permissions.
    #
    # @option params [String] :ip_protocol
    #   The IP protocol name (`tcp`, `udp`, `icmp`) or number (see [Protocol
    #   Numbers][1]). Use `-1` to specify all.
    #
    #
    #
    #   [1]: http://www.iana.org/assignments/protocol-numbers/protocol-numbers.xhtml
    #
    # @option params [String] :source_security_group_name
    #   \[Default VPC\] The name of the source security group. You can't
    #   specify this parameter in combination with the following parameters:
    #   the CIDR IP address range, the start of the port range, the IP
    #   protocol, and the end of the port range. The source security group
    #   must be in the same VPC. To revoke a specific rule for an IP protocol
    #   and port range, use a set of IP permissions instead.
    #
    # @option params [String] :source_security_group_owner_id
    #   Not supported.
    #
    # @option params [Integer] :to_port
    #   If the protocol is TCP or UDP, this is the end of the port range. If
    #   the protocol is ICMP, this is the ICMP code or -1 (all ICMP codes).
    #
    # @option params [Array<String>] :security_group_rule_ids
    #   The IDs of the security group rules.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::RevokeSecurityGroupIngressResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RevokeSecurityGroupIngressResult#return #return} => Boolean
    #   * {Types::RevokeSecurityGroupIngressResult#unknown_ip_permissions #unknown_ip_permissions} => Array&lt;Types::IpPermission&gt;
    #   * {Types::RevokeSecurityGroupIngressResult#revoked_security_group_rules #revoked_security_group_rules} => Array&lt;Types::RevokedSecurityGroupRule&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.revoke_security_group_ingress({
    #     cidr_ip: "String",
    #     from_port: 1,
    #     group_id: "SecurityGroupId",
    #     group_name: "SecurityGroupName",
    #     ip_permissions: [
    #       {
    #         ip_protocol: "String",
    #         from_port: 1,
    #         to_port: 1,
    #         user_id_group_pairs: [
    #           {
    #             description: "String",
    #             user_id: "String",
    #             group_name: "String",
    #             group_id: "String",
    #             vpc_id: "String",
    #             vpc_peering_connection_id: "String",
    #             peering_status: "String",
    #           },
    #         ],
    #         ip_ranges: [
    #           {
    #             description: "String",
    #             cidr_ip: "String",
    #           },
    #         ],
    #         ipv_6_ranges: [
    #           {
    #             description: "String",
    #             cidr_ipv_6: "String",
    #           },
    #         ],
    #         prefix_list_ids: [
    #           {
    #             description: "String",
    #             prefix_list_id: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     ip_protocol: "String",
    #     source_security_group_name: "String",
    #     source_security_group_owner_id: "String",
    #     to_port: 1,
    #     security_group_rule_ids: ["String"],
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #   resp.unknown_ip_permissions #=> Array
    #   resp.unknown_ip_permissions[0].ip_protocol #=> String
    #   resp.unknown_ip_permissions[0].from_port #=> Integer
    #   resp.unknown_ip_permissions[0].to_port #=> Integer
    #   resp.unknown_ip_permissions[0].user_id_group_pairs #=> Array
    #   resp.unknown_ip_permissions[0].user_id_group_pairs[0].description #=> String
    #   resp.unknown_ip_permissions[0].user_id_group_pairs[0].user_id #=> String
    #   resp.unknown_ip_permissions[0].user_id_group_pairs[0].group_name #=> String
    #   resp.unknown_ip_permissions[0].user_id_group_pairs[0].group_id #=> String
    #   resp.unknown_ip_permissions[0].user_id_group_pairs[0].vpc_id #=> String
    #   resp.unknown_ip_permissions[0].user_id_group_pairs[0].vpc_peering_connection_id #=> String
    #   resp.unknown_ip_permissions[0].user_id_group_pairs[0].peering_status #=> String
    #   resp.unknown_ip_permissions[0].ip_ranges #=> Array
    #   resp.unknown_ip_permissions[0].ip_ranges[0].description #=> String
    #   resp.unknown_ip_permissions[0].ip_ranges[0].cidr_ip #=> String
    #   resp.unknown_ip_permissions[0].ipv_6_ranges #=> Array
    #   resp.unknown_ip_permissions[0].ipv_6_ranges[0].description #=> String
    #   resp.unknown_ip_permissions[0].ipv_6_ranges[0].cidr_ipv_6 #=> String
    #   resp.unknown_ip_permissions[0].prefix_list_ids #=> Array
    #   resp.unknown_ip_permissions[0].prefix_list_ids[0].description #=> String
    #   resp.unknown_ip_permissions[0].prefix_list_ids[0].prefix_list_id #=> String
    #   resp.revoked_security_group_rules #=> Array
    #   resp.revoked_security_group_rules[0].security_group_rule_id #=> String
    #   resp.revoked_security_group_rules[0].group_id #=> String
    #   resp.revoked_security_group_rules[0].is_egress #=> Boolean
    #   resp.revoked_security_group_rules[0].ip_protocol #=> String
    #   resp.revoked_security_group_rules[0].from_port #=> Integer
    #   resp.revoked_security_group_rules[0].to_port #=> Integer
    #   resp.revoked_security_group_rules[0].cidr_ipv_4 #=> String
    #   resp.revoked_security_group_rules[0].cidr_ipv_6 #=> String
    #   resp.revoked_security_group_rules[0].prefix_list_id #=> String
    #   resp.revoked_security_group_rules[0].referenced_group_id #=> String
    #   resp.revoked_security_group_rules[0].description #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RevokeSecurityGroupIngress AWS API Documentation
    #
    # @overload revoke_security_group_ingress(params = {})
    # @param [Hash] params ({})
    def revoke_security_group_ingress(params = {}, options = {})
      req = build_request(:revoke_security_group_ingress, params)
      req.send_request(options)
    end

    # Launches the specified number of instances using an AMI for which you
    # have permissions.
    #
    # You can specify a number of options, or leave the default options. The
    # following rules apply:
    #
    # * If you don't specify a subnet ID, we choose a default subnet from
    #   your default VPC for you. If you don't have a default VPC, you must
    #   specify a subnet ID in the request.
    #
    # * All instances have a network interface with a primary private IPv4
    #   address. If you don't specify this address, we choose one from the
    #   IPv4 range of your subnet.
    #
    # * Not all instance types support IPv6 addresses. For more information,
    #   see [Instance types][1].
    #
    # * If you don't specify a security group ID, we use the default
    #   security group for the VPC. For more information, see [Security
    #   groups][2].
    #
    # * If any of the AMIs have a product code attached for which the user
    #   has not subscribed, the request fails.
    #
    # You can create a [launch template][3], which is a resource that
    # contains the parameters to launch an instance. When you launch an
    # instance using RunInstances, you can specify the launch template
    # instead of specifying the launch parameters.
    #
    # To ensure faster instance launches, break up large requests into
    # smaller batches. For example, create five separate launch requests for
    # 100 instances each instead of one launch request for 500 instances.
    #
    # `RunInstances` is subject to both request rate limiting and resource
    # rate limiting. For more information, see [Request throttling][4].
    #
    # An instance is ready for you to use when it's in the `running` state.
    # You can check the state of your instance using DescribeInstances. You
    # can tag instances and EBS volumes during launch, after launch, or
    # both. For more information, see CreateTags and [Tagging your Amazon
    # EC2 resources][5].
    #
    # Linux instances have access to the public key of the key pair at boot.
    # You can use this key to provide secure access to the instance. Amazon
    # EC2 public images use this feature to provide secure access without
    # passwords. For more information, see [Key pairs][6].
    #
    # For troubleshooting, see [What to do if an instance immediately
    # terminates][7], and [Troubleshooting connecting to your instance][8].
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-network-security.html
    # [3]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-launch-templates.html
    # [4]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-throttling.html
    # [5]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_Tags.html
    # [6]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-key-pairs.html
    # [7]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Using_InstanceStraightToTerminated.html
    # [8]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/TroubleshootingInstancesConnecting.html
    #
    # @option params [Array<Types::BlockDeviceMapping>] :block_device_mappings
    #   The block device mapping, which defines the EBS volumes and instance
    #   store volumes to attach to the instance at launch. For more
    #   information, see [Block device mappings][1] in the *Amazon EC2 User
    #   Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/block-device-mapping-concepts.html
    #
    # @option params [String] :image_id
    #   The ID of the AMI. An AMI ID is required to launch an instance and
    #   must be specified here or in a launch template.
    #
    # @option params [String] :instance_type
    #   The instance type. For more information, see [Amazon EC2 instance
    #   types][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-types.html
    #
    # @option params [Integer] :ipv_6_address_count
    #   The number of IPv6 addresses to associate with the primary network
    #   interface. Amazon EC2 chooses the IPv6 addresses from the range of
    #   your subnet. You cannot specify this option and the option to assign
    #   specific IPv6 addresses in the same request. You can specify this
    #   option if you've specified a minimum number of instances to launch.
    #
    #   You cannot specify this option and the network interfaces option in
    #   the same request.
    #
    # @option params [Array<Types::InstanceIpv6Address>] :ipv_6_addresses
    #   The IPv6 addresses from the range of the subnet to associate with the
    #   primary network interface. You cannot specify this option and the
    #   option to assign a number of IPv6 addresses in the same request. You
    #   cannot specify this option if you've specified a minimum number of
    #   instances to launch.
    #
    #   You cannot specify this option and the network interfaces option in
    #   the same request.
    #
    # @option params [String] :kernel_id
    #   The ID of the kernel.
    #
    #   We recommend that you use PV-GRUB instead of kernels and RAM disks.
    #   For more information, see [PV-GRUB][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedkernels.html
    #
    # @option params [String] :key_name
    #   The name of the key pair. You can create a key pair using
    #   [CreateKeyPair][1] or [ImportKeyPair][2].
    #
    #   If you do not specify a key pair, you can't connect to the instance
    #   unless you choose an AMI that is configured to allow users another way
    #   to log in.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateKeyPair.html
    #   [2]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ImportKeyPair.html
    #
    # @option params [required, Integer] :max_count
    #   The maximum number of instances to launch. If you specify a value that
    #   is more capacity than Amazon EC2 can launch in the target Availability
    #   Zone, Amazon EC2 launches the largest possible number of instances
    #   above the specified minimum count.
    #
    #   Constraints: Between 1 and the quota for the specified instance type
    #   for your account for this Region. For more information, see [Amazon
    #   EC2 instance type quotas][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/instancetypes/ec2-instance-quotas.html
    #
    # @option params [required, Integer] :min_count
    #   The minimum number of instances to launch. If you specify a value that
    #   is more capacity than Amazon EC2 can provide in the target
    #   Availability Zone, Amazon EC2 does not launch any instances.
    #
    #   Constraints: Between 1 and the quota for the specified instance type
    #   for your account for this Region. For more information, see [Amazon
    #   EC2 instance type quotas][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/instancetypes/ec2-instance-quotas.html
    #
    # @option params [Types::RunInstancesMonitoringEnabled] :monitoring
    #   Specifies whether detailed monitoring is enabled for the instance.
    #
    # @option params [Types::Placement] :placement
    #   The placement for the instance.
    #
    # @option params [String] :ramdisk_id
    #   The ID of the RAM disk to select. Some kernels require additional
    #   drivers at launch. Check the kernel requirements for information about
    #   whether you need to specify a RAM disk. To find kernel requirements,
    #   go to the Amazon Web Services Resource Center and search for the
    #   kernel ID.
    #
    #   We recommend that you use PV-GRUB instead of kernels and RAM disks.
    #   For more information, see [PV-GRUB][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/UserProvidedkernels.html
    #
    # @option params [Array<String>] :security_group_ids
    #   The IDs of the security groups. You can create a security group using
    #   [CreateSecurityGroup][1].
    #
    #   If you specify a network interface, you must specify any security
    #   groups as part of the network interface instead of using this
    #   parameter.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateSecurityGroup.html
    #
    # @option params [Array<String>] :security_groups
    #   \[Default VPC\] The names of the security groups.
    #
    #   If you specify a network interface, you must specify any security
    #   groups as part of the network interface instead of using this
    #   parameter.
    #
    #   Default: Amazon EC2 uses the default security group.
    #
    # @option params [String] :subnet_id
    #   The ID of the subnet to launch the instance into.
    #
    #   If you specify a network interface, you must specify any subnets as
    #   part of the network interface instead of using this parameter.
    #
    # @option params [String] :user_data
    #   The user data to make available to the instance. User data must be
    #   base64-encoded. Depending on the tool or SDK that you're using, the
    #   base64-encoding might be performed for you. For more information, see
    #   [Work with instance user data][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instancedata-add-user-data.html
    #
    # @option params [Array<Types::ElasticGpuSpecification>] :elastic_gpu_specification
    #   An elastic GPU to associate with the instance.
    #
    #   <note markdown="1"> Amazon Elastic Graphics reached end of life on January 8, 2024.
    #
    #    </note>
    #
    # @option params [Array<Types::ElasticInferenceAccelerator>] :elastic_inference_accelerators
    #   An elastic inference accelerator to associate with the instance.
    #
    #   <note markdown="1"> Amazon Elastic Inference is no longer available.
    #
    #    </note>
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply to the resources that are created during instance
    #   launch.
    #
    #   You can specify tags for the following resources only:
    #
    #   * Instances
    #
    #   * Volumes
    #
    #   * Spot Instance requests
    #
    #   * Network interfaces
    #
    #   To tag a resource after it has been created, see [CreateTags][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_CreateTags.html
    #
    # @option params [Types::LaunchTemplateSpecification] :launch_template
    #   The launch template. Any additional parameters that you specify for
    #   the new instance overwrite the corresponding parameters included in
    #   the launch template.
    #
    # @option params [Types::InstanceMarketOptionsRequest] :instance_market_options
    #   The market (purchasing) option for the instances.
    #
    #   For RunInstances, persistent Spot Instance requests are only supported
    #   when **InstanceInterruptionBehavior** is set to either `hibernate` or
    #   `stop`.
    #
    # @option params [Types::CreditSpecificationRequest] :credit_specification
    #   The credit option for CPU usage of the burstable performance instance.
    #   Valid values are `standard` and `unlimited`. To change this attribute
    #   after launch, use [ ModifyInstanceCreditSpecification][1]. For more
    #   information, see [Burstable performance instances][2] in the *Amazon
    #   EC2 User Guide*.
    #
    #   Default: `standard` (T2 instances) or `unlimited` (T3/T3a/T4g
    #   instances)
    #
    #   For T3 instances with `host` tenancy, only `standard` is supported.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/API_ModifyInstanceCreditSpecification.html
    #   [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/burstable-performance-instances.html
    #
    # @option params [Types::CpuOptionsRequest] :cpu_options
    #   The CPU options for the instance. For more information, see [Optimize
    #   CPU options][1] in the *Amazon EC2 User Guide*.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/instance-optimize-cpu.html
    #
    # @option params [Types::CapacityReservationSpecification] :capacity_reservation_specification
    #   Information about the Capacity Reservation targeting option. If you do
    #   not specify this parameter, the instance's Capacity Reservation
    #   preference defaults to `open`, which enables it to run in any open
    #   Capacity Reservation that has matching attributes (instance type,
    #   platform, Availability Zone, and tenancy).
    #
    # @option params [Types::HibernationOptionsRequest] :hibernation_options
    #   Indicates whether an instance is enabled for hibernation. This
    #   parameter is valid only if the instance meets the [hibernation
    #   prerequisites][1]. For more information, see [Hibernate your Amazon
    #   EC2 instance][2] in the *Amazon EC2 User Guide*.
    #
    #   You can't enable hibernation and Amazon Web Services Nitro Enclaves
    #   on the same instance.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/hibernating-prerequisites.html
    #   [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html
    #
    # @option params [Array<Types::LicenseConfigurationRequest>] :license_specifications
    #   The license configurations.
    #
    # @option params [Types::InstanceMetadataOptionsRequest] :metadata_options
    #   The metadata options for the instance. For more information, see
    #   [Instance metadata and user data][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-metadata.html
    #
    # @option params [Types::EnclaveOptionsRequest] :enclave_options
    #   Indicates whether the instance is enabled for Amazon Web Services
    #   Nitro Enclaves. For more information, see [What is Amazon Web Services
    #   Nitro Enclaves?][1] in the *Amazon Web Services Nitro Enclaves User
    #   Guide*.
    #
    #   You can't enable Amazon Web Services Nitro Enclaves and hibernation
    #   on the same instance.
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/enclaves/latest/user/nitro-enclave.html
    #
    # @option params [Types::PrivateDnsNameOptionsRequest] :private_dns_name_options
    #   The options for the instance hostname. The default values are
    #   inherited from the subnet. Applies only if creating a network
    #   interface, not attaching an existing one.
    #
    # @option params [Types::InstanceMaintenanceOptionsRequest] :maintenance_options
    #   The maintenance and recovery options for the instance.
    #
    # @option params [Boolean] :disable_api_stop
    #   Indicates whether an instance is enabled for stop protection. For more
    #   information, see [Stop protection][1].
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Stop_Start.html#Using_StopProtection
    #
    # @option params [Boolean] :enable_primary_ipv_6
    #   If you’re launching an instance into a dual-stack or IPv6-only subnet,
    #   you can enable assigning a primary IPv6 address. A primary IPv6
    #   address is an IPv6 GUA address associated with an ENI that you have
    #   enabled to use a primary IPv6 address. Use this option if an instance
    #   relies on its IPv6 address not changing. When you launch the instance,
    #   Amazon Web Services will automatically assign an IPv6 address
    #   associated with the ENI attached to your instance to be the primary
    #   IPv6 address. Once you enable an IPv6 GUA address to be a primary
    #   IPv6, you cannot disable it. When you enable an IPv6 GUA address to be
    #   a primary IPv6, the first IPv6 GUA will be made the primary IPv6
    #   address until the instance is terminated or the network interface is
    #   detached. If you have multiple IPv6 addresses associated with an ENI
    #   attached to your instance and you enable a primary IPv6 address, the
    #   first IPv6 GUA address associated with the ENI becomes the primary
    #   IPv6 address.
    #
    # @option params [Types::InstanceNetworkPerformanceOptionsRequest] :network_performance_options
    #   Contains settings for the network performance options for the
    #   instance.
    #
    # @option params [Types::OperatorRequest] :operator
    #   Reserved for internal use.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Boolean] :disable_api_termination
    #   Indicates whether termination protection is enabled for the instance.
    #   The default is `false`, which means that you can terminate the
    #   instance using the Amazon EC2 console, command line tools, or API. You
    #   can enable termination protection when you launch an instance, while
    #   the instance is running, or while the instance is stopped.
    #
    # @option params [String] :instance_initiated_shutdown_behavior
    #   Indicates whether an instance stops or terminates when you initiate
    #   shutdown from the instance (using the operating system command for
    #   system shutdown).
    #
    #   Default: `stop`
    #
    # @option params [String] :private_ip_address
    #   The primary IPv4 address. You must specify a value from the IPv4
    #   address range of the subnet.
    #
    #   Only one private IP address can be designated as primary. You can't
    #   specify this option if you've specified the option to designate a
    #   private IP address as the primary IP address in a network interface
    #   specification. You cannot specify this option if you're launching
    #   more than one instance in the request.
    #
    #   You cannot specify this option and the network interfaces option in
    #   the same request.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier you provide to ensure the
    #   idempotency of the request. If you do not specify a client token, a
    #   randomly generated token is used for the request to ensure
    #   idempotency.
    #
    #   For more information, see [Ensuring Idempotency][1].
    #
    #   Constraints: Maximum 64 ASCII characters
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [String] :additional_info
    #   Reserved.
    #
    # @option params [Array<Types::InstanceNetworkInterfaceSpecification>] :network_interfaces
    #   The network interfaces to associate with the instance.
    #
    # @option params [Types::IamInstanceProfileSpecification] :iam_instance_profile
    #   The name or Amazon Resource Name (ARN) of an IAM instance profile.
    #
    # @option params [Boolean] :ebs_optimized
    #   Indicates whether the instance is optimized for Amazon EBS I/O. This
    #   optimization provides dedicated throughput to Amazon EBS and an
    #   optimized configuration stack to provide optimal Amazon EBS I/O
    #   performance. This optimization isn't available with all instance
    #   types. Additional usage charges apply when using an EBS-optimized
    #   instance.
    #
    #   Default: `false`
    #
    # @return [Types::Reservation] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::Reservation#reservation_id #reservation_id} => String
    #   * {Types::Reservation#owner_id #owner_id} => String
    #   * {Types::Reservation#requester_id #requester_id} => String
    #   * {Types::Reservation#groups #groups} => Array&lt;Types::GroupIdentifier&gt;
    #   * {Types::Reservation#instances #instances} => Array&lt;Types::Instance&gt;
    #
    #
    # @example Example: To launch an instance
    #
    #   # This example launches an instance using the specified AMI, instance type, security group, subnet, block device mapping,
    #   # and tags.
    #
    #   resp = client.run_instances({
    #     block_device_mappings: [
    #       {
    #         device_name: "/dev/sdh", 
    #         ebs: {
    #           volume_size: 100, 
    #         }, 
    #       }, 
    #     ], 
    #     image_id: "ami-abc12345", 
    #     instance_type: "t2.micro", 
    #     key_name: "my-key-pair", 
    #     max_count: 1, 
    #     min_count: 1, 
    #     security_group_ids: [
    #       "sg-1a2b3c4d", 
    #     ], 
    #     subnet_id: "subnet-6e7f829e", 
    #     tag_specifications: [
    #       {
    #         resource_type: "instance", 
    #         tags: [
    #           {
    #             key: "Purpose", 
    #             value: "test", 
    #           }, 
    #         ], 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.run_instances({
    #     block_device_mappings: [
    #       {
    #         ebs: {
    #           delete_on_termination: false,
    #           iops: 1,
    #           snapshot_id: "SnapshotId",
    #           volume_size: 1,
    #           volume_type: "standard", # accepts standard, io1, io2, gp2, sc1, st1, gp3
    #           kms_key_id: "String",
    #           throughput: 1,
    #           outpost_arn: "String",
    #           encrypted: false,
    #         },
    #         no_device: "String",
    #         device_name: "String",
    #         virtual_name: "String",
    #       },
    #     ],
    #     image_id: "ImageId",
    #     instance_type: "a1.medium", # accepts a1.medium, a1.large, a1.xlarge, a1.2xlarge, a1.4xlarge, a1.metal, c1.medium, c1.xlarge, c3.large, c3.xlarge, c3.2xlarge, c3.4xlarge, c3.8xlarge, c4.large, c4.xlarge, c4.2xlarge, c4.4xlarge, c4.8xlarge, c5.large, c5.xlarge, c5.2xlarge, c5.4xlarge, c5.9xlarge, c5.12xlarge, c5.18xlarge, c5.24xlarge, c5.metal, c5a.large, c5a.xlarge, c5a.2xlarge, c5a.4xlarge, c5a.8xlarge, c5a.12xlarge, c5a.16xlarge, c5a.24xlarge, c5ad.large, c5ad.xlarge, c5ad.2xlarge, c5ad.4xlarge, c5ad.8xlarge, c5ad.12xlarge, c5ad.16xlarge, c5ad.24xlarge, c5d.large, c5d.xlarge, c5d.2xlarge, c5d.4xlarge, c5d.9xlarge, c5d.12xlarge, c5d.18xlarge, c5d.24xlarge, c5d.metal, c5n.large, c5n.xlarge, c5n.2xlarge, c5n.4xlarge, c5n.9xlarge, c5n.18xlarge, c5n.metal, c6g.medium, c6g.large, c6g.xlarge, c6g.2xlarge, c6g.4xlarge, c6g.8xlarge, c6g.12xlarge, c6g.16xlarge, c6g.metal, c6gd.medium, c6gd.large, c6gd.xlarge, c6gd.2xlarge, c6gd.4xlarge, c6gd.8xlarge, c6gd.12xlarge, c6gd.16xlarge, c6gd.metal, c6gn.medium, c6gn.large, c6gn.xlarge, c6gn.2xlarge, c6gn.4xlarge, c6gn.8xlarge, c6gn.12xlarge, c6gn.16xlarge, c6i.large, c6i.xlarge, c6i.2xlarge, c6i.4xlarge, c6i.8xlarge, c6i.12xlarge, c6i.16xlarge, c6i.24xlarge, c6i.32xlarge, c6i.metal, cc1.4xlarge, cc2.8xlarge, cg1.4xlarge, cr1.8xlarge, d2.xlarge, d2.2xlarge, d2.4xlarge, d2.8xlarge, d3.xlarge, d3.2xlarge, d3.4xlarge, d3.8xlarge, d3en.xlarge, d3en.2xlarge, d3en.4xlarge, d3en.6xlarge, d3en.8xlarge, d3en.12xlarge, dl1.24xlarge, f1.2xlarge, f1.4xlarge, f1.16xlarge, g2.2xlarge, g2.8xlarge, g3.4xlarge, g3.8xlarge, g3.16xlarge, g3s.xlarge, g4ad.xlarge, g4ad.2xlarge, g4ad.4xlarge, g4ad.8xlarge, g4ad.16xlarge, g4dn.xlarge, g4dn.2xlarge, g4dn.4xlarge, g4dn.8xlarge, g4dn.12xlarge, g4dn.16xlarge, g4dn.metal, g5.xlarge, g5.2xlarge, g5.4xlarge, g5.8xlarge, g5.12xlarge, g5.16xlarge, g5.24xlarge, g5.48xlarge, g5g.xlarge, g5g.2xlarge, g5g.4xlarge, g5g.8xlarge, g5g.16xlarge, g5g.metal, hi1.4xlarge, hpc6a.48xlarge, hs1.8xlarge, h1.2xlarge, h1.4xlarge, h1.8xlarge, h1.16xlarge, i2.xlarge, i2.2xlarge, i2.4xlarge, i2.8xlarge, i3.large, i3.xlarge, i3.2xlarge, i3.4xlarge, i3.8xlarge, i3.16xlarge, i3.metal, i3en.large, i3en.xlarge, i3en.2xlarge, i3en.3xlarge, i3en.6xlarge, i3en.12xlarge, i3en.24xlarge, i3en.metal, im4gn.large, im4gn.xlarge, im4gn.2xlarge, im4gn.4xlarge, im4gn.8xlarge, im4gn.16xlarge, inf1.xlarge, inf1.2xlarge, inf1.6xlarge, inf1.24xlarge, is4gen.medium, is4gen.large, is4gen.xlarge, is4gen.2xlarge, is4gen.4xlarge, is4gen.8xlarge, m1.small, m1.medium, m1.large, m1.xlarge, m2.xlarge, m2.2xlarge, m2.4xlarge, m3.medium, m3.large, m3.xlarge, m3.2xlarge, m4.large, m4.xlarge, m4.2xlarge, m4.4xlarge, m4.10xlarge, m4.16xlarge, m5.large, m5.xlarge, m5.2xlarge, m5.4xlarge, m5.8xlarge, m5.12xlarge, m5.16xlarge, m5.24xlarge, m5.metal, m5a.large, m5a.xlarge, m5a.2xlarge, m5a.4xlarge, m5a.8xlarge, m5a.12xlarge, m5a.16xlarge, m5a.24xlarge, m5ad.large, m5ad.xlarge, m5ad.2xlarge, m5ad.4xlarge, m5ad.8xlarge, m5ad.12xlarge, m5ad.16xlarge, m5ad.24xlarge, m5d.large, m5d.xlarge, m5d.2xlarge, m5d.4xlarge, m5d.8xlarge, m5d.12xlarge, m5d.16xlarge, m5d.24xlarge, m5d.metal, m5dn.large, m5dn.xlarge, m5dn.2xlarge, m5dn.4xlarge, m5dn.8xlarge, m5dn.12xlarge, m5dn.16xlarge, m5dn.24xlarge, m5dn.metal, m5n.large, m5n.xlarge, m5n.2xlarge, m5n.4xlarge, m5n.8xlarge, m5n.12xlarge, m5n.16xlarge, m5n.24xlarge, m5n.metal, m5zn.large, m5zn.xlarge, m5zn.2xlarge, m5zn.3xlarge, m5zn.6xlarge, m5zn.12xlarge, m5zn.metal, m6a.large, m6a.xlarge, m6a.2xlarge, m6a.4xlarge, m6a.8xlarge, m6a.12xlarge, m6a.16xlarge, m6a.24xlarge, m6a.32xlarge, m6a.48xlarge, m6g.metal, m6g.medium, m6g.large, m6g.xlarge, m6g.2xlarge, m6g.4xlarge, m6g.8xlarge, m6g.12xlarge, m6g.16xlarge, m6gd.metal, m6gd.medium, m6gd.large, m6gd.xlarge, m6gd.2xlarge, m6gd.4xlarge, m6gd.8xlarge, m6gd.12xlarge, m6gd.16xlarge, m6i.large, m6i.xlarge, m6i.2xlarge, m6i.4xlarge, m6i.8xlarge, m6i.12xlarge, m6i.16xlarge, m6i.24xlarge, m6i.32xlarge, m6i.metal, mac1.metal, p2.xlarge, p2.8xlarge, p2.16xlarge, p3.2xlarge, p3.8xlarge, p3.16xlarge, p3dn.24xlarge, p4d.24xlarge, r3.large, r3.xlarge, r3.2xlarge, r3.4xlarge, r3.8xlarge, r4.large, r4.xlarge, r4.2xlarge, r4.4xlarge, r4.8xlarge, r4.16xlarge, r5.large, r5.xlarge, r5.2xlarge, r5.4xlarge, r5.8xlarge, r5.12xlarge, r5.16xlarge, r5.24xlarge, r5.metal, r5a.large, r5a.xlarge, r5a.2xlarge, r5a.4xlarge, r5a.8xlarge, r5a.12xlarge, r5a.16xlarge, r5a.24xlarge, r5ad.large, r5ad.xlarge, r5ad.2xlarge, r5ad.4xlarge, r5ad.8xlarge, r5ad.12xlarge, r5ad.16xlarge, r5ad.24xlarge, r5b.large, r5b.xlarge, r5b.2xlarge, r5b.4xlarge, r5b.8xlarge, r5b.12xlarge, r5b.16xlarge, r5b.24xlarge, r5b.metal, r5d.large, r5d.xlarge, r5d.2xlarge, r5d.4xlarge, r5d.8xlarge, r5d.12xlarge, r5d.16xlarge, r5d.24xlarge, r5d.metal, r5dn.large, r5dn.xlarge, r5dn.2xlarge, r5dn.4xlarge, r5dn.8xlarge, r5dn.12xlarge, r5dn.16xlarge, r5dn.24xlarge, r5dn.metal, r5n.large, r5n.xlarge, r5n.2xlarge, r5n.4xlarge, r5n.8xlarge, r5n.12xlarge, r5n.16xlarge, r5n.24xlarge, r5n.metal, r6g.medium, r6g.large, r6g.xlarge, r6g.2xlarge, r6g.4xlarge, r6g.8xlarge, r6g.12xlarge, r6g.16xlarge, r6g.metal, r6gd.medium, r6gd.large, r6gd.xlarge, r6gd.2xlarge, r6gd.4xlarge, r6gd.8xlarge, r6gd.12xlarge, r6gd.16xlarge, r6gd.metal, r6i.large, r6i.xlarge, r6i.2xlarge, r6i.4xlarge, r6i.8xlarge, r6i.12xlarge, r6i.16xlarge, r6i.24xlarge, r6i.32xlarge, r6i.metal, t1.micro, t2.nano, t2.micro, t2.small, t2.medium, t2.large, t2.xlarge, t2.2xlarge, t3.nano, t3.micro, t3.small, t3.medium, t3.large, t3.xlarge, t3.2xlarge, t3a.nano, t3a.micro, t3a.small, t3a.medium, t3a.large, t3a.xlarge, t3a.2xlarge, t4g.nano, t4g.micro, t4g.small, t4g.medium, t4g.large, t4g.xlarge, t4g.2xlarge, u-6tb1.56xlarge, u-6tb1.112xlarge, u-9tb1.112xlarge, u-12tb1.112xlarge, u-6tb1.metal, u-9tb1.metal, u-12tb1.metal, u-18tb1.metal, u-24tb1.metal, vt1.3xlarge, vt1.6xlarge, vt1.24xlarge, x1.16xlarge, x1.32xlarge, x1e.xlarge, x1e.2xlarge, x1e.4xlarge, x1e.8xlarge, x1e.16xlarge, x1e.32xlarge, x2iezn.2xlarge, x2iezn.4xlarge, x2iezn.6xlarge, x2iezn.8xlarge, x2iezn.12xlarge, x2iezn.metal, x2gd.medium, x2gd.large, x2gd.xlarge, x2gd.2xlarge, x2gd.4xlarge, x2gd.8xlarge, x2gd.12xlarge, x2gd.16xlarge, x2gd.metal, z1d.large, z1d.xlarge, z1d.2xlarge, z1d.3xlarge, z1d.6xlarge, z1d.12xlarge, z1d.metal, x2idn.16xlarge, x2idn.24xlarge, x2idn.32xlarge, x2iedn.xlarge, x2iedn.2xlarge, x2iedn.4xlarge, x2iedn.8xlarge, x2iedn.16xlarge, x2iedn.24xlarge, x2iedn.32xlarge, c6a.large, c6a.xlarge, c6a.2xlarge, c6a.4xlarge, c6a.8xlarge, c6a.12xlarge, c6a.16xlarge, c6a.24xlarge, c6a.32xlarge, c6a.48xlarge, c6a.metal, m6a.metal, i4i.large, i4i.xlarge, i4i.2xlarge, i4i.4xlarge, i4i.8xlarge, i4i.16xlarge, i4i.32xlarge, i4i.metal, x2idn.metal, x2iedn.metal, c7g.medium, c7g.large, c7g.xlarge, c7g.2xlarge, c7g.4xlarge, c7g.8xlarge, c7g.12xlarge, c7g.16xlarge, mac2.metal, c6id.large, c6id.xlarge, c6id.2xlarge, c6id.4xlarge, c6id.8xlarge, c6id.12xlarge, c6id.16xlarge, c6id.24xlarge, c6id.32xlarge, c6id.metal, m6id.large, m6id.xlarge, m6id.2xlarge, m6id.4xlarge, m6id.8xlarge, m6id.12xlarge, m6id.16xlarge, m6id.24xlarge, m6id.32xlarge, m6id.metal, r6id.large, r6id.xlarge, r6id.2xlarge, r6id.4xlarge, r6id.8xlarge, r6id.12xlarge, r6id.16xlarge, r6id.24xlarge, r6id.32xlarge, r6id.metal, r6a.large, r6a.xlarge, r6a.2xlarge, r6a.4xlarge, r6a.8xlarge, r6a.12xlarge, r6a.16xlarge, r6a.24xlarge, r6a.32xlarge, r6a.48xlarge, r6a.metal, p4de.24xlarge, u-3tb1.56xlarge, u-18tb1.112xlarge, u-24tb1.112xlarge, trn1.2xlarge, trn1.32xlarge, hpc6id.32xlarge, c6in.large, c6in.xlarge, c6in.2xlarge, c6in.4xlarge, c6in.8xlarge, c6in.12xlarge, c6in.16xlarge, c6in.24xlarge, c6in.32xlarge, m6in.large, m6in.xlarge, m6in.2xlarge, m6in.4xlarge, m6in.8xlarge, m6in.12xlarge, m6in.16xlarge, m6in.24xlarge, m6in.32xlarge, m6idn.large, m6idn.xlarge, m6idn.2xlarge, m6idn.4xlarge, m6idn.8xlarge, m6idn.12xlarge, m6idn.16xlarge, m6idn.24xlarge, m6idn.32xlarge, r6in.large, r6in.xlarge, r6in.2xlarge, r6in.4xlarge, r6in.8xlarge, r6in.12xlarge, r6in.16xlarge, r6in.24xlarge, r6in.32xlarge, r6idn.large, r6idn.xlarge, r6idn.2xlarge, r6idn.4xlarge, r6idn.8xlarge, r6idn.12xlarge, r6idn.16xlarge, r6idn.24xlarge, r6idn.32xlarge, c7g.metal, m7g.medium, m7g.large, m7g.xlarge, m7g.2xlarge, m7g.4xlarge, m7g.8xlarge, m7g.12xlarge, m7g.16xlarge, m7g.metal, r7g.medium, r7g.large, r7g.xlarge, r7g.2xlarge, r7g.4xlarge, r7g.8xlarge, r7g.12xlarge, r7g.16xlarge, r7g.metal, c6in.metal, m6in.metal, m6idn.metal, r6in.metal, r6idn.metal, inf2.xlarge, inf2.8xlarge, inf2.24xlarge, inf2.48xlarge, trn1n.32xlarge, i4g.large, i4g.xlarge, i4g.2xlarge, i4g.4xlarge, i4g.8xlarge, i4g.16xlarge, hpc7g.4xlarge, hpc7g.8xlarge, hpc7g.16xlarge, c7gn.medium, c7gn.large, c7gn.xlarge, c7gn.2xlarge, c7gn.4xlarge, c7gn.8xlarge, c7gn.12xlarge, c7gn.16xlarge, p5.48xlarge, m7i.large, m7i.xlarge, m7i.2xlarge, m7i.4xlarge, m7i.8xlarge, m7i.12xlarge, m7i.16xlarge, m7i.24xlarge, m7i.48xlarge, m7i-flex.large, m7i-flex.xlarge, m7i-flex.2xlarge, m7i-flex.4xlarge, m7i-flex.8xlarge, m7a.medium, m7a.large, m7a.xlarge, m7a.2xlarge, m7a.4xlarge, m7a.8xlarge, m7a.12xlarge, m7a.16xlarge, m7a.24xlarge, m7a.32xlarge, m7a.48xlarge, m7a.metal-48xl, hpc7a.12xlarge, hpc7a.24xlarge, hpc7a.48xlarge, hpc7a.96xlarge, c7gd.medium, c7gd.large, c7gd.xlarge, c7gd.2xlarge, c7gd.4xlarge, c7gd.8xlarge, c7gd.12xlarge, c7gd.16xlarge, m7gd.medium, m7gd.large, m7gd.xlarge, m7gd.2xlarge, m7gd.4xlarge, m7gd.8xlarge, m7gd.12xlarge, m7gd.16xlarge, r7gd.medium, r7gd.large, r7gd.xlarge, r7gd.2xlarge, r7gd.4xlarge, r7gd.8xlarge, r7gd.12xlarge, r7gd.16xlarge, r7a.medium, r7a.large, r7a.xlarge, r7a.2xlarge, r7a.4xlarge, r7a.8xlarge, r7a.12xlarge, r7a.16xlarge, r7a.24xlarge, r7a.32xlarge, r7a.48xlarge, c7i.large, c7i.xlarge, c7i.2xlarge, c7i.4xlarge, c7i.8xlarge, c7i.12xlarge, c7i.16xlarge, c7i.24xlarge, c7i.48xlarge, mac2-m2pro.metal, r7iz.large, r7iz.xlarge, r7iz.2xlarge, r7iz.4xlarge, r7iz.8xlarge, r7iz.12xlarge, r7iz.16xlarge, r7iz.32xlarge, c7a.medium, c7a.large, c7a.xlarge, c7a.2xlarge, c7a.4xlarge, c7a.8xlarge, c7a.12xlarge, c7a.16xlarge, c7a.24xlarge, c7a.32xlarge, c7a.48xlarge, c7a.metal-48xl, r7a.metal-48xl, r7i.large, r7i.xlarge, r7i.2xlarge, r7i.4xlarge, r7i.8xlarge, r7i.12xlarge, r7i.16xlarge, r7i.24xlarge, r7i.48xlarge, dl2q.24xlarge, mac2-m2.metal, i4i.12xlarge, i4i.24xlarge, c7i.metal-24xl, c7i.metal-48xl, m7i.metal-24xl, m7i.metal-48xl, r7i.metal-24xl, r7i.metal-48xl, r7iz.metal-16xl, r7iz.metal-32xl, c7gd.metal, m7gd.metal, r7gd.metal, g6.xlarge, g6.2xlarge, g6.4xlarge, g6.8xlarge, g6.12xlarge, g6.16xlarge, g6.24xlarge, g6.48xlarge, gr6.4xlarge, gr6.8xlarge, c7i-flex.large, c7i-flex.xlarge, c7i-flex.2xlarge, c7i-flex.4xlarge, c7i-flex.8xlarge, u7i-12tb.224xlarge, u7in-16tb.224xlarge, u7in-24tb.224xlarge, u7in-32tb.224xlarge, u7ib-12tb.224xlarge, c7gn.metal, r8g.medium, r8g.large, r8g.xlarge, r8g.2xlarge, r8g.4xlarge, r8g.8xlarge, r8g.12xlarge, r8g.16xlarge, r8g.24xlarge, r8g.48xlarge, r8g.metal-24xl, r8g.metal-48xl, mac2-m1ultra.metal, g6e.xlarge, g6e.2xlarge, g6e.4xlarge, g6e.8xlarge, g6e.12xlarge, g6e.16xlarge, g6e.24xlarge, g6e.48xlarge, c8g.medium, c8g.large, c8g.xlarge, c8g.2xlarge, c8g.4xlarge, c8g.8xlarge, c8g.12xlarge, c8g.16xlarge, c8g.24xlarge, c8g.48xlarge, c8g.metal-24xl, c8g.metal-48xl, m8g.medium, m8g.large, m8g.xlarge, m8g.2xlarge, m8g.4xlarge, m8g.8xlarge, m8g.12xlarge, m8g.16xlarge, m8g.24xlarge, m8g.48xlarge, m8g.metal-24xl, m8g.metal-48xl, x8g.medium, x8g.large, x8g.xlarge, x8g.2xlarge, x8g.4xlarge, x8g.8xlarge, x8g.12xlarge, x8g.16xlarge, x8g.24xlarge, x8g.48xlarge, x8g.metal-24xl, x8g.metal-48xl, i7ie.large, i7ie.xlarge, i7ie.2xlarge, i7ie.3xlarge, i7ie.6xlarge, i7ie.12xlarge, i7ie.18xlarge, i7ie.24xlarge, i7ie.48xlarge, i8g.large, i8g.xlarge, i8g.2xlarge, i8g.4xlarge, i8g.8xlarge, i8g.12xlarge, i8g.16xlarge, i8g.24xlarge, i8g.metal-24xl, u7i-6tb.112xlarge, u7i-8tb.112xlarge, u7inh-32tb.480xlarge, p5e.48xlarge, p5en.48xlarge, f2.12xlarge, f2.48xlarge, trn2.48xlarge
    #     ipv_6_address_count: 1,
    #     ipv_6_addresses: [
    #       {
    #         ipv_6_address: "String",
    #         is_primary_ipv_6: false,
    #       },
    #     ],
    #     kernel_id: "KernelId",
    #     key_name: "KeyPairName",
    #     max_count: 1, # required
    #     min_count: 1, # required
    #     monitoring: {
    #       enabled: false, # required
    #     },
    #     placement: {
    #       affinity: "String",
    #       group_name: "PlacementGroupName",
    #       partition_number: 1,
    #       host_id: "String",
    #       tenancy: "default", # accepts default, dedicated, host
    #       spread_domain: "String",
    #       host_resource_group_arn: "String",
    #       group_id: "PlacementGroupId",
    #       availability_zone: "String",
    #     },
    #     ramdisk_id: "RamdiskId",
    #     security_group_ids: ["SecurityGroupId"],
    #     security_groups: ["SecurityGroupName"],
    #     subnet_id: "SubnetId",
    #     user_data: "RunInstancesUserData",
    #     elastic_gpu_specification: [
    #       {
    #         type: "String", # required
    #       },
    #     ],
    #     elastic_inference_accelerators: [
    #       {
    #         type: "String", # required
    #         count: 1,
    #       },
    #     ],
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     launch_template: {
    #       launch_template_id: "LaunchTemplateId",
    #       launch_template_name: "String",
    #       version: "String",
    #     },
    #     instance_market_options: {
    #       market_type: "spot", # accepts spot, capacity-block
    #       spot_options: {
    #         max_price: "String",
    #         spot_instance_type: "one-time", # accepts one-time, persistent
    #         block_duration_minutes: 1,
    #         valid_until: Time.now,
    #         instance_interruption_behavior: "hibernate", # accepts hibernate, stop, terminate
    #       },
    #     },
    #     credit_specification: {
    #       cpu_credits: "String", # required
    #     },
    #     cpu_options: {
    #       core_count: 1,
    #       threads_per_core: 1,
    #       amd_sev_snp: "enabled", # accepts enabled, disabled
    #     },
    #     capacity_reservation_specification: {
    #       capacity_reservation_preference: "capacity-reservations-only", # accepts capacity-reservations-only, open, none
    #       capacity_reservation_target: {
    #         capacity_reservation_id: "CapacityReservationId",
    #         capacity_reservation_resource_group_arn: "String",
    #       },
    #     },
    #     hibernation_options: {
    #       configured: false,
    #     },
    #     license_specifications: [
    #       {
    #         license_configuration_arn: "String",
    #       },
    #     ],
    #     metadata_options: {
    #       http_tokens: "optional", # accepts optional, required
    #       http_put_response_hop_limit: 1,
    #       http_endpoint: "disabled", # accepts disabled, enabled
    #       http_protocol_ipv_6: "disabled", # accepts disabled, enabled
    #       instance_metadata_tags: "disabled", # accepts disabled, enabled
    #     },
    #     enclave_options: {
    #       enabled: false,
    #     },
    #     private_dns_name_options: {
    #       hostname_type: "ip-name", # accepts ip-name, resource-name
    #       enable_resource_name_dns_a_record: false,
    #       enable_resource_name_dns_aaaa_record: false,
    #     },
    #     maintenance_options: {
    #       auto_recovery: "disabled", # accepts disabled, default
    #     },
    #     disable_api_stop: false,
    #     enable_primary_ipv_6: false,
    #     network_performance_options: {
    #       bandwidth_weighting: "default", # accepts default, vpc-1, ebs-1
    #     },
    #     operator: {
    #       principal: "String",
    #     },
    #     dry_run: false,
    #     disable_api_termination: false,
    #     instance_initiated_shutdown_behavior: "stop", # accepts stop, terminate
    #     private_ip_address: "String",
    #     client_token: "String",
    #     additional_info: "String",
    #     network_interfaces: [
    #       {
    #         associate_public_ip_address: false,
    #         delete_on_termination: false,
    #         description: "String",
    #         device_index: 1,
    #         groups: ["SecurityGroupId"],
    #         ipv_6_address_count: 1,
    #         ipv_6_addresses: [
    #           {
    #             ipv_6_address: "String",
    #             is_primary_ipv_6: false,
    #           },
    #         ],
    #         network_interface_id: "NetworkInterfaceId",
    #         private_ip_address: "String",
    #         private_ip_addresses: [
    #           {
    #             primary: false,
    #             private_ip_address: "String",
    #           },
    #         ],
    #         secondary_private_ip_address_count: 1,
    #         subnet_id: "String",
    #         associate_carrier_ip_address: false,
    #         interface_type: "String",
    #         network_card_index: 1,
    #         ipv_4_prefixes: [
    #           {
    #             ipv_4_prefix: "String",
    #           },
    #         ],
    #         ipv_4_prefix_count: 1,
    #         ipv_6_prefixes: [
    #           {
    #             ipv_6_prefix: "String",
    #           },
    #         ],
    #         ipv_6_prefix_count: 1,
    #         primary_ipv_6: false,
    #         ena_srd_specification: {
    #           ena_srd_enabled: false,
    #           ena_srd_udp_specification: {
    #             ena_srd_udp_enabled: false,
    #           },
    #         },
    #         connection_tracking_specification: {
    #           tcp_established_timeout: 1,
    #           udp_stream_timeout: 1,
    #           udp_timeout: 1,
    #         },
    #       },
    #     ],
    #     iam_instance_profile: {
    #       arn: "String",
    #       name: "String",
    #     },
    #     ebs_optimized: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.reservation_id #=> String
    #   resp.owner_id #=> String
    #   resp.requester_id #=> String
    #   resp.groups #=> Array
    #   resp.groups[0].group_id #=> String
    #   resp.groups[0].group_name #=> String
    #   resp.instances #=> Array
    #   resp.instances[0].architecture #=> String, one of "i386", "x86_64", "arm64", "x86_64_mac", "arm64_mac"
    #   resp.instances[0].block_device_mappings #=> Array
    #   resp.instances[0].block_device_mappings[0].device_name #=> String
    #   resp.instances[0].block_device_mappings[0].ebs.attach_time #=> Time
    #   resp.instances[0].block_device_mappings[0].ebs.delete_on_termination #=> Boolean
    #   resp.instances[0].block_device_mappings[0].ebs.status #=> String, one of "attaching", "attached", "detaching", "detached"
    #   resp.instances[0].block_device_mappings[0].ebs.volume_id #=> String
    #   resp.instances[0].block_device_mappings[0].ebs.associated_resource #=> String
    #   resp.instances[0].block_device_mappings[0].ebs.volume_owner_id #=> String
    #   resp.instances[0].block_device_mappings[0].ebs.operator.managed #=> Boolean
    #   resp.instances[0].block_device_mappings[0].ebs.operator.principal #=> String
    #   resp.instances[0].client_token #=> String
    #   resp.instances[0].ebs_optimized #=> Boolean
    #   resp.instances[0].ena_support #=> Boolean
    #   resp.instances[0].hypervisor #=> String, one of "ovm", "xen"
    #   resp.instances[0].iam_instance_profile.arn #=> String
    #   resp.instances[0].iam_instance_profile.id #=> String
    #   resp.instances[0].instance_lifecycle #=> String, one of "spot", "scheduled", "capacity-block"
    #   resp.instances[0].elastic_gpu_associations #=> Array
    #   resp.instances[0].elastic_gpu_associations[0].elastic_gpu_id #=> String
    #   resp.instances[0].elastic_gpu_associations[0].elastic_gpu_association_id #=> String
    #   resp.instances[0].elastic_gpu_associations[0].elastic_gpu_association_state #=> String
    #   resp.instances[0].elastic_gpu_associations[0].elastic_gpu_association_time #=> String
    #   resp.instances[0].elastic_inference_accelerator_associations #=> Array
    #   resp.instances[0].elastic_inference_accelerator_associations[0].elastic_inference_accelerator_arn #=> String
    #   resp.instances[0].elastic_inference_accelerator_associations[0].elastic_inference_accelerator_association_id #=> String
    #   resp.instances[0].elastic_inference_accelerator_associations[0].elastic_inference_accelerator_association_state #=> String
    #   resp.instances[0].elastic_inference_accelerator_associations[0].elastic_inference_accelerator_association_time #=> Time
    #   resp.instances[0].network_interfaces #=> Array
    #   resp.instances[0].network_interfaces[0].association.carrier_ip #=> String
    #   resp.instances[0].network_interfaces[0].association.customer_owned_ip #=> String
    #   resp.instances[0].network_interfaces[0].association.ip_owner_id #=> String
    #   resp.instances[0].network_interfaces[0].association.public_dns_name #=> String
    #   resp.instances[0].network_interfaces[0].association.public_ip #=> String
    #   resp.instances[0].network_interfaces[0].attachment.attach_time #=> Time
    #   resp.instances[0].network_interfaces[0].attachment.attachment_id #=> String
    #   resp.instances[0].network_interfaces[0].attachment.delete_on_termination #=> Boolean
    #   resp.instances[0].network_interfaces[0].attachment.device_index #=> Integer
    #   resp.instances[0].network_interfaces[0].attachment.status #=> String, one of "attaching", "attached", "detaching", "detached"
    #   resp.instances[0].network_interfaces[0].attachment.network_card_index #=> Integer
    #   resp.instances[0].network_interfaces[0].attachment.ena_srd_specification.ena_srd_enabled #=> Boolean
    #   resp.instances[0].network_interfaces[0].attachment.ena_srd_specification.ena_srd_udp_specification.ena_srd_udp_enabled #=> Boolean
    #   resp.instances[0].network_interfaces[0].description #=> String
    #   resp.instances[0].network_interfaces[0].groups #=> Array
    #   resp.instances[0].network_interfaces[0].groups[0].group_id #=> String
    #   resp.instances[0].network_interfaces[0].groups[0].group_name #=> String
    #   resp.instances[0].network_interfaces[0].ipv_6_addresses #=> Array
    #   resp.instances[0].network_interfaces[0].ipv_6_addresses[0].ipv_6_address #=> String
    #   resp.instances[0].network_interfaces[0].ipv_6_addresses[0].is_primary_ipv_6 #=> Boolean
    #   resp.instances[0].network_interfaces[0].mac_address #=> String
    #   resp.instances[0].network_interfaces[0].network_interface_id #=> String
    #   resp.instances[0].network_interfaces[0].owner_id #=> String
    #   resp.instances[0].network_interfaces[0].private_dns_name #=> String
    #   resp.instances[0].network_interfaces[0].private_ip_address #=> String
    #   resp.instances[0].network_interfaces[0].private_ip_addresses #=> Array
    #   resp.instances[0].network_interfaces[0].private_ip_addresses[0].association.carrier_ip #=> String
    #   resp.instances[0].network_interfaces[0].private_ip_addresses[0].association.customer_owned_ip #=> String
    #   resp.instances[0].network_interfaces[0].private_ip_addresses[0].association.ip_owner_id #=> String
    #   resp.instances[0].network_interfaces[0].private_ip_addresses[0].association.public_dns_name #=> String
    #   resp.instances[0].network_interfaces[0].private_ip_addresses[0].association.public_ip #=> String
    #   resp.instances[0].network_interfaces[0].private_ip_addresses[0].primary #=> Boolean
    #   resp.instances[0].network_interfaces[0].private_ip_addresses[0].private_dns_name #=> String
    #   resp.instances[0].network_interfaces[0].private_ip_addresses[0].private_ip_address #=> String
    #   resp.instances[0].network_interfaces[0].source_dest_check #=> Boolean
    #   resp.instances[0].network_interfaces[0].status #=> String, one of "available", "associated", "attaching", "in-use", "detaching"
    #   resp.instances[0].network_interfaces[0].subnet_id #=> String
    #   resp.instances[0].network_interfaces[0].vpc_id #=> String
    #   resp.instances[0].network_interfaces[0].interface_type #=> String
    #   resp.instances[0].network_interfaces[0].ipv_4_prefixes #=> Array
    #   resp.instances[0].network_interfaces[0].ipv_4_prefixes[0].ipv_4_prefix #=> String
    #   resp.instances[0].network_interfaces[0].ipv_6_prefixes #=> Array
    #   resp.instances[0].network_interfaces[0].ipv_6_prefixes[0].ipv_6_prefix #=> String
    #   resp.instances[0].network_interfaces[0].connection_tracking_configuration.tcp_established_timeout #=> Integer
    #   resp.instances[0].network_interfaces[0].connection_tracking_configuration.udp_stream_timeout #=> Integer
    #   resp.instances[0].network_interfaces[0].connection_tracking_configuration.udp_timeout #=> Integer
    #   resp.instances[0].network_interfaces[0].operator.managed #=> Boolean
    #   resp.instances[0].network_interfaces[0].operator.principal #=> String
    #   resp.instances[0].outpost_arn #=> String
    #   resp.instances[0].root_device_name #=> String
    #   resp.instances[0].root_device_type #=> String, one of "ebs", "instance-store"
    #   resp.instances[0].security_groups #=> Array
    #   resp.instances[0].security_groups[0].group_id #=> String
    #   resp.instances[0].security_groups[0].group_name #=> String
    #   resp.instances[0].source_dest_check #=> Boolean
    #   resp.instances[0].spot_instance_request_id #=> String
    #   resp.instances[0].sriov_net_support #=> String
    #   resp.instances[0].state_reason.code #=> String
    #   resp.instances[0].state_reason.message #=> String
    #   resp.instances[0].tags #=> Array
    #   resp.instances[0].tags[0].key #=> String
    #   resp.instances[0].tags[0].value #=> String
    #   resp.instances[0].virtualization_type #=> String, one of "hvm", "paravirtual"
    #   resp.instances[0].cpu_options.core_count #=> Integer
    #   resp.instances[0].cpu_options.threads_per_core #=> Integer
    #   resp.instances[0].cpu_options.amd_sev_snp #=> String, one of "enabled", "disabled"
    #   resp.instances[0].capacity_reservation_id #=> String
    #   resp.instances[0].capacity_reservation_specification.capacity_reservation_preference #=> String, one of "capacity-reservations-only", "open", "none"
    #   resp.instances[0].capacity_reservation_specification.capacity_reservation_target.capacity_reservation_id #=> String
    #   resp.instances[0].capacity_reservation_specification.capacity_reservation_target.capacity_reservation_resource_group_arn #=> String
    #   resp.instances[0].hibernation_options.configured #=> Boolean
    #   resp.instances[0].licenses #=> Array
    #   resp.instances[0].licenses[0].license_configuration_arn #=> String
    #   resp.instances[0].metadata_options.state #=> String, one of "pending", "applied"
    #   resp.instances[0].metadata_options.http_tokens #=> String, one of "optional", "required"
    #   resp.instances[0].metadata_options.http_put_response_hop_limit #=> Integer
    #   resp.instances[0].metadata_options.http_endpoint #=> String, one of "disabled", "enabled"
    #   resp.instances[0].metadata_options.http_protocol_ipv_6 #=> String, one of "disabled", "enabled"
    #   resp.instances[0].metadata_options.instance_metadata_tags #=> String, one of "disabled", "enabled"
    #   resp.instances[0].enclave_options.enabled #=> Boolean
    #   resp.instances[0].boot_mode #=> String, one of "legacy-bios", "uefi", "uefi-preferred"
    #   resp.instances[0].platform_details #=> String
    #   resp.instances[0].usage_operation #=> String
    #   resp.instances[0].usage_operation_update_time #=> Time
    #   resp.instances[0].private_dns_name_options.hostname_type #=> String, one of "ip-name", "resource-name"
    #   resp.instances[0].private_dns_name_options.enable_resource_name_dns_a_record #=> Boolean
    #   resp.instances[0].private_dns_name_options.enable_resource_name_dns_aaaa_record #=> Boolean
    #   resp.instances[0].ipv_6_address #=> String
    #   resp.instances[0].tpm_support #=> String
    #   resp.instances[0].maintenance_options.auto_recovery #=> String, one of "disabled", "default"
    #   resp.instances[0].current_instance_boot_mode #=> String, one of "legacy-bios", "uefi"
    #   resp.instances[0].network_performance_options.bandwidth_weighting #=> String, one of "default", "vpc-1", "ebs-1"
    #   resp.instances[0].operator.managed #=> Boolean
    #   resp.instances[0].operator.principal #=> String
    #   resp.instances[0].instance_id #=> String
    #   resp.instances[0].image_id #=> String
    #   resp.instances[0].state.code #=> Integer
    #   resp.instances[0].state.name #=> String, one of "pending", "running", "shutting-down", "terminated", "stopping", "stopped"
    #   resp.instances[0].private_dns_name #=> String
    #   resp.instances[0].public_dns_name #=> String
    #   resp.instances[0].state_transition_reason #=> String
    #   resp.instances[0].key_name #=> String
    #   resp.instances[0].ami_launch_index #=> Integer
    #   resp.instances[0].product_codes #=> Array
    #   resp.instances[0].product_codes[0].product_code_id #=> String
    #   resp.instances[0].product_codes[0].product_code_type #=> String, one of "devpay", "marketplace"
    #   resp.instances[0].instance_type #=> String, one of "a1.medium", "a1.large", "a1.xlarge", "a1.2xlarge", "a1.4xlarge", "a1.metal", "c1.medium", "c1.xlarge", "c3.large", "c3.xlarge", "c3.2xlarge", "c3.4xlarge", "c3.8xlarge", "c4.large", "c4.xlarge", "c4.2xlarge", "c4.4xlarge", "c4.8xlarge", "c5.large", "c5.xlarge", "c5.2xlarge", "c5.4xlarge", "c5.9xlarge", "c5.12xlarge", "c5.18xlarge", "c5.24xlarge", "c5.metal", "c5a.large", "c5a.xlarge", "c5a.2xlarge", "c5a.4xlarge", "c5a.8xlarge", "c5a.12xlarge", "c5a.16xlarge", "c5a.24xlarge", "c5ad.large", "c5ad.xlarge", "c5ad.2xlarge", "c5ad.4xlarge", "c5ad.8xlarge", "c5ad.12xlarge", "c5ad.16xlarge", "c5ad.24xlarge", "c5d.large", "c5d.xlarge", "c5d.2xlarge", "c5d.4xlarge", "c5d.9xlarge", "c5d.12xlarge", "c5d.18xlarge", "c5d.24xlarge", "c5d.metal", "c5n.large", "c5n.xlarge", "c5n.2xlarge", "c5n.4xlarge", "c5n.9xlarge", "c5n.18xlarge", "c5n.metal", "c6g.medium", "c6g.large", "c6g.xlarge", "c6g.2xlarge", "c6g.4xlarge", "c6g.8xlarge", "c6g.12xlarge", "c6g.16xlarge", "c6g.metal", "c6gd.medium", "c6gd.large", "c6gd.xlarge", "c6gd.2xlarge", "c6gd.4xlarge", "c6gd.8xlarge", "c6gd.12xlarge", "c6gd.16xlarge", "c6gd.metal", "c6gn.medium", "c6gn.large", "c6gn.xlarge", "c6gn.2xlarge", "c6gn.4xlarge", "c6gn.8xlarge", "c6gn.12xlarge", "c6gn.16xlarge", "c6i.large", "c6i.xlarge", "c6i.2xlarge", "c6i.4xlarge", "c6i.8xlarge", "c6i.12xlarge", "c6i.16xlarge", "c6i.24xlarge", "c6i.32xlarge", "c6i.metal", "cc1.4xlarge", "cc2.8xlarge", "cg1.4xlarge", "cr1.8xlarge", "d2.xlarge", "d2.2xlarge", "d2.4xlarge", "d2.8xlarge", "d3.xlarge", "d3.2xlarge", "d3.4xlarge", "d3.8xlarge", "d3en.xlarge", "d3en.2xlarge", "d3en.4xlarge", "d3en.6xlarge", "d3en.8xlarge", "d3en.12xlarge", "dl1.24xlarge", "f1.2xlarge", "f1.4xlarge", "f1.16xlarge", "g2.2xlarge", "g2.8xlarge", "g3.4xlarge", "g3.8xlarge", "g3.16xlarge", "g3s.xlarge", "g4ad.xlarge", "g4ad.2xlarge", "g4ad.4xlarge", "g4ad.8xlarge", "g4ad.16xlarge", "g4dn.xlarge", "g4dn.2xlarge", "g4dn.4xlarge", "g4dn.8xlarge", "g4dn.12xlarge", "g4dn.16xlarge", "g4dn.metal", "g5.xlarge", "g5.2xlarge", "g5.4xlarge", "g5.8xlarge", "g5.12xlarge", "g5.16xlarge", "g5.24xlarge", "g5.48xlarge", "g5g.xlarge", "g5g.2xlarge", "g5g.4xlarge", "g5g.8xlarge", "g5g.16xlarge", "g5g.metal", "hi1.4xlarge", "hpc6a.48xlarge", "hs1.8xlarge", "h1.2xlarge", "h1.4xlarge", "h1.8xlarge", "h1.16xlarge", "i2.xlarge", "i2.2xlarge", "i2.4xlarge", "i2.8xlarge", "i3.large", "i3.xlarge", "i3.2xlarge", "i3.4xlarge", "i3.8xlarge", "i3.16xlarge", "i3.metal", "i3en.large", "i3en.xlarge", "i3en.2xlarge", "i3en.3xlarge", "i3en.6xlarge", "i3en.12xlarge", "i3en.24xlarge", "i3en.metal", "im4gn.large", "im4gn.xlarge", "im4gn.2xlarge", "im4gn.4xlarge", "im4gn.8xlarge", "im4gn.16xlarge", "inf1.xlarge", "inf1.2xlarge", "inf1.6xlarge", "inf1.24xlarge", "is4gen.medium", "is4gen.large", "is4gen.xlarge", "is4gen.2xlarge", "is4gen.4xlarge", "is4gen.8xlarge", "m1.small", "m1.medium", "m1.large", "m1.xlarge", "m2.xlarge", "m2.2xlarge", "m2.4xlarge", "m3.medium", "m3.large", "m3.xlarge", "m3.2xlarge", "m4.large", "m4.xlarge", "m4.2xlarge", "m4.4xlarge", "m4.10xlarge", "m4.16xlarge", "m5.large", "m5.xlarge", "m5.2xlarge", "m5.4xlarge", "m5.8xlarge", "m5.12xlarge", "m5.16xlarge", "m5.24xlarge", "m5.metal", "m5a.large", "m5a.xlarge", "m5a.2xlarge", "m5a.4xlarge", "m5a.8xlarge", "m5a.12xlarge", "m5a.16xlarge", "m5a.24xlarge", "m5ad.large", "m5ad.xlarge", "m5ad.2xlarge", "m5ad.4xlarge", "m5ad.8xlarge", "m5ad.12xlarge", "m5ad.16xlarge", "m5ad.24xlarge", "m5d.large", "m5d.xlarge", "m5d.2xlarge", "m5d.4xlarge", "m5d.8xlarge", "m5d.12xlarge", "m5d.16xlarge", "m5d.24xlarge", "m5d.metal", "m5dn.large", "m5dn.xlarge", "m5dn.2xlarge", "m5dn.4xlarge", "m5dn.8xlarge", "m5dn.12xlarge", "m5dn.16xlarge", "m5dn.24xlarge", "m5dn.metal", "m5n.large", "m5n.xlarge", "m5n.2xlarge", "m5n.4xlarge", "m5n.8xlarge", "m5n.12xlarge", "m5n.16xlarge", "m5n.24xlarge", "m5n.metal", "m5zn.large", "m5zn.xlarge", "m5zn.2xlarge", "m5zn.3xlarge", "m5zn.6xlarge", "m5zn.12xlarge", "m5zn.metal", "m6a.large", "m6a.xlarge", "m6a.2xlarge", "m6a.4xlarge", "m6a.8xlarge", "m6a.12xlarge", "m6a.16xlarge", "m6a.24xlarge", "m6a.32xlarge", "m6a.48xlarge", "m6g.metal", "m6g.medium", "m6g.large", "m6g.xlarge", "m6g.2xlarge", "m6g.4xlarge", "m6g.8xlarge", "m6g.12xlarge", "m6g.16xlarge", "m6gd.metal", "m6gd.medium", "m6gd.large", "m6gd.xlarge", "m6gd.2xlarge", "m6gd.4xlarge", "m6gd.8xlarge", "m6gd.12xlarge", "m6gd.16xlarge", "m6i.large", "m6i.xlarge", "m6i.2xlarge", "m6i.4xlarge", "m6i.8xlarge", "m6i.12xlarge", "m6i.16xlarge", "m6i.24xlarge", "m6i.32xlarge", "m6i.metal", "mac1.metal", "p2.xlarge", "p2.8xlarge", "p2.16xlarge", "p3.2xlarge", "p3.8xlarge", "p3.16xlarge", "p3dn.24xlarge", "p4d.24xlarge", "r3.large", "r3.xlarge", "r3.2xlarge", "r3.4xlarge", "r3.8xlarge", "r4.large", "r4.xlarge", "r4.2xlarge", "r4.4xlarge", "r4.8xlarge", "r4.16xlarge", "r5.large", "r5.xlarge", "r5.2xlarge", "r5.4xlarge", "r5.8xlarge", "r5.12xlarge", "r5.16xlarge", "r5.24xlarge", "r5.metal", "r5a.large", "r5a.xlarge", "r5a.2xlarge", "r5a.4xlarge", "r5a.8xlarge", "r5a.12xlarge", "r5a.16xlarge", "r5a.24xlarge", "r5ad.large", "r5ad.xlarge", "r5ad.2xlarge", "r5ad.4xlarge", "r5ad.8xlarge", "r5ad.12xlarge", "r5ad.16xlarge", "r5ad.24xlarge", "r5b.large", "r5b.xlarge", "r5b.2xlarge", "r5b.4xlarge", "r5b.8xlarge", "r5b.12xlarge", "r5b.16xlarge", "r5b.24xlarge", "r5b.metal", "r5d.large", "r5d.xlarge", "r5d.2xlarge", "r5d.4xlarge", "r5d.8xlarge", "r5d.12xlarge", "r5d.16xlarge", "r5d.24xlarge", "r5d.metal", "r5dn.large", "r5dn.xlarge", "r5dn.2xlarge", "r5dn.4xlarge", "r5dn.8xlarge", "r5dn.12xlarge", "r5dn.16xlarge", "r5dn.24xlarge", "r5dn.metal", "r5n.large", "r5n.xlarge", "r5n.2xlarge", "r5n.4xlarge", "r5n.8xlarge", "r5n.12xlarge", "r5n.16xlarge", "r5n.24xlarge", "r5n.metal", "r6g.medium", "r6g.large", "r6g.xlarge", "r6g.2xlarge", "r6g.4xlarge", "r6g.8xlarge", "r6g.12xlarge", "r6g.16xlarge", "r6g.metal", "r6gd.medium", "r6gd.large", "r6gd.xlarge", "r6gd.2xlarge", "r6gd.4xlarge", "r6gd.8xlarge", "r6gd.12xlarge", "r6gd.16xlarge", "r6gd.metal", "r6i.large", "r6i.xlarge", "r6i.2xlarge", "r6i.4xlarge", "r6i.8xlarge", "r6i.12xlarge", "r6i.16xlarge", "r6i.24xlarge", "r6i.32xlarge", "r6i.metal", "t1.micro", "t2.nano", "t2.micro", "t2.small", "t2.medium", "t2.large", "t2.xlarge", "t2.2xlarge", "t3.nano", "t3.micro", "t3.small", "t3.medium", "t3.large", "t3.xlarge", "t3.2xlarge", "t3a.nano", "t3a.micro", "t3a.small", "t3a.medium", "t3a.large", "t3a.xlarge", "t3a.2xlarge", "t4g.nano", "t4g.micro", "t4g.small", "t4g.medium", "t4g.large", "t4g.xlarge", "t4g.2xlarge", "u-6tb1.56xlarge", "u-6tb1.112xlarge", "u-9tb1.112xlarge", "u-12tb1.112xlarge", "u-6tb1.metal", "u-9tb1.metal", "u-12tb1.metal", "u-18tb1.metal", "u-24tb1.metal", "vt1.3xlarge", "vt1.6xlarge", "vt1.24xlarge", "x1.16xlarge", "x1.32xlarge", "x1e.xlarge", "x1e.2xlarge", "x1e.4xlarge", "x1e.8xlarge", "x1e.16xlarge", "x1e.32xlarge", "x2iezn.2xlarge", "x2iezn.4xlarge", "x2iezn.6xlarge", "x2iezn.8xlarge", "x2iezn.12xlarge", "x2iezn.metal", "x2gd.medium", "x2gd.large", "x2gd.xlarge", "x2gd.2xlarge", "x2gd.4xlarge", "x2gd.8xlarge", "x2gd.12xlarge", "x2gd.16xlarge", "x2gd.metal", "z1d.large", "z1d.xlarge", "z1d.2xlarge", "z1d.3xlarge", "z1d.6xlarge", "z1d.12xlarge", "z1d.metal", "x2idn.16xlarge", "x2idn.24xlarge", "x2idn.32xlarge", "x2iedn.xlarge", "x2iedn.2xlarge", "x2iedn.4xlarge", "x2iedn.8xlarge", "x2iedn.16xlarge", "x2iedn.24xlarge", "x2iedn.32xlarge", "c6a.large", "c6a.xlarge", "c6a.2xlarge", "c6a.4xlarge", "c6a.8xlarge", "c6a.12xlarge", "c6a.16xlarge", "c6a.24xlarge", "c6a.32xlarge", "c6a.48xlarge", "c6a.metal", "m6a.metal", "i4i.large", "i4i.xlarge", "i4i.2xlarge", "i4i.4xlarge", "i4i.8xlarge", "i4i.16xlarge", "i4i.32xlarge", "i4i.metal", "x2idn.metal", "x2iedn.metal", "c7g.medium", "c7g.large", "c7g.xlarge", "c7g.2xlarge", "c7g.4xlarge", "c7g.8xlarge", "c7g.12xlarge", "c7g.16xlarge", "mac2.metal", "c6id.large", "c6id.xlarge", "c6id.2xlarge", "c6id.4xlarge", "c6id.8xlarge", "c6id.12xlarge", "c6id.16xlarge", "c6id.24xlarge", "c6id.32xlarge", "c6id.metal", "m6id.large", "m6id.xlarge", "m6id.2xlarge", "m6id.4xlarge", "m6id.8xlarge", "m6id.12xlarge", "m6id.16xlarge", "m6id.24xlarge", "m6id.32xlarge", "m6id.metal", "r6id.large", "r6id.xlarge", "r6id.2xlarge", "r6id.4xlarge", "r6id.8xlarge", "r6id.12xlarge", "r6id.16xlarge", "r6id.24xlarge", "r6id.32xlarge", "r6id.metal", "r6a.large", "r6a.xlarge", "r6a.2xlarge", "r6a.4xlarge", "r6a.8xlarge", "r6a.12xlarge", "r6a.16xlarge", "r6a.24xlarge", "r6a.32xlarge", "r6a.48xlarge", "r6a.metal", "p4de.24xlarge", "u-3tb1.56xlarge", "u-18tb1.112xlarge", "u-24tb1.112xlarge", "trn1.2xlarge", "trn1.32xlarge", "hpc6id.32xlarge", "c6in.large", "c6in.xlarge", "c6in.2xlarge", "c6in.4xlarge", "c6in.8xlarge", "c6in.12xlarge", "c6in.16xlarge", "c6in.24xlarge", "c6in.32xlarge", "m6in.large", "m6in.xlarge", "m6in.2xlarge", "m6in.4xlarge", "m6in.8xlarge", "m6in.12xlarge", "m6in.16xlarge", "m6in.24xlarge", "m6in.32xlarge", "m6idn.large", "m6idn.xlarge", "m6idn.2xlarge", "m6idn.4xlarge", "m6idn.8xlarge", "m6idn.12xlarge", "m6idn.16xlarge", "m6idn.24xlarge", "m6idn.32xlarge", "r6in.large", "r6in.xlarge", "r6in.2xlarge", "r6in.4xlarge", "r6in.8xlarge", "r6in.12xlarge", "r6in.16xlarge", "r6in.24xlarge", "r6in.32xlarge", "r6idn.large", "r6idn.xlarge", "r6idn.2xlarge", "r6idn.4xlarge", "r6idn.8xlarge", "r6idn.12xlarge", "r6idn.16xlarge", "r6idn.24xlarge", "r6idn.32xlarge", "c7g.metal", "m7g.medium", "m7g.large", "m7g.xlarge", "m7g.2xlarge", "m7g.4xlarge", "m7g.8xlarge", "m7g.12xlarge", "m7g.16xlarge", "m7g.metal", "r7g.medium", "r7g.large", "r7g.xlarge", "r7g.2xlarge", "r7g.4xlarge", "r7g.8xlarge", "r7g.12xlarge", "r7g.16xlarge", "r7g.metal", "c6in.metal", "m6in.metal", "m6idn.metal", "r6in.metal", "r6idn.metal", "inf2.xlarge", "inf2.8xlarge", "inf2.24xlarge", "inf2.48xlarge", "trn1n.32xlarge", "i4g.large", "i4g.xlarge", "i4g.2xlarge", "i4g.4xlarge", "i4g.8xlarge", "i4g.16xlarge", "hpc7g.4xlarge", "hpc7g.8xlarge", "hpc7g.16xlarge", "c7gn.medium", "c7gn.large", "c7gn.xlarge", "c7gn.2xlarge", "c7gn.4xlarge", "c7gn.8xlarge", "c7gn.12xlarge", "c7gn.16xlarge", "p5.48xlarge", "m7i.large", "m7i.xlarge", "m7i.2xlarge", "m7i.4xlarge", "m7i.8xlarge", "m7i.12xlarge", "m7i.16xlarge", "m7i.24xlarge", "m7i.48xlarge", "m7i-flex.large", "m7i-flex.xlarge", "m7i-flex.2xlarge", "m7i-flex.4xlarge", "m7i-flex.8xlarge", "m7a.medium", "m7a.large", "m7a.xlarge", "m7a.2xlarge", "m7a.4xlarge", "m7a.8xlarge", "m7a.12xlarge", "m7a.16xlarge", "m7a.24xlarge", "m7a.32xlarge", "m7a.48xlarge", "m7a.metal-48xl", "hpc7a.12xlarge", "hpc7a.24xlarge", "hpc7a.48xlarge", "hpc7a.96xlarge", "c7gd.medium", "c7gd.large", "c7gd.xlarge", "c7gd.2xlarge", "c7gd.4xlarge", "c7gd.8xlarge", "c7gd.12xlarge", "c7gd.16xlarge", "m7gd.medium", "m7gd.large", "m7gd.xlarge", "m7gd.2xlarge", "m7gd.4xlarge", "m7gd.8xlarge", "m7gd.12xlarge", "m7gd.16xlarge", "r7gd.medium", "r7gd.large", "r7gd.xlarge", "r7gd.2xlarge", "r7gd.4xlarge", "r7gd.8xlarge", "r7gd.12xlarge", "r7gd.16xlarge", "r7a.medium", "r7a.large", "r7a.xlarge", "r7a.2xlarge", "r7a.4xlarge", "r7a.8xlarge", "r7a.12xlarge", "r7a.16xlarge", "r7a.24xlarge", "r7a.32xlarge", "r7a.48xlarge", "c7i.large", "c7i.xlarge", "c7i.2xlarge", "c7i.4xlarge", "c7i.8xlarge", "c7i.12xlarge", "c7i.16xlarge", "c7i.24xlarge", "c7i.48xlarge", "mac2-m2pro.metal", "r7iz.large", "r7iz.xlarge", "r7iz.2xlarge", "r7iz.4xlarge", "r7iz.8xlarge", "r7iz.12xlarge", "r7iz.16xlarge", "r7iz.32xlarge", "c7a.medium", "c7a.large", "c7a.xlarge", "c7a.2xlarge", "c7a.4xlarge", "c7a.8xlarge", "c7a.12xlarge", "c7a.16xlarge", "c7a.24xlarge", "c7a.32xlarge", "c7a.48xlarge", "c7a.metal-48xl", "r7a.metal-48xl", "r7i.large", "r7i.xlarge", "r7i.2xlarge", "r7i.4xlarge", "r7i.8xlarge", "r7i.12xlarge", "r7i.16xlarge", "r7i.24xlarge", "r7i.48xlarge", "dl2q.24xlarge", "mac2-m2.metal", "i4i.12xlarge", "i4i.24xlarge", "c7i.metal-24xl", "c7i.metal-48xl", "m7i.metal-24xl", "m7i.metal-48xl", "r7i.metal-24xl", "r7i.metal-48xl", "r7iz.metal-16xl", "r7iz.metal-32xl", "c7gd.metal", "m7gd.metal", "r7gd.metal", "g6.xlarge", "g6.2xlarge", "g6.4xlarge", "g6.8xlarge", "g6.12xlarge", "g6.16xlarge", "g6.24xlarge", "g6.48xlarge", "gr6.4xlarge", "gr6.8xlarge", "c7i-flex.large", "c7i-flex.xlarge", "c7i-flex.2xlarge", "c7i-flex.4xlarge", "c7i-flex.8xlarge", "u7i-12tb.224xlarge", "u7in-16tb.224xlarge", "u7in-24tb.224xlarge", "u7in-32tb.224xlarge", "u7ib-12tb.224xlarge", "c7gn.metal", "r8g.medium", "r8g.large", "r8g.xlarge", "r8g.2xlarge", "r8g.4xlarge", "r8g.8xlarge", "r8g.12xlarge", "r8g.16xlarge", "r8g.24xlarge", "r8g.48xlarge", "r8g.metal-24xl", "r8g.metal-48xl", "mac2-m1ultra.metal", "g6e.xlarge", "g6e.2xlarge", "g6e.4xlarge", "g6e.8xlarge", "g6e.12xlarge", "g6e.16xlarge", "g6e.24xlarge", "g6e.48xlarge", "c8g.medium", "c8g.large", "c8g.xlarge", "c8g.2xlarge", "c8g.4xlarge", "c8g.8xlarge", "c8g.12xlarge", "c8g.16xlarge", "c8g.24xlarge", "c8g.48xlarge", "c8g.metal-24xl", "c8g.metal-48xl", "m8g.medium", "m8g.large", "m8g.xlarge", "m8g.2xlarge", "m8g.4xlarge", "m8g.8xlarge", "m8g.12xlarge", "m8g.16xlarge", "m8g.24xlarge", "m8g.48xlarge", "m8g.metal-24xl", "m8g.metal-48xl", "x8g.medium", "x8g.large", "x8g.xlarge", "x8g.2xlarge", "x8g.4xlarge", "x8g.8xlarge", "x8g.12xlarge", "x8g.16xlarge", "x8g.24xlarge", "x8g.48xlarge", "x8g.metal-24xl", "x8g.metal-48xl", "i7ie.large", "i7ie.xlarge", "i7ie.2xlarge", "i7ie.3xlarge", "i7ie.6xlarge", "i7ie.12xlarge", "i7ie.18xlarge", "i7ie.24xlarge", "i7ie.48xlarge", "i8g.large", "i8g.xlarge", "i8g.2xlarge", "i8g.4xlarge", "i8g.8xlarge", "i8g.12xlarge", "i8g.16xlarge", "i8g.24xlarge", "i8g.metal-24xl", "u7i-6tb.112xlarge", "u7i-8tb.112xlarge", "u7inh-32tb.480xlarge", "p5e.48xlarge", "p5en.48xlarge", "f2.12xlarge", "f2.48xlarge", "trn2.48xlarge"
    #   resp.instances[0].launch_time #=> Time
    #   resp.instances[0].placement.affinity #=> String
    #   resp.instances[0].placement.group_name #=> String
    #   resp.instances[0].placement.partition_number #=> Integer
    #   resp.instances[0].placement.host_id #=> String
    #   resp.instances[0].placement.tenancy #=> String, one of "default", "dedicated", "host"
    #   resp.instances[0].placement.spread_domain #=> String
    #   resp.instances[0].placement.host_resource_group_arn #=> String
    #   resp.instances[0].placement.group_id #=> String
    #   resp.instances[0].placement.availability_zone #=> String
    #   resp.instances[0].kernel_id #=> String
    #   resp.instances[0].ramdisk_id #=> String
    #   resp.instances[0].platform #=> String, one of "Windows"
    #   resp.instances[0].monitoring.state #=> String, one of "disabled", "disabling", "enabled", "pending"
    #   resp.instances[0].subnet_id #=> String
    #   resp.instances[0].vpc_id #=> String
    #   resp.instances[0].private_ip_address #=> String
    #   resp.instances[0].public_ip_address #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RunInstances AWS API Documentation
    #
    # @overload run_instances(params = {})
    # @param [Hash] params ({})
    def run_instances(params = {}, options = {})
      req = build_request(:run_instances, params)
      req.send_request(options)
    end

    # Launches the specified Scheduled Instances.
    #
    # Before you can launch a Scheduled Instance, you must purchase it and
    # obtain an identifier using PurchaseScheduledInstances.
    #
    # You must launch a Scheduled Instance during its scheduled time period.
    # You can't stop or reboot a Scheduled Instance, but you can terminate
    # it as needed. If you terminate a Scheduled Instance before the current
    # scheduled time period ends, you can launch it again after a few
    # minutes.
    #
    # @option params [String] :client_token
    #   Unique, case-sensitive identifier that ensures the idempotency of the
    #   request. For more information, see [Ensuring Idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/APIReference/Run_Instance_Idempotency.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Integer] :instance_count
    #   The number of instances.
    #
    #   Default: 1
    #
    # @option params [required, Types::ScheduledInstancesLaunchSpecification] :launch_specification
    #   The launch specification. You must match the instance type,
    #   Availability Zone, network, and platform of the schedule that you
    #   purchased.
    #
    # @option params [required, String] :scheduled_instance_id
    #   The Scheduled Instance ID.
    #
    # @return [Types::RunScheduledInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::RunScheduledInstancesResult#instance_id_set #instance_id_set} => Array&lt;String&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.run_scheduled_instances({
    #     client_token: "String",
    #     dry_run: false,
    #     instance_count: 1,
    #     launch_specification: { # required
    #       block_device_mappings: [
    #         {
    #           device_name: "String",
    #           ebs: {
    #             delete_on_termination: false,
    #             encrypted: false,
    #             iops: 1,
    #             snapshot_id: "SnapshotId",
    #             volume_size: 1,
    #             volume_type: "String",
    #           },
    #           no_device: "String",
    #           virtual_name: "String",
    #         },
    #       ],
    #       ebs_optimized: false,
    #       iam_instance_profile: {
    #         arn: "String",
    #         name: "String",
    #       },
    #       image_id: "ImageId", # required
    #       instance_type: "String",
    #       kernel_id: "KernelId",
    #       key_name: "KeyPairName",
    #       monitoring: {
    #         enabled: false,
    #       },
    #       network_interfaces: [
    #         {
    #           associate_public_ip_address: false,
    #           delete_on_termination: false,
    #           description: "String",
    #           device_index: 1,
    #           groups: ["SecurityGroupId"],
    #           ipv_6_address_count: 1,
    #           ipv_6_addresses: [
    #             {
    #               ipv_6_address: "Ipv6Address",
    #             },
    #           ],
    #           network_interface_id: "NetworkInterfaceId",
    #           private_ip_address: "String",
    #           private_ip_address_configs: [
    #             {
    #               primary: false,
    #               private_ip_address: "String",
    #             },
    #           ],
    #           secondary_private_ip_address_count: 1,
    #           subnet_id: "SubnetId",
    #         },
    #       ],
    #       placement: {
    #         availability_zone: "String",
    #         group_name: "PlacementGroupName",
    #       },
    #       ramdisk_id: "RamdiskId",
    #       security_group_ids: ["SecurityGroupId"],
    #       subnet_id: "SubnetId",
    #       user_data: "String",
    #     },
    #     scheduled_instance_id: "ScheduledInstanceId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_id_set #=> Array
    #   resp.instance_id_set[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/RunScheduledInstances AWS API Documentation
    #
    # @overload run_scheduled_instances(params = {})
    # @param [Hash] params ({})
    def run_scheduled_instances(params = {}, options = {})
      req = build_request(:run_scheduled_instances, params)
      req.send_request(options)
    end

    # Searches for routes in the specified local gateway route table.
    #
    # @option params [required, String] :local_gateway_route_table_id
    #   The ID of the local gateway route table.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters.
    #
    #   * `prefix-list-id` - The ID of the prefix list.
    #
    #   * `route-search.exact-match` - The exact match of the specified
    #     filter.
    #
    #   * `route-search.longest-prefix-match` - The longest prefix that
    #     matches the route.
    #
    #   * `route-search.subnet-of-match` - The routes with a subnet that match
    #     the specified CIDR filter.
    #
    #   * `route-search.supernet-of-match` - The routes with a CIDR that
    #     encompass the CIDR filter. For example, if you have 10.0.1.0/29 and
    #     10.0.1.0/31 routes in your route table and you specify
    #     `supernet-of-match` as 10.0.1.0/30, then the result returns
    #     10.0.1.0/29.
    #
    #   * `state` - The state of the route.
    #
    #   * `type` - The route type.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::SearchLocalGatewayRoutesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::SearchLocalGatewayRoutesResult#routes #routes} => Array&lt;Types::LocalGatewayRoute&gt;
    #   * {Types::SearchLocalGatewayRoutesResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.search_local_gateway_routes({
    #     local_gateway_route_table_id: "LocalGatewayRoutetableId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.routes #=> Array
    #   resp.routes[0].destination_cidr_block #=> String
    #   resp.routes[0].local_gateway_virtual_interface_group_id #=> String
    #   resp.routes[0].type #=> String, one of "static", "propagated"
    #   resp.routes[0].state #=> String, one of "pending", "active", "blackhole", "deleting", "deleted"
    #   resp.routes[0].local_gateway_route_table_id #=> String
    #   resp.routes[0].local_gateway_route_table_arn #=> String
    #   resp.routes[0].owner_id #=> String
    #   resp.routes[0].subnet_id #=> String
    #   resp.routes[0].coip_pool_id #=> String
    #   resp.routes[0].network_interface_id #=> String
    #   resp.routes[0].destination_prefix_list_id #=> String
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/SearchLocalGatewayRoutes AWS API Documentation
    #
    # @overload search_local_gateway_routes(params = {})
    # @param [Hash] params ({})
    def search_local_gateway_routes(params = {}, options = {})
      req = build_request(:search_local_gateway_routes, params)
      req.send_request(options)
    end

    # Searches one or more transit gateway multicast groups and returns the
    # group membership information.
    #
    # @option params [required, String] :transit_gateway_multicast_domain_id
    #   The ID of the transit gateway multicast domain.
    #
    # @option params [Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `group-ip-address` - The IP address of the transit gateway multicast
    #     group.
    #
    #   * `is-group-member` - The resource is a group member. Valid values are
    #     `true` \| `false`.
    #
    #   * `is-group-source` - The resource is a group source. Valid values are
    #     `true` \| `false`.
    #
    #   * `member-type` - The member type. Valid values are `igmp` \|
    #     `static`.
    #
    #   * `resource-id` - The ID of the resource.
    #
    #   * `resource-type` - The type of resource. Valid values are `vpc` \|
    #     `vpn` \| `direct-connect-gateway` \| `tgw-peering`.
    #
    #   * `source-type` - The source type. Valid values are `igmp` \|
    #     `static`.
    #
    #   * `subnet-id` - The ID of the subnet.
    #
    #   * `transit-gateway-attachment-id` - The id of the transit gateway
    #     attachment.
    #
    # @option params [Integer] :max_results
    #   The maximum number of results to return with a single call. To
    #   retrieve the remaining results, make another call with the returned
    #   `nextToken` value.
    #
    # @option params [String] :next_token
    #   The token for the next page of results.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::SearchTransitGatewayMulticastGroupsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::SearchTransitGatewayMulticastGroupsResult#multicast_groups #multicast_groups} => Array&lt;Types::TransitGatewayMulticastGroup&gt;
    #   * {Types::SearchTransitGatewayMulticastGroupsResult#next_token #next_token} => String
    #
    # The returned {Seahorse::Client::Response response} is a pageable response and is Enumerable. For details on usage see {Aws::PageableResponse PageableResponse}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.search_transit_gateway_multicast_groups({
    #     transit_gateway_multicast_domain_id: "TransitGatewayMulticastDomainId", # required
    #     filters: [
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     next_token: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.multicast_groups #=> Array
    #   resp.multicast_groups[0].group_ip_address #=> String
    #   resp.multicast_groups[0].transit_gateway_attachment_id #=> String
    #   resp.multicast_groups[0].subnet_id #=> String
    #   resp.multicast_groups[0].resource_id #=> String
    #   resp.multicast_groups[0].resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.multicast_groups[0].resource_owner_id #=> String
    #   resp.multicast_groups[0].network_interface_id #=> String
    #   resp.multicast_groups[0].group_member #=> Boolean
    #   resp.multicast_groups[0].group_source #=> Boolean
    #   resp.multicast_groups[0].member_type #=> String, one of "static", "igmp"
    #   resp.multicast_groups[0].source_type #=> String, one of "static", "igmp"
    #   resp.next_token #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/SearchTransitGatewayMulticastGroups AWS API Documentation
    #
    # @overload search_transit_gateway_multicast_groups(params = {})
    # @param [Hash] params ({})
    def search_transit_gateway_multicast_groups(params = {}, options = {})
      req = build_request(:search_transit_gateway_multicast_groups, params)
      req.send_request(options)
    end

    # Searches for routes in the specified transit gateway route table.
    #
    # @option params [required, String] :transit_gateway_route_table_id
    #   The ID of the transit gateway route table.
    #
    # @option params [required, Array<Types::Filter>] :filters
    #   One or more filters. The possible values are:
    #
    #   * `attachment.transit-gateway-attachment-id`- The id of the transit
    #     gateway attachment.
    #
    #   * `attachment.resource-id` - The resource id of the transit gateway
    #     attachment.
    #
    #   * `attachment.resource-type` - The attachment resource type. Valid
    #     values are `vpc` \| `vpn` \| `direct-connect-gateway` \| `peering`
    #     \| `connect`.
    #
    #   * `prefix-list-id` - The ID of the prefix list.
    #
    #   * `route-search.exact-match` - The exact match of the specified
    #     filter.
    #
    #   * `route-search.longest-prefix-match` - The longest prefix that
    #     matches the route.
    #
    #   * `route-search.subnet-of-match` - The routes with a subnet that match
    #     the specified CIDR filter.
    #
    #   * `route-search.supernet-of-match` - The routes with a CIDR that
    #     encompass the CIDR filter. For example, if you have 10.0.1.0/29 and
    #     10.0.1.0/31 routes in your route table and you specify
    #     supernet-of-match as 10.0.1.0/30, then the result returns
    #     10.0.1.0/29.
    #
    #   * `state` - The state of the route (`active` \| `blackhole`).
    #
    #   * `type` - The type of route (`propagated` \| `static`).
    #
    # @option params [Integer] :max_results
    #   The maximum number of routes to return. If a value is not provided,
    #   the default is 1000.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::SearchTransitGatewayRoutesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::SearchTransitGatewayRoutesResult#routes #routes} => Array&lt;Types::TransitGatewayRoute&gt;
    #   * {Types::SearchTransitGatewayRoutesResult#additional_routes_available #additional_routes_available} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.search_transit_gateway_routes({
    #     transit_gateway_route_table_id: "TransitGatewayRouteTableId", # required
    #     filters: [ # required
    #       {
    #         name: "String",
    #         values: ["String"],
    #       },
    #     ],
    #     max_results: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.routes #=> Array
    #   resp.routes[0].destination_cidr_block #=> String
    #   resp.routes[0].prefix_list_id #=> String
    #   resp.routes[0].transit_gateway_route_table_announcement_id #=> String
    #   resp.routes[0].transit_gateway_attachments #=> Array
    #   resp.routes[0].transit_gateway_attachments[0].resource_id #=> String
    #   resp.routes[0].transit_gateway_attachments[0].transit_gateway_attachment_id #=> String
    #   resp.routes[0].transit_gateway_attachments[0].resource_type #=> String, one of "vpc", "vpn", "direct-connect-gateway", "connect", "peering", "tgw-peering"
    #   resp.routes[0].type #=> String, one of "static", "propagated"
    #   resp.routes[0].state #=> String, one of "pending", "active", "blackhole", "deleting", "deleted"
    #   resp.additional_routes_available #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/SearchTransitGatewayRoutes AWS API Documentation
    #
    # @overload search_transit_gateway_routes(params = {})
    # @param [Hash] params ({})
    def search_transit_gateway_routes(params = {}, options = {})
      req = build_request(:search_transit_gateway_routes, params)
      req.send_request(options)
    end

    # Sends a diagnostic interrupt to the specified Amazon EC2 instance to
    # trigger a *kernel panic* (on Linux instances), or a *blue
    # screen*/*stop error* (on Windows instances). For instances based on
    # Intel and AMD processors, the interrupt is received as a *non-maskable
    # interrupt* (NMI).
    #
    # In general, the operating system crashes and reboots when a kernel
    # panic or stop error is triggered. The operating system can also be
    # configured to perform diagnostic tasks, such as generating a memory
    # dump file, loading a secondary kernel, or obtaining a call trace.
    #
    # Before sending a diagnostic interrupt to your instance, ensure that
    # its operating system is configured to perform the required diagnostic
    # tasks.
    #
    # For more information about configuring your operating system to
    # generate a crash dump when a kernel panic or stop error occurs, see
    # [Send a diagnostic interrupt (for advanced users)][1] in the *Amazon
    # EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/diagnostic-interrupt.html
    #
    # @option params [required, String] :instance_id
    #   The ID of the instance.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.send_diagnostic_interrupt({
    #     instance_id: "InstanceId", # required
    #     dry_run: false,
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/SendDiagnosticInterrupt AWS API Documentation
    #
    # @overload send_diagnostic_interrupt(params = {})
    # @param [Hash] params ({})
    def send_diagnostic_interrupt(params = {}, options = {})
      req = build_request(:send_diagnostic_interrupt, params)
      req.send_request(options)
    end

    # Generates an account status report. The report is generated
    # asynchronously, and can take several hours to complete.
    #
    # The report provides the current status of all attributes supported by
    # declarative policies for the accounts within the specified scope. The
    # scope is determined by the specified `TargetId`, which can represent
    # an individual account, or all the accounts that fall under the
    # specified organizational unit (OU) or root (the entire Amazon Web
    # Services Organization).
    #
    # The report is saved to your specified S3 bucket, using the following
    # path structure (with the *italicized placeholders* representing your
    # specific values):
    #
    # `s3://amzn-s3-demo-bucket/your-optional-s3-prefix/ec2_targetId_reportId_yyyyMMddThhmmZ.csv`
    #
    # **Prerequisites for generating a report**
    #
    # * The `StartDeclarativePoliciesReport` API can only be called by the
    #   management account or delegated administrators for the organization.
    #
    # * An S3 bucket must be available before generating the report (you can
    #   create a new one or use an existing one), it must be in the same
    #   Region where the report generation request is made, and it must have
    #   an appropriate bucket policy. For a sample S3 policy, see *Sample
    #   Amazon S3 policy* under .
    #
    # * Trusted access must be enabled for the service for which the
    #   declarative policy will enforce a baseline configuration. If you use
    #   the Amazon Web Services Organizations console, this is done
    #   automatically when you enable declarative policies. The API uses the
    #   following service principal to identify the EC2 service:
    #   `ec2.amazonaws.com`. For more information on how to enable trusted
    #   access with the Amazon Web Services CLI and Amazon Web Services
    #   SDKs, see [Using Organizations with other Amazon Web Services
    #   services][1] in the *Amazon Web Services Organizations User Guide*.
    #
    # * Only one report per organization can be generated at a time.
    #   Attempting to generate a report while another is in progress will
    #   result in an error.
    #
    # For more information, including the required IAM permissions to run
    # this API, see [Generating the account status report for declarative
    # policies][2] in the *Amazon Web Services Organizations User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/organizations/latest/userguide/orgs_integrate_services.html
    # [2]: https://docs.aws.amazon.com/organizations/latest/userguide/orgs_manage_policies_declarative_status-report.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :s3_bucket
    #   The name of the S3 bucket where the report will be saved. The bucket
    #   must be in the same Region where the report generation request is
    #   made.
    #
    # @option params [String] :s3_prefix
    #   The prefix for your S3 object.
    #
    # @option params [required, String] :target_id
    #   The root ID, organizational unit ID, or account ID.
    #
    #   Format:
    #
    #   * For root: `r-ab12`
    #
    #   * For OU: `ou-ab12-cdef1234`
    #
    #   * For account: `123456789012`
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply.
    #
    # @return [Types::StartDeclarativePoliciesReportResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::StartDeclarativePoliciesReportResult#report_id #report_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.start_declarative_policies_report({
    #     dry_run: false,
    #     s3_bucket: "String", # required
    #     s3_prefix: "String",
    #     target_id: "String", # required
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.report_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/StartDeclarativePoliciesReport AWS API Documentation
    #
    # @overload start_declarative_policies_report(params = {})
    # @param [Hash] params ({})
    def start_declarative_policies_report(params = {}, options = {})
      req = build_request(:start_declarative_policies_report, params)
      req.send_request(options)
    end

    # Starts an Amazon EBS-backed instance that you've previously stopped.
    #
    # Instances that use Amazon EBS volumes as their root devices can be
    # quickly stopped and started. When an instance is stopped, the compute
    # resources are released and you are not billed for instance usage.
    # However, your root partition Amazon EBS volume remains and continues
    # to persist your data, and you are charged for Amazon EBS volume usage.
    # You can restart your instance at any time. Every time you start your
    # instance, Amazon EC2 charges a one-minute minimum for instance usage,
    # and thereafter charges per second for instance usage.
    #
    # Before stopping an instance, make sure it is in a state from which it
    # can be restarted. Stopping an instance does not preserve data stored
    # in RAM.
    #
    # Performing this operation on an instance that uses an instance store
    # as its root device returns an error.
    #
    # If you attempt to start a T3 instance with `host` tenancy and the
    # `unlimited` CPU credit option, the request fails. The `unlimited` CPU
    # credit option is not supported on Dedicated Hosts. Before you start
    # the instance, either change its CPU credit option to `standard`, or
    # change its tenancy to `default` or `dedicated`.
    #
    # For more information, see [Stop and start Amazon EC2 instances][1] in
    # the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Stop_Start.html
    #
    # @option params [required, Array<String>] :instance_ids
    #   The IDs of the instances.
    #
    # @option params [String] :additional_info
    #   Reserved.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::StartInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::StartInstancesResult#starting_instances #starting_instances} => Array&lt;Types::InstanceStateChange&gt;
    #
    #
    # @example Example: To start a stopped EC2 instance
    #
    #   # This example starts the specified EC2 instance.
    #
    #   resp = client.start_instances({
    #     instance_ids: [
    #       "i-1234567890abcdef0", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     starting_instances: [
    #       {
    #         current_state: {
    #           code: 0, 
    #           name: "pending", 
    #         }, 
    #         instance_id: "i-1234567890abcdef0", 
    #         previous_state: {
    #           code: 80, 
    #           name: "stopped", 
    #         }, 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.start_instances({
    #     instance_ids: ["InstanceId"], # required
    #     additional_info: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.starting_instances #=> Array
    #   resp.starting_instances[0].instance_id #=> String
    #   resp.starting_instances[0].current_state.code #=> Integer
    #   resp.starting_instances[0].current_state.name #=> String, one of "pending", "running", "shutting-down", "terminated", "stopping", "stopped"
    #   resp.starting_instances[0].previous_state.code #=> Integer
    #   resp.starting_instances[0].previous_state.name #=> String, one of "pending", "running", "shutting-down", "terminated", "stopping", "stopped"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/StartInstances AWS API Documentation
    #
    # @overload start_instances(params = {})
    # @param [Hash] params ({})
    def start_instances(params = {}, options = {})
      req = build_request(:start_instances, params)
      req.send_request(options)
    end

    # Starts analyzing the specified Network Access Scope.
    #
    # @option params [required, String] :network_insights_access_scope_id
    #   The ID of the Network Access Scope.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply.
    #
    # @option params [required, String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [How to ensure
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @return [Types::StartNetworkInsightsAccessScopeAnalysisResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::StartNetworkInsightsAccessScopeAnalysisResult#network_insights_access_scope_analysis #network_insights_access_scope_analysis} => Types::NetworkInsightsAccessScopeAnalysis
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.start_network_insights_access_scope_analysis({
    #     network_insights_access_scope_id: "NetworkInsightsAccessScopeId", # required
    #     dry_run: false,
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.network_insights_access_scope_analysis.network_insights_access_scope_analysis_id #=> String
    #   resp.network_insights_access_scope_analysis.network_insights_access_scope_analysis_arn #=> String
    #   resp.network_insights_access_scope_analysis.network_insights_access_scope_id #=> String
    #   resp.network_insights_access_scope_analysis.status #=> String, one of "running", "succeeded", "failed"
    #   resp.network_insights_access_scope_analysis.status_message #=> String
    #   resp.network_insights_access_scope_analysis.warning_message #=> String
    #   resp.network_insights_access_scope_analysis.start_date #=> Time
    #   resp.network_insights_access_scope_analysis.end_date #=> Time
    #   resp.network_insights_access_scope_analysis.findings_found #=> String, one of "true", "false", "unknown"
    #   resp.network_insights_access_scope_analysis.analyzed_eni_count #=> Integer
    #   resp.network_insights_access_scope_analysis.tags #=> Array
    #   resp.network_insights_access_scope_analysis.tags[0].key #=> String
    #   resp.network_insights_access_scope_analysis.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/StartNetworkInsightsAccessScopeAnalysis AWS API Documentation
    #
    # @overload start_network_insights_access_scope_analysis(params = {})
    # @param [Hash] params ({})
    def start_network_insights_access_scope_analysis(params = {}, options = {})
      req = build_request(:start_network_insights_access_scope_analysis, params)
      req.send_request(options)
    end

    # Starts analyzing the specified path. If the path is reachable, the
    # operation returns the shortest feasible path.
    #
    # @option params [required, String] :network_insights_path_id
    #   The ID of the path.
    #
    # @option params [Array<String>] :additional_accounts
    #   The member accounts that contain resources that the path can traverse.
    #
    # @option params [Array<String>] :filter_in_arns
    #   The Amazon Resource Names (ARN) of the resources that the path must
    #   traverse.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Array<Types::TagSpecification>] :tag_specifications
    #   The tags to apply.
    #
    # @option params [required, String] :client_token
    #   Unique, case-sensitive identifier that you provide to ensure the
    #   idempotency of the request. For more information, see [How to ensure
    #   idempotency][1].
    #
    #   **A suitable default value is auto-generated.** You should normally
    #   not need to pass this option.**
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/ec2/latest/devguide/ec2-api-idempotency.html
    #
    # @return [Types::StartNetworkInsightsAnalysisResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::StartNetworkInsightsAnalysisResult#network_insights_analysis #network_insights_analysis} => Types::NetworkInsightsAnalysis
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.start_network_insights_analysis({
    #     network_insights_path_id: "NetworkInsightsPathId", # required
    #     additional_accounts: ["String"],
    #     filter_in_arns: ["ResourceArn"],
    #     dry_run: false,
    #     tag_specifications: [
    #       {
    #         resource_type: "capacity-reservation", # accepts capacity-reservation, client-vpn-endpoint, customer-gateway, carrier-gateway, coip-pool, declarative-policies-report, dedicated-host, dhcp-options, egress-only-internet-gateway, elastic-ip, elastic-gpu, export-image-task, export-instance-task, fleet, fpga-image, host-reservation, image, import-image-task, import-snapshot-task, instance, instance-event-window, internet-gateway, ipam, ipam-pool, ipam-scope, ipv4pool-ec2, ipv6pool-ec2, key-pair, launch-template, local-gateway, local-gateway-route-table, local-gateway-virtual-interface, local-gateway-virtual-interface-group, local-gateway-route-table-vpc-association, local-gateway-route-table-virtual-interface-group-association, natgateway, network-acl, network-interface, network-insights-analysis, network-insights-path, network-insights-access-scope, network-insights-access-scope-analysis, placement-group, prefix-list, replace-root-volume-task, reserved-instances, route-table, security-group, security-group-rule, snapshot, spot-fleet-request, spot-instances-request, subnet, subnet-cidr-reservation, traffic-mirror-filter, traffic-mirror-session, traffic-mirror-target, transit-gateway, transit-gateway-attachment, transit-gateway-connect-peer, transit-gateway-multicast-domain, transit-gateway-policy-table, transit-gateway-route-table, transit-gateway-route-table-announcement, volume, vpc, vpc-endpoint, vpc-endpoint-connection, vpc-endpoint-service, vpc-endpoint-service-permission, vpc-peering-connection, vpn-connection, vpn-gateway, vpc-flow-log, capacity-reservation-fleet, traffic-mirror-filter-rule, vpc-endpoint-connection-device-type, verified-access-instance, verified-access-group, verified-access-endpoint, verified-access-policy, verified-access-trust-provider, vpn-connection-device-type, vpc-block-public-access-exclusion, ipam-resource-discovery, ipam-resource-discovery-association, instance-connect-endpoint, verified-access-endpoint-target, ipam-external-resource-verification-token
    #         tags: [
    #           {
    #             key: "String",
    #             value: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     client_token: "String", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.network_insights_analysis.network_insights_analysis_id #=> String
    #   resp.network_insights_analysis.network_insights_analysis_arn #=> String
    #   resp.network_insights_analysis.network_insights_path_id #=> String
    #   resp.network_insights_analysis.additional_accounts #=> Array
    #   resp.network_insights_analysis.additional_accounts[0] #=> String
    #   resp.network_insights_analysis.filter_in_arns #=> Array
    #   resp.network_insights_analysis.filter_in_arns[0] #=> String
    #   resp.network_insights_analysis.start_date #=> Time
    #   resp.network_insights_analysis.status #=> String, one of "running", "succeeded", "failed"
    #   resp.network_insights_analysis.status_message #=> String
    #   resp.network_insights_analysis.warning_message #=> String
    #   resp.network_insights_analysis.network_path_found #=> Boolean
    #   resp.network_insights_analysis.forward_path_components #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].sequence_number #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].acl_rule.cidr #=> String
    #   resp.network_insights_analysis.forward_path_components[0].acl_rule.egress #=> Boolean
    #   resp.network_insights_analysis.forward_path_components[0].acl_rule.port_range.from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].acl_rule.port_range.to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].acl_rule.protocol #=> String
    #   resp.network_insights_analysis.forward_path_components[0].acl_rule.rule_action #=> String
    #   resp.network_insights_analysis.forward_path_components[0].acl_rule.rule_number #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].attached_to.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].attached_to.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].attached_to.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].component.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].component.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].component.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].destination_vpc.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].destination_vpc.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].destination_vpc.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].outbound_header.destination_addresses #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].outbound_header.destination_addresses[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].outbound_header.destination_port_ranges #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].outbound_header.destination_port_ranges[0].from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].outbound_header.destination_port_ranges[0].to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].outbound_header.protocol #=> String
    #   resp.network_insights_analysis.forward_path_components[0].outbound_header.source_addresses #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].outbound_header.source_addresses[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].outbound_header.source_port_ranges #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].outbound_header.source_port_ranges[0].from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].outbound_header.source_port_ranges[0].to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].inbound_header.destination_addresses #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].inbound_header.destination_addresses[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].inbound_header.destination_port_ranges #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].inbound_header.destination_port_ranges[0].from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].inbound_header.destination_port_ranges[0].to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].inbound_header.protocol #=> String
    #   resp.network_insights_analysis.forward_path_components[0].inbound_header.source_addresses #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].inbound_header.source_addresses[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].inbound_header.source_port_ranges #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].inbound_header.source_port_ranges[0].from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].inbound_header.source_port_ranges[0].to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].route_table_route.destination_cidr #=> String
    #   resp.network_insights_analysis.forward_path_components[0].route_table_route.destination_prefix_list_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].route_table_route.egress_only_internet_gateway_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].route_table_route.gateway_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].route_table_route.instance_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].route_table_route.nat_gateway_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].route_table_route.network_interface_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].route_table_route.origin #=> String
    #   resp.network_insights_analysis.forward_path_components[0].route_table_route.transit_gateway_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].route_table_route.vpc_peering_connection_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].route_table_route.state #=> String
    #   resp.network_insights_analysis.forward_path_components[0].route_table_route.carrier_gateway_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].route_table_route.core_network_arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].route_table_route.local_gateway_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].security_group_rule.cidr #=> String
    #   resp.network_insights_analysis.forward_path_components[0].security_group_rule.direction #=> String
    #   resp.network_insights_analysis.forward_path_components[0].security_group_rule.security_group_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].security_group_rule.port_range.from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].security_group_rule.port_range.to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].security_group_rule.prefix_list_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].security_group_rule.protocol #=> String
    #   resp.network_insights_analysis.forward_path_components[0].source_vpc.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].source_vpc.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].source_vpc.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].subnet.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].subnet.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].subnet.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].vpc.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].vpc.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].vpc.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].additional_detail_type #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].component.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].component.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].component.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].vpc_endpoint_service.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].vpc_endpoint_service.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].vpc_endpoint_service.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].rule_options #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].rule_options[0].keyword #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].rule_options[0].settings #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].rule_options[0].settings[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].rule_group_type_pairs #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].rule_group_type_pairs[0].rule_group_arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].rule_group_type_pairs[0].rule_group_type #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].rule_group_rule_options_pairs #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_group_arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].keyword #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].settings #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].settings[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].service_name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].load_balancers #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].load_balancers[0].id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].load_balancers[0].arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].additional_details[0].load_balancers[0].name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].transit_gateway.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].transit_gateway.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].transit_gateway.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].transit_gateway_route_table_route.destination_cidr #=> String
    #   resp.network_insights_analysis.forward_path_components[0].transit_gateway_route_table_route.state #=> String
    #   resp.network_insights_analysis.forward_path_components[0].transit_gateway_route_table_route.route_origin #=> String
    #   resp.network_insights_analysis.forward_path_components[0].transit_gateway_route_table_route.prefix_list_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].transit_gateway_route_table_route.attachment_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].transit_gateway_route_table_route.resource_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].transit_gateway_route_table_route.resource_type #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].acl.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].acl.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].acl.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].acl_rule.cidr #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].acl_rule.egress #=> Boolean
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].acl_rule.port_range.from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].acl_rule.port_range.to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].acl_rule.protocol #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].acl_rule.rule_action #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].acl_rule.rule_number #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].address #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].addresses #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].addresses[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].attached_to.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].attached_to.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].attached_to.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].availability_zones #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].availability_zones[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].cidrs #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].cidrs[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].component.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].component.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].component.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].customer_gateway.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].customer_gateway.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].customer_gateway.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].destination.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].destination.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].destination.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].destination_vpc.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].destination_vpc.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].destination_vpc.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].direction #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].explanation_code #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].ingress_route_table.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].ingress_route_table.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].ingress_route_table.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].internet_gateway.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].internet_gateway.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].internet_gateway.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].classic_load_balancer_listener.load_balancer_port #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].classic_load_balancer_listener.instance_port #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_listener_port #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_target.address #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_target.availability_zone #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_target.instance.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_target.instance.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_target.instance.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_target.port #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_target_group.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_target_group.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_target_group.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_target_groups #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_target_groups[0].id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_target_groups[0].arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_target_groups[0].name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].load_balancer_target_port #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].elastic_load_balancer_listener.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].elastic_load_balancer_listener.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].elastic_load_balancer_listener.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].missing_component #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].nat_gateway.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].nat_gateway.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].nat_gateway.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].network_interface.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].network_interface.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].network_interface.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].packet_field #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpc_peering_connection.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpc_peering_connection.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpc_peering_connection.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].port #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].port_ranges #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].port_ranges[0].from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].port_ranges[0].to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].prefix_list.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].prefix_list.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].prefix_list.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].protocols #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].protocols[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table_route.destination_cidr #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table_route.destination_prefix_list_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table_route.egress_only_internet_gateway_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table_route.gateway_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table_route.instance_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table_route.nat_gateway_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table_route.network_interface_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table_route.origin #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table_route.transit_gateway_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table_route.vpc_peering_connection_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table_route.state #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table_route.carrier_gateway_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table_route.core_network_arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table_route.local_gateway_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].route_table.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].security_group.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].security_group.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].security_group.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].security_group_rule.cidr #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].security_group_rule.direction #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].security_group_rule.security_group_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].security_group_rule.port_range.from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].security_group_rule.port_range.to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].security_group_rule.prefix_list_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].security_group_rule.protocol #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].security_groups #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].security_groups[0].id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].security_groups[0].arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].security_groups[0].name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].source_vpc.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].source_vpc.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].source_vpc.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].state #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].subnet.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].subnet.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].subnet.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].subnet_route_table.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].subnet_route_table.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].subnet_route_table.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpc.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpc.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpc.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpc_endpoint.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpc_endpoint.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpc_endpoint.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpn_connection.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpn_connection.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpn_connection.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpn_gateway.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpn_gateway.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].vpn_gateway.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway_route_table.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway_route_table.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway_route_table.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway_route_table_route.destination_cidr #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway_route_table_route.state #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway_route_table_route.route_origin #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway_route_table_route.prefix_list_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway_route_table_route.attachment_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway_route_table_route.resource_id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway_route_table_route.resource_type #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway_attachment.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway_attachment.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].transit_gateway_attachment.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].component_account #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].component_region #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.rule_group_arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.sources #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.sources[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.destinations #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.destinations[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.source_ports #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.destination_ports #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.protocols #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.protocols[0] #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.rule_action #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateless_rule.priority #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateful_rule.rule_group_arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateful_rule.sources #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateful_rule.sources[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateful_rule.destinations #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateful_rule.destinations[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateful_rule.source_ports #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateful_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateful_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateful_rule.destination_ports #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateful_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateful_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateful_rule.protocol #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateful_rule.rule_action #=> String
    #   resp.network_insights_analysis.forward_path_components[0].explanations[0].firewall_stateful_rule.direction #=> String
    #   resp.network_insights_analysis.forward_path_components[0].elastic_load_balancer_listener.id #=> String
    #   resp.network_insights_analysis.forward_path_components[0].elastic_load_balancer_listener.arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].elastic_load_balancer_listener.name #=> String
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.rule_group_arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.sources #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.sources[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.destinations #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.destinations[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.source_ports #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.destination_ports #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.protocols #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.protocols[0] #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.rule_action #=> String
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateless_rule.priority #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateful_rule.rule_group_arn #=> String
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateful_rule.sources #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateful_rule.sources[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateful_rule.destinations #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateful_rule.destinations[0] #=> String
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateful_rule.source_ports #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateful_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateful_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateful_rule.destination_ports #=> Array
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateful_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateful_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateful_rule.protocol #=> String
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateful_rule.rule_action #=> String
    #   resp.network_insights_analysis.forward_path_components[0].firewall_stateful_rule.direction #=> String
    #   resp.network_insights_analysis.forward_path_components[0].service_name #=> String
    #   resp.network_insights_analysis.return_path_components #=> Array
    #   resp.network_insights_analysis.return_path_components[0].sequence_number #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].acl_rule.cidr #=> String
    #   resp.network_insights_analysis.return_path_components[0].acl_rule.egress #=> Boolean
    #   resp.network_insights_analysis.return_path_components[0].acl_rule.port_range.from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].acl_rule.port_range.to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].acl_rule.protocol #=> String
    #   resp.network_insights_analysis.return_path_components[0].acl_rule.rule_action #=> String
    #   resp.network_insights_analysis.return_path_components[0].acl_rule.rule_number #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].attached_to.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].attached_to.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].attached_to.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].component.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].component.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].component.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].destination_vpc.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].destination_vpc.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].destination_vpc.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].outbound_header.destination_addresses #=> Array
    #   resp.network_insights_analysis.return_path_components[0].outbound_header.destination_addresses[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].outbound_header.destination_port_ranges #=> Array
    #   resp.network_insights_analysis.return_path_components[0].outbound_header.destination_port_ranges[0].from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].outbound_header.destination_port_ranges[0].to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].outbound_header.protocol #=> String
    #   resp.network_insights_analysis.return_path_components[0].outbound_header.source_addresses #=> Array
    #   resp.network_insights_analysis.return_path_components[0].outbound_header.source_addresses[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].outbound_header.source_port_ranges #=> Array
    #   resp.network_insights_analysis.return_path_components[0].outbound_header.source_port_ranges[0].from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].outbound_header.source_port_ranges[0].to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].inbound_header.destination_addresses #=> Array
    #   resp.network_insights_analysis.return_path_components[0].inbound_header.destination_addresses[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].inbound_header.destination_port_ranges #=> Array
    #   resp.network_insights_analysis.return_path_components[0].inbound_header.destination_port_ranges[0].from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].inbound_header.destination_port_ranges[0].to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].inbound_header.protocol #=> String
    #   resp.network_insights_analysis.return_path_components[0].inbound_header.source_addresses #=> Array
    #   resp.network_insights_analysis.return_path_components[0].inbound_header.source_addresses[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].inbound_header.source_port_ranges #=> Array
    #   resp.network_insights_analysis.return_path_components[0].inbound_header.source_port_ranges[0].from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].inbound_header.source_port_ranges[0].to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].route_table_route.destination_cidr #=> String
    #   resp.network_insights_analysis.return_path_components[0].route_table_route.destination_prefix_list_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].route_table_route.egress_only_internet_gateway_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].route_table_route.gateway_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].route_table_route.instance_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].route_table_route.nat_gateway_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].route_table_route.network_interface_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].route_table_route.origin #=> String
    #   resp.network_insights_analysis.return_path_components[0].route_table_route.transit_gateway_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].route_table_route.vpc_peering_connection_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].route_table_route.state #=> String
    #   resp.network_insights_analysis.return_path_components[0].route_table_route.carrier_gateway_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].route_table_route.core_network_arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].route_table_route.local_gateway_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].security_group_rule.cidr #=> String
    #   resp.network_insights_analysis.return_path_components[0].security_group_rule.direction #=> String
    #   resp.network_insights_analysis.return_path_components[0].security_group_rule.security_group_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].security_group_rule.port_range.from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].security_group_rule.port_range.to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].security_group_rule.prefix_list_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].security_group_rule.protocol #=> String
    #   resp.network_insights_analysis.return_path_components[0].source_vpc.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].source_vpc.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].source_vpc.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].subnet.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].subnet.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].subnet.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].vpc.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].vpc.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].vpc.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details #=> Array
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].additional_detail_type #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].component.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].component.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].component.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].vpc_endpoint_service.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].vpc_endpoint_service.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].vpc_endpoint_service.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].rule_options #=> Array
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].rule_options[0].keyword #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].rule_options[0].settings #=> Array
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].rule_options[0].settings[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].rule_group_type_pairs #=> Array
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].rule_group_type_pairs[0].rule_group_arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].rule_group_type_pairs[0].rule_group_type #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].rule_group_rule_options_pairs #=> Array
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_group_arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options #=> Array
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].keyword #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].settings #=> Array
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].rule_group_rule_options_pairs[0].rule_options[0].settings[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].service_name #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].load_balancers #=> Array
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].load_balancers[0].id #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].load_balancers[0].arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].additional_details[0].load_balancers[0].name #=> String
    #   resp.network_insights_analysis.return_path_components[0].transit_gateway.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].transit_gateway.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].transit_gateway.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].transit_gateway_route_table_route.destination_cidr #=> String
    #   resp.network_insights_analysis.return_path_components[0].transit_gateway_route_table_route.state #=> String
    #   resp.network_insights_analysis.return_path_components[0].transit_gateway_route_table_route.route_origin #=> String
    #   resp.network_insights_analysis.return_path_components[0].transit_gateway_route_table_route.prefix_list_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].transit_gateway_route_table_route.attachment_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].transit_gateway_route_table_route.resource_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].transit_gateway_route_table_route.resource_type #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].acl.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].acl.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].acl.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].acl_rule.cidr #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].acl_rule.egress #=> Boolean
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].acl_rule.port_range.from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].acl_rule.port_range.to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].acl_rule.protocol #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].acl_rule.rule_action #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].acl_rule.rule_number #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].address #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].addresses #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].addresses[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].attached_to.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].attached_to.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].attached_to.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].availability_zones #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].availability_zones[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].cidrs #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].cidrs[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].component.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].component.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].component.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].customer_gateway.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].customer_gateway.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].customer_gateway.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].destination.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].destination.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].destination.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].destination_vpc.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].destination_vpc.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].destination_vpc.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].direction #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].explanation_code #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].ingress_route_table.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].ingress_route_table.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].ingress_route_table.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].internet_gateway.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].internet_gateway.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].internet_gateway.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].classic_load_balancer_listener.load_balancer_port #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].classic_load_balancer_listener.instance_port #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_listener_port #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_target.address #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_target.availability_zone #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_target.instance.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_target.instance.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_target.instance.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_target.port #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_target_group.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_target_group.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_target_group.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_target_groups #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_target_groups[0].id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_target_groups[0].arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_target_groups[0].name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].load_balancer_target_port #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].elastic_load_balancer_listener.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].elastic_load_balancer_listener.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].elastic_load_balancer_listener.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].missing_component #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].nat_gateway.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].nat_gateway.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].nat_gateway.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].network_interface.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].network_interface.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].network_interface.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].packet_field #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpc_peering_connection.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpc_peering_connection.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpc_peering_connection.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].port #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].port_ranges #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].port_ranges[0].from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].port_ranges[0].to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].prefix_list.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].prefix_list.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].prefix_list.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].protocols #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].protocols[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table_route.destination_cidr #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table_route.destination_prefix_list_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table_route.egress_only_internet_gateway_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table_route.gateway_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table_route.instance_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table_route.nat_gateway_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table_route.network_interface_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table_route.origin #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table_route.transit_gateway_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table_route.vpc_peering_connection_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table_route.state #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table_route.carrier_gateway_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table_route.core_network_arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table_route.local_gateway_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].route_table.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].security_group.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].security_group.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].security_group.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].security_group_rule.cidr #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].security_group_rule.direction #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].security_group_rule.security_group_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].security_group_rule.port_range.from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].security_group_rule.port_range.to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].security_group_rule.prefix_list_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].security_group_rule.protocol #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].security_groups #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].security_groups[0].id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].security_groups[0].arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].security_groups[0].name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].source_vpc.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].source_vpc.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].source_vpc.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].state #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].subnet.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].subnet.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].subnet.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].subnet_route_table.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].subnet_route_table.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].subnet_route_table.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpc.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpc.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpc.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpc_endpoint.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpc_endpoint.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpc_endpoint.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpn_connection.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpn_connection.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpn_connection.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpn_gateway.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpn_gateway.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].vpn_gateway.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway_route_table.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway_route_table.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway_route_table.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway_route_table_route.destination_cidr #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway_route_table_route.state #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway_route_table_route.route_origin #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway_route_table_route.prefix_list_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway_route_table_route.attachment_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway_route_table_route.resource_id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway_route_table_route.resource_type #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway_attachment.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway_attachment.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].transit_gateway_attachment.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].component_account #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].component_region #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.rule_group_arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.sources #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.sources[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.destinations #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.destinations[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.source_ports #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.destination_ports #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.protocols #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.protocols[0] #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.rule_action #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateless_rule.priority #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateful_rule.rule_group_arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateful_rule.sources #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateful_rule.sources[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateful_rule.destinations #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateful_rule.destinations[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateful_rule.source_ports #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateful_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateful_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateful_rule.destination_ports #=> Array
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateful_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateful_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateful_rule.protocol #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateful_rule.rule_action #=> String
    #   resp.network_insights_analysis.return_path_components[0].explanations[0].firewall_stateful_rule.direction #=> String
    #   resp.network_insights_analysis.return_path_components[0].elastic_load_balancer_listener.id #=> String
    #   resp.network_insights_analysis.return_path_components[0].elastic_load_balancer_listener.arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].elastic_load_balancer_listener.name #=> String
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.rule_group_arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.sources #=> Array
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.sources[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.destinations #=> Array
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.destinations[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.source_ports #=> Array
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.destination_ports #=> Array
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.protocols #=> Array
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.protocols[0] #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.rule_action #=> String
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateless_rule.priority #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateful_rule.rule_group_arn #=> String
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateful_rule.sources #=> Array
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateful_rule.sources[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateful_rule.destinations #=> Array
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateful_rule.destinations[0] #=> String
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateful_rule.source_ports #=> Array
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateful_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateful_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateful_rule.destination_ports #=> Array
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateful_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateful_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateful_rule.protocol #=> String
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateful_rule.rule_action #=> String
    #   resp.network_insights_analysis.return_path_components[0].firewall_stateful_rule.direction #=> String
    #   resp.network_insights_analysis.return_path_components[0].service_name #=> String
    #   resp.network_insights_analysis.explanations #=> Array
    #   resp.network_insights_analysis.explanations[0].acl.id #=> String
    #   resp.network_insights_analysis.explanations[0].acl.arn #=> String
    #   resp.network_insights_analysis.explanations[0].acl.name #=> String
    #   resp.network_insights_analysis.explanations[0].acl_rule.cidr #=> String
    #   resp.network_insights_analysis.explanations[0].acl_rule.egress #=> Boolean
    #   resp.network_insights_analysis.explanations[0].acl_rule.port_range.from #=> Integer
    #   resp.network_insights_analysis.explanations[0].acl_rule.port_range.to #=> Integer
    #   resp.network_insights_analysis.explanations[0].acl_rule.protocol #=> String
    #   resp.network_insights_analysis.explanations[0].acl_rule.rule_action #=> String
    #   resp.network_insights_analysis.explanations[0].acl_rule.rule_number #=> Integer
    #   resp.network_insights_analysis.explanations[0].address #=> String
    #   resp.network_insights_analysis.explanations[0].addresses #=> Array
    #   resp.network_insights_analysis.explanations[0].addresses[0] #=> String
    #   resp.network_insights_analysis.explanations[0].attached_to.id #=> String
    #   resp.network_insights_analysis.explanations[0].attached_to.arn #=> String
    #   resp.network_insights_analysis.explanations[0].attached_to.name #=> String
    #   resp.network_insights_analysis.explanations[0].availability_zones #=> Array
    #   resp.network_insights_analysis.explanations[0].availability_zones[0] #=> String
    #   resp.network_insights_analysis.explanations[0].cidrs #=> Array
    #   resp.network_insights_analysis.explanations[0].cidrs[0] #=> String
    #   resp.network_insights_analysis.explanations[0].component.id #=> String
    #   resp.network_insights_analysis.explanations[0].component.arn #=> String
    #   resp.network_insights_analysis.explanations[0].component.name #=> String
    #   resp.network_insights_analysis.explanations[0].customer_gateway.id #=> String
    #   resp.network_insights_analysis.explanations[0].customer_gateway.arn #=> String
    #   resp.network_insights_analysis.explanations[0].customer_gateway.name #=> String
    #   resp.network_insights_analysis.explanations[0].destination.id #=> String
    #   resp.network_insights_analysis.explanations[0].destination.arn #=> String
    #   resp.network_insights_analysis.explanations[0].destination.name #=> String
    #   resp.network_insights_analysis.explanations[0].destination_vpc.id #=> String
    #   resp.network_insights_analysis.explanations[0].destination_vpc.arn #=> String
    #   resp.network_insights_analysis.explanations[0].destination_vpc.name #=> String
    #   resp.network_insights_analysis.explanations[0].direction #=> String
    #   resp.network_insights_analysis.explanations[0].explanation_code #=> String
    #   resp.network_insights_analysis.explanations[0].ingress_route_table.id #=> String
    #   resp.network_insights_analysis.explanations[0].ingress_route_table.arn #=> String
    #   resp.network_insights_analysis.explanations[0].ingress_route_table.name #=> String
    #   resp.network_insights_analysis.explanations[0].internet_gateway.id #=> String
    #   resp.network_insights_analysis.explanations[0].internet_gateway.arn #=> String
    #   resp.network_insights_analysis.explanations[0].internet_gateway.name #=> String
    #   resp.network_insights_analysis.explanations[0].load_balancer_arn #=> String
    #   resp.network_insights_analysis.explanations[0].classic_load_balancer_listener.load_balancer_port #=> Integer
    #   resp.network_insights_analysis.explanations[0].classic_load_balancer_listener.instance_port #=> Integer
    #   resp.network_insights_analysis.explanations[0].load_balancer_listener_port #=> Integer
    #   resp.network_insights_analysis.explanations[0].load_balancer_target.address #=> String
    #   resp.network_insights_analysis.explanations[0].load_balancer_target.availability_zone #=> String
    #   resp.network_insights_analysis.explanations[0].load_balancer_target.instance.id #=> String
    #   resp.network_insights_analysis.explanations[0].load_balancer_target.instance.arn #=> String
    #   resp.network_insights_analysis.explanations[0].load_balancer_target.instance.name #=> String
    #   resp.network_insights_analysis.explanations[0].load_balancer_target.port #=> Integer
    #   resp.network_insights_analysis.explanations[0].load_balancer_target_group.id #=> String
    #   resp.network_insights_analysis.explanations[0].load_balancer_target_group.arn #=> String
    #   resp.network_insights_analysis.explanations[0].load_balancer_target_group.name #=> String
    #   resp.network_insights_analysis.explanations[0].load_balancer_target_groups #=> Array
    #   resp.network_insights_analysis.explanations[0].load_balancer_target_groups[0].id #=> String
    #   resp.network_insights_analysis.explanations[0].load_balancer_target_groups[0].arn #=> String
    #   resp.network_insights_analysis.explanations[0].load_balancer_target_groups[0].name #=> String
    #   resp.network_insights_analysis.explanations[0].load_balancer_target_port #=> Integer
    #   resp.network_insights_analysis.explanations[0].elastic_load_balancer_listener.id #=> String
    #   resp.network_insights_analysis.explanations[0].elastic_load_balancer_listener.arn #=> String
    #   resp.network_insights_analysis.explanations[0].elastic_load_balancer_listener.name #=> String
    #   resp.network_insights_analysis.explanations[0].missing_component #=> String
    #   resp.network_insights_analysis.explanations[0].nat_gateway.id #=> String
    #   resp.network_insights_analysis.explanations[0].nat_gateway.arn #=> String
    #   resp.network_insights_analysis.explanations[0].nat_gateway.name #=> String
    #   resp.network_insights_analysis.explanations[0].network_interface.id #=> String
    #   resp.network_insights_analysis.explanations[0].network_interface.arn #=> String
    #   resp.network_insights_analysis.explanations[0].network_interface.name #=> String
    #   resp.network_insights_analysis.explanations[0].packet_field #=> String
    #   resp.network_insights_analysis.explanations[0].vpc_peering_connection.id #=> String
    #   resp.network_insights_analysis.explanations[0].vpc_peering_connection.arn #=> String
    #   resp.network_insights_analysis.explanations[0].vpc_peering_connection.name #=> String
    #   resp.network_insights_analysis.explanations[0].port #=> Integer
    #   resp.network_insights_analysis.explanations[0].port_ranges #=> Array
    #   resp.network_insights_analysis.explanations[0].port_ranges[0].from #=> Integer
    #   resp.network_insights_analysis.explanations[0].port_ranges[0].to #=> Integer
    #   resp.network_insights_analysis.explanations[0].prefix_list.id #=> String
    #   resp.network_insights_analysis.explanations[0].prefix_list.arn #=> String
    #   resp.network_insights_analysis.explanations[0].prefix_list.name #=> String
    #   resp.network_insights_analysis.explanations[0].protocols #=> Array
    #   resp.network_insights_analysis.explanations[0].protocols[0] #=> String
    #   resp.network_insights_analysis.explanations[0].route_table_route.destination_cidr #=> String
    #   resp.network_insights_analysis.explanations[0].route_table_route.destination_prefix_list_id #=> String
    #   resp.network_insights_analysis.explanations[0].route_table_route.egress_only_internet_gateway_id #=> String
    #   resp.network_insights_analysis.explanations[0].route_table_route.gateway_id #=> String
    #   resp.network_insights_analysis.explanations[0].route_table_route.instance_id #=> String
    #   resp.network_insights_analysis.explanations[0].route_table_route.nat_gateway_id #=> String
    #   resp.network_insights_analysis.explanations[0].route_table_route.network_interface_id #=> String
    #   resp.network_insights_analysis.explanations[0].route_table_route.origin #=> String
    #   resp.network_insights_analysis.explanations[0].route_table_route.transit_gateway_id #=> String
    #   resp.network_insights_analysis.explanations[0].route_table_route.vpc_peering_connection_id #=> String
    #   resp.network_insights_analysis.explanations[0].route_table_route.state #=> String
    #   resp.network_insights_analysis.explanations[0].route_table_route.carrier_gateway_id #=> String
    #   resp.network_insights_analysis.explanations[0].route_table_route.core_network_arn #=> String
    #   resp.network_insights_analysis.explanations[0].route_table_route.local_gateway_id #=> String
    #   resp.network_insights_analysis.explanations[0].route_table.id #=> String
    #   resp.network_insights_analysis.explanations[0].route_table.arn #=> String
    #   resp.network_insights_analysis.explanations[0].route_table.name #=> String
    #   resp.network_insights_analysis.explanations[0].security_group.id #=> String
    #   resp.network_insights_analysis.explanations[0].security_group.arn #=> String
    #   resp.network_insights_analysis.explanations[0].security_group.name #=> String
    #   resp.network_insights_analysis.explanations[0].security_group_rule.cidr #=> String
    #   resp.network_insights_analysis.explanations[0].security_group_rule.direction #=> String
    #   resp.network_insights_analysis.explanations[0].security_group_rule.security_group_id #=> String
    #   resp.network_insights_analysis.explanations[0].security_group_rule.port_range.from #=> Integer
    #   resp.network_insights_analysis.explanations[0].security_group_rule.port_range.to #=> Integer
    #   resp.network_insights_analysis.explanations[0].security_group_rule.prefix_list_id #=> String
    #   resp.network_insights_analysis.explanations[0].security_group_rule.protocol #=> String
    #   resp.network_insights_analysis.explanations[0].security_groups #=> Array
    #   resp.network_insights_analysis.explanations[0].security_groups[0].id #=> String
    #   resp.network_insights_analysis.explanations[0].security_groups[0].arn #=> String
    #   resp.network_insights_analysis.explanations[0].security_groups[0].name #=> String
    #   resp.network_insights_analysis.explanations[0].source_vpc.id #=> String
    #   resp.network_insights_analysis.explanations[0].source_vpc.arn #=> String
    #   resp.network_insights_analysis.explanations[0].source_vpc.name #=> String
    #   resp.network_insights_analysis.explanations[0].state #=> String
    #   resp.network_insights_analysis.explanations[0].subnet.id #=> String
    #   resp.network_insights_analysis.explanations[0].subnet.arn #=> String
    #   resp.network_insights_analysis.explanations[0].subnet.name #=> String
    #   resp.network_insights_analysis.explanations[0].subnet_route_table.id #=> String
    #   resp.network_insights_analysis.explanations[0].subnet_route_table.arn #=> String
    #   resp.network_insights_analysis.explanations[0].subnet_route_table.name #=> String
    #   resp.network_insights_analysis.explanations[0].vpc.id #=> String
    #   resp.network_insights_analysis.explanations[0].vpc.arn #=> String
    #   resp.network_insights_analysis.explanations[0].vpc.name #=> String
    #   resp.network_insights_analysis.explanations[0].vpc_endpoint.id #=> String
    #   resp.network_insights_analysis.explanations[0].vpc_endpoint.arn #=> String
    #   resp.network_insights_analysis.explanations[0].vpc_endpoint.name #=> String
    #   resp.network_insights_analysis.explanations[0].vpn_connection.id #=> String
    #   resp.network_insights_analysis.explanations[0].vpn_connection.arn #=> String
    #   resp.network_insights_analysis.explanations[0].vpn_connection.name #=> String
    #   resp.network_insights_analysis.explanations[0].vpn_gateway.id #=> String
    #   resp.network_insights_analysis.explanations[0].vpn_gateway.arn #=> String
    #   resp.network_insights_analysis.explanations[0].vpn_gateway.name #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway.id #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway.arn #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway.name #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway_route_table.id #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway_route_table.arn #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway_route_table.name #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway_route_table_route.destination_cidr #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway_route_table_route.state #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway_route_table_route.route_origin #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway_route_table_route.prefix_list_id #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway_route_table_route.attachment_id #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway_route_table_route.resource_id #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway_route_table_route.resource_type #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway_attachment.id #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway_attachment.arn #=> String
    #   resp.network_insights_analysis.explanations[0].transit_gateway_attachment.name #=> String
    #   resp.network_insights_analysis.explanations[0].component_account #=> String
    #   resp.network_insights_analysis.explanations[0].component_region #=> String
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.rule_group_arn #=> String
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.sources #=> Array
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.sources[0] #=> String
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.destinations #=> Array
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.destinations[0] #=> String
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.source_ports #=> Array
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.destination_ports #=> Array
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.protocols #=> Array
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.protocols[0] #=> Integer
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.rule_action #=> String
    #   resp.network_insights_analysis.explanations[0].firewall_stateless_rule.priority #=> Integer
    #   resp.network_insights_analysis.explanations[0].firewall_stateful_rule.rule_group_arn #=> String
    #   resp.network_insights_analysis.explanations[0].firewall_stateful_rule.sources #=> Array
    #   resp.network_insights_analysis.explanations[0].firewall_stateful_rule.sources[0] #=> String
    #   resp.network_insights_analysis.explanations[0].firewall_stateful_rule.destinations #=> Array
    #   resp.network_insights_analysis.explanations[0].firewall_stateful_rule.destinations[0] #=> String
    #   resp.network_insights_analysis.explanations[0].firewall_stateful_rule.source_ports #=> Array
    #   resp.network_insights_analysis.explanations[0].firewall_stateful_rule.source_ports[0].from #=> Integer
    #   resp.network_insights_analysis.explanations[0].firewall_stateful_rule.source_ports[0].to #=> Integer
    #   resp.network_insights_analysis.explanations[0].firewall_stateful_rule.destination_ports #=> Array
    #   resp.network_insights_analysis.explanations[0].firewall_stateful_rule.destination_ports[0].from #=> Integer
    #   resp.network_insights_analysis.explanations[0].firewall_stateful_rule.destination_ports[0].to #=> Integer
    #   resp.network_insights_analysis.explanations[0].firewall_stateful_rule.protocol #=> String
    #   resp.network_insights_analysis.explanations[0].firewall_stateful_rule.rule_action #=> String
    #   resp.network_insights_analysis.explanations[0].firewall_stateful_rule.direction #=> String
    #   resp.network_insights_analysis.alternate_path_hints #=> Array
    #   resp.network_insights_analysis.alternate_path_hints[0].component_id #=> String
    #   resp.network_insights_analysis.alternate_path_hints[0].component_arn #=> String
    #   resp.network_insights_analysis.suggested_accounts #=> Array
    #   resp.network_insights_analysis.suggested_accounts[0] #=> String
    #   resp.network_insights_analysis.tags #=> Array
    #   resp.network_insights_analysis.tags[0].key #=> String
    #   resp.network_insights_analysis.tags[0].value #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/StartNetworkInsightsAnalysis AWS API Documentation
    #
    # @overload start_network_insights_analysis(params = {})
    # @param [Hash] params ({})
    def start_network_insights_analysis(params = {}, options = {})
      req = build_request(:start_network_insights_analysis, params)
      req.send_request(options)
    end

    # Initiates the verification process to prove that the service provider
    # owns the private DNS name domain for the endpoint service.
    #
    # The service provider must successfully perform the verification before
    # the consumer can use the name to access the service.
    #
    # Before the service provider runs this command, they must add a record
    # to the DNS server.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [required, String] :service_id
    #   The ID of the endpoint service.
    #
    # @return [Types::StartVpcEndpointServicePrivateDnsVerificationResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::StartVpcEndpointServicePrivateDnsVerificationResult#return_value #return_value} => Boolean
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.start_vpc_endpoint_service_private_dns_verification({
    #     dry_run: false,
    #     service_id: "VpcEndpointServiceId", # required
    #   })
    #
    # @example Response structure
    #
    #   resp.return_value #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/StartVpcEndpointServicePrivateDnsVerification AWS API Documentation
    #
    # @overload start_vpc_endpoint_service_private_dns_verification(params = {})
    # @param [Hash] params ({})
    def start_vpc_endpoint_service_private_dns_verification(params = {}, options = {})
      req = build_request(:start_vpc_endpoint_service_private_dns_verification, params)
      req.send_request(options)
    end

    # Stops an Amazon EBS-backed instance. For more information, see [Stop
    # and start Amazon EC2 instances][1] in the *Amazon EC2 User Guide*.
    #
    # You can use the Stop action to hibernate an instance if the instance
    # is [enabled for hibernation][2] and it meets the [hibernation
    # prerequisites][3]. For more information, see [Hibernate your Amazon
    # EC2 instance][4] in the *Amazon EC2 User Guide*.
    #
    # We don't charge usage for a stopped instance, or data transfer fees;
    # however, your root partition Amazon EBS volume remains and continues
    # to persist your data, and you are charged for Amazon EBS volume usage.
    # Every time you start your instance, Amazon EC2 charges a one-minute
    # minimum for instance usage, and thereafter charges per second for
    # instance usage.
    #
    # You can't stop or hibernate instance store-backed instances. You
    # can't use the Stop action to hibernate Spot Instances, but you can
    # specify that Amazon EC2 should hibernate Spot Instances when they are
    # interrupted. For more information, see [Hibernating interrupted Spot
    # Instances][5] in the *Amazon EC2 User Guide*.
    #
    # When you stop or hibernate an instance, we shut it down. You can
    # restart your instance at any time. Before stopping or hibernating an
    # instance, make sure it is in a state from which it can be restarted.
    # Stopping an instance does not preserve data stored in RAM, but
    # hibernating an instance does preserve data stored in RAM. If an
    # instance cannot hibernate successfully, a normal shutdown occurs.
    #
    # Stopping and hibernating an instance is different to rebooting or
    # terminating it. For example, when you stop or hibernate an instance,
    # the root device and any other devices attached to the instance
    # persist. When you terminate an instance, the root device and any other
    # devices attached during the instance launch are automatically deleted.
    # For more information about the differences between rebooting,
    # stopping, hibernating, and terminating instances, see [Instance
    # lifecycle][6] in the *Amazon EC2 User Guide*.
    #
    # When you stop an instance, we attempt to shut it down forcibly after a
    # short while. If your instance appears stuck in the stopping state
    # after a period of time, there may be an issue with the underlying host
    # computer. For more information, see [Troubleshoot stopping your
    # instance][7] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Stop_Start.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/enabling-hibernation.html
    # [3]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/hibernating-prerequisites.html
    # [4]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html
    # [5]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/spot-interruptions.html#hibernate-spot-instances
    # [6]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-lifecycle.html
    # [7]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/TroubleshootingInstancesStopping.html
    #
    # @option params [required, Array<String>] :instance_ids
    #   The IDs of the instances.
    #
    # @option params [Boolean] :hibernate
    #   Hibernates the instance if the instance was enabled for hibernation at
    #   launch. If the instance cannot hibernate successfully, a normal
    #   shutdown occurs. For more information, see [Hibernate your
    #   instance][1] in the *Amazon EC2 User Guide*.
    #
    #   Default: `false`
    #
    #
    #
    #   [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/Hibernate.html
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [Boolean] :force
    #   Forces the instances to stop. The instances do not have an opportunity
    #   to flush file system caches or file system metadata. If you use this
    #   option, you must perform file system check and repair procedures. This
    #   option is not recommended for Windows instances.
    #
    #   Default: `false`
    #
    # @return [Types::StopInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::StopInstancesResult#stopping_instances #stopping_instances} => Array&lt;Types::InstanceStateChange&gt;
    #
    #
    # @example Example: To stop a running EC2 instance
    #
    #   # This example stops the specified EC2 instance.
    #
    #   resp = client.stop_instances({
    #     instance_ids: [
    #       "i-1234567890abcdef0", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     stopping_instances: [
    #       {
    #         current_state: {
    #           code: 64, 
    #           name: "stopping", 
    #         }, 
    #         instance_id: "i-1234567890abcdef0", 
    #         previous_state: {
    #           code: 16, 
    #           name: "running", 
    #         }, 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.stop_instances({
    #     instance_ids: ["InstanceId"], # required
    #     hibernate: false,
    #     dry_run: false,
    #     force: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.stopping_instances #=> Array
    #   resp.stopping_instances[0].instance_id #=> String
    #   resp.stopping_instances[0].current_state.code #=> Integer
    #   resp.stopping_instances[0].current_state.name #=> String, one of "pending", "running", "shutting-down", "terminated", "stopping", "stopped"
    #   resp.stopping_instances[0].previous_state.code #=> Integer
    #   resp.stopping_instances[0].previous_state.name #=> String, one of "pending", "running", "shutting-down", "terminated", "stopping", "stopped"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/StopInstances AWS API Documentation
    #
    # @overload stop_instances(params = {})
    # @param [Hash] params ({})
    def stop_instances(params = {}, options = {})
      req = build_request(:stop_instances, params)
      req.send_request(options)
    end

    # Terminates active Client VPN endpoint connections. This action can be
    # used to terminate a specific client connection, or up to five
    # connections established by a specific user.
    #
    # @option params [required, String] :client_vpn_endpoint_id
    #   The ID of the Client VPN endpoint to which the client is connected.
    #
    # @option params [String] :connection_id
    #   The ID of the client connection to be terminated.
    #
    # @option params [String] :username
    #   The name of the user who initiated the connection. Use this option to
    #   terminate all active connections for the specified user. This option
    #   can only be used if the user has established up to five connections.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::TerminateClientVpnConnectionsResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::TerminateClientVpnConnectionsResult#client_vpn_endpoint_id #client_vpn_endpoint_id} => String
    #   * {Types::TerminateClientVpnConnectionsResult#username #username} => String
    #   * {Types::TerminateClientVpnConnectionsResult#connection_statuses #connection_statuses} => Array&lt;Types::TerminateConnectionStatus&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.terminate_client_vpn_connections({
    #     client_vpn_endpoint_id: "ClientVpnEndpointId", # required
    #     connection_id: "String",
    #     username: "String",
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.client_vpn_endpoint_id #=> String
    #   resp.username #=> String
    #   resp.connection_statuses #=> Array
    #   resp.connection_statuses[0].connection_id #=> String
    #   resp.connection_statuses[0].previous_status.code #=> String, one of "active", "failed-to-terminate", "terminating", "terminated"
    #   resp.connection_statuses[0].previous_status.message #=> String
    #   resp.connection_statuses[0].current_status.code #=> String, one of "active", "failed-to-terminate", "terminating", "terminated"
    #   resp.connection_statuses[0].current_status.message #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/TerminateClientVpnConnections AWS API Documentation
    #
    # @overload terminate_client_vpn_connections(params = {})
    # @param [Hash] params ({})
    def terminate_client_vpn_connections(params = {}, options = {})
      req = build_request(:terminate_client_vpn_connections, params)
      req.send_request(options)
    end

    # Shuts down the specified instances. This operation is idempotent; if
    # you terminate an instance more than once, each call succeeds.
    #
    # If you specify multiple instances and the request fails (for example,
    # because of a single incorrect instance ID), none of the instances are
    # terminated.
    #
    # If you terminate multiple instances across multiple Availability
    # Zones, and one or more of the specified instances are enabled for
    # termination protection, the request fails with the following results:
    #
    # * The specified instances that are in the same Availability Zone as
    #   the protected instance are not terminated.
    #
    # * The specified instances that are in different Availability Zones,
    #   where no other specified instances are protected, are successfully
    #   terminated.
    #
    # For example, say you have the following instances:
    #
    # * Instance A: `us-east-1a`; Not protected
    #
    # * Instance B: `us-east-1a`; Not protected
    #
    # * Instance C: `us-east-1b`; Protected
    #
    # * Instance D: `us-east-1b`; not protected
    #
    # If you attempt to terminate all of these instances in the same
    # request, the request reports failure with the following results:
    #
    # * Instance A and Instance B are successfully terminated because none
    #   of the specified instances in `us-east-1a` are enabled for
    #   termination protection.
    #
    # * Instance C and Instance D fail to terminate because at least one of
    #   the specified instances in `us-east-1b` (Instance C) is enabled for
    #   termination protection.
    #
    # Terminated instances remain visible after termination (for
    # approximately one hour).
    #
    # By default, Amazon EC2 deletes all EBS volumes that were attached when
    # the instance launched. Volumes attached after instance launch continue
    # running.
    #
    # You can stop, start, and terminate EBS-backed instances. You can only
    # terminate instance store-backed instances. What happens to an instance
    # differs if you stop it or terminate it. For example, when you stop an
    # instance, the root device and any other devices attached to the
    # instance persist. When you terminate an instance, any attached EBS
    # volumes with the `DeleteOnTermination` block device mapping parameter
    # set to `true` are automatically deleted. For more information about
    # the differences between stopping and terminating instances, see
    # [Instance lifecycle][1] in the *Amazon EC2 User Guide*.
    #
    # For more information about troubleshooting, see [Troubleshooting
    # terminating your instance][2] in the *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/ec2-instance-lifecycle.html
    # [2]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/TroubleshootingInstancesShuttingDown.html
    #
    # @option params [required, Array<String>] :instance_ids
    #   The IDs of the instances.
    #
    #   Constraints: Up to 1000 instance IDs. We recommend breaking up this
    #   request into smaller batches.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::TerminateInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::TerminateInstancesResult#terminating_instances #terminating_instances} => Array&lt;Types::InstanceStateChange&gt;
    #
    #
    # @example Example: To terminate an EC2 instance
    #
    #   # This example terminates the specified EC2 instance.
    #
    #   resp = client.terminate_instances({
    #     instance_ids: [
    #       "i-1234567890abcdef0", 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #     terminating_instances: [
    #       {
    #         current_state: {
    #           code: 32, 
    #           name: "shutting-down", 
    #         }, 
    #         instance_id: "i-1234567890abcdef0", 
    #         previous_state: {
    #           code: 16, 
    #           name: "running", 
    #         }, 
    #       }, 
    #     ], 
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.terminate_instances({
    #     instance_ids: ["InstanceId"], # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.terminating_instances #=> Array
    #   resp.terminating_instances[0].instance_id #=> String
    #   resp.terminating_instances[0].current_state.code #=> Integer
    #   resp.terminating_instances[0].current_state.name #=> String, one of "pending", "running", "shutting-down", "terminated", "stopping", "stopped"
    #   resp.terminating_instances[0].previous_state.code #=> Integer
    #   resp.terminating_instances[0].previous_state.name #=> String, one of "pending", "running", "shutting-down", "terminated", "stopping", "stopped"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/TerminateInstances AWS API Documentation
    #
    # @overload terminate_instances(params = {})
    # @param [Hash] params ({})
    def terminate_instances(params = {}, options = {})
      req = build_request(:terminate_instances, params)
      req.send_request(options)
    end

    # Unassigns the specified IPv6 addresses or Prefix Delegation prefixes
    # from a network interface.
    #
    # @option params [Array<String>] :ipv_6_prefixes
    #   The IPv6 prefixes to unassign from the network interface.
    #
    # @option params [required, String] :network_interface_id
    #   The ID of the network interface.
    #
    # @option params [Array<String>] :ipv_6_addresses
    #   The IPv6 addresses to unassign from the network interface.
    #
    # @return [Types::UnassignIpv6AddressesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::UnassignIpv6AddressesResult#network_interface_id #network_interface_id} => String
    #   * {Types::UnassignIpv6AddressesResult#unassigned_ipv_6_addresses #unassigned_ipv_6_addresses} => Array&lt;String&gt;
    #   * {Types::UnassignIpv6AddressesResult#unassigned_ipv_6_prefixes #unassigned_ipv_6_prefixes} => Array&lt;String&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.unassign_ipv_6_addresses({
    #     ipv_6_prefixes: ["String"],
    #     network_interface_id: "NetworkInterfaceId", # required
    #     ipv_6_addresses: ["String"],
    #   })
    #
    # @example Response structure
    #
    #   resp.network_interface_id #=> String
    #   resp.unassigned_ipv_6_addresses #=> Array
    #   resp.unassigned_ipv_6_addresses[0] #=> String
    #   resp.unassigned_ipv_6_prefixes #=> Array
    #   resp.unassigned_ipv_6_prefixes[0] #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/UnassignIpv6Addresses AWS API Documentation
    #
    # @overload unassign_ipv_6_addresses(params = {})
    # @param [Hash] params ({})
    def unassign_ipv_6_addresses(params = {}, options = {})
      req = build_request(:unassign_ipv_6_addresses, params)
      req.send_request(options)
    end

    # Unassigns the specified secondary private IP addresses or IPv4 Prefix
    # Delegation prefixes from a network interface.
    #
    # @option params [Array<String>] :ipv_4_prefixes
    #   The IPv4 prefixes to unassign from the network interface.
    #
    # @option params [required, String] :network_interface_id
    #   The ID of the network interface.
    #
    # @option params [Array<String>] :private_ip_addresses
    #   The secondary private IP addresses to unassign from the network
    #   interface. You can specify this option multiple times to unassign more
    #   than one IP address.
    #
    # @return [Struct] Returns an empty {Seahorse::Client::Response response}.
    #
    #
    # @example Example: To unassign a secondary private IP address from a network interface
    #
    #   # This example unassigns the specified private IP address from the specified network interface.
    #
    #   resp = client.unassign_private_ip_addresses({
    #     network_interface_id: "eni-e5aa89a3", 
    #     private_ip_addresses: [
    #       "10.0.0.82", 
    #     ], 
    #   })
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.unassign_private_ip_addresses({
    #     ipv_4_prefixes: ["String"],
    #     network_interface_id: "NetworkInterfaceId", # required
    #     private_ip_addresses: ["String"],
    #   })
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/UnassignPrivateIpAddresses AWS API Documentation
    #
    # @overload unassign_private_ip_addresses(params = {})
    # @param [Hash] params ({})
    def unassign_private_ip_addresses(params = {}, options = {})
      req = build_request(:unassign_private_ip_addresses, params)
      req.send_request(options)
    end

    # Unassigns secondary private IPv4 addresses from a private NAT gateway.
    # You cannot unassign your primary private IP. For more information, see
    # [Edit secondary IP address associations][1] in the *Amazon VPC User
    # Guide*.
    #
    # While unassigning is in progress, you cannot assign/unassign
    # additional IP addresses while the connections are being drained. You
    # are, however, allowed to delete the NAT gateway.
    #
    # A private IP address will only be released at the end of
    # MaxDrainDurationSeconds. The private IP addresses stay associated and
    # support the existing connections, but do not support any new
    # connections (new connections are distributed across the remaining
    # assigned private IP address). After the existing connections drain
    # out, the private IP addresses are released.
    #
    #
    #
    #
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/vpc/latest/userguide/nat-gateway-working-with.html#nat-gateway-edit-secondary
    #
    # @option params [required, String] :nat_gateway_id
    #   The ID of the NAT gateway.
    #
    # @option params [required, Array<String>] :private_ip_addresses
    #   The private IPv4 addresses you want to unassign.
    #
    # @option params [Integer] :max_drain_duration_seconds
    #   The maximum amount of time to wait (in seconds) before forcibly
    #   releasing the IP addresses if connections are still in progress.
    #   Default value is 350 seconds.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::UnassignPrivateNatGatewayAddressResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::UnassignPrivateNatGatewayAddressResult#nat_gateway_id #nat_gateway_id} => String
    #   * {Types::UnassignPrivateNatGatewayAddressResult#nat_gateway_addresses #nat_gateway_addresses} => Array&lt;Types::NatGatewayAddress&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.unassign_private_nat_gateway_address({
    #     nat_gateway_id: "NatGatewayId", # required
    #     private_ip_addresses: ["String"], # required
    #     max_drain_duration_seconds: 1,
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.nat_gateway_id #=> String
    #   resp.nat_gateway_addresses #=> Array
    #   resp.nat_gateway_addresses[0].allocation_id #=> String
    #   resp.nat_gateway_addresses[0].network_interface_id #=> String
    #   resp.nat_gateway_addresses[0].private_ip #=> String
    #   resp.nat_gateway_addresses[0].public_ip #=> String
    #   resp.nat_gateway_addresses[0].association_id #=> String
    #   resp.nat_gateway_addresses[0].is_primary #=> Boolean
    #   resp.nat_gateway_addresses[0].failure_message #=> String
    #   resp.nat_gateway_addresses[0].status #=> String, one of "assigning", "unassigning", "associating", "disassociating", "succeeded", "failed"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/UnassignPrivateNatGatewayAddress AWS API Documentation
    #
    # @overload unassign_private_nat_gateway_address(params = {})
    # @param [Hash] params ({})
    def unassign_private_nat_gateway_address(params = {}, options = {})
      req = build_request(:unassign_private_nat_gateway_address, params)
      req.send_request(options)
    end

    # Unlocks a snapshot that is locked in governance mode or that is locked
    # in compliance mode but still in the cooling-off period. You can't
    # unlock a snapshot that is locked in compliance mode after the
    # cooling-off period has expired.
    #
    # @option params [required, String] :snapshot_id
    #   The ID of the snapshot to unlock.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::UnlockSnapshotResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::UnlockSnapshotResult#snapshot_id #snapshot_id} => String
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.unlock_snapshot({
    #     snapshot_id: "SnapshotId", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.snapshot_id #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/UnlockSnapshot AWS API Documentation
    #
    # @overload unlock_snapshot(params = {})
    # @param [Hash] params ({})
    def unlock_snapshot(params = {}, options = {})
      req = build_request(:unlock_snapshot, params)
      req.send_request(options)
    end

    # Disables detailed monitoring for a running instance. For more
    # information, see [Monitoring your instances and volumes][1] in the
    # *Amazon EC2 User Guide*.
    #
    #
    #
    # [1]: https://docs.aws.amazon.com/AWSEC2/latest/UserGuide/using-cloudwatch.html
    #
    # @option params [required, Array<String>] :instance_ids
    #   The IDs of the instances.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the operation,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::UnmonitorInstancesResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::UnmonitorInstancesResult#instance_monitorings #instance_monitorings} => Array&lt;Types::InstanceMonitoring&gt;
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.unmonitor_instances({
    #     instance_ids: ["InstanceId"], # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.instance_monitorings #=> Array
    #   resp.instance_monitorings[0].instance_id #=> String
    #   resp.instance_monitorings[0].monitoring.state #=> String, one of "disabled", "disabling", "enabled", "pending"
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/UnmonitorInstances AWS API Documentation
    #
    # @overload unmonitor_instances(params = {})
    # @param [Hash] params ({})
    def unmonitor_instances(params = {}, options = {})
      req = build_request(:unmonitor_instances, params)
      req.send_request(options)
    end

    # Updates the description of an egress (outbound) security group rule.
    # You can replace an existing description, or add a description to a
    # rule that did not have one previously. You can remove a description
    # for a security group rule by omitting the description parameter in the
    # request.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :group_id
    #   The ID of the security group. You must specify either the security
    #   group ID or the security group name in the request. For security
    #   groups in a nondefault VPC, you must specify the security group ID.
    #
    # @option params [String] :group_name
    #   \[Default VPC\] The name of the security group. You must specify
    #   either the security group ID or the security group name.
    #
    # @option params [Array<Types::IpPermission>] :ip_permissions
    #   The IP permissions for the security group rule. You must specify
    #   either the IP permissions or the description.
    #
    # @option params [Array<Types::SecurityGroupRuleDescription>] :security_group_rule_descriptions
    #   The description for the egress security group rules. You must specify
    #   either the description or the IP permissions.
    #
    # @return [Types::UpdateSecurityGroupRuleDescriptionsEgressResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::UpdateSecurityGroupRuleDescriptionsEgressResult#return #return} => Boolean
    #
    #
    # @example Example: To update an outbound security group rule description
    #
    #   # This example updates the description for the specified security group rule.
    #
    #   resp = client.update_security_group_rule_descriptions_egress({
    #     group_id: "sg-123abc12", 
    #     ip_permissions: [
    #       {
    #         from_port: 80, 
    #         ip_protocol: "tcp", 
    #         ip_ranges: [
    #           {
    #             cidr_ip: "203.0.113.0/24", 
    #             description: "Outbound HTTP access to server 2", 
    #           }, 
    #         ], 
    #         to_port: 80, 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.update_security_group_rule_descriptions_egress({
    #     dry_run: false,
    #     group_id: "SecurityGroupId",
    #     group_name: "SecurityGroupName",
    #     ip_permissions: [
    #       {
    #         ip_protocol: "String",
    #         from_port: 1,
    #         to_port: 1,
    #         user_id_group_pairs: [
    #           {
    #             description: "String",
    #             user_id: "String",
    #             group_name: "String",
    #             group_id: "String",
    #             vpc_id: "String",
    #             vpc_peering_connection_id: "String",
    #             peering_status: "String",
    #           },
    #         ],
    #         ip_ranges: [
    #           {
    #             description: "String",
    #             cidr_ip: "String",
    #           },
    #         ],
    #         ipv_6_ranges: [
    #           {
    #             description: "String",
    #             cidr_ipv_6: "String",
    #           },
    #         ],
    #         prefix_list_ids: [
    #           {
    #             description: "String",
    #             prefix_list_id: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     security_group_rule_descriptions: [
    #       {
    #         security_group_rule_id: "String",
    #         description: "String",
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/UpdateSecurityGroupRuleDescriptionsEgress AWS API Documentation
    #
    # @overload update_security_group_rule_descriptions_egress(params = {})
    # @param [Hash] params ({})
    def update_security_group_rule_descriptions_egress(params = {}, options = {})
      req = build_request(:update_security_group_rule_descriptions_egress, params)
      req.send_request(options)
    end

    # Updates the description of an ingress (inbound) security group rule.
    # You can replace an existing description, or add a description to a
    # rule that did not have one previously. You can remove a description
    # for a security group rule by omitting the description parameter in the
    # request.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @option params [String] :group_id
    #   The ID of the security group. You must specify either the security
    #   group ID or the security group name in the request. For security
    #   groups in a nondefault VPC, you must specify the security group ID.
    #
    # @option params [String] :group_name
    #   \[Default VPC\] The name of the security group. You must specify
    #   either the security group ID or the security group name. For security
    #   groups in a nondefault VPC, you must specify the security group ID.
    #
    # @option params [Array<Types::IpPermission>] :ip_permissions
    #   The IP permissions for the security group rule. You must specify
    #   either IP permissions or a description.
    #
    # @option params [Array<Types::SecurityGroupRuleDescription>] :security_group_rule_descriptions
    #   The description for the ingress security group rules. You must specify
    #   either a description or IP permissions.
    #
    # @return [Types::UpdateSecurityGroupRuleDescriptionsIngressResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::UpdateSecurityGroupRuleDescriptionsIngressResult#return #return} => Boolean
    #
    #
    # @example Example: To update an inbound security group rule description
    #
    #   # This example updates the description for the specified security group rule.
    #
    #   resp = client.update_security_group_rule_descriptions_ingress({
    #     group_id: "sg-123abc12", 
    #     ip_permissions: [
    #       {
    #         from_port: 22, 
    #         ip_protocol: "tcp", 
    #         ip_ranges: [
    #           {
    #             cidr_ip: "203.0.113.0/16", 
    #             description: "SSH access from the LA office", 
    #           }, 
    #         ], 
    #         to_port: 22, 
    #       }, 
    #     ], 
    #   })
    #
    #   resp.to_h outputs the following:
    #   {
    #   }
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.update_security_group_rule_descriptions_ingress({
    #     dry_run: false,
    #     group_id: "SecurityGroupId",
    #     group_name: "SecurityGroupName",
    #     ip_permissions: [
    #       {
    #         ip_protocol: "String",
    #         from_port: 1,
    #         to_port: 1,
    #         user_id_group_pairs: [
    #           {
    #             description: "String",
    #             user_id: "String",
    #             group_name: "String",
    #             group_id: "String",
    #             vpc_id: "String",
    #             vpc_peering_connection_id: "String",
    #             peering_status: "String",
    #           },
    #         ],
    #         ip_ranges: [
    #           {
    #             description: "String",
    #             cidr_ip: "String",
    #           },
    #         ],
    #         ipv_6_ranges: [
    #           {
    #             description: "String",
    #             cidr_ipv_6: "String",
    #           },
    #         ],
    #         prefix_list_ids: [
    #           {
    #             description: "String",
    #             prefix_list_id: "String",
    #           },
    #         ],
    #       },
    #     ],
    #     security_group_rule_descriptions: [
    #       {
    #         security_group_rule_id: "String",
    #         description: "String",
    #       },
    #     ],
    #   })
    #
    # @example Response structure
    #
    #   resp.return #=> Boolean
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/UpdateSecurityGroupRuleDescriptionsIngress AWS API Documentation
    #
    # @overload update_security_group_rule_descriptions_ingress(params = {})
    # @param [Hash] params ({})
    def update_security_group_rule_descriptions_ingress(params = {}, options = {})
      req = build_request(:update_security_group_rule_descriptions_ingress, params)
      req.send_request(options)
    end

    # Stops advertising an address range that is provisioned as an address
    # pool.
    #
    # You can perform this operation at most once every 10 seconds, even if
    # you specify different address ranges each time.
    #
    # It can take a few minutes before traffic to the specified addresses
    # stops routing to Amazon Web Services because of BGP propagation
    # delays.
    #
    # @option params [required, String] :cidr
    #   The address range, in CIDR notation.
    #
    # @option params [Boolean] :dry_run
    #   Checks whether you have the required permissions for the action,
    #   without actually making the request, and provides an error response.
    #   If you have the required permissions, the error response is
    #   `DryRunOperation`. Otherwise, it is `UnauthorizedOperation`.
    #
    # @return [Types::WithdrawByoipCidrResult] Returns a {Seahorse::Client::Response response} object which responds to the following methods:
    #
    #   * {Types::WithdrawByoipCidrResult#byoip_cidr #byoip_cidr} => Types::ByoipCidr
    #
    # @example Request syntax with placeholder values
    #
    #   resp = client.withdraw_byoip_cidr({
    #     cidr: "String", # required
    #     dry_run: false,
    #   })
    #
    # @example Response structure
    #
    #   resp.byoip_cidr.cidr #=> String
    #   resp.byoip_cidr.description #=> String
    #   resp.byoip_cidr.asn_associations #=> Array
    #   resp.byoip_cidr.asn_associations[0].asn #=> String
    #   resp.byoip_cidr.asn_associations[0].cidr #=> String
    #   resp.byoip_cidr.asn_associations[0].status_message #=> String
    #   resp.byoip_cidr.asn_associations[0].state #=> String, one of "disassociated", "failed-disassociation", "failed-association", "pending-disassociation", "pending-association", "associated"
    #   resp.byoip_cidr.status_message #=> String
    #   resp.byoip_cidr.state #=> String, one of "advertised", "deprovisioned", "failed-deprovision", "failed-provision", "pending-deprovision", "pending-provision", "provisioned", "provisioned-not-publicly-advertisable"
    #   resp.byoip_cidr.network_border_group #=> String
    #
    # @see http://docs.aws.amazon.com/goto/WebAPI/ec2-2016-11-15/WithdrawByoipCidr AWS API Documentation
    #
    # @overload withdraw_byoip_cidr(params = {})
    # @param [Hash] params ({})
    def withdraw_byoip_cidr(params = {}, options = {})
      req = build_request(:withdraw_byoip_cidr, params)
      req.send_request(options)
    end

    # @!endgroup

    # @param params ({})
    # @api private
    def build_request(operation_name, params = {})
      handlers = @handlers.for(operation_name)
      tracer = config.telemetry_provider.tracer_provider.tracer(
        Aws::Telemetry.module_to_tracer_name('Aws::EC2')
      )
      context = Seahorse::Client::RequestContext.new(
        operation_name: operation_name,
        operation: config.api.operation(operation_name),
        client: self,
        params: params,
        config: config,
        tracer: tracer
      )
      context[:gem_name] = 'aws-sdk-ec2'
      context[:gem_version] = '1.512.0'
      Seahorse::Client::Request.new(handlers, context)
    end

    # Polls an API operation until a resource enters a desired state.
    #
    # ## Basic Usage
    #
    # A waiter will call an API operation until:
    #
    # * It is successful
    # * It enters a terminal state
    # * It makes the maximum number of attempts
    #
    # In between attempts, the waiter will sleep.
    #
    #     # polls in a loop, sleeping between attempts
    #     client.wait_until(waiter_name, params)
    #
    # ## Configuration
    #
    # You can configure the maximum number of polling attempts, and the
    # delay (in seconds) between each polling attempt. You can pass
    # configuration as the final arguments hash.
    #
    #     # poll for ~25 seconds
    #     client.wait_until(waiter_name, params, {
    #       max_attempts: 5,
    #       delay: 5,
    #     })
    #
    # ## Callbacks
    #
    # You can be notified before each polling attempt and before each
    # delay. If you throw `:success` or `:failure` from these callbacks,
    # it will terminate the waiter.
    #
    #     started_at = Time.now
    #     client.wait_until(waiter_name, params, {
    #
    #       # disable max attempts
    #       max_attempts: nil,
    #
    #       # poll for 1 hour, instead of a number of attempts
    #       before_wait: -> (attempts, response) do
    #         throw :failure if Time.now - started_at > 3600
    #       end
    #     })
    #
    # ## Handling Errors
    #
    # When a waiter is unsuccessful, it will raise an error.
    # All of the failure errors extend from
    # {Aws::Waiters::Errors::WaiterFailed}.
    #
    #     begin
    #       client.wait_until(...)
    #     rescue Aws::Waiters::Errors::WaiterFailed
    #       # resource did not enter the desired state in time
    #     end
    #
    # ## Valid Waiters
    #
    # The following table lists the valid waiter names, the operations they call,
    # and the default `:delay` and `:max_attempts` values.
    #
    # | waiter_name                     | params                                    | :delay   | :max_attempts |
    # | ------------------------------- | ----------------------------------------- | -------- | ------------- |
    # | bundle_task_complete            | {Client#describe_bundle_tasks}            | 15       | 40            |
    # | conversion_task_cancelled       | {Client#describe_conversion_tasks}        | 15       | 40            |
    # | conversion_task_completed       | {Client#describe_conversion_tasks}        | 15       | 40            |
    # | conversion_task_deleted         | {Client#describe_conversion_tasks}        | 15       | 40            |
    # | customer_gateway_available      | {Client#describe_customer_gateways}       | 15       | 40            |
    # | export_task_cancelled           | {Client#describe_export_tasks}            | 15       | 40            |
    # | export_task_completed           | {Client#describe_export_tasks}            | 15       | 40            |
    # | image_available                 | {Client#describe_images}                  | 15       | 40            |
    # | image_exists                    | {Client#describe_images}                  | 15       | 40            |
    # | instance_exists                 | {Client#describe_instances}               | 5        | 40            |
    # | instance_running                | {Client#describe_instances}               | 15       | 40            |
    # | instance_status_ok              | {Client#describe_instance_status}         | 15       | 40            |
    # | instance_stopped                | {Client#describe_instances}               | 15       | 40            |
    # | instance_terminated             | {Client#describe_instances}               | 15       | 40            |
    # | internet_gateway_exists         | {Client#describe_internet_gateways}       | 5        | 6             |
    # | key_pair_exists                 | {Client#describe_key_pairs}               | 5        | 6             |
    # | nat_gateway_available           | {Client#describe_nat_gateways}            | 15       | 40            |
    # | nat_gateway_deleted             | {Client#describe_nat_gateways}            | 15       | 40            |
    # | network_interface_available     | {Client#describe_network_interfaces}      | 20       | 10            |
    # | password_data_available         | {Client#get_password_data}                | 15       | 40            |
    # | security_group_exists           | {Client#describe_security_groups}         | 5        | 6             |
    # | snapshot_completed              | {Client#describe_snapshots}               | 15       | 40            |
    # | snapshot_imported               | {Client#describe_import_snapshot_tasks}   | 15       | 40            |
    # | spot_instance_request_fulfilled | {Client#describe_spot_instance_requests}  | 15       | 40            |
    # | store_image_task_complete       | {Client#describe_store_image_tasks}       | 5        | 40            |
    # | subnet_available                | {Client#describe_subnets}                 | 15       | 40            |
    # | system_status_ok                | {Client#describe_instance_status}         | 15       | 40            |
    # | volume_available                | {Client#describe_volumes}                 | 15       | 40            |
    # | volume_deleted                  | {Client#describe_volumes}                 | 15       | 40            |
    # | volume_in_use                   | {Client#describe_volumes}                 | 15       | 40            |
    # | vpc_available                   | {Client#describe_vpcs}                    | 15       | 40            |
    # | vpc_exists                      | {Client#describe_vpcs}                    | 1        | 5             |
    # | vpc_peering_connection_deleted  | {Client#describe_vpc_peering_connections} | 15       | 40            |
    # | vpc_peering_connection_exists   | {Client#describe_vpc_peering_connections} | 15       | 40            |
    # | vpn_connection_available        | {Client#describe_vpn_connections}         | 15       | 40            |
    # | vpn_connection_deleted          | {Client#describe_vpn_connections}         | 15       | 40            |
    #
    # @raise [Errors::FailureStateError] Raised when the waiter terminates
    #   because the waiter has entered a state that it will not transition
    #   out of, preventing success.
    #
    # @raise [Errors::TooManyAttemptsError] Raised when the configured
    #   maximum number of attempts have been made, and the waiter is not
    #   yet successful.
    #
    # @raise [Errors::UnexpectedError] Raised when an error is encounted
    #   while polling for a resource that is not expected.
    #
    # @raise [Errors::NoSuchWaiterError] Raised when you request to wait
    #   for an unknown state.
    #
    # @return [Boolean] Returns `true` if the waiter was successful.
    # @param [Symbol] waiter_name
    # @param [Hash] params ({})
    # @param [Hash] options ({})
    # @option options [Integer] :max_attempts
    # @option options [Integer] :delay
    # @option options [Proc] :before_attempt
    # @option options [Proc] :before_wait
    def wait_until(waiter_name, params = {}, options = {})
      w = waiter(waiter_name, options)
      yield(w.waiter) if block_given? # deprecated
      w.wait(params)
    end

    # @api private
    # @deprecated
    def waiter_names
      waiters.keys
    end

    private

    # @param [Symbol] waiter_name
    # @param [Hash] options ({})
    def waiter(waiter_name, options = {})
      waiter_class = waiters[waiter_name]
      if waiter_class
        waiter_class.new(options.merge(client: self))
      else
        raise Aws::Waiters::Errors::NoSuchWaiterError.new(waiter_name, waiters.keys)
      end
    end

    def waiters
      {
        bundle_task_complete: Waiters::BundleTaskComplete,
        conversion_task_cancelled: Waiters::ConversionTaskCancelled,
        conversion_task_completed: Waiters::ConversionTaskCompleted,
        conversion_task_deleted: Waiters::ConversionTaskDeleted,
        customer_gateway_available: Waiters::CustomerGatewayAvailable,
        export_task_cancelled: Waiters::ExportTaskCancelled,
        export_task_completed: Waiters::ExportTaskCompleted,
        image_available: Waiters::ImageAvailable,
        image_exists: Waiters::ImageExists,
        instance_exists: Waiters::InstanceExists,
        instance_running: Waiters::InstanceRunning,
        instance_status_ok: Waiters::InstanceStatusOk,
        instance_stopped: Waiters::InstanceStopped,
        instance_terminated: Waiters::InstanceTerminated,
        internet_gateway_exists: Waiters::InternetGatewayExists,
        key_pair_exists: Waiters::KeyPairExists,
        nat_gateway_available: Waiters::NatGatewayAvailable,
        nat_gateway_deleted: Waiters::NatGatewayDeleted,
        network_interface_available: Waiters::NetworkInterfaceAvailable,
        password_data_available: Waiters::PasswordDataAvailable,
        security_group_exists: Waiters::SecurityGroupExists,
        snapshot_completed: Waiters::SnapshotCompleted,
        snapshot_imported: Waiters::SnapshotImported,
        spot_instance_request_fulfilled: Waiters::SpotInstanceRequestFulfilled,
        store_image_task_complete: Waiters::StoreImageTaskComplete,
        subnet_available: Waiters::SubnetAvailable,
        system_status_ok: Waiters::SystemStatusOk,
        volume_available: Waiters::VolumeAvailable,
        volume_deleted: Waiters::VolumeDeleted,
        volume_in_use: Waiters::VolumeInUse,
        vpc_available: Waiters::VpcAvailable,
        vpc_exists: Waiters::VpcExists,
        vpc_peering_connection_deleted: Waiters::VpcPeeringConnectionDeleted,
        vpc_peering_connection_exists: Waiters::VpcPeeringConnectionExists,
        vpn_connection_available: Waiters::VpnConnectionAvailable,
        vpn_connection_deleted: Waiters::VpnConnectionDeleted
      }
    end

    class << self

      # @api private
      attr_reader :identifier

      # @api private
      def errors_module
        Errors
      end

    end
  end
end
